Package: matrixCorr
Type: Package
Title: Collection of Correlation, Agreement, and Reliability Estimators
Version: 0.12.2
Authors@R: c(
    person(
        given = "Thiago de Paula",
        family = "Oliveira",
        email = "thiago.paula.oliveira@gmail.com",
        role = c("aut", "cre"),
        comment = c(ORCID = "0000-0002-4555-2584")
    )
    )
Author: Thiago de Paula Oliveira [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4555-2584>)
Maintainer: Thiago de Paula Oliveira <thiago.paula.oliveira@gmail.com>
Description: Compute correlation, association, agreement, and reliability
    measures for small to high-dimensional datasets through a consistent
    matrix-oriented interface. Supports classical correlations (Pearson,
    Spearman, Kendall, Chatterjee's rank correlation), distance correlation, partial correlation with
    regularised estimators, shrinkage correlation for p >= n settings, robust
    correlations including biweight mid-correlation, percentage-bend,
    Winsorized, and skipped correlation, latent-variable methods for binary
    and ordinal data, pairwise and overall intraclass correlation for wide
    data, repeated-measures correlation, and agreement/reliability analyses
    based on Cohen's kappa, weighted kappa, multi-rater kappa, Gwet's AC1/AC2,
    Krippendorff's alpha, Bland-Altman methods, Lin's concordance correlation
    coefficient, Poisson GLMM concordance for count data, and
    repeated-measures intraclass/concordance correlation.
    Implemented with optimized C++ backends using BLAS/OpenMP and memory-aware
    symmetric updates, and returns standard R objects with print/summary/plot
    methods plus optional Shiny viewers for matrix inspection. Methods based
    on Ledoit and Wolf (2004) <doi:10.1016/S0047-259X(03)00096-4>;
    high-dimensional shrinkage covariance estimation
    <doi:10.2202/1544-6115.1175>; Lin (1989) <doi:10.2307/2532051>; Wilcox
    (1994) <doi:10.1007/BF02294395>; Wilcox (2004)
    <doi:10.1080/0266476032000148821>; Hayes and Krippendorff (2007)
    <doi:10.1080/19312450709336664>; weighted repeated-measures correlation by
    Kondo et al. (2025) <doi:10.1002/sim.70046>.
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 4.4.0)
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp (>= 1.1.0), ggplot2 (>= 3.5.2), Matrix (>= 1.7.2), cli,
        generics, rlang
Suggests: knitr, rmarkdown, MASS, mnormt, shiny, shinyWidgets,
        viridisLite, testthat (>= 3.0.0)
VignetteBuilder: knitr
Enhances: plotly
RoxygenNote: 7.3.3
URL: https://github.com/Prof-ThiagoOliveira/matrixCorr
BugReports: https://github.com/Prof-ThiagoOliveira/matrixCorr/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-05-31 12:40:49 UTC; ThiagoOliveira
Repository: CRAN
Date/Publication: 2026-05-31 15:00:18 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-31 19:12:08 UTC; unix
Archs: matrixCorr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("$",ba)
S3method("$",rmcorr)
S3method("$",skipped_corr)
S3method("$",summary.corr_result)
S3method("[[",ba)
S3method("[[",rmcorr)
S3method("[[",skipped_corr)
S3method("[[",summary.corr_result)
S3method(as.data.frame,corr_edge_list)
S3method(ci,ba)
S3method(ci,ba_matrix)
S3method(ci,ba_repeated)
S3method(ci,ba_repeated_matrix)
S3method(ci,ccc)
S3method(ci,ccc_ci)
S3method(ci,ccc_glmm)
S3method(ci,cia)
S3method(ci,cia_ci)
S3method(ci,cia_rm)
S3method(ci,cohen_kappa)
S3method(ci,corr_result)
S3method(ci,default)
S3method(ci,gwet_ac)
S3method(ci,icc)
S3method(ci,icc_overall)
S3method(ci,icc_rm_reml)
S3method(ci,krippendorff_alpha)
S3method(ci,multirater_kappa)
S3method(ci,partial_corr)
S3method(ci,prob_agree)
S3method(ci,rmcorr)
S3method(ci,rmcorr_matrix)
S3method(ci,weighted_kappa)
S3method(coef,ccc_glmm)
S3method(coef,corr_result)
S3method(confint,ba)
S3method(confint,ba_matrix)
S3method(confint,ba_repeated)
S3method(confint,ba_repeated_matrix)
S3method(confint,ccc)
S3method(confint,ccc_ci)
S3method(confint,ccc_glmm)
S3method(confint,chatterjee_xi_scalar)
S3method(confint,cia)
S3method(confint,cia_ci)
S3method(confint,cia_rm)
S3method(confint,cohen_kappa)
S3method(confint,corr_result)
S3method(confint,gwet_ac)
S3method(confint,icc)
S3method(confint,icc_overall)
S3method(confint,icc_rm_reml)
S3method(confint,krippendorff_alpha)
S3method(confint,multirater_kappa)
S3method(confint,partial_corr)
S3method(confint,prob_agree)
S3method(confint,rmcorr)
S3method(confint,rmcorr_matrix)
S3method(confint,summary.corr_result)
S3method(confint,summary.matrixCorr)
S3method(confint,weighted_kappa)
S3method(estimate,ba)
S3method(estimate,ba_matrix)
S3method(estimate,ba_repeated)
S3method(estimate,ba_repeated_matrix)
S3method(estimate,ccc_glmm)
S3method(estimate,corr_result)
S3method(estimate,default)
S3method(estimate,partial_corr)
S3method(estimate,summary.corr_result)
S3method(estimate,summary.matrixCorr)
S3method(names,ba)
S3method(names,rmcorr)
S3method(names,skipped_corr)
S3method(plot,ba)
S3method(plot,ba_matrix)
S3method(plot,ba_repeated)
S3method(plot,ba_repeated_matrix)
S3method(plot,bicor)
S3method(plot,biserial_corr)
S3method(plot,ccc)
S3method(plot,cia)
S3method(plot,cia_rm)
S3method(plot,cohen_kappa)
S3method(plot,corr_edge_list)
S3method(plot,corr_matrix)
S3method(plot,corr_sparse)
S3method(plot,dcor)
S3method(plot,dgCMatrix)
S3method(plot,dsCMatrix)
S3method(plot,gwet_ac)
S3method(plot,hsic)
S3method(plot,kendall_matrix)
S3method(plot,krippendorff_alpha)
S3method(plot,multirater_kappa)
S3method(plot,partial_corr)
S3method(plot,pbcor)
S3method(plot,pearson_corr)
S3method(plot,polychoric_corr)
S3method(plot,polyserial_corr)
S3method(plot,prob_agree)
S3method(plot,rmcorr)
S3method(plot,rmcorr_matrix)
S3method(plot,robust_dcor)
S3method(plot,schafer_corr)
S3method(plot,shrinkage_corr)
S3method(plot,skipped_corr)
S3method(plot,sparseMatrix)
S3method(plot,spearman_rho)
S3method(plot,tetrachoric_corr)
S3method(plot,wincor)
S3method(print,ba)
S3method(print,ba_matrix)
S3method(print,ba_repeated)
S3method(print,ba_repeated_matrix)
S3method(print,bicor)
S3method(print,biserial_corr)
S3method(print,ccc)
S3method(print,ccc_ci)
S3method(print,ccc_glmm)
S3method(print,chatterjee_xi)
S3method(print,chatterjee_xi_scalar)
S3method(print,cia)
S3method(print,cia_ci)
S3method(print,cia_rm)
S3method(print,cohen_kappa)
S3method(print,corr_edge_list)
S3method(print,dcor)
S3method(print,gwet_ac)
S3method(print,hsic)
S3method(print,icc)
S3method(print,icc_overall)
S3method(print,icc_rm_reml)
S3method(print,kendall_matrix)
S3method(print,krippendorff_alpha)
S3method(print,matrixCorr_ccc)
S3method(print,matrixCorr_ccc_ci)
S3method(print,multirater_kappa)
S3method(print,partial_corr)
S3method(print,pbcor)
S3method(print,pearson_corr)
S3method(print,polychoric_corr)
S3method(print,polyserial_corr)
S3method(print,prob_agree)
S3method(print,rmcorr)
S3method(print,rmcorr_matrix)
S3method(print,robust_dcor)
S3method(print,schafer_corr)
S3method(print,shrinkage_corr)
S3method(print,skipped_corr)
S3method(print,spearman_rho)
S3method(print,summary.ba)
S3method(print,summary.ba_matrix)
S3method(print,summary.ba_repeated)
S3method(print,summary.ba_repeated_matrix)
S3method(print,summary.bicor)
S3method(print,summary.biserial_corr)
S3method(print,summary.ccc)
S3method(print,summary.ccc_glmm)
S3method(print,summary.ccc_rm_reml)
S3method(print,summary.chatterjee_xi_scalar)
S3method(print,summary.cia)
S3method(print,summary.cia_rm)
S3method(print,summary.cohen_kappa)
S3method(print,summary.corr_matrix)
S3method(print,summary.corr_result)
S3method(print,summary.dcor)
S3method(print,summary.gwet_ac)
S3method(print,summary.hsic)
S3method(print,summary.icc)
S3method(print,summary.icc_overall)
S3method(print,summary.icc_rm_reml)
S3method(print,summary.kendall_matrix)
S3method(print,summary.krippendorff_alpha)
S3method(print,summary.latent_corr)
S3method(print,summary.matrixCorr)
S3method(print,summary.multirater_kappa)
S3method(print,summary.partial_corr)
S3method(print,summary.pbcor)
S3method(print,summary.pearson_corr)
S3method(print,summary.polychoric_corr)
S3method(print,summary.polyserial_corr)
S3method(print,summary.prob_agree)
S3method(print,summary.rmcorr)
S3method(print,summary.rmcorr_matrix)
S3method(print,summary.skipped_corr)
S3method(print,summary.spearman_rho)
S3method(print,summary.tetrachoric_corr)
S3method(print,summary.wincor)
S3method(print,tetrachoric_corr)
S3method(print,weighted_kappa)
S3method(print,wincor)
S3method(summary,ba)
S3method(summary,ba_matrix)
S3method(summary,ba_repeated)
S3method(summary,ba_repeated_matrix)
S3method(summary,bicor)
S3method(summary,biserial_corr)
S3method(summary,ccc)
S3method(summary,ccc_glmm)
S3method(summary,ccc_rm_reml)
S3method(summary,chatterjee_xi_scalar)
S3method(summary,cia)
S3method(summary,cia_rm)
S3method(summary,cohen_kappa)
S3method(summary,corr_edge_list)
S3method(summary,corr_matrix)
S3method(summary,corr_sparse)
S3method(summary,dcor)
S3method(summary,dgCMatrix)
S3method(summary,dsCMatrix)
S3method(summary,gwet_ac)
S3method(summary,hsic)
S3method(summary,icc)
S3method(summary,icc_overall)
S3method(summary,icc_rm_reml)
S3method(summary,kendall_matrix)
S3method(summary,krippendorff_alpha)
S3method(summary,multirater_kappa)
S3method(summary,partial_corr)
S3method(summary,pbcor)
S3method(summary,pearson_corr)
S3method(summary,polychoric_corr)
S3method(summary,polyserial_corr)
S3method(summary,prob_agree)
S3method(summary,rmcorr)
S3method(summary,rmcorr_matrix)
S3method(summary,robust_dcor)
S3method(summary,schafer_corr)
S3method(summary,shrinkage_corr)
S3method(summary,skipped_corr)
S3method(summary,sparseMatrix)
S3method(summary,spearman_rho)
S3method(summary,tetrachoric_corr)
S3method(summary,wincor)
S3method(tidy,ba)
S3method(tidy,ba_matrix)
S3method(tidy,ba_repeated)
S3method(tidy,ba_repeated_matrix)
S3method(tidy,corr_result)
S3method(tidy,default)
S3method(tidy,partial_corr)
S3method(tidy,summary.corr_result)
S3method(tidy,summary.matrixCorr)
export(ba)
export(ba_rm)
export(bicor)
export(biserial)
export(biweight_mid_corr)
export(bland_altman)
export(bland_altman_repeated)
export(ccc)
export(ccc_glmm)
export(ccc_lmm_reml)
export(ccc_pairwise_u_stat)
export(ccc_rm_reml)
export(ccc_rm_ustat)
export(ci)
export(cia)
export(cia_rm)
export(cohen_kappa)
export(dcor)
export(diag.bicor)
export(distance_corr)
export(estimate)
export(gwet_ac)
export(hsic)
export(icc)
export(icc_rm_reml)
export(kendall_tau)
export(krippendorff_alpha)
export(multirater_kappa)
export(partial_correlation)
export(pbcor)
export(pcorr)
export(pearson_corr)
export(polychoric)
export(polyserial)
export(prob_agree)
export(rmcorr)
export(rmcorr_weighted)
export(robust_dcor)
export(schafer_corr)
export(shrinkage_corr)
export(skipped_corr)
export(skipped_corr_masks)
export(spearman_rho)
export(tetrachoric)
export(tidy)
export(view_corr_shiny)
export(view_rmcorr_shiny)
export(weighted_kappa)
export(wincor)
export(xi_corr)
import(ggplot2)
importFrom(Matrix,Matrix)
importFrom(Rcpp,evalCpp)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(generics,tidy)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(rlang,.data)
importFrom(rlang,arg_match)
importFrom(rlang,is_bool)
importFrom(rlang,is_scalar_character)
importFrom(rlang,is_scalar_integerish)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,hclust)
importFrom(stats,model.matrix)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,capture.output)
useDynLib(matrixCorr, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# matrixCorr <img src="man/figures/logo.png" align="right" height="160" />

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version/matrixCorr)](https://CRAN.R-project.org/package=matrixCorr)
![CRAN
downloads](https://cranlogs.r-pkg.org/badges/grand-total/matrixCorr)
[![R-CMD-check.yaml](https://github.com/Prof-ThiagoOliveira/matrixCorr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Prof-ThiagoOliveira/matrixCorr/actions/workflows/R-CMD-check.yaml)
[![test-coverage.yaml](https://github.com/Prof-ThiagoOliveira/matrixCorr/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/Prof-ThiagoOliveira/matrixCorr/actions/workflows/test-coverage.yaml)
<!-- badges: end -->

`matrixCorr` computes correlation and related association matrices from
small to high-dimensional data using simple, consistent functions and
sensible defaults. It includes shrinkage and robust options for noisy or
**p \>= n** settings, plus convenient print/plot/summary methods.
Performance-critical paths are implemented in C++ with BLAS/OpenMP and
memory-aware symmetric updates. The API accepts base matrices and data
frames and returns standard R objects via a consistent S3 interface.

Contributions from other researchers who want to add new correlation
methods are very welcome. A central goal of `matrixCorr` is to keep
efficient correlation and agreement estimation in one package with a
common interface and consistent outputs, so methods can be extended,
compared, and used without repeated translation across packages.

Supported measures include Pearson, Spearman, Kendall, distance
correlation, partial correlation, kernel dependence via the
Hilbert-Schmidt independence criterion, directed Chatterjee rank
correlation, robust biweight mid-correlation, percentage bend,
Winsorized, skipped correlation, and latent categorical/ordinal
correlations (tetrachoric, polychoric, polyserial, and biserial), plus
repeated-measures correlation; agreement tools cover Cohen’s kappa for
nominal ratings, weighted Cohen’s kappa for ordered two-rater agreement,
Gwet’s AC1/AC2, multi-rater kappa for nominal panel agreement,
Krippendorff’s alpha for panel-level reliability, Bland-Altman
(two-method and repeated-measures), the coefficient of individual
agreement for replicated and repeated-measures designs, Lin’s
concordance correlation coefficient (including repeated-measures
LMM/REML extensions and Poisson GLMM count-data CCC), and intraclass
correlation for both wide and repeated-measures designs.

## Features

| Area | Functions and tools |
|----|----|
| Backend | High-performance C++ backend using `Rcpp` |
| General correlations | `pearson_corr()`, `spearman_rho()`, `kendall_tau()` |
| Robust correlations | `bicor()`, `pbcor()`, `wincor()`, `skipped_corr()` |
| Distance correlation | `dcor()`, `robust_dcor()` |
| Kernel dependence | `hsic()` for raw biased/unbiased HSIC, normalised kCor-style dependence, kernel bandwidth rules, and permutation p-values |
| Partial correlation | `pcorr()` |
| Directed dependence | `xi_corr()` for Chatterjee rank correlation, with directed/asymmetric matrices and m-out-of-n bootstrap confidence intervals |
| Latent categorical/ordinal correlations | `tetrachoric()`, `polychoric()`, `polyserial()`, `biserial()` |
| Repeated-measures correlation | `rmcorr()` |
| Shrinkage for $p >> n$ | `shrinkage_corr()` |
| Agreement: two-rater categorical ratings | `cohen_kappa()` and `gwet_ac()` for nominal AC1/AC2 agreement, `weighted_kappa()` for ordered categories |
| Agreement: multi-rater and panel reliability | `multirater_kappa()`, `gwet_ac()` for panel AC1/AC2 agreement, `krippendorff_alpha()` |
| Agreement: Bland-Altman | Two-method or pairwise wide-input `ba()`, repeated-measures `ba_rm()` |
| Agreement: individual agreement | Replicated long-format `cia()`, repeated-measures `cia_rm()` |
| Agreement: probability of agreement | `prob_agree()` |
| Agreement: concordance | Pairwise Lin’s CCC `ccc()`, repeated-measures LMM/REML `ccc_rm_reml()`, Poisson GLMM count-data CCC `ccc_glmm()`, non-parametric `ccc_rm_ustat()` |
| Agreement: intraclass correlation | Wide-data `icc()` with pairwise and overall scope, repeated-measures REML `icc_rm_reml()` |
| Interactive viewers | Matrix-style Shiny viewers, including the repeated-measures correlation viewer `view_rmcorr_shiny()` |

## Installation

``` r
# Install from CRAN
install.packages("matrixCorr")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("Prof-ThiagoOliveira/matrixCorr")
```

## Thread settings (`n_threads`)

Most computational functions expose an `n_threads` argument and default
to:

``` r
getOption("matrixCorr.threads", 1L)
```

To define a package-wide default once per session:

``` r
options(matrixCorr.threads = parallel::detectCores(logical = FALSE))
```

## Quick start

### Wide-data correlation workflow

``` r
library(matrixCorr)

set.seed(1)
X <- as.data.frame(matrix(rnorm(300 * 6), ncol = 6))
names(X) <- paste0("V", 1:6)

R_pear <- pearson_corr(X, ci = TRUE)
R_bicor <- bicor(X)

print(R_pear, digits = 2)
#> Pearson correlation matrix
#>   method      : pearson
#>   dimensions  : 6 x 6
#>   ci          : yes
#> 
#>       V1    V2    V3    V4    V5    V6
#> V1  1.00  0.02  0.04 -0.02 -0.07  0.01
#> V2  0.02  1.00  0.04  0.03 -0.05  0.13
#> V3  0.04  0.04  1.00 -0.06  0.08 -0.14
#> V4 -0.02  0.03 -0.06  1.00  0.07  0.03
#> V5 -0.07 -0.05  0.08  0.07  1.00  0.04
#> V6  0.01  0.13 -0.14  0.03  0.04  1.00
summary(R_pear)
#> Pearson correlation summary
#>   output      : matrix
#>   dimensions  : 6 x 6
#>   retained_pairs: 15
#>   threshold   : 0.0000
#>   diag        : included
#>   estimate    : -0.1410 to 0.1272
#>   ci          : 95%
#>   ci_method   : fisher_z
#>   ci_width    : 0.222 to 0.226
#>   cross_zero  : 13 pair(s)
#> 
#>  item1 item2 estimate n_complete lwr     upr     fisher_z statistic p_value
#>  V3    V6    -0.1410  300        -0.2502 -0.0282 -0.1419  -2.4459   0.0144 
#>  V2    V6    0.1272   300        0.0142  0.2371  0.1279   2.2047    0.0275 
#>  V3    V5    0.0776   300        -0.0360 0.1892  0.0778   1.3401    0.1802 
#>  V4    V5    0.0724   300        -0.0412 0.1841  0.0725   1.2491    0.2116 
#>  V1    V5    -0.0650  300        -0.1770 0.0486  -0.0651  -1.1222   0.2618 
#>  ...   ...   ...      ...        ...     ...     ...      ...       ...    
#>  V2    V4    0.0335   300        -0.0801 0.1462  0.0335   0.5773    0.5638 
#>  V4    V6    0.0324   300        -0.0811 0.1451  0.0324   0.5584    0.5766 
#>  V1    V2    0.0236   300        -0.0899 0.1365  0.0236   0.4064    0.6845 
#>  V1    V4    -0.0185  300        -0.1315 0.0949  -0.0185  -0.3187   0.7500 
#>  V1    V6    0.0130   300        -0.1004 0.1261  0.0130   0.2243    0.8225 
#> ... 5 more rows not shown (omitted)
#> Use as.data.frame()/tidy()/as.matrix() to inspect the full result.
#> 
#> Strongest pairs by |estimate|
#> 
#>  item1 item2 estimate n_complete lwr     upr     fisher_z statistic p_value
#>  V3    V6    -0.1410  300        -0.2502 -0.0282 -0.1419  -2.4459   0.0144 
#>  V2    V6    0.1272   300        0.0142  0.2371  0.1279   2.2047    0.0275 
#>  V3    V5    0.0776   300        -0.0360 0.1892  0.0778   1.3401    0.1802 
#>  V4    V5    0.0724   300        -0.0412 0.1841  0.0725   1.2491    0.2116 
#>  V1    V5    -0.0650  300        -0.1770 0.0486  -0.0651  -1.1222   0.2618
plot(R_bicor)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="" width="100%" />

The same matrix-style workflow extends to Spearman, Kendall, distance
correlation, HSIC/kernel dependence, partial correlation, shrinkage
correlation, latent correlation, and the robust estimators `pbcor()`,
`wincor()`, and `skipped_corr()`.

`hsic()` computes pairwise kernel dependence matrices with Gaussian,
linear, Laplace, and polynomial kernels. It defaults to the package’s
normalised kCor-style output; use `normalise = FALSE` to return raw HSIC
estimates, and `p_value = TRUE` for permutation-based independence
tests.

### Agreement and repeated-measures workflow

``` r
set.seed(6)
S <- 24
Tm <- 4
id <- factor(rep(seq_len(S), each = 2 * Tm))
method <- factor(rep(rep(c("A", "B"), each = Tm), times = S))
time <- rep(rep(seq_len(Tm), times = 2), times = S)

u <- rnorm(S, 0, 0.9)[as.integer(id)]
um <- rnorm(S * 2, 0, 0.25)
um <- um[(as.integer(id) - 1L) * 2L + as.integer(method)]
y <- u + um + (method == "B") * 0.2 + rnorm(length(id), 0, 0.35)

dat_rm <- data.frame(y, id, method, time)

fit_ccc_rm <- ccc_rm_reml(
  dat_rm,
  response = "y",
  subject = "id",
  method = "method",
  time = "time"
)

summary(fit_ccc_rm)
#> 
#> Repeated-measures concordance (REML)
#> 
#> Concordance estimates
#> 
#>  item1 item2 estimate n_subjects n_obs SB     se_ccc
#>  A     B     0.8328   24         192   0.1096 0.0381
#> 
#> Variance components
#> 
#>  sigma2_subject sigma2_subject_method sigma2_subject_time sigma2_error
#>  0.7848         0.0192                0.007               0.1168      
#> 
#> AR(1) diagnostics
#> 
#>  ar1_rho ar1_rho_lag1 ar1_rho_mom ar1_pairs ar1_pval use_ar1 ar1_recommend
#>  -0.1073 -0.1073      -0.1073     144       0.198    FALSE   FALSE
```

Agreement and reliability methods use the same general inspection
pattern, but they target different quantities. The package includes
Cohen’s kappa for nominal two-rater ratings, weighted kappa for ordered
two-rater ratings, Gwet’s AC1/AC2, multi-rater kappa for nominal panel
agreement, Krippendorff’s alpha for panel-level reliability,
Bland-Altman analysis, concordance correlation, and intraclass
correlation for both wide and repeated-measures designs.

## Vignettes

The package documentation is organised as a set of workflow vignettes.
The README is intentionally brief; the vignettes are the main user
guide.

Start here:

- `vignette("v01-matrixCorr-introduction", package = "matrixCorr")`
  introduces the package structure, common object behaviour, and shared
  display conventions.
- `vignette("v02-wide-correlation-workflows", package = "matrixCorr")`
  covers Pearson, Spearman, Kendall, distance correlation, and the
  general wide-data matrix workflow.
- `vignette("v03-robust-and-highdim-correlation", package = "matrixCorr")`
  covers robust estimators, shrinkage, and high-dimensional settings.
- `vignette("v04-latent-and-mixed-scale-correlation", package = "matrixCorr")`
  covers tetrachoric, polychoric, polyserial, and biserial correlation.
- `vignette("v05-agreement-and-icc-wide", package = "matrixCorr")`
  covers Bland-Altman analysis, concordance, and intraclass correlation
  for wide data.
- `vignette("v06-repeated-measures-workflows", package = "matrixCorr")`
  covers repeated-measures correlation, repeated agreement, and repeated
  reliability workflows.

If you want a compact overview of the available estimators, start with
the introduction vignette and then move to the workflow family that
matches your data layout and scientific question.

## Contributing

Issues and pull requests are welcome. Please see `CONTRIBUTING.md` for
guidelines and `cran-comments.md`/`DESCRIPTION` for package metadata.

## License

GPL (>= 3) [Thiago de Paula Oliveira](https://orcid.org/0000-0002-4555-2584)

This is a copyleft license: modified versions distributed to others must be
distributed under the same GPL terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)

## -----------------------------------------------------------------------------
library(matrixCorr)

set.seed(50)
n_id <- 14
n_time <- 4

dat <- expand.grid(
  id = factor(seq_len(n_id)),
  time = factor(seq_len(n_time)),
  method = factor(c("A", "B"))
)

dat$time_index <- as.integer(dat$time)

subj <- rnorm(n_id, sd = 1.0)[dat$id]
subject_method <- rnorm(n_id * 2, sd = 0.25)
sm <- subject_method[(as.integer(dat$id) - 1L) * 2L + as.integer(dat$method)]
subject_time <- rnorm(n_id * n_time, sd = 0.75)
st <- subject_time[(as.integer(dat$id) - 1L) * n_time + as.integer(dat$time)]

dat$y <- subj + sm + st + 0.35 * (dat$method == "B") +
  rnorm(nrow(dat), sd = 0.35)

## -----------------------------------------------------------------------------
set.seed(51)
dat_rmcorr <- data.frame(
  id = rep(seq_len(n_id), each = n_time),
  x = rnorm(n_id * n_time),
  y = rnorm(n_id * n_time),
  z = rnorm(n_id * n_time)
)

dat_rmcorr$y <- 0.7 * dat_rmcorr$x +
  rnorm(n_id, sd = 1)[dat_rmcorr$id] +
  rnorm(nrow(dat_rmcorr), sd = 0.3)

fit_rmcorr <- rmcorr(dat_rmcorr, response = c("x", "y", "z"), subject = "id")
summary(fit_rmcorr)

## -----------------------------------------------------------------------------
fit_ba_rm <- ba_rm(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index"
)

summary(fit_ba_rm)

## -----------------------------------------------------------------------------
fit_ccc_ustat <- ccc_rm_ustat(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index"
)

fit_ccc_reml <- ccc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  ci = FALSE
)

summary(fit_ccc_ustat)
summary(fit_ccc_reml)

## -----------------------------------------------------------------------------
fit_icc_cons <- icc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  type = "consistency",
  ci = TRUE
)

fit_icc_agr <- icc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  type = "agreement",
  ci = FALSE
)

summary(fit_icc_cons)
summary(fit_icc_agr)
estimate(fit_icc_cons)
confint(fit_icc_cons)
ci(fit_icc_cons)
tidy(fit_icc_cons)

## -----------------------------------------------------------------------------
data.frame(
  method = c("Repeated CCC (REML)", "Repeated ICC (agreement, REML)"),
  estimate = c(
    fit_ccc_reml[1, 2],
    fit_icc_agr[1, 2]
  )
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)

## -----------------------------------------------------------------------------
library(matrixCorr)

set.seed(1)
X <- as.data.frame(matrix(rnorm(120), ncol = 4))
names(X) <- paste0("V", 1:4)

fit_pearson <- pearson_corr(X, ci = TRUE)
fit_spearman <- spearman_rho(X, ci = TRUE)

print(fit_pearson, digits = 2)
summary(fit_spearman)

## -----------------------------------------------------------------------------
estimate(fit_pearson)
confint(fit_pearson)
ci(fit_pearson)
tidy(fit_pearson)

## -----------------------------------------------------------------------------
set.seed(2)
ref <- rnorm(40, mean = 10, sd = 2)
alt <- ref + 0.3 + rnorm(40, sd = 0.8)

fit_ba <- ba(ref, alt)
print(fit_ba)

wide_methods <- data.frame(
  m1 = ref + rnorm(40, sd = 0.2),
  m2 = ref + 0.2 + rnorm(40, sd = 0.3),
  m3 = ref - 0.1 + rnorm(40, sd = 0.4)
)

fit_ccc <- ccc(wide_methods)
fit_icc <- icc(wide_methods, scope = "pairwise")

summary(fit_ccc)
summary(fit_icc)

## -----------------------------------------------------------------------------
fit_icc_overall <- icc(wide_methods, scope = "overall", ci = TRUE)
print(fit_icc_overall)
summary(fit_icc_overall)
estimate(fit_icc_overall)
confint(fit_icc_overall)
ci(fit_icc_overall)
tidy(fit_icc_overall)

## -----------------------------------------------------------------------------
set.seed(3)
n_subject <- 12
n_rep <- 3

subject <- rep(seq_len(n_subject), each = n_rep)
signal <- rnorm(n_subject * n_rep)
subject_x <- rnorm(n_subject, sd = 1.2)[subject]
subject_y <- rnorm(n_subject, sd = 1.0)[subject]

dat_rm <- data.frame(
  id = subject,
  x = subject_x + signal + rnorm(n_subject * n_rep, sd = 0.2),
  y = subject_y + 0.7 * signal + rnorm(n_subject * n_rep, sd = 0.3),
  z = subject_y - 0.4 * signal + rnorm(n_subject * n_rep, sd = 0.4)
)

fit_rmcorr <- rmcorr(dat_rm, response = c("x", "y", "z"), subject = "id")
print(fit_rmcorr, digits = 2)
summary(fit_rmcorr)

## -----------------------------------------------------------------------------
set.seed(4)
n_id <- 10
n_time <- 3

dat_agree <- expand.grid(
  id = factor(seq_len(n_id)),
  time = factor(seq_len(n_time)),
  method = factor(c("A", "B"))
)

subj <- rnorm(n_id, sd = 1.0)[dat_agree$id]
subj_method <- rnorm(n_id * 2, sd = 0.2)
sm <- subj_method[(as.integer(dat_agree$id) - 1L) * 2L + as.integer(dat_agree$method)]

dat_agree$y <- subj + sm + 0.25 * (dat_agree$method == "B") +
  rnorm(nrow(dat_agree), sd = 0.35)

fit_icc_rm <- icc_rm_reml(
  dat_agree,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  type = "consistency"
)

summary(fit_icc_rm)

## ----eval = FALSE-------------------------------------------------------------
# options(
#   matrixCorr.threads = parallel::detectCores(logical = FALSE),
#   matrixCorr.print_max_rows = 20L,
#   matrixCorr.print_topn = 5L,
#   matrixCorr.print_max_vars = 10L,
#   matrixCorr.print_show_ci = "yes",
#   matrixCorr.summary_max_rows = 12L,
#   matrixCorr.summary_topn = 5L,
#   matrixCorr.summary_max_vars = 10L,
#   matrixCorr.summary_show_ci = "yes"
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)

## -----------------------------------------------------------------------------
library(matrixCorr)

set.seed(20)
Y <- data.frame(
  x1 = rnorm(60),
  x2 = rnorm(60),
  x3 = rnorm(60),
  x4 = rnorm(60)
)

idx <- sample.int(nrow(Y), 5)
Y$x1[idx] <- Y$x1[idx] + 8
Y$x2[idx] <- Y$x2[idx] - 6

R_pear <- pearson_corr(Y)
R_bicor <- bicor(Y)
R_pb <- pbcor(Y)
R_win <- wincor(Y)
R_skip <- skipped_corr(Y)

summary(R_pear)
summary(R_bicor)

## -----------------------------------------------------------------------------
summary(R_skip)

## -----------------------------------------------------------------------------
fit_bicor_ci <- bicor(Y, ci = TRUE)
summary(fit_bicor_ci)
estimate(fit_bicor_ci)
confint(fit_bicor_ci)
ci(fit_bicor_ci)
tidy(fit_bicor_ci)

fit_pb_inf <- pbcor(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_pb_inf)

fit_win_inf <- wincor(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_win_inf)

fit_skip_inf <- skipped_corr(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_skip_inf)

## -----------------------------------------------------------------------------
set.seed(21)
n <- 300
x2 <- rnorm(n)
x1 <- 0.8 * x2 + rnorm(n, sd = 0.4)
x3 <- 0.8 * x2 + rnorm(n, sd = 0.4)
x4 <- 0.7 * x3 + rnorm(n, sd = 0.5)
x5 <- rnorm(n)
x6 <- rnorm(n)
X <- data.frame(x1, x2, x3, x4, x5, x6)

fit_pcor_sample <- pcorr(X, method = "sample")
fit_pcor_oas <- pcorr(X, method = "oas")
R_raw <- pearson_corr(X)

round(c(
  raw_x1_x3 = R_raw["x1", "x3"],
  partial_x1_x3 = estimate(fit_pcor_sample)["x1", "x3"]
), 2)

print(fit_pcor_sample, digits = 2)
summary(fit_pcor_oas)

## -----------------------------------------------------------------------------
fit_pcor_inf <- pcorr(
  X,
  method = "sample",
  return_p_value = TRUE,
  ci = TRUE
)

summary(fit_pcor_inf)
tidy(fit_pcor_inf)
ci(fit_pcor_inf)
confint(fit_pcor_inf)

## -----------------------------------------------------------------------------
set.seed(22)
n <- 40
p_block <- 30

make_block <- function(f) {
  sapply(seq_len(p_block), function(j) 0.8 * f + rnorm(n, sd = 0.8))
}

f1 <- rnorm(n)
f2 <- rnorm(n)
f3 <- rnorm(n)

Xd <- cbind(make_block(f1), make_block(f2), make_block(f3))
p <- ncol(Xd)
colnames(Xd) <- paste0("G", seq_len(p))

R_raw <- stats::cor(Xd)
fit_shr <- shrinkage_corr(Xd)
block <- rep(1:3, each = p_block)
within <- outer(block, block, "==") & upper.tri(R_raw)
between <- outer(block, block, "!=") & upper.tri(R_raw)

round(c(
  raw_within = mean(abs(R_raw[within])),
  raw_between = mean(abs(R_raw[between])),
  shrink_within = mean(abs(fit_shr[within])),
  shrink_between = mean(abs(fit_shr[between]))
), 3)

print(fit_shr, digits = 2, max_rows = 6, max_vars = 6)
summary(fit_shr)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)

## -----------------------------------------------------------------------------
library(matrixCorr)

set.seed(40)
ref <- rnorm(50, mean = 100, sd = 10)
m1 <- ref + rnorm(50, sd = 2)
m2 <- ref + 1.2 + rnorm(50, sd = 3)

fit_ba <- ba(m1, m2)
fit_ccc <- ccc(data.frame(m1 = m1, m2 = m2), ci = TRUE)

print(fit_ba)
summary(fit_ccc)
estimate(fit_ccc)
confint(fit_ccc)
ci(fit_ccc)
tidy(fit_ccc)

## -----------------------------------------------------------------------------
fit_ba_pairwise <- ba(data.frame(m1 = m1, m2 = m2, m3 = ref - 0.8 + rnorm(50, sd = 2.5)))
print(fit_ba_pairwise)
summary(fit_ba_pairwise)

## -----------------------------------------------------------------------------
wide_methods <- data.frame(
  J1 = ref + rnorm(50, sd = 1.5),
  J2 = ref + 4.0 + rnorm(50, sd = 1.8),
  J3 = ref - 3.0 + rnorm(50, sd = 2.0),
  J4 = ref + rnorm(50, sd = 1.6)
)

fit_icc_pair <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "pairwise"
)

fit_icc_overall <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "overall",
  ci = TRUE
)

print(fit_icc_pair, digits = 2)
summary(fit_icc_pair)
print(fit_icc_overall)

## -----------------------------------------------------------------------------
fit_icc_cons <- icc(
  wide_methods,
  model = "twoway_random",
  type = "consistency",
  unit = "single",
  scope = "overall",
  ci = FALSE
)

fit_icc_agr <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "overall",
  ci = FALSE
)

data.frame(
  type = c("consistency", "agreement"),
  selected_coefficient = c(
    attr(fit_icc_cons, "selected_coefficient"),
    attr(fit_icc_agr, "selected_coefficient")
  ),
  estimate = c(
    attr(fit_icc_cons, "selected_row")$estimate,
    attr(fit_icc_agr, "selected_row")$estimate
  )
)

## ----echo = FALSE-------------------------------------------------------------
agreement_summary <- data.frame(
  Function = c(
    "`ccc()`",
    "`ccc_rm_reml()`",
    "`ccc_rm_ustat()`",
    "`icc()`",
    "`icc_rm_reml()`",
    "`cia()`",
    "`cia_rm()`"
  ),
  `Index family` = c(
    "Lin CCC",
    "Repeated-measures CCC",
    "Repeated-measures CCC",
    "Classical ICC",
    "Repeated-measures ICC",
    "CIA",
    "Repeated-measures CIA"
  ),
  `Target question` = c(
    "Pairwise concordance combining precision and accuracy",
    "Pairwise repeated-measures concordance from fitted variance components",
    "Pairwise repeated-measures concordance from U-statistic distances",
    "Reliability or agreement under the selected ICC model, type, and unit",
    "Pairwise repeated-measures reliability/agreement from fitted variance components",
    "Individual agreement/interchangeability relative to within-method replicate disagreement",
    "Individual agreement/interchangeability across matched repeated conditions"
  ),
  `Data/design supported` = c(
    "Numeric wide data; rows are paired observational units",
    "Long repeated-measures data; subject plus optional method/time structure",
    "Long repeated-measures data; balanced method/time coverage per pair",
    "Numeric wide data; pairwise or overall all-column scope",
    "Long repeated-measures data; subject plus optional method/time structure",
    "Long replicated method-comparison data with replicate identifiers",
    "Long matched repeated-measures data with one observation per subject-method-condition cell"
  ),
  `Estimation approach` = c(
    "Moment CCC with Lin delta-method/Fisher-z CI when requested",
    "REML mixed-model variance components and fixed-effect bias term",
    "Nonparametric U-statistic estimator with Fisher-z CI when requested",
    "Classical ANOVA mean-square formulas",
    "REML mixed-model variance components and fixed-effect bias term",
    "Method-of-moments disagreement ratios; optional bounded variance-component variant",
    "Categorical-condition ANOVA estimator"
  ),
  `Inference reported` = c(
    "Estimate; optional confidence interval",
    "Estimate; optional confidence interval; variance-component diagnostics",
    "Estimate; optional confidence interval",
    "Pairwise: estimate and optional CI. Overall: coefficient table with F statistic, df, p-value, and optional CI",
    "Estimate; optional confidence interval; variance-component diagnostics",
    "Estimate; optional confidence interval",
    "Estimate; optional confidence interval; homogeneity F statistic and p-value"
  ),
  `Formal hypothesis test implemented? Yes/No/Requires verification` = c(
    "No",
    "No for the CCC parameter; variance-component selection tests are not CCC tests",
    "No",
    "Requires verification for overall p-values; no pairwise ICC test is documented",
    "No for the ICC parameter; variance-component selection tests are not ICC tests",
    "No",
    "Yes: homogeneity of agreement across conditions"
  ),
  check.names = FALSE
)

knitr::kable(agreement_summary, escape = FALSE)

#!/usr/bin/env Rscript

suppressPackageStartupMessages(library(matrixCorr))

run_case <- function(label, expr) {
  cat(sprintf("\n== %s ==\n", label))
  gc()
  flush.console()
  force(expr)
  cat(sprintf("OK: %s\n", label))
}

run_case("Case 1: pairwise, no time", {
  set.seed(123)
  n_subj <- 500L
  id <- factor(rep(seq_len(n_subj), each = 2L))
  method <- factor(rep(c("A", "B"), times = n_subj))

  sigA <- 1
  sigE <- 0.5
  biasB <- 0.2

  u <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
  e <- rnorm(n_subj * 2L, 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + e

  df <- data.frame(y, id, method)

  cfit <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    ci = TRUE
  )

  print(cfit)
})

run_case("Case 2: method + time + vc_select = auto", {
  set.seed(102)
  n_subj <- 60L
  n_time <- 8L

  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))

  sigA <- 0.6
  sigAM <- 0.3
  sigAT <- 0.5
  sigE <- 0.4
  biasB <- 0.2

  u_i <- rnorm(n_subj, 0, sqrt(sigA))
  u <- u_i[as.integer(id)]

  sm <- interaction(id, method, drop = TRUE)
  w_im_lv <- rnorm(nlevels(sm), 0, sqrt(sigAM))
  w_im <- w_im_lv[as.integer(sm)]

  st <- interaction(id, time, drop = TRUE)
  g_it_lv <- rnorm(nlevels(st), 0, sqrt(sigAT))
  g_it <- g_it_lv[as.integer(st)]

  e <- rnorm(length(id), 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + w_im + g_it + e

  dat_both <- data.frame(y, id, method, time)

  fit_both <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "auto",
    verbose = TRUE
  )

  print(fit_both)
})

run_case("Vignette ICC consistency case", {
  set.seed(50)
  n_id <- 14L
  n_time <- 4L

  dat <- expand.grid(
    id = factor(seq_len(n_id)),
    time = factor(seq_len(n_time)),
    method = factor(c("A", "B"))
  )

  dat$time_index <- as.integer(dat$time)

  subj <- rnorm(n_id, sd = 1.0)[dat$id]
  subject_method <- rnorm(n_id * 2L, sd = 0.25)
  sm <- subject_method[(as.integer(dat$id) - 1L) * 2L + as.integer(dat$method)]
  subject_time <- rnorm(n_id * n_time, sd = 0.75)
  st <- subject_time[(as.integer(dat$id) - 1L) * n_time + as.integer(dat$time)]

  dat$y <- subj + sm + st + 0.35 * (dat$method == "B") +
    rnorm(nrow(dat), sd = 0.35)

  fit_icc_cons <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time_index",
    type = "consistency",
    ci = TRUE
  )

  print(fit_icc_cons)
})

run_case("Vignette CCC no-CI time_index case", {
  set.seed(50)
  n_id <- 14L
  n_time <- 4L

  dat <- expand.grid(
    id = factor(seq_len(n_id)),
    time = factor(seq_len(n_time)),
    method = factor(c("A", "B"))
  )

  dat$time_index <- as.integer(dat$time)

  subj <- rnorm(n_id, sd = 1.0)[dat$id]
  subject_method <- rnorm(n_id * 2L, sd = 0.25)
  sm <- subject_method[(as.integer(dat$id) - 1L) * 2L + as.integer(dat$method)]
  subject_time <- rnorm(n_id * n_time, sd = 0.75)
  st <- subject_time[(as.integer(dat$id) - 1L) * n_time + as.integer(dat$time)]

  dat$y <- subj + sm + st + 0.35 * (dat$method == "B") +
    rnorm(nrow(dat), sd = 0.35)

  fit_ccc_reml <- ccc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time_index",
    ci = FALSE
  )

  print(fit_ccc_reml)
})

#' Pairwise Chatterjee Rank Correlation
#'
#' @description
#' Computes the directed Chatterjee rank correlation coefficient for numeric
#' vectors or for all directed column pairs of a numeric matrix/data frame.
#' The matrix orientation is \code{result[i, j] = xi(V_i, V_j)}, where
#' \code{V_i} is the predictor/sorting variable and \code{V_j} is the
#' response/ranked variable.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns, or a numeric predictor vector when \code{y} is supplied. All
#' non-numeric columns will be excluded in matrix/data-frame mode. Each
#' retained column must have at least two non-missing values.
#' @param y Optional numeric response vector for two-vector mode. When supplied,
#'   \code{xi_corr(data, y)} estimates \eqn{\xi(data, y)}, where \code{data}
#'   is the sorting/predictor variable and \code{y} is the response/ranked
#'   variable.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each directed pair on its own pairwise
#'   complete-case overlap. This is permissive, but different directed entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes all
#'   directed entries on the same complete sample.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#'   m-out-of-n bootstrap confidence intervals for the directed Chatterjee
#'   estimates. In two-vector mode the scalar estimate carries a \code{ci}
#'   attribute.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#'   \code{0.95}.
#' @param ci_method Confidence interval method:
#'   \itemize{
#'   \item \code{"auto"} (default): use \code{"dette_kroll"} when the
#'   pair-specific complete-case sample size is less than or equal to
#'   \code{large_sample_cutoff}, and \code{"n_choose_m"} for larger samples.
#'   \item \code{"dette_kroll"}: m-out-of-n variance estimation followed by a
#'   normal interval, following Dette and Kroll.
#'   \item \code{"n_choose_m"}: non-parametric m-out-of-n basic interval based
#'   on the centred and scaled bootstrap statistic, following the large-sample
#'   n-choose-m approach used by Dalitz, Arning and Goebbels.
#'   }
#' @param bootstrap_reps Number of m-out-of-n bootstrap replicates used when
#'   \code{ci = TRUE}. Larger values reduce Monte Carlo noise but increase
#'   runtime.
#' @param m Optional subsample size for the m-out-of-n bootstrap. If
#'   \code{NULL}, method-specific defaults are used: \code{floor(sqrt(n))} for
#'   \code{"dette_kroll"} and \code{round(2 * sqrt(n))} for
#'   \code{"n_choose_m"}, each bounded to \code{[2, n - 1]}.
#' @param large_sample_cutoff For \code{ci_method = "auto"}, use
#'   \code{"n_choose_m"} when pair-specific \code{n_complete} is greater than
#'   this cutoff; otherwise use \code{"dette_kroll"}.
#' @param bias_correction Finite-sample normalisation:
#'   \itemize{
#'   \item \code{"none"} (default): return Chatterjee's original finite-sample
#'   statistic.
#'   \item \code{"upper_bound"}: divide by the raw finite-sample upper-bound
#'   estimate \eqn{\xi_n(Y,Y)} and return
#'   \eqn{\max(-1, \xi_n(X,Y) / \xi_n(Y,Y))}. This can reduce the finite-sample
#'   downward bias of the raw statistic, but it is not the default.
#'   }
#' @param tie_method How ties in the sorting variable \eqn{X} are broken:
#'   \itemize{
#'   \item \code{"random"} (default): randomly permute observations within tied
#'   \eqn{X} groups, matching Chatterjee's random tie-breaking definition.
#'   \item \code{"first"}: preserve stable input order within tied \eqn{X}
#'   groups, useful for deterministic diagnostics and reproducible comparisons.
#'   }
#' @param seed Optional positive integer seed for reproducible tie breaking and
#'   bootstrap resampling.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}. Random tie breaking and
#'   confidence-interval paths are evaluated serially to preserve R RNG
#'   semantics.
#' @param output Output representation for the computed directed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense directed matrix; best when you
#'   need matrix algebra, heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing retained
#'   directed entries; useful when thresholding leaves relatively few values.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, and \code{value}; convenient for filtering,
#'   joins, and graph/network workflows. Because \eqn{\xi(X,Y)} is directed,
#'   both directions are represented when retained.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger directed associations
#'   with \code{output = "sparse"} or \code{"edge_list"}. Must be \code{0}
#'   when \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#' @param ... Compatibility arguments. The deprecated \code{check_na} alias is
#'   accepted.
#'
#' @return
#' A directed numeric matrix where the \code{(i, j)}-th element is
#' Chatterjee's \eqn{\xi} from the \code{i}-th numeric column to the
#' \code{j}-th numeric column. The dense matrix inherits from
#' \code{c("corr_matrix", "chatterjee_xi", "corr_result", "matrix")}. When
#' \code{ci = TRUE}, the object also carries a \code{ci} attribute with
#' elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
#' \code{ci.method}, \code{se}, \code{m}, and \code{bootstrap_reps}. When
#' pairwise-complete evaluation is used, pairwise sample sizes are stored in
#' \code{attr(x, "diagnostics")$n_complete}. In two-vector mode, a numeric
#' scalar is returned; when \code{ci = TRUE}, it carries confidence-interval
#' attributes.
#'
#' @details
#' Let \eqn{(X_k,Y_k), k=1,\ldots,n}, be complete paired observations.
#' Chatterjee's rank correlation is directed: \eqn{\xi(X,Y)} measures how well
#' \eqn{Y} is functionally determined by \eqn{X}. It is not a symmetric
#' association measure, so in matrix mode
#' \deqn{\mathrm{result}_{ij} = \xi(X_{\cdot i}, X_{\cdot j})}
#' need not equal \eqn{\mathrm{result}_{ji}}. The row variable is always the
#' sorting/predictor variable and the column variable is the response/ranked
#' variable.
#'
#' For a given directed pair, observations are sorted by \eqn{X}. Let
#' \eqn{r_i} be the number of response values \eqn{Y_j \le Y_i}, and let
#' \eqn{l_i} be the number of response values \eqn{Y_j \ge Y_i}, evaluated for
#' the \eqn{i}-th observation in sorted-by-\eqn{X} order. The tied-response
#' finite-sample estimator is
#' \deqn{
#' \xi_n = 1 -
#' \frac{n \sum_{i=1}^{n-1} |r_{i+1} - r_i|}
#'      {2 \sum_{i=1}^n l_i (n - l_i)}.
#' }
#' If all response values are equal, the denominator is zero and the estimate
#' is \code{NA}. When there are no ties in \eqn{Y}, this reduces to the familiar
#' rank-difference expression
#' \deqn{
#' \xi_n = 1 -
#' \frac{3 \sum_{i=1}^{n-1}|r_{i+1}-r_i|}{n^2 - 1}.
#' }
#' The raw finite-sample statistic is not forced to one on the diagonal; with
#' no ties and \code{X = Y}, it equals \eqn{(n - 2)/(n + 1)}. Use
#' \code{bias_correction = "upper_bound"} only when this finite-sample
#' upper-bound normalisation is desired.
#'
#' Ties in the sorting variable \eqn{X} are handled before computing adjacent
#' rank differences. Chatterjee's definition uses random tie breaking, provided
#' here by \code{tie_method = "random"}. For reproducible diagnostics or when
#' the input order should define tied-\eqn{X} ordering, use
#' \code{tie_method = "first"}. Ties in the response variable \eqn{Y} are
#' handled by the general \eqn{r_i,l_i} formula above and do not require random
#' tie breaking.
#'
#' \strong{Confidence intervals.} The ordinary n-out-of-n bootstrap is not
#' used for Chatterjee's coefficient. Both available intervals use
#' m-out-of-n subsampling without replacement, consistent with the bootstrap
#' family considered for Chatterjee's rank correlation.
#' With \code{ci_method = "dette_kroll"}, subsamples of size \eqn{m} are drawn
#' without replacement and the limiting standard deviation is estimated as
#' \deqn{\widehat{\sigma} = \sqrt{m}\,\mathrm{sd}(\xi_m^*).}
#' The reported interval is the normal interval
#' \deqn{
#' \xi_n \pm z_{1-\alpha/2}\widehat{\sigma}/\sqrt{n},
#' \quad \alpha = 1-\texttt{conf\_level}.
#' }
#' If \code{m = NULL}, this method uses \eqn{m=\lfloor\sqrt{n}\rfloor}, bounded
#' to \eqn{[2,n-1]}.
#'
#' With \code{ci_method = "n_choose_m"}, subsamples are also drawn without
#' replacement, but the interval is obtained by inverting the empirical
#' quantiles of the centred and scaled statistic
#' \deqn{T^* = \sqrt{m}(\xi_m^* - \xi_n).}
#' If \eqn{q_{\alpha/2}} and \eqn{q_{1-\alpha/2}} are the bootstrap quantiles,
#' the basic m-out-of-n limits are
#' \deqn{
#' \xi_n - q_{1-\alpha/2}/\sqrt{n}
#' \quad\text{and}\quad
#' \xi_n - q_{\alpha/2}/\sqrt{n}.
#' }
#' If finite Monte Carlo quantiles fall entirely on one side of zero, the
#' inversion is anchored at zero so that the reported interval contains the
#' observed estimate; the bounds are not clipped to the population parameter
#' range. If \code{m = NULL}, this method uses
#' \eqn{m=\mathrm{round}(2\sqrt{n})}, bounded to \eqn{[2,n-1]}, following the
#' implementation rule used by Dalitz, Arning and Goebbels.
#'
#' \strong{Which CI method should be used?} The Dette-Kroll interval is the
#' more conservative default for small to moderate complete-case sample sizes
#' in this implementation, because it uses the m-out-of-n bootstrap only to
#' estimate a normal-approximation standard error. The non-parametric
#' n-choose-m interval is useful for larger samples when a direct
#' m-out-of-n bootstrap interval is preferred, but Dalitz, Arning and Goebbels
#' report that this interval may need fairly large \eqn{n} to approach nominal
#' coverage in some settings. Therefore \code{ci_method = "auto"} uses
#' \code{"dette_kroll"} when pair-specific \code{n_complete <=}
#' \code{large_sample_cutoff} and \code{"n_choose_m"} when
#' \code{n_complete > large_sample_cutoff}. Increase
#' \code{bootstrap_reps} for final analyses to reduce Monte Carlo error, and
#' consider setting \code{m} explicitly when a study protocol requires a
#' fixed subsample size.
#'
#' \strong{Computation.} For complete finite matrices without confidence
#' intervals, the C++ backend precomputes each column's sorting order and
#' response ranks and reuses them across directed pairs. This costs
#' \eqn{O(p\,n\log n + p^2 n)} time and \eqn{O(pn + p^2)} memory. Pairwise
#' missing-data evaluation and bootstrap confidence intervals recompute on the
#' relevant complete-case samples and are correspondingly more expensive.
#'
#' @references
#' Chatterjee, S. (2021). A New Coefficient of Correlation. \emph{Journal of
#' the American Statistical Association}, 116, 2009-2022.
#'
#' Dette, H. and Kroll, M. (2025). A simple bootstrap for Chatterjee's rank
#' correlation. \emph{Biometrika}, 112(1), asae045.
#' \doi{10.1093/biomet/asae045}.
#'
#' Lin, Z. and Han, F. (2025). Limit theorems of Chatterjee's rank correlation.
#' arXiv:2204.08031v4. \doi{10.48550/arXiv.2204.08031}.
#'
#' Dalitz, C., Arning, J. and Goebbels, S. (2024). A Simple Bias Reduction for
#' Chatterjee's Correlation.
#'
#' @examples
#' ## Example 1: independence versus functional dependence
#' ## Chatterjee's xi targets whether Y is determined by X.
#' set.seed(1)
#' n <- 300
#' x <- runif(n, -1, 1)
#' y_independent <- rnorm(n)
#' y_function <- sin(2 * pi * x)
#' c(
#'   independent = xi_corr(x, y_independent, tie_method = "first"),
#'   functional = xi_corr(x, y_function, tie_method = "first")
#' )
#'
#' ## Example 2: non-monotone functional dependence is directed
#' ## x determines x^2, but x^2 does not determine the sign of x.
#' ## The reverse raw finite-sample estimate can therefore be much smaller,
#' ## and may be negative when sorted response ranks oscillate strongly.
#' x <- seq(-1, 1, length.out = 300)
#' y <- x^2
#' c(
#'   xi_x_to_y = xi_corr(x, y, tie_method = "first"),
#'   xi_y_to_x = xi_corr(y, x, tie_method = "first")
#' )
#'
#' ## Example 3: the raw finite-sample diagonal is not forced to one
#' ## The optional upper-bound normalisation rescales this finite-sample ceiling.
#' z <- 1:20
#' c(
#'   raw = xi_corr(z, z, tie_method = "first"),
#'   upper_bound = xi_corr(
#'     z, z,
#'     tie_method = "first",
#'     bias_correction = "upper_bound"
#'   )
#' )
#'
#' ## Example 4: directed matrix workflow
#' X <- cbind(
#'   x = x,
#'   square = x^2,
#'   sine = sin(2 * pi * x),
#'   noise = rnorm(length(x))
#' )
#' xi <- xi_corr(X, tie_method = "first")
#' print(xi, digits = 3)
#' summary(xi)
#' estimate(xi)
#' tidy(xi)
#'
#' ## Example 5: Dette-Kroll bootstrap interval for a moderate sample
#' \donttest{
#' xi_ci <- xi_corr(
#'   X[, c("x", "sine")],
#'   ci = TRUE,
#'   ci_method = "dette_kroll",
#'   bootstrap_reps = 49,
#'   seed = 1,
#'   tie_method = "first"
#' )
#' summary(xi_ci)
#' ci(xi_ci)
#' confint(xi_ci)
#' plot(xi_ci)
#' }
#'
#' ## Example 6: n-choose-m interval for larger samples
#' \donttest{
#' set.seed(2)
#' x_large <- runif(1200, -1, 1)
#' y_large <- sin(2 * pi * x_large) + rnorm(1200, sd = 0.2)
#' xi_corr(
#'   x_large,
#'   y_large,
#'   ci = TRUE,
#'   ci_method = "n_choose_m",
#'   bootstrap_reps = 99,
#'   seed = 2,
#'   tie_method = "first"
#' )
#' }
#'
#' @useDynLib matrixCorr, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @author Thiago de Paula Oliveira
#' @export
xi_corr <- function(data,
                    y = NULL,
                    na_method = c("error", "pairwise", "complete"),
                    ci = FALSE,
                    conf_level = 0.95,
                    ci_method = c("auto", "dette_kroll", "n_choose_m"),
                    bootstrap_reps = 999L,
                    m = NULL,
                    large_sample_cutoff = 1000L,
                    bias_correction = c("none", "upper_bound"),
                    tie_method = c("random", "first"),
                    seed = NULL,
                    n_threads = getOption("matrixCorr.threads", 1L),
                    output = c("matrix", "sparse", "edge_list"),
                    threshold = 0,
                    diag = TRUE,
                    ...) {
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )

  if (...length() == 0L && missing(na_method)) {
    na_cfg <- list(na_method = "error", check_na = TRUE)
  } else {
    legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
    na_cfg <- resolve_na_args(
      na_method = na_method,
      check_na = legacy_args$check_na %||% NULL,
      na_method_missing = missing(na_method),
      allowed = c("error", "pairwise", "complete")
    )
  }

  check_bool(ci, arg = "ci")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
    bootstrap_reps <- check_scalar_int_pos(bootstrap_reps, arg = "bootstrap_reps")
    large_sample_cutoff <- check_scalar_int_pos(large_sample_cutoff, arg = "large_sample_cutoff")
    if (!is.null(m)) {
      m <- check_scalar_int_pos(m, arg = "m")
      if (m < 2L) {
        abort_bad_arg("m", message = "must be at least 2.")
      }
    }
  }
  ci_method <- match.arg(ci_method)
  bias_correction <- match.arg(bias_correction)
  tie_method <- match.arg(tie_method)
  if (!is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  eval_seeded <- function(expr) .mc_eval_with_seed(seed, expr)

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "non-matrix output modes are unavailable in two-vector mode."
      )
    }
    if (!is.numeric(data) || !is.numeric(y)) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must be numeric vectors for two-vector mode."
      )
    }
    check_same_length(data, y, arg_x = "data", arg_y = "y")
    data <- as.numeric(data)
    y <- as.numeric(y)
    if (na_cfg$check_na && (any(!is.finite(data)) || any(!is.finite(y)))) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must be free of NA/NaN/Inf when {.arg na_method} is {.val error}.",
        .hint = "Set `na_method = \"pairwise\"` or `na_method = \"complete\"` to use complete paired observations."
      )
    }
    if (!identical(na_cfg$na_method, "error")) {
      keep <- is.finite(data) & is.finite(y)
      if (sum(keep) < 2L) {
        abort_bad_arg(
          "data",
          message = "must retain at least 2 complete paired observations; retained {sum(keep)}."
        )
      }
      data <- data[keep]
      y <- y[keep]
    }

    if (!isTRUE(ci)) {
      return(eval_seeded(chatterjee_xi_vec_cpp(
        data,
        y,
        tie_method = tie_method,
        bias_correction = bias_correction
      )))
    }

    pair <- eval_seeded(chatterjee_xi_matrix_pairwise_cpp(
      cbind(data, y),
      return_ci = TRUE,
      conf_level = conf_level,
      ci_method = ci_method,
      bootstrap_reps = bootstrap_reps,
      m = if (is.null(m)) NULL else as.integer(m),
      large_sample_cutoff = large_sample_cutoff,
      tie_method = tie_method,
      bias_correction = bias_correction,
      n_threads = as.integer(n_threads)
    ))
    out <- as.numeric(pair$est[1L, 2L])
    attr(out, "ci") <- list(
      lwr = as.numeric(pair$lwr[1L, 2L]),
      upr = as.numeric(pair$upr[1L, 2L]),
      conf.level = pair$conf_level,
      ci.method = as.character(pair$ci_method[1L, 2L]),
      se = as.numeric(pair$se[1L, 2L]),
      m = as.integer(pair$m[1L, 2L]),
      bootstrap_reps = as.integer(pair$bootstrap_reps),
      n_complete = as.integer(pair$n_complete[1L, 2L])
    )
    attr(out, "conf.level") <- pair$conf_level
    attr(out, "ci.method") <- as.character(pair$ci_method[1L, 2L])
    class(out) <- c("chatterjee_xi_scalar", "chatterjee_xi", class(out))
    return(out)
  }

  numeric_data <- validate_corr_input(data, check_na = na_cfg$check_na)
  diagnostics_extra <- NULL
  if (identical(na_cfg$na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 2L, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }

  colnames_data <- colnames(numeric_data)
  dn <- if (is.null(colnames_data)) NULL else .mc_square_dimnames(colnames_data)
  diagnostics <- NULL
  ci_attr <- NULL
  extra_attrs <- list()

  if (!identical(na_cfg$na_method, "pairwise") && !isTRUE(ci)) {
    result <- eval_seeded(chatterjee_xi_matrix_cpp(
      numeric_data,
      tie_method = tie_method,
      bias_correction = bias_correction,
      n_threads = as.integer(n_threads)
    ))
  } else {
    pairwise <- eval_seeded(chatterjee_xi_matrix_pairwise_cpp(
      numeric_data,
      return_ci = ci,
      conf_level = conf_level,
      ci_method = ci_method,
      bootstrap_reps = bootstrap_reps,
      m = if (is.null(m)) NULL else as.integer(m),
      large_sample_cutoff = large_sample_cutoff,
      tie_method = tie_method,
      bias_correction = bias_correction,
      n_threads = as.integer(n_threads)
    ))
    result <- pairwise$est
    diagnostics <- list(
      n_complete = .mc_set_matrix_dimnames(pairwise$n_complete, colnames_data)
    )
    if (isTRUE(ci)) {
      ci_method_payload <- pairwise$ci_method
      dimnames(ci_method_payload) <- dn
      se_payload <- .mc_set_matrix_dimnames(pairwise$se, colnames_data)
      m_payload <- .mc_set_matrix_dimnames(pairwise$m, colnames_data)
      diagnostics$ci_method <- ci_method_payload
      diagnostics$m <- m_payload
      diagnostics$bootstrap_reps <- as.integer(pairwise$bootstrap_reps)
      diagnostics$se <- se_payload
      ci_attr <- list(
        est = .mc_set_matrix_dimnames(unclass(result), colnames_data),
        lwr.ci = .mc_set_matrix_dimnames(unclass(pairwise$lwr), colnames_data),
        upr.ci = .mc_set_matrix_dimnames(unclass(pairwise$upr), colnames_data),
        conf.level = pairwise$conf_level,
        ci.method = ci_method_payload,
        se = se_payload,
        m = m_payload,
        bootstrap_reps = as.integer(pairwise$bootstrap_reps)
      )
      extra_attrs$ci.method <- if (identical(ci_method, "auto")) "auto" else ci_method
    }
  }
  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)

  out <- .mc_structure_corr_matrix(
    result,
    class_name = "chatterjee_xi",
    method = "chatterjee_xi",
    description = "Pairwise Chatterjee rank correlation coefficient matrix (directed)",
    symmetric = FALSE,
    diagnostics = diagnostics,
    dimnames = dn,
    extra_attrs = c(
      list(
        bias_correction = bias_correction,
        tie_method = tie_method
      ),
      if (!is.null(ci_attr)) {
        c(
          list(
            ci = ci_attr,
            conf.level = conf_level
          ),
          extra_attrs
        )
      } else {
        list()
      }
    )
  )
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

#' @rdname xi_corr
#' @method print chatterjee_xi
#' @param x An object of class \code{chatterjee_xi}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; \code{NULL}
#'   derives this from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits for confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.chatterjee_xi <- function(x,
                                digits = 4,
                                n = NULL,
                                topn = NULL,
                                max_vars = NULL,
                                width = NULL,
                                ci_digits = 3,
                                show_ci = NULL,
                                ...) {
  .mc_print_corr_matrix(
    x,
    header = "Chatterjee xi directed-dependence matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname xi_corr
#' @method print chatterjee_xi_scalar
#' @export
print.chatterjee_xi_scalar <- function(x,
                                       digits = 4,
                                       ci_digits = 4,
                                       ...) {
  ci <- attr(x, "ci", exact = TRUE)
  cat("Chatterjee xi directed-dependence estimate\n")
  cat("  method      : chatterjee_xi\n")
  cat("  estimate    : ", formatC(unname(as.numeric(x)[1L]), digits = digits, format = "f"), "\n", sep = "")
  if (is.list(ci)) {
    cat("  ci          : yes\n")
    cat("  ci_method   : ", ci$ci.method %||% NA_character_, "\n", sep = "")
    cat("  conf_level  : ", ci$conf.level %||% attr(x, "conf.level", exact = TRUE) %||% NA_real_, "\n", sep = "")
    cat("  n_complete  : ", ci$n_complete %||% NA_integer_, "\n", sep = "")
    cat("  m           : ", ci$m %||% NA_integer_, "\n", sep = "")
    cat("  boot_reps   : ", ci$bootstrap_reps %||% NA_integer_, "\n", sep = "")
    cat(
      "  interval    : [",
      formatC(ci$lwr %||% NA_real_, digits = ci_digits, format = "f"),
      ", ",
      formatC(ci$upr %||% NA_real_, digits = ci_digits, format = "f"),
      "]\n",
      sep = ""
    )
  } else {
    cat("  ci          : no\n")
  }
  invisible(x)
}

#' @rdname xi_corr
#' @method summary chatterjee_xi_scalar
#' @param object An object of class \code{chatterjee_xi_scalar}.
#' @export
summary.chatterjee_xi_scalar <- function(object,
                                         digits = 4,
                                         ci_digits = 4,
                                         ...) {
  ci <- attr(object, "ci", exact = TRUE)
  out <- data.frame(
    estimate = unname(as.numeric(object)[1L]),
    lwr = if (is.list(ci)) ci$lwr %||% NA_real_ else NA_real_,
    upr = if (is.list(ci)) ci$upr %||% NA_real_ else NA_real_,
    conf_level = if (is.list(ci)) ci$conf.level %||% attr(object, "conf.level", exact = TRUE) %||% NA_real_ else NA_real_,
    ci_method = if (is.list(ci)) ci$ci.method %||% NA_character_ else NA_character_,
    se = if (is.list(ci)) ci$se %||% NA_real_ else NA_real_,
    m = if (is.list(ci)) ci$m %||% NA_integer_ else NA_integer_,
    bootstrap_reps = if (is.list(ci)) ci$bootstrap_reps %||% NA_integer_ else NA_integer_,
    n_complete = if (is.list(ci)) ci$n_complete %||% NA_integer_ else NA_integer_,
    stringsAsFactors = FALSE
  )
  class(out) <- c("summary.chatterjee_xi_scalar", "summary.matrixCorr", "data.frame")
  attr(out, "method") <- "chatterjee_xi"
  attr(out, "summary_title") <- "Chatterjee xi directed-dependence estimate"
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  out
}

#' @rdname xi_corr
#' @method print summary.chatterjee_xi_scalar
#' @export
print.summary.chatterjee_xi_scalar <- function(x,
                                               digits = NULL,
                                               ci_digits = NULL,
                                               ...) {
  digits <- digits %||% attr(x, "digits", exact = TRUE) %||% 4L
  ci_digits <- ci_digits %||% attr(x, "ci_digits", exact = TRUE) %||% digits
  cat(attr(x, "summary_title", exact = TRUE) %||% "Chatterjee xi directed-dependence estimate", "\n")
  cat("  method      : chatterjee_xi\n")
  cat("  estimate    : ", formatC(x$estimate[[1L]], digits = digits, format = "f"), "\n", sep = "")
  if (is.finite(x$lwr[[1L]]) || is.finite(x$upr[[1L]])) {
    cat("  ci          : yes\n")
    cat("  ci_method   : ", x$ci_method[[1L]], "\n", sep = "")
    cat("  conf_level  : ", x$conf_level[[1L]], "\n", sep = "")
    cat("  n_complete  : ", x$n_complete[[1L]], "\n", sep = "")
    cat("  m           : ", x$m[[1L]], "\n", sep = "")
    cat("  boot_reps   : ", x$bootstrap_reps[[1L]], "\n", sep = "")
    cat(
      "  interval    : [",
      formatC(x$lwr[[1L]], digits = ci_digits, format = "f"),
      ", ",
      formatC(x$upr[[1L]], digits = ci_digits, format = "f"),
      "]\n",
      sep = ""
    )
    if ("se" %in% names(x) && is.finite(x$se[[1L]])) {
      cat("  se          : ", formatC(x$se[[1L]], digits = ci_digits, format = "f"), "\n", sep = "")
    }
  } else {
    cat("  ci          : no\n")
  }
  invisible(x)
}

manual_xi_no_y_ties <- function(x, y) {
  o <- order(x)
  r <- rank(y, ties.method = "first")
  n <- length(x)
  1 - 3 * sum(abs(diff(r[o]))) / (n^2 - 1)
}

test_that("chatterjee_xi matches deterministic monotone values without y ties", {
  n <- 25
  expected <- (n - 2) / (n + 1)

  expect_equal(
    xi_corr(1:n, 1:n, tie_method = "first"),
    expected,
    tolerance = 1e-12
  )
  expect_equal(
    xi_corr(1:n, n:1, tie_method = "first"),
    expected,
    tolerance = 1e-12
  )
})

test_that("chatterjee_xi captures directed non-monotone deterministic dependence", {
  n <- 401
  x <- seq(-1, 1, length.out = n)
  y <- x^2
  xy <- xi_corr(x, y, tie_method = "first")
  yx <- xi_corr(y, x, tie_method = "first")

  expect_gt(xy, 0.75)
  expect_lt(yx, xy - 0.20)
})

test_that("chatterjee_xi is near zero under independence", {
  set.seed(12)
  x <- rnorm(2000)
  y <- rnorm(2000)
  expect_lt(abs(xi_corr(x, y, tie_method = "first")), 0.08)
})

test_that("chatterjee_xi handles ties and optional bias correction", {
  expect_true(is.na(xi_corr(1:5, rep(1, 5), tie_method = "first")))

  x <- c(1, 1, 1, 2, 2, 3, 3, 3)
  y <- c(2, 4, 1, 3, 5, 8, 7, 6)
  first_1 <- xi_corr(x, y, tie_method = "first")
  first_2 <- xi_corr(x, y, tie_method = "first")
  expect_identical(first_1, first_2)

  random_1 <- xi_corr(x, y, tie_method = "random", seed = 99)
  random_2 <- xi_corr(x, y, tie_method = "random", seed = 99)
  expect_identical(random_1, random_2)

  n <- 20
  expected <- (n - 2) / (n + 1)
  expect_equal(
    xi_corr(1:n, 1:n, tie_method = "first", bias_correction = "none"),
    expected,
    tolerance = 1e-12
  )
  expect_equal(
    xi_corr(1:n, 1:n, tie_method = "first", bias_correction = "upper_bound"),
    1,
    tolerance = 1e-12
  )
})

test_that("chatterjee_xi matrix orientation is row predictor and column response", {
  x <- seq(-1, 1, length.out = 101)
  X <- cbind(x = x, square = x^2, reverse = rev(x))
  fit <- xi_corr(X, tie_method = "first")

  expect_equal(
    unname(fit["x", "square"]),
    xi_corr(X[, "x"], X[, "square"], tie_method = "first"),
    tolerance = 1e-12
  )
  expect_equal(
    unname(fit["square", "x"]),
    xi_corr(X[, "square"], X[, "x"], tie_method = "first"),
    tolerance = 1e-12
  )
  expect_false(isSymmetric(unclass(fit)))
  expect_false(isTRUE(attr(fit, "corr_symmetric", exact = TRUE)))
})

test_that("chatterjee_xi missing-data modes follow package contracts", {
  X <- cbind(
    a = c(1, 2, 3, 4, NA, 6),
    b = c(1, 4, 9, 16, 25, 36),
    c = c(6, 5, Inf, 3, 2, 1)
  )

  expect_error(xi_corr(X, na_method = "error"), "Missing values|NA|Inf")

  complete <- xi_corr(X, na_method = "complete", tie_method = "first")
  d_complete <- attr(complete, "diagnostics", exact = TRUE)
  expect_identical(d_complete$na_method, "complete")
  expect_identical(d_complete$n_complete, 4L)

  pairwise <- xi_corr(X, na_method = "pairwise", tie_method = "first")
  d_pairwise <- attr(pairwise, "diagnostics", exact = TRUE)
  expect_true(is.matrix(d_pairwise$n_complete))
  expect_identical(unname(d_pairwise$n_complete["a", "b"]), 5L)
  expect_identical(unname(d_pairwise$n_complete["a", "c"]), 4L)
})

test_that("chatterjee_xi confidence intervals are finite and auto-select methods", {
  set.seed(42)
  X <- cbind(a = rnorm(80), b = rnorm(80), c = rnorm(80))
  fit <- xi_corr(
    X,
    ci = TRUE,
    ci_method = "dette_kroll",
    bootstrap_reps = 19,
    seed = 1,
    tie_method = "first"
  )
  ci <- attr(fit, "ci", exact = TRUE)
  expect_true(is.finite(ci$lwr.ci["a", "b"]))
  expect_true(is.finite(ci$upr.ci["a", "b"]))
  expect_identical(as.character(ci$ci.method["a", "b"]), "dette_kroll")

  big <- cbind(a = rnorm(1001), b = rnorm(1001))
  fit_big <- xi_corr(
    big,
    ci = TRUE,
    ci_method = "n_choose_m",
    bootstrap_reps = 9,
    seed = 2,
    tie_method = "first"
  )
  ci_big <- attr(fit_big, "ci", exact = TRUE)
  expect_true(is.finite(ci_big$lwr.ci["a", "b"]))
  expect_true(is.finite(ci_big$upr.ci["a", "b"]))
  expect_lte(ci_big$lwr.ci["a", "b"], fit_big["a", "b"])
  expect_gte(ci_big$upr.ci["a", "b"], fit_big["a", "b"])
  expect_identical(as.character(ci_big$ci.method["a", "b"]), "n_choose_m")

  set.seed(1299)
  x <- runif(1200, -1, 1)
  y <- sin(2 * pi * x) + rnorm(1200, sd = 0.2)
  scalar_big <- xi_corr(
    x,
    y,
    ci = TRUE,
    ci_method = "n_choose_m",
    bootstrap_reps = 19,
    seed = 123,
    tie_method = "first"
  )
  scalar_ci <- attr(scalar_big, "ci", exact = TRUE)
  scalar_est <- unname(as.numeric(scalar_big))
  expect_lte(scalar_ci$lwr, scalar_est)
  expect_gte(scalar_ci$upr, scalar_est)
  expect_s3_class(scalar_big, "chatterjee_xi_scalar")
  scalar_print <- capture.output(print(scalar_big))
  expect_true(any(grepl("Chatterjee xi directed-dependence estimate", scalar_print, fixed = TRUE)))
  expect_true(any(grepl("interval", scalar_print, fixed = TRUE)))
  scalar_summary <- summary(scalar_big)
  expect_s3_class(scalar_summary, "summary.chatterjee_xi_scalar")
  expect_equal(scalar_summary$estimate, scalar_est, tolerance = 1e-12)
  expect_equal(scalar_summary$lwr, scalar_ci$lwr, tolerance = 1e-12)
  summary_print <- capture.output(print(scalar_summary))
  expect_true(any(grepl("Chatterjee xi directed-dependence estimate", summary_print, fixed = TRUE)))
  expect_true(any(grepl("ci_method", summary_print, fixed = TRUE)))

  small_auto <- xi_corr(
    cbind(a = rnorm(1000), b = rnorm(1000)),
    ci = TRUE,
    ci_method = "auto",
    bootstrap_reps = 9,
    seed = 3,
    tie_method = "first"
  )
  large_auto <- xi_corr(
    cbind(a = rnorm(1001), b = rnorm(1001)),
    ci = TRUE,
    ci_method = "auto",
    bootstrap_reps = 9,
    seed = 3,
    tie_method = "first"
  )
  expect_identical(as.character(attr(small_auto, "ci")$ci.method["a", "b"]), "dette_kroll")
  expect_identical(as.character(attr(large_auto, "ci")$ci.method["a", "b"]), "n_choose_m")

  rep1 <- xi_corr(X, ci = TRUE, bootstrap_reps = 9, seed = 10, tie_method = "first")
  rep2 <- xi_corr(X, ci = TRUE, bootstrap_reps = 9, seed = 10, tie_method = "first")
  expect_equal(attr(rep1, "ci", exact = TRUE), attr(rep2, "ci", exact = TRUE))

  sm <- summary(fit)
  expect_s3_class(sm, "summary.chatterjee_xi")
  expect_true(all(c("lwr", "upr", "ci_method", "m", "se", "bootstrap_reps") %in% names(sm)))
  expect_identical(attr(sm, "ci_method", exact = TRUE), "dette_kroll")
})

test_that("chatterjee_xi output classes and directed output modes are correct", {
  x <- seq(-1, 1, length.out = 51)
  X <- cbind(a = x, b = x^2, c = rev(x))
  dense <- xi_corr(X, tie_method = "first")
  mat <- as.matrix(dense)

  expect_s3_class(dense, "corr_matrix")
  expect_s3_class(dense, "chatterjee_xi")
  expect_identical(attr(dense, "method", exact = TRUE), "chatterjee_xi")
  expect_false(isTRUE(attr(dense, "corr_symmetric", exact = TRUE)))

  edge <- xi_corr(X, tie_method = "first", output = "edge_list", threshold = 0, diag = FALSE)
  expect_s3_class(edge, "corr_edge_list")
  edge_df <- as.data.frame(edge, stringsAsFactors = FALSE)
  expect_equal(nrow(edge_df), ncol(X) * (ncol(X) - 1L))
  expect_equal(
    edge_df$value[edge_df$row == "b" & edge_df$col == "a"],
    unname(mat["b", "a"])
  )

  sparse <- xi_corr(X, tie_method = "first", output = "sparse", threshold = 0, diag = FALSE)
  expect_s4_class(sparse, "sparseMatrix")
  expect_false(isTRUE(attr(sparse, "corr_symmetric", exact = TRUE)))
  expect_equal(as.matrix(sparse)["a", "b"], mat["a", "b"], tolerance = 1e-12)
  expect_equal(as.matrix(sparse)["b", "a"], mat["b", "a"], tolerance = 1e-12)
})

test_that("estimate returns a plain numeric matrix for dense correlation results", {
  set.seed(101)
  X <- cbind(
    a = rnorm(30),
    b = rnorm(30),
    c = rnorm(30)
  )

  fit <- pearson_corr(X, ci = TRUE)
  est <- estimate(fit)

  expect_true(is.matrix(est))
  expect_type(est, "double")
  expect_false(inherits(est, "corr_matrix"))
  expect_equal(est, unclass(as.matrix(fit)), tolerance = 1e-12, ignore_attr = TRUE)
  expect_equal(dim(est), dim(fit))
  expect_equal(dimnames(est), dimnames(fit))
  expect_equal(coef(fit), est, tolerance = 1e-12)
})

test_that("tidy returns unique upper-triangle pairs by default", {
  set.seed(102)
  X <- cbind(
    a = rnorm(30),
    b = rnorm(30),
    c = rnorm(30)
  )

  fit <- pearson_corr(X, ci = TRUE)
  td <- tidy(fit)

  expect_s3_class(td, "data.frame")
  expect_true(all(c("item1", "item2", "estimate") %in% names(td)))
  expect_false(any(td$item1 == td$item2))
  expect_equal(nrow(td), choose(ncol(X), 2L))
  expect_false(any(duplicated(paste(pmin(td$item1, td$item2), pmax(td$item1, td$item2)))))
})

test_that("tidy triangle and diagonal controls are predictable", {
  set.seed(103)
  X <- cbind(
    a = rnorm(20),
    b = rnorm(20),
    c = rnorm(20)
  )

  fit <- pearson_corr(X, ci = TRUE)
  p <- ncol(X)

  expect_equal(nrow(tidy(fit)), choose(p, 2L))
  expect_equal(nrow(tidy(fit, diag = TRUE)), choose(p, 2L) + p)
  expect_equal(nrow(tidy(fit, triangle = "full")), p * (p - 1L))
  expect_equal(nrow(tidy(fit, triangle = "full", diag = TRUE)), p * p)
  expect_true(any(tidy(fit, diag = TRUE)$item1 == tidy(fit, diag = TRUE)$item2))
})

test_that("confint and ci expose confidence intervals cleanly", {
  set.seed(104)
  X <- cbind(
    a = rnorm(30),
    b = rnorm(30),
    c = rnorm(30)
  )

  fit <- pearson_corr(X, ci = TRUE, conf_level = 0.95)
  ci_df <- confint(fit)

  expect_identical(ci(fit), attr(fit, "ci", exact = TRUE))
  expect_s3_class(ci_df, "data.frame")
  expect_true(all(c("item1", "item2", "lwr", "upr") %in% names(ci_df)))
  expect_false("conf.level" %in% names(ci_df))
  expect_false("conf_level" %in% names(ci_df))
  expect_false("ci.method" %in% names(ci_df))
  expect_no_error(confint(fit, level = attr(fit, "conf.level", exact = TRUE)))
  expect_error(
    confint(fit, level = 0.90),
    "This object contains pre-computed confidence intervals at conf.level"
  )
})

test_that("ci and confint fail clearly when intervals are absent", {
  set.seed(105)
  X <- cbind(
    a = rnorm(20),
    b = rnorm(20),
    c = rnorm(20)
  )

  fit <- pearson_corr(X, ci = FALSE)

  expect_error(ci(fit), "No confidence intervals are available for this object.")
  expect_error(confint(fit), "does not contain confidence limits")
})

test_that("estimate handles edge-list outputs predictably", {
  set.seed(106)
  X <- cbind(
    a = rnorm(20),
    b = rnorm(20),
    c = rnorm(20)
  )

  fit <- pearson_corr(X, output = "edge_list")
  est <- estimate(fit)

  expect_s3_class(est, "data.frame")
  expect_equal(names(est), c("item1", "item2", "estimate"))
  expect_false(inherits(est, "corr_edge_list"))
})

test_that("generic tidy preserves directed matrix orientation", {
  x <- seq(-1, 1, length.out = 80)
  X <- cbind(x = x, x2 = x^2)

  fit <- xi_corr(X, tie_method = "first", ci = FALSE)
  td <- tidy(fit)

  expect_equal(estimate(fit), unclass(as.matrix(fit)), tolerance = 1e-12, ignore_attr = TRUE)
  expect_true(all(c("x", "x2") %in% td$item1))
  expect_true(all(c("x", "x2") %in% td$item2))
  expect_equal(
    td$estimate[td$item1 == "x" & td$item2 == "x2"],
    unname(fit["x", "x2"]),
    tolerance = 1e-12
  )
  expect_equal(
    td$estimate[td$item1 == "x2" & td$item2 == "x"],
    unname(fit["x2", "x"]),
    tolerance = 1e-12
  )
})

test_that("generic accessors work for scalar CI results", {
  set.seed(107)
  x <- runif(120)
  y <- sin(2 * pi * x) + rnorm(120, sd = 0.2)

  fit <- xi_corr(
    x,
    y,
    ci = TRUE,
    ci_method = "dette_kroll",
    bootstrap_reps = 9,
    seed = 107,
    tie_method = "first"
  )

  td <- tidy(fit)
  ci_df <- confint(fit)

  expect_equal(estimate(fit), unname(as.numeric(fit)), tolerance = 1e-12)
  expect_true(all(c("estimate", "lwr", "upr", "m", "bootstrap_reps") %in% names(td)))
  expect_false("ci.method" %in% names(td))
  expect_false("n_complete" %in% names(td))
  expect_true(all(c("lwr", "upr") %in% names(ci_df)))
  expect_false("conf.level" %in% names(ci_df))
  expect_false("ci.method" %in% names(ci_df))
  expect_false("conf_level" %in% names(ci_df))
  expect_false("ci_method" %in% names(ci_df))
  expect_equal(ci_df$lwr, td$lwr, tolerance = 1e-12)
  expect_equal(ci_df$upr, td$upr, tolerance = 1e-12)
})

test_that("generic accessors work for scalar agreement results", {
  ratings <- data.frame(
    r1 = c(1, 1, 2, 2, 3, 3, 1, 2),
    r2 = c(1, 2, 2, 2, 3, 1, 1, 2)
  )

  fit <- cohen_kappa(ratings$r1, ratings$r2, ci = TRUE)
  td <- tidy(fit)
  ci_df <- confint(fit)

  expect_equal(estimate(fit), unname(as.numeric(fit)), tolerance = 1e-12)
  expect_true(all(c("estimate", "lwr", "upr") %in% names(td)))
  expect_true(all(c("lwr", "upr") %in% names(ci_df)))
})

test_that("generic accessors work for overall CIA results", {
  set.seed(110)
  subjects <- sprintf("s%02d", 1:10)
  methods <- c("A", "B", "C")
  replicates <- sprintf("r%02d", 1:4)
  dat <- expand.grid(
    subject = subjects,
    method = methods,
    replicate = replicates,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  subject_effect <- stats::rnorm(length(subjects), sd = 3)
  method_shift <- c(A = 0, B = 0.15, C = -0.10)
  dat$value <- subject_effect[match(dat$subject, subjects)] +
    method_shift[dat$method] +
    stats::rnorm(nrow(dat), sd = 0.4)

  fit <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "overall"
  )
  td <- tidy(fit)

  expect_equal(estimate(fit), fit$cia, tolerance = 1e-12)
  expect_equal(estimate(summary(fit)), fit$cia, tolerance = 1e-12)
  expect_true(all(c("estimate", "cia_raw", "tau2") %in% names(td)))
  expect_equal(td$estimate, fit$cia, tolerance = 1e-12)
})

test_that("generic tidy handles list-style CI outputs", {
  X <- cbind(
    a = c(1, 2, 3, 4, 5, 6),
    b = c(1.1, 2.1, 2.9, 4.2, 4.9, 6.1),
    c = c(6, 5, 4, 3, 2, 1)
  )

  fit <- icc(X, ci = TRUE)
  est <- estimate(fit)
  td <- tidy(fit)
  ci_df <- confint(fit)

  expect_true(is.matrix(est))
  expect_false(inherits(est, "icc"))
  expect_equal(dim(est), dim(fit$est))
  expect_equal(dimnames(est), dimnames(fit$est))
  expect_true(all(c("item1", "item2", "estimate") %in% names(td)))
  expect_true(all(c("item1", "item2", "lwr", "upr") %in% names(ci_df)))
  expect_false("n_complete" %in% names(td))
  expect_false(any(td$item1 == td$item2))
  expect_equal(nrow(td), choose(ncol(X), 2L))
})

test_that("estimate preserves matrix shape for matrixCorr matrix-like classes", {
  X <- cbind(
    m1 = c(1, 2, 3, 4, 5, 6),
    m2 = c(1.1, 2.1, 2.9, 4.2, 4.9, 6.1),
    m3 = c(6, 5, 4, 3, 2, 1)
  )

  fit <- icc(
    X,
    model = "twoway_random",
    type = "agreement",
    unit = "single",
    scope = "pairwise"
  )
  est <- estimate(fit)

  expect_true(is.matrix(est))
  expect_false(inherits(est, "icc"))
  expect_equal(dim(est), dim(fit))
  expect_equal(dimnames(est), dimnames(fit))
})

test_that("ccc_glmm accessors return matrices and CI payloads", {
  set.seed(109)
  n_subjects <- 10L
  reps <- 2L
  dat <- data.frame(
    subject = factor(rep(seq_len(n_subjects), each = 2L * reps)),
    method = factor(rep(rep(c("A", "B"), each = reps), times = n_subjects)),
    replicate = factor(rep(seq_len(reps), times = 2L * n_subjects))
  )
  subject_eff <- rnorm(n_subjects, 0, 0.5)
  dat$eta <- 1.1 + subject_eff[as.integer(dat$subject)] +
    ifelse(dat$method == "B", 0.1, 0)
  dat$y <- rpois(nrow(dat), exp(dat$eta))

  fit <- ccc_glmm(
    dat,
    "y",
    "subject",
    "method",
    replicate = "replicate",
    ci = TRUE
  )
  est <- estimate(fit)
  payload <- ci(fit)

  expect_true(is.matrix(est))
  expect_false(inherits(est, "ccc_glmm"))
  expect_equal(dim(est), dim(fit))
  expect_equal(dimnames(est), dimnames(fit))
  expect_equal(coef(fit), est, tolerance = 1e-12)
  expect_true(is.list(payload))
  expect_false(is.logical(payload))
  expect_true(all(c("lwr", "upr", "se", "total", "inter", "intra_method1", "intra_method2") %in% names(payload)))
  expect_true(is.matrix(payload$lwr))
  expect_true(is.matrix(payload$upr))
  expect_equal(dim(payload$lwr), dim(fit))
  expect_equal(dim(payload$upr), dim(fit))
})

test_that("ci works for agreement list and Bland-Altman matrix outputs", {
  set.seed(108)
  Sigma <- matrix(c(1, 0.5, 0.3, 0.5, 1, 0.4, 0.3, 0.4, 1), nrow = 3)
  X <- MASS::mvrnorm(n = 60, mu = rep(0, 3), Sigma = Sigma)

  ccc_fit <- ccc(X, ci = TRUE)
  ccc_ci <- ci(ccc_fit)
  ccc_td <- tidy(ccc_fit)
  ccc_limits <- confint(ccc_fit)

  expect_true(all(c("lwr", "upr") %in% names(ccc_ci)))
  expect_false(any(ccc_td$item1 == ccc_td$item2))
  expect_equal(nrow(ccc_td), choose(ncol(X), 2L))
  expect_equal(nrow(ccc_limits), choose(ncol(X), 2L))

  wide <- data.frame(
    ref = rnorm(50, 100, 8),
    m2 = rnorm(50, 101, 8),
    m3 = rnorm(50, 99, 9)
  )
  ba_fit <- ba(wide)
  ba_ci <- ci(ba_fit)
  ba_limits <- confint(ba_fit)

  expect_true(all(c("bias", "loa_low", "loa_up", "conf.level") %in% names(ba_ci)))
  expect_true(all(c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr") %in% names(ba_limits)))
  expect_false("conf.level" %in% names(ba_limits))
})

#' @title Bland-Altman for repeated measurements
#' @description
#' Repeated-measures Bland-Altman (BA) analysis for method comparison based on a
#' mixed-effects model fitted to **subject-time matched paired differences**.
#' The fitted model includes a subject-specific random intercept and, optionally,
#' an AR(1) residual correlation structure within subject.
#'
#' The function accepts either exactly two methods or \eqn{\ge 3} methods.
#' With exactly two methods it returns a single fitted BA object. With
#' \eqn{\ge 3} methods it fits the same model to every unordered method pair and
#' returns pairwise matrices of results.
#'
#' **Required variables**
#' \itemize{
#'   \item \code{response}: numeric measurements.
#'   \item \code{subject}: subject identifier.
#'   \item \code{method}: method label with at least two distinct levels.
#'   \item \code{time}: replicate/time key used to form within-subject pairs.
#' }
#'
#' For any analysed pair of methods, only records where both methods are present
#' for the same \code{subject} and integer-coerced \code{time} contribute to the
#' fit. Rows with missing values in any required field are excluded for that
#' analysed pair.
#'
#' @param data Optional data frame-like object. If supplied, `response`,
#'   `subject`, `method`, and `time` may be column names. Objects not already
#'   inheriting from `data.frame` are first coerced with `as.data.frame()`.
#' @param response Numeric response vector, or a single character string naming
#'   the response column in `data`.
#' @param subject Subject identifier vector (integer, numeric, or factor), or a
#'   single character string naming the subject column in `data`.
#' @param method Method label vector (character, factor, integer, or numeric), or
#'   a single character string naming the method column in `data`. At least two
#'   distinct method levels are required.
#' @param time Replicate/time index vector (integer or numeric), or a single
#'   character string naming the time column in `data`. Values are coerced to
#'   integer before pairing and before AR(1) contiguity checks.
#' @param loa_multiplier Positive scalar giving the SD multiplier used to form
#'   the limits of agreement. Default is `1.96`. In the exported `ba_rm()`
#'   interface this default is fixed and does not depend on `conf_level`.
#' @param conf_level Confidence level for Wald confidence intervals for the
#'   reported bias and both LoA endpoints. Must lie in `(0, 1)`. Default `0.95`.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param include_slope Logical. If `TRUE`, the model includes the paired mean as
#'   a fixed effect and estimates a proportional-bias slope. The reported BA
#'   centre remains the fitted mean difference at the centred reference paired
#'   mean used internally by the backend; the returned LoA remain horizontal
#'   bands and are not regression-adjusted curves.
#' @param use_ar1 Logical. If `TRUE`, request an AR(1) residual structure within
#'   subject over contiguous integer time blocks.
#' @param ar1_rho Optional AR(1) parameter. Must satisfy `abs(ar1_rho) < 0.999`
#'   when supplied. If `NA` and `use_ar1 = TRUE`, the backend estimates `rho`
#'   separately for each analysed pair.
#' @param max_iter Maximum number of EM/GLS iterations used by the backend.
#' @param tol Convergence tolerance for the backend EM/GLS iterations.
#' @param verbose Logical. If `TRUE`, print progress for the pairwise
#'   \eqn{\ge 3}-method path. It has no material effect in the exactly-two-method
#'   path.
#'
#' @return
#' Either a \code{"ba_repeated"} object (exactly two methods) or a
#' \code{"ba_repeated_matrix"} object (\eqn{\ge 3} methods).
#'
#' \strong{If exactly two methods are supplied}, the returned
#' \code{"ba_repeated"} object is a list with components:
#' \itemize{
#'   \item \code{means}: numeric vector of paired means
#'   \eqn{(y_1 + y_2)/2} used for plotting helpers.
#'   \item \code{diffs}: numeric vector of paired differences
#'   \eqn{y_2 - y_1} used for plotting helpers.
#'   \item \code{n_obs}: integer number of complete subject-time pairs used.
#'   \item \code{based.on}: compatibility alias for \code{n_obs}.
#'   \item \code{mean.diffs}: scalar model-based BA centre. When
#'   `include_slope = FALSE`, this is the fitted intercept of the paired-
#'   difference model. When `include_slope = TRUE`, this is the fitted mean
#'   difference at the centred reference paired mean used internally by the
#'   backend.
#'   \item \code{lower.limit}, \code{upper.limit}: scalar limits of agreement,
#'   computed as
#'   \eqn{\mu_0 \pm \texttt{loa\_multiplier}\sqrt{\sigma_u^2 + \sigma_e^2}}.
#'   \item \code{lines}: named numeric vector with entries `lower`, `mean`, and
#'   `upper`.
#'   \item \code{CI.lines}: named numeric vector containing Wald confidence
#'   interval bounds for the bias and both LoA endpoints:
#'   `mean.diff.ci.lower`, `mean.diff.ci.upper`,
#'   `lower.limit.ci.lower`, `lower.limit.ci.upper`,
#'   `upper.limit.ci.lower`, `upper.limit.ci.upper`.
#'   \item \code{loa_multiplier}: scalar LoA multiplier actually used.
#'   \item \code{critical.diff}: scalar LoA half-width
#'   \eqn{\texttt{loa\_multiplier} \times \texttt{sd\_loa}}.
#'   \item \code{include_slope}: logical, copied from the call.
#'   \item \code{beta_slope}: proportional-bias slope on the original paired-mean
#'   scale when `include_slope = TRUE`; otherwise `NA`.
#'   \item \code{sigma2_subject}: estimated variance of the subject-level random
#'   intercept on paired differences.
#'   \item \code{sigma2_resid}: estimated residual variance on paired differences.
#'   \item \code{use_ar1}: logical, copied from the call.
#'   \item \code{residual_model}: either `"ar1"` or `"iid"`, indicating the final
#'   residual structure actually used.
#'   \item \code{ar1_rho}: AR(1) correlation actually used in the final fit when
#'   `residual_model == "ar1"`; otherwise `NA`.
#'   \item \code{ar1_estimated}: logical indicating whether `ar1_rho` was
#'   estimated internally (`TRUE`) or supplied by the user (`FALSE`) when the
#'   final residual model is AR(1); otherwise `NA`.
#' }
#'
#' The confidence level is stored as `attr(x, "conf.level")`.
#'
#' \strong{If \eqn{\ge 3} methods are supplied}, the returned
#' \code{"ba_repeated_matrix"} object is a list with components:
#' \itemize{
#'   \item \code{bias}: numeric \eqn{m \times m} matrix of model-based BA centres.
#'   For indices \eqn{(j, k)} with \eqn{j < k}, \code{bias[j, k]} estimates
#'   \eqn{\mathrm{method}_k - \mathrm{method}_j}. Thus the matrix orientation is
#'   **column minus row**, not row minus column. The diagonal is `NA`.
#'   \item \code{sd_loa}: numeric \eqn{m \times m} matrix of LoA SDs,
#'   \eqn{\sqrt{\sigma_u^2 + \sigma_e^2}}. This matrix is symmetric.
#'   \item \code{loa_lower}, \code{loa_upper}: numeric \eqn{m \times m} matrices
#'   of LoA endpoints corresponding to `bias`. These satisfy
#'   \eqn{\texttt{loa\_lower}[j,k] = -\texttt{loa\_upper}[k,j]} and
#'   \eqn{\texttt{loa\_upper}[j,k] = -\texttt{loa\_lower}[k,j]}.
#'   \item \code{width}: numeric \eqn{m \times m} matrix of LoA widths,
#'   \code{loa_upper - loa_lower}. This matrix is symmetric.
#'   \item \code{n}: integer \eqn{m \times m} matrix giving the number of complete
#'   subject-time pairs used for each analysed contrast. Pairs with fewer than
#'   two complete matches are left as `NA` in the estimate matrices.
#'   \item \code{mean_ci_low}, \code{mean_ci_high}: numeric \eqn{m \times m}
#'   matrices of Wald confidence interval bounds for `bias`.
#'   \item \code{loa_lower_ci_low}, \code{loa_lower_ci_high}: numeric
#'   \eqn{m \times m} matrices of Wald confidence interval bounds for the lower
#'   LoA.
#'   \item \code{loa_upper_ci_low}, \code{loa_upper_ci_high}: numeric
#'   \eqn{m \times m} matrices of Wald confidence interval bounds for the upper
#'   LoA.
#'   \item \code{slope}: optional numeric \eqn{m \times m} matrix of
#'   proportional-bias slopes on the original paired-mean scale when
#'   `include_slope = TRUE`; otherwise `NULL`. This matrix is antisymmetric in
#'   sign because each fitted contrast is reversed across the transpose.
#'   \item \code{methods}: character vector of method levels defining matrix row
#'   and column order.
#'   \item \code{loa_multiplier}: scalar LoA multiplier actually used.
#'   \item \code{conf_level}: scalar confidence level used for the reported Wald
#'   intervals.
#'   \item \code{use_ar1}: logical, copied from the call.
#'   \item \code{ar1_rho}: scalar equal to the user-supplied common `ar1_rho`
#'   when `use_ar1 = TRUE` and a value was supplied; otherwise `NA`. This field
#'   does \emph{not} store the per-pair estimated AR(1) parameters.
#'   \item \code{residual_model}: character \eqn{m \times m} matrix whose entries
#'   are `"ar1"`, `"iid"`, or `NA`, indicating the final residual structure used
#'   for each pair.
#'   \item \code{sigma2_subject}: numeric \eqn{m \times m} matrix of estimated
#'   subject-level random-intercept variances.
#'   \item \code{sigma2_resid}: numeric \eqn{m \times m} matrix of estimated
#'   residual variances.
#'   \item \code{ar1_rho_pair}: optional numeric \eqn{m \times m} matrix giving
#'   the AR(1) correlation actually used for each pair when the final residual
#'   model is `"ar1"`; otherwise `NA` for that entry. Present only when
#'   `use_ar1 = TRUE`.
#'   \item \code{ar1_estimated}: optional logical \eqn{m \times m} matrix
#'   indicating whether the pair-specific `ar1_rho_pair` was estimated internally
#'   (`TRUE`) or supplied by the user (`FALSE`) for entries whose final residual
#'   model is `"ar1"`; otherwise `NA`. Present only when `use_ar1 = TRUE`.
#'   \item \code{data_long}: canonical long-form data frame with columns
#'   \code{.response}, \code{.subject}, \code{.method}, and \code{.time}
#'   retained for pairwise plotting helpers.
#'   \item \code{mapping}: named list mapping \code{response}, \code{subject},
#'   \code{method}, and \code{time} to the canonical stored columns in
#'   \code{data_long}.
#' }
#'
#' @details
#' For a selected pair of methods \eqn{(a, b)}, the backend first forms complete
#' within-subject pairs at matched \code{subject} and integer-coerced
#' \code{time}. Let
#' \deqn{
#' d_{it} = y_{itb} - y_{ita},
#' \qquad
#' m_{it} = \frac{y_{ita} + y_{itb}}{2},
#' }
#' where \eqn{d_{it}} is the paired difference and \eqn{m_{it}} is the paired
#' mean for subject \eqn{i} at time/replicate \eqn{t}. Only complete
#' subject-time matches contribute to that pairwise fit.
#'
#' If multiple rows are present for the same `subject`-`time`-`method`
#' combination within an analysed pair, the backend keeps the last encountered
#' value for that combination when forming the pair. The function therefore
#' implicitly assumes at most one observation per `subject`-`time`-`method`
#' cell for each analysed contrast.
#'
#' The fitted model for each analysed pair is
#' \deqn{
#' d_{it} = \beta_0 + \beta_1 x_{it} + u_i + \varepsilon_{it},
#' }
#' where \eqn{x_{it} = m_{it}} if `include_slope = TRUE` and the term is omitted
#' otherwise; \eqn{u_i \sim \mathcal{N}(0, \sigma_u^2)} is a subject-specific
#' random intercept; and the within-subject residual vector satisfies
#' \eqn{\mathrm{Cov}(\varepsilon_i) = \sigma_e^2 R_i}.
#'
#' When `use_ar1 = FALSE`, \eqn{R_i = I}. When `use_ar1 = TRUE`, the backend
#' works with the residual \emph{precision} matrix \eqn{C_i = R_i^{-1}} over
#' contiguous time blocks within subject and uses \eqn{\sigma_e^2 C_i^{-1}} as
#' the residual covariance.
#'
#' \subsection{AR(1) residual structure}{
#' Within each subject, paired observations are ordered by integer-coerced
#' `time`. AR(1) correlation is applied only over strictly contiguous runs
#' satisfying \eqn{t_{k+1} = t_k + 1}. Gaps break the run. Negative times, and
#' any isolated positions not belonging to a contiguous run, are treated as
#' independent singletons.
#'
#' For a contiguous run of length \eqn{L} and correlation parameter \eqn{\rho},
#' the block precision matrix is
#' \deqn{
#' C
#' =
#' \frac{1}{1-\rho^2}
#' \begin{bmatrix}
#' 1      & -\rho &        &        &  \\
#' -\rho  & 1+\rho^2 & -\rho &      &  \\
#'         & \ddots & \ddots & \ddots & \\
#'         &        & -\rho & 1+\rho^2 & -\rho \\
#'         &        &       & -\rho & 1
#' \end{bmatrix},
#' }
#' with a very small ridge added to the diagonal for numerical stability.
#'
#' If `use_ar1 = TRUE` and `ar1_rho` is supplied, that value is used after
#' validation and clipping to the admissible numerical range handled by the
#' backend.
#'
#' If `use_ar1 = TRUE` and `ar1_rho = NA`, the backend estimates `rho`
#' separately for each analysed pair by:
#' \enumerate{
#'   \item fitting the corresponding iid model;
#'   \item computing a moments-based lag-1 estimate from detrended residuals
#'   within contiguous blocks, used only as a seed; and
#'   \item refining that seed by a short profile search over `rho` using the
#'   profiled REML log-likelihood.
#' }
#'
#' In the exported `ba_rm()` wrapper, if an AR(1) fit for a given analysed pair
#' fails specifically because the backend EM/GLS routine did not converge to
#' admissible finite variance-component estimates, the wrapper retries that pair
#' with iid residuals. If the iid refit succeeds, the final reported residual
#' model for that pair is `"iid"` and a warning is issued. Other AR(1) failures
#' are not simplified and are propagated as errors.
#' }
#'
#' \subsection{Internal centring and scaling for the proportional-bias slope}{
#' When `include_slope = TRUE`, the paired mean regressor is centred and scaled
#' internally before fitting. Let \eqn{\bar m} be the mean of the observed paired
#' means. The backend chooses a scaling denominator from:
#' \itemize{
#'   \item the sample SD;
#'   \item the IQR-based scale \eqn{\mathrm{IQR}(m)/1.349};
#'   \item the MAD-based scale \eqn{1.4826\,\mathrm{MAD}(m)}.
#' }
#' It uses the first of these that is not judged near-zero relative to the
#' largest finite positive candidate scale, under a threshold proportional to
#' \eqn{\sqrt{\epsilon_{\mathrm{mach}}}}. If all candidate scales are treated as
#' near-zero, the fit stops with an error because the proportional-bias slope is
#' not estimable on the observed paired-mean scale.
#'
#' The returned `beta_slope` is back-transformed to the original paired-mean
#' scale. The returned BA centre is the fitted mean difference at the centred
#' reference paired mean \eqn{\bar m}, not the original-scale intercept
#' coefficient.
#' }
#'
#' \subsection{Estimation}{
#' The backend uses a stabilised EM/GLS scheme.
#'
#' Conditional on current variance components, the fixed effects are updated by
#' GLS using the marginal precision of the paired differences after integrating
#' out the random subject intercept. The resulting fixed-effect covariance used
#' in the confidence-interval calculations is the GLS covariance
#' \deqn{
#' \mathrm{Var}(\hat\beta \mid \hat\theta)
#' =
#' \left( \sum_i X_i^\top V_i^{-1} X_i \right)^{-1}.
#' }
#'
#' Given updated fixed effects, the variance components are refreshed by EM using
#' the conditional moments of the subject random intercept and the residual
#' quadratic forms. Variance updates are ratio-damped and clipped to admissible
#' ranges for numerical stability.
#' }
#'
#' \subsection{Reported BA centre and limits of agreement}{
#' The reported BA centre is always model-based.
#'
#' When `include_slope = FALSE`, it is the fitted intercept of the paired-
#' difference mixed model.
#'
#' When `include_slope = TRUE`, it is the fitted mean difference at the centred
#' reference paired mean used internally by the backend.
#'
#' The reported limits of agreement are
#' \deqn{
#' \mu_0 \pm \texttt{loa\_multiplier}\sqrt{\sigma_u^2 + \sigma_e^2},
#' }
#' where \eqn{\mu_0} is the reported model-based BA centre. These LoA are for a
#' single new paired difference from a random subject under the fitted model.
#'
#' Under the implemented parameterisation, AR(1) correlation affects the
#' off-diagonal within-subject covariance structure and therefore the estimation
#' of the model parameters and their uncertainty, but not the marginal variance
#' of a single paired difference. Consequently `rho` does not appear explicitly
#' in the LoA point-estimate formula.
#' }
#'
#' \subsection{Confidence intervals}{
#' The backend returns Wald confidence intervals for the reported BA centre and
#' for both LoA endpoints.
#'
#' These intervals combine:
#' \itemize{
#'   \item the conditional GLS uncertainty in the fixed effects at the fitted
#'   covariance parameters; and
#'   \item a delta-method propagation of covariance-parameter uncertainty from
#'   the observed information matrix of the profiled REML log-likelihood.
#' }
#'
#' The covariance-parameter vector is profiled on transformed scales:
#' log-variances for \eqn{\sigma_u^2} and \eqn{\sigma_e^2}, and, when `rho` is
#' estimated internally under AR(1), a transformed correlation parameter
#' mapped back by \eqn{\rho = 0.95\tanh(z)}.
#'
#' Numerical central finite differences are used to approximate both the
#' observed Hessian of the profiled REML log-likelihood and the gradients of the
#' reported derived quantities. The resulting variances are combined and the
#' final intervals are formed with the normal quantile corresponding to
#' `conf_level`.
#' }
#'
#' \subsection{Exactly two methods versus \eqn{\ge 3} methods}{
#' With exactly two methods, at least two complete subject-time pairs are
#' required; otherwise the function errors.
#'
#' With \eqn{\ge 3} methods, the function analyses every unordered pair of method
#' levels. For a given pair with fewer than two complete subject-time matches,
#' that contrast is skipped and the corresponding matrix entries remain `NA`.
#'
#' For a fitted contrast between methods in matrix positions \eqn{(j, k)} with
#' \eqn{j < k}, the stored orientation is:
#' \deqn{
#' \texttt{bias}[j,k] \approx \mathrm{method}_k - \mathrm{method}_j.
#' }
#' Hence the transposed entry changes sign, while `sd_loa` and `width` are
#' symmetric.
#' }
#'
#' \subsection{Identifiability and safeguards}{
#' Separate estimation of the residual and subject-level variance components
#' requires sufficient complete within-subject replication after pairing. If the
#' paired data are not adequate to separate these components, the fit stops with
#' an identifiability error.
#'
#' If the model is conceptually estimable but no finite positive pooled
#' within-subject variance can be formed during initialisation, the backend uses
#' \eqn{0.5 \times v_{\mathrm{ref}}} only as a temporary positive starting value
#' for the EM routine and records a warning string in the backend output. The
#' exported wrapper does not otherwise modify the final estimates.
#'
#' If the EM/GLS routine fails to reach admissible finite variance-component
#' estimates, the backend throws an explicit convergence error rather than
#' returning fallback estimates.
#' }
#'
#' @examples
#' # -------- Simulate repeated-measures data --------
#' set.seed(1)
#'
#' # design (no AR)
#' # subjects
#' S   <- 30L
#' # replicates per subject
#' Tm  <- 15L
#' subj <- rep(seq_len(S), each = Tm)
#' time <- rep(seq_len(Tm), times = S)
#'
#' # subject signal centered at 0 so BA "bias" won't be driven by the mean level
#' mu_s  <- rnorm(S, mean = 0, sd = 8)
#' # constant within subject across replicates
#' true  <- mu_s[subj]
#'
#' # common noise (no AR, i.i.d.)
#' sd_e <- 2
#' e0   <- rnorm(length(true), 0, sd_e)
#'
#' # --- Methods ---
#' # M1: signal + noise
#' y1 <- true + e0
#'
#' # M2: same precision as M1; here identical so M3 can be
#' #     almost perfectly the inverse of both M1 and M2
#' y2 <- y1 + rnorm(length(true), 0, 0.01)
#'
#' # M3: perfect inverse of M1 and M2
#' y3 <- -y1   # = -(true + e0)
#'
#' # M4: unrelated to all others (pure noise, different scale)
#' y4 <- rnorm(length(true), 3, 6)
#'
#' data <- rbind(
#'   data.frame(y = y1, subject = subj, method = "M1", time = time),
#'   data.frame(y = y2, subject = subj, method = "M2", time = time),
#'   data.frame(y = y3, subject = subj, method = "M3", time = time),
#'   data.frame(y = y4, subject = subj, method = "M4", time = time)
#' )
#' data$method <- factor(data$method, levels = c("M1","M2","M3","M4"))
#'
#' # quick sanity checks
#' with(data, {
#'   Y <- split(y, method)
#'   round(cor(cbind(M1 = Y$M1, M2 = Y$M2, M3 = Y$M3, M4 = Y$M4)), 3)
#' })
#'
#' # Run BA (no AR)
#' ba4 <- ba_rm(
#'   data = data,
#'   response = "y", subject = "subject", method = "method", time = "time",
#'   loa_multiplier = 1.96, conf_level = 0.95,
#'   include_slope = FALSE, use_ar1 = FALSE
#' )
#' summary(ba4)
#' estimate(ba4)
#' tidy(ba4)
#' confint(ba4)
#' plot(ba4)
#'
#' # -------- Simulate repeated-measures with AR(1) data --------
#' set.seed(123)
#' S <- 40L                      # subjects
#' Tm <- 50L                     # replicates per subject
#' methods <- c("A","B","C")     # N = 3 methods
#' rho <- 0.4                    # AR(1) within-subject across time
#'
#' ar1_sim <- function(n, rho, sd = 1) {
#'   z <- rnorm(n)
#'   e <- numeric(n)
#'   e[1] <- z[1] * sd
#'   if (n > 1) for (t in 2:n) e[t] <- rho * e[t-1] + sqrt(1 - rho^2) * z[t] * sd
#'   e
#' }
#'
#' # Subject baseline + time trend (latent "true" signal)
#' subj <- rep(seq_len(S), each = Tm)
#' time <- rep(seq_len(Tm), times = S)
#' # subject effects
#' mu_s  <- rnorm(S, 50, 7)
#' trend <- rep(seq_len(Tm) - mean(seq_len(Tm)), times = S) * 0.8
#' true  <- mu_s[subj] + trend
#'
#' # Method-specific biases (B has +1.5 constant; C has slight proportional bias)
#' bias  <- c(A = 0, B = 1.5, C = -0.5)
#' # proportional component on "true"
#' prop  <- c(A = 0.00, B = 0.00, C = 0.10)
#'
#' # Build long data: for each method, add AR(1) noise within subject over time
#' make_method <- function(meth, sd = 3) {
#'   e <- unlist(lapply(split(seq_along(time), subj),
#'                      function(ix) ar1_sim(length(ix), rho, sd)))
#'   y <- true * (1 + prop[meth]) + bias[meth] + e
#'   data.frame(y = y, subject = subj, method = meth, time = time,
#'              check.names = FALSE)
#' }
#'
#' data <- do.call(rbind, lapply(methods, make_method))
#' data$method <- factor(data$method, levels = methods)
#'
#' # -------- Repeated BA (pairwise matrix) ---------------------
#' baN <- ba_rm(
#'   response = data$y, subject = data$subject, method = data$method, time = data$time,
#'   loa_multiplier = 1.96, conf_level = 0.95,
#'   include_slope = FALSE,         # estimate proportional bias per pair
#'   use_ar1 = TRUE, ar1_rho = rho
#' )
#'
#' # Matrices (row - column orientation)
#' print(baN)
#' summary(baN)
#' tidy(baN)
#'
#' # Faceted BA scatter by pair
#' plot(baN, smoother = "lm", facet_scales = "free_y")
#'
#' # -------- Two-method AR(1) path (A vs B only) ------------------------------
#' data_AB <- subset(data, method %in% c("A","B"))
#' baAB <- ba_rm(
#'   response = data_AB$y, subject = data_AB$subject,
#'   method = droplevels(data_AB$method), time = data_AB$time,
#'   include_slope = FALSE, use_ar1 = TRUE, ar1_rho = 0.4
#' )
#' print(baAB)
#' plot(baAB)
#'
#' @author Thiago de Paula Oliveira
#' @export
ba_rm <- function(data = NULL, response, subject, method, time,
                                  conf_level = 0.95,
                                  n_threads = getOption("matrixCorr.threads", 1L),
                                  verbose = FALSE,
                                  loa_multiplier = 1.96,
                                  include_slope = FALSE,
                                  use_ar1 = FALSE, ar1_rho = NA_real_,
                                  max_iter = 200L, tol = 1e-6) {

  # legacy positional signature support: ba_rm(response, subject, method, time, ...)
  if (!missing(data) && !inherits(data, "data.frame") && missing(time)) {
    time <- method
    method <- subject
    subject <- response
    response <- data
    data <- NULL
  }

  if (!is.null(data) && !inherits(data, "data.frame")) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }

  # --- resolve columns if 'data' provided and names given ---
  if (!is.null(data)) {
    if (!inherits(data, "data.frame")) {
      abort_bad_arg("data",
        message = "must be a data.frame or data.table."
      )
    }
    pull <- function(x) {
      if (is.character(x) && length(x) == 1L) {
        if (!x %in% names(data)) {
          abort_bad_arg("data",
            message = "Column '{x}' not found in `data`.",
            x = x
          )
        }
        data[[x]]
      } else x
    }
    response <- pull(response)
    subject  <- pull(subject)
    method   <- pull(method)
    time     <- pull(time)
    mapping <- list(
      response =
        if (is.character(substitute(response)) && length(substitute(response)) == 1L &&
                     is.character(match.call()$response)) as.character(match.call()$response) else ".response",
      subject  = if (is.character(match.call()$subject)) as.character(match.call()$subject) else ".subject",
      method   = if (is.character(match.call()$method))  as.character(match.call()$method)  else ".method",
      time     = if (is.character(match.call()$time))    as.character(match.call()$time)    else ".time"
    )

    # always use canonical internal names in the stored table
    data_long <- setNames(
      data.frame(response, subject, method, time, check.names = FALSE),
      c(".response", ".subject", ".method", ".time")
    )
    mapping <- list(
      response = ".response",
      subject  = ".subject",
      method   = ".method",
      time     = ".time"
    )
  } else {
    # vectors supplied; build canonical internal columns
    mapping <- list(
      response = ".response",
      subject  = ".subject",
      method   = ".method",
      time     = ".time"
    )
    data_long <- data.frame(
      ".response" = response,
      ".subject"  = subject,
      ".method"   = method,
      ".time"     = time,
      check.names = FALSE
    )
  }

  # ---- validate / normalise types (work with local copies) ----
  y <- data_long[[mapping$response]]
  s <- data_long[[mapping$subject ]]
  m <- data_long[[mapping$method  ]]
  t <- data_long[[mapping$time    ]]

  if (!is.numeric(y) || length(y) < 2L) {
    abort_bad_arg("response",
      message = "must be numeric with length > 1."
    )
  }
  if (!(is.integer(s) || is.factor(s) || is.numeric(s))) {
    abort_bad_arg("subject",
      message = "must be integer, factor, or numeric."
    )
  }
  s <- as.integer(as.factor(s))
  if (!is.factor(m)) m <- as.factor(m)
  m <- droplevels(m)
  mlev <- levels(m)
  if (length(mlev) < 2L) {
    abort_bad_arg("method",
      message = "Need at least 2 distinct methods in `method`."
    )
  }
  if (!(is.integer(t) || is.numeric(t))) {
    abort_bad_arg("time",
      message = "must be integer or numeric."
    )
  }
  t <- as.integer(t)

  if (!is.numeric(loa_multiplier) || length(loa_multiplier) != 1L || !is.finite(loa_multiplier) || loa_multiplier <= 0) {
    abort_bad_arg("loa_multiplier",
      message = "`loa_multiplier` must be a positive scalar."
    )
  }
  if (!is.numeric(conf_level) || length(conf_level) != 1L ||
      !is.finite(conf_level) || conf_level <= 0 || conf_level >= 1) {
    abort_bad_arg("conf_level",
      message = "`conf_level` must be in (0,1)."
    )
  }
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(include_slope, arg = "include_slope")
  check_bool(use_ar1, arg = "use_ar1")
  check_bool(verbose, arg = "verbose")
  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  if (isTRUE(use_ar1)) {
    if (!is.na(ar1_rho)) {
      ar1_rho <- check_ar1_rho(ar1_rho, arg = "ar1_rho", bound = 0.999)
    }
  } else {
    ar1_rho <- NA_real_
  }

  if (isTRUE(verbose)) {
    cli::cli_inform("Using {n_threads} OpenMP thread{?s}.")
  }

  # ---- two-method path -------------------------------------------------------
  if (length(mlev) == 2L) {
    idx <- m %in% mlev & is.finite(y) & !is.na(s) & !is.na(t)
    res1 <- .ba_rep_two_methods(
      response = y[idx],
      subject  = s[idx],
      method12 = as.integer(m[idx] == mlev[2L]) + 1L,
      time     = t[idx],
      loa_multiplier = loa_multiplier, conf_level = conf_level, n_threads = n_threads,
      include_slope = include_slope,
      use_ar1 = use_ar1, ar1_rho = ar1_rho, max_iter = max_iter, tol = tol
    )
    attr(res1, "conf.level") <- conf_level
    return(res1)
  }

  # ---- N-method pairwise path ------------------------------------------------
  if (isTRUE(verbose)) {
    cli::cli_inform(
      "Repeated BA (pairwise): {length(mlev)} methods -> {choose(length(mlev), 2L)} pairs."
    )
  }

  methods <- mlev
  mm <- length(methods)
  valid_common <- is.finite(y) & !is.na(s) & !is.na(t) & !is.na(m)
  y_valid <- y[valid_common]
  s_valid <- s[valid_common]
  t_valid <- t[valid_common]
  m_valid <- m[valid_common]
  method_code <- as.integer(m_valid)
  idx_split <- split(seq_along(method_code), method_code)
  idx_by_method <- vector("list", mm)
  for (idx in seq_len(mm)) {
    hit <- idx_split[[as.character(idx)]]
    idx_by_method[[idx]] <- if (is.null(hit)) integer(0L) else hit
  }

  bias         <- .make_named_matrix_ba(methods)
  sd_loa       <- .make_named_matrix_ba(methods)
  loa_lower    <- .make_named_matrix_ba(methods)
  loa_upper    <- .make_named_matrix_ba(methods)
  width        <- .make_named_matrix_ba(methods)
  n_mat        <- .make_named_matrix_ba(methods, NA_integer_, "integer")
  mean_ci_low  <- .make_named_matrix_ba(methods)
  mean_ci_high <- .make_named_matrix_ba(methods)
  lo_ci_low    <- .make_named_matrix_ba(methods)
  lo_ci_high   <- .make_named_matrix_ba(methods)
  hi_ci_low    <- .make_named_matrix_ba(methods)
  hi_ci_high   <- .make_named_matrix_ba(methods)
  slope_mat    <- if (isTRUE(include_slope)) .make_named_matrix_ba(methods) else NULL
  vc_subject   <- .make_named_matrix_ba(methods)
  vc_resid     <- .make_named_matrix_ba(methods)
  ar1_rho_mat   <- if (isTRUE(use_ar1)) .make_named_matrix_ba(methods) else NULL
  residual_model <- matrix(NA_character_, mm, mm, dimnames = list(methods, methods))
  ar1_estimated <- if (isTRUE(use_ar1)) {
    z <- matrix(NA, mm, mm, dimnames = list(methods, methods))
    storage.mode(z) <- "logical"
    z
  } else NULL
  ar1_simplified_pairs <- character()

  .recompose_pair <- function(fit) {
    list(
      md    = as.numeric(fit$bias_mu0),
      sd    = as.numeric(fit$sd_loa),
      lo    = as.numeric(fit$loa_lower),
      hi    = as.numeric(fit$loa_upper),
      bias_l= as.numeric(fit$bias_lwr),
      bias_u= as.numeric(fit$bias_upr),
      lo_l  = as.numeric(fit$loa_lower_lwr),
      lo_u  = as.numeric(fit$loa_lower_upr),
      hi_l  = as.numeric(fit$loa_upper_lwr),
      hi_u  = as.numeric(fit$loa_upper_upr)
    )
  }

  for (j in 1:(mm - 1L)) for (k in (j + 1L):mm) {
    lev_j <- methods[j]; lev_k <- methods[k]
    idx_j <- idx_by_method[[j]]
    idx_k <- idx_by_method[[k]]
    if (!length(idx_j) && !length(idx_k)) next
    idx_pair <- sort.int(c(idx_j, idx_k), method = "radix")
    pair_y <- y_valid[idx_pair]
    pair_s <- s_valid[idx_pair]
    pair_t <- t_valid[idx_pair]
    pair_m12 <- as.integer(method_code[idx_pair] == k) + 1L

    pair_label <- paste(lev_j, lev_k, sep = "-")
    fit_info <- tryCatch(
      .fit_ba_rm_pair_model(
        response = pair_y, subject = pair_s, method12 = pair_m12, time = pair_t,
        n_threads = n_threads,
        include_slope = include_slope, use_ar1 = use_ar1, ar1_rho = ar1_rho,
        max_iter = max_iter, tol = tol, conf_level = conf_level,
        loa_multiplier = loa_multiplier, pair_label = pair_label
      ),
      error = identity
    )

    if (inherits(fit_info, "error")) {
      n_pair <- .count_ba_rm_complete_pairs(pair_y, pair_s, pair_m12, pair_t)
      n_mat[j,k] <- n_mat[k,j] <- n_pair
      if (n_pair >= 2L) {
        .mc_abort_condition(fit_info)
      }
      bias[j,k]      <- bias[k,j]      <- NA_real_
      sd_loa[j,k]    <- sd_loa[k,j]    <- NA_real_
      loa_lower[j,k] <- loa_lower[k,j] <- NA_real_
      loa_upper[j,k] <- loa_upper[k,j] <- NA_real_
      width[j,k]     <- width[k,j]     <- NA_real_

      mean_ci_low[j,k]  <- mean_ci_low[k,j]   <- NA_real_
      mean_ci_high[j,k] <- mean_ci_high[k,j]  <- NA_real_
      lo_ci_low[j,k]    <- lo_ci_low[k,j]     <- NA_real_
      lo_ci_high[j,k]   <- lo_ci_high[k,j]    <- NA_real_
      hi_ci_low[j,k]    <- hi_ci_low[k,j]     <- NA_real_
      hi_ci_high[j,k]   <- hi_ci_high[k,j]    <- NA_real_

      vc_subject[j,k] <- vc_subject[k,j] <- NA_real_
      vc_resid[j,k]   <- vc_resid[k,j]   <- NA_real_
      residual_model[j,k] <- residual_model[k,j] <- NA_character_

      if (!is.null(slope_mat)) {
        slope_mat[j,k] <- slope_mat[k,j] <- NA_real_
      }
      if (isTRUE(use_ar1)) {
        ar1_rho_mat[j,k] <- ar1_rho_mat[k,j] <- NA_real_
        ar1_estimated[j,k] <- ar1_estimated[k,j] <- NA
      }

      inform_if_verbose(
        "Skipping Bland-Altman pair {.val {lev_j}} vs {.val {lev_k}}: need at least 2 complete subject-time pairs (found {n_pair}).",
        .verbose = verbose
      )
      next
    }

    fit <- fit_info$fit
    n_pair <- as.integer(fit$n_pairs)
    n_mat[j,k] <- n_mat[k,j] <- n_pair
    if (n_pair < 2L) {
      bias[j,k]      <- bias[k,j]      <- NA_real_
      sd_loa[j,k]    <- sd_loa[k,j]    <- NA_real_
      loa_lower[j,k] <- loa_lower[k,j] <- NA_real_
      loa_upper[j,k] <- loa_upper[k,j] <- NA_real_
      width[j,k]     <- width[k,j]     <- NA_real_

      mean_ci_low[j,k]  <- mean_ci_low[k,j]   <- NA_real_
      mean_ci_high[j,k] <- mean_ci_high[k,j]  <- NA_real_
      lo_ci_low[j,k]    <- lo_ci_low[k,j]     <- NA_real_
      lo_ci_high[j,k]   <- lo_ci_high[k,j]    <- NA_real_
      hi_ci_low[j,k]    <- hi_ci_low[k,j]     <- NA_real_
      hi_ci_high[j,k]   <- hi_ci_high[k,j]    <- NA_real_

      vc_subject[j,k] <- vc_subject[k,j] <- NA_real_
      vc_resid[j,k]   <- vc_resid[k,j]   <- NA_real_
      residual_model[j,k] <- residual_model[k,j] <- NA_character_

      if (!is.null(slope_mat)) {
        slope_mat[j,k] <- slope_mat[k,j] <- NA_real_
      }
      if (isTRUE(use_ar1)) {
        ar1_rho_mat[j,k] <- ar1_rho_mat[k,j] <- NA_real_
        ar1_estimated[j,k] <- ar1_estimated[k,j] <- NA
      }

      inform_if_verbose(
        "Skipping Bland-Altman pair {.val {lev_j}} vs {.val {lev_k}}: need at least 2 complete subject-time pairs (found {n_pair}).",
        .verbose = verbose
      )
      next
    }
    comp <- .recompose_pair(fit)

    bias[j,k]      <- comp$md;  bias[k,j]      <- -comp$md
    sd_loa[j,k]    <- comp$sd;  sd_loa[k,j]    <-  comp$sd
    loa_lower[j,k] <- comp$lo;  loa_lower[k,j] <- -comp$hi
    loa_upper[j,k] <- comp$hi;  loa_upper[k,j] <- -comp$lo
    width[j,k]     <- comp$hi - comp$lo; width[k,j] <- width[j,k]

    mean_ci_low[j,k]  <- comp$bias_l; mean_ci_low[k,j]  <- -comp$bias_u
    mean_ci_high[j,k] <- comp$bias_u; mean_ci_high[k,j] <- -comp$bias_l
    lo_ci_low[j,k]    <- comp$lo_l;   lo_ci_low[k,j]    <- -comp$hi_u
    lo_ci_high[j,k]   <- comp$lo_u;   lo_ci_high[k,j]   <- -comp$hi_l
    hi_ci_low[j,k]    <- comp$hi_l;   hi_ci_low[k,j]    <- -comp$lo_u
    hi_ci_high[j,k]   <- comp$hi_u;   hi_ci_high[k,j]   <- -comp$lo_l

    vc_subject[j,k] <- vc_subject[k,j] <- as.numeric(fit$sigma2_subject)
    vc_resid[j,k]   <- vc_resid[k,j]   <- as.numeric(fit$sigma2_resid)
    residual_model[j,k] <- residual_model[k,j] <- fit_info$residual_model

    if (isTRUE(use_ar1)) {
      ar1_rho_used <- if (identical(fit_info$residual_model, "ar1")) as.numeric(fit$ar1_rho) else NA_real_
      ar1_rho_mat[j,k] <- ar1_rho_mat[k,j] <- ar1_rho_used
      ar1_estimated[j,k] <- ar1_estimated[k,j] <- if (identical(fit_info$residual_model, "ar1")) isTRUE(fit$ar1_estimated) else NA
      if (isTRUE(fit_info$ar1_simplified)) {
        ar1_simplified_pairs <- c(ar1_simplified_pairs, pair_label)
      }
    }

    if (!is.null(slope_mat)) {
      slope <- as.numeric(fit$beta_slope)
      slope_mat[j,k] <-  slope
      slope_mat[k,j] <- -slope
    }

    if (isTRUE(verbose)) {
      cli::cli_inform(
        "Pair {.val {lev_j}} - {.val {lev_k}}: n={as.integer(fit$n_pairs)}, bias={format(round(comp$md, 4), nsmall = 4)}, sd={format(round(comp$sd, 4), nsmall = 4)}."
      )
    }
  }

  ba_repeated <- list(
    bias = bias,
    sd_loa = sd_loa,
    loa_lower = loa_lower,
    loa_upper = loa_upper,
    width = width,
    n = n_mat,
    mean_ci_low = mean_ci_low,
    mean_ci_high = mean_ci_high,
    loa_lower_ci_low = lo_ci_low,
    loa_lower_ci_high = lo_ci_high,
    loa_upper_ci_low = hi_ci_low,
    loa_upper_ci_high = hi_ci_high,
    slope = slope_mat,
    methods = methods,
    loa_multiplier = loa_multiplier,
    conf_level = conf_level,
    use_ar1 = use_ar1,
    ar1_rho = if (use_ar1) ar1_rho else NA_real_,
    residual_model = residual_model,
    sigma2_subject = vc_subject,
    sigma2_resid   = vc_resid,
    ar1_rho_pair   = if (use_ar1) ar1_rho_mat else NULL,
    ar1_estimated  = if (use_ar1) ar1_estimated else NULL,
    data_long = data_long,
    mapping = mapping
  )
  ba_repeated <- structure(ba_repeated, class = c("ba_repeated_matrix","list"))
  attr(ba_repeated, "conf.level") <- conf_level
  if (length(ar1_simplified_pairs)) {
    .warn_ba_rm_ar1_fallback(ar1_simplified_pairs)
  }
  ba_repeated
}




#' @keywords internal
.count_ba_rm_complete_pairs <- function(response, subject, method12, time) {
  ba_rm_complete_pairs_cpp(response, subject, method12, time)
}


.ba_rm_is_ar1_convergence_failure <- function(cnd) {
  grepl(
    "failed to converge to admissible finite variance-component estimates",
    conditionMessage(cnd),
    fixed = TRUE
  )
}

.warn_ba_rm_ar1_fallback <- function(pair_labels = NULL) {
  where <- if (length(pair_labels)) {
    sprintf(" for pair(s): %s", paste(unique(pair_labels), collapse = ", "))
  } else {
    " for this fit"
  }

  cli::cli_warn(
    "Requested AR(1) residual structure could not be fit{where}; using iid residuals instead."
  )
}


.fit_ba_rm_pair_model <- function(response, subject, method12, time,
                                  n_threads,
                                  include_slope, use_ar1, ar1_rho,
                                  max_iter, tol, conf_level, loa_multiplier,
                                  pair_label = NULL) {
  run_backend <- function(use_ar1_fit, ar1_rho_fit = NA_real_) {
    bland_altman_repeated_em_ext_cpp(
      y = response, subject = subject, method = method12, time = time,
      include_slope = include_slope,
      use_ar1 = use_ar1_fit, ar1_rho = ar1_rho_fit,
      max_iter = max_iter, tol = tol, conf_level = conf_level,
      loa_multiplier_arg = loa_multiplier,
      n_threads = n_threads
    )
  }

  if (!isTRUE(use_ar1)) {
    return(list(
      fit = run_backend(FALSE),
      residual_model = "iid",
      ar1_simplified = FALSE,
      ar1_simplification_reason = NA_character_
    ))
  }

  fit_ar1 <- tryCatch(
    run_backend(TRUE, ar1_rho),
    error = identity
  )
  if (!inherits(fit_ar1, "error")) {
    return(list(
      fit = fit_ar1,
      residual_model = "ar1",
      ar1_simplified = FALSE,
      ar1_simplification_reason = NA_character_
    ))
  }

  if (!.ba_rm_is_ar1_convergence_failure(fit_ar1)) {
    .mc_abort_condition(fit_ar1)
  }

  fit_iid <- tryCatch(
    run_backend(FALSE),
    error = identity
  )
  if (inherits(fit_iid, "error")) {
    .mc_abort_condition(fit_iid)
  }

  list(
    fit = fit_iid,
    residual_model = "iid",
    ar1_simplified = TRUE,
    ar1_simplification_reason = "ar1_fit_failed_then_iid_succeeded",
    pair_label = pair_label
  )
}


#' two-method helper
#' @keywords internal
.ba_rep_two_methods <- function(response, subject, method12, time,
                                loa_multiplier, conf_level, n_threads, include_slope,
                                use_ar1, ar1_rho, max_iter, tol) {
  fit_info <- tryCatch(
    .fit_ba_rm_pair_model(
      response = response, subject = subject, method12 = method12, time = time,
      n_threads = n_threads,
      include_slope = include_slope, use_ar1 = use_ar1, ar1_rho = ar1_rho,
      max_iter = max_iter, tol = tol, conf_level = conf_level,
      loa_multiplier = loa_multiplier
    ),
    error = identity
  )
  if (inherits(fit_info, "error")) {
    n_pairs <- .count_ba_rm_complete_pairs(response, subject, method12, time)
    if (n_pairs < 2L) {
      abort_bad_arg("response",
        message = "must provide at least two subject-time matched pairs after removing missing observations; only {n_pairs} available.",
        n_pairs = n_pairs
      )
    }
    .mc_abort_condition(fit_info)
  }
  fit <- fit_info$fit
  n_pairs <- as.integer(fit$n_pairs)
  if (n_pairs < 2L) {
    abort_bad_arg("response",
      message = "must provide at least two subject-time matched pairs after removing missing observations; only {n_pairs} available.",
      n_pairs = n_pairs
    )
  }

  md  <- as.numeric(fit$bias_mu0)
  sdL <- as.numeric(fit$sd_loa)

  loa_lower <- as.numeric(fit$loa_lower)
  loa_upper <- as.numeric(fit$loa_upper)

  CI.lines <- c(
    "mean.diff.ci.lower"   = as.numeric(fit$bias_lwr),
    "mean.diff.ci.upper"   = as.numeric(fit$bias_upr),
    "lower.limit.ci.lower" = as.numeric(fit$loa_lower_lwr),
    "lower.limit.ci.upper" = as.numeric(fit$loa_lower_upr),
    "upper.limit.ci.lower" = as.numeric(fit$loa_upper_lwr),
    "upper.limit.ci.upper" = as.numeric(fit$loa_upper_upr)
  )

  means <- as.numeric(fit$pairs_mean)
  diffs <- as.numeric(fit$pairs_diff)

  ba_repeated <- list(
    means         = means,
    diffs         = diffs,
    n_obs         = n_pairs,
    based.on      = n_pairs,
    lower.limit   = loa_lower,
    mean.diffs    = md,
    upper.limit   = loa_upper,
    lines         = c(lower = loa_lower, mean = md, upper = loa_upper),
    CI.lines      = CI.lines,
    loa_multiplier = loa_multiplier,
    critical.diff = loa_multiplier * sdL,
    include_slope = include_slope,
    beta_slope    = if (include_slope) as.numeric(fit$beta_slope) else NA_real_,
    sigma2_subject= as.numeric(fit$sigma2_subject),
    sigma2_resid  = as.numeric(fit$sigma2_resid),
    use_ar1       = use_ar1,
    residual_model = fit_info$residual_model,
    ar1_rho       = if (identical(fit_info$residual_model, "ar1")) as.numeric(fit$ar1_rho) else NA_real_,
    ar1_estimated = if (identical(fit_info$residual_model, "ar1")) isTRUE(fit$ar1_estimated) else NA
  )
  ba_repeated <- structure(ba_repeated, class = c("ba_repeated","list"))
  attr(ba_repeated, "conf.level") <- conf_level
  if (isTRUE(fit_info$ar1_simplified)) {
    .warn_ba_rm_ar1_fallback()
  }
  ba_repeated
}




# ---------- printers ----------
#' @rdname ba_rm
#' @method print ba_repeated
#' @param x A \code{"ba_repeated"} object.
#' @param digits Number of digits for estimates (default 3).
#' @param ci_digits Number of digits for CI bounds (default 3).
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#' @export
print.ba_repeated <- function(x, digits = 3, ci_digits = 3,
                              n = NULL, topn = NULL,
                              max_vars = NULL, width = NULL,
                              show_ci = NULL, ...) {
  check_inherits(x, "ba_repeated")
  show_ci <- .mc_resolve_show_ci(show_ci, context = "print")
  display_width <- width
  n   <- as.integer(x$based.on)
  loa_multiplier <- as.numeric(x$loa_multiplier)
  cl  <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  if (!is.finite(cl)) cl <- NA_real_

  md    <- as.numeric(x$mean.diffs)
  loaL  <- as.numeric(x$lower.limit)
  loaU  <- as.numeric(x$upper.limit)
  sd_d  <- as.numeric(x$critical.diff) / loa_multiplier
  loa_width <- loaU - loaL

  cil <- function(nm) as.numeric(x$CI.lines[[nm]])
  bias_l <- cil("mean.diff.ci.lower"); bias_u <- cil("mean.diff.ci.upper")
  lo_l   <- cil("lower.limit.ci.lower"); lo_u <- cil("lower.limit.ci.upper")
  hi_l   <- cil("upper.limit.ci.lower"); hi_u <- cil("upper.limit.ci.upper")

  .mc_print_named_digest(
    c(
      pairs = n,
      loa_rule = sprintf("mean +/- %.3g * SD", loa_multiplier),
      if (identical(show_ci, "yes") && is.finite(cl)) c(ci = sprintf("%g%%", 100 * cl)),
      sd_single_pair = formatC(sd_d, format = "f", digits = digits),
      width = formatC(loa_width, format = "f", digits = digits)
    ),
    header = "Repeated-measures Bland-Altman preview:"
  )
  cat("\n")

  df <- data.frame(
    quantity = c("Mean difference", "Lower LoA", "Upper LoA"),
    estimate = c(md, loaL, loaU),
    lwr      = c(bias_l, lo_l, hi_l),
    upr      = c(bias_u, lo_u, hi_u),
    check.names = FALSE
  )
  df$estimate <- formatC(df$estimate, format = "f", digits = digits)
  if (identical(show_ci, "yes")) {
    df$lwr <- formatC(df$lwr, format = "f", digits = ci_digits)
    df$upr <- formatC(df$upr, format = "f", digits = ci_digits)
  } else {
    df$lwr <- NULL
    df$upr <- NULL
  }
  .mc_print_preview_table(
    df,
    n = .mc_coalesce(n, .mc_display_option("print_max_rows", 20L)),
    topn = .mc_coalesce(topn, .mc_display_option("print_topn", 5L)),
    max_vars = .mc_coalesce(max_vars, .mc_display_option("print_max_vars", NULL)),
    width = .mc_coalesce(display_width, getOption("width", 80L)),
    context = "print",
    full_hint = TRUE,
    summary_hint = TRUE,
    ...
  )

  if (isTRUE(x$include_slope) && is.finite(x$beta_slope)) {
    cat(sprintf("\nProportional bias slope (vs pair mean): %s\n",
                formatC(x$beta_slope, format = "f", digits = digits)))
  } else {
    cat("\n")
  }
  invisible(x)
}

#' @rdname ba_rm
#' @method print ba_repeated_matrix
#' @param x A \code{"ba_repeated_matrix"} object.
#' @param digits Number of digits for estimates (default 3).
#' @param ci_digits Number of digits for CI bounds (default 3).
#' @param style Show as pairs or matrix format?
#' @param ... Unused.
#' @export
print.ba_repeated_matrix <- function(x,
                                     digits = 3,
                                     ci_digits = 3,
                                     n = NULL,
                                     topn = NULL,
                                     max_vars = NULL,
                                     width = NULL,
                                     show_ci = NULL,
                                     style = c("pairs","matrices"),
                                     ...) {
  check_inherits(x, "ba_repeated_matrix")
  style <- match.arg(style)
  show_ci <- .mc_resolve_show_ci(show_ci, context = "print")
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  has_ci <- all(c("mean_ci_low","mean_ci_high",
                  "loa_lower_ci_low","loa_lower_ci_high",
                  "loa_upper_ci_low","loa_upper_ci_high") %in% names(x))

  if (style == "matrices") {
    .mc_print_named_digest(
      c(
        methods = nrow(x$bias),
        if (identical(show_ci, "yes") && is.finite(cl)) c(ci = sprintf("%g%%", 100 * cl)),
        components = "bias, sd_loa, loa_low, loa_up, width, n"
      ),
      header = "Repeated-measures Bland-Altman matrix preview:"
    )
    cat("\nPrimary component: bias\n\n")
    .mc_print_preview_matrix(
      x$bias,
      digits = digits,
      n = .mc_coalesce(n, .mc_display_option("print_max_rows", 20L)),
      topn = .mc_coalesce(topn, .mc_display_option("print_topn", 5L)),
      max_vars = .mc_coalesce(max_vars, .mc_display_option("print_max_vars", NULL)),
      width = .mc_coalesce(width, getOption("width", 80L)),
      context = "print",
      full_hint = TRUE,
      summary_hint = TRUE,
      ...
    )
    return(invisible(x))
  }

  sm <- summary(x, digits = digits, ci_digits = ci_digits)
  cols <- c("method1", "method2", "bias", "sd_loa", "loa_low", "loa_up", "width", "n_obs")
  if ("slope" %in% names(sm)) cols <- c(cols, "slope")
  df <- as.data.frame(sm)[, cols[cols %in% names(sm)], drop = FALSE]

  header <- .mc_header_with_ci("Bland-Altman (row \u2212 column)", cl, if (has_ci) show_ci else "no")
  .mc_print_summary_table(
    df,
    header = header,
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @method summary ba_repeated
#' @export
summary.ba_repeated <- function(object,
                                digits = 3,
                                ci_digits = 3,
                                ...) {
  check_inherits(object, "ba_repeated")
  cl <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  n  <- as.integer(object$based.on)

  ba_repeated <- data.frame(
    method1   = "method 1",
    method2   = "method 2",
    bias      = round(num_or_na_ba(object$mean.diffs), digits),
    sd_loa    = round(num_or_na_ba(object$critical.diff) / num_or_na_ba(object$loa_multiplier), digits),
    loa_low   = round(num_or_na_ba(object$lower.limit), digits),
    loa_up    = round(num_or_na_ba(object$upper.limit), digits),
    width     = round(num_or_na_ba(object$upper.limit - object$lower.limit), digits),
    n_obs     = n,
    stringsAsFactors = FALSE, check.names = FALSE
  )

  if (isTRUE(object$include_slope) && is.finite(num_or_na_ba(object$beta_slope))) {
    ba_repeated$slope <- round(num_or_na_ba(object$beta_slope), digits)
  }

  cil <- function(nm) num_or_na_ba(object$CI.lines[[nm]])
  if (!any(is.na(c(cil("mean.diff.ci.lower"), cil("mean.diff.ci.upper"),
                   cil("lower.limit.ci.lower"), cil("lower.limit.ci.upper"),
                   cil("upper.limit.ci.lower"), cil("upper.limit.ci.upper"))))) {
    ba_repeated$bias_lwr <- round(cil("mean.diff.ci.lower"), ci_digits)
    ba_repeated$bias_upr <- round(cil("mean.diff.ci.upper"), ci_digits)
    ba_repeated$lo_lwr   <- round(cil("lower.limit.ci.lower"), ci_digits)
    ba_repeated$lo_upr   <- round(cil("lower.limit.ci.upper"), ci_digits)
    ba_repeated$up_lwr   <- round(cil("upper.limit.ci.lower"), ci_digits)
    ba_repeated$up_upr   <- round(cil("upper.limit.ci.upper"), ci_digits)
  }

  ba_repeated$sigma2_subject <- round(num_or_na_ba(object$sigma2_subject), digits)
  ba_repeated$sigma2_resid   <- round(num_or_na_ba(object$sigma2_resid),   digits)
  ba_repeated$use_ar1        <- isTRUE(object$use_ar1)
  ba_repeated$residual_model <- if (!is.null(object$residual_model)) object$residual_model else if (isTRUE(object$use_ar1)) "ar1" else "iid"
  if (identical(ba_repeated$residual_model, "ar1")) {
    ba_repeated$ar1_rho       <- round(num_or_na_ba(object$ar1_rho), digits)
    ba_repeated$ar1_estimated <- object$ar1_estimated
  }

  ba_repeated <- .mc_finalize_summary_df(
    ba_repeated,
    class_name = "summary.ba_repeated",
    repeated = TRUE
  )
  attr(ba_repeated, "conf.level") <- cl
  ba_repeated
}

#' @method summary ba_repeated_matrix
#' @export
summary.ba_repeated_matrix <- function(object,
                                       digits = 3,
                                       ci_digits = 3,
                                       ...) {
  check_inherits(object, "ba_repeated_matrix")
  cl <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  methods <- object$methods
  m <- length(methods)

  has_ci <- all(c("mean_ci_low","mean_ci_high",
                  "loa_lower_ci_low","loa_lower_ci_high",
                  "loa_upper_ci_low","loa_upper_ci_high") %in% names(object))

  rows <- vector("list", m * (m - 1L) / 2L)
  k <- 0L
  for (i in 1:(m - 1L)) for (j in (i + 1L):m) {
    k <- k + 1L
    row <- list(
      method1 = methods[i],
      method2 = methods[j],
      bias    = round(object$bias[i, j], digits),
      sd_loa  = round(object$sd_loa[i, j], digits),
      loa_low = round(object$loa_lower[i, j], digits),
      loa_up  = round(object$loa_upper[i, j], digits),
      width   = round(object$width[i, j], digits),
      n_obs   = suppressWarnings(as.integer(object$n[i, j])),
      sigma2_subject = round(object$sigma2_subject[i, j], digits),
      sigma2_resid   = round(object$sigma2_resid[i, j],   digits)
    )

    if (!is.null(object$slope)) {
      row$slope <- round(object$slope[i, j], digits)
    }

    if (has_ci && is.finite(cl)) {
      row$bias_lwr <- round(object$mean_ci_low[i, j],       ci_digits)
      row$bias_upr <- round(object$mean_ci_high[i, j],      ci_digits)
      row$lo_lwr   <- round(object$loa_lower_ci_low[i, j],  ci_digits)
      row$lo_upr   <- round(object$loa_lower_ci_high[i, j], ci_digits)
      row$up_lwr   <- round(object$loa_upper_ci_low[i, j],  ci_digits)
      row$up_upr   <- round(object$loa_upper_ci_high[i, j], ci_digits)
    }
    if (!is.null(object$residual_model)) {
      row$residual_model <- object$residual_model[i, j]
    }
    if (isTRUE(object$use_ar1)) {
      rho_ij <- if (!is.null(object$ar1_rho_pair)) object$ar1_rho_pair[i, j] else NA_real_
      est_ij <- if (!is.null(object$ar1_estimated)) object$ar1_estimated[i, j] else NA
      row$ar1_rho       <- round(rho_ij, digits)
      row$ar1_estimated <- est_ij
    }
    rows[[k]] <- row
  }
  df <- .mc_finalize_summary_df(
    do.call(rbind.data.frame, rows),
    class_name = "summary.ba_repeated_matrix",
    repeated = TRUE
  )
  if ("ar1_rho" %in% names(df) && all(is.na(df$ar1_rho))) {
    df$ar1_rho <- NULL
  }
  if ("ar1_estimated" %in% names(df) && all(is.na(df$ar1_estimated))) {
    df$ar1_estimated <- NULL
  }
  attr(df, "conf.level") <- cl
  df
}

.print_ba_summary_blocks <- function(x, ...) {
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Agreement estimates",
        cols = c("method1", "method2", "n_obs", "bias", "sd_loa",
                 "loa_low", "loa_up", "width", "slope")
      ),
      list(
        title = "Confidence intervals",
        cols = c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr")
      ),
      list(
        title = "Model details",
        cols = c("sigma2_subject", "sigma2_resid",
                 "residual_model", "ar1_rho", "ar1_estimated")
      )
    ),
    header = NULL,
    ...
  )
}


#' @method print summary.ba_repeated
#' @export
print.summary.ba_repeated <- function(x, digits = NULL, n = NULL,
                                      topn = NULL, max_vars = NULL,
                                      width = NULL, show_ci = NULL, ...) {
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  header <- .mc_header_with_ci("Bland-Altman (two methods)", cl, show_ci)
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Agreement estimates",
        cols = c("method1", "method2", "n_obs", "bias", "sd_loa",
                 "loa_low", "loa_up", "width", "slope")
      ),
      list(
        title = "Confidence intervals",
        cols = c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr")
      ),
      list(
        title = "Model details",
        cols = c("sigma2_subject", "sigma2_resid",
                 "residual_model", "ar1_rho", "ar1_estimated")
      )
    ),
    header = header,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @method print summary.ba_repeated_matrix
#' @export
print.summary.ba_repeated_matrix <- function(x, digits = NULL, n = NULL,
                                             topn = NULL, max_vars = NULL,
                                             width = NULL, show_ci = NULL, ...) {
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  header <- .mc_header_with_ci("Bland-Altman (pairwise)", cl, show_ci)
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Agreement estimates",
        cols = c("method1", "method2", "n_obs", "bias", "sd_loa",
                 "loa_low", "loa_up", "width", "slope")
      ),
      list(
        title = "Confidence intervals",
        cols = c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr")
      ),
      list(
        title = "Model details",
        cols = c("sigma2_subject", "sigma2_resid",
                 "residual_model", "ar1_rho", "ar1_estimated")
      )
    ),
    header = header,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#-------- Plots ------------

#' @rdname ba_rm
#' @method plot ba_repeated
#' @param title Plot title (character scalar). Defaults to
#'   `"Bland-Altman (repeated measurements)"` for two methods and
#'   `"Bland-Altman (repeated, pairwise)"` for the faceted matrix plot.
#' @param subtitle Optional subtitle (character scalar). If `NULL`, a compact
#'   summary is shown using the fitted object.
#' @param point_alpha Numeric in `[0, 1]`. Transparency for scatter points
#'   drawn at (pair mean, pair difference) when point data are available.
#'   Passed to `ggplot2::geom_point(alpha = ...)`. Default `0.7`.
#'
#' @param point_size Positive numeric. Size of scatter points; passed to
#'   `ggplot2::geom_point(size = ...)`. Default `2.2`.
#'
#' @param line_size Positive numeric. Line width for horizontal bands
#'   (bias and both LoA) and, when requested, the proportional-bias line.
#'   Passed to `ggplot2::geom_hline(linewidth = ...)` (and `geom_abline`).
#'   Default `0.8`.
#'
#' @param shade_ci Logical. If `TRUE` and confidence intervals are available in
#'   the object (`CI.lines` for two methods; `*_ci_*` matrices for the pairwise
#'   case), semi-transparent rectangles are drawn to indicate CI bands for the
#'   bias and each LoA. If `FALSE`, dashed horizontal CI lines are drawn instead.
#'   Has no effect if CIs are not present. Default `TRUE`.
#'
#' @param shade_alpha Numeric in `[0, 1]`. Opacity of the CI shading
#'   rectangles when `shade_ci = TRUE`. Passed to `ggplot2::annotate("rect", alpha = ...)`.
#'   Default `0.08`.
#'
#' @param smoother One of `"none"`, `"loess"`, or `"lm"`. Adds an overlaid trend
#'   for differences vs means when points are drawn, to visualise proportional
#'   bias. `"lm"` fits a straight line with no SE ribbon; `"loess"` draws a
#'   locally-smoothed curve (span 0.9) with no SE ribbon; `"none"` draws no
#'   smoother. Ignored if `show_points = FALSE` or if no point data are available.
#'
#' @param symmetrize_y Logical (two-method plot only). If `TRUE`, the y-axis is
#'   centred at the estimated bias and expanded symmetrically to cover all
#'   elements used to compute the range (bands, CIs, and points if shown).
#'   Default `TRUE`.
#'
#' @param show_points Logical. If `TRUE`, per-pair points are drawn for the
#'   exactly-two-method path from the stored `means` and `diffs` vectors.
#'   Pairwise matrix plots draw points reconstructed from the canonical
#'   long-form columns stored by `ba_rm()`. If `FALSE` or if point data are
#'   unavailable, only the bands (and optional CI indicators) are drawn.
#'   Default `TRUE`.
#' @param ... Additional theme adjustments passed to `ggplot2::theme(...)`
#'   (e.g., `plot.title.position = "plot"`, `axis.title.x = element_text(size=11)`).
#' @param show_value Logical; included for a consistent plotting interface.
#'   Repeated-measures Bland-Altman plots do not overlay numeric cell values,
#'   so this argument currently has no effect.
#' @importFrom graphics abline lines par rect plot
#' @export
plot.ba_repeated <- function(x,
                             title = "Bland-Altman (repeated measurements)",
                             subtitle = NULL,
                             point_alpha = 0.7,
                             point_size  = 2.2,
                             line_size   = 0.8,
                             shade_ci    = TRUE,
                             shade_alpha = 0.08,
                             smoother    = c("none", "loess", "lm"),
                             symmetrize_y = TRUE,
                             show_points = TRUE,
                             show_value = TRUE,
                             ...) {
  check_inherits(x, "ba_repeated")
  check_bool(show_value, arg = "show_value")
  smoother <- match.arg(smoother)

  # Prefer pre-computed points if present
  has_pts <- !is.null(x$means) && !is.null(x$diffs)
  means <- if (has_pts) as.numeric(x$means) else numeric()
  diffs <- if (has_pts) as.numeric(x$diffs) else numeric()
  if (!has_pts) show_points <- FALSE

  # Bands
  md    <- as.numeric(x$mean.diffs)
  loaL  <- as.numeric(x$lower.limit)
  loaU  <- as.numeric(x$upper.limit)
  loa_multiplier <- as.numeric(x$loa_multiplier)
  n     <- as.integer(x$based.on)
  cl    <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  ci_val <- function(nm) if (!is.null(x$CI.lines)) suppressWarnings(as.numeric(x$CI.lines[[nm]])) else NA_real_
  has_ci <- !is.null(x$CI.lines) &&
    all(is.finite(c(ci_val("mean.diff.ci.lower"), ci_val("mean.diff.ci.upper"),
                    ci_val("lower.limit.ci.lower"), ci_val("lower.limit.ci.upper"),
                    ci_val("upper.limit.ci.lower"), ci_val("upper.limit.ci.upper"))))

  if (is.null(subtitle)) {
    subtitle <- if (is.finite(cl)) {
      sprintf("pairs = %d |  mean diff = %.3g |  LoA = [%.3g, %.3g] |  %g%% CI",
              n, md, loaL, loaU, 100*cl)
    } else {
      sprintf("pairs = %d |  mean diff = %.3g |  LoA = [%.3g, %.3g]",
              n, md, loaL, loaU)
    }
  }

  # y-range
  y_bits <- c(loaL, loaU, md)
  if (has_ci) {
    y_bits <- c(y_bits,
                ci_val("mean.diff.ci.lower"), ci_val("mean.diff.ci.upper"),
                ci_val("lower.limit.ci.lower"), ci_val("lower.limit.ci.upper"),
                ci_val("upper.limit.ci.lower"), ci_val("upper.limit.ci.upper"))
  }
  if (show_points && length(diffs)) y_bits <- c(y_bits, diffs)
  y_rng <- range(y_bits, na.rm = TRUE)
  if (isTRUE(symmetrize_y) && all(is.finite(c(y_rng, md)))) {
    half <- max(abs(c(y_rng[1] - md, y_rng[2] - md)))
    y_rng <- c(md - half, md + half)
  }

  p <- ggplot2::ggplot()

  if (isTRUE(has_ci) && shade_ci) {
    p <- p +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci_val("mean.diff.ci.lower"), ymax = ci_val("mean.diff.ci.upper"),
                        alpha = shade_alpha) +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci_val("lower.limit.ci.lower"), ymax = ci_val("lower.limit.ci.upper"),
                        alpha = shade_alpha) +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci_val("upper.limit.ci.lower"), ymax = ci_val("upper.limit.ci.upper"),
                        alpha = shade_alpha)
  } else if (isTRUE(has_ci)) {
    p <- p + ggplot2::geom_hline(yintercept = c(
      ci_val("mean.diff.ci.lower"),  ci_val("mean.diff.ci.upper"),
      ci_val("lower.limit.ci.lower"),ci_val("lower.limit.ci.upper"),
      ci_val("upper.limit.ci.lower"),ci_val("upper.limit.ci.upper")),
      linetype = "dashed"
    )
  }

  p <- p +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.4, linetype = "dotted", colour = "grey40") +
    ggplot2::geom_hline(yintercept = md,   linewidth = line_size) +
    ggplot2::geom_hline(yintercept = loaL, linewidth = line_size) +
    ggplot2::geom_hline(yintercept = loaU, linewidth = line_size)

  if (show_points && length(means) && length(diffs)) {
    df <- data.frame(means = means, diffs = diffs)
    p <- p + ggplot2::geom_point(data = df, ggplot2::aes(x = means, y = diffs),
                                 alpha = point_alpha, size = point_size)
    if (smoother == "lm") {
      p <- p + ggplot2::geom_smooth(data = df, ggplot2::aes(x = means, y = diffs),
                                    method = "lm", se = FALSE, linewidth = 0.7)
    } else if (smoother == "loess") {
      p <- p + ggplot2::geom_smooth(data = df, ggplot2::aes(x = means, y = diffs),
                                    method = "loess", se = FALSE, linewidth = 0.7, span = 0.9)
    }
    if (isTRUE(x$include_slope) && is.finite(x$beta_slope)) {
      p <- p + ggplot2::geom_abline(intercept = x$mean.diffs, slope = x$beta_slope, linewidth = line_size)
    }
  }

  p +
    ggplot2::coord_cartesian(ylim = y_rng, expand = TRUE) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid = ggplot2::element_blank(), ...) +
    ggplot2::labs(title = title, subtitle = subtitle,
                  x = if (show_points) "Pair mean" else NULL,
                  y = "Difference (method 2 \u2212 method 1)")
}

#' @rdname ba_rm
#' @method plot ba_repeated_matrix
#' @param pairs (Faceted pairwise plot only.) Optional character vector of
#'   labels specifying which method contrasts to display. Labels must match the
#'   "row - column" convention used by `print()`/`summary()` (e.g., `"B - A"`).
#'   Defaults to all upper-triangle pairs.
#'
#' @param against (Faceted pairwise plot only.) Optional single method name.
#'   If supplied, facets are restricted to contrasts of the chosen method
#'   against all others. Ignored when `pairs` is provided.
#'
#' @param facet_scales (Faceted pairwise plot only.) Either `"free_y"` (default)
#'   to allow each facet its own y-axis limits, or `"fixed"` for a common scale
#'   across facets. Passed to `ggplot2::facet_wrap(scales = ...)`.
#' @importFrom ggplot2 ggplot annotate geom_hline geom_point geom_smooth
#' @importFrom ggplot2 coord_cartesian theme_minimal theme labs facet_wrap
#' @importFrom ggplot2 element_blank
#' @export
plot.ba_repeated_matrix <- function(
    x,
    pairs = NULL, against = NULL,
    facet_scales = c("free_y","fixed"),
    title = "Bland-Altman (repeated, pairwise)",
    point_alpha = 0.6, point_size = 1.8,
    line_size = 0.7, shade_ci = TRUE, shade_alpha = 0.08,
    smoother = c("none","loess","lm"),
    show_points = TRUE,
    show_value = TRUE,
    ...
) {
  check_inherits(x, "ba_repeated_matrix")
  check_bool(show_value, arg = "show_value")
  facet_scales <- match.arg(facet_scales)
  smoother <- match.arg(smoother)
  methods <- x$methods
  m <- length(methods)
  lab_pair <- function(i,j) paste(methods[i], "\u2212", methods[j])

  idx_upper <- which(upper.tri(matrix(NA_real_, m, m)), arr.ind = TRUE)
  all_pairs <- data.frame(
    j   = idx_upper[,1],
    k   = idx_upper[,2],
    lab = lab_pair(idx_upper[,1], idx_upper[,2]),
    stringsAsFactors = FALSE
  )

  if (!is.null(against)) {
    if (!against %in% methods) {
      abort_bad_arg("against",
        message = "must be one of: {methods*?{, }{ and }}.",
        methods = methods
      )
    }
    js <- match(against, methods)
    all_pairs <- subset(all_pairs, j == js | k == js)
  } else if (!is.null(pairs)) {
    all_pairs <- subset(all_pairs, lab %in% pairs)
    if (!nrow(all_pairs)) {
      abort_bad_arg("pairs",
        message = "None of the requested pairs matched available contrasts.",
        .hint = "Use the \"row - column\" labels shown in print/summary output."
      )
    }
  }
  pairs_order <- all_pairs$lab

  bands <- data.frame(
    pair = lab_pair(idx_upper[,1], idx_upper[,2]),
    md   = as.vector(x$bias[idx_upper]),
    loaL = as.vector(x$loa_lower[idx_upper]),
    loaU = as.vector(x$loa_upper[idx_upper]),
    md_l = as.vector(x$mean_ci_low[idx_upper]),
    md_u = as.vector(x$mean_ci_high[idx_upper]),
    lo_l = as.vector(x$loa_lower_ci_low[idx_upper]),
    lo_u = as.vector(x$loa_lower_ci_high[idx_upper]),
    hi_l = as.vector(x$loa_upper_ci_low[idx_upper]),
    hi_u = as.vector(x$loa_upper_ci_high[idx_upper]),
    stringsAsFactors = FALSE
  )
  bands <- bands[bands$pair %in% pairs_order, , drop = FALSE]
  bands$pair <- factor(bands$pair, levels = pairs_order)

  has_ci <- with(bands, all(is.finite(c(md_l, md_u, lo_l, lo_u, hi_l, hi_u))))

  # Build point data from stored mapping (no hard-coded names)
  pts <- NULL
  if (isTRUE(show_points) && !is.null(x$data_long) && !is.null(x$mapping)) {
    df  <- as.data.frame(x$data_long, check.names = FALSE)
    map <- x$mapping
    mY <- .resolve_map_col_ba(df, map, "response")
    mS <- .resolve_map_col_ba(df, map, "subject")
    mM <- .resolve_map_col_ba(df, map, "method")
    mT <- .resolve_map_col_ba(df, map, "time")

    build_panel <- function(lev_j, lev_k, lab) {
      a <- df[df[[mM]] == lev_j, c(mS, mT, mY)]
      b <- df[df[[mM]] == lev_k, c(mS, mT, mY)]
      names(a) <- c("subject","time","yA"); names(b) <- c("subject","time","yB")
      ab <- merge(a, b, by = c("subject","time"), all = FALSE)
      if (!nrow(ab)) return(NULL)
      data.frame(pair = lab,
                 means = (ab$yA + ab$yB)/2,
                 diffs =  ab$yB - ab$yA,
                 stringsAsFactors = FALSE)
    }

    pts <- do.call(rbind, lapply(seq_len(nrow(all_pairs)), function(i) {
      build_panel(methods[all_pairs$j[i]], methods[all_pairs$k[i]], all_pairs$lab[i])
    }))
    if (!is.null(pts) && nrow(pts)) {
      pts$pair <- factor(pts$pair, levels = pairs_order)
    } else {
      pts <- NULL
    }
  }

  p <- ggplot2::ggplot() + ggplot2::facet_wrap(~ pair, scales = facet_scales)

  # Ensure y-scale trained even without points
  p <- p +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = md))   +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = loaL)) +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = loaU))

  if (has_ci) {
    if (shade_ci) {
      p <- p +
        ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = md_l, ymax = md_u),
                           inherit.aes = FALSE, alpha = shade_alpha) +
        ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = lo_l, ymax = lo_u),
                           inherit.aes = FALSE, alpha = shade_alpha) +
        ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = hi_l, ymax = hi_u),
                           inherit.aes = FALSE, alpha = shade_alpha)
    } else {
      p <- p +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md_l)) +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md_u)) +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = lo_l)) +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = lo_u)) +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = hi_l)) +
        ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = hi_u))
    }
  }

  p <- p +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.35, linetype = "dotted", colour = "grey40") +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md),   linewidth = line_size) +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = loaL), linewidth = line_size) +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = loaU), linewidth = line_size)

  if (!is.null(pts)) {
    p <- p + ggplot2::geom_point(data = pts, ggplot2::aes(x = means, y = diffs),
                                 alpha = point_alpha, size = point_size)
    if (smoother == "lm") {
      p <- p + ggplot2::geom_smooth(data = pts, ggplot2::aes(x = means, y = diffs),
                                    method = "lm", se = FALSE, linewidth = 0.7)
    } else if (smoother == "loess") {
      p <- p + ggplot2::geom_smooth(data = pts, ggplot2::aes(x = means, y = diffs),
                                    method = "loess", se = FALSE, linewidth = 0.7, span = 0.9)
    }
  }

  p +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid = ggplot2::element_blank(), ...) +
    ggplot2::labs(title = title,
                  x = if (!is.null(pts)) "Pair mean" else NULL,
                  y = "Difference (row \u2212 column)")
}

#' @keywords internal
num_or_na_ba <- function(x) {
  y <- suppressWarnings(as.numeric(x))
  y[!is.finite(y)] <- NA_real_
  y
}

#' @keywords internal
.resolve_map_col_ba <- function(df, mapping, key) {
  candidate <- mapping[[key]] %||% switch(key,
                                          response = ".response",
                                          subject  = ".subject",
                                          method   = ".method",
                                          time     = ".time"
  )
  if (candidate %in% names(df)) return(candidate)

  canonical <- switch(key,
                      response = ".response",
                      subject  = ".subject",
                      method   = ".method",
                      time     = ".time"
  )
  if (canonical %in% names(df)) return(canonical)

  abort_bad_arg("mapping",
    message = "Column for '{key}' not found in stored data_long.",
    key = key
  )
}

#' @keywords internal
.make_named_matrix_ba <- function(methods, fill = NA_real_, storage = "double") {
  m <- length(methods)
  out <- matrix(fill, m, m, dimnames = list(methods, methods))
  storage.mode(out) <- storage
  out
}

# Internal exact percentage-bend helpers. These stay in R until the compiled
# backend reproduces the established wide-matrix reference behaviour exactly.
.mc_pbcor_scores <- function(z, beta = 0.2) {
  med <- stats::median(z)
  omega <- sort(abs(z - med))[floor((1 - beta) * length(z))]
  if (!is.finite(omega) || omega <= 0) return(rep(NA_real_, length(z)))

  psi <- (z - med) / omega
  i1 <- sum(psi < -1)
  i2 <- sum(psi > 1)
  sx <- z
  sx[psi < -1 | psi > 1] <- 0
  denom <- length(z) - i1 - i2
  if (denom <= 0) return(rep(NA_real_, length(z)))

  theta <- (sum(sx) + omega * (i2 - i1)) / denom
  a <- (z - theta) / omega
  a[a <= -1] <- -1
  a[a >= 1] <- 1
  a
}

.mc_pbcor_pair_exact <- function(x, y, beta = 0.2) {
  ax <- .mc_pbcor_scores(x, beta = beta)
  ay <- .mc_pbcor_scores(y, beta = beta)
  if (anyNA(ax) || anyNA(ay)) return(NA_real_)

  den_x <- sum(ax^2)
  den_y <- sum(ay^2)
  if (!is.finite(den_x) || !is.finite(den_y) || den_x <= 0 || den_y <= 0) {
    return(NA_real_)
  }

  sum(ax * ay) / sqrt(den_x * den_y)
}

.mc_pbcor_matrix_exact <- function(X, beta = 0.2) {
  n <- nrow(X)
  p <- ncol(X)
  if (p < 2L) {
    abort_bad_arg("X", message = "must contain at least two numeric columns.")
  }
  scores <- matrix(0, nrow = n, ncol = p)
  valid <- rep(FALSE, p)

  for (j in seq_len(p)) {
    a <- .mc_pbcor_scores(X[, j], beta = beta)
    if (!anyNA(a)) {
      ss <- sum(a^2)
      if (is.finite(ss) && ss > 0) {
        scores[, j] <- a
        valid[j] <- TRUE
      }
    }
  }

  ss <- colSums(scores^2)
  out <- as.matrix(crossprod(scores))
  denom <- sqrt(outer(ss, ss))
  out <- as.matrix(out / denom)
  out <- pmin(1, pmax(-1, out))
  out <- matrix(out, nrow = p, ncol = p, dimnames = list(colnames(X), colnames(X)))

  if (any(!valid)) {
    bad <- which(!valid)
    out[bad, ] <- NA_real_
    out[, bad] <- NA_real_
  }
  diag(out) <- 1
  out
}

.mc_pbcor_matrix_pairwise_exact <- function(X, beta = 0.2, min_n = 5L) {
  p <- ncol(X)
  if (p < 2L) {
    abort_bad_arg("X", message = "must contain at least two numeric columns.")
  }
  out <- matrix(NA_real_, nrow = p, ncol = p)
  dimnames(out) <- list(colnames(X), colnames(X))

  finite_n <- colSums(is.finite(X))
  diag(out) <- 1

  for (j in seq_len(p - 1L)) {
    for (k in seq.int(j + 1L, p)) {
      ok <- is.finite(X[, j]) & is.finite(X[, k])
      if (sum(ok) < min_n) next
      out[j, k] <- .mc_pbcor_pair_exact(X[ok, j], X[ok, k], beta = beta)
      out[k, j] <- out[j, k]
    }
  }

  if (any(finite_n < 2L)) {
    bad <- which(finite_n < 2L)
    out[bad, ] <- NA_real_
    out[, bad] <- NA_real_
    diag(out) <- 1
  }
  out
}

.mc_pbcor_pair_inference <- function(x,
                                     y,
                                     beta = 0.2,
                                     ci = FALSE,
                                     p_value = FALSE,
                                     conf_level = 0.95,
                                     n_boot = 500L,
                                     seed = NULL) {
  est <- .mc_pbcor_pair_exact(x, y, beta = beta)
  n_complete <- length(x)
  statistic <- NA_real_
  p_val <- NA_real_
  parameter <- NA_real_
  lwr <- NA_real_
  upr <- NA_real_

  if (isTRUE(p_value) && is.finite(est) && n_complete > 2L) {
    parameter <- n_complete - 2
    if (abs(est) >= 1) {
      statistic <- sign(est) * Inf
      p_val <- 0
    } else {
      statistic <- est * sqrt((n_complete - 2) / (1 - est^2))
      p_val <- 2 * stats::pt(abs(statistic), df = parameter, lower.tail = FALSE)
    }
  }

  if (isTRUE(ci) && is.finite(est) && n_complete > 1L) {
    boot_vals <- .mc_eval_with_seed(
      seed,
      replicate(
        n_boot,
        {
          idx <- sample.int(n_complete, size = n_complete, replace = TRUE)
          .mc_pbcor_pair_exact(x[idx], y[idx], beta = beta)
        }
      )
    )
    ci_vals <- .mc_percentile_boot_ci(boot_vals, conf_level = conf_level)
    lwr <- ci_vals[[1L]]
    upr <- ci_vals[[2L]]
  }

  list(
    estimate = est,
    statistic = statistic,
    p_value = p_val,
    parameter = parameter,
    lwr = lwr,
    upr = upr,
    n_complete = n_complete
  )
}

.mc_pbcor_pairwise_payload <- function(X,
                                       est,
                                       beta = 0.2,
                                       ci = FALSE,
                                       p_value = FALSE,
                                       conf_level = 0.95,
                                       n_boot = 500L,
                                       seed = NULL) {
  est <- as.matrix(est)
  p <- ncol(est)
  dn <- dimnames(est)
  n_complete <- .mc_corr_n_complete(X)
  dimnames(n_complete) <- dn

  ci_lwr <- ci_upr <- NULL
  if (isTRUE(ci)) {
    ci_lwr <- matrix(NA_real_, p, p, dimnames = dn)
    ci_upr <- matrix(NA_real_, p, p, dimnames = dn)
    diag(ci_lwr) <- 1
    diag(ci_upr) <- 1
  }

  stat_mat <- p_mat <- df_mat <- NULL
  if (isTRUE(p_value)) {
    stat_mat <- matrix(NA_real_, p, p, dimnames = dn)
    p_mat <- matrix(NA_real_, p, p, dimnames = dn)
    df_mat <- matrix(NA_real_, p, p, dimnames = dn)
    diag(stat_mat) <- Inf
    diag(p_mat) <- 0
  }

  pair_id <- 0L
  for (i in seq_len(p - 1L)) {
    for (j in seq.int(i + 1L, p)) {
      pair_id <- pair_id + 1L
      ok <- is.finite(X[, i]) & is.finite(X[, j])
      n_ij <- sum(ok)
      n_complete[i, j] <- n_complete[j, i] <- n_ij
      if (n_ij < 5L) {
        next
      }

      info <- .mc_pbcor_pair_inference(
        X[ok, i],
        X[ok, j],
        beta = beta,
        ci = ci,
        p_value = p_value,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = .mc_seed_offset(seed, pair_id - 1L)
      )

      if (isTRUE(ci)) {
        ci_lwr[i, j] <- ci_lwr[j, i] <- info$lwr
        ci_upr[i, j] <- ci_upr[j, i] <- info$upr
      }
      if (isTRUE(p_value)) {
        stat_mat[i, j] <- stat_mat[j, i] <- info$statistic
        p_mat[i, j] <- p_mat[j, i] <- info$p_value
        df_mat[i, j] <- df_mat[j, i] <- info$parameter
      }
    }
  }

  diagnostics <- list(n_complete = n_complete)
  ci_attr <- NULL
  if (isTRUE(ci)) {
    ci_attr <- list(
      est = unclass(est),
      lwr.ci = ci_lwr,
      upr.ci = ci_upr,
      conf.level = conf_level,
      ci.method = "percentile_bootstrap"
    )
  }

  inference_attr <- NULL
  if (isTRUE(p_value)) {
    inference_attr <- list(
      method = "percentage_bend_t_test",
      estimate = unclass(est),
      statistic = stat_mat,
      p_value = p_mat,
      parameter = df_mat,
      n_obs = n_complete,
      alternative = "two.sided"
    )
  }

  list(
    diagnostics = diagnostics,
    ci = ci_attr,
    inference = inference_attr
  )
}

.mc_pbcor_pairwise_summary <- function(object,
                                       digits = 4,
                                       ci_digits = 3,
                                       p_digits = 4,
                                       show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "pbcor")

  ci <- .mc_ci_attr(object)
  inf <- .mc_inference_attr(object)
  include_p <- !is.null(inf) && !is.null(inf$p_value)
  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.pbcor",
    digits = digits,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci,
    ci_attr = ci,
    inference_attr = inf,
    include_p = include_p,
    extra_columns = if (isTRUE(include_p)) {
      list(
        statistic = list(matrix = inf$statistic, digits = digits),
        p_value = list(matrix = inf$p_value, digits = p_digits)
      )
    },
    extra_attrs = list(
      inference_method = if (is.null(inf)) NA_character_ else inf$method,
      n_boot = if (is.null(ci)) NA_integer_ else attr(object, "n_boot", exact = TRUE) %||% NA_integer_
    )
  )
}

#' Percentage bend correlation
#'
#' @description
#' Computes all pairwise percentage bend correlations for the numeric columns of
#' a matrix or data frame. Percentage bend correlation limits the influence of
#' extreme marginal observations by bending standardised deviations into the
#' interval \eqn{[-1, 1]}, yielding a Pearson-like measure that is robust to
#' outliers and heavy tails.
#'
#' @param data A numeric matrix or data frame containing numeric columns.
#' @param beta Bending constant in \code{[0, 0.5)} that sets the cutoff used to
#'   bend standardised deviations toward the interval \eqn{[-1, 1]}. Larger
#'   values cause more observations to be bent and increase resistance to
#'   marginal outliers. Default \code{0.2}. See Details.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each estimate on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#'   With \code{"pairwise"}, each correlation is computed on the overlapping
#'   complete rows for the column pair.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads used for the
#'   point-estimate matrix computation. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach percentile
#'   bootstrap confidence intervals for each pairwise estimate.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach the
#'   method-specific large-sample test statistic and two-sided p-value for each
#'   pairwise estimate.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default
#'   \code{0.95}.
#' @param n_boot Integer \eqn{\geq 1}. Number of bootstrap resamples used when
#'   \code{ci = TRUE}. Default \code{500}.
#' @param seed Optional positive integer used to seed the bootstrap resampling
#'   when \code{ci = TRUE}. If \code{NULL}, the current random-number stream is
#'   used.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#' @param x An object of class \code{pbcor}.
#' @param digits Integer; number of digits to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits used for confidence limits in pairwise
#'   summaries.
#' @param p_digits Integer; digits used for p-values in pairwise summaries.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to the underlying print or plot helper.
#' @param title Character; plot title.
#' @param low_color,high_color,mid_color Colors used in the heatmap.
#' @param value_text_size Numeric text size for overlaid cell values.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#'
#' @return A symmetric correlation matrix with class \code{pbcor} and
#'   attributes \code{method = "percentage_bend_correlation"},
#'   \code{description}, and \code{package = "matrixCorr"}. When
#'   \code{ci = TRUE}, the returned object also carries a \code{ci} attribute
#'   with elements \code{est}, \code{lwr.ci}, \code{upr.ci},
#'   \code{conf.level}, and \code{ci.method}, plus
#'   \code{attr(x, "conf.level")}. When \code{p_value = TRUE}, it also carries
#'   an \code{inference} attribute with elements \code{estimate},
#'   \code{statistic}, \code{parameter}, \code{p_value}, \code{n_obs}, and
#'   \code{alternative}. When either inferential option is requested, the
#'   object also carries \code{diagnostics$n_complete}.
#'
#' @details
#' Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
#' observations and columns as variables. For a column
#' \eqn{x = (x_i)_{i=1}^n}, let \eqn{m_x = \mathrm{med}(x)} and define
#' \eqn{\omega_\beta(x)} as the \eqn{\lfloor (1-\beta)n \rfloor}-th order
#' statistic of \eqn{|x_i - m_x|}. Larger values of \code{beta} reduce
#' \eqn{\omega_\beta(x)}, so more observations are bent to the bounds
#' \eqn{-1} and \eqn{1}.
#'
#' The one-step percentage-bend location is
#' \deqn{
#' \hat\theta_{pb}(x) =
#' \frac{\sum_{i: |\psi_i| \le 1} x_i + \omega_\beta(x)(i_2 - i_1)}
#'      {n - i_1 - i_2},
#' \qquad
#' \psi_i = \frac{x_i - m_x}{\omega_\beta(x)},
#' }
#' where \eqn{i_1 = \sum_{i=1}^n \mathbf{1}(\psi_i < -1)} and
#' \eqn{i_2 = \sum_{i=1}^n \mathbf{1}(\psi_i > 1)}. The bent scores are
#' \deqn{
#' a_i = \max\!\left\{-1,\; \min\!\left(1,\frac{x_i - \hat\theta_{pb}(x)}
#' {\omega_\beta(x)}\right)\right\},
#' }
#' and likewise \eqn{b_i} for a second column \eqn{y}. The percentage bend
#' correlation for the pair \eqn{(x,y)} is
#' \deqn{
#' r_{pb}(x,y) =
#' \frac{\sum_{i=1}^n a_i b_i}
#'      {\sqrt{\sum_{i=1}^n a_i^2}\sqrt{\sum_{i=1}^n b_i^2}}.
#' }
#'
#' In the complete-data path, the bent score vectors are computed once per
#' column and collected into a matrix \eqn{A = [a_{\cdot 1}, \ldots, a_{\cdot p}]},
#' after which the correlation matrix is formed from their cross-products:
#' \deqn{
#' R_{pb} = D_A^{-1/2} A^\top A D_A^{-1/2},
#' \qquad
#' D_A = \mathrm{diag}(a_{\cdot 1}^\top a_{\cdot 1}, \ldots,
#' a_{\cdot p}^\top a_{\cdot p}).
#' }
#' If a column yields an undefined bent-score denominator, the corresponding row
#' and column are returned as \code{NA}. With \code{na_method = "pairwise"},
#' each pair is recomputed on its complete-case overlap. As with pairwise
#' Pearson correlation, this pairwise path can break positive semidefiniteness.
#'
#' When \code{p_value = TRUE}, the method-specific test statistic for a pairwise
#' estimate \eqn{r_{pb}} based on \eqn{n_{ij}} complete observations is
#' \deqn{
#' T_{ij} = r_{pb,ij}\sqrt{\frac{n_{ij} - 2}{1 - r_{pb,ij}^2}},
#' }
#' and the reported p-value is the two-sided Student-\eqn{t} tail probability
#' with \eqn{n_{ij}-2} degrees of freedom. When \code{ci = TRUE}, the interval
#' is a percentile bootstrap interval based on \eqn{n_{\mathrm{boot}}}
#' resamples drawn from the pairwise complete cases. If
#' \eqn{\tilde r_{pb,(1)} \le \cdots \le \tilde r_{pb,(B)}} denotes the sorted
#' bootstrap sample of finite estimates with \eqn{B} retained resamples, the
#' reported limits are
#' \deqn{
#' \tilde r_{pb,(\ell)} \quad \text{and} \quad \tilde r_{pb,(u)},
#' }
#' where \eqn{\ell = \lfloor (\alpha/2) B + 0.5 \rfloor} and
#' \eqn{u = \lfloor (1-\alpha/2) B + 0.5 \rfloor} for
#' \eqn{\alpha = 1 - \mathrm{conf\_level}}. Resamples that yield undefined
#' estimates are discarded before the percentile limits are formed.
#'
#' \strong{Computational complexity.} In the complete-data path, forming the
#' bent scores requires sorting within each column and the cross-product step
#' costs \eqn{O(n p^2)} with \eqn{O(p^2)} output storage. When
#' \code{ci = TRUE}, the bootstrap cost is incurred separately for each column
#' pair.
#'
#' @references
#' Wilcox, R. R. (1994). The percentage bend correlation coefficient.
#' Psychometrika, 59(4), 601-616. \doi{10.1007/BF02294395}
#'
#' @seealso [wincor()], [skipped_corr()], [bicor()]
#'
#' @examples
#' set.seed(10)
#' X <- matrix(rnorm(150 * 4), ncol = 4)
#' X[sample(length(X), 8)] <- X[sample(length(X), 8)] + 10
#'
#' R <- pbcor(X)
#' print(R, digits = 2)
#' summary(R)
#' estimate(R)
#' tidy(R)
#' plot(R)
#'
#' ## Bootstrap confidence intervals
#' R_ci <- pbcor(X, ci = TRUE, n_boot = 49, seed = 10)
#' ci(R_ci)
#' confint(R_ci)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(R)
#' }
#'
#' @author Thiago de Paula Oliveira
#' @export
pbcor <- function(data,
                  na_method = c("error", "pairwise", "complete"),
                  ci = FALSE,
                  p_value = FALSE,
                  conf_level = 0.95,
                  n_threads = getOption("matrixCorr.threads", 1L),
                  beta = 0.2,
                  n_boot = 500L,
                  seed = NULL,
                  output = c("matrix", "sparse", "edge_list"),
                  threshold = 0,
                  diag = TRUE) {
  check_scalar_numeric(beta,
                       arg = "beta",
                       lower = 0,
                       upper = 0.5,
                       closed_lower = TRUE,
                       closed_upper = FALSE)
  na_method <- match.arg(na_method)
  desc <- paste0(
    "Percentage bend correlation; beta = ", beta,
    "; NA mode = ", na_method, "."
  )

  .mc_inference_corr_wrapper(
    data = data,
    na_method = na_method,
    ci = ci,
    p_value = p_value,
    conf_level = conf_level,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag,
    estimator_class = "pbcor",
    method = "percentage_bend_correlation",
    description = desc,
    kernel_matrix = function(x, n_threads) {
      pbcor_matrix_cpp(x, beta = beta, n_threads = n_threads)
    },
    kernel_pairwise = function(x, n_threads) {
      pbcor_matrix_pairwise_cpp(x, beta = beta, min_n = 5L, n_threads = n_threads)
    },
    kernel_threshold = function(x, threshold, diag, n_threads) {
      pbcor_threshold_triplets_cpp(
        x,
        beta = beta,
        threshold = threshold,
        diag = diag,
        n_threads = n_threads
      )
    },
    payload_builder = function(x, est, ci, p_value, conf_level, n_boot, seed) {
      .mc_pbcor_pairwise_payload(
        x,
        est = est,
        beta = beta,
        ci = ci,
        p_value = p_value,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = seed
      )
    },
    min_n = 5L,
    direct_method = "pbcor",
    symmetric = TRUE,
    n_boot = n_boot,
    seed = seed
  )
}

#' @rdname pbcor
#' @method print pbcor
#' @export
print.pbcor <- function(x, digits = 4, n = NULL, topn = NULL,
                        max_vars = NULL, width = NULL,
                        show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Percentage bend correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname pbcor
#' @method plot pbcor
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.pbcor <- function(x,
                       title = "Percentage bend correlation heatmap",
                       low_color = "indianred1",
                       high_color = "steelblue1",
                       mid_color = "white",
                       value_text_size = 4,
                       show_value = TRUE,
                       ...) {
  .mc_plot_corr_matrix(
    x,
    class_name = "pbcor",
    fill_name = "pbcor",
    title = title,
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    show_value = show_value,
    ...
  )
}

#' @rdname pbcor
#' @method summary pbcor
#' @param object An object of class \code{pbcor}.
#' @export
summary.pbcor <- function(object, n = NULL, topn = NULL,
                          max_vars = NULL, width = NULL,
                          ci_digits = 3,
                          p_digits = 4,
                          show_ci = NULL, ...) {
  check_inherits(object, "pbcor")
  if (is.null(.mc_ci_attr(object)) && is.null(.mc_inference_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_pbcor_pairwise_summary(
    object,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname pbcor
#' @method print summary.pbcor
#' @param x An object of class \code{summary.pbcor}.
#' @export
print.summary.pbcor <- function(x, digits = NULL, n = NULL,
                                topn = NULL, max_vars = NULL,
                                width = NULL, show_ci = NULL, ...) {
  extra_items <- c(
    inference = attr(x, "inference_method", exact = TRUE),
    n_boot = if (isTRUE(attr(x, "has_ci", exact = TRUE))) {
      as.character(attr(x, "n_boot", exact = TRUE))
    } else {
      NA_character_
    }
  )
  .mc_print_pairwise_summary_digest(
    x,
    title = "Percentage bend correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "percentile_bootstrap",
    extra_items = extra_items,
    ...
  )
  invisible(x)
}

expect_pairwise_summary_equal <- function(actual, expected) {
  expect_identical(names(actual), names(expected))
  expect_identical(class(actual), class(expected))
  expect_identical(setdiff(names(attributes(actual)), "row.names"),
                   setdiff(names(attributes(expected)), "row.names"))
  expect_identical(rownames(actual), rownames(expected))

  for (nm in names(expected)) {
    if (is.numeric(expected[[nm]])) {
      expect_equal(actual[[nm]], expected[[nm]], tolerance = 1e-12, info = nm)
    } else {
      expect_identical(actual[[nm]], expected[[nm]], info = nm)
    }
  }

  actual_attrs <- attributes(actual)
  expected_attrs <- attributes(expected)
  actual_attrs$row.names <- NULL
  expected_attrs$row.names <- NULL
  expect_equal(actual_attrs, expected_attrs, tolerance = 1e-12)
}

reference_pairwise_summary <- function(object,
                                       class_name,
                                       digits = 4,
                                       ci_digits = 3,
                                       p_digits = 4,
                                       show_ci = "yes",
                                       ci = attr(object, "ci", exact = TRUE),
                                       inf = attr(object, "inference", exact = TRUE),
                                       diagnostics = attr(object, "diagnostics", exact = TRUE),
                                       include_ci = identical(show_ci, "yes") && !is.null(ci),
                                       include_p = FALSE,
                                       extra = NULL,
                                       overview = matrixCorr:::.mc_summary_corr_matrix(object),
                                       attrs = list()) {
  est <- as.matrix(object)
  rn <- rownames(est)
  cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  rows <- vector("list", nrow(est) * (ncol(est) - 1L) / 2L)
  k <- 0L
  for (i in seq_len(nrow(est) - 1L)) {
    for (j in (i + 1L):ncol(est)) {
      k <- k + 1L
      rec <- list(
        var1 = rn[[i]],
        var2 = cn[[j]],
        estimate = round(est[i, j], digits)
      )
      if (is.list(diagnostics) && is.matrix(diagnostics$n_complete)) {
        rec$n_complete <- as.integer(diagnostics$n_complete[i, j])
      }
      if (include_ci) {
        rec$lwr <- if (!is.null(ci$lwr.ci) && is.finite(ci$lwr.ci[i, j])) {
          round(ci$lwr.ci[i, j], ci_digits)
        } else {
          NA_real_
        }
        rec$upr <- if (!is.null(ci$upr.ci) && is.finite(ci$upr.ci[i, j])) {
          round(ci$upr.ci[i, j], ci_digits)
        } else {
          NA_real_
        }
      }
      if (is.function(extra)) {
        rec <- c(rec, extra(i, j, inf, diagnostics))
      }
      rows[[k]] <- rec
    }
  }

  df <- do.call(rbind.data.frame, rows)
  rownames(df) <- NULL
  for (nm in intersect(c("estimate", "lwr", "upr", "statistic", "df",
                        "fisher_z", "p_value", "p_value_adjusted",
                        "skipped_prop"), names(df))) {
    df[[nm]] <- as.numeric(df[[nm]])
  }
  for (nm in intersect(c("n_complete", "skipped_n"), names(df))) {
    df[[nm]] <- as.integer(df[[nm]])
  }

  out <- matrixCorr:::.mc_finalize_summary_df(df, class_name = class_name)
  attr(out, "overview") <- overview
  attr(out, "has_ci") <- include_ci
  if (!is.null(include_p)) attr(out, "has_p") <- include_p
  attr(out, "conf.level") <- if (is.null(ci)) NA_real_ else ci$conf.level
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  if (!is.null(p_digits)) attr(out, "p_digits") <- p_digits
  for (nm in names(attrs)) attr(out, nm) <- attrs[[nm]]
  out
}

test_that("shared pairwise summaries preserve estimator-specific data frames", {
  set.seed(20260522)
  X <- matrix(rnorm(72), nrow = 18, ncol = 4)
  colnames(X) <- paste0("S", seq_len(ncol(X)))

  pearson <- pearson_corr(X, ci = TRUE)
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_pearson_pairwise_summary(pearson),
    reference_pairwise_summary(pearson, "summary.pearson_corr", p_digits = NULL, include_p = NULL)
  )

  spearman <- spearman_rho(X, ci = TRUE)
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_spearman_pairwise_summary(spearman),
    reference_pairwise_summary(spearman, "summary.spearman_rho", p_digits = NULL, include_p = NULL)
  )

  kendall <- kendall_tau(X, ci = TRUE)
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_kendall_pairwise_summary(kendall),
    reference_pairwise_summary(
      kendall,
      "summary.kendall_matrix",
      p_digits = NULL,
      include_p = NULL,
      attrs = list(ci.method = attr(kendall, "ci", exact = TRUE)$ci.method)
    )
  )

  bic <- bicor(X, ci = TRUE)
  bic_extra <- function(i, j, inf, diagnostics) {
    list(
      statistic = if (is.finite(inf$statistic[i, j])) round(inf$statistic[i, j], 4) else NA_real_,
      fisher_z = if (is.finite(inf$Z[i, j])) round(inf$Z[i, j], 4) else NA_real_,
      p_value = if (is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], 4) else NA_real_
    )
  }
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_bicor_pairwise_summary(bic),
    reference_pairwise_summary(
      bic,
      "summary.bicor",
      include_p = TRUE,
      extra = bic_extra,
      attrs = list(inference_method = attr(bic, "inference", exact = TRUE)$method)
    )
  )

  dc <- dcor(X, p_value = TRUE)
  dc_extra <- function(i, j, inf, diagnostics) {
    list(
      statistic = if (is.matrix(inf$statistic) && is.finite(inf$statistic[i, j])) round(inf$statistic[i, j], 4) else NA_real_,
      df = if (is.matrix(inf$parameter) && is.finite(inf$parameter[i, j])) round(inf$parameter[i, j], 4) else NA_real_,
      p_value = if (is.matrix(inf$p_value) && is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], 4) else NA_real_
    )
  }
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_dcor_pairwise_summary(dc),
    reference_pairwise_summary(
      dc,
      "summary.dcor",
      ci_digits = NULL,
      include_ci = FALSE,
      include_p = TRUE,
      extra = dc_extra,
      attrs = list(inference_method = attr(dc, "inference", exact = TRUE)$method)
    )
  )
})

test_that("shared robust and partial summaries preserve extra columns", {
  set.seed(20260523)
  X <- matrix(rnorm(72), nrow = 18, ncol = 4)
  colnames(X) <- paste0("R", seq_len(ncol(X)))

  robust_extra <- function(i, j, inf, diagnostics) {
    list(
      statistic = if (is.finite(inf$statistic[i, j])) round(inf$statistic[i, j], 4) else NA_real_,
      p_value = if (is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], 4) else NA_real_
    )
  }

  pb <- pbcor(X, ci = TRUE, p_value = TRUE, n_boot = 20L, seed = 1L)
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_pbcor_pairwise_summary(pb),
    reference_pairwise_summary(
      pb,
      "summary.pbcor",
      include_p = TRUE,
      extra = robust_extra,
      attrs = list(
        inference_method = attr(pb, "inference", exact = TRUE)$method,
        n_boot = attr(pb, "n_boot", exact = TRUE)
      )
    )
  )

  win <- wincor(X, ci = TRUE, p_value = TRUE, n_boot = 20L, seed = 2L)
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_wincor_pairwise_summary(win),
    reference_pairwise_summary(
      win,
      "summary.wincor",
      include_p = TRUE,
      extra = robust_extra,
      attrs = list(
        inference_method = attr(win, "inference", exact = TRUE)$method,
        n_boot = attr(win, "n_boot", exact = TRUE)
      )
    )
  )

  pc <- pcorr(X, method = "sample", ci = TRUE, return_p_value = TRUE, return_details = TRUE)
  pc_extra <- function(i, j, inf, diagnostics) {
    list(p_value = pc$p_value[i, j])
  }
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_partial_corr_pairwise_summary(pc),
    reference_pairwise_summary(
      pc$pcor,
      "summary.partial_corr",
      p_digits = NULL,
      ci = pc$ci,
      diagnostics = pc$diagnostics,
      include_p = NULL,
      extra = pc_extra,
      overview = matrixCorr:::.mc_summary_corr_matrix(pc$pcor)
    )
  )

  skipped <- skipped_corr(
    X,
    ci = TRUE,
    p_value = TRUE,
    n_boot = 20L,
    n_mc = 20L,
    seed = 3L
  )
  skip_extra <- function(i, j, inf, diagnostics) {
    out <- list(
      p_value = if (!is.null(inf$p_value) && is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], 4) else NA_real_
    )
    if (!is.null(inf$p_value_adjusted)) {
      out$p_value_adjusted <- if (is.finite(inf$p_value_adjusted[i, j])) round(inf$p_value_adjusted[i, j], 4) else NA_real_
    }
    if (!is.null(inf$reject)) out$reject <- isTRUE(inf$reject[i, j])
    if (is.matrix(diagnostics$skipped_n)) out$skipped_n <- as.integer(diagnostics$skipped_n[i, j])
    if (is.matrix(diagnostics$skipped_prop)) out$skipped_prop <- round(diagnostics$skipped_prop[i, j], 4)
    out
  }
  expect_pairwise_summary_equal(
    matrixCorr:::.mc_skipcor_pairwise_summary(skipped),
    reference_pairwise_summary(
      skipped,
      "summary.skipped_corr",
      ci_digits = 2,
      include_p = TRUE,
      extra = skip_extra,
      attrs = list(
        inference_method = attr(skipped, "inference", exact = TRUE)$method,
        p_adjust = attr(skipped, "inference", exact = TRUE)$p_adjust,
        critical_p_value = attr(skipped, "inference", exact = TRUE)$critical_p_value %||% NA_real_
      )
    )
  )
})

test_that("matrix previews truncate rows and columns without mutating objects", {
  set.seed(1)
  X <- matrix(rnorm(20 * 12), nrow = 20, ncol = 12)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  obj <- pearson_corr(X)
  original <- unclass(obj)

  txt <- capture.output(print(obj, n = 8, topn = 2, max_vars = 4, show_ci = "no"))

  expect_true(any(grepl("Pearson correlation matrix", txt, fixed = TRUE)))
  expect_true(any(grepl("... 8 more rows not shown", txt, fixed = TRUE)))
  expect_true(any(grepl("... 8 more variables not shown", txt, fixed = TRUE)))
  expect_true(any(grepl("Use summary() for a richer digest.", txt, fixed = TRUE)))
  expect_equal(unclass(obj), original)
})

test_that("summary output remains bounded and preserves full underlying data", {
  set.seed(2)
  X <- matrix(rnorm(30 * 12), nrow = 30, ncol = 12)
  colnames(X) <- paste0("S", seq_len(ncol(X)))

  obj <- spearman_rho(X)
  sm <- summary(obj, topn = 6)
  txt <- capture.output(print(sm, n = 10, topn = 3, max_vars = 6))

  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "summary.corr_matrix")
  expect_equal(nrow(sm$top_results), 6L)
  expect_true(any(grepl("^Correlation summary$", txt)))
  expect_false(any(grepl("^Strongest pairs by \\|estimate\\|$", txt)))
  expect_true(any(grepl("\\bitem1\\b", txt)))
  expect_true(any(grepl("\\bestimate\\b", txt)))
  expect_true(any(grepl("Use as.data.frame()/tidy()/as.matrix() to inspect the full result.", txt, fixed = TRUE)))
})

test_that("corr_result compact summary table respects print digits", {
  set.seed(20260428)
  X <- matrix(rnorm(60), nrow = 20, ncol = 3)
  colnames(X) <- c("A", "B", "C")

  sm <- summary(pearson_corr(X, ci = TRUE))
  txt <- capture.output(print(sm, digits = 2, topn = 3))

  expect_false(any(grepl("^Strongest pairs by \\|estimate\\|$", txt)))
  expect_true(any(grepl("95% CI", txt, fixed = TRUE)))
  table_block <- txt[grep("\\bitem1\\b", txt):length(txt)]
  expect_false(any(grepl("\\b-?\\d+\\.\\d{3,}\\b", table_block)))
})

test_that("summary numeric columns align signed and unsigned estimates", {
  df <- data.frame(
    item1 = c("x", "square", "sine"),
    item2 = c("sine", "x", "noise"),
    estimate = c(0.9603, -0.4668, 0.0550),
    stringsAsFactors = FALSE
  )

  out <- matrixCorr:::`.mc_format_summary_numeric_table`(df, digits = 4)

  expect_identical(out$estimate, c(" 0.9603", "-0.4668", " 0.0550"))
})

test_that("width-aware truncation responds to width and max_vars overrides", {
  set.seed(3)
  X <- matrix(rnorm(25 * 10), nrow = 25, ncol = 10)
  colnames(X) <- paste0("LongVariable", seq_len(ncol(X)))

  obj <- kendall_tau(X)

  txt_narrow <- capture.output(print(obj, width = 40, max_vars = NULL, show_ci = "no"))
  txt_wide <- capture.output(print(obj, width = 220, max_vars = NULL, show_ci = "no"))
  txt_override <- capture.output(print(obj, width = 220, max_vars = 4, show_ci = "no"))

  expect_true(any(grepl("more variables not shown", txt_narrow, fixed = TRUE)))
  expect_false(any(grepl("more variables not shown", txt_wide, fixed = TRUE)))
  expect_true(any(grepl("... 6 more variables not shown", txt_override, fixed = TRUE)))
})

test_that("binary show_ci controls bounded CI digest display for CI-enabled summaries", {
  set.seed(4)
  X <- matrix(rnorm(40 * 5), nrow = 40, ncol = 5)
  colnames(X) <- paste0("P", seq_len(ncol(X)))

  obj <- pearson_corr(X, ci = TRUE)
  sm <- summary(obj, show_ci = "yes")

  txt_yes <- capture.output(print(sm, show_ci = "yes"))
  txt_no <- capture.output(print(sm, show_ci = "no"))

  expect_true(any(grepl("^\\s*ci\\s*:\\s*95%", txt_yes)))
  expect_true(any(grepl("^\\s*ci_width\\s*:", txt_yes)))
  expect_false(any(grepl("^\\s*ci\\s*:\\s*95%", txt_no)))
  expect_false(any(grepl("^\\s*ci_width\\s*:", txt_no)))
  expect_false(any(grepl("\\blwr\\b", txt_no)))
  expect_false(any(grepl("\\bupr\\b", txt_no)))
})

test_that("display helpers reject auto and invalid display arguments", {
  set.seed(5)
  X <- matrix(rnorm(30 * 6), nrow = 30, ncol = 6)
  colnames(X) <- paste0("T", seq_len(ncol(X)))

  obj <- spearman_rho(X, ci = TRUE)
  sm <- summary(obj, show_ci = "yes")

  expect_error(print(obj, show_ci = "auto"), class = "matrixCorr_arg_error")
  expect_error(summary(obj, show_ci = "auto"), class = "matrixCorr_arg_error")
  expect_error(print(sm, show_ci = "auto"), class = "matrixCorr_arg_error")

  expect_error(print(obj, n = 1), class = "matrixCorr_arg_error")
  expect_error(print(obj, topn = 0), class = "matrixCorr_arg_error")
  expect_error(print(obj, max_vars = 0), class = "matrixCorr_arg_error")
  expect_error(print(obj, width = 0), class = "matrixCorr_arg_error")
  expect_warning(capture.output(print(obj, n = 6, topn = 4)), class = "matrixCorr_display_warning")
  expect_no_warning(capture.output(print(obj, n = 6)))
})

test_that("package display options control defaults", {
  set.seed(6)
  X <- matrix(rnorm(30 * 12), nrow = 30, ncol = 12)
  colnames(X) <- paste0("O", seq_len(ncol(X)))

  obj <- pearson_corr(X, ci = TRUE)
  old <- options(
    matrixCorr.print_max_rows = 8L,
    matrixCorr.print_topn = 2L,
    matrixCorr.print_max_vars = 4L,
    matrixCorr.print_show_ci = "no"
  )
  on.exit(options(old), add = TRUE)

  txt <- capture.output(print(obj))

  expect_true(any(grepl("... 8 more rows not shown", txt, fixed = TRUE)))
  expect_true(any(grepl("... 8 more variables not shown", txt, fixed = TRUE)))
  expect_false(any(grepl("Pearson correlation summary", txt, fixed = TRUE)))
})

test_that("non-correlation S3 classes use the same bounded preview philosophy", {
  set.seed(7)
  x <- rnorm(40)
  y <- x + rnorm(40, sd = 0.5)

  fit <- ba(x, y)

  txt_print <- capture.output(print(fit, show_ci = "no"))
  txt_summary <- capture.output(print(summary(fit), show_ci = "no"))

  expect_true(any(grepl("^Bland-Altman preview:", txt_print)))
  expect_true(any(grepl("^Bland-Altman \\(two methods\\)$", txt_summary)))
  expect_false(any(grepl("bias_lwr", txt_summary, fixed = TRUE)))
  expect_false(any(grepl("bias_upr", txt_summary, fixed = TRUE)))
})

test_that("CI-enabled correlation print methods keep a single primary body", {
  set.seed(8)
  X <- matrix(rnorm(60), nrow = 20, ncol = 3)
  colnames(X) <- c("A", "B", "C")

  p_txt <- capture.output(print(pearson_corr(X, ci = TRUE), show_ci = "yes"))
  s_txt <- capture.output(print(spearman_rho(X, ci = TRUE), show_ci = "yes"))
  k_txt <- capture.output(print(kendall_tau(X, ci = TRUE), show_ci = "yes"))

  expect_false(any(grepl("Pearson correlation summary", p_txt, fixed = TRUE)))
  expect_false(any(grepl("Spearman correlation summary", s_txt, fixed = TRUE)))
  expect_false(any(grepl("Kendall correlation summary", k_txt, fixed = TRUE)))
})

test_that("plot methods honour show_value for numeric overlays", {
  skip_if_not_installed("ggplot2")

  geom_names <- function(p) vapply(p$layers, function(layer) class(layer$geom)[1], character(1))

  set.seed(9)
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  p1 <- plot(pearson_corr(X, ci = TRUE), show_value = FALSE)
  p2 <- plot(pbcor(X), show_value = FALSE)
  p3 <- plot(shrinkage_corr(X), show_value = FALSE)
  p4 <- plot(shrinkage_corr(X), show_value = TRUE)

  expect_false(any(geom_names(p1) == "GeomText"))
  expect_false(any(geom_names(p2) == "GeomText"))
  expect_false(any(geom_names(p3) == "GeomText"))
  expect_true(any(geom_names(p4) == "GeomText"))
})

test_that("na_method complete matches manual listwise deletion", {
  set.seed(1)
  X <- matrix(rnorm(25 * 5), 25, 5)
  X[sample(length(X), 8)] <- NA_real_
  colnames(X) <- paste0("V", 1:5)
  Xcc <- X[stats::complete.cases(X) & apply(is.finite(X), 1L, all), , drop = FALSE]

  funs <- list(
    pearson_corr = function(z, na_method) pearson_corr(z, na_method = na_method, ci = FALSE),
    spearman_rho = function(z, na_method) spearman_rho(z, na_method = na_method, ci = FALSE),
    kendall_tau = function(z, na_method) kendall_tau(z, na_method = na_method, ci = FALSE),
    dcor = function(z, na_method) dcor(z, na_method = na_method, p_value = FALSE),
    robust_dcor = function(z, na_method) robust_dcor(z, na_method = na_method, p_value = FALSE),
    wincor = function(z, na_method) wincor(z, na_method = na_method, ci = FALSE, p_value = FALSE),
    pbcor = function(z, na_method) pbcor(z, na_method = na_method, ci = FALSE, p_value = FALSE),
    bicor = function(z, na_method) bicor(z, na_method = na_method, ci = FALSE),
    skipped_corr = function(z, na_method) skipped_corr(z, na_method = na_method, ci = FALSE, p_value = FALSE)
  )

  for (nm in names(funs)) {
    R_complete <- funs[[nm]](X, "complete")
    R_manual <- funs[[nm]](Xcc, "error")
    mat_complete <- unclass(as.matrix(R_complete))
    mat_manual <- unclass(as.matrix(R_manual))
    attributes(mat_complete) <- attributes(mat_complete)[c("dim", "dimnames")]
    attributes(mat_manual) <- attributes(mat_manual)[c("dim", "dimnames")]
    expect_equal(
      mat_complete,
      mat_manual,
      tolerance = 1e-8,
      info = nm
    )
  }
})

test_that("complete diagnostics record the common sample", {
  set.seed(1)
  X <- matrix(rnorm(25 * 5), 25, 5)
  X[sample(length(X), 8)] <- NA_real_
  colnames(X) <- paste0("V", 1:5)
  Xcc <- X[stats::complete.cases(X) & apply(is.finite(X), 1L, all), , drop = FALSE]

  R <- pearson_corr(X, na_method = "complete")
  diag <- attr(R, "diagnostics")

  expect_equal(diag$na_method, "complete")
  expect_equal(diag$n_original, nrow(X))
  expect_equal(diag$n_complete, nrow(Xcc))
  expect_equal(diag$n_removed, nrow(X) - nrow(Xcc))
  expect_true(diag$common_sample)
})

test_that("complete-case handling works for integer matrices with NA", {
  X <- matrix(c(1L, NA_integer_, 3L, 4L, 5L, 6L), ncol = 2)

  res <- pearson_corr(X, na_method = "complete")
  expected <- stats::cor(X[stats::complete.cases(X), , drop = FALSE])

  expect_equal(estimate(res), expected)
  expect_equal(attr(res, "diagnostics")$n_complete, 2L)
})

test_that("error and pairwise behaviours remain available", {
  set.seed(2)
  X <- matrix(rnorm(50), 10, 5)
  colnames(X) <- paste0("V", 1:5)
  X_na <- X
  X_na[1, 1] <- NA_real_

  expect_equal(
    pearson_corr(X, na_method = "error"),
    pearson_corr(X, na_method = "error")
  )
  R_pairwise <- pearson_corr(X_na, na_method = "pairwise")
  expect_true(!is.null(attr(R_pairwise, "diagnostics")$n_complete))
})

test_that("complete errors when too few rows remain", {
  X_bad <- matrix(NA_real_, 5, 3)
  X_bad[1, ] <- c(1, 2, 3)

  expect_error(
    pearson_corr(X_bad, na_method = "complete"),
    "complete rows"
  )
})

test_that("pcorr complete matches manual listwise deletion", {
  set.seed(3)
  X <- matrix(rnorm(25 * 5), 25, 5)
  X[sample(length(X), 8)] <- NA_real_
  colnames(X) <- paste0("V", 1:5)
  Xcc <- X[stats::complete.cases(X) & apply(is.finite(X), 1L, all), , drop = FALSE]

  R_complete <- pcorr(X, na_method = "complete")
  R_manual <- pcorr(Xcc, na_method = "error")

  expect_equal(
    unclass(estimate(R_complete)),
    unclass(estimate(R_manual)),
    tolerance = 1e-8
  )
})

#' @title Pairwise Spearman's rank correlation
#'
#' @description
#' Computes pairwise Spearman's rank correlations for the numeric columns of a
#' matrix or data frame using a high-performance 'C++' backend. Optional
#' confidence intervals are available via a jackknife Euclidean-likelihood
#' method.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns will be excluded. Each column must have
#' at least two non-missing values.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each correlation on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#' jackknife Euclidean-likelihood confidence intervals for the off-diagonal
#' Spearman correlations.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return A symmetric numeric matrix where the \code{(i, j)}-th element is
#' the Spearman correlation between the \code{i}-th and \code{j}-th
#' numeric columns of the input. When \code{ci = TRUE}, the object also
#' carries a \code{ci} attribute with elements \code{est}, \code{lwr.ci},
#' \code{upr.ci}, and \code{conf.level}. When pairwise-complete evaluation is
#' used, pairwise sample sizes are stored in \code{attr(x, "diagnostics")$n_complete}.
#'
#' @details
#' For each column \eqn{j=1,\ldots,p}, let
#' \eqn{R_{\cdot j} \in \{1,\ldots,n\}^n} denote the (mid-)ranks of
#' \eqn{X_{\cdot j}}, assigning average ranks to ties. The mean rank is
#' \eqn{\bar R_j = (n+1)/2} regardless of ties. Define the centred rank
#' vectors \eqn{\tilde R_{\cdot j} = R_{\cdot j} - \bar R_j \mathbf{1}},
#' where \eqn{\mathbf{1}\in\mathbb{R}^n} is the all-ones vector. The
#' Spearman correlation between columns \eqn{i} and \eqn{j} is the Pearson
#' correlation of their rank vectors:
#' \deqn{
#' \rho_S(i,j) \;=\;
#' \frac{\sum_{k=1}^n (R_{ki}-\bar R_i)(R_{kj}-\bar R_j)}
#'      {\sqrt{\sum_{k=1}^n (R_{ki}-\bar R_i)^2}\;
#'       \sqrt{\sum_{k=1}^n (R_{kj}-\bar R_j)^2}}.
#' }
#' In matrix form, with \eqn{R=[R_{\cdot 1},\ldots,R_{\cdot p}]},
#' \eqn{\mu=(n+1)\mathbf{1}_p/2} for \eqn{\mathbf{1}_p\in\mathbb{R}^p}, and
#' \eqn{S_R=\bigl(R-\mathbf{1}\mu^\top\bigr)^\top
#'            \bigl(R-\mathbf{1}\mu^\top\bigr)/(n-1)},
#' the Spearman correlation matrix is
#' \deqn{
#' \widehat{\rho}_S \;=\; D^{-1/2} S_R D^{-1/2}, \qquad
#' D \;=\; \mathrm{diag}(\mathrm{diag}(S_R)).
#' }
#' When there are no ties, the familiar rank-difference formula obtains
#' \deqn{
#' \rho_S(i,j) \;=\; 1 - \frac{6}{n(n^2-1)} \sum_{k=1}^n d_k^2,
#' \quad d_k \;=\; R_{ki}-R_{kj},
#' }
#' but this expression does \emph{not} hold under ties; computing Pearson on
#' mid-ranks (as above) is the standard tie-robust approach. Without ties,
#' \eqn{\mathrm{Var}(R_{\cdot j})=(n^2-1)/12}; with ties, the variance is
#' smaller.
#'
#' \eqn{\rho_S(i,j) \in [-1,1]} and \eqn{\widehat{\rho}_S} is symmetric
#' positive semi-definite by construction (up to floating-point error). The
#' implementation symmetrises the result to remove round-off asymmetry.
#' Spearman's correlation is invariant to strictly monotone transformations
#' applied separately to each variable.
#'
#' \strong{Computation.} Each column is ranked (mid-ranks) to form \eqn{R}.
#' The product \eqn{R^\top R} is computed via a 'BLAS' symmetric rank update
#' ('SYRK'), and centred using
#' \deqn{
#' (R-\mathbf{1}\mu^\top)^\top (R-\mathbf{1}\mu^\top)
#' \;=\; R^\top R \;-\; n\,\mu\mu^\top,
#' }
#' avoiding an explicit centred copy. Division by \eqn{n-1} yields the sample
#' covariance of ranks; standardising by \eqn{D^{-1/2}} gives \eqn{\widehat{\rho}_S}.
#' Columns with zero rank variance (all values equal) are returned as \code{NA}
#' along their row/column; the corresponding diagonal entry is also \code{NA}.
#'
#' When \code{na_method = "pairwise"}, each \eqn{(i,j)} estimate is recomputed
#' on the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}. When
#' \code{ci = TRUE}, confidence intervals are computed in 'C++' using the
#' jackknife Euclidean-likelihood method of de Carvalho and Marques (2012).
#' For a pairwise estimate \eqn{U = \hat\rho_S}, delete-one jackknife
#' pseudo-values are formed as
#' \deqn{
#' Z_i = nU - (n-1)U_{(-i)}, \qquad i = 1,\ldots,n,
#' }
#' where \eqn{U_{(-i)}} is the Spearman correlation after removing observation
#' \eqn{i}. The confidence limits solve
#' \deqn{
#' \frac{n(U-\theta)^2}{n^{-1}\sum_{i=1}^n (Z_i - \theta)^2}
#' = \chi^2_{1,\;\texttt{conf\_level}}.
#' }
#'
#' Ranking costs
#' \eqn{O\!\bigl(p\,n\log n\bigr)}; forming and normalising
#' \eqn{R^\top R} costs \eqn{O\!\bigl(n p^2\bigr)} with \eqn{O(p^2)} additional
#' memory. The optional jackknife Euclidean-likelihood confidence intervals add
#' per-pair delete-one recomputation work and are intended for inference rather
#' than raw-matrix throughput.
#'
#' @note Missing values are rejected when \code{na_method = "error"}. Columns
#' with fewer than two usable observations are excluded.
#'
#' @references
#' Spearman, C. (1904). The proof and measurement of association between
#' two things. International Journal of Epidemiology, 39(5), 1137-1150.
#'
#' de Carvalho, M., & Marques, F. (2012). Jackknife Euclidean
#' likelihood-based inference for Spearman's rho. North American Actuarial
#' Journal, 16(4), 487-492.
#'
#' @examples
#' ## Monotone transformation invariance (Spearman is rank-based)
#' set.seed(123)
#' n <- 400; p <- 6; rho <- 0.6
#' Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
#' L <- chol(Sigma)
#' X <- matrix(rnorm(n * p), n, p) %*% L
#' colnames(X) <- paste0("V", seq_len(p))
#'
#' X_mono <- X
#' X_mono[, 1] <- exp(X_mono[, 1])
#' X_mono[, 2] <- log1p(exp(X_mono[, 2]))
#' X_mono[, 3] <- X_mono[, 3]^3
#'
#' sp_X <- spearman_rho(X)
#' sp_m <- spearman_rho(X_mono)
#' summary(sp_X)
#' round(max(abs(sp_X - sp_m)), 3)
#' plot(sp_X)
#'
#' ## Confidence intervals
#' sp_ci <- spearman_rho(X[, 1:3], ci = TRUE)
#' print(sp_ci, show_ci = "yes")
#' summary(sp_ci)
#' estimate(sp_ci)
#' tidy(sp_ci)
#' ci(sp_ci)
#' confint(sp_ci)
#'
#' ## Ties handled via mid-ranks
#' tied <- cbind(
#'   a = rep(1:5, each = 20),
#'   b = rep(5:1, each = 20) + rnorm(100, sd = 0.1),
#'   c = as.numeric(gl(10, 10))
#' )
#' sp_tied <- spearman_rho(tied, ci = TRUE)
#' print(sp_tied, digits = 2, show_ci = "yes")
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(sp_X)
#' }
#' @useDynLib matrixCorr, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @seealso \code{\link{print.spearman_rho}}, \code{\link{plot.spearman_rho}}
#' @author Thiago de Paula Oliveira
#' @export
spearman_rho <- function(data,
                         na_method = c("error", "pairwise", "complete"),
                         ci = FALSE,
                         conf_level = 0.95,
                         n_threads = getOption("matrixCorr.threads", 1L),
                         output = c("matrix", "sparse", "edge_list"),
                         threshold = 0,
                         diag = TRUE,
                         ...) {
  .mc_corr_wrapper(
    data = data,
    dots = list(...),
    na_method = na_method,
    na_method_missing = missing(na_method),
    ci = ci,
    conf_level = conf_level,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag,
    estimator_class = "spearman_rho",
    method = "spearman",
    description = "Pairwise Spearman's rank correlation matrix",
    kernel_matrix = spearman_matrix_cpp,
    kernel_pairwise = spearman_matrix_pairwise_cpp,
    kernel_threshold = spearman_threshold_triplets_cpp,
    min_n = 2L,
    symmetric = TRUE,
    thresholded_output = TRUE
  )
}

.mc_spearman_pairwise_summary <- function(object,
                                          digits = 4,
                                          ci_digits = 3,
                                          show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "spearman_rho")

  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.spearman_rho",
    digits = digits,
    ci_digits = ci_digits,
    show_ci = show_ci,
    include_p = NULL
  )
}

#' @rdname spearman_rho
#' @method print spearman_rho
#' @title Print Method for \code{spearman_rho} Objects
#'
#' @param x An object of class \code{spearman_rho}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits for Spearman confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print}.
#'
#' @return Invisibly returns the \code{spearman_rho} object.
#' @export
print.spearman_rho <- function(x,
                               digits = 4,
                               n = NULL,
                               topn = NULL,
                               max_vars = NULL,
                               width = NULL,
                               ci_digits = 3,
                               show_ci = NULL,
                               ...) {
  .mc_print_corr_matrix(
    x,
    header = "Spearman correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname spearman_rho
#' @method plot spearman_rho
#' @title Plot Method for \code{spearman_rho} Objects
#'
#' @param x An object of class \code{spearman_rho}.
#' @param title Plot title. Default is \code{"Spearman's rank correlation
#' heatmap"}.
#' @param low_color Color for the minimum rho value. Default is
#'  \code{"indianred1"}.
#' @param high_color Color for the maximum rho value. Default is
#' \code{"steelblue1"}.
#' @param mid_color Color for zero correlation. Default is \code{"white"}.
#' @param value_text_size Font size for displaying correlation values. Default
#' is \code{4}.
#' @param ci_text_size Text size for confidence intervals in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other
#' \code{ggplot2} layers.
#'
#' @return A \code{ggplot} object representing the heatmap.
#' @import ggplot2
#' @export
plot.spearman_rho <-
  function(x, title = "Spearman's rank correlation heatmap",
           low_color = "indianred1", high_color = "steelblue1",
           mid_color = "white", value_text_size = 4,
           ci_text_size = 3, show_value = TRUE, ...) {
    check_bool(show_value, arg = "show_value")
    ci <- .mc_ci_attr(x)
    if (is.null(ci) || is.null(ci$lwr.ci) || is.null(ci$upr.ci)) {
      return(.mc_plot_corr_matrix(
        x, class_name = "spearman_rho", fill_name = "Rho",
        title = title, low_color = low_color, high_color = high_color,
        mid_color = mid_color, value_text_size = value_text_size,
        show_value = show_value, ...
      ))
    }

    est_mat <- as.matrix(x)
    df_est <- as.data.frame(as.table(est_mat))
    names(df_est) <- c("Var1", "Var2", "rho")

    df_lwr <- as.data.frame(as.table(ci$lwr.ci))
    names(df_lwr)[3] <- "lwr"
    df_upr <- as.data.frame(as.table(ci$upr.ci))
    names(df_upr)[3] <- "upr"
    df <- Reduce(
      function(a, b) merge(a, b, by = c("Var1", "Var2"), all = TRUE),
      list(df_est, df_lwr, df_upr)
    )

    diag_idx <- df$Var1 == df$Var2
    df$lwr[diag_idx] <- NA_real_
    df$upr[diag_idx] <- NA_real_
    df$ci_label <- ifelse(
      is.na(df$lwr) | is.na(df$upr),
      NA_character_,
      sprintf("[%.3f, %.3f]", df$lwr, df$upr)
    )

    lev_row <- unique(df_est$Var1)
    lev_col <- unique(df_est$Var2)
    df$Var1 <- factor(df$Var1, levels = rev(lev_row))
    df$Var2 <- factor(df$Var2, levels = lev_col)
    df$label <- sprintf("%.2f", df$rho)

    p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = .data$rho)) +
      ggplot2::geom_tile(color = "white") +
      ggplot2::scale_fill_gradient2(
        low = low_color,
        high = high_color,
        mid = mid_color,
        midpoint = 0,
        limits = c(-1, 1),
        name = "Rho"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        panel.grid = ggplot2::element_blank(),
        ...
      ) +
      ggplot2::coord_fixed() +
      ggplot2::labs(title = title, x = NULL, y = NULL)

    if (isTRUE(show_value)) {
      p <- p + ggplot2::geom_text(ggplot2::aes(label = label), size = value_text_size, color = "black")
    }

    if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.25),
        size = ci_text_size,
        color = "gray30",
        na.rm = TRUE
      )
    }

    p
  }

#' @rdname spearman_rho
#' @method summary spearman_rho
#' @param object An object of class \code{spearman_rho}.
#' @param ci_digits Integer; digits for Spearman confidence limits in the
#'   pairwise summary.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
summary.spearman_rho <- function(object,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 ci_digits = 3,
                                 show_ci = NULL,
                                 ...) {
  check_inherits(object, "spearman_rho")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  if (is.null(.mc_ci_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_spearman_pairwise_summary(
    object,
    ci_digits = ci_digits,
    show_ci = show_ci
  )
}

#' @rdname spearman_rho
#' @method print summary.spearman_rho
#' @param x An object of class \code{summary.spearman_rho}.
#' @export
print.summary.spearman_rho <- function(x, digits = NULL, n = NULL,
                                       topn = NULL, max_vars = NULL,
                                       width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Spearman correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "jackknife_euclidean_likelihood",
    ...
  )
  invisible(x)
}

test_that("correlation outputs dispatch summary/plot by representation class", {
  skip_if_not_installed("ggplot2")

  set.seed(20260415)
  X <- matrix(rnorm(300 * 6), nrow = 300, ncol = 6)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  dense <- pearson_corr(X, ci = TRUE)
  expect_s3_class(dense, "corr_matrix")
  expect_s3_class(dense, "pearson_corr")
  expect_identical(attr(dense, "method", exact = TRUE), "pearson")
  expect_true(is.list(attr(dense, "ci", exact = TRUE)))

  sm_dense <- summary(dense)
  expect_s3_class(sm_dense, "summary.corr_result")
  expect_s3_class(sm_dense, "summary.corr_matrix")
  expect_identical(attr(sm_dense, "method", exact = TRUE), "pearson")
  expect_true(isTRUE(attr(sm_dense, "has_ci", exact = TRUE)))
  expect_false(inherits(sm_dense, "summaryDefault"))
  expect_s3_class(plot(dense, show_value = FALSE), "ggplot")

  sparse <- pearson_corr(X, output = "sparse", threshold = 0.30, diag = FALSE, ci = TRUE)
  expect_s4_class(sparse, "sparseMatrix")
  expect_identical(attr(sparse, "output", exact = TRUE), "sparse")
  expect_identical(attr(sparse, "method", exact = TRUE), "pearson")
  expect_true(is.list(attr(sparse, "ci", exact = TRUE)))

  sm_sparse <- summary(sparse)
  expect_s3_class(sm_sparse, "summary.corr_result")
  expect_s3_class(sm_sparse, "summary.corr_sparse")
  expect_identical(attr(sm_sparse, "output", exact = TRUE), "sparse")
  expect_true(isTRUE(attr(sm_sparse, "has_ci", exact = TRUE)))
  expect_false(inherits(sm_sparse, "summaryDefault"))
  expect_s3_class(plot(sparse, show_value = FALSE), "ggplot")

  expect_error(pearson_corr(X, output = "packed_upper", threshold = 0.30, diag = FALSE))

  edge <- pearson_corr(X, output = "edge_list", threshold = 0.30, diag = FALSE)
  expect_s3_class(edge, "corr_edge_list")
  expect_identical(attr(edge, "output", exact = TRUE), "edge_list")
  expect_identical(attr(edge, "method", exact = TRUE), "pearson")

  sm_edge <- summary(edge)
  expect_s3_class(sm_edge, "summary.corr_result")
  expect_s3_class(sm_edge, "summary.corr_edge_list")
  expect_identical(attr(sm_edge, "output", exact = TRUE), "edge_list")
  expect_false(inherits(sm_edge, "summaryDefault"))
  expect_s3_class(plot(edge, show_value = FALSE), "ggplot")
  edge_all <- pearson_corr(X, output = "edge_list", threshold = 0, diag = TRUE)
  edge_print <- capture.output(print(edge_all, n = 8, topn = 2))
  expect_true(any(grepl("V\\d+\\s+V\\d+", edge_print)))
  expect_false(any(grepl("^X(\\.|\\s)", edge_print)))
})

test_that("correlation summary objects keep standardized overview metadata", {
  set.seed(42)
  X <- matrix(rnorm(160 * 4), nrow = 160, ncol = 4)
  colnames(X) <- paste0("M", seq_len(ncol(X)))

  out <- list(
    pearson_corr(X),
    pearson_corr(X, output = "sparse", threshold = 0.25, diag = FALSE),
    pearson_corr(X, output = "edge_list", threshold = 0.25, diag = FALSE)
  )

  for (obj in out) {
    sm <- summary(obj)
    ov <- attr(sm, "overview", exact = TRUE)
    expect_true(is.list(ov))
    expect_identical(attr(sm, "method", exact = TRUE), "pearson")
    expect_true(attr(sm, "output", exact = TRUE) %in% c("matrix", "sparse", "edge_list"))
    expect_true(is.logical(attr(sm, "diag", exact = TRUE)))
    expect_true(is.numeric(attr(sm, "threshold", exact = TRUE)))
  }
})

test_that("sparse and edge-list summaries do not reconstruct dense matrices", {
  set.seed(43)
  X <- matrix(rnorm(180 * 5), nrow = 180, ncol = 5)
  X[, 2] <- X[, 1] + rnorm(180, sd = 0.05)
  X[, 3] <- -X[, 1] + rnorm(180, sd = 0.05)
  colnames(X) <- paste0("M", seq_len(ncol(X)))

  sparse <- pearson_corr(X, output = "sparse", threshold = 0.70, diag = FALSE)
  edge <- pearson_corr(X, output = "edge_list", threshold = 0.70, diag = FALSE)

  testthat::local_mocked_bindings(
    .mc_corr_as_dense_matrix = function(...) {
      stop("dense reconstruction should not be used", call. = FALSE)
    },
    .package = "matrixCorr"
  )

  sm_sparse <- summary(sparse)
  sm_edge <- summary(edge)

  expect_s3_class(sm_sparse, "summary.corr_sparse")
  expect_s3_class(sm_edge, "summary.corr_edge_list")
  expect_equal(attr(sm_sparse, "overview", exact = TRUE)$output, "sparse")
  expect_equal(attr(sm_edge, "overview", exact = TRUE)$output, "edge_list")
})

test_that("summary() dispatches CI-aware sparse outputs across correlation estimators", {
  set.seed(20260417)
  X <- matrix(rnorm(260 * 5), nrow = 260, ncol = 5)
  colnames(X) <- paste0("S", seq_len(ncol(X)))

  methods <- list(
    pearson_corr = function(...) pearson_corr(X, ci = TRUE, ...),
    spearman_rho = function(...) spearman_rho(X, ci = TRUE, ...),
    kendall_tau = function(...) kendall_tau(X, ci = TRUE, ...)
  )

  for (nm in names(methods)) {
    fn <- methods[[nm]]

    dense <- fn(output = "matrix")
    sm_dense <- summary(dense)
    expect_s3_class(sm_dense, "summary.corr_result")
    expect_s3_class(sm_dense, "summary.corr_matrix")
    expect_true(isTRUE(attr(sm_dense, "has_ci", exact = TRUE)), info = paste(nm, "dense"))

    sparse <- fn(output = "sparse", threshold = 0.20, diag = FALSE)
    sm_sparse <- summary(sparse)
    expect_s3_class(sm_sparse, "summary.corr_result")
    expect_s3_class(sm_sparse, "summary.corr_sparse")
    expect_true(isTRUE(attr(sm_sparse, "has_ci", exact = TRUE)), info = paste(nm, "sparse"))
    expect_false(inherits(sm_sparse, "summaryDefault"), info = paste(nm, "sparse"))

    edge <- fn(output = "edge_list", threshold = 0.20, diag = FALSE)
    sm_edge <- summary(edge)
    expect_s3_class(sm_edge, "summary.corr_result")
    expect_s3_class(sm_edge, "summary.corr_edge_list")
    expect_true(isTRUE(attr(sm_edge, "has_ci", exact = TRUE)), info = paste(nm, "edge_list"))
  }
})

test_that("corr_result heatmaps show CI labels when ci=TRUE for all output modes", {
  skip_if_not_installed("ggplot2")

  set.seed(20260418)
  x1 <- rnorm(260)
  x2 <- x1 + rnorm(260, sd = 0.15)
  x3 <- -x1 + rnorm(260, sd = 0.15)
  x4 <- rnorm(260)
  X <- cbind(x1 = x1, x2 = x2, x3 = x3, x4 = x4)

  count_text_layers <- function(p) {
    sum(vapply(p$layers, function(layer) inherits(layer$geom, "GeomText"), logical(1)))
  }

  matrix_fit <- pearson_corr(X, ci = TRUE)
  sparse_fit <- pearson_corr(X, output = "sparse", threshold = 0.6, diag = FALSE, ci = TRUE)
  edge_fit <- pearson_corr(X, output = "edge_list", threshold = 0.6, diag = FALSE, ci = TRUE)

  gp_matrix <- plot(matrix_fit, show_value = TRUE)
  gp_sparse <- plot(sparse_fit, show_value = TRUE)
  gp_edge <- plot(edge_fit, show_value = TRUE)

  expect_gte(count_text_layers(gp_matrix), 2L)
  expect_gte(count_text_layers(gp_sparse), 2L)
  expect_gte(count_text_layers(gp_edge), 2L)
})

test_that("viewer helpers coerce package outputs", {
  skip_if_not_installed("stats")

  pearson <- pearson_corr(mtcars)
  spearman <- spearman_rho(mtcars)
  distance <- dcor(mtcars)
  partial <- pcorr(mtcars, method = "ridge", lambda = 1e-2)
  kendall <- kendall_tau(mtcars)
  biweight <- bicor(mtcars)
  shrinkage <- shrinkage_corr(mtcars)

  res <- matrixCorr:::`.mc_prepare_corr_inputs`(list(
    Pearson = pearson,
    Spearman = spearman,
    Distance = distance,
    Partial = partial,
    Kendall = kendall,
    Biweight = biweight,
    Shrinkage = shrinkage
  ))

  expect_true(all(vapply(res, function(x) is.matrix(x$matrix), logical(1))))
  expect_equal(res$Distance$signed, FALSE)
  expect_true(res$Pearson$signed)
  expect_equal(colnames(res$Partial$matrix), colnames(estimate(partial)))
  expect_equal(res$Kendall$class, paste(class(kendall), collapse = ", "))
  expect_equal(res$Biweight$class, paste(class(biweight), collapse = ", "))
  expect_equal(res$Shrinkage$class, paste(class(shrinkage), collapse = ", "))
})

test_that("viewer helpers accept thresholded sparse and edge-list correlation outputs", {
  X <- mtcars
  sparse <- pearson_corr(X, output = "sparse", threshold = 0.4, diag = FALSE)
  edge <- pearson_corr(X, output = "edge_list", threshold = 0.4, diag = FALSE)

  parsed_sparse <- matrixCorr:::`.mc_prepare_corr_inputs`(sparse)
  parsed_edge <- matrixCorr:::`.mc_prepare_corr_inputs`(edge)

  expect_length(parsed_sparse, 1L)
  expect_length(parsed_edge, 1L)
  expect_true(is.matrix(parsed_sparse$default$matrix))
  expect_true(is.matrix(parsed_edge$default$matrix))
  expect_equal(dim(parsed_sparse$default$matrix), dim(as.matrix(pearson_corr(X))))
  expect_equal(dim(parsed_edge$default$matrix), dim(as.matrix(pearson_corr(X))))
  expect_true(any(is.finite(parsed_edge$default$matrix)))
  expect_true(any(is.na(parsed_edge$default$matrix)))
})

test_that("viewer helpers treat list-based CI outputs as single objects", {
  ci_obj <- ccc(mtcars[, 1:5], ci = TRUE)
  parsed <- matrixCorr:::`.mc_prepare_corr_inputs`(ci_obj)

  expect_length(parsed, 1L)
  expect_true("default" %in% names(parsed))
  expect_true(is.matrix(parsed$default$matrix))
  expect_equal(parsed$default$matrix, as.matrix(ci_obj$est), tolerance = 1e-12)
})

test_that("heatmap helper returns ggplot when plotly missing", {
  mat <- matrix(c(1, 0.4, 0.4, 1), nrow = 2)
  p <- matrixCorr:::`.mc_build_heatmap`(
    mat = mat,
    signed = TRUE,
    show_values = TRUE,
    use_abs = FALSE,
    use_plotly = FALSE
  )
  expect_s3_class(p, "ggplot")
})

test_that("distribution helpers use unique off-diagonal pairs", {
  mat <- matrix(
    c(1, 0.4, -0.2,
      0.4, 1, 0.1,
      -0.2, 0.1, 1),
    nrow = 3,
    byrow = TRUE
  )

  vals_signed <- matrixCorr:::`.mc_corr_distribution_values`(mat, use_abs = FALSE)
  vals_abs <- matrixCorr:::`.mc_corr_distribution_values`(mat, use_abs = TRUE)

  expect_equal(vals_signed, c(0.4, -0.2, 0.1))
  expect_equal(vals_abs, c(0.4, 0.2, 0.1))
})

test_that("distribution summary table reports visible pair statistics", {
  tbl <- matrixCorr:::`.mc_corr_summary_table`(c(0.4, -0.2, 0.1))

  expect_equal(tbl$Metric[[1]], "Pairs shown")
  expect_equal(tbl$Value[[1]], "3")
  expect_true("Median" %in% tbl$Metric)
  expect_true("Mean" %in% tbl$Metric)
})

test_that("distribution axis limits adapt to visible correlation range", {
  limits_signed <- matrixCorr:::`.mc_corr_axis_limits`(c(0.1, 0.3, 0.6), use_abs = FALSE)
  limits_abs <- matrixCorr:::`.mc_corr_axis_limits`(c(0.1, 0.3, 0.6), use_abs = TRUE)

  expect_true(limits_signed[[1]] > -1)
  expect_true(limits_signed[[2]] < 1)
  expect_true(limits_signed[[1]] <= 0.1)
  expect_true(limits_signed[[2]] >= 0.6)
  expect_true(limits_abs[[1]] >= 0)
  expect_true(limits_abs[[2]] <= 1)
  expect_true(limits_abs[[1]] <= 0.1)
  expect_true(limits_abs[[2]] >= 0.6)
})

test_that("matrix reorder helper handles absolute and signed modes", {
  mat <- matrix(c(1, 0.2, 0.8,
                  0.2, 1, 0.3,
                  0.8, 0.3, 1), nrow = 3, byrow = TRUE)
  res_abs <- matrixCorr:::`.mc_reorder_matrix`(mat, mode = "abs", method = "complete")
  expect_null(res_abs$message)
  expect_equal(dim(res_abs$matrix), dim(mat))
  expect_true(isSymmetric(res_abs$matrix, tol = 1e-12))
  expect_equal(diag(res_abs$matrix), diag(mat)[res_abs$order])
  expect_equal(sort(res_abs$order), seq_len(nrow(mat)))

  res_signed <- matrixCorr:::`.mc_reorder_matrix`(mat, mode = "signed", method = "average")
  expect_null(res_signed$message)
  expect_equal(dim(res_signed$matrix), dim(mat))
  expect_true(isSymmetric(res_signed$matrix, tol = 1e-12))
  expect_equal(diag(res_signed$matrix), diag(mat)[res_signed$order])
  expect_equal(sort(res_signed$order), seq_len(nrow(mat)))
})

test_that("correlation range filter subsets variables in signed mode", {
  mat <- matrix(
    c(1, 0.8, 0.1, -0.4,
      0.8, 1, 0.3, -0.2,
      0.1, 0.3, 1, 0.05,
      -0.4, -0.2, 0.05, 1),
    nrow = 4,
    byrow = TRUE,
    dimnames = list(c("A", "B", "C", "D"), c("A", "B", "C", "D"))
  )

  res <- matrixCorr:::`.mc_filter_matrix_by_range`(
    mat = mat,
    corr_range = c(0.25, 0.85),
    use_abs = FALSE
  )

  expect_equal(colnames(res$matrix), c("A", "B", "C"))
  expect_match(res$message, "kept 3 of 4 variables")
})

test_that("correlation range filter subsets variables in absolute mode", {
  mat <- matrix(
    c(1, 0.8, 0.1, -0.4,
      0.8, 1, 0.3, -0.2,
      0.1, 0.3, 1, 0.05,
      -0.4, -0.2, 0.05, 1),
    nrow = 4,
    byrow = TRUE,
    dimnames = list(c("A", "B", "C", "D"), c("A", "B", "C", "D"))
  )

  res <- matrixCorr:::`.mc_filter_matrix_by_range`(
    mat = mat,
    corr_range = c(0.35, 0.5),
    use_abs = TRUE
  )

  expect_equal(colnames(res$matrix), c("A", "D"))
  expect_match(res$message, "kept 2 of 4 variables")
})

test_that("correlation range helper clamps out-of-bounds ranges", {
  expect_equal(matrixCorr:::`.mc_clamp_corr_range`(c(-2, 0.4), use_abs = TRUE), c(0, 0.4))
  expect_equal(matrixCorr:::`.mc_clamp_corr_range`(c(0.8, -2), use_abs = FALSE), c(-1, 0.8))
})

test_that("viewer helpers resolve logo asset and sanitize filenames", {
  expect_true(file.exists(matrixCorr:::`.mc_logo_path`()))
  expect_equal(matrixCorr:::`.mc_sanitize_filename`("Repeated measures / matrix"), "Repeated-measures-matrix")
})

test_that("signed plotly palette keeps blue at positive and red at negative", {
  scale <- matrixCorr:::`.mc_plotly_colorscale`(TRUE)

  expect_equal(scale[[1]], list(0, "#FF6A6A"))
  expect_equal(scale[[2]], list(0.5, "white"))
  expect_equal(scale[[3]], list(1, "#63B8FF"))
})

test_that("heatmap click helpers recover selected variable pairs", {
  mat <- matrix(
    c(1, 0.4,
      0.4, 1),
    nrow = 2,
    byrow = TRUE,
    dimnames = list(c("A", "B"), c("A", "B"))
  )
  df <- matrixCorr:::`.mc_heatmap_df`(mat)

  expect_equal(
    matrixCorr:::`.mc_plotly_click_pair`(data.frame(x = "B", y = "A"), vars = c("A", "B")),
    c("B", "A")
  )

  expect_equal(
    matrixCorr:::`.mc_static_click_pair`(df, list(x = "B", y = "A")),
    c("B", "A")
  )
})

manual_spearman_jel_ci_R <- function(x, y, conf_level = 0.95) {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  n <- length(x)
  est <- suppressWarnings(stats::cor(x, y, method = "spearman"))

  out <- list(estimate = est, lwr = NA_real_, upr = NA_real_, n = n)
  if (!is.finite(est) || n < 3L) return(out)

  z <- vapply(seq_len(n), function(i) {
    suppressWarnings(n * est - (n - 1) * stats::cor(x[-i], y[-i], method = "spearman"))
  }, numeric(1))
  if (any(!is.finite(z))) return(out)

  s_u <- mean((z - est)^2)
  if (!is.finite(s_u)) return(out)
  if (s_u <= 1e-14) {
    out$lwr <- est
    out$upr <- est
    return(out)
  }

  crit <- stats::qchisq(conf_level, df = 1)
  g <- function(theta) {
    s_theta <- mean((z - theta)^2)
    if (!(s_theta > 0)) {
      return(if ((est - theta)^2 <= 1e-18) -crit else Inf)
    }
    n * (est - theta)^2 / s_theta - crit
  }

  if (est <= -1) {
    out$lwr <- est
  } else if (!is.finite(g(-1)) || g(-1) <= 0) {
    out$lwr <- -1
  } else {
    out$lwr <- uniroot(g, interval = c(-1, est), tol = 1e-11)$root
  }

  if (est >= 1) {
    out$upr <- est
  } else if (!is.finite(g(1)) || g(1) <= 0) {
    out$upr <- 1
  } else {
    out$upr <- uniroot(g, interval = c(est, 1), tol = 1e-11)$root
  }

  out
}

test_that("spearman_rho matches base::cor(..., method = 'spearman')", {
  for (i in 1:10) {
    n <- sample(c(10, 50, 100, 500), 1)
    p <- sample(2:6, 1)
    mat <- replicate(p, rnorm(n))
    colnames(mat) <- paste0("S", seq_len(p))

    base_cor <- cor(mat, method = "spearman")
    fast_cor <- suppressWarnings(spearman_rho(mat))

    attributes(fast_cor) <- NULL
    attributes(base_cor) <- NULL

    expect_equal(
      fast_cor,
      base_cor,
      tolerance = 1e-8,
      info = paste("Mismatch on Spearman test dataset", i, "n =", n, "p =", p)
    )
  }
})

test_that("spearman_rho handles ties correctly and matches base::cor", {
  for (i in 1:5) {
    n <- sample(c(50, 100, 200), 1)
    p <- sample(2:5, 1)

    mat <- replicate(p, sample(rep(1:5, length.out = n)))
    colnames(mat) <- paste0("T", seq_len(p))

    base_cor <- cor(mat, method = "spearman")
    fast_cor <- suppressWarnings(spearman_rho(mat))

    attributes(fast_cor) <- NULL
    attributes(base_cor) <- NULL

    expect_equal(
      fast_cor,
      base_cor,
      tolerance = 1e-8,
      info = paste("Mismatch on tied Spearman dataset", i, "n =", n, "p =", p)
    )
  }
})

test_that("spearman_rho is invariant to strictly monotone transformations", {
  set.seed(123)
  X <- matrix(rnorm(200), ncol = 2)
  X_mono <- X
  X_mono[,1] <- exp(X_mono[,1])     # monotone transform
  X_mono[,2] <- log1p(exp(X_mono[,2]))  # monotone transform

  base_cor <- cor(X, method = "spearman")
  fast_cor <- spearman_rho(X_mono)

  expect_equal(as.numeric(fast_cor),
               as.numeric(unname(base_cor)),
               tolerance = 1e-12)
})

test_that("spearman_rho returns NA when a column is constant", {
  X <- cbind(a = rnorm(20), b = rep(1, 20))
  sp <- spearman_rho(X)

  expect_true(all(is.na(sp["b", ])))
  expect_true(all(is.na(sp[, "b"])))
})

test_that("spearman_rho agrees with base::cor on known values (small matrix)", {
  X <- matrix(c(1, 2, 3, 4,
                1, 4, 9, 16), ncol = 2)
  colnames(X) <- c("x", "y")

  base_cor <- cor(X, method = "spearman")
  fast_cor <- spearman_rho(X)

  expect_equal(as.numeric(fast_cor),
               as.numeric(unname(base_cor)),
               tolerance = 1e-12)
})

test_that("theoretical BVN formula for Spearman holds approximately", {
  set.seed(321)
  rhos <- c(-0.8, -0.4, 0, 0.4, 0.8)
  n <- 2000

  for (r in rhos) {
    Sigma <- matrix(c(1, r, r, 1), 2, 2)
    Z <- MASS::mvrnorm(n, mu = c(0,0), Sigma = Sigma)

    est <- spearman_rho(Z)[1,2]
    theory <- (6/pi) * asin(r/2)

    expect_equal(est, theory, tolerance = 0.08,
                 info = paste("Mismatch for true rho =", r))
  }
})

test_that("spearman_rho print/plot methods cover options", {
  skip_if_not_installed("ggplot2")

  set.seed(101)
  X <- matrix(rnorm(40), nrow = 10, ncol = 4)
  colnames(X) <- paste0("S", seq_len(4))
  sp <- spearman_rho(X)

  out <- capture.output(print(sp, digits = 3, max_rows = 2, max_cols = 3))
  expect_true(any(grepl("omitted", out)))

  p <- plot(sp, title = "Spearman plot", low_color = "blue", high_color = "red", mid_color = "white", value_text_size = 3)
  expect_s3_class(p, "ggplot")
  scale <- p$scales$get_scales("fill")
  expect_equal(scale$limits, c(-1, 1))
})

test_that("spearman_rho rejects missing values when check_na = TRUE", {
  X <- cbind(a = c(1, 2, NA, 4), b = c(1, 2, 3, 4))
  expect_error(spearman_rho(X), "Missing values are not allowed.")
})

test_that("spearman_rho default CI behaviour remains estimate-only", {
  set.seed(2026)
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)
  colnames(X) <- paste0("S", 1:4)

  sp <- spearman_rho(X)
  sm <- summary(sp)

  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "summary.corr_matrix")
  expect_null(attr(sp, "ci", exact = TRUE))
  expect_null(attr(sp, "conf.level", exact = TRUE))
})

test_that("spearman_rho honors n_threads without changing estimates", {
  set.seed(654)
  X <- matrix(rnorm(240), nrow = 40, ncol = 6)
  colnames(X) <- paste0("S", seq_len(ncol(X)))

  fit1 <- spearman_rho(X, n_threads = 1L)
  fit2 <- spearman_rho(X, n_threads = 2L)
  fit1_ci <- spearman_rho(X, ci = TRUE, n_threads = 1L)
  fit2_ci <- spearman_rho(X, ci = TRUE, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(attr(fit1_ci, "ci", exact = TRUE), attr(fit2_ci, "ci", exact = TRUE), tolerance = 1e-12)
})

test_that("spearman_rho CI matches reference JEL implementation for positive association", {
  x <- c(1, 2, 3, 4, 5, 6, 7, 8)
  y <- c(1.0, 1.8, 2.9, 4.1, 4.9, 6.2, 7.1, 8.0)
  ref <- manual_spearman_jel_ci_R(x, y, conf_level = 0.95)
  sp <- spearman_rho(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(sp, "ci", exact = TRUE)

  expect_equal(unname(sp["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("spearman_rho CI matches reference JEL implementation for negative association", {
  x <- c(1, 2, 3, 4, 5, 6, 7, 8, 9)
  y <- c(9, 7, 8, 6, 5, 4, 3, 2, 1)
  ref <- manual_spearman_jel_ci_R(x, y, conf_level = 0.95)
  sp <- spearman_rho(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(sp, "ci", exact = TRUE)

  expect_equal(unname(sp["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("spearman_rho CI matches reference JEL implementation near zero", {
  x <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  y <- c(3, 8, 1, 9, 4, 10, 2, 7, 5, 6)
  ref <- manual_spearman_jel_ci_R(x, y, conf_level = 0.95)
  sp <- spearman_rho(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(sp, "ci", exact = TRUE)

  expect_equal(unname(sp["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("spearman_rho custom conf_level changes JEL interval width", {
  x <- 1:20
  y <- c(1, 3, 2, 4, 5, 7, 6, 8, 10, 9, 11, 13, 12, 14, 15, 17, 16, 18, 20, 19)

  sp90 <- spearman_rho(cbind(x = x, y = y), ci = TRUE, conf_level = 0.90)
  sp99 <- spearman_rho(cbind(x = x, y = y), ci = TRUE, conf_level = 0.99)

  ci90 <- attr(sp90, "ci", exact = TRUE)
  ci99 <- attr(sp99, "ci", exact = TRUE)
  width90 <- ci90$upr.ci["x", "y"] - ci90$lwr.ci["x", "y"]
  width99 <- ci99$upr.ci["x", "y"] - ci99$lwr.ci["x", "y"]

  expect_equal(attr(sp90, "conf.level", exact = TRUE), 0.90)
  expect_equal(attr(sp99, "conf.level", exact = TRUE), 0.99)
  expect_true(width99 > width90)
})

test_that("spearman_rho CI works for pairwise-complete input and reports n_complete", {
  X <- cbind(
    x = c(1, 2, 3, 4, 5, 6, NA, 8),
    y = c(2, 1, 4, 3, 6, 5, 7, NA),
    z = c(8, 7, 6, 5, 4, NA, 2, 1)
  )

  sp <- spearman_rho(X, na_method = "pairwise", ci = TRUE)
  diag_attr <- attr(sp, "diagnostics", exact = TRUE)
  ci <- attr(sp, "ci", exact = TRUE)
  ref_xy <- manual_spearman_jel_ci_R(X[, "x"], X[, "y"], conf_level = 0.95)

  expect_identical(diag_attr$n_complete["x", "y"], as.integer(ref_xy$n))
  expect_equal(unname(sp["x", "y"]), ref_xy$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref_xy$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref_xy$upr, tolerance = 1e-8)
})

test_that("spearman_rho CI matches reference implementation with ties and small sample", {
  x <- c(1, 1, 2, 2, 3, 3)
  y <- c(1, 2, 2, 3, 3, 4)
  ref <- manual_spearman_jel_ci_R(x, y, conf_level = 0.95)
  sp <- spearman_rho(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(sp, "ci", exact = TRUE)

  expect_equal(unname(sp["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("spearman_rho CI-aware print, summary, and plot follow the CI contract", {
  skip_if_not_installed("ggplot2")

  X <- cbind(
    A = c(1, 2, 3, 4, 5, 6, 7),
    B = c(1, 3, 2, 4, 6, 5, 7),
    C = c(7, 6, 5, 4, 3, 2, 1)
  )

  sp <- spearman_rho(X, ci = TRUE)
  sm <- summary(sp)
  txt_print <- capture.output(print(sp, show_ci = "yes"))
  txt_summary <- capture.output(print(sm))
  p <- plot(sp, ci_text_size = 2)

  expect_s3_class(sm, "summary.spearman_rho")
  expect_equal(nrow(sm), choose(ncol(X), 2))
  expect_true(all(c("item1", "item2", "estimate", "n_complete", "lwr", "upr") %in% names(sm)))
  expect_match(paste(txt_print, collapse = "\n"), "Spearman correlation matrix")
  expect_false(any(grepl("Spearman correlation summary", txt_print, fixed = TRUE)))
  expect_match(paste(txt_summary, collapse = "\n"), "Spearman correlation summary")
  expect_match(paste(txt_summary, collapse = "\n"), "jackknife_euclidean_likelihood")
  expect_match(paste(txt_summary, collapse = "\n"), "\\bn\\b")
  expect_s3_class(p, "ggplot")
})

manual_kendall_fieller_ci_R <- function(x, y, conf_level = 0.95) {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  n <- length(x)
  tau <- suppressWarnings(stats::cor(x, y, method = "kendall"))

  out <- list(estimate = tau, lwr = NA_real_, upr = NA_real_, n = n)
  if (!is.finite(tau) || n <= 4L) return(out)

  z <- atanh(max(min(tau, 1 - 1e-15), -1 + 1e-15))
  crit <- stats::qnorm(0.5 * (1 + conf_level))
  se <- sqrt(0.437 / (n - 4))
  out$lwr <- max(-1, tanh(z - crit * se))
  out$upr <- min(1, tanh(z + crit * se))
  out
}

manual_kendall_brown_benedetti_ci_R <- function(x, y, conf_level = 0.95) {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  n <- length(x)

  out <- list(estimate = NA_real_, lwr = NA_real_, upr = NA_real_, n = n)
  if (n < 2L) return(out)

  tab <- table(x, y)
  if (nrow(tab) < 2L || ncol(tab) < 2L) return(out)

  pi_c <- matrix(0, nrow = nrow(tab), ncol = ncol(tab))
  pi_d <- matrix(0, nrow = nrow(tab), ncol = ncol(tab))

  for (i in seq_len(nrow(tab))) {
    i_lt <- if (i > 1L) seq_len(i - 1L) else integer(0)
    i_gt <- if (i < nrow(tab)) seq.int(i + 1L, nrow(tab)) else integer(0)
    for (j in seq_len(ncol(tab))) {
      j_lt <- if (j > 1L) seq_len(j - 1L) else integer(0)
      j_gt <- if (j < ncol(tab)) seq.int(j + 1L, ncol(tab)) else integer(0)
      pi_c[i, j] <- sum(tab[i_lt, j_lt, drop = FALSE]) +
        sum(tab[i_gt, j_gt, drop = FALSE])
      pi_d[i, j] <- sum(tab[i_lt, j_gt, drop = FALSE]) +
        sum(tab[i_gt, j_lt, drop = FALSE])
    }
  }

  C <- sum(pi_c * tab) / 2
  D <- sum(pi_d * tab) / 2
  n0 <- n * (n - 1) / 2
  ti <- rowSums(tab)
  uj <- colSums(tab)
  n1 <- sum(ti * (ti - 1) / 2)
  n2 <- sum(uj * (uj - 1) / 2)
  den <- sqrt((n0 - n1) * (n0 - n2))
  if (!(den > 0)) return(out)

  tau_b <- (C - D) / den
  pi <- tab / sum(tab)
  pdiff <- (pi_c - pi_d) / sum(tab)
  Pdiff <- 2 * (C - D) / sum(tab)^2
  rowsum <- rowSums(pi)
  colsum <- colSums(pi)
  rowmat <- matrix(rep(rowsum, ncol(tab)), ncol = ncol(tab))
  colmat <- matrix(rep(colsum, nrow(tab)), nrow = nrow(tab), byrow = TRUE)
  delta1 <- sqrt(1 - sum(rowsum^2))
  delta2 <- sqrt(1 - sum(colsum^2))
  if (!(delta1 > 0) || !(delta2 > 0)) {
    out$estimate <- tau_b
    return(out)
  }

  tauphi <- (2 * pdiff + Pdiff * colmat) * delta2 * delta1 +
    (Pdiff * rowmat * delta2) / delta1
  sigma2 <- ((sum(pi * tauphi^2) - sum(pi * tauphi)^2) /
    (delta1 * delta2)^4) / n
  if (sigma2 < .Machine$double.eps * 10) sigma2 <- 0
  if (!is.finite(sigma2) || sigma2 < 0) {
    out$estimate <- tau_b
    return(out)
  }

  crit <- stats::qnorm(1 - (1 - conf_level) / 2)
  ci <- tau_b + crit * sqrt(sigma2) * c(-1, 1)
  out$estimate <- tau_b
  out$lwr <- max(ci[1], -1)
  out$upr <- min(ci[2], 1)
  out
}

manual_kendall_ifel_ci_R <- function(x, y, conf_level = 0.95) {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  n <- length(x)
  tau <- suppressWarnings(stats::cor(x, y, method = "kendall"))

  out <- list(estimate = tau, lwr = NA_real_, upr = NA_real_, n = n)
  if (!is.finite(tau) || n < 3L) return(out)

  s_sum <- numeric(n)
  a_sum <- numeric(n)
  b_sum <- numeric(n)
  s_total <- 0
  a_total <- 0
  b_total <- 0

  for (i in seq_len(n - 1L)) {
    for (j in (i + 1L):n) {
      sx <- sign(x[i] - x[j])
      sy <- sign(y[i] - y[j])
      s_ij <- sx * sy
      a_ij <- as.numeric(sx != 0)
      b_ij <- as.numeric(sy != 0)

      s_sum[i] <- s_sum[i] + s_ij
      s_sum[j] <- s_sum[j] + s_ij
      a_sum[i] <- a_sum[i] + a_ij
      a_sum[j] <- a_sum[j] + a_ij
      b_sum[i] <- b_sum[i] + b_ij
      b_sum[j] <- b_sum[j] + b_ij

      s_total <- s_total + s_ij
      a_total <- a_total + a_ij
      b_total <- b_total + b_ij
    }
  }

  if (!(a_total > 0) || !(b_total > 0)) return(out)

  n0 <- n * (n - 1) / 2
  tau_hat <- s_total / sqrt(a_total * b_total)
  s_bar <- s_total / n0
  a_bar <- a_total / n0
  b_bar <- b_total / n0
  if (!(a_bar > 0) || !(b_bar > 0)) return(out)

  den <- sqrt(a_bar * b_bar)
  s_i <- s_sum / (n - 1)
  a_i <- a_sum / (n - 1)
  b_i <- b_sum / (n - 1)
  psi <- 2 * ((s_i - s_bar) / den -
    0.5 * tau_hat * ((a_i - a_bar) / a_bar + (b_i - b_bar) / b_bar))
  w <- tau_hat + psi

  if (!all(is.finite(w))) return(out)
  spread <- mean((w - tau_hat)^2)
  if (!is.finite(spread)) return(out)
  if (spread <= 1e-14) {
    out$estimate <- tau_hat
    out$lwr <- tau_hat
    out$upr <- tau_hat
    return(out)
  }

  elr <- function(theta) {
    v <- w - theta
    if (max(abs(v)) <= 1e-12) return(0)
    if (!(min(v) < 0 && max(v) > 0)) return(Inf)

    lower <- max(-1 / v[v > 0]) + 1e-12
    upper <- min(-1 / v[v < 0]) - 1e-12
    g <- function(lambda) sum(v / (1 + lambda * v))
    lambda <- if (abs(sum(v)) <= 1e-12) {
      0
    } else {
      stats::uniroot(g, interval = c(lower, upper), tol = 1e-11)$root
    }
    den_l <- 1 + lambda * v
    if (any(den_l <= 0)) return(Inf)
    2 * sum(log(den_l))
  }

  crit <- stats::qchisq(conf_level, df = 1)
  f <- function(theta) elr(theta) - crit

  left_edge <- max(-1, min(w))
  right_edge <- min(1, max(w))

  if (tau_hat <= left_edge + 1e-10) {
    out$lwr <- left_edge
  } else {
    left_start <- min(tau_hat, left_edge + 1e-10)
    if (!is.finite(f(left_start)) || f(left_start) > 0) {
      out$lwr <- stats::uniroot(f, interval = c(left_start, tau_hat), tol = 1e-11)$root
    } else {
      out$lwr <- left_edge
    }
  }

  if (tau_hat >= right_edge - 1e-10) {
    out$upr <- right_edge
  } else {
    right_start <- max(tau_hat, right_edge - 1e-10)
    if (!is.finite(f(right_start)) || f(right_start) > 0) {
      out$upr <- stats::uniroot(f, interval = c(tau_hat, right_start), tol = 1e-11)$root
    } else {
      out$upr <- right_edge
    }
  }

  out$estimate <- tau_hat
  out
}

test_that("kendall_tau matches base::cor(..., method = 'kendall')", {
  for (i in 1:10) {
    n <- sample(c(10, 50, 100, 500), 1)
    p <- sample(2:6, 1)
    mat <- replicate(p, rnorm(n))
    colnames(mat) <- paste0("V", seq_len(p))

    base_cor <- cor(mat, method = "kendall")
    fast_cor <- suppressWarnings(kendall_tau(mat))

    attributes(fast_cor) <- NULL
    attributes(base_cor) <- NULL

    expect_equal(
      fast_cor,
      base_cor,
      tolerance = 1e-8,
      info = paste("Mismatch on test dataset", i, "n =", n, "p =", p)
    )
  }
})

test_that("kendall_tau handles ties correctly and matches base::cor", {
  for (i in 1:5) {
    n <- sample(c(50, 100, 200), 1)
    p <- sample(2:5, 1)

    mat <- replicate(p, sample(rep(1:5, length.out = n)))
    colnames(mat) <- paste0("T", seq_len(p))

    base_cor <- cor(mat, method = "kendall")
    fast_cor <- suppressWarnings(kendall_tau(mat))

    attributes(fast_cor) <- NULL
    attributes(base_cor) <- NULL

    expect_equal(
      fast_cor,
      base_cor,
      tolerance = 1e-8,
      info = paste("Mismatch on tied test dataset", i, "n =", n, "p =", p)
    )
  }
})

test_that("kendall_tau numerics match stats::cor(Kendall)", {
  set.seed(123)

  X <- matrix(rnorm(200), ncol = 2)
  X_mono <- cbind(exp(X[, 1]), log1p(exp(X[, 2])))

  base_cor <- unname(cor(X, method = "kendall"))

  fast_raw <- kendall_tau(X_mono)
  fast_cor <- unname(as.matrix(fast_raw))
  class(fast_cor) <- NULL
  dimnames(fast_cor) <- NULL

  expect_equal(dim(fast_cor), dim(base_cor))
  expect_true(isSymmetric(matrix(fast_cor, nrow = nrow(base_cor))))
  expect_equal(diag(fast_cor), rep(1, ncol(X)))

  expect_equal(as.numeric(fast_cor),
    as.numeric(unname(cor(X, method = "kendall"))),
    tolerance = 1e-12
  )

  x <- X[, 1]
  y <- X[, 2]
  base12 <- cor(x, y, method = "kendall")
  fast12 <- as.numeric(as.matrix(kendall_tau(cbind(x, y)))[1, 2])
  expect_equal(fast12, base12, tolerance = 1e-12)

  y_rev <- -y
  expect_equal(
    as.numeric(as.matrix(kendall_tau(cbind(x, y_rev)))[1, 2]),
    -fast12,
    tolerance = 1e-12
  )
})

test_that("kendall_tau honors n_threads without changing estimates", {
  set.seed(987)
  X <- matrix(rnorm(240), nrow = 40, ncol = 6)
  colnames(X) <- paste0("K", seq_len(ncol(X)))

  fit1 <- kendall_tau(X, n_threads = 1L)
  fit2 <- kendall_tau(X, n_threads = 2L)
  fit1_ci <- kendall_tau(X, ci = TRUE, n_threads = 1L)
  fit2_ci <- kendall_tau(X, ci = TRUE, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(attr(fit1_ci, "ci", exact = TRUE), attr(fit2_ci, "ci", exact = TRUE), tolerance = 1e-12)
})

test_that("kendall_tau two-vector mode returns a scalar matching base::cor", {
  set.seed(456)
  x <- rnorm(300)
  y <- -0.4 * x + rnorm(300)
  kt <- kendall_tau(x, y)
  expect_type(kt, "double")
  expect_length(kt, 1L)
  expect_equal(
    kt,
    cor(x, y, method = "kendall"),
    tolerance = 1e-12
  )
})

test_that("kendall_tau returns NA when a column is constant", {
  X <- cbind(a = rnorm(20), b = rep(1, 20))
  kt <- kendall_tau(X)

  expect_true(all(is.na(kt["b", 1])))
  expect_true(all(is.na(kt[1, "b"])))
})

test_that("kendall_tau matches base::cor on a known toy dataset", {
  X <- matrix(c(
    1, 2, 3, 4,
    4, 3, 2, 1
  ), ncol = 2)
  colnames(X) <- c("x", "y")

  base_cor <- cor(X, method = "kendall")
  fast_cor <- kendall_tau(X)

  expect_equal(as.numeric(fast_cor),
    as.numeric(unname(base_cor)),
    tolerance = 1e-12
  )
})

test_that("kendall_tau rejects NA input in two-column mode", {
  X <- cbind(a = c(1, 2, NA, 4), b = c(0, 1, 2, 3))
  expect_error(kendall_tau(X), "Missing values are not allowed.")
})

test_that("kendall_tau estimates agree with theoretical BVN relationship", {
  set.seed(321)
  rhos <- c(-0.8, -0.4, 0, 0.4, 0.8)
  n <- 2000

  for (r in rhos) {
    Sigma <- matrix(c(1, r, r, 1), 2, 2)
    Z <- MASS::mvrnorm(n, mu = c(0, 0), Sigma = Sigma)

    est <- kendall_tau(Z)[1, 2]
    theory <- (2 / pi) * asin(r)

    expect_equal(est, theory, tolerance = 0.08,
      info = paste("Mismatch for true rho =", r)
    )
  }
})

test_that("kendall_tau default CI behaviour remains estimate-only", {
  set.seed(2026)
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)
  colnames(X) <- paste0("K", 1:4)

  kt <- kendall_tau(X)
  sm <- summary(kt)

  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "summary.corr_matrix")
  expect_null(attr(kt, "ci", exact = TRUE))
  expect_null(attr(kt, "conf.level", exact = TRUE))
  expect_null(attr(kt, "ci.method", exact = TRUE))
})

test_that("kendall_tau rejects CI requests in two-vector mode", {
  expect_error(
    kendall_tau(1:6, 6:1, ci = TRUE),
    "Confidence intervals are not available"
  )
})

test_that("kendall_tau Fieller CI matches reference calculation for positive association", {
  x <- c(1, 2, 3, 4, 5, 6, 7, 8)
  y <- c(1.1, 2.2, 2.8, 4.0, 5.1, 5.9, 7.0, 8.3)
  ref <- manual_kendall_fieller_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-10)
  expect_identical(ci$ci.method, "fieller")
})

test_that("kendall_tau Fieller CI matches reference calculation for negative association", {
  x <- c(1, 2, 3, 4, 5, 6, 7, 8)
  y <- c(8, 7, 6, 5, 4, 2, 3, 1)
  ref <- manual_kendall_fieller_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-10)
})

test_that("kendall_tau Fieller CI matches reference calculation near zero", {
  x <- 1:10
  y <- c(2, 7, 1, 10, 4, 9, 3, 8, 6, 5)
  ref <- manual_kendall_fieller_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE)
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-10)
})

test_that("kendall_tau custom conf_level changes Fieller interval width", {
  x <- 1:20
  y <- c(1, 3, 2, 4, 5, 7, 6, 8, 10, 9, 11, 13, 12, 14, 15, 17, 16, 18, 20, 19)

  kt90 <- kendall_tau(cbind(x = x, y = y), ci = TRUE, conf_level = 0.90)
  kt99 <- kendall_tau(cbind(x = x, y = y), ci = TRUE, conf_level = 0.99)

  ci90 <- attr(kt90, "ci", exact = TRUE)
  ci99 <- attr(kt99, "ci", exact = TRUE)
  width90 <- ci90$upr.ci["x", "y"] - ci90$lwr.ci["x", "y"]
  width99 <- ci99$upr.ci["x", "y"] - ci99$lwr.ci["x", "y"]

  expect_equal(attr(kt90, "conf.level", exact = TRUE), 0.90)
  expect_equal(attr(kt99, "conf.level", exact = TRUE), 0.99)
  expect_true(width99 > width90)
})

test_that("kendall_tau Fieller CI handles ties and pairwise-complete sample sizes", {
  X <- cbind(
    x = c(1, 1, 2, 2, 3, 3, NA, 4),
    y = c(1, 2, 2, 3, 3, 4, 4, NA),
    z = c(4, 4, 3, 3, 2, NA, 1, 1)
  )

  kt <- kendall_tau(X, na_method = "pairwise", ci = TRUE)
  diag_attr <- attr(kt, "diagnostics", exact = TRUE)
  ci <- attr(kt, "ci", exact = TRUE)
  ref_xy <- manual_kendall_fieller_ci_R(X[, "x"], X[, "y"], conf_level = 0.95)

  expect_identical(diag_attr$n_complete["x", "y"], as.integer(ref_xy$n))
  expect_equal(unname(kt["x", "y"]), ref_xy$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref_xy$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref_xy$upr, tolerance = 1e-10)
})

test_that("kendall_tau Brown-Benedetti CI matches reference calculation for tied positive association", {
  x <- c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5)
  y <- c(1, 2, 2, 3, 3, 4, 4, 5, 5, 5)
  ref <- manual_kendall_brown_benedetti_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "brown_benedetti")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-10)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-10)
  expect_identical(ci$ci.method, "brown_benedetti")
  expect_identical(attr(kt, "ci.method", exact = TRUE), "brown_benedetti")
})

test_that("kendall_tau Brown-Benedetti CI matches reference calculation for tied negative association", {
  x <- c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5)
  y <- c(5, 5, 4, 4, 3, 3, 2, 2, 1, 1)
  ref <- manual_kendall_brown_benedetti_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "brown_benedetti")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-10)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-10)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-10)
})

test_that("kendall_tau Brown-Benedetti custom conf_level changes interval width", {
  x <- c(rep(1:6, each = 2), 7, 7, 8, 8)
  y <- c(1, 1, 2, 3, 3, 4, 4, 4, 5, 6, 6, 7, 7, 8, 8, 8)

  kt90 <- kendall_tau(cbind(x = x, y = y), ci = TRUE, conf_level = 0.90, ci_method = "brown_benedetti")
  kt99 <- kendall_tau(cbind(x = x, y = y), ci = TRUE, conf_level = 0.99, ci_method = "brown_benedetti")

  ci90 <- attr(kt90, "ci", exact = TRUE)
  ci99 <- attr(kt99, "ci", exact = TRUE)
  width90 <- ci90$upr.ci["x", "y"] - ci90$lwr.ci["x", "y"]
  width99 <- ci99$upr.ci["x", "y"] - ci99$lwr.ci["x", "y"]

  expect_equal(attr(kt90, "conf.level", exact = TRUE), 0.90)
  expect_equal(attr(kt99, "conf.level", exact = TRUE), 0.99)
  expect_true(width99 > width90)
})

test_that("kendall_tau IF-EL CI matches reference implementation for positive association", {
  x <- 1:8
  y <- c(1.0, 2.2, 3.1, 4.0, 5.0, 5.7, 7.2, 8.0)
  ref <- manual_kendall_ifel_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "if_el")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
  expect_identical(ci$ci.method, "if_el")
  expect_identical(attr(kt, "ci.method", exact = TRUE), "if_el")
})

test_that("kendall_tau IF-EL CI matches reference implementation for negative association", {
  x <- 1:9
  y <- c(9, 8, 7, 5, 6, 4, 3, 2, 1)
  ref <- manual_kendall_ifel_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "if_el")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("kendall_tau IF-EL CI matches reference implementation near zero", {
  x <- 1:10
  y <- c(3, 9, 1, 10, 5, 7, 4, 8, 2, 6)
  ref <- manual_kendall_ifel_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "if_el")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("kendall_tau IF-EL CI handles ties and small samples", {
  x <- c(1, 1, 2, 2, 3, 3)
  y <- c(1, 2, 2, 3, 3, 4)
  ref <- manual_kendall_ifel_ci_R(x, y, conf_level = 0.95)
  kt <- kendall_tau(cbind(x = x, y = y), ci = TRUE, ci_method = "if_el")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_equal(unname(kt["x", "y"]), ref$estimate, tolerance = 1e-12)
  expect_equal(unname(ci$lwr.ci["x", "y"]), ref$lwr, tolerance = 1e-8)
  expect_equal(unname(ci$upr.ci["x", "y"]), ref$upr, tolerance = 1e-8)
})

test_that("kendall_tau IF-EL returns NA bounds when the interval cannot be formed", {
  X <- cbind(
    x = c(1, 2),
    y = c(2, 1)
  )

  kt <- kendall_tau(X, ci = TRUE, ci_method = "if_el")
  ci <- attr(kt, "ci", exact = TRUE)

  expect_true(is.na(ci$lwr.ci["x", "y"]))
  expect_true(is.na(ci$upr.ci["x", "y"]))
})

test_that("kendall_tau CI-aware print, summary, and plot follow the CI contract", {
  skip_if_not_installed("ggplot2")

  X <- cbind(
    A = c(1, 2, 3, 4, 5, 6, 7),
    B = c(1, 3, 2, 4, 6, 5, 7),
    C = c(7, 6, 5, 4, 3, 2, 1)
  )

  kt <- kendall_tau(X, ci = TRUE, ci_method = "fieller")
  sm <- summary(kt)
  txt_print <- capture.output(print(kt, show_ci = "yes"))
  txt_summary <- capture.output(print(sm))
  p <- plot(kt, ci_text_size = 2)

  expect_s3_class(sm, "summary.kendall_matrix")
  expect_equal(nrow(sm), choose(ncol(X), 2))
  expect_true(all(c("item1", "item2", "estimate", "n_complete", "lwr", "upr") %in% names(sm)))
  expect_match(paste(txt_print, collapse = "\n"), "Kendall correlation matrix")
  expect_false(any(grepl("Kendall correlation summary", txt_print, fixed = TRUE)))
  expect_match(paste(txt_summary, collapse = "\n"), "Kendall correlation summary")
  expect_match(paste(txt_summary, collapse = "\n"), "fieller")
  expect_match(paste(txt_summary, collapse = "\n"), "\\bn\\b")
  expect_s3_class(p, "ggplot")
})

test_that("kendall_tau CI-enabled objects keep the same outer contract as other methods", {
  X <- cbind(
    A = c(1, 2, 3, 4, 5, 6),
    B = c(1, 2, 4, 3, 5, 6),
    C = c(6, 5, 4, 3, 2, 1)
  )

  k_fieller <- kendall_tau(X, ci = TRUE, ci_method = "fieller")
  k_bb <- kendall_tau(X, ci = TRUE, ci_method = "brown_benedetti")
  k_ifel <- kendall_tau(X, ci = TRUE, ci_method = "if_el")
  p <- pearson_corr(X, ci = TRUE)
  s <- spearman_rho(X, ci = TRUE)

  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level") %in% names(attr(k_fieller, "ci", exact = TRUE))))
  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level") %in% names(attr(k_bb, "ci", exact = TRUE))))
  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level") %in% names(attr(p, "ci", exact = TRUE))))
  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level") %in% names(attr(s, "ci", exact = TRUE))))
  expect_equal(names(summary(k_fieller)), names(summary(k_bb)))
  expect_equal(names(summary(k_fieller)), names(summary(k_ifel)))
})

#' @title Pairwise Cohen's Kappa for Nominal Ratings
#'
#' @description
#' Computes unweighted Cohen's kappa for either a pair of nominal rating
#' vectors or all pairwise combinations of nominal columns in a matrix or data
#' frame.
#'
#' @param data In matrix mode, a matrix or data frame whose rows are
#'   observational units and whose columns are raters or classifiers. Supported
#'   column types are factor, ordered factor, character, logical, integer, and
#'   numeric, all treated as nominal categories here. If the ratings are truly
#'   ordinal and disagreements should be weighted by distance, use
#'   [weighted_kappa()] instead. In two-vector mode, the first nominal rating
#'   vector.
#' @param y Optional second nominal rating vector. When supplied, the function
#'   returns a single Cohen's kappa estimate for \code{data} and \code{y}.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"complete"} applies listwise
#'   deletion across retained columns before computing the matrix. In matrix
#'   mode, \code{"pairwise"} uses pair-specific complete observations.
#' @param ci Logical; if \code{TRUE}, attach large-sample delta-method
#'   confidence intervals.
#' @param p_value Logical; if \code{TRUE}, attach large-sample delta-method
#'   Wald test statistics and p-values.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#'   \code{0.95}.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   matrix C++ backend. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for matrix mode.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries after thresholding.
#'   \item \code{"edge_list"}: long-form data frame representation.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   retain entries with \code{abs(value) >= threshold}. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in sparse and
#'   edge-list outputs.
#' @param ... Reserved for future extensions. Unsupported extra arguments are
#'   rejected.
#'
#' @details
#' Cohen's kappa is an agreement coefficient for two raters assigning the same
#' units to mutually exclusive nominal categories. For contingency-table cell
#' proportions \eqn{p_{ij}},
#' \deqn{
#' \kappa = \frac{p_o - p_e}{1 - p_e},
#' }
#' where \eqn{p_o = \sum_i p_{ii}} is the observed agreement and
#' \eqn{p_e = \sum_i p_{i+} p_{+i}} is the chance agreement implied by the
#' marginal category proportions.
#'
#' This implementation is strictly the original unweighted nominal-scale Cohen's
#' kappa. If the categories are ordinal and near disagreements should count as
#' less severe than distant disagreements, use [weighted_kappa()] instead.
#'
#' In matrix mode, columns are treated as raters/classifiers and rows as shared
#' observational units. All pairwise Cohen's kappas between columns are
#' computed. Category labels are encoded in R before dispatch to C++, using a
#' common label mapping across all columns so that identical labels in different
#' columns correspond to the same category code.
#'
#' Missing values are encoded as \code{NA_integer_} before entering the C++
#' backend. With \code{na_method = "error"}, missing values are rejected before
#' computation. With \code{na_method = "complete"}, listwise deletion is
#' applied across retained columns. With \code{na_method = "pairwise"}, each
#' pair uses its own complete observations. Pairwise complete counts are stored
#' in \code{attr(x, "diagnostics")$n_complete}.
#'
#' \strong{Confidence intervals and standard errors.}
#' The implementation uses the exact large-sample formula. Let \eqn{p_{ab}}
#' be the empirical cell proportions,
#' \eqn{q_a = \sum_b p_{ab}} the row margins, \eqn{r_b = \sum_a p_{ab}} the
#' column margins, and \eqn{D = 1 - p_e}. For each cell \eqn{(a,b)}, define
#' the influence contribution
#' \deqn{ g_{ab}
#'       \;=\;
#'       \frac{\mathbf{1}(a=b)\,D + (p_o - 1)\,(r_a + q_b)}{D^2}. }
#' The variance estimator used by the code is
#' \deqn{ \widehat{\mathrm{Var}}(\hat\kappa)
#'       \;=\;
#'       \frac{1}{n}\left(
#'       \sum_{a,b} p_{ab} g_{ab}^2 -
#'       \Big(\sum_{a,b} p_{ab} g_{ab}\Big)^2
#'       \right), }
#' with \eqn{n} the number of complete paired ratings and
#' \eqn{\widehat{\mathrm{se}}(\hat\kappa)=\sqrt{\widehat{\mathrm{Var}}(\hat\kappa)}}.
#' The confidence interval is the Wald interval
#' \deqn{ \hat\kappa \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa), }
#' truncated to \eqn{[-1, 1]} in the returned result.
#'
#' @return
#' If \code{y} is supplied, a scalar S3 object of class \code{"cohen_kappa"}
#' backed by a numeric value, with attributes \code{diagnostics}, and
#' optionally \code{ci}, \code{inference}, and \code{conf.level}. Otherwise a
#' symmetric matrix-style result with estimator class \code{cohen_kappa}.
#'
#' @references
#' Cohen, J. (1960). A coefficient of agreement for nominal scales.
#' \emph{Educational and Psychological Measurement}, 20(1), 37-46.
#' \doi{10.1177/001316446002000104}
#'
#' @seealso
#' [weighted_kappa()] for two-rater ordered-category agreement with
#' distance-sensitive disagreement weights; [multirater_kappa()] for
#' panel-level nominal agreement among three or more raters.
#'
#' @examples
#' x <- factor(c("A", "A", "B", "B", "A", "B"))
#' y <- factor(c("A", "B", "B", "B", "A", "A"))
#' cohen_kappa(x, y)
#'
#' raters <- data.frame(
#'   r1 = factor(c("low", "low", "high", "high", "mid")),
#'   r2 = factor(c("low", "mid", "high", "high", "mid")),
#'   r3 = c("low", "low", "high", "mid", "mid")
#' )
#' ck <- cohen_kappa(raters)
#' print(ck)
#' summary(ck)
#' estimate(ck)
#' tidy(ck)
#' plot(ck)
#'
#' @author Thiago de Paula Oliveira
#' @export
cohen_kappa <- function(data,
                        y = NULL,
                        na_method = c("error", "pairwise", "complete"),
                        ci = FALSE,
                        p_value = FALSE,
                        conf_level = 0.95,
                        n_threads = getOption("matrixCorr.threads", 1L),
                        output = c("matrix", "sparse", "edge_list"),
                        threshold = 0,
                        diag = TRUE,
                        ...) {
  .mc_extract_legacy_aliases(list(...), allowed = character())
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )
  na_method <- validate_na_method(
    na_method,
    arg = "na_method",
    allowed = c("error", "pairwise", "complete")
  )
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg y} is supplied."
      )
    }
    x_labels <- .mc_nominal_vector_labels(data, arg = "data")
    y_labels <- .mc_nominal_vector_labels(y, arg = "y")
    check_same_length(x_labels, y_labels, arg_x = "data", arg_y = "y")

    if (identical(na_method, "error") && (anyNA(x_labels) || anyNA(y_labels))) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must not contain missing values when {.arg na_method} is {.val error}.",
        .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to drop incomplete pairs."
      )
    }

    if (!identical(na_method, "error")) {
      keep <- stats::complete.cases(x_labels, y_labels)
      x_labels <- x_labels[keep]
      y_labels <- y_labels[keep]
    }

    enc <- .mc_encode_nominal_pair_labels(x_labels, y_labels)
    fit <- cohen_kappa_pair_cpp(
      enc$x,
      enc$y,
      n_levels = enc$n_levels,
      return_inference = isTRUE(ci) || isTRUE(p_value),
      conf_level = conf_level
    )
    return(.mc_cohen_kappa_scalar(
      fit,
      ci = ci,
      p_value = p_value,
      conf_level = conf_level
    ))
  }

  enc <- .mc_extract_nominal_columns(data, arg = "data", min_cols = 2L)
  X <- enc$matrix
  col_names <- enc$names
  dn <- .mc_square_dimnames(col_names)
  n_levels <- enc$n_levels

  listwise_meta <- NULL
  if (identical(na_method, "error") && anyNA(X)) {
    abort_bad_arg(
      "data",
      message = "contains missing values when {.arg na_method} is {.val error}.",
      .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to handle incomplete units."
    )
  }
  if (identical(na_method, "complete")) {
    keep <- stats::complete.cases(X)
    listwise_meta <- list(
      na_method = "complete",
      n_original = nrow(X),
      n_complete_rows = sum(keep),
      n_removed = nrow(X) - sum(keep),
      complete_rows = keep,
      common_sample = TRUE
    )
    X <- X[keep, , drop = FALSE]
  }

  direct_threshold <- !isTRUE(ci) &&
    !isTRUE(p_value) &&
    !identical(na_method, "pairwise") &&
    isTRUE(output_cfg$thresholded)

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  if (direct_threshold) {
    trip <- cohen_kappa_threshold_triplets_cpp(
      X,
      n_levels = n_levels,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      n_threads = n_threads
    )
    n_complete_mat <- matrix(
      as.integer(nrow(X)),
      nrow = ncol(X),
      ncol = ncol(X),
      dimnames = dn
    )
    diag(n_complete_mat) <- colSums(!is.na(X))
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = "cohen_kappa",
      method = "cohen_kappa",
      description = "Pairwise Cohen's kappa agreement matrix",
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = as.integer(c(ncol(X), ncol(X))),
      source_dimnames = dn,
      diagnostics = .mc_merge_diagnostics(
        list(n_complete = n_complete_mat),
        listwise_meta
      ),
      symmetric = TRUE
    ))
  }

  fit <- cohen_kappa_matrix_cpp(
    X,
    n_levels = n_levels,
    pairwise_complete = identical(na_method, "pairwise"),
    return_inference = isTRUE(ci) || isTRUE(p_value),
    conf_level = conf_level,
    n_threads = n_threads
  )
  fit$est <- .mc_set_matrix_dimnames(fit$est, col_names)
  fit$n_complete <- .mc_set_matrix_dimnames(fit$n_complete, col_names)
  fit$po <- .mc_set_matrix_dimnames(fit$po, col_names)
  fit$pe <- .mc_set_matrix_dimnames(fit$pe, col_names)
  if (!is.null(fit$se)) {
    fit$se <- .mc_set_matrix_dimnames(fit$se, col_names)
    fit$lwr <- .mc_set_matrix_dimnames(fit$lwr, col_names)
    fit$upr <- .mc_set_matrix_dimnames(fit$upr, col_names)
    fit$statistic <- .mc_set_matrix_dimnames(fit$statistic, col_names)
    fit$p_value <- .mc_set_matrix_dimnames(fit$p_value, col_names)
  }

  diagnostics <- .mc_merge_diagnostics(
    list(
      n_complete = fit$n_complete,
      observed_agreement = fit$po,
      expected_agreement = fit$pe
    ),
    if (identical(na_method, "pairwise")) {
      list(
        na_method = "pairwise",
        common_sample = FALSE
      )
    } else {
      listwise_meta
    }
  )
  ci_attr <- if (isTRUE(ci)) {
    list(
      est = fit$est,
      lwr.ci = fit$lwr,
      upr.ci = fit$upr,
      conf.level = conf_level,
      ci.method = "delta"
    )
  } else {
    NULL
  }
  inference_attr <- if (isTRUE(p_value)) {
    list(
      method = "wald_z_delta_kappa",
      estimate = fit$est,
      se = fit$se,
      statistic = fit$statistic,
      p_value = fit$p_value,
      n_obs = fit$n_complete
    )
  } else {
    NULL
  }

  out <- .mc_new_corr_matrix(
    mat = fit$est,
    estimator_class = "cohen_kappa",
    method = "cohen_kappa",
    description = "Pairwise Cohen's kappa agreement matrix",
    diagnostics = diagnostics,
    ci = ci_attr,
    conf.level = if (isTRUE(ci)) conf_level else NULL,
    symmetric = TRUE,
    extra_attrs = list(inference = inference_attr)
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_is_plain_nominal_atomic <- function(x) {
  cls <- class(x)
  identical(cls, "character") ||
    identical(cls, "logical") ||
    identical(cls, "integer") ||
    identical(cls, "numeric")
}

.mc_nominal_vector_labels <- function(x,
                                      arg = as.character(substitute(x))) {
  if (is.list(x) && !is.data.frame(x)) {
    abort_bad_arg(arg, message = "must not be a list.")
  }
  if (inherits(x, "Date") || inherits(x, "POSIXt") || inherits(x, "difftime")) {
    abort_bad_arg(
      arg,
      message = "must not be a Date/POSIXct/POSIXlt/difftime object."
    )
  }
  if (is.factor(x)) {
    out <- as.character(x)
    out[is.na(x)] <- NA_character_
    return(out)
  }
  if (.mc_is_plain_nominal_atomic(x)) {
    if (is.numeric(x) && any(!is.na(x) & !is.finite(x))) {
      abort_bad_arg(
        arg,
        message = "must not contain NaN or infinite values."
      )
    }
    out <- as.character(x)
    out[is.na(x)] <- NA_character_
    return(out)
  }
  abort_bad_arg(
    arg,
    message = paste(
      "must be a factor, ordered factor, character, logical, integer,",
      "or numeric nominal vector."
    )
  )
}

.mc_extract_nominal_columns <- function(data,
                                        arg = as.character(substitute(data)),
                                        min_cols = 2L) {
  cols <- NULL
  col_names <- NULL
  if (is.data.frame(data)) {
    cols <- unclass(data)
    col_names <- names(data)
  } else if (is.matrix(data)) {
    cols <- lapply(seq_len(ncol(data)), function(j) data[, j])
    col_names <- colnames(data)
  } else {
    abort_bad_arg(
      arg,
      message = "must be a matrix or data frame in matrix mode."
    )
  }

  if (length(cols) < min_cols) {
    abort_bad_arg(
      arg,
      message = "must contain at least {min_cols} nominal column{?s}.",
      min_cols = min_cols
    )
  }

  labels <- vector("list", length(cols))
  for (j in seq_along(cols)) {
    col_arg <- if (!is.null(col_names) && nzchar(col_names[[j]] %||% "")) {
      sprintf("%s$%s", arg, col_names[[j]])
    } else {
      sprintf("%s[[%d]]", arg, j)
    }
    labels[[j]] <- .mc_nominal_vector_labels(cols[[j]], arg = col_arg)
  }
  names(labels) <- col_names
  .mc_encode_nominal_label_list(labels)
}

.mc_encode_nominal_pair_labels <- function(x_labels, y_labels) {
  levels_all <- unique(c(
    x_labels[!is.na(x_labels)],
    y_labels[!is.na(y_labels)]
  ))
  list(
    x = .mc_match_nominal_levels(x_labels, levels_all),
    y = .mc_match_nominal_levels(y_labels, levels_all),
    levels = levels_all,
    n_levels = length(levels_all)
  )
}

.mc_encode_nominal_label_list <- function(labels) {
  levels_all <- unique(unlist(
    lapply(labels, function(z) z[!is.na(z)]),
    use.names = FALSE
  ))
  X <- matrix(
    NA_integer_,
    nrow = length(labels[[1L]]),
    ncol = length(labels),
    dimnames = list(NULL, names(labels))
  )
  for (j in seq_along(labels)) {
    X[, j] <- .mc_match_nominal_levels(labels[[j]], levels_all)
  }
  list(
    matrix = X,
    levels = levels_all,
    n_levels = length(levels_all),
    names = names(labels)
  )
}

.mc_match_nominal_levels <- function(labels, levels_all) {
  out <- match(labels, levels_all)
  out[is.na(labels)] <- NA_integer_
  as.integer(out)
}

.mc_cohen_kappa_scalar <- function(fit,
                                   ci = FALSE,
                                   p_value = FALSE,
                                   conf_level = 0.95) {
  est <- as.numeric(fit$est)
  out <- structure(est, class = c("cohen_kappa", "numeric"))
  attr(out, "method") <- "cohen_kappa"
  attr(out, "description") <- "Pairwise Cohen's kappa agreement"
  attr(out, "package") <- "matrixCorr"
  attr(out, "estimate") <- est
  attr(out, "diagnostics") <- list(
    n_complete = as.integer(fit$n_complete),
    observed_agreement = as.numeric(fit$po),
    expected_agreement = as.numeric(fit$pe)
  )
  if (isTRUE(ci)) {
    attr(out, "ci") <- list(
      est = est,
      lwr.ci = as.numeric(fit$lwr),
      upr.ci = as.numeric(fit$upr),
      conf.level = conf_level,
      ci.method = "delta"
    )
    attr(out, "conf.level") <- conf_level
    attr(out, "conf_level") <- conf_level
  }
  if (isTRUE(p_value)) {
    attr(out, "inference") <- list(
      method = "wald_z_delta_kappa",
      estimate = est,
      se = as.numeric(fit$se),
      statistic = as.numeric(fit$statistic),
      p_value = as.numeric(fit$p_value),
      n_obs = as.integer(fit$n_complete)
    )
  }
  out
}

#' @rdname cohen_kappa
#' @method summary cohen_kappa
#' @param object A scalar or matrix-style \code{cohen_kappa} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param ci_digits Integer; number of decimal places for confidence limits.
#' @param p_digits Integer; number of decimal places for p-values.
#' @param ... Unused.
#' @export
summary.cohen_kappa <- function(object,
                                digits = 4,
                                ci_digits = 3,
                                p_digits = 4,
                                ...) {
  if (inherits(object, "corr_result")) {
    return(summary.corr_matrix(object, ...))
  }
  check_inherits(object, "cohen_kappa")

  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  ci_attr <- attr(object, "ci", exact = TRUE)
  inf_attr <- attr(object, "inference", exact = TRUE)

  df <- data.frame(
    estimate = as.numeric(object),
    n_complete = as.integer(diag_attr$n_complete %||% NA_integer_),
    observed_agreement = as.numeric(diag_attr$observed_agreement %||% NA_real_),
    expected_agreement = as.numeric(diag_attr$expected_agreement %||% NA_real_),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (is.list(ci_attr)) {
    df$lwr <- as.numeric(ci_attr$lwr.ci %||% NA_real_)
    df$upr <- as.numeric(ci_attr$upr.ci %||% NA_real_)
  }
  if (is.list(inf_attr)) {
    df$se <- as.numeric(inf_attr$se %||% NA_real_)
    df$statistic <- as.numeric(inf_attr$statistic %||% NA_real_)
    df$p_value <- as.numeric(inf_attr$p_value %||% NA_real_)
  }

  out <- .mc_finalize_summary_df(df, class_name = "summary.cohen_kappa")
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "p_digits") <- p_digits
  attr(out, "has_ci") <- is.list(ci_attr)
  attr(out, "has_p") <- is.list(inf_attr)
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE) %||%
    attr(object, "conf_level", exact = TRUE) %||%
    NA_real_
  attr(out, "ci_method") <- if (is.list(ci_attr)) ci_attr$ci.method %||% NA_character_ else NA_character_
  attr(out, "inference_method") <- if (is.list(inf_attr)) inf_attr$method %||% NA_character_ else NA_character_
  out
}

#' @rdname cohen_kappa
#' @method print summary.cohen_kappa
#' @param x A \code{summary.cohen_kappa} object.
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.summary.cohen_kappa <- function(x,
                                      digits = NULL,
                                      n = NULL,
                                      topn = NULL,
                                      max_vars = NULL,
                                      width = NULL,
                                      show_ci = NULL,
                                      ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  digits <- .mc_coalesce(digits, .mc_coalesce(attr(x, "digits", exact = TRUE), 4L))
  ci_digits <- .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)
  p_digits <- .mc_coalesce(attr(x, "p_digits", exact = TRUE), digits)

  digest <- c(
    method = "cohen_kappa",
    estimate = formatC(as.numeric(x$estimate[[1L]]), format = "f", digits = digits),
    n_complete = .mc_count_fmt(x$n_complete[[1L]]),
    observed_agreement = formatC(as.numeric(x$observed_agreement[[1L]]), format = "f", digits = digits),
    expected_agreement = formatC(as.numeric(x$expected_agreement[[1L]]), format = "f", digits = digits)
  )
  if (isTRUE(attr(x, "has_ci", exact = TRUE))) {
    digest <- c(
      digest,
      ci = sprintf("%g%%", 100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE)))),
      ci_method = attr(x, "ci_method", exact = TRUE) %||% NA_character_
    )
  }
  if (isTRUE(attr(x, "has_p", exact = TRUE))) {
    digest <- c(
      digest,
      inference = attr(x, "inference_method", exact = TRUE) %||% NA_character_
    )
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = "Cohen's kappa agreement summary")
  cat("\n")

  preview <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  if (identical(show_ci, "no")) {
    preview <- preview[, setdiff(names(preview), c("lwr", "upr")), drop = FALSE]
  }
  if ("estimate" %in% names(preview)) preview$estimate <- round(preview$estimate, digits)
  if ("observed_agreement" %in% names(preview)) preview$observed_agreement <- round(preview$observed_agreement, digits)
  if ("expected_agreement" %in% names(preview)) preview$expected_agreement <- round(preview$expected_agreement, digits)
  if ("lwr" %in% names(preview)) preview$lwr <- round(preview$lwr, ci_digits)
  if ("upr" %in% names(preview)) preview$upr <- round(preview$upr, ci_digits)
  if ("se" %in% names(preview)) preview$se <- round(preview$se, digits)
  if ("statistic" %in% names(preview)) preview$statistic <- round(preview$statistic, digits)
  if ("p_value" %in% names(preview)) preview$p_value <- round(preview$p_value, p_digits)

  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  .mc_print_preview_table(
    preview,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = FALSE,
    summary_hint = FALSE
  )
  invisible(x)
}

#' @rdname cohen_kappa
#' @method print cohen_kappa
#' @param x A scalar or matrix-style \code{cohen_kappa} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to downstream print helpers.
#' @export
print.cohen_kappa <- function(x, digits = 4, n = NULL, topn = NULL,
                              max_vars = NULL, width = NULL,
                              show_ci = NULL, ...) {
  if (!inherits(x, "corr_result")) {
    return(invisible(print.summary.cohen_kappa(
      summary.cohen_kappa(x),
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      ...
    )))
  }
  .mc_print_corr_matrix(
    x,
    header = "Cohen's kappa agreement matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname cohen_kappa
#' @method plot cohen_kappa
#' @param x A scalar or matrix-style \code{cohen_kappa} object.
#' @param title Optional plot title.
#' @param low_color Fill/color used for negative agreement.
#' @param high_color Fill/color used for positive agreement.
#' @param mid_color Unused placeholder for API consistency with matrix heatmaps.
#' @param value_text_size Text size for the estimate label.
#' @param ci_text_size Text size for the CI label.
#' @param show_value Logical; whether to print the estimate and CI labels.
#' @param ... Additional theme arguments.
#' @export
plot.cohen_kappa <- function(x,
                             title = NULL,
                             low_color = "indianred1",
                             high_color = "steelblue1",
                             mid_color = "white",
                             value_text_size = 4,
                             ci_text_size = 3,
                             show_value = TRUE,
                             ...) {
  if (inherits(x, "corr_result")) {
    return(.mc_plot_corr_result(
      x,
      title = title %||% "Cohen's kappa agreement heatmap",
      low_color = low_color,
      high_color = high_color,
      mid_color = mid_color,
      value_text_size = value_text_size,
      ci_text_size = ci_text_size,
      show_value = show_value,
      ...
    ))
  }
  check_inherits(x, "cohen_kappa")
  check_bool(show_value, arg = "show_value")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  ci_attr <- attr(x, "ci", exact = TRUE)
  est <- as.numeric(x)
  df <- data.frame(
    label = "Cohen's kappa",
    estimate = est,
    lwr = if (is.list(ci_attr)) as.numeric(ci_attr$lwr.ci %||% NA_real_) else NA_real_,
    upr = if (is.list(ci_attr)) as.numeric(ci_attr$upr.ci %||% NA_real_) else NA_real_,
    stringsAsFactors = FALSE
  )
  df$label_y <- 1
  df$estimate_label <- sprintf("%.2f", df$estimate)
  df$ci_label <- ifelse(
    is.finite(df$lwr) & is.finite(df$upr),
    sprintf("[%.2f, %.2f]", df$lwr, df$upr),
    NA_character_
  )
  point_color <- if (is.finite(est) && est < 0) low_color else high_color

  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$estimate, y = .data$label_y)) +
    ggplot2::geom_vline(xintercept = 0, color = "gray85", linewidth = 0.4) +
    ggplot2::geom_vline(xintercept = 1, color = "gray90", linewidth = 0.4, linetype = "dashed")
  if (is.finite(df$lwr[[1L]]) && is.finite(df$upr[[1L]])) {
    p <- p + ggplot2::geom_segment(
      ggplot2::aes(x = .data$lwr, xend = .data$upr, y = .data$label_y, yend = .data$label_y),
      linewidth = 1,
      color = point_color
    )
  }
  p <- p +
    ggplot2::geom_point(size = 3, color = point_color) +
    ggplot2::scale_x_continuous(limits = c(-1, 1)) +
    ggplot2::scale_y_continuous(breaks = 1, labels = "Cohen's kappa") +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::labs(
      title = title %||% "Cohen's kappa agreement",
      x = "Kappa"
    )

  if (isTRUE(show_value)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = .data$estimate_label),
      nudge_y = 0.12,
      size = value_text_size,
      color = "black"
    )
    if (any(!is.na(df$ci_label))) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(label = .data$ci_label),
        nudge_y = -0.12,
        size = ci_text_size,
        color = "gray30",
        na.rm = TRUE
      )
    }
  }
  p
}

manual_cohen_kappa <- function(x, y) {
  keep <- stats::complete.cases(x, y)
  x <- as.character(x[keep])
  y <- as.character(y[keep])
  if (length(x) < 2L) {
    return(NA_real_)
  }
  lev <- unique(c(x, y))
  tab <- table(
    factor(x, levels = lev),
    factor(y, levels = lev)
  )
  p <- unclass(tab) / sum(tab)
  po <- sum(diag(p))
  pe <- sum(rowSums(p) * colSums(p))
  if (!is.finite(1 - pe) || (1 - pe) <= 0) {
    return(NA_real_)
  }
  (po - pe) / (1 - pe)
}

test_that("cohen_kappa reproduces the manual 2x2 example", {
  x <- c(rep("c1", 25), rep("c2", 25))
  y <- c(
    rep("c1", 20), rep("c2", 5),
    rep("c1", 10), rep("c2", 15)
  )

  fit <- cohen_kappa(x, y)
  expect_s3_class(fit, "cohen_kappa")
  expect_equal(as.numeric(fit), 0.4, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$observed_agreement, 0.7, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$expected_agreement, 0.5, tolerance = 1e-12)
})

test_that("cohen_kappa returns the verified 2-category estimate and delta CI", {
  x <- factor(c(rep("c1", 25), rep("c2", 25)))
  y <- factor(c(
    rep("c1", 20), rep("c2", 5),
    rep("c1", 10), rep("c2", 15)
  ))

  fit <- cohen_kappa(x, y, ci = TRUE, p_value = TRUE)
  ci_attr <- attr(fit, "ci")
  inf_attr <- attr(fit, "inference")

  expect_equal(as.numeric(fit), 0.4, tolerance = 1e-12)
  expect_equal(ci_attr$lwr.ci, 0.1510923, tolerance = 1e-6)
  expect_equal(ci_attr$upr.ci, 0.6489077, tolerance = 1e-6)
  expect_equal(inf_attr$se, 0.1269961, tolerance = 1e-6)
})

test_that("cohen_kappa returns the verified 3-category estimate and delta CI", {
  tab <- matrix(
    c(
      12, 3, 1,
      4, 10, 2,
      2, 5, 11
    ),
    nrow = 3,
    byrow = TRUE,
    dimnames = list(x = c("A", "B", "C"), y = c("A", "B", "C"))
  )

  x <- rep(rownames(tab), times = rowSums(tab))
  y <- unlist(lapply(seq_len(nrow(tab)), function(i) {
    rep(colnames(tab), times = tab[i, ])
  }), use.names = FALSE)

  fit <- cohen_kappa(
    factor(x, levels = rownames(tab)),
    factor(y, levels = colnames(tab)),
    ci = TRUE,
    p_value = TRUE
  )
  ci_attr <- attr(fit, "ci")
  inf_attr <- attr(fit, "inference")

  expect_equal(as.numeric(fit), 0.4916268, tolerance = 1e-6)
  expect_equal(ci_attr$lwr.ci, 0.2964395, tolerance = 1e-6)
  expect_equal(ci_attr$upr.ci, 0.6868141, tolerance = 1e-6)
  expect_equal(inf_attr$se, 0.0995872, tolerance = 1e-6)
})

test_that("cohen_kappa matrix mode is symmetric and matches manual pairwise values", {
  raters <- data.frame(
    r1 = factor(c("A", "A", "B", "B", "C", "C", "A", "B")),
    r2 = c("A", "B", "B", "B", "C", "A", "A", "B"),
    r3 = c(1, 1, 2, 2, 3, 3, 1, 2),
    r4 = c(TRUE, FALSE, FALSE, FALSE, TRUE, TRUE, TRUE, FALSE),
    stringsAsFactors = FALSE
  )

  fit <- cohen_kappa(raters)
  mat <- as.matrix(fit)
  expect_true(isSymmetric(mat))
  expect_equal(unname(diag(mat)), rep(1, ncol(raters)))

  expected <- matrix(NA_real_, ncol(raters), ncol(raters), dimnames = dimnames(mat))
  diag(expected) <- 1
  for (j in seq_len(ncol(raters) - 1L)) {
    for (k in (j + 1L):ncol(raters)) {
      expected[j, k] <- manual_cohen_kappa(raters[[j]], raters[[k]])
      expected[k, j] <- expected[j, k]
    }
  }

  expect_identical(dimnames(mat), dimnames(expected))
  expect_equal(as.vector(mat), as.vector(expected), tolerance = 1e-12)
})

test_that("cohen_kappa missing-data modes behave as documented", {
  raters <- data.frame(
    a = c("A", "A", "B", NA, "C", "A"),
    b = c("A", "B", "B", "B", NA, "A"),
    c = c("A", "A", NA, "B", "C", "A"),
    stringsAsFactors = FALSE
  )

  expect_error(
    cohen_kappa(raters, na_method = "error"),
    "contains missing values"
  )

  fit_complete <- cohen_kappa(raters, na_method = "complete")
  keep <- stats::complete.cases(raters)
  expected_complete <- matrix(sum(keep), ncol(raters), ncol(raters))
  diag(expected_complete) <- rep(sum(keep), ncol(raters))
  dimnames(expected_complete) <- dimnames(as.matrix(fit_complete))
  expect_equal(attr(fit_complete, "diagnostics")$n_complete, expected_complete)
  expect_equal(
    as.vector(as.matrix(fit_complete)),
    as.vector(as.matrix(cohen_kappa(raters[keep, , drop = FALSE], na_method = "error"))),
    tolerance = 1e-12
  )

  fit_pairwise <- cohen_kappa(raters, na_method = "pairwise")
  n_complete <- attr(fit_pairwise, "diagnostics")$n_complete
  expect_equal(n_complete["a", "b"], sum(stats::complete.cases(raters[c("a", "b")])))
  expect_equal(n_complete["a", "c"], sum(stats::complete.cases(raters[c("a", "c")])))
  expect_equal(n_complete["b", "c"], sum(stats::complete.cases(raters[c("b", "c")])))
})

test_that("cohen_kappa scalar mode has dedicated print, summary, and plot methods", {
  skip_if_not_installed("ggplot2")

  x <- factor(c("A", "A", "B", "B", "A", "B"))
  y <- factor(c("A", "B", "B", "B", "A", "A"))
  fit <- cohen_kappa(x, y, ci = TRUE, p_value = TRUE)

  txt <- capture.output(print(fit))
  expect_true(any(grepl("Cohen's kappa agreement summary", txt, fixed = TRUE)))

  sm <- summary(fit)
  expect_s3_class(sm, "summary.cohen_kappa")
  expect_true(all(c("estimate", "n_complete", "observed_agreement", "expected_agreement") %in% names(sm)))
  expect_true(all(c("lwr", "upr", "se", "statistic", "p_value") %in% names(sm)))

  p <- plot(fit, show_value = FALSE)
  expect_s3_class(p, "ggplot")
})

test_that("cohen_kappa attaches CI and inference metadata", {
  raters <- data.frame(
    a = c("A", "A", "B", "B", "C", "A", "B", "C", "A", "C"),
    b = c("A", "B", "B", "B", "C", "A", "B", "C", "A", "B"),
    c = c("A", "A", "B", "C", "C", "A", "B", "B", "A", "C"),
    stringsAsFactors = FALSE
  )

  fit_ci <- cohen_kappa(raters, ci = TRUE)
  ci_attr <- attr(fit_ci, "ci")
  expect_true(is.list(ci_attr))
  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level", "ci.method") %in% names(ci_attr)))
  expect_identical(ci_attr$ci.method, "delta")

  fit_p <- cohen_kappa(raters, p_value = TRUE)
  inf_attr <- attr(fit_p, "inference")
  expect_true(is.list(inf_attr))
  expect_true(all(c("statistic", "p_value") %in% names(inf_attr)))
})

test_that("cohen_kappa handles degenerate and low-information cases", {
  raters <- data.frame(
    a = rep("A", 6),
    b = rep("A", 6),
    c = c("A", "B", "A", "B", "A", "B"),
    stringsAsFactors = FALSE
  )
  fit <- cohen_kappa(raters)
  mat <- as.matrix(fit)
  expect_true(is.na(mat["a", "b"]))
  expect_equal(unname(diag(mat)), c(1, 1, 1))

  low_info <- data.frame(
    a = c("A", NA),
    b = c("A", "B"),
    c = c("A", "A"),
    stringsAsFactors = FALSE
  )
  fit_low <- cohen_kappa(low_info, na_method = "pairwise")
  mat_low <- as.matrix(fit_low)
  expect_true(is.na(mat_low["a", "b"]))
  expect_true(is.na(mat_low["a", "a"]))
  expect_equal(mat_low["b", "b"], 1)
  expect_equal(mat_low["c", "c"], 1)
})

#' Probability of Agreement for Reliability Curves
#'
#' @description
#' `prob_agree()` implements the probability of agreement following Stevens and
#' Anderson-Cook (2017). It fits binomial reliability curves by population or
#' generation and estimates, for each pair, the probability that the fitted
#' reliabilities differ by no more than a user-specified practically negligible
#' amount.
#'
#' This is not a correlation coefficient and should not be interpreted as
#' linear association. It is a tolerance-based agreement measure. It is
#' computed from the sampling distribution of the estimated difference between
#' two reliability curves.
#'
#' @details
#' For each pair of fitted reliability curves \eqn{\hat\pi_1(a)} and
#' \eqn{\hat\pi_2(a)}, Stevens and Anderson-Cook define
#' \deqn{\theta(a) = P\{|\tilde\pi_1(a) - \tilde\pi_2(a)| \le \delta(a) \mid a\}.}
#' The models are fit with a binomial GLM using either a logit or probit link,
#' with \eqn{g(\pi_i) = \beta_0 + \beta_1 a_i}. The C++ backend evaluates the
#' two-population large-sample normal approximation described in the paper and
#' Supplementary Material A; when more than two groups are supplied, `prob_agree()`
#' applies that calculation to all pairwise group comparisons.
#'
#' Missing rows in `response`, `predictor`, or `group` are removed before model
#' fitting. The response must be binary, coded as `0`/`1`, `FALSE`/`TRUE`, or a
#' two-level factor.
#'
#' @param data A data frame containing the response, predictor, and group
#'   variables.
#' @param response Character scalar naming the binary pass/fail response column.
#' @param predictor Character scalar naming the numeric age, time, or operating
#'   condition column.
#' @param group Character scalar naming the population/generation column. All
#'   pairwise comparisons among the observed non-missing levels are evaluated.
#' @param delta Positive scalar or vector tolerance for symmetric limits
#'   `-delta` and `delta`. A length-two vector with a negative lower bound and a
#'   positive upper bound, for example `c(-0.03, 0.05)`, is treated as an
#'   asymmetric tolerance interval. No default is provided because the tolerance
#'   is part of the scientific estimand.
#' @param limits Numeric vector `c(lower, upper)` for asymmetric tolerance
#'   limits. Mutually exclusive with `delta`.
#' @param link Link used for the reliability curve: `"logit"` or `"probit"`.
#' @param newdata Optional data frame containing predictor values where the
#'   probability of agreement is evaluated. If `NULL`, a regular grid over the
#'   observed predictor range is used.
#' @param grid_size Integer number of evaluation points when `newdata = NULL`.
#' @param ci Logical; if `TRUE`, attach pointwise confidence limits.
#' @param conf_level Confidence level for intervals.
#' @param max_iter Maximum number of IRLS iterations for each fitted curve.
#' @param tol Convergence tolerance for IRLS coefficient updates.
#' @param verbose Logical; if `TRUE`, emits a short `cli` message.
#'
#' @return A data frame with class
#' `c("prob_agree_curve", "prob_agree", "data.frame")` and columns `group1`,
#' `group2`, the predictor, `prob_agree`, and optional `lwr.ci`, `upr.ci`.
#'
#' @references
#' Stevens, N. T. and Anderson-Cook, C. M. (2017). Comparing the Reliability of
#' Related Populations With the Probability of Agreement. *Technometrics*,
#' 59(3), 371-380. doi:10.1080/00401706.2016.1214180.
#'
#' @examples
#' # Stevens and Anderson-Cook's probability of agreement evaluates whether
#' # two related populations have reliability curves that are similar enough
#' # to be treated as practically homogeneous. At each age, the agreement
#' # hypothesis is that the difference between the two fitted reliabilities
#' # lies inside the user-specified practical tolerance interval.
#' set.seed(1)
#' n <- 160
#' dat <- data.frame(
#'   age = c(runif(n, 0, 60), runif(n, 0, 45)),
#'   generation = rep(c("Gen1", "Gen2"), each = n)
#' )
#' eta <- ifelse(
#'   dat$generation == "Gen1",
#'   4.3 - 0.045 * dat$age,
#'   4.0 - 0.040 * dat$age
#' )
#' dat$pass <- rbinom(nrow(dat), size = 1, prob = plogis(eta))
#'
#' fit_pa <- prob_agree(
#'   dat,
#'   response = "pass",
#'   predictor = "age",
#'   group = "generation",
#'   delta = 0.05,
#'   link = "logit",
#'   ci = TRUE
#' )
#' print(fit_pa)
#' summary(fit_pa)
#' estimate(fit_pa)
#' tidy(fit_pa)
#' confint(fit_pa)
#' plot(fit_pa)
#'
#' # Four generations are compared as all pairwise two-generation contrasts.
#' set.seed(2)
#' n4 <- 120
#' dat4 <- data.frame(
#'   age = rep(runif(n4, 0, 55), times = 4),
#'   generation = rep(paste0("Gen", 1:4), each = n4)
#' )
#' shifts <- c(4.2, 4.0, 3.8, 3.6)
#' slopes <- c(-0.040, -0.042, -0.044, -0.046)
#' gen_id <- match(dat4$generation, paste0("Gen", 1:4))
#' eta4 <- shifts[gen_id] + slopes[gen_id] * dat4$age
#' dat4$pass <- rbinom(nrow(dat4), size = 1, prob = plogis(eta4))
#'
#' fit4 <- prob_agree(
#'   dat4,
#'   response = "pass",
#'   predictor = "age",
#'   group = "generation",
#'   limits = c(-0.03, 0.05),
#'   link = "logit",
#'   ci = FALSE
#' )
#' print(fit4)
#' plot(fit4)
#'
#' @author Thiago de Paula Oliveira
#' @export
prob_agree <- function(data,
                       response,
                       predictor,
                       group,
                       delta = NULL,
                       limits = NULL,
                       link = c("logit", "probit"),
                       newdata = NULL,
                       grid_size = 100L,
                       ci = TRUE,
                       conf_level = 0.95,
                       max_iter = 50L,
                       tol = 1e-8,
                       verbose = FALSE) {
  link <- match.arg(link)
  check_bool(ci, arg = "ci")
  check_bool(verbose, arg = "verbose")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  grid_size <- check_scalar_int_pos(grid_size, arg = "grid_size")
  max_iter <- check_scalar_int_pos(max_iter, arg = "max_iter")
  check_scalar_numeric(tol, arg = "tol", lower = 0, finite = TRUE)

  if (!is.data.frame(data)) {
    abort_bad_arg("data", message = "must be a data frame.")
  }
  for (arg in c("response", "predictor", "group")) {
    val <- get(arg, envir = environment())
    if (!is.character(val) || length(val) != 1L || is.na(val) || !nzchar(val)) {
      abort_bad_arg(arg, message = "must be a single column name.")
    }
    if (!val %in% names(data)) {
      abort_bad_arg(arg, message = "must name a column in {.arg data}.")
    }
  }

  y <- .pa_response01(data[[response]])
  x <- data[[predictor]]
  if (!is.numeric(x)) {
    abort_bad_arg("predictor", message = "must name a numeric column.")
  }
  g <- factor(data[[group]])
  keep <- stats::complete.cases(y, x, g)
  if (sum(keep) < 6L) {
    abort_bad_arg("data", message = "must contain at least six complete rows across the groups.")
  }
  y <- y[keep]
  x <- as.numeric(x[keep])
  g <- droplevels(g[keep])
  if (nlevels(g) < 2L) {
    abort_bad_arg("group", message = "must retain at least two levels after removing missing rows.")
  }

  eval_x <- .pa_eval_predictor(data[[predictor]], predictor, newdata, grid_size)
  tol_obj <- .pa_tolerance(delta, limits, length(eval_x))
  link_code <- switch(link, logit = 1L, probit = 2L)
  levels_g <- levels(g)
  pair_mat <- utils::combn(levels_g, 2L)
  pair_outputs <- vector("list", ncol(pair_mat))
  pair_models <- vector("list", ncol(pair_mat))

  for (j in seq_len(ncol(pair_mat))) {
    pair <- pair_mat[, j]
    idx <- g %in% pair
    pair_g <- droplevels(g[idx])
    raw <- prob_agree_fit_cpp(
      response = y[idx],
      predictor = x[idx],
      group = as.integer(pair_g),
      eval_predictor = eval_x,
      link_code = link_code,
      lower = tol_obj$lower,
      upper = tol_obj$upper,
      ci = ci,
      ci_method = if (ci) 1L else 0L,
      conf_level = conf_level,
      max_iter = max_iter,
      tol = tol
    )
    pair_outputs[[j]] <- .pa_build_output(
      raw,
      ci = ci,
      predictor = predictor,
      group1 = pair[[1L]],
      group2 = pair[[2L]]
    )
    pair_models[[j]] <- list(
      group1 = pair[[1L]],
      group2 = pair[[2L]],
      coefficients = stats::setNames(
        list(
          stats::setNames(as.numeric(raw$beta1), c("(Intercept)", predictor)),
          stats::setNames(as.numeric(raw$beta2), c("(Intercept)", predictor))
        ),
        pair
      ),
      vcov = stats::setNames(list(raw$vcov1, raw$vcov2), pair),
      n_obs = stats::setNames(as.integer(c(raw$n1, raw$n2)), pair),
      converged = stats::setNames(as.logical(raw$converged), pair),
      iterations = stats::setNames(as.integer(raw$iterations), pair)
    )
  }
  out <- do.call(rbind, pair_outputs)
  row.names(out) <- NULL
  class(out) <- c("prob_agree_curve", "prob_agree", "data.frame")

  attr(out, "method") <- "probability_of_agreement"
  attr(out, "description") <- "Stevens-Anderson-Cook probability of agreement"
  attr(out, "package") <- "matrixCorr"
  attr(out, "estimator") <- "large_sample_normal"
  attr(out, "tolerance") <- tol_obj$tolerance
  attr(out, "link") <- link
  attr(out, "response") <- response
  attr(out, "predictor") <- predictor
  attr(out, "group") <- group
  attr(out, "group.levels") <- levels_g
  attr(out, "pairs") <- data.frame(
    group1 = pair_mat[1L, ],
    group2 = pair_mat[2L, ],
    pair = paste(pair_mat[1L, ], pair_mat[2L, ], sep = " vs "),
    stringsAsFactors = FALSE
  )
  attr(out, "pair_models") <- stats::setNames(pair_models, attr(out, "pairs")$pair)
  attr(out, "ci.method") <- if (ci) "delta_method" else "none"
  attr(out, "conf.level") <- conf_level
  attr(out, "reference") <- "Stevens and Anderson-Cook (2017), Technometrics, doi:10.1080/00401706.2016.1214180"

  inform_if_verbose(
    "Computed probability of agreement for {ncol(pair_mat)} pairwise comparison{?s} at {length(eval_x)} evaluation point{?s}.",
    .verbose = verbose
  )
  out
}

.pa_response01 <- function(x) {
  if (is.logical(x)) return(as.numeric(x))
  if (is.factor(x)) {
    if (nlevels(droplevels(x)) != 2L) abort_bad_arg("response", message = "must be binary.")
    return(as.numeric(droplevels(x)) - 1)
  }
  if (is.numeric(x) || is.integer(x)) {
    out <- as.numeric(x)
    vals <- stats::na.omit(unique(out))
    if (!all(vals %in% c(0, 1))) abort_bad_arg("response", message = "must contain only 0/1 values.")
    return(out)
  }
  abort_bad_arg("response", message = "must be binary numeric, logical, or a two-level factor.")
}

.pa_eval_predictor <- function(x, predictor, newdata, grid_size) {
  if (is.null(newdata)) {
    rng <- range(x, na.rm = TRUE)
    if (!all(is.finite(rng)) || rng[[1L]] == rng[[2L]]) {
      abort_bad_arg("predictor", message = "must have a finite non-zero range when {.arg newdata} is NULL.")
    }
    return(seq(rng[[1L]], rng[[2L]], length.out = grid_size))
  }
  out <- if (is.data.frame(newdata)) {
    if (!predictor %in% names(newdata)) abort_bad_arg("newdata", message = "must contain the predictor column {.val {predictor}}.")
    newdata[[predictor]]
  } else {
    newdata
  }
  if (!is.numeric(out) || any(!is.finite(out))) abort_bad_arg("newdata", message = "must provide finite numeric predictor values.")
  as.numeric(out)
}

.pa_tolerance <- function(delta, limits, n) {
  if (!is.null(delta) && !is.null(limits)) cli::cli_abort("{.arg delta} and {.arg limits} are mutually exclusive.")
  if (is.null(delta) && is.null(limits)) cli::cli_abort("A practical tolerance must be supplied via {.arg delta} or {.arg limits}.")
  if (!is.null(delta)) {
    delta <- as.numeric(delta)
    delta_input <- delta
    if (length(delta) == 2L && any(delta < 0)) {
      if (any(!is.finite(delta)) || delta[[1L]] > delta[[2L]]) {
        abort_bad_arg("delta", message = "must be a finite c(lower, upper) vector with lower <= upper when used as asymmetric tolerance.")
      }
      return(list(
        lower = rep(delta[[1L]], n),
        upper = rep(delta[[2L]], n),
        tolerance = list(delta = NULL, limits = delta)
      ))
    }
    if (!(length(delta) %in% c(1L, n)) || any(!is.finite(delta)) || any(delta < 0)) {
      abort_bad_arg("delta", message = "must be a non-negative finite scalar or vector, or c(lower, upper) for asymmetric tolerance.")
    }
    if (length(delta) == 1L) delta <- rep(delta, n)
    return(list(
      lower = -delta,
      upper = delta,
      tolerance = list(delta = delta_input, limits = NULL)
    ))
  }
  limits <- as.numeric(limits)
  if (length(limits) != 2L || any(!is.finite(limits)) || limits[[1L]] > limits[[2L]]) {
    abort_bad_arg("limits", message = "must be a finite numeric vector c(lower, upper) with lower <= upper.")
  }
  list(
    lower = rep(limits[[1L]], n),
    upper = rep(limits[[2L]], n),
    tolerance = list(delta = NULL, limits = limits)
  )
}

.pa_build_output <- function(raw, ci, predictor, group1, group2) {
  out <- data.frame(
    group1 = group1,
    group2 = group2,
    predictor_value = as.numeric(raw$predictor),
    prob_agree = as.numeric(raw$prob_agree),
    check.names = FALSE
  )
  names(out)[[3L]] <- predictor
  if (isTRUE(ci)) {
    out$lwr.ci <- as.numeric(raw$lwr.ci)
    out$upr.ci <- as.numeric(raw$upr.ci)
  }
  structure(out, class = c("prob_agree_curve", "prob_agree", "data.frame"))
}

#' @method print prob_agree
#' @export
print.prob_agree <- function(x, digits = 4, ...) {
  rng <- range(x$prob_agree, na.rm = TRUE)
  tol <- attr(x, "tolerance", exact = TRUE)
  tol_txt <- if (!is.null(tol$delta)) {
    if (length(tol$delta) == 1L) sprintf("symmetric delta = %s", format(tol$delta, digits = digits)) else "symmetric vector delta"
  } else {
    sprintf("limits = [%s, %s]", format(tol$limits[[1L]], digits = digits), format(tol$limits[[2L]], digits = digits))
  }
  groups <- attr(x, "group.levels", exact = TRUE)
  pairs <- attr(x, "pairs", exact = TRUE)
  n_eval <- length(unique(x[[attr(x, "predictor", exact = TRUE)]]))
  cat("Probability-of-agreement result\n")
  cat("  method      : Stevens-Anderson-Cook large-sample normal\n")
  cat("  groups      :", paste(groups, collapse = ", "), "\n")
  cat("  comparisons :", nrow(pairs), "\n")
  cat("  link        :", attr(x, "link", exact = TRUE), "\n")
  cat("  tolerance   :", tol_txt, "\n")
  cat("  evaluations :", n_eval, "per comparison\n")
  cat("  range       :", paste(format(round(rng, digits), nsmall = digits), collapse = " to "), "\n")
  cat("  ci          :", if (all(c("lwr.ci", "upr.ci") %in% names(x))) "yes" else "no", "\n\n")
  print(.pa_preview_df(x, digits = digits), row.names = FALSE, ...)
  invisible(x)
}

#' @method summary prob_agree
#' @export
summary.prob_agree <- function(object, digits = 4, ...) {
  out <- as.data.frame(object)
  num_cols <- vapply(out, is.numeric, logical(1))
  out[num_cols] <- lapply(out[num_cols], round, digits = digits)
  structure(out, class = c("summary.prob_agree", "data.frame"))
}

#' @method print summary.prob_agree
#' @export
print.summary.prob_agree <- function(x, ...) {
  cat("Probability-of-agreement summary\n\n")
  print.data.frame(.pa_preview_df(x), row.names = FALSE, ...)
  n_omitted <- nrow(x) - nrow(.pa_preview_df(x))
  if (n_omitted > 0L) {
    cat("\n", n_omitted, " additional evaluation point", if (n_omitted == 1L) "" else "s", " not shown\n", sep = "")
  }
  invisible(x)
}

.pa_preview_df <- function(x, n = 10L, digits = 4) {
  out <- as.data.frame(x)
  if (nrow(out) > n) out <- utils::head(out, n)
  num_cols <- vapply(out, is.numeric, logical(1))
  out[num_cols] <- lapply(out[num_cols], round, digits = digits)
  out
}

#' Plot probability-of-agreement results
#'
#' @param x An object returned by `prob_agree()`.
#' @param threshold Optional probability threshold shown in curve plots.
#' @param title Optional plot title.
#' @param style Plot style: `"auto"` and `"curve"` draw pairwise probability
#'   curves; `"facet"` draws one curve panel per comparison; `"heatmap"` shows
#'   probability by predictor value and comparison.
#' @param show_ci Logical; for curve plots, controls whether confidence ribbons
#'   are shown. By default ribbons are shown for one comparison and suppressed
#'   for multiple comparisons.
#' @param ... Additional arguments passed to [ggplot2::theme()].
#'
#' @return A `ggplot` object.
#' @method plot prob_agree
#' @export
plot.prob_agree <- function(x,
                            threshold = NULL,
                            title = "Probability of agreement",
                            style = c("auto", "curve", "facet", "heatmap"),
                            show_ci = NULL,
                            ...) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  style <- match.arg(style)
  predictor <- attr(x, "predictor", exact = TRUE) %||% names(x)[[2L]]
  df <- as.data.frame(x)
  df <- df[order(df[[predictor]]), , drop = FALSE]
  df$pair <- paste(df$group1, df$group2, sep = " vs ")
  n_pairs <- length(unique(df$pair))
  if (identical(style, "auto")) style <- "curve"

  if (identical(style, "heatmap")) {
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[predictor]], y = .data$pair, fill = .data$prob_agree)) +
      ggplot2::geom_tile() +
      ggplot2::scale_fill_viridis_c(
        limits = c(0, 1),
        option = "C",
        name = "Pr(agreement)"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(panel.grid = ggplot2::element_blank(), ...) +
      ggplot2::labs(title = title, x = predictor, y = "Comparison")
    if (!is.null(threshold)) {
      check_scalar_numeric(threshold, arg = "threshold", lower = 0, upper = 1)
    }
    return(p)
  }

  if (identical(style, "facet") && is.null(show_ci)) {
    show_ci <- all(c("lwr.ci", "upr.ci") %in% names(df))
  }
  if (is.null(show_ci)) show_ci <- n_pairs == 1L
  check_bool(show_ci, arg = "show_ci")
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[predictor]], y = .data$prob_agree, colour = .data$pair))
  has_ci_ribbon <- isTRUE(show_ci) && all(c("lwr.ci", "upr.ci") %in% names(df))
  if (has_ci_ribbon) {
    p <- p + ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data$lwr.ci, ymax = .data$upr.ci, fill = .data$pair),
      alpha = 0.15,
      colour = NA
    )
  }
  p <- p +
    ggplot2::geom_line(linewidth = 0.7) +
    ggplot2::coord_cartesian(ylim = c(0, 1)) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
    ggplot2::labs(title = title, x = predictor, y = "Probability of agreement", colour = "Comparison")
  if (identical(style, "facet")) {
    p <- p +
      ggplot2::facet_wrap(ggplot2::vars(.data$pair)) +
      ggplot2::guides(colour = "none", fill = "none")
  }
  if (has_ci_ribbon) {
    p <- p + ggplot2::labs(fill = "Comparison")
  }
  if (n_pairs == 1L) {
    p <- p + ggplot2::guides(colour = "none", fill = "none")
  }
  if (!is.null(threshold)) {
    check_scalar_numeric(threshold, arg = "threshold", lower = 0, upper = 1)
    p <- p + ggplot2::geom_hline(yintercept = threshold, linetype = "dashed", colour = "grey35")
  }
  p
}

pa_sim_data <- function(n = 120L) {
  set.seed(11)
  dat <- data.frame(
    age = c(runif(n, 0, 50), runif(n, 0, 50)),
    generation = rep(c("Gen1", "Gen2"), each = n)
  )
  eta <- ifelse(dat$generation == "Gen1", 3.8 - 0.05 * dat$age, 3.5 - 0.045 * dat$age)
  dat$pass <- rbinom(2L * n, 1L, plogis(eta))
  dat
}

test_that("prob_agree fits logit reliability curves from data", {
  dat <- pa_sim_data()
  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    delta = 0.05,
    link = "logit",
    newdata = data.frame(age = c(0, 25, 50)),
    ci = TRUE
  )

  expect_s3_class(fit, "prob_agree")
  expect_equal(names(fit), c("group1", "group2", "age", "prob_agree", "lwr.ci", "upr.ci"))
  expect_equal(fit$age, c(0, 25, 50))
  expect_equal(unique(fit$group1), "Gen1")
  expect_equal(unique(fit$group2), "Gen2")
  expect_true(all(fit$prob_agree >= 0 & fit$prob_agree <= 1))
  expect_true(all(fit$lwr.ci >= 0 & fit$lwr.ci <= 1))
  expect_true(all(fit$upr.ci >= 0 & fit$upr.ci <= 1))
  expect_equal(attr(fit, "group.levels"), c("Gen1", "Gen2"))
  expect_equal(attr(fit, "pairs")$pair, "Gen1 vs Gen2")
  expect_equal(names(attr(fit, "pair_models")[[1]]$coefficients), c("Gen1", "Gen2"))
})

test_that("prob_agree supports probit link and auto grid", {
  dat <- pa_sim_data()
  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    delta = 0.10,
    link = "probit",
    grid_size = 7,
    ci = FALSE
  )

  expect_equal(nrow(fit), 7)
  expect_false("lwr.ci" %in% names(fit))
  expect_equal(attr(fit, "link"), "probit")
})

test_that("prob_agree supports asymmetric limits", {
  dat <- pa_sim_data()
  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    limits = c(-0.05, 0.10),
    newdata = c(10, 20),
    ci = FALSE
  )

  expect_equal(attr(fit, "tolerance")$limits, c(-0.05, 0.10))
})

test_that("prob_agree accepts asymmetric delta", {
  dat <- pa_sim_data()
  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    delta = c(-0.03, 0.05),
    newdata = c(10, 20),
    ci = FALSE
  )

  expect_equal(names(fit), c("group1", "group2", "age", "prob_agree"))
  expect_equal(attr(fit, "tolerance")$limits, c(-0.03, 0.05))
})

test_that("prob_agree computes all pairwise comparisons for more than two groups", {
  set.seed(12)
  n <- 70L
  dat <- data.frame(
    age = rep(runif(n, 0, 45), times = 4),
    generation = rep(paste0("Gen", 1:4), each = n)
  )
  gen_id <- match(dat$generation, paste0("Gen", 1:4))
  dat$pass <- rbinom(
    nrow(dat),
    1L,
    plogis(c(3.8, 3.6, 3.4, 3.2)[gen_id] - c(0.04, 0.042, 0.044, 0.046)[gen_id] * dat$age)
  )

  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    delta = 0.05,
    newdata = c(0, 20, 40),
    ci = FALSE
  )

  expect_equal(nrow(fit), 18)
  expect_equal(nrow(attr(fit, "pairs")), 6)
  expect_equal(unique(paste(fit$group1, fit$group2, sep = " vs ")), attr(fit, "pairs")$pair)
  expect_true(all(fit$prob_agree >= 0 & fit$prob_agree <= 1))
  skip_if_not_installed("ggplot2")
  expect_warning(p <- plot(fit), NA)
  expect_s3_class(p, "ggplot")
  expect_s3_class(plot(fit, style = "curve"), "ggplot")
  expect_s3_class(plot(fit, style = "facet"), "ggplot")
  expect_s3_class(plot(fit, style = "heatmap"), "ggplot")
})

test_that("prob_agree validates data and tolerances", {
  dat <- pa_sim_data()
  expect_error(prob_agree(dat, "pass", "age", "generation"), "tolerance")
  expect_error(
    prob_agree(dat, "pass", "age", "generation", delta = 0.1, limits = c(-0.1, 0.1)),
    "mutually exclusive"
  )
  expect_error(
    prob_agree(dat, "pass", "age", "generation", limits = c(0.1, -0.1)),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    prob_agree(dat, "pass", "age", "generation", delta = -0.1),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    prob_agree(transform(dat, generation = "one"), "pass", "age", "generation", delta = 0.1),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    prob_agree(transform(dat, pass = 2), "pass", "age", "generation", delta = 0.1),
    class = "matrixCorr_arg_error"
  )
})

test_that("prob_agree S3 methods work", {
  skip_if_not_installed("ggplot2")
  dat <- pa_sim_data()
  fit <- prob_agree(
    dat,
    response = "pass",
    predictor = "age",
    group = "generation",
    delta = 0.05,
    newdata = c(0, 20, 40),
    ci = TRUE
  )

  expect_true(length(capture.output(print(fit))) > 0L)
  sm <- summary(fit)
  expect_s3_class(sm, "summary.prob_agree")
  expect_true(all(c(
    "group1", "group2", "age", "prob_agree", "lwr.ci", "upr.ci"
  ) %in% names(sm)))
  expect_s3_class(plot(fit), "ggplot")
  expect_s3_class(plot(fit, style = "curve"), "ggplot")
  expect_s3_class(plot(fit, style = "facet"), "ggplot")
})

#' Interactive Shiny viewer for matrixCorr objects
#'
#' Launches an interactive Shiny gadget that displays correlation heatmaps with
#' filtering, clustering, and hover inspection. The viewer accepts any
#' matrixCorr correlation result (for example the outputs from
#' [pearson_corr()], [spearman_rho()], [kendall_tau()], [bicor()],
#' [pbcor()], [wincor()], [skipped_corr()], [pcorr()], [dcor()], or
#' [shrinkage_corr()]), a plain
#' matrix, or a named list of such objects. When a list is supplied the gadget
#' offers a picker to switch between results.
#' @param x A correlation result, a numeric matrix, or a named list of those
#'   objects. Each element must be square with matching row/column names.
#' @param title Optional character title shown at the top of the gadget.
#' @param default_max_vars Integer; maximum number of variables pre-selected
#'   when the app opens. Defaults to 40 so very wide matrices start collapsed.
#' @return Invisibly returns `NULL`; the function is called for its side
#'   effect of launching a Shiny gadget.
#' @details This helper lives in `Suggests`; it requires the `shiny` and
#'   `shinyWidgets` packages at runtime and will optionally convert the plot to
#'   an interactive widget when `plotly` is installed. Variable selection uses
#'   a searchable picker, and clustering controls let you reorder variables via
#'   hierarchical clustering on either absolute or signed correlations with a
#'   choice of linkage methods.
#'
#' @importFrom rlang .data
#' @examples
#' if (interactive()) {
#'   data <- mtcars
#'   results <- list(
#'     Pearson = pearson_corr(data),
#'     Spearman = spearman_rho(data),
#'     Kendall = kendall_tau(data)
#'   )
#'   view_corr_shiny(results)
#' }
#'
#' @export
view_corr_shiny <- function(x, title = NULL, default_max_vars = 40L) {
  if (missing(x)) {
    abort_bad_arg("x", message = "must be supplied (correlation result or list).")
  }

  if (!requireNamespace("shiny", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg shiny} is required for {.fn view_corr_shiny}.")
  }
  if (!requireNamespace("shinyWidgets", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg shinyWidgets} is required for {.fn view_corr_shiny}.")
  }

  prepared <- .mc_prepare_corr_inputs(x)
  if (length(prepared) == 0L) {
    cli::cli_abort("{.fn view_corr_shiny} did not find a usable correlation matrix.")
  }

  use_plotly <- .mc_has_namespace("plotly")
  app_title <- title %||% "matrixCorr correlation viewer"
  logo_src <- .mc_register_logo_resource(.mc_logo_path())

  plot_widget <- if (use_plotly) {
    .mc_plotly_fn("plotlyOutput")("heatmap", height = "650px")
  } else {
    shiny::plotOutput("heatmap", height = "650px")
  }

  ui <- shiny::fluidPage(
    .mc_viewer_head(),
    shiny::div(
      class = "mc-app-shell",
      .mc_viewer_header(
        title = app_title,
        subtitle = "Explore matrix-style correlation outputs with searchable subsetting, responsive heatmaps, and focused visual inspection.",
        logo_src = logo_src,
        badges = list(
          .mc_header_badge("Correlation Viewer", tone = "neutral"),
          .mc_header_badge(
            if (use_plotly) "Interactive heatmap" else "Static heatmap",
            tone = if (use_plotly) "accent" else "neutral"
          )
        )
      ),
      shiny::sidebarLayout(
        shiny::sidebarPanel(
          width = 4,
          .mc_control_section(
            title = "Data",
            shinyWidgets::pickerInput(
              inputId = "matrix_choice",
              label = "Correlation object",
              choices = names(prepared),
              selected = names(prepared)[[1L]],
              multiple = FALSE,
              options = list(`live-search` = TRUE)
            ),
            shinyWidgets::pickerInput(
              inputId = "var_choice",
              label = "Variables (select to subset)",
              choices = colnames(prepared[[1L]]$matrix),
              selected = .mc_default_vars(prepared[[1L]]$matrix, default_max_vars),
              multiple = TRUE,
              options = list(`live-search` = TRUE, `actions-box` = TRUE)
            )
          ),
          .mc_control_section(
            title = "Display",
            shiny::sliderInput(
              inputId = "threshold",
              label = "Hide cells with |value| below",
              min = 0,
              max = 1,
              step = 0.05,
              value = 0
            ),
            shiny::sliderInput(
              inputId = "corr_range",
              label = "Keep variables with correlation in range",
              min = -1,
              max = 1,
              step = 0.05,
              value = c(-1, 1)
            ),
            shiny::checkboxInput("use_abs", "Colour by absolute value", value = FALSE),
            shiny::checkboxInput("mask_diag", "Hide diagonal", value = FALSE),
            shiny::selectInput(
              inputId = "cluster_mode",
              label = "Cluster variables",
              choices = c(
                "None" = "none",
                "Absolute correlation (|r|)" = "abs",
                "Signed correlation (r)" = "signed"
              ),
              selected = "none"
            ),
            shiny::conditionalPanel(
              condition = "input.cluster_mode !== 'none'",
              shiny::selectInput(
                inputId = "cluster_method",
                label = "Linkage method",
                choices = c(
                  "Complete" = "complete",
                  "Average" = "average",
                  "Single" = "single",
                  "Ward (D2)" = "ward.D2"
                ),
                selected = "complete"
              )
            ),
            shiny::checkboxInput("show_values", "Show cell labels", value = TRUE)
          ),
          .mc_control_section(
            title = "Actions",
            shiny::div(
              class = "mc-viewer-controls",
              shiny::actionButton("reset_controls", "Reset controls"),
              shiny::downloadButton("download_matrix", "Download displayed matrix")
            )
          )
        ),
        shiny::mainPanel(
          width = 8,
          shiny::div(
            class = "mc-tabset-shell",
            shiny::tabsetPanel(
              id = "corr_view",
              shiny::tabPanel(
                title = "Heatmap",
                shiny::div(
                  class = "mc-card mc-main-card",
                  .mc_card_header(
                    title = "Heatmap",
                    subtitle = "Filter, cluster, and inspect the currently displayed matrix."
                  ),
                  plot_widget,
                  shiny::uiOutput("status_alerts")
                ),
                shiny::div(
                  class = "mc-card mc-meta-card",
                  .mc_card_header(
                    title = "Summary",
                    subtitle = "Current object and display state."
                  ),
                  shiny::uiOutput("meta")
                )
              ),
              shiny::tabPanel(
                title = "Distribution",
                shiny::fluidRow(
                  shiny::column(
                    width = 7,
                    shiny::div(
                      class = "mc-card mc-main-card",
                      .mc_card_header(
                        title = "Correlation Distribution",
                        subtitle = "Boxplot of displayed off-diagonal values using unique variable pairs."
                      ),
                      shiny::plotOutput("dist_plot", height = "430px")
                    )
                  ),
                  shiny::column(
                    width = 5,
                    shiny::div(
                      class = "mc-card mc-meta-card",
                      .mc_card_header(
                        title = "Summary Table",
                        subtitle = "Computed from the currently displayed upper triangle."
                      ),
                      shiny::tableOutput("dist_summary")
                    )
                  )
                )
              )
            )
          )
        )
      )
    )
  )

  server <- function(input, output, session) {
    shiny::observe({
      .mc_update_corr_range_slider(
        session = session,
        input_id = "corr_range",
        use_abs = isTRUE(input$use_abs),
        value = input$corr_range
      )
    })

    shiny::observeEvent(input$matrix_choice, {
      info <- prepared[[input$matrix_choice]]
      vars <- colnames(info$matrix)
      shinyWidgets::updatePickerInput(
        session,
        inputId = "var_choice",
        choices = vars,
        selected = .mc_default_vars(info$matrix, default_max_vars)
      )
      shiny::updateSliderInput(session, "threshold", min = 0, max = 1, value = min(input$threshold %||% 0, 1))
    }, ignoreNULL = FALSE)

    shiny::observeEvent(input$reset_controls, {
      info <- current_matrix()
      shinyWidgets::updatePickerInput(
        session,
        inputId = "var_choice",
        choices = colnames(info$matrix),
        selected = .mc_default_vars(info$matrix, default_max_vars)
      )
      shiny::updateSliderInput(session, "threshold", value = 0)
      shiny::updateCheckboxInput(session, "use_abs", value = FALSE)
      shiny::updateCheckboxInput(session, "mask_diag", value = FALSE)
      shiny::updateSelectInput(session, "cluster_mode", selected = "none")
      shiny::updateSelectInput(session, "cluster_method", selected = "complete")
      shiny::updateCheckboxInput(session, "show_values", value = TRUE)
      .mc_update_corr_range_slider(
        session = session,
        input_id = "corr_range",
        use_abs = FALSE,
        value = c(-1, 1)
      )
    })

    current_matrix <- shiny::reactive({
      prepared[[input$matrix_choice]]
    })

    filtered_matrix <- shiny::reactive({
      info <- current_matrix()
      vars <- input$var_choice
      if (is.null(vars) || !length(vars)) {
        vars <- colnames(info$matrix)
      }
      vars <- intersect(vars, colnames(info$matrix))
      shiny::validate(shiny::need(length(vars) >= 2L, "Select at least two variables."))
      M <- info$matrix[vars, vars, drop = FALSE]
      range_filter <- .mc_filter_matrix_by_range(
        mat = M,
        corr_range = input$corr_range,
        use_abs = isTRUE(input$use_abs)
      )
      M <- range_filter$matrix
      shiny::validate(shiny::need(
        ncol(M) >= 2L,
        "Correlation range filter left fewer than two variables. Widen the range or select more variables."
      ))
      diag_map <- diag(M)
      if (!is.null(colnames(M))) names(diag_map) <- colnames(M)
      cluster_mode <- input$cluster_mode %||% "none"
      cluster_method <- input$cluster_method %||% "complete"
      status_messages <- range_filter$message
      if (!identical(cluster_mode, "none")) {
        if (nrow(M) < 3L) {
          status_messages <- c(status_messages, "Clustering requires at least three variables.")
        } else {
          res <- .mc_reorder_matrix(
            M,
            mode = cluster_mode,
            method = cluster_method
          )
          M <- res$matrix
          if (length(diag_map)) {
            if (!is.null(res$order)) {
              diag_map <- diag_map[res$order]
            }
            if (!is.null(colnames(M))) {
              names(diag_map) <- colnames(M)
            }
          }
          status_messages <- c(status_messages, res$message)
        }
      }
      thr <- input$threshold %||% 0
      if (thr > 0) {
        suppress <- abs(M) < thr
        M[suppress] <- NA_real_
      }
      if (isTRUE(input$mask_diag)) {
        diag(M) <- NA_real_
      } else if (length(diag_map)) {
        if (!is.null(colnames(M))) {
          diag(M) <- diag_map[colnames(M)]
        } else {
          diag(M) <- diag_map
        }
      }
      if (isTRUE(input$show_values) && prod(dim(M)) > .mc_heatmap_label_limit(use_plotly = use_plotly)) {
        status_messages <- c(
          status_messages,
          "Cell labels are hidden automatically for larger heatmaps to keep the viewer responsive."
        )
      }
      list(
        matrix = M,
        meta = info,
        status_messages = status_messages[!is.na(status_messages) & nzchar(status_messages)]
      )
    })

    output$meta <- shiny::renderUI({
      res <- filtered_matrix()
      info <- res$meta
      dims <- dim(res$matrix)
      total_vars <- ncol(info$matrix)
      .mc_meta_grid(c(
        "Result" = info$label,
        "Object class" = info$class,
        "Displayed variables" = sprintf("%d of %d", dims[[1L]], total_vars),
        "Signed values" = if (info$signed) "Yes" else "No"
      ))
    })

    output$status_alerts <- shiny::renderUI({
      .mc_status_notices(filtered_matrix()$status_messages)
    })

    output$download_matrix <- shiny::downloadHandler(
      filename = function() {
        sprintf(
          "%s-displayed-matrix.csv",
          .mc_sanitize_filename(input$matrix_choice %||% "matrixCorr")
        )
      },
      content = function(file) {
        utils::write.csv(filtered_matrix()$matrix, file = file, row.names = TRUE)
      }
    )

    plot_data <- shiny::reactive({
      res <- filtered_matrix()
      mat <- res$matrix
      signed <- res$meta$signed
      list(
        plot = .mc_build_heatmap(
          mat = mat,
          signed = signed,
          show_values = isTRUE(input$show_values),
          use_abs = isTRUE(input$use_abs),
          use_plotly = use_plotly
        ),
        signed = signed
      )
    })

    distribution_values <- shiny::reactive({
      .mc_corr_distribution_values(
        mat = filtered_matrix()$matrix,
        use_abs = isTRUE(input$use_abs)
      )
    })

    if (use_plotly) {
      output$heatmap <- .mc_plotly_fn("renderPlotly")({
        plot_data()$plot
      })
    } else {
      output$heatmap <- shiny::renderPlot({
        plot_data()$plot
      })
    }

    output$dist_plot <- shiny::renderPlot({
      values <- distribution_values()
      shiny::validate(shiny::need(
        length(values) > 0L,
        "No off-diagonal correlations are visible after filtering."
      ))
      .mc_build_corr_boxplot(values = values, use_abs = isTRUE(input$use_abs))
    })

    output$dist_summary <- shiny::renderTable(
      {
        .mc_corr_summary_table(distribution_values())
      },
      striped = TRUE,
      bordered = FALSE,
      spacing = "s",
      width = "100%",
      rownames = FALSE
    )
  }

  shiny::runApp(shiny::shinyApp(ui = ui, server = server))
  invisible(NULL)
}

.mc_default_vars <- function(mat, max_vars) {
  vars <- colnames(mat)
  if (is.null(vars)) {
    vars <- paste0("V", seq_len(ncol(mat)))
  }
  vars[seq_len(min(length(vars), max_vars))]
}

.mc_prepare_corr_inputs <- function(x) {
  parsed_single <- try(.mc_parse_corr_object(x, label = "default"), silent = TRUE)
  if (!inherits(parsed_single, "try-error")) {
    return(list(default = parsed_single))
  }

  if (!(is.list(x) && !is.matrix(x) && !inherits(x, "Matrix"))) {
    return(list())
  }

  objects <- x
  if (is.null(names(objects))) {
    names(objects) <- paste0("Matrix ", seq_along(objects))
  }

  out <- list()
  for (nm in names(objects)) {
    obj <- objects[[nm]]
    parsed <- try(.mc_parse_corr_object(obj, label = nm), silent = TRUE)
    if (!inherits(parsed, "try-error")) {
      out[[parsed$name]] <- parsed
    }
  }
  out
}

.mc_is_corr_result_like <- function(x) {
  if (inherits(x, "corr_result")) {
    return(TRUE)
  }
  if (isTRUE(attr(x, "corr_result", exact = TRUE))) {
    return(TRUE)
  }
  output_class <- attr(x, "corr_output_class", exact = TRUE)
  if (is.character(output_class) && length(output_class) == 1L && nzchar(output_class)) {
    return(TRUE)
  }
  output <- attr(x, "output", exact = TRUE)
  is.character(output) && length(output) == 1L &&
    output %in% c("matrix", "sparse", "edge_list")
}

.mc_parse_corr_object <- function(obj, label) {
  mat <- NULL
  signed <- TRUE
  desc <- NULL
  cls <- class(obj)
  if (inherits(obj, "partial_corr")) {
    mat <- obj$pcor
    desc <- attr(obj, "method", exact = TRUE) %||% "partial correlation"
  } else if (.mc_is_corr_result_like(obj)) {
    mat <- .mc_corr_as_dense_matrix(obj)
    desc <- attr(obj, "description", exact = TRUE) %||%
      attr(obj, "method", exact = TRUE) %||%
      label
  } else if (inherits(obj, "Matrix")) {
    mat <- as.matrix(obj)
  } else if (is.matrix(obj)) {
    mat <- obj
  } else if (is.list(obj) && !is.null(obj$pcor) && is.matrix(obj$pcor)) {
    mat <- obj$pcor
    desc <- "partial correlation"
  } else if (is.list(obj) && !is.null(obj$est) && is.matrix(obj$est)) {
    mat <- obj$est
    desc <- attr(obj, "description", exact = TRUE) %||%
      attr(obj, "method", exact = TRUE) %||%
      label
  } else {
    cli::cli_abort("Unsupported object class.")
  }
  mat <- as.matrix(mat)
  if (is.null(colnames(mat))) {
    colnames(mat) <- rownames(mat) <- paste0("V", seq_len(ncol(mat)))
  }
  if (nrow(mat) != ncol(mat)) {
    cli::cli_abort("Correlation matrices must be square.")
  }
  if (is.null(desc)) {
    desc <- attr(obj, "description", exact = TRUE) %||%
      attr(obj, "method", exact = TRUE) %||% label
  }
  signed <- any(mat < 0, na.rm = TRUE)
  list(
    name = label,
    matrix = mat,
    signed = signed,
    label = desc,
    class = paste(cls, collapse = ", ")
  )
}

.mc_reorder_matrix <- function(mat, mode = c("abs", "signed"), method = "complete") {
  mode <- match.arg(mode, c("abs", "signed"))
  allowed_methods <- c(
    complete = "complete",
    average  = "average",
    single   = "single",
    ward.d   = "ward.D",
    ward.d2  = "ward.D2",
    mcquitty = "mcquitty",
    median   = "median",
    centroid = "centroid"
  )
  method_key <- match.arg(tolower(method), names(allowed_methods))
  method <- allowed_methods[[method_key]]

  base_mat <- pmax(pmin(mat, 1), -1)
  dist_mat <- switch(
    mode,
    abs = 1 - abs(base_mat),
    signed = 0.5 * (1 - base_mat)
  )
  dist_mat <- (dist_mat + t(dist_mat)) / 2
  dist_mat[dist_mat < 0] <- 0
  dist_mat[dist_mat > 1] <- 1
  diag(dist_mat) <- 0
  dist_mat[!is.finite(dist_mat)] <- 1

  dist_obj <- try(suppressWarnings(stats::as.dist(dist_mat)), silent = TRUE)
  if (inherits(dist_obj, "try-error") || is.null(dist_obj)) {
    return(list(
      matrix = mat,
      message = "Unable to compute a distance matrix for clustering.",
      order = seq_len(nrow(mat))
    ))
  }

  hc <- try(stats::hclust(dist_obj, method = method), silent = TRUE)
  if (inherits(hc, "try-error")) {
    cond <- attr(hc, "condition")
    msg <- if (inherits(cond, "condition")) {
      conditionMessage(cond)
    } else {
      as.character(hc)
    }
    return(list(
      matrix = mat,
      message = paste("Clustering failed:", msg),
      order = seq_len(nrow(mat))
    ))
  }

  ord <- hc$order
  list(matrix = mat[ord, ord, drop = FALSE], message = NULL, order = ord)
}

.mc_filter_matrix_by_range <- function(mat, corr_range = NULL, use_abs = FALSE) {
  corr_range <- .mc_clamp_corr_range(corr_range, use_abs = use_abs)
  values <- if (isTRUE(use_abs)) abs(mat) else mat
  diag(values) <- NA_real_

  keep <- apply(
    values,
    2,
    function(x) any(!is.na(x) & x >= corr_range[[1L]] & x <= corr_range[[2L]])
  )
  if (!any(keep)) {
    return(list(
      matrix = mat[FALSE, FALSE, drop = FALSE],
      kept = character(),
      message = sprintf(
        "No variables have %s within [%s, %s].",
        if (isTRUE(use_abs)) "|correlation|" else "correlation",
        format(signif(corr_range[[1L]], digits = 3)),
        format(signif(corr_range[[2L]], digits = 3))
      )
    ))
  }

  kept <- colnames(mat)[keep]
  message <- NULL
  if (sum(!keep) > 0L) {
    message <- sprintf(
      "Correlation range filter kept %d of %d variables using %s within [%s, %s].",
      sum(keep),
      length(keep),
      if (isTRUE(use_abs)) "|correlation|" else "correlation",
      format(signif(corr_range[[1L]], digits = 3)),
      format(signif(corr_range[[2L]], digits = 3))
    )
  }

  list(
    matrix = mat[keep, keep, drop = FALSE],
    kept = kept,
    message = message
  )
}

.mc_clamp_corr_range <- function(value, use_abs = FALSE) {
  bounds <- if (isTRUE(use_abs)) c(0, 1) else c(-1, 1)
  if (is.null(value) || length(value) != 2L || any(!is.finite(value))) {
    return(bounds)
  }
  value <- sort(value)
  c(
    max(bounds[[1L]], min(bounds[[2L]], value[[1L]])),
    max(bounds[[1L]], min(bounds[[2L]], value[[2L]]))
  )
}

.mc_update_corr_range_slider <- function(session, input_id, use_abs = FALSE, value = NULL) {
  bounds <- if (isTRUE(use_abs)) c(0, 1) else c(-1, 1)
  label <- if (isTRUE(use_abs)) {
    "Keep variables with |correlation| in range"
  } else {
    "Keep variables with correlation in range"
  }
  shiny::updateSliderInput(
    session = session,
    inputId = input_id,
    label = label,
    min = bounds[[1L]],
    max = bounds[[2L]],
    value = .mc_clamp_corr_range(value, use_abs = use_abs),
    step = 0.05
  )
}

.mc_build_heatmap <- function(mat, signed, show_values, use_abs, use_plotly,
                              plotly_source = NULL) {
  if (use_abs) {
    plot_mat <- abs(mat)
    signed <- FALSE
  } else {
    plot_mat <- mat
  }
  show_values <- isTRUE(show_values) && prod(dim(plot_mat)) <= .mc_heatmap_label_limit(use_plotly = use_plotly)
  if (use_plotly) {
    return(.mc_build_heatmap_plotly(
      mat = plot_mat,
      signed = signed,
      show_values = show_values,
      plotly_source = plotly_source
    ))
  }
  df <- .mc_heatmap_df(plot_mat)
  fill_scale <- if (signed) {
    ggplot2::scale_fill_gradient2(
      low = "#FF6A6A",
      mid = "white",
      high = "#63B8FF",
      limits = c(-1, 1),
      midpoint = 0,
      na.value = "grey90",
      name = "r"
    )
  } else {
    ggplot2::scale_fill_gradient(
      low = "white",
      high = "steelblue1",
      limits = c(0, 1),
      na.value = "grey90",
      name = "|r|"
    )
  }
  p <- ggplot2::ggplot(
    df,
    ggplot2::aes(x = .data$Var2, y = .data$Var1, fill = .data$Value)
  ) +
    ggplot2::geom_tile(color = "white") +
    fill_scale +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank()
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(x = NULL, y = NULL)
  if (show_values) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(
        label = ifelse(is.na(.data$Value), "", sprintf("%.2f", .data$Value))
      ),
      size = 3,
      colour = "black"
    )
  }
  p
}

.mc_heatmap_df <- function(mat) {
  row_levels <- rownames(mat)
  if (is.null(row_levels)) {
    row_levels <- sprintf("V%02d", seq_len(nrow(mat)))
    rownames(mat) <- row_levels
  }
  col_levels <- colnames(mat)
  if (is.null(col_levels)) {
    col_levels <- sprintf("V%02d", seq_len(ncol(mat)))
    colnames(mat) <- col_levels
  }
  df <- as.data.frame(as.table(mat), stringsAsFactors = FALSE)
  names(df) <- c("Var1", "Var2", "Value")
  df$Var1 <- factor(df$Var1, levels = rev(row_levels))
  df$Var2 <- factor(df$Var2, levels = col_levels)
  df
}

.mc_corr_distribution_values <- function(mat, use_abs = FALSE) {
  if (!is.matrix(mat) || nrow(mat) < 2L || ncol(mat) < 2L) {
    return(numeric())
  }
  values <- if (isTRUE(use_abs)) abs(mat) else mat
  values <- values[upper.tri(values, diag = FALSE)]
  values[is.finite(values)]
}

.mc_corr_summary_table <- function(values) {
  if (!length(values)) {
    return(data.frame(
      Metric = c("Pairs shown", "Status"),
      Value = c("0", "No off-diagonal correlations visible"),
      stringsAsFactors = FALSE
    ))
  }

  quants <- stats::quantile(values, probs = c(0.25, 0.5, 0.75), names = FALSE, na.rm = TRUE)
  data.frame(
    Metric = c("Pairs shown", "Min", "Q1", "Median", "Mean", "Q3", "Max", "SD"),
    Value = c(
      as.character(length(values)),
      .mc_format_metric(min(values)),
      .mc_format_metric(quants[[1L]]),
      .mc_format_metric(quants[[2L]]),
      .mc_format_metric(mean(values)),
      .mc_format_metric(quants[[3L]]),
      .mc_format_metric(max(values)),
      .mc_format_metric(stats::sd(values))
    ),
    stringsAsFactors = FALSE
  )
}

.mc_format_metric <- function(x, digits = 3L) {
  if (!length(x) || anyNA(x) || !is.finite(x)) {
    return("NA")
  }
  format(signif(x, digits = digits), trim = TRUE)
}

.mc_corr_axis_limits <- function(values, use_abs = FALSE) {
  bounds <- if (isTRUE(use_abs)) c(0, 1) else c(-1, 1)
  if (!length(values) || all(!is.finite(values))) {
    return(bounds)
  }

  rng <- range(values, na.rm = TRUE, finite = TRUE)
  span <- diff(rng)
  pad <- if (span <= 0) {
    max(0.04, abs(rng[[1L]]) * 0.12)
  } else {
    max(0.03, span * 0.12)
  }
  limits <- c(rng[[1L]] - pad, rng[[2L]] + pad)
  limits[[1L]] <- max(bounds[[1L]], limits[[1L]])
  limits[[2L]] <- min(bounds[[2L]], limits[[2L]])

  if (diff(limits) < 0.08) {
    centre <- mean(limits)
    limits <- c(centre - 0.04, centre + 0.04)
    limits[[1L]] <- max(bounds[[1L]], limits[[1L]])
    limits[[2L]] <- min(bounds[[2L]], limits[[2L]])
  }

  limits
}

.mc_build_corr_boxplot <- function(values, use_abs = FALSE) {
  df <- data.frame(Group = "Displayed pairs", Value = values)
  limits <- .mc_corr_axis_limits(values = values, use_abs = use_abs)
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$Group, y = .data$Value))
  if (!isTRUE(use_abs)) {
    p <- p + ggplot2::geom_hline(
      yintercept = 0,
      colour = "#C8D2DD",
      linewidth = 0.6,
      linetype = "dashed"
    )
  }
  p <- p +
    ggplot2::geom_boxplot(
      width = 0.22,
      outlier.shape = NA,
      fill = if (isTRUE(use_abs)) "#A7D8FF" else "#C6D8FF",
      colour = "#153A5B",
      linewidth = 0.8
    ) +
    ggplot2::geom_jitter(
      width = 0.08,
      height = 0,
      size = 1.7,
      alpha = 0.3,
      colour = if (isTRUE(use_abs)) "#1874CD" else "#24557A"
    ) +
    ggplot2::coord_cartesian(ylim = limits) +
    ggplot2::labs(
      x = NULL,
      y = if (isTRUE(use_abs)) "|r|" else "r"
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      panel.grid.minor = ggplot2::element_blank(),
      panel.grid.major.x = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_text(colour = "#274055", face = "bold"),
      axis.text.y = ggplot2::element_text(colour = "#425466"),
      plot.background = ggplot2::element_rect(fill = "white", colour = NA),
      panel.background = ggplot2::element_rect(fill = "white", colour = NA)
    )
  p
}

.mc_plotly_click_pair <- function(event, vars = NULL) {
  if (is.null(event) || !NROW(event)) {
    return(NULL)
  }
  x <- event[["x"]][[1L]]
  y <- event[["y"]][[1L]]
  if (is.null(x) || is.null(y)) {
    return(NULL)
  }
  pair <- c(as.character(x), as.character(y))
  if (!is.null(vars) && !all(pair %in% vars)) {
    return(NULL)
  }
  pair
}

.mc_static_click_pair <- function(df, click, threshold = 15) {
  if (is.null(click) || is.null(df) || !nrow(df)) {
    return(NULL)
  }
  if (is.character(click[["x"]]) && is.character(click[["y"]])) {
    pair <- c(as.character(click[["x"]]), as.character(click[["y"]]))
    vars <- unique(c(as.character(df$Var1), as.character(df$Var2)))
    if (all(pair %in% vars)) {
      return(pair)
    }
  }
  hit <- shiny::nearPoints(
    df,
    click,
    xvar = "Var2",
    yvar = "Var1",
    threshold = threshold,
    maxpoints = 1,
    addDist = FALSE
  )
  if (!nrow(hit)) {
    return(NULL)
  }
  c(as.character(hit$Var2[[1L]]), as.character(hit$Var1[[1L]]))
}

.mc_heatmap_label_limit <- function(use_plotly = FALSE) {
  if (isTRUE(use_plotly)) 196L else 400L
}

.mc_build_heatmap_plotly <- function(mat, signed, show_values, plotly_source = NULL) {
  cols <- colnames(mat)
  if (is.null(cols)) {
    cols <- sprintf("V%02d", seq_len(ncol(mat)))
  }
  rows <- rownames(mat)
  if (is.null(rows)) {
    rows <- sprintf("V%02d", seq_len(nrow(mat)))
  }
  z <- mat[nrow(mat):1L, , drop = FALSE]
  y <- rev(rows)
  text_mat <- if (isTRUE(show_values)) {
    matrix(ifelse(is.na(z), "", sprintf("%.2f", z)), nrow = nrow(z), ncol = ncol(z))
  } else {
    NULL
  }
  plot_ly <- .mc_plotly_fn("plot_ly")
  layout_fn <- .mc_plotly_fn("layout")
  p <- plot_ly(
    x = cols,
    y = y,
    z = z,
    type = "heatmap",
    source = plotly_source,
    colorscale = .mc_plotly_colorscale(signed),
    reversescale = FALSE,
    zmin = if (signed) -1 else 0,
    zmax = 1,
    hoverongaps = FALSE,
    text = text_mat,
    texttemplate = if (isTRUE(show_values)) "%{text}" else NULL,
    textfont = if (isTRUE(show_values)) list(color = "black", size = 11) else NULL,
    hovertemplate = if (signed) {
      paste("x: %{x}<br>y: %{y}<br>r: %{z:.3f}<extra></extra>")
    } else {
      paste("x: %{x}<br>y: %{y}<br>|r|: %{z:.3f}<extra></extra>")
    },
    colorbar = list(title = if (signed) "r" else "|r|")
  )
  layout_fn(
    p,
    margin = list(l = 70, r = 30, b = 90, t = 20),
    xaxis = list(title = NULL, tickangle = 45),
    yaxis = list(title = NULL),
    paper_bgcolor = "white",
    plot_bgcolor = "white"
  )
}

.mc_plotly_colorscale <- function(signed) {
  if (isTRUE(signed)) {
    list(
      list(0.0, "#FF6A6A"),
      list(0.5, "white"),
      list(1.0, "#63B8FF")
    )
  } else {
    list(
      list(0.0, "white"),
      list(1.0, "#63B8FF")
    )
  }
}

.mc_has_namespace <- function(pkg) {
  !inherits(try(getNamespace(pkg), silent = TRUE), "try-error")
}

.mc_plotly_fn <- function(name) {
  if (!.mc_has_namespace("plotly")) {
    cli::cli_abort("Package {.pkg plotly} is required for {.arg use_plotly} = TRUE.")
  }
  getExportedValue("plotly", name)
}

.mc_viewer_header <- function(title, subtitle = NULL, badges = NULL, logo_src = NULL) {
  shiny::tags$div(
    class = "mc-hero",
    shiny::tags$div(
      class = "mc-hero-copy",
      shiny::tags$div(
        class = "mc-hero-title-row",
        if (!is.null(logo_src) && nzchar(logo_src)) shiny::tags$img(
          class = "mc-hero-logo",
          src = logo_src,
          alt = "matrixCorr logo"
        ),
        shiny::tags$div(class = "mc-hero-title", title)
      ),
      if (!is.null(subtitle) && nzchar(subtitle)) shiny::tags$p(class = "mc-hero-subtitle", subtitle)
    ),
    if (length(badges)) shiny::tags$div(class = "mc-hero-badges", badges)
  )
}

.mc_header_badge <- function(text, tone = c("neutral", "accent")) {
  tone <- match.arg(tone)
  shiny::tags$span(class = paste("mc-header-badge", paste0("mc-header-badge-", tone)), text)
}

.mc_control_section <- function(title, ...) {
  shiny::tags$div(
    class = "mc-control-section",
    shiny::tags$div(class = "mc-section-title", title),
    ...
  )
}

.mc_card_header <- function(title, subtitle = NULL) {
  shiny::tags$div(
    class = "mc-card-header",
    shiny::tags$div(class = "mc-card-title", title),
    if (!is.null(subtitle) && nzchar(subtitle)) shiny::tags$div(class = "mc-card-subtitle", subtitle)
  )
}

.mc_meta_grid <- function(items) {
  shiny::tags$div(
    class = "mc-meta-grid",
    lapply(seq_along(items), function(i) {
      shiny::tags$div(
        class = "mc-stat-card",
        shiny::tags$div(class = "mc-stat-label", names(items)[[i]]),
        shiny::tags$div(class = "mc-stat-value", as.character(items[[i]]))
      )
    })
  )
}

.mc_status_notices <- function(msgs) {
  if (is.null(msgs) || !length(msgs)) {
    return(NULL)
  }
  shiny::tagList(lapply(msgs, function(msg) {
    shiny::tags$div(
      class = "mc-status-note",
      shiny::tags$span(class = "mc-status-dot"),
      shiny::tags$span(msg)
    )
  }))
}

.mc_viewer_head <- function() {
  shiny::tags$head(
    shiny::tags$style(shiny::HTML(
      paste(
        "body { background: linear-gradient(180deg, #F3F7FB 0%, #ECF2F8 100%); color: #162535; }",
        ".container-fluid { max-width: 1580px; padding: 24px 24px 112px; }",
        ".mc-app-shell { position: relative; }",
        ".mc-hero { display: flex; justify-content: space-between; align-items: flex-start; gap: 24px; margin-bottom: 22px; padding: 28px 30px; border-radius: 26px; background: linear-gradient(135deg, #12263F 0%, #1C4E72 58%, #2B7A8E 100%); box-shadow: 0 18px 42px rgba(17, 37, 62, 0.18); color: #FFFFFF; }",
        ".mc-hero-copy { max-width: 860px; }",
        ".mc-hero-title-row { display: flex; align-items: center; gap: 16px; margin-bottom: 8px; }",
        ".mc-hero-logo { width: 74px; height: auto; display: block; flex: 0 0 auto; filter: drop-shadow(0 8px 18px rgba(5, 14, 24, 0.22)); }",
        ".mc-hero-title { font-size: 30px; line-height: 1.15; font-weight: 700; letter-spacing: -0.02em; margin: 0; }",
        ".mc-hero-subtitle { margin: 0; max-width: 760px; font-size: 14px; line-height: 1.6; color: rgba(255,255,255,0.82); }",
        ".mc-hero-badges { display: flex; flex-wrap: wrap; gap: 10px; justify-content: flex-end; }",
        ".mc-header-badge { display: inline-flex; align-items: center; padding: 8px 12px; border-radius: 999px; font-size: 12px; font-weight: 700; letter-spacing: 0.04em; text-transform: uppercase; }",
        ".mc-header-badge-neutral { background: rgba(255,255,255,0.16); color: #FFFFFF; border: 1px solid rgba(255,255,255,0.22); }",
        ".mc-header-badge-accent { background: #E7F5FA; color: #14405E; }",
        ".well { background: rgba(255,255,255,0.94); border: 1px solid #D9E4EE; border-radius: 24px; box-shadow: 0 14px 32px rgba(21, 37, 56, 0.08); padding: 22px; }",
        ".mc-control-section + .mc-control-section { margin-top: 20px; padding-top: 18px; border-top: 1px solid #E8EEF5; }",
        ".mc-section-title { margin: 0 0 12px; font-size: 11px; font-weight: 800; letter-spacing: 0.08em; text-transform: uppercase; color: #587086; }",
        ".mc-card { margin-bottom: 18px; padding: 22px 24px; border-radius: 24px; border: 1px solid #D9E4EE; background: rgba(255,255,255,0.96); box-shadow: 0 14px 34px rgba(21, 37, 56, 0.08); }",
        ".mc-card-header { margin-bottom: 16px; }",
        ".mc-card-title { font-size: 18px; font-weight: 700; color: #15283D; }",
        ".mc-card-subtitle { margin-top: 4px; font-size: 13px; line-height: 1.5; color: #60758A; }",
        ".mc-tabset-shell .nav-tabs { border-bottom: 0; margin-bottom: 18px; display: flex; gap: 10px; }",
        ".mc-tabset-shell .nav-tabs > li { float: none; }",
        ".mc-tabset-shell .nav-tabs > li > a { margin-right: 0; border: 0; border-radius: 999px; background: rgba(255,255,255,0.72); color: #33516A; font-weight: 700; padding: 10px 16px; }",
        ".mc-tabset-shell .nav-tabs > li.active > a, .mc-tabset-shell .nav-tabs > li.active > a:focus, .mc-tabset-shell .nav-tabs > li.active > a:hover { border: 0; background: #153A5B; color: #FFFFFF; box-shadow: 0 10px 20px rgba(21, 58, 91, 0.18); }",
        ".mc-meta-grid { display: grid; grid-template-columns: repeat(auto-fit, minmax(180px, 1fr)); gap: 14px; }",
        ".mc-stat-card { padding: 16px 18px; border-radius: 18px; background: linear-gradient(180deg, #F9FBFD 0%, #F1F6FA 100%); border: 1px solid #E3EBF3; }",
        ".mc-stat-label { font-size: 11px; font-weight: 800; letter-spacing: 0.08em; text-transform: uppercase; color: #6E8193; }",
        ".mc-stat-value { margin-top: 8px; font-size: 16px; line-height: 1.4; font-weight: 700; color: #17324A; word-break: break-word; }",
        ".mc-status-note { display: flex; align-items: flex-start; gap: 10px; margin-top: 12px; padding: 12px 14px; border-radius: 16px; background: #FFF8E8; border: 1px solid #F2DEAE; color: #6F5515; }",
        ".mc-status-dot { width: 9px; height: 9px; border-radius: 50%; margin-top: 5px; flex: 0 0 auto; background: #D9A321; }",
        ".mc-viewer-controls { display: flex; flex-wrap: wrap; gap: 10px; }",
        ".mc-viewer-controls .btn, .mc-viewer-controls .btn-default { margin: 0; border-radius: 12px; border: 0; padding: 10px 14px; font-weight: 700; box-shadow: none; }",
        ".mc-viewer-controls .btn-default { background: #153A5B; color: #FFFFFF; }",
        ".mc-viewer-controls .btn-default:hover, .mc-viewer-controls .btn-default:focus { background: #1D4C77; color: #FFFFFF; }",
        ".bootstrap-select > .dropdown-toggle, .form-control { min-height: 44px; border-radius: 14px; border-color: #D8E2EC; box-shadow: none; }",
        ".bootstrap-select > .dropdown-toggle { background: #FFFFFF; }",
        ".control-label { font-size: 12px; font-weight: 700; letter-spacing: 0.02em; color: #274055; }",
        ".checkbox label { color: #274055; font-weight: 600; }",
        ".irs--shiny .irs-bar, .irs--shiny .irs-from, .irs--shiny .irs-to, .irs--shiny .irs-single { background: #1C5A7B; border-top-color: #1C5A7B; border-bottom-color: #1C5A7B; }",
        ".irs--shiny .irs-handle > i:first-child { background: #153A5B; }",
        ".table { margin-bottom: 0; }",
        ".table > thead > tr > th { border-bottom: 1px solid #D9E4EE; font-size: 11px; font-weight: 800; letter-spacing: 0.08em; text-transform: uppercase; color: #6E8193; }",
        ".table > tbody > tr > td { border-top: 1px solid #EDF2F7; color: #1C3249; font-weight: 600; }",
        "@media (max-width: 768px) {",
        "  .container-fluid { padding: 16px 14px 84px; }",
        "  .mc-hero { padding: 22px 18px; border-radius: 22px; }",
        "  .mc-hero-title-row { align-items: flex-start; }",
        "  .mc-hero-logo { width: 62px; }",
        "  .mc-hero-title { font-size: 24px; }",
        "  .mc-hero-badges { justify-content: flex-start; }",
        "  .mc-card { padding: 18px 16px; border-radius: 20px; }",
        "}",
        sep = "\n"
      )
    ))
  )
}

.mc_viewer_footer <- function(logo_src) {
  if (is.null(logo_src) || !nzchar(logo_src)) {
    return(NULL)
  }
  shiny::tags$div(
    class = "mc-viewer-footer",
    shiny::tags$img(src = logo_src, alt = "matrixCorr logo"),
    shiny::tags$div(class = "mc-viewer-footer-text", "matrixCorr")
  )
}

.mc_logo_path <- function() {
  candidates <- c(
    system.file("app-www", "logo.svg", package = "matrixCorr"),
    file.path("inst", "app-www", "logo.svg"),
    file.path("man", "figures", "logo.svg")
  )
  candidates <- candidates[nzchar(candidates)]
  existing <- candidates[file.exists(candidates)]
  if (!length(existing)) {
    return(NULL)
  }
  normalizePath(existing[[1L]], winslash = "/", mustWork = FALSE)
}

.mc_register_logo_resource <- function(path) {
  if (is.null(path) || !nzchar(path) || !file.exists(path)) {
    return(NULL)
  }
  prefix <- paste0("matrixcorr-assets-", as.integer(stats::runif(1L, min = 1, max = 1e9)))
  shiny::addResourcePath(prefix, dirname(path))
  sprintf("%s/%s", prefix, basename(path))
}

.mc_sanitize_filename <- function(x) {
  out <- gsub("[^A-Za-z0-9_-]+", "-", x)
  out <- gsub("(^-+|-+$)", "", out)
  if (!nzchar(out)) "matrixCorr" else out
}

#' @title Gwet's AC1 and AC2 Agreement Coefficients
#'
#' @description
#' Estimates Gwet's chance-corrected agreement coefficient for either
#' unweighted nominal agreement (AC1) or weighted agreement (AC2) in two-rater
#' pairwise form or panel-level multi-rater form.
#'
#' @param data Input data. If \code{y} is supplied, \code{data} is the first
#'   rater vector. Otherwise:
#'   for \code{input = "pairwise"}, rows are units and columns are raters;
#'   for \code{input = "ratings"}, rows are items and columns are raters;
#'   for \code{input = "counts"}, rows are items and columns are categories
#'   containing rater counts.
#' @param y Optional second rater vector for scalar two-rater agreement.
#' @param weights Weight specification. \code{"unweighted"} yields AC1.
#'   Weighted schemes yield AC2. Supported built-in schemes are
#'   \code{"linear"}, \code{"quadratic"}, \code{"ordinal"},
#'   \code{"radical"}, \code{"ratio"}, \code{"circular"}, and
#'   \code{"bipolar"}. A custom numeric agreement-weight matrix may also be
#'   supplied. Character schemes must be supplied as a single value.
#' @param levels Optional explicit category labels. This is mainly useful when
#'   unused categories should still be retained in the calculation, and is the
#'   recommended way to define ordering for ordinal AC2 weights.
#' @param input One of \code{"pairwise"}, \code{"ratings"}, or \code{"counts"}.
#' @param na_method Missing-data rule. For scalar two-rater and pairwise matrix
#'   modes, \code{"error"} rejects missing values, \code{"complete"} applies
#'   listwise deletion in matrix mode, and \code{"pairwise"} uses complete
#'   pairs within each pair of raters. For \code{input = "ratings"},
#'   \code{"available"} retains items with at least \code{min_raters}
#'   observed ratings. For \code{input = "counts"}, missing values are not
#'   allowed.
#' @param min_raters Minimum number of observed ratings required for an item to
#'   be retained in panel mode. Must be at least \code{2}.
#' @param by_category Logical; if \code{TRUE}, attach one-vs-all category-wise
#'   AC1 estimates for panel-level fits. This is available only for the
#'   unweighted coefficient.
#' @param ci Logical; if \code{TRUE}, attach confidence intervals.
#' @param p_value Logical; if \code{TRUE}, attach test statistics and p-values.
#' @param conf_level Confidence level for intervals. Default is \code{0.95}.
#' @param se_method Standard-error method. \code{"asymptotic"} uses the
#'   analytic large-sample formula. \code{"jackknife"} is retained as an
#'   alternative for panel-level fits. \code{"none"} disables inferential
#'   output.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   C++ backend.
#' @param output One of \code{"matrix"}, \code{"sparse"}, or
#'   \code{"edge_list"} for pairwise matrix mode.
#' @param threshold Numeric threshold used only for thresholded pairwise matrix
#'   output.
#' @param diag Logical; include diagonal entries in thresholded pairwise matrix
#'   output.
#' @param verbose Logical; if \code{TRUE}, emit short progress messages.
#' @param ... Reserved for future extensions. Unsupported extra arguments are
#'   rejected.
#'
#' @details
#' For two raters, let \eqn{w_{ab}} denote the agreement weight for cell
#' \eqn{(a,b)}, \eqn{p_{ab}} the cell proportion, and
#' \eqn{\pi_k = (p_{k+} + p_{+k})/2}. Then
#' \deqn{ P_o = \sum_{a,b} w_{ab} p_{ab}, }
#' \deqn{ P_e =
#'        \frac{\sum_{a,b} w_{ab}}{q(q-1)}
#'        \sum_{k=1}^q \pi_k(1-\pi_k), }
#' and
#' \deqn{ AC = \frac{P_o - P_e}{1 - P_e}. }
#' With identity weights \eqn{w_{ab} = 1(a=b)} this is Gwet's AC1, the
#' nominal-agreement coefficient. With non-identity agreement weights this is
#' Gwet's AC2, which extends the same chance-correction idea to ordered or
#' partially creditable disagreement by letting near disagreements receive
#' larger agreement weights than distant disagreements.
#'
#' \strong{Category ordering for weighted AC2.} Built-in weighted schemes use
#' the category order to define near and distant disagreements. If
#' \code{levels} is supplied, that order is used. Otherwise, factor inputs use
#' their factor levels, numeric/integer/logical inputs use sorted observed
#' values, character inputs use first-observed order, and counts inputs use
#' column order (or \code{levels}, when supplied). For ordinal analyses,
#' supplying \code{levels} is recommended whenever first-observed character
#' order is not the intended scale order.
#'
#' For panel-level counts, if \eqn{r_{ik}} is the number of raters assigning
#' item \eqn{i} to category \eqn{k}, \eqn{r_i = \sum_k r_{ik}}, and
#' \eqn{w_{kl}} is the agreement-weight matrix, then the item-level agreement is
#' \deqn{ A_i =
#'        \frac{1}{r_i(r_i-1)}
#'        \sum_k r_{ik}\left(\sum_l w_{kl}r_{il}-1\right). }
#' The observed agreement is the mean of \eqn{A_i} over retained items. The
#' chance term uses the average item-wise category proportions
#' \eqn{\pi_k = m^{-1}\sum_i r_{ik}/r_i}, where \eqn{m} is the number of
#' retained items.
#'
#' \strong{Confidence intervals and inference.}
#'
#' The primary inferential path is the analytic large-sample method. For two
#' raters, define
#' \deqn{ s_{ab}
#'        =
#'        w_{ab}
#'        -
#'        \frac{2(1-AC)\sum_{u,v} w_{uv}}{q(q-1)}
#'        \left(1 - \frac{\pi_a + \pi_b}{2}\right). }
#' The backend variance estimator is
#' \deqn{ \widehat{\mathrm{Var}}(\widehat{AC})
#'        =
#'        \frac{1}{n(1-P_e)^2}
#'        \left[
#'        \sum_{a,b} p_{ab} s_{ab}^2
#'        -
#'        \left\{P_o - 2(1-AC)P_e\right\}^2
#'        \right], }
#' where \eqn{n} is the number of complete paired ratings.
#'
#' For panel-level counts, write
#' \deqn{ AC_i = \frac{A_i - P_e}{1 - P_e}, }
#' and define the item-specific chance term
#' \deqn{ P_{e,i}
#'        =
#'        \frac{\sum_{k,l} w_{kl}}{q(q-1)}
#'        \sum_{k=1}^q \frac{r_{ik}}{r_i}(1-\pi_k). }
#' The asymptotic linearised contribution used by the backend is
#' \deqn{ \xi_i
#'        =
#'        AC_i
#'        -
#'        \frac{2(1-AC)}{1-P_e}(P_{e,i} - P_e), }
#' giving
#' \deqn{ \widehat{\mathrm{Var}}(\widehat{AC})
#'        =
#'        \frac{1}{m(m-1)}
#'        \sum_{i=1}^m (\xi_i - AC)^2, }
#' where \eqn{m} is the number of retained items.
#'
#' The reported standard error is
#' \eqn{\widehat{\mathrm{se}}(\widehat{AC}) =
#' \sqrt{\widehat{\mathrm{Var}}(\widehat{AC})}}, and the confidence interval is
#' the t interval
#' \deqn{ \widehat{AC} \pm t_{1-\alpha/2,\nu}\,
#'        \widehat{\mathrm{se}}(\widehat{AC}), }
#' truncated to \eqn{[-1,1]}. Here \eqn{\nu = n-1} for the two-rater path and
#' \eqn{\nu = m-1} for panel-level asymptotic inference. The reported test
#' statistic is the corresponding t ratio for \eqn{H_0: AC = 0}. For
#' panel-level fits, \code{se_method = "jackknife"} remains available as a
#' second option, using the leave-one-item-out variance
#' \deqn{ \widehat{\mathrm{Var}}_{\mathrm{JK}}(\widehat{AC})
#'        =
#'        \frac{m-1}{m}
#'        \sum_{i=1}^m
#'        \left(\widehat{AC}_{(-i)} - \overline{AC}_{(-\cdot)}\right)^2. }
#'
#' @return
#' If \code{y} is supplied, a scalar numeric object of class
#' \code{c("gwet_ac", "numeric")}. If \code{input = "pairwise"}, a
#' \code{corr_result}. If \code{input} is \code{"ratings"} or \code{"counts"},
#' a one-row data frame with class
#' \code{c("gwet_ac", "agreement_result", "data.frame")}.
#'
#' @references
#' Gwet, K. L. (2008). Computing inter-rater reliability and its variance in
#' the presence of high agreement. \emph{British Journal of Mathematical and
#' Statistical Psychology}, 61, 29-48. \doi{10.1348/000711006X126600}
#'
#' @seealso
#' [cohen_kappa()] for two-rater nominal kappa;
#' [multirater_kappa()] for panel-level nominal kappa;
#' [weighted_kappa()] for weighted Cohen-type agreement.
#'
#' @examples
#' x <- c("A", "A", "B", "B", "A", "C")
#' y <- c("A", "B", "B", "B", "A", "C")
#' gwet_ac(x, y)
#' gwet_ac(x, y, weights = "quadratic", levels = c("A", "B", "C"))
#'
#' raters <- data.frame(
#'   r1 = c("A", "A", "B", "C", "A", "B"),
#'   r2 = c("A", "B", "B", "C", "A", "B"),
#'   r3 = c("A", "A", "B", "B", "A", "C"),
#'   stringsAsFactors = FALSE
#' )
#'
#' fit_pw <- gwet_ac(raters)
#' fit_panel <- gwet_ac(raters, input = "ratings")
#' print(fit_pw)
#' print(fit_panel)
#' estimate(fit_pw)
#' tidy(fit_pw)
#' tidy(fit_panel)
#'
#' @author Thiago de Paula Oliveira
#' @export
gwet_ac <- function(data,
                    y = NULL,
                    weights = c("unweighted", "linear", "quadratic", "ordinal", "radical", "ratio", "circular", "bipolar"),
                    levels = NULL,
                    input = c("pairwise", "ratings", "counts"),
                    na_method = c("error", "pairwise", "complete", "available"),
                    min_raters = 2L,
                    by_category = FALSE,
                    ci = FALSE,
                    p_value = FALSE,
                    conf_level = 0.95,
                    se_method = c("asymptotic", "jackknife", "none"),
                    n_threads = getOption("matrixCorr.threads", 1L),
                    output = c("matrix", "sparse", "edge_list"),
                    threshold = 0,
                    diag = TRUE,
                    verbose = FALSE,
                    ...) {
  .mc_extract_legacy_aliases(list(...), allowed = character())
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )
  input <- match_arg(input, c("pairwise", "ratings", "counts"), arg_name = "input")
  na_method <- validate_na_method(
    na_method,
    arg = "na_method",
    allowed = c("error", "pairwise", "complete", "available")
  )
  min_raters <- check_scalar_int_pos(min_raters, arg = "min_raters")
  if (min_raters < 2L) {
    abort_bad_arg("min_raters", message = "must be >= 2.")
  }
  check_bool(by_category, arg = "by_category")
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_bool(verbose, arg = "verbose")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  se_method <- match_arg(se_method, c("asymptotic", "jackknife", "none"), arg_name = "se_method")
  if ((isTRUE(ci) || isTRUE(p_value)) && identical(se_method, "none")) {
    abort_bad_arg("se_method", message = "must not be 'none' when ci or p_value is TRUE.")
  }

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg("output", message = "must be {.val matrix} when {.arg y} is supplied.")
    }
    if (identical(na_method, "available")) {
      abort_bad_arg("na_method", message = "na_method = 'available' is not defined when {.arg y} is supplied.")
    }
    if (identical(se_method, "jackknife") && (isTRUE(ci) || isTRUE(p_value))) {
      abort_bad_arg("se_method", message = "{.val jackknife} is not defined for two-rater AC1/AC2.")
    }

    x_labels <- .mc_nominal_vector_labels(data, arg = "data")
    y_labels <- .mc_nominal_vector_labels(y, arg = "y")
    check_same_length(x_labels, y_labels, arg_x = "data", arg_y = "y")
    if (identical(na_method, "error") && (anyNA(x_labels) || anyNA(y_labels))) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must not contain missing values when {.arg na_method} is {.val error}.",
        .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to drop incomplete pairs."
      )
    }
    if (!identical(na_method, "error")) {
      keep <- stats::complete.cases(x_labels, y_labels)
      x_labels <- x_labels[keep]
      y_labels <- y_labels[keep]
    }

    analysis_levels <- .mc_resolve_gwet_category_levels(
      cols = list(data, y),
      labels = list(x_labels, y_labels),
      levels = levels
    )
    enc <- .mc_encode_gwet_pair_labels(x_labels, y_labels, levels = analysis_levels)
    W <- .mc_resolve_gwet_weights(weights, enc$levels)
    fit <- gwet_ac_pair_cpp(
      enc$x,
      enc$y,
      weights = W,
      return_inference = isTRUE(ci) || isTRUE(p_value),
      conf_level = conf_level
    )
    return(.mc_gwet_ac_scalar(
      fit,
      weights = W,
      levels = enc$levels,
      ci = ci,
      p_value = p_value,
      conf_level = conf_level
    ))
  }

  if (identical(input, "pairwise")) {
    if (identical(na_method, "available")) {
      abort_bad_arg("na_method", message = "na_method = 'available' is not defined for pairwise matrix agreement.")
    }
    if (identical(se_method, "jackknife") && (isTRUE(ci) || isTRUE(p_value))) {
      abort_bad_arg("se_method", message = "{.val jackknife} is not defined for pairwise matrix AC1/AC2.")
    }
    return(.mc_gwet_ac_pairwise_matrix(
      data = data,
      weights = weights,
      levels = levels,
      na_method = na_method,
      ci = ci,
      p_value = p_value,
      conf_level = conf_level,
      n_threads = n_threads,
      output_cfg = output_cfg
    ))
  }

  if (!identical(output_cfg$output, "matrix") || output_cfg$thresholded) {
    abort_bad_arg(
      "output",
      message = "must be {.val matrix} with {.arg threshold} = 0 for panel-level agreement."
    )
  }
  if (identical(input, "ratings") && identical(na_method, "pairwise")) {
    abort_bad_arg(
      "na_method",
      message = "na_method = 'pairwise' is not defined for panel-level agreement; use 'available' to allow item-specific missing ratings."
    )
  }

  prep <- if (identical(input, "ratings")) {
    .mc_prepare_multirater_kappa_ratings(
      data = data,
      levels = levels,
      na_method = na_method,
      min_raters = min_raters
    )
  } else {
    .mc_prepare_multirater_kappa_counts(
      data = data,
      levels = levels,
      min_raters = min_raters
    )
  }
  W <- .mc_resolve_gwet_weights(weights, prep$categories)
  if (isTRUE(by_category) && !identical(attr(W, "weight_type", exact = TRUE), "unweighted")) {
    abort_bad_arg("by_category", message = "must be FALSE for weighted AC2 fits.")
  }

  inference_requested <- isTRUE(ci) || isTRUE(p_value)
  inference_code <- if (!inference_requested) {
    0L
  } else if (identical(se_method, "jackknife")) {
    2L
  } else {
    1L
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  raw <- if (identical(input, "ratings")) {
    gwet_ac_ratings_cpp(
      ratings = prep$X,
      n_levels = prep$n_levels,
      weights = W,
      na_code = switch(prep$na_method, error = 1L, complete = 2L, available = 3L),
      min_raters = min_raters,
      by_category = by_category,
      inference_code = inference_code,
      conf_level = conf_level,
      n_threads = n_threads
    )
  } else {
    gwet_ac_counts_cpp(
      counts = prep$counts,
      weights = W,
      by_category = by_category,
      inference_code = inference_code,
      conf_level = conf_level,
      n_threads = n_threads
    )
  }

  out <- .mc_build_gwet_ac_output(
    raw = raw,
    input = input,
    categories = prep$categories,
    weights = W,
    ci = ci,
    p_value = p_value,
    conf_level = conf_level,
    na_method = if (identical(input, "ratings")) prep$na_method else "available",
    min_raters = min_raters,
    se_method = if (isTRUE(inference_requested)) {
      if (!is.null(raw$se_method)) as.character(raw$se_method) else se_method
    } else {
      "none"
    },
    by_category = by_category,
    prep_diagnostics = prep$diagnostics
  )

  inform_if_verbose(
    "Computed {attr(out, 'coefficient', exact = TRUE)} for {attr(out, 'n_items', exact = TRUE)} item{?s} across {length(prep$categories)} categor{?y/ies}.",
    .verbose = verbose
  )
  out
}

.mc_resolve_gwet_category_levels <- function(cols, labels, levels = NULL) {
  if (!is.atomic(levels) || length(levels) < 2L) {
    if (!is.null(levels)) {
      abort_bad_arg("levels", message = "must be an atomic vector with at least two categories.")
    }
  }

  if (!is.null(levels)) {
    if (anyNA(levels)) {
      abort_bad_arg("levels", message = "must not contain missing values.")
    }
    level_keys <- as.character(levels)
    if (anyDuplicated(level_keys)) {
      abort_bad_arg("levels", message = "must not contain duplicate categories.")
    }
    observed <- unlist(lapply(labels, function(z) z[!is.na(z)]), use.names = FALSE)
    if (length(observed) && any(!observed %in% level_keys)) {
      abort_bad_arg("levels", message = "must contain every non-missing observed category.")
    }
    return(levels)
  }

  all_factor <- all(vapply(cols, is.factor, logical(1)))
  all_numericish <- all(vapply(cols, function(z) is.numeric(z) || is.integer(z), logical(1)))
  all_logical <- all(vapply(cols, is.logical, logical(1)))

  if (all_factor) {
    out <- unique(unlist(lapply(cols, levels), use.names = FALSE))
  } else if (all_numericish || all_logical) {
    observed <- unlist(lapply(cols, function(z) z[!is.na(z)]), use.names = FALSE)
    out <- sort(unique(observed))
  } else {
    out <- unique(unlist(lapply(labels, function(z) z[!is.na(z)]), use.names = FALSE))
  }

  if (length(out) < 2L) {
    abort_bad_arg("levels", message = "must define at least two categories.")
  }
  out
}

.mc_encode_gwet_pair_labels <- function(x_labels, y_labels, levels = NULL) {
  if (is.null(levels)) {
    return(.mc_encode_nominal_pair_labels(x_labels, y_labels))
  }
  level_keys <- as.character(levels)
  list(
    x = .mc_match_nominal_levels(x_labels, level_keys),
    y = .mc_match_nominal_levels(y_labels, level_keys),
    levels = levels,
    n_levels = length(level_keys)
  )
}

.mc_extract_gwet_pairwise_columns <- function(data, levels = NULL) {
  if (!is.data.frame(data) && !is.matrix(data)) {
    abort_bad_arg("data", message = "must be a matrix or data frame in matrix mode.")
  }

  cols <- if (is.data.frame(data)) unclass(data) else lapply(seq_len(ncol(data)), function(j) data[, j])
  col_names <- if (is.data.frame(data)) names(data) else colnames(data)
  if (length(cols) < 2L) {
    abort_bad_arg("data", message = "must contain at least two nominal columns.")
  }
  labels <- vector("list", length(cols))
  for (j in seq_along(cols)) {
    col_arg <- if (!is.null(col_names) && nzchar(col_names[[j]] %||% "")) {
      sprintf("data$%s", col_names[[j]])
    } else {
      sprintf("data[[%d]]", j)
    }
    labels[[j]] <- .mc_nominal_vector_labels(cols[[j]], arg = col_arg)
  }
  categories <- .mc_resolve_gwet_category_levels(cols = cols, labels = labels, levels = levels)
  level_keys <- as.character(categories)

  X <- matrix(
    NA_integer_,
    nrow = length(labels[[1L]]),
    ncol = length(labels),
    dimnames = list(NULL, col_names)
  )
  for (j in seq_along(labels)) {
    X[, j] <- .mc_match_nominal_levels(labels[[j]], level_keys)
  }
  list(
    matrix = X,
    levels = categories,
    n_levels = length(level_keys),
    names = col_names
  )
}

.mc_gwet_weight_support <- function(categories) {
  if (is.numeric(categories) || is.integer(categories)) {
    return(as.numeric(categories))
  }
  seq_along(categories)
}

.mc_resolve_gwet_weights <- function(weights, categories) {
  n_levels <- length(categories)
  check_scalar_numeric(n_levels, arg = "categories", lower = 1, closed_lower = TRUE)

  if (is.character(weights)) {
    choices <- c("unweighted", "linear", "quadratic", "ordinal", "radical", "ratio", "circular", "bipolar")
    if (length(weights) != 1L) {
      if (identical(weights, choices)) {
        weights <- choices[[1L]]
      } else {
        abort_bad_arg("weights", message = "must be a single character weight scheme or a numeric square matrix.")
      }
    }
    if (is.na(weights)) {
      abort_bad_arg("weights", message = "must be a single character weight scheme or a numeric square matrix.")
    }
    key <- tolower(weights[[1L]])
    alias_map <- c(
      unweighted = "unweighted",
      identity = "unweighted",
      linear = "linear",
      quadratic = "quadratic",
      ordinal = "ordinal",
      radical = "radical",
      ratio = "ratio",
      circular = "circular",
      bipolar = "bipolar"
    )
    resolved <- unname(alias_map[key])
    if (!length(resolved) || is.na(resolved)) {
      abort_bad_arg(
        "weights",
        message = paste(
          "must be one of {.val unweighted}, {.val linear}, {.val quadratic},",
          "{.val ordinal}, {.val radical}, {.val ratio}, {.val circular},",
          "{.val bipolar}, or a numeric square matrix."
        )
      )
    }

    K <- as.integer(n_levels)
    if (identical(resolved, "unweighted")) {
      W <- diag(1, nrow = K, ncol = K)
    } else {
      support <- .mc_gwet_weight_support(categories)
      if (!is.numeric(support) || any(!is.finite(support))) {
        support <- seq_len(K)
      }
      if (length(support) != K) {
        support <- seq_len(K)
      }
      xmin <- min(support)
      xmax <- max(support)
      span <- xmax - xmin
      idx_i <- matrix(rep(support, each = K), nrow = K)
      idx_j <- matrix(rep(support, times = K), nrow = K)
      if (identical(resolved, "linear")) {
        W <- if (K <= 1L || span == 0) matrix(1, K, K) else 1 - abs(idx_i - idx_j) / abs(span)
      } else if (identical(resolved, "quadratic")) {
        W <- if (K <= 1L || span == 0) matrix(1, K, K) else 1 - ((idx_i - idx_j)^2) / (span^2)
      } else if (identical(resolved, "radical")) {
        W <- if (K <= 1L || span == 0) matrix(1, K, K) else 1 - sqrt(abs(idx_i - idx_j)) / sqrt(abs(span))
      } else if (identical(resolved, "ratio")) {
        denom <- ((xmax - xmin) / (xmax + xmin))^2
        if (!is.finite(denom) || denom <= 0) {
          support <- seq_len(K)
          idx_i <- matrix(rep(support, each = K), nrow = K)
          idx_j <- matrix(rep(support, times = K), nrow = K)
          denom <- ((max(support) - min(support)) / (max(support) + min(support)))^2
        }
        W <- 1 - (((idx_i - idx_j) / (idx_i + idx_j))^2) / denom
        W[!is.finite(W)] <- 0
      } else if (identical(resolved, "circular")) {
        U <- span + 1
        base <- sin(pi * (idx_i - idx_j) / U)^2
        W <- 1 - base / max(base)
        W[!is.finite(W)] <- 1
      } else if (identical(resolved, "bipolar")) {
        W <- matrix(0, K, K)
        for (i in seq_len(K)) {
          for (j in seq_len(K)) {
            if (i == j) {
              W[i, j] <- 1
            } else {
              denom <- (((support[i] + support[j]) - 2 * xmin) * (2 * xmax - (support[i] + support[j])))
              W[i, j] <- if (!is.finite(denom) || denom == 0) 0 else
                (support[i] - support[j])^2 / denom
            }
          }
        }
        W <- 1 - W / max(W)
        W[!is.finite(W)] <- 1
      } else {
        W <- matrix(0, K, K)
        for (i in seq_len(K)) {
          for (j in seq_len(K)) {
            nkl <- abs(i - j) + 1
            W[i, j] <- nkl * (nkl - 1) / 2
          }
        }
        W <- 1 - W / max(W)
      }
    }
    storage.mode(W) <- "double"
    diag(W) <- 1
    W[W < 0] <- 0
    W[W > 1] <- 1
    dimnames(W) <- list(as.character(categories), as.character(categories))
    attr(W, "weight_type") <- resolved
    attr(W, "coefficient") <- if (identical(resolved, "unweighted")) "AC1" else "AC2"
    return(W)
  }

  if (!is.numeric(weights) || !is.matrix(weights)) {
    abort_bad_arg("weights", message = "must be a supported character scheme or a numeric square matrix.")
  }
  check_matrix_dims(weights, arg = "weights")
  if (nrow(weights) != ncol(weights)) {
    abort_bad_arg("weights", message = "must be a square matrix.")
  }
  if (nrow(weights) != n_levels) {
    abort_bad_arg("weights", message = "must have dimension {n_levels} x {n_levels}.", n_levels = n_levels)
  }
  if (any(!is.finite(weights))) {
    abort_bad_arg("weights", message = "must contain only finite values.")
  }
  if (any(weights < 0 | weights > 1)) {
    abort_bad_arg("weights", message = "must contain values in [0, 1].")
  }
  if (any(abs(diag(weights) - 1) > sqrt(.Machine$double.eps))) {
    abort_bad_arg("weights", message = "must have diagonal entries equal to 1.")
  }
  if (isTRUE(max(abs(weights - t(weights))) > sqrt(.Machine$double.eps))) {
    abort_bad_arg("weights", message = "must be symmetric.")
  }
  W <- unclass(weights)
  storage.mode(W) <- "double"
  attr(W, "weight_type") <- "custom"
  attr(W, "coefficient") <- if (isTRUE(all.equal(W, diag(n_levels), tolerance = sqrt(.Machine$double.eps)))) "AC1" else "AC2"
  W
}

.mc_gwet_ac_pairwise_matrix <- function(data,
                                        weights,
                                        levels,
                                        na_method,
                                        ci,
                                        p_value,
                                        conf_level,
                                        n_threads,
                                        output_cfg) {
  enc <- .mc_extract_gwet_pairwise_columns(data, levels = levels)
  X <- enc$matrix
  col_names <- enc$names
  dn <- .mc_square_dimnames(col_names)
  W <- .mc_resolve_gwet_weights(weights, enc$levels)

  listwise_meta <- NULL
  if (identical(na_method, "error") && anyNA(X)) {
    abort_bad_arg(
      "data",
      message = "contains missing values when {.arg na_method} is {.val error}.",
      .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to handle incomplete units."
    )
  }
  if (identical(na_method, "complete")) {
    keep <- stats::complete.cases(X)
    listwise_meta <- list(
      na_method = "complete",
      n_original = nrow(X),
      n_complete_rows = sum(keep),
      n_removed = nrow(X) - sum(keep),
      complete_rows = keep,
      common_sample = TRUE
    )
    X <- X[keep, , drop = FALSE]
  }

  direct_threshold <- !isTRUE(ci) &&
    !isTRUE(p_value) &&
    !identical(na_method, "pairwise") &&
    isTRUE(output_cfg$thresholded)

  if (direct_threshold) {
    trip <- gwet_ac_threshold_triplets_cpp(
      X,
      weights = W,
      drop_unused_levels = is.null(levels),
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      n_threads = n_threads
    )
    n_complete_mat <- matrix(
      as.integer(nrow(X)),
      nrow = ncol(X),
      ncol = ncol(X),
      dimnames = dn
    )
    diag(n_complete_mat) <- colSums(!is.na(X))
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = "gwet_ac",
      method = "gwet_ac",
      description = sprintf("Pairwise Gwet %s agreement matrix", attr(W, "coefficient", exact = TRUE)),
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = as.integer(c(ncol(X), ncol(X))),
      source_dimnames = dn,
      diagnostics = .mc_merge_diagnostics(
        list(n_complete = n_complete_mat),
        listwise_meta
      ),
      symmetric = TRUE
    ))
  }

  fit <- gwet_ac_matrix_cpp(
    X,
    weights = W,
    drop_unused_levels = is.null(levels),
    pairwise_complete = identical(na_method, "pairwise"),
    return_inference = isTRUE(ci) || isTRUE(p_value),
    conf_level = conf_level,
    n_threads = n_threads
  )
  fit$est <- .mc_set_matrix_dimnames(fit$est, col_names)
  fit$n_complete <- .mc_set_matrix_dimnames(fit$n_complete, col_names)
  fit$po <- .mc_set_matrix_dimnames(fit$po, col_names)
  fit$pe <- .mc_set_matrix_dimnames(fit$pe, col_names)
  if (!is.null(fit$se)) {
    fit$se <- .mc_set_matrix_dimnames(fit$se, col_names)
    fit$lwr <- .mc_set_matrix_dimnames(fit$lwr, col_names)
    fit$upr <- .mc_set_matrix_dimnames(fit$upr, col_names)
    fit$statistic <- .mc_set_matrix_dimnames(fit$statistic, col_names)
    fit$p_value <- .mc_set_matrix_dimnames(fit$p_value, col_names)
  }

  diagnostics <- .mc_merge_diagnostics(
    list(
      n_complete = fit$n_complete,
      observed_agreement = fit$po,
      expected_agreement = fit$pe,
      weights = W,
      weight_type = attr(W, "weight_type", exact = TRUE)
    ),
    if (identical(na_method, "pairwise")) {
      list(na_method = "pairwise", common_sample = FALSE)
    } else {
      listwise_meta
    }
  )
  ci_attr <- if (isTRUE(ci)) {
    list(
      est = fit$est,
      lwr.ci = fit$lwr,
      upr.ci = fit$upr,
      conf.level = conf_level,
      ci.method = "asymptotic"
    )
  } else {
    NULL
  }
  inference_attr <- if (isTRUE(p_value)) {
    list(
      method = "t_asymptotic_gwet",
      estimate = fit$est,
      se = fit$se,
      statistic = fit$statistic,
      p_value = fit$p_value,
      n_obs = fit$n_complete
    )
  } else {
    NULL
  }

  out <- .mc_new_corr_matrix(
    mat = fit$est,
    estimator_class = "gwet_ac",
    method = "gwet_ac",
    description = sprintf("Pairwise Gwet %s agreement matrix", attr(W, "coefficient", exact = TRUE)),
    diagnostics = diagnostics,
    ci = ci_attr,
    conf.level = if (isTRUE(ci)) conf_level else NULL,
    symmetric = TRUE,
    extra_attrs = list(
      inference = inference_attr,
      weights = W,
      weight_type = attr(W, "weight_type", exact = TRUE),
      coefficient = attr(W, "coefficient", exact = TRUE),
      levels = enc$levels
    )
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_gwet_ac_scalar <- function(fit,
                               weights,
                               levels,
                               ci = FALSE,
                               p_value = FALSE,
                               conf_level = 0.95) {
  est <- as.numeric(fit$est)
  out <- structure(est, class = c("gwet_ac", "numeric"))
  attr(out, "method") <- "gwet_ac"
  attr(out, "description") <- sprintf("Pairwise Gwet %s agreement", attr(weights, "coefficient", exact = TRUE))
  attr(out, "package") <- "matrixCorr"
  attr(out, "estimate") <- est
  attr(out, "levels") <- levels
  attr(out, "weights") <- weights
  attr(out, "weight_type") <- attr(weights, "weight_type", exact = TRUE)
  attr(out, "coefficient") <- attr(weights, "coefficient", exact = TRUE)
  attr(out, "diagnostics") <- list(
    n_complete = as.integer(fit$n_complete),
    observed_agreement = as.numeric(fit$po),
    expected_agreement = as.numeric(fit$pe)
  )
  if (isTRUE(ci)) {
    attr(out, "ci") <- list(
      est = est,
      lwr.ci = as.numeric(fit$lwr),
      upr.ci = as.numeric(fit$upr),
      conf.level = conf_level,
      ci.method = "asymptotic"
    )
    attr(out, "conf.level") <- conf_level
    attr(out, "conf_level") <- conf_level
  }
  if (isTRUE(p_value)) {
    attr(out, "inference") <- list(
      method = "t_asymptotic_gwet",
      estimate = est,
      se = as.numeric(fit$se),
      statistic = as.numeric(fit$statistic),
      p_value = as.numeric(fit$p_value),
      n_obs = as.integer(fit$n_complete)
    )
  }
  out
}

.mc_build_gwet_ac_output <- function(raw,
                                     input,
                                     categories,
                                     weights,
                                     ci,
                                     p_value,
                                     conf_level,
                                     na_method,
                                     min_raters,
                                     se_method,
                                     by_category,
                                     prep_diagnostics) {
  coefficient <- attr(weights, "coefficient", exact = TRUE)
  inference_requested <- isTRUE(ci) || isTRUE(p_value)
  out <- data.frame(
    method = "gwet_ac",
    ac = as.numeric(raw$estimate),
    observed_agreement = as.numeric(raw$observed_agreement),
    expected_agreement = as.numeric(raw$expected_agreement),
    n_items = as.integer(raw$n_items),
    n_categories = as.integer(raw$n_categories),
    n_raters_min = as.integer(raw$n_raters_min),
    n_raters_max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced),
    n_ratings_total = as.integer(raw$n_ratings_total),
    se = if (!is.null(raw$se)) as.numeric(raw$se) else NA_real_,
    statistic = if (isTRUE(p_value) && !is.null(raw$statistic)) as.numeric(raw$statistic) else NA_real_,
    p_value = if (isTRUE(p_value) && !is.null(raw$p_value)) as.numeric(raw$p_value) else NA_real_,
    lwr.ci = if (isTRUE(ci) && !is.null(raw$lwr)) as.numeric(raw$lwr) else NA_real_,
    upr.ci = if (isTRUE(ci) && !is.null(raw$upr)) as.numeric(raw$upr) else NA_real_,
    conf.level = if (isTRUE(ci)) conf_level else NA_real_,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  by_category_df <- NULL
  if (isTRUE(by_category)) {
    by_category_df <- data.frame(
      category = as.character(categories),
      ac = as.numeric(raw$by_category_estimate),
      observed_agreement = as.numeric(raw$by_category_observed_agreement),
      expected_agreement = as.numeric(raw$by_category_expected_agreement),
      se = if (!is.null(raw$by_category_se)) as.numeric(raw$by_category_se) else NA_real_,
      statistic = if (isTRUE(p_value) && !is.null(raw$by_category_statistic)) as.numeric(raw$by_category_statistic) else NA_real_,
      p_value = if (isTRUE(p_value) && !is.null(raw$by_category_p_value)) as.numeric(raw$by_category_p_value) else NA_real_,
      lwr.ci = if (isTRUE(ci) && !is.null(raw$by_category_lwr)) as.numeric(raw$by_category_lwr) else NA_real_,
      upr.ci = if (isTRUE(ci) && !is.null(raw$by_category_upr)) as.numeric(raw$by_category_upr) else NA_real_,
      n_items = as.integer(raw$n_items),
      n_ratings_total = as.integer(raw$n_ratings_total),
      category_count = as.integer(raw$category_counts),
      category_proportion = as.numeric(raw$category_proportions),
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  }

  class(out) <- c("gwet_ac", "agreement_result", "data.frame")
  attr(out, "method") <- "gwet_ac"
  attr(out, "description") <- sprintf("Gwet %s chance-corrected agreement", coefficient)
  attr(out, "package") <- "matrixCorr"
  attr(out, "input") <- input
  attr(out, "categories") <- categories
  attr(out, "n_categories") <- length(categories)
  attr(out, "n_items") <- as.integer(raw$n_items)
  attr(out, "n_raters") <- list(
    min = as.integer(raw$n_raters_min),
    max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced)
  )
  attr(out, "na_method") <- na_method
  attr(out, "min_raters") <- min_raters
  attr(out, "weights") <- weights
  attr(out, "weight_type") <- attr(weights, "weight_type", exact = TRUE)
  attr(out, "coefficient") <- coefficient
  attr(out, "diagnostics") <- c(
    list(
      category_counts = as.integer(raw$category_counts),
      category_proportions = as.numeric(raw$category_proportions),
      item_category_counts = {
        mat <- raw$item_category_counts
        if (!is.null(mat)) {
          mat <- as.matrix(mat)
          colnames(mat) <- as.character(categories)
          rownames(mat) <- paste0("item_", seq_len(nrow(mat)))
        }
        mat
      },
      item_agreement = as.numeric(raw$item_agreement),
      n_raters_by_item = as.integer(raw$n_raters_by_item)
    ),
    prep_diagnostics
  )
  attr(out, "by_category") <- by_category_df
  attr(out, "se.method") <- if (isTRUE(inference_requested)) se_method else "none"
  attr(out, "conf.level") <- if (isTRUE(ci)) conf_level else NULL
  attr(out, "reference") <- "Gwet (2008), British Journal of Mathematical and Statistical Psychology, 61, 29-48, doi:10.1348/000711006X126600"
  out
}

.mc_as_multirater_plot_input <- function(x) {
  tmp <- x
  tmp$kappa <- tmp$ac
  tmp$ac <- NULL
  by_cat <- attr(tmp, "by_category", exact = TRUE)
  if (is.data.frame(by_cat) && "ac" %in% names(by_cat)) {
    by_cat$kappa <- by_cat$ac
    by_cat$ac <- NULL
    attr(tmp, "by_category") <- by_cat
  }
  class(tmp) <- c("multirater_kappa", "agreement_result", "data.frame")
  attr(tmp, "kappa_method") <- attr(x, "coefficient", exact = TRUE) %||% "gwet_ac"
  attr(tmp, "exact") <- FALSE
  tmp
}

#' @rdname gwet_ac
#' @method summary gwet_ac
#' @param object A \code{gwet_ac} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param ci_digits Integer; number of decimal places for confidence limits.
#' @param p_digits Integer; number of decimal places for p-values.
#' @param ... Unused.
#' @export
summary.gwet_ac <- function(object,
                            digits = 4,
                            ci_digits = 3,
                            p_digits = 4,
                            ...) {
  if (inherits(object, "corr_result")) {
    return(summary.corr_matrix(object, ...))
  }
  if (inherits(object, "agreement_result") && is.data.frame(object)) {
    out <- as.data.frame(object, stringsAsFactors = FALSE, check.names = FALSE)
    preferred_order <- c(
      "method", "ac", "observed_agreement", "expected_agreement", "n_items",
      "n_categories", "n_raters_min", "n_raters_max", "balanced",
      "n_ratings_total", "se", "statistic", "p_value", "lwr.ci", "upr.ci",
      "conf.level"
    )
    out <- out[, c(intersect(preferred_order, names(out)), setdiff(names(out), preferred_order)), drop = FALSE]
    drop_if_all_na <- c("se", "statistic", "p_value", "lwr.ci", "upr.ci", "conf.level")
    keep_cols <- vapply(names(out), function(nm) {
      if (!nm %in% drop_if_all_na) {
        return(TRUE)
      }
      !all(is.na(out[[nm]]))
    }, logical(1))
    out <- out[, keep_cols, drop = FALSE]
    num_cols <- vapply(out, is.numeric, logical(1))
    out[num_cols] <- lapply(out[num_cols], round, digits = digits)
    out <- .mc_finalize_summary_df(out, class_name = "summary.gwet_ac")
    attr(out, "result_type") <- "multirater"
    attr(out, "digits") <- digits
    attr(out, "by_category") <- attr(object, "by_category", exact = TRUE)
    attr(out, "se.method") <- attr(object, "se.method", exact = TRUE)
    attr(out, "coefficient") <- attr(object, "coefficient", exact = TRUE)
    return(out)
  }

  check_inherits(object, "gwet_ac")
  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  ci_attr <- attr(object, "ci", exact = TRUE)
  inf_attr <- attr(object, "inference", exact = TRUE)

  df <- data.frame(
    estimate = as.numeric(object),
    n_complete = as.integer(diag_attr$n_complete %||% NA_integer_),
    observed_agreement = as.numeric(diag_attr$observed_agreement %||% NA_real_),
    expected_agreement = as.numeric(diag_attr$expected_agreement %||% NA_real_),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (is.list(ci_attr)) {
    df$lwr <- as.numeric(ci_attr$lwr.ci %||% NA_real_)
    df$upr <- as.numeric(ci_attr$upr.ci %||% NA_real_)
  }
  if (is.list(inf_attr)) {
    df$se <- as.numeric(inf_attr$se %||% NA_real_)
    df$statistic <- as.numeric(inf_attr$statistic %||% NA_real_)
    df$p_value <- as.numeric(inf_attr$p_value %||% NA_real_)
  }
  out <- .mc_finalize_summary_df(df, class_name = "summary.gwet_ac")
  attr(out, "result_type") <- "scalar"
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "p_digits") <- p_digits
  attr(out, "has_ci") <- is.list(ci_attr)
  attr(out, "has_p") <- is.list(inf_attr)
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE) %||%
    attr(object, "conf_level", exact = TRUE) %||%
    NA_real_
  attr(out, "ci_method") <- if (is.list(ci_attr)) ci_attr$ci.method %||% NA_character_ else NA_character_
  attr(out, "inference_method") <- if (is.list(inf_attr)) inf_attr$method %||% NA_character_ else NA_character_
  attr(out, "coefficient") <- attr(object, "coefficient", exact = TRUE)
  out
}

#' @rdname gwet_ac
#' @method print summary.gwet_ac
#' @param x A \code{summary.gwet_ac} object.
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.summary.gwet_ac <- function(x,
                                  digits = NULL,
                                  n = NULL,
                                  topn = NULL,
                                  max_vars = NULL,
                                  width = NULL,
                                  show_ci = NULL,
                                  ...) {
  coefficient <- attr(x, "coefficient", exact = TRUE) %||% "AC"
  if (identical(attr(x, "result_type", exact = TRUE), "multirater")) {
    digits <- .mc_coalesce(digits, attr(x, "digits", exact = TRUE) %||% 4L)
    digest <- c(
      coefficient = coefficient,
      ac = formatC(x$ac[[1L]], format = "f", digits = digits),
      items = .mc_count_fmt(x$n_items[[1L]]),
      categories = .mc_count_fmt(x$n_categories[[1L]])
    )
    se_method <- attr(x, "se.method", exact = TRUE) %||% "none"
    if (!identical(se_method, "none")) {
      digest <- c(digest, se_method = se_method)
    }
    .mc_print_named_digest(digest, header = "Gwet agreement summary")
    cat("\n")
    .mc_print_summary_table(
      x,
      header = "Estimate table",
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      print_overview = FALSE,
      ...
    )
    return(invisible(x))
  }

  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  digits <- .mc_coalesce(digits, .mc_coalesce(attr(x, "digits", exact = TRUE), 4L))
  ci_digits <- .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)
  p_digits <- .mc_coalesce(attr(x, "p_digits", exact = TRUE), digits)

  digest <- c(
    coefficient = coefficient,
    estimate = formatC(as.numeric(x$estimate[[1L]]), format = "f", digits = digits),
    n_complete = .mc_count_fmt(x$n_complete[[1L]]),
    observed_agreement = formatC(as.numeric(x$observed_agreement[[1L]]), format = "f", digits = digits),
    expected_agreement = formatC(as.numeric(x$expected_agreement[[1L]]), format = "f", digits = digits)
  )
  if (isTRUE(attr(x, "has_ci", exact = TRUE))) {
    digest <- c(
      digest,
      ci = sprintf("%g%%", 100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE)))),
      ci_method = attr(x, "ci_method", exact = TRUE) %||% NA_character_
    )
  }
  if (isTRUE(attr(x, "has_p", exact = TRUE))) {
    digest <- c(
      digest,
      inference = attr(x, "inference_method", exact = TRUE) %||% NA_character_
    )
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = "Gwet agreement summary")
  cat("\n")

  preview <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  if (identical(show_ci, "no")) {
    preview <- preview[, setdiff(names(preview), c("lwr", "upr")), drop = FALSE]
  }
  if ("estimate" %in% names(preview)) preview$estimate <- round(preview$estimate, digits)
  if ("observed_agreement" %in% names(preview)) preview$observed_agreement <- round(preview$observed_agreement, digits)
  if ("expected_agreement" %in% names(preview)) preview$expected_agreement <- round(preview$expected_agreement, digits)
  if ("lwr" %in% names(preview)) preview$lwr <- round(preview$lwr, ci_digits)
  if ("upr" %in% names(preview)) preview$upr <- round(preview$upr, ci_digits)
  if ("se" %in% names(preview)) preview$se <- round(preview$se, digits)
  if ("statistic" %in% names(preview)) preview$statistic <- round(preview$statistic, digits)
  if ("p_value" %in% names(preview)) preview$p_value <- round(preview$p_value, p_digits)

  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  .mc_print_preview_table(
    preview,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = FALSE,
    summary_hint = FALSE
  )
  invisible(x)
}

#' @rdname gwet_ac
#' @method print gwet_ac
#' @param x A \code{gwet_ac} object.
#' @param show_by_category Logical; whether to print attached category-wise
#'   panel results when available.
#' @export
print.gwet_ac <- function(x,
                          digits = 4,
                          n = NULL,
                          topn = NULL,
                          max_vars = NULL,
                          width = NULL,
                          show_ci = NULL,
                          show_by_category = FALSE,
                          ...) {
  coefficient <- attr(x, "coefficient", exact = TRUE) %||% "AC"
  if (inherits(x, "corr_result")) {
    .mc_print_corr_matrix(
      x,
      header = sprintf("Gwet %s agreement matrix", coefficient),
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      ...
    )
    return(invisible(x))
  }

  if (inherits(x, "agreement_result") && is.data.frame(x)) {
    check_bool(show_by_category, arg = "show_by_category")
    cat("Gwet agreement\n")
    cat("  coefficient        :", coefficient, "\n")
    cat("  estimate           :", formatC(x$ac[[1L]], format = "f", digits = digits), "\n")
    cat("  observed agreement :", formatC(x$observed_agreement[[1L]], format = "f", digits = digits), "\n")
    cat("  expected agreement :", formatC(x$expected_agreement[[1L]], format = "f", digits = digits), "\n")
    cat("  items              :", .mc_count_fmt(x$n_items[[1L]]), "\n")
    cat("  categories         :", .mc_count_fmt(x$n_categories[[1L]]), "\n")
    nr <- attr(x, "n_raters", exact = TRUE)
    cat(
      "  raters per item    :",
      paste0(.mc_count_fmt(nr$min), " to ", .mc_count_fmt(nr$max)),
      if (isTRUE(nr$balanced)) "(balanced)" else "(unbalanced)",
      "\n"
    )
    if (is.finite(x$lwr.ci[[1L]]) && is.finite(x$upr.ci[[1L]])) {
      cat(
        "  CI                 :",
        sprintf(
          "%g%% [%s, %s]",
          100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))),
          formatC(x$lwr.ci[[1L]], format = "f", digits = digits),
          formatC(x$upr.ci[[1L]], format = "f", digits = digits)
        ),
        "\n"
      )
    }
    if (is.finite(x$p_value[[1L]])) {
      cat("  p-value            :", formatC(x$p_value[[1L]], format = "f", digits = digits), "\n")
    }
    if (isTRUE(show_by_category)) {
      by_cat <- attr(x, "by_category", exact = TRUE)
      if (is.data.frame(by_cat) && nrow(by_cat)) {
        cat("\nBy-category AC1\n\n")
        print.data.frame(by_cat, row.names = FALSE, ...)
      }
    }
    return(invisible(x))
  }

  invisible(print.summary.gwet_ac(
    summary.gwet_ac(x),
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  ))
}

#' @rdname gwet_ac
#' @method plot gwet_ac
#' @param title Optional plot title.
#' @param low_color Fill/color used for negative agreement.
#' @param high_color Fill/color used for positive agreement.
#' @param mid_color Unused placeholder for API consistency with matrix heatmaps.
#' @param value_text_size Text size for estimate labels in matrix plots.
#' @param ci_text_size Text size for CI labels in matrix plots.
#' @param show_value Logical; whether to print estimate and CI labels.
#' @param type Plot type for panel-level fits: \code{"agreement_map"},
#'   \code{"estimate"}, \code{"item_agreement"}, \code{"category_proportion"},
#'   or \code{"by_category"}.
#' @param bins Integer number of bins retained for compatibility with the
#'   panel-level item-agreement plot.
#' @export
plot.gwet_ac <- function(x,
                         title = NULL,
                         low_color = "indianred1",
                         high_color = "steelblue1",
                         mid_color = "white",
                         value_text_size = 4,
                         ci_text_size = 3,
                         show_value = TRUE,
                         type = c("agreement_map", "estimate", "item_agreement", "category_proportion", "by_category"),
                         bins = 30L,
                         ...) {
  coefficient <- attr(x, "coefficient", exact = TRUE) %||% "AC"
  if (inherits(x, "corr_result")) {
    return(.mc_plot_corr_result(
      x,
      title = title %||% sprintf("Gwet %s agreement heatmap", coefficient),
      low_color = low_color,
      high_color = high_color,
      mid_color = mid_color,
      value_text_size = value_text_size,
      ci_text_size = ci_text_size,
      show_value = show_value,
      ...
    ))
  }

  if (inherits(x, "agreement_result") && is.data.frame(x)) {
    return(plot.multirater_kappa(
      .mc_as_multirater_plot_input(x),
      type = match.arg(type),
      bins = bins,
      ...
    ))
  }

  check_inherits(x, "gwet_ac")
  check_bool(show_value, arg = "show_value")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  ci_attr <- attr(x, "ci", exact = TRUE)
  est <- as.numeric(x)
  df <- data.frame(
    label = coefficient,
    estimate = est,
    lwr = if (is.list(ci_attr)) as.numeric(ci_attr$lwr.ci %||% NA_real_) else NA_real_,
    upr = if (is.list(ci_attr)) as.numeric(ci_attr$upr.ci %||% NA_real_) else NA_real_,
    stringsAsFactors = FALSE
  )
  df$label_y <- 1
  df$estimate_label <- sprintf("%.2f", df$estimate)
  df$ci_label <- ifelse(
    is.finite(df$lwr) & is.finite(df$upr),
    sprintf("[%.2f, %.2f]", df$lwr, df$upr),
    NA_character_
  )
  point_color <- if (is.finite(est) && est < 0) low_color else high_color

  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$estimate, y = .data$label_y)) +
    ggplot2::geom_vline(xintercept = 0, color = "gray85", linewidth = 0.4) +
    ggplot2::geom_vline(xintercept = 1, color = "gray90", linewidth = 0.4, linetype = "dashed")
  if (is.finite(df$lwr[[1L]]) && is.finite(df$upr[[1L]])) {
    p <- p + ggplot2::geom_segment(
      ggplot2::aes(x = .data$lwr, xend = .data$upr, y = .data$label_y, yend = .data$label_y),
      linewidth = 1,
      color = point_color
    )
  }
  p <- p +
    ggplot2::geom_point(size = 3, color = point_color) +
    ggplot2::scale_x_continuous(limits = c(-1, 1)) +
    ggplot2::scale_y_continuous(breaks = 1, labels = coefficient) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.title.y = ggplot2::element_blank(),
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::labs(
      title = title %||% sprintf("Gwet %s agreement", coefficient),
      x = coefficient
    )

  if (isTRUE(show_value)) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(label = .data$estimate_label),
        nudge_y = 0.18,
        size = 4
      )
    if (is.finite(df$lwr[[1L]]) && is.finite(df$upr[[1L]])) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = .data$ci_label),
          nudge_y = -0.18,
          size = 3
        )
    }
  }
  p
}

manual_gwet_pair <- function(x, y, levels = NULL, weights = NULL) {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  if (is.null(levels)) {
    levels <- unique(c(as.character(x), as.character(y)))
  }
  q <- length(levels)
  if (q < 2L || length(x) < 2L) {
    return(list(ac = NA_real_, observed = NA_real_, expected = NA_real_, n = length(x)))
  }

  if (is.null(weights)) {
    weights <- diag(q)
  }
  tab <- table(
    factor(as.character(x), levels = as.character(levels)),
    factor(as.character(y), levels = as.character(levels))
  )
  p <- unclass(tab) / sum(tab)
  pi_k <- (rowSums(p) + colSums(p)) / 2
  observed <- sum(weights * p)
  expected <- sum(weights) * sum(pi_k * (1 - pi_k)) / (q * (q - 1))
  ac <- if (!is.finite(1 - expected) || (1 - expected) <= 0) {
    NA_real_
  } else {
    (observed - expected) / (1 - expected)
  }
  list(ac = ac, observed = observed, expected = expected, n = sum(tab))
}

manual_gwet_counts <- function(counts, weights = NULL) {
  counts <- as.matrix(counts)
  n_items <- nrow(counts)
  q <- ncol(counts)
  if (n_items < 2L || q < 2L) {
    return(list(ac = NA_real_, observed = NA_real_, expected = NA_real_))
  }
  if (is.null(weights)) {
    weights <- diag(q)
  }

  n_i <- rowSums(counts)
  item_agreement <- vapply(seq_len(n_items), function(i) {
    ri <- n_i[[i]]
    if (ri < 2L) {
      return(NA_real_)
    }
    sum(counts[i, ] * (drop(weights %*% counts[i, ]) - 1)) / (ri * (ri - 1))
  }, numeric(1))
  observed <- mean(item_agreement, na.rm = TRUE)
  pi_k <- colMeans(counts / n_i)
  expected <- sum(weights) * sum(pi_k * (1 - pi_k)) / (q * (q - 1))
  ac <- if (!is.finite(1 - expected) || (1 - expected) <= 0) {
    NA_real_
  } else {
    (observed - expected) / (1 - expected)
  }

  list(
    ac = ac,
    observed = observed,
    expected = expected,
    item_agreement = item_agreement,
    category_proportions = colSums(counts) / sum(counts),
    pi_k = pi_k
  )
}

collapse_counts_for_category_ac1 <- function(counts, j) {
  counts <- as.matrix(counts)
  cbind(counts[, j], rowSums(counts) - counts[, j])
}

ratings_to_counts <- function(ratings, levels = NULL) {
  ratings <- as.matrix(ratings)
  if (is.null(levels)) {
    levels <- unique(as.character(ratings[!is.na(ratings)]))
  }
  out <- matrix(0L, nrow(ratings), length(levels), dimnames = list(NULL, levels))
  for (i in seq_len(nrow(ratings))) {
    tab <- table(factor(as.character(ratings[i, !is.na(ratings[i, ])]), levels = levels))
    out[i, ] <- as.integer(tab)
  }
  out
}

test_that("gwet_ac reproduces a manual two-rater binary AC1 example", {
  x <- c(rep("A", 25), rep("B", 25))
  y <- c(rep("A", 20), rep("B", 5), rep("A", 10), rep("B", 15))
  manual <- manual_gwet_pair(x, y)
  fit <- gwet_ac(x, y)

  expect_s3_class(fit, "gwet_ac")
  expect_equal(as.numeric(fit), manual$ac, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$observed_agreement, manual$observed, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$expected_agreement, manual$expected, tolerance = 1e-12)
})

test_that("gwet_ac reproduces a manual two-rater three-category AC1 example", {
  tab <- matrix(
    c(
      12, 3, 1,
      4, 10, 2,
      2, 5, 11
    ),
    nrow = 3,
    byrow = TRUE,
    dimnames = list(x = c("A", "B", "C"), y = c("A", "B", "C"))
  )
  x <- rep(rownames(tab), times = rowSums(tab))
  y <- unlist(lapply(seq_len(nrow(tab)), function(i) {
    rep(colnames(tab), times = tab[i, ])
  }), use.names = FALSE)

  manual <- manual_gwet_pair(x, y, levels = c("A", "B", "C"))
  fit <- gwet_ac(
    factor(x, levels = c("A", "B", "C")),
    factor(y, levels = c("A", "B", "C"))
  )

  expect_equal(attr(fit, "diagnostics")$observed_agreement, manual$observed, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$expected_agreement, manual$expected, tolerance = 1e-12)
  expect_equal(as.numeric(fit), manual$ac, tolerance = 1e-12)
})

test_that("gwet_ac reproduces a manual weighted AC2 example", {
  x <- factor(c("low", "low", "mid", "mid", "high", "high"), levels = c("low", "mid", "high"))
  y <- factor(c("low", "mid", "mid", "high", "high", "mid"), levels = c("low", "mid", "high"))
  W <- matrix(
    c(
      1.0, 0.5, 0.0,
      0.5, 1.0, 0.5,
      0.0, 0.5, 1.0
    ),
    nrow = 3,
    byrow = TRUE
  )
  manual <- manual_gwet_pair(x, y, levels = levels(x), weights = W)
  fit <- gwet_ac(x, y, weights = W)

  expect_equal(as.numeric(fit), manual$ac, tolerance = 1e-12)
  expect_identical(attr(fit, "coefficient"), "AC2")
  expect_equal(attr(fit, "diagnostics")$observed_agreement, manual$observed, tolerance = 1e-12)
  expect_equal(attr(fit, "diagnostics")$expected_agreement, manual$expected, tolerance = 1e-12)
})

test_that("gwet_ac rejects ambiguous multi-value weight schemes", {
  x <- c("A", "A", "B", "B")
  y <- c("A", "B", "B", "A")

  expect_error(
    gwet_ac(x, y, weights = c("linear", "quadratic")),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    .mc_resolve_gwet_weights(c("linear", "quadratic"), c("A", "B")),
    class = "matrixCorr_arg_error"
  )
  expect_s3_class(gwet_ac(x, y), "gwet_ac")
})

test_that("gwet_ac ordinal weights respect explicit and implicit category ordering", {
  x <- factor(c("low", "high", "mid", "low"), levels = c("high", "mid", "low"))
  y <- factor(c("high", "high", "mid", "low"), levels = c("high", "mid", "low"))
  fit_factor <- gwet_ac(x, y, weights = "linear")

  expect_identical(attr(fit_factor, "levels"), c("high", "mid", "low"))
  expect_identical(dimnames(attr(fit_factor, "weights"))[[1L]], c("high", "mid", "low"))
  expect_equal(attr(fit_factor, "weights")["high", "low"], 0)
  expect_equal(attr(fit_factor, "weights")["high", "mid"], 0.5)

  fit_explicit <- gwet_ac(
    as.character(x),
    as.character(y),
    weights = "linear",
    levels = c("low", "mid", "high")
  )
  expect_identical(attr(fit_explicit, "levels"), c("low", "mid", "high"))
  expect_identical(dimnames(attr(fit_explicit, "weights"))[[1L]], c("low", "mid", "high"))
  expect_equal(attr(fit_explicit, "weights")["low", "high"], 0)

  counts <- matrix(
    c(2, 0, 1, 0, 1, 2),
    nrow = 2,
    byrow = TRUE,
    dimnames = list(NULL, c("high", "mid", "low"))
  )
  fit_counts <- gwet_ac(counts, input = "counts", weights = "linear")
  expect_identical(attr(fit_counts, "categories"), c("high", "mid", "low"))
  expect_identical(dimnames(attr(fit_counts, "weights"))[[1L]], c("high", "mid", "low"))
})

test_that("gwet_ac differs from Cohen's kappa in a high-prevalence example", {
  x <- c(rep("A", 90), rep("B", 10))
  y <- c(rep("A", 85), rep("B", 5), rep("A", 5), rep("B", 5))

  fit_gwet <- gwet_ac(x, y)
  fit_cohen <- cohen_kappa(x, y)
  manual <- manual_gwet_pair(x, y)

  expect_equal(as.numeric(fit_gwet), manual$ac, tolerance = 1e-12)
  expect_false(isTRUE(all.equal(as.numeric(fit_gwet), as.numeric(fit_cohen), tolerance = 1e-6)))
})

test_that("gwet_ac matrix mode is symmetric and matches scalar pairwise values", {
  raters <- data.frame(
    r1 = c("A", "A", "B", "B", "C", "C", "A", "B"),
    r2 = c("A", "B", "B", "B", "C", "A", "A", "B"),
    r3 = c(1, 1, 2, 2, 3, 3, 1, 2),
    r4 = c(TRUE, FALSE, FALSE, FALSE, TRUE, TRUE, TRUE, FALSE),
    stringsAsFactors = FALSE
  )

  fit <- gwet_ac(raters)
  mat <- as.matrix(fit)
  expect_true(isSymmetric(mat))
  expect_equal(unname(diag(mat)), rep(1, ncol(raters)))

  for (j in seq_len(ncol(raters) - 1L)) {
    for (k in (j + 1L):ncol(raters)) {
      scalar <- gwet_ac(raters[[j]], raters[[k]])
      expect_equal(mat[j, k], as.numeric(scalar), tolerance = 1e-12)
    }
  }
})

test_that("gwet_ac output modes integrate with corr_result helpers", {
  raters <- data.frame(
    a = c("A", "A", "B", "B", "C", "A", "B", "C"),
    b = c("A", "B", "B", "B", "C", "A", "B", "C"),
    c = c("A", "A", "B", "C", "C", "A", "B", "B"),
    d = c("A", "B", "A", "B", "C", "C", "B", "C"),
    stringsAsFactors = FALSE
  )

  fit_matrix <- gwet_ac(raters, output = "matrix")
  expect_s3_class(fit_matrix, c("corr_matrix", "gwet_ac", "corr_result", "matrix"))

  edge <- gwet_ac(raters, output = "edge_list", threshold = 0.2, diag = FALSE)
  edge_df <- as.data.frame(edge, stringsAsFactors = FALSE)
  expect_true(all(c("row", "col", "value") %in% names(edge_df)))

  sparse <- gwet_ac(raters, output = "sparse", threshold = 0.2, diag = FALSE)
  expect_s4_class(sparse, "sparseMatrix")

  expect_error(
    gwet_ac(raters, input = "ratings", output = "edge_list", threshold = 0.1),
    "must be .*matrix"
  )
})

test_that("gwet_ac missing-data modes behave as documented", {
  raters <- data.frame(
    a = c("A", "A", "B", NA, "C", "A"),
    b = c("A", "B", "B", "B", NA, "A"),
    c = c("A", "A", NA, "B", "C", "A"),
    stringsAsFactors = FALSE
  )

  expect_error(gwet_ac(raters, na_method = "error"), "contains missing values")

  fit_complete <- gwet_ac(raters, na_method = "complete")
  keep <- stats::complete.cases(raters)
  expected_complete <- matrix(sum(keep), ncol(raters), ncol(raters))
  diag(expected_complete) <- rep(sum(keep), ncol(raters))
  dimnames(expected_complete) <- dimnames(as.matrix(fit_complete))
  expect_equal(attr(fit_complete, "diagnostics")$n_complete, expected_complete)

  fit_pairwise <- gwet_ac(raters, na_method = "pairwise")
  n_complete <- attr(fit_pairwise, "diagnostics")$n_complete
  expect_equal(n_complete["a", "b"], sum(stats::complete.cases(raters[c("a", "b")])))
  expect_equal(n_complete["a", "c"], sum(stats::complete.cases(raters[c("a", "c")])))
  expect_equal(n_complete["b", "c"], sum(stats::complete.cases(raters[c("b", "c")])))
})

test_that("gwet_ac attaches asymptotic CI and inference metadata", {
  x <- factor(c("A", "A", "B", "B", "C", "A", "B", "C", "A", "C"))
  y <- factor(c("A", "B", "B", "B", "C", "A", "B", "C", "A", "B"))

  fit <- gwet_ac(x, y, ci = TRUE, p_value = TRUE)
  ci_attr <- attr(fit, "ci")
  inf_attr <- attr(fit, "inference")

  expect_true(is.list(ci_attr))
  expect_identical(ci_attr$ci.method, "asymptotic")
  expect_true(all(is.finite(c(ci_attr$lwr.ci, ci_attr$upr.ci))))
  expect_true(is.list(inf_attr))
  expect_true(all(is.finite(c(inf_attr$se, inf_attr$statistic, inf_attr$p_value))))
})

test_that("gwet_ac returns NA for a degenerate custom-weight denominator", {
  x <- factor(c("A", "A", "B", "B"), levels = c("A", "B"))
  y <- factor(c("A", "B", "A", "B"), levels = c("A", "B"))
  W <- matrix(1, 2, 2)
  fit <- gwet_ac(x, y, weights = W, ci = TRUE, p_value = TRUE)

  expect_true(is.na(as.numeric(fit)))
  expect_true(is.na(attr(fit, "inference")$se))
  expect_true(is.na(attr(fit, "ci")$lwr.ci))
  expect_true(is.na(attr(fit, "ci")$upr.ci))
})

test_that("gwet_ac multi-rater ratings matches a manual counts implementation", {
  ratings <- data.frame(
    r1 = c("A", "A", "B", "C", "A", "B"),
    r2 = c("A", "B", "B", "C", "A", "B"),
    r3 = c("A", "A", "B", "B", "A", "C"),
    r4 = c("A", "A", "B", "C", "B", "B"),
    stringsAsFactors = FALSE
  )

  counts <- ratings_to_counts(ratings, levels = c("A", "B", "C"))
  manual <- manual_gwet_counts(counts)
  fit <- gwet_ac(ratings, input = "ratings")

  expect_equal(fit$observed_agreement[[1L]], manual$observed, tolerance = 1e-12)
  expect_equal(fit$expected_agreement[[1L]], manual$expected, tolerance = 1e-12)
  expect_equal(fit$ac[[1L]], manual$ac, tolerance = 1e-12)
})

test_that("gwet_ac weighted multi-rater AC2 matches a manual counts implementation", {
  ratings <- data.frame(
    r1 = c("L", "L", "M", "H", "M", "H"),
    r2 = c("L", "M", "M", "H", "H", "H"),
    r3 = c("L", "L", "H", "H", "M", "M"),
    stringsAsFactors = FALSE
  )
  counts <- ratings_to_counts(ratings, levels = c("L", "M", "H"))
  W <- matrix(
    c(
      1.0, 0.5, 0.0,
      0.5, 1.0, 0.5,
      0.0, 0.5, 1.0
    ),
    nrow = 3,
    byrow = TRUE
  )
  manual <- manual_gwet_counts(counts, weights = W)
  fit <- gwet_ac(ratings, input = "ratings", weights = W)

  expect_identical(attr(fit, "coefficient"), "AC2")
  expect_equal(fit$observed_agreement[[1L]], manual$observed, tolerance = 1e-12)
  expect_equal(fit$expected_agreement[[1L]], manual$expected, tolerance = 1e-12)
  expect_equal(fit$ac[[1L]], manual$ac, tolerance = 1e-12)
})

test_that("gwet_ac ratings and counts panel inputs agree", {
  ratings <- data.frame(
    r1 = c("A", "A", "B", "C", "A", "B"),
    r2 = c("A", "B", "B", "C", "A", "B"),
    r3 = c("A", "A", "B", "B", "A", "C"),
    r4 = c("A", "A", "B", "C", "B", "B"),
    stringsAsFactors = FALSE
  )
  counts <- ratings_to_counts(ratings, levels = c("A", "B", "C"))
  colnames(counts) <- c("A", "B", "C")

  fit_ratings <- gwet_ac(ratings, input = "ratings")
  fit_counts <- gwet_ac(counts, input = "counts")
  expect_equal(fit_ratings$ac[[1L]], fit_counts$ac[[1L]], tolerance = 1e-12)
  expect_equal(fit_ratings$observed_agreement[[1L]], fit_counts$observed_agreement[[1L]], tolerance = 1e-12)
  expect_equal(fit_ratings$expected_agreement[[1L]], fit_counts$expected_agreement[[1L]], tolerance = 1e-12)
})

test_that("gwet_ac by-category panel results match manual binary collapses", {
  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 3, 0),
    c(0, 1, 2)
  )
  colnames(counts) <- c("A", "B", "C")

  fit <- gwet_ac(counts, input = "counts", by_category = TRUE)
  by_cat <- attr(fit, "by_category")

  expect_true(is.data.frame(by_cat))
  expect_equal(nrow(by_cat), ncol(counts))
  for (j in seq_len(ncol(counts))) {
    manual <- manual_gwet_counts(collapse_counts_for_category_ac1(counts, j))
    expect_equal(by_cat$ac[[j]], manual$ac, tolerance = 1e-12)
  }
})

test_that("gwet_ac panel inference keeps asymptotic primary and jackknife optional", {
  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 3, 0),
    c(0, 1, 2),
    c(1, 2, 0)
  )
  colnames(counts) <- c("A", "B", "C")

  fit_default <- gwet_ac(counts, input = "counts", ci = TRUE, p_value = TRUE)
  fit_jk <- gwet_ac(counts, input = "counts", ci = TRUE, p_value = TRUE, se_method = "jackknife")

  expect_identical(attr(fit_default, "se.method"), "asymptotic")
  expect_identical(attr(fit_jk, "se.method"), "jackknife")
  expect_equal(fit_default$ac[[1L]], fit_jk$ac[[1L]], tolerance = 1e-12)
  expect_true(is.finite(fit_default$se[[1L]]) || is.na(fit_default$se[[1L]]))
  expect_true(is.finite(fit_jk$se[[1L]]) || is.na(fit_jk$se[[1L]]))
})

test_that("gwet_ac rejects by-category AC2 output", {
  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 3, 0)
  )
  colnames(counts) <- c("L", "M", "H")

  expect_error(
    gwet_ac(counts, input = "counts", weights = "linear", by_category = TRUE),
    "must be FALSE for weighted AC2 fits"
  )
})

test_that("gwet_ac thread count does not change estimates", {
  raters <- data.frame(
    a = c("A", "A", "B", "B", "C", "A", "B", "C", "A", "C"),
    b = c("A", "B", "B", "B", "C", "A", "B", "C", "A", "B"),
    c = c("A", "A", "B", "C", "C", "A", "B", "B", "A", "C"),
    d = c("A", "B", "A", "B", "C", "C", "B", "C", "A", "C"),
    stringsAsFactors = FALSE
  )

  fit1 <- gwet_ac(raters, n_threads = 1L)
  fit2 <- gwet_ac(raters, n_threads = 2L)
  expect_equal(as.matrix(fit1), as.matrix(fit2), tolerance = 1e-12)

  counts <- ratings_to_counts(raters, levels = c("A", "B", "C"))
  fit_panel_1 <- gwet_ac(counts, input = "counts", ci = TRUE, p_value = TRUE, n_threads = 1L)
  fit_panel_2 <- gwet_ac(counts, input = "counts", ci = TRUE, p_value = TRUE, n_threads = 2L)
  expect_equal(fit_panel_1$ac[[1L]], fit_panel_2$ac[[1L]], tolerance = 1e-12)
  expect_equal(fit_panel_1$se[[1L]], fit_panel_2$se[[1L]], tolerance = 1e-12)
})

test_that("gwet_ac print summary and plot methods work across result types", {
  skip_if_not_installed("ggplot2")

  x <- factor(c("A", "A", "B", "B", "A", "B"))
  y <- factor(c("A", "B", "B", "B", "A", "A"))
  fit_scalar <- gwet_ac(x, y, ci = TRUE, p_value = TRUE)

  expect_true(length(capture.output(print(fit_scalar))) > 0L)
  sm_scalar <- summary(fit_scalar)
  expect_s3_class(sm_scalar, "summary.gwet_ac")
  expect_s3_class(plot(fit_scalar, show_value = FALSE), "ggplot")

  raters <- data.frame(
    a = c("A", "A", "B", "B", "C", "A", "B", "C"),
    b = c("A", "B", "B", "B", "C", "A", "B", "C"),
    c = c("A", "A", "B", "C", "C", "A", "B", "B"),
    stringsAsFactors = FALSE
  )
  fit_matrix <- gwet_ac(raters)
  expect_true(length(capture.output(print(fit_matrix))) > 0L)
  expect_s3_class(summary(fit_matrix), "summary.corr_result")
  expect_s3_class(plot(fit_matrix, show_value = FALSE), "ggplot")
  expect_error(
    plot(fit_matrix, type = "estimate"),
    "not used for matrix-style correlation heatmaps"
  )

  counts <- ratings_to_counts(raters, levels = c("A", "B", "C"))
  fit_panel <- gwet_ac(counts, input = "counts", by_category = TRUE, ci = TRUE)
  expect_true(length(capture.output(print(fit_panel, show_by_category = TRUE))) > 0L)
  sm_panel <- summary(fit_panel)
  expect_s3_class(sm_panel, "summary.gwet_ac")
  expect_s3_class(plot(fit_panel, type = "estimate"), "ggplot")
  expect_s3_class(plot(fit_panel, type = "by_category"), "ggplot")
})

test_that("gwet_ac matches fixed benchmark values for estimates and CIs", {
  benchmark_tol <- 1e-6

  pair1_x <- c(rep("A", 12), rep("A", 3), rep("A", 1), rep("B", 4), rep("B", 10), rep("B", 2), rep("C", 2), rep("C", 5), rep("C", 11))
  pair1_y <- c(rep("A", 12), rep("B", 3), rep("C", 1), rep("A", 4), rep("B", 10), rep("C", 2), rep("A", 2), rep("B", 5), rep("C", 11))
  fit_pair1 <- gwet_ac(pair1_x, pair1_y, ci = TRUE)
  expect_equal(as.numeric(fit_pair1), 0.49010198, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair1, "ci")$lwr.ci, 0.2882755, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair1, "ci")$upr.ci, 0.6919285, tolerance = benchmark_tol)

  pair2_x <- c(rep("A", 90), rep("B", 10))
  pair2_y <- c(rep("A", 85), rep("B", 5), rep("A", 5), rep("B", 5))
  fit_pair2 <- gwet_ac(pair2_x, pair2_y, ci = TRUE)
  expect_equal(as.numeric(fit_pair2), 0.87804878, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair2, "ci")$lwr.ci, 0.7984957, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair2, "ci")$upr.ci, 0.9576018, tolerance = benchmark_tol)

  counts_u1 <- rbind(
    c(5, 0, 0),
    c(2, 1, 0),
    c(0, 3, 1),
    c(0, 1, 2),
    c(1, 1, 0),
    c(0, 0, 4)
  )
  colnames(counts_u1) <- c("A", "B", "C")
  fit_u1 <- gwet_ac(counts_u1, input = "counts", ci = TRUE)
  expect_equal(fit_u1$ac[[1L]], 0.29228101, tolerance = benchmark_tol)
  expect_equal(fit_u1$lwr.ci[[1L]], -0.3393303, tolerance = benchmark_tol)
  expect_equal(fit_u1$upr.ci[[1L]], 0.9238923, tolerance = benchmark_tol)

  counts_u2 <- rbind(
    c(4, 1, 0, 0),
    c(1, 2, 0, 0),
    c(0, 2, 2, 0),
    c(0, 1, 1, 1),
    c(0, 0, 2, 3),
    c(1, 0, 0, 1)
  )
  colnames(counts_u2) <- c("W", "X", "Y", "Z")
  fit_u2 <- gwet_ac(counts_u2, input = "counts", ci = TRUE)
  expect_equal(fit_u2$ac[[1L]], 0.03861956, tolerance = benchmark_tol)
  expect_equal(fit_u2$lwr.ci[[1L]], -0.2970932, tolerance = benchmark_tol)
  expect_equal(fit_u2$upr.ci[[1L]], 0.3743323, tolerance = benchmark_tol)
})

test_that("gwet_ac AC2 matches fixed benchmark values for estimates and CIs", {
  benchmark_tol <- 1e-6

  levels3 <- c("L", "M", "H")
  W <- matrix(
    c(
      1.0, 0.5, 0.0,
      0.5, 1.0, 0.5,
      0.0, 0.5, 1.0
    ),
    nrow = 3,
    byrow = TRUE
  )

  pair_x <- factor(c("L", "L", "M", "M", "H", "H"), levels = levels3)
  pair_y <- factor(c("L", "M", "M", "H", "H", "M"), levels = levels3)
  fit_pair <- gwet_ac(pair_x, pair_y, weights = W, ci = TRUE)
  expect_equal(as.numeric(fit_pair), 0.4517767, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair, "ci")$lwr.ci, -0.1044028, tolerance = benchmark_tol)
  expect_equal(attr(fit_pair, "ci")$upr.ci, 1.0000000, tolerance = benchmark_tol)

  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 2, 1),
    c(0, 0, 3),
    c(0, 2, 1),
    c(0, 1, 2)
  )
  colnames(counts) <- levels3
  fit_panel <- gwet_ac(counts, input = "counts", weights = W, ci = TRUE)
  expect_equal(fit_panel$ac[[1L]], 0.5057208, tolerance = benchmark_tol)
  expect_equal(fit_panel$lwr.ci[[1L]], 0.1044950, tolerance = benchmark_tol)
  expect_equal(fit_panel$upr.ci[[1L]], 0.9069466, tolerance = benchmark_tol)
})

#' @title Bland-Altman statistics with confidence intervals
#'
#' @description
#' Computes Bland-Altman mean difference and limits of agreement (LoA)
#' between two numeric measurement vectors, including t-based confidence
#' intervals for the mean difference and for each LoA using 'C++' backend.
#' If `group2` is omitted and `group1` is a numeric matrix or data frame with
#' at least two numeric columns, `ba()` computes all pairwise contrasts across
#' methods and returns a pairwise Bland-Altman matrix object.
#'
#' Note: Lin's concordance correlation coefficient (CCC) is a complementary,
#' single-number summary of agreement (precision + accuracy). It is useful for
#' quick screening or reporting an overall CI, but may miss systematic or
#' magnitude-dependent bias; consider reporting CCC alongside Bland-Altman.
#'
#' @details
#' Given paired measurements \eqn{(x_i, y_i)}, Bland-Altman analysis uses
#' \eqn{d_i = x_i - y_i} (or \eqn{y_i - x_i} if \code{mode = 2}) and
#' \eqn{m_i = (x_i + y_i)/2}. The mean difference \eqn{\bar d} estimates bias.
#' The limits of agreement (LoA) are \eqn{\bar d \pm z \cdot s_d}, where
#' \eqn{s_d} is the sample standard deviation of \eqn{d_i} and \eqn{z}
#' (argument \code{loa_multiplier}) is typically 1.96 for nominal 95% LoA.
#'
#' When `group2` is omitted and `group1` is a wide numeric matrix or data
#' frame, the same two-method calculation is applied to every unordered column
#' pair. The returned pairwise matrices follow the requested `mode`: with
#' `mode = 1`, upper-triangle entries represent row minus column; with
#' `mode = 2`, upper-triangle entries represent column minus row.
#'
#' Confidence intervals use Student's \eqn{t} distribution with \eqn{n-1}
#' degrees of freedom, with
#' \itemize{
#'   \item Mean-difference CI given by \eqn{\bar d \pm t_{n-1,\,1-\alpha/2}\,
#'   s_d/\sqrt{n}}; and
#'   \item LoA CI given by \eqn{(\bar d \pm z\, s_d) \;\pm\;
#'   t_{n-1,\,1-\alpha/2}\, s_d\,\sqrt{3/n}}.
#' }
#'
#' Assumptions include approximately normal differences and roughly constant
#' variability across the measurement range; if differences increase with
#' magnitude, consider a transformation before analysis. Missing values are
#' removed pairwise (rows with an \code{NA} in either input are dropped before
#' calling the C++ backend).
#'
#' Probability of agreement, available through \code{\link{prob_agree}}, is a
#' tolerance-based companion to Bland-Altman analysis. Bland-Altman reports bias
#' and limits of agreement for paired differences. `prob_agree()` instead uses
#' the sampling distribution of estimated differences to quantify the
#' probability that two estimated quantities or curves agree within a
#' user-specified practical tolerance.
#'
#' @param group1 Numeric vector of paired measurements, or a numeric matrix/data
#'   frame with at least two numeric columns when `group2` is omitted.
#' @param group2 Optional numeric vector of paired measurements. If omitted,
#'   `group1` must be a numeric matrix or data frame and all unordered numeric
#'   column pairs are analysed.
#' @param loa_multiplier Positive scalar; the multiple of the standard deviation used to
#'   define the LoA (default 1.96 for nominal 95\% agreement). The confidence
#'   intervals always use \eqn{t_{n-1,\,1-\alpha/2}} regardless of this choice.
#' @param mode Integer; 1 uses \code{group1 - group2}, 2 uses \code{group2 - group1}.
#' @param conf_level Confidence level for CIs (default 0.95).
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param verbose Logical; if TRUE, prints how many OpenMP threads are used.
#'
#' @return
#' If `group1` and `group2` are both supplied, an object of class
#' \code{"ba"} (list) with elements:
#' \itemize{
#'   \item \code{means}, \code{diffs}: numeric vectors
#'   \item \code{groups}: data.frame used after NA removal
#'   \item \code{n_obs}: integer, number of complete pairs used.
#'   \item \code{based.on}: compatibility alias for \code{n_obs}.
#'   \item \code{lower.limit}, \code{mean.diffs}, \code{upper.limit}
#'   \item \code{lines}: named numeric vector (lower, mean, upper)
#'   \item \code{CI.lines}: named numeric vector for CIs of those lines
#'   \item \code{loa_multiplier}, \code{critical.diff}
#' }
#'
#' If `group2` is omitted and `group1` is a wide numeric matrix or data frame,
#' an object of class \code{"ba_matrix"} with pairwise components:
#' \itemize{
#'   \item \code{bias}: pairwise matrix of Bland-Altman mean differences.
#'   \item \code{sd_loa}: pairwise matrix of SDs of differences.
#'   \item \code{loa_lower}, \code{loa_upper}: pairwise matrices of LoA
#'   endpoints.
#'   \item \code{width}: pairwise matrix of LoA widths.
#'   \item \code{n}: integer pairwise matrix of complete-case counts.
#'   \item \code{mean_ci_low}, \code{mean_ci_high}: pairwise matrices of CI
#'   bounds for the mean difference.
#'   \item \code{loa_lower_ci_low}, \code{loa_lower_ci_high}: pairwise matrices
#'   of CI bounds for the lower LoA.
#'   \item \code{loa_upper_ci_low}, \code{loa_upper_ci_high}: pairwise matrices
#'   of CI bounds for the upper LoA.
#'   \item \code{methods}: character vector of analysed method names.
#'   \item \code{loa_multiplier}, \code{mode}: calculation settings reused for
#'   every pair.
#' }
#'
#' @seealso \code{\link{print.ba}}, \code{\link{plot.ba}},
#'  \code{\link{ccc}}, \code{\link{prob_agree}}, \code{\link{ccc_rm_ustat}},
#'  \code{\link{ccc_rm_reml}}
#'
#' @examples
#' set.seed(1)
#' x <- rnorm(100, 100, 10)
#' y <- x + rnorm(100, 0, 8)
#' fit_ba <- ba(x, y)
#' print(fit_ba)
#' estimate(fit_ba)
#' tidy(fit_ba)
#' confint(fit_ba)
#' plot(fit_ba)
#'
#' # Pairwise Bland-Altman across 3 methods
#' set.seed(7)
#' wide3 <- data.frame(
#'   ref = rnorm(80, 100, 8),
#'   m2  = rnorm(80, 101, 8),
#'   m3  = rnorm(80,  99, 9)
#' )
#' fit_ba3 <- ba(wide3)
#' print(fit_ba3)
#' summary(fit_ba3)
#' tidy(fit_ba3)
#' plot(fit_ba3)
#'
#' @references
#' Bland JM, Altman DG (1986). Statistical methods for assessing agreement
#' between two methods of clinical measurement. *The Lancet*, 307-310.
#' @references
#' Bland JM, Altman DG (1999). Measuring agreement in method comparison studies.
#' *Statistical Methods in Medical Research*, 8(2), 135-160.
#'
#' @author Thiago de Paula Oliveira
#'
#' @export
ba <- function(group1,
               group2,
               loa_multiplier = 1.96,
               mode = 1L,
               conf_level = 0.95,
               n_threads = getOption("matrixCorr.threads", 1L),
               verbose = FALSE) {
  if (missing(group2)) {
    return(.mc_ba_fit_matrix(
      data = group1,
      loa_multiplier = loa_multiplier,
      mode = mode,
      conf_level = conf_level,
      n_threads = n_threads,
      verbose = verbose
    ))
  }

  .mc_ba_fit_two(
    group1 = group1,
    group2 = group2,
    loa_multiplier = loa_multiplier,
    mode = mode,
    conf_level = conf_level,
    n_threads = n_threads,
    verbose = verbose
  )
}

.mc_ba_fit_two <- function(group1,
                           group2,
                           loa_multiplier,
                           mode,
                           conf_level,
                           n_threads,
                           verbose) {
  # -- validate ---------------------------------------------------------------
  if (!is.numeric(group1) || !is.numeric(group2)) {
    abort_bad_arg("group1",
      message = "and {.arg group2} must be numeric vectors."
    )
  }
  check_same_length(group1, group2, arg_x = "group1", arg_y = "group2")
  if (!is.numeric(loa_multiplier) || length(loa_multiplier) != 1L || !is.finite(loa_multiplier) || loa_multiplier <= 0) {
    abort_bad_arg("loa_multiplier",
      message = "`{arg}` must be a positive scalar."
    )
  }
  if (!rlang::is_scalar_integerish(mode) || !(as.integer(mode) %in% c(1L, 2L))) {
    abort_bad_arg("mode",
      message = "must be 1 or 2."
    )
  }
  mode <- as.integer(mode)
  if (!is.numeric(conf_level) || length(conf_level) != 1L ||
      !is.finite(conf_level) || conf_level <= 0 || conf_level >= 1) {
    abort_bad_arg("conf_level",
      message = "`conf_level` must be in (0, 1)."
    )
  }
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")

  called.with <- length(group1)
  complete_pairs <- !is.na(group1) & !is.na(group2)
  n_pairs <- sum(complete_pairs)
  if (n_pairs < 2L) {
    abort_bad_arg("group1",
      message = "must provide at least two complete pairs with {.arg group2} after removing missing values (found {n_pairs}).",
      n_pairs = n_pairs
    )
  }
  inform_if_verbose(
    "Using {n_threads} OpenMP thread{?s}.",
    .verbose = verbose
  )

  # -- compute in C++ ---------------------------------------------------------
  prev_threads <- get_omp_threads()
  on.exit(set_omp_threads(as.integer(prev_threads)), add = TRUE)
  ba_out <- bland_altman_cpp(
    group1 = group1,
    group2 = group2,
    loa_multiplier = loa_multiplier,
    mode = mode,
    conf_level = conf_level,
    n_threads = n_threads,
    include_legacy_payload = FALSE
  )
  ba_out <- structure(
    list(
      means = as.numeric(ba_out$means),
      diffs = as.numeric(ba_out$diffs),
      n_obs = as.integer(ba_out$based.on),
      lower.limit = as.numeric(ba_out$lower.limit),
      mean.diffs = as.numeric(ba_out$mean.diffs),
      upper.limit = as.numeric(ba_out$upper.limit),
      loa_multiplier = as.numeric(ba_out$loa_multiplier),
      critical.diff = as.numeric(ba_out$critical.diff),
      ci = .mc_ba_normalize_ci_lines(ba_out$CI.lines),
      mode = mode
    ),
    class = "ba"
  )
  attr(ba_out, "method") <- "Bland-Altman"
  attr(ba_out, "description") <- "Mean difference and limits of agreement with CIs"
  attr(ba_out, "package") <- "matrixCorr"
  attr(ba_out, "conf.level") <- conf_level
  attr(ba_out, "called.with") <- length(group1)
  ba_out
}

.mc_ba_fit_matrix <- function(data,
                              loa_multiplier,
                              mode,
                              conf_level,
                              n_threads,
                              verbose) {
  if (is.data.frame(data)) {
    is_num <- vapply(data, is.numeric, logical(1))
    if (!any(is_num)) {
      abort_bad_arg("group1",
        message = "must contain at least two numeric columns when {.arg group2} is omitted."
      )
    }
    data <- data[is_num]
  }

  if (!is.matrix(data) && !is.data.frame(data)) {
    abort_bad_arg("group1",
      message = "must be a numeric matrix or data frame when {.arg group2} is omitted."
    )
  }
  if (is.matrix(data) && !is.numeric(data)) {
    abort_bad_arg("group1",
      message = "must be a numeric matrix when {.arg group2} is omitted."
    )
  }

  data_wide <- as.data.frame(data, check.names = FALSE, stringsAsFactors = FALSE)
  if (ncol(data_wide) < 2L) {
    abort_bad_arg("group1",
      message = "must contain at least two numeric columns when {.arg group2} is omitted."
    )
  }
  methods <- names(data_wide)
  if (is.null(methods) || anyNA(methods) || any(methods == "")) {
    methods <- paste0("method", seq_len(ncol(data_wide)))
    names(data_wide) <- methods
  }

  if (!is.numeric(loa_multiplier) || length(loa_multiplier) != 1L || !is.finite(loa_multiplier) || loa_multiplier <= 0) {
    abort_bad_arg("loa_multiplier",
      message = "`{arg}` must be a positive scalar."
    )
  }
  if (!rlang::is_scalar_integerish(mode) || !(as.integer(mode) %in% c(1L, 2L))) {
    abort_bad_arg("mode",
      message = "must be 1 or 2."
    )
  }
  mode <- as.integer(mode)
  if (!is.numeric(conf_level) || length(conf_level) != 1L ||
      !is.finite(conf_level) || conf_level <= 0 || conf_level >= 1) {
    abort_bad_arg("conf_level",
      message = "`conf_level` must be in (0, 1)."
    )
  }
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")

  if (isTRUE(verbose)) {
    inform_if_verbose(
      "Pairwise Bland-Altman: {ncol(data_wide)} methods -> {choose(ncol(data_wide), 2L)} pairs.",
      .verbose = TRUE
    )
  }

  mm <- length(methods)
  bias <- .make_named_matrix_ba(methods)
  sd_loa <- .make_named_matrix_ba(methods)
  loa_lower <- .make_named_matrix_ba(methods)
  loa_upper <- .make_named_matrix_ba(methods)
  width <- .make_named_matrix_ba(methods)
  n_mat <- .make_named_matrix_ba(methods, NA_integer_, "integer")
  mean_ci_low <- .make_named_matrix_ba(methods)
  mean_ci_high <- .make_named_matrix_ba(methods)
  lo_ci_low <- .make_named_matrix_ba(methods)
  lo_ci_high <- .make_named_matrix_ba(methods)
  hi_ci_low <- .make_named_matrix_ba(methods)
  hi_ci_high <- .make_named_matrix_ba(methods)

  for (i in 1:(mm - 1L)) for (j in (i + 1L):mm) {
    keep <- !is.na(data_wide[[i]]) & !is.na(data_wide[[j]])
    n_pair <- sum(keep)
    n_mat[i, j] <- n_pair
    n_mat[j, i] <- n_pair
    if (n_pair < 2L) {
      next
    }

    fit <- .mc_ba_fit_two(
      group1 = data_wide[[i]],
      group2 = data_wide[[j]],
      loa_multiplier = loa_multiplier,
      mode = mode,
      conf_level = conf_level,
      n_threads = n_threads,
      verbose = FALSE
    )

    md <- as.numeric(fit$mean.diffs)
    sdv <- as.numeric(fit$critical.diff) / as.numeric(fit$loa_multiplier)
    lo <- as.numeric(fit$lower.limit)
    hi <- as.numeric(fit$upper.limit)

    bias[i, j] <- md
    bias[j, i] <- -md
    sd_loa[i, j] <- sdv
    sd_loa[j, i] <- sdv
    loa_lower[i, j] <- lo
    loa_lower[j, i] <- -hi
    loa_upper[i, j] <- hi
    loa_upper[j, i] <- -lo
    width[i, j] <- hi - lo
    width[j, i] <- hi - lo

    mean_ci_low[i, j] <- as.numeric(fit$CI.lines[["mean.diff.ci.lower"]])
    mean_ci_high[i, j] <- as.numeric(fit$CI.lines[["mean.diff.ci.upper"]])
    lo_ci_low[i, j] <- as.numeric(fit$CI.lines[["lower.limit.ci.lower"]])
    lo_ci_high[i, j] <- as.numeric(fit$CI.lines[["lower.limit.ci.upper"]])
    hi_ci_low[i, j] <- as.numeric(fit$CI.lines[["upper.limit.ci.lower"]])
    hi_ci_high[i, j] <- as.numeric(fit$CI.lines[["upper.limit.ci.upper"]])

    mean_ci_low[j, i] <- -mean_ci_high[i, j]
    mean_ci_high[j, i] <- -mean_ci_low[i, j]
    lo_ci_low[j, i] <- -hi_ci_high[i, j]
    lo_ci_high[j, i] <- -hi_ci_low[i, j]
    hi_ci_low[j, i] <- -lo_ci_high[i, j]
    hi_ci_high[j, i] <- -lo_ci_low[i, j]
  }

  out <- structure(
    list(
      bias = bias,
      sd_loa = sd_loa,
      loa_lower = loa_lower,
      loa_upper = loa_upper,
      width = width,
      n = n_mat,
      mean_ci_low = mean_ci_low,
      mean_ci_high = mean_ci_high,
      loa_lower_ci_low = lo_ci_low,
      loa_lower_ci_high = lo_ci_high,
      loa_upper_ci_low = hi_ci_low,
      loa_upper_ci_high = hi_ci_high,
      methods = methods,
      loa_multiplier = loa_multiplier,
      mode = mode,
      data_wide = data_wide
    ),
    class = c("ba_matrix", "list")
  )
  attr(out, "method") <- "Bland-Altman"
  attr(out, "description") <- "Pairwise mean differences and limits of agreement with CIs"
  attr(out, "package") <- "matrixCorr"
  attr(out, "conf.level") <- conf_level
  out
}

.mc_ba_normalize_ci_lines <- function(x) {
  out <- as.numeric(x)
  nms <- names(x)
  if (is.null(nms) && length(out) == 6L) {
    nms <- c(
      "lower.limit.ci.lower",
      "lower.limit.ci.upper",
      "mean.diff.ci.lower",
      "mean.diff.ci.upper",
      "upper.limit.ci.lower",
      "upper.limit.ci.upper"
    )
  }
  stats::setNames(out, nms)
}

.mc_ba_groups <- function(x) {
  means <- as.numeric(unclass(x)$means)
  diffs <- as.numeric(unclass(x)$diffs)
  mode <- as.integer(unclass(x)$mode %||% 1L)
  if (identical(mode, 2L)) {
    group1 <- means - diffs / 2
    group2 <- means + diffs / 2
  } else {
    group1 <- means + diffs / 2
    group2 <- means - diffs / 2
  }
  data.frame(group1 = group1, group2 = group2, check.names = FALSE)
}

.mc_ba_lines <- function(x) {
  c(
    lower.limit = as.numeric(unclass(x)$lower.limit),
    mean.diffs = as.numeric(unclass(x)$mean.diffs),
    upper.limit = as.numeric(unclass(x)$upper.limit)
  )
}

.mc_ba_ci_lines <- function(x) {
  ci <- .mc_ba_normalize_ci_lines(unclass(x)$ci %||% numeric())
  if (length(ci)) {
    return(ci)
  }
  numeric()
}

#' @export
names.ba <- function(x) {
  unique(c(
    names(unclass(x)),
    "based.on",
    "groups",
    "lines",
    "CI.lines"
  ))
}

#' @export
`$.ba` <- function(x, name) {
  core <- unclass(x)
  switch(name,
    based.on = core$n_obs,
    groups = .mc_ba_groups(x),
    lines = .mc_ba_lines(x),
    CI.lines = .mc_ba_ci_lines(x),
    core[[name]]
  )
}

#' @export
`[[.ba` <- function(x, i, ...) {
  if (is.numeric(i)) {
    nms <- names(x)
    if (length(i) != 1L || is.na(i) || i < 1L || i > length(nms)) {
      abort_bad_arg("i", message = "must be a valid component index.")
    }
    i <- nms[[i]]
  }
  `$.ba`(x, i)
}

#' @rdname ba
#' @method print ba
#' @param x A \code{"ba"} object.
#' @param digits Number of digits for estimates (default 3).
#' @param ci_digits Number of digits for CI bounds (default 3).
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Unused.
#' @export
print.ba <- function(x, digits = 3, ci_digits = 3,
                     n = NULL, topn = NULL,
                     max_vars = NULL, width = NULL,
                     show_ci = NULL, ...) {
  check_inherits(x, "ba")
  show_ci <- .mc_resolve_show_ci(show_ci, context = "print")

  n   <- as.integer(x$based.on)
  loa_multiplier <- as.numeric(x$loa_multiplier)
  cl  <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  if (!is.finite(cl)) cl <- NA_real_

  # core numbers (as scalars)
  bias   <- as.numeric(x$mean.diffs)
  loa_lo <- as.numeric(x$lower.limit)
  loa_hi <- as.numeric(x$upper.limit)
  sd_d   <- as.numeric(x$critical.diff) / loa_multiplier
  loa_width <- loa_hi - loa_lo

  # CIs (robust extraction by name)
  cil <- function(nm) as.numeric(x$CI.lines[[nm]])
  bias_l <- cil("mean.diff.ci.lower"); bias_u <- cil("mean.diff.ci.upper")
  lo_l   <- cil("lower.limit.ci.lower"); lo_u <- cil("lower.limit.ci.upper")
  hi_l   <- cil("upper.limit.ci.lower"); hi_u <- cil("upper.limit.ci.upper")

  .mc_print_named_digest(
    c(
      based_on = n,
      loa_rule = sprintf("mean +/- %.3g * SD", loa_multiplier),
      if (identical(show_ci, "yes") && is.finite(cl)) c(ci = sprintf("%g%%", 100 * cl)),
      sd_diff = formatC(sd_d, format = "f", digits = digits),
      width = formatC(loa_width, format = "f", digits = digits)
    ),
    header = "Bland-Altman preview:"
  )
  cat("\n")

  # nicely aligned three-row table
  df <- data.frame(
    quantity = c("Mean difference", "Lower LoA", "Upper LoA"),
    estimate = c(bias, loa_lo, loa_hi),
    lwr      = c(bias_l, lo_l, hi_l),
    upr      = c(bias_u, lo_u, hi_u),
    check.names = FALSE,
    row.names   = NULL
  )
  df$estimate <- formatC(df$estimate, format = "f", digits = digits)
  if (identical(show_ci, "yes")) {
    df$lwr <- formatC(df$lwr, format = "f", digits = ci_digits)
    df$upr <- formatC(df$upr, format = "f", digits = ci_digits)
  } else {
    df$lwr <- NULL
    df$upr <- NULL
  }

  .mc_print_preview_table(
    df,
    n = .mc_coalesce(n, .mc_display_option("print_max_rows", 20L)),
    topn = .mc_coalesce(topn, .mc_display_option("print_topn", 5L)),
    max_vars = .mc_coalesce(max_vars, .mc_display_option("print_max_vars", NULL)),
    width = .mc_coalesce(width, getOption("width", 80L)),
    context = "print",
    full_hint = TRUE,
    summary_hint = TRUE,
    ...
  )
  invisible(x)
}

#' @rdname ba
#' @method summary ba
#' @param object A \code{"ba"} object.
#' @export
summary.ba <- function(object,
                       digits = 3,
                       ci_digits = 3,
                       ...) {
  check_inherits(object, "ba")

  cil <- function(nm) as.numeric(object$CI.lines[[nm]])

  out <- data.frame(
    n_obs          = as.integer(object$based.on),
    bias           = round(as.numeric(object$mean.diffs), digits),
    sd_loa         = round(as.numeric(object$critical.diff) / as.numeric(object$loa_multiplier), digits),
    loa_low        = round(as.numeric(object$lower.limit), digits),
    loa_up         = round(as.numeric(object$upper.limit), digits),
    width          = round(as.numeric(object$upper.limit - object$lower.limit), digits),
    loa_multiplier = round(as.numeric(object$loa_multiplier), digits),
    bias_lwr       = round(cil("mean.diff.ci.lower"), ci_digits),
    bias_upr       = round(cil("mean.diff.ci.upper"), ci_digits),
    lo_lwr         = round(cil("lower.limit.ci.lower"), ci_digits),
    lo_upr         = round(cil("lower.limit.ci.upper"), ci_digits),
    up_lwr         = round(cil("upper.limit.ci.lower"), ci_digits),
    up_upr         = round(cil("upper.limit.ci.upper"), ci_digits),
    check.names = FALSE
  )

  out <- .mc_finalize_summary_df(out, class_name = "summary.ba")
  attr(out, "conf.level") <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  out
}

#' @rdname ba
#' @method print summary.ba
#' @param ... Passed to \code{\link[base]{print.data.frame}}.
#' @export
print.summary.ba <- function(x, digits = NULL, n = NULL,
                             topn = NULL, max_vars = NULL,
                             width = NULL, show_ci = NULL, ...) {
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  header <- .mc_header_with_ci("Bland-Altman (two methods)", cl, show_ci)
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Agreement estimates",
        cols = c("n_obs", "bias", "sd_loa", "loa_low", "loa_up", "width", "loa_multiplier")
      ),
      list(
        title = "Confidence intervals",
        cols = c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr")
      )
    ),
    header = header,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @rdname ba
#' @method plot ba
#' @param x A \code{"ba"} object.
#' @param title Plot title.
#' @param subtitle Optional subtitle. If NULL, shows n and LoA summary.
#' @param point_alpha Point transparency.
#' @param point_size Point size.
#' @param line_size Line width for mean/LoA.
#' @param shade_ci Logical; if TRUE, draw shaded CI bands instead of 6 dashed
#' lines.
#' @param shade_alpha Transparency of CI bands.
#' @param smoother One of "none", "loess", "lm" to visualize proportional bias.
#' @param symmetrize_y Logical; if TRUE, y-axis centered at mean difference
#' with symmetric limits.
#' @param show_value Logical; included for a consistent plotting interface.
#'   Bland-Altman plots do not overlay numeric cell values, so this argument
#'   currently has no effect.
#' @param ... Passed to \code{ggplot2::theme()} (ggplot path) or \code{plot()}.
#' @importFrom graphics abline lines par rect
#' @export
plot.ba <- function(x,
                    title = "Bland-Altman Plot",
                    subtitle = NULL,
                    point_alpha = 0.7,
                    point_size  = 2.2,
                    line_size   = 0.8,
                    shade_ci    = TRUE,
                    shade_alpha = 0.08,
                    smoother    = c("none", "loess", "lm"),
                    symmetrize_y = TRUE,
                    show_value = TRUE,
                    ...) {
  check_inherits(x, "ba")
  check_bool(show_value, arg = "show_value")
  smoother <- match.arg(smoother)

  means <- as.numeric(x$means)
  diffs <- as.numeric(x$diffs)

  # scalars
  md    <- as.numeric(x$mean.diffs)
  loaL  <- as.numeric(x$lower.limit)
  loaU  <- as.numeric(x$upper.limit)
  loa_multiplier <- as.numeric(x$loa_multiplier)
  n     <- as.integer(x$based.on)
  sd_d  <- as.numeric(x$critical.diff) / loa_multiplier
  cl    <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  ci    <- function(nm) as.numeric(x$CI.lines[[nm]])

  if (is.null(subtitle)) {
    subtitle <- if (is.finite(cl)) {
      sprintf("n = %d; mean diff = %.2f; LoA = [%.2f, %.2f]; %g%% CI shown",
              n, md, loaL, loaU, 100*cl)
    } else {
      sprintf("n = %d; mean diff = %.2f; LoA = [%.2f, %.2f]",
              n, md, loaL, loaU)
    }
  }

  # y limits (symmetric around md if requested)
  y_rng <- range(c(diffs, loaL, loaU), na.rm = TRUE)
  if (isTRUE(symmetrize_y)) {
    half <- max(abs(y_rng - md))
    y_rng <- c(md - half, md + half)
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    ## ---------- Base fallback ----------
    plot(means, diffs,
         xlab = "Mean of methods", ylab = "Difference between methods",
         main = title, sub = subtitle,
         pch = 16, cex = point_size / 2.2,
         col = grDevices::gray(0.2, alpha = point_alpha),
         ylim = y_rng, ...)
    # shaded CI bands (rectangles)
    if (shade_ci) {
      usr <- par("usr")
      rect(xleft = usr[1], xright = usr[2],
           ybottom = ci("mean.diff.ci.lower"), ytop = ci("mean.diff.ci.upper"),
           border = NA, col = grDevices::gray(0.2, alpha = shade_alpha))
      rect(xleft = usr[1], xright = usr[2],
           ybottom = ci("lower.limit.ci.lower"), ytop = ci("lower.limit.ci.upper"),
           border = NA, col = grDevices::gray(0.2, alpha = shade_alpha))
      rect(xleft = usr[1], xright = usr[2],
           ybottom = ci("upper.limit.ci.lower"), ytop = ci("upper.limit.ci.upper"),
           border = NA, col = grDevices::gray(0.2, alpha = shade_alpha))
    } else {
      abline(h = c(ci("mean.diff.ci.lower"), ci("mean.diff.ci.upper")), lty = 2)
      abline(h = c(ci("lower.limit.ci.lower"), ci("lower.limit.ci.upper")), lty = 2)
      abline(h = c(ci("upper.limit.ci.lower"), ci("upper.limit.ci.upper")), lty = 2)
    }
    # reference & main lines
    abline(h = 0,   col = "grey70", lty = 3)
    abline(h = md,  lwd = line_size * 1.4)
    abline(h = loaL, lwd = line_size * 1.2)
    abline(h = loaU, lwd = line_size * 1.2)

    # optional smoother
    if (smoother != "none") {
      fit <- if (smoother == "lm") stats::lm(diffs ~ means) else stats::loess(diffs ~ means)
      xs <- seq(min(means), max(means), length.out = 200)
      ys <- stats::predict(fit, newdata = data.frame(means = xs))
      lines(xs, ys, lty = 1, col = "grey40")
    }
    return(invisible(NULL))
  }

  ## ---------- ggplot path ----------
  df <- data.frame(means = means, diffs = diffs)
  xr <- range(means, na.rm = TRUE)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = means, y = diffs))

  if (shade_ci) {
    p <- p +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci("mean.diff.ci.lower"), ymax = ci("mean.diff.ci.upper"),
                        alpha = shade_alpha) +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci("lower.limit.ci.lower"), ymax = ci("lower.limit.ci.upper"),
                        alpha = shade_alpha) +
      ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                        ymin = ci("upper.limit.ci.lower"), ymax = ci("upper.limit.ci.upper"),
                        alpha = shade_alpha)
  } else {
    p <- p +
      ggplot2::geom_hline(yintercept = c(ci("mean.diff.ci.lower"),
                                         ci("mean.diff.ci.upper"),
                                         ci("lower.limit.ci.lower"),
                                         ci("lower.limit.ci.upper"),
                                         ci("upper.limit.ci.lower"),
                                         ci("upper.limit.ci.upper")),
                          linetype = "dashed")
  }

  p <- p +
    ggplot2::geom_point(alpha = point_alpha, size = point_size) +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.4, linetype = "dotted", color = "grey40") +
    ggplot2::geom_hline(yintercept = md,   linewidth = line_size) +
    ggplot2::geom_hline(yintercept = loaL, linewidth = line_size) +
    ggplot2::geom_hline(yintercept = loaU, linewidth = line_size)

  if (smoother == "lm") {
    p <- p + ggplot2::geom_smooth(method = "lm", se = FALSE, linewidth = 0.7)
  } else if (smoother == "loess") {
    p <- p + ggplot2::geom_smooth(method = "loess", se = FALSE, linewidth = 0.7, span = 0.9)
  }

  p <- p +
    ggplot2::coord_cartesian(ylim = y_rng, expand = TRUE) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid = ggplot2::element_blank(), ...) +
    ggplot2::labs(title = title, subtitle = subtitle,
                  x = "Mean of methods", y = "Difference between methods")

  p
}

#' @rdname ba
#' @method print ba_matrix
#' @param x A \code{"ba_matrix"} object.
#' @param style Show the pairwise result as \code{"pairs"} or \code{"matrices"}.
#' @export
print.ba_matrix <- function(x,
                            digits = 3,
                            ci_digits = 3,
                            n = NULL,
                            topn = NULL,
                            max_vars = NULL,
                            width = NULL,
                            show_ci = NULL,
                            style = c("pairs", "matrices"),
                            ...) {
  check_inherits(x, "ba_matrix")
  style <- match.arg(style)
  show_ci <- .mc_resolve_show_ci(show_ci, context = "print")
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))

  if (style == "matrices") {
    .mc_print_named_digest(
      c(
        methods = length(x$methods),
        mode = if (identical(as.integer(x$mode), 1L)) "row - column" else "column - row",
        if (identical(show_ci, "yes") && is.finite(cl)) c(ci = sprintf("%g%%", 100 * cl)),
        components = "bias, sd_loa, loa_low, loa_up, width, n"
      ),
      header = "Bland-Altman matrix preview:"
    )
    cat("\nPrimary component: bias\n\n")
    .mc_print_preview_matrix(
      x$bias,
      digits = digits,
      n = .mc_coalesce(n, .mc_display_option("print_max_rows", 20L)),
      topn = .mc_coalesce(topn, .mc_display_option("print_topn", 5L)),
      max_vars = .mc_coalesce(max_vars, .mc_display_option("print_max_vars", NULL)),
      width = .mc_coalesce(width, getOption("width", 80L)),
      context = "print",
      full_hint = TRUE,
      summary_hint = TRUE,
      ...
    )
    return(invisible(x))
  }

  sm <- summary(x, digits = digits, ci_digits = ci_digits)
  cols <- c("method1", "method2", "bias", "sd_loa", "loa_low", "loa_up", "width", "n_obs")
  df <- as.data.frame(sm)[, cols[cols %in% names(sm)], drop = FALSE]
  header <- .mc_header_with_ci(
    sprintf("Bland-Altman (%s)", if (identical(as.integer(x$mode), 1L)) "row - column" else "column - row"),
    cl,
    show_ci
  )
  .mc_print_summary_table(
    df,
    header = header,
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @rdname ba
#' @method summary ba_matrix
#' @export
summary.ba_matrix <- function(object,
                              digits = 3,
                              ci_digits = 3,
                              ...) {
  check_inherits(object, "ba_matrix")
  methods <- object$methods
  m <- length(methods)

  rows <- vector("list", m * (m - 1L) / 2L)
  k <- 0L
  for (i in 1:(m - 1L)) for (j in (i + 1L):m) {
    k <- k + 1L
    rows[[k]] <- list(
      method1 = methods[i],
      method2 = methods[j],
      bias = round(object$bias[i, j], digits),
      sd_loa = round(object$sd_loa[i, j], digits),
      loa_low = round(object$loa_lower[i, j], digits),
      loa_up = round(object$loa_upper[i, j], digits),
      width = round(object$width[i, j], digits),
      n_obs = suppressWarnings(as.integer(object$n[i, j])),
      loa_multiplier = round(as.numeric(object$loa_multiplier), digits),
      bias_lwr = round(object$mean_ci_low[i, j], ci_digits),
      bias_upr = round(object$mean_ci_high[i, j], ci_digits),
      lo_lwr = round(object$loa_lower_ci_low[i, j], ci_digits),
      lo_upr = round(object$loa_lower_ci_high[i, j], ci_digits),
      up_lwr = round(object$loa_upper_ci_low[i, j], ci_digits),
      up_upr = round(object$loa_upper_ci_high[i, j], ci_digits)
    )
  }

  out <- .mc_finalize_summary_df(
    do.call(rbind.data.frame, rows),
    class_name = "summary.ba_matrix"
  )
  attr(out, "conf.level") <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  attr(out, "mode") <- as.integer(object$mode)
  out
}

#' @rdname ba
#' @method print summary.ba_matrix
#' @export
print.summary.ba_matrix <- function(x, digits = NULL, n = NULL,
                                    topn = NULL, max_vars = NULL,
                                    width = NULL, show_ci = NULL, ...) {
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  mode <- as.integer(attr(x, "mode") %||% 1L)
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  header <- .mc_header_with_ci(
    sprintf("Bland-Altman (%s)", if (identical(mode, 1L)) "pairwise row - column" else "pairwise column - row"),
    cl,
    show_ci
  )
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Agreement estimates",
        cols = c("method1", "method2", "n_obs", "bias", "sd_loa", "loa_low", "loa_up", "width", "loa_multiplier")
      ),
      list(
        title = "Confidence intervals",
        cols = c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr")
      )
    ),
    header = header,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @rdname ba
#' @method plot ba_matrix
#' @param pairs Optional character vector of pair labels to display.
#' @param against Optional single method name; if supplied, only contrasts
#'   involving that method are plotted.
#' @param facet_scales Either \code{"free_y"} (default) or \code{"fixed"}.
#' @export
plot.ba_matrix <- function(x,
                           pairs = NULL,
                           against = NULL,
                           facet_scales = c("free_y", "fixed"),
                           title = "Bland-Altman (pairwise)",
                           point_alpha = 0.6,
                           point_size = 1.8,
                           line_size = 0.7,
                           shade_ci = TRUE,
                           shade_alpha = 0.08,
                           smoother = c("none", "loess", "lm"),
                           show_value = TRUE,
                           ...) {
  check_inherits(x, "ba_matrix")
  check_bool(show_value, arg = "show_value")
  facet_scales <- match.arg(facet_scales)
  smoother <- match.arg(smoother)
  methods <- x$methods
  mode <- as.integer(x$mode %||% 1L)
  m <- length(methods)
  lab_pair <- function(i, j) {
    if (identical(mode, 2L)) {
      paste(methods[j], "\u2212", methods[i])
    } else {
      paste(methods[i], "\u2212", methods[j])
    }
  }

  idx_upper <- which(upper.tri(matrix(NA_real_, m, m)), arr.ind = TRUE)
  all_pairs <- data.frame(
    i = idx_upper[, 1],
    j = idx_upper[, 2],
    lab = lab_pair(idx_upper[, 1], idx_upper[, 2]),
    stringsAsFactors = FALSE
  )

  if (!is.null(against)) {
    if (!against %in% methods) {
      abort_bad_arg("against",
        message = "must be one of: {methods*?{, }{ and }}.",
        methods = methods
      )
    }
    hit <- match(against, methods)
    keep_against <- all_pairs[["i"]] == hit | all_pairs[["j"]] == hit
    all_pairs <- all_pairs[keep_against, , drop = FALSE]
  } else if (!is.null(pairs)) {
    keep_pairs <- all_pairs[["lab"]] %in% pairs
    all_pairs <- all_pairs[keep_pairs, , drop = FALSE]
    if (!nrow(all_pairs)) {
      abort_bad_arg("pairs",
        message = "None of the requested pairs matched available contrasts."
      )
    }
  }
  pairs_order <- all_pairs$lab

  bands <- data.frame(
    pair = lab_pair(idx_upper[, 1], idx_upper[, 2]),
    md = as.vector(x$bias[idx_upper]),
    loaL = as.vector(x$loa_lower[idx_upper]),
    loaU = as.vector(x$loa_upper[idx_upper]),
    md_l = as.vector(x$mean_ci_low[idx_upper]),
    md_u = as.vector(x$mean_ci_high[idx_upper]),
    lo_l = as.vector(x$loa_lower_ci_low[idx_upper]),
    lo_u = as.vector(x$loa_lower_ci_high[idx_upper]),
    hi_l = as.vector(x$loa_upper_ci_low[idx_upper]),
    hi_u = as.vector(x$loa_upper_ci_high[idx_upper]),
    stringsAsFactors = FALSE
  )
  bands <- bands[bands$pair %in% pairs_order, , drop = FALSE]
  bands$pair <- factor(bands$pair, levels = pairs_order)

  pts <- do.call(rbind, lapply(seq_len(nrow(all_pairs)), function(k) {
    i <- all_pairs$i[k]
    j <- all_pairs$j[k]
    a <- x$data_wide[[i]]
    b <- x$data_wide[[j]]
    keep <- !is.na(a) & !is.na(b)
    if (!any(keep)) {
      return(NULL)
    }
    means <- (a[keep] + b[keep]) / 2
    diffs <- if (identical(mode, 2L)) b[keep] - a[keep] else a[keep] - b[keep]
    data.frame(pair = all_pairs$lab[k], means = means, diffs = diffs, stringsAsFactors = FALSE)
  }))
  if (!is.null(pts) && nrow(pts)) {
    pts$pair <- factor(pts$pair, levels = pairs_order)
  } else {
    pts <- NULL
  }

  p <- ggplot2::ggplot() + ggplot2::facet_wrap(~ pair, scales = facet_scales)
  p <- p +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = md)) +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = loaL)) +
    ggplot2::geom_blank(data = bands, ggplot2::aes(x = 0, y = loaU))

  if (shade_ci) {
    p <- p +
      ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = md_l, ymax = md_u),
                         inherit.aes = FALSE, alpha = shade_alpha) +
      ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = lo_l, ymax = lo_u),
                         inherit.aes = FALSE, alpha = shade_alpha) +
      ggplot2::geom_rect(data = bands, ggplot2::aes(xmin = -Inf, xmax = Inf, ymin = hi_l, ymax = hi_u),
                         inherit.aes = FALSE, alpha = shade_alpha)
  } else {
    p <- p +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md_l), linetype = "dashed") +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md_u), linetype = "dashed") +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = lo_l), linetype = "dashed") +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = lo_u), linetype = "dashed") +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = hi_l), linetype = "dashed") +
      ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = hi_u), linetype = "dashed")
  }

  p <- p +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.35, linetype = "dotted", colour = "grey40") +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = md), linewidth = line_size) +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = loaL), linewidth = line_size) +
    ggplot2::geom_hline(data = bands, ggplot2::aes(yintercept = loaU), linewidth = line_size)

  if (!is.null(pts)) {
    p <- p + ggplot2::geom_point(data = pts, ggplot2::aes(x = means, y = diffs),
                                 alpha = point_alpha, size = point_size)
    if (smoother == "lm") {
      p <- p + ggplot2::geom_smooth(data = pts, ggplot2::aes(x = means, y = diffs),
                                    method = "lm", se = FALSE, linewidth = 0.7)
    } else if (smoother == "loess") {
      p <- p + ggplot2::geom_smooth(data = pts, ggplot2::aes(x = means, y = diffs),
                                    method = "loess", se = FALSE, linewidth = 0.7, span = 0.9)
    }
  }

  p +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid = ggplot2::element_blank(), ...) +
    ggplot2::labs(
      title = title,
      x = if (!is.null(pts)) "Mean of methods" else NULL,
      y = if (identical(mode, 2L)) "Difference (column - row)" else "Difference (row - column)"
    )
}

#' @title Pairwise Distance Correlation (dCor)
#'
#' @description
#' Computes pairwise distance correlations for the numeric columns of a matrix
#' or data frame using a high-performance 'C++' backend. Distance correlation
#' detects general dependence, including non-linear relationships. Optional
#' p-values are available via the bias-corrected distance-correlation t-test.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns are dropped. Columns must be numeric.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each association on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' pairwise p-values, test statistics, and degrees of freedom from the
#' distance-correlation t-test of independence.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return A symmetric numeric matrix where the \code{(i, j)} entry is the
#' unbiased distance correlation between the \code{i}-th and \code{j}-th
#' numeric columns. The object has class \code{dcor} with attributes
#' \code{method = "distance_correlation"}, \code{description}, and
#' \code{package = "matrixCorr"}. When \code{p_value = TRUE}, the object also
#' carries an \code{inference} attribute with matrices \code{estimate},
#' \code{statistic}, \code{parameter}, and \code{p_value}, plus
#' \code{attr(x, "diagnostics")$n_complete}. The main returned matrix remains
#' the usual non-negative unbiased distance-correlation estimate.
#'
#' @details
#' Let \eqn{x \in \mathbb{R}^n} and \eqn{D^{(x)}} be the pairwise distance matrix
#' with zero diagonal: \eqn{D^{(x)}_{ii} = 0}, \eqn{D^{(x)}_{ij} = |x_i - x_j|} for
#' \eqn{i \neq j}. Define row sums \eqn{r^{(x)}_i = \sum_{k \neq i} D^{(x)}_{ik}} and
#' grand sum \eqn{S^{(x)} = \sum_{i \neq k} D^{(x)}_{ik}}. The U-centred matrix is
#' \deqn{A^{(x)}_{ij} =
#'       \begin{cases}
#'         D^{(x)}_{ij} - \dfrac{r^{(x)}_i + r^{(x)}_j}{n - 2}
#'         + \dfrac{S^{(x)}}{(n - 1)(n - 2)}, & i \neq j,\\[6pt]
#'         0, & i = j~.
#'       \end{cases}}
#' For two variables \eqn{x,y}, the unbiased distance covariance and variances are
#' \deqn{\widehat{\mathrm{dCov}}^2_u(x,y) = \frac{2}{n(n-3)} \sum_{i<j} A^{(x)}_{ij} A^{(y)}_{ij}
#' \;=\; \frac{1}{n(n-3)} \sum_{i \neq j} A^{(x)}_{ij} A^{(y)}_{ij},}
#' with \eqn{\widehat{\mathrm{dVar}}^2_u(x)} defined analogously from \eqn{A^{(x)}}.
#' The unbiased distance correlation is
#' \deqn{\widehat{\mathrm{dCor}}_u(x,y) =
#'       \frac{\widehat{\mathrm{dCov}}_u(x,y)}
#'            {\sqrt{\widehat{\mathrm{dVar}}_u(x)\,\widehat{\mathrm{dVar}}_u(y)}} \in [0,1].}
#'
#' \strong{Computation.} All heavy lifting (distance matrices, U-centering,
#' and unbiased scaling) is implemented in C++ (\code{ustat_dcor_matrix_cpp}),
#' so the R wrapper only validates/coerces the input. OpenMP parallelises the
#' upper-triangular loops. The implementation includes a Huo-Szekely style
#' univariate \eqn{O(n \log n)} dispatch for pairwise terms. We also have an exact
#' unbiased \eqn{O(n^2)} fallback retained for robustness in small-sample or
#' non-finite-path cases; no external dependencies are used.
#'
#' \strong{Inference.} When \code{p_value = TRUE}, the package computes the
#' bias-corrected distance-correlation t-test of independence of Szekely and
#' Rizzo (2013). Let \eqn{\widehat{\mathrm{dCor}}^\ast(x,y)} denote the signed
#' bias-corrected distance correlation used internally by the test (that is,
#' the same ratio before the package's usual clipping to \eqn{[0,1]}). With
#' \deqn{M = \frac{n(n-3)}{2},}
#' the test statistic is
#' \deqn{
#' T = \sqrt{M - 1}\;
#' \frac{\widehat{\mathrm{dCor}}^\ast(x,y)}
#'      {\sqrt{1 - \{\widehat{\mathrm{dCor}}^\ast(x,y)\}^2}},
#' }
#' referenced to a Student \eqn{t}-distribution with \eqn{M - 1} degrees of
#' freedom. The reported p-value uses the upper-tail probability
#' \eqn{P(t_{M-1} \ge T)}. This inference payload is attached as metadata; the
#' main returned matrix is unchanged unless \code{p_value} is explicitly
#' requested. The t reference is an asymptotic approximation. For small
#' complete-case sample sizes, especially when \eqn{M - 1} is small, p-values
#' can be unstable and should be interpreted cautiously; a permutation-based
#' dependence test is preferable when exact small-sample calibration matters.
#'
#' @note Requires \eqn{n \ge 4}. Columns with (near) zero unbiased distance
#' variance yield \code{NA} in their row/column. Typical per-pair cost uses
#' the \eqn{O(n \log n)} fast path, with \eqn{O(n^2)} fallback when needed.
#'
#' @references
#' Szekely, G. J., Rizzo, M. L., & Bakirov, N. K. (2007).
#' Measuring and testing dependence by correlation of distances.
#' \emph{Annals of Statistics}, 35(6), 2769-2794.
#'
#' Szekely, G. J., & Rizzo, M. L. (2013).
#' The distance correlation t-test of independence.
#' \emph{Journal of Multivariate Analysis}, 117, 193-213.
#'
#' Rizzo, M. L., & Szekely, G. J. (2024). \pkg{energy}: E-statistics
#' (energy statistics). R package version 1.7-12.
#'
#' @examples
#' \donttest{
#' ## Independent variables -> dCor ~ 0
#' set.seed(1)
#' X <- cbind(a = rnorm(200), b = rnorm(200))
#' D <- dcor(X)
#' print(D, digits = 3)
#' summary(D)
#'
#' ## Non-linear dependence: Pearson ~ 0, but unbiased dCor > 0
#' set.seed(42)
#' n <- 200
#' x <- rnorm(n)
#' y <- x^2 + rnorm(n, sd = 0.2)
#' XY <- cbind(x = x, y = y)
#' D2 <- dcor(XY)
#' # Compare Pearson vs unbiased distance correlation
#' round(c(pearson = cor(XY)[1, 2], dcor = D2["x", "y"]), 3)
#' summary(D2)
#' plot(D2, title = "Unbiased distance correlation (non-linear example)")
#'
#' ## Small AR(1) multivariate normal example
#' set.seed(7)
#' p <- 5; n <- 150; rho <- 0.6
#' Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
#' X3 <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
#' colnames(X3) <- paste0("V", seq_len(p))
#' D3 <- dcor(X3)
#' print(D3[1:3, 1:3], digits = 2)
#'
#' ## Optional inference
#' D4 <- dcor(XY, p_value = TRUE)
#' summary(D4)
#' estimate(D4)
#' tidy(D4)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(D)
#' }
#' }
#'
#' @author Thiago de paula Oliveira
#'
#' @export
dcor <- function(data,
                 na_method = c("error", "pairwise", "complete"),
                 p_value = FALSE,
                 n_threads = getOption("matrixCorr.threads", 1L),
                 output = c("matrix", "sparse", "edge_list"),
                 threshold = 0,
                 diag = TRUE,
                 ...) {
  output_cfg <- .mc_prepare_corr_output(
    output = output,
    threshold = threshold,
    diag = diag
  )
  if (...length() == 0L && missing(na_method) && isFALSE(p_value)) {
    input <- .mc_prepare_corr_input(
      data,
      na_cfg = list(na_method = "error", check_na = TRUE),
      min_n = 4L
    )
    return(.mc_with_omp_threads(
      n_threads,
      {
        .mc_finalize_corr_result(
          mat = ustat_dcor_matrix_cpp(input$data),
          class_name = "dcor",
          method = "distance_correlation",
          description = "Pairwise distance correlation matrix (unbiased)",
          output_cfg = output_cfg,
          dimnames = input$dimnames,
          symmetric = TRUE
        )
      }
    ))
  }

  na_cfg <- .mc_resolve_corr_na(
    na_method = na_method,
    dots = list(...),
    na_method_missing = missing(na_method),
    allowed = c("error", "pairwise", "complete")
  )
  if (!isFALSE(p_value)) {
    check_bool(p_value, arg = "p_value")
  } else if (!is.logical(p_value) || length(p_value) != 1L || is.na(p_value)) {
    check_bool(p_value, arg = "p_value")
  }
  input <- .mc_prepare_corr_input(data, na_cfg = na_cfg, min_n = 4L)
  diagnostics <- NULL
  inference_attr <- NULL

  .mc_with_omp_threads(
    n_threads,
    {
      if (isTRUE(p_value) || identical(na_cfg$na_method, "pairwise")) {
        pairwise <- ustat_dcor_matrix_pairwise_cpp(
          input$data,
          return_inference = p_value
        )
        dcor_matrix <- pairwise$est
        diagnostics <- list(
          n_complete = .mc_set_matrix_dimnames(unclass(pairwise$n_complete), input$colnames)
        )

        if (isTRUE(p_value)) {
          inference_attr <- list(
            method = "dcor_t_test",
            estimate = .mc_set_matrix_dimnames(unclass(pairwise$estimate), input$colnames),
            statistic = .mc_set_matrix_dimnames(unclass(pairwise$statistic), input$colnames),
            parameter = .mc_set_matrix_dimnames(unclass(pairwise$parameter), input$colnames),
            p_value = .mc_set_matrix_dimnames(unclass(pairwise$p_value), input$colnames),
            alternative = "greater"
          )
        }
      } else {
        dcor_matrix <- ustat_dcor_matrix_cpp(input$data)
      }

      .mc_finalize_corr_result(
        mat = dcor_matrix,
        class_name = "dcor",
        method = "distance_correlation",
        description = "Pairwise distance correlation matrix (unbiased)",
        output_cfg = output_cfg,
        diagnostics = .mc_merge_diagnostics(diagnostics, input$diagnostics),
        dimnames = input$dimnames,
        symmetric = TRUE,
        extra_attrs = if (!is.null(inference_attr)) list(inference = inference_attr)
      )
    }
  )
}

.mc_dcor_pairwise_summary <- function(object,
                                      digits = 4,
                                      p_digits = 4) {
  check_inherits(object, "dcor")

  inf <- .mc_inference_attr(object)
  has_p <- is.list(inf) && is.matrix(inf$p_value)
  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.dcor",
    digits = digits,
    p_digits = p_digits,
    include_ci = FALSE,
    include_p = has_p,
    extra_columns = if (is.list(inf)) {
      list(
        statistic = list(matrix = inf$statistic, digits = digits),
        df = list(matrix = inf$parameter, digits = digits),
        p_value = list(matrix = inf$p_value, digits = p_digits)
      )
    },
    extra_attrs = list(
      inference_method = if (isTRUE(has_p)) inf$method %||% NA_character_ else NA_character_
    )
  )
}

#' @rdname dcor
#' @method print dcor
#' @title Print Method for \code{dcor} Objects
#'
#' @param x An object of class \code{dcor}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print}.
#'
#' @return Invisibly returns \code{x}.
#' @export
print.dcor <- function(x, digits = 4, n = NULL, topn = NULL,
                       max_vars = NULL, width = NULL,
                       show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Distance correlation (dCor) matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname dcor
#' @method plot dcor
#' @title Plot Method for \code{dcor} Objects
#'
#' @param x An object of class \code{dcor}.
#' @param title Plot title. Default is \code{"Distance correlation heatmap"}.
#' @param low_color Colour for zero correlation. Default is \code{"white"}.
#' @param high_color Colour for strong correlation. Default is \code{"steelblue1"}.
#' @param value_text_size Font size for displaying values. Default is \code{4}.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other
#' \code{ggplot2} layers.
#'
#' @return A \code{ggplot} object representing the heatmap.
#' @import ggplot2
#' @export
plot.dcor <-
  function(x, title = "Distance correlation heatmap",
           low_color = "white", high_color = "steelblue1",
           value_text_size = 4, show_value = TRUE, ...) {

    check_inherits(x, "dcor")
    check_bool(show_value, arg = "show_value")
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
    }

    mat <- as.matrix(x)
    df <- as.data.frame(as.table(mat))
    colnames(df) <- c("Var1", "Var2", "dCor")

    df$Var1 <- factor(df$Var1, levels = rev(unique(df$Var1)))

    p <- ggplot2::ggplot(df, ggplot2::aes(Var2, Var1, fill = dCor)) +
      ggplot2::geom_tile(color = "white") +
      ggplot2::scale_fill_gradient(
        low = low_color, high = high_color,
        limits = c(0, 1), name = "dCor"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        panel.grid = ggplot2::element_blank(),
        ...
      ) +
      ggplot2::coord_fixed() +
      ggplot2::labs(title = title, x = NULL, y = NULL)

    if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(label = sprintf("%.2f", dCor)),
        size = value_text_size,
        color = "black"
      )
    }

    p
  }

#' @rdname dcor
#' @method summary dcor
#' @param object An object of class \code{dcor}.
#' @export
summary.dcor <- function(object, n = NULL, topn = NULL,
                         max_vars = NULL, width = NULL,
                         show_ci = NULL, ...) {
  check_inherits(object, "dcor")
  inf <- .mc_inference_attr(object)
  if (!is.list(inf) || !is.matrix(inf$p_value)) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_dcor_pairwise_summary(object)
}

#' @rdname dcor
#' @method print summary.dcor
#' @param x An object of class \code{summary.dcor}.
#' @export
print.summary.dcor <- function(x, digits = NULL, n = NULL,
                               topn = NULL, max_vars = NULL,
                               width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Distance correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    extra_items = c(inference = attr(x, "inference_method", exact = TRUE)),
    ...
  )
  invisible(x)
}

test_that("returns correct structure and handles simple cases", {
  n <- 5
  mat <- cbind(
    x = 1:n,
    y = 2:(n+1),               # perfectly linear with x
    z = c(0.34, -0.75, 0.12, 1.09, -0.22)  # fixed values
  )

  res <- dcor(mat)

  expect_s3_class(res, "dcor")
  expect_equal(dim(res), c(3, 3))
  expect_true(all(diag(res) == 1))

  expect_gt(res["x","y"], 0.95)
  expect_lt(res["x","z"], 0.5)  # pick a margin that passes for this fixed z
})

test_that("detects non-linear dependence missed by Pearson", {
  set.seed(42)
  n <- 10000
  x <- rnorm(n)
  y <- x^2
  X <- cbind(x, y)
  colnames(X) <- c("x", "y")

  pearson <- pearson_corr(X)[1, 2]
  dcor <- dcor(X)[1, 2]

  # Pearson correlation near 0 due to symmetry
  expect_lt(abs(pearson), 0.02)

  # Distance correlation should be clearly positive
  expect_gt(dcor, 0.25)
})

test_that("handles constant variables and produces NAs", {
  mat <- cbind(rep(1, 10), rnorm(10))
  colnames(mat) <- c("const", "noise")

  res <- suppressWarnings(dcor(mat))
  expect_true(is.na(res["const", "noise"]))
})

test_that("matches expected dCor in AR(1) matrix", {
  set.seed(1)
  p <- 3; n <- 100; rho <- 0.7
  Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
  L <- chol(Sigma)
  X <- matrix(rnorm(n * p), n, p) %*% L
  colnames(X) <- paste0("V", 1:p)

  res <- dcor(X)

  # Expect symmetry and diagonal = 1
  expect_true(all.equal(res, t(res)))
  expect_true(all(diag(res) == 1))

  # dCor between adjacent variables should be stronger than non-adjacent
  expect_gt(res["V1", "V2"], res["V1", "V3"])
})

test_that("distance-correlation inference matches the reference t-test", {
  set.seed(2026)
  x <- rnorm(80)
  y <- x^2 + rnorm(80, sd = 0.25)
  z <- -0.4 * x + rnorm(80, sd = 0.8)
  X <- cbind(x = x, y = y, z = z)

  dc <- dcor(X, p_value = TRUE)
  inf <- attr(dc, "inference", exact = TRUE)
  ref_estimate <- matrix(
    c(
      NA_real_, 0.248471103773309, 0.0998513936435007,
      0.248471103773309, NA_real_, -0.0165169652366662,
      0.0998513936435007, -0.0165169652366662, NA_real_
    ),
    nrow = 3,
    byrow = TRUE
  )
  ref_statistic <- matrix(
    c(
      NA_real_, 14.2337297104894, 5.56845748933565,
      14.2337297104894, NA_real_, -0.916630998257919,
      5.56845748933565, -0.916630998257919, NA_real_
    ),
    nrow = 3,
    byrow = TRUE
  )
  ref_df <- matrix(
    c(
      NA_real_, 3079, 3079,
      3079, NA_real_, 3079,
      3079, 3079, NA_real_
    ),
    nrow = 3,
    byrow = TRUE
  )
  ref_p_value <- matrix(
    c(
      NA_real_, 0, 1.3955280708799478e-08,
      0, NA_real_, 0.82029609217836286,
      1.3955280708799478e-08, 0.82029609217836286, NA_real_
    ),
    nrow = 3,
    byrow = TRUE
  )

  expect_type(inf, "list")
  expect_identical(inf$method, "dcor_t_test")
  expect_true(is.matrix(inf$estimate))
  expect_true(is.matrix(inf$statistic))
  expect_true(is.matrix(inf$parameter))
  expect_true(is.matrix(inf$p_value))

  idx <- which(upper.tri(dc), arr.ind = TRUE)
  for (k in seq_len(nrow(idx))) {
    i <- idx[k, 1]
    j <- idx[k, 2]
    ref_est <- ref_estimate[i, j]
    ref_t <- ref_statistic[i, j]
    ref_df_ij <- ref_df[i, j]
    ref_p <- ref_p_value[i, j]

    expect_lt(abs(inf$estimate[i, j] - ref_est), 1e-6)
    expect_lt(abs(inf$statistic[i, j] - ref_t), 1e-5)
    expect_equal(inf$parameter[i, j], ref_df_ij, tolerance = 1e-10)
    if (isTRUE(is.finite(ref_p) && ref_p == 0)) {
      expect_gte(inf$p_value[i, j], 0)
      expect_lt(inf$p_value[i, j], .Machine$double.eps)
    } else {
      expect_lt(abs(inf$p_value[i, j] - ref_p), 1e-10)
    }
    expect_lt(abs(dc[i, j] - max(ref_est, 0)), 1e-6)
  }
})

test_that("requesting inference does not change the dcor estimate matrix", {
  X <- cbind(
    a = c(-2.0, -0.5, 0.0, 1.5, 2.0, 2.5),
    b = c(1.0, 0.2, -0.1, 0.5, 1.1, 1.8),
    c = c(0.5, -1.1, 0.7, -0.3, 1.6, -0.8)
  )

  est_only <- dcor(X)
  with_test <- dcor(X, p_value = TRUE)
  est_only_mat <- unclass(est_only)
  with_test_mat <- unclass(with_test)
  attributes(est_only_mat) <- attributes(est_only_mat)[c("dim", "dimnames")]
  attributes(with_test_mat) <- attributes(with_test_mat)[c("dim", "dimnames")]

  expect_equal(est_only_mat, with_test_mat, tolerance = 1e-12)
})

test_that("dcor stores inference payload only when requested", {
  X <- cbind(
    a = c(1, 2, 3, 4, 5, 6),
    b = c(2, 1, 4, 3, 6, 5),
    c = c(1, 4, 2, 5, 3, 6)
  )

  fit <- dcor(X)
  fit_p <- dcor(X, p_value = TRUE)

  expect_null(attr(fit, "inference", exact = TRUE))
  expect_null(attr(fit, "diagnostics", exact = TRUE))

  inf <- attr(fit_p, "inference", exact = TRUE)
  expect_type(inf, "list")
  expect_identical(names(inf), c("method", "estimate", "statistic", "parameter", "p_value", "alternative"))
  expect_true(is.matrix(inf$estimate))
  expect_true(is.matrix(inf$statistic))
  expect_true(is.matrix(inf$parameter))
  expect_true(is.matrix(inf$p_value))
})

test_that("dcor honors n_threads without changing estimates", {
  set.seed(246)
  X <- matrix(rnorm(240), nrow = 40, ncol = 6)
  colnames(X) <- paste0("D", seq_len(ncol(X)))

  fit1 <- dcor(X, n_threads = 1L)
  fit2 <- dcor(X, n_threads = 2L)
  fit1_p <- dcor(X, p_value = TRUE, n_threads = 1L)
  fit2_p <- dcor(X, p_value = TRUE, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(attr(fit1_p, "inference", exact = TRUE), attr(fit2_p, "inference", exact = TRUE), tolerance = 1e-12)
})

test_that("dcor summary switches to pairwise inference view when requested", {
  set.seed(11)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)
  colnames(X) <- paste0("D", seq_len(ncol(X)))

  dc_no_p <- dcor(X)
  sm_no_p <- summary(dc_no_p)
  expect_false(isTRUE(attr(sm_no_p, "has_p", exact = TRUE)))
  expect_false("p_value" %in% names(sm_no_p))

  dc <- dcor(X, p_value = TRUE)
  sm <- summary(dc)

  expect_s3_class(sm, "summary.dcor")
  expect_s3_class(sm, "data.frame")
  expect_true(all(c("item1", "item2", "estimate", "n_complete", "statistic", "df", "p_value") %in% names(sm)))

  txt <- capture.output(print(sm))
  expect_true(any(grepl("^Distance correlation summary$", txt)))
  expect_true(any(grepl("p_value", txt, fixed = TRUE)))
})

test_that("dcor print/plot cover optional parameters", {
  skip_if_not_installed("ggplot2")

  set.seed(303)
  X <- matrix(rnorm(60), nrow = 15, ncol = 4)
  colnames(X) <- paste0("D", seq_len(4))
  dc <- dcor(X)

  out <- capture.output(print(dc, digits = 3, max_rows = 2, max_cols = 3))
  expect_true(any(grepl("omitted", out)))

  p <- plot(dc, title = "Distance plot", low_color = "white", high_color = "navy", value_text_size = 3)
  expect_s3_class(p, "ggplot")
})

test_that("dcor rejects missing values by default", {
  X <- cbind(a = c(1, 2, NA, 4), b = c(1, 2, 3, 4), c = c(1, 2, 3, 4))
  expect_error(dcor(X), "Missing values are not allowed.")
})

#' @title Shrinkage Correlation
#'
#' @description
#' Computes a shrinkage correlation matrix for numeric data using a
#' high-performance 'C++' backend. The current implementation uses the
#' Schafer-Strimmer shrinkage estimator to stabilise Pearson correlation
#' estimates by shrinking off-diagonal entries towards zero.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns will be excluded. Columns must be numeric
#' and contain no \code{NA}s.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return A symmetric numeric matrix of class \code{shrinkage_corr} (with
#' compatibility class \code{schafer_corr}) where entry \code{(i, j)} is the
#' shrunk correlation between the \code{i}-th and \code{j}-th numeric columns.
#' Attributes:
#' \itemize{
#'   \item \code{method} = \code{"schafer_shrinkage"}
#'   \item \code{description} = \code{"Schafer-Strimmer shrinkage correlation
#'   matrix"}
#'   \item \code{package} = \code{"matrixCorr"}
#' }
#' Columns with zero variance are set to \code{NA} across row/column (including
#' the diagonal), matching \code{pearson_corr()} behaviour.
#'
#' @details
#' Let \eqn{R} be the sample Pearson correlation matrix. The Schafer-Strimmer
#' shrinkage estimator targets the identity in correlation space and uses
#' \eqn{\hat\lambda = \frac{\sum_{i<j}\widehat{\mathrm{Var}}(r_{ij})}
#' {\sum_{i<j} r_{ij}^2}} (clamped to \eqn{[0,1]}), where
#' \eqn{\widehat{\mathrm{Var}}(r_{ij}) \approx \frac{(1-r_{ij}^2)^2}{n-1}}.
#' The returned estimator is \eqn{R_{\mathrm{shr}} = (1-\hat\lambda)R +
#' \hat\lambda I}.
#'
#' @note
#' No missing values are permitted. Columns with fewer than two observations
#' or zero variance are flagged as \code{NA} (row/column).
#'
#' @references
#' Schafer, J. & Strimmer, K. (2005). A shrinkage approach to large-scale
#' covariance matrix estimation and implications for functional genomics.
#' \emph{Statistical Applications in Genetics and Molecular Biology}, 4(1).
#'
#' @examples
#' ## Multivariate normal with AR(1) dependence (Toeplitz correlation)
#' set.seed(1)
#' n <- 80; p <- 40; rho <- 0.6
#' d <- abs(outer(seq_len(p), seq_len(p), "-"))
#' Sigma <- rho^d
#'
#' X <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
#' colnames(X) <- paste0("V", seq_len(p))
#'
#' Rshr <- shrinkage_corr(X)
#' print(Rshr, digits = 2, n = 6, max_vars = 6)
#' summary(Rshr)
#' estimate(Rshr)
#' tidy(Rshr)
#' plot(Rshr)
#'
#' ## Shrinkage typically moves the sample correlation closer to the truth
#' Rraw <- stats::cor(X)
#' off  <- upper.tri(Sigma, diag = FALSE)
#' mae_raw <- mean(abs(Rraw[off] - Sigma[off]))
#' mae_shr <- mean(abs(Rshr[off] - Sigma[off]))
#' print(c(MAE_raw = mae_raw, MAE_shrunk = mae_shr))
#' plot(Rshr, title = "Schafer-Strimmer shrinkage correlation")
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(Rshr)
#' }
#'
#' @seealso \code{\link{print.shrinkage_corr}},
#'   \code{\link{plot.shrinkage_corr}}, \code{\link{pearson_corr}}
#' @author Thiago de Paula Oliveira
#' @export
shrinkage_corr <- function(data,
                           n_threads = getOption("matrixCorr.threads", 1L),
                           output = c("matrix", "sparse", "edge_list"),
                           threshold = 0,
                           diag = TRUE) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )
  numeric_data <- validate_corr_input(data)
  colnames_data <- colnames(numeric_data)

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  # call the C++ backend
  result <- sss_cor_cpp(numeric_data)
  if (!is.null(colnames_data)) {
    dimnames(result) <- .mc_square_dimnames(colnames_data)
  }

  out <- .mc_structure_corr_matrix(
    result,
    class_name = "shrinkage_corr",
    method = "schafer_shrinkage",
    description = "Schafer-Strimmer shrinkage correlation matrix",
    symmetric = TRUE,
    classes = c("shrinkage_corr", "schafer_corr", "matrix")
  )
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

#' Compatibility alias for \code{shrinkage_corr()}.
#'
#' @rdname shrinkage_corr
#' @export
schafer_corr <- function(data,
                         n_threads = getOption("matrixCorr.threads", 1L),
                         output = c("matrix", "sparse", "edge_list"),
                         threshold = 0,
                         diag = TRUE) {
  shrinkage_corr(
    data,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag
  )
}

#' @rdname shrinkage_corr
#' @method print shrinkage_corr
#' @title Print Method for \code{shrinkage_corr} Objects
#'
#' @param x An object of class \code{shrinkage_corr} or
#'   \code{schafer_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print}.
#'
#' @return Invisibly returns \code{x}.
#' @export
print.shrinkage_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                 max_vars = NULL, width = NULL,
                                 show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Schafer-Strimmer shrinkage correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname shrinkage_corr
#' @method print schafer_corr
#' @export
print.schafer_corr <- print.shrinkage_corr

#' @rdname shrinkage_corr
#' @method plot shrinkage_corr
#' @title Plot Method for \code{shrinkage_corr} Objects
#'
#' @param x An object of class \code{shrinkage_corr} or
#'   \code{schafer_corr}.
#' @param title Plot title.
#' @param cluster Logical; if TRUE, reorder rows/cols by hierarchical clustering
#'        on distance \eqn{1 - r}.
#' @param hclust_method Linkage method for \code{hclust}; default \code{"complete"}.
#' @param triangle One of \code{"full"}, \code{"upper"}, \code{"lower"}.
#' Default to \code{upper}.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles (subject to \code{value_text_limit}).
#' @param show_values Deprecated compatibility alias for \code{show_value}. If
#'   supplied, it overrides \code{show_value}.
#' @param value_text_limit Integer threshold controlling when values are drawn.
#' @param value_text_size Font size for values if shown.
#' @param palette Character; \code{"diverging"} (default) or \code{"viridis"}.
#' @param ... Additional arguments passed to \code{ggplot2::theme()}.
#'
#' @return A \code{ggplot} object.
#' @import ggplot2
#' @export
plot.shrinkage_corr <- function(
    x,
    title = "Schafer-Strimmer shrinkage correlation",
    cluster = TRUE,
    hclust_method = "complete",
    triangle = c("upper", "lower", "full"),
    show_value = TRUE,
    show_values = NULL,
    value_text_limit = 60,
    value_text_size = 3,
    palette = c("diverging", "viridis"),
    ...
) {
  check_inherits(x, c("shrinkage_corr", "schafer_corr"))
  if (!is.null(show_values)) {
    show_value <- show_values
  }
  check_bool(show_value, arg = "show_value")
  triangle <- match.arg(triangle)
  palette  <- match.arg(palette)

  mat <- as.matrix(x)
  p   <- ncol(mat)

  # clustering to reveal blocks
  if (cluster && p > 1) {
    d <- stats::as.dist(1 - pmax(pmin(mat, 1), -1))
    hc <- stats::hclust(d, method = hclust_method)
    ord <- hc$order
    mat <- mat[ord, ord, drop = FALSE]
  }

  if (triangle == "upper") {
    mat[lower.tri(mat)] <- NA_real_
  } else if (triangle == "lower") {
    mat[upper.tri(mat)] <- NA_real_
  }
  rn <- rownames(mat); if (is.null(rn)) rn <- as.character(seq_len(p))
  cn <- colnames(mat); if (is.null(cn)) cn <- as.character(seq_len(p))
  df <- data.frame(
    Var1 = factor(rep(rn, each = p), levels = rev(rn)),
    Var2 = factor(rep(cn, times = p), levels = cn),
    r    = as.vector(mat),
    stringsAsFactors = FALSE
  )
  df <- df[is.finite(df$r), , drop = FALSE]   # drop masked NAs

  # Palette
  if (palette == "diverging") {
    fill_scale <- ggplot2::scale_fill_gradient2(
      low = "indianred1", mid = "white", high = "steelblue1",
      midpoint = 0, limits = c(-1, 1), name = "r"
    )
  } else {
    if (!requireNamespace("viridisLite", quietly = TRUE)) {
      cli::cli_abort("Package {.pkg viridisLite} is required for {.arg palette} = \"viridis\".")
    }
    fill_scale <- ggplot2::scale_fill_gradientn(
      colours = viridisLite::viridis(256, option = "B"),
      limits = c(-1, 1), name = "r"
    )
  }

  p_ <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = r)) +
    ggplot2::geom_raster() +
    fill_scale +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      ...
    )

  # Draw numbers only for small matrices
  if (show_value && p <= value_text_limit) {
    p_ <- p_ + ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", r)),
      size = value_text_size, colour = "black"
    )
  } else if (p > 60) {
    # Hide tick labels for very large matrices
    p_ <- p_ + ggplot2::theme(
      axis.text.x = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_blank()
    )
  }

  p_
}

#' @rdname shrinkage_corr
#' @method plot schafer_corr
#' @export
plot.schafer_corr <- plot.shrinkage_corr

#' @rdname shrinkage_corr
#' @method summary shrinkage_corr
#' @param object An object of class \code{shrinkage_corr} or
#'   \code{schafer_corr}.
#' @export
summary.shrinkage_corr <- function(object, n = NULL, topn = NULL,
                                   max_vars = NULL, width = NULL,
                                   show_ci = NULL, ...) {
  .mc_summary_corr_matrix(object, topn = topn)
}

#' @rdname shrinkage_corr
#' @method summary schafer_corr
#' @export
summary.schafer_corr <- summary.shrinkage_corr

schafer_shrink_base <- function(X) {
  stopifnot(is.matrix(X))
  n <- nrow(X)
  R <- stats::cor(X)

  zero <- is.na(diag(R))
  if (any(zero)) {
    R[zero, ] <- NA_real_
    R[, zero] <- NA_real_
  }

  off <- upper.tri(R, diag = FALSE)
  r <- R[off]
  r <- r[is.finite(r)]
  sum_sq <- sum(r^2)

  if (sum_sq > 0) {
    v <- ((1 - r^2)^2) / (n - 1)
    lambda <- sum(v) / sum_sq
    lambda <- min(max(lambda, 0), 1)
  } else {
    lambda <- 1
  }

  Rsh <- R
  Rsh[off] <- (1 - lambda) * R[off]
  Rsh[lower.tri(Rsh)] <- t(Rsh)[lower.tri(Rsh)]
  diag(Rsh) <- ifelse(is.na(diag(R)), NA_real_, 1)
  attr(Rsh, "lambda") <- lambda
  Rsh
}

drop_schafer_attrs <- function(x) {
  m <- as.matrix(x)
  attributes(m) <- attributes(m)[c("dim", "dimnames")]
  m
}

test_that("shrinkage_corr returns shrinkage matrix with metadata", {
  set.seed(123)
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)
  colnames(X) <- paste0("V", seq_len(4))

  R <- shrinkage_corr(X)

  expect_s3_class(R, "shrinkage_corr")
  expect_s3_class(R, "schafer_corr")
  expect_equal(dim(R), c(4, 4))
  expect_true(all(diag(R) == 1))
  expect_equal(attr(R, "method"), "schafer_shrinkage")
  expect_equal(attr(R, "package"), "matrixCorr")

  R_raw <- stats::cor(X)
  off <- upper.tri(R_raw, diag = FALSE)
  expect_true(mean(abs(R[off])) <= mean(abs(R_raw[off])) + 1e-10)
})

test_that("base-R reference agrees within tolerance and attributes are sane", {
  set.seed(123)
  n <- 80
  p <- 30
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))

  R_pkg <- shrinkage_corr(X)
  R_base <- schafer_shrink_base(X)

  expect_s3_class(R_pkg, "shrinkage_corr")
  expect_s3_class(R_pkg, "schafer_corr")
  expect_true(is.matrix(R_pkg))
  expect_true(isSymmetric(R_pkg))
  expect_identical(attr(R_pkg, "method"), "schafer_shrinkage")
  expect_true(grepl("Scha", attr(R_pkg, "description"), fixed = TRUE))
  expect_equal(drop_schafer_attrs(R_pkg), drop_schafer_attrs(R_base), tolerance = 2e-4)
  expect_true(all(diag(R_pkg) == 1))
})

test_that("deterministic small example matches the base reference tightly", {
  X <- matrix(
    c(
      1.1, 0.3, -0.7,
      0.9, 0.4, -0.5,
      1.2, 0.2, -0.6,
      0.8, 0.5, -0.4,
      1.0, 0.1, -0.8
    ),
    nrow = 5, ncol = 3, byrow = TRUE
  )
  colnames(X) <- c("A", "B", "C")

  R_pkg <- shrinkage_corr(X)
  R_base <- schafer_shrink_base(X)

  expect_equal(unname(drop_schafer_attrs(R_pkg)),
               unname(drop_schafer_attrs(R_base)),
               tolerance = 1e-10)
})

test_that("shrinkage_corr flags zero-variance columns as NA", {
  X <- cbind(const = rep(2, 6), var = rnorm(6))

  R <- shrinkage_corr(X)

  expect_true(all(is.na(R["const", ])))
  expect_true(all(is.na(R[, "const"])))
})

test_that("data.frame path ignores non-numeric columns and preserves names", {
  set.seed(7)
  n <- 40
  df <- data.frame(
    a = rnorm(n),
    b = rnorm(n),
    grp = rep(letters[1:4], length.out = n),
    flag = rep(c(TRUE, FALSE), length.out = n)
  )
  out <- shrinkage_corr(df)

  expect_equal(dim(out), c(2, 2))
  expect_setequal(colnames(out), c("a", "b"))
  expect_true(isSymmetric(out))
  expect_true(all(diag(out) == 1))
})

test_that("independent columns are shrunk closer to identity than raw correlations", {
  set.seed(99)
  n <- 60
  p <- 40
  X <- matrix(rnorm(n * p), n, p)

  R_raw <- stats::cor(X)
  R_shr <- shrinkage_corr(X)
  I <- diag(p)

  err_raw <- sqrt(sum((R_raw - I)^2))
  err_shr <- sqrt(sum((R_shr - I)^2))
  expect_lt(err_shr, err_raw)

  off <- upper.tri(R_raw, diag = FALSE)
  r <- R_raw[off]
  lambda_hat <- {
    r2 <- r^2
    sum_sq <- sum(r2[is.finite(r2)])
    if (sum_sq > 0) {
      v <- ((1 - r^2)^2) / (n - 1)
      min(max(sum(v[is.finite(v)]) / sum_sq, 0), 1)
    } else {
      1
    }
  }
  expect_equal(unname(as.vector(R_shr[off])),
               unname(as.vector((1 - lambda_hat) * r)),
               tolerance = 5e-5)
})

test_that("p >> n returns a valid symmetric PSD correlation matrix", {
  set.seed(2024)
  n <- 30
  p <- 120
  X <- matrix(rnorm(n * p), n, p)

  R_shr <- shrinkage_corr(X)

  expect_equal(dim(R_shr), c(p, p))
  expect_true(isSymmetric(R_shr))
  ev <- eigen(R_shr, symmetric = TRUE, only.values = TRUE)$values
  expect_gte(min(ev), -1e-8)
})

test_that("print and plot methods cover optional arguments", {
  skip_if_not_installed("ggplot2")

  set.seed(456)
  X <- matrix(rnorm(60), nrow = 15, ncol = 4)
  colnames(X) <- paste0("G", seq_len(4))
  R <- shrinkage_corr(X)

  out <- capture.output(print(R, digits = 3, max_rows = 2, max_cols = 3))
  expect_true(any(grepl("omitted", out)))

  p1 <- plot(R, cluster = TRUE, triangle = "lower", show_values = TRUE, value_text_limit = 10)
  expect_s3_class(p1, "ggplot")

  if (requireNamespace("viridisLite", quietly = TRUE)) {
    p2 <- plot(R, cluster = FALSE, triangle = "upper", palette = "viridis")
    expect_s3_class(p2, "ggplot")
  }
})

test_that("schafer_corr remains a compatibility alias", {
  set.seed(55)
  X <- matrix(rnorm(50), nrow = 10, ncol = 5)

  new_fit <- shrinkage_corr(X)
  old_fit <- schafer_corr(X)

  expect_identical(unclass(new_fit), unclass(old_fit))
  expect_s3_class(old_fit, "shrinkage_corr")
  expect_s3_class(old_fit, "schafer_corr")
})

test_that("shrinkage_corr honors n_threads without changing estimates", {
  set.seed(808)
  X <- matrix(rnorm(240), nrow = 40, ncol = 6)
  colnames(X) <- paste0("G", seq_len(ncol(X)))

  fit1 <- shrinkage_corr(X, n_threads = 1L)
  fit2 <- shrinkage_corr(X, n_threads = 2L)
  alias_fit <- schafer_corr(X, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(unclass(fit2), unclass(alias_fit), tolerance = 1e-12)
})

#' @title Krippendorff's Alpha Reliability Coefficient
#'
#' @description
#' Estimates Krippendorff's alpha as a panel-level reliability/agreement
#' coefficient among two or more coders, raters, observers, judges, or
#' instruments. Missing ratings are supported through \code{na_method}, and
#' nominal, ordinal, interval, and ratio disagreement functions are available.
#'
#' @param data Input data.
#'   For \code{input = "ratings"}, rows are units/items and columns are
#'   coders/raters. For \code{input = "counts"}, rows are units/items and
#'   columns are categories containing per-item category counts.
#' @param levels Optional category labels in analysis order. For ordinal
#'   character data, explicit \code{levels} are required unless the data are
#'   ordered factors. For interval and ratio alpha, \code{levels} must be
#'   coercible to finite numeric values.
#' @param input One of \code{"ratings"} or \code{"counts"}.
#' @param level Measurement level: \code{"nominal"}, \code{"ordinal"},
#'   \code{"interval"}, or \code{"ratio"}.
#' @param method Estimator. \code{"customary"} uses the coincidence-matrix
#'   estimator of Krippendorff/Hayes. \code{"analytical"} uses the improved
#'   estimator and jackknife inference direction described by Hughes.
#' @param na_method Missing-data rule. \code{"error"} rejects missing ratings,
#'   \code{"complete"} removes any item with one or more missing ratings, and
#'   \code{"available"} retains items with at least \code{min_raters} observed
#'   ratings. For \code{input = "counts"}, missing counts are not allowed and
#'   rows with total counts below \code{min_raters} are dropped.
#' @param min_raters Minimum number of observed ratings required for a retained
#'   item. Must be at least \code{2}.
#' @param ci Logical; if \code{TRUE}, attach confidence intervals.
#' @param p_value Logical; if \code{TRUE}, attach test statistics and p-values.
#'   This is available only for \code{method = "analytical"} with jackknife
#'   inference.
#' @param conf_level Confidence level for intervals. Default is \code{0.95}.
#' @param se_method Standard-error method. \code{"auto"} resolves to
#'   \code{"bootstrap"} for customary alpha when \code{ci = TRUE}, and to
#'   \code{"jackknife"} for analytical alpha when \code{ci = TRUE} or
#'   \code{p_value = TRUE}. \code{"bootstrap"} is allowed only for customary
#'   alpha. \code{"jackknife"} is the primary inference path for analytical
#'   alpha. \code{"none"} disables uncertainty output.
#' @param n_boot Number of bootstrap resamples used for customary alpha when
#'   \code{ci = TRUE}.
#' @param seed Optional positive integer seed used for reproducible bootstrap
#'   resampling.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   C++ backend.
#' @param return_matrices Logical; if \code{TRUE}, attach the retained count
#'   matrix, observed coincidence matrix, expected coincidence matrix, and
#'   disagreement matrix in \code{attr(x, "matrices")}.
#' @param verbose Logical; if \code{TRUE}, emit short progress messages.
#' @param ... Reserved for future extensions. Unsupported extra arguments are
#'   rejected.
#'
#' @details
#' Krippendorff's alpha is a panel-level reliability coefficient, not a
#' pairwise correlation matrix. The customary coincidence-matrix estimator is
#' \deqn{ \alpha = 1 - \frac{D_o}{D_e}, }
#' where \eqn{D_o} is the observed disagreement and \eqn{D_e} is the expected
#' disagreement under chance assignment.
#'
#' For retained item \eqn{u} with \eqn{m_u} observed ratings and category
#' counts \eqn{n_{uc}}, the observed coincidence matrix updates by
#' \deqn{ O_{ck} \leftarrow O_{ck} +
#'        \frac{n_{uc}(n_{uk} - 1(c=k))}{m_u - 1}. }
#' Let \eqn{n_c = \sum_k O_{ck}} and \eqn{n = \sum_c n_c}. Then
#' \deqn{ D_o = \frac{1}{n}\sum_{c,k} O_{ck}\,\delta^2_{ck}, }
#' \deqn{ D_e = \frac{1}{n(n-1)}\sum_{c,k} n_c n_k\,\delta^2_{ck}, }
#' where \eqn{\delta^2_{ck}} is the level-specific disagreement matrix.
#'
#' The disagreement functions implemented here are:
#' \itemize{
#'   \item nominal: \eqn{\delta^2_{ck} = 1(c \ne k)}
#'   \item ordinal: Krippendorff's cumulative-margin disagreement based on the
#'   pooled category margins
#'   \item interval: \eqn{\delta^2_{ck} = (v_c - v_k)^2}
#'   \item ratio:
#'   \eqn{\delta^2_{ck} = (v_c - v_k)^2 / (v_c + v_k)^2}, with
#'   \eqn{\delta^2_{ck}=0} when both values are zero
#' }
#'
#' \strong{Analytical estimator and inference.}
#' Hughes (2024) recommends a different point estimator based on within-item
#' and pooled pairwise disagreement. Let \eqn{a} be the number of retained
#' items, \eqn{m_i} the number of ratings in item \eqn{i}, and
#' \eqn{N = \sum_i m_i}. Define
#' \deqn{ \bar n = \frac{N - \sum_i m_i^2 / N}{a - 1}. }
#' The within-item mean square is
#' \deqn{ MSE =
#'        \frac{1}{N}\sum_{i=1}^a
#'        \frac{\sum_{j<k}\delta^2(x_{ij}, x_{ik})}{m_i - 1}, }
#' the pooled total disagreement is
#' \deqn{ SST = \frac{1}{N}\sum_{r<s}\delta^2(z_r, z_s), }
#' where \eqn{z_1,\ldots,z_N} are the pooled observed ratings, and
#' \deqn{ SSA = SST - (N-a)\,MSE, \qquad
#'        MSA = \frac{SSA}{a-1}. }
#' With \eqn{\theta = MSA/MSE} and \eqn{\eta = \log(\theta)}, the analytical
#' alpha estimate is
#' \deqn{ \alpha =
#'        \frac{\theta - 1}{\theta + \bar n - 1}. }
#'
#' When analytical inference is requested, the implementation uses the
#' delete-one-item jackknife on \eqn{\eta}. If \eqn{\eta_{(-i)}} is the
#' leave-one-item-out transform and \eqn{a} is the number of retained items,
#' the jackknife pseudo-values are
#' \deqn{ p_i = a\,\hat\eta - (a-1)\,\hat\eta_{(-i)}, }
#' with standard error
#' \deqn{ \widehat{\mathrm{se}}(\hat\eta)
#'        = \sqrt{\mathrm{Var}(p_1,\ldots,p_a) / a}. }
#' The confidence interval is formed on the \eqn{\eta}-scale with a
#' \eqn{t_{a-1}} critical value and back-transformed to alpha using the full
#' data \eqn{\bar n}. The analytical p-value tests \eqn{H_0: \alpha = 0},
#' equivalently \eqn{H_0: \eta = 0}, via the t statistic
#' \deqn{ t = \hat\eta / \widehat{\mathrm{se}}(\hat\eta). }
#'
#' For customary alpha, percentile bootstrap intervals are available by
#' resampling retained items with replacement and recomputing the customary
#' estimate.
#'
#' @return
#' A one-row data frame with class
#' \code{c("krippendorff_alpha", "agreement_result", "data.frame")}. The
#' object stores method metadata, diagnostics, and optional matrices in
#' attributes.
#'
#' @references
#' Krippendorff, K. (2011/2013). Computing Krippendorff's alpha-reliability.
#'
#' Hayes, A. F. and Krippendorff, K. (2007). Answering the call for a standard
#' reliability measure for coding data. \emph{Communication Methods and
#' Measures}, 1(1), 77-89.
#'
#' Hughes, J. (2024). Toward improved inference for Krippendorff's Alpha
#' agreement coefficient. \emph{Journal of Statistical Planning and
#' Inference}, 233, 106170.
#'
#' @seealso
#' [multirater_kappa()] for nominal multi-rater kappa;
#' [gwet_ac()] for AC1/AC2 agreement coefficients.
#'
#' @examples
#' raters <- data.frame(
#'   r1 = c("A", "A", "B", "B", "C", "A"),
#'   r2 = c("A", "B", "B", "B", "C", "A"),
#'   r3 = c("A", "A", "B", "C", "C", "A"),
#'   stringsAsFactors = FALSE
#' )
#'
#' fit <- krippendorff_alpha(raters, level = "nominal", na_method = "available")
#' print(fit)
#' summary(fit)
#' estimate(fit)
#' tidy(fit)
#' plot(fit)
#'
#' @author Thiago de Paula Oliveira
#' @export
krippendorff_alpha <- function(data,
                               levels = NULL,
                               input = c("ratings", "counts"),
                               level = c("nominal", "ordinal", "interval", "ratio"),
                               method = c("customary", "analytical"),
                               na_method = c("error", "complete", "available"),
                               min_raters = 2L,
                               ci = FALSE,
                               p_value = FALSE,
                               conf_level = 0.95,
                               se_method = c("auto", "bootstrap", "jackknife", "none"),
                               n_boot = 1000L,
                               seed = NULL,
                               n_threads = getOption("matrixCorr.threads", 1L),
                               return_matrices = FALSE,
                               verbose = FALSE,
                               ...) {
  .mc_extract_legacy_aliases(list(...), allowed = character())

  input <- match_arg(input, c("ratings", "counts"), arg_name = "input")
  level <- match_arg(level, c("nominal", "ordinal", "interval", "ratio"), arg_name = "level")
  method <- match_arg(method, c("customary", "analytical"), arg_name = "method")
  na_method <- validate_na_method(
    na_method,
    arg = "na_method",
    allowed = c("error", "complete", "available")
  )
  min_raters <- check_scalar_int_pos(min_raters, arg = "min_raters")
  if (min_raters < 2L) {
    abort_bad_arg("min_raters", message = "must be >= 2.")
  }
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_bool(return_matrices, arg = "return_matrices")
  check_bool(verbose, arg = "verbose")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  if (!is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  }

  se_method <- match_arg(
    se_method,
    c("auto", "bootstrap", "jackknife", "none"),
    arg_name = "se_method"
  )
  resolved_se_method <- .mc_krippendorff_resolve_se_method(
    method = method,
    se_method = se_method,
    ci = ci,
    p_value = p_value
  )

  if (identical(resolved_se_method, "bootstrap")) {
    n_boot <- check_scalar_int_pos(n_boot, arg = "n_boot")
  }

  prep <- if (identical(input, "ratings")) {
    .mc_prepare_agreement_ratings(
      data = data,
      levels = levels,
      level = level,
      na_method = na_method,
      min_raters = min_raters
    )
  } else {
    .mc_prepare_agreement_counts(
      data = data,
      levels = levels,
      level = level,
      min_raters = min_raters
    )
  }

  delta2 <- .mc_krippendorff_delta2(
    categories = prep$categories,
    level = level,
    coincidence_margins = colSums(prep$counts)
  )

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  raw <- krippendorff_alpha_core_cpp(
    counts = prep$counts,
    delta2 = delta2,
    method_code = if (identical(method, "analytical")) 2L else 1L,
    return_matrices = return_matrices,
    n_threads = n_threads
  )

  estimate <- .mc_clamp_unit(as.numeric(raw$estimate))
  se_out <- NA_real_
  statistic_out <- NA_real_
  p_value_out <- NA_real_
  lwr <- NA_real_
  upr <- NA_real_
  ci_method <- NULL
  boot_vals <- NULL
  boot_kept <- NULL
  jackknife <- NULL

  if (identical(resolved_se_method, "bootstrap")) {
    boot_vals <- .mc_krippendorff_bootstrap_customary(
      counts = prep$counts,
      delta2 = delta2,
      n_boot = n_boot,
      seed = seed,
      n_threads = n_threads
    )
    boot_kept <- sum(is.finite(boot_vals))
    if (boot_kept >= 2L) {
      se_out <- stats::sd(boot_vals[is.finite(boot_vals)])
    }
    if (isTRUE(ci)) {
      ci_vals <- .mc_percentile_boot_ci(boot_vals, conf_level = conf_level)
      lwr <- .mc_clamp_unit(ci_vals[[1L]])
      upr <- .mc_clamp_unit(ci_vals[[2L]])
      ci_method <- "percentile_bootstrap"
    }
  } else if (identical(resolved_se_method, "jackknife")) {
    jackknife <- .mc_krippendorff_analytical_jackknife(
      counts = prep$counts,
      delta2 = delta2,
      full_raw = raw,
      conf_level = conf_level,
      ci = ci,
      p_value = p_value,
      n_threads = n_threads
    )
    se_out <- jackknife$se
    statistic_out <- jackknife$statistic
    p_value_out <- jackknife$p_value
    lwr <- jackknife$lwr
    upr <- jackknife$upr
    ci_method <- if (isTRUE(ci)) "jackknife_t_eta" else NULL
  }

  out <- data.frame(
    method = "krippendorff_alpha",
    alpha = estimate,
    level = level,
    estimator = method,
    observed_disagreement = as.numeric(raw$observed_disagreement),
    expected_disagreement = as.numeric(raw$expected_disagreement),
    n_items = as.integer(raw$n_items),
    n_categories = as.integer(raw$n_categories),
    n_raters_min = as.integer(raw$n_raters_min),
    n_raters_max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced),
    n_ratings_total = as.integer(raw$n_ratings_total),
    se = se_out,
    statistic = statistic_out,
    p_value = p_value_out,
    lwr.ci = if (isTRUE(ci)) lwr else NA_real_,
    upr.ci = if (isTRUE(ci)) upr else NA_real_,
    conf.level = if (isTRUE(ci)) conf_level else NA_real_,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
  class(out) <- c("krippendorff_alpha", "agreement_result", "data.frame")

  matrices_attr <- NULL
  if (isTRUE(return_matrices)) {
    coincidence <- as.matrix(raw$coincidence)
    expected_coincidence <- as.matrix(raw$expected_coincidence)
    colnames(coincidence) <- rownames(coincidence) <- as.character(prep$categories)
    colnames(expected_coincidence) <- rownames(expected_coincidence) <- as.character(prep$categories)
    colnames(delta2) <- rownames(delta2) <- as.character(prep$categories)
    counts_mat <- prep$counts
    colnames(counts_mat) <- as.character(prep$categories)
    matrices_attr <- list(
      counts = counts_mat,
      coincidence = coincidence,
      expected_coincidence = expected_coincidence,
      delta2 = delta2
    )
  }

  diagnostics <- c(
    list(
      category_counts = as.integer(raw$category_counts),
      category_proportions = as.numeric(raw$category_proportions),
      item_category_counts = prep$counts,
      n_raters_by_item = as.integer(raw$n_raters_by_item),
      item_disagreement = as.numeric(raw$item_disagreement),
      customary_alpha = as.numeric(raw$customary_alpha),
      analytical_alpha = as.numeric(raw$analytical_alpha),
      mse = as.numeric(raw$mse),
      sst = as.numeric(raw$sst),
      ssa = as.numeric(raw$ssa),
      msa = as.numeric(raw$msa),
      theta = as.numeric(raw$theta),
      eta = as.numeric(raw$eta),
      n_bar = as.numeric(raw$n_bar),
      ci_method = ci_method,
      boot_finite = boot_kept,
      eta_jackknife = if (!is.null(jackknife)) jackknife$eta_minus else NULL,
      eta_pseudovalues = if (!is.null(jackknife)) jackknife$pseudo else NULL
    ),
    prep$diagnostics
  )

  attr(out, "method") <- "krippendorff_alpha"
  attr(out, "description") <- "Krippendorff's alpha reliability coefficient"
  attr(out, "package") <- "matrixCorr"
  attr(out, "input") <- input
  attr(out, "level") <- level
  attr(out, "estimator") <- method
  attr(out, "categories") <- prep$categories
  attr(out, "n_categories") <- length(prep$categories)
  attr(out, "n_items") <- as.integer(raw$n_items)
  attr(out, "n_raters") <- list(
    min = as.integer(raw$n_raters_min),
    max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced)
  )
  attr(out, "na_method") <- if (identical(input, "counts")) na_method else prep$na_method
  attr(out, "min_raters") <- min_raters
  attr(out, "se.method") <- resolved_se_method
  attr(out, "conf.level") <- if (isTRUE(ci)) conf_level else NULL
  attr(out, "n_boot") <- if (identical(resolved_se_method, "bootstrap")) n_boot else NULL
  attr(out, "diagnostics") <- diagnostics
  attr(out, "reference") <- if (identical(method, "analytical")) {
    paste(
      "Hughes (2024), Journal of Statistical Planning and Inference, 233, 106170;",
      "Krippendorff (2011/2013), Computing Krippendorff's alpha-reliability"
    )
  } else {
    "Krippendorff (2011/2013), Computing Krippendorff's alpha-reliability"
  }
  if (isTRUE(return_matrices)) {
    attr(out, "matrices") <- matrices_attr
  }

  inform_if_verbose(
    "Computed Krippendorff's alpha ({.val {method}}, {.val {level}}) for {attr(out, 'n_items', exact = TRUE)} item{?s} across {length(prep$categories)} categor{?y/ies}.",
    .verbose = verbose
  )
  out
}

.mc_agreement_rating_column <- function(x,
                                        arg = as.character(substitute(x))) {
  out <- .mc_weighted_kappa_prepare_vector(x, arg = arg)
  out$is_factor <- is.factor(x)
  out$factor_levels <- if (is.factor(x)) levels(x) else NULL
  out
}

.mc_resolve_agreement_levels <- function(prepared,
                                         levels = NULL,
                                         level = c("nominal", "ordinal", "interval", "ratio")) {
  level <- match.arg(level)
  observed <- unlist(lapply(prepared, function(z) z$key[!is.na(z$key)]), use.names = FALSE)

  if (!is.null(levels)) {
    lev <- .mc_weighted_kappa_normalize_levels(levels)
    lev_key <- as.character(lev)
    if (length(observed) && any(!observed %in% lev_key)) {
      abort_bad_arg("levels", message = "must contain every non-missing observed category.")
    }
    if (level %in% c("interval", "ratio")) {
      num <- suppressWarnings(as.numeric(lev_key))
      if (anyNA(num) || any(!is.finite(num))) {
        abort_bad_arg("levels", message = "must be coercible to finite numeric values for interval/ratio alpha.")
      }
      if (identical(level, "ratio") && any(num < 0)) {
        abort_bad_arg("levels", message = "must be non-negative for ratio alpha.")
      }
    }
    return(lev)
  }

  if (!length(observed)) {
    abort_bad_arg(
      "levels",
      message = "must be supplied when no non-missing ratings are available to infer categories."
    )
  }

  if (identical(level, "nominal")) {
    if (all(vapply(prepared, function(z) isTRUE(z$numeric_infer), logical(1)))) {
      vals <- unlist(lapply(prepared, function(z) z$numeric_values[!is.na(z$numeric_values)]), use.names = FALSE)
      return(sort(unique(vals)))
    }
    from_levels <- unlist(lapply(prepared, function(z) z$factor_levels), use.names = FALSE)
    if (length(from_levels)) {
      out <- unique(c(from_levels, observed))
      return(out)
    }
    return(unique(observed))
  }

  if (identical(level, "ordinal")) {
    if (all(vapply(prepared, `[[`, logical(1), "is_ordered"))) {
      base_levels <- prepared[[1L]]$ordered_levels
      same_levels <- vapply(
        prepared,
        function(z) identical(z$ordered_levels, base_levels),
        logical(1)
      )
      if (all(same_levels)) {
        return(base_levels)
      }
    }
    numeric_ok <- suppressWarnings(as.numeric(observed))
    if (all(is.finite(numeric_ok))) {
      return(sort(unique(numeric_ok)))
    }
    abort_bad_arg(
      "levels",
      message = paste(
        "must be supplied for ordinal character/unordered-factor data.",
        "Automatic order inference is limited to identical ordered-factor levels or numeric values."
      )
    )
  }

  numeric_ok <- suppressWarnings(as.numeric(observed))
  if (anyNA(numeric_ok) || any(!is.finite(numeric_ok))) {
    abort_bad_arg(
      "levels",
      message = "must be supplied as numeric/coercible values for interval/ratio alpha."
    )
  }
  out <- sort(unique(numeric_ok))
  if (identical(level, "ratio") && any(out < 0)) {
    abort_bad_arg("data", message = "must not contain negative categories for ratio alpha.")
  }
  out
}

.mc_prepare_agreement_ratings <- function(data,
                                          levels = NULL,
                                          level = c("nominal", "ordinal", "interval", "ratio"),
                                          na_method = c("error", "complete", "available"),
                                          min_raters = 2L) {
  level <- match.arg(level)
  na_method <- match.arg(na_method)
  if (!is.data.frame(data) && !is.matrix(data)) {
    abort_bad_arg("data", message = "must be a matrix or data frame when {.arg input} is {.val ratings}.")
  }

  df <- if (is.data.frame(data)) data else as.data.frame(data, stringsAsFactors = FALSE)
  if (ncol(df) < 2L) {
    abort_bad_arg("data", message = "must contain at least two rater columns.")
  }

  cols <- vector("list", ncol(df))
  for (j in seq_along(cols)) {
    col_arg <- names(df)[[j]]
    if (is.null(col_arg) || !nzchar(col_arg)) {
      col_arg <- sprintf("data[[%d]]", j)
    } else {
      col_arg <- sprintf("data$%s", col_arg)
    }
    cols[[j]] <- .mc_agreement_rating_column(df[[j]], arg = col_arg)
  }
  categories <- .mc_resolve_agreement_levels(cols, levels = levels, level = level)
  category_keys <- as.character(categories)
  if (length(categories) < 2L) {
    abort_bad_arg("levels", message = "must define at least two categories.")
  }

  X <- matrix(
    NA_integer_,
    nrow = nrow(df),
    ncol = ncol(df),
    dimnames = list(NULL, names(df))
  )
  for (j in seq_along(cols)) {
    codes <- match(cols[[j]]$key, category_keys)
    codes[is.na(cols[[j]]$key)] <- NA_integer_
    bad <- !is.na(cols[[j]]$key) & is.na(codes)
    if (any(bad)) {
      abort_bad_arg("data", message = "contains values not present in {.arg levels}.")
    }
    X[, j] <- as.integer(codes)
  }

  n_available <- rowSums(!is.na(X))
  keep <- rep(TRUE, nrow(X))
  if (identical(na_method, "error") && anyNA(X)) {
    abort_bad_arg(
      "data",
      message = "contains missing ratings when {.arg na_method} is {.val error}."
    )
  }
  if (identical(na_method, "complete")) {
    keep <- stats::complete.cases(X)
  } else if (identical(na_method, "available")) {
    keep <- n_available >= min_raters
  }
  X_retained <- X[keep, , drop = FALSE]
  retained_available <- n_available[keep]
  if (length(retained_available)) {
    keep_raters <- retained_available >= min_raters
    X_retained <- X_retained[keep_raters, , drop = FALSE]
    idx_keep <- which(keep)
    keep[idx_keep[!keep_raters]] <- FALSE
  }

  if (nrow(X_retained) < 2L) {
    abort_bad_arg(
      "data",
      message = "must retain at least two items with at least {.val {min_raters}} observed ratings."
    )
  }

  counts <- matrix(
    0L,
    nrow = nrow(X_retained),
    ncol = length(categories),
    dimnames = list(paste0("item_", seq_len(nrow(X_retained))), as.character(categories))
  )
  for (i in seq_len(nrow(X_retained))) {
    xi <- X_retained[i, ]
    xi <- xi[!is.na(xi)]
    if (length(xi)) {
      counts[i, ] <- tabulate(xi, nbins = ncol(counts))
    }
  }

  list(
    counts = counts,
    categories = categories,
    n_levels = length(categories),
    retained_rows = keep,
    dropped_rows = !keep,
    na_method = na_method,
    diagnostics = list(
      n_original = nrow(df),
      n_retained = nrow(counts),
      retained_rows = keep,
      dropped_rows = !keep,
      n_available_by_item = n_available
    )
  )
}

.mc_prepare_agreement_counts <- function(data,
                                         levels = NULL,
                                         level = c("nominal", "ordinal", "interval", "ratio"),
                                         min_raters = 2L) {
  level <- match.arg(level)
  if (!is.data.frame(data) && !is.matrix(data)) {
    abort_bad_arg("data", message = "must be a matrix or data frame when {.arg input} is {.val counts}.")
  }
  X <- if (is.matrix(data)) data else as.matrix(data)
  if (!is.numeric(X)) {
    abort_bad_arg("data", message = "must contain numeric count columns when {.arg input} is {.val counts}.")
  }
  if (ncol(X) < 2L) {
    abort_bad_arg("data", message = "must contain at least two category columns.")
  }
  if (anyNA(X)) {
    abort_bad_arg("data", message = "must not contain missing counts.")
  }
  if (any(!is.finite(X))) {
    abort_bad_arg("data", message = "must contain only finite counts.")
  }
  if (any(X < 0)) {
    abort_bad_arg("data", message = "must contain non-negative counts.")
  }
  if (any(abs(X - round(X)) > sqrt(.Machine$double.eps))) {
    abort_bad_arg("data", message = "must contain integer-like count values.")
  }

  x_names <- colnames(X)
  if (!is.null(x_names) && anyDuplicated(x_names)) {
    abort_bad_arg("data", message = "must not contain duplicate category column names.")
  }

  categories <- NULL
  counts_aligned <- NULL
  if (!is.null(levels)) {
    categories <- .mc_resolve_agreement_levels(
      prepared = list(list(
        key = if (!is.null(x_names)) x_names else character(),
        is_ordered = FALSE,
        ordered_levels = NULL,
        numeric_values = NULL,
        numeric_infer = FALSE,
        is_factor = FALSE,
        factor_levels = NULL
      )),
      levels = levels,
      level = level
    )
    cat_keys <- as.character(categories)
    if (is.null(x_names)) {
      if (length(categories) != ncol(X)) {
        abort_bad_arg(
          "levels",
          message = "must have length equal to ncol(data) when count columns are unnamed."
        )
      }
      counts_aligned <- matrix(
        as.integer(round(X)),
        nrow = nrow(X),
        ncol = ncol(X),
        dimnames = list(NULL, cat_keys)
      )
    } else {
      idx <- match(x_names, cat_keys)
      if (anyNA(idx)) {
        abort_bad_arg("levels", message = "must contain every observed count column name.")
      }
      if (anyDuplicated(idx)) {
        abort_bad_arg("data", message = "maps multiple columns to the same category in {.arg levels}.")
      }
      counts_aligned <- matrix(
        0L,
        nrow = nrow(X),
        ncol = length(categories),
        dimnames = list(NULL, cat_keys)
      )
      counts_aligned[, idx] <- as.integer(round(X))
    }
  } else {
    if (is.null(x_names)) {
      if (level %in% c("interval", "ratio")) {
        abort_bad_arg(
          "levels",
          message = "must be supplied or inferred from count-column names for interval/ratio alpha."
        )
      }
      categories <- seq_len(ncol(X))
    } else {
      categories <- if (level %in% c("interval", "ratio")) {
        num <- suppressWarnings(as.numeric(x_names))
        if (anyNA(num) || any(!is.finite(num))) {
          abort_bad_arg(
            "levels",
            message = "count-column names must be coercible to finite numeric values for interval/ratio alpha."
          )
        }
        if (identical(level, "ratio") && any(num < 0)) {
          abort_bad_arg("data", message = "must not contain negative categories for ratio alpha.")
        }
        num
      } else if (identical(level, "ordinal")) {
        x_names
      } else {
        x_names
      }
    }
    categories <- .mc_resolve_agreement_levels(
      prepared = list(list(
        key = as.character(categories),
        is_ordered = FALSE,
        ordered_levels = NULL,
        numeric_values = if (is.numeric(categories)) categories else NULL,
        numeric_infer = is.numeric(categories),
        is_factor = FALSE,
        factor_levels = NULL
      )),
      levels = categories,
      level = level
    )
    counts_aligned <- matrix(
      as.integer(round(X)),
      nrow = nrow(X),
      ncol = ncol(X),
      dimnames = list(NULL, as.character(categories))
    )
  }

  row_sums <- rowSums(counts_aligned)
  keep <- row_sums >= min_raters
  counts_retained <- counts_aligned[keep, , drop = FALSE]
  rownames(counts_retained) <- paste0("item_", seq_len(nrow(counts_retained)))
  if (nrow(counts_retained) < 2L) {
    abort_bad_arg(
      "data",
      message = "must retain at least two items with at least {.val {min_raters}} ratings."
    )
  }

  list(
    counts = counts_retained,
    categories = categories,
    n_levels = length(categories),
    retained_rows = keep,
    dropped_rows = !keep,
    na_method = "available",
    diagnostics = list(
      n_original = nrow(counts_aligned),
      n_retained = nrow(counts_retained),
      retained_rows = keep,
      dropped_rows = !keep,
      row_sums = row_sums
    )
  )
}

.mc_krippendorff_delta2 <- function(categories,
                                    level = c("nominal", "ordinal", "interval", "ratio"),
                                    coincidence_margins = NULL) {
  level <- match.arg(level)
  K <- length(categories)
  out <- matrix(0, nrow = K, ncol = K, dimnames = list(as.character(categories), as.character(categories)))
  if (K <= 1L) {
    return(out)
  }

  if (identical(level, "nominal")) {
    out[row(out) != col(out)] <- 1
    return(out)
  }

  if (identical(level, "ordinal")) {
    if (is.null(coincidence_margins) || length(coincidence_margins) != K) {
      abort_bad_arg(
        "coincidence_margins",
        message = "must be supplied with one pooled category margin per category for ordinal alpha."
      )
    }
    ng <- as.numeric(coincidence_margins)
    if (any(!is.finite(ng)) || any(ng < 0)) {
      abort_bad_arg("coincidence_margins", message = "must contain finite non-negative margins.")
    }
    cs <- cumsum(ng)
    for (c in seq_len(K)) {
      if (c >= K) {
        next
      }
      for (k in seq.int(c + 1L, K)) {
        mass_ck <- cs[[k]] - if (c > 1L) cs[[c - 1L]] else 0
        val <- (mass_ck - (ng[[c]] + ng[[k]]) / 2)^2
        out[c, k] <- val
        out[k, c] <- val
      }
    }
    diag(out) <- 0
    return(out)
  }

  vals <- suppressWarnings(as.numeric(as.character(categories)))
  if (anyNA(vals) || any(!is.finite(vals))) {
    abort_bad_arg("levels", message = "must be coercible to finite numeric values for interval/ratio alpha.")
  }
  if (identical(level, "ratio") && any(vals < 0)) {
    abort_bad_arg("levels", message = "must be non-negative for ratio alpha.")
  }

  for (c in seq_len(K)) {
    for (k in seq_len(K)) {
      if (c == k) {
        next
      }
      if (identical(level, "interval")) {
        out[c, k] <- (vals[[c]] - vals[[k]])^2
      } else {
        denom <- (vals[[c]] + vals[[k]])^2
        out[c, k] <- if (denom <= 0) 0 else ((vals[[c]] - vals[[k]])^2) / denom
      }
    }
  }
  out
}

.mc_clamp_unit <- function(x, tol = 1e-12) {
  if (!is.finite(x)) {
    return(NA_real_)
  }
  if (x > 1 && x <= 1 + tol) {
    return(1)
  }
  if (x < -1 && x >= -1 - tol) {
    return(-1)
  }
  x
}

.mc_krippendorff_resolve_se_method <- function(method,
                                               se_method,
                                               ci = FALSE,
                                               p_value = FALSE) {
  inference_requested <- isTRUE(ci) || isTRUE(p_value)
  if (!inference_requested) {
    return("none")
  }

  if (identical(method, "customary")) {
    if (isTRUE(p_value)) {
      abort_bad_arg(
        "p_value",
        message = "must be FALSE for the customary estimator.",
        .hint = "P-values are available only for {.arg method} = {.val analytical} with jackknife inference."
      )
    }
    if (identical(se_method, "auto")) {
      return("bootstrap")
    }
    if (!identical(se_method, "bootstrap")) {
      abort_bad_arg(
        "se_method",
        message = "must be {.val bootstrap}, {.val auto}, or {.val none} for customary alpha."
      )
    }
    return("bootstrap")
  }

  if (identical(se_method, "auto")) {
    return("jackknife")
  }
  if (identical(se_method, "bootstrap")) {
    abort_bad_arg(
      "se_method",
      message = "{.val bootstrap} is not available for the analytical estimator.",
      .hint = "Use {.arg se_method} = {.val jackknife}."
    )
  }
  if (identical(se_method, "none")) {
    abort_bad_arg(
      "se_method",
      message = "must not be {.val none} when ci or p_value is TRUE."
    )
  }
  "jackknife"
}

.mc_krippendorff_bootstrap_customary <- function(counts,
                                                 delta2,
                                                 n_boot,
                                                 seed = NULL,
                                                 n_threads = 1L) {
  n_items <- nrow(counts)
  .mc_eval_with_seed(seed, {
    replicate(n_boot, {
      idx <- sample.int(n_items, size = n_items, replace = TRUE)
      raw_b <- krippendorff_alpha_core_cpp(
        counts = counts[idx, , drop = FALSE],
        delta2 = delta2,
        method_code = 1L,
        return_matrices = FALSE,
        n_threads = n_threads
      )
      .mc_clamp_unit(as.numeric(raw_b$estimate))
    })
  })
}

.mc_krippendorff_alpha_from_eta <- function(eta, n_bar) {
  theta <- exp(eta)
  .mc_clamp_unit((theta - 1) / (theta + n_bar - 1))
}

.mc_krippendorff_analytical_jackknife <- function(counts,
                                                  delta2,
                                                  full_raw,
                                                  conf_level = 0.95,
                                                  ci = FALSE,
                                                  p_value = FALSE,
                                                  n_threads = 1L) {
  out <- list(
    se = NA_real_,
    statistic = NA_real_,
    p_value = NA_real_,
    lwr = NA_real_,
    upr = NA_real_,
    eta_minus = rep(NA_real_, nrow(counts)),
    pseudo = rep(NA_real_, nrow(counts))
  )

  a <- as.integer(full_raw$n_items)
  eta_hat <- as.numeric(full_raw$eta)
  n_bar <- as.numeric(full_raw$n_bar)
  if (a < 3L || !is.finite(eta_hat) || !is.finite(n_bar)) {
    return(out)
  }

  eta_minus <- vapply(seq_len(a), function(i) {
    raw_i <- krippendorff_alpha_core_cpp(
      counts = counts[-i, , drop = FALSE],
      delta2 = delta2,
      method_code = 2L,
      return_matrices = FALSE,
      n_threads = n_threads
    )
    as.numeric(raw_i$eta)
  }, numeric(1))

  if (any(!is.finite(eta_minus))) {
    out$eta_minus <- eta_minus
    return(out)
  }

  pseudo <- a * eta_hat - (a - 1) * eta_minus
  se_eta <- sqrt(stats::var(pseudo) / a)
  out$eta_minus <- eta_minus
  out$pseudo <- pseudo
  out$se <- if (is.finite(se_eta) && se_eta > 0) se_eta else NA_real_
  if (!is.finite(out$se) || out$se <= 0) {
    return(out)
  }

  df <- a - 1
  tcrit <- stats::qt(1 - (1 - conf_level) / 2, df = df)
  if (isTRUE(ci)) {
    lo_eta <- eta_hat - tcrit * out$se
    hi_eta <- eta_hat + tcrit * out$se
    out$lwr <- .mc_krippendorff_alpha_from_eta(lo_eta, n_bar)
    out$upr <- .mc_krippendorff_alpha_from_eta(hi_eta, n_bar)
  }
  if (isTRUE(p_value)) {
    out$statistic <- eta_hat / out$se
    out$p_value <- 2 * stats::pt(abs(out$statistic), df = df, lower.tail = FALSE)
  }
  out
}

#' @method print krippendorff_alpha
#' @rdname krippendorff_alpha
#' @param x A \code{krippendorff_alpha} object.
#' @param digits Integer; number of decimal places for displayed numeric values.
#' @export
print.krippendorff_alpha <- function(x,
                                     digits = 4,
                                     ...) {
  check_inherits(x, "krippendorff_alpha")
  nr <- attr(x, "n_raters", exact = TRUE)
  has_ci <- is.finite(x$lwr.ci[[1L]]) && is.finite(x$upr.ci[[1L]])
  digest <- c(
    estimator = attr(x, "estimator", exact = TRUE),
    level = attr(x, "level", exact = TRUE),
    alpha = formatC(x$alpha[[1L]], format = "f", digits = digits),
    items = .mc_count_fmt(x$n_items[[1L]]),
    categories = .mc_count_fmt(x$n_categories[[1L]]),
    raters_per_item = paste0(
      .mc_count_fmt(nr$min), " to ", .mc_count_fmt(nr$max),
      if (isTRUE(nr$balanced)) " (balanced)" else " (unbalanced)"
    ),
    ci = if (has_ci) "yes" else "no",
    se_method = .mc_coalesce(attr(x, "se.method", exact = TRUE), "none")
  )
  .mc_print_named_digest(digest, header = "Krippendorff's alpha reliability coefficient")
  if (has_ci) {
    cat(
      sprintf(
        "  %-12s: %g%% [%s, %s]\n",
        "interval",
        100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))),
        formatC(x$lwr.ci[[1L]], format = "f", digits = digits),
        formatC(x$upr.ci[[1L]], format = "f", digits = digits)
      )
    )
  }
  if (is.finite(x$p_value[[1L]])) {
    cat(sprintf("  %-12s: %s\n", "p_value", formatC(x$p_value[[1L]], format = "f", digits = digits)))
  }
  invisible(x)
}

#' @method summary krippendorff_alpha
#' @rdname krippendorff_alpha
#' @param object A \code{krippendorff_alpha} object.
#' @param digits Integer; number of decimal places for rounded numeric columns.
#' @param ... Unused.
#' @export
summary.krippendorff_alpha <- function(object,
                                       digits = 4,
                                       ...) {
  check_inherits(object, "krippendorff_alpha")
  out <- as.data.frame(object, stringsAsFactors = FALSE, check.names = FALSE)
  preferred_order <- c(
    "method", "alpha", "lwr.ci", "upr.ci", "se", "statistic", "p_value",
    "observed_disagreement", "expected_disagreement", "n_items",
    "n_categories", "n_ratings_total", "n_raters_min", "n_raters_max",
    "balanced", "level", "estimator"
  )
  out$se_method <- .mc_coalesce(attr(object, "se.method", exact = TRUE), "none")
  out <- out[, c(intersect(preferred_order, names(out)), "se_method", setdiff(names(out), c(preferred_order, "se_method"))), drop = FALSE]
  drop_if_all_na <- c("lwr.ci", "upr.ci", "se", "statistic", "p_value", "conf.level")
  keep_cols <- vapply(names(out), function(nm) {
    if (!nm %in% drop_if_all_na) {
      return(TRUE)
    }
    !all(is.na(out[[nm]]))
  }, logical(1))
  out <- out[, keep_cols, drop = FALSE]
  num_cols <- vapply(out, is.numeric, logical(1))
  out[num_cols] <- lapply(out[num_cols], round, digits = digits)
  out <- .mc_finalize_summary_df(out, class_name = "summary.krippendorff_alpha")
  class(out) <- unique(c("summary.krippendorff_alpha", "summary.agreement_result", class(out)))
  attr(out, "digits") <- digits
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE)
  attr(out, "se.method") <- attr(object, "se.method", exact = TRUE)
  attr(out, "estimator") <- attr(object, "estimator", exact = TRUE)
  attr(out, "level") <- attr(object, "level", exact = TRUE)
  out
}

#' @method print summary.krippendorff_alpha
#' @rdname krippendorff_alpha
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.summary.krippendorff_alpha <- function(x,
                                             digits = NULL,
                                             n = NULL,
                                             topn = NULL,
                                             max_vars = NULL,
                                             width = NULL,
                                             show_ci = NULL,
                                             ...) {
  digits <- .mc_coalesce(digits, .mc_coalesce(attr(x, "digits", exact = TRUE), 4L))
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  digest <- c(
    estimator = attr(x, "estimator", exact = TRUE),
    level = attr(x, "level", exact = TRUE),
    alpha = formatC(x$alpha[[1L]], format = "f", digits = digits),
    items = .mc_count_fmt(x$n_items[[1L]]),
    categories = .mc_count_fmt(x$n_categories[[1L]])
  )
  se_method <- .mc_coalesce(attr(x, "se.method", exact = TRUE), "none")
  if (!identical(se_method, "none")) {
    digest <- c(digest, se_method = se_method)
  }
  .mc_print_named_digest(
    digest,
    header = .mc_header_with_ci(
      "Krippendorff's alpha summary",
      suppressWarnings(as.numeric(.mc_coalesce(attr(x, "conf.level", exact = TRUE), NA_real_))),
      show_ci = show_ci
    )
  )
  cat("\n")
  .mc_print_summary_table(
    x,
    header = "Estimate table",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @method plot krippendorff_alpha
#' @rdname krippendorff_alpha
#' @param type Plot type: \code{"estimate"}, \code{"item_disagreement"},
#'   \code{"category_proportion"}, or \code{"coincidence"}.
#' @export
plot.krippendorff_alpha <- function(x,
                                    type = c("estimate", "item_disagreement", "category_proportion", "coincidence"),
                                    ...) {
  check_inherits(x, "krippendorff_alpha")
  type <- match.arg(type)
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  diag_attr <- attr(x, "diagnostics", exact = TRUE)
  if (identical(type, "estimate")) {
    df <- data.frame(
      label = "Krippendorff's alpha",
      x = x$alpha[[1L]],
      lwr = x$lwr.ci[[1L]],
      upr = x$upr.ci[[1L]],
      stringsAsFactors = FALSE
    )
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$x, y = .data$label)) +
      ggplot2::geom_vline(xintercept = 0, linewidth = 0.5, color = "grey80") +
      ggplot2::geom_vline(xintercept = 1, linewidth = 0.4, color = "grey90", linetype = "dashed") +
      ggplot2::geom_vline(xintercept = -1, linewidth = 0.4, color = "grey90", linetype = "dashed")
    if (is.finite(df$lwr[[1L]]) && is.finite(df$upr[[1L]])) {
      p <- p + ggplot2::geom_segment(
        ggplot2::aes(x = .data$lwr, xend = .data$upr, y = .data$label, yend = .data$label),
        linewidth = 1.2,
        color = "#5B7C99"
      )
    }
    return(
      p +
        ggplot2::geom_point(size = 3.2, color = "#1F4E79") +
        ggplot2::coord_cartesian(xlim = c(-1, 1)) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(
          panel.grid.major.y = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          axis.title.y = ggplot2::element_blank(),
          ...
        ) +
        ggplot2::labs(
          title = "Krippendorff's alpha estimate",
          subtitle = sprintf(
            "%s estimator; %s level",
            attr(x, "estimator", exact = TRUE),
            attr(x, "level", exact = TRUE)
          ),
          x = "Alpha"
        )
    )
  }

  if (identical(type, "item_disagreement")) {
        item_dis <- as.numeric(.mc_coalesce(diag_attr$item_disagreement, numeric()))
    if (!length(item_dis) || !any(is.finite(item_dis))) {
      abort_bad_arg("type", message = "cannot be {.val item_disagreement} because item-level disagreement diagnostics are unavailable.")
    }
    df <- data.frame(
      item = seq_along(item_dis),
      item_disagreement = sort(item_dis),
      stringsAsFactors = FALSE
    )
    return(
      ggplot2::ggplot(df, ggplot2::aes(x = .data$item, y = .data$item_disagreement)) +
        ggplot2::geom_hline(
          yintercept = mean(item_dis, na.rm = TRUE),
          linewidth = 0.6,
          linetype = "dashed",
          color = "#B04A5A"
        ) +
        ggplot2::geom_linerange(
          ggplot2::aes(ymin = 0, ymax = .data$item_disagreement),
          linewidth = 0.8,
          color = "#7AA6C2"
        ) +
        ggplot2::geom_point(size = 2.3, color = "#1F4E79") +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
        ggplot2::labs(
          title = "Item-level disagreement profile",
          subtitle = "Dashed line shows the mean item-level disagreement",
          x = "Items ordered by disagreement",
          y = "Disagreement"
        )
    )
  }

  if (identical(type, "category_proportion")) {
    props <- as.numeric(.mc_coalesce(diag_attr$category_proportions, numeric()))
    cats <- as.character(attr(x, "categories", exact = TRUE))
    df <- data.frame(
      category = factor(cats, levels = rev(cats)),
      proportion = props,
      stringsAsFactors = FALSE
    )
    return(
      ggplot2::ggplot(df, ggplot2::aes(y = .data$category, x = .data$proportion)) +
        ggplot2::geom_segment(
          ggplot2::aes(x = 0, xend = .data$proportion, yend = .data$category),
          linewidth = 1,
          color = "#C5A46D"
        ) +
        ggplot2::geom_point(size = 3, color = "#8C5A2B") +
        ggplot2::coord_cartesian(xlim = c(0, 1)) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
        ggplot2::labs(
          title = "Marginal category proportions",
          x = "Proportion",
          y = "Category"
        )
    )
  }

  mats <- attr(x, "matrices", exact = TRUE)
  if (is.null(mats) || !is.matrix(mats$coincidence)) {
    abort_bad_arg(
      "type",
      message = "cannot be {.val coincidence} unless {.arg return_matrices} was requested in the original fit."
    )
  }
  df <- as.data.frame(as.table(mats$coincidence), stringsAsFactors = FALSE)
  names(df) <- c("category1", "category2", "value")
  ggplot2::ggplot(df, ggplot2::aes(x = .data$category2, y = .data$category1, fill = .data$value)) +
    ggplot2::geom_tile(color = "white", linewidth = 0.4) +
    ggplot2::geom_text(
      ggplot2::aes(label = formatC(.data$value, format = "f", digits = 1)),
      size = 3
    ) +
    ggplot2::scale_fill_gradient(low = "#F4EBD0", high = "#2E5EAA", name = "Coincidence") +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      axis.title = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::labs(title = "Observed coincidence matrix")
}

ratings_to_counts <- function(x, levels, min_raters = 2L) {
  x <- as.matrix(x)
  keep <- rowSums(!is.na(x)) >= min_raters
  x <- x[keep, , drop = FALSE]
  out <- matrix(0L, nrow = nrow(x), ncol = length(levels))
  colnames(out) <- as.character(levels)
  for (i in seq_len(nrow(x))) {
    xi <- x[i, ]
    xi <- xi[!is.na(xi)]
    out[i, ] <- tabulate(match(as.character(xi), as.character(levels)), nbins = ncol(out))
  }
  out
}

nominal_example <- function() {
  matrix(c(
    1, 2, 3, 3, 2, 1, 4, 1, 2, NA, NA, NA,
    1, 2, 3, 3, 2, 2, 4, 1, 2, 5, NA, 3,
    NA, 3, 3, 3, 2, 3, 4, 2, 2, 5, 1, NA,
    1, 2, 3, 3, 2, 4, 4, 1, 2, 5, 1, NA
  ), nrow = 12, ncol = 4)
}

interval_example <- function() {
  matrix(c(
    10, 10, 12, 11,
    20, 19, 21, 20,
    15, 15, NA, 16,
    30, 31, 29, 30,
    40, 39, 40, 41,
    50, 52, 51, 50,
    80, 78, 79, NA,
    100, 101, 99, 100
  ), byrow = TRUE, ncol = 4)
}

ratio_example <- function() {
  matrix(c(
    1, 1, 2, 1,
    2, 2, 2, 4,
    4, 4, 8, 4,
    8, 8, 8, 16,
    16, 16, 32, NA,
    32, 32, 32, 64,
    64, 64, 64, 64,
    128, 128, NA, 256
  ), byrow = TRUE, ncol = 4)
}

test_that("krippendorff_alpha customary nominal matches classic benchmark", {
  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available"
  )

  expect_s3_class(fit, "krippendorff_alpha")
  expect_equal(fit$alpha, 0.7434, tolerance = 5e-4)
  expect_equal(fit$n_items, 11L)
  expect_equal(fit$n_categories, 5L)
})

test_that("krippendorff_alpha smoke-test benchmarks match frozen estimate values", {
  nominal <- nominal_example()
  interval_dat <- interval_example()
  ratio_dat <- ratio_example()

  fit_nominal_customary <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available"
  )
  fit_nominal_analytical <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "analytical",
    na_method = "available",
    ci = TRUE,
    p_value = TRUE
  )
  fit_interval_customary <- krippendorff_alpha(
    interval_dat,
    level = "interval",
    method = "customary",
    na_method = "available"
  )
  fit_interval_analytical <- krippendorff_alpha(
    interval_dat,
    level = "interval",
    method = "analytical",
    na_method = "available",
    ci = TRUE,
    p_value = TRUE
  )
  fit_ratio_customary <- krippendorff_alpha(
    ratio_dat,
    level = "ratio",
    method = "customary",
    na_method = "available"
  )
  fit_ratio_analytical <- krippendorff_alpha(
    ratio_dat,
    level = "ratio",
    method = "analytical",
    na_method = "available",
    ci = TRUE,
    p_value = TRUE
  )

  expect_equal(as.numeric(fit_nominal_customary$alpha[[1L]]), 0.74342105, tolerance = 5e-7)
  expect_equal(as.numeric(fit_nominal_analytical$alpha[[1L]]), 0.75711893, tolerance = 5e-7)
  expect_equal(as.numeric(fit_interval_customary$alpha[[1L]]), 0.99919521, tolerance = 5e-7)
  expect_equal(as.numeric(fit_interval_analytical$alpha[[1L]]), 0.99927285, tolerance = 5e-7)
  expect_equal(as.numeric(fit_ratio_customary$alpha[[1L]]), 0.88950587, tolerance = 5e-7)
  expect_equal(as.numeric(fit_ratio_analytical$alpha[[1L]]), 0.89909781, tolerance = 5e-7)

  expect_equal(as.numeric(fit_nominal_analytical$lwr.ci[[1L]]), 0.23035062, tolerance = 5e-7)
  expect_equal(as.numeric(fit_nominal_analytical$upr.ci[[1L]]), 0.95178774, tolerance = 5e-7)
  expect_equal(as.numeric(fit_interval_analytical$lwr.ci[[1L]]), 0.99625147, tolerance = 5e-7)
  expect_equal(as.numeric(fit_interval_analytical$upr.ci[[1L]]), 0.99985920, tolerance = 5e-7)
  expect_equal(as.numeric(fit_ratio_analytical$lwr.ci[[1L]]), 0.82735250, tolerance = 5e-7)
  expect_equal(as.numeric(fit_ratio_analytical$upr.ci[[1L]]), 0.94246775, tolerance = 5e-7)
})

test_that("ratings input and counts input return identical customary nominal alpha", {
  nominal <- nominal_example()
  lev <- 1:5
  counts <- ratings_to_counts(nominal, levels = lev, min_raters = 2L)

  fit_ratings <- krippendorff_alpha(
    nominal,
    levels = lev,
    input = "ratings",
    level = "nominal",
    method = "customary",
    na_method = "available"
  )
  fit_counts <- krippendorff_alpha(
    counts,
    levels = lev,
    input = "counts",
    level = "nominal",
    method = "customary"
  )

  expect_equal(fit_ratings$alpha, fit_counts$alpha, tolerance = 1e-12)
  expect_equal(fit_ratings$observed_disagreement, fit_counts$observed_disagreement, tolerance = 1e-12)
  expect_equal(fit_ratings$expected_disagreement, fit_counts$expected_disagreement, tolerance = 1e-12)
})

test_that("na_method = error rejects missing ratings", {
  nominal <- nominal_example()
  expect_error(
    krippendorff_alpha(
      nominal,
      level = "nominal",
      method = "customary",
      na_method = "error"
    ),
    "missing ratings"
  )
})

test_that("na_method = complete drops incomplete items", {
  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "complete"
  )
  expect_equal(fit$n_items, sum(stats::complete.cases(nominal)))
})

test_that("na_method = available keeps rows with at least min_raters ratings", {
  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available",
    min_raters = 3L
  )
  expect_equal(fit$n_items, sum(rowSums(!is.na(nominal)) >= 3L))
})

test_that("explicit levels preserve unused categories", {
  x <- data.frame(
    r1 = factor(c("A", "A", "B", "C"), levels = c("A", "B", "C", "D")),
    r2 = factor(c("A", "B", "B", "C"), levels = c("A", "B", "C", "D")),
    r3 = factor(c("A", "A", "B", "C"), levels = c("A", "B", "C", "D"))
  )
  fit <- krippendorff_alpha(
    x,
    levels = c("A", "B", "C", "D"),
    level = "nominal",
    method = "customary",
    na_method = "available"
  )

  expect_equal(fit$n_categories, 4L)
  expect_identical(as.character(attr(fit, "categories", exact = TRUE)), c("A", "B", "C", "D"))
})

test_that("ordered factor + ordinal respects factor order", {
  lev <- c("low", "mid", "high")
  x <- data.frame(
    r1 = ordered(c("low", "mid", "high", "mid", "low"), levels = lev),
    r2 = ordered(c("low", "high", "high", "mid", "low"), levels = lev),
    r3 = ordered(c("mid", "mid", "high", "mid", "low"), levels = lev)
  )

  fit_factor <- krippendorff_alpha(
    x,
    level = "ordinal",
    method = "customary",
    na_method = "available"
  )
  fit_levels <- krippendorff_alpha(
    data.frame(lapply(x, as.character), stringsAsFactors = FALSE),
    levels = lev,
    level = "ordinal",
    method = "customary",
    na_method = "available"
  )

  expect_equal(fit_factor$alpha, fit_levels$alpha, tolerance = 1e-12)
})

test_that("character ordinal data without levels errors", {
  x <- data.frame(
    r1 = c("low", "mid", "high"),
    r2 = c("low", "high", "high"),
    r3 = c("mid", "mid", "high"),
    stringsAsFactors = FALSE
  )
  expect_error(
    krippendorff_alpha(
      x,
      level = "ordinal",
      method = "customary",
      na_method = "available"
    ),
    "must be supplied"
  )
})

test_that("interval alpha requires numeric categories", {
  x <- data.frame(
    r1 = c("low", "mid", "high"),
    r2 = c("low", "high", "high"),
    r3 = c("mid", "mid", "high"),
    stringsAsFactors = FALSE
  )
  expect_error(
    krippendorff_alpha(
      x,
      level = "interval",
      method = "customary",
      na_method = "available"
    ),
    "numeric"
  )
})

test_that("ratio alpha rejects negative categories", {
  x <- data.frame(
    r1 = c(-1, 0, 1, 2),
    r2 = c(-1, 1, 1, 2),
    r3 = c(0, 0, 1, 2)
  )
  expect_error(
    krippendorff_alpha(
      x,
      level = "ratio",
      method = "customary",
      na_method = "available"
    ),
    "negative"
  )
})

test_that("analytical estimator returns finite jackknife CI and p-value", {
  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "analytical",
    na_method = "available",
    ci = TRUE,
    p_value = TRUE
  )

  expect_true(is.finite(fit$alpha))
  expect_true(is.finite(fit$lwr.ci))
  expect_true(is.finite(fit$upr.ci))
  expect_true(is.finite(fit$se))
  expect_true(is.finite(fit$statistic))
  expect_true(is.finite(fit$p_value))
  expect_identical(attr(fit, "se.method", exact = TRUE), "jackknife")
})

test_that("customary alpha rejects p_value = TRUE", {
  nominal <- nominal_example()
  expect_error(
    krippendorff_alpha(
      nominal,
      level = "nominal",
      method = "customary",
      na_method = "available",
      p_value = TRUE
    ),
    "P-values are available only"
  )
})

test_that("customary bootstrap CI is reproducible with fixed seed", {
  nominal <- nominal_example()
  fit1 <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available",
    ci = TRUE,
    n_boot = 100L,
    seed = 11L
  )
  fit2 <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available",
    ci = TRUE,
    n_boot = 100L,
    seed = 11L
  )

  expect_equal(fit1$lwr.ci, fit2$lwr.ci, tolerance = 0)
  expect_equal(fit1$upr.ci, fit2$upr.ci, tolerance = 0)
  expect_equal(fit1$se, fit2$se, tolerance = 0)
  expect_identical(attr(fit1, "se.method", exact = TRUE), "bootstrap")
})

test_that("return_matrices attaches counts, coincidence, expected coincidence and delta2", {
  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available",
    return_matrices = TRUE
  )
  mats <- attr(fit, "matrices", exact = TRUE)

  expect_true(is.list(mats))
  expect_true(all(c("counts", "coincidence", "expected_coincidence", "delta2") %in% names(mats)))
  expect_true(is.matrix(mats$counts))
  expect_true(is.matrix(mats$coincidence))
  expect_true(is.matrix(mats$expected_coincidence))
  expect_true(is.matrix(mats$delta2))
  expect_equal(ncol(mats$counts), fit$n_categories)
})

test_that("print, summary, and plot methods do not error", {
  skip_if_not_installed("ggplot2")

  nominal <- nominal_example()
  fit <- krippendorff_alpha(
    nominal,
    level = "nominal",
    method = "customary",
    na_method = "available",
    ci = TRUE,
    n_boot = 50L,
    seed = 1L,
    return_matrices = TRUE
  )

  expect_no_error(capture.output(print(fit)))
  sm <- summary(fit)
  expect_s3_class(sm, "summary.krippendorff_alpha")
  expect_no_error(capture.output(print(sm)))
  expect_s3_class(plot(fit, type = "estimate"), "ggplot")
  expect_s3_class(plot(fit, type = "item_disagreement"), "ggplot")
  expect_s3_class(plot(fit, type = "category_proportion"), "ggplot")
  expect_s3_class(plot(fit, type = "coincidence"), "ggplot")
})

#' Access MatrixCorr Estimates, Confidence Intervals, and Tidy Results
#'
#' @description
#' Lightweight accessors for matrixCorr result objects. These functions do not
#' change the stored object structure; they just provide a stable way to extract the
#' estimate matrix, confidence intervals, or a pairwise data frame without
#' reading attributes directly.
#'
#' @param x,object A matrixCorr result, summary object, or scalar result.
#' @param ... Additional arguments passed to methods.
#' @param diag Logical; include diagonal entries for matrix-style results.
#'   Default is \code{FALSE}.
#' @param triangle For matrix-style correlation results, which triangle to
#'   return from \code{tidy()}: \code{"upper"} (default), \code{"lower"}, or
#'   \code{"full"}. The default avoids duplicate pairs for symmetric matrices;
#'   directed matrices use all off-diagonal entries unless \code{triangle} is
#'   supplied explicitly.
#' @param parm Ignored; included for compatibility with \code{stats::confint()}.
#' @param level Confidence level requested by \code{stats::confint()}. MatrixCorr
#'   objects store pre-computed intervals; if \code{level} differs from the
#'   stored level, recompute the original object with the desired level.
#'
#' @return
#' \code{estimate()} returns the primary estimate in its natural shape: a
#' matrix-like result returns a matrix, an edge-list result returns a data frame,
#' and a scalar result returns a numeric value.
#'
#' \code{tidy()} returns a data frame with columns such as \code{item1},
#' \code{item2}, \code{estimate}, \code{lwr}, \code{upr}, diagnostics, and
#' inferential quantities when available.
#'
#' \code{confint()} returns a data frame containing confidence limits when they
#' are available.
#'
#' \code{ci()} returns the stored confidence-interval payload. It is mainly a
#' structured alternative to reading \code{attr(x, "ci")} directly.
#'
#' @examples
#' X <- cbind(a = 1:10, b = 1:10 + rnorm(10), c = rnorm(10))
#' fit <- pearson_corr(X, ci = TRUE)
#'
#' estimate(fit)
#' tidy(fit)
#' ci(fit)
#' confint(fit)
#'
#' @author Thiago de Paula Oliveira
#' @export
estimate <- function(x, ...) {
  UseMethod("estimate")
}

#' @importFrom generics tidy
#' @rdname estimate
#' @export
generics::tidy

#' @rdname estimate
#' @export
ci <- function(x, ...) {
  UseMethod("ci")
}

#' @rdname estimate
#' @export
estimate.corr_result <- function(x, ...) {
  if (inherits(x, "corr_edge_list")) {
    out <- .mc_corr_as_edge_df(x)
    names(out)[names(out) == "row"] <- "item1"
    names(out)[names(out) == "col"] <- "item2"
    names(out)[names(out) == "value"] <- "estimate"
    out <- out[, intersect(c("item1", "item2", "estimate"), names(out)), drop = FALSE]
    rownames(out) <- NULL
    return(out)
  }
  .mc_plain_numeric_matrix(.mc_corr_as_dense_matrix(x))
}

#' @rdname estimate
#' @export
estimate.summary.corr_result <- function(x, ...) {
  df <- .mc_tidy_summary_df(x)
  if ("estimate" %in% names(df)) {
    return(df$estimate)
  }
  abort_bad_arg("x", message = "does not contain an `estimate` column.")
}

#' @rdname estimate
#' @export
estimate.summary.matrixCorr <- estimate.summary.corr_result

#' @rdname estimate
#' @export
coef.corr_result <- function(object, ...) {
  estimate(object, ...)
}

#' @rdname estimate
#' @export
ci.corr_result <- function(x, ...) {
  out <- attr(x, "ci", exact = TRUE)
  if (is.null(out)) {
    cli::cli_abort("No confidence intervals are available for this object.")
  }
  out
}

#' @rdname estimate
#' @export
ci.default <- function(x, ...) {
  .mc_ci_payload(x)
}

#' @rdname estimate
#' @export
ci.ba <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ba_matrix <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ba_repeated <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ba_repeated_matrix <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ccc <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ccc_ci <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.ccc_glmm <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.cia <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.cia_ci <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.cia_rm <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.cohen_kappa <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.gwet_ac <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.icc <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.icc_overall <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.icc_rm_reml <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.krippendorff_alpha <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.multirater_kappa <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.partial_corr <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.prob_agree <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.rmcorr <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.rmcorr_matrix <- function(x, ...) .mc_ci_payload(x)

#' @rdname estimate
#' @export
ci.weighted_kappa <- function(x, ...) .mc_ci_payload(x)

.mc_ci_payload <- function(x) {
  if (inherits(x, "ccc_glmm")) {
    return(.mc_ccc_glmm_ci_payload(x))
  }
  out <- attr(x, "ci", exact = TRUE)
  if (!is.null(out) && !is.logical(out)) {
    return(out)
  }
  if (inherits(x, c("ba", "ba_repeated"))) {
    return(.mc_ba_lines_ci_payload(x))
  }
  if (inherits(x, c("ba_matrix", "ba_repeated_matrix"))) {
    return(.mc_ba_matrix_ci_payload(x))
  }
  if (.mc_is_matrixcorr_like(x) && is.list(x)) {
    out <- x[["ci", exact = TRUE]]
    if (!is.null(out)) {
      return(out)
    }
    lwr <- x[["lwr", exact = TRUE]] %||% x[["lwr.ci", exact = TRUE]]
    upr <- x[["upr", exact = TRUE]] %||% x[["upr.ci", exact = TRUE]]
    if (!is.null(lwr) && !is.null(upr)) {
      return(list(
        lwr = lwr,
        upr = upr,
        conf.level = x[["conf.level", exact = TRUE]] %||%
          x[["conf_level", exact = TRUE]] %||%
          attr(x, "conf.level", exact = TRUE),
        ci.method = x[["ci.method", exact = TRUE]] %||%
          x[["ci_method", exact = TRUE]] %||%
          attr(x, "ci.method", exact = TRUE)
      ))
    }
  }
  sm <- tryCatch(summary(x), error = function(e) NULL)
  if (is.data.frame(sm) && all(c("lwr", "upr") %in% names(sm))) {
    return(list(
      lwr = sm$lwr,
      upr = sm$upr,
      conf.level = attr(sm, "conf.level", exact = TRUE),
      ci.method = attr(sm, "ci.method", exact = TRUE)
    ))
  }
  cli::cli_abort("No confidence intervals are available for this object.")
}

#' @rdname estimate
#' @export
estimate.partial_corr <- function(x, ...) {
  if (!is.null(x$pcor)) {
    return(as.matrix(x$pcor))
  }
  estimate.default(x, ...)
}

#' @rdname estimate
#' @export
estimate.ba <- function(x, ...) .mc_ba_estimate_columns(tidy(x, ...))

#' @rdname estimate
#' @export
estimate.ba_matrix <- estimate.ba

#' @rdname estimate
#' @export
estimate.ba_repeated <- estimate.ba

#' @rdname estimate
#' @export
estimate.ba_repeated_matrix <- estimate.ba

#' @rdname estimate
#' @export
estimate.ccc_glmm <- function(x, ...) .mc_plain_numeric_matrix(x)

#' @rdname estimate
#' @export
coef.ccc_glmm <- function(object, ...) estimate(object, ...)

#' @rdname estimate
#' @export
estimate.default <- function(x, ...) {
  if (is.data.frame(x) && .mc_is_matrixcorr_like(x)) {
    for (nm in c("estimate", "kappa", "alpha", "ac", "rho", "ccc", "cia", "prob_agree")) {
      if (nm %in% names(x)) {
        return(x[[nm]])
      }
    }
  }
  est <- if (is.list(x) && .mc_is_matrixcorr_like(x)) x[["est", exact = TRUE]] else NULL
  if (!is.null(est)) {
    if (is.matrix(est) || is.data.frame(est)) {
      return(.mc_plain_numeric_matrix(est))
    }
    return(as.numeric(est)[1L])
  }
  estimate_value <- if (is.list(x) && .mc_is_matrixcorr_like(x)) x[["estimate", exact = TRUE]] else NULL
  if (!is.null(estimate_value)) {
    return(estimate_value)
  }
  ac <- if (is.list(x) && .mc_is_matrixcorr_like(x)) x[["ac", exact = TRUE]] else NULL
  if (!is.null(ac)) {
    return(ac)
  }
  est_attr <- attr(x, "estimate", exact = TRUE)
  if (!is.null(est_attr)) {
    return(est_attr)
  }
  if (is.matrix(x) && .mc_is_matrixcorr_like(x)) {
    return(.mc_plain_numeric_matrix(x))
  }
  if (is.numeric(x) && length(x) >= 1L && .mc_is_matrixcorr_like(x)) {
    return(unname(as.numeric(x)))
  }
  sm <- tryCatch(summary(x), error = function(e) NULL)
  if (is.data.frame(sm) && .mc_is_matrixcorr_like(sm) && "estimate" %in% names(sm)) {
    return(sm$estimate)
  }
  abort_bad_arg("x", message = "does not expose a matrixCorr estimate.")
}

#' @rdname estimate
#' @export
tidy.corr_result <- function(x,
                             diag = FALSE,
                             triangle = c("upper", "lower", "full"),
                             ...) {
  check_bool(diag, arg = "diag")
  triangle_missing <- missing(triangle)
  triangle <- match.arg(triangle)
  out <- .mc_tidy_corr_result(
    x,
    diag = diag,
    triangle = triangle,
    triangle_missing = triangle_missing
  )
  rownames(out) <- NULL
  out
}

#' @rdname estimate
#' @export
tidy.summary.corr_result <- function(x, ...) {
  .mc_tidy_summary_df(x)
}

#' @rdname estimate
#' @export
tidy.summary.matrixCorr <- tidy.summary.corr_result

#' @rdname estimate
#' @export
tidy.partial_corr <- function(x,
                              diag = FALSE,
  triangle = c("upper", "lower", "full"),
                              ...) {
  if (!is.null(x$pcor)) {
    triangle <- match.arg(triangle)
    pcor_obj <- .mc_structure_corr_matrix(
      x$pcor,
      class_name = "partial_corr_matrix",
      method = x$method %||% attr(x, "method", exact = TRUE) %||% "partial_corr",
      description = "Partial correlation matrix",
      classes = c("partial_corr_matrix", "matrix"),
      extra_attrs = list(
        ci = x$ci %||% attr(x, "ci", exact = TRUE),
        diagnostics = x$diagnostics %||% attr(x, "diagnostics", exact = TRUE),
        inference = x$inference %||% attr(x, "inference", exact = TRUE),
        conf.level = x$conf.level %||% attr(x, "conf.level", exact = TRUE),
        ci.method = x$ci.method %||% attr(x, "ci.method", exact = TRUE)
      )
    )
    return(tidy.corr_result(pcor_obj, diag = diag, triangle = triangle, ...))
  }
  tidy.default(x, ...)
}

#' @rdname estimate
#' @export
tidy.ba <- function(x, ...) .mc_tidy_summary_with_conf_level(summary(x, ...))

#' @rdname estimate
#' @export
tidy.ba_matrix <- tidy.ba

#' @rdname estimate
#' @export
tidy.ba_repeated <- tidy.ba

#' @rdname estimate
#' @export
tidy.ba_repeated_matrix <- tidy.ba

#' @rdname estimate
#' @export
tidy.default <- function(x,
                         diag = FALSE,
                         triangle = c("upper", "lower", "full"),
                         ...) {
  check_bool(diag, arg = "diag")
  triangle <- match.arg(triangle)
  if (is.list(x) && .mc_is_matrixcorr_like(x) && !is.null(x[["est", exact = TRUE]])) {
    return(.mc_tidy_estimate_list(x, diag = diag, triangle = triangle))
  }
  if (is.list(x) && .mc_is_matrixcorr_like(x) && !is.null(x[["estimate", exact = TRUE]])) {
    return(.mc_tidy_named_estimate_list(x))
  }
  sm <- tryCatch(summary(x), error = function(e) NULL)
  if (is.data.frame(sm) && .mc_is_matrixcorr_like(sm)) {
    out <- .mc_tidy_summary_df(sm)
    if ("estimate" %in% names(out)) {
      return(out)
    }
  }
  if (is.numeric(x) && length(x) >= 1L && .mc_is_matrixcorr_like(x)) {
    return(.mc_tidy_scalar_result(x))
  }
  if (is.list(sm) && .mc_is_matrixcorr_like(sm) && !is.null(sm[["estimate", exact = TRUE]])) {
    return(.mc_tidy_named_estimate_list(sm))
  }
  abort_bad_arg("x", message = "does not expose a tidy matrixCorr representation.")
}

#' @rdname estimate
#' @export
confint.corr_result <- function(object, parm, level = NULL, ...) {
  .mc_confint_from_object(object, level = level, ...)
}

#' @rdname estimate
#' @export
confint.summary.corr_result <- function(object, parm, level = NULL, ...) {
  .mc_check_confint_level_value(attr(object, "conf.level", exact = TRUE), level = level)
  .mc_confint_from_tidy(tidy(object, ...), level = NULL)
}

#' @rdname estimate
#' @export
confint.summary.matrixCorr <- confint.summary.corr_result

#' @rdname estimate
#' @export
confint.ba <- function(object, parm, level = NULL, ...) .mc_confint_ba_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ba_repeated <- function(object, parm, level = NULL, ...) .mc_confint_ba_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ba_matrix <- function(object, parm, level = NULL, ...) .mc_confint_ba_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ba_repeated_matrix <- function(object, parm, level = NULL, ...) .mc_confint_ba_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ccc <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ccc_ci <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.ccc_glmm <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.chatterjee_xi_scalar <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.cia <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.cia_ci <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.cia_rm <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.cohen_kappa <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.gwet_ac <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.icc <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.icc_overall <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.icc_rm_reml <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.krippendorff_alpha <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.multirater_kappa <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.prob_agree <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.partial_corr <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.rmcorr <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.rmcorr_matrix <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

#' @rdname estimate
#' @export
confint.weighted_kappa <- function(object, parm, level = NULL, ...) .mc_confint_from_object(object, level = level, ...)

.mc_is_matrixcorr_like <- function(x) {
  cls <- class(x)
  isTRUE(attr(x, "corr_result", exact = TRUE)) ||
    identical(attr(x, "package", exact = TRUE), "matrixCorr") ||
    any(cls %in% c(
      "corr_result", "agreement_result", "matrixCorr_ccc_ci",
      "ba", "ba_matrix", "ba_repeated", "ba_repeated_matrix",
      "ccc", "ccc_ci", "ccc_glmm", "cia", "cia_ci", "cia_rm",
      "cohen_kappa", "weighted_kappa", "gwet_ac", "icc", "icc_overall",
      "icc_rm_reml", "krippendorff_alpha", "multirater_kappa",
      "partial_corr", "prob_agree", "rmcorr", "rmcorr_matrix",
      "chatterjee_xi_scalar", "summary.matrixCorr"
    ))
}

.mc_plain_numeric_matrix <- function(x) {
  x <- as.matrix(x)
  matrix(
    as.numeric(x),
    nrow = nrow(x),
    ncol = ncol(x),
    dimnames = dimnames(x)
  )
}

.mc_tidy_index_matrix <- function(mat, diag = FALSE, triangle = c("upper", "lower", "full")) {
  triangle <- match.arg(triangle)
  idx <- switch(
    triangle,
    upper = upper.tri(mat, diag = isTRUE(diag)),
    lower = lower.tri(mat, diag = isTRUE(diag)),
    full = matrix(TRUE, nrow = nrow(mat), ncol = ncol(mat))
  )
  if (!isTRUE(diag) && identical(triangle, "full")) {
    d <- seq_len(min(nrow(mat), ncol(mat)))
    idx[cbind(d, d)] <- FALSE
  }
  idx
}

.mc_complete_symmetric_matrix <- function(mat, symmetric = FALSE) {
  if (!is.matrix(mat) || !isTRUE(symmetric) || nrow(mat) != ncol(mat)) {
    return(mat)
  }
  out <- mat
  miss <- is.na(out)
  transpose <- t(out)
  out[miss & !is.na(transpose)] <- transpose[miss & !is.na(transpose)]
  out
}

.mc_standard_conf_level <- function(x) {
  out <- suppressWarnings(as.numeric(x)[1L])
  if (length(out) && is.finite(out)) out else NA_real_
}

.mc_check_confint_level <- function(df, level = NULL) {
  stored <- .mc_standard_conf_level(df[["conf.level", exact = TRUE]] %||% attr(df, "conf.level", exact = TRUE))
  .mc_check_confint_level_value(stored, level = level)
}

.mc_object_conf_level <- function(object) {
  if (is.list(object)) {
    object[["conf.level", exact = TRUE]] %||%
      object[["conf_level", exact = TRUE]] %||%
      attr(object, "conf.level", exact = TRUE)
  } else {
    attr(object, "conf.level", exact = TRUE)
  }
}

.mc_check_confint_level_value <- function(stored, level = NULL) {
  stored <- .mc_standard_conf_level(stored)
  if (!is.null(level)) {
    requested <- .mc_standard_conf_level(level)
    if (!is.finite(requested)) {
      abort_bad_arg("level", message = "must be a finite confidence level.")
    }
    if (is.finite(stored) && !isTRUE(all.equal(requested, stored, tolerance = sqrt(.Machine$double.eps)))) {
      cli::cli_abort(sprintf(
        "This object contains pre-computed confidence intervals at conf.level = %s. Recompute the correlation object to use a different level.",
        format(stored)
      ))
    }
  }
  invisible(stored)
}

.mc_confint_from_object <- function(object, level = NULL, ...) {
  .mc_check_confint_level_value(.mc_object_conf_level(object), level = level)
  .mc_confint_from_tidy(tidy(object, ...), level = NULL)
}

.mc_confint_ba_from_object <- function(object, level = NULL, ...) {
  .mc_check_confint_level_value(.mc_object_conf_level(object), level = level)
  .mc_confint_ba(tidy(object, ...), level = NULL)
}

.mc_clean_extractor_metadata <- function(df) {
  if (!is.data.frame(df)) {
    return(df)
  }

  conf_level <- df[["conf.level", exact = TRUE]] %||%
    df[["conf_level", exact = TRUE]] %||%
    attr(df, "conf.level", exact = TRUE)
  conf_level <- .mc_standard_conf_level(conf_level)
  df$conf.level <- NULL
  df$conf_level <- NULL
  attr(df, "conf.level") <- NULL

  df$ci.method <- NULL
  df$ci_method <- NULL
  attr(df, "ci.method") <- NULL
  attr(df, "ci_method") <- NULL

  if ("n_complete" %in% names(df)) {
    n_complete <- unique(stats::na.omit(df$n_complete))
    if (length(n_complete) <= 1L) {
      df$n_complete <- NULL
    }
  }

  attributes(df) <- attributes(df)[intersect(names(attributes(df)), c("names", "row.names", "class"))]
  df
}

.mc_tidy_corr_result <- function(x,
                                 diag = FALSE,
                                 triangle = c("upper", "lower", "full"),
                                 triangle_missing = FALSE) {
  triangle <- match.arg(triangle)
  mat <- .mc_plain_numeric_matrix(.mc_corr_as_dense_matrix(x))
  dm <- dim(mat)
  dn <- dimnames(mat)
  symmetric <- isTRUE(attr(x, "corr_symmetric", exact = TRUE)) &&
    length(dm) == 2L &&
    dm[[1L]] == dm[[2L]] &&
    identical(dn[[1L]], dn[[2L]])
  use_triangle <- if (isTRUE(symmetric)) triangle else if (isTRUE(triangle_missing)) "full" else triangle
  cross_matrix <- !identical(dn[[1L]], dn[[2L]]) && identical(use_triangle, "full")
  idx_mat <- .mc_tidy_index_matrix(
    mat,
    diag = isTRUE(diag) || isTRUE(cross_matrix),
    triangle = use_triangle
  )
  idx <- which(idx_mat, arr.ind = TRUE)
  rn <- dn[[1L]] %||% as.character(seq_len(dm[[1L]]))
  cn <- dn[[2L]] %||% as.character(seq_len(dm[[2L]]))
  out <- data.frame(
    item1 = rn[idx[, 1L]],
    item2 = cn[idx[, 2L]],
    estimate = as.numeric(mat[idx]),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  out <- out[is.finite(out$estimate), , drop = FALSE]
  idx <- list(i = idx[, 1L], j = idx[, 2L], ok = rep(TRUE, nrow(idx)))
  if (nrow(out) != length(idx$i)) {
    keep <- is.finite(as.numeric(mat[cbind(idx$i, idx$j)]))
    idx <- list(i = idx$i[keep], j = idx$j[keep], ok = rep(TRUE, sum(keep)))
  }

  ci <- attr(x, "ci", exact = TRUE)
  if (is.list(ci)) {
    lwr_mat <- .mc_complete_symmetric_matrix(ci$lwr.ci %||% ci$lwr, symmetric = symmetric)
    upr_mat <- .mc_complete_symmetric_matrix(ci$upr.ci %||% ci$upr, symmetric = symmetric)
    out$lwr <- .mc_extract_pair_matrix(lwr_mat, idx, dm)
    out$upr <- .mc_extract_pair_matrix(upr_mat, idx, dm)
    conf_level <- .mc_standard_conf_level(ci$conf.level %||% attr(x, "conf.level", exact = TRUE))
    if (is.finite(conf_level)) {
      attr(out, "conf.level") <- conf_level
    }
    ci_method <- ci$ci.method %||% ci$ci_method %||% attr(x, "ci.method", exact = TRUE) %||%
      attr(x, "ci_method", exact = TRUE)
    if (is.matrix(ci_method)) {
      out$ci.method <- .mc_extract_pair_matrix(
        .mc_complete_symmetric_matrix(ci_method, symmetric = symmetric),
        idx,
        dm,
        character = TRUE
      )
    } else if (!is.null(ci_method) && length(ci_method) >= 1L) {
      attr(out, "ci.method") <- as.character(ci_method[[1L]])
    }
  }

  diag_attr <- attr(x, "diagnostics", exact = TRUE)
  if (is.list(diag_attr)) {
    out$n_complete <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(diag_attr$n_complete, symmetric = symmetric),
      idx,
      dm,
      integer = TRUE
    )
    out$m <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(diag_attr$m, symmetric = symmetric),
      idx,
      dm,
      integer = TRUE
    )
    out$se <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(diag_attr$se, symmetric = symmetric),
      idx,
      dm
    )
    if (!is.null(diag_attr$bootstrap_reps)) {
      out$bootstrap_reps <- as.integer(diag_attr$bootstrap_reps[[1L]])
    }
  }

  inf <- attr(x, "inference", exact = TRUE)
  if (is.list(inf)) {
    out$statistic <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(inf$statistic, symmetric = symmetric),
      idx,
      dm
    )
    out$df <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(inf$parameter %||% inf$df, symmetric = symmetric),
      idx,
      dm
    )
    out$p_value <- .mc_extract_pair_matrix(
      .mc_complete_symmetric_matrix(inf$p_value, symmetric = symmetric),
      idx,
      dm
    )
  }

  out <- .mc_clean_extractor_metadata(.mc_drop_all_na_columns(out))
  rownames(out) <- NULL
  out
}

.mc_pair_indices <- function(df, dm, dn) {
  if (!all(c("item1", "item2") %in% names(df)) || length(dm) != 2L) {
    return(list(i = integer(), j = integer(), ok = logical()))
  }
  rn <- dn[[1L]]
  cn <- dn[[2L]]
  ii <- if (is.null(rn)) suppressWarnings(as.integer(df$item1)) else match(df$item1, rn)
  jj <- if (is.null(cn)) suppressWarnings(as.integer(df$item2)) else match(df$item2, cn)
  ok <- is.finite(ii) & is.finite(jj) &
    ii >= 1L & ii <= dm[[1L]] &
    jj >= 1L & jj <= dm[[2L]]
  list(i = as.integer(ii), j = as.integer(jj), ok = ok)
}

.mc_extract_pair_matrix <- function(mat,
                                    idx,
                                    dm,
                                    integer = FALSE,
                                    character = FALSE) {
  n <- length(idx$ok)
  if (isTRUE(character)) {
    out <- rep(NA_character_, n)
  } else if (isTRUE(integer)) {
    out <- rep(NA_integer_, n)
  } else {
    out <- rep(NA_real_, n)
  }
  if (!is.matrix(mat) || !identical(dim(mat), dm) || !n) {
    return(out)
  }
  out[idx$ok] <- mat[cbind(idx$i[idx$ok], idx$j[idx$ok])]
  if (isTRUE(integer)) {
    return(as.integer(round(out)))
  }
  if (isTRUE(character)) {
    return(as.character(out))
  }
  as.numeric(out)
}

.mc_tidy_summary_df <- function(x) {
  conf_level <- attr(x, "conf.level", exact = TRUE)
  out <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  if ("row" %in% names(out) && !"item1" %in% names(out)) names(out)[names(out) == "row"] <- "item1"
  if ("col" %in% names(out) && !"item2" %in% names(out)) names(out)[names(out) == "col"] <- "item2"
  if ("method1" %in% names(out) && !"item1" %in% names(out)) names(out)[names(out) == "method1"] <- "item1"
  if ("method2" %in% names(out) && !"item2" %in% names(out)) names(out)[names(out) == "method2"] <- "item2"
  if ("value" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "value"] <- "estimate"
  if ("ac" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "ac"] <- "estimate"
  if ("kappa" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "kappa"] <- "estimate"
  if ("alpha" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "alpha"] <- "estimate"
  if ("rho" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "rho"] <- "estimate"
  if ("ccc" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "ccc"] <- "estimate"
  if ("cia" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "cia"] <- "estimate"
  if ("prob_agree" %in% names(out) && !"estimate" %in% names(out)) names(out)[names(out) == "prob_agree"] <- "estimate"
  if ("lwr.ci" %in% names(out) && !"lwr" %in% names(out)) names(out)[names(out) == "lwr.ci"] <- "lwr"
  if ("upr.ci" %in% names(out) && !"upr" %in% names(out)) names(out)[names(out) == "upr.ci"] <- "upr"
  if ("conf_level" %in% names(out) && !"conf.level" %in% names(out)) names(out)[names(out) == "conf_level"] <- "conf.level"
  if ("ci_method" %in% names(out) && !"ci.method" %in% names(out)) names(out)[names(out) == "ci_method"] <- "ci.method"
  if ("conf_level" %in% names(out) && "conf.level" %in% names(out)) out$conf_level <- NULL
  if ("ci_method" %in% names(out) && "ci.method" %in% names(out)) out$ci_method <- NULL
  rownames(out) <- NULL
  attr(out, "conf.level") <- conf_level
  .mc_clean_extractor_metadata(out)
}

.mc_tidy_summary_with_conf_level <- function(x) {
  out <- .mc_tidy_summary_df(x)
  conf_level <- suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE)))
  if (length(conf_level) && is.finite(conf_level[1L])) {
    attr(out, "conf.level") <- conf_level[1L]
  }
  .mc_clean_extractor_metadata(out)
}

.mc_ba_estimate_columns <- function(df) {
  keep <- intersect(
    c(
      "item1", "item2", "n_obs", "bias", "sd_loa", "loa_low", "loa_up",
      "width", "loa_multiplier", "sigma2_subject", "sigma2_resid",
      "residual_model", "ar1_rho"
    ),
    names(df)
  )
  if (!length(keep)) {
    abort_bad_arg("x", message = "does not contain Bland-Altman estimates.")
  }
  out <- df[, keep, drop = FALSE]
  rownames(out) <- NULL
  .mc_drop_all_na_columns(out)
}

.mc_confint_ba <- function(df, level = NULL) {
  df <- .mc_tidy_summary_df(df)
  keep <- intersect(
    c("item1", "item2", "bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr"),
    names(df)
  )
  if (!all(c("bias_lwr", "bias_upr", "lo_lwr", "lo_upr", "up_lwr", "up_upr") %in% names(df))) {
    abort_bad_arg("object", message = "does not contain Bland-Altman confidence limits.")
  }
  .mc_check_confint_level(df, level = level)
  out <- df[, keep, drop = FALSE]
  rownames(out) <- NULL
  attr(out, "conf.level") <- attr(df, "conf.level", exact = TRUE)
  out <- .mc_clean_extractor_metadata(out)
  attr(out, "conf.level") <- NULL
  out
}

.mc_ba_lines_ci_payload <- function(x) {
  ci_lines <- x[["CI.lines", exact = TRUE]]
  if (is.null(ci_lines)) {
    ci_lines <- x[["ci", exact = TRUE]]
  }
  if (is.null(ci_lines)) {
    cli::cli_abort("No confidence intervals are available for this object.")
  }
  get <- function(nm) suppressWarnings(as.numeric(ci_lines[[nm]]))[1L]
  out <- list(
    bias = c(lwr = get("mean.diff.ci.lower"), upr = get("mean.diff.ci.upper")),
    loa_low = c(lwr = get("lower.limit.ci.lower"), upr = get("lower.limit.ci.upper")),
    loa_up = c(lwr = get("upper.limit.ci.lower"), upr = get("upper.limit.ci.upper")),
    conf.level = attr(x, "conf.level", exact = TRUE),
    ci.method = attr(x, "ci.method", exact = TRUE)
  )
  if (all(!is.finite(unlist(out[1:3], use.names = FALSE)))) {
    cli::cli_abort("No confidence intervals are available for this object.")
  }
  out
}

.mc_ba_matrix_ci_payload <- function(x) {
  fields <- list(
    bias = c("mean_ci_low", "mean_ci_high"),
    loa_low = c("loa_lower_ci_low", "loa_lower_ci_high"),
    loa_up = c("loa_upper_ci_low", "loa_upper_ci_high")
  )
  if (!all(unlist(fields, use.names = FALSE) %in% names(x))) {
    cli::cli_abort("No confidence intervals are available for this object.")
  }
  out <- lapply(fields, function(nm) {
    list(lwr = x[[nm[[1L]], exact = TRUE]], upr = x[[nm[[2L]], exact = TRUE]])
  })
  out$conf.level <- attr(x, "conf.level", exact = TRUE)
  out$ci.method <- attr(x, "ci.method", exact = TRUE)
  out
}

.mc_ccc_glmm_ci_payload <- function(x) {
  lwr <- attr(x, "rho_ccc_lwr.ci", exact = TRUE)
  upr <- attr(x, "rho_ccc_upr.ci", exact = TRUE)
  se <- attr(x, "rho_ccc_se", exact = TRUE)
  if (is.null(lwr) || is.null(upr)) {
    cli::cli_abort("No confidence intervals are available for this object.")
  }

  out <- list(
    lwr = lwr,
    upr = upr,
    se = se,
    total = list(lwr = lwr, upr = upr, se = se),
    inter = list(
      lwr = attr(x, "rho_ccc_inter_lwr.ci", exact = TRUE),
      upr = attr(x, "rho_ccc_inter_upr.ci", exact = TRUE),
      se = attr(x, "rho_ccc_inter_se", exact = TRUE)
    ),
    intra_method1 = list(
      lwr = attr(x, "rho_ccc_intra_method1_lwr.ci", exact = TRUE),
      upr = attr(x, "rho_ccc_intra_method1_upr.ci", exact = TRUE),
      se = attr(x, "rho_ccc_intra_method1_se", exact = TRUE)
    ),
    intra_method2 = list(
      lwr = attr(x, "rho_ccc_intra_method2_lwr.ci", exact = TRUE),
      upr = attr(x, "rho_ccc_intra_method2_upr.ci", exact = TRUE),
      se = attr(x, "rho_ccc_intra_method2_se", exact = TRUE)
    )
  )
  conf_level <- attr(x, "conf.level", exact = TRUE) %||% attr(x, "conf_level", exact = TRUE)
  if (!is.null(conf_level)) {
    out$conf.level <- conf_level
  }
  ci_method <- attr(x, "ci.method", exact = TRUE) %||% attr(x, "ci_method", exact = TRUE)
  if (!is.null(ci_method)) {
    out$ci.method <- ci_method
  }
  out
}

.mc_tidy_estimate_list <- function(x,
                                   diag = FALSE,
                                   triangle = c("upper", "lower", "full")) {
  triangle <- match.arg(triangle)
  est <- x[["est", exact = TRUE]]
  if (is.matrix(est)) {
    out <- .mc_matrix_estimate_table(est, diag = diag, triangle = triangle)
    if (is.matrix(x$lwr.ci)) out$lwr <- x$lwr.ci[cbind(out$.i, out$.j)]
    if (is.matrix(x$upr.ci)) out$upr <- x$upr.ci[cbind(out$.i, out$.j)]
    out$.i <- out$.j <- NULL
    attr(out, "conf.level") <- x[["conf.level", exact = TRUE]] %||% attr(x, "conf.level", exact = TRUE)
    return(.mc_clean_extractor_metadata(.mc_drop_all_na_columns(out)))
  }
  out <- data.frame(
    estimate = as.numeric(est)[1L],
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (!is.null(x$lwr.ci)) out$lwr <- as.numeric(x$lwr.ci)[1L]
  if (!is.null(x$upr.ci)) out$upr <- as.numeric(x$upr.ci)[1L]
  attr(out, "conf.level") <- x[["conf.level", exact = TRUE]] %||% attr(x, "conf.level", exact = TRUE)
  .mc_clean_extractor_metadata(.mc_drop_all_na_columns(out))
}

.mc_tidy_named_estimate_list <- function(x) {
  out <- data.frame(
    estimate = as.numeric(x[["estimate", exact = TRUE]])[1L],
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  for (nm in c(
    "lwr", "upr", "conf.level", "conf_level", "ci.method", "ci_method",
    "p_value", "statistic", "df", "slope", "n_obs", "n_subjects",
    "n_complete", "n_boot", "bootstrap_reps"
  )) {
    value <- x[[nm, exact = TRUE]]
    if (!is.null(value)) {
      out[[nm]] <- value[[1L]]
    }
  }
  .mc_clean_extractor_metadata(.mc_tidy_summary_df(.mc_drop_all_na_columns(out)))
}

.mc_matrix_estimate_table <- function(mat,
                                      diag = FALSE,
                                      triangle = c("upper", "lower", "full")) {
  triangle <- match.arg(triangle)
  mat <- as.matrix(mat)
  rn <- rownames(mat) %||% as.character(seq_len(nrow(mat)))
  cn <- colnames(mat) %||% as.character(seq_len(ncol(mat)))
  same_items <- identical(rn, cn) && nrow(mat) == ncol(mat)
  use_triangle <- if (same_items) triangle else "full"
  idx_mat <- .mc_tidy_index_matrix(
    mat,
    diag = isTRUE(diag) || !same_items,
    triangle = use_triangle
  )
  idx <- which(idx_mat, arr.ind = TRUE)
  data.frame(
    item1 = rn[idx[, 1L]],
    item2 = cn[idx[, 2L]],
    estimate = as.numeric(mat[idx]),
    .i = idx[, 1L],
    .j = idx[, 2L],
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

.mc_tidy_scalar_result <- function(x) {
  out <- data.frame(
    estimate = unname(as.numeric(x)[1L]),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  ci <- attr(x, "ci", exact = TRUE)
  if (is.list(ci)) {
    out$lwr <- as.numeric(ci$lwr %||% ci$lwr.ci %||% NA_real_)[1L]
    out$upr <- as.numeric(ci$upr %||% ci$upr.ci %||% NA_real_)[1L]
    out$conf.level <- as.numeric(ci$conf.level %||% attr(x, "conf.level", exact = TRUE) %||% NA_real_)[1L]
    out$se <- as.numeric(ci$se %||% NA_real_)[1L]
    out$m <- as.integer(ci$m %||% NA_integer_)[1L]
    out$bootstrap_reps <- as.integer(ci$bootstrap_reps %||% NA_integer_)[1L]
    out$n_complete <- as.integer(ci$n_complete %||% NA_integer_)[1L]
  }
  .mc_clean_extractor_metadata(.mc_drop_all_na_columns(out))
}

.mc_confint_from_tidy <- function(df, level = NULL) {
  df <- .mc_tidy_summary_df(df)
  if (!all(c("lwr", "upr") %in% names(df))) {
    abort_bad_arg("object", message = "does not contain confidence limits.")
  }
  .mc_check_confint_level(df, level = level)
  keep <- intersect(c("item1", "item2", "lwr", "upr"), names(df))
  out <- df[, keep, drop = FALSE]
  rownames(out) <- NULL
  attr(out, "conf.level") <- attr(df, "conf.level", exact = TRUE)
  out <- .mc_clean_extractor_metadata(out)
  attr(out, "conf.level") <- NULL
  out
}

.mc_drop_all_na_columns <- function(df) {
  if (!is.data.frame(df) || !ncol(df)) {
    return(df)
  }
  keep <- vapply(df, function(z) {
    if (is.list(z)) {
      return(TRUE)
    }
    !all(is.na(z))
  }, logical(1L))
  df[, keep, drop = FALSE]
}

dat_ref2 <- data.frame(
  subject = c(
    "s1", "s1", "s1", "s1",
    "s2", "s2", "s2", "s2"
  ),
  method = c(
    "R", "R", "A", "A",
    "R", "R", "A", "A"
  ),
  replicate = c(
    "r1", "r2", "r1", "r2",
    "r1", "r2", "r1", "r2"
  ),
  value = c(
    0, 2, 0, 0,
    10, 12, 10, 10
  )
)

dat_ref3 <- data.frame(
  subject = c(
    "s1", "s1", "s1", "s1", "s1", "s1",
    "s2", "s2", "s2", "s2", "s2", "s2"
  ),
  method = c(
    "R", "R", "A", "A", "B", "B",
    "R", "R", "A", "A", "B", "B"
  ),
  replicate = c(
    "r1", "r2", "r1", "r2", "r1", "r2",
    "r1", "r2", "r1", "r2", "r1", "r2"
  ),
  value = c(
    0, 2, 0, 0, 4, 4,
    10, 12, 10, 10, 14, 14
  )
)

dat_overall_bal <- data.frame(
  subject = c(
    rep("s1", 6),
    rep("s2", 6),
    rep("s3", 6)
  ),
  method = rep(c("R", "R", "A", "A", "B", "B"), times = 3),
  replicate = rep(c("r1", "r2"), times = 9),
  value = c(
    0, 2, 0, 0, 4, 4,
    10, 12, 10, 10, 14, 14,
    20, 24, 20, 20, 25, 25
  )
)

manual_overall_cia <- function(dat, reference = NULL, conf_level = 0.95) {
  subjects <- unique(dat$subject)
  methods <- unique(dat$method)
  n <- length(subjects)
  J <- length(methods)
  counts <- xtabs(~ subject + method, data = dat)
  stopifnot(length(unique(as.integer(counts))) == 1L)
  K <- unique(as.integer(counts))[[1L]]

  Ybar <- matrix(NA_real_, n, J, dimnames = list(subjects, methods))
  A <- Ybar
  for (i in seq_along(subjects)) {
    for (j in seq_along(methods)) {
      vals <- dat$value[dat$subject == subjects[[i]] & dat$method == methods[[j]]]
      mu <- mean(vals)
      Ybar[i, j] <- mu
      A[i, j] <- sum((vals - mu)^2) / (K - 1)
    }
  }

  if (is.null(reference)) {
    A_i <- rowMeans(A)
    Y_i_dot_dot <- rowMeans(Ybar)
    B_i <- rowSums((Ybar - Y_i_dot_dot)^2) / (J - 1) + (1 - 1 / K) * A_i
    A_bar <- mean(A_i)
    B_bar <- mean(B_i)
    psi <- A_bar / B_bar
    var_A <- stats::var(A_i) / n
    var_B <- stats::var(B_i) / n
    cov_A_B <- stats::cov(A_i, B_i) / n
    var_psi <-
      (1 / (B_bar^2)) * var_A +
      (A_bar^2 / B_bar^4) * var_B -
      (2 * A_bar / B_bar^3) * cov_A_B
    numerator_term <- A_bar
    denominator_term <- B_bar
  } else {
    ref_idx <- match(reference, methods)
    nonref_idx <- setdiff(seq_len(J), ref_idx)
    A_i <- A[, ref_idx]
    B_i <- rowSums((Ybar[, nonref_idx, drop = FALSE] - Ybar[, ref_idx])^2) / (J - 1) +
      (1 - 1 / K) * rowSums(A[, nonref_idx, drop = FALSE]) / (J - 1) +
      (1 - 1 / K) * A[, ref_idx]
    A_bar <- mean(A_i)
    B_bar <- mean(B_i)
    psi <- 2 * A_bar / B_bar
    var_A <- stats::var(A_i) / n
    var_B <- stats::var(B_i) / n
    cov_A_B <- stats::cov(A_i, B_i) / n
    var_ratio <-
      (1 / (B_bar^2)) * var_A +
      (A_bar^2 / B_bar^4) * var_B -
      (2 * A_bar / B_bar^3) * cov_A_B
    var_psi <- 4 * var_ratio
    numerator_term <- 2 * A_bar
    denominator_term <- B_bar
  }

  z <- stats::qnorm(1 - (1 - conf_level) / 2)
  se <- sqrt(max(var_psi, 0))
  list(
    psi = psi,
    lwr = psi - z * se,
    upr = psi + z * se,
    se = se,
    A_i = A_i,
    B_i = B_i,
    A_bar = mean(A_i),
    B_bar = mean(B_i),
    numerator_term = numerator_term,
    denominator_term = denominator_term
  )
}

test_that("pairwise reference CIA uses the unhalved reference replicate disagreement", {
  fit <- suppressWarnings(cia(
    dat_ref2,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  mat <- as.matrix(fit)
  diag_attr <- attr(fit, "diagnostics", exact = TRUE)

  expect_equal(mat["A", "R"], 2, tolerance = 1e-12)
  expect_equal(unname(diag_attr$within_msd["R"]), 4, tolerance = 1e-12)
  expect_equal(unname(diag_attr$sigma_within["R"]), 2, tolerance = 1e-12)
  expect_equal(unname(diag_attr$between_msd["A", "R"]), 2, tolerance = 1e-12)
})

test_that("overall reference CIA uses 2 * A_bar / B_bar", {
  manual <- manual_overall_cia(dat_overall_bal, reference = "R")
  fit <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "overall",
    estimator = "mom_unconstrained"
  )

  expect_equal(fit$cia[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(fit$numerator_term[[1]], manual$numerator_term, tolerance = 1e-12)
  expect_equal(fit$denominator_term[[1]], manual$denominator_term, tolerance = 1e-12)
  expect_equal(fit$within_msd[[1]], manual$numerator_term, tolerance = 1e-12)
  expect_equal(fit$between_msd[[1]], manual$denominator_term, tolerance = 1e-12)
})

test_that("overall no-reference CIA uses mean(A_i_dot) / mean(B_N_i)", {
  manual <- manual_overall_cia(dat_overall_bal, reference = NULL)
  fit <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "overall",
    estimator = "mom_unconstrained"
  )

  expect_equal(fit$cia[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(fit$numerator_term[[1]], manual$numerator_term, tolerance = 1e-12)
  expect_equal(fit$denominator_term[[1]], manual$denominator_term, tolerance = 1e-12)
})

test_that("pairwise reference CIA is correctly scaled for all non-reference methods", {
  fit <- suppressWarnings(cia(
    dat_ref3,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  mat <- as.matrix(fit)

  expect_equal(mat["A", "R"], 2, tolerance = 1e-12)
  expect_equal(mat["B", "R"], 0.4, tolerance = 1e-12)
})

test_that("pairwise no-reference CIA uses the two-method subject-level estimator", {
  fit <- suppressWarnings(cia(
    dat_ref2,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = NULL,
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  mat <- as.matrix(fit)

  expect_equal(mat["A", "R"], 1, tolerance = 1e-12)
})

test_that("C++ pairwise reference estimate uses the unhalved reference numerator", {
  cpp_fun <- get("cia_pairwise_stats_cpp", envir = asNamespace("matrixCorr"))

  y <- c(
    0, 2, 0, 0, 4, 4,
    10, 12, 10, 10, 14, 14
  )
  subject_code <- c(
    1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2
  )
  method_code <- c(
    1, 1, 2, 2, 3, 3,
    1, 1, 2, 2, 3, 3
  )
  replicate_code <- c(
    1, 2, 1, 2, 1, 2,
    1, 2, 1, 2, 1, 2
  )

  raw <- cpp_fun(
    y = y,
    subject = subject_code,
    method = method_code,
    replicate = replicate_code,
    n_methods = 3L,
    reference_method = 1L,
    has_reference = TRUE,
    n_threads = 1L
  )

  expect_equal(raw$A_bar[2, 1], 4, tolerance = 1e-12)
  expect_equal(raw$B_bar[2, 1], 2, tolerance = 1e-12)
  expect_equal(raw$B_bar[3, 1], 10, tolerance = 1e-12)
  expect_equal(raw$estimate_raw[2, 1], 2, tolerance = 1e-12)
  expect_equal(raw$estimate_raw[3, 1], 0.4, tolerance = 1e-12)
})

test_that("C++ overall no-reference estimator matches the balanced moment formula", {
  cpp_fun <- get("cia_overall_balanced_cpp", envir = asNamespace("matrixCorr"))
  y <- dat_overall_bal$value
  subject_code <- as.integer(factor(dat_overall_bal$subject))
  method_code <- as.integer(factor(dat_overall_bal$method, levels = c("R", "A", "B")))
  replicate_code <- as.integer(factor(dat_overall_bal$replicate))
  manual <- manual_overall_cia(dat_overall_bal, reference = NULL)

  raw <- cpp_fun(
    y = y,
    subject = subject_code,
    method = method_code,
    replicate = replicate_code,
    n_methods = 3L,
    reference_method = 0L,
    has_reference = FALSE,
    n_threads = 1L
  )

  expect_equal(raw$estimate_raw[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(raw$A_bar[[1]], manual$A_bar, tolerance = 1e-12)
  expect_equal(raw$B_bar[[1]], manual$B_bar, tolerance = 1e-12)
})

test_that("C++ overall reference estimator matches 2 * A_bar / B_bar and not the old pooled ratio", {
  cpp_fun <- get("cia_overall_balanced_cpp", envir = asNamespace("matrixCorr"))
  y <- dat_overall_bal$value
  subject_code <- as.integer(factor(dat_overall_bal$subject))
  method_code <- as.integer(factor(dat_overall_bal$method, levels = c("R", "A", "B")))
  replicate_code <- as.integer(factor(dat_overall_bal$replicate))
  manual <- manual_overall_cia(dat_overall_bal, reference = "R")
  wrong_old <- manual$A_bar / manual$B_bar

  raw <- cpp_fun(
    y = y,
    subject = subject_code,
    method = method_code,
    replicate = replicate_code,
    n_methods = 3L,
    reference_method = 1L,
    has_reference = TRUE,
    n_threads = 1L
  )

  expect_equal(raw$estimate_raw[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(raw$A_bar[[1]], manual$A_bar, tolerance = 1e-12)
  expect_equal(raw$B_bar[[1]], manual$B_bar, tolerance = 1e-12)
  expect_false(isTRUE(all.equal(raw$estimate_raw[[1]], wrong_old, tolerance = 1e-12)))
})

test_that("diagnostics backend no longer exposes the old pooled overall estimator", {
  cpp_fun <- get("cia_moments_cpp", envir = asNamespace("matrixCorr"))
  y <- dat_overall_bal$value
  subject_code <- as.integer(factor(dat_overall_bal$subject))
  method_code <- as.integer(factor(dat_overall_bal$method, levels = c("R", "A", "B")))
  replicate_code <- as.integer(factor(dat_overall_bal$replicate))

  raw <- cpp_fun(
    y = y,
    subject = subject_code,
    method = method_code,
    replicate = replicate_code,
    n_methods = 3L,
    reference_method = 1L,
    has_reference = TRUE,
    pairwise = FALSE,
    n_threads = 1L
  )

  expect_null(raw$estimate)
})

test_that("unbalanced replication uses equal subject weighting in pairwise CIA", {
  dat_unbalanced <- data.frame(
    subject = c(
      rep("s1", 5),
      rep("s2", 4)
    ),
    method = c(
      "A", "A", "A", "B", "B",
      "A", "A", "B", "B"
    ),
    replicate = c(
      "r1", "r2", "r3", "r1", "r2",
      "r1", "r2", "r1", "r2"
    ),
    value = c(
      0, 2, 4, 0, 0,
      10, 12, 10, 10
    )
  )

  fit <- suppressWarnings(cia(
    dat_unbalanced,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  expect_equal(as.matrix(fit)["A", "B"], 9 / 13, tolerance = 1e-12)
})

test_that("pairwise CIA uses delta normal CI by default", {
  dat_big <- do.call(
    rbind,
    lapply(seq_len(12), function(i) {
      data.frame(
        subject = rep(sprintf("s%02d", i), 4),
        method = c("A", "A", "B", "B"),
        replicate = c("r1", "r2", "r1", "r2"),
        value = c(i, i + 1, i + 0.1, i + 1.1)
      )
    })
  )

  fit <- cia(
    dat_big,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    ci = TRUE
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "delta_normal")
  expect_true(all(c("est", "lwr.ci", "upr.ci") %in% names(fit)))
})

test_that("pairwise bootstrap CIA remains available", {
  dat_big <- do.call(
    rbind,
    lapply(seq_len(12), function(i) {
      data.frame(
        subject = rep(sprintf("s%02d", i), 4),
        method = c("A", "A", "B", "B"),
        replicate = c("r1", "r2", "r1", "r2"),
        value = c(i, i + 1, i + 0.1, i + 1.1)
      )
    })
  )

  fit <- cia(
    dat_big,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    ci = TRUE,
    inference = "bootstrap",
    B = 20L,
    seed = 1L
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "subject_bootstrap_percentile")
  expect_true(all(c("est", "lwr.ci", "upr.ci") %in% names(fit)))
})

test_that("overall reference delta CI follows the paper's ratio delta method", {
  manual <- manual_overall_cia(dat_overall_bal, reference = "R")
  fit <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "overall",
    estimator = "mom_unconstrained",
    ci = TRUE,
    inference = "delta"
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "delta_normal")
  expect_equal(fit$cia[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(fit$se[[1]], manual$se, tolerance = 1e-12)
  expect_equal(fit$lwr.ci[[1]], manual$lwr, tolerance = 1e-12)
  expect_equal(fit$upr.ci[[1]], manual$upr, tolerance = 1e-12)
})

test_that("overall no-reference delta CI follows the paper's ratio delta method", {
  manual <- manual_overall_cia(dat_overall_bal, reference = NULL)
  fit <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "overall",
    estimator = "mom_unconstrained",
    ci = TRUE,
    inference = "delta"
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "delta_normal")
  expect_equal(fit$cia[[1]], manual$psi, tolerance = 1e-12)
  expect_equal(fit$se[[1]], manual$se, tolerance = 1e-12)
  expect_equal(fit$lwr.ci[[1]], manual$lwr, tolerance = 1e-12)
  expect_equal(fit$upr.ci[[1]], manual$upr, tolerance = 1e-12)
})

test_that("overall CIA rejects unbalanced replication", {
  dat_bad <- dat_overall_bal[-1, , drop = FALSE]

  expect_error(
    cia(
      dat_bad,
      response = "value",
      subject = "subject",
      method = "method",
      replicate = "replicate",
      scope = "overall"
    ),
    "balanced-replication"
  )
})

test_that("overall bootstrap recomputes the corrected moment estimator", {
  fit <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "overall",
    estimator = "mom_unconstrained",
    ci = TRUE,
    inference = "bootstrap",
    B = 20L,
    seed = 1L
  )
  fit_plain <- cia(
    dat_overall_bal,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "overall",
    estimator = "mom_unconstrained"
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "subject_bootstrap_percentile")
  expect_equal(fit$cia[[1]], fit_plain$cia[[1]], tolerance = 1e-12)
  expect_true(all(c("lwr.ci", "upr.ci") %in% names(fit)))
})

test_that("constrained overall estimator rejects delta CI", {
  expect_error(
    cia(
      dat_overall_bal,
      response = "value",
      subject = "subject",
      method = "method",
      replicate = "replicate",
      scope = "overall",
      estimator = "vc_constrained",
      ci = TRUE,
      inference = "delta"
    ),
    "unconstrained moment estimator"
  )
})

test_that("reference pairwise CIA can exceed 1 under the raw MOM estimator", {
  dat_gt1 <- data.frame(
    subject = c(
      "s1", "s1", "s1", "s1",
      "s2", "s2", "s2", "s2"
    ),
    method = c(
      "R", "R", "A", "A",
      "R", "R", "A", "A"
    ),
    replicate = c(
      "r1", "r2", "r1", "r2",
      "r1", "r2", "r1", "r2"
    ),
    value = c(
      0, 2, 1, 1,
      10, 12, 11, 11
    )
  )

  fit <- suppressWarnings(cia(
    dat_gt1,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "R",
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  expect_gt(as.matrix(fit)["A", "R"], 1)
})

test_that("pairwise CIA warns when fewer than 10 eligible subjects are used", {
  expect_warning(
    cia(
      dat_ref2,
      response = "value",
      subject = "subject",
      method = "method",
      replicate = "replicate",
      scope = "pairwise",
      estimator = "mom_unconstrained"
    ),
    "fewer than 10 eligible subjects"
  )
})

pcor_from_precision <- function(Theta) {
  d <- diag(Theta)
  S <- -Theta / sqrt(outer(d, d))
  diag(S) <- 1
  S
}

strip_matrix_metadata <- function(x) {
  attrs <- attributes(x)
  keep <- intersect(names(attrs), c("dim", "dimnames"))
  attributes(x) <- attrs[keep]
  x
}

manual_partial_fisher_ci_R <- function(X, conf_level = 0.95) {
  X <- as.matrix(X)
  n <- nrow(X)
  p <- ncol(X)
  c_vars <- p - 2L
  S <- stats::cov(X)
  Theta <- solve(S)
  pcor <- pcor_from_precision(Theta)

  lwr <- matrix(NA_real_, p, p, dimnames = dimnames(pcor))
  upr <- matrix(NA_real_, p, p, dimnames = dimnames(pcor))
  diag(lwr) <- 1
  diag(upr) <- 1

  se <- 1 / sqrt(n - 3 - c_vars)
  crit <- stats::qnorm(0.5 * (1 + conf_level))
  eps <- sqrt(.Machine$double.eps)

  for (j in seq_len(p - 1L)) {
    for (i in (j + 1L):p) {
      r <- pcor[j, i]
      if (!is.finite(r) || abs(r) >= 1) next
      z <- atanh(max(min(r, 1 - eps), -1 + eps))
      lo <- tanh(z - crit * se)
      hi <- tanh(z + crit * se)
      lwr[j, i] <- lwr[i, j] <- lo
      upr[j, i] <- upr[i, j] <- hi
    }
  }

  list(
    pcor = pcor,
    lwr = lwr,
    upr = upr,
    n_complete = n,
    n_conditioning = c_vars
  )
}

oas_shrink_R <- function(X) {
  n <- nrow(X)
  p <- ncol(X)
  mu <- colMeans(X)
  Xc <- sweep(X, 2, mu, check.margin = FALSE)
  S_mle <- crossprod(Xc) / n
  trS <- sum(diag(S_mle))
  trS2 <- sum(S_mle * S_mle)
  mu_sc <- trS / p
  num <- (1 - 2 / p) * trS2 + trS^2
  den <- (n + 1 - 2 / p) * (trS2 - trS^2 / p)
  rho <- if (den > 0) num / den else 1
  rho <- max(0, min(1, rho))
  Sigma <- (1 - rho) * S_mle
  diag(Sigma) <- diag(Sigma) + rho * mu_sc
  list(Sigma = Sigma, rho = rho)
}

glasso_reference_case <- function() {
  vars <- paste0("V", seq_len(6))

  Sigma <- structure(
    c(0.892577511863956, -3.47460427750079e-05, -0.048912146675666,
      0.00145719602008398, 0.00184184223658341, 7.32384941200259e-08,
      -3.47460427750079e-05, 1.18178348379229, 0.000607657901650682,
      -1.81034106257749e-05, -0.0211063204776342, -0.00249098993197849,
      -0.048912146675666, 0.000607657901650682, 0.855402516097313,
      -0.0254842452589996, -0.0322111497962843, -1.28083505639143e-06,
      0.00145719602008398, -1.81034106257749e-05, -0.0254842452589996,
      1.08365876399129, 0.000959638095557692, 3.81587780019788e-08,
      0.00184184223658341, -0.0211063204776342, -0.0322111497962843,
      0.000959638095557692, 1.11881841527388, 4.44883792437081e-05,
      7.32384941200259e-08, -0.00249098993197849, -1.28083505639143e-06,
      3.81587780019788e-08, 4.44883792437081e-05, 0.894638069650798),
    dim = c(6L, 6L),
    dimnames = list(vars, vars)
  )

  Theta <- structure(
    c(1.12387243015938, 0, 0.0642633288004597, 0, 0, 0,
      0, 0.846468847136127, 0, 0, 0.0159683981780265, 0.0023560756458958,
      0.0642633288004597, 0, 1.1748032714163, 0.0275114249784953,
      0.033693582191095, 0, 0, 0, 0.0275114249784953, 0.923446698493756, 0, 0,
      0, 0.0159683981780265, 0.033693582191095, 0, 0.895071380200203, 0,
      0, 0.0023560756458958, 0, 0, 0, 1.11777701272096),
    dim = c(6L, 6L),
    dimnames = list(vars, vars)
  )

  list(lambda = 0.08, Sigma = Sigma, Theta = Theta)
}

test_that("pcorr returns expected components for each method", {
  set.seed(111)
  X <- matrix(rnorm(200), nrow = 40, ncol = 5)
  colnames(X) <- paste0("V", seq_len(5))

  samp <- pcorr(X, method = "sample", return_cov_precision = TRUE, return_details = TRUE)
  expect_s3_class(samp, "partial_corr")
  expect_equal(dim(samp$pcor), c(5, 5))
  expect_true(all(diag(samp$pcor) == 1))
  expect_true(is.matrix(samp$cov))
  expect_true(is.matrix(samp$precision))
  expect_equal(samp$method, "sample")
  expect_true(is.na(samp$lambda))
  expect_true(is.na(samp$rho))

  ridge <- pcorr(X, method = "ridge", lambda = 1e-2, return_cov_precision = TRUE,
                 return_details = TRUE)
  expect_equal(ridge$method, "ridge")
  expect_equal(ridge$lambda, 1e-2)
  expect_true(is.na(ridge$rho))

  oas <- pcorr(X, method = "oas", return_cov_precision = TRUE, return_details = TRUE)
  expect_equal(oas$method, "oas")
  expect_true(is.na(oas$lambda))
  expect_true(oas$rho >= 0 && oas$rho <= 1)

  glasso <- pcorr(X, method = "glasso", lambda = 0.05, return_cov_precision = TRUE,
                  return_details = TRUE)
  expect_equal(glasso$method, "glasso")
  expect_equal(glasso$lambda, 0.05)
  expect_true(is.na(glasso$rho))
})

test_that("pcorr print and plot methods cover options", {
  skip_if_not_installed("ggplot2")

  set.seed(222)
  X <- matrix(rnorm(150), nrow = 30, ncol = 5)
  colnames(X) <- paste0("G", seq_len(5))
  pc <- pcorr(X, method = "ridge", lambda = 5e-3, return_cov_precision = TRUE,
              return_details = TRUE)

  out1 <- capture.output(print(pc, digits = 4, max_rows = 3, max_cols = 4))
  expect_true(any(grepl("omitted", out1)))

  out2 <- capture.output(print(pc, show_method = FALSE, max_rows = 2, max_cols = 2))
  expect_true(any(grepl("Partial correlation matrix", out2)))

  p <- plot(pc, mask_diag = FALSE, reorder = TRUE, value_text_size = 3,
            low_color = "blue", high_color = "red", mid_color = "white")
  expect_s3_class(p, "ggplot")
})

test_that("pcorr validates lambda", {
  set.seed(1)
  X <- matrix(rnorm(40), nrow = 10, ncol = 4)
  expect_error(pcorr(X, method = "ridge", lambda = -1), "must be >=")
})

test_that("pcorr returns matrix output by default and details only when requested", {
  set.seed(2)
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  fit <- pcorr(X)
  expect_s3_class(fit, "corr_matrix")
  expect_s3_class(fit, "partial_corr_matrix")
  expect_false(inherits(fit, "partial_corr"))
  expect_true(is.matrix(fit))
  expect_equal(dim(estimate(fit)), c(4L, 4L))
  expect_equal(dimnames(estimate(fit)), list(colnames(X), colnames(X)))

  detailed <- pcorr(X, return_details = TRUE)
  expect_s3_class(detailed, "partial_corr")
  expect_true(is.matrix(detailed$pcor))
  expect_error(
    pcorr(X, return_cov_precision = TRUE),
    "return_details"
  )
})

test_that("pcorr rejects non-finite matrix inputs without an R-side copy", {
  X <- matrix(rnorm(16), nrow = 4)
  X[3] <- Inf
  expect_error(pcorr(X, method = "sample"), "finite values")
})

test_that("pcorr exposes shrinkage metadata without cov/precision", {
  set.seed(333)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)

  oas <- pcorr(X, method = "oas")
  expect_true(is.numeric(attr(oas, "rho", exact = TRUE)))
  expect_true(isTRUE(attr(oas, "rho", exact = TRUE) >= 0 &&
                     attr(oas, "rho", exact = TRUE) <= 1))
  expect_true(is.numeric(attr(oas, "jitter", exact = TRUE)))
  expect_false(is.na(attr(oas, "jitter", exact = TRUE)))
  expect_null(attr(oas, "cov", exact = TRUE))
  expect_null(attr(oas, "precision", exact = TRUE))

  samp <- pcorr(X, method = "sample")
  expect_true(is.na(attr(samp, "rho", exact = TRUE)))
  expect_true(is.na(attr(samp, "lambda", exact = TRUE)))
  expect_true(is.numeric(attr(samp, "jitter", exact = TRUE)))
  expect_null(attr(samp, "p_value", exact = TRUE))
})

test_that("pcorr default CI behaviour remains estimate-only", {
  set.seed(334)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)

  fit <- pcorr(X, method = "sample")

  expect_s3_class(fit, "partial_corr_matrix")
  expect_s3_class(fit, "corr_matrix")
  expect_false(inherits(fit, "partial_corr"))
  expect_true(is.matrix(estimate(fit)))
  expect_null(attr(fit, "ci", exact = TRUE))
  expect_null(attr(fit, "conf.level", exact = TRUE))
  expect_true(is.list(attr(fit, "diagnostics", exact = TRUE)))
})

test_that("pcorr Fisher-z CI matches manual partial-correlation calculation", {
  set.seed(335)
  X <- matrix(rnorm(240), nrow = 60, ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  fit <- pcorr(X, method = "sample", ci = TRUE, return_p_value = TRUE)
  ref <- manual_partial_fisher_ci_R(X, conf_level = 0.95)
  ci <- attr(fit, "ci", exact = TRUE)

  expect_true(is.list(ci))
  expect_identical(ci$ci.method, "fisher_z_partial")
  expect_equal(ci$conf.level, 0.95)
  expect_equal(estimate(fit), ref$pcor, tolerance = 1e-10)
  expect_equal(ci$lwr.ci, ref$lwr, tolerance = 1e-10)
  expect_equal(ci$upr.ci, ref$upr, tolerance = 1e-10)
  diagnostics <- attr(fit, "diagnostics", exact = TRUE)
  expect_identical(unique(as.integer(diagnostics$n_complete)), as.integer(ref$n_complete))
  expect_identical(unique(as.integer(diagnostics$n_conditioning[upper.tri(diagnostics$n_conditioning)])), as.integer(ref$n_conditioning))
  inference <- attr(fit, "inference", exact = TRUE)
  expect_true(is.matrix(inference$p_value))
  expect_null(attr(estimate(fit), "ci", exact = TRUE))
  expect_null(attr(estimate(fit), "diagnostics", exact = TRUE))
})

test_that("pcorr custom conf_level changes CI width", {
  set.seed(336)
  X <- matrix(rnorm(320), nrow = 80, ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  fit90 <- pcorr(X, method = "sample", ci = TRUE, conf_level = 0.90)
  fit99 <- pcorr(X, method = "sample", ci = TRUE, conf_level = 0.99)

  ci90 <- attr(fit90, "ci", exact = TRUE)
  ci99 <- attr(fit99, "ci", exact = TRUE)
  width90 <- ci90$upr.ci["V1", "V2"] - ci90$lwr.ci["V1", "V2"]
  width99 <- ci99$upr.ci["V1", "V2"] - ci99$lwr.ci["V1", "V2"]

  expect_equal(attr(fit90, "conf.level", exact = TRUE), 0.90)
  expect_equal(attr(fit99, "conf.level", exact = TRUE), 0.99)
  expect_true(width99 > width90)
})

test_that("pcorr CI integrates with the existing summary contract", {
  set.seed(337)
  X <- matrix(rnorm(240), nrow = 60, ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  fit <- pcorr(X, method = "sample", ci = TRUE)
  sm <- summary(fit)
  txt <- capture.output(print(sm, show_ci = "yes"))

  expect_s3_class(sm, "summary.corr_matrix")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "data.frame")
  expect_true(isTRUE(attr(sm, "has_ci")))
  expect_true(all(c("estimate", "lwr", "upr") %in% names(sm)))
  expect_match(paste(txt, collapse = "\n"), "Partial correlation summary")
  expect_match(paste(txt, collapse = "\n"), "ci_method")
  expect_match(paste(txt, collapse = "\n"), "ci_width")
  expect_false(any(grepl("Strongest pairs by \\|estimate\\|", txt)))
  expect_true(any(grepl("95% CI", txt, fixed = TRUE)))
})

test_that("sample partial-correlation p-values match the reference example", {
  y.data <- data.frame(
    hl = c(7, 15, 19, 15, 21, 22, 57, 15, 20, 18),
    disp = c(0.000, 0.964, 0.000, 0.000, 0.921, 0.000, 0.000, 1.006, 0.000, 1.011),
    deg = c(9, 2, 3, 4, 1, 3, 1, 3, 6, 1),
    BC = c(1.78e-02, 1.05e-06, 1.37e-05, 7.18e-03, 0.00e+00,
           0.00e+00, 0.00e+00, 4.48e-03, 2.10e-06, 0.00e+00)
  )

  ours <- pcorr(y.data, method = "sample", return_p_value = TRUE, return_details = TRUE)

  vars <- c("hl", "disp", "deg", "BC")
  expected_pcor <- structure(
    c(
      1.0000000, -0.6720863, -0.6161163,  0.1148459,
     -0.6720863,  1.0000000, -0.7215522,  0.2855420,
     -0.6161163, -0.7215522,  1.0000000,  0.6940953,
      0.1148459,  0.2855420,  0.6940953,  1.0000000
    ),
    dim = c(4L, 4L),
    dimnames = list(vars, vars)
  )

  expected_p_value <- structure(
    c(
      0.00000000, 0.06789202, 0.10383620, 0.78654997,
      0.06789202, 0.00000000, 0.04332869, 0.49299871,
      0.10383620, 0.04332869, 0.00000000, 0.05615021,
      0.78654997, 0.49299871, 0.05615021, 0.00000000
    ),
    dim = c(4L, 4L),
    dimnames = list(vars, vars)
  )

  expect_equal(
    strip_matrix_metadata(ours$pcor),
    strip_matrix_metadata(expected_pcor),
    tolerance = 1e-7
  )
  expect_equal(ours$p_value, expected_p_value, tolerance = 1e-8)
  expect_equal(unname(diag(ours$p_value)), rep(0, ncol(y.data)))
})

test_that("pcorr p-values are restricted to the classical sample setting", {
  X <- matrix(rnorm(60), nrow = 20, ncol = 3)

  expect_error(
    pcorr(X, method = "ridge", return_p_value = TRUE),
    "available only for"
  )

  X_wide <- matrix(rnorm(36), nrow = 6, ncol = 6)
  expect_error(
    pcorr(X_wide, method = "sample", return_p_value = TRUE),
    "requires .*n > p"
  )
})

test_that("pcorr CI is restricted to the classical sample setting", {
  X <- matrix(rnorm(80), nrow = 20, ncol = 4)

  expect_error(
    pcorr(X, method = "ridge", ci = TRUE),
    "available only for the classical .*method = \"sample\""
  )
  expect_error(
    pcorr(X, method = "oas", ci = TRUE),
    "available only for the classical .*method = \"sample\""
  )
  expect_error(
    pcorr(X, method = "glasso", ci = TRUE),
    "available only for the classical .*method = \"sample\""
  )
})

test_that("pcorr CI requires positive Fisher-z residual degrees of freedom", {
  X <- matrix(rnorm(20), nrow = 5, ncol = 4)

  expect_error(
    pcorr(X, method = "sample", ci = TRUE),
    "requires .*n > p \\+ 1"
  )
})

test_that("pcorr CI warns and returns NA bounds when the sample covariance needs jitter", {
  x1 <- 1:20
  x2 <- x1
  x3 <- rep(c(0, 1), each = 10)
  X <- cbind(x1 = x1, x2 = x2, x3 = x3)

  expect_warning(
    fit <- pcorr(X, method = "sample", ci = TRUE),
    class = "matrixCorr_ci_warning"
  )

  ci <- attr(fit, "ci", exact = TRUE)
  expect_true(all(is.na(ci$lwr.ci[upper.tri(ci$lwr.ci)])))
  expect_true(all(is.na(ci$upr.ci[upper.tri(ci$upr.ci)])))
})

test_that("sample partial correlation is close to truth in a structured MVN model", {
  skip_if_not_installed("MASS")
  set.seed(20240819)

  p <- 8
  alpha <- 0.3
  Theta <- diag(p)
  for (j in 1:(p - 1)) Theta[j, j + 1] <- Theta[j + 1, j] <- -alpha
  Sigma <- solve(Theta)

  n <- 1500
  X <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
  colnames(X) <- paste0("V", seq_len(p))

  ours <- estimate(pcorr(X, method = "sample", return_cov_precision = FALSE))
  truth <- pcor_from_precision(Theta)

  ut <- upper.tri(ours, diag = FALSE)
  z_hat <- atanh(ours[ut])
  z_true <- atanh(truth[ut])
  se_z <- 1 / sqrt(n - p - 1)

  expect_lt(max(abs(z_hat - z_true)), 3 * se_z)

  non_edge <- matrix(TRUE, p, p)
  diag(non_edge) <- FALSE
  non_edge[abs(row(non_edge) - col(non_edge)) == 1] <- FALSE
  expect_lt(max(abs(ours[non_edge])), 0.07)
  expect_true(isSymmetric(ours, tol = 1e-12))
  expect_equal(as.numeric(diag(ours)), rep(1, p), tolerance = 1e-12)
})

test_that("sample method equals the base-R precision construction", {
  set.seed(123)
  n <- 200
  p <- 12
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))

  ours <- pcorr(X, method = "sample", return_cov_precision = TRUE, return_details = TRUE)

  S_unb <- stats::cov(X)
  Theta <- solve(S_unb)
  ref <- pcor_from_precision(Theta)

  expect_equal(
    strip_matrix_metadata(ours$pcor),
    strip_matrix_metadata(ref),
    tolerance = 1e-10
  )
  expect_true(isSymmetric(ours$pcor))
  expect_equal(as.numeric(diag(ours$pcor)), rep(1, p))
})

test_that("ridge method equals the base-R ridge construction", {
  set.seed(99)
  n <- 180
  p <- 10
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  lambda <- 5e-3

  ours <- pcorr(X, method = "ridge", lambda = lambda,
                return_cov_precision = TRUE, return_details = TRUE)

  S_unb <- stats::cov(X)
  diag(S_unb) <- diag(S_unb) + lambda
  Theta <- solve(S_unb)
  ref <- pcor_from_precision(Theta)

  expect_equal(
    strip_matrix_metadata(ours$pcor),
    strip_matrix_metadata(ref),
    tolerance = 1e-10
  )
  I <- diag(p)
  dimnames(I) <- dimnames(S_unb)
  expect_equal(S_unb %*% Theta, I, tolerance = 1e-8)
})

test_that("OAS method matches an R implementation of the same formula", {
  set.seed(7)
  n <- 120
  p <- 25
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))

  ours <- pcorr(X, method = "oas", return_cov_precision = TRUE, return_details = TRUE)

  oas <- oas_shrink_R(X)
  Sigma <- oas$Sigma
  Theta <- solve(Sigma)
  ref <- pcor_from_precision(Theta)

  expect_equal(
    strip_matrix_metadata(ours$pcor),
    strip_matrix_metadata(ref),
    tolerance = 1e-10
  )
  expect_true(isSymmetric(ours$pcor))
  expect_equal(as.numeric(diag(ours$pcor)), rep(1, p))
})

test_that("glasso method matches fixed graphical-lasso reference values", {
  set.seed(1234)
  n <- 120
  p <- 6
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  ref <- glasso_reference_case()
  lambda <- ref$lambda

  ours <- pcorr(X, method = "glasso", lambda = lambda, return_cov_precision = TRUE,
                return_details = TRUE)
  ref_pcor <- pcor_from_precision(ref$Theta)
  dimnames(ref_pcor) <- dimnames(ours$pcor)

  expect_equal(ours$precision, ref$Theta, tolerance = 2e-4)
  expect_equal(ours$cov, ref$Sigma, tolerance = 2e-4)
  expect_equal(
    strip_matrix_metadata(ours$pcor),
    strip_matrix_metadata(ref_pcor),
    tolerance = 2e-4
  )
  expect_true(isSymmetric(ours$pcor, tol = 1e-12))
  expect_equal(as.numeric(diag(ours$pcor)), rep(1, p), tolerance = 1e-12)
})

test_that("p >> n: OAS returns a finite, well-formed matrix", {
  set.seed(4242)
  n <- 40
  p <- 80
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))

  oas <- pcorr(X, method = "oas", return_cov_precision = TRUE, return_details = TRUE)
  M <- oas$pcor
  expect_true(is.matrix(M))
  expect_true(isSymmetric(M, tol = 1e-12))
  expect_equal(as.numeric(diag(M)), rep(1, p), tolerance = 1e-12)
  expect_true(all(is.finite(M)))
  expect_true(all(abs(M[upper.tri(M, diag = FALSE)]) <= 1 + 1e-10))

  I <- diag(p)
  dimnames(I) <- dimnames(oas$cov)
  expect_equal(oas$cov %*% oas$precision, I, tolerance = 1e-6)
})

test_that("non-numeric columns are ignored and dimnames propagate", {
  set.seed(1)
  X <- data.frame(
    a = rnorm(50),
    b = rnorm(50),
    f = factor(sample(letters[1:3], 50, TRUE)),
    c = rnorm(50),
    s = sample(c("x", "y", "z"), 50, TRUE),
    l = sample(c(TRUE, FALSE), 50, TRUE)
  )
  cols <- c("a", "b", "c")

  pc <- estimate(pcorr(X, method = "oas", return_cov_precision = FALSE))
  expect_equal(dim(pc), c(3L, 3L))
  expect_equal(dimnames(pc), list(cols, cols))
})

#' @title Validation and normalisation for correlation
#'
#' @description
#' Validates and normalises input for correlation computations. Accepts either a
#' numeric matrix or a data frame, filters numeric columns, checks dimensions and
#' (optionally) missing values, and returns a numeric (double) matrix with
#' preserved column names.
#'
#' @details
#' Rules enforced:
#' \itemize{
#'   \item Input must be a matrix or data.frame.
#'   \item Only numeric (integer or double) columns are retained (data.frame path).
#'   \item At least two numeric columns are required.
#'   \item All columns must have the same length and \eqn{\ge} 2 observations.
#'   \item Missing and non-finite values are not allowed when
#'     \code{check_na = TRUE}.
#'   \item Returns a \code{double} matrix; integer input is converted once.
#' }
#'
#' @param data A matrix or data frame. Non-numeric columns are dropped
#'   (data.frame path). For matrix input, storage mode must be integer or double.
#' @param check_na Logical (default \code{TRUE}). If \code{TRUE}, validate and
#'   reject inputs containing \code{NA}/\code{NaN}/\code{Inf}. Set to
#'   \code{FALSE} when an upstream routine (e.g., pairwise-complete kernels)
#'   will handle missingness per pair.
#'
#' @return A numeric matrix (type \code{double}) with column names preserved.
#'
#' @seealso [pearson_corr()], [spearman_rho()], [kendall_tau()]
#'
#' @author Thiago de Paula Oliveira
#' @keywords internal
#' @name matrixCorr-internal
validate_corr_input <- function(data, check_na = TRUE) {
  check_bool(check_na)
  validate_corr_input_cpp(data, check_na)
}

test_that("validate_corr_input rejects non-finite real matrix values", {
  X <- cbind(a = c(1, 2, Inf, 4), b = c(1, 2, 3, 4))

  expect_error(
    validate_corr_input(X, check_na = TRUE),
    "Missing values are not allowed.*non-finite values are not allowed"
  )

  expect_error(
    validate_corr_input(replace(X, 3, -Inf), check_na = TRUE),
    "Missing values are not allowed.*non-finite values are not allowed"
  )
})

test_that("validate_corr_input rejects non-finite real data-frame values", {
  df <- data.frame(a = c(1, 2, NaN, 4), b = c(1, 2, Inf, 4))

  expect_error(
    validate_corr_input(df, check_na = TRUE),
    "Missing values are not allowed.*non-finite values are not allowed"
  )
})

test_that("validate_corr_input permits non-finite values only when requested", {
  X <- cbind(a = c(1, 2, Inf, 4), b = c(1, 2, 3, 4))

  out <- validate_corr_input(X, check_na = FALSE)

  expect_true(is.matrix(out))
  expect_true(is.infinite(out[3, "a"]))
})

test_that("integer NA is preserved when check_na = FALSE", {
  X <- matrix(c(1L, NA_integer_, 3L, 4L), ncol = 2)

  out <- validate_corr_input(X, check_na = FALSE)

  expect_true(is.na(out[2, 1]))
  expect_false(is.finite(out[2, 1]))
  expect_false(identical(out[2, 1], -2147483648))
})

test_that("integer NA is preserved in data frames when check_na = FALSE", {
  df <- data.frame(
    a = c(1L, NA_integer_, 3L),
    b = c(4L, 5L, 6L),
    c = letters[1:3]
  )

  out <- validate_corr_input(df, check_na = FALSE)

  expect_true(is.na(out[2, "a"]))
  expect_false(is.finite(out[2, "a"]))
  expect_false(identical(out[2, "a"], -2147483648))
})

#' Biweight mid-correlation (bicor)
#'
#' @description
#' Computes pairwise biweight mid-correlations for numeric data. Bicor is a
#' robust, Pearson-like correlation that down-weights outliers and heavy-tailed
#' observations. Optional large-sample confidence intervals are available as a
#' derived feature.
#'
#'
#' @param data A numeric matrix or a data frame containing numeric columns.
#'   Factors, logicals and common time classes are dropped in the data-frame
#'   path. Missing values are not allowed unless \code{na_method = "pairwise"}.
#' @param c_const Positive numeric. Tukey biweight tuning constant applied to the
#'   \emph{raw} MAD; default \code{9} (Langfelder & Horvath's convention).
#' @param max_p_outliers Numeric in \code{(0, 1]}. Optional cap on the maximum
#'   proportion of outliers \emph{on each side}; if \code{< 1}, side-specific
#'   rescaling maps those quantiles to \code{|u|=1}. Use \code{1} to disable.
#' @param pearson_fallback Character scalar indicating the fallback policy.
#'   One of:
#'   \itemize{
#'     \item \code{"hybrid"} (default): if a column has MAD = 0, that column
#'       uses Pearson standardisation, yielding a hybrid correlation.
#'     \item \code{"none"}: return \code{NA} if a column has MAD = 0 or becomes
#'       degenerate after weighting.
#'     \item \code{"all"}: force ordinary Pearson for all columns.
#'   }
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each estimate on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param mad_consistent Logical; if \code{TRUE}, use the normal-consistent MAD
#'   (\code{MAD_raw * 1.4826}) in the bicor weights. Default \code{FALSE} to
#'   match Langfelder & Horvath (2012).
#' @param w Optional non-negative numeric vector of length \code{nrow(data)}
#'   giving \emph{row weights}. When supplied, weighted medians/MADs are used
#'   and Tukey weights are multiplied by \code{w} before normalisation.
#' @param sparse_threshold Optional numeric \eqn{\geq 0}. If supplied, sets
#'   entries with \code{|r| < sparse_threshold} to 0 and returns a sparse
#'   \code{"ddiMatrix"} (requires \pkg{Matrix}).
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach approximate
#'   Fisher-z confidence intervals for the off-diagonal biweight
#'   mid-correlations. This confidence interval is provided as an additional
#'   large-sample approximation.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#'   \code{0.95}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return A symmetric correlation matrix with class \code{bicor}
#'   (or a \code{dgCMatrix} if \code{sparse_threshold} is used), with attributes:
#'   \code{method = "biweight_mid_correlation"}, \code{description},
#'   and \code{package = "matrixCorr"}. Downstream code should be prepared to
#'   handle either a dense numeric matrix or a sparse \code{dgCMatrix}. When
#'   \code{ci = TRUE}, the object also carries a \code{ci} attribute with
#'   elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and
#'   \code{ci.method}, together with an \code{inference} attribute containing
#'   the standard large-sample summary matrices \code{estimate},
#'   \code{statistic}, \code{p_value}, \code{Z}, and \code{n_obs}. Pairwise
#'   complete-case counts are stored in \code{attr(x, "diagnostics")$n_complete}.
#' Internally, all medians/MADs, Tukey weights, optional pairwise-NA handling,
#' and OpenMP loops are implemented in the C++ helpers
#' (\code{bicor_*_cpp()}), so the R wrapper mostly validates arguments and
#' dispatches to the appropriate backend.
#'
#' @details
#'
#' For a column \eqn{x = (x_a)_{a=1}^m}, let \eqn{\mathrm{med}(x)} be the median and
#' \eqn{\mathrm{MAD}(x) = \mathrm{med}(|x - \mathrm{med}(x)|)} the (raw) median
#' absolute deviation. If \code{mad_consistent = TRUE}, the consistent scale
#' \eqn{\mathrm{MAD}^\star(x) = 1.4826\,\mathrm{MAD}(x)} is used. With tuning constant
#' \eqn{c>0}, define
#' \deqn{u_a = \frac{x_a - \mathrm{med}(x)}{c\,\mathrm{MAD}^{(\star)}(x)}.}
#' The Tukey biweight gives per-observation weights
#' \deqn{w_a = (1 - u_a^2)^2\,\mathbf{1}\{|u_a| < 1\}.}
#' Robust standardisation of a column is
#' \deqn{\tilde x_a =
#' \frac{(x_a - \mathrm{med}(x))\,w_a}{
#'       \sqrt{\sum_{b=1}^m \big[(x_b - \mathrm{med}(x))\,w_b\big]^2}}.}
#' For two columns \eqn{x,y}, the biweight mid-correlation is
#' \deqn{\mathrm{bicor}(x,y) = \sum_{a=1}^m \tilde x_a\,\tilde y_a \in [-1,1].}
#'
#' \strong{Capping the maximum proportion of outliers (\code{max_p_outliers}).}
#' If \code{max_p_outliers < 1}, let \eqn{q_L = Q_x(\text{max\_p\_outliers})} and
#' \eqn{q_U = Q_x(1-\text{max\_p\_outliers})} be the lower/upper quantiles of \eqn{x}.
#' If the corresponding \eqn{|u|} at either quantile exceeds 1, \eqn{u} is rescaled
#' \emph{separately} on the negative and positive sides so that those quantiles land at
#' \eqn{|u|=1}. This guarantees that all observations between the two quantiles receive
#' positive weight. Note the bound applies per side, so up to \eqn{2\,\text{max\_p\_outliers}}
#' of observations can be treated as outliers overall.
#'
#' \strong{Fallback when for zero MAD / degeneracy (\code{pearson_fallback}).}
#' If a column has \eqn{\mathrm{MAD}=0} or the robust denominator becomes zero,
#' the following rules apply:
#' \itemize{
#'   \item \code{"none"} when correlations involving that column are \code{NA} (diagonal
#'         remains 1).
#'   \item \code{"hybrid"} when only the affected column switches to Pearson standardisation
#'         \eqn{\bar x_a = (x_a - \overline{x}) / \sqrt{\sum_b (x_b - \overline{x})^2}},
#'         yielding the hybrid correlation
#'         \deqn{\mathrm{bicor}_{\mathrm{hyb}}(x,y) = \sum_a \bar x_a\,\tilde y_a,}
#'         with the other column still robust-standardised.
#'   \item \code{"all"} when all columns use ordinary Pearson standardisation; the result
#'         equals \code{stats::cor(..., method="pearson")} when the NA policy matches.
#' }
#'
#' \strong{Handling missing values (\code{na_method}).}
#' \itemize{
#'   \item \code{"error"} (default): inputs must be finite; this yields a symmetric,
#'         positive semidefinite (PSD) matrix since \eqn{R = \tilde X^\top \tilde X}.
#'   \item \code{"pairwise"}: each \eqn{R_{jk}} is computed on the intersection of
#'         rows where both columns are finite. Pairs with fewer than 5 overlapping
#'         rows return \code{NA} (guarding against instability). Pairwise deletion can
#'         break PSD, as in the Pearson case.
#' }
#'
#' \strong{Row weights (\code{w}).}
#' When \code{w} is supplied (non-negative, length \eqn{m}), the weighted median
#' \eqn{\mathrm{med}_w(x)} and weighted MAD
#' \eqn{\mathrm{MAD}_w(x) = \mathrm{med}_w(|x - \mathrm{med}_w(x)|)} are used to form
#' \eqn{u}. The Tukey weights are then multiplied by the observation weights prior
#' to normalisation:
#' \deqn{\tilde x_a =
#' \frac{(x_a - \mathrm{med}_w(x))\,w_a\,w^{(\mathrm{obs})}_a}{
#'       \sqrt{\sum_b \big[(x_b - \mathrm{med}_w(x))\,w_b\,w^{(\mathrm{obs})}_b\big]^2}},}
#' where \eqn{w^{(\mathrm{obs})}_a \ge 0} are the user-supplied row weights and \eqn{w_a}
#' are the Tukey biweights built from the weighted median/MAD. Weighted pairwise
#' behaves analogously on each column pair's overlap.
#'
#' \strong{MAD choice (\code{mad_consistent}).}
#' Setting \code{mad_consistent = TRUE} multiplies the raw MAD by 1.4826 inside
#' \eqn{u}. Equivalently, it uses an effective tuning constant
#' \eqn{c^\star = c \times 1.4826}. The default \code{FALSE} reproduces the convention
#' in Langfelder & Horvath (2012).
#'
#' \strong{Optional sparsification (\code{sparse_threshold}).}
#' If provided, entries with \eqn{|r| < \text{sparse\_threshold}} are set to 0 and the
#' result is returned as a \code{"ddiMatrix"} (diagonal is forced to 1). This is a
#' post-processing step that does not alter the per-pair estimates.
#'
#' \strong{Computation and threads.}
#' Columns are robust-standardised in parallel and the matrix is formed as
#' \eqn{R = \tilde X^\top \tilde X}. \code{n_threads} selects the number of OpenMP
#' threads; by default it uses \code{getOption("matrixCorr.threads", 1L)}.
#'
#' \strong{Large-sample inference.}
#' For a pairwise estimate \eqn{r} computed from \eqn{n_{jk}} observed rows, the
#' standard large-sample summaries use
#' \deqn{
#' Z_{jk} = \frac{1}{2}\log\!\left(\frac{1 + r}{1 - r}\right)\sqrt{n_{jk} - 2}
#' }
#' and
#' \deqn{
#' T_{jk} = \sqrt{n_{jk} - 2}\;\frac{|r|}{\sqrt{1-r^2}}.
#' }
#' The reported p-value is the two-sided Student-\eqn{t} tail probability with
#' \eqn{n_{jk}-2} degrees of freedom. When \code{ci = TRUE}, the package also
#' reports an approximate Fisher-z confidence interval obtained from
#' \deqn{
#' z_{jk} = \operatorname{atanh}(r), \qquad
#' \operatorname{SE}(z_{jk}) = \frac{1}{\sqrt{n_{jk}-3}},
#' }
#' followed by back-transformation with \code{tanh()}. Confidence intervals are currently available only for dense,
#' unweighted outputs.
#'
#' \strong{Basic properties.}
#' \eqn{\mathrm{bicor}(a x + b,\; c y + d) = \mathrm{sign}(ac)\,\mathrm{bicor}(x,y)}.
#' With no missing data (and with per-column hybrid/robust standardisation), the
#' output is symmetric and PSD. As with Pearson, affine equivariance does not hold
#' for the associated biweight midcovariance.
#'
#' @references
#' Langfelder, P. & Horvath, S. (2012).
#' Fast R Functions for Robust Correlations and Hierarchical Clustering.
#' Journal of Statistical Software, 46(11), 1-17. \doi{10.18637/jss.v046.i11}
#'
#' @importFrom Matrix Matrix
#' @examples
#' set.seed(1)
#' X <- matrix(rnorm(2000 * 40), 2000, 40)
#' R <- bicor(X, c_const = 9, max_p_outliers = 1,
#'                        pearson_fallback = "hybrid")
#' print(attr(R, "method"))
#' summary(R)
#' R_ci <- bicor(X[, 1:5], ci = TRUE)
#' summary(R_ci)
#' estimate(R_ci)
#' tidy(R_ci)
#' ci(R_ci)
#' confint(R_ci)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(R)
#' }
#'
#' @author Thiago de Paula Oliveira
#' @export
bicor <- function(
    data,
    na_method        = c("error", "pairwise", "complete"),
    ci               = FALSE,
    conf_level       = 0.95,
    n_threads        = getOption("matrixCorr.threads", 1L),
    output           = c("matrix", "sparse", "edge_list"),
    threshold        = 0,
    diag             = TRUE,
    c_const          = 9,
    max_p_outliers   = 1,
    pearson_fallback = c("hybrid", "none", "all"),
    mad_consistent   = FALSE,
    w                = NULL,
    sparse_threshold = NULL
) {
  output_missing <- missing(output)
  threshold_missing <- missing(threshold)
  if (!is.null(sparse_threshold)) {
    check_scalar_nonneg(sparse_threshold, arg = "sparse_threshold", strict = FALSE)
    if (isTRUE(output_missing)) {
      output <- "sparse"
    } else if (!identical(output, "sparse")) {
      abort_bad_arg(
        "output",
        message = "must be {.val sparse} when {.arg sparse_threshold} is supplied."
      )
    }
    if (isTRUE(threshold_missing)) {
      threshold <- sparse_threshold
    } else if (!isTRUE(all.equal(as.numeric(threshold), as.numeric(sparse_threshold)))) {
      abort_bad_arg(
        "threshold",
        message = "must match {.arg sparse_threshold} when both are supplied."
      )
    }
  }
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )

  if (missing(na_method) &&
      isFALSE(ci) &&
      missing(n_threads) &&
      identical(output_cfg$output, "matrix") &&
      identical(output_cfg$threshold, 0) &&
      isTRUE(output_cfg$diag) &&
      identical(c_const, 9) &&
      identical(max_p_outliers, 1) &&
      missing(pearson_fallback) &&
      isFALSE(mad_consistent) &&
      is.null(w) &&
      is.null(sparse_threshold)) {
    numeric_data <- validate_corr_input(data)
    colnames_data <- colnames(numeric_data)
    prev_threads <- .mc_prepare_omp_threads(
      n_threads
    )
    if (!is.null(prev_threads)) {
      on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
    }
    res <- bicor_matrix_cpp(
      numeric_data,
      c_const = 9,
      maxPOutliers = 1,
      pearson_fallback = 1L,
      n_threads = n_threads
    )
    if (!is.null(colnames_data)) {
      dimnames(res) <- .mc_square_dimnames(colnames_data)
    }
    return(.mc_structure_corr_matrix(
      res,
      class_name = "bicor",
      method = "biweight_mid_correlation",
      description = paste0(
        "Median/MAD-based biweight mid-correlation (bicor); max_p_outliers = 1",
        ", MAD = raw; fallback = hybrid; NA mode = error."
      ),
      symmetric = TRUE
    ))
  }

  pf <- match.arg(pearson_fallback)
  pf_int <- switch(pf, "none" = 0L, "hybrid" = 1L, "all" = 2L)
  na_method <- match.arg(na_method)

  # --- checks
  check_scalar_nonneg(c_const, arg = "c_const", strict = TRUE)
  check_scalar_numeric(max_p_outliers,
                       arg          = "max_p_outliers",
                       lower        = 0,
                       upper        = 1,
                       closed_lower = FALSE,
                       closed_upper = TRUE)
  check_bool(mad_consistent, arg = "mad_consistent")
  check_bool(ci, arg = "ci")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  if (!is.null(sparse_threshold)) {
    check_scalar_nonneg(sparse_threshold, arg = "sparse_threshold", strict = FALSE)
  }

  # --- validate/coerce input (allow NA only in pairwise mode)
  numeric_data <- if (na_method == "error") {
    validate_corr_input(data)
  } else {
    validate_corr_input(data, check_na = FALSE)
  }
  w <- check_weights(w, n = nrow(numeric_data), arg = "w")
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 5L, arg = "data")
    numeric_data <- cc$data
    if (!is.null(w)) {
      w <- w[cc$diagnostics$complete_rows]
    }
    diagnostics_extra <- cc$diagnostics
  }
  colnames_data <- colnames(numeric_data)
  if (isTRUE(ci) && !is.null(w)) {
    abort_bad_arg(
      "ci",
      message = "Confidence intervals are unavailable when {.arg w} is supplied."
    )
  }

  # --- MAD consistency via effective c
  c_eff <- if (isTRUE(mad_consistent)) c_const * 1.4826 else c_const
  dn <- .mc_square_dimnames(colnames_data)
  desc <- paste0(
    "Median/MAD-based biweight mid-correlation (bicor); max_p_outliers = ", max_p_outliers,
    ", MAD = ", if (mad_consistent) "normal-consistent (1.4826 * raw)" else "raw",
    "; fallback = ", pf, "; NA mode = ", na_method, "."
  )

  if (.mc_supports_direct_threshold_path(
    method = "bicor",
    na_method = na_method,
    ci = ci,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    pairwise = identical(na_method, "pairwise"),
    has_ci = ci,
    weighted = !is.null(w)
  )) {
    prev_threads <- .mc_prepare_omp_threads(
      n_threads
    )
    if (!is.null(prev_threads)) {
      on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
    }
    trip <- bicor_threshold_triplets_cpp(
      numeric_data,
      c_const = c_eff,
      maxPOutliers = max_p_outliers,
      pearson_fallback = pf_int,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      n_threads = n_threads
    )
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = "bicor",
      method = "biweight_mid_correlation",
      description = desc,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = as.integer(c(ncol(numeric_data), ncol(numeric_data))),
      source_dimnames = dn,
      symmetric = TRUE
    ))
  }

  # --- choose backend
  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  if (is.null(w) && !identical(na_method, "pairwise")) {
    res <- bicor_matrix_cpp(
      numeric_data,
      c_const          = c_eff,
      maxPOutliers     = max_p_outliers,
      pearson_fallback = pf_int,
      n_threads        = n_threads
    )
  } else if (is.null(w) && identical(na_method, "pairwise")) {
    res <- bicor_matrix_pairwise_cpp(
      numeric_data,
      c_const          = c_eff,
      maxPOutliers     = max_p_outliers,
      pearson_fallback = pf_int,
      min_n            = 5L,
      n_threads        = n_threads
    )
  } else if (!is.null(w) && !identical(na_method, "pairwise")) {
    res <- bicor_matrix_weighted_cpp(
      numeric_data, w,
      c_const          = c_eff,
      maxPOutliers     = max_p_outliers,
      pearson_fallback = pf_int,
      n_threads        = n_threads
    )
  } else { # weighted + pairwise
    res <- bicor_matrix_weighted_pairwise_cpp(
      numeric_data, w,
      c_const          = c_eff,
      maxPOutliers     = max_p_outliers,
      pearson_fallback = pf_int,
      min_n            = 5L,
      n_threads        = n_threads
    )
  }
  res <- .mc_set_matrix_dimnames(res, colnames_data)

  # --- names & metadata
  diagnostics <- NULL
  ci_attr <- NULL
  inference_attr <- NULL
  if (isTRUE(ci)) {
    n_complete <- .mc_bicor_n_complete(numeric_data)
    diagnostics <- list(n_complete = .mc_set_matrix_dimnames(n_complete, colnames_data))
    inference_attr <- .mc_bicor_large_sample_inference(
      est = res,
      n_complete = diagnostics$n_complete
    )
    ci_attr <- .mc_bicor_fisher_ci(
      est = res,
      n_complete = diagnostics$n_complete,
      conf_level = conf_level
    )
  }
  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)

  out <- .mc_structure_corr_matrix(
    res,
    class_name = "bicor",
    method = "biweight_mid_correlation",
    description = desc,
    symmetric = TRUE,
    diagnostics = diagnostics,
    extra_attrs = c(
      if (!is.null(ci_attr)) {
        list(
          ci = ci_attr,
          conf.level = conf_level
        )
      },
      if (!is.null(inference_attr)) {
        list(inference = inference_attr)
      }
    )
  )
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_bicor_n_complete <- function(x) {
  x <- as.matrix(x)
  fin <- is.finite(x)
  n_complete <- crossprod(unclass(fin))
  storage.mode(n_complete) <- "integer"
  n_complete
}

.mc_bicor_large_sample_inference <- function(est, n_complete) {
  est <- as.matrix(est)
  n_complete <- as.matrix(n_complete)
  p <- ncol(est)
  dn <- dimnames(est)

  statistic <- matrix(NA_real_, p, p, dimnames = dn)
  p_value <- matrix(NA_real_, p, p, dimnames = dn)
  Z <- matrix(NA_real_, p, p, dimnames = dn)
  parameter <- matrix(NA_real_, p, p, dimnames = dn)

  valid <- is.finite(est) & is.finite(n_complete) & n_complete > 2
  exact <- valid & abs(est) >= 1
  approx <- valid & !exact

  parameter[valid] <- n_complete[valid] - 2
  if (any(exact)) {
    statistic[exact] <- Inf
    p_value[exact] <- 0
    Z[exact] <- sign(est[exact]) * Inf
  }
  if (any(approx)) {
    eps <- sqrt(.Machine$double.eps)
    est_clip <- pmax(pmin(est[approx], 1 - eps), -1 + eps)
    df <- n_complete[approx] - 2
    statistic[approx] <- sqrt(df) * abs(est_clip) / sqrt(1 - est_clip^2)
    p_value[approx] <- 2 * stats::pt(statistic[approx], df = df, lower.tail = FALSE)
    Z[approx] <- atanh(est_clip) * sqrt(df)
  }

  diag(parameter) <- NA_real_
  diag(p_value) <- 0
  diag(statistic) <- Inf
  diag(Z) <- Inf

  list(
    method = "large_sample_bicor",
    estimate = unclass(est),
    statistic = statistic,
    p_value = p_value,
    Z = Z,
    parameter = parameter,
    n_obs = n_complete,
    alternative = "two.sided"
  )
}

.mc_bicor_fisher_ci <- function(est, n_complete, conf_level = 0.95) {
  est <- as.matrix(est)
  n_complete <- as.matrix(n_complete)
  p <- ncol(est)
  dn <- dimnames(est)
  lwr <- matrix(NA_real_, p, p, dimnames = dn)
  upr <- matrix(NA_real_, p, p, dimnames = dn)
  diag(lwr) <- 1
  diag(upr) <- 1

  valid <- is.finite(est) & is.finite(n_complete) & n_complete > 3
  exact <- valid & abs(est) >= 1
  approx <- valid & !exact
  if (any(exact)) {
    lwr[exact] <- est[exact]
    upr[exact] <- est[exact]
  }
  if (any(approx)) {
    eps <- sqrt(.Machine$double.eps)
    est_clip <- pmax(pmin(est[approx], 1 - eps), -1 + eps)
    z <- atanh(est_clip)
    se <- 1 / sqrt(n_complete[approx] - 3)
    crit <- stats::qnorm(0.5 * (1 + conf_level))
    lwr[approx] <- tanh(z - crit * se)
    upr[approx] <- tanh(z + crit * se)
  }

  list(
    est = unclass(est),
    lwr.ci = lwr,
    upr.ci = upr,
    conf.level = conf_level,
    ci.method = "fisher_z_bicor"
  )
}

.mc_bicor_pairwise_summary <- function(object,
                                       digits = 4,
                                       ci_digits = 3,
                                       p_digits = 4,
                                       show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "bicor")

  ci <- .mc_ci_attr(object)
  inf <- .mc_inference_attr(object)
  include_p <- !is.null(inf) && !is.null(inf$p_value)
  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.bicor",
    digits = digits,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci,
    ci_attr = ci,
    inference_attr = inf,
    include_p = include_p,
    extra_columns = if (isTRUE(include_p)) {
      list(
        statistic = list(matrix = inf$statistic, digits = digits),
        fisher_z = list(matrix = inf$Z, digits = digits),
        p_value = list(matrix = inf$p_value, digits = p_digits)
      )
    },
    extra_attrs = list(
      inference_method = if (is.null(inf)) NA_character_ else inf$method
    )
  )
}

#' @rdname bicor
#' @export
diag.bicor <- function(x, ...) {
  base::diag(as.matrix(x), ...)
}



#' @rdname bicor
#' @method print bicor
#' @title Print Method for \code{bicor} Objects
#'
#' @param x An object of class \code{bicor}.
#' @param digits Integer; number of decimal places used for the matrix.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits for bicor confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param na_print Character; how to display missing values.
#' @param ... Additional arguments passed to \code{print()}.
#'
#' @return Invisibly returns \code{x}.
#' @export
print.bicor <- function(x,
                        digits   = 4,
                        n = NULL,
                        topn = NULL,
                        max_vars = NULL,
                        width    = NULL,
                        ci_digits = 3,
                        show_ci = NULL,
                        na_print = "NA",
                        ...) {
  check_inherits(x, "bicor")
  .mc_print_corr_matrix(
    x,
    header = "Biweight mid-correlation matrix (bicor)",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    na.print = na_print,
    ...
  )
}

#' @rdname bicor
#' @method plot bicor
#' @title Plot Method for \code{bicor} Objects
#'
#' @param x An object of class \code{bicor}.
#' @param title Plot title. Default is \code{"Biweight mid-correlation heatmap"}.
#' @param reorder Character; one of \code{"none"} (default) or \code{"hclust"}.
#'   If \code{"hclust"}, variables are reordered by complete-linkage clustering
#'   on the distance \eqn{d = 1 - r}, after replacing \code{NA} by 0 for
#'   clustering purposes only.
#' @param triangle One of \code{"full"} (default), \code{"lower"}, or \code{"upper"}
#'   to display the full matrix or a single triangle.
#' @param low_color,mid_color,high_color Colours for the gradient in
#'   \code{scale_fill_gradient2}. Defaults are \code{"indianred1"},
#'   \code{"white"}, \code{"steelblue1"}.
#' @param value_text_size Numeric; font size for cell labels. Set to \code{NULL}
#'   to suppress labels (recommended for large matrices).
#' @param ci_text_size Text size for confidence-interval labels in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param na_fill Fill colour for \code{NA} cells. Default \code{"grey90"}.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other layers.
#'
#' @return A \code{ggplot} object.
#' @import ggplot2
#' @importFrom stats as.dist hclust
#' @export
plot.bicor <- function(
    x,
    title = "Biweight mid-correlation heatmap",
    reorder  = c("none", "hclust"),
    triangle = c("full", "lower", "upper"),
    low_color = "indianred1",
    mid_color = "white",
    high_color = "steelblue1",
    value_text_size = 3,
    ci_text_size = 2.5,
    show_value = TRUE,
    na_fill = "grey90",
    ...
) {
  check_inherits(x, "bicor")
  check_bool(show_value, arg = "show_value")

  reorder  <- match.arg(reorder)
  triangle <- match.arg(triangle)

  mat <- as.matrix(x)

  # Optional reordering via robust-inspired clustering on 1 - r
  if (reorder == "hclust" && ncol(mat) > 1) {
    R_fill <- mat
    R_fill[is.na(R_fill)] <- 0   # for clustering only
    R_fill[R_fill > 1] <- 1; R_fill[R_fill < -1] <- -1
    d <- stats::as.dist(1 - R_fill)
    ord <- stats::hclust(d, method = "complete")$order
    mat <- mat[ord, ord, drop = FALSE]
  }

  # Prepare long format with indices for triangle filtering
  rn <- rownames(mat); cn <- colnames(mat)
  if (is.null(rn)) rn <- seq_len(nrow(mat)) else rn <- as.character(rn)
  if (is.null(cn)) cn <- seq_len(ncol(mat)) else cn <- as.character(cn)
  dimnames(mat) <- list(rn, cn)

  idx_r <- rep(rn, each = length(cn))
  idx_c <- rep(cn, times = length(rn))

  tm <- as.data.frame(as.table(mat), stringsAsFactors = FALSE)
  names(tm) <- c("Var1", "Var2", "bicor")
  tm$Var1 <- as.character(tm$Var1)
  tm$Var2 <- as.character(tm$Var2)
  miss_r <- is.na(tm$Var1) | tm$Var1 == ""
  miss_c <- is.na(tm$Var2) | tm$Var2 == ""
  if (any(miss_r)) tm$Var1[miss_r] <- idx_r[miss_r]
  if (any(miss_c)) tm$Var2[miss_c] <- idx_c[miss_c]

  df <- tm
  df$Var1 <- factor(as.character(df$Var1), levels = rev(rn))
  df$Var2 <- factor(as.character(df$Var2), levels = cn)
  ci <- .mc_ci_attr(x)
  if (!is.null(ci) && is.matrix(ci$lwr.ci) && is.matrix(ci$upr.ci)) {
    lwr_mat <- ci$lwr.ci
    upr_mat <- ci$upr.ci
    if (reorder == "hclust" && ncol(mat) > 1) {
      lwr_mat <- lwr_mat[ord, ord, drop = FALSE]
      upr_mat <- upr_mat[ord, ord, drop = FALSE]
    }
    dimnames(lwr_mat) <- list(rn, cn)
    dimnames(upr_mat) <- list(rn, cn)
    df_lwr <- as.data.frame(as.table(lwr_mat), stringsAsFactors = FALSE)
    df_upr <- as.data.frame(as.table(upr_mat), stringsAsFactors = FALSE)
    names(df_lwr) <- c("Var1", "Var2", "lwr")
    names(df_upr) <- c("Var1", "Var2", "upr")
    df <- Reduce(
      function(a, b) merge(a, b, by = c("Var1", "Var2"), all.x = TRUE),
      list(df, df_lwr, df_upr)
    )
    df$Var1 <- factor(as.character(df$Var1), levels = rev(rn))
    df$Var2 <- factor(as.character(df$Var2), levels = cn)
  } else {
    df$lwr <- NA_real_
    df$upr <- NA_real_
  }

  # Triangle filtering
  if (triangle != "full") {
    # map factors back to integer indices
    i <- as.integer(df$Var1)         # note: Var1 is reversed
    j <- as.integer(df$Var2)
    # Because Var1 is reversed, convert to row indices in original order
    n <- length(levels(df$Var1))
    i_true <- n - i + 1
    keep <- switch(triangle,
                   lower = i_true >= j,
                   upper = i_true <= j)
    df <- df[keep, , drop = FALSE]
  }
  df$ci_label <- ifelse(
    is.na(df$lwr) | is.na(df$upr) | (as.character(df$Var1) == as.character(df$Var2)),
    NA_character_,
    sprintf("[%.3f, %.3f]", df$lwr, df$upr)
  )

  p <- ggplot2::ggplot(df, ggplot2::aes(Var2, Var1, fill = bicor)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color, mid = mid_color, high = high_color,
      midpoint = 0, limits = c(-1, 1), na.value = na_fill, name = "bicor"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ifelse(is.na(bicor), "NA", sprintf("%.2f", bicor))),
      size = value_text_size, color = "black"
    )
  }
  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.22),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }

  return(p)
}

#' @rdname bicor
#' @method summary bicor
#' @param object An object of class \code{bicor}.
#' @param ci_digits Integer; digits for bicor confidence limits in the pairwise
#'   summary.
#' @param p_digits Integer; digits for bicor p-values in the pairwise summary.
#' @export
summary.bicor <- function(object, n = NULL, topn = NULL,
                          max_vars = NULL, width = NULL,
                          ci_digits = 3,
                          p_digits = 4,
                          show_ci = NULL, ...) {
  check_inherits(object, "bicor")
  if (is.null(.mc_ci_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_bicor_pairwise_summary(
    object,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname bicor
#' @method print summary.bicor
#' @param x An object of class \code{summary.bicor}.
#' @export
print.summary.bicor <- function(x, digits = NULL, n = NULL,
                                topn = NULL, max_vars = NULL,
                                width = NULL, show_ci = NULL, ...) {
  extra_items <- c(inference = attr(x, "inference_method", exact = TRUE))
  .mc_print_pairwise_summary_digest(
    x,
    title = "Biweight mid-correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "fisher_z_bicor",
    extra_items = extra_items,
    ...
  )
  invisible(x)
}

# tests/testthat/test-bicor.R

# helper - defined using R
bicor_manual_test <- function(x, y, c_const = 9) {
  rx <- {
    mx  <- median(x); madx <- median(abs(x - mx))
    u   <- (x - mx) / (c_const * madx)
    w   <- ifelse(abs(u) < 1, (1 - u^2)^2, 0)
    r   <- (x - mx) * w
    r / sqrt(sum(r^2))
  }
  ry <- {
    my  <- median(y); mady <- median(abs(y - my))
    u   <- (y - my) / (c_const * mady)
    w   <- ifelse(abs(u) < 1, (1 - u^2)^2, 0)
    r   <- (y - my) * w
    r / sqrt(sum(r^2))
  }
  sum(rx * ry)
}

biweight_standardise_R <- function(x, c_const = 9, max_p_outliers = 1) {
  stopifnot(is.numeric(x), length(x) >= 2L)
  mx  <- stats::median(x)
  mad <- stats::median(abs(x - mx))
  if (!(mad > 0)) return(rep(NaN, length(x)))

  # Side-cap disabled here for exactness in most tests
  stopifnot(max_p_outliers == 1)
  u <- (x - mx) / (c_const * mad)
  w <- ifelse(abs(u) < 1, (1 - u^2)^2, 0)
  r <- (x - mx) * w
  s2 <- sum(r^2)
  if (!(s2 > 0)) return(rep(NaN, length(x)))
  r / sqrt(s2)
}

bicor_manual_R <- function(x, y, c_const = 9, max_p_outliers = 1) {
  zx <- biweight_standardise_R(x, c_const, max_p_outliers)
  zy <- biweight_standardise_R(y, c_const, max_p_outliers)
  if (any(!is.finite(zx)) || any(!is.finite(zy))) return(NaN)
  sum(zx * zy)
}

# Weighted versions (for small n testcases)
w_quantile <- function(x, w, p) {
  o  <- order(x, method = "radix")
  xs <- x[o]; ws <- w[o]
  W  <- sum(ws)
  if (!(W > 0)) return(NaN)
  if (p <= 0) return(xs[1L])
  if (p >= 1) return(xs[length(xs)])
  target <- p * W
  csum   <- 0
  for (i in seq_along(xs)) {
    csum <- csum + ws[i]
    if (csum >= target) {
      if (i == 1L) return(xs[1L])
      cprev <- csum - ws[i]
      frac  <- (target - cprev) / ws[i]
      return(xs[i-1L] + frac * (xs[i] - xs[i-1L]))
    }
  }
  xs[length(xs)]
}

w_median  <- function(x, w)  w_quantile(x, w, 0.5)
w_mad     <- function(x, w, med) w_median(abs(x - med), w)

biweight_standardise_w_R <- function(x, w, c_const = 9, max_p_outliers = 1) {
  stopifnot(is.numeric(x), is.numeric(w), length(x) == length(w))
  W <- sum(w); if (!(W > 0)) return(rep(NaN, length(x)))
  med <- w_median(x, w)
  mad <- w_mad(x, w, med)
  if (!(mad > 0)) {
    # weighted Pearson fallback (for tests that expect it)
    mu <- sum(w * x) / W
    r  <- (x - mu) * w
    s2 <- sum(r^2)
    if (!(s2 > 0)) return(rep(NaN, length(x)))
    return(r / sqrt(s2))
  }
  stopifnot(max_p_outliers == 1)
  u  <- (x - med) / (c_const * mad)
  wt <- ifelse(abs(u) < 1, (1 - u^2)^2, 0) * w
  r  <- (x - med) * wt
  s2 <- sum(r^2)
  if (!(s2 > 0)) return(rep(NaN, length(x)))
  r / sqrt(s2)
}

bicor_weighted_manual_R <- function(x, y, w, c_const = 9, max_p_outliers = 1) {
  zx <- biweight_standardise_w_R(x, w, c_const, max_p_outliers)
  zy <- biweight_standardise_w_R(y, w, c_const, max_p_outliers)
  if (any(!is.finite(zx)) || any(!is.finite(zy))) return(NaN)
  sum(zx * zy)
}

test_that("affine transformations yield ±1 exactly", {
  x     <- 1:5
  y_pos <- 2*x + 7    # strictly increasing affine transform
  y_neg <- -3*x + 1   # strictly decreasing affine transform

  M <- cbind(x = x, y_pos = y_pos, y_neg = y_neg)

  R <- bicor(
    M, c_const = 9, max_p_outliers = 1,
    pearson_fallback = "none", n_threads = 1L
  )

  expect_equal(R["x", "y_pos"],  1, tolerance = 1e-12)
  expect_equal(R["x", "y_neg"], -1, tolerance = 1e-12)
  expect_equal(as.numeric(diag(R)), rep(1, 3))
})

test_that("single gross same-direction leaves bicor = 1", {
  x <- c(1:5, 1000)
  y <- c(1:5, 1000)
  R <- bicor(cbind(x, y), pearson_fallback = "none")
  expect_gt(R["x", "y"], 0.999)
})

test_that("opposite-direction outlier: bicor matches manual definition", {
  x <- c(1:5, 1000)
  y <- c(1:5, -1000)
  R <- bicor(cbind(x, y), pearson_fallback = "none")
  expect_equal(R["x","y"], bicor_manual_test(x, y), tolerance = 1e-12)
  expect_lt(R["x","y"], 0.9)  # sanity: not near 1
})

test_that("fallback policy: 'none' and 'hybrid' produce NA for constant column; 'all' matches Pearson", {
  A <- rep(1, 4)            # constant -> MAD = 0 and variance = 0
  B <- 1:4
  C <- 2*B + 5
  M <- cbind(A = A, B = B, C = C)

  # 'none' => NA for all correlations involving A; B~C remains exactly 1
  R_none <- bicor(M, pearson_fallback = "none")
  expect_true(R_none["A","A"]==1)
  expect_true(all(is.na(R_none["A", c("B","C")])))
  expect_equal(R_none["B","C"], 1, tolerance = 1e-12)

  # 'hybrid' tries Pearson for zero-MAD column; but A has zero variance,
  # so it must still be NA. B~C remains exactly 1.
  R_hybr <- bicor(M, pearson_fallback = "hybrid")
  expect_true(R_hybr["A","A"]==1)
  expect_true(all(is.na(R_hybr["A", c("B","C")])))
  expect_equal(R_hybr["B","C"], 1, tolerance = 1e-12)
})

test_that("threading does not change results beyond tiny numerical noise", {
  set.seed(42)
  X <- matrix(rnorm(500 * 20), 500, 20)

  R1 <- bicor(X, n_threads = 1L)
  R2 <- bicor(X, n_threads = 2L)

  expect_equal(R1, R2, tolerance = 1e-8)
})

test_that("data-frame path: drops non-numeric columns and preserves numeric names", {
  df <- data.frame(
    num1 = 1:5,
    fac  = factor(c("a","b","a","b","a")),
    logi = c(TRUE, FALSE, TRUE, FALSE, TRUE),
    num2 = 2*(1:5) + 7,
    date = as.Date("2020-01-01") + 0:4
  )

  R <- bicor(df, pearson_fallback = "none")
  expect_equal(colnames(R), c("num1", "num2"))
  expect_equal(rownames(R), c("num1", "num2"))
  expect_equal(R["num1","num2"], 1, tolerance = 1e-12)
})

test_that("validator rejects missing values", {
  df_na <- data.frame(a = c(1, 2, NA, 4), b = c(2, 3, 4, 5))
  expect_error(
    bicor(df_na),
    regexp = "Missing values are not allowed"
  )
})

test_that("attributes and class are set correctly", {
  set.seed(1)
  X <- matrix(rnorm(200), 50, 4)
  R <- bicor(X)

  expect_s3_class(R, "bicor")
  expect_equal(attr(R, "method"), "biweight_mid_correlation")
  expect_true(grepl("biweight mid-correlation", attr(R, "description")))
  expect_equal(attr(R, "package"), "matrixCorr")
  expect_null(attr(R, "ci", exact = TRUE))
  expect_null(attr(R, "inference", exact = TRUE))
})

test_that("bicor large-sample inference follows the established reference behaviour", {
  set.seed(151)
  X <- matrix(rnorm(90 * 4), nrow = 90, ncol = 4)
  X[, 2] <- 0.7 * X[, 1] + rnorm(90, sd = 0.4)
  X[, 4] <- -0.4 * X[, 3] + rnorm(90, sd = 0.6)
  colnames(X) <- paste0("B", seq_len(ncol(X)))

  ours <- bicor(X, ci = TRUE, pearson_fallback = "hybrid")
  inf <- attr(ours, "inference", exact = TRUE)
  ref_estimate <- structure(
    c(
      1, 0.87211433189796, -0.139821080789143, 0.0200531551858707,
      0.87211433189796, 1, -0.100737855987848, 0.0600146995629313,
      -0.139821080789143, -0.100737855987848, 1, -0.501164943217541,
      0.0200531551858707, 0.0600146995629313, -0.501164943217541, 1
    ),
    dim = c(4L, 4L)
  )
  ref_p_value <- structure(
    c(
      0, 4.66844299908808e-29, 0.188716903130168, 0.851189832850908,
      4.66844299908808e-29, 0, 0.344796594702779, 0.574185746219993,
      0.188716903130168, 0.344796594702779, 0, 4.87601871367649e-07,
      0.851189832850908, 0.574185746219993, 4.87601871367649e-07, 0
    ),
    dim = c(4L, 4L)
  )
  ref_statistic <- structure(
    c(
      Inf, 16.7200525192994, 1.32465032686288, 0.188153104953045,
      16.7200525192994, Inf, 0.949836670568733, 0.564004406966281,
      1.32465032686288, 0.949836670568733, 363463325.618328, 5.43287021000477,
      0.188153104953045, 0.564004406966281, 5.43287021000477, 363463325.618328
    ),
    dim = c(4L, 4L)
  )
  ref_Z <- structure(
    c(
      Inf, 12.5876081889807, -1.32028716414974, 0.188140491858905,
      12.5876081889807, Inf, -0.948221133852547, 0.563665166175343,
      -1.32028716414974, -0.948221133852547, 170.409068424375, -5.16753057745717,
      0.188140491858905, 0.563665166175343, -5.16753057745717, 170.409068424375
    ),
    dim = c(4L, 4L)
  )
  ref_n_obs <- structure(rep(90, 16), dim = c(4L, 4L))

  idx <- upper.tri(ours, diag = FALSE)
  expect_lt(max(abs(unclass(ours)[idx] - ref_estimate[idx]), na.rm = TRUE), 1e-8)
  expect_lt(max(abs(inf$p_value[idx] - ref_p_value[idx]), na.rm = TRUE), 1e-8)
  expect_lt(max(abs(inf$statistic[idx] - ref_statistic[idx]), na.rm = TRUE), 1e-8)
  expect_lt(max(abs(inf$Z[idx] - ref_Z[idx]), na.rm = TRUE), 1e-8)
  expect_equal(attr(ours, "diagnostics", exact = TRUE)$n_complete[idx], ref_n_obs[idx], tolerance = 0)
})

test_that("bicor returns confidence intervals only when requested", {
  set.seed(152)
  X <- matrix(rnorm(70 * 4), nrow = 70, ncol = 4)
  X[, 2] <- 0.6 * X[, 1] + rnorm(70, sd = 0.5)
  colnames(X) <- paste0("C", seq_len(ncol(X)))

  fit0 <- bicor(X, ci = FALSE)
  fit1 <- bicor(X, ci = TRUE)

  expect_null(attr(fit0, "ci", exact = TRUE))
  expect_null(attr(fit0, "inference", exact = TRUE))

  ci <- attr(fit1, "ci", exact = TRUE)
  inf <- attr(fit1, "inference", exact = TRUE)
  expect_true(is.list(ci))
  expect_true(is.list(inf))
  expect_identical(ci$ci.method, "fisher_z_bicor")
  expect_true(is.matrix(ci$lwr.ci))
  expect_true(is.matrix(ci$upr.ci))
  expect_true(is.matrix(inf$p_value))
})

test_that("bicor confidence intervals are ordered and bounded", {
  set.seed(153)
  X <- matrix(rnorm(80 * 5), nrow = 80, ncol = 5)
  X[, 2] <- 0.8 * X[, 1] + rnorm(80, sd = 0.25)
  colnames(X) <- paste0("D", seq_len(ncol(X)))

  fit <- bicor(X, ci = TRUE)
  ci <- attr(fit, "ci", exact = TRUE)
  idx <- upper.tri(fit, diag = FALSE)

  expect_true(all(ci$lwr.ci[idx] <= ci$upr.ci[idx], na.rm = TRUE))
  expect_true(all(ci$lwr.ci[idx] >= -1, na.rm = TRUE))
  expect_true(all(ci$upr.ci[idx] <= 1, na.rm = TRUE))
  expect_true(all(fit[idx] >= ci$lwr.ci[idx] & fit[idx] <= ci$upr.ci[idx], na.rm = TRUE))
})

test_that("bicor core estimate is unchanged when ci is disabled", {
  set.seed(154)
  X <- matrix(rnorm(60 * 4), nrow = 60, ncol = 4)
  X[, 3] <- -0.5 * X[, 2] + rnorm(60, sd = 0.4)

  fit_base <- bicor(X)
  fit_ci_false <- bicor(X, ci = FALSE)

  base_mat <- unclass(fit_base)
  false_mat <- unclass(fit_ci_false)
  attributes(base_mat) <- attributes(base_mat)[c("dim", "dimnames")]
  attributes(false_mat) <- attributes(false_mat)[c("dim", "dimnames")]
  expect_equal(base_mat, false_mat, tolerance = 1e-12)
})

test_that("bicor summary switches to pairwise inference view when ci is requested", {
  set.seed(155)
  X <- matrix(rnorm(75 * 4), nrow = 75, ncol = 4)
  X[, 2] <- 0.65 * X[, 1] + rnorm(75, sd = 0.4)
  colnames(X) <- paste0("S", seq_len(ncol(X)))

  fit <- bicor(X, ci = TRUE)
  sm <- summary(fit)
  txt <- capture.output(print(sm))

  expect_s3_class(sm, "summary.bicor")
  expect_s3_class(sm, "data.frame")
  expect_true(all(c("item1", "item2", "estimate", "n_complete", "lwr", "upr", "statistic", "fisher_z", "p_value") %in% names(sm)))
  expect_true(isTRUE(attr(sm, "has_ci")))
  expect_true(isTRUE(attr(sm, "has_p")))
  expect_true(any(grepl("^Biweight mid-correlation summary$", txt)))
  expect_true(any(grepl("p_value", txt, fixed = TRUE)))
})

test_that("NA policy: error vs pairwise", {
  set.seed(1)
  x <- c(rnorm(4), NA, rnorm(3))
  y <- c(rnorm(3), NA, rnorm(4))

  M <- cbind(x = x, y = y)
  # error mode must reject
  expect_error(bicor(M, na_method = "error"),
               "Missing values are not allowed")

  # pairwise mode: compute on finite overlap
  idx <- is.finite(x) & is.finite(y)
  expect_true(sum(idx) >= 5)  # ensure enough points for a stable check
  Rpw <- bicor(M, na_method = "pairwise",
                           pearson_fallback = "none",
                           c_const = 9, max_p_outliers = 1)
  expect_equal(Rpw["x","y"],
               bicor_manual_R(x[idx], y[idx], c_const = 9),
               tolerance = 1e-12)
  expect_equal(as.numeric(diag(Rpw)), rep(1, 2))
})

test_that("weighted, NA-free: matches manual weighted bicor", {
  x <- c(-2, -1, 0, 1, 2)
  y <- c(-4, -2, 0, 2, 4)  # perfectly monotone
  w <- c(1, 1, 5, 1, 1)    # emphasise the centre

  Rw <- bicor(cbind(x = x, y = y),
                          w = w,
                          na_method = "error",
                          pearson_fallback = "none",
                          c_const = 9, max_p_outliers = 1)
  expected <- bicor_weighted_manual_R(x, y, w, c_const = 9)
  expect_equal(Rw["x","y"], expected, tolerance = 1e-12)
  expect_equal(as.numeric(diag(Rw)), rep(1, 2))
})

test_that("weighted + pairwise: matches manual on overlapping rows (hybrid fallback)", {
  x <- c(0, 0, 0, 1, 2, NA)
  y <- c(0, 0, 0, 2, 4,  5)
  w <- c(1, 1, 5, 1, 1,  3)

  idx <- is.finite(x) & is.finite(y)
  Rwp <- bicor(cbind(x = x, y = y),
                           w = w, na_method = "pairwise",
                           pearson_fallback = "hybrid",   # ← change here
                           c_const = 9, max_p_outliers = 1)

  expect_equal(Rwp["x","y"],
               bicor_weighted_manual_R(x[idx], y[idx], w[idx], c_const = 9),
               tolerance = 1e-12)
})

test_that("fallback = 'all' reproduces Pearson (including constant columns)", {
  A <- rep(1, 5); B <- 1:5; C <- 2*B + 5
  M <- cbind(A = A, B = B, C = C)

  R_all <- bicor(M, pearson_fallback = "all", na_method = "error")
  R_p   <- suppressWarnings(stats::cor(M, method = "pearson"))

  to_na <- function(m) { m[is.nan(m)] <- NA_real_; m }

  # Diagonal: ignore names on the vector
  expect_equal(unname(diag(R_all)), rep(1, 3))

  # Off-diagonals: align NaN from bicor to NA as in base cor()
  R_all_na <- to_na(R_all)
  expect_equal(R_all_na[upper.tri(R_all_na)], R_p[upper.tri(R_p)])
  expect_equal(R_all_na[lower.tri(R_all_na)], R_p[lower.tri(R_p)])
})


test_that("fallback = 'hybrid' when one column has MAD=0 but variance>0", {
  # x has MAD = 0 (at least half zeros) but variance > 0
  x <- c(0, 0, 0, 1, 2)
  y <- c(0, 1, 2, 3, 4)

  # Expected: Pearson standardisation for x, biweight for y
  mu  <- mean(x); zx <- (x - mu) / sqrt(sum((x - mu)^2))
  zy  <- biweight_standardise_R(y, c_const = 9)
  expected <- sum(zx * zy)

  Rh <- bicor(cbind(x = x, y = y),
                          pearson_fallback = "hybrid",
                          c_const = 9, max_p_outliers = 1)
  expect_equal(Rh["x","y"], expected, tolerance = 1e-12)
})

test_that("mad_consistent=TRUE equals using c_const*1.4826", {
  set.seed(123)
  X <- matrix(rnorm(200), 50, 4)

  R1 <- bicor(X, mad_consistent = TRUE,  c_const = 9)
  R2 <- bicor(X, mad_consistent = FALSE, c_const = 9 * 1.4826)

  expect_equal(R1, R2, tolerance = 1e-12, ignore_attr = TRUE)
})


test_that("max_p_outliers side-cap improves robustness in one-sided outliers", {
  # Perfect linear pattern + a few very large positive outliers on y
  set.seed(7)
  x <- seq(-2, 2, length.out = 40)
  y <- 3*x + rnorm(length(x), sd = 0.02)
  y[38:40] <- y[38:40] + 50  # large positive outliers (one-sided)

  R0 <- bicor(cbind(x, y),
                          c_const = 9, max_p_outliers = 1,
                          pearson_fallback = "none")
  R1 <- bicor(cbind(x, y),
                          c_const = 9, max_p_outliers = 0.10,
                          pearson_fallback = "none")
  # With side-cap, more points remain with positive weight on the + side -> correlation should not decrease
  expect_gte(R1["x","y"], R0["x","y"])
})

test_that("sparse_threshold alias returns unified sparse output contract", {
  skip_if_not_installed("Matrix")

  # Small matrix with modest correlations
  set.seed(99)
  X <- matrix(rnorm(200), 50, 4)
  R  <- bicor(X, sparse_threshold = 0.9)  # most off-diagonals -> 0

  expect_s4_class(R, "sparseMatrix")
  expect_identical(attr(R, "output"), "sparse")
  expect_equal(as.numeric(attr(R, "threshold")), 0.9)
  expect_true(isTRUE(attr(R, "diag")))

  meta <- attr(R, "matrixCorr_meta", exact = TRUE)
  expect_true(is.list(meta))
  expect_true("bicor" %in% meta$source_class)
  expect_identical(meta$method, "biweight_mid_correlation")

  # diagonal remains 1 for finite bicor diagonals
  expect_equal(as.numeric(diag(as.matrix(R))), rep(1, ncol(X)))
  # confirm many structural zeros (at least one)
  nz  <- Matrix::nnzero(R)
  expect_lt(nz, ncol(X)^2)  # sparsity achieved
})

test_that("sparse output drops non-finite dense entries and keeps finite thresholded values", {
  skip_if_not_installed("Matrix")

  M <- cbind(
    const = rep(1, 6),
    var   = c(0, 1, 2, 3, 4, 5)
  )

  R_dense <- bicor(M, pearson_fallback = "none")
  R <- bicor(M, pearson_fallback = "none", sparse_threshold = 0.4)
  R_sparse_dense <- as.matrix(R)

  expect_s4_class(R, "sparseMatrix")
  finite_keep <- is.finite(R_dense) & abs(R_dense) >= 0.4
  expect_equal(R_sparse_dense[finite_keep], R_dense[finite_keep], tolerance = 1e-12)
  expect_true(all(R_sparse_dense[!is.finite(R_dense)] == 0))
})

test_that("plot methods do not error and return expected classes", {
  skip_if_not_installed("ggplot2")

  set.seed(10)
  X <- matrix(rnorm(100), 20, 5)
  R <- bicor(X)

  # plot method: should return a ggplot object
  p <- plot(R, title = "bicor heatmap")
  expect_s3_class(p, "ggplot")
})

#' @title Repeated-Measures Correlation (rmcorr)
#'
#' @description
#' Computes repeated-measures correlation for two or more continuous responses
#' observed repeatedly within subjects. Supply a \code{data.frame} plus column
#' names, or pass the response matrix/data frame and subject vector directly.
#' The repeated observations are indexed only by \code{subject}, thus no explicit
#' time variable is modeled, and the method targets a common within-subject
#' linear association after removing subject-specific means.
#'
#' @param data Optional \code{data.frame}/matrix containing the repeated-measures
#'   dataset.
#' @param response Either:
#'   \itemize{
#'     \item a character vector of at least two column names in \code{data}, or
#'     \item a numeric matrix/data frame with at least two columns.
#'   }
#'   If exactly two responses are supplied, the function returns a pairwise
#'   repeated-measures correlation object of class \code{"rmcorr"}. If three or
#'   more responses are supplied, a symmetric matrix of class
#'   \code{"rmcorr_matrix"} is returned.
#' @param subject Subject identifier (factor/character/integer/numeric) or a
#'   single character string naming the subject column in \code{data}.
#' @param conf_level Confidence level used for Wald confidence intervals on the
#'   repeated-measures correlation (default \code{0.95}).
#' @param estimator Character scalar selecting the repeated-measures
#'   correlation estimator. \code{"ancova"} (default) is the classical
#'   subject-centred repeated-measures correlation. \code{"weighted"} is the
#'   weighted repeated-measures correlation coefficient of Kondo et al. (2025),
#'   designed for incomplete or unbalanced repeated-measurement schedules.
#' @param ci_method Confidence-interval engine. \code{"auto"} (default)
#'   resolves to \code{"fisher_z"} for \code{estimator = "ancova"} and to
#'   subject-level \code{"bootstrap"} for \code{estimator = "weighted"} when
#'   \code{n_boot > 0}; otherwise \code{"none"}. \code{"none"} returns
#'   \code{NA} bounds.
#' @param n_boot Integer \eqn{\ge 0}. Number of subject-level bootstrap
#'   resamples when \code{ci_method = "bootstrap"}.
#' @param seed Optional positive integer seed used for bootstrap
#'   reproducibility.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values in the selected response columns or
#'   \code{subject}. \code{"pairwise"} uses pairwise complete cases and drops
#'   subjects with fewer than two complete pairs for the specific contrast.
#'   \code{"complete"} removes rows with missing or non-finite values in
#'   \code{subject} and all selected response variables before fitting.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   C++ backend in matrix mode. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param keep_data Logical (default \code{FALSE}). If \code{TRUE}, returned
#'   objects retain a compact copy of the numeric response matrix plus encoded
#'   subject identifiers. For pairwise fits this enables \code{plot()} to
#'   reconstruct subject-level fitted lines lazily; for matrix outputs it also
#'   allows \code{view_rmcorr_shiny()} to rebuild pairwise scatterplots from the
#'   returned object alone.
#' @param verbose Logical; if \code{TRUE}, prints a short note about the thread
#'   count used in matrix mode.
#' @param ... Deprecated compatibility aliases. The legacy \code{check_na}
#'   argument is still accepted here temporarily.
#'
#' @return
#' Either a \code{"rmcorr"} object (exactly two responses) or a
#' \code{"rmcorr_matrix"} object (pairwise results when \eqn{\ge}3 responses).
#'
#' \strong{If \code{"rmcorr"} (exactly two responses)}, outputs include:
#' \itemize{
#'   \item \code{estimate}; repeated-measures correlation estimate.
#'   \item \code{p_value}; two-sided p-value for the common within-subject slope.
#'   \item \code{lwr}, \code{upr}; confidence interval limits for
#'         \code{estimate}.
#'   \item \code{slope}; common within-subject slope.
#'   \item \code{df}; residual degrees of freedom \eqn{N - S - 1}.
#'   \item \code{n_obs}; number of complete observations retained after
#'         dropping subjects with fewer than two repeated pairs.
#'   \item \code{n_subjects}; number of contributing subjects.
#'   \item \code{responses}; names of the fitted response variables.
#'   \item compatibility aliases \code{r}, \code{conf_int}, and \code{based.on}
#'         are reconstructed on access without duplicate storage.
#'   \item when \code{keep_data = TRUE}, compact source data are retained so
#'         \code{plot()} can lazily reconstruct \code{data_long},
#'         \code{intercepts}, and fitted lines; these are otherwise not stored.
#' }
#'
#' \strong{If \code{"rmcorr_matrix"} (\eqn{\ge}3 responses)}, outputs are:
#' \itemize{
#'   \item a symmetric numeric matrix of pairwise repeated-measures correlations.
#'   \item attributes \code{method}, \code{description}, and
#'         \code{package = "matrixCorr"}.
#'   \item \code{diagnostics}; a list with square matrices for \code{slope},
#'         \code{p_value}, \code{df}, \code{n_complete}, \code{n_subjects},
#'         \code{conf_low}, and \code{conf_high}, plus scalar
#'         \code{conf_level}.
#' }
#'
#' @details
#' Repeated-measures correlation estimates the common \emph{within-subject}
#' linear association between two variables measured repeatedly on the same
#' subjects. It differs from agreement methods such as Lin's CCC or
#' Bland-Altman analysis because those target concordance or interchangeability,
#' whereas repeated-measures correlation targets the strength of the
#' subject-centred association.
#'
#' For subject \eqn{i = 1,\ldots,S} and repeated observations
#' \eqn{j = 1,\ldots,n_i}, let \eqn{x_{ij}} and \eqn{y_{ij}} denote the two
#' responses. Define subject-specific means
#' \deqn{\bar x_i = \frac{1}{n_i}\sum_{j=1}^{n_i} x_{ij}, \qquad
#'       \bar y_i = \frac{1}{n_i}\sum_{j=1}^{n_i} y_{ij}.}
#' The repeated-measures correlation uses within-subject centred values
#' \deqn{x^\ast_{ij} = x_{ij} - \bar x_i, \qquad
#'       y^\ast_{ij} = y_{ij} - \bar y_i}
#' and computes
#' \deqn{r_{\mathrm{rm}} =
#'       \frac{\sum_i \sum_j x^\ast_{ij} y^\ast_{ij}}
#'            {\sqrt{\left(\sum_i \sum_j {x^\ast_{ij}}^2\right)
#'                   \left(\sum_i \sum_j {y^\ast_{ij}}^2\right)}}.}
#'
#' Equivalently, this is the correlation implied by an ANCOVA model with a
#' common slope and subject-specific intercepts:
#' \deqn{y_{ij} = \alpha_i + \beta x_{ij} + \varepsilon_{ij}.}
#' The returned slope is
#' \deqn{\hat\beta =
#'       \frac{\sum_i \sum_j x^\ast_{ij} y^\ast_{ij}}
#'            {\sum_i \sum_j {x^\ast_{ij}}^2},}
#' and the subject-specific fitted intercepts are
#' \eqn{\hat\alpha_i = \bar y_i - \hat\beta \bar x_i}. Residual degrees of
#' freedom are \eqn{N - S - 1}, where \eqn{N = \sum_i n_i} after filtering to
#' complete observations and retaining only subjects with at least two repeated
#' pairs.
#'
#' Confidence intervals are computed with a Fisher \eqn{z}-transformation of
#' \eqn{r_{\mathrm{rm}}} and then back-transformed to the correlation scale. In
#' matrix mode, the same estimator is applied to every pair of selected
#' response columns.
#'
#' With \code{estimator = "weighted"}, the package implements the weighted
#' repeated-measures estimator from Kondo et al. (2025) using complete observed
#' \eqn{(x,y)} pairs per contrast. This estimator does not impute missing data;
#' it uses per-subject complete-pair sets and weighted sums of explained and
#' residual quantities from the fixed-subject ANCOVA decomposition.
#'
#' Bootstrap confidence intervals use non-parametric subject-level resampling
#' (subjects are resampled as blocks, not rows).
#'
#' @references
#' Bakdash, J. Z., & Marusich, L. R. (2017). Repeated Measures Correlation.
#' \emph{Frontiers in Psychology}, 8, 456.
#' \doi{10.3389/fpsyg.2017.00456}
#'
#' Kondo, M., Nagashima, K., Isono, S., & Sato, Y. (2025). Weighted Repeated
#' Measures Correlation Coefficient: A New Correlation Coefficient for Handling
#' Missing Data With Repeated Measures. \emph{Statistics in Medicine},
#' 44(10-12), e70046. \doi{10.1002/sim.70046}
#'
#' @examples
#' set.seed(2026)
#' n_subjects <- 20
#' n_rep <- 4
#' subject <- rep(seq_len(n_subjects), each = n_rep)
#' subj_eff_x <- rnorm(n_subjects, sd = 1.5)
#' subj_eff_y <- rnorm(n_subjects, sd = 2.0)
#' within_signal <- rnorm(n_subjects * n_rep)
#'
#' dat <- data.frame(
#'   subject = subject,
#'   x = subj_eff_x[subject] + within_signal + rnorm(n_subjects * n_rep, sd = 0.2),
#'   y = subj_eff_y[subject] + 0.8 * within_signal + rnorm(n_subjects * n_rep, sd = 0.3),
#'   z = subj_eff_y[subject] - 0.4 * within_signal + rnorm(n_subjects * n_rep, sd = 0.4)
#' )
#'
#' fit_xy <- rmcorr(dat, response = c("x", "y"), subject = "subject", keep_data = TRUE)
#' print(fit_xy)
#' summary(fit_xy)
#' estimate(fit_xy)
#' tidy(fit_xy)
#' ci(fit_xy)
#' confint(fit_xy)
#' plot(fit_xy)
#'
#' fit_mat <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject")
#' print(fit_mat, digits = 3)
#' summary(fit_mat)
#' tidy(fit_mat)
#' plot(fit_mat)
#'
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   fit_mat_view <- rmcorr(
#'     dat,
#'     response = c("x", "y", "z"),
#'     subject = "subject",
#'     keep_data = TRUE
#'   )
#'   view_rmcorr_shiny(fit_mat_view)
#' }
#' @author Thiago de Paula Oliveira
#' @export
rmcorr <- function(data = NULL, response, subject,
                   na_method = c("error", "pairwise", "complete"), conf_level = 0.95,
                   n_threads = getOption("matrixCorr.threads", 1L),
                   keep_data = FALSE,
                   verbose = FALSE,
                   estimator = c("ancova", "weighted"),
                   ci_method = c("auto", "fisher_z", "bootstrap", "none"),
                   n_boot = 999L,
                   seed = NULL,
                   ...) {
  # legacy positional signature support: rmcorr(response, subject, ...)
  if (!missing(data) && missing(subject)) {
    data_nrow <- tryCatch(NROW(data), error = function(...) NA_integer_)
    response_is_subject_vector <-
      !missing(response) &&
      isTRUE(length(response) == data_nrow) &&
      !(
        is.character(response) &&
        length(response) >= 2L &&
        inherits(data, "data.frame") &&
        all(response %in% names(data))
      )

    if (!inherits(data, "data.frame") || response_is_subject_vector) {
      subject <- response
      response <- data
      data <- NULL
    }
  }

  estimator <- match.arg(estimator)
  ci_method <- match.arg(ci_method)
  n_boot <- .mc_rmcorr_check_n_boot(n_boot)
  if (!is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  }
  ci_method_resolved <- .mc_rmcorr_resolve_ci_method(
    estimator = estimator,
    ci_method = ci_method,
    n_boot = n_boot
  )

  if (...length() == 0L &&
      missing(na_method) &&
      isFALSE(keep_data) &&
      isFALSE(verbose) &&
      identical(estimator, "ancova") &&
      identical(ci_method_resolved, "fisher_z")) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
    n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

    resolved <- .mc_rmcorr_resolve_inputs(data = data, response = response, subject = subject)
    response_mat <- resolved$response
    subject_vec <- resolved$subject
    response_names <- colnames(response_mat)

    if (ncol(response_mat) < 2L) {
      abort_bad_arg(
        "response",
        message = "must supply at least two numeric response columns."
      )
    }
    if (nrow(response_mat) != length(subject_vec)) {
      abort_bad_arg(
        "response",
        message = "and {.arg subject} must have the same number of observations."
      )
    }

    .mc_check_latent_missing(
      list(response = response_mat, subject = subject_vec),
      check_na = TRUE,
      arg = "response"
    )

    subj_info <- .mc_rmcorr_encode_subject(subject_vec, arg = "subject")

    if (ncol(response_mat) == 2L) {
      stats <- rmcorr_pair_cpp(
        x = response_mat[, 1L],
        y = response_mat[, 2L],
        subject = subj_info$code,
        conf_level = conf_level
      )
      return(.mc_rmcorr_build_pair_object(
        stats = stats,
        response_mat = response_mat,
        subject = subject_vec,
        response_names = response_names,
        subject_name = resolved$subject_name,
        na_method = "error",
        conf_level = conf_level,
        keep_data = FALSE
      ))
    }

    prev_threads <- .mc_prepare_omp_threads(
      n_threads
    )
    if (!is.null(prev_threads)) {
      on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
    }

    out <- rmcorr_matrix_cpp(
      x = response_mat,
      y = response_mat,
      subject = subj_info$code,
      symmetric = TRUE,
      conf_level = conf_level,
      n_threads = n_threads
    )

    est <- out$estimate
    dimnames(est) <- list(response_names, response_names)
    diagnostics <- list(
      slope = .mc_rmcorr_set_dimnames(out$slope, response_names),
      p_value = .mc_rmcorr_set_dimnames(out$p_value, response_names),
      df = .mc_rmcorr_set_dimnames(out$df, response_names),
      n_complete = .mc_rmcorr_set_dimnames(out$n_complete, response_names),
      n_subjects = .mc_rmcorr_set_dimnames(out$n_subjects, response_names),
      conf_low = .mc_rmcorr_set_dimnames(out$conf_low, response_names),
      conf_high = .mc_rmcorr_set_dimnames(out$conf_high, response_names),
      conf_level = conf_level
    )
    return(.mc_structure_corr_matrix(
      est,
      class_name = "rmcorr_matrix",
      method = "rmcorr",
      description = "Repeated-measures correlation matrix",
      symmetric = TRUE,
      diagnostics = diagnostics
    ))
  }

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method),
    allowed = c("error", "pairwise", "complete")
  )

  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(keep_data, arg = "keep_data")
  check_bool(verbose, arg = "verbose")
  ci_conf_level_cpp <- if (identical(ci_method_resolved, "fisher_z")) conf_level else -1

  resolved <- .mc_rmcorr_resolve_inputs(data = data, response = response, subject = subject)
  response_mat <- resolved$response
  subject_vec <- resolved$subject
  diagnostics_extra <- NULL
  if (identical(na_cfg$na_method, "complete")) {
    cc <- .mc_rmcorr_complete_cases(response_mat, subject_vec, min_n = 2L)
    response_mat <- cc$response
    subject_vec <- cc$subject
    diagnostics_extra <- cc$diagnostics
  }
  response_names <- colnames(response_mat)

  if (ncol(response_mat) < 2L) {
    abort_bad_arg(
      "response",
      message = "must supply at least two numeric response columns."
    )
  }

  if (nrow(response_mat) != length(subject_vec)) {
    abort_bad_arg(
      "response",
      message = "and {.arg subject} must have the same number of observations."
    )
  }

  if (na_cfg$check_na) {
    .mc_check_latent_missing(
      list(response = response_mat, subject = subject_vec),
      check_na = TRUE,
      arg = "response"
    )
  }

  subj_info <- .mc_rmcorr_encode_subject(subject_vec, arg = "subject")

  if (ncol(response_mat) == 2L) {
    stats <- .mc_rmcorr_pair_stats_dispatch(
      x = response_mat[, 1L],
      y = response_mat[, 2L],
      subject_code = subj_info$code,
      estimator = estimator,
      conf_level = ci_conf_level_cpp
    )
    if (identical(ci_method_resolved, "bootstrap")) {
      dat_pair <- .mc_rmcorr_pair_data(response_mat, subject_vec)
      ci_vals <- .mc_rmcorr_bootstrap_ci_from_pair_data(
        dat = dat_pair,
        estimator = estimator,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = seed
      )
      stats$conf_low <- ci_vals[[1L]]
      stats$conf_high <- ci_vals[[2L]]
    } else if (identical(ci_method_resolved, "none")) {
      stats$conf_low <- NA_real_
      stats$conf_high <- NA_real_
    }

    include_extended <- !(
      identical(estimator, "ancova") &&
      identical(ci_method_resolved, "fisher_z")
    )
    return(.mc_rmcorr_build_pair_object(
      stats = stats,
      response_mat = response_mat,
      subject = subject_vec,
      response_names = response_names,
      subject_name = resolved$subject_name,
      na_method = na_cfg$na_method,
      conf_level = conf_level,
      keep_data = keep_data,
      estimator = if (include_extended) estimator else NULL,
      ci_method = if (include_extended) ci_method_resolved else NULL,
      n_boot = if (include_extended && identical(ci_method_resolved, "bootstrap")) n_boot else NULL,
      seed = if (include_extended && identical(ci_method_resolved, "bootstrap")) seed else NULL,
      diagnostics_extra = diagnostics_extra
    ))
  }

  if (verbose) {
    cli::cli_inform("Using {n_threads} OpenMP thread{?s}.")
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  out <- .mc_rmcorr_matrix_stats_dispatch(
    response_mat = response_mat,
    subject_code = subj_info$code,
    estimator = estimator,
    conf_level = ci_conf_level_cpp,
    n_threads = n_threads
  )

  est <- out$estimate
  dimnames(est) <- list(response_names, response_names)
  conf_low <- .mc_rmcorr_set_dimnames(out$conf_low, response_names)
  conf_high <- .mc_rmcorr_set_dimnames(out$conf_high, response_names)
  if (identical(ci_method_resolved, "bootstrap")) {
    boot_ci <- .mc_rmcorr_matrix_bootstrap_ci(
      response_mat = response_mat,
      subject = subject_vec,
      estimator = estimator,
      conf_level = conf_level,
      n_boot = n_boot,
      seed = seed,
      diag_values = diag(est)
    )
    conf_low <- boot_ci$conf_low
    conf_high <- boot_ci$conf_high
  } else if (identical(ci_method_resolved, "none")) {
    conf_low[,] <- NA_real_
    conf_high[,] <- NA_real_
  }

  diagnostics <- list(
    slope = .mc_rmcorr_set_dimnames(out$slope, response_names),
    p_value = .mc_rmcorr_set_dimnames(out$p_value, response_names),
    df = .mc_rmcorr_set_dimnames(out$df, response_names),
    n_complete = .mc_rmcorr_set_dimnames(out$n_complete, response_names),
    n_subjects = .mc_rmcorr_set_dimnames(out$n_subjects, response_names),
    conf_low = conf_low,
    conf_high = conf_high,
    conf_level = conf_level
  )
  if (identical(estimator, "weighted")) {
    diagnostics$weighted_ss_x <- .mc_rmcorr_set_dimnames(out$weighted_ss_x, response_names)
    diagnostics$weighted_ss_e <- .mc_rmcorr_set_dimnames(out$weighted_ss_e, response_names)
    diagnostics$obs_per_subject_min <- .mc_rmcorr_set_dimnames(out$obs_per_subject_min, response_names)
    diagnostics$obs_per_subject_max <- .mc_rmcorr_set_dimnames(out$obs_per_subject_max, response_names)
    diagnostics$valid <- .mc_rmcorr_set_dimnames(out$valid, response_names)
    diag(diagnostics$valid) <- NA
    diagnostics$estimator <- "weighted"
    diagnostics$ci_method <- ci_method_resolved
  } else if (!identical(ci_method_resolved, "fisher_z")) {
    diagnostics$ci_method <- ci_method_resolved
  }
  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)

  out_obj <- .mc_structure_corr_matrix(
    est,
    class_name = "rmcorr_matrix",
    method = "rmcorr",
    description = "Repeated-measures correlation matrix",
    symmetric = TRUE,
    diagnostics = diagnostics
  )
  if (keep_data) {
    attr(out_obj, "source_data") <- .mc_rmcorr_compact_source_data(
      response_mat = response_mat,
      subject = subject_vec,
      subject_name = resolved$subject_name,
      conf_level = conf_level,
      estimator = estimator,
      ci_method = ci_method_resolved,
      n_boot = if (identical(ci_method_resolved, "bootstrap")) n_boot else NULL
    )
  }
  out_obj
}

#' @rdname rmcorr
#' @export
rmcorr_weighted <- function(data = NULL, response, subject, ...) {
  rmcorr(
    data = data,
    response = response,
    subject = subject,
    estimator = "weighted",
    ...
  )
}

.mc_rmcorr_check_n_boot <- function(n_boot) {
  ok <- length(n_boot) == 1L &&
    is.numeric(n_boot) &&
    !is.na(n_boot) &&
    is.finite(n_boot) &&
    n_boot >= 0 &&
    abs(n_boot - round(n_boot)) <= sqrt(.Machine$double.eps)
  if (!ok) {
    abort_bad_arg("n_boot", message = "must be a non-negative integer.")
  }
  as.integer(n_boot)
}

.mc_rmcorr_resolve_ci_method <- function(estimator, ci_method, n_boot) {
  if (identical(ci_method, "auto")) {
    if (identical(estimator, "ancova")) {
      return("fisher_z")
    }
    return(if (n_boot > 0L) "bootstrap" else "none")
  }
  if (identical(ci_method, "bootstrap") && n_boot <= 0L) {
    abort_bad_arg(
      "n_boot",
      message = "must be positive when {.arg ci_method} is {.val bootstrap}."
    )
  }
  ci_method
}

.mc_rmcorr_pair_stats_dispatch <- function(x, y, subject_code, estimator, conf_level) {
  if (identical(estimator, "weighted")) {
    return(rmcorr_pair_weighted_cpp(
      x = x,
      y = y,
      subject = subject_code,
      conf_level = conf_level
    ))
  }
  rmcorr_pair_cpp(
    x = x,
    y = y,
    subject = subject_code,
    conf_level = conf_level
  )
}

.mc_rmcorr_matrix_stats_dispatch <- function(response_mat, subject_code, estimator,
                                             conf_level, n_threads) {
  if (identical(estimator, "weighted")) {
    return(rmcorr_matrix_weighted_cpp(
      x = response_mat,
      y = response_mat,
      subject = subject_code,
      symmetric = TRUE,
      conf_level = conf_level,
      n_threads = n_threads
    ))
  }
  rmcorr_matrix_cpp(
    x = response_mat,
    y = response_mat,
    subject = subject_code,
    symmetric = TRUE,
    conf_level = conf_level,
    n_threads = n_threads
  )
}

.mc_rmcorr_resample_subject_blocks <- function(dat, draw = NULL, keep_source = FALSE) {
  if (!nrow(dat)) {
    return(list(data = dat, draw = integer()))
  }
  dat$.subject <- droplevels(factor(dat$.subject))
  idx_by_subject <- split(seq_len(nrow(dat)), dat$.subject, drop = TRUE)
  n_subjects <- length(idx_by_subject)
  if (!n_subjects) {
    return(list(data = dat[0, , drop = FALSE], draw = integer()))
  }
  if (is.null(draw)) {
    draw <- sample.int(n_subjects, size = n_subjects, replace = TRUE)
  } else {
    draw <- as.integer(draw)
    bad_draw <- !length(draw) ||
      any(is.na(draw)) ||
      any(draw < 1L | draw > n_subjects, na.rm = TRUE)
    if (bad_draw) {
      abort_bad_arg("draw", message = "contains invalid subject indices.")
    }
  }

  counts <- vapply(idx_by_subject, length, integer(1))
  n_out <- sum(counts[draw], na.rm = TRUE)
  out <- data.frame(
    .response1 = numeric(n_out),
    .response2 = numeric(n_out),
    .subject = integer(n_out),
    check.names = FALSE
  )
  if (isTRUE(keep_source)) {
    out$.subject_source <- character(n_out)
  }

  cursor <- 1L
  subj_levels <- levels(dat$.subject)
  src_row_names <- attr(dat, "row.names")
  if (length(src_row_names) == 2L &&
      is.na(src_row_names[[1L]]) &&
      src_row_names[[2L]] < 0L) {
    src_row_names <- seq_len(nrow(dat))
  }
  out_row_names <- vector(mode = typeof(src_row_names), length = n_out)
  for (b in seq_along(draw)) {
    src <- draw[[b]]
    idx <- idx_by_subject[[src]]
    len <- length(idx)
    if (!len) next
    rng <- seq.int(cursor, cursor + len - 1L)
    out$.response1[rng] <- dat$.response1[idx]
    out$.response2[rng] <- dat$.response2[idx]
    out$.subject[rng] <- b
    out_row_names[rng] <- src_row_names[idx]
    if (isTRUE(keep_source)) {
      out$.subject_source[rng] <- subj_levels[[src]]
    }
    cursor <- cursor + len
  }

  out$.subject <- factor(out$.subject, levels = seq_along(draw))
  if (isTRUE(keep_source)) {
    out$.subject_source <- factor(out$.subject_source, levels = subj_levels)
  }
  attr(out, "row.names") <- out_row_names
  list(data = out, draw = draw)
}

.mc_rmcorr_bootstrap_ci_from_pair_data <- function(dat, estimator, conf_level, n_boot, seed = NULL) {
  if (n_boot <= 0L || !nrow(dat)) {
    return(c(NA_real_, NA_real_))
  }
  dat$.subject <- droplevels(factor(dat$.subject))
  if (nlevels(dat$.subject) < 2L) {
    return(c(NA_real_, NA_real_))
  }
  if (any(table(dat$.subject) < 2L)) {
    return(c(NA_real_, NA_real_))
  }

  boot_vals <- .mc_eval_with_seed(
    seed,
    replicate(
      n_boot,
      {
        sampled <- .mc_rmcorr_resample_subject_blocks(dat)$data
        stats <- .mc_rmcorr_pair_stats_dispatch(
          x = sampled$.response1,
          y = sampled$.response2,
          subject_code = as.integer(sampled$.subject),
          estimator = estimator,
          conf_level = -1
        )
        if (isTRUE(stats$valid) && is.finite(stats$estimate)) {
          as.numeric(stats$estimate)
        } else {
          NA_real_
        }
      }
    )
  )
  .mc_percentile_boot_ci(boot_vals, conf_level = conf_level)
}

.mc_rmcorr_matrix_bootstrap_ci <- function(response_mat,
                                           subject,
                                           estimator,
                                           conf_level,
                                           n_boot,
                                           seed = NULL,
                                           diag_values = NULL) {
  p <- ncol(response_mat)
  nm <- colnames(response_mat)
  out_low <- matrix(NA_real_, nrow = p, ncol = p, dimnames = list(nm, nm))
  out_high <- matrix(NA_real_, nrow = p, ncol = p, dimnames = list(nm, nm))

  if (p < 2L || n_boot <= 0L) {
    if (p >= 1L && !is.null(diag_values)) {
      diag(out_low) <- diag_values
      diag(out_high) <- diag_values
    }
    return(list(conf_low = out_low, conf_high = out_high))
  }

  if (!is.null(diag_values) && length(diag_values) == p) {
    diag(out_low) <- diag_values
    diag(out_high) <- diag_values
  }

  pair_id <- 0L
  for (j in seq_len(p - 1L)) {
    for (k in seq.int(j + 1L, p)) {
      pair_id <- pair_id + 1L
      dat <- .mc_rmcorr_pair_data(response_mat[, c(j, k), drop = FALSE], subject)
      ci_vals <- .mc_rmcorr_bootstrap_ci_from_pair_data(
        dat = dat,
        estimator = estimator,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = .mc_seed_offset(seed, pair_id - 1L)
      )
      out_low[j, k] <- out_low[k, j] <- ci_vals[[1L]]
      out_high[j, k] <- out_high[k, j] <- ci_vals[[2L]]
    }
  }

  list(conf_low = out_low, conf_high = out_high)
}

.mc_rmcorr_resolve_inputs <- function(data, response, subject) {
  pull_data_col <- function(df, x, arg) {
    if (is.character(x) && length(x) >= 1L) {
      missing_cols <- setdiff(x, names(df))
      if (length(missing_cols) > 0L) {
        abort_bad_arg(
          "data",
          message = "is missing required column(s): {paste(missing_cols, collapse = ', ')}.",
          missing_cols = missing_cols
        )
      }
      if (identical(arg, "response")) {
        return(df[x])
      }
      if (length(x) != 1L) {
        abort_bad_arg(
          arg,
          message = "must be a vector or a single column name in {.arg data}."
        )
      }
      return(df[[x]])
    }
    x
  }

  if (!is.null(data)) {
    if (!inherits(data, "data.frame")) {
      data <- as.data.frame(data, stringsAsFactors = FALSE)
    }
    response <- pull_data_col(data, response, "response")
    subject_name <-
      if (is.character(subject) && length(subject) == 1L) subject else "subject"
    subject <- pull_data_col(data, subject, "subject")
  } else {
    subject_name <- "subject"
  }

  response_mat <- .mc_extract_continuous_matrix(response, arg = "response", min_cols = 2L)

  if (!is.atomic(subject) && !is.factor(subject)) {
    abort_bad_arg(
      "subject",
      message = "must be a vector or a single column name in {.arg data}."
    )
  }

  list(
    response = response_mat,
    subject = subject,
    subject_name = subject_name
  )
}

.mc_rmcorr_encode_subject <- function(subject, arg = "subject") {
  if (is.numeric(subject)) {
    bad <- is.na(subject) | is.nan(subject) | is.infinite(subject)
    if (any(bad)) subject[bad] <- NA_real_
  }
  f <- factor(subject)
  if (nlevels(f) < 2L) {
    abort_bad_arg(
      arg,
      message = "must contain at least two distinct subjects."
    )
  }
  list(code = as.integer(f), levels = levels(f))
}

.mc_rmcorr_pair_data <- function(response_mat, subject) {
  dat <- data.frame(
    .response1 = as.numeric(response_mat[, 1L]),
    .response2 = as.numeric(response_mat[, 2L]),
    .subject = subject,
    check.names = FALSE
  )
  dat <- dat[stats::complete.cases(dat), , drop = FALSE]
  if (!nrow(dat)) return(dat)
  counts <- table(dat$.subject)
  keep <- names(counts[counts >= 2L])
  dat <- dat[dat$.subject %in% keep, , drop = FALSE]
  dat$.subject <- droplevels(factor(dat$.subject))
  rownames(dat) <- NULL
  dat
}

.mc_rmcorr_complete_cases <- function(response_mat, subject, min_n = 2L) {
  finite_response <- apply(is.finite(response_mat), 1L, all)
  valid_subject <- !is.na(subject)
  if (is.numeric(subject)) {
    valid_subject <- valid_subject & is.finite(subject)
  }
  keep <- finite_response & valid_subject
  n_original <- nrow(response_mat)
  n_complete <- sum(keep)
  if (n_complete < min_n) {
    abort_bad_arg(
      "response",
      message = "must retain at least {min_n} complete rows when {.arg na_method} is {.val complete}; retained {n_complete}.",
      min_n = min_n,
      n_complete = n_complete
    )
  }
  list(
    response = response_mat[keep, , drop = FALSE],
    subject = subject[keep],
    diagnostics = list(
      na_method = "complete",
      n_original = n_original,
      n_complete = n_complete,
      n_removed = n_original - n_complete,
      complete_rows = keep,
      common_sample = TRUE
    )
  )
}

.mc_rmcorr_build_pair_object <- function(stats, response_mat, subject,
                                         response_names, subject_name,
                                         na_method,
                                         conf_level, keep_data = FALSE,
                                         estimator = NULL,
                                         ci_method = NULL,
                                         n_boot = NULL,
                                         seed = NULL,
                                         diagnostics_extra = NULL) {
  dat <- .mc_rmcorr_pair_data(response_mat, subject)
  slope <- as.numeric(stats$slope)
  valid <- isTRUE(stats$valid) && is.finite(stats$estimate)

  subj_counts <- if (nrow(dat)) as.integer(table(dat$.subject)) else integer()
  payload <- list(
    estimate = as.numeric(stats$estimate),
    p_value = as.numeric(stats$p_value),
    lwr = as.numeric(stats$conf_low),
    upr = as.numeric(stats$conf_high),
    conf_level = conf_level,
    slope = slope,
    df = as.numeric(stats$df),
    n_obs = as.integer(stats$n_complete),
    n_subjects = as.integer(stats$n_subjects),
    na_method = na_method,
    responses = response_names,
    subject_name = subject_name,
    obs_per_subject_min = if (length(subj_counts)) min(subj_counts) else NA_integer_,
    obs_per_subject_max = if (length(subj_counts)) max(subj_counts) else NA_integer_,
    valid = valid
  )

  if (!is.null(estimator)) {
    payload$estimator <- estimator
    payload$ci_method <- ci_method %||% NA_character_
    payload$n_complete <- as.integer(stats$n_complete)
    if (!is.null(n_boot)) payload$n_boot <- as.integer(n_boot)
    if (!is.null(seed)) payload$seed <- as.integer(seed)
    payload$weighted_ss_x <- as.numeric(stats$weighted_ss_x)
    payload$weighted_ss_e <- as.numeric(stats$weighted_ss_e)
  }
  payload <- .mc_merge_diagnostics(payload, diagnostics_extra)

  out <- structure(
    payload,
    class = "rmcorr",
    method = "rmcorr",
    description = "Repeated-measures correlation",
    package = "matrixCorr"
  )
  if (isTRUE(keep_data)) {
    attr(out, "source_data") <- .mc_rmcorr_compact_source_data(
      response_mat = response_mat,
      subject = subject,
      subject_name = subject_name,
      conf_level = conf_level,
      estimator = estimator,
      ci_method = ci_method,
      n_boot = n_boot
    )
  }
  out
}

.mc_rmcorr_set_dimnames <- function(x, nm) {
  x <- as.matrix(x)
  dimnames(x) <- list(nm, nm)
  x
}

.mc_rmcorr_pick_diag_vals <- function(diag_attr, object, field) {
  x <- diag_attr[[field]]
  m <- as.matrix(object)
  if (!is.matrix(x) || !identical(dim(x), dim(m))) return(numeric())
  selector <- if (isTRUE(nrow(m) == ncol(m)) && nrow(m) > 1L) {
    upper.tri(m, diag = FALSE)
  } else {
    matrix(TRUE, nrow(m), ncol(m))
  }
  vals <- x[selector]
  vals[is.finite(vals)]
}

.mc_rmcorr_compact_source_data <- function(response_mat, subject, subject_name,
                                           conf_level,
                                           estimator = NULL,
                                           ci_method = NULL,
                                           n_boot = NULL) {
  f <- factor(subject)
  out <- list(
    response = unname(response_mat),
    response_names = colnames(response_mat),
    subject_code = as.integer(f),
    subject_levels = levels(f),
    subject_name = subject_name,
    conf_level = conf_level
  )
  if (!is.null(estimator)) out$estimator <- estimator
  if (!is.null(ci_method)) out$ci_method <- ci_method
  if (!is.null(n_boot)) out$n_boot <- as.integer(n_boot)
  out
}

.mc_rmcorr_pair_plot_payload <- function(x) {
  source_data <- attr(x, "source_data", exact = TRUE)
  if (!is.list(source_data) || is.null(source_data$response) || is.null(source_data$subject_code)) {
    return(NULL)
  }

  subject <- factor(
    source_data$subject_code,
    levels = seq_along(source_data$subject_levels),
    labels = source_data$subject_levels
  )
  dat <- .mc_rmcorr_pair_data(source_data$response, subject)
  if (!nrow(dat)) {
    return(list(data_long = dat, intercepts = numeric(), fitted = dat[0, c(".subject"), drop = FALSE]))
  }

  slope <- as.numeric(x$estimate) * 0 + as.numeric(x$slope)
  intercepts <- tapply(
    dat$.response2 - slope * dat$.response1,
    dat$.subject,
    mean
  )
  fitted <- do.call(
    rbind,
    lapply(names(intercepts), function(s) {
      idx <- dat$.subject == s
      x_min <- min(dat$.response1[idx])
      x_max <- max(dat$.response1[idx])
      b0 <- as.numeric(intercepts[[s]])
      data.frame(
        .subject = s,
        x = c(x_min, x_max),
        y = c(b0 + slope * x_min, b0 + slope * x_max),
        check.names = FALSE
      )
    })
  )
  list(data_long = dat, intercepts = intercepts, fitted = fitted)
}

.mc_rmcorr_component_names <- function(x) {
  out <- c(
    "estimate", "p_value", "lwr", "upr", "conf_level", "slope", "df",
    "n_obs", "n_subjects", "na_method", "responses", "subject_name",
    "obs_per_subject_min", "obs_per_subject_max", "valid"
  )
  extra <- intersect(
    c(
      "estimator", "ci_method", "n_boot", "seed", "n_complete",
      "weighted_ss_x", "weighted_ss_e"
    ),
    names(unclass(x))
  )
  out <- c(out, extra)
  out <- c(out, "r", "conf_int", "based.on")
  if (!is.null(attr(x, "source_data", exact = TRUE))) {
    out <- c(out, "source_data", "data_long", "intercepts", "fitted")
  }
  unique(out)
}

#' @export
names.rmcorr <- function(x) {
  .mc_rmcorr_component_names(x)
}

#' @export
`$.rmcorr` <- function(x, name) {
  core <- unclass(x)
  switch(name,
    r = core$estimate,
    conf_int = c(lower = core$lwr, upper = core$upr),
    based.on = core$n_obs,
    source_data = attr(x, "source_data", exact = TRUE),
    data_long = {
      payload <- .mc_rmcorr_pair_plot_payload(x)
      if (is.null(payload)) NULL else payload$data_long
    },
    intercepts = {
      payload <- .mc_rmcorr_pair_plot_payload(x)
      if (is.null(payload)) NULL else payload$intercepts
    },
    fitted = {
      payload <- .mc_rmcorr_pair_plot_payload(x)
      if (is.null(payload)) NULL else payload$fitted
    },
    core[[name]]
  )
}

#' @export
`[[.rmcorr` <- function(x, i, ...) {
  if (is.numeric(i)) {
    nms <- names(x)
    if (length(i) != 1L || is.na(i) || i < 1L || i > length(nms)) {
      abort_bad_arg("i", message = "must be a valid component index.")
    }
    i <- nms[[i]]
  }
  `$.rmcorr`(x, i)
}

#' @title Methods for Pairwise rmcorr Objects
#' @description Print, summarize, and plot methods for pairwise
#'   repeated-measures correlation objects of class \code{"rmcorr"} and
#'   \code{"summary.rmcorr"}.
#' @param x An object of class \code{"rmcorr"} or \code{"summary.rmcorr"}.
#' @param object An object of class \code{"rmcorr"}.
#' @param digits Number of significant digits to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param title Optional plot title for \code{plot.rmcorr()}. Defaults to a
#'   title containing the estimated repeated-measures correlation.
#' @param point_alpha Alpha transparency for scatterplot points.
#' @param line_width Line width for subject-specific fitted lines.
#' @param show_legend Logical; if \code{TRUE}, shows the subject legend in the
#'   pairwise scatterplot.
#' @param show_value Logical; included for a consistent plotting interface.
#'   Pairwise repeated-measures plots do not overlay numeric cell values, so this
#'   argument currently has no effect.
#' @param ... Additional arguments passed to downstream methods. For
#'   \code{plot.rmcorr()}, these are passed to \code{ggplot2::theme()}.
#' @rdname rmcorr_methods
#' @method print rmcorr
#' @export
print.rmcorr <- function(x, digits = 4, n = NULL, topn = NULL,
                         max_vars = NULL, width = NULL,
                         show_ci = NULL, ...) {
  check_inherits(x, "rmcorr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )
  digest <- c(
    method = attr(x, "method"),
    if (!is.null(x$estimator)) c(estimator = x$estimator),
    responses = sprintf("%s vs %s", x$responses[[1L]], x$responses[[2L]]),
    n_obs = x$n_obs,
    n_subjects = x$n_subjects,
    estimate = format(signif(x$estimate, digits = digits)),
    if (identical(show_ci, "yes")) {
      c(ci = sprintf(
        "[%s, %s] (%g%%)",
        format(signif(x$lwr, digits = digits)),
        format(signif(x$upr, digits = digits)),
        100 * x$conf_level
      ))
    }
  )
  .mc_print_named_digest(digest, header = "Repeated-measures correlation preview:")
  invisible(x)
}

#' @rdname rmcorr_methods
#' @method summary rmcorr
#' @export
summary.rmcorr <- function(object, n = NULL, topn = NULL,
                           max_vars = NULL, width = NULL,
                           show_ci = NULL, ...) {
  check_inherits(object, "rmcorr")
  ci_method <- object$ci_method %||% "fisher_z"
  out <- list(
    class = "rmcorr",
    method = attr(object, "method"),
    description = attr(object, "description"),
    responses = object$responses,
    n_obs = object$n_obs,
    n_subjects = object$n_subjects,
    df = object$df,
    estimate = object$estimate,
    slope = object$slope,
    p_value = object$p_value,
    lwr = object$lwr,
    upr = object$upr,
    conf_level = object$conf_level,
    obs_per_subject_min = object$obs_per_subject_min %||% NA_integer_,
    obs_per_subject_max = object$obs_per_subject_max %||% NA_integer_,
    n_intercepts = object$n_subjects %||% NA_integer_,
    ci_method = ci_method,
    ci_width = if (all(is.finite(c(object$lwr, object$upr)))) object$upr - object$lwr else NA_real_,
    valid = isTRUE(object$valid)
  )
  if (!is.null(object$estimator)) out$estimator <- object$estimator
  if (!is.null(object$n_complete)) out$n_complete <- object$n_complete
  if (!is.null(object$weighted_ss_x)) out$weighted_ss_x <- object$weighted_ss_x
  if (!is.null(object$weighted_ss_e)) out$weighted_ss_e <- object$weighted_ss_e
  if (!is.null(object$n_boot)) out$n_boot <- object$n_boot
  if (!is.null(object$seed)) out$seed <- object$seed
  .mc_finalize_summary_list(out, class_name = "summary.rmcorr")
}

#' @rdname rmcorr_methods
#' @method print summary.rmcorr
#' @export
print.summary.rmcorr <- function(x, digits = 4, n = NULL, topn = NULL,
                                 max_vars = NULL, width = NULL,
                                 show_ci = NULL, ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  digest <- c(
    method = x$method,
    if (!is.null(x$estimator)) c(estimator = x$estimator),
    responses = sprintf("%s vs %s", x$responses[[1L]], x$responses[[2L]]),
    n_obs = x$n_obs,
    n_subjects = x$n_subjects,
    obs_per_subject = .mc_format_scalar_or_range(
      x$obs_per_subject_min,
      x$obs_per_subject_max,
      digits = digits,
      integer = TRUE
    ),
    df = format(signif(x$df, digits = digits)),
    estimate = format(signif(x$estimate, digits = digits)),
    slope = format(signif(x$slope, digits = digits)),
    p_value = format(signif(x$p_value, digits = digits)),
    if (identical(show_ci, "yes")) {
      c(
        interval = sprintf(
          "[%s, %s]",
          format(signif(x$lwr, digits = digits)),
          format(signif(x$upr, digits = digits))
        ),
        ci_level = sprintf("%g%%", 100 * x$conf_level),
        ci_method = x$ci_method,
        ci_width = format(signif(x$ci_width, digits = digits))
      )
    }
  )
  if (is.finite(x$n_intercepts) && x$n_intercepts > 0L) {
    digest <- c(digest, fitted_subjects = x$n_intercepts)
  }
  if (!is.null(x$n_complete) && is.finite(x$n_complete)) {
    digest <- c(digest, n_complete = x$n_complete)
  }
  if (!is.null(x$weighted_ss_x) && is.finite(x$weighted_ss_x)) {
    digest <- c(digest, weighted_ss_x = format(signif(x$weighted_ss_x, digits = digits)))
  }
  if (!is.null(x$weighted_ss_e) && is.finite(x$weighted_ss_e)) {
    digest <- c(digest, weighted_ss_e = format(signif(x$weighted_ss_e, digits = digits)))
  }
  if (!is.null(x$n_boot) && is.finite(x$n_boot)) {
    digest <- c(digest, n_boot = x$n_boot)
  }
  if (!isTRUE(x$valid)) {
    digest <- c(digest, valid = "no")
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = "Repeated-measures correlation summary:")
  invisible(x)
}

#' @rdname rmcorr_methods
#' @method plot rmcorr
#' @export
plot.rmcorr <- function(x,
                        title = NULL,
                        point_alpha = 0.8,
                        line_width = 0.8,
                        show_legend = FALSE,
                        show_value = TRUE,
                        ...) {
  check_inherits(x, "rmcorr")
  check_bool(show_value, arg = "show_value")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }
  payload <- .mc_rmcorr_pair_plot_payload(x)
  if (is.null(payload)) {
    cli::cli_abort(
      c(
        "The repeated-measures correlation fit does not retain plotting data.",
        "i" = "Refit with {.code keep_data = TRUE} to enable {.fn plot} for pairwise repeated-measures correlations."
      )
    )
  }
  if (!nrow(payload$data_long) || !isTRUE(x$valid)) {
    cli::cli_abort("The repeated-measures correlation fit is not valid for plotting.")
  }

  title <- title %||% sprintf(
    "Repeated-measures correlation: r = %.2f",
    x$r
  )

  p <- ggplot2::ggplot(
    payload$data_long,
    ggplot2::aes(
      x = .data$.response1,
      y = .data$.response2,
      colour = .data$.subject
    )
  ) +
    ggplot2::geom_point(alpha = point_alpha) +
    ggplot2::geom_line(
      data = payload$fitted,
      ggplot2::aes(x = .data$x, y = .data$y, colour = .data$.subject, group = .data$.subject),
      linewidth = line_width,
      inherit.aes = FALSE
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      legend.position = if (show_legend) "right" else "none",
      ...
    ) +
    ggplot2::labs(
      title = title,
      x = x$responses[[1L]],
      y = x$responses[[2L]],
      colour = x$subject_name
    )

  p
}

#' @title Methods for rmcorr Matrix Objects
#' @description Print, summarize, and plot methods for repeated-measures
#'   correlation matrix objects of class \code{"rmcorr_matrix"} and
#'   \code{"summary.rmcorr_matrix"}.
#' @param x An object of class \code{"rmcorr_matrix"} or
#'   \code{"summary.rmcorr_matrix"}.
#' @param object An object of class \code{"rmcorr_matrix"}.
#' @param digits Number of significant digits to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param title Plot title for \code{plot.rmcorr_matrix()}.
#' @param low_color,high_color,mid_color Colours used for negative, positive,
#'   and midpoint values in the heatmap.
#' @param value_text_size Size of the overlaid numeric value labels in the
#'   heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on heatmap tiles when the plot type supports them.
#' @param ... Additional arguments passed to downstream methods.
#' @rdname rmcorr_matrix_methods
#' @method print rmcorr_matrix
#' @export
print.rmcorr_matrix <- function(x, digits = 4, n = NULL,
                                topn = NULL, max_vars = NULL,
                                width = NULL, show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Repeated-measures correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname rmcorr_matrix_methods
#' @method plot rmcorr_matrix
#' @export
plot.rmcorr_matrix <- function(x, title = "Repeated-measures correlation heatmap",
                               low_color = "indianred1",
                               high_color = "steelblue1",
                               mid_color = "white",
                               value_text_size = 4,
                               show_value = TRUE, ...) {
  .mc_plot_corr_matrix(
    x,
    class_name = "rmcorr_matrix",
    fill_name = "rmcorr",
    title = title,
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    show_value = show_value,
    ...
  )
}

#' @rdname rmcorr_matrix_methods
#' @method summary rmcorr_matrix
#' @export
summary.rmcorr_matrix <- function(object, n = NULL, topn = NULL,
                                  max_vars = NULL, width = NULL,
                                  show_ci = NULL, ...) {
  check_inherits(object, "rmcorr_matrix")
  diag_attr <- attr(object, "diagnostics")
  m <- as.matrix(object)
  symmetric <- isTRUE(nrow(m) == ncol(m)) && isTRUE(isSymmetric(unclass(m)))
  vals <- if (symmetric && nrow(m) > 1L) {
    m[upper.tri(m, diag = FALSE)]
  } else {
    as.vector(m)
  }
  vals <- vals[is.finite(vals)]
  n_pairs <- if (symmetric) {
    stats::setNames(nrow(m) * (nrow(m) - 1L) / 2L, NULL)
  } else {
    stats::setNames(nrow(m) * ncol(m), NULL)
  }

  out <- list(
    class = class(object)[1L],
    method = attr(object, "method"),
    description = attr(object, "description"),
    estimator = diag_attr$estimator %||% "ancova",
    ci_method = diag_attr$ci_method %||% "fisher_z",
    n_variables = if (symmetric) nrow(m) else NA_integer_,
    n_pairs = as.integer(n_pairs),
    n_rows = nrow(m),
    n_cols = ncol(m),
    symmetric = symmetric,
    n_missing = sum(is.na(m)),
    estimate_min = if (length(vals)) min(vals) else NA_real_,
    estimate_max = if (length(vals)) max(vals) else NA_real_,
    header = "Repeated-measures correlation summary"
  )
  out$df_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "df")
    if (length(vals)) min(vals) else NA_real_
  }
  out$df_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "df")
    if (length(vals)) max(vals) else NA_real_
  }
  out$n_complete_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "n_complete")
    if (length(vals)) min(vals) else NA_real_
  }
  out$n_complete_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "n_complete")
    if (length(vals)) max(vals) else NA_real_
  }
  out$n_subjects_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "n_subjects")
    if (length(vals)) min(vals) else NA_real_
  }
  out$n_subjects_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "n_subjects")
    if (length(vals)) max(vals) else NA_real_
  }
  out$p_value_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "p_value")
    if (length(vals)) min(vals) else NA_real_
  }
  out$p_value_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "p_value")
    if (length(vals)) max(vals) else NA_real_
  }
  out$conf_level <- diag_attr$conf_level %||% NA_real_
  out$weighted_ss_x_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "weighted_ss_x")
    if (length(vals)) min(vals) else NA_real_
  }
  out$weighted_ss_x_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "weighted_ss_x")
    if (length(vals)) max(vals) else NA_real_
  }
  out$weighted_ss_e_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "weighted_ss_e")
    if (length(vals)) min(vals) else NA_real_
  }
  out$weighted_ss_e_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "weighted_ss_e")
    if (length(vals)) max(vals) else NA_real_
  }
  out$obs_per_subject_min <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "obs_per_subject_min")
    if (length(vals)) min(vals) else NA_real_
  }
  out$obs_per_subject_max <- {
    vals <- .mc_rmcorr_pick_diag_vals(diag_attr, object, "obs_per_subject_max")
    if (length(vals)) max(vals) else NA_real_
  }
  out$valid_rate <- {
    vv <- diag_attr$valid
    if (is.matrix(vv) && identical(dim(vv), dim(m))) {
      selector <- if (symmetric && nrow(m) > 1L) upper.tri(m, diag = FALSE) else matrix(TRUE, nrow(m), ncol(m))
      vals <- as.logical(vv[selector])
      vals <- vals[!is.na(vals)]
      if (length(vals)) mean(vals) else NA_real_
    } else {
      NA_real_
    }
  }
  out$top_results <- .mc_summary_top_pairs(object, digits = 4, topn = .mc_coalesce(topn, .mc_display_option("summary_topn", 5L)))
  .mc_finalize_summary_list(out, class_name = "summary.rmcorr_matrix")
}

#' @rdname rmcorr_matrix_methods
#' @method print summary.rmcorr_matrix
#' @export
print.summary.rmcorr_matrix <- function(x, digits = 4, n = NULL,
                                        topn = NULL, max_vars = NULL,
                                        width = NULL, show_ci = NULL, ...) {
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  digest <- c(
    method = x$method,
    estimator = x$estimator %||% "ancova",
    dimensions = sprintf("%d x %d", x$n_rows, x$n_cols),
    pairs = .mc_count_fmt(x$n_pairs),
    estimate = .mc_format_scalar_or_range(x$estimate_min, x$estimate_max, digits = digits)
  )
  if (!is.null(x$ci_method) && nzchar(x$ci_method)) {
    digest <- c(digest, ci_method = x$ci_method)
  }
  if (!is.na(x$n_variables)) digest <- c(digest, n_variables = x$n_variables)
  if (is.finite(x$conf_level)) digest <- c(digest, conf_level = format(signif(x$conf_level, digits = digits)))
  if (is.finite(x$n_complete_min) && is.finite(x$n_complete_max)) {
    digest <- c(digest, n_complete = .mc_format_scalar_or_range(x$n_complete_min, x$n_complete_max, digits = digits))
  }
  if (is.finite(x$df_min) && is.finite(x$df_max)) {
    digest <- c(digest, df = .mc_format_scalar_or_range(x$df_min, x$df_max, digits = digits))
  }
  if (is.finite(x$n_subjects_min) && is.finite(x$n_subjects_max)) {
    digest <- c(digest, n_subjects = .mc_format_scalar_or_range(x$n_subjects_min, x$n_subjects_max, digits = digits))
  }
  if (is.finite(x$p_value_min) && is.finite(x$p_value_max)) {
    digest <- c(digest, p_value = .mc_format_scalar_or_range(x$p_value_min, x$p_value_max, digits = digits))
  }
  if (is.finite(x$obs_per_subject_min) && is.finite(x$obs_per_subject_max)) {
    digest <- c(digest, obs_per_subject = .mc_format_scalar_or_range(x$obs_per_subject_min, x$obs_per_subject_max, digits = digits))
  }
  if (is.finite(x$weighted_ss_x_min) && is.finite(x$weighted_ss_x_max)) {
    digest <- c(digest, weighted_ss_x = .mc_format_scalar_or_range(x$weighted_ss_x_min, x$weighted_ss_x_max, digits = digits))
  }
  if (is.finite(x$weighted_ss_e_min) && is.finite(x$weighted_ss_e_max)) {
    digest <- c(digest, weighted_ss_e = .mc_format_scalar_or_range(x$weighted_ss_e_min, x$weighted_ss_e_max, digits = digits))
  }
  if (is.finite(x$valid_rate)) {
    digest <- c(digest, valid_rate = sprintf("%.1f%%", 100 * x$valid_rate))
  }
  if (isTRUE(x$n_missing > 0L)) {
    digest <- c(digest, missing = .mc_count_fmt(x$n_missing))
  }
  .mc_print_named_digest(digest, header = x$header %||% "Repeated-measures correlation summary")
  if (is.data.frame(x$top_results) && nrow(x$top_results)) {
    .mc_print_ranked_pairs_preview(
      x$top_results,
      header = "Strongest pairs by |estimate|",
      topn = cfg$topn,
      max_vars = cfg$max_vars,
      width = cfg$width,
      digits = digits,
      ci_digits = .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits),
      show_ci = cfg$show_ci,
      ...
    )
  }
  invisible(x)
}

print.summary_rmcorr <- print.summary.rmcorr
print.summary_rmcorr_matrix <- print.summary.rmcorr_matrix

manual_rmcorr <- function(x, y, subject, conf_level = 0.95) {
  dat <- data.frame(x = x, y = y, subject = subject, check.names = FALSE)
  dat <- dat[stats::complete.cases(dat), , drop = FALSE]
  if (!nrow(dat)) {
    return(list(valid = FALSE))
  }

  counts <- table(dat$subject)
  keep <- names(counts[counts >= 2L])
  dat <- dat[dat$subject %in% keep, , drop = FALSE]
  dat$subject <- droplevels(factor(dat$subject))

  n_complete <- nrow(dat)
  n_subjects <- nlevels(dat$subject)
  if (n_subjects < 2L) {
    return(list(valid = FALSE, n_complete = n_complete, n_subjects = n_subjects))
  }

  xc <- dat$x - ave(dat$x, dat$subject)
  yc <- dat$y - ave(dat$y, dat$subject)
  sxx <- sum(xc * xc)
  syy <- sum(yc * yc)
  sxy <- sum(xc * yc)
  r <- sxy / sqrt(sxx * syy)
  slope <- sxy / sxx
  df <- n_complete - n_subjects - 1L
  sse <- syy - (sxy * sxy) / sxx
  if (sse < 0 && sse > -1e-12) {
    sse <- 0
  }
  mse <- sse / df
  se_slope <- sqrt(mse / sxx)
  t_value <- if (se_slope == 0) {
    if (slope == 0) 0 else Inf
  } else {
    slope / se_slope
  }
  p_value <- 2 * stats::pt(-abs(t_value), df = df)

  ci <- c(lower = NA_real_, upper = NA_real_)
  eff_n <- df + 2
  if (abs(r) >= 1) {
    ci[] <- r
  } else if (eff_n > 3) {
    zr <- atanh(r)
    zcrit <- stats::qnorm(0.5 * (1 + conf_level))
    se_z <- 1 / sqrt(eff_n - 3)
    ci <- c(
      lower = tanh(zr - zcrit * se_z),
      upper = tanh(zr + zcrit * se_z)
    )
  }

  list(
    valid = TRUE,
    r = r,
    slope = slope,
    p_value = p_value,
    conf_int = ci,
    df = df,
    n_complete = n_complete,
    n_subjects = n_subjects
  )
}

sim_rmcorr_long <- function(seed = 2026L) {
  set.seed(seed)
  n_subjects <- 18L
  n_rep <- 4L
  subject <- factor(rep(seq_len(n_subjects), each = n_rep))
  subj_x <- rnorm(n_subjects, sd = 1.2)
  subj_y <- rnorm(n_subjects, sd = 1.5)
  signal <- rnorm(n_subjects * n_rep)

  data.frame(
    subject = subject,
    x = subj_x[subject] + signal + rnorm(n_subjects * n_rep, sd = 0.15),
    y = subj_y[subject] + 0.7 * signal + rnorm(n_subjects * n_rep, sd = 0.2),
    z = subj_y[subject] - 0.4 * signal + rnorm(n_subjects * n_rep, sd = 0.25),
    check.names = FALSE
  )
}

test_that("rmcorr pair interface matches manual within-subject formulas", {
  dat <- sim_rmcorr_long()
  ref <- manual_rmcorr(dat$x, dat$y, dat$subject)

  fit <- rmcorr(dat, response = c("x", "y"), subject = "subject")
  fit_positional <- rmcorr(dat[c("x", "y")], dat$subject)

  expect_s3_class(fit, "rmcorr")
  expect_equal(fit$r, ref$r, tolerance = 1e-10)
  expect_equal(fit$estimate, ref$r, tolerance = 1e-10)
  expect_equal(fit$slope, ref$slope, tolerance = 1e-10)
  expect_equal(fit$p_value, ref$p_value, tolerance = 1e-10)
  expect_equal(fit$conf_int, ref$conf_int, tolerance = 1e-10)
  expect_equal(c(fit$lwr, fit$upr), unname(ref$conf_int), tolerance = 1e-10)
  expect_equal(fit$df, ref$df)
  expect_equal(fit$based.on, ref$n_complete)
  expect_equal(fit$n_obs, ref$n_complete)
  expect_equal(fit$n_subjects, ref$n_subjects)
  expect_equal(fit$r, fit_positional$r, tolerance = 1e-10)
  expect_equal(fit$slope, fit_positional$slope, tolerance = 1e-10)

  sm <- summary(fit)
  expect_s3_class(sm, "summary.rmcorr")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_true(any(grepl("Repeated-measures correlation", capture.output(print(fit)))))
  sum_out <- capture.output(matrixCorr:::print.summary.rmcorr(sm))
  expect_true(any(grepl("^Repeated-measures correlation summary:", sum_out)))
  expect_true(any(grepl("obs_per_subject", sum_out, fixed = TRUE)))
  expect_true(any(grepl("ci_method", sum_out, fixed = TRUE)))
  expect_named(sm, c(
    "class", "method", "description", "responses", "n_obs", "n_subjects", "df",
    "estimate", "slope", "p_value", "lwr", "upr", "conf_level",
    "obs_per_subject_min", "obs_per_subject_max", "n_intercepts",
    "ci_method", "ci_width", "valid"
  ))
  expect_equal(fit$r, fit$estimate, tolerance = 1e-12)
  expect_equal(fit$conf_int, c(lower = fit$lwr, upper = fit$upr), tolerance = 1e-12)
  expect_equal(fit$based.on, fit$n_obs)
  expect_false(any(c("r", "conf_int", "based.on", "data_long", "intercepts", "fitted") %in% names(unclass(fit))))
  expect_null(attr(fit, "source_data", exact = TRUE))
  expect_error(plot(fit), "keep_data = TRUE", fixed = TRUE)

  fit_keep <- rmcorr(dat, response = c("x", "y"), subject = "subject", keep_data = TRUE)
  expect_s3_class(plot(fit_keep), "ggplot")
  expect_true(is.list(attr(fit_keep, "source_data", exact = TRUE)))
})

test_that("rmcorr matrix output is symmetric and agrees with pair fits", {
  dat <- sim_rmcorr_long(seed = 44L)

  fit_xy <- rmcorr(dat, response = c("x", "y"), subject = "subject")
  fit_xz <- rmcorr(dat, response = c("x", "z"), subject = "subject")
  fit_mat <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject", n_threads = 1L)

  expect_s3_class(fit_mat, "rmcorr_matrix")
  expect_equal(dim(fit_mat), c(3L, 3L))
  expect_equal(unname(diag(fit_mat)), c(1, 1, 1), tolerance = 1e-12)
  expect_equal(unclass(fit_mat), t(unclass(fit_mat)), tolerance = 1e-10)
  expect_equal(unname(fit_mat["x", "y"]), fit_xy$r, tolerance = 1e-10)
  expect_equal(unname(fit_mat["x", "z"]), fit_xz$r, tolerance = 1e-10)

  diag_attr <- attr(fit_mat, "diagnostics")
  expect_true(is.list(diag_attr))
  expect_equal(unname(diag_attr$slope["x", "y"]), fit_xy$slope, tolerance = 1e-10)
  expect_equal(unname(diag_attr$n_complete["x", "y"]), fit_xy$based.on)
  expect_equal(unname(diag_attr$n_subjects["x", "z"]), fit_xz$n_subjects)

  sm <- summary(fit_mat)
  expect_s3_class(sm, "summary.rmcorr_matrix")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_true(any(grepl("Repeated-measures correlation matrix", capture.output(print(fit_mat)))))
  expect_s3_class(plot(fit_mat), "ggplot")
  expect_null(attr(fit_mat, "source_data", exact = TRUE))
})

test_that("rmcorr matrix stores compact source data only when requested", {
  dat <- sim_rmcorr_long(seed = 77L)
  fit_mat <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    keep_data = TRUE,
    n_threads = 1L
  )

  source_data <- attr(fit_mat, "source_data", exact = TRUE)
  expect_true(is.list(source_data))
  expect_equal(dim(source_data$response), c(nrow(dat), 3L))
  expect_null(colnames(source_data$response))
  expect_equal(source_data$response_names, c("x", "y", "z"))
  expect_equal(source_data$subject_code, as.integer(dat$subject))
  expect_equal(source_data$subject_levels, levels(dat$subject))
  expect_equal(source_data$subject_name, "subject")
  expect_equal(source_data$conf_level, 0.95)
})

test_that("rmcorr pair keeps compact source data only when requested", {
  dat <- sim_rmcorr_long(seed = 79L)
  fit_base <- rmcorr(dat, response = c("x", "y"), subject = "subject")
  fit_keep <- rmcorr(dat, response = c("x", "y"), subject = "subject", keep_data = TRUE)

  expect_null(attr(fit_base, "source_data", exact = TRUE))
  expect_true(is.list(attr(fit_keep, "source_data", exact = TRUE)))
  expect_lt(as.numeric(object.size(fit_base)), as.numeric(object.size(fit_keep)))
})

test_that("rmcorr check_na = FALSE uses pairwise complete cases", {
  dat <- data.frame(
    subject = c(1, 1, 2, 2, 2, 3, 3),
    x = c(1, 2, 3, 4, 5, 1, 2),
    y = c(2, 4, NA, 8, 10, 2, 4),
    z = c(1, 3, 3, 5, NA, 2, 5)
  )

  expect_error(
    rmcorr(dat, response = c("x", "y"), subject = "subject", na_method = "error"),
    "missing, NaN, or infinite values"
  )

  ref_xy <- manual_rmcorr(dat$x, dat$y, dat$subject)
  ref_xz <- manual_rmcorr(dat$x, dat$z, dat$subject)

  expect_warning(
    fit_xy <- rmcorr(dat, response = c("x", "y"), subject = "subject", check_na = FALSE),
    "deprecated"
  )
  expect_warning(
    fit_mat <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject", check_na = FALSE),
    "deprecated"
  )

  expect_equal(fit_xy$based.on, ref_xy$n_complete)
  expect_equal(fit_xy$n_obs, ref_xy$n_complete)
  expect_equal(fit_xy$n_subjects, ref_xy$n_subjects)
  expect_equal(fit_xy$r, ref_xy$r, tolerance = 1e-10)

  diag_attr <- attr(fit_mat, "diagnostics")
  expect_equal(unname(diag_attr$n_complete["x", "y"]), ref_xy$n_complete)
  expect_equal(unname(diag_attr$n_complete["x", "z"]), ref_xz$n_complete)
  expect_equal(unname(fit_mat["x", "z"]), ref_xz$r, tolerance = 1e-10)
})

test_that("rmcorr canonical na_method matches legacy check_na behavior", {
  dat <- data.frame(
    subject = c(1, 1, 2, 2, 2, 3, 3),
    x = c(1, 2, 3, 4, 5, 1, 2),
    y = c(2, 4, NA, 8, 10, 2, 4)
  )

  fit_new <- rmcorr(dat, response = c("x", "y"), subject = "subject", na_method = "pairwise")
  expect_warning(
    fit_old <- rmcorr(dat, response = c("x", "y"), subject = "subject", check_na = FALSE),
    "deprecated"
  )

  expect_equal(fit_new$estimate, fit_old$estimate, tolerance = 1e-12)
  expect_equal(fit_new$n_obs, fit_old$n_obs)
  expect_equal(fit_new$n_subjects, fit_old$n_subjects)
})

test_that("rmcorr validates subject structure", {
  dat <- data.frame(
    subject = rep(1, 4),
    x = c(1, 2, 3, 4),
    y = c(2, 3, 4, 5)
  )

  expect_error(
    rmcorr(dat, response = c("x", "y"), subject = "subject"),
    "at least two distinct subjects"
  )
})

test_that("rmcorr viewer helpers preserve source data and rebuild pair fits", {
  set.seed(2026)
  n_subjects <- 12
  n_rep <- 4
  subject <- rep(seq_len(n_subjects), each = n_rep)
  subj_eff_x <- rnorm(n_subjects, sd = 1.2)
  subj_eff_y <- rnorm(n_subjects, sd = 1.5)
  within_signal <- rnorm(n_subjects * n_rep)

  dat <- data.frame(
    subject = subject,
    x = subj_eff_x[subject] + within_signal + rnorm(n_subjects * n_rep, sd = 0.2),
    y = subj_eff_y[subject] + 0.7 * within_signal + rnorm(n_subjects * n_rep, sd = 0.3),
    z = subj_eff_y[subject] - 0.4 * within_signal + rnorm(n_subjects * n_rep, sd = 0.4)
  )

  fit_mat <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    keep_data = TRUE
  )
  prepared <- matrixCorr:::`.mc_prepare_rmcorr_inputs`(fit_mat)

  expect_length(prepared, 1L)
  expect_true(inherits(prepared$default$matrix, "matrix"))
  expect_null(colnames(prepared$default$source_data$response))
  expect_equal(prepared$default$source_data$response_names, c("x", "y", "z"))
  expect_equal(prepared$default$source_data$subject_code, as.integer(dat$subject))
  expect_equal(prepared$default$source_data$subject_levels, as.character(seq_len(n_subjects)))
  expect_equal(prepared$default$n_subjects, n_subjects)

  fit_xy_direct <- rmcorr(dat, response = c("x", "y"), subject = "subject")
  fit_xy_view <- matrixCorr:::`.mc_rmcorr_view_pair_fit`(
    prepared$default,
    x_var = "x",
    y_var = "y"
  )

  expect_s3_class(fit_xy_view, "rmcorr")
  expect_equal(fit_xy_view$responses, c("x", "y"))
  expect_equal(fit_xy_view$r, fit_xy_direct$r, tolerance = 1e-12)
  expect_equal(fit_xy_view$slope, fit_xy_direct$slope, tolerance = 1e-12)
  expect_equal(fit_xy_view$p_value, fit_xy_direct$p_value, tolerance = 1e-12)
  expect_equal(fit_xy_view$based.on, fit_xy_direct$based.on)
  expect_equal(fit_xy_view$n_subjects, fit_xy_direct$n_subjects)
})

test_that("rmcorr viewer parser rejects matrix objects without retained data", {
  dat <- data.frame(
    subject = rep(1:3, each = 3),
    x = rnorm(9),
    y = rnorm(9),
    z = rnorm(9)
  )
  bad <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject")

  expect_error(
    matrixCorr:::`.mc_parse_rmcorr_object`(bad, label = "bad"),
    "keep_data = TRUE"
  )
})

reference_rmcorr_weighted <- function(data, x, y, subject) {
  dat <- data.frame(
    x = data[[x]],
    y = data[[y]],
    subject = data[[subject]]
  )
  dat <- dat[stats::complete.cases(dat), , drop = FALSE]
  dat$subject <- factor(dat$subject)

  counts <- table(dat$subject)
  keep <- names(counts[counts >= 2L])
  dat <- dat[dat$subject %in% keep, , drop = FALSE]
  dat$subject <- droplevels(dat$subject)

  if (nlevels(dat$subject) < 2L) {
    return(list(
      estimate = NA_real_,
      slope = NA_real_,
      weighted_ss_x = NA_real_,
      weighted_ss_e = NA_real_,
      n_complete = nrow(dat),
      n_subjects = nlevels(dat$subject),
      valid = FALSE
    ))
  }

  df <- nrow(dat) - nlevels(dat$subject) - 1L
  if (df <= 0L) {
    return(list(
      estimate = NA_real_,
      slope = NA_real_,
      weighted_ss_x = NA_real_,
      weighted_ss_e = NA_real_,
      n_complete = nrow(dat),
      n_subjects = nlevels(dat$subject),
      valid = FALSE
    ))
  }

  fit_null <- stats::lm(y ~ subject, data = dat)
  fit_full <- stats::lm(y ~ x + subject, data = dat)

  beta <- unname(stats::coef(fit_full)[["x"]])

  e_null <- stats::residuals(fit_null)
  e_full <- stats::residuals(fit_full)

  dat$qx <- e_null^2 - e_full^2
  dat$qe <- e_full^2

  by_subject <- stats::aggregate(
    cbind(qx, qe) ~ subject,
    data = dat,
    FUN = mean
  )

  weighted_ss_x <- sum(by_subject$qx)
  weighted_ss_e <- sum(by_subject$qe)
  denom <- weighted_ss_x + weighted_ss_e

  if (!is.finite(beta) ||
      !is.finite(weighted_ss_x) ||
      !is.finite(weighted_ss_e) ||
      denom <= 0) {
    est <- NA_real_
    valid <- FALSE
  } else {
    if (weighted_ss_x < 0 && weighted_ss_x > -sqrt(.Machine$double.eps)) {
      weighted_ss_x <- 0
    }
    if (weighted_ss_x < 0) {
      est <- NA_real_
      valid <- FALSE
    } else {
      est <- sign(beta) * sqrt(weighted_ss_x / denom)
      est <- max(-1, min(1, est))
      valid <- is.finite(est)
    }
  }

  list(
    estimate = est,
    slope = beta,
    weighted_ss_x = weighted_ss_x,
    weighted_ss_e = weighted_ss_e,
    n_complete = nrow(dat),
    n_subjects = nlevels(dat$subject),
    df = df,
    valid = valid
  )
}

make_balanced_complete_data <- function() {
  data.frame(
    subject = rep(1:4, each = 4),
    x = rep(c(-1, 0, 1, 2), times = 4) + rep(c(0.0, 0.2, -0.1, 0.15), each = 4),
    y = rep(c(10, 11, 12, 13), each = 4) +
      1.25 * rep(c(-1, 0, 1, 2), times = 4) +
      rep(c(0.10, -0.05, 0.03, -0.02), times = 4),
    z = rep(c(4, 6, 5, 7), each = 4) -
      0.75 * rep(c(-1, 0, 1, 2), times = 4)
  )
}

make_unbalanced_weight_data <- function() {
  data.frame(
    subject = c(
      rep("A", 8),
      rep("B", 2),
      rep("C", 2)
    ),
    x = c(1:8, 1, 2, 1, 2),
    y = c(1:8, 2, 1, 2, 1)
  )
}

test_that("rmcorr default ANCOVA output matches explicit ancova configuration", {
  dat <- make_balanced_complete_data()

  fit_default <- rmcorr(dat, response = c("x", "y"), subject = "subject")
  fit_ancova <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "ancova",
    ci_method = "auto"
  )

  expect_identical(unclass(fit_default), unclass(fit_ancova))
  expect_identical(attributes(fit_default), attributes(fit_ancova))

  mat_default <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject", n_threads = 1L)
  mat_ancova <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    estimator = "ancova",
    ci_method = "auto",
    n_threads = 1L
  )
  expect_equal(unclass(mat_default), unclass(mat_ancova), tolerance = 1e-12)
  expect_equal(attr(mat_default, "diagnostics"), attr(mat_ancova, "diagnostics"))
})

test_that("weighted rmcorr matches lm reference implementation exactly", {
  dat <- data.frame(
    subject = c(
      rep("S1", 5),
      rep("S2", 3),
      rep("S3", 2),
      rep("S4", 4)
    ),
    x = c(1, 2, 3, 4, 5, 2, 3, 4, 3, 5, 1, 2, 4, 6),
    y = c(2, 4, 6, 7, 10, 4, 5, 8, 5, 9, 2, 4, 7, 11)
  )

  ref <- reference_rmcorr_weighted(dat, "x", "y", "subject")
  fit <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )

  expect_equal(fit$estimate, ref$estimate, tolerance = 1e-10)
  expect_equal(fit$slope, ref$slope, tolerance = 1e-10)
  expect_equal(fit$weighted_ss_x, ref$weighted_ss_x, tolerance = 1e-10)
  expect_equal(fit$weighted_ss_e, ref$weighted_ss_e, tolerance = 1e-10)
  expect_equal(fit$n_complete, ref$n_complete)
  expect_equal(fit$n_subjects, ref$n_subjects)
  expect_equal(fit$df, ref$df)
  expect_identical(fit$valid, ref$valid)
})

test_that("weighted equals ancova on balanced complete repeated data", {
  dat <- make_balanced_complete_data()
  fit_ancova <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "ancova",
    ci_method = "none"
  )
  fit_weighted <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )

  expect_equal(fit_weighted$estimate, fit_ancova$estimate, tolerance = 1e-12)
})

test_that("weighted differs from ancova on unbalanced subject counts", {
  dat <- make_unbalanced_weight_data()

  fit_weighted <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )
  fit_ancova <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "ancova",
    ci_method = "none"
  )
  ref <- reference_rmcorr_weighted(dat, "x", "y", "subject")

  expect_equal(fit_weighted$estimate, ref$estimate, tolerance = 1e-10)
  expect_equal(fit_weighted$weighted_ss_x, ref$weighted_ss_x, tolerance = 1e-10)
  expect_equal(fit_weighted$weighted_ss_e, ref$weighted_ss_e, tolerance = 1e-10)
  expect_false(isTRUE(all.equal(fit_weighted$estimate, fit_ancova$estimate, tolerance = 1e-8)))
})

test_that("weighted matrix mode applies pairwise filtering per response pair", {
  dat <- data.frame(
    subject = rep(1:4, each = 3),
    x = c(1, 2, 3, 1, 3, 5, 2, 4, 6, 1, 2, 3),
    y = c(2, 4, NA, 1, 2, 3, 4, 6, 8, 2, NA, 4),
    z = c(NA, 3, 4, 2, 4, 6, 3, 5, NA, 1, 2, 4)
  )

  fit <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    estimator = "weighted",
    na_method = "pairwise",
    ci_method = "none",
    n_threads = 1L
  )
  diag_attr <- attr(fit, "diagnostics")

  ref_xy <- reference_rmcorr_weighted(dat, "x", "y", "subject")
  ref_xz <- reference_rmcorr_weighted(dat, "x", "z", "subject")
  ref_yz <- reference_rmcorr_weighted(dat, "y", "z", "subject")

  expect_equal(unname(fit["x", "y"]), ref_xy$estimate, tolerance = 1e-10)
  expect_equal(unname(fit["x", "z"]), ref_xz$estimate, tolerance = 1e-10)
  expect_equal(unname(fit["y", "z"]), ref_yz$estimate, tolerance = 1e-10)

  expect_equal(unname(diag_attr$n_complete["x", "y"]), ref_xy$n_complete)
  expect_equal(unname(diag_attr$n_complete["x", "z"]), ref_xz$n_complete)
  expect_equal(unname(diag_attr$n_complete["y", "z"]), ref_yz$n_complete)

  expect_equal(unname(diag_attr$n_subjects["x", "y"]), ref_xy$n_subjects)
  expect_equal(unname(diag_attr$n_subjects["x", "z"]), ref_xz$n_subjects)
  expect_equal(unname(diag_attr$n_subjects["y", "z"]), ref_yz$n_subjects)

  pair_n <- c(
    unname(diag_attr$n_complete["x", "y"]),
    unname(diag_attr$n_complete["x", "z"]),
    unname(diag_attr$n_complete["y", "z"])
  )
  expect_gt(length(unique(pair_n)), 1L)
})

test_that("weighted estimator respects na_method error and pairwise", {
  dat <- data.frame(
    subject = c(1, 1, 2, 2, 3, 3),
    x = c(1, 2, 1, NA, 2, 3),
    y = c(2, 4, 2, 3, 3, 5)
  )

  expect_error(
    rmcorr(dat, response = c("x", "y"), subject = "subject", estimator = "weighted", na_method = "error"),
    "missing, NaN, or infinite values"
  )
  expect_no_error(
    rmcorr(dat, response = c("x", "y"), subject = "subject", estimator = "weighted", na_method = "pairwise")
  )
})

test_that("subjects with fewer than two complete pairs are dropped", {
  dat <- data.frame(
    subject = c(1, 1, 1, 2, 2, 3, 3),
    x = c(1, 2, 3, 1, 2, 1, 2),
    y = c(2, 4, 6, 3, 6, 5, NA)
  )
  fit <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    na_method = "pairwise",
    ci_method = "none"
  )

  expect_equal(fit$n_subjects, 2L)
  expect_equal(fit$n_complete, 5L)
  expect_equal(fit$obs_per_subject_min, 2L)
  expect_equal(fit$obs_per_subject_max, 3L)
})

test_that("weighted estimator preserves negative association sign", {
  dat <- data.frame(
    subject = rep(1:3, each = 4),
    x = rep(c(1, 2, 3, 4), times = 3),
    y = rep(c(10, 12, 14), each = 4) - 1.5 * rep(c(1, 2, 3, 4), times = 3)
  )
  ref <- reference_rmcorr_weighted(dat, "x", "y", "subject")
  fit <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )

  expect_lt(fit$slope, 0)
  expect_lt(fit$estimate, 0)
  expect_equal(abs(fit$estimate), abs(ref$estimate), tolerance = 1e-10)
})

test_that("weighted estimator handles zero within-subject x variance", {
  dat <- data.frame(
    subject = c(1, 1, 2, 2, 3, 3),
    x = c(1, 1, 2, 2, 3, 3),
    y = c(1, 2, 3, 4, 5, 6)
  )
  fit <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )

  expect_true(is.na(fit$estimate))
  expect_true(is.na(fit$slope))
  expect_false(fit$valid)
})

test_that("weighted estimator handles zero within-subject y variance", {
  dat <- data.frame(
    subject = c(1, 1, 2, 2, 3, 3),
    x = c(1, 2, 1, 2, 1, 2),
    y = c(5, 5, 7, 7, 9, 9)
  )
  fit <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none"
  )

  expect_true(is.na(fit$estimate))
  expect_false(fit$valid)
})

test_that("weighted bootstrap CI is reproducible with fixed seed", {
  dat <- make_balanced_complete_data()

  fit1 <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "bootstrap",
    n_boot = 199L,
    seed = 2026L
  )
  fit2 <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "bootstrap",
    n_boot = 199L,
    seed = 2026L
  )

  expect_equal(c(fit1$lwr, fit1$upr), c(fit2$lwr, fit2$upr))
  expect_true(all(is.finite(c(fit1$lwr, fit1$upr))))
  expect_identical(fit1$ci_method, "bootstrap")
})

test_that("weighted bootstrap resamples subjects as blocks with reassigned IDs", {
  dat <- data.frame(
    .response1 = c(1, 2, 3, 4, 5, 6, 7, 8, 9),
    .response2 = c(2, 3, 2, 5, 7, 6, 8, 9, 11),
    .subject = factor(c("A", "A", "B", "B", "B", "C", "C", "C", "C"))
  )

  sampled <- matrixCorr:::`.mc_rmcorr_resample_subject_blocks`(
    dat,
    draw = c(2L, 2L, 1L),
    keep_source = TRUE
  )$data

  expect_equal(as.integer(table(sampled$.subject)), c(3L, 3L, 2L))
  expect_identical(as.character(unique(sampled$.subject_source[sampled$.subject == 1L])), "B")
  expect_identical(as.character(unique(sampled$.subject_source[sampled$.subject == 2L])), "B")
  expect_identical(as.character(unique(sampled$.subject_source[sampled$.subject == 3L])), "A")

  orig_A <- dat[dat$.subject == "A", c(".response1", ".response2")]
  orig_B <- dat[dat$.subject == "B", c(".response1", ".response2")]
  new_1 <- sampled[sampled$.subject == 1L, c(".response1", ".response2")]
  new_2 <- sampled[sampled$.subject == 2L, c(".response1", ".response2")]
  new_3 <- sampled[sampled$.subject == 3L, c(".response1", ".response2")]

  expect_equal(new_1, orig_B)
  expect_equal(new_2, orig_B)
  expect_equal(new_3, orig_A)
})

test_that("weighted matrix output is symmetric with symmetric diagnostics", {
  dat <- data.frame(
    subject = rep(1:5, each = 3),
    x = c(1, 2, 3, 2, 4, 6, 1, 3, 5, 2, 3, 4, 4, 5, 6),
    y = c(2, 3, 5, 3, 5, 7, 1, 2, 4, 3, 4, 5, 5, 6, 8),
    z = c(5, 4, 3, 4, 3, 2, 6, 5, 4, 3, 2, 1, 7, 6, 5)
  )
  fit <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none",
    n_threads = 1L
  )
  diag_attr <- attr(fit, "diagnostics")

  expect_equal(unclass(fit), t(unclass(fit)), tolerance = 1e-12)
  expect_equal(rownames(fit), c("x", "y", "z"))
  expect_equal(colnames(fit), c("x", "y", "z"))

  matrix_fields <- c(
    "slope", "p_value", "df", "n_complete", "n_subjects",
    "conf_low", "conf_high", "weighted_ss_x", "weighted_ss_e",
    "obs_per_subject_min", "obs_per_subject_max", "valid"
  )
  for (nm in matrix_fields) {
    expect_equal(diag_attr[[nm]], t(diag_attr[[nm]]), tolerance = 1e-12)
  }

  valid_diag <- diag(diag_attr$valid)
  est_diag <- diag(as.matrix(fit))
  idx <- which(!is.na(valid_diag) & valid_diag)
  if (length(idx)) {
    expect_equal(est_diag[idx], rep(1, length(idx)), tolerance = 1e-12)
  }
})

test_that("weighted outputs remain compatible with rmcorr/rmcorr_matrix methods", {
  dat <- make_balanced_complete_data()

  fit_pair <- rmcorr(
    dat,
    response = c("x", "y"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none",
    keep_data = TRUE
  )
  expect_s3_class(fit_pair, "rmcorr")
  expect_equal(fit_pair$r, fit_pair$estimate, tolerance = 1e-12)
  expect_equal(fit_pair$conf_int, c(lower = fit_pair$lwr, upper = fit_pair$upr), tolerance = 1e-12)
  expect_true(length(capture.output(print(fit_pair))) > 0L)
  expect_no_error(summary(fit_pair))
  expect_s3_class(plot(fit_pair), "ggplot")

  fit_matrix <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    estimator = "weighted",
    ci_method = "none",
    n_threads = 1L
  )
  expect_s3_class(fit_matrix, "rmcorr_matrix")
  expect_no_error(summary(fit_matrix))
  expect_s3_class(plot(fit_matrix), "ggplot")
})

test_that("weighted matrix results are thread-consistent", {
  dat <- data.frame(
    subject = rep(1:10, each = 4),
    x = rep(1:4, times = 10) + rep(seq_len(10) / 10, each = 4),
    y = rep(c(2, 3, 5, 6), times = 10) + rep(seq_len(10) / 20, each = 4),
    z = rep(c(6, 5, 4, 3), times = 10) - rep(seq_len(10) / 25, each = 4),
    w = rep(c(1, 4, 2, 5), times = 10)
  )
  dat$y[c(3, 17, 29)] <- NA_real_
  dat$z[c(6, 14, 31)] <- NA_real_

  fit1 <- rmcorr(
    dat,
    response = c("x", "y", "z", "w"),
    subject = "subject",
    estimator = "weighted",
    na_method = "pairwise",
    ci_method = "none",
    n_threads = 1L
  )
  fit2 <- rmcorr(
    dat,
    response = c("x", "y", "z", "w"),
    subject = "subject",
    estimator = "weighted",
    na_method = "pairwise",
    ci_method = "none",
    n_threads = 2L
  )

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)

  d1 <- attr(fit1, "diagnostics")
  d2 <- attr(fit2, "diagnostics")
  expect_equal(names(d1), names(d2))
  for (nm in names(d1)) {
    if (is.matrix(d1[[nm]]) || is.array(d1[[nm]])) {
      expect_equal(d1[[nm]], d2[[nm]], tolerance = 1e-12)
    } else {
      expect_equal(d1[[nm]], d2[[nm]])
    }
  }
})

#' Internal validation utilities
#'
#' Centralised argument and consistency checks using cli/rlang.
#' These helpers are not exported and are intended for internal use.
#'
#' @keywords internal
#' @name matrixCorr-utils
#' @noRd
#' @importFrom cli cli_abort cli_warn cli_inform
#' @importFrom rlang is_bool is_scalar_integerish is_scalar_character arg_match
NULL

#' Null-coalescing helper
#' @keywords internal
#' @noRd
`%||%` <- function(a, b) if (!is.null(a)) a else b

#' Abort for internal errors (should not happen)
#' @keywords internal
abort_internal <- function(message, ...,
                           .class = character()) {
  cli::cli_abort(
    message,
    ...,
    class = c(.class, "matrixCorr_error", "matrixCorr_internal_error")
  )
}

#' Rethrow a caught condition as a cli error
#' @keywords internal
.mc_abort_condition <- function(cnd,
                                .class = character()) {
  msg <- conditionMessage(cnd)
  cli::cli_abort(
    "{msg}",
    msg = msg,
    parent = cnd,
    class = c(.class, "matrixCorr_error", "matrixCorr_backend_error")
  )
}

#' Abort with a standardised argument error
#' @keywords internal
abort_bad_arg <- function(arg,
                          message,
                          ...,
                          .hint = NULL,
                          .class = character()) {
  env <- rlang::env_clone(rlang::caller_env())
  rlang::env_bind(env, arg = arg)
  bullets <- c(
    "Problem with {.arg {arg}}.",
    "x" = message
  )
  if (!is.null(.hint)) {
    bullets <- c(bullets, "i" = .hint)
  }

  cli::cli_abort(
    bullets,
    arg = arg,
    ...,
    .envir = env,
    class = c(.class, "matrixCorr_error", "matrixCorr_arg_error")
  )
}

#' Check a single logical flag
#' @keywords internal
check_bool <- function(x,
                       arg = as.character(substitute(x))) {
  if (!is.logical(x) || length(x) != 1L || is.na(x)) {
    abort_bad_arg(arg,
      message = "must be a single TRUE or FALSE."
    )
  }
  invisible(x)
}

#' Check scalar numeric (with optional bounds)
#' @keywords internal
check_scalar_numeric <- function(x,
                                 arg = as.character(substitute(x)),
                                 finite = TRUE,
                                 lower = -Inf,
                                 upper = Inf,
                                 allow_na = FALSE,
                                 closed_lower = TRUE,
                                 closed_upper = TRUE) {
  ok_len <- length(x) == 1L
  ok_num <- is.numeric(x)
  if (!ok_len || !ok_num) {
    abort_bad_arg(arg,
      message = "must be a single numeric value."
    )
  }

  if (!allow_na && (is.na(x) || is.nan(x))) {
    abort_bad_arg(arg,
      message = "must not be NA or NaN."
    )
  }

  if (finite && !(allow_na && is.na(x)) && any(!is.finite(x))) {
    abort_bad_arg(arg,
      message = "must be finite."
    )
  }

  if (!is.na(lower)) {
    if (closed_lower && x < lower) {
      abort_bad_arg(arg,
        message = "must be >= {lower}.",
        lower = lower
      )
    } else if (!closed_lower && x <= lower) {
      abort_bad_arg(arg,
        message = "must be > {lower}.",
        lower = lower
      )
    }
  }

  if (!is.na(upper)) {
    if (closed_upper && x > upper) {
      abort_bad_arg(arg,
        message = "must be <= {upper}.",
        upper = upper
      )
    } else if (!closed_upper && x >= upper) {
      abort_bad_arg(arg,
        message = "must be < {upper}.",
        upper = upper
      )
    }
  }

  invisible(x)
}

#' Check scalar character (non-empty)
#' @keywords internal
check_scalar_character <- function(x,
                                   arg = as.character(substitute(x)),
                                   allow_empty = FALSE) {
  if (!rlang::is_scalar_character(x)) {
    abort_bad_arg(arg,
      message = "must be a single character string."
    )
  }
  if (!allow_empty && !nzchar(x)) {
    abort_bad_arg(arg,
      message = "must be a non-empty character string."
    )
  }
  invisible(x)
}

#' Check that a data frame contains required columns
#' @keywords internal
check_required_cols <- function(df,
                                cols,
                                df_arg = "data") {
  missing <- setdiff(cols, names(df))
  if (length(missing) > 0L) {
    abort_bad_arg(df_arg,
      message = "is missing required column(s): {paste(missing, collapse = ', ')}.",
      .hint   = "Check spelling and ensure all required variables are present.",
      missing = missing
    )
  }
  invisible(df)
}

#' Check that two vectors have the same length
#' @keywords internal
check_same_length <- function(x, y,
                              arg_x = as.character(substitute(x)),
                              arg_y = as.character(substitute(y))) {
  if (length(x) != length(y)) {
    cli::cli_abort(
      c(
        "Length mismatch: {.arg {arg_x}} and {.arg {arg_y}} must have the same length.",
        "x" = "{.arg {arg_x}} has length {length(x)}.",
        "x" = "{.arg {arg_y}} has length {length(y)}."
      ),
      arg_x = arg_x,
      arg_y = arg_y,
      class = c("matrixCorr_error", "matrixCorr_arg_error")
    )
  }
  invisible(NULL)
}

#' Check matrix dimensions
#' @keywords internal
check_matrix_dims <- function(M,
                              nrow = NULL,
                              ncol = NULL,
                              arg = as.character(substitute(M))) {
  if (!is.matrix(M)) {
    abort_bad_arg(arg,
      message = "must be a matrix."
    )
  }
  dm <- dim(M)
  if (!is.null(nrow) && dm[1L] != nrow) {
    abort_bad_arg(arg,
      message = "must have {nrow} row{?s}, not {dm[1L]}.",
      nrow = nrow, dm = dm
    )
  }
  if (!is.null(ncol) && dm[2L] != ncol) {
    abort_bad_arg(arg,
      message = "must have {ncol} column{?s}, not {dm[2L]}.",
      ncol = ncol, dm = dm
    )
  }
  invisible(M)
}

#' Check matrix symmetry
#' @keywords internal
check_symmetric_matrix <- function(M,
                                   tol = .Machine$double.eps^0.5,
                                   arg = as.character(substitute(M))) {
  check_matrix_dims(M, arg = arg)
  if (!isSymmetric(M, tol = tol)) {
    abort_bad_arg(arg,
      message = "must be symmetric within tolerance {tol}.",
      tol = tol
    )
  }
  invisible(M)
}

#' Check a non-negative scalar (>= 0)
#' @keywords internal
check_scalar_nonneg <- function(x,
                                arg = as.character(substitute(x)),
                                strict = FALSE) {
  if (strict) {
    # Strictly positive: custom message to surface "positive"
    if (!is.numeric(x) || length(x) != 1L) {
      abort_bad_arg(arg, message = "must be a single numeric value.")
    }
    if (is.na(x) || is.nan(x)) {
      abort_bad_arg(arg, message = "must not be NA or NaN.")
    }
    if (!is.finite(x)) {
      abort_bad_arg(arg, message = "must be finite.")
    }
    if (x <= 0) {
      abort_bad_arg(arg, message = "must be a positive value (> 0).")
    }
  } else {
    check_scalar_numeric(x, arg = arg, lower = 0, closed_lower = TRUE)
  }
  invisible(x)
}

#' Check strictly positive scalar integer
#' @keywords internal
check_scalar_int_pos <- function(x,
                                 arg = as.character(substitute(x))) {
  ok <- length(x) == 1L &&
    is.numeric(x) &&
    !is.na(x) &&
    is.finite(x) &&
    x > 0 &&
    abs(x - round(x)) <= sqrt(.Machine$double.eps)
  if (!ok) {
    abort_bad_arg(arg,
      message = "must be a positive integer."
    )
  }
  as.integer(x)
}

#' Check probability in unit interval
#' @keywords internal
check_prob_scalar <- function(x,
                              arg = as.character(substitute(x)),
                              open_ends = FALSE) {
  if (open_ends) {
    check_scalar_numeric(x, arg = arg,
                         lower = 0, closed_lower = FALSE,
                         upper = 1, closed_upper = FALSE)
  } else {
    check_scalar_numeric(x, arg = arg,
                         lower = 0, closed_lower = TRUE,
                         upper = 1, closed_upper = TRUE)
  }
  invisible(x)
}

#' Check AR(1) correlation parameter
#' @keywords internal
check_ar1_rho <- function(x,
                          arg = as.character(substitute(x)),
                          bound = 0.999) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) ||
      x <= -bound || x >= bound) {
    abort_bad_arg(arg,
      message = "`{arg}` must be in (-{bound}, {bound}).",
      bound = bound
    )
  }
  as.numeric(x)
}

#' Check weights vector (non-negative, finite, correct length)
#' @keywords internal
check_weights <- function(w,
                          n,
                          arg = as.character(substitute(w))) {
  if (is.null(w)) return(invisible(NULL))

  w <- as.numeric(w)
  if (length(w) != n) {
    abort_bad_arg(arg,
      message = "must have length {n}, not {length(w)}.",
      n = n, w = w
    )
  }
  if (anyNA(w) || any(!is.finite(w)) || any(w < 0)) {
    abort_bad_arg(arg,
      message = "must be non-negative, finite, and contain no missing values."
    )
  }
  invisible(w)
}

#' Check object class
#' @keywords internal
check_inherits <- function(x,
                           class,
                           arg = as.character(substitute(x))) {
  if (!inherits(x, class)) {
    abort_bad_arg(arg,
      message = "must inherit from class{?es}: {cli::qty(class)}{class*?{, }{ and }}.",
      class = class
    )
  }
  invisible(x)
}

#' Match argument to allowed values
#' @keywords internal
match_arg <- function(arg,
                      values,
                      arg_name = as.character(substitute(arg))) {
  if (is.character(arg) &&
      length(arg) == 1L &&
      !is.na(arg) &&
      identical(match(arg, values, nomatch = 0L) > 0L, TRUE)) {
    return(arg)
  }
  rlang::arg_match(arg = arg, values = values, multiple = FALSE, error_arg = arg_name)
}

#' Validate the package-wide missing-data method selector
#' @keywords internal
#' @noRd
validate_na_method <- function(na_method,
                               arg = as.character(substitute(na_method)),
                               allowed = c("error", "pairwise")) {
  if (is.logical(na_method) && length(na_method) == 1L && !is.na(na_method)) {
    resolved <- if (isTRUE(na_method)) "error" else "pairwise"
    if (resolved %in% allowed) return(resolved)
  }
  if (is.character(na_method) &&
      length(na_method) == 1L &&
      !is.na(na_method) &&
      (na_method %in% allowed)) {
    return(na_method)
  }
  match_arg(
    na_method,
    values = allowed,
    arg_name = arg
  )
}

#' Resolve canonical/deprecated missing-data arguments
#' @keywords internal
#' @noRd
resolve_na_args <- function(na_method = "error",
                            check_na = NULL,
                            na_method_missing = FALSE,
                            arg_na_method = "na_method",
                            arg_check_na = "check_na",
                            allowed = c("error", "pairwise"),
                            warn = TRUE) {
  allowed <- match.arg(allowed, c("error", "pairwise", "complete"), several.ok = TRUE)
  if (is.null(check_na) && isTRUE(na_method_missing)) {
    return(list(
      na_method = "error",
      check_na = TRUE,
      common_sample = TRUE
    ))
  }
  if (!is.null(check_na)) {
    check_bool(check_na, arg = arg_check_na)
    if (!isTRUE(na_method_missing)) {
      abort_bad_arg(
        arg_na_method,
        message = "and {.arg check_na} cannot both be supplied.",
        .hint = "Use only {.arg na_method}; {.arg check_na} is deprecated."
      )
    }
    if (isTRUE(warn)) {
      replacement <- if (isTRUE(check_na)) "error" else "pairwise"
      cli::cli_warn(
        c(
          "{.arg check_na} is deprecated.",
          "i" = "Use {.arg na_method} = {.val {replacement}} instead."
        ),
        replacement = replacement,
        class = c("matrixCorr_warning", "matrixCorr_deprecated_warning")
      )
    }
    resolved <- if (isTRUE(check_na)) "error" else "pairwise"
    if (!resolved %in% allowed) {
      resolved <- "error"
    }
    return(list(
      na_method = resolved,
      check_na = identical(resolved, "error"),
      common_sample = !identical(resolved, "pairwise")
    ))
  }

  if (!isTRUE(na_method_missing) && rlang::is_bool(na_method)) {
    if (isTRUE(warn)) {
      replacement <- if (isTRUE(na_method)) "error" else "pairwise"
      cli::cli_warn(
        c(
          "Logical {.arg na_method} is deprecated compatibility input.",
          "i" = "Use {.arg na_method} = {.val {replacement}} instead."
        ),
        replacement = replacement,
        class = c("matrixCorr_warning", "matrixCorr_deprecated_warning")
      )
    }
  }

  resolved <- validate_na_method(na_method, arg = arg_na_method, allowed = allowed)
  list(
    na_method = resolved,
    check_na = identical(resolved, "error"),
    common_sample = !identical(resolved, "pairwise")
  )
}

#' Apply listwise finite-row deletion for na_method = "complete"
#' @keywords internal
#' @noRd
.mc_complete_case_matrix <- function(x,
                                     min_n = 2L,
                                     arg = "data") {
  x <- as.matrix(x)
  min_n <- as.integer(min_n)
  filtered <- complete_case_matrix_cpp(x)
  keep <- filtered$complete_rows

  n_original <- filtered$n_original
  n_complete <- filtered$n_complete

  if (n_complete < min_n) {
    abort_bad_arg(
      arg,
      message = "must retain at least {min_n} complete rows when {.arg na_method} is {.val complete}; retained {n_complete}.",
      min_n = min_n,
      n_complete = n_complete
    )
  }

  list(
    data = filtered$data,
    diagnostics = list(
      na_method = "complete",
      n_original = n_original,
      n_complete = n_complete,
      n_removed = n_original - n_complete,
      complete_rows = keep,
      common_sample = TRUE
    )
  )
}

#' Merge correlation diagnostics
#' @keywords internal
#' @noRd
.mc_merge_diagnostics <- function(x, y) {
  if (is.null(x)) return(y)
  if (is.null(y)) return(x)
  utils::modifyList(x, y)
}

#' Set OpenMP threads only when a change is required
#' @keywords internal
#' @noRd
.mc_enter_omp_threads <- function(n_threads) {
  current <- as.integer(get_omp_threads())
  target <- as.integer(n_threads)
  if (identical(current, target)) {
    return(NULL)
  }
  set_omp_threads(target)
  current
}

#' Restore OpenMP thread count after `.mc_enter_omp_threads()`
#' @keywords internal
#' @noRd
.mc_exit_omp_threads <- function(prev_threads) {
  if (!is.null(prev_threads)) {
    set_omp_threads(as.integer(prev_threads))
  }
  invisible(NULL)
}

#' Prepare OpenMP thread state for a wrapper call
#' @keywords internal
#' @noRd
.mc_prepare_omp_threads <- function(n_threads,
                                    arg = "n_threads") {
  .mc_enter_omp_threads(check_scalar_int_pos(n_threads, arg = arg))
}

#' Extract deprecated compatibility aliases from dots
#' @keywords internal
#' @noRd
.mc_extract_legacy_aliases <- function(dots,
                                       allowed = character(),
                                       dots_arg = "...") {
  dots <- if (is.null(dots)) list() else dots
  nms <- names(dots)
  if (length(dots) && (is.null(nms) || anyNA(nms) || any(!nzchar(nms)))) {
    abort_bad_arg(
      dots_arg,
      message = "does not allow unnamed extra arguments."
    )
  }
  unknown <- setdiff(nms, allowed)
  if (length(unknown)) {
    abort_bad_arg(
      dots_arg,
      message = "contains unsupported argument(s): {paste(unknown, collapse = ', ')}."
    )
  }
  dots
}

#' Validate common correlation output arguments
#' @keywords internal
#' @noRd
.mc_validate_output_args <- function(output = c("matrix", "sparse", "edge_list"),
                                     threshold = 0,
                                     diag = TRUE,
                                     arg_output = "output",
                                     arg_threshold = "threshold",
                                     arg_diag = "diag") {
  output <- match_arg(
    output,
    values = c("matrix", "sparse", "edge_list"),
    arg_name = arg_output
  )
  check_scalar_numeric(
    threshold,
    arg = arg_threshold,
    lower = 0,
    closed_lower = TRUE
  )
  check_bool(diag, arg = arg_diag)

  threshold <- as.numeric(threshold)
  if (identical(output, "matrix") && threshold > 0) {
    abort_bad_arg(
      arg_threshold,
      message = "must be 0 when {.arg output} is {.val matrix}.",
      .hint = "Use {.arg output} = {.val sparse} or {.val edge_list} for thresholded output."
    )
  }

  list(
    output = output,
    threshold = threshold,
    diag = isTRUE(diag)
  )
}

#' Validate thresholded-output routing request
#' @keywords internal
#' @noRd
.mc_validate_thresholded_output_request <- function(output = c("matrix", "sparse", "edge_list"),
                                                    threshold = 0,
                                                    diag = TRUE,
                                                    arg_output = "output",
                                                    arg_threshold = "threshold",
                                                    arg_diag = "diag") {
  cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag,
    arg_output = arg_output,
    arg_threshold = arg_threshold,
    arg_diag = arg_diag
  )
  cfg$thresholded <- identical(cfg$output, "sparse") ||
    identical(cfg$output, "edge_list")
  cfg$thresholded <- isTRUE(cfg$thresholded) && isTRUE(cfg$threshold > 0)
  cfg
}

#' Report whether direct thresholded path is eligible
#' @keywords internal
#' @noRd
.mc_supports_direct_threshold_path <- function(method,
                                               na_method = "error",
                                               ci = FALSE,
                                               output = "matrix",
                                               threshold = 0,
                                               symmetric = TRUE,
                                               exact = TRUE,
                                               pairwise = FALSE,
                                               has_ci = FALSE,
                                               has_inference = FALSE,
                                               weighted = FALSE,
                                               ...) {
  method <- tolower(as.character(method %||% ""))
  if (!method %in% c("pearson", "spearman", "ccc", "bicor", "pbcor", "wincor", "cohen_kappa", "weighted_kappa")) {
    return(FALSE)
  }
  if (identical(method, "cohen_kappa")) {
    if (!na_method %in% c("error", "complete")) {
      return(FALSE)
    }
  } else if (identical(method, "weighted_kappa")) {
    if (!identical(na_method, "error")) {
      return(FALSE)
    }
  } else if (!identical(na_method, "error")) {
    return(FALSE)
  }
  if (!identical(output, "sparse") && !identical(output, "edge_list")) {
    return(FALSE)
  }
  if (!(is.numeric(threshold) && length(threshold) == 1L && is.finite(threshold) && threshold > 0)) {
    return(FALSE)
  }
  if (!isTRUE(symmetric) || !isTRUE(exact)) {
    return(FALSE)
  }
  if (isTRUE(ci) || isTRUE(has_ci) || isTRUE(has_inference)) {
    return(FALSE)
  }
  if (isTRUE(pairwise) || isTRUE(weighted)) {
    return(FALSE)
  }
  TRUE
}

#' First estimator class from a correlation result
#' @keywords internal
#' @noRd
.mc_corr_estimator_class <- function(x, default = "corr_estimator") {
  cls <- class(x)
  if (is.null(cls) || !length(cls)) {
    return(default)
  }
  skip <- c(
    "corr_matrix", "corr_sparse", "corr_edge_list",
    "corr_result", "matrix", "array", "data.frame", "list",
    "sparseMatrix", "Matrix", "mMatrix", "dMatrix", "symmetricMatrix",
    "CsparseMatrix", "dsparseMatrix", "dCsparseMatrix", "generalMatrix"
  )
  keep <- setdiff(cls, skip)
  if (!length(keep)) {
    return(default)
  }
  keep[[1L]]
}

#' Collect non-structural attributes for propagation across outputs
#' @keywords internal
#' @noRd
.mc_corr_extra_attrs <- function(x) {
  attrs <- attributes(x)
  if (is.null(attrs) || !length(attrs)) {
    return(list())
  }
  drop <- c(
    "dim", "dimnames", "class", "names", "row.names",
    "method", "description", "package", "output", "threshold", "diag",
    "diagnostics", "ci", "conf.level", "corr_result", "corr_output_class",
    "corr_estimator_class", "corr_dim", "corr_dimnames", "corr_symmetric"
  )
  attrs[setdiff(names(attrs), drop)]
}

#' Attach arbitrary attributes from a named list
#' @keywords internal
#' @noRd
.mc_set_attrs <- function(x, attrs) {
  if (is.null(attrs) || !length(attrs)) {
    return(x)
  }
  nm <- names(attrs)
  if (is.null(nm) || any(!nzchar(nm))) {
    return(x)
  }
  for (i in seq_along(attrs)) {
    attr(x, nm[[i]]) <- attrs[[i]]
  }
  x
}

#' New dense correlation result
#' @keywords internal
#' @noRd
.mc_new_corr_matrix <- function(mat,
                                estimator_class,
                                method,
                                description,
                                output = "matrix",
                                threshold = 0,
                                diag = TRUE,
                                diagnostics = NULL,
                                ci = NULL,
                                conf.level = NULL,
                                symmetric = NULL,
                                package_name = "matrixCorr",
                                extra_attrs = list(),
                                extra_classes = character()) {
  check_matrix_dims(mat, arg = "mat")
  cls <- unique(c(
    "corr_matrix",
    estimator_class,
    "corr_result",
    extra_classes,
    "matrix"
  ))
  out <- structure(
    mat,
    class = cls,
    method = method,
    description = description,
    package = package_name,
    output = output,
    threshold = as.numeric(threshold),
    diag = isTRUE(diag),
    diagnostics = diagnostics,
    ci = ci,
    conf.level = conf.level,
    corr_result = TRUE,
    corr_output_class = "corr_matrix",
    corr_estimator_class = estimator_class,
    corr_dim = as.integer(dim(mat)),
    corr_dimnames = dimnames(mat),
    corr_symmetric = if (is.null(symmetric)) {
      isTRUE(nrow(mat) == ncol(mat)) && isTRUE(isSymmetric(mat, check.attributes = FALSE))
    } else {
      isTRUE(symmetric)
    }
  )
  .mc_set_attrs(out, extra_attrs)
}

#' New sparse correlation result
#' @keywords internal
#' @noRd
.mc_new_corr_sparse <- function(x,
                                estimator_class,
                                method,
                                description,
                                output = "sparse",
                                threshold = 0,
                                diag = TRUE,
                                diagnostics = NULL,
                                ci = NULL,
                                conf.level = NULL,
                                symmetric = NULL,
                                package_name = "matrixCorr",
                                extra_attrs = list()) {
  out <- x
  attr(out, "method") <- method
  attr(out, "description") <- description
  attr(out, "package") <- package_name
  attr(out, "output") <- output
  attr(out, "threshold") <- as.numeric(threshold)
  attr(out, "diag") <- isTRUE(diag)
  attr(out, "diagnostics") <- diagnostics
  attr(out, "ci") <- ci
  attr(out, "conf.level") <- conf.level
  attr(out, "corr_result") <- TRUE
  attr(out, "corr_output_class") <- "corr_sparse"
  attr(out, "corr_estimator_class") <- estimator_class
  attr(out, "corr_dim") <- as.integer(dim(out))
  attr(out, "corr_dimnames") <- dimnames(out)
  attr(out, "corr_symmetric") <- if (is.null(symmetric)) {
    isTRUE(nrow(out) == ncol(out))
  } else {
    isTRUE(symmetric)
  }
  .mc_set_attrs(out, extra_attrs)
}

#' New edge-list correlation result
#' @keywords internal
#' @noRd
.mc_new_corr_edge_list <- function(df,
                                   estimator_class,
                                   method,
                                   description,
                                   threshold = 0,
                                   diag = TRUE,
                                   diagnostics = NULL,
                                   ci = NULL,
                                   conf.level = NULL,
                                   source_dim = NULL,
                                   source_dimnames = NULL,
                                   symmetric = NULL,
                                   package_name = "matrixCorr",
                                   extra_attrs = list()) {
  out <- data.frame(df, stringsAsFactors = FALSE, check.names = FALSE)
  if (!all(c("row", "col", "value") %in% names(out)) &&
      all(c("i", "j", "x") %in% names(out))) {
    dn <- source_dimnames
    ii <- as.integer(out$i)
    jj <- as.integer(out$j)
    rn <- if (is.list(dn) && length(dn) == 2L) dn[[1L]] else NULL
    cn <- if (is.list(dn) && length(dn) == 2L) dn[[2L]] else NULL
    out <- data.frame(
      row = if (is.null(rn)) as.character(ii) else rn[ii],
      col = if (is.null(cn)) as.character(jj) else cn[jj],
      value = as.numeric(out$x),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
  }
  if (!all(c("row", "col", "value") %in% names(out))) {
    abort_bad_arg("df", message = "must contain columns `row`, `col`, and `value`.")
  }
  out <- out[, c("row", "col", "value"), drop = FALSE]
  class(out) <- c("corr_edge_list", estimator_class, "corr_result", "data.frame")
  attr(out, "method") <- method
  attr(out, "description") <- description
  attr(out, "package") <- package_name
  attr(out, "output") <- "edge_list"
  attr(out, "threshold") <- as.numeric(threshold)
  attr(out, "diag") <- isTRUE(diag)
  attr(out, "diagnostics") <- diagnostics
  attr(out, "ci") <- ci
  attr(out, "conf.level") <- conf.level
  attr(out, "corr_result") <- TRUE
  attr(out, "corr_output_class") <- "corr_edge_list"
  attr(out, "corr_estimator_class") <- estimator_class
  attr(out, "corr_dim") <- as.integer(source_dim %||% c(NA_integer_, NA_integer_))
  attr(out, "corr_dimnames") <- source_dimnames
  attr(out, "corr_symmetric") <- if (is.null(symmetric)) NA else isTRUE(symmetric)
  .mc_set_attrs(out, extra_attrs)
}

#' Convert C++ triplets to sparse matrix
#' @keywords internal
#' @noRd
.mc_triplets_to_sparse <- function(triplets,
                                   dim,
                                   dimnames = NULL,
                                   symmetric = TRUE) {
  check_same_length(triplets$i, triplets$j, arg_x = "triplets$i", arg_y = "triplets$j")
  check_same_length(triplets$i, triplets$x, arg_x = "triplets$i", arg_y = "triplets$x")
  dm <- as.integer(dim)
  if (length(dm) != 2L) {
    abort_bad_arg("dim", message = "must be length 2.")
  }

  i <- as.integer(triplets$i)
  j <- as.integer(triplets$j)
  x <- as.numeric(triplets$x)

  keep <- is.finite(i) & is.finite(j) & is.finite(x) &
    i >= 1L & i <= dm[[1L]] &
    j >= 1L & j <= dm[[2L]]
  i <- i[keep]
  j <- j[keep]
  x <- x[keep]

  out <- Matrix::sparseMatrix(
    i = i,
    j = j,
    x = x,
    dims = dm,
    dimnames = dimnames
  )
  if (isTRUE(symmetric)) {
    return(Matrix::forceSymmetric(out, uplo = "U"))
  }
  out
}

#' Convert C++ triplets to public edge-list payload
#' @keywords internal
#' @noRd
.mc_triplets_to_edge_list <- function(triplets,
                                      dimnames = NULL) {
  check_same_length(triplets$i, triplets$j, arg_x = "triplets$i", arg_y = "triplets$j")
  check_same_length(triplets$i, triplets$x, arg_x = "triplets$i", arg_y = "triplets$x")
  ii <- as.integer(triplets$i)
  jj <- as.integer(triplets$j)

  rn <- NULL
  cn <- NULL
  if (is.list(dimnames) && length(dimnames) == 2L) {
    rn <- dimnames[[1L]]
    cn <- dimnames[[2L]]
  }

  data.frame(
    row = if (is.null(rn)) as.character(ii) else rn[ii],
    col = if (is.null(cn)) as.character(jj) else cn[jj],
    value = as.numeric(triplets$x),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

#' Build sparse/edge output directly from retained triplets
#' @keywords internal
#' @noRd
.mc_finalize_triplets_output <- function(triplets,
                                         output = c("sparse", "edge_list"),
                                         estimator_class,
                                         method,
                                         description,
                                         threshold,
                                         diag,
                                         source_dim,
                                         source_dimnames = NULL,
                                         diagnostics = NULL,
                                         ci = NULL,
                                         conf.level = NULL,
                                         symmetric = TRUE,
                                         package_name = "matrixCorr",
                                         extra_attrs = list()) {
  output <- match.arg(output)
  source_dim <- as.integer(source_dim)

  meta <- list(
    source_class = estimator_class,
    method = method,
    description = description,
    package = package_name,
    diagnostics = diagnostics
  )

  if (identical(output, "sparse")) {
    out <- .mc_new_corr_sparse(
      x = .mc_triplets_to_sparse(
        triplets,
        dim = source_dim,
        dimnames = source_dimnames,
        symmetric = symmetric
      ),
      estimator_class = estimator_class,
      method = method,
      description = description,
      output = "sparse",
      threshold = threshold,
      diag = diag,
      diagnostics = diagnostics,
      ci = ci,
      conf.level = conf.level,
      symmetric = symmetric,
      package_name = package_name,
      extra_attrs = extra_attrs
    )
    attr(out, "matrixCorr_meta") <- meta
    return(out)
  }

  out <- .mc_new_corr_edge_list(
    df = .mc_triplets_to_edge_list(triplets, dimnames = source_dimnames),
    estimator_class = estimator_class,
    method = method,
    description = description,
    threshold = threshold,
    diag = diag,
    diagnostics = diagnostics,
    ci = ci,
    conf.level = conf.level,
    source_dim = source_dim,
    source_dimnames = source_dimnames,
    symmetric = symmetric,
    package_name = package_name,
    extra_attrs = extra_attrs
  )
  attr(out, "matrixCorr_meta") <- meta
  out
}

#' Convert a symmetric matrix to an upper-triangle edge frame
#' @keywords internal
#' @noRd
.mc_matrix_to_edge_list <- function(mat,
                                    threshold = 0,
                                    diag = TRUE,
                                    symmetric = NULL) {
  check_matrix_dims(mat, arg = "mat")
  check_scalar_numeric(threshold, arg = "threshold", lower = 0, closed_lower = TRUE)
  check_bool(diag, arg = "diag")

  if (nrow(mat) == 0L || ncol(mat) == 0L) {
    return(data.frame(
      row = character(),
      col = character(),
      value = numeric(),
      stringsAsFactors = FALSE,
      check.names = FALSE
    ))
  }

  same_dimnames <- isTRUE(identical(rownames(mat), colnames(mat))) ||
    (is.null(rownames(mat)) && is.null(colnames(mat)))
  use_upper <- if (is.null(symmetric)) {
    isTRUE(nrow(mat) == ncol(mat)) &&
      same_dimnames &&
      isTRUE(isSymmetric(mat, check.attributes = FALSE))
  } else {
    isTRUE(symmetric) &&
      isTRUE(nrow(mat) == ncol(mat)) &&
      same_dimnames
  }
  if (use_upper) {
    idx <- upper.tri(mat, diag = isTRUE(diag))
  } else {
    idx <- matrix(TRUE, nrow = nrow(mat), ncol = ncol(mat))
    if (!isTRUE(diag)) {
      d <- seq_len(min(nrow(mat), ncol(mat)))
      idx[cbind(d, d)] <- FALSE
    }
  }
  i <- row(mat)[idx]
  j <- col(mat)[idx]
  vals <- mat[idx]
  keep <- is.finite(vals) & (abs(vals) >= as.numeric(threshold))

  rn <- rownames(mat)
  cn <- colnames(mat)
  row_out <- if (is.null(rn)) as.character(i[keep]) else rn[i[keep]]
  col_out <- if (is.null(cn)) as.character(j[keep]) else cn[j[keep]]

  data.frame(
    row = row_out,
    col = col_out,
    value = as.numeric(vals[keep]),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

#' Convert a symmetric matrix to thresholded sparse matrix
#' @keywords internal
#' @noRd
.mc_matrix_to_sparse_thresholded <- function(mat,
                                             threshold = 0,
                                             diag = TRUE,
                                             symmetric = NULL) {
  check_matrix_dims(mat, arg = "mat")
  check_scalar_numeric(threshold, arg = "threshold", lower = 0, closed_lower = TRUE)
  check_bool(diag, arg = "diag")

  dn <- dimnames(mat)
  same_dimnames <- isTRUE(identical(rownames(mat), colnames(mat))) ||
    (is.null(rownames(mat)) && is.null(colnames(mat)))
  use_upper <- if (is.null(symmetric)) {
    isTRUE(nrow(mat) == ncol(mat)) &&
      same_dimnames &&
      isTRUE(isSymmetric(mat, check.attributes = FALSE))
  } else {
    isTRUE(symmetric) &&
      isTRUE(nrow(mat) == ncol(mat)) &&
      same_dimnames
  }
  if (use_upper) {
    idx <- upper.tri(mat, diag = isTRUE(diag))
  } else {
    idx <- matrix(TRUE, nrow = nrow(mat), ncol = ncol(mat))
    if (!isTRUE(diag)) {
      d <- seq_len(min(nrow(mat), ncol(mat)))
      idx[cbind(d, d)] <- FALSE
    }
  }
  i <- row(mat)[idx]
  j <- col(mat)[idx]
  vals <- mat[idx]
  keep <- is.finite(vals) & (abs(vals) >= as.numeric(threshold))

  out <- Matrix::sparseMatrix(
    i = as.integer(i[keep]),
    j = as.integer(j[keep]),
    x = as.numeric(vals[keep]),
    dims = dim(mat),
    dimnames = dn
  )
  if (use_upper) {
    return(Matrix::forceSymmetric(out, uplo = "U"))
  }
  out
}

#' Fast route for already-validated output routing
#' @keywords internal
#' @noRd
.mc_finalize_corr_output_fast <- function(x,
                                          output = "matrix",
                                          threshold = 0,
                                          diag = TRUE) {
  if (identical(output, "matrix") &&
    isTRUE(is.finite(threshold)) && isTRUE(as.numeric(threshold) == 0) &&
    isTRUE(diag) &&
    inherits(x, "corr_matrix")) {
    attr(x, "output") <- "matrix"
    attr(x, "threshold") <- 0
    attr(x, "diag") <- TRUE
    return(x)
  }
  .mc_finalize_corr_output(
    x = x,
    output = output,
    threshold = threshold,
    diag = diag
  )
}

#' Route a computed correlation matrix object to requested output shape
#' @keywords internal
#' @noRd
.mc_finalize_corr_output <- function(x,
                                     output = c("matrix", "sparse", "edge_list"),
                                     threshold = 0,
                                     diag = TRUE) {
  if (inherits(x, "corr_matrix") &&
    is.character(output) &&
    length(output) == 1L &&
    identical(output, "matrix") &&
    is.numeric(threshold) &&
    length(threshold) == 1L &&
    !is.na(threshold) &&
    as.numeric(threshold) == 0 &&
    isTRUE(diag)) {
    attr(x, "output") <- "matrix"
    attr(x, "threshold") <- 0
    attr(x, "diag") <- TRUE
    return(x)
  }
  cfg <- .mc_validate_output_args(output = output, threshold = threshold, diag = diag)
  if (identical(cfg$output, "matrix") && inherits(x, "corr_matrix")) {
    attr(x, "output") <- "matrix"
    attr(x, "threshold") <- 0
    attr(x, "diag") <- TRUE
    return(x)
  }
  mat <- as.matrix(x)
  estimator_class <- .mc_corr_estimator_class(x)
  method <- attr(x, "method", exact = TRUE) %||% estimator_class
  description <- attr(x, "description", exact = TRUE) %||% "Correlation result"
  diagnostics <- attr(x, "diagnostics", exact = TRUE)
  ci <- attr(x, "ci", exact = TRUE)
  conf.level <- attr(x, "conf.level", exact = TRUE)
  package_name <- attr(x, "package", exact = TRUE) %||% "matrixCorr"
  extra_attrs <- .mc_corr_extra_attrs(x)
  meta <- list(
    source_class = class(x),
    method = method,
    description = description,
    package = package_name,
    diagnostics = diagnostics
  )
  source_dim <- dim(mat)
  source_dimnames <- dimnames(mat)
  source_symmetric_attr <- attr(x, "corr_symmetric", exact = TRUE)
  source_symmetric <- if (is.logical(source_symmetric_attr) &&
    length(source_symmetric_attr) == 1L &&
    !is.na(source_symmetric_attr)) {
    isTRUE(source_symmetric_attr)
  } else {
    isTRUE(nrow(mat) == ncol(mat)) &&
      (isTRUE(identical(rownames(mat), colnames(mat))) ||
        (is.null(rownames(mat)) && is.null(colnames(mat)))) &&
      isTRUE(isSymmetric(mat, check.attributes = FALSE))
  }

  if (identical(cfg$output, "matrix")) {
    return(.mc_new_corr_matrix(
      mat = mat,
      estimator_class = estimator_class,
      method = method,
      description = description,
      output = "matrix",
      threshold = 0,
      diag = TRUE,
      diagnostics = diagnostics,
      ci = ci,
      conf.level = conf.level,
      symmetric = source_symmetric,
      package_name = package_name,
      extra_attrs = extra_attrs,
      extra_classes = setdiff(class(x), c(estimator_class, "matrix"))
    ))
  }

  if (identical(cfg$output, "edge_list")) {
    out <- .mc_new_corr_edge_list(
      df = .mc_matrix_to_edge_list(
        mat,
        threshold = cfg$threshold,
        diag = cfg$diag,
        symmetric = source_symmetric
      ),
      estimator_class = estimator_class,
      method = method,
      description = description,
      threshold = cfg$threshold,
      diag = cfg$diag,
      diagnostics = diagnostics,
      ci = ci,
      conf.level = conf.level,
      source_dim = source_dim,
      source_dimnames = source_dimnames,
      symmetric = source_symmetric,
      package_name = package_name,
      extra_attrs = extra_attrs
    )
    attr(out, "matrixCorr_meta") <- meta
    return(out)
  }

  out <- .mc_new_corr_sparse(
    x = .mc_matrix_to_sparse_thresholded(
      mat,
      threshold = cfg$threshold,
      diag = cfg$diag,
      symmetric = source_symmetric
    ),
    estimator_class = estimator_class,
    method = method,
    description = description,
    output = "sparse",
    threshold = cfg$threshold,
    diag = cfg$diag,
    diagnostics = diagnostics,
    ci = ci,
    conf.level = conf.level,
    symmetric = source_symmetric,
    package_name = package_name,
    extra_attrs = extra_attrs
  )
  attr(out, "matrixCorr_meta") <- meta
  out
}

#' Correlation result dimensions
#' @keywords internal
#' @noRd
.mc_corr_dim <- function(x) {
  dm <- attr(x, "corr_dim", exact = TRUE)
  if (!is.null(dm) && length(dm) == 2L) {
    return(as.integer(dm))
  }
  as.integer(dim(x))
}

#' Correlation result dimnames
#' @keywords internal
#' @noRd
.mc_corr_dimnames <- function(x) {
  dn <- attr(x, "corr_dimnames", exact = TRUE)
  if (is.list(dn) && length(dn) == 2L) {
    return(dn)
  }
  dimnames(x)
}

#' Coerce correlation object to edge data frame
#' @keywords internal
#' @noRd
.mc_corr_as_edge_df <- function(x) {
  output <- attr(x, "output", exact = TRUE) %||% "matrix"

  if (inherits(x, "corr_edge_list")) {
    out <- x
    class(out) <- "data.frame"
    out <- as.data.frame(out, stringsAsFactors = FALSE, check.names = FALSE)
    if (all(c("row", "col", "value") %in% names(out))) {
      return(out[, c("row", "col", "value"), drop = FALSE])
    }
    if (all(c("i", "j", "x") %in% names(out))) {
      dn <- .mc_corr_dimnames(x)
      ii <- as.integer(out$i)
      jj <- as.integer(out$j)
      vv <- as.numeric(out$x)
      row_lab <- if (is.null(dn[[1L]])) as.character(ii) else dn[[1L]][ii]
      col_lab <- if (is.null(dn[[2L]])) as.character(jj) else dn[[2L]][jj]
      return(data.frame(
        row = row_lab,
        col = col_lab,
        value = vv,
        stringsAsFactors = FALSE,
        check.names = FALSE
      ))
    }
    abort_bad_arg("x", "must contain columns `row`, `col`, and `value` (or internal `i`, `j`, `x`).")
  }

  if (identical(output, "sparse") || inherits(x, "sparseMatrix")) {
    sx <- Matrix::summary(x)
    ii <- sx$i
    jj <- sx$j
    vv <- sx$x
    dn <- .mc_corr_dimnames(x)
    row_lab <- if (is.null(dn[[1L]])) as.character(ii) else dn[[1L]][ii]
    col_lab <- if (is.null(dn[[2L]])) as.character(jj) else dn[[2L]][jj]
    return(data.frame(
      row = row_lab,
      col = col_lab,
      value = as.numeric(vv),
      stringsAsFactors = FALSE,
      check.names = FALSE
    ))
  }

  .mc_matrix_to_edge_list(
    as.matrix(x),
    threshold = if (identical(output, "matrix")) 0 else (attr(x, "threshold", exact = TRUE) %||% 0),
    diag = if (identical(output, "matrix")) TRUE else (attr(x, "diag", exact = TRUE) %||% TRUE),
    symmetric = attr(x, "corr_symmetric", exact = TRUE)
  )
}

#' Coerce correlation object to dense matrix
#' @keywords internal
#' @noRd
.mc_corr_as_dense_matrix <- function(x) {
  if (is.matrix(x) || inherits(x, "corr_matrix")) {
    return(as.matrix(x))
  }
  if (identical(attr(x, "output", exact = TRUE), "sparse") || inherits(x, "sparseMatrix")) {
    return(as.matrix(x))
  }

  dm <- .mc_corr_dim(x)
  if (length(dm) != 2L || any(!is.finite(dm))) {
    abort_bad_arg("x", "does not expose matrix dimensions.")
  }
  dn <- .mc_corr_dimnames(x)
  mat <- matrix(NA_real_, nrow = dm[[1L]], ncol = dm[[2L]], dimnames = dn)
  edf <- .mc_corr_as_edge_df(x)
  if (!nrow(edf)) {
    return(mat)
  }

  rmap <- dn[[1L]]
  cmap <- dn[[2L]]
  ii <- if (is.null(rmap)) as.integer(edf$row) else match(edf$row, rmap)
  jj <- if (is.null(cmap)) as.integer(edf$col) else match(edf$col, cmap)
  keep <- is.finite(ii) & is.finite(jj)
  ii <- ii[keep]
  jj <- jj[keep]
  vv <- as.numeric(edf$value[keep])
  mat[cbind(ii, jj)] <- vv

  symmetric <- isTRUE(attr(x, "corr_symmetric", exact = TRUE))
  if (symmetric) {
    mat[cbind(jj, ii)] <- vv
  }
  mat
}

#' Correlation pairs abstraction
#' @keywords internal
#' @noRd
.mc_corr_pairs <- function(x, diag = FALSE) {
  check_bool(diag, arg = "diag")
  out <- .mc_corr_as_edge_df(x)
  if (!isTRUE(diag) && nrow(out)) {
    out <- out[out$row != out$col, , drop = FALSE]
  }
  rownames(out) <- NULL
  out
}

#' Correlation overview abstraction
#' @keywords internal
#' @noRd
.mc_corr_overview <- function(x) {
  dm <- .mc_corr_dim(x)
  edges <- .mc_corr_pairs(x, diag = TRUE)
  method <- attr(x, "method", exact = TRUE) %||% .mc_corr_estimator_class(x)
  output <- attr(x, "output", exact = TRUE) %||% "matrix"
  thr <- attr(x, "threshold", exact = TRUE) %||% 0
  use_diag <- attr(x, "diag", exact = TRUE) %||% TRUE
  symmetric <- isTRUE(attr(x, "corr_symmetric", exact = TRUE))
  pair_edges <- if (isTRUE(symmetric)) {
    .mc_corr_pairs(x, diag = FALSE)
  } else {
    edges
  }
  has_ci <- !is.null(attr(x, "ci", exact = TRUE))
  inf_attr <- attr(x, "inference", exact = TRUE)
  has_p <- is.list(inf_attr) && is.matrix(inf_attr$p_value)
  conf.level <- attr(x, "conf.level", exact = TRUE) %||%
    (attr(x, "ci", exact = TRUE)$conf.level %||% NA_real_)
  ci_method <- attr(x, "ci", exact = TRUE)$ci.method %||%
    attr(x, "ci.method", exact = TRUE) %||%
    NA_character_
  inference_method <- if (is.list(inf_attr) && is.character(inf_attr$method) &&
                          length(inf_attr$method) == 1L) {
    inf_attr$method
  } else {
    NA_character_
  }

  matrix_output <- identical(output, "matrix") || inherits(x, "corr_matrix")
  use_matrix_overview <- isTRUE(matrix_output)
  selector <- NULL
  if (isTRUE(use_matrix_overview)) {
    m <- .mc_corr_as_dense_matrix(x)
    selector <- if (symmetric && dm[[1L]] == dm[[2L]]) {
      upper.tri(m, diag = FALSE)
    } else {
      matrix(TRUE, nrow = dm[[1L]], ncol = dm[[2L]])
    }
    vals <- if (is.matrix(m) && identical(dim(m), dm)) {
      as.numeric(m[selector])
    } else {
      numeric()
    }
    n_missing <- if (is.matrix(m)) sum(is.na(m)) else NA_integer_
  } else {
    vals <- as.numeric(edges$value)
    if (symmetric && dm[[1L]] == dm[[2L]] && nrow(edges)) {
      vals <- vals[as.character(edges$row) != as.character(edges$col)]
    }
    n_missing <- sum(is.na(edges$value))
  }
  vals <- vals[is.finite(vals)]
  pick_range <- function(mat) {
    if (!is.matrix(mat) || !identical(dim(mat), dm)) {
      return(c(NA_real_, NA_real_))
    }
    idx <- selector
    if (is.null(idx)) {
      idx <- if (symmetric && dm[[1L]] == dm[[2L]]) {
        upper.tri(mat, diag = FALSE)
      } else {
        matrix(TRUE, nrow = dm[[1L]], ncol = dm[[2L]])
      }
    }
    vv <- as.numeric(mat[idx])
    vv <- vv[is.finite(vv)]
    if (!length(vv)) {
      return(c(NA_real_, NA_real_))
    }
    c(min(vv), max(vv))
  }
  diag_attr <- attr(x, "diagnostics", exact = TRUE)
  skipped_n <- if (is.list(diag_attr)) pick_range(diag_attr$skipped_n) else c(NA_real_, NA_real_)
  skipped_prop <- if (is.list(diag_attr)) pick_range(diag_attr$skipped_prop) else c(NA_real_, NA_real_)
  threshold_sets <- {
    thr <- attr(x, "thresholds", exact = TRUE)
    if (is.list(thr)) length(thr) else NA_integer_
  }
  thr_num <- if (is.list(thr)) numeric() else suppressWarnings(as.numeric(thr))
  if (!length(thr_num) || !is.finite(thr_num[[1L]])) {
    thr_num <- 0
  } else {
    thr_num <- thr_num[[1L]]
  }
  conf_num <- suppressWarnings(as.numeric(conf.level))
  if (!length(conf_num) || !is.finite(conf_num[[1L]])) {
    conf_num <- NA_real_
  } else {
    conf_num <- conf_num[[1L]]
  }
  if (length(ci_method) != 1L || is.list(ci_method)) {
    ci_method <- NA_character_
  }
  if (length(inference_method) != 1L || is.list(inference_method)) {
    inference_method <- NA_character_
  }

  list(
    class = .mc_corr_estimator_class(x),
    method = method,
    output = output,
    threshold = thr_num,
    diag = isTRUE(use_diag),
    n_variables = if (symmetric) as.integer(dm[[1L]]) else NA_integer_,
    n_rows = as.integer(dm[[1L]]),
    n_cols = as.integer(dm[[2L]]),
    n_pairs = nrow(pair_edges),
    n_pairs_retained = nrow(pair_edges),
    symmetric = symmetric,
    n_missing = n_missing,
    threshold_sets = threshold_sets,
    estimate_min = if (length(vals)) min(vals) else NA_real_,
    estimate_max = if (length(vals)) max(vals) else NA_real_,
    has_ci = has_ci,
    has_p = has_p,
    conf.level = conf_num,
    ci_method = ci_method,
    inference_method = inference_method,
    skipped_n_min = skipped_n[[1L]],
    skipped_n_max = skipped_n[[2L]],
    skipped_prop_min = skipped_prop[[1L]],
    skipped_prop_max = skipped_prop[[2L]]
  )
}

#' Inform only when verbose
#' @keywords internal
inform_if_verbose <- function(...,
                              .verbose = getOption("matrixCorr.verbose", TRUE),
                              .envir = parent.frame()) {
  if (isTRUE(.verbose)) {
    cli::cli_inform(..., .envir = .envir)
  }
  invisible(NULL)
}

test_that("check_bool validates logical flags", {
  expect_invisible(check_bool(TRUE))
  expect_invisible(check_bool(FALSE))

  expect_error(check_bool(c(TRUE, FALSE)), class = "matrixCorr_arg_error")
  expect_error(check_bool(1), class = "matrixCorr_arg_error")
})

test_that("check_scalar_numeric enforces bounds and types", {
  expect_invisible(check_scalar_numeric(1))
  expect_invisible(check_scalar_numeric(0, lower = 0, upper = 1))

  expect_error(check_scalar_numeric("a"), class = "matrixCorr_arg_error")
  expect_error(check_scalar_numeric(-1, lower = 0), class = "matrixCorr_arg_error")
  expect_error(check_scalar_numeric(2, upper = 1), class = "matrixCorr_arg_error")
})

test_that("check_required_cols detects missing columns", {
  df <- data.frame(a = 1, b = 2)
  expect_invisible(check_required_cols(df, c("a")))
  expect_error(check_required_cols(df, c("a", "c")), class = "matrixCorr_arg_error")
})

test_that("check_same_length enforces matching lengths", {
  expect_invisible(check_same_length(1:3, 4:6))
  expect_error(check_same_length(1:3, 4:7), class = "matrixCorr_arg_error")
})

test_that("check_prob_scalar covers open/closed intervals", {
  expect_invisible(check_prob_scalar(0))
  expect_invisible(check_prob_scalar(1, open_ends = FALSE))
  expect_error(check_prob_scalar(-0.1), class = "matrixCorr_arg_error")
  expect_error(check_prob_scalar(1, open_ends = TRUE), class = "matrixCorr_arg_error")
})

test_that("check_weights validates length and non-negativity", {
  expect_invisible(check_weights(NULL, n = 2))
  expect_invisible(check_weights(c(0.2, 0.8), n = 2))
  expect_error(check_weights(c(1, -1), n = 2), class = "matrixCorr_arg_error")
  expect_error(check_weights(1:3, n = 2), class = "matrixCorr_arg_error")
})

test_that("match_arg wraps rlang::arg_match", {
  expect_equal(match_arg("a", c("a", "b")), "a")
  expect_error(match_arg("c", c("a", "b")), class = "rlang_error")
})

test_that("inform_if_verbose obeys verbosity flag", {
  expect_silent(inform_if_verbose("quiet", .verbose = FALSE))
  expect_message(inform_if_verbose("loud", .verbose = TRUE), "loud")
})

#' S3 Summary for Dense Correlation Results
#'
#' Representation-first summary for dense correlation outputs.
#'
#' @param object A dense correlation result (`corr_matrix`).
#' @param topn Optional number of head/tail rows when preview is truncated.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#'
#' @return A standardized summary data frame with class
#'   `c("summary.corr_result", "data.frame")` (plus compatibility classes).
#' @method summary corr_matrix
#' @export
summary.corr_matrix <- function(object, topn = NULL, show_ci = NULL, ...) {
  .mc_summary_corr_result(
    object,
    output_class = "summary.corr_matrix",
    topn = topn,
    show_ci = show_ci
  )
}

#' S3 Summary for Sparse Correlation Results
#'
#' Representation-first summary for sparse correlation outputs.
#'
#' @param object A sparse correlation result.
#' @param topn Optional number of head/tail rows when preview is truncated.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#'
#' @return A standardized summary data frame with class
#'   `c("summary.corr_result", "data.frame")` (plus compatibility classes).
#' @method summary corr_sparse
#' @export
summary.corr_sparse <- function(object, topn = NULL, show_ci = NULL, ...) {
  .mc_summary_corr_result(
    object,
    output_class = "summary.corr_sparse",
    topn = topn,
    show_ci = show_ci
  )
}

#' S3 Summary for Edge-List Correlation Results
#'
#' Representation-first summary for edge-list outputs.
#'
#' @param object An edge-list correlation result.
#' @param topn Optional number of head/tail rows when preview is truncated.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#'
#' @return A standardized summary data frame with class
#'   `c("summary.corr_result", "data.frame")` (plus compatibility classes).
#' @method summary corr_edge_list
#' @export
summary.corr_edge_list <- function(object, topn = NULL, show_ci = NULL, ...) {
  .mc_summary_corr_result(
    object,
    output_class = "summary.corr_edge_list",
    topn = topn,
    show_ci = show_ci
  )
}

#' Edge-list data-frame view
#' @param x A `corr_edge_list` object.
#' @param row.names Ignored.
#' @param optional Ignored.
#' @param ... Unused.
#' @return A data frame with columns `row`, `col`, `value`.
#' @method as.data.frame corr_edge_list
#' @export
as.data.frame.corr_edge_list <- function(x, row.names = NULL, optional = FALSE, ...) {
  .mc_corr_as_edge_df(x)
}

#' Matrix sparse fallback for correlation sparse output
#' @keywords internal
#' @exportS3Method summary sparseMatrix
#' @noRd
summary.sparseMatrix <- function(object, topn = NULL, show_ci = NULL, ...) {
  if (identical(attr(object, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(object, "output", exact = TRUE), "sparse")) {
    return(summary.corr_sparse(object, topn = topn, show_ci = show_ci, ...))
  }
  NextMethod()
}

#' @keywords internal
#' @exportS3Method summary dsCMatrix
#' @noRd
summary.dsCMatrix <- function(object, topn = NULL, show_ci = NULL, ...) {
  if (identical(attr(object, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(object, "output", exact = TRUE), "sparse")) {
    return(summary.corr_sparse(object, topn = topn, show_ci = show_ci, ...))
  }
  NextMethod()
}

#' @keywords internal
#' @exportS3Method summary dgCMatrix
#' @noRd
summary.dgCMatrix <- function(object, topn = NULL, show_ci = NULL, ...) {
  if (identical(attr(object, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(object, "output", exact = TRUE), "sparse")) {
    return(summary.corr_sparse(object, topn = topn, show_ci = show_ci, ...))
  }
  NextMethod()
}

#' @title Print Edge-List Correlation Results
#' @param x An edge-list correlation result.
#' @param digits Number of digits for numeric values.
#' @param n Optional preview row threshold.
#' @param topn Optional number of head/tail rows when preview is truncated.
#' @param max_vars Optional maximum number of visible columns in preview.
#' @param width Optional output width.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#' @method print corr_edge_list
#' @export
print.corr_edge_list <- function(x,
                                 digits = 4,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 show_ci = NULL,
                                 ...) {
  .mc_print_corr_records(
    x,
    output_label = "edge list",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @title Print Standardized Correlation Summaries
#' @param x A `summary.corr_result` object.
#' @param digits Number of digits for numeric values.
#' @param n Optional preview row threshold.
#' @param topn Optional number of head/tail rows when preview is truncated.
#' @param max_vars Optional maximum number of visible columns in preview.
#' @param width Optional output width.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#' @return Invisibly returns `x`.
#' @method print summary.corr_result
#' @export
print.summary.corr_result <- function(x,
                                      digits = 4,
                                      n = NULL,
                                      topn = NULL,
                                      max_vars = NULL,
                                      width = NULL,
                                      show_ci = NULL,
                                      ...) {
  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  overview <- attr(x, "overview", exact = TRUE)
  header <- attr(x, "summary_title", exact = TRUE) %||%
    sprintf("%s summary", attr(x, "method", exact = TRUE) %||%
      (overview$method %||% "correlation"))
  digest <- c(
    output = attr(x, "output", exact = TRUE) %||% (overview$output %||% NA_character_),
    dimensions = if (is.list(overview)) sprintf("%d x %d", overview$n_rows, overview$n_cols) else NA_character_,
    retained_pairs = if (is.list(overview)) .mc_count_fmt(overview$n_pairs_retained) else NA_character_,
    threshold = if (is.list(overview)) formatC(overview$threshold, format = "f", digits = digits) else NA_character_,
    diag = if (is.list(overview)) if (isTRUE(overview$diag)) "included" else "excluded" else NA_character_,
    estimate = if (is.list(overview)) {
      .mc_format_scalar_or_range(overview$estimate_min, overview$estimate_max, digits = digits)
    } else {
      NA_character_
    }
  )
  if (is.list(overview) && is.finite(overview$threshold_sets) && overview$threshold_sets > 0L) {
    digest <- c(digest, thresholds = sprintf("%s set(s)", .mc_count_fmt(overview$threshold_sets)))
  }
  if (is.list(overview)) {
    skipped_n_txt <- .mc_format_scalar_or_range(overview$skipped_n_min, overview$skipped_n_max, digits = digits, integer = TRUE)
    if (!is.null(skipped_n_txt)) {
      digest <- c(digest, skipped_n = skipped_n_txt)
    }
    skipped_prop_txt <- .mc_format_scalar_or_range(overview$skipped_prop_min, overview$skipped_prop_max, digits = digits, integer = FALSE)
    if (!is.null(skipped_prop_txt)) {
      digest <- c(digest, skipped_prop = skipped_prop_txt)
    }
    if (!is.null(overview$inference_method) &&
        length(overview$inference_method) == 1L &&
        !is.na(overview$inference_method) &&
        nzchar(overview$inference_method)) {
      digest <- c(digest, inference = overview$inference_method)
    }
  }
  if (identical(cfg$show_ci, "yes")) {
    summary_df <- as.data.frame(x, stringsAsFactors = FALSE)
    digest <- c(
      digest,
      .mc_ci_digest(
        summary_df,
        conf_level = suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))),
        ci_method = attr(x, "ci_method", exact = TRUE) %||%
          (attr(x, "overview", exact = TRUE)$ci_method %||% NULL),
        digits = 3
      )
    )
    constant_m <- .mc_single_display_value(summary_df$m)
    if (!is.null(constant_m)) {
      digest <- c(digest, m = constant_m)
    }
    constant_boot <- .mc_single_display_value(summary_df$bootstrap_reps)
    if (!is.null(constant_boot)) {
      digest <- c(digest, boot_reps = constant_boot)
    }
    se_vals <- suppressWarnings(as.numeric(summary_df$se))
    se_vals <- se_vals[is.finite(se_vals)]
    if (length(se_vals)) {
      se_txt <- .mc_format_scalar_or_range(
        min(se_vals),
        max(se_vals),
        digits = 3,
        integer = FALSE
      )
      if (!is.null(se_txt)) {
        digest <- c(digest, se = se_txt)
      }
    }
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = header)
  cat("\n")

  preview <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  ci_digits <- .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)
  preview <- .mc_compact_corr_summary_table(
    preview,
    show_ci = cfg$show_ci,
    conf_level = suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))),
    digits = digits,
    ci_digits = ci_digits
  )
  preview <- .mc_format_summary_numeric_table(
    preview,
    digits = digits,
    ci_digits = ci_digits
  )
  .mc_print_preview_table(
    preview,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = TRUE,
    summary_hint = FALSE
  )
  invisible(x)
}

.mc_single_display_value <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }
  vals <- unique(x[!is.na(x)])
  if (length(vals) != 1L) {
    return(NULL)
  }
  if (is.numeric(vals)) {
    vals <- vals[is.finite(vals)]
    if (length(vals) != 1L) {
      return(NULL)
    }
    return(format(vals[[1L]], trim = TRUE, scientific = FALSE))
  }
  vals <- as.character(vals)
  if (!nzchar(vals[[1L]])) {
    return(NULL)
  }
  vals[[1L]]
}

.mc_compact_corr_summary_table <- function(df,
                                           show_ci = "yes",
                                           conf_level = NA_real_,
                                           digits = 4,
                                           ci_digits = digits) {
  if (!is.data.frame(df) || !nrow(df)) {
    return(df)
  }

  out <- df
  if ("n_complete" %in% names(out)) {
    names(out)[match("n_complete", names(out))] <- "n"
  }

  if (identical(show_ci, "yes") && all(c("lwr", "upr") %in% names(out))) {
    ci_label <- if (is.finite(conf_level)) {
      sprintf("%g%% CI", 100 * conf_level)
    } else {
      "CI"
    }
    lwr <- suppressWarnings(as.numeric(out$lwr))
    upr <- suppressWarnings(as.numeric(out$upr))
    out[[ci_label]] <- ifelse(
      is.finite(lwr) & is.finite(upr),
      sprintf(
        "[%s, %s]",
        formatC(lwr, format = "f", digits = ci_digits),
        formatC(upr, format = "f", digits = ci_digits)
      ),
      NA_character_
    )
  }

  always_drop <- c(
    "lwr", "upr", "fisher_z", "statistic", "df", "se",
    "bootstrap_reps", "skipped_n", "skipped_prop"
  )
  if (identical(show_ci, "no")) {
    always_drop <- c(always_drop, grep("CI$", names(out), value = TRUE))
  }

  if ("ci_method" %in% names(out) && !is.null(.mc_single_display_value(out$ci_method))) {
    always_drop <- c(always_drop, "ci_method")
  }
  if ("m" %in% names(out) && !is.null(.mc_single_display_value(out$m))) {
    always_drop <- c(always_drop, "m")
  }
  out <- out[, setdiff(names(out), always_drop), drop = FALSE]

  preferred <- c(
    "item1", "item2", "estimate", "n",
    grep("CI$", names(out), value = TRUE),
    "p_value", "p_value_adjusted", "ci_method", "m"
  )
  preferred <- unique(preferred[preferred %in% names(out)])
  out[, c(preferred, setdiff(names(out), preferred)), drop = FALSE]
}

#' S3 Plot for Dense Correlation Results
#'
#' @param x A dense correlation result (`corr_matrix`).
#' @param title Optional plot title.
#' @param low_color Fill color for -1.
#' @param high_color Fill color for +1.
#' @param mid_color Fill color for 0.
#' @param value_text_size Text size for optional overlaid values.
#' @param ci_text_size Text size for optional confidence-interval labels.
#' @param show_value Logical; overlay values if `TRUE`.
#' @param ... Additional theme arguments.
#'
#' @return A `ggplot` heatmap.
#' @method plot corr_matrix
#' @export
plot.corr_matrix <- function(x,
                             title = NULL,
                             low_color = "indianred1",
                             high_color = "steelblue1",
                             mid_color = "white",
                             value_text_size = 4,
                             ci_text_size = 3,
                             show_value = TRUE,
                             ...) {
  .mc_plot_corr_result(
    x,
    title = title %||% "Correlation heatmap",
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    ci_text_size = ci_text_size,
    show_value = show_value,
    ...
  )
}

#' S3 Plot for Sparse Correlation Results
#'
#' @inheritParams plot.corr_matrix
#' @param x A sparse correlation result.
#' @method plot corr_sparse
#' @export
plot.corr_sparse <- function(x,
                             title = NULL,
                             low_color = "indianred1",
                             high_color = "steelblue1",
                             mid_color = "white",
                             value_text_size = 4,
                             ci_text_size = 3,
                             show_value = TRUE,
                             ...) {
  .mc_plot_corr_result(
    x,
    title = title %||% "Retained sparse correlation heatmap",
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    ci_text_size = ci_text_size,
    show_value = show_value,
    ...
  )
}

#' S3 Plot for Edge-List Correlation Results
#'
#' @inheritParams plot.corr_matrix
#' @param x An edge-list correlation result.
#' @method plot corr_edge_list
#' @export
plot.corr_edge_list <- function(x,
                                title = NULL,
                                low_color = "indianred1",
                                high_color = "steelblue1",
                                mid_color = "white",
                                value_text_size = 4,
                                ci_text_size = 3,
                                show_value = TRUE,
                                ...) {
  .mc_plot_corr_result(
    x,
    title = title %||% "Edge-list correlation heatmap",
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    ci_text_size = ci_text_size,
    show_value = show_value,
    ...
  )
}

#' Matrix sparse fallback for correlation sparse output
#' @keywords internal
#' @exportS3Method plot sparseMatrix
#' @noRd
plot.sparseMatrix <- function(x,
                              title = NULL,
                              low_color = "indianred1",
                              high_color = "steelblue1",
                              mid_color = "white",
                              value_text_size = 4,
                              ci_text_size = 3,
                              show_value = TRUE,
                              ...) {
  if (identical(attr(x, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(x, "output", exact = TRUE), "sparse")) {
    return(plot.corr_sparse(
      x,
      title = title,
      low_color = low_color,
      high_color = high_color,
      mid_color = mid_color,
      value_text_size = value_text_size,
      ci_text_size = ci_text_size,
      show_value = show_value,
      ...
    ))
  }
  graphics::plot(as.matrix(x), ...)
}

#' @keywords internal
#' @exportS3Method plot dsCMatrix
#' @noRd
plot.dsCMatrix <- function(x,
                           title = NULL,
                           low_color = "indianred1",
                           high_color = "steelblue1",
                           mid_color = "white",
                           value_text_size = 4,
                           ci_text_size = 3,
                           show_value = TRUE,
                           ...) {
  if (identical(attr(x, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(x, "output", exact = TRUE), "sparse")) {
    return(plot.corr_sparse(
      x,
      title = title,
      low_color = low_color,
      high_color = high_color,
      mid_color = mid_color,
      value_text_size = value_text_size,
      ci_text_size = ci_text_size,
      show_value = show_value,
      ...
    ))
  }
  NextMethod()
}

#' @keywords internal
#' @exportS3Method plot dgCMatrix
#' @noRd
plot.dgCMatrix <- function(x,
                           title = NULL,
                           low_color = "indianred1",
                           high_color = "steelblue1",
                           mid_color = "white",
                           value_text_size = 4,
                           ci_text_size = 3,
                           show_value = TRUE,
                           ...) {
  if (identical(attr(x, "corr_output_class", exact = TRUE), "corr_sparse") ||
      identical(attr(x, "output", exact = TRUE), "sparse")) {
    return(plot.corr_sparse(
      x,
      title = title,
      low_color = low_color,
      high_color = high_color,
      mid_color = mid_color,
      value_text_size = value_text_size,
      ci_text_size = ci_text_size,
      show_value = show_value,
      ...
    ))
  }
  NextMethod()
}

#' @keywords internal
#' @noRd
.mc_summary_corr_result <- function(object,
                                    output_class,
                                    topn = NULL,
                                    show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  raw_pairs <- .mc_corr_pairs(object, diag = FALSE)
  estimator_class <- attr(object, "corr_estimator_class", exact = TRUE) %||%
    .mc_corr_estimator_class(object)
  estimator_class <- if (is.character(estimator_class) &&
    length(estimator_class) >= 1L &&
    !is.na(estimator_class[[1L]]) &&
    nzchar(estimator_class[[1L]])) {
    estimator_class[[1L]]
  } else {
    "corr_estimator"
  }
  pairs <- data.frame(
    item1 = as.character(raw_pairs$row %||% character()),
    item2 = as.character(raw_pairs$col %||% character()),
    estimate = as.numeric(raw_pairs$value %||% numeric()),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )

  dm <- .mc_corr_dim(object)
  dn <- .mc_corr_dimnames(object)
  rn <- dn[[1L]]
  cn <- dn[[2L]]
  ii <- if (is.null(rn)) suppressWarnings(as.integer(pairs$item1)) else match(pairs$item1, rn)
  jj <- if (is.null(cn)) suppressWarnings(as.integer(pairs$item2)) else match(pairs$item2, cn)
  idx_ok <- is.finite(ii) & is.finite(jj)

  add_from_matrix <- function(mat) {
    if (!is.matrix(mat) || length(dm) != 2L || !identical(dim(mat), dm) || !nrow(pairs)) {
      return(rep(NA_real_, nrow(pairs)))
    }
    out <- rep(NA_real_, nrow(pairs))
    out[idx_ok] <- mat[cbind(ii[idx_ok], jj[idx_ok])]
    out
  }

  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
    pairs$n_complete <- as.integer(round(add_from_matrix(diag_attr$n_complete)))
  }
  if (identical(estimator_class, "chatterjee_xi") && is.list(diag_attr)) {
    if (is.matrix(diag_attr$ci_method)) {
      method_vals <- rep(NA_character_, nrow(pairs))
      method_vals[idx_ok] <- as.character(diag_attr$ci_method[cbind(ii[idx_ok], jj[idx_ok])])
      pairs$ci_method <- method_vals
    }
    if (is.matrix(diag_attr$m)) {
      pairs$m <- as.integer(round(add_from_matrix(diag_attr$m)))
    }
    if (is.matrix(diag_attr$se)) {
      pairs$se <- as.numeric(add_from_matrix(diag_attr$se))
    }
    if (!is.null(diag_attr$bootstrap_reps)) {
      pairs$bootstrap_reps <- as.integer(diag_attr$bootstrap_reps[[1L]])
    }
  }

  ci_attr <- attr(object, "ci", exact = TRUE)
  if (is.list(ci_attr)) {
    if (is.matrix(ci_attr$lwr.ci)) pairs$lwr <- as.numeric(add_from_matrix(ci_attr$lwr.ci))
    if (is.matrix(ci_attr$upr.ci)) pairs$upr <- as.numeric(add_from_matrix(ci_attr$upr.ci))
  }

  inf <- attr(object, "inference", exact = TRUE)
  if (is.list(inf)) {
    if (is.matrix(inf$statistic)) pairs$statistic <- as.numeric(add_from_matrix(inf$statistic))
    param_mat <- inf$parameter %||% inf$df
    if (is.matrix(param_mat)) pairs$df <- as.numeric(add_from_matrix(param_mat))
    if (is.matrix(inf$p_value)) pairs$p_value <- as.numeric(add_from_matrix(inf$p_value))
  } else {
    needs_fallback <- estimator_class %in% c(
      "bicor", "pearson_corr", "spearman_rho",
      "kendall_matrix", "pbcor", "wincor", "skipped_corr",
      "partial_corr_matrix"
    )
    if (isTRUE(needs_fallback)) {
      # Backward-compatible fallback for CI-only summaries that historically
      # exposed Fisher-z and p-value columns.
      z <- suppressWarnings(atanh(pairs$estimate))
      z[!is.finite(z)] <- NA_real_
      pairs$fisher_z <- z
      if ("n_complete" %in% names(pairs) && all(is.finite(pairs$n_complete))) {
        se <- 1 / sqrt(pmax(1, as.numeric(pairs$n_complete) - 3))
        pairs$statistic <- z / se
        pairs$p_value <- 2 * stats::pnorm(abs(pairs$statistic), lower.tail = FALSE)
      }
    }
  }

  if (!"fisher_z" %in% names(pairs)) {
    z <- suppressWarnings(atanh(pairs$estimate))
    z[!is.finite(z)] <- NA_real_
    pairs$fisher_z <- z
  }

  if (nrow(pairs)) {
    pairs <- pairs[order(abs(pairs$estimate), decreasing = TRUE, na.last = TRUE), , drop = FALSE]
  }
  rownames(pairs) <- NULL

  overview <- .mc_corr_overview(object)
  summary_classes <- c(
    "summary.corr_result",
    output_class,
    paste0("summary.", estimator_class),
    if (estimator_class %in% c("tetrachoric_corr", "polychoric_corr", "polyserial_corr", "biserial_corr")) {
      "summary.latent_corr"
    } else {
      character()
    },
    "summary.matrixCorr",
    "data.frame"
  )
  class(pairs) <- unique(summary_classes)

  topn_use <- .mc_coalesce(topn, .mc_display_option("summary_topn", 5L))
  topn_use <- .mc_validate_optional_count(topn_use, arg = "topn", allow_null = FALSE)
  top_tbl <- if (nrow(pairs)) {
    pairs[utils::head(order(abs(pairs$estimate), decreasing = TRUE, na.last = TRUE), topn_use), , drop = FALSE]
  } else {
    pairs
  }
  attr(pairs, "overview") <- overview
  attr(pairs, "method") <- overview$method
  attr(pairs, "output") <- overview$output
  attr(pairs, "threshold") <- overview$threshold
  attr(pairs, "diag") <- overview$diag
  attr(pairs, "has_ci") <- any(c("lwr", "upr") %in% names(pairs))
  attr(pairs, "has_p") <- "p_value" %in% names(pairs)
  attr(pairs, "conf.level") <- overview$conf.level
  ci_method_attr <- attr(object, "ci", exact = TRUE)$ci.method %||%
    attr(object, "ci.method", exact = TRUE) %||%
    NA_character_
  if (is.list(ci_method_attr)) {
    ci_method_attr <- NA_character_
  } else if (length(ci_method_attr) != 1L) {
    method_values <- unique(as.character(ci_method_attr))
    method_values <- method_values[!is.na(method_values) & nzchar(method_values)]
    ci_method_attr <- if (length(method_values) == 1L) {
      method_values
    } else if (length(method_values) > 1L) {
      "mixed"
    } else {
      NA_character_
    }
  }
  ci_method_attr <- as.character(ci_method_attr)[[1L]]
  attr(pairs, "ci_method") <- ci_method_attr
  if ((is.null(attr(pairs, "ci_method", exact = TRUE)) ||
       is.na(attr(pairs, "ci_method", exact = TRUE)) ||
       !nzchar(attr(pairs, "ci_method", exact = TRUE))) &&
      isTRUE(attr(pairs, "has_ci"))) {
    ci_map <- c(
      pearson_corr = "fisher_z",
      spearman_rho = "jackknife_euclidean_likelihood",
      kendall_matrix = "fieller"
    )
    attr(pairs, "ci_method") <- unname(ci_map[estimator_class] %||% NA_character_)
  }
  attr(pairs, "top_results") <- top_tbl
  attr(pairs, "show_ci") <- show_ci
  attr(pairs, "summary_title") <- .mc_corr_summary_title(
    estimator_class,
    has_ci = isTRUE(attr(pairs, "has_ci")),
    has_p = isTRUE(attr(pairs, "has_p"))
  )
  pairs
}

#' @keywords internal
#' @noRd
.mc_corr_summary_title <- function(estimator_class, has_ci = FALSE, has_p = FALSE) {
  estimator_class <- if (is.character(estimator_class) &&
    length(estimator_class) >= 1L &&
    !is.na(estimator_class[[1L]]) &&
    nzchar(estimator_class[[1L]])) {
    estimator_class[[1L]]
  } else {
    "corr_estimator"
  }
  if (estimator_class %in% c("tetrachoric_corr", "polychoric_corr", "polyserial_corr", "biserial_corr")) {
    if (!isTRUE(has_ci) && !isTRUE(has_p)) {
      return("Latent correlation summary")
    }
  }
  map <- c(
    pearson_corr = "Pearson correlation summary",
    chatterjee_xi = "Chatterjee xi directed-dependence summary",
    spearman_rho = if (isTRUE(has_ci) || isTRUE(has_p)) "Spearman correlation summary" else "Correlation summary",
    kendall_matrix = "Kendall correlation summary",
    cohen_kappa = "Cohen's kappa agreement summary",
    weighted_kappa = "Weighted Cohen's kappa agreement summary",
    bicor = "Biweight mid-correlation summary",
    dcor = "Distance correlation summary",
    robust_dcor = "Robust distance correlation summary",
    pbcor = "Percentage bend correlation summary",
    wincor = "Winsorized correlation summary",
    skipped_corr = "Skipped correlation summary",
    shrinkage_corr = "Correlation summary",
    schafer_corr = "Correlation summary",
    tetrachoric_corr = "Tetrachoric correlation summary",
    polychoric_corr = "Polychoric correlation summary",
    polyserial_corr = "Polyserial correlation summary",
    biserial_corr = "Biserial correlation summary",
    partial_corr_matrix = "Partial correlation summary"
  )
  label <- unname(map[estimator_class])
  if (length(label) == 0L || is.na(label) || !nzchar(label)) {
    "Correlation summary"
  } else {
    label
  }
}

#' Summary Accessor for Correlation Summaries
#' @param x A `summary.corr_result` object.
#' @param name A column name or summary metadata key.
#' @return A summary column (if present) or summary metadata entry.
#' @method $ summary.corr_result
#' @export
`$.summary.corr_result` <- function(x, name) {
  if (name %in% names(x)) {
    return(NextMethod("$"))
  }
  if (identical(name, "top_results")) {
    return(attr(x, "top_results", exact = TRUE))
  }
  overview <- attr(x, "overview", exact = TRUE)
  if (is.list(overview) && name %in% names(overview)) {
    return(overview[[name]])
  }
  attr(x, name, exact = TRUE)
}

#' Summary Accessor for Correlation Summaries
#' @param x A `summary.corr_result` object.
#' @param i A column name or summary metadata key.
#' @param ... Unused.
#' @return A summary column (if present) or summary metadata entry.
#' @method [[ summary.corr_result
#' @export
`[[.summary.corr_result` <- function(x, i, ...) {
  if (is.character(i) && length(i) == 1L && !i %in% names(x)) {
    return(`$.summary.corr_result`(x, i))
  }
  NextMethod("[[")
}

#' @keywords internal
#' @noRd
.mc_method_label <- function(x) {
  method <- attr(x, "method", exact = TRUE) %||%
    attr(x, "corr_estimator_class", exact = TRUE) %||%
    "correlation"
  if (!is.character(method) || length(method) != 1L || is.na(method) || !nzchar(method)) {
    return("Correlation")
  }
  paste0(toupper(substr(method, 1L, 1L)), substr(method, 2L, nchar(method)))
}

#' @keywords internal
#' @noRd
.mc_print_corr_records <- function(x,
                                   output_label,
                                   digits = 4,
                                   n = NULL,
                                   topn = NULL,
                                   max_vars = NULL,
                                   width = NULL,
                                   show_ci = NULL,
                                   ...) {
  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "print",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  df <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  header <- sprintf("%s correlation %s", .mc_method_label(x), output_label)
  digest <- c(
    method = attr(x, "method", exact = TRUE) %||% NA_character_,
    dimensions = sprintf("%d x %d", nrow(df), ncol(df))
  )
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = header)
  cat("\n")

  if ("value" %in% names(df)) {
    df$value <- formatC(as.numeric(df$value), format = "f", digits = digits)
  }
  .mc_print_preview_table(
    df,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "print",
    full_hint = TRUE,
    summary_hint = TRUE
  )
}

#' @keywords internal
#' @noRd
.mc_corr_plot_grid <- function(x) {
  dm <- .mc_corr_dim(x)
  dn <- .mc_corr_dimnames(x)
  edf <- .mc_corr_as_edge_df(x)
  if (!nrow(edf)) {
    return(data.frame(
      row = character(),
      col = character(),
      value = numeric(),
      lwr = numeric(),
      upr = numeric(),
      stringsAsFactors = FALSE,
      check.names = FALSE
    ))
  }
  edf$row <- as.character(edf$row)
  edf$col <- as.character(edf$col)
  edf$value <- as.numeric(edf$value)

  ci_attr <- attr(x, "ci", exact = TRUE)
  has_ci <- is.list(ci_attr) &&
    is.matrix(ci_attr$lwr.ci) &&
    is.matrix(ci_attr$upr.ci) &&
    identical(dim(ci_attr$lwr.ci), dm) &&
    identical(dim(ci_attr$upr.ci), dm)
  if (isTRUE(has_ci)) {
    ii <- if (!is.null(dn[[1L]])) match(edf$row, dn[[1L]]) else suppressWarnings(as.integer(edf$row))
    jj <- if (!is.null(dn[[2L]])) match(edf$col, dn[[2L]]) else suppressWarnings(as.integer(edf$col))
    ok <- is.finite(ii) & is.finite(jj) &
      ii >= 1L & ii <= dm[[1L]] &
      jj >= 1L & jj <= dm[[2L]]
    lwr <- rep(NA_real_, nrow(edf))
    upr <- rep(NA_real_, nrow(edf))
    if (any(ok)) {
      lwr[ok] <- ci_attr$lwr.ci[cbind(ii[ok], jj[ok])]
      upr[ok] <- ci_attr$upr.ci[cbind(ii[ok], jj[ok])]
    }
    edf$lwr <- as.numeric(lwr)
    edf$upr <- as.numeric(upr)
  } else {
    edf$lwr <- NA_real_
    edf$upr <- NA_real_
  }

  symm <- isTRUE(attr(x, "corr_symmetric", exact = TRUE))
  if (symm) {
    off <- edf$row != edf$col
    if (any(off)) {
      mirror <- edf[off, , drop = FALSE]
      mirror$row <- edf$col[off]
      mirror$col <- edf$row[off]
      edf <- rbind(
        edf,
        mirror
      )
    }
  }

  if (!is.null(dn[[1L]])) {
    edf$row <- factor(edf$row, levels = rev(dn[[1L]]))
  } else {
    edf$row <- factor(edf$row, levels = rev(as.character(seq_len(dm[[1L]]))))
  }
  if (!is.null(dn[[2L]])) {
    edf$col <- factor(edf$col, levels = dn[[2L]])
  } else {
    edf$col <- factor(edf$col, levels = as.character(seq_len(dm[[2L]])))
  }
  edf
}

#' @keywords internal
#' @noRd
.mc_plot_corr_result <- function(x,
                                 title,
                                 low_color,
                                 high_color,
                                 mid_color,
                                 value_text_size,
                                 ci_text_size,
                                 show_value,
                                 ...) {
  dots <- list(...)
  if ("type" %in% names(dots)) {
    abort_bad_arg(
      "type",
      message = "is not used for matrix-style correlation heatmaps.",
      .hint = "For estimator-specific plot modes such as {.arg type} = {.val estimate}, use a non-matrix result object."
    )
  }
  check_bool(show_value, arg = "show_value")
  df <- .mc_corr_plot_grid(x)
  if (!"ci_label" %in% names(df)) {
    has_ci_cols <- all(c("lwr", "upr") %in% names(df))
    if (isTRUE(has_ci_cols)) {
      same_cell <- as.character(df$row) == as.character(df$col)
      df$ci_label <- ifelse(
        is.finite(df$lwr) & is.finite(df$upr) & !same_cell,
        sprintf("[%.3f, %.3f]", df$lwr, df$upr),
        NA_character_
      )
    } else {
      df$ci_label <- NA_character_
    }
  }
  p <- ggplot2::ggplot(df, ggplot2::aes(.data$col, .data$row, fill = .data$value)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = "Correlation"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value) && is.finite(value_text_size) && !is.null(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", .data$value)),
      size = value_text_size,
      color = "black"
    )
  }
  if (isTRUE(show_value) &&
      is.finite(ci_text_size) &&
      !is.null(ci_text_size) &&
      any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = .data$ci_label, y = as.numeric(.data$row) - 0.25),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }
  p
}

#' Interactive Shiny Viewer for Repeated-Measures Correlation
#'
#' Launches a dedicated Shiny gadget for repeated-measures correlation matrix
#' objects of class \code{"rmcorr_matrix"}. The viewer combines the correlation
#' heatmap with a pairwise scatterplot panel that rebuilds the corresponding
#' two-variable \code{"rmcorr"} fit for user-selected variables.
#'
#' @param x An object of class \code{"rmcorr_matrix"} or a named list of such
#'   objects.
#' @param title Optional character title shown at the top of the gadget.
#' @param default_max_vars Integer; maximum number of variables pre-selected
#'   in the heatmap view when the app opens. Defaults to 40.
#'
#' @return Invisibly returns \code{NULL}; the function is called for its side
#'   effect of launching a Shiny gadget.
#'
#' @details This helper requires the \pkg{shiny} and \pkg{shinyWidgets}
#' packages at runtime and will optionally use \pkg{plotly} for the heatmap
#' when available. The pairwise panel reuses the package's regular
#' \code{plot.rmcorr()} method, so the Shiny scatterplot matches the standard
#' pairwise repeated-measures correlation plot. To rebuild pairwise fits from a
#' returned \code{"rmcorr_matrix"} object, the matrix must have been created
#' with \code{keep_data = TRUE}.
#'
#' @examples
#' if (interactive()) {
#'   set.seed(2026)
#'   n_subjects <- 20
#'   n_rep <- 4
#'   subject <- rep(seq_len(n_subjects), each = n_rep)
#'   subj_eff_x <- rnorm(n_subjects, sd = 1.5)
#'   subj_eff_y <- rnorm(n_subjects, sd = 2.0)
#'   within_signal <- rnorm(n_subjects * n_rep)
#'
#'   dat <- data.frame(
#'     subject = subject,
#'     x = subj_eff_x[subject] + within_signal + rnorm(n_subjects * n_rep, sd = 0.2),
#'     y = subj_eff_y[subject] + 0.8 * within_signal + rnorm(n_subjects * n_rep, sd = 0.3),
#'     z = subj_eff_y[subject] - 0.4 * within_signal + rnorm(n_subjects * n_rep, sd = 0.4)
#'   )
#'
#'   fit_mat <- rmcorr(
#'     dat,
#'     response = c("x", "y", "z"),
#'     subject = "subject",
#'     keep_data = TRUE
#'   )
#'   view_rmcorr_shiny(fit_mat)
#' }
#'
#' @export
view_rmcorr_shiny <- function(x, title = NULL, default_max_vars = 40L) {
  if (missing(x)) {
    abort_bad_arg("x", message = "must be supplied (an rmcorr_matrix result or list).")
  }

  if (!requireNamespace("shiny", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg shiny} is required for {.fn view_rmcorr_shiny}.")
  }
  if (!requireNamespace("shinyWidgets", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg shinyWidgets} is required for {.fn view_rmcorr_shiny}.")
  }

  prepared <- .mc_prepare_rmcorr_inputs(x)
  if (!length(prepared)) {
    cli::cli_abort("{.fn view_rmcorr_shiny} did not find a usable {.cls rmcorr_matrix} object.")
  }

  use_plotly <- .mc_has_namespace("plotly")
  app_title <- title %||% "matrixCorr repeated-measures correlation viewer"
  logo_src <- .mc_register_logo_resource(.mc_logo_path())

  heatmap_widget <- if (use_plotly) {
    .mc_plotly_fn("plotlyOutput")("heatmap", height = "650px")
  } else {
    shiny::plotOutput("heatmap", click = "heatmap_click", height = "650px")
  }

  ui <- shiny::fluidPage(
    .mc_viewer_head(),
    shiny::div(
      class = "mc-app-shell",
      .mc_viewer_header(
        title = app_title,
        subtitle = "Inspect repeated-measures correlation matrices with linked heatmap, pairwise fit, and distribution summaries.",
        logo_src = logo_src,
        badges = list(
          .mc_header_badge("Repeated Measures", tone = "neutral"),
          .mc_header_badge(
            if (use_plotly) "Interactive heatmap" else "Static heatmap",
            tone = if (use_plotly) "accent" else "neutral"
          )
        )
      ),
      shiny::sidebarLayout(
        shiny::sidebarPanel(
          width = 4,
          .mc_control_section(
            title = "Data",
            shinyWidgets::pickerInput(
              inputId = "matrix_choice",
              label = "Repeated-measures correlation object",
              choices = names(prepared),
              selected = names(prepared)[[1L]],
              multiple = FALSE,
              options = list(`live-search` = TRUE)
            ),
            shinyWidgets::pickerInput(
              inputId = "var_choice",
              label = "Heatmap variables",
              choices = colnames(prepared[[1L]]$matrix),
              selected = .mc_default_vars(prepared[[1L]]$matrix, default_max_vars),
              multiple = TRUE,
              options = list(`live-search` = TRUE, `actions-box` = TRUE)
            )
          ),
          .mc_control_section(
            title = "Heatmap",
            shiny::sliderInput(
              inputId = "threshold",
              label = "Hide cells with |value| below",
              min = 0,
              max = 1,
              step = 0.05,
              value = 0
            ),
            shiny::sliderInput(
              inputId = "corr_range",
              label = "Keep variables with correlation in range",
              min = -1,
              max = 1,
              step = 0.05,
              value = c(-1, 1)
            ),
            shiny::checkboxInput("use_abs", "Colour by absolute value", value = FALSE),
            shiny::checkboxInput("mask_diag", "Hide diagonal", value = FALSE),
            shiny::selectInput(
              inputId = "cluster_mode",
              label = "Cluster variables",
              choices = c(
                "None" = "none",
                "Absolute correlation (|r|)" = "abs",
                "Signed correlation (r)" = "signed"
              ),
              selected = "none"
            ),
            shiny::conditionalPanel(
              condition = "input.cluster_mode !== 'none'",
              shiny::selectInput(
                inputId = "cluster_method",
                label = "Linkage method",
                choices = c(
                  "Complete" = "complete",
                  "Average" = "average",
                  "Single" = "single",
                  "Ward (D2)" = "ward.D2"
                ),
                selected = "complete"
              )
            ),
            shiny::checkboxInput("show_values", "Show heatmap cell labels", value = TRUE)
          ),
          .mc_control_section(
            title = "Pair Plot",
            shiny::selectInput(
              inputId = "pair_x",
              label = "Pair plot X variable",
              choices = colnames(prepared[[1L]]$matrix),
              selected = colnames(prepared[[1L]]$matrix)[[1L]]
            ),
            shiny::selectInput(
              inputId = "pair_y",
              label = "Pair plot Y variable",
              choices = colnames(prepared[[1L]]$matrix),
              selected = colnames(prepared[[1L]]$matrix)[[2L]]
            ),
            shiny::checkboxInput("pair_show_legend", "Show subject legend", value = FALSE)
          ),
          .mc_control_section(
            title = "Actions",
            shiny::div(
              class = "mc-viewer-controls",
              shiny::actionButton("reset_controls", "Reset controls"),
              shiny::downloadButton("download_matrix", "Download displayed matrix")
            )
          )
        ),
        shiny::mainPanel(
          width = 8,
          shiny::div(
            class = "mc-tabset-shell",
            shiny::tabsetPanel(
              id = "rmcorr_view",
              shiny::tabPanel(
                title = "Heatmap",
                shiny::div(
                  class = "mc-card mc-main-card",
                  .mc_card_header(
                    title = "Heatmap",
                    subtitle = "Click a cell to open the corresponding repeated-measures pair plot."
                  ),
                  heatmap_widget,
                  shiny::uiOutput("status_alerts")
                ),
                shiny::div(
                  class = "mc-card mc-meta-card",
                  .mc_card_header(
                    title = "Summary",
                    subtitle = "Current repeated-measures matrix and display state."
                  ),
                  shiny::uiOutput("heatmap_meta")
                )
              ),
              shiny::tabPanel(
                title = "Distribution",
                shiny::fluidRow(
                  shiny::column(
                    width = 7,
                    shiny::div(
                      class = "mc-card mc-main-card",
                      .mc_card_header(
                        title = "Correlation Distribution",
                        subtitle = "Boxplot of displayed off-diagonal values using unique variable pairs."
                      ),
                      shiny::plotOutput("dist_plot", height = "430px")
                    )
                  ),
                  shiny::column(
                    width = 5,
                    shiny::div(
                      class = "mc-card mc-meta-card",
                      .mc_card_header(
                        title = "Summary Table",
                        subtitle = "Computed from the currently displayed upper triangle."
                      ),
                      shiny::tableOutput("dist_summary")
                    )
                  )
                )
              ),
              shiny::tabPanel(
                title = "Pair Plot",
                shiny::div(
                  class = "mc-card mc-main-card",
                  .mc_card_header(
                    title = "Pair Plot",
                    subtitle = "Repeated-measures fit for the currently selected variable pair."
                  ),
                  shiny::plotOutput("pair_plot", height = "650px")
                ),
                shiny::div(
                  class = "mc-card mc-meta-card",
                  .mc_card_header(
                    title = "Pair Summary",
                    subtitle = "Model estimates for the selected repeated-measures pair."
                  ),
                  shiny::uiOutput("pair_meta")
                )
              )
            )
          )
        )
      )
    )
  )

  server <- function(input, output, session) {
    shiny::observe({
      .mc_update_corr_range_slider(
        session = session,
        input_id = "corr_range",
        use_abs = isTRUE(input$use_abs),
        value = input$corr_range
      )
    })

    shiny::observeEvent(input$matrix_choice, {
      info <- prepared[[input$matrix_choice]]
      vars <- colnames(info$matrix)
      pair_default <- .mc_rmcorr_default_pair(vars)
      shinyWidgets::updatePickerInput(
        session,
        inputId = "var_choice",
        choices = vars,
        selected = .mc_default_vars(info$matrix, default_max_vars)
      )
      shiny::updateSelectInput(session, "pair_x", choices = vars, selected = pair_default[[1L]])
      shiny::updateSelectInput(session, "pair_y", choices = vars, selected = pair_default[[2L]])
      shiny::updateSliderInput(
        session,
        "threshold",
        min = 0,
        max = 1,
        value = min(input$threshold %||% 0, 1)
      )
    }, ignoreNULL = FALSE)

    shiny::observeEvent(input$reset_controls, {
      info <- current_info()
      vars <- colnames(info$matrix)
      pair_default <- .mc_rmcorr_default_pair(vars)
      shinyWidgets::updatePickerInput(
        session,
        inputId = "var_choice",
        choices = vars,
        selected = .mc_default_vars(info$matrix, default_max_vars)
      )
      shiny::updateSliderInput(session, "threshold", value = 0)
      shiny::updateCheckboxInput(session, "use_abs", value = FALSE)
      shiny::updateCheckboxInput(session, "mask_diag", value = FALSE)
      shiny::updateSelectInput(session, "cluster_mode", selected = "none")
      shiny::updateSelectInput(session, "cluster_method", selected = "complete")
      shiny::updateCheckboxInput(session, "show_values", value = TRUE)
      shiny::updateSelectInput(session, "pair_x", choices = vars, selected = pair_default[[1L]])
      shiny::updateSelectInput(session, "pair_y", choices = vars, selected = pair_default[[2L]])
      shiny::updateCheckboxInput(session, "pair_show_legend", value = FALSE)
      .mc_update_corr_range_slider(
        session = session,
        input_id = "corr_range",
        use_abs = FALSE,
        value = c(-1, 1)
      )
    })

    current_info <- shiny::reactive({
      prepared[[input$matrix_choice]]
    })

    filtered_heatmap <- shiny::reactive({
      info <- current_info()
      vars <- input$var_choice
      if (is.null(vars) || !length(vars)) {
        vars <- colnames(info$matrix)
      }
      vars <- intersect(vars, colnames(info$matrix))
      shiny::validate(shiny::need(length(vars) >= 2L, "Select at least two variables."))
      M <- info$matrix[vars, vars, drop = FALSE]
      range_filter <- .mc_filter_matrix_by_range(
        mat = M,
        corr_range = input$corr_range,
        use_abs = isTRUE(input$use_abs)
      )
      M <- range_filter$matrix
      shiny::validate(shiny::need(
        ncol(M) >= 2L,
        "Correlation range filter left fewer than two variables. Widen the range or select more variables."
      ))
      diag_map <- diag(M)
      if (!is.null(colnames(M))) names(diag_map) <- colnames(M)
      cluster_mode <- input$cluster_mode %||% "none"
      cluster_method <- input$cluster_method %||% "complete"
      status_messages <- range_filter$message
      if (!identical(cluster_mode, "none")) {
        if (nrow(M) < 3L) {
          status_messages <- c(status_messages, "Clustering requires at least three variables.")
        } else {
          res <- .mc_reorder_matrix(M, mode = cluster_mode, method = cluster_method)
          M <- res$matrix
          if (length(diag_map)) {
            if (!is.null(res$order)) {
              diag_map <- diag_map[res$order]
            }
            if (!is.null(colnames(M))) {
              names(diag_map) <- colnames(M)
            }
          }
          status_messages <- c(status_messages, res$message)
        }
      }
      thr <- input$threshold %||% 0
      if (thr > 0) {
        suppress <- abs(M) < thr
        M[suppress] <- NA_real_
      }
      if (isTRUE(input$mask_diag)) {
        diag(M) <- NA_real_
      } else if (length(diag_map)) {
        if (!is.null(colnames(M))) {
          diag(M) <- diag_map[colnames(M)]
        } else {
          diag(M) <- diag_map
        }
      }
      if (isTRUE(input$show_values) && prod(dim(M)) > .mc_heatmap_label_limit(use_plotly = use_plotly)) {
        status_messages <- c(
          status_messages,
          "Cell labels are hidden automatically for larger heatmaps to keep the viewer responsive."
        )
      }
      list(
        matrix = M,
        meta = info,
        status_messages = status_messages[!is.na(status_messages) & nzchar(status_messages)]
      )
    })

    pair_fit <- shiny::reactive({
      info <- current_info()
      vars <- colnames(info$matrix)
      x_var <- input$pair_x
      y_var <- input$pair_y
      shiny::validate(shiny::need(x_var %in% vars, "Choose a valid X variable."))
      shiny::validate(shiny::need(y_var %in% vars, "Choose a valid Y variable."))
      shiny::validate(shiny::need(!identical(x_var, y_var), "Choose two distinct variables."))
      .mc_rmcorr_view_pair_fit(info, x_var = x_var, y_var = y_var)
    })

    heatmap_click_pair <- shiny::reactive({
      if (use_plotly) {
        .mc_plotly_click_pair(
          .mc_plotly_fn("event_data")("plotly_click", source = "matrixcorr-rmcorr-heatmap"),
          vars = colnames(filtered_heatmap()$matrix)
        )
      } else {
        .mc_static_click_pair(
          .mc_heatmap_df(if (isTRUE(input$use_abs)) abs(filtered_heatmap()$matrix) else filtered_heatmap()$matrix),
          input$heatmap_click
        )
      }
    })

    shiny::observeEvent(heatmap_click_pair(), {
      pair <- heatmap_click_pair()
      if (is.null(pair) || length(pair) != 2L) {
        return()
      }
      shiny::updateSelectInput(session, "pair_x", selected = pair[[1L]])
      shiny::updateSelectInput(session, "pair_y", selected = pair[[2L]])
      shiny::updateTabsetPanel(session, "rmcorr_view", selected = "Pair Plot")
    }, ignoreInit = TRUE)

    output$heatmap_meta <- shiny::renderUI({
      res <- filtered_heatmap()
      info <- res$meta
      dims <- dim(res$matrix)
      total_vars <- ncol(info$matrix)
      diag_attr <- attr(info$matrix, "diagnostics", exact = TRUE)
      conf_level <- diag_attr$conf_level %||% info$conf_level
      .mc_meta_grid(c(
        "Result" = info$label,
        "Object class" = info$class,
        "Displayed variables" = sprintf("%d of %d", dims[[1L]], total_vars),
        "Subjects" = as.character(info$n_subjects),
        "Confidence level" = format(signif(conf_level, digits = 3))
      ))
    })

    output$status_alerts <- shiny::renderUI({
      .mc_status_notices(filtered_heatmap()$status_messages)
    })

    output$download_matrix <- shiny::downloadHandler(
      filename = function() {
        sprintf(
          "%s-displayed-matrix.csv",
          .mc_sanitize_filename(input$matrix_choice %||% "matrixCorr-rmcorr")
        )
      },
      content = function(file) {
        utils::write.csv(filtered_heatmap()$matrix, file = file, row.names = TRUE)
      }
    )

    heatmap_plot <- shiny::reactive({
      res <- filtered_heatmap()
      .mc_build_heatmap(
        mat = res$matrix,
        signed = TRUE,
        show_values = isTRUE(input$show_values),
        use_abs = isTRUE(input$use_abs),
        use_plotly = use_plotly,
        plotly_source = "matrixcorr-rmcorr-heatmap"
      )
    })

    distribution_values <- shiny::reactive({
      .mc_corr_distribution_values(
        mat = filtered_heatmap()$matrix,
        use_abs = isTRUE(input$use_abs)
      )
    })

    if (use_plotly) {
      output$heatmap <- .mc_plotly_fn("renderPlotly")({
        heatmap_plot()
      })
    } else {
      output$heatmap <- shiny::renderPlot({
        heatmap_plot()
      })
    }

    output$dist_plot <- shiny::renderPlot({
      values <- distribution_values()
      shiny::validate(shiny::need(
        length(values) > 0L,
        "No off-diagonal correlations are visible after filtering."
      ))
      .mc_build_corr_boxplot(values = values, use_abs = isTRUE(input$use_abs))
    })

    output$dist_summary <- shiny::renderTable(
      {
        .mc_corr_summary_table(distribution_values())
      },
      striped = TRUE,
      bordered = FALSE,
      spacing = "s",
      width = "100%",
      rownames = FALSE
    )

    output$pair_plot <- shiny::renderPlot({
      print(plot(pair_fit(), show_legend = isTRUE(input$pair_show_legend)))
    })

    output$pair_meta <- shiny::renderUI({
      fit <- pair_fit()
      .mc_meta_grid(c(
        "Pair" = sprintf("%s vs %s", fit$responses[[1L]], fit$responses[[2L]]),
        "Correlation" = format(signif(fit$r, digits = 4)),
        "Slope" = format(signif(fit$slope, digits = 4)),
        "P-value" = format(signif(fit$p_value, digits = 4)),
        "Degrees of freedom" = format(signif(fit$df, digits = 4)),
        "Subjects" = as.character(fit$n_subjects),
        "Observations" = as.character(fit$n_obs)
      ))
    })
  }

  shiny::runApp(shiny::shinyApp(ui = ui, server = server))
  invisible(NULL)
}

.mc_prepare_rmcorr_inputs <- function(x) {
  objects <-
    if (is.list(x) && !is.matrix(x) && !inherits(x, "Matrix") &&
        !inherits(x, "rmcorr_matrix")) {
      if (is.null(names(x))) {
        names(x) <- paste0("rmcorr ", seq_along(x))
      }
      x
    } else {
      list(default = x)
    }

  out <- list()
  for (nm in names(objects)) {
    obj <- objects[[nm]]
    parsed <- try(.mc_parse_rmcorr_object(obj, label = nm), silent = TRUE)
    if (!inherits(parsed, "try-error")) {
      out[[parsed$name]] <- parsed
    }
  }
  out
}

.mc_parse_rmcorr_object <- function(obj, label) {
  if (!inherits(obj, "rmcorr_matrix")) {
    cli::cli_abort("Unsupported object class.")
  }

  mat <- as.matrix(obj)
  if (!is.matrix(mat) || nrow(mat) != ncol(mat)) {
    cli::cli_abort("Repeated-measures correlation matrices must be square.")
  }
  if (is.null(colnames(mat))) {
    colnames(mat) <- rownames(mat) <- paste0("V", seq_len(ncol(mat)))
  }

  source_data <- attr(obj, "source_data", exact = TRUE)
  if (!is.list(source_data)) {
    cli::cli_abort(
      "{.cls rmcorr_matrix} objects need embedded {.field source_data} for pair plots. Refit with {.code keep_data = TRUE}."
    )
  }
  if (is.null(source_data$response) ||
      is.null(source_data$response_names) ||
      is.null(source_data$subject_code) ||
      is.null(source_data$subject_levels)) {
    cli::cli_abort("{.field source_data} is incomplete for rebuilding pair plots.")
  }

  response <- as.matrix(source_data$response)
  colnames(response) <- source_data$response_names
  if (!identical(colnames(response), colnames(mat))) {
    cli::cli_abort("{.field source_data} response columns do not match the matrix variables.")
  }
  if (nrow(response) != length(source_data$subject_code)) {
    cli::cli_abort("{.field source_data} subject length does not match the stored responses.")
  }

  list(
    name = label,
    matrix = mat,
    label = attr(obj, "description", exact = TRUE) %||% label,
    class = paste(class(obj), collapse = ", "),
    source_data = source_data,
    conf_level = attr(obj, "diagnostics", exact = TRUE)$conf_level %||% NA_real_,
    n_subjects = length(source_data$subject_levels)
  )
}

.mc_rmcorr_default_pair <- function(vars) {
  if (length(vars) < 2L) {
    cli::cli_abort("Repeated-measures correlation viewers require at least two variables.")
  }
  vars[1:2]
}

.mc_rmcorr_view_pair_fit <- function(info, x_var, y_var) {
  source_data <- info$source_data
  response <- as.matrix(source_data$response)
  colnames(response) <- source_data$response_names
  subject <- factor(
    source_data$subject_levels[source_data$subject_code],
    levels = source_data$subject_levels
  )
  estimator <- source_data$estimator %||% "ancova"
  ci_method <- source_data$ci_method %||% "auto"
  n_boot <- source_data$n_boot %||% 999L

  dat <- data.frame(
    .x = response[, x_var],
    .y = response[, y_var],
    .subject = subject,
    check.names = FALSE
  )

  fit <- rmcorr(
    data = dat,
    response = c(".x", ".y"),
    subject = ".subject",
    na_method = "pairwise",
    conf_level = source_data$conf_level %||% info$conf_level %||% 0.95,
    estimator = estimator,
    ci_method = ci_method,
    n_boot = n_boot,
    keep_data = TRUE
  )
  fit$responses <- c(x_var, y_var)
  fit
}

# Latent categorical / ordinal correlation wrappers

.mc_has_missing_or_bad <- function(x) {
  if (is.matrix(x)) {
    if (is.logical(x)) {
      return(anyNA(x))
    }
    return(anyNA(x) || !all(is.finite(x)))
  }
  if (is.factor(x) || is.logical(x) || is.character(x)) {
    return(anyNA(x))
  }
  anyNA(x) || any(!is.finite(x))
}

.mc_is_integerish_numeric <- function(x, tol = 1e-8) {
  is.numeric(x) && all(is.na(x) | abs(x - round(x)) <= tol)
}

.mc_as_data_frame <- function(data, arg = as.character(substitute(data))) {
  if (is.data.frame(data)) {
    return(data)
  }
  if (is.matrix(data)) {
    return(as.data.frame(data, stringsAsFactors = FALSE))
  }
  abort_bad_arg(
    arg,
    message = "must be a matrix or data frame."
  )
}

.mc_encode_ordinal_vector <- function(x, binary = FALSE) {
  if (is.logical(x)) {
    levels_x <- c("FALSE", "TRUE")
    code <- ifelse(is.na(x), NA_integer_, ifelse(x, 2L, 1L))
  } else if (is.factor(x)) {
    levels_x <- levels(x)
    code <- as.integer(x)
  } else if (.mc_is_integerish_numeric(x)) {
    vals <- sort(unique(as.numeric(x[!is.na(x)])))
    if (length(vals) == 0L) {
      return(NULL)
    }
    levels_x <- as.character(vals)
    code <- match(as.numeric(x), vals)
  } else {
    return(NULL)
  }

  if (length(levels_x) < 2L) {
    return(NULL)
  }
  if (binary && length(levels_x) != 2L) {
    return(NULL)
  }

  list(code = as.integer(code), levels = levels_x)
}

.mc_extract_discrete_columns <- function(data, kind = c("ordinal", "binary"),
                                         arg = as.character(substitute(data)),
                                         min_cols = 1L) {
  kind <- match.arg(kind)
  want_binary <- identical(kind, "binary")

  if (is.null(dim(data)) && is.atomic(data)) {
    enc <- .mc_encode_ordinal_vector(data, binary = want_binary)
    if (is.null(enc)) {
      msg <- if (want_binary) {
        "must be a binary (two-level) variable."
      } else {
        "must be an ordinal variable."
      }
      abort_bad_arg(arg, message = msg)
    }
    out <- list(enc)
    names(out) <- arg
    return(out)
  }

  df <- .mc_as_data_frame(data, arg = arg)

  keep <- vapply(df, function(col) {
    !is.null(.mc_encode_ordinal_vector(col, binary = want_binary))
  }, logical(1))

  df <- df[keep]
  if (ncol(df) < min_cols) {
    msg <- if (want_binary) {
      "must contain at least {min_cols} binary column{?s}."
    } else {
      paste0(
        "must contain at least {min_cols} ordinal column{?s} ",
        "(factor, ordered factor, logical, or integer-like numeric)."
      )
    }
    abort_bad_arg(arg, message = msg, min_cols = min_cols)
  }

  encoded <- lapply(df, .mc_encode_ordinal_vector, binary = want_binary)
  names(encoded) <- names(df)
  encoded
}

.mc_extract_continuous_matrix <- function(x,
                                          arg = as.character(substitute(x)),
                                          min_cols = 1L) {
  if (is.null(dim(x)) && is.atomic(x) && !is.factor(x)) {
    if (!is.numeric(x)) {
      abort_bad_arg(arg, message = "must be numeric.")
    }
    mat <- matrix(as.numeric(x), ncol = 1L)
    colnames(mat) <- arg
    return(mat)
  }

  df <- .mc_as_data_frame(x, arg = arg)
  keep <- vapply(df, is.numeric, logical(1))
  df <- df[keep]
  if (ncol(df) < min_cols) {
    abort_bad_arg(
      arg,
      message = "must contain at least {min_cols} numeric column{?s}.",
      min_cols = min_cols
    )
  }
  data.matrix(df)
}

.mc_check_latent_missing <- function(x, check_na, arg) {
  check_bool(check_na)
  if (!check_na) {
    return(invisible(NULL))
  }
  if (is.matrix(x)) {
    if (.mc_has_missing_or_bad(x)) {
      abort_bad_arg(
        arg,
        message = "contains missing, NaN, or infinite values.",
        .hint = "Use `na_method = \"pairwise\"` to use pairwise complete cases."
      )
    }
    return(invisible(NULL))
  }
  if (!is.list(x)) {
    x <- list(x)
  }
  bad <- vapply(x, .mc_has_missing_or_bad, logical(1))
  if (any(bad)) {
    abort_bad_arg(
      arg,
      message = "contains missing, NaN, or infinite values.",
      .hint = "Use `na_method = \"pairwise\"` to use pairwise complete cases."
    )
  }
  invisible(NULL)
}

.mc_scalar_or_matrix <- function(mat, scalar) {
  if (scalar) {
    return(as.numeric(mat[1L, 1L]))
  }
  mat
}

.mc_attach_scalar_latent <- function(x, method, description,
                                     diagnostics = NULL, thresholds = NULL) {
  extra_attrs <- list(estimate = as.numeric(x))
  if (is.list(diagnostics) && length(diagnostics$n_complete) == 1L) {
    extra_attrs$n_obs <- as.integer(diagnostics$n_complete)
  }
  do.call(
    structure,
    c(
      list(
        .Data = as.numeric(x),
        method = method,
        description = description,
        package = "matrixCorr",
        diagnostics = diagnostics,
        thresholds = thresholds
      ),
      extra_attrs
    )
  )
}

.mc_attach_latent_inference <- function(x, payload, conf_level = NULL) {
  attrs <- attributes(x)
  if (is.list(payload$inference)) {
    attrs$inference <- payload$inference
    if (length(payload$inference$estimate) == 1L) {
      attrs$estimate <- as.numeric(payload$inference$estimate)
    }
    if (length(payload$inference$n_obs) == 1L) {
      attrs$n_obs <- as.integer(payload$inference$n_obs)
    }
  }
  if (is.list(payload$ci)) {
    attrs$ci <- payload$ci
    attrs$estimate <- as.numeric(payload$ci$est)
    attrs$lwr <- as.numeric(payload$ci$lwr.ci)
    attrs$upr <- as.numeric(payload$ci$upr.ci)
    attrs$conf.level <- conf_level %||% payload$ci$conf.level
    attrs$conf_level <- conf_level %||% payload$ci$conf.level
  }
  attributes(x) <- attrs
  x
}

.mc_latent_ci_attr <- function(x) {
  attr(x, "ci", exact = TRUE)
}

.mc_latent_inference_attr <- function(x) {
  attr(x, "inference", exact = TRUE)
}

.mc_latent_scalar_payload <- function(fit,
                                      return_ci = FALSE,
                                      return_p = FALSE) {
  list(
    inference = if (isTRUE(return_p)) {
      list(
        method = as.character(fit$inference.method),
        estimate = as.numeric(fit$estimate),
        statistic = as.numeric(fit$statistic),
        parameter = as.numeric(fit$parameter),
        p_value = as.numeric(fit$p_value),
        n_obs = as.integer(fit$n_obs),
        alternative = "two.sided"
      )
    } else {
      NULL
    },
    ci = if (isTRUE(return_ci)) {
      list(
        est = as.numeric(fit$estimate),
        lwr.ci = as.numeric(fit$lwr),
        upr.ci = as.numeric(fit$upr),
        conf.level = as.numeric(fit$conf.level),
        ci.method = as.character(fit$ci.method)
      )
    } else {
      NULL
    }
  )
}

.mc_new_metric_matrix <- function(template, mode = c("double", "integer")) {
  mode <- match.arg(mode)
  matrix(
    if (identical(mode, "integer")) NA_integer_ else NA_real_,
    nrow = nrow(template),
    ncol = ncol(template),
    dimnames = dimnames(template)
  )
}

.mc_latent_matrix_payload <- function(estimate,
                                      statistic = NULL,
                                      parameter = NULL,
                                      p_value = NULL,
                                      n_obs = NULL,
                                      lwr = NULL,
                                      upr = NULL,
                                      conf_level = NULL,
                                      ci_method = NULL,
                                      inference_method = NULL,
                                      return_ci = FALSE,
                                      return_p = FALSE) {
  list(
    inference = if (isTRUE(return_p)) {
      list(
        method = inference_method,
        estimate = estimate,
        statistic = statistic,
        parameter = parameter,
        p_value = p_value,
        n_obs = n_obs,
        alternative = "two.sided"
      )
    } else {
      NULL
    },
    ci = if (isTRUE(return_ci)) {
      list(
        est = estimate,
        lwr.ci = lwr,
        upr.ci = upr,
        conf.level = conf_level,
        ci.method = ci_method
      )
    } else {
      NULL
    }
  )
}

.mc_recenter_wald_fit <- function(fit, estimate, conf_level) {
  fit$estimate <- as.numeric(estimate)
  se <- as.numeric(fit$se)
  est <- as.numeric(fit$estimate)

  if (is.finite(est) && is.finite(se) && se > 0) {
    z_crit <- stats::qnorm(0.5 * (1 + conf_level))
    fit$statistic <- est / se
    fit$p_value <- 2 * stats::pnorm(abs(fit$statistic), lower.tail = FALSE)
    fit$lwr <- max(-1, est - z_crit * se)
    fit$upr <- min(1, est + z_crit * se)
  } else {
    fit$statistic <- NA_real_
    fit$p_value <- NA_real_
    fit$lwr <- NA_real_
    fit$upr <- NA_real_
  }

  fit
}

.mc_tetrachoric_inference_one <- function(tab, correct, conf_level) {
  fit <- matrixCorr_tetrachoric_inference_cpp(
    tab = unclass(tab),
    correct = correct,
    conf_level = conf_level
  )
  .mc_recenter_wald_fit(
    fit,
    estimate = .mc_tetrachoric_table_estimate(tab, correct = correct),
    conf_level = conf_level
  )
}

.mc_polychoric_inference_one <- function(tab, correct, conf_level) {
  fit <- matrixCorr_polychoric_inference_cpp(
    tab = unclass(tab),
    correct = correct,
    conf_level = conf_level
  )
  .mc_recenter_wald_fit(
    fit,
    estimate = .mc_polychoric_table_estimate(tab, correct = correct),
    conf_level = conf_level
  )
}

.mc_polyserial_inference_one <- function(x, y, check_na, conf_level) {
  pair <- .mc_pair_complete(x, y, check_na)
  out <- list(
    estimate = NA_real_,
    statistic = NA_real_,
    parameter = NA_real_,
    p_value = NA_real_,
    n_obs = as.integer(length(pair$x)),
    lwr = NA_real_,
    upr = NA_real_,
    ci.method = "wald_information_polyserial",
    inference.method = "wald_z_polyserial"
  )

  if (length(pair$x) < 2L) {
    return(out)
  }
  yy <- pair$y[!is.na(pair$y)]
  if (length(unique(yy)) < 2L) {
    return(out)
  }

  fit <- matrixCorr_polyserial_inference_cpp(
    x = as.numeric(pair$x),
    y = as.integer(as.factor(pair$y)),
    conf_level = conf_level
  )
  fit$n_obs <- as.integer(fit$n_obs)
  fit
}

.mc_print_corr_matrix <- function(x,
                                  header,
                                  digits = 4,
                                  n = NULL,
                                  topn = NULL,
                                  max_vars = NULL,
                                  width = NULL,
                                  show_ci = NULL,
                                  mat = NULL,
                                  ...) {
  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "print",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  m <- as.matrix(.mc_coalesce(mat, x))
  attributes(m) <- attributes(m)[c("dim", "dimnames")]
  ci_present <- .mc_has_ci_payload(x)
  p_present <- .mc_has_p_payload(x)

  cat(header, "\n", sep = "")
  digest <- c(
      method = .mc_coalesce(attr(x, "method", exact = TRUE), NA_character_),
      dimensions = sprintf("%d x %d", nrow(m), ncol(m)),
      if (ci_present) c(ci = "yes"),
      if (p_present) c(p_values = "yes")
    )
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest)
  cat("\n")
  do.call(
    .mc_print_preview_matrix,
    c(
      list(
        m = m,
        digits = digits,
        n = cfg$n,
        topn = cfg$topn,
        max_vars = cfg$max_vars,
        width = cfg$width,
        context = "print",
        full_hint = TRUE,
        summary_hint = TRUE
      ),
      legacy$dots
    )
  )

  invisible(x)
}

.mc_plot_corr_matrix <- function(x, class_name, fill_name, title,
                                 low_color = "indianred1",
                                 high_color = "steelblue1",
                                 mid_color = "white",
                                 value_text_size = 4,
                                 show_value = TRUE, ...) {
  check_inherits(x, class_name)
  check_bool(show_value, arg = "show_value")

  mat <- as.matrix(x)
  df <- as.data.frame(as.table(mat))
  colnames(df) <- c("Var1", "Var2", fill_name)
  df$Var1 <- factor(df$Var1, levels = rev(unique(df$Var1)))

  p <- ggplot2::ggplot(df, ggplot2::aes(Var2, Var1, fill = .data[[fill_name]])) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = fill_name
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", .data[[fill_name]])),
      size = value_text_size,
      color = "black"
    )
  }

  p
}

.mc_summary_corr_matrix <- function(object,
                                    header = "Correlation summary",
                                    topn = NULL) {
  m <- as.matrix(object)
  symmetric <- isTRUE(nrow(m) == ncol(m)) && isTRUE(isSymmetric(unclass(m)))
  vals <- if (symmetric && nrow(m) > 1L) {
    m[upper.tri(m, diag = FALSE)]
  } else {
    as.vector(m)
  }
  vals <- vals[is.finite(vals)]
  n_pairs <- if (symmetric) {
    stats::setNames(nrow(m) * (nrow(m) - 1L) / 2L, NULL)
  } else {
    stats::setNames(nrow(m) * ncol(m), NULL)
  }

  out <- list(
    class = class(object)[1L],
    method = attr(object, "method"),
    description = attr(object, "description"),
    correct = attr(object, "correct"),
    n_variables = if (symmetric) nrow(m) else NA_integer_,
    n_pairs = as.integer(n_pairs),
    n_rows = nrow(m),
    n_cols = ncol(m),
    symmetric = symmetric,
    n_missing = sum(is.na(m)),
    estimate_min = if (length(vals)) min(vals) else NA_real_,
    estimate_max = if (length(vals)) max(vals) else NA_real_,
    threshold_sets = {
      thr <- attr(object, "thresholds")
      if (is.list(thr)) length(thr) else NA_integer_
    },
    has_ci = .mc_has_ci_payload(object),
    has_p = .mc_has_p_payload(object)
  )
  extremes <- .mc_pair_extremes(object, digits = 4)
  out$most_negative <- extremes$most_negative
  out$most_positive <- extremes$most_positive

  diag_attr <- attr(object, "diagnostics")
  if (is.list(diag_attr)) {
    selector <- if (symmetric && nrow(m) > 1L) upper.tri(m, diag = FALSE) else matrix(TRUE, nrow(m), ncol(m))
    pair_pick <- function(x, transform = identity, default = NA_integer_) {
      if (!is.matrix(x) || !identical(dim(x), dim(m))) return(default)
      vals <- transform(x[selector])
      vals <- vals[!is.na(vals)]
      vals <- vals[is.finite(vals)]
      if (!length(vals)) return(default)
      vals
    }

    zero_vals <- pair_pick(diag_attr$zero_cells)
    ncomp_vals <- pair_pick(diag_attr$n_complete)
    skipped_n_vals <- pair_pick(diag_attr$skipped_n, default = integer())
    skipped_prop_vals <- pair_pick(diag_attr$skipped_prop, default = numeric())
    boundary_vals <- pair_pick(diag_attr$boundary, as.logical, NA)
    near_vals <- pair_pick(diag_attr$near_boundary, as.logical, NA)
    corrected_vals <- pair_pick(diag_attr$corrected, as.logical, NA)
    conv_vals <- pair_pick(diag_attr$converged, as.logical, NA)

    out$zero_cell_pairs <- if (is.atomic(zero_vals) && length(zero_vals)) sum(zero_vals > 0, na.rm = TRUE) else NA_integer_
    out$boundary_pairs <- if (is.atomic(boundary_vals) && length(boundary_vals)) sum(boundary_vals, na.rm = TRUE) else NA_integer_
    out$near_boundary_pairs <- if (is.atomic(near_vals) && length(near_vals)) sum(near_vals, na.rm = TRUE) else NA_integer_
    out$corrected_pairs <- if (is.atomic(corrected_vals) && length(corrected_vals)) sum(corrected_vals, na.rm = TRUE) else NA_integer_
    out$converged_pairs <- if (is.atomic(conv_vals) && length(conv_vals)) sum(conv_vals, na.rm = TRUE) else NA_integer_
    out$n_complete_min <- if (is.atomic(ncomp_vals) && length(ncomp_vals)) min(ncomp_vals, na.rm = TRUE) else NA_integer_
    out$n_complete_max <- if (is.atomic(ncomp_vals) && length(ncomp_vals)) max(ncomp_vals, na.rm = TRUE) else NA_integer_
    out$skipped_n_min <- if (is.atomic(skipped_n_vals) && length(skipped_n_vals)) min(skipped_n_vals, na.rm = TRUE) else NA_integer_
    out$skipped_n_max <- if (is.atomic(skipped_n_vals) && length(skipped_n_vals)) max(skipped_n_vals, na.rm = TRUE) else NA_integer_
    out$skipped_prop_min <- if (is.atomic(skipped_prop_vals) && length(skipped_prop_vals)) min(skipped_prop_vals, na.rm = TRUE) else NA_real_
    out$skipped_prop_max <- if (is.atomic(skipped_prop_vals) && length(skipped_prop_vals)) max(skipped_prop_vals, na.rm = TRUE) else NA_real_
      out$optimizer_tol <- if (is.null(diag_attr$optimizer_tol)) NA_real_ else diag_attr$optimizer_tol
    } else {
    out$zero_cell_pairs <- NA_integer_
    out$boundary_pairs <- NA_integer_
    out$near_boundary_pairs <- NA_integer_
    out$corrected_pairs <- NA_integer_
    out$converged_pairs <- NA_integer_
    out$n_complete_min <- NA_integer_
    out$n_complete_max <- NA_integer_
    out$skipped_n_min <- NA_integer_
    out$skipped_n_max <- NA_integer_
    out$skipped_prop_min <- NA_real_
    out$skipped_prop_max <- NA_real_
      out$optimizer_tol <- NA_real_
    }

  ci_attr <- attr(object, "ci", exact = TRUE)
  out$ci_conf_level <- NA_real_
  out$ci_method <- NA_character_
  out$ci_width_min <- NA_real_
  out$ci_width_max <- NA_real_
  out$ci_cross_zero <- NA_integer_
  if (is.list(ci_attr) &&
      is.matrix(ci_attr$lwr.ci) &&
      is.matrix(ci_attr$upr.ci) &&
      identical(dim(ci_attr$lwr.ci), dim(m)) &&
      identical(dim(ci_attr$upr.ci), dim(m))) {
    selector <- if (symmetric && nrow(m) > 1L) upper.tri(m, diag = FALSE) else matrix(TRUE, nrow(m), ncol(m))
    lwr_vals <- ci_attr$lwr.ci[selector]
    upr_vals <- ci_attr$upr.ci[selector]
    keep_ci <- is.finite(lwr_vals) & is.finite(upr_vals)
    out$ci_conf_level <- suppressWarnings(as.numeric(ci_attr$conf.level %||% attr(object, "conf.level", exact = TRUE)))
    out$ci_method <- as.character(ci_attr$ci.method %||% attr(object, "ci.method", exact = TRUE) %||% NA_character_)
    if (any(keep_ci)) {
      widths <- upr_vals[keep_ci] - lwr_vals[keep_ci]
      out$ci_width_min <- min(widths)
      out$ci_width_max <- max(widths)
      out$ci_cross_zero <- sum(lwr_vals[keep_ci] <= 0 & upr_vals[keep_ci] >= 0, na.rm = TRUE)
    }
  }
  
  out$header <- header
  out$top_results <- .mc_summary_top_pairs(
    object,
    digits = 4,
    topn = .mc_coalesce(topn, .mc_display_option("summary_topn", 5L))
  )
  out <- .mc_finalize_summary_list(
    out,
    class_name = if (identical(header, "Latent correlation summary")) {
      "summary.latent_corr"
    } else {
      "summary.matrixCorr"
    }
  )
  if (identical(class(out)[1L], "summary.matrixCorr")) {
    class(out) <- c("summary.matrixCorr", "summary.corr_matrix")
  } else {
    class(out) <- c("summary.latent_corr", "summary.matrixCorr", "summary.corr_matrix")
  }
  out
}

.mc_diag_discrete <- function(code) {
  keep <- !is.na(code)
  if (sum(keep) < 2L || length(unique(code[keep])) < 2L) {
    return(NA_real_)
  }
  1
}

.mc_pair_complete <- function(x, y, check_na) {
  if (check_na) {
    return(list(x = x, y = y))
  }
  keep <- stats::complete.cases(x, y)
  list(x = x[keep], y = y[keep])
}

.mc_fast_binary_table01 <- function(x, y) {
  matrix(
    tabulate(as.integer(x) + 2L * as.integer(y) + 1L, nbins = 4L),
    nrow = 2L, ncol = 2L
  )
}

.mc_fast_ordinal_table <- function(x, y, n_x, n_y) {
  matrix(
    tabulate(as.integer(x) + (as.integer(y) - 1L) * as.integer(n_x),
             nbins = as.integer(n_x) * as.integer(n_y)),
    nrow = as.integer(n_x),
    ncol = as.integer(n_y)
  )
}

.mc_binary_tau <- function(x01) {
  x01 <- x01[!is.na(x01)]
  -stats::qnorm(mean(x01))
}

.mc_global_cutpoints <- function(code, n_levels) {
  if (n_levels < 2L) {
    return(numeric())
  }
  code <- as.integer(code)
  code <- code[!is.na(code)]
  freq <- tabulate(code, nbins = as.integer(n_levels))
  probs <- freq / sum(freq)
  stats::qnorm(cumsum(probs)[-n_levels])
}

.mc_phi2_exact <- function(a, b, rho) {
  stats::integrate(
    function(z) stats::dnorm(z) * stats::pnorm((b - rho * z) / sqrt(1 - rho^2)),
    lower = -Inf,
    upper = a,
    rel.tol = 1e-12,
    abs.tol = 0
  )$value
}

.mc_binary_prob_exact <- function(rc, cc, rho) {
  p11 <- .mc_phi2_exact(rc, cc, rho)
  p21 <- stats::pnorm(rc) - p11
  p12 <- stats::pnorm(cc) - p11
  p22 <- 1 - stats::pnorm(rc) - p12
  matrix(c(p11, p21, p12, p22), 2L, 2L)
}

.mc_binary_prob_exact_poly <- function(rc, cc, rho) {
  p11 <- .mc_phi2_exact(rc, cc, rho)
  p12 <- stats::pnorm(rc) - p11
  p21 <- stats::pnorm(cc) - p11
  p22 <- 1 - stats::pnorm(rc) - p21
  matrix(c(p11, p21, p12, p22), 2L, 2L)
}

.mc_tetra_opt_exact <- function(tab, rc, cc) {
  fn <- function(rho) {
    P <- .mc_binary_prob_exact(rc, cc, rho)
    if (any(tab > 0 & (!is.finite(P) | P <= 0))) {
      return(Inf)
    }
    -sum(tab * log(P))
  }
  suppressWarnings(stats::optimize(fn, c(-1, 1))$minimum)
}

.mc_poly_opt_exact_binary <- function(tab, rc, cc) {
  fn <- function(rho) {
    P <- .mc_binary_prob_exact_poly(rc, cc, rho)
    P[P <= 0] <- NA_real_
    lP <- log(P)
    lP[!is.finite(lP)] <- NA_real_
    -sum(tab * lP, na.rm = TRUE)
  }
  suppressWarnings(stats::optimize(fn, c(-1, 1))$minimum)
}

.mc_tetra_table_thresholds <- function(tab, correct) {
  tab <- as.matrix(tab)
  if (correct > 0) {
    tab[tab == 0] <- correct
  }
  tab <- tab / sum(tab)
  list(
    row = if (nrow(tab) == 2L) stats::qnorm(rowSums(tab))[1L] else stats::qnorm(cumsum(rowSums(tab))[-nrow(tab)]),
    col = if (ncol(tab) == 2L) stats::qnorm(colSums(tab))[1L] else stats::qnorm(cumsum(colSums(tab))[-ncol(tab)])
  )
}

.mc_poly_table_thresholds <- function(tab, correct) {
  tab <- as.matrix(tab)
  tot <- sum(tab)
  tab <- tab / tot
  if (correct > 0) {
    tab[tab == 0] <- correct / tot
  }
  list(
    row = stats::qnorm(cumsum(rowSums(tab))[-nrow(tab)]),
    col = stats::qnorm(cumsum(colSums(tab))[-ncol(tab)])
  )
}

.mc_tetrachoric_table_estimate <- function(tab, correct) {
  tab <- as.matrix(tab)
  if (all(dim(tab) == c(2L, 2L))) {
    if (correct > 0) {
      tab[tab == 0] <- correct
    }
    tab <- tab / sum(tab)
    rc <- stats::qnorm(colSums(tab))[1L]
    cc <- stats::qnorm(rowSums(tab))[1L]
    return(.mc_tetra_opt_exact(tab, rc = rc, cc = cc))
  }
  matrixCorr_tetrachoric_mle_cpp(unclass(tab), correct = correct)
}

.mc_polychoric_table_estimate <- function(tab, correct) {
  tab <- as.matrix(tab)
  if (all(dim(tab) == c(2L, 2L))) {
    tot <- sum(tab)
    tab <- tab / tot
    if (correct > 0) {
      tab[tab == 0] <- correct / tot
    }
    cc <- stats::qnorm(cumsum(colSums(tab))[-ncol(tab)])
    rc <- stats::qnorm(cumsum(rowSums(tab))[-nrow(tab)])
    return(.mc_poly_opt_exact_binary(tab, rc = rc, cc = cc))
  }
  matrixCorr_polychoric_mle_cpp(unclass(tab), correct = correct)
}

.mc_tetrachoric_estimate_internal <- function(x, y, correct,
                                              tau_x = NULL, tau_y = NULL,
                                              global = FALSE) {
  x01 <- as.integer(x) - min(as.integer(x), na.rm = TRUE)
  y01 <- as.integer(y) - min(as.integer(y), na.rm = TRUE)
  tab <- .mc_fast_binary_table01(x01, y01)

  if ((sum(tab) > 1L) && (min(tab) == 0L) && (correct > 0)) {
    tab[tab == 0L] <- correct
  }

  if (any(tab == correct) || any(tab == 0L)) {
    if (global) {
      cc <- if (is.null(tau_x)) .mc_binary_tau(x01) else tau_x
      rc <- if (is.null(tau_y)) .mc_binary_tau(y01) else tau_y
      return(.mc_tetra_opt_exact(tab, rc = rc, cc = cc))
    }
    tab_local <- tab / sum(tab)
    rc <- stats::qnorm(colSums(tab_local))[1L]
    cc <- stats::qnorm(rowSums(tab_local))[1L]
    return(.mc_tetra_opt_exact(tab_local, rc = rc, cc = cc))
  }

  if (global) {
    cc <- if (is.null(tau_x)) .mc_binary_tau(x01) else tau_x
    rc <- if (is.null(tau_y)) .mc_binary_tau(y01) else tau_y
    return(matrixCorr_tetrachoric_fixed_cpp(unclass(tab), rc = rc, cc = cc, correct = correct))
  } else {
    return(matrixCorr_tetrachoric_mle_cpp(unclass(tab), correct = correct))
  }
}

.mc_polychoric_estimate_internal <- function(x, y, n_x, n_y, correct,
                                             tau_x = NULL, tau_y = NULL,
                                             global = FALSE) {
  tab <- .mc_fast_ordinal_table(x, y, n_x = n_x, n_y = n_y)
  tot <- sum(tab)
  if (tot <= 0) {
    return(NA_real_)
  }
  tab <- tab / tot

  if (n_x == 2L && n_y == 2L && any(tab == 0)) {
    if (global) {
      return(.mc_poly_opt_exact_binary(tab, rc = tau_x, cc = tau_y))
    }
    tab_local <- tab
    if (correct > 0 && any(tab_local == 0)) {
      tab_local[tab_local == 0] <- correct / tot
    }
    cc <- stats::qnorm(cumsum(colSums(tab_local))[-ncol(tab_local)])
    rc <- stats::qnorm(cumsum(rowSums(tab_local))[-nrow(tab_local)])
    return(.mc_poly_opt_exact_binary(tab_local, rc = rc, cc = cc))
  }

  if (global) {
    return(matrixCorr_polychoric_fixed_cpp(unclass(tab), rc_in = tau_x, cc_in = tau_y))
  } else {
    return(matrixCorr_polychoric_mle_cpp(unclass(tab), correct = correct))
  }
}

.mc_pair_tetrachoric <- function(x, y, correct, check_na,
                                 tau_x = NULL, tau_y = NULL,
                                 global = TRUE) {
  pair <- .mc_pair_complete(x, y, check_na)
  if (length(pair$x) < 2L) {
    return(NA_real_)
  }
  .mc_tetrachoric_from_pair(
    pair$x, pair$y,
    correct = correct,
    tau_x = tau_x,
    tau_y = tau_y,
    global = global
  )
}

.mc_tetrachoric_from_pair <- function(x_pair, y_pair, correct,
                                      tau_x = NULL, tau_y = NULL,
                                      global = TRUE) {
  ux <- unique(x_pair)
  uy <- unique(y_pair)
  ux <- ux[!is.na(ux)]
  uy <- uy[!is.na(uy)]
  if (length(ux) < 2L || length(uy) < 2L) {
    return(NA_real_)
  }
  .mc_tetrachoric_estimate_internal(
    x_pair, y_pair, correct = correct,
    tau_x = tau_x, tau_y = tau_y, global = global
  )
}

.mc_pair_polychoric <- function(x, y, n_x, n_y, correct, check_na,
                                tau_x = NULL, tau_y = NULL,
                                global = identical(n_x, n_y)) {
  pair <- .mc_pair_complete(x, y, check_na)
  if (length(pair$x) < 2L) {
    return(NA_real_)
  }
  .mc_polychoric_from_pair(
    pair$x, pair$y,
    n_x = n_x,
    n_y = n_y,
    correct = correct,
    tau_x = tau_x,
    tau_y = tau_y,
    global = global
  )
}

.mc_polychoric_from_pair <- function(x_pair, y_pair, n_x, n_y, correct,
                                     tau_x = NULL, tau_y = NULL,
                                     global = identical(n_x, n_y)) {
  ux <- unique(x_pair)
  uy <- unique(y_pair)
  ux <- ux[!is.na(ux)]
  uy <- uy[!is.na(uy)]
  if (length(ux) < 2L || length(uy) < 2L) {
    return(NA_real_)
  }
  .mc_polychoric_estimate_internal(
    x_pair, y_pair,
    n_x = n_x, n_y = n_y,
    correct = correct,
    tau_x = tau_x, tau_y = tau_y,
    global = global
  )
}

.mc_pair_polyserial <- function(x, y, check_na) {
  pair <- .mc_pair_complete(x, y, check_na)
  if (length(pair$x) < 2L) {
    return(NA_real_)
  }
  if (length(unique(pair$y[!is.na(pair$y)])) < 2L) {
    return(NA_real_)
  }
  matrixCorr_polyserial_mle_cpp(
    x = as.numeric(pair$x),
    y = as.integer(pair$y)
  )
}

.mc_pair_biserial <- function(x, y, check_na) {
  pair <- .mc_pair_complete(x, y, check_na)
  if (length(pair$x) < 2L) {
    return(NA_real_)
  }
  yy <- pair$y[!is.na(pair$y)]
  if (length(unique(yy)) < 2L) {
    return(NA_real_)
  }
  matrixCorr_biserial_latent_cpp(as.numeric(pair$x), as.logical(pair$y == 2L))
}

.mc_biserial_inference_one <- function(x, y, check_na,
                                       return_ci = FALSE,
                                       conf_level = 0.95) {
  pair <- .mc_pair_complete(x, y, check_na)
  n_complete <- length(pair$x)
  out <- list(
    estimate = NA_real_,
    statistic = NA_real_,
    parameter = NA_real_,
    p_value = NA_real_,
    n_obs = as.integer(n_complete),
    lwr = NA_real_,
    upr = NA_real_
  )

  if (n_complete < 2L) {
    return(out)
  }

  yy <- pair$y[!is.na(pair$y)]
  if (length(unique(yy)) < 2L) {
    return(out)
  }

  est <- matrixCorr_biserial_latent_cpp(
    as.numeric(pair$x),
    as.logical(pair$y == 2L)
  )
  out$estimate <- est

  df <- n_complete - 2L
  out$parameter <- if (df > 0L) as.numeric(df) else NA_real_
  if (is.finite(est) && df > 0L) {
    if (abs(est) >= 1) {
      statistic <- sign(est) * Inf
    } else {
      statistic <- est * sqrt(df / (1 - est^2))
    }
    out$statistic <- statistic
    out$p_value <- 2 * stats::pt(abs(statistic), df = df, lower.tail = FALSE)
  }

  if (isTRUE(return_ci) && is.finite(est) && n_complete > 3L) {
    if (abs(est) >= 1) {
      out$lwr <- est
      out$upr <- est
    } else {
      z_est <- atanh(est)
      z_crit <- stats::qnorm(0.5 * (1 + conf_level))
      z_half_width <- z_crit / sqrt(n_complete - 3)
      out$lwr <- tanh(z_est - z_half_width)
      out$upr <- tanh(z_est + z_half_width)
    }
  }

  out
}

.mc_biserial_ci_attr <- function(x) {
  attr(x, "ci", exact = TRUE)
}

.mc_biserial_inference_attr <- function(x) {
  attr(x, "inference", exact = TRUE)
}

.mc_encode_list_to_matrix <- function(enc) {
  out <- matrix(NA_integer_, nrow = length(enc[[1L]]$code), ncol = length(enc))
  for (j in seq_along(enc)) {
    out[, j] <- as.integer(enc[[j]]$code)
  }
  colnames(out) <- names(enc)
  out
}

.mc_tau_matrix <- function(enc) {
  max_cuts <- max(vapply(enc, function(z) max(length(z$levels) - 1L, 0L), integer(1)))
  out <- matrix(NA_real_, nrow = max_cuts, ncol = length(enc))
  for (j in seq_along(enc)) {
    tau_j <- .mc_global_cutpoints(enc[[j]]$code, length(enc[[j]]$levels))
    if (length(tau_j) > 0L) {
      out[seq_along(tau_j), j] <- tau_j
    }
  }
  out
}

.mc_latent_pair_metadata <- function(est, tab, n_complete, correct, thresholds,
                                     optimizer_tol = 1e-8) {
  zero_cells <- sum(tab == 0)
  boundary <- isTRUE(correct == 0) && zero_cells > 0L
  near_boundary <- isTRUE(boundary) || (is.finite(est) && abs(est) >= 0.99)
  list(
    zero_cells = as.integer(zero_cells),
    correct = correct,
    corrected = isTRUE(correct > 0) && zero_cells > 0L,
    boundary = boundary,
    near_boundary = near_boundary,
    n_complete = as.integer(n_complete),
    converged = is.finite(est),
    optimizer_tol = optimizer_tol,
    thresholds = thresholds
  )
}

.mc_matrix_diagnostics_template <- function(p, dimnames) {
  mat_int <- matrix(NA_integer_, nrow = p, ncol = p, dimnames = dimnames)
  mat_lgl <- matrix(NA, nrow = p, ncol = p, dimnames = dimnames)
  list(
    zero_cells = mat_int,
    n_complete = mat_int,
    corrected = mat_lgl,
    boundary = mat_lgl,
    near_boundary = mat_lgl,
    converged = mat_lgl,
    optimizer_tol = 1e-8
  )
}

.mc_fill_pair_diag <- function(diag, j, k, est, tab, n_complete, correct) {
  zero_cells <- as.integer(sum(tab == 0))
  boundary <- isTRUE(correct == 0) && zero_cells > 0L
  near_boundary <- isTRUE(boundary) || (is.finite(est) && abs(est) >= 0.99)
  corrected <- isTRUE(correct > 0) && zero_cells > 0L
  converged <- is.finite(est)

  diag$zero_cells[j, k] <- zero_cells
  diag$zero_cells[k, j] <- zero_cells
  diag$n_complete[j, k] <- as.integer(n_complete)
  diag$n_complete[k, j] <- as.integer(n_complete)
  diag$corrected[j, k] <- corrected
  diag$corrected[k, j] <- corrected
  diag$boundary[j, k] <- boundary
  diag$boundary[k, j] <- boundary
  diag$near_boundary[j, k] <- near_boundary
  diag$near_boundary[k, j] <- near_boundary
  diag$converged[j, k] <- converged
  diag$converged[k, j] <- converged
  diag
}

#' @title Pairwise Tetrachoric Correlation
#'
#' @description
#' Computes the tetrachoric correlation for either a pair of binary variables
#' or all pairwise combinations of binary columns in a matrix/data frame.
#'
#' @param data A binary vector, matrix, or data frame. In matrix/data-frame mode,
#' only binary columns are retained.
#' @param y Optional second binary vector. When supplied, the function returns a
#' single tetrachoric correlation estimate.
#' @param correct Non-negative continuity correction added to zero-count cells.
#' Default is \code{0.5}.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
#'   complete cases.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald confidence intervals derived from the observed
#' information matrix of the latent-variable likelihood.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald p-values and test statistics for each
#' estimated latent correlation.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return
#' If \code{y} is supplied, a numeric scalar with attributes
#' \code{diagnostics} and \code{thresholds}. Otherwise a symmetric matrix of
#' class \code{tetrachoric_corr} with attributes \code{method},
#' \code{description}, \code{package = "matrixCorr"}, \code{diagnostics},
#' \code{thresholds}, and \code{correct}. When \code{p_value = TRUE}, the
#' returned object also carries an \code{inference} attribute with elements
#' \code{estimate}, \code{statistic}, \code{parameter}, \code{p_value}, and
#' \code{n_obs}. When \code{ci = TRUE}, it also carries a \code{ci} attribute
#' with elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
#' and \code{ci.method}, plus \code{attr(x, "conf.level")}. Scalar outputs keep
#' the same point estimate and gain the same metadata only when inference is
#' requested. In matrix mode, \code{output = "edge_list"} returns a data frame with columns
#' \code{row}, \code{col}, \code{value}; \code{output = "sparse"} returns a
#' symmetric sparse matrix.
#'
#' @details
#' The tetrachoric correlation assumes that the observed binary variables arise
#' by dichotomising latent standard-normal variables. Let
#' \eqn{Z_1, Z_2 \sim N(0, 1)} with latent correlation \eqn{\rho}, and define
#' observed binary variables by thresholds \eqn{\tau_1, \tau_2}:
#' \deqn{
#' X = \mathbf{1}\{Z_1 > \tau_1\},
#' \qquad
#' Y = \mathbf{1}\{Z_2 > \tau_2\}.
#' }
#' If the observed \eqn{2 \times 2} table has counts
#' \eqn{n_{ij}} for \eqn{i,j \in \{0,1\}}, the marginal proportions determine
#' the thresholds:
#' \deqn{
#' \tau_1 = \Phi^{-1}\!\big(P(X = 0)\big),
#' \qquad
#' \tau_2 = \Phi^{-1}\!\big(P(Y = 0)\big).
#' }
#' The estimator returned here is the maximum-likelihood estimate of the latent
#' correlation \eqn{\rho}, obtained by maximizing the multinomial log-likelihood
#' built from the rectangle probabilities of the bivariate normal distribution:
#' \deqn{
#' \ell(\rho) = \sum_{i=0}^1 \sum_{j=0}^1 n_{ij}\log \pi_{ij}(\rho;\tau_1,\tau_2),
#' }
#' where \eqn{\pi_{ij}} are the four bivariate-normal cell probabilities implied
#' by \eqn{\rho} and the fixed thresholds. The implementation evaluates the
#' likelihood over \eqn{\rho \in (-1,1)} by a coarse search followed by Brent
#' refinement in C++.
#'
#' The argument \code{correct} adds a continuity correction only to zero-count
#' cells before threshold estimation and likelihood evaluation. This stabilises
#' the estimator for sparse tables and mirrors the conventional
#' \code{correct = 0.5} continuity-correction behaviour used in several
#' latent-correlation implementations.
#' When \code{correct = 0} and the observed contingency table contains zero
#' cells, the fit is non-regular and may be boundary-driven. In those cases the
#' returned object stores sparse-fit diagnostics, including whether the fit was
#' classified as \code{boundary} or \code{near_boundary}.
#'
#' \strong{Assumptions.} The coefficient is appropriate when both observed
#' binary variables are viewed as thresholded versions of jointly normal latent
#' variables. The optional p-values and confidence intervals adopt this
#' latent-normal interpretation and use the same likelihood that defines the
#' tetrachoric estimate. These inferential quantities are therefore model-based
#' and should not be interpreted as distribution-free summaries.
#'
#' \strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
#' function refits the pairwise tetrachoric model by maximum likelihood and
#' obtains the observed information matrix numerically in C++. The reported
#' confidence interval is a Wald interval
#' \eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
#' reported p-value is from the large-sample Wald \eqn{z}-test for
#' \eqn{H_0:\rho = 0}. These inferential quantities are only computed when
#' explicitly requested.
#'
#' In matrix/data-frame mode, all pairwise tetrachoric correlations are computed
#' between binary columns. Diagonal entries are \code{1} for non-degenerate
#' columns and \code{NA} for columns with fewer than two observed levels.
#' Variable-specific latent thresholds are stored in the \code{thresholds}
#' attribute, and pairwise sparse-fit diagnostics are stored in
#' \code{diagnostics}.
#'
#' \strong{Computational complexity.} For \eqn{p} binary variables, the matrix
#' path evaluates \eqn{p(p-1)/2} pairwise likelihoods. Each pair uses a
#' one-dimensional optimisation with negligible memory overhead beyond the
#' output matrix.
#'
#' @references
#' Pearson, K. (1900). Mathematical contributions to the theory of evolution.
#' VII. On the correlation of characters not quantitatively measurable.
#' \emph{Philosophical Transactions of the Royal Society A}, 195, 1-47.
#'
#' Olsson, U. (1979). Maximum likelihood estimation of the polychoric
#' correlation coefficient. \emph{Psychometrika}, 44(4), 443-460.
#'
#' @examplesIf requireNamespace("mnormt", quietly = TRUE)
#' set.seed(123)
#' n <- 1000
#' Sigma <- matrix(c(
#'   1.00, 0.55, 0.35,
#'   0.55, 1.00, 0.45,
#'   0.35, 0.45, 1.00
#' ), 3, 3, byrow = TRUE)
#'
#' Z <- mnormt::rmnorm(n = n, mean = rep(0, 3), varcov = Sigma)
#' X <- data.frame(
#'   item1 = Z[, 1] > stats::qnorm(0.70),
#'   item2 = Z[, 2] > stats::qnorm(0.60),
#'   item3 = Z[, 3] > stats::qnorm(0.50)
#' )
#'
#' tc <- tetrachoric(X)
#' print(tc, digits = 3)
#' summary(tc)
#' estimate(tc)
#' tidy(tc)
#' tc_ci <- tetrachoric(X, ci = TRUE)
#' ci(tc_ci)
#' confint(tc_ci)
#' plot(tc)
#' tetrachoric(X, output = "edge_list", diag = FALSE)
#' tetrachoric(X, output = "sparse", threshold = 0.4, diag = FALSE)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(tc)
#' }
#'
#' # latent Pearson correlations used to generate the binary items
#' round(stats::cor(Z), 2)
#'
#' @author Thiago de Paula Oliveira
#' @export
tetrachoric <- function(data,
                        y = NULL,
                        na_method = c("error", "pairwise"),
                        ci = FALSE,
                        p_value = FALSE,
                        conf_level = 0.95,
                        correct = 0.5,
                        output = c("matrix", "sparse", "edge_list"),
                        threshold = 0,
                        diag = TRUE,
                        ...) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )
  if (missing(na_method) && isFALSE(ci) && isFALSE(p_value) && ...length() == 0L) {
    check_scalar_nonneg(correct, arg = "correct")

    if (is.null(y) && inherits(data, "table")) {
      if (!identical(output_cfg$output, "matrix")) {
        abort_bad_arg(
          "output",
          message = "must be {.val matrix} when {.arg data} is a contingency table."
        )
      }
      tab <- as.matrix(data)
      est <- .mc_tetrachoric_table_estimate(tab, correct = correct)
      thresholds <- .mc_tetra_table_thresholds(tab, correct = correct)
      diagnostics <- .mc_latent_pair_metadata(
        est = est,
        tab = tab,
        n_complete = sum(tab),
        correct = correct,
        thresholds = thresholds
      )
      return(.mc_attach_scalar_latent(
        est,
        method = "tetrachoric",
        description = "Pairwise tetrachoric correlation",
        diagnostics = diagnostics,
        thresholds = thresholds
      ))
    }

    if (!is.null(y)) {
      if (!identical(output_cfg$output, "matrix")) {
        abort_bad_arg(
          "output",
          message = "must be {.val matrix} when {.arg y} is supplied."
        )
      }
      check_same_length(data, y, arg_x = "data", arg_y = "y")
      .mc_check_latent_missing(list(data = data, y = y), check_na = TRUE, arg = "data")

      enc_x <- .mc_encode_ordinal_vector(data, binary = TRUE)
      enc_y <- .mc_encode_ordinal_vector(y, binary = TRUE)
      if (is.null(enc_x) || is.null(enc_y)) {
        abort_bad_arg(
          "data",
          message = "and {.arg y} must both be binary (two-level) variables."
        )
      }

      pair <- .mc_pair_complete(enc_x$code, enc_y$code, TRUE)
      est <- .mc_tetrachoric_from_pair(
        pair$x, pair$y,
        correct = correct,
        global = TRUE
      )
      x01 <- as.integer(pair$x) - min(as.integer(pair$x), na.rm = TRUE)
      y01 <- as.integer(pair$y) - min(as.integer(pair$y), na.rm = TRUE)
      tab <- .mc_fast_binary_table01(x01, y01)
      thresholds <- list(data = .mc_binary_tau(x01), y = .mc_binary_tau(y01))
      diagnostics <- .mc_latent_pair_metadata(
        est = est,
        tab = tab,
        n_complete = length(pair$x),
        correct = correct,
        thresholds = thresholds
      )
      return(.mc_attach_scalar_latent(
        est,
        method = "tetrachoric",
        description = "Pairwise tetrachoric correlation",
        diagnostics = diagnostics,
        thresholds = thresholds
      ))
    }

    enc <- .mc_extract_discrete_columns(data, kind = "binary", arg = "data", min_cols = 2L)
    .mc_check_latent_missing(lapply(enc, `[[`, "code"), check_na = TRUE, arg = "data")

    x_mat <- .mc_encode_list_to_matrix(enc)
    tau <- vapply(enc, function(z) {
      code <- as.integer(z$code)
      .mc_binary_tau(code - min(code, na.rm = TRUE))
    }, numeric(1))
    out <- matrixCorr_tetrachoric_matrix_cpp(
      x = x_mat,
      tau = tau,
      correct = correct,
      pairwise_complete = FALSE
    )
    dimnames(out) <- list(names(enc), names(enc))

    p <- length(enc)
    diag_info <- .mc_matrix_diagnostics_template(p, dimnames(out))
    diag(diag_info$zero_cells) <- 0L
    diag(diag_info$n_complete) <- colSums(!is.na(x_mat))
    diag(diag_info$corrected) <- FALSE
    diag(diag_info$boundary) <- FALSE
    diag(diag_info$near_boundary) <- FALSE
    diag(diag_info$converged) <- is.finite(diag(out))
    for (j in seq_len(p - 1L)) {
      for (k in (j + 1L):p) {
        pair <- .mc_pair_complete(enc[[j]]$code, enc[[k]]$code, TRUE)
        x01 <- as.integer(pair$x) - min(as.integer(pair$x), na.rm = TRUE)
        y01 <- as.integer(pair$y) - min(as.integer(pair$y), na.rm = TRUE)
        tab <- .mc_fast_binary_table01(x01, y01)
        diag_info <- .mc_fill_pair_diag(diag_info, j, k, out[j, k], tab, length(pair$x), correct)
      }
    }
    thresholds <- as.list(tau)
    names(thresholds) <- names(enc)
    out <- .mc_structure_corr_matrix(
      out,
      class_name = "tetrachoric_corr",
      method = "tetrachoric",
      description = "Pairwise tetrachoric correlation matrix",
      symmetric = TRUE,
      diagnostics = diag_info,
      thresholds = thresholds,
      correct = correct
    )
    return(.mc_finalize_corr_output_fast(
      out,
      output = output_cfg$output,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag
    ))
  }

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method)
  )
  check_na <- na_cfg$check_na
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_scalar_nonneg(correct, arg = "correct")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  if (is.null(y) && inherits(data, "table")) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg data} is a contingency table."
      )
    }
    tab <- as.matrix(data)
    fit <- if (isTRUE(ci) || isTRUE(p_value)) {
      .mc_tetrachoric_inference_one(tab, correct = correct, conf_level = conf_level)
    } else {
      NULL
    }
    est <- if (is.null(fit)) .mc_tetrachoric_table_estimate(tab, correct = correct) else as.numeric(fit$estimate)
    thresholds <- .mc_tetra_table_thresholds(tab, correct = correct)
    diagnostics <- .mc_latent_pair_metadata(
      est = est,
      tab = tab,
      n_complete = sum(tab),
      correct = correct,
      thresholds = thresholds
    )
    out <- .mc_attach_scalar_latent(
      est,
      method = "tetrachoric",
      description = "Pairwise tetrachoric correlation",
      diagnostics = diagnostics,
      thresholds = thresholds
    )
    if (isTRUE(ci) || isTRUE(p_value)) {
      out <- .mc_attach_latent_inference(
        out,
        .mc_latent_scalar_payload(fit, return_ci = ci, return_p = p_value),
        conf_level = conf_level
      )
    }
    return(out)
  }

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg y} is supplied."
      )
    }
    check_same_length(data, y, arg_x = "data", arg_y = "y")
    .mc_check_latent_missing(list(data = data, y = y), check_na = check_na, arg = "data")

    enc_x <- .mc_encode_ordinal_vector(data, binary = TRUE)
    enc_y <- .mc_encode_ordinal_vector(y, binary = TRUE)
    if (is.null(enc_x) || is.null(enc_y)) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must both be binary (two-level) variables."
      )
    }
    pair <- .mc_pair_complete(enc_x$code, enc_y$code, check_na)
    x01 <- as.integer(pair$x) - min(as.integer(pair$x), na.rm = TRUE)
    y01 <- as.integer(pair$y) - min(as.integer(pair$y), na.rm = TRUE)
    tab <- .mc_fast_binary_table01(x01, y01)
    fit <- if (isTRUE(ci) || isTRUE(p_value)) {
      .mc_tetrachoric_inference_one(tab, correct = correct, conf_level = conf_level)
    } else {
      NULL
    }
    est <- .mc_tetrachoric_from_pair(
      pair$x, pair$y,
      correct = correct,
      global = TRUE
    )
    if (!is.null(fit)) {
      fit <- .mc_recenter_wald_fit(fit, estimate = est, conf_level = conf_level)
    }
    thresholds <- list(
      data = .mc_binary_tau(x01),
      y = .mc_binary_tau(y01)
    )
    diagnostics <- .mc_latent_pair_metadata(
      est = est,
      tab = tab,
      n_complete = length(pair$x),
      correct = correct,
      thresholds = thresholds
    )
    out <- .mc_attach_scalar_latent(
      est,
      method = "tetrachoric",
      description = "Pairwise tetrachoric correlation",
      diagnostics = diagnostics,
      thresholds = thresholds
    )
    if (isTRUE(ci) || isTRUE(p_value)) {
      out <- .mc_attach_latent_inference(
        out,
        .mc_latent_scalar_payload(fit, return_ci = ci, return_p = p_value),
        conf_level = conf_level
      )
    }
    return(out)
  }

  enc <- .mc_extract_discrete_columns(data, kind = "binary", arg = "data", min_cols = 2L)
  .mc_check_latent_missing(
    lapply(enc, `[[`, "code"),
    check_na = check_na,
    arg = "data"
  )

  p <- length(enc)
  x_mat <- .mc_encode_list_to_matrix(enc)
  tau <- vapply(enc, function(z) {
    code <- as.integer(z$code)
    .mc_binary_tau(code - min(code, na.rm = TRUE))
  }, numeric(1))
  out <- if (!isTRUE(ci) && !isTRUE(p_value)) {
    matrixCorr_tetrachoric_matrix_cpp(
      x = x_mat,
      tau = tau,
      correct = correct,
      pairwise_complete = !check_na
    )
  } else {
    matrix(NA_real_, nrow = p, ncol = p)
  }
  dimnames(out) <- list(names(enc), names(enc))
  diag_info <- .mc_matrix_diagnostics_template(p, dimnames(out))
  diag(diag_info$zero_cells) <- 0L
  diag(diag_info$n_complete) <- colSums(!is.na(x_mat))
  diag(diag_info$corrected) <- FALSE
  diag(diag_info$boundary) <- FALSE
  diag(diag_info$near_boundary) <- FALSE
  diag(diag_info$converged) <- TRUE
  diag(out) <- ifelse(vapply(enc, function(z) length(unique(z$code[!is.na(z$code)])) >= 2L, logical(1)), 1, NA_real_)
  statistic <- parameter <- p_mat <- lwr <- upr <- NULL
  if (isTRUE(ci) || isTRUE(p_value)) {
    statistic <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    parameter <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    p_mat <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    lwr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
    upr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
  }
  for (j in seq_len(p - 1L)) {
    for (k in (j + 1L):p) {
      pair <- .mc_pair_complete(enc[[j]]$code, enc[[k]]$code, check_na)
      x01 <- as.integer(pair$x) - min(as.integer(pair$x), na.rm = TRUE)
      y01 <- as.integer(pair$y) - min(as.integer(pair$y), na.rm = TRUE)
      tab <- .mc_fast_binary_table01(x01, y01)
      if (isTRUE(ci) || isTRUE(p_value)) {
        est_jk <- .mc_tetrachoric_from_pair(
          pair$x, pair$y,
          correct = correct,
          tau_x = tau[j],
          tau_y = tau[k],
          global = TRUE
        )
        fit <- .mc_tetrachoric_inference_one(tab, correct = correct, conf_level = conf_level)
        fit <- .mc_recenter_wald_fit(fit, estimate = est_jk, conf_level = conf_level)
        out[j, k] <- est_jk
        out[k, j] <- out[j, k]
        if (isTRUE(p_value)) {
          statistic[j, k] <- as.numeric(fit$statistic)
          statistic[k, j] <- statistic[j, k]
          parameter[j, k] <- as.numeric(fit$parameter)
          parameter[k, j] <- parameter[j, k]
          p_mat[j, k] <- as.numeric(fit$p_value)
          p_mat[k, j] <- p_mat[j, k]
        }
        if (isTRUE(ci)) {
          lwr[j, k] <- as.numeric(fit$lwr)
          lwr[k, j] <- lwr[j, k]
          upr[j, k] <- as.numeric(fit$upr)
          upr[k, j] <- upr[j, k]
        }
      }
      diag_info <- .mc_fill_pair_diag(diag_info, j, k, out[j, k], tab, length(pair$x), correct)
    }
  }
  thresholds <- as.list(tau)
  names(thresholds) <- names(enc)

  out <- .mc_structure_corr_matrix(
    out,
    class_name = "tetrachoric_corr",
    method = "tetrachoric",
    description = "Pairwise tetrachoric correlation matrix",
    symmetric = TRUE,
    diagnostics = diag_info,
    thresholds = thresholds,
    correct = correct
  )
  if (isTRUE(ci) || isTRUE(p_value)) {
    out <- .mc_attach_latent_inference(
      out,
      .mc_latent_matrix_payload(
        estimate = unclass(out),
        statistic = statistic,
        parameter = parameter,
        p_value = p_mat,
        n_obs = diag_info$n_complete,
        lwr = lwr,
        upr = upr,
        conf_level = conf_level,
        ci_method = "wald_information_tetrachoric",
        inference_method = "wald_z_tetrachoric",
        return_ci = ci,
        return_p = p_value
      ),
      conf_level = conf_level
    )
  }
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

#' @rdname tetrachoric
#' @method print tetrachoric_corr
#' @param x An object of class \code{tetrachoric_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print()}.
#' @export
print.tetrachoric_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                   max_vars = NULL, width = NULL,
                                   show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x, header = "Tetrachoric correlation matrix",
    digits = digits, n = n, topn = topn,
    max_vars = max_vars, width = width, show_ci = show_ci, ...
  )
}

#' @rdname tetrachoric
#' @method plot tetrachoric_corr
#' @param title Plot title. Default is \code{"Tetrachoric correlation heatmap"}.
#' @param low_color Color for the minimum correlation.
#' @param high_color Color for the maximum correlation.
#' @param mid_color Color for zero correlation.
#' @param value_text_size Font size used in tile labels.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.tetrachoric_corr <- function(x, title = "Tetrachoric correlation heatmap",
                                  low_color = "indianred1",
                                  high_color = "steelblue1",
                                  mid_color = "white",
                                  value_text_size = 4,
                                  show_value = TRUE, ...) {
  .mc_plot_corr_matrix(
    x, class_name = "tetrachoric_corr", fill_name = "Tetrachoric",
    title = title, low_color = low_color, high_color = high_color,
    mid_color = mid_color, value_text_size = value_text_size,
    show_value = show_value, ...
  )
}

#' @rdname tetrachoric
#' @method summary tetrachoric_corr
#' @param object An object of class \code{tetrachoric_corr}.
#' @param ci_digits Integer; digits for confidence limits in the pairwise
#'   summary.
#' @param p_digits Integer; digits for p-values in the pairwise summary.
#' @export
summary.tetrachoric_corr <- function(object, n = NULL, topn = NULL,
                                     max_vars = NULL, width = NULL,
                                     ci_digits = 3,
                                     p_digits = 4,
                                     show_ci = NULL, ...) {
  check_inherits(object, "tetrachoric_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  ci <- .mc_latent_ci_attr(object)
  inf <- .mc_latent_inference_attr(object)
  if (is.null(ci) && (is.null(inf) || is.null(inf$p_value))) {
    return(.mc_summary_corr_matrix(object, header = "Latent correlation summary", topn = topn))
  }

  .mc_latent_pairwise_summary(
    object,
    summary_class = "summary.tetrachoric_corr",
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname tetrachoric
#' @method print summary.tetrachoric_corr
#' @param x An object of class \code{summary.tetrachoric_corr}.
#' @export
print.summary.tetrachoric_corr <- function(x, digits = NULL, n = NULL,
                                           topn = NULL, max_vars = NULL,
                                           width = NULL, show_ci = NULL, ...) {
  .mc_print_latent_pairwise_summary(
    x,
    title = "Tetrachoric correlation summary",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @title Pairwise Polychoric Correlation
#'
#' @description
#' Computes the polychoric correlation for either a pair of ordinal variables
#' or all pairwise combinations of ordinal columns in a matrix/data frame.
#'
#' @param data An ordinal vector, matrix, or data frame. Supported columns are
#' factors, ordered factors, logical values, or integer-like numerics.
#' In matrix/data-frame mode, only supported ordinal columns are retained.
#' @param y Optional second ordinal vector. When supplied, the function returns
#' a single polychoric correlation estimate.
#' @param correct Non-negative continuity correction added to zero-count cells.
#' Default is \code{0.5}.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
#'   complete cases.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald confidence intervals derived from the observed
#' information matrix of the latent-variable likelihood.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald p-values and test statistics for each
#' estimated latent correlation.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return
#' If \code{y} is supplied, a numeric scalar with attributes
#' \code{diagnostics} and \code{thresholds}. Otherwise a symmetric matrix of
#' class \code{polychoric_corr} with attributes \code{method},
#' \code{description}, \code{package = "matrixCorr"}, \code{diagnostics},
#' \code{thresholds}, and \code{correct}. When \code{p_value = TRUE}, the
#' returned object also carries an \code{inference} attribute with elements
#' \code{estimate}, \code{statistic}, \code{parameter}, \code{p_value}, and
#' \code{n_obs}. When \code{ci = TRUE}, it also carries a \code{ci} attribute
#' with elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
#' and \code{ci.method}, plus \code{attr(x, "conf.level")}. Scalar outputs keep
#' the same point estimate and gain the same metadata only when inference is
#' requested. In matrix mode, \code{output = "edge_list"} returns a data frame with columns
#' \code{row}, \code{col}, \code{value}; \code{output = "sparse"} returns a
#' symmetric sparse matrix.
#'
#' @details
#' The polychoric correlation generalises the tetrachoric model to ordered
#' categorical variables with more than two levels. It assumes latent
#' standard-normal variables \eqn{Z_1, Z_2} with correlation \eqn{\rho}, and
#' cut-points
#' \eqn{-\infty = \alpha_0 < \alpha_1 < \cdots < \alpha_R = \infty} and
#' \eqn{-\infty = \beta_0 < \beta_1 < \cdots < \beta_C = \infty} such that
#' \deqn{
#' X = r \iff \alpha_{r-1} < Z_1 \le \alpha_r,
#' \qquad
#' Y = c \iff \beta_{c-1} < Z_2 \le \beta_c.
#' }
#' For an observed \eqn{R \times C} contingency table with counts \eqn{n_{rc}},
#' the thresholds are estimated from the marginal cumulative proportions:
#' \deqn{
#' \alpha_r = \Phi^{-1}\!\Big(\sum_{k \le r} P(X = k)\Big),
#' \qquad
#' \beta_c = \Phi^{-1}\!\Big(\sum_{k \le c} P(Y = k)\Big).
#' }
#' Holding those thresholds fixed, the log-likelihood for the latent
#' correlation is
#' \deqn{
#' \ell(\rho) = \sum_{r=1}^{R}\sum_{c=1}^{C}
#' n_{rc} \log \Pr\!\big(
#' \alpha_{r-1} < Z_1 \le \alpha_r,\;
#' \beta_{c-1} < Z_2 \le \beta_c
#' \mid \rho \big),
#' }
#' and the estimator returned is the maximiser over \eqn{\rho \in (-1,1)}.
#' The C++ implementation performs a dense one-dimensional search followed by
#' Brent refinement.
#'
#' The argument \code{correct} adds a non-negative continuity correction to
#' empty cells before marginal threshold estimation and likelihood evaluation.
#' This avoids numerical failures for sparse tables with structurally zero cells.
#' When \code{correct = 0} and zero cells are present, the corresponding fit can
#' be boundary-driven rather than a regular interior maximum-likelihood problem.
#' The returned object stores sparse-fit diagnostics and the thresholds used for
#' estimation so those cases can be inspected explicitly.
#'
#' \strong{Assumptions.} The coefficient is appropriate when both observed
#' ordinal variables are viewed as discretisations of jointly normal latent
#' variables. The optional p-values and confidence intervals adopt this
#' latent-normal interpretation and use the same likelihood that defines the
#' polychoric estimate. These inferential quantities are therefore model-based
#' and should not be interpreted as distribution-free summaries.
#'
#' \strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
#' function refits the pairwise polychoric model by maximum likelihood and
#' obtains the observed information matrix numerically in C++. The reported
#' confidence interval is a Wald interval
#' \eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
#' reported p-value is from the large-sample Wald \eqn{z}-test for
#' \eqn{H_0:\rho = 0}. These inferential quantities are only computed when
#' explicitly requested.
#'
#' In matrix/data-frame mode, all pairwise polychoric correlations are computed
#' between supported ordinal columns. Diagonal entries are \code{1} for
#' non-degenerate columns and \code{NA} when a column has fewer than two
#' observed levels.
#'
#' \strong{Computational complexity.} For \eqn{p} ordinal variables, the matrix
#' path evaluates \eqn{p(p-1)/2} bivariate likelihoods. Each pair optimises a
#' single scalar parameter \eqn{\rho}, so the main cost is repeated evaluation
#' of bivariate normal rectangle probabilities.
#'
#' @references
#' Olsson, U. (1979). Maximum likelihood estimation of the polychoric
#' correlation coefficient. \emph{Psychometrika}, 44(4), 443-460.
#'
#' @examplesIf requireNamespace("mnormt", quietly = TRUE)
#' set.seed(124)
#' n <- 1200
#' Sigma <- matrix(c(
#'   1.00, 0.60, 0.40,
#'   0.60, 1.00, 0.50,
#'   0.40, 0.50, 1.00
#' ), 3, 3, byrow = TRUE)
#'
#' Z <- mnormt::rmnorm(n = n, mean = rep(0, 3), varcov = Sigma)
#' Y <- data.frame(
#'   y1 = ordered(cut(
#'     Z[, 1],
#'     breaks = c(-Inf, -0.7, 0.4, Inf),
#'     labels = c("low", "mid", "high")
#'   )),
#'   y2 = ordered(cut(
#'     Z[, 2],
#'     breaks = c(-Inf, -1.0, -0.1, 0.8, Inf),
#'     labels = c("1", "2", "3", "4")
#'   )),
#'   y3 = ordered(cut(
#'     Z[, 3],
#'     breaks = c(-Inf, -0.4, 0.2, 1.1, Inf),
#'     labels = c("A", "B", "C", "D")
#'   ))
#' )
#'
#' pc <- polychoric(Y)
#' print(pc, digits = 3)
#' summary(pc)
#' estimate(pc)
#' tidy(pc)
#' pc_ci <- polychoric(Y, ci = TRUE)
#' ci(pc_ci)
#' confint(pc_ci)
#' plot(pc)
#' polychoric(Y, output = "edge_list", threshold = 0.3, diag = FALSE)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(pc)
#' }
#'
#' # latent Pearson correlations used to generate the ordinal variables
#' round(stats::cor(Z), 2)
#' @author Thiago de Paula Oliveira
#' @export
polychoric <- function(data,
                       y = NULL,
                       na_method = c("error", "pairwise"),
                       ci = FALSE,
                       p_value = FALSE,
                       conf_level = 0.95,
                       correct = 0.5,
                       output = c("matrix", "sparse", "edge_list"),
                       threshold = 0,
                       diag = TRUE,
                       ...) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )
  if (missing(na_method) && isFALSE(ci) && isFALSE(p_value) && ...length() == 0L) {
    check_scalar_nonneg(correct, arg = "correct")

    if (is.null(y) && inherits(data, "table")) {
      if (!identical(output_cfg$output, "matrix")) {
        abort_bad_arg(
          "output",
          message = "must be {.val matrix} when {.arg data} is a contingency table."
        )
      }
      tab <- as.matrix(data)
      est <- .mc_polychoric_table_estimate(tab, correct = correct)
      thresholds <- .mc_poly_table_thresholds(tab, correct = correct)
      diagnostics <- .mc_latent_pair_metadata(
        est = est,
        tab = tab,
        n_complete = sum(tab),
        correct = correct,
        thresholds = thresholds
      )
      return(.mc_attach_scalar_latent(
        est,
        method = "polychoric",
        description = "Pairwise polychoric correlation",
        diagnostics = diagnostics,
        thresholds = thresholds
      ))
    }

    if (!is.null(y)) {
      if (!identical(output_cfg$output, "matrix")) {
        abort_bad_arg(
          "output",
          message = "must be {.val matrix} when {.arg y} is supplied."
        )
      }
      check_same_length(data, y, arg_x = "data", arg_y = "y")
      .mc_check_latent_missing(list(data = data, y = y), check_na = TRUE, arg = "data")

      enc_x <- .mc_encode_ordinal_vector(data, binary = FALSE)
      enc_y <- .mc_encode_ordinal_vector(y, binary = FALSE)
      if (is.null(enc_x) || is.null(enc_y)) {
        abort_bad_arg("data", message = "and {.arg y} must both be ordinal variables.")
      }
      n_x <- length(enc_x$levels)
      n_y <- length(enc_y$levels)
      global <- identical(n_x, n_y)
      pair <- .mc_pair_complete(enc_x$code, enc_y$code, TRUE)
      est <- .mc_polychoric_from_pair(
        pair$x, pair$y,
        n_x = n_x,
        n_y = n_y,
        correct = correct,
        tau_x = if (global) .mc_global_cutpoints(pair$x, n_x) else NULL,
        tau_y = if (global) .mc_global_cutpoints(pair$y, n_y) else NULL,
        global = global
      )
      tab <- .mc_fast_ordinal_table(pair$x, pair$y, n_x = n_x, n_y = n_y)
      thresholds <- list(
        data = .mc_global_cutpoints(pair$x, n_x),
        y = .mc_global_cutpoints(pair$y, n_y)
      )
      diagnostics <- .mc_latent_pair_metadata(
        est = est,
        tab = tab,
        n_complete = length(pair$x),
        correct = correct,
        thresholds = thresholds
      )
      return(.mc_attach_scalar_latent(
        est,
        method = "polychoric",
        description = "Pairwise polychoric correlation",
        diagnostics = diagnostics,
        thresholds = thresholds
      ))
    }

    enc <- .mc_extract_discrete_columns(data, kind = "ordinal", arg = "data", min_cols = 2L)
    .mc_check_latent_missing(lapply(enc, `[[`, "code"), check_na = TRUE, arg = "data")

    p <- length(enc)
    n_levels <- vapply(enc, function(z) length(z$levels), integer(1))
    global_all <- length(unique(n_levels)) == 1L
    tau_mat <- if (global_all) .mc_tau_matrix(enc) else matrix(NA_real_, nrow = 0L, ncol = 0L)
    x_mat <- .mc_encode_list_to_matrix(enc)
    out <- matrixCorr_polychoric_matrix_cpp(
      x = x_mat,
      n_levels = n_levels,
      tau_mat = tau_mat,
      global_all = global_all,
      correct = correct,
      pairwise_complete = FALSE
    )
    dimnames(out) <- list(names(enc), names(enc))

    diag_info <- .mc_matrix_diagnostics_template(p, dimnames(out))
    diag(diag_info$zero_cells) <- 0L
    diag(diag_info$n_complete) <- vapply(enc, function(z) sum(!is.na(z$code)), integer(1))
    diag(diag_info$corrected) <- FALSE
    diag(diag_info$boundary) <- FALSE
    diag(diag_info$near_boundary) <- FALSE
    diag(diag_info$converged) <- is.finite(diag(out))
    for (j in seq_len(p - 1L)) {
      for (k in (j + 1L):p) {
        pair <- .mc_pair_complete(enc[[j]]$code, enc[[k]]$code, TRUE)
        tab <- .mc_fast_ordinal_table(
          pair$x, pair$y,
          n_x = length(enc[[j]]$levels),
          n_y = length(enc[[k]]$levels)
        )
        diag_info <- .mc_fill_pair_diag(diag_info, j, k, out[j, k], tab, length(pair$x), correct)
      }
    }
    thresholds <- lapply(enc, function(z) .mc_global_cutpoints(z$code, length(z$levels)))
    names(thresholds) <- names(enc)
    out <- .mc_structure_corr_matrix(
      out,
      class_name = "polychoric_corr",
      method = "polychoric",
      description = "Pairwise polychoric correlation matrix",
      symmetric = TRUE,
      diagnostics = diag_info,
      thresholds = thresholds,
      correct = correct
    )
    return(.mc_finalize_corr_output_fast(
      out,
      output = output_cfg$output,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag
    ))
  }

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method)
  )
  check_na <- na_cfg$check_na
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_scalar_nonneg(correct, arg = "correct")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  if (is.null(y) && inherits(data, "table")) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg data} is a contingency table."
      )
    }
    tab <- as.matrix(data)
    fit <- if (isTRUE(ci) || isTRUE(p_value)) {
      .mc_polychoric_inference_one(tab, correct = correct, conf_level = conf_level)
    } else {
      NULL
    }
    est <- if (is.null(fit)) .mc_polychoric_table_estimate(tab, correct = correct) else as.numeric(fit$estimate)
    thresholds <- .mc_poly_table_thresholds(tab, correct = correct)
    diagnostics <- .mc_latent_pair_metadata(
      est = est,
      tab = tab,
      n_complete = sum(tab),
      correct = correct,
      thresholds = thresholds
    )
    out <- .mc_attach_scalar_latent(
      est,
      method = "polychoric",
      description = "Pairwise polychoric correlation",
      diagnostics = diagnostics,
      thresholds = thresholds
    )
    if (isTRUE(ci) || isTRUE(p_value)) {
      out <- .mc_attach_latent_inference(
        out,
        .mc_latent_scalar_payload(fit, return_ci = ci, return_p = p_value),
        conf_level = conf_level
      )
    }
    return(out)
  }

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg y} is supplied."
      )
    }
    check_same_length(data, y, arg_x = "data", arg_y = "y")
    .mc_check_latent_missing(list(data = data, y = y), check_na = check_na, arg = "data")

    enc_x <- .mc_encode_ordinal_vector(data, binary = FALSE)
    enc_y <- .mc_encode_ordinal_vector(y, binary = FALSE)
    if (is.null(enc_x) || is.null(enc_y)) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must both be ordinal variables."
      )
    }
    n_x <- length(enc_x$levels)
    n_y <- length(enc_y$levels)
    global <- identical(n_x, n_y)
    pair <- .mc_pair_complete(enc_x$code, enc_y$code, check_na)
    tab <- .mc_fast_ordinal_table(pair$x, pair$y, n_x = n_x, n_y = n_y)
    fit <- if (isTRUE(ci) || isTRUE(p_value)) {
      .mc_polychoric_inference_one(tab, correct = correct, conf_level = conf_level)
    } else {
      NULL
    }
    est <- .mc_polychoric_from_pair(
      pair$x, pair$y,
      n_x = n_x, n_y = n_y,
      correct = correct,
      tau_x = if (global) .mc_global_cutpoints(pair$x, n_x) else NULL,
      tau_y = if (global) .mc_global_cutpoints(pair$y, n_y) else NULL,
      global = global
    )
    if (!is.null(fit)) {
      fit <- .mc_recenter_wald_fit(fit, estimate = est, conf_level = conf_level)
    }
    thresholds <- list(
      data = .mc_global_cutpoints(pair$x, n_x),
      y = .mc_global_cutpoints(pair$y, n_y)
    )
    diagnostics <- .mc_latent_pair_metadata(
      est = est,
      tab = tab,
      n_complete = length(pair$x),
      correct = correct,
      thresholds = thresholds
    )
    out <- .mc_attach_scalar_latent(
      est,
      method = "polychoric",
      description = "Pairwise polychoric correlation",
      diagnostics = diagnostics,
      thresholds = thresholds
    )
    if (isTRUE(ci) || isTRUE(p_value)) {
      out <- .mc_attach_latent_inference(
        out,
        .mc_latent_scalar_payload(fit, return_ci = ci, return_p = p_value),
        conf_level = conf_level
      )
    }
    return(out)
  }

  enc <- .mc_extract_discrete_columns(data, kind = "ordinal", arg = "data", min_cols = 2L)
  .mc_check_latent_missing(
    lapply(enc, `[[`, "code"),
    check_na = check_na,
    arg = "data"
  )

  p <- length(enc)
  n_levels <- vapply(enc, function(z) length(z$levels), integer(1))
  global_all <- length(unique(n_levels)) == 1L
  tau_mat <- if (global_all) {
    .mc_tau_matrix(enc)
  } else {
    matrix(NA_real_, nrow = 0L, ncol = 0L)
  }
  global_thresholds <- lapply(enc, function(z) .mc_global_cutpoints(z$code, length(z$levels)))
  names(global_thresholds) <- names(enc)
  x_mat <- .mc_encode_list_to_matrix(enc)
  out <- if (!isTRUE(ci) && !isTRUE(p_value)) {
    matrixCorr_polychoric_matrix_cpp(
      x = x_mat,
      n_levels = n_levels,
      tau_mat = tau_mat,
      global_all = global_all,
      correct = correct,
      pairwise_complete = !check_na
    )
  } else {
    matrix(NA_real_, nrow = p, ncol = p)
  }
  dimnames(out) <- list(names(enc), names(enc))
  diag_info <- .mc_matrix_diagnostics_template(p, dimnames(out))
  diag(diag_info$zero_cells) <- 0L
  diag(diag_info$n_complete) <- vapply(enc, function(z) sum(!is.na(z$code)), integer(1))
  diag(diag_info$corrected) <- FALSE
  diag(diag_info$boundary) <- FALSE
  diag(diag_info$near_boundary) <- FALSE
  diag(diag_info$converged) <- TRUE
  diag(out) <- ifelse(vapply(enc, function(z) length(unique(z$code[!is.na(z$code)])) >= 2L, logical(1)), 1, NA_real_)
  statistic <- parameter <- p_mat <- lwr <- upr <- NULL
  if (isTRUE(ci) || isTRUE(p_value)) {
    statistic <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    parameter <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    p_mat <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    lwr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
    upr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
  }
  for (j in seq_len(p - 1L)) {
    for (k in (j + 1L):p) {
      pair <- .mc_pair_complete(enc[[j]]$code, enc[[k]]$code, check_na)
      tab <- .mc_fast_ordinal_table(
        pair$x, pair$y,
        n_x = length(enc[[j]]$levels),
        n_y = length(enc[[k]]$levels)
      )
      global <- if (global_all) TRUE else identical(n_levels[j], n_levels[k])
      if (isTRUE(ci) || isTRUE(p_value)) {
        est_jk <- .mc_polychoric_from_pair(
          pair$x, pair$y,
          n_x = n_levels[j],
          n_y = n_levels[k],
          correct = correct,
          tau_x = if (global) global_thresholds[[j]] else NULL,
          tau_y = if (global) global_thresholds[[k]] else NULL,
          global = global
        )
        fit <- .mc_polychoric_inference_one(tab, correct = correct, conf_level = conf_level)
        fit <- .mc_recenter_wald_fit(fit, estimate = est_jk, conf_level = conf_level)
        out[j, k] <- est_jk
        out[k, j] <- out[j, k]
        if (isTRUE(p_value)) {
          statistic[j, k] <- as.numeric(fit$statistic)
          statistic[k, j] <- statistic[j, k]
          parameter[j, k] <- as.numeric(fit$parameter)
          parameter[k, j] <- parameter[j, k]
          p_mat[j, k] <- as.numeric(fit$p_value)
          p_mat[k, j] <- p_mat[j, k]
        }
        if (isTRUE(ci)) {
          lwr[j, k] <- as.numeric(fit$lwr)
          lwr[k, j] <- lwr[j, k]
          upr[j, k] <- as.numeric(fit$upr)
          upr[k, j] <- upr[j, k]
        }
      }
      diag_info <- .mc_fill_pair_diag(diag_info, j, k, out[j, k], tab, length(pair$x), correct)
    }
  }
  thresholds <- global_thresholds

  out <- .mc_structure_corr_matrix(
    out,
    class_name = "polychoric_corr",
    method = "polychoric",
    description = "Pairwise polychoric correlation matrix",
    symmetric = TRUE,
    diagnostics = diag_info,
    thresholds = thresholds,
    correct = correct
  )
  if (isTRUE(ci) || isTRUE(p_value)) {
    out <- .mc_attach_latent_inference(
      out,
      .mc_latent_matrix_payload(
        estimate = unclass(out),
        statistic = statistic,
        parameter = parameter,
        p_value = p_mat,
        n_obs = diag_info$n_complete,
        lwr = lwr,
        upr = upr,
        conf_level = conf_level,
        ci_method = "wald_information_polychoric",
        inference_method = "wald_z_polychoric",
        return_ci = ci,
        return_p = p_value
      ),
      conf_level = conf_level
    )
  }
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

#' @rdname polychoric
#' @method print polychoric_corr
#' @param x An object of class \code{polychoric_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print()}.
#' @export
print.polychoric_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                  max_vars = NULL, width = NULL,
                                  show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x, header = "Polychoric correlation matrix",
    digits = digits, n = n, topn = topn,
    max_vars = max_vars, width = width, show_ci = show_ci, ...
  )
}

#' @rdname polychoric
#' @method plot polychoric_corr
#' @param title Plot title. Default is \code{"Polychoric correlation heatmap"}.
#' @param low_color Color for the minimum correlation.
#' @param high_color Color for the maximum correlation.
#' @param mid_color Color for zero correlation.
#' @param value_text_size Font size used in tile labels.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.polychoric_corr <- function(x, title = "Polychoric correlation heatmap",
                                 low_color = "indianred1",
                                 high_color = "steelblue1",
                                 mid_color = "white",
                                 value_text_size = 4,
                                 show_value = TRUE, ...) {
  .mc_plot_corr_matrix(
    x, class_name = "polychoric_corr", fill_name = "Polychoric",
    title = title, low_color = low_color, high_color = high_color,
    mid_color = mid_color, value_text_size = value_text_size,
    show_value = show_value, ...
  )
}

#' @rdname polychoric
#' @method summary polychoric_corr
#' @param object An object of class \code{polychoric_corr}.
#' @param ci_digits Integer; digits for confidence limits in the pairwise
#'   summary.
#' @param p_digits Integer; digits for p-values in the pairwise summary.
#' @export
summary.polychoric_corr <- function(object, n = NULL, topn = NULL,
                                    max_vars = NULL, width = NULL,
                                    ci_digits = 3,
                                    p_digits = 4,
                                    show_ci = NULL, ...) {
  check_inherits(object, "polychoric_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  ci <- .mc_latent_ci_attr(object)
  inf <- .mc_latent_inference_attr(object)
  if (is.null(ci) && (is.null(inf) || is.null(inf$p_value))) {
    return(.mc_summary_corr_matrix(object, header = "Latent correlation summary", topn = topn))
  }

  .mc_latent_pairwise_summary(
    object,
    summary_class = "summary.polychoric_corr",
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname polychoric
#' @method print summary.polychoric_corr
#' @param x An object of class \code{summary.polychoric_corr}.
#' @export
print.summary.polychoric_corr <- function(x, digits = NULL, n = NULL,
                                          topn = NULL, max_vars = NULL,
                                          width = NULL, show_ci = NULL, ...) {
  .mc_print_latent_pairwise_summary(
    x,
    title = "Polychoric correlation summary",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @title Polyserial Correlation Between Continuous and Ordinal Variables
#'
#' @description
#' Computes polyserial correlations between continuous variables in \code{data}
#' and ordinal variables in \code{y}. Both pairwise vector mode and rectangular
#' matrix/data-frame mode are supported.
#'
#' @usage polyserial(data, y, na_method = c("error", "pairwise"), ci = FALSE, p_value = FALSE,
#'   conf_level = 0.95, ...)
#'
#' @param data A numeric vector, matrix, or data frame containing continuous
#' variables.
#' @param y An ordinal vector, matrix, or data frame containing ordinal
#' variables. Supported columns are factors, ordered factors, logical values,
#' or integer-like numerics.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
#'   complete cases.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald confidence intervals derived from the observed
#' information matrix of the latent-variable likelihood.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample Wald p-values and test statistics for each
#' estimated latent correlation.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#'
#' @return
#' If both \code{data} and \code{y} are vectors, a numeric scalar. Otherwise a
#' numeric matrix of class \code{polyserial_corr} with rows corresponding to
#' the continuous variables in \code{data} and columns to the ordinal variables
#' in \code{y}. Matrix outputs carry attributes \code{method},
#' \code{description}, and \code{package = "matrixCorr"}. When
#' \code{p_value = TRUE}, the returned object also carries an \code{inference}
#' attribute with elements \code{estimate}, \code{statistic}, \code{parameter},
#' \code{p_value}, and \code{n_obs}. When \code{ci = TRUE}, it also carries a
#' \code{ci} attribute with elements \code{est}, \code{lwr.ci},
#' \code{upr.ci}, \code{conf.level}, and \code{ci.method}, plus
#' \code{attr(x, "conf.level")}. Scalar outputs keep the same point estimate
#' and gain the same metadata only when inference is requested.
#'
#' @details
#' The polyserial correlation assumes a latent bivariate normal model between a
#' continuous variable and an unobserved continuous propensity underlying an
#' ordinal variable. Let
#' \eqn{(X, Z)^\top \sim N_2(0, \Sigma)} with
#' \eqn{\mathrm{corr}(X,Z)=\rho}, and suppose the observed ordinal response
#' \eqn{Y} is formed by cut-points
#' \eqn{-\infty = \beta_0 < \beta_1 < \cdots < \beta_K = \infty}:
#' \deqn{
#' Y = k \iff \beta_{k-1} < Z \le \beta_k.
#' }
#' After standardising the observed continuous variable \eqn{X}, the thresholds
#' are estimated from the marginal proportions of \eqn{Y}. Conditional on an
#' observed \eqn{x_i}, the category probability is
#' \deqn{
#' \Pr(Y_i = k \mid X_i = x_i, \rho)
#' =
#' \Phi\!\left(\frac{\beta_k - \rho x_i}{\sqrt{1-\rho^2}}\right)
#' -
#' \Phi\!\left(\frac{\beta_{k-1} - \rho x_i}{\sqrt{1-\rho^2}}\right).
#' }
#' The returned estimate maximises the log-likelihood
#' \deqn{
#' \ell(\rho) = \sum_{i=1}^{n}\log \Pr(Y_i = y_i \mid X_i = x_i, \rho)
#' }
#' over \eqn{\rho \in (-1,1)} via a one-dimensional Brent search in C++.
#'
#' \strong{Assumptions.} The coefficient is appropriate when the ordinal
#' variable is viewed as the discretised version of a latent normal variable
#' that is jointly normal with the observed continuous variable. The optional
#' p-values and confidence intervals adopt this latent-normal interpretation
#' and use the same likelihood that defines the polyserial estimate. These
#' inferential quantities are therefore model-based and should not be
#' interpreted as distribution-free summaries.
#'
#' \strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
#' function refits the pairwise polyserial model by maximum likelihood and
#' obtains the observed information matrix numerically in C++. The reported
#' confidence interval is a Wald interval
#' \eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
#' reported p-value is from the large-sample Wald \eqn{z}-test for
#' \eqn{H_0:\rho = 0}. These inferential quantities are only computed when
#' explicitly requested.
#'
#' In vector mode a single estimate is returned. In matrix/data-frame mode,
#' every numeric column of \code{data} is paired with every ordinal column of
#' \code{y}, producing a rectangular matrix of continuous-by-ordinal
#' polyserial correlations.
#'
#' \strong{Computational complexity.} If \code{data} has \eqn{p_x} continuous
#' columns and \code{y} has \eqn{p_y} ordinal columns, the matrix path computes
#' \eqn{p_x p_y} separate one-parameter likelihood optimisations.
#'
#' @references
#' Olsson, U., Drasgow, F., & Dorans, N. J. (1982). The polyserial
#' correlation coefficient. \emph{Psychometrika}, 47(3), 337-347.
#' @param ... Deprecated compatibility aliases. The legacy \code{check_na}
#'   argument is still accepted temporarily.
#'
#' @examplesIf requireNamespace("mnormt", quietly = TRUE)
#' set.seed(125)
#' n <- 1000
#' Sigma <- matrix(c(
#'   1.00, 0.30, 0.55, 0.20,
#'   0.30, 1.00, 0.25, 0.50,
#'   0.55, 0.25, 1.00, 0.40,
#'   0.20, 0.50, 0.40, 1.00
#' ), 4, 4, byrow = TRUE)
#'
#' Z <- mnormt::rmnorm(n = n, mean = rep(0, 4), varcov = Sigma)
#' X <- data.frame(x1 = Z[, 1], x2 = Z[, 2])
#' Y <- data.frame(
#'   y1 = ordered(cut(
#'     Z[, 3],
#'     breaks = c(-Inf, -0.5, 0.7, Inf),
#'     labels = c("low", "mid", "high")
#'   )),
#'   y2 = ordered(cut(
#'     Z[, 4],
#'     breaks = c(-Inf, -1.0, 0.0, 1.0, Inf),
#'     labels = c("1", "2", "3", "4")
#'   ))
#' )
#'
#' ps <- polyserial(X, Y)
#' print(ps, digits = 3)
#' summary(ps)
#' estimate(ps)
#' tidy(ps)
#' ps_ci <- polyserial(X, Y, ci = TRUE)
#' ci(ps_ci)
#' confint(ps_ci)
#' plot(ps)
#' @author Thiago de Paula Oliveira
#' @export
polyserial <- function(data,
                       y,
                       na_method = c("error", "pairwise"),
                       ci = FALSE,
                       p_value = FALSE,
                       conf_level = 0.95,
                       ...) {
  if (missing(na_method) && isFALSE(ci) && isFALSE(p_value) && ...length() == 0L) {
    scalar <- is.null(dim(data)) && is.atomic(data) && !is.factor(data) &&
      is.null(dim(y)) && is.atomic(y)

    x_mat <- .mc_extract_continuous_matrix(data, arg = "data", min_cols = 1L)
    y_enc <- .mc_extract_discrete_columns(y, kind = "ordinal", arg = "y", min_cols = 1L)
    if (nrow(x_mat) != length(y_enc[[1L]]$code)) {
      abort_bad_arg("data", message = "and {.arg y} must have the same number of observations.")
    }
    .mc_check_latent_missing(
      c(list(.x_mat = x_mat), lapply(y_enc, `[[`, "code")),
      check_na = TRUE,
      arg = "data"
    )

    if (scalar) {
      return(.mc_pair_polyserial(x_mat[, 1L], y_enc[[1L]]$code, TRUE))
    }

    out <- matrix(
      NA_real_,
      nrow = ncol(x_mat),
      ncol = length(y_enc),
      dimnames = list(colnames(x_mat), names(y_enc))
    )
    for (j in seq_len(ncol(x_mat))) {
      xj <- x_mat[, j]
      for (k in seq_along(y_enc)) {
        out[j, k] <- .mc_pair_polyserial(xj, y_enc[[k]]$code, TRUE)
      }
    }
    return(.mc_structure_corr_matrix(
      out,
      class_name = "polyserial_corr",
      method = "polyserial",
      description = "Polyserial correlation matrix (continuous x ordinal)"
    ))
  }

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method)
  )
  check_na <- na_cfg$check_na
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  scalar <- is.null(dim(data)) && is.atomic(data) && !is.factor(data) &&
    is.null(dim(y)) && is.atomic(y)

  x_mat <- .mc_extract_continuous_matrix(data, arg = "data", min_cols = 1L)
  y_enc <- .mc_extract_discrete_columns(y, kind = "ordinal", arg = "y", min_cols = 1L)

  if (nrow(x_mat) != length(y_enc[[1L]]$code)) {
    abort_bad_arg(
      "data",
      message = "and {.arg y} must have the same number of observations."
    )
  }

  .mc_check_latent_missing(
    c(list(.x_mat = x_mat), lapply(y_enc, `[[`, "code")),
    check_na = check_na,
    arg = "data"
  )

  if (scalar) {
    if (!isTRUE(ci) && !isTRUE(p_value)) {
      return(.mc_pair_polyserial(x_mat[, 1L], y_enc[[1L]]$code, check_na))
    }
    fit <- .mc_polyserial_inference_one(
      x_mat[, 1L],
      y_enc[[1L]]$code,
      check_na = check_na,
      conf_level = conf_level
    )
    out <- .mc_attach_scalar_latent(
      fit$estimate,
      method = "polyserial",
      description = "Polyserial correlation",
      diagnostics = list(n_complete = as.integer(fit$n_obs))
    )
    out <- .mc_attach_latent_inference(
      out,
      .mc_latent_scalar_payload(fit, return_ci = ci, return_p = p_value),
      conf_level = conf_level
    )
    return(out)
  }

  out <- matrix(NA_real_, nrow = ncol(x_mat), ncol = length(y_enc),
                dimnames = list(colnames(x_mat), names(y_enc)))
  n_complete <- NULL
  if (isTRUE(ci) || isTRUE(p_value)) {
    n_complete <- matrix(
      NA_integer_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    )
  }
  statistic <- parameter <- p_mat <- lwr <- upr <- NULL
  if (isTRUE(ci) || isTRUE(p_value)) {
    statistic <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    parameter <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    p_mat <- if (isTRUE(p_value)) .mc_new_metric_matrix(out, "double") else NULL
    lwr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
    upr <- if (isTRUE(ci)) .mc_new_metric_matrix(out, "double") else NULL
  }

  for (j in seq_len(ncol(x_mat))) {
    xj <- x_mat[, j]
    for (k in seq_along(y_enc)) {
      if (isTRUE(ci) || isTRUE(p_value)) {
        fit <- .mc_polyserial_inference_one(
          xj,
          y_enc[[k]]$code,
          check_na = check_na,
          conf_level = conf_level
        )
        out[j, k] <- as.numeric(fit$estimate)
        n_complete[j, k] <- as.integer(fit$n_obs)
        if (isTRUE(p_value)) {
          statistic[j, k] <- as.numeric(fit$statistic)
          parameter[j, k] <- as.numeric(fit$parameter)
          p_mat[j, k] <- as.numeric(fit$p_value)
        }
        if (isTRUE(ci)) {
          lwr[j, k] <- as.numeric(fit$lwr)
          upr[j, k] <- as.numeric(fit$upr)
        }
      } else {
        out[j, k] <- .mc_pair_polyserial(xj, y_enc[[k]]$code, check_na)
      }
    }
  }

  out <- .mc_structure_corr_matrix(
    out,
    class_name = "polyserial_corr",
    method = "polyserial",
    description = "Polyserial correlation matrix (continuous x ordinal)",
    diagnostics = if (isTRUE(ci) || isTRUE(p_value)) list(n_complete = n_complete) else NULL
  )
  if (isTRUE(ci) || isTRUE(p_value)) {
    out <- .mc_attach_latent_inference(
      out,
      .mc_latent_matrix_payload(
        estimate = unclass(out),
        statistic = statistic,
        parameter = parameter,
        p_value = p_mat,
        n_obs = n_complete,
        lwr = lwr,
        upr = upr,
        conf_level = conf_level,
        ci_method = "wald_information_polyserial",
        inference_method = "wald_z_polyserial",
        return_ci = ci,
        return_p = p_value
      ),
      conf_level = conf_level
    )
  }
  out
}

#' @rdname polyserial
#' @method print polyserial_corr
#' @param x An object of class \code{polyserial_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print()}.
#' @export
print.polyserial_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                  max_vars = NULL, width = NULL,
                                  show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x, header = "Polyserial correlation matrix",
    digits = digits, n = n, topn = topn,
    max_vars = max_vars, width = width, show_ci = show_ci, ...
  )
}

#' @rdname polyserial
#' @method plot polyserial_corr
#' @param title Plot title. Default is \code{"Polyserial correlation heatmap"}.
#' @param low_color Color for the minimum correlation.
#' @param high_color Color for the maximum correlation.
#' @param mid_color Color for zero correlation.
#' @param value_text_size Font size used in tile labels.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.polyserial_corr <- function(x, title = "Polyserial correlation heatmap",
                                 low_color = "indianred1",
                                 high_color = "steelblue1",
                                 mid_color = "white",
                                 value_text_size = 4,
                                 show_value = TRUE, ...) {
  .mc_plot_corr_matrix(
    x, class_name = "polyserial_corr", fill_name = "Polyserial",
    title = title, low_color = low_color, high_color = high_color,
    mid_color = mid_color, value_text_size = value_text_size,
    show_value = show_value, ...
  )
}

#' @rdname polyserial
#' @method summary polyserial_corr
#' @param object An object of class \code{polyserial_corr}.
#' @param ci_digits Integer; digits for confidence limits in the pairwise
#'   summary.
#' @param p_digits Integer; digits for p-values in the pairwise summary.
#' @export
summary.polyserial_corr <- function(object, n = NULL, topn = NULL,
                                    max_vars = NULL, width = NULL,
                                    ci_digits = 3,
                                    p_digits = 4,
                                    show_ci = NULL, ...) {
  check_inherits(object, "polyserial_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  ci <- .mc_latent_ci_attr(object)
  inf <- .mc_latent_inference_attr(object)
  if (is.null(ci) && (is.null(inf) || is.null(inf$p_value))) {
    return(.mc_summary_corr_matrix(object, header = "Latent correlation summary", topn = topn))
  }

  .mc_latent_pairwise_summary(
    object,
    summary_class = "summary.polyserial_corr",
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname polyserial
#' @method print summary.polyserial_corr
#' @param x An object of class \code{summary.polyserial_corr}.
#' @export
print.summary.polyserial_corr <- function(x, digits = NULL, n = NULL,
                                          topn = NULL, max_vars = NULL,
                                          width = NULL, show_ci = NULL, ...) {
  .mc_print_latent_pairwise_summary(
    x,
    title = "Polyserial correlation summary",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @title Biserial Correlation Between Continuous and Binary Variables
#'
#' @description
#' Computes biserial correlations between continuous variables in \code{data}
#' and binary variables in \code{y}. Both pairwise vector mode and rectangular
#' matrix/data-frame mode are supported.
#'
#' @usage biserial(data, y, na_method = c("error", "pairwise"), ci = FALSE, p_value = FALSE,
#'   conf_level = 0.95, ...)
#'
#' @param data A numeric vector, matrix, or data frame containing continuous
#' variables.
#' @param y A binary vector, matrix, or data frame. In data-frame mode, only
#' two-level columns are retained.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
#'   complete cases.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach
#' approximate large-sample confidence intervals derived from a Fisher
#' \eqn{z}-transformation of the biserial estimate.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' model-based large-sample p-values, test statistics, and degrees of freedom
#' for each biserial estimate.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#'
#' @return
#' If both \code{data} and \code{y} are vectors, a numeric scalar. Otherwise a
#' numeric matrix of class \code{biserial_corr} with rows corresponding to
#' the continuous variables in \code{data} and columns to the binary variables
#' in \code{y}. Matrix outputs carry attributes \code{method},
#' \code{description}, and \code{package = "matrixCorr"}. When
#' \code{p_value = TRUE}, the object also carries an \code{inference}
#' attribute with matrices \code{estimate}, \code{statistic},
#' \code{parameter}, \code{p_value}, and \code{n_obs}. When \code{ci = TRUE},
#' it additionally carries a \code{ci} attribute with matrices
#' \code{lwr.ci} and \code{upr.ci}, plus \code{attr(x, "conf.level")}. Scalar
#' outputs keep the same point estimate and gain the same metadata only when
#' inference is requested.
#'
#' @details
#' The biserial correlation is the special two-category case of the polyserial
#' model. It assumes that a binary variable \eqn{Y} arises by thresholding an
#' unobserved standard-normal variable \eqn{Z} that is jointly normal with a
#' continuous variable \eqn{X}. Writing \eqn{p = P(Y = 1)} and
#' \eqn{q = 1-p}, let \eqn{z_p = \Phi^{-1}(p)} and \eqn{\phi(z_p)} be the
#' standard-normal density evaluated at \eqn{z_p}. If \eqn{\bar x_1} and
#' \eqn{\bar x_0} denote the sample means of \eqn{X} in the two observed groups
#' and \eqn{s_x} is the sample standard deviation of \eqn{X}, the usual
#' biserial estimator is
#' \deqn{
#' r_b =
#' \frac{\bar x_1 - \bar x_0}{s_x}
#' \frac{pq}{\phi(z_p)}.
#' }
#' This is exactly the estimator implemented in the underlying C++ kernel.
#'
#' \strong{Assumptions.} The biserial coefficient is appropriate when the
#' observed binary variable is viewed as a thresholded version of an
#' unobserved continuous latent variable that is jointly normal with the
#' observed continuous variable. The optional p-values and confidence
#' intervals adopt this latent-normal interpretation together with the usual
#' large-sample approximations used for correlation coefficients. These
#' inferential quantities are therefore model-based and should not be
#' interpreted as distribution-free summaries.
#'
#' \strong{Inference.} When \code{p_value = TRUE}, the package reports the
#' large-sample \eqn{t}-statistic
#' \deqn{
#' t = r_b \sqrt{\frac{n - 2}{1 - r_b^2}},
#' }
#' referenced to a Student \eqn{t}-distribution with \eqn{n - 2} degrees of
#' freedom. When \code{ci = TRUE}, the package forms an approximate Fisher
#' \eqn{z}-interval by transforming \eqn{r_b} with
#' \eqn{z = \operatorname{atanh}(r_b)}, using standard error
#' \eqn{1 / \sqrt{n - 3}}, and mapping the limits back with
#' \eqn{\tanh(\cdot)}. The CI is therefore an internal large-sample
#' extension and is only computed when explicitly requested.
#'
#' In vector mode a single biserial correlation is returned. In
#' matrix/data-frame mode, every numeric column of \code{data} is paired with every
#' binary column of \code{y}, producing a rectangular matrix of
#' continuous-by-binary biserial correlations.
#'
#' Unlike the point-biserial correlation, which is just Pearson correlation on a
#' 0/1 coding of the binary variable, the biserial coefficient explicitly
#' assumes an underlying latent normal threshold model and rescales the mean
#' difference accordingly.
#'
#' \strong{Computational complexity.} If \code{data} has \eqn{p_x} continuous
#' columns and \code{y} has \eqn{p_y} binary columns, the matrix path computes
#' \eqn{p_x p_y} closed-form estimates with negligible extra memory beyond the
#' output matrix.
#'
#' @references
#' Olsson, U., Drasgow, F., & Dorans, N. J. (1982). The polyserial
#' correlation coefficient. \emph{Psychometrika}, 47(3), 337-347.
#'
#' Fisher, R. A. (1921). On the probable error of a coefficient of
#' correlation deduced from a small sample. \emph{Metron}, 1, 3-32.
#' @param ... Deprecated compatibility aliases. The legacy \code{check_na}
#'   argument is still accepted temporarily.
#'
#' @examplesIf requireNamespace("mnormt", quietly = TRUE)
#' set.seed(126)
#' n <- 1000
#' Sigma <- matrix(c(
#'   1.00, 0.35, 0.50, 0.25,
#'   0.35, 1.00, 0.30, 0.55,
#'   0.50, 0.30, 1.00, 0.40,
#'   0.25, 0.55, 0.40, 1.00
#' ), 4, 4, byrow = TRUE)
#'
#' Z <- mnormt::rmnorm(n = n, mean = rep(0, 4), varcov = Sigma)
#' X <- data.frame(x1 = Z[, 1], x2 = Z[, 2])
#' Y <- data.frame(
#'   g1 = Z[, 3] > stats::qnorm(0.65),
#'   g2 = Z[, 4] > stats::qnorm(0.55)
#' )
#'
#' bs <- biserial(X, Y, ci = TRUE, p_value = TRUE)
#' print(bs, digits = 3)
#' summary(bs)
#' estimate(bs)
#' tidy(bs)
#' ci(bs)
#' confint(bs)
#' plot(bs)
#' @author Thiago de Paula Oliveira
#' @export
biserial <- function(data,
                     y,
                     na_method = c("error", "pairwise"),
                     ci = FALSE,
                     p_value = FALSE,
                     conf_level = 0.95,
                     ...) {
  if (missing(na_method) && isFALSE(ci) && isFALSE(p_value) && ...length() == 0L) {
    scalar <- is.null(dim(data)) && is.atomic(data) && !is.factor(data) &&
      is.null(dim(y)) && is.atomic(y)

    x_mat <- .mc_extract_continuous_matrix(data, arg = "data", min_cols = 1L)
    y_enc <- .mc_extract_discrete_columns(y, kind = "binary", arg = "y", min_cols = 1L)
    if (nrow(x_mat) != length(y_enc[[1L]]$code)) {
      abort_bad_arg("data", message = "and {.arg y} must have the same number of observations.")
    }
    .mc_check_latent_missing(
      c(list(.x_mat = x_mat), lapply(y_enc, `[[`, "code")),
      check_na = TRUE,
      arg = "data"
    )

    out <- matrix(
      NA_real_,
      nrow = ncol(x_mat),
      ncol = length(y_enc),
      dimnames = list(colnames(x_mat), names(y_enc))
    )
    for (j in seq_len(ncol(x_mat))) {
      xj <- x_mat[, j]
      for (k in seq_along(y_enc)) {
        out[j, k] <- .mc_pair_biserial(xj, y_enc[[k]]$code, TRUE)
      }
    }
    out <- .mc_scalar_or_matrix(out, scalar = scalar)
    if (scalar) {
      return(out)
    }
    return(.mc_structure_corr_matrix(
      out,
      class_name = "biserial_corr",
      method = "biserial",
      description = "Biserial correlation matrix (continuous x binary)"
    ))
  }

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method)
  )
  check_na <- na_cfg$check_na
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  scalar <- is.null(dim(data)) && is.atomic(data) && !is.factor(data) &&
    is.null(dim(y)) && is.atomic(y)

  x_mat <- .mc_extract_continuous_matrix(data, arg = "data", min_cols = 1L)
  y_enc <- .mc_extract_discrete_columns(y, kind = "binary", arg = "y", min_cols = 1L)

  if (nrow(x_mat) != length(y_enc[[1L]]$code)) {
    abort_bad_arg(
      "data",
      message = "and {.arg y} must have the same number of observations."
    )
  }

  .mc_check_latent_missing(
    c(list(.x_mat = x_mat), lapply(y_enc, `[[`, "code")),
    check_na = check_na,
    arg = "data"
  )

  out <- matrix(NA_real_, nrow = ncol(x_mat), ncol = length(y_enc),
                dimnames = list(colnames(x_mat), names(y_enc)))
  diagnostics <- NULL
  inference_attr <- NULL
  ci_attr <- NULL

  if (isTRUE(ci) || isTRUE(p_value)) {
    n_complete <- matrix(
      NA_integer_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    )
    statistic <- if (isTRUE(p_value)) matrix(
      NA_real_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    ) else NULL
    parameter <- if (isTRUE(p_value)) matrix(
      NA_real_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    ) else NULL
    p_mat <- if (isTRUE(p_value)) matrix(
      NA_real_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    ) else NULL
    lwr <- if (isTRUE(ci)) matrix(
      NA_real_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    ) else NULL
    upr <- if (isTRUE(ci)) matrix(
      NA_real_,
      nrow = nrow(out),
      ncol = ncol(out),
      dimnames = dimnames(out)
    ) else NULL
  }

  for (j in seq_len(ncol(x_mat))) {
    xj <- x_mat[, j]
    for (k in seq_along(y_enc)) {
      if (isTRUE(ci) || isTRUE(p_value)) {
        fit <- .mc_biserial_inference_one(
          xj,
          y_enc[[k]]$code,
          check_na = check_na,
          return_ci = ci,
          conf_level = conf_level
        )
        out[j, k] <- fit$estimate
        n_complete[j, k] <- fit$n_obs
        if (isTRUE(p_value)) {
          statistic[j, k] <- fit$statistic
          parameter[j, k] <- fit$parameter
          p_mat[j, k] <- fit$p_value
        }
        if (isTRUE(ci)) {
          lwr[j, k] <- fit$lwr
          upr[j, k] <- fit$upr
        }
      } else {
        out[j, k] <- .mc_pair_biserial(xj, y_enc[[k]]$code, check_na)
      }
    }
  }

  estimate_mat <- out
  attributes(estimate_mat) <- attributes(estimate_mat)[c("dim", "dimnames")]
  out <- .mc_scalar_or_matrix(out, scalar = scalar)
  if (scalar) {
    if (!isTRUE(ci) && !isTRUE(p_value)) {
      return(out)
    }
    diagnostics <- list(n_complete = as.integer(n_complete[1L, 1L]))
    scalar_out <- .mc_attach_scalar_latent(
      out,
      method = "biserial",
      description = "Biserial correlation",
      diagnostics = diagnostics
    )
    if (isTRUE(p_value)) {
      attr(scalar_out, "inference") <- list(
        method = "biserial_t_test",
        estimate = as.numeric(out),
        statistic = as.numeric(statistic[1L, 1L]),
        parameter = as.numeric(parameter[1L, 1L]),
        p_value = as.numeric(p_mat[1L, 1L]),
        n_obs = as.integer(n_complete[1L, 1L]),
        alternative = "two.sided"
      )
    }
    if (isTRUE(ci)) {
      attr(scalar_out, "ci") <- list(
        est = as.numeric(out),
        lwr.ci = as.numeric(lwr[1L, 1L]),
        upr.ci = as.numeric(upr[1L, 1L]),
        conf.level = conf_level,
        ci.method = "fisher_z_biserial"
      )
      attr(scalar_out, "conf.level") <- conf_level
    }
    return(scalar_out)
  }

  diagnostics <- if (isTRUE(ci) || isTRUE(p_value)) {
    list(n_complete = n_complete)
  } else {
    NULL
  }

  out <- .mc_structure_corr_matrix(
    out,
    class_name = "biserial_corr",
    method = "biserial",
    description = "Biserial correlation matrix (continuous x binary)",
    diagnostics = diagnostics
  )
    if (isTRUE(p_value)) {
      inference_attr <- list(
        method = "biserial_t_test",
        estimate = estimate_mat,
        statistic = statistic,
        parameter = parameter,
        p_value = p_mat,
      n_obs = n_complete,
      alternative = "two.sided"
    )
    attr(out, "inference") <- inference_attr
  }
  if (isTRUE(ci)) {
    ci_attr <- list(
      est = estimate_mat,
      lwr.ci = lwr,
      upr.ci = upr,
      conf.level = conf_level,
      ci.method = "fisher_z_biserial"
    )
    attr(out, "ci") <- ci_attr
    attr(out, "conf.level") <- conf_level
  }
  out
}

#' @rdname biserial
#' @method print biserial_corr
#' @param x An object of class \code{biserial_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print()}.
#' @export
print.biserial_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                max_vars = NULL, width = NULL,
                                show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x, header = "Biserial correlation matrix",
    digits = digits, n = n, topn = topn,
    max_vars = max_vars, width = width, show_ci = show_ci, ...
  )
}

#' @rdname biserial
#' @method plot biserial_corr
#' @param title Plot title. Default is \code{"Biserial correlation heatmap"}.
#' @param low_color Color for the minimum correlation.
#' @param high_color Color for the maximum correlation.
#' @param mid_color Color for zero correlation.
#' @param value_text_size Font size used in tile labels.
#' @param ci_text_size Text size for confidence intervals in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.biserial_corr <- function(x, title = "Biserial correlation heatmap",
                               low_color = "indianred1",
                               high_color = "steelblue1",
                               mid_color = "white",
                               value_text_size = 4,
                               ci_text_size = 3,
                               show_value = TRUE, ...) {
  check_bool(show_value, arg = "show_value")
  ci <- .mc_biserial_ci_attr(x)
  if (is.null(ci) || is.null(ci$lwr.ci) || is.null(ci$upr.ci)) {
    return(.mc_plot_corr_matrix(
      x, class_name = "biserial_corr", fill_name = "Biserial",
      title = title, low_color = low_color, high_color = high_color,
      mid_color = mid_color, value_text_size = value_text_size,
      show_value = show_value, ...
    ))
  }

  est_mat <- as.matrix(x)
  df_est <- as.data.frame(as.table(est_mat))
  names(df_est) <- c("Var1", "Var2", "biserial")

  df_lwr <- as.data.frame(as.table(ci$lwr.ci))
  names(df_lwr)[3L] <- "lwr"
  df_upr <- as.data.frame(as.table(ci$upr.ci))
  names(df_upr)[3L] <- "upr"
  df <- Reduce(
    function(a, b) merge(a, b, by = c("Var1", "Var2"), all = TRUE),
    list(df_est, df_lwr, df_upr)
  )

  lev_row <- unique(df_est$Var1)
  lev_col <- unique(df_est$Var2)
  df$Var1 <- factor(df$Var1, levels = rev(lev_row))
  df$Var2 <- factor(df$Var2, levels = lev_col)
  df$ci_label <- ifelse(
    is.na(df$lwr) | is.na(df$upr),
    NA_character_,
    sprintf("[%.3f, %.3f]", df$lwr, df$upr)
  )

  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = .data$biserial)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = "Biserial"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ifelse(is.na(biserial), "NA", sprintf("%.2f", biserial))),
      size = value_text_size,
      color = "black"
    )
  }
  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.22),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }

  p
}

.mc_latent_pairwise_summary <- function(object,
                                        summary_class,
                                        digits = 4,
                                        ci_digits = 3,
                                        p_digits = 4,
                                        show_ci = "yes") {
  est <- as.matrix(object)
  rn <- rownames(est)
  cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  ci <- .mc_latent_ci_attr(object)
  inf <- .mc_latent_inference_attr(object)
  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  include_ci <- identical(show_ci, "yes") && is.list(ci)
  include_p <- is.list(inf) && !is.null(inf$p_value)
  symmetric <- isTRUE(nrow(est) == ncol(est)) && isTRUE(isSymmetric(unclass(est)))

  n_pairs <- if (symmetric) {
    nrow(est) * (ncol(est) - 1L) / 2L
  } else {
    nrow(est) * ncol(est)
  }
  rows <- vector("list", n_pairs)
  k <- 0L

  for (i in seq_len(nrow(est))) {
    j_start <- if (symmetric) i + 1L else 1L
    if (j_start > ncol(est)) {
      next
    }
    for (j in j_start:ncol(est)) {
      k <- k + 1L
      rec <- list(
        var1 = rn[i],
        var2 = cn[j],
        estimate = round(est[i, j], digits)
      )
      if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
        rec$n_complete <- as.integer(diag_attr$n_complete[i, j])
      }
      if (include_ci) {
        rec$lwr <- if (is.finite(ci$lwr.ci[i, j])) round(ci$lwr.ci[i, j], ci_digits) else NA_real_
        rec$upr <- if (is.finite(ci$upr.ci[i, j])) round(ci$upr.ci[i, j], ci_digits) else NA_real_
      }
      if (include_p) {
        rec$statistic <- if (is.finite(inf$statistic[i, j])) round(inf$statistic[i, j], digits) else NA_real_
        rec$df <- if (is.finite(inf$parameter[i, j])) round(inf$parameter[i, j], digits) else NA_real_
        rec$p_value <- if (is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], p_digits) else NA_real_
      }
      rows[[k]] <- rec
    }
  }

  if (!k) {
    df <- data.frame(var1 = character(), var2 = character(), estimate = numeric())
  } else {
    rows <- rows[seq_len(k)]
    df <- do.call(rbind.data.frame, rows)
  }
  rownames(df) <- NULL
  num_cols <- intersect(c("estimate", "lwr", "upr", "statistic", "df", "p_value"), names(df))
  int_cols <- intersect(c("n_complete"), names(df))
  for (nm in num_cols) df[[nm]] <- as.numeric(df[[nm]])
  for (nm in int_cols) df[[nm]] <- as.integer(df[[nm]])

  out <- .mc_finalize_summary_df(df, class_name = summary_class)
  attr(out, "overview") <- .mc_summary_corr_matrix(object, header = "Latent correlation summary")
  attr(out, "has_ci") <- include_ci
  attr(out, "has_p") <- include_p
  attr(out, "conf.level") <- if (is.null(ci)) NA_real_ else ci$conf.level
  attr(out, "ci_method") <- if (is.null(ci)) NA_character_ else ci$ci.method
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "p_digits") <- p_digits
  attr(out, "inference_method") <- if (is.null(inf)) NA_character_ else inf$method
  out
}

.mc_print_latent_pairwise_summary <- function(x,
                                              title,
                                              digits = NULL,
                                              n = NULL,
                                              topn = NULL,
                                              max_vars = NULL,
                                              width = NULL,
                                              show_ci = NULL,
                                              ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = title,
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = if (isTRUE(attr(x, "has_ci"))) attr(x, "ci_method", exact = TRUE) else NULL,
    extra_items = c(inference = attr(x, "inference_method", exact = TRUE)),
    ...
  )
  invisible(x)
}

.mc_biserial_pairwise_summary <- function(object,
                                          digits = 4,
                                          ci_digits = 3,
                                          p_digits = 4,
                                          show_ci = "yes") {
  check_inherits(object, "biserial_corr")

  est <- as.matrix(object)
  rn <- rownames(est)
  cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  ci <- .mc_biserial_ci_attr(object)
  inf <- .mc_biserial_inference_attr(object)
  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  include_ci <- identical(show_ci, "yes") && is.list(ci)
  include_p <- is.list(inf) && !is.null(inf$p_value)

  rows <- vector("list", nrow(est) * ncol(est))
  k <- 0L
  for (i in seq_len(nrow(est))) {
    for (j in seq_len(ncol(est))) {
      k <- k + 1L
      rec <- list(
        var1 = rn[i],
        var2 = cn[j],
        estimate = round(est[i, j], digits)
      )
      if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
        rec$n_complete <- as.integer(diag_attr$n_complete[i, j])
      }
      if (include_ci) {
        rec$lwr <- if (is.finite(ci$lwr.ci[i, j])) round(ci$lwr.ci[i, j], ci_digits) else NA_real_
        rec$upr <- if (is.finite(ci$upr.ci[i, j])) round(ci$upr.ci[i, j], ci_digits) else NA_real_
      }
      if (include_p) {
        rec$statistic <- if (is.finite(inf$statistic[i, j])) round(inf$statistic[i, j], digits) else NA_real_
        rec$df <- if (is.finite(inf$parameter[i, j])) round(inf$parameter[i, j], digits) else NA_real_
        rec$p_value <- if (is.finite(inf$p_value[i, j])) round(inf$p_value[i, j], p_digits) else NA_real_
      }
      rows[[k]] <- rec
    }
  }

  df <- do.call(rbind.data.frame, rows)
  rownames(df) <- NULL
  num_cols <- intersect(c("estimate", "lwr", "upr", "statistic", "df", "p_value"), names(df))
  int_cols <- intersect(c("n_complete"), names(df))
  for (nm in num_cols) df[[nm]] <- as.numeric(df[[nm]])
  for (nm in int_cols) df[[nm]] <- as.integer(df[[nm]])

  out <- .mc_finalize_summary_df(df, class_name = "summary.biserial_corr")
  attr(out, "overview") <- .mc_summary_corr_matrix(object)
  attr(out, "has_ci") <- include_ci
  attr(out, "has_p") <- include_p
  attr(out, "conf.level") <- if (is.null(ci)) NA_real_ else ci$conf.level
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "p_digits") <- p_digits
  attr(out, "inference_method") <- if (is.null(inf)) NA_character_ else inf$method
  out
}

#' @rdname biserial
#' @method summary biserial_corr
#' @param object An object of class \code{biserial_corr}.
#' @param ci_digits Integer; digits for biserial confidence limits in the
#'   pairwise summary.
#' @param p_digits Integer; digits for biserial p-values in the pairwise
#'   summary.
#' @export
summary.biserial_corr <- function(object, n = NULL, topn = NULL,
                                  max_vars = NULL, width = NULL,
                                  ci_digits = 3,
                                  p_digits = 4,
                                  show_ci = NULL, ...) {
  check_inherits(object, "biserial_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  ci <- .mc_biserial_ci_attr(object)
  inf <- .mc_biserial_inference_attr(object)
  if (is.null(ci) && (is.null(inf) || is.null(inf$p_value))) {
    return(.mc_summary_corr_matrix(object, header = "Latent correlation summary", topn = topn))
  }

  .mc_biserial_pairwise_summary(
    object,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname biserial
#' @method print summary.biserial_corr
#' @param x An object of class \code{summary.biserial_corr}.
#' @export
print.summary.biserial_corr <- function(x, digits = NULL, n = NULL,
                                        topn = NULL, max_vars = NULL,
                                        width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Biserial correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = if (isTRUE(attr(x, "has_ci"))) "fisher_z_biserial" else NULL,
    extra_items = c(inference = attr(x, "inference_method", exact = TRUE)),
    ...
  )
  invisible(x)
}

#' @title Summary Method for Correlation Matrices
#'
#' @description
#' Prints compact summary statistics returned by matrix-style
#' \code{summary()} methods in \pkg{matrixCorr}.
#'
#' @param x An object of class \code{summary.matrixCorr}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Unused.
#'
#' @return Invisibly returns \code{x}.
#' @method print summary.matrixCorr
#' @export
print.summary.matrixCorr <- function(x,
                                     digits = 4,
                                     n = NULL,
                                     topn = NULL,
                                     max_vars = NULL,
                                     width = NULL,
                                     show_ci = NULL,
                                     ...) {
  if (inherits(x, "summary.corr_result")) {
    x2 <- x
    attr(x2, "summary_title") <- "Correlation summary"
    return(print.summary.corr_result(
      x2,
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      ...
    ))
  }
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )

  digest <- .mc_corr_summary_digest_items(x, digits = digits, show_ci = cfg$show_ci)
  if (isTRUE(x$has_ci) && identical(cfg$show_ci, "yes")) {
    if (is.finite(x$ci_conf_level)) {
      digest <- c(digest, ci_level = sprintf("%g%%", 100 * x$ci_conf_level))
    }
    if (!is.null(x$ci_method) && length(x$ci_method) == 1L &&
        !is.na(x$ci_method) && nzchar(x$ci_method)) {
      digest <- c(digest, ci_method = x$ci_method)
    }
    ci_width_txt <- .mc_format_scalar_or_range(
      x$ci_width_min,
      x$ci_width_max,
      digits = 3,
      integer = FALSE
    )
    if (!is.null(ci_width_txt)) {
      digest <- c(digest, ci_width = ci_width_txt)
    }
    if (is.finite(x$ci_cross_zero) && x$ci_cross_zero > 0L) {
      digest <- c(digest, cross_zero = sprintf("%s pair(s)", .mc_count_fmt(x$ci_cross_zero)))
    }
  }
  if (!is.null(x$lambda) && is.finite(x$lambda)) {
    digest <- c(digest, lambda = format(signif(x$lambda, digits = digits)))
  }
  if (!is.null(x$rho) && is.finite(x$rho)) {
    digest <- c(digest, rho = format(signif(x$rho, digits = digits)))
  }
  if (!is.null(x$jitter) && is.finite(x$jitter)) {
    digest <- c(digest, jitter = format(signif(x$jitter, digits = digits)))
  }

  .mc_print_named_digest(
    digest,
    header = .mc_coalesce(x$header, "Correlation summary")
  )

  if (is.data.frame(x$top_results) && nrow(x$top_results)) {
    .mc_print_ranked_pairs_preview(
      x$top_results,
      header = "Strongest pairs by |estimate|",
      topn = cfg$topn,
      max_vars = cfg$max_vars,
      width = cfg$width,
      digits = digits,
      ci_digits = .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits),
      show_ci = cfg$show_ci,
      ...
    )
  }
  invisible(x)
}

#' @method print summary.corr_matrix
#' @export
print.summary.corr_matrix <- function(x,
                                      digits = 4,
                                      n = NULL,
                                      topn = NULL,
                                      max_vars = NULL,
                                      width = NULL,
                                      show_ci = NULL,
                                      ...) {
  print.summary.matrixCorr(
    x,
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

print.summary_corr_matrix <- print.summary.corr_matrix

#' @title Summary Method for Latent Correlation Matrices
#'
#' @description
#' Prints compact summary statistics returned by
#' \code{summary.tetrachoric_corr()}, \code{summary.polychoric_corr()},
#' \code{summary.polyserial_corr()}, and \code{summary.biserial_corr()}.
#'
#' @param x An object of class \code{summary.latent_corr}.
#' @param digits Integer; number of decimal places to print.
#' @param ... Unused.
#'
#' @return Invisibly returns \code{x}.
#' @method print summary.latent_corr
#' @export
print.summary.latent_corr <- function(x, digits = 4, ...) {
  if (inherits(x, "summary.corr_result")) {
    x2 <- x
    attr(x2, "summary_title") <- "Latent correlation summary"
    return(print.summary.corr_result(x2, digits = digits, ...))
  }
  print.summary.matrixCorr(x, digits = digits, ...)
}

print.summary_latent_corr <- print.summary.latent_corr

expected_edge_df <- function(mat, threshold = 0, diag = TRUE) {
  idx <- upper.tri(mat, diag = diag)
  i <- row(mat)[idx]
  j <- col(mat)[idx]
  vals <- mat[idx]
  keep <- is.finite(vals) & abs(vals) >= threshold
  rn <- rownames(mat)
  cn <- colnames(mat)
  row_out <- if (is.null(rn)) as.character(i[keep]) else rn[i[keep]]
  col_out <- if (is.null(cn)) as.character(j[keep]) else cn[j[keep]]
  data.frame(
    row = row_out,
    col = col_out,
    value = as.numeric(vals[keep]),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

expected_sparse_dense <- function(mat, threshold = 0, diag = TRUE) {
  out <- matrix(0, nrow = nrow(mat), ncol = ncol(mat), dimnames = dimnames(mat))
  idx <- upper.tri(mat, diag = diag)
  i <- row(mat)[idx]
  j <- col(mat)[idx]
  vals <- mat[idx]
  keep <- is.finite(vals) & abs(vals) >= threshold
  out[cbind(i[keep], j[keep])] <- vals[keep]
  out[cbind(j[keep], i[keep])] <- vals[keep]
  out
}

test_that("output modes are consistent across dense correlation methods", {
  set.seed(20260415)
  X <- matrix(rnorm(200 * 5), nrow = 200, ncol = 5)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  methods <- list(
    pearson = function(...) pearson_corr(X, na_method = "error", ci = FALSE, ...),
    spearman = function(...) spearman_rho(X, na_method = "error", ci = FALSE, ...),
    kendall = function(...) kendall_tau(X, na_method = "error", ci = FALSE, ...),
    bicor = function(...) bicor(X, na_method = "error", ci = FALSE, ...),
    dcor = function(...) dcor(X, na_method = "error", p_value = FALSE, ...),
    pbcor = function(...) pbcor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...),
    wincor = function(...) wincor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...),
    skipped = function(...) skipped_corr(
      X,
      method = "pearson",
      na_method = "error",
      ci = FALSE,
      p_value = FALSE,
      return_masks = FALSE,
      ...
    ),
    shrinkage = function(...) shrinkage_corr(X, ...),
    ccc = function(...) ccc(X, ci = FALSE, ...)
  )

  for (nm in names(methods)) {
    fn <- methods[[nm]]

    base <- fn()
    mat <- as.matrix(base)
    expect_equal(colnames(mat), colnames(X), info = nm)
    expect_equal(rownames(mat), colnames(X), info = nm)

    explicit_matrix <- fn(output = "matrix", threshold = 0, diag = TRUE)
    expect_equal(as.matrix(explicit_matrix), mat, tolerance = 1e-12, info = nm)

    expect_error(
      fn(output = "matrix", threshold = 0.25),
      "must be 0 when",
      info = nm
    )

    expect_error(fn(output = "packed_upper"), info = nm)

    thr <- 0.30
    edge <- fn(output = "edge_list", threshold = thr, diag = FALSE)
    expected_edge <- expected_edge_df(mat, threshold = thr, diag = FALSE)
    edge_df <- as.data.frame(edge, stringsAsFactors = FALSE)[, c("row", "col", "value")]
    expect_equal(edge_df, expected_edge, tolerance = 1e-12, info = nm)
    if (nrow(edge) > 0L) {
      expect_true(all(abs(edge$value) >= thr), info = nm)
    }

    sparse <- fn(output = "sparse", threshold = thr, diag = FALSE)
    expect_s4_class(sparse, "sparseMatrix")
    expect_equal(dimnames(sparse), dimnames(mat), info = nm)
    dense_sparse <- as.matrix(sparse)
    expect_true(isSymmetric(dense_sparse), info = nm)
    expect_equal(
      dense_sparse,
      expected_sparse_dense(mat, threshold = thr, diag = FALSE),
      tolerance = 1e-12,
      info = nm
    )
  }
})

test_that("eligible methods use direct triplet payloads and match dense filtering", {
  set.seed(20260416)
  X <- matrix(rnorm(320 * 6), nrow = 320, ncol = 6)
  colnames(X) <- paste0("T", seq_len(ncol(X)))

  methods <- list(
    pearson = function(...) pearson_corr(X, na_method = "error", ci = FALSE, ...),
    spearman = function(...) spearman_rho(X, na_method = "error", ci = FALSE, ...),
    ccc = function(...) ccc(X, ci = FALSE, ...),
    bicor = function(...) bicor(X, na_method = "error", ci = FALSE, ...),
    pbcor = function(...) pbcor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...),
    wincor = function(...) wincor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...)
  )

  thr <- 0.35
  for (nm in names(methods)) {
    fn <- methods[[nm]]
    dense <- fn(output = "matrix")
    mat <- as.matrix(dense)

    edge <- fn(output = "edge_list", threshold = thr, diag = FALSE)
    expect_true(all(c("row", "col", "value") %in% names(edge)), info = nm)
    expect_false(any(c("i", "j", "x") %in% names(edge)), info = nm)
    edge_df <- .mc_corr_as_edge_df(edge)
    expect_equal(
      edge_df,
      expected_edge_df(mat, threshold = thr, diag = FALSE),
      tolerance = 1e-12,
      info = nm
    )

    sparse <- fn(output = "sparse", threshold = thr, diag = FALSE)
    expect_s4_class(sparse, "sparseMatrix")
    expect_equal(dimnames(sparse), dimnames(mat), info = nm)
    expect_true(isSymmetric(as.matrix(sparse)), info = nm)
    expect_equal(
      as.matrix(sparse),
      expected_sparse_dense(mat, threshold = thr, diag = FALSE),
      tolerance = 1e-12,
      info = nm
    )
  }
})

test_that("thresholded direct outputs are materially smaller than dense outputs", {
  set.seed(20260417)
  X <- matrix(rnorm(250 * 150), nrow = 250, ncol = 150)
  colnames(X) <- paste0("M", seq_len(ncol(X)))

  methods <- list(
    pearson = function(...) pearson_corr(X, na_method = "error", ci = FALSE, ...),
    spearman = function(...) spearman_rho(X, na_method = "error", ci = FALSE, ...),
    ccc = function(...) ccc(X, ci = FALSE, ...),
    bicor = function(...) bicor(X, na_method = "error", ci = FALSE, ...),
    pbcor = function(...) pbcor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...),
    wincor = function(...) wincor(X, na_method = "error", ci = FALSE, p_value = FALSE, ...)
  )

  thr <- 0.90
  for (nm in names(methods)) {
    fn <- methods[[nm]]
    dense <- fn(output = "matrix")
    sparse <- fn(output = "sparse", threshold = thr, diag = FALSE)
    edge <- fn(output = "edge_list", threshold = thr, diag = FALSE)

    dense_size <- as.numeric(object.size(as.matrix(dense)))
    sparse_size <- as.numeric(object.size(sparse))
    edge_size <- as.numeric(object.size(edge))
    retained_ratio <- Matrix::nnzero(sparse) / (ncol(X) * ncol(X))

    expect_lt(retained_ratio, 0.20)

    expect_lt(sparse_size, dense_size * 0.90)
    expect_lt(edge_size, dense_size * 0.90)
  }
})

test_that("pearson edge_list threshold=0 keeps edge-list payload contract", {
  set.seed(20260417)
  X <- matrix(rnorm(240 * 8), nrow = 240, ncol = 8)
  colnames(X) <- paste0("E", seq_len(ncol(X)))

  mat <- as.matrix(pearson_corr(X, na_method = "error", ci = FALSE, output = "matrix"))
  edge <- pearson_corr(X, na_method = "error", ci = FALSE, output = "edge_list", threshold = 0, diag = FALSE)

  expect_true(all(c("row", "col", "value") %in% names(edge)))
  expect_false(any(c("i", "j", "x") %in% names(edge)))

  got <- as.data.frame(edge, stringsAsFactors = FALSE)[, c("row", "col", "value")]
  expected <- expected_edge_df(mat, threshold = 0, diag = FALSE)

  got <- got[order(got$col, got$row), , drop = FALSE]
  expected <- expected[order(expected$col, expected$row), , drop = FALSE]
  rownames(got) <- NULL
  rownames(expected) <- NULL

  expect_equal(got, expected, tolerance = 1e-12)
})

test_that("pcorr supports output modes for point-estimate path", {
  set.seed(11)
  X <- matrix(rnorm(180 * 5), nrow = 180, ncol = 5)
  colnames(X) <- paste0("P", seq_len(ncol(X)))

  fit <- pcorr(X, method = "sample")
  mat <- estimate(fit)
  expect_equal(colnames(mat), colnames(X))
  expect_equal(rownames(mat), colnames(X))

  expect_error(
    pcorr(X, method = "sample", output = "matrix", threshold = 0.2),
    "must be 0 when"
  )
  expect_error(
    pcorr(X, method = "sample", output = "edge_list", return_cov_precision = TRUE,
          return_details = TRUE),
    "point estimates only"
  )

  expect_error(pcorr(X, method = "sample", output = "packed_upper"))

  edge <- pcorr(X, method = "sample", output = "edge_list", threshold = 0.25, diag = FALSE)
  edge_df <- as.data.frame(edge, stringsAsFactors = FALSE)[, c("row", "col", "value")]
  expect_equal(edge_df, expected_edge_df(mat, threshold = 0.25, diag = FALSE), tolerance = 1e-12)

  sparse <- pcorr(X, method = "sample", output = "sparse", threshold = 0.25, diag = FALSE)
  expect_s4_class(sparse, "sparseMatrix")
  expect_equal(dimnames(sparse), dimnames(mat))
  expect_equal(
    as.matrix(sparse),
    expected_sparse_dense(mat, threshold = 0.25, diag = FALSE),
    tolerance = 1e-12
  )
})

test_that("latent symmetric methods support matrix/sparse/edge outputs", {
  set.seed(4242)
  Z <- matrix(rnorm(600 * 4), nrow = 600, ncol = 4)
  X_bin <- data.frame(
    b1 = Z[, 1] > -0.2,
    b2 = Z[, 2] > 0.1,
    b3 = Z[, 3] > 0.4,
    b4 = Z[, 4] > -0.5
  )
  X_ord <- data.frame(
    o1 = ordered(cut(Z[, 1], breaks = c(-Inf, -0.4, 0.3, Inf), labels = c("L", "M", "H"))),
    o2 = ordered(cut(Z[, 2], breaks = c(-Inf, -0.8, -0.1, 0.6, Inf), labels = c("1", "2", "3", "4"))),
    o3 = ordered(cut(Z[, 3], breaks = c(-Inf, -0.2, 0.5, Inf), labels = c("A", "B", "C"))),
    o4 = ordered(cut(Z[, 4], breaks = c(-Inf, -0.3, 0.2, Inf), labels = c("Q", "R", "S")))
  )

  methods <- list(
    tetrachoric = function(...) tetrachoric(X_bin, ci = FALSE, p_value = FALSE, ...),
    polychoric = function(...) polychoric(X_ord, ci = FALSE, p_value = FALSE, ...)
  )

  for (nm in names(methods)) {
    fn <- methods[[nm]]
    base <- fn()
    mat <- as.matrix(base)

    expect_error(fn(output = "matrix", threshold = 0.2), "must be 0 when", info = nm)

    expect_error(fn(output = "packed_upper"), info = nm)

    edge <- fn(output = "edge_list", threshold = 0.25, diag = FALSE)
    edge_df <- as.data.frame(edge, stringsAsFactors = FALSE)[, c("row", "col", "value")]
    expect_equal(edge_df, expected_edge_df(mat, threshold = 0.25, diag = FALSE), tolerance = 1e-12, info = nm)

    sparse <- fn(output = "sparse", threshold = 0.25, diag = FALSE)
    expect_s4_class(sparse, "sparseMatrix")
    expect_equal(as.matrix(sparse), expected_sparse_dense(mat, threshold = 0.25, diag = FALSE), tolerance = 1e-12, info = nm)
  }
})

test_that("cohen_kappa supports matrix/sparse/edge outputs", {
  X_nom <- data.frame(
    k1 = c("A", "A", "B", "B", "C", "A", "B", "C"),
    k2 = c("A", "B", "B", "B", "C", "A", "B", "C"),
    k3 = c("A", "A", "B", "C", "C", "A", "B", "B"),
    k4 = c("A", "B", "A", "B", "C", "C", "B", "C"),
    stringsAsFactors = FALSE
  )

  base <- cohen_kappa(X_nom, na_method = "error", ci = FALSE, p_value = FALSE)
  mat <- as.matrix(base)

  expect_error(
    cohen_kappa(X_nom, output = "matrix", threshold = 0.2),
    "must be 0 when"
  )

  edge <- cohen_kappa(
    X_nom,
    na_method = "error",
    ci = FALSE,
    p_value = FALSE,
    output = "edge_list",
    threshold = 0.2,
    diag = FALSE
  )
  edge_df <- .mc_corr_as_edge_df(edge)
  edge_df <- edge_df[order(edge_df$col, edge_df$row), , drop = FALSE]
  expected <- expected_edge_df(mat, threshold = 0.2, diag = FALSE)
  expected <- expected[order(expected$col, expected$row), , drop = FALSE]
  rownames(edge_df) <- NULL
  rownames(expected) <- NULL
  expect_equal(
    edge_df,
    expected,
    tolerance = 1e-12
  )

  sparse <- cohen_kappa(
    X_nom,
    na_method = "error",
    ci = FALSE,
    p_value = FALSE,
    output = "sparse",
    threshold = 0.2,
    diag = FALSE
  )
  expect_s4_class(sparse, "sparseMatrix")
  expect_equal(
    as.matrix(sparse),
    expected_sparse_dense(mat, threshold = 0.2, diag = FALSE),
    tolerance = 1e-12
  )
})

test_that("weighted_kappa supports matrix/sparse/edge outputs", {
  lev <- c("low", "mid", "high")
  X_ord <- data.frame(
    k1 = ordered(c("low", "low", "mid", "mid", "high", "high", "low", "high"), levels = lev),
    k2 = ordered(c("low", "mid", "mid", "high", "high", "mid", "low", "high"), levels = lev),
    k3 = ordered(c("low", "low", "mid", "high", "high", "high", "low", "mid"), levels = lev),
    k4 = ordered(c("mid", "mid", "mid", "high", "high", "high", "low", "mid"), levels = lev)
  )

  base <- weighted_kappa(X_ord, na_method = "error", ci = FALSE, p_value = FALSE)
  mat <- as.matrix(base)

  expect_error(
    weighted_kappa(X_ord, output = "matrix", threshold = 0.2),
    "must be 0 when"
  )

  edge <- weighted_kappa(
    X_ord,
    na_method = "error",
    ci = FALSE,
    p_value = FALSE,
    output = "edge_list",
    threshold = 0.2,
    diag = FALSE
  )
  edge_df <- .mc_corr_as_edge_df(edge)
  edge_df <- edge_df[order(edge_df$col, edge_df$row), , drop = FALSE]
  expected <- expected_edge_df(mat, threshold = 0.2, diag = FALSE)
  expected <- expected[order(expected$col, expected$row), , drop = FALSE]
  rownames(edge_df) <- NULL
  rownames(expected) <- NULL
  expect_equal(edge_df, expected, tolerance = 1e-12)

  sparse <- weighted_kappa(
    X_ord,
    na_method = "error",
    ci = FALSE,
    p_value = FALSE,
    output = "sparse",
    threshold = 0.2,
    diag = FALSE
  )
  expect_s4_class(sparse, "sparseMatrix")
  expect_equal(
    as.matrix(sparse),
    expected_sparse_dense(mat, threshold = 0.2, diag = FALSE),
    tolerance = 1e-12
  )
})

test_that("rectangular latent methods keep legacy interface without output arguments", {
  set.seed(5252)
  Z <- matrix(rnorm(400 * 4), nrow = 400, ncol = 4)
  X <- data.frame(x1 = Z[, 1], x2 = Z[, 2])
  Y_ord <- data.frame(
    y1 = ordered(cut(Z[, 3], breaks = c(-Inf, -0.3, 0.5, Inf), labels = c("L", "M", "H"))),
    y2 = ordered(cut(Z[, 4], breaks = c(-Inf, -0.8, 0.0, 0.8, Inf), labels = c("1", "2", "3", "4")))
  )
  Y_bin <- data.frame(
    g1 = Z[, 3] > 0,
    g2 = Z[, 4] > -0.2
  )

  expect_error(polyserial(X, Y_ord, output = "sparse"), "unsupported argument\\(s\\): output")
  expect_error(biserial(X, Y_bin, output = "sparse"), "unsupported argument\\(s\\): output")
})

biserial_manual_test <- function(x, y) {
  y <- as.integer(as.logical(y))
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]

  p <- mean(y == 1L)
  q <- 1 - p
  z_p <- stats::qnorm(p)
  phi_z <- stats::dnorm(z_p)
  x1 <- mean(x[y == 1L])
  x0 <- mean(x[y == 0L])
  sx <- stats::sd(x)

  ((x1 - x0) / sx) * (p * q / phi_z)
}

# Brent optimization in the C++ latent estimators currently targets 1e-8,
# so allow for small platform/compiler drift around these regression values.
latent_reg_tol <- 1e-7

expect_equal_abs <- function(object, expected, tolerance = latent_reg_tol) {
  expect_lte(
    max(abs(as.numeric(object) - as.numeric(expected))),
    tolerance
  )
}

polyserial_reference_old <- function(x, y, check_na = TRUE) {
  if (!check_na) {
    keep <- stats::complete.cases(x, y)
    x <- x[keep]
    y <- y[keep]
  }

  if (length(x) < 2L || length(unique(y[!is.na(y)])) < 2L) {
    return(NA_real_)
  }

  x <- as.numeric(x)
  y <- as.integer(as.factor(y))
  z <- as.numeric(scale(x))
  tab <- table(y)
  n <- sum(tab)
  s <- length(tab)
  if (s < 2L || sum(tab != 0L) < 2L) {
    return(NA_real_)
  }

  cuts <- stats::qnorm(cumsum(tab) / n)[-s]
  rho <- sqrt((n - 1) / n) * stats::sd(y) * stats::cor(x, y) /
    sum(stats::dnorm(cuts))
  maxcor <- 0.9999

  if (abs(rho) > maxcor) {
    rho <- sign(rho) * maxcor
  }

  out <- suppressWarnings(
    tryCatch(
      stats::optim(
        c(rho, cuts),
        function(pars) matrixCorr:::matrixCorr_polyserial_negloglik_cpp(z, y, pars, maxcor = maxcor)
      )$par[1L],
      error = function(e) NA_real_
    )
  )

  if (is.na(out)) {
    return(NA_real_)
  }
  if (out > 1) {
    out <- maxcor
  } else if (out < -1) {
    out <- -maxcor
  }
  as.numeric(out)
}

latent_estimate_template <- function(fit) {
  if (is.null(dim(fit))) {
    return(as.numeric(fit))
  }

  est_mat <- as.matrix(fit)
  attributes(est_mat) <- attributes(est_mat)[c("dim", "dimnames")]
  est_mat
}

expect_same_shape <- function(object, template) {
  if (is.null(dim(template))) {
    expect_length(object, length(template))
    return(invisible(NULL))
  }

  expect_equal(dim(object), dim(template))
  expect_identical(dimnames(object), dimnames(template))
  invisible(NULL)
}

expect_latent_wald_payloads <- function(fit,
                                        ci_method,
                                        inference_method) {
  est <- latent_estimate_template(fit)
  inf <- attr(fit, "inference", exact = TRUE)
  ci <- attr(fit, "ci", exact = TRUE)

  if (is.list(inf)) {
    expect_identical(inf$method, inference_method)
    expect_identical(inf$alternative, "two.sided")
    expect_same_shape(inf$estimate, est)
    expect_same_shape(inf$statistic, est)
    expect_same_shape(inf$parameter, est)
    expect_same_shape(inf$p_value, est)
    expect_same_shape(inf$n_obs, est)
    expect_equal(as.numeric(inf$estimate), as.numeric(est), tolerance = 1e-12)
    expect_true(all(is.na(as.numeric(inf$parameter))))
    expect_true(all(is.na(as.numeric(inf$statistic)) | is.finite(as.numeric(inf$statistic))))
    expect_true(all(is.na(as.numeric(inf$p_value)) |
      (as.numeric(inf$p_value) >= 0 & as.numeric(inf$p_value) <= 1)))
    expect_true(all(is.na(as.numeric(inf$n_obs)) | as.numeric(inf$n_obs) >= 0))
  }

  if (is.list(ci)) {
    expect_identical(ci$ci.method, ci_method)
    expect_same_shape(ci$est, est)
    expect_same_shape(ci$lwr.ci, est)
    expect_same_shape(ci$upr.ci, est)
    expect_equal(as.numeric(ci$est), as.numeric(est), tolerance = 1e-12)
    expect_equal(attr(fit, "conf.level", exact = TRUE), ci$conf.level, tolerance = 1e-12)
    expect_true(all(is.na(as.numeric(ci$lwr.ci)) |
      (as.numeric(ci$lwr.ci) >= -1 & as.numeric(ci$lwr.ci) <= 1)))
    expect_true(all(is.na(as.numeric(ci$upr.ci)) |
      (as.numeric(ci$upr.ci) >= -1 & as.numeric(ci$upr.ci) <= 1)))
    expect_true(all(
      is.na(as.numeric(ci$lwr.ci)) |
      is.na(as.numeric(ci$upr.ci)) |
      as.numeric(ci$lwr.ci) <= as.numeric(ci$upr.ci)
    ))
    keep <- is.finite(as.numeric(est)) &
      is.finite(as.numeric(ci$lwr.ci)) &
      is.finite(as.numeric(ci$upr.ci))
    expect_true(all(
      as.numeric(est)[keep] >= as.numeric(ci$lwr.ci)[keep] &
        as.numeric(est)[keep] <= as.numeric(ci$upr.ci)[keep]
    ))
  }
}

test_that("tetrachoric pair and matrix modes match fixed regression values", {
  set.seed(1001)
  x <- sample(c(FALSE, TRUE), 300, replace = TRUE)
  y <- sample(c(FALSE, TRUE), 300, replace = TRUE)

  est_pair <- tetrachoric(x, y)
  expect_equal_abs(as.numeric(est_pair), -0.272117497540896)
  expect_true(is.list(attr(est_pair, "diagnostics")))
  expect_true(is.list(attr(est_pair, "thresholds")))

  dat <- data.frame(
    a = x,
    b = y,
    c = sample(c(FALSE, TRUE), 300, replace = TRUE)
  )
  est_mat <- tetrachoric(dat)
  ref_mat <- matrix(
    c(
      1, -0.272117497540896, 0.0624107955993122,
      -0.272117497540896, 1, 0.0348797202693802,
      0.0624107955993122, 0.0348797202693802, 1
    ),
    nrow = 3,
    byrow = TRUE,
    dimnames = list(names(dat), names(dat))
  )

  expect_s3_class(est_mat, "tetrachoric_corr")
  expect_equal_abs(as.numeric(est_mat), as.numeric(ref_mat))
  expect_equal_abs(as.numeric(est_mat["a", "b"]), as.numeric(est_pair))
})

test_that("polychoric pair and matrix modes match fixed regression values", {
  set.seed(1002)
  x <- ordered(sample(1:4, 400, replace = TRUE))
  y <- ordered(sample(1:5, 400, replace = TRUE))

  est_pair <- polychoric(x, y)
  expect_equal_abs(as.numeric(est_pair), 0.0034270943899109)

  dat <- data.frame(
    x = x,
    y = y,
    z = ordered(sample(1:3, 400, replace = TRUE))
  )
  est_mat <- polychoric(dat)
  ref_mat <- matrix(
    c(
      1, 0.0034270943899109, 0.00263097305519399,
      0.0034270943899109, 1, 0.0231228372629091,
      0.00263097305519399, 0.0231228372629091, 1
    ),
    nrow = 3,
    byrow = TRUE,
    dimnames = list(names(dat), names(dat))
  )

  expect_s3_class(est_mat, "polychoric_corr")
  expect_equal_abs(as.numeric(est_mat), as.numeric(ref_mat))
  expect_equal_abs(as.numeric(est_mat["x", "y"]), as.numeric(est_pair))
})

test_that("polyserial pair and matrix modes match fixed regression values", {
  set.seed(1003)
  x <- rnorm(350)
  y <- ordered(sample(1:4, 350, replace = TRUE))

  est_pair <- polyserial(x, y)
  expect_equal(as.numeric(est_pair), 0.0418913522648746, tolerance = 1e-12)

  X <- data.frame(x1 = rnorm(350), x2 = rnorm(350))
  Y <- data.frame(
    y1 = y,
    y2 = ordered(sample(1:5, 350, replace = TRUE))
  )
  est_mat <- polyserial(X, Y)
  expect_s3_class(est_mat, "polyserial_corr")
  expect_equal(
    as.numeric(est_mat),
    c(
      -0.0127484729795922, -0.0159091216414217,
      0.0706339051579041, -0.0149611262803363
    ),
    tolerance = 1e-12
  )
})

test_that("polyserial matches the previous R optim reference path", {
  set.seed(2468)
  x <- rnorm(240)
  y <- ordered(sample(c(1, 2, 4), 240, replace = TRUE))

  expect_equal(
    polyserial(x, y),
    polyserial_reference_old(x, y),
    tolerance = 1e-12
  )

  x[c(5, 19)] <- NA_real_
  y[c(11, 19)] <- NA
  expect_equal(
    polyserial(x, y, na_method = "pairwise"),
    polyserial_reference_old(x, y, check_na = FALSE),
    tolerance = 1e-12
  )

  X <- data.frame(x1 = rnorm(240), x2 = rnorm(240))
  Y <- data.frame(
    y1 = ordered(sample(1:4, 240, replace = TRUE)),
    y2 = ordered(sample(c(1, 3, 4), 240, replace = TRUE))
  )
  ref_mat <- outer(
    seq_len(ncol(X)),
    seq_len(ncol(Y)),
    Vectorize(function(j, k) polyserial_reference_old(X[[j]], Y[[k]]))
  )
  dimnames(ref_mat) <- list(names(X), names(Y))

  expect_equal(
    as.numeric(polyserial(X, Y)),
    as.numeric(ref_mat),
    tolerance = 1e-12
  )
})

test_that("tetrachoric Wald inference preserves the established estimates", {
  set.seed(1101)
  x <- sample(c(FALSE, TRUE), 320, replace = TRUE)
  y <- sample(c(FALSE, TRUE), 320, replace = TRUE)
  dat <- data.frame(
    a = x,
    b = y,
    c = sample(c(FALSE, TRUE), 320, replace = TRUE)
  )

  fit_pair <- tetrachoric(x, y, ci = TRUE, p_value = TRUE)
  fit_mat <- tetrachoric(dat, ci = TRUE, p_value = TRUE)

  expect_equal_abs(as.numeric(fit_pair), as.numeric(tetrachoric(x, y)))
  expect_equal_abs(as.numeric(fit_mat), as.numeric(tetrachoric(dat)))
  expect_latent_wald_payloads(
    fit_pair,
    ci_method = "wald_information_tetrachoric",
    inference_method = "wald_z_tetrachoric"
  )
  expect_latent_wald_payloads(
    fit_mat,
    ci_method = "wald_information_tetrachoric",
    inference_method = "wald_z_tetrachoric"
  )
})

test_that("polychoric Wald inference preserves the established estimates", {
  set.seed(1102)
  x <- ordered(sample(1:4, 360, replace = TRUE))
  y <- ordered(sample(1:5, 360, replace = TRUE))
  dat <- data.frame(
    x = x,
    y = y,
    z = ordered(sample(1:3, 360, replace = TRUE))
  )

  fit_pair <- polychoric(x, y, ci = TRUE, p_value = TRUE)
  fit_mat <- polychoric(dat, ci = TRUE, p_value = TRUE)

  expect_equal_abs(as.numeric(fit_pair), as.numeric(polychoric(x, y)))
  expect_equal_abs(as.numeric(fit_mat), as.numeric(polychoric(dat)))
  expect_latent_wald_payloads(
    fit_pair,
    ci_method = "wald_information_polychoric",
    inference_method = "wald_z_polychoric"
  )
  expect_latent_wald_payloads(
    fit_mat,
    ci_method = "wald_information_polychoric",
    inference_method = "wald_z_polychoric"
  )
})

test_that("polyserial Wald inference preserves the established estimates", {
  set.seed(1103)
  X <- data.frame(x1 = rnorm(280), x2 = rnorm(280))
  Y <- data.frame(
    y1 = ordered(sample(1:4, 280, replace = TRUE)),
    y2 = ordered(sample(c(1, 3, 4), 280, replace = TRUE))
  )

  fit_pair <- polyserial(X$x1, Y$y1, ci = TRUE, p_value = TRUE)
  fit_mat <- polyserial(X, Y, ci = TRUE, p_value = TRUE)

  expect_equal_abs(as.numeric(fit_pair), as.numeric(polyserial(X$x1, Y$y1)))
  expect_equal_abs(as.numeric(fit_mat), as.numeric(polyserial(X, Y)))
  expect_latent_wald_payloads(
    fit_pair,
    ci_method = "wald_information_polyserial",
    inference_method = "wald_z_polyserial"
  )
  expect_latent_wald_payloads(
    fit_mat,
    ci_method = "wald_information_polyserial",
    inference_method = "wald_z_polyserial"
  )
})

test_that("latent inference payloads are attached only when requested", {
  set.seed(1104)
  x_bin <- sample(c(FALSE, TRUE), 180, replace = TRUE)
  y_bin <- sample(c(FALSE, TRUE), 180, replace = TRUE)
  x_ord <- ordered(sample(1:4, 180, replace = TRUE))
  y_ord <- ordered(sample(1:5, 180, replace = TRUE))
  X <- data.frame(x1 = rnorm(180), x2 = rnorm(180))
  Y <- data.frame(y1 = x_ord, y2 = y_ord)

  tetra_0 <- tetrachoric(data.frame(a = x_bin, b = y_bin), ci = FALSE, p_value = FALSE)
  tetra_ci <- tetrachoric(data.frame(a = x_bin, b = y_bin), ci = TRUE, p_value = FALSE)
  tetra_p <- tetrachoric(data.frame(a = x_bin, b = y_bin), ci = FALSE, p_value = TRUE)
  tetra_both <- tetrachoric(data.frame(a = x_bin, b = y_bin), ci = TRUE, p_value = TRUE)

  poly_0 <- polychoric(data.frame(x = x_ord, y = y_ord), ci = FALSE, p_value = FALSE)
  poly_ci <- polychoric(data.frame(x = x_ord, y = y_ord), ci = TRUE, p_value = FALSE)
  poly_p <- polychoric(data.frame(x = x_ord, y = y_ord), ci = FALSE, p_value = TRUE)
  poly_both <- polychoric(data.frame(x = x_ord, y = y_ord), ci = TRUE, p_value = TRUE)

  ps_0 <- polyserial(X, Y, ci = FALSE, p_value = FALSE)
  ps_ci <- polyserial(X, Y, ci = TRUE, p_value = FALSE)
  ps_p <- polyserial(X, Y, ci = FALSE, p_value = TRUE)
  ps_both <- polyserial(X, Y, ci = TRUE, p_value = TRUE)

  for (fit in list(tetra_0, poly_0, ps_0)) {
    expect_null(attr(fit, "ci", exact = TRUE))
    expect_null(attr(fit, "inference", exact = TRUE))
  }

  for (fit in list(tetra_ci, poly_ci, ps_ci)) {
    expect_true(is.list(attr(fit, "ci", exact = TRUE)))
    expect_null(attr(fit, "inference", exact = TRUE))
  }

  for (fit in list(tetra_p, poly_p, ps_p)) {
    expect_true(is.list(attr(fit, "inference", exact = TRUE)))
    expect_null(attr(fit, "ci", exact = TRUE))
  }

  for (fit in list(tetra_both, poly_both, ps_both)) {
    expect_true(is.list(attr(fit, "ci", exact = TRUE)))
    expect_true(is.list(attr(fit, "inference", exact = TRUE)))
  }
})

test_that("latent core estimates are unchanged when CI and p-values are disabled", {
  set.seed(1105)
  dat_bin <- data.frame(
    a = sample(c(FALSE, TRUE), 160, replace = TRUE),
    b = sample(c(FALSE, TRUE), 160, replace = TRUE),
    c = sample(c(FALSE, TRUE), 160, replace = TRUE)
  )
  dat_ord <- data.frame(
    x = ordered(sample(1:4, 160, replace = TRUE)),
    y = ordered(sample(1:5, 160, replace = TRUE)),
    z = ordered(sample(1:3, 160, replace = TRUE))
  )
  X <- data.frame(x1 = rnorm(160), x2 = rnorm(160))
  Y <- data.frame(
    y1 = ordered(sample(1:4, 160, replace = TRUE)),
    y2 = ordered(sample(c(1, 3, 4), 160, replace = TRUE))
  )

  fit_t_base <- tetrachoric(dat_bin)
  fit_t_false <- tetrachoric(dat_bin, ci = FALSE, p_value = FALSE)
  fit_p_base <- polychoric(dat_ord)
  fit_p_false <- polychoric(dat_ord, ci = FALSE, p_value = FALSE)
  fit_ps_base <- polyserial(X, Y)
  fit_ps_false <- polyserial(X, Y, ci = FALSE, p_value = FALSE)

  expect_equal(unclass(fit_t_base), unclass(fit_t_false), tolerance = 1e-12)
  expect_equal(unclass(fit_p_base), unclass(fit_p_false), tolerance = 1e-12)
  expect_equal(unclass(fit_ps_base), unclass(fit_ps_false), tolerance = 1e-12)
  expect_identical(attributes(fit_t_base), attributes(fit_t_false))
  expect_identical(attributes(fit_p_base), attributes(fit_p_false))
  expect_identical(attributes(fit_ps_base), attributes(fit_ps_false))
})

test_that("latent inference summaries switch to the pairwise view only when requested", {
  set.seed(1106)
  dat_bin <- data.frame(
    a = sample(c(FALSE, TRUE), 150, replace = TRUE),
    b = sample(c(FALSE, TRUE), 150, replace = TRUE),
    c = sample(c(FALSE, TRUE), 150, replace = TRUE)
  )
  dat_ord <- data.frame(
    x = ordered(sample(1:4, 150, replace = TRUE)),
    y = ordered(sample(1:5, 150, replace = TRUE)),
    z = ordered(sample(1:3, 150, replace = TRUE))
  )
  X <- data.frame(x1 = rnorm(150), x2 = rnorm(150))
  Y <- data.frame(
    y1 = ordered(sample(1:4, 150, replace = TRUE)),
    y2 = ordered(sample(c(1, 3, 4), 150, replace = TRUE))
  )

  fits0 <- list(
    tetrachoric(dat_bin),
    polychoric(dat_ord),
    polyserial(X, Y)
  )
  fits1 <- list(
    tetrachoric(dat_bin, ci = TRUE, p_value = TRUE),
    polychoric(dat_ord, ci = TRUE, p_value = TRUE),
    polyserial(X, Y, ci = TRUE, p_value = TRUE)
  )
  summary_classes <- c("summary.tetrachoric_corr", "summary.polychoric_corr", "summary.polyserial_corr")
  summary_titles <- c("Tetrachoric correlation summary", "Polychoric correlation summary", "Polyserial correlation summary")

  for (fit in fits0) {
    expect_s3_class(summary(fit), "summary.latent_corr")
    expect_s3_class(summary(fit), "summary.matrixCorr")
  }

  for (i in seq_along(fits1)) {
    sm <- summary(fits1[[i]])
    expect_s3_class(sm, summary_classes[[i]])
    expect_true(all(c("item1", "item2", "estimate", "lwr", "upr", "statistic", "df", "p_value", "n_complete") %in% names(sm)))
    out <- capture.output(print(sm, digits = 3))
    expect_true(any(grepl(summary_titles[[i]], out, fixed = TRUE)))
  }
})

test_that("biserial matches the closed-form estimator in pair and matrix mode", {
  set.seed(1004)
  x <- rnorm(320)
  y <- sample(c(FALSE, TRUE), 320, replace = TRUE)

  est_pair <- biserial(x, y)
  expect_equal(est_pair, biserial_manual_test(x, y), tolerance = 1e-12)

  X <- data.frame(x1 = rnorm(320), x2 = rnorm(320))
  Y <- data.frame(
    b1 = y,
    b2 = sample(c(FALSE, TRUE), 320, replace = TRUE)
  )
  est_mat <- biserial(X, Y)
  ref_mat <- outer(
    seq_len(ncol(X)),
    seq_len(ncol(Y)),
    Vectorize(function(j, k) biserial_manual_test(X[[j]], Y[[k]]))
  )
  dimnames(ref_mat) <- list(names(X), names(Y))

  expect_s3_class(est_mat, "biserial_corr")
  expect_equal(as.numeric(est_mat), as.numeric(ref_mat), tolerance = 1e-12)
})

test_that("biserial large-sample inference follows the established reference behaviour", {
  set.seed(160)
  n <- 180
  z1 <- rnorm(n)
  z2 <- rnorm(n)
  X <- data.frame(
    bx1 = 0.7 * z1 + rnorm(n, sd = 0.6),
    bx2 = -0.5 * z2 + 0.2 * z1 + rnorm(n, sd = 0.7)
  )
  Y <- data.frame(
    g1 = z1 > stats::qnorm(0.58),
    g2 = (0.4 * z1 + 0.8 * z2 + rnorm(n, sd = 0.5)) > stats::qnorm(0.52)
  )

  fit <- biserial(X, Y, ci = TRUE, p_value = TRUE)
  inf <- attr(fit, "inference", exact = TRUE)
  ci <- attr(fit, "ci", exact = TRUE)
  est_mat <- fit
  attributes(est_mat) <- attributes(est_mat)[c("dim", "dimnames")]

  ref_estimate <- structure(
    c(0.792193280766738, 0.103575171678435, 0.260351318125141, -0.330748623730774),
    dim = c(2L, 2L),
    dimnames = list(names(X), names(Y))
  )
  ref_statistic <- structure(
    c(17.3188455939721, 1.38933751928313, 3.59758636169522, -4.67590196928005),
    dim = c(2L, 2L),
    dimnames = list(names(X), names(Y))
  )
  ref_p_value <- structure(
    c(4.99830673160238e-40, 0.166465648114298, 0.000416008709530862, 5.76492513948137e-06),
    dim = c(2L, 2L),
    dimnames = list(names(X), names(Y))
  )
  ref_lwr <- structure(
    c(0.730581588076149, -0.0433447862124336, 0.118604440134517, -0.455000651378134),
    dim = c(2L, 2L),
    dimnames = list(names(X), names(Y))
  )
  ref_upr <- structure(
    c(0.841009788517119, 0.246110105580895, 0.391698804407742, -0.193863734545158),
    dim = c(2L, 2L),
    dimnames = list(names(X), names(Y))
  )
  ref_n_obs <- structure(rep(180L, 4L), dim = c(2L, 2L), dimnames = list(names(X), names(Y)))

  expect_equal(est_mat, ref_estimate, tolerance = 1e-12)
  expect_equal(inf$statistic, ref_statistic, tolerance = 1e-12)
  expect_equal(inf$p_value, ref_p_value, tolerance = 1e-12)
  expect_equal(ci$lwr.ci, ref_lwr, tolerance = 1e-12)
  expect_equal(ci$upr.ci, ref_upr, tolerance = 1e-12)
  expect_equal(inf$n_obs, ref_n_obs, tolerance = 0)
  expect_equal(attr(fit, "diagnostics", exact = TRUE)$n_complete, ref_n_obs, tolerance = 0)
})

test_that("biserial returns CI and p-value payloads only when requested", {
  set.seed(1007)
  x <- rnorm(150)
  y <- x + rnorm(150, sd = 0.7)
  g <- y > stats::qnorm(0.55)
  X <- data.frame(x1 = x, x2 = rnorm(150))
  Y <- data.frame(g1 = g, g2 = sample(c(FALSE, TRUE), 150, replace = TRUE))

  fit0 <- biserial(X, Y, ci = FALSE, p_value = FALSE)
  fit_ci <- biserial(X, Y, ci = TRUE, p_value = FALSE)
  fit_p <- biserial(X, Y, ci = FALSE, p_value = TRUE)
  fit_both <- biserial(X, Y, ci = TRUE, p_value = TRUE)

  expect_null(attr(fit0, "ci", exact = TRUE))
  expect_null(attr(fit0, "inference", exact = TRUE))

  expect_true(is.list(attr(fit_ci, "ci", exact = TRUE)))
  expect_null(attr(fit_ci, "inference", exact = TRUE))

  expect_true(is.list(attr(fit_p, "inference", exact = TRUE)))
  expect_null(attr(fit_p, "ci", exact = TRUE))

  expect_true(is.list(attr(fit_both, "ci", exact = TRUE)))
  expect_true(is.list(attr(fit_both, "inference", exact = TRUE)))
})

test_that("biserial confidence intervals are ordered and bounded", {
  set.seed(1008)
  z <- rnorm(170)
  X <- data.frame(
    x1 = 0.8 * z + rnorm(170, sd = 0.5),
    x2 = rnorm(170)
  )
  Y <- data.frame(
    g1 = z > stats::qnorm(0.6),
    g2 = (0.5 * z + rnorm(170, sd = 0.8)) > stats::qnorm(0.45)
  )

  fit <- biserial(X, Y, ci = TRUE)
  ci <- attr(fit, "ci", exact = TRUE)

  expect_true(all(ci$lwr.ci <= ci$upr.ci, na.rm = TRUE))
  expect_true(all(ci$lwr.ci >= -1, na.rm = TRUE))
  expect_true(all(ci$upr.ci <= 1, na.rm = TRUE))
  expect_true(all(unclass(fit) >= ci$lwr.ci & unclass(fit) <= ci$upr.ci, na.rm = TRUE))
})

test_that("biserial core estimate is unchanged when CI and p-values are disabled", {
  set.seed(1009)
  X <- data.frame(
    x1 = rnorm(140),
    x2 = rnorm(140)
  )
  Y <- data.frame(
    g1 = sample(c(FALSE, TRUE), 140, replace = TRUE),
    g2 = sample(c(FALSE, TRUE), 140, replace = TRUE)
  )

  fit_base <- biserial(X, Y)
  fit_flags_false <- biserial(X, Y, ci = FALSE, p_value = FALSE)

  expect_equal(unclass(fit_base), unclass(fit_flags_false), tolerance = 1e-12)
  expect_equal(attributes(fit_base)[c("dim", "dimnames", "class", "method", "description", "package", "diagnostics", "thresholds", "correct")],
               attributes(fit_flags_false)[c("dim", "dimnames", "class", "method", "description", "package", "diagnostics", "thresholds", "correct")])
})

test_that("biserial summary switches to pairwise inference view when requested", {
  set.seed(1010)
  z <- rnorm(160)
  X <- data.frame(
    x1 = 0.6 * z + rnorm(160, sd = 0.6),
    x2 = rnorm(160)
  )
  Y <- data.frame(
    g1 = z > stats::qnorm(0.57),
    g2 = sample(c(FALSE, TRUE), 160, replace = TRUE)
  )

  fit0 <- biserial(X, Y)
  fit1 <- biserial(X, Y, ci = TRUE, p_value = TRUE)

  sm0 <- summary(fit0)
  sm1 <- summary(fit1)

  expect_s3_class(sm0, "summary.latent_corr")
  expect_s3_class(sm0, "summary.matrixCorr")
  expect_s3_class(sm1, "summary.biserial_corr")
  expect_true(all(c("item1", "item2", "estimate", "lwr", "upr", "statistic", "df", "p_value", "n_complete") %in% names(sm1)))

  out <- capture.output(print(sm1, digits = 3))
  expect_true(any(grepl("Biserial correlation summary", out, fixed = TRUE)))
})

test_that("latent correlation methods support pairwise complete cases", {
  x_bin <- c(FALSE, TRUE, NA, FALSE, TRUE, FALSE)
  y_bin <- c(TRUE, FALSE, TRUE, NA, FALSE, TRUE)
  expect_false(is.na(tetrachoric(x_bin, y_bin, na_method = "pairwise")))

  x_cont <- c(0.1, -0.4, NA, 0.7, 1.2, -0.5)
  y_ord <- ordered(c(1, 2, 3, NA, 2, 1))
  expect_false(is.na(polyserial(x_cont, y_ord, na_method = "pairwise")))
  expect_false(is.na(biserial(x_cont, x_bin, na_method = "pairwise")))
})

test_that("latent correlation print, plot, and summary methods work", {
  skip_if_not_installed("ggplot2")

  set.seed(1005)
  dat <- data.frame(
    a = sample(c(FALSE, TRUE), 120, replace = TRUE),
    b = sample(c(FALSE, TRUE), 120, replace = TRUE),
    c = sample(c(FALSE, TRUE), 120, replace = TRUE)
  )
  tc <- tetrachoric(dat)

  out <- capture.output(print(tc, digits = 3, max_rows = 2, max_cols = 2))
  expect_true(any(grepl("omitted", out)))

  p <- plot(tc, title = "Tetra plot", low_color = "blue", high_color = "red",
            mid_color = "white", value_text_size = 3)
  expect_s3_class(p, "ggplot")
  scale <- p$scales$get_scales("fill")
  expect_equal(scale$limits, c(-1, 1))

  sm <- summary(tc)
  expect_s3_class(sm, "summary.latent_corr")
  expect_s3_class(sm, "summary.matrixCorr")
  sum_out <- capture.output(print(sm, digits = 3))
  expect_true(any(grepl("Latent correlation summary", sum_out)))
  expect_true(any(grepl("thresholds", sum_out)))
  expect_false(any(grepl("zero_cells", sum_out)))
})

test_that("latent summaries keep all pairs for square but non-symmetric outputs", {
  set.seed(1006)
  X <- data.frame(x1 = rnorm(200), x2 = rnorm(200))
  Y <- data.frame(
    g1 = sample(c(FALSE, TRUE), 200, replace = TRUE),
    g2 = sample(c(FALSE, TRUE), 200, replace = TRUE)
  )

  bs <- biserial(X, Y)
  expect_false(isSymmetric(unclass(as.matrix(bs))))

  sm <- summary(bs)

  expect_s3_class(sm, "summary.latent_corr")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_identical(sm$n_pairs, 4L)
  expect_s3_class(sm$top_results, "data.frame")
  expect_identical(nrow(sm$top_results), 4L)

  out <- capture.output(print(sm, digits = 3))
  expect_true(any(grepl("x1", out, fixed = TRUE)))
  expect_true(any(grepl("x2", out, fixed = TRUE)))
  expect_true(any(grepl("g1", out, fixed = TRUE)))
  expect_true(any(grepl("g2", out, fixed = TRUE)))
})

test_that("regular table-input latent fits match known regression values", {
  tab_2x2 <- as.table(matrix(c(61661, 1610, 85, 20), 2, 2))
  tab_3x3 <- as.table(matrix(c(13, 69, 41, 6, 113, 132, 0, 22, 104), 3, 3))

  expect_equal_abs(as.numeric(tetrachoric(tab_2x2, correct = 0.5)), 0.348073131513184)
  expect_equal_abs(as.numeric(polychoric(tab_2x2, correct = 0.5)), 0.348073131513616)

  pc_3x3 <- polychoric(tab_3x3, correct = 0)
  expect_equal_abs(as.numeric(pc_3x3), 0.491371943011718)
  thr <- attr(pc_3x3, "thresholds")
  expect_equal(unname(thr$row), c(-1.77438191034496, -0.135773931302112), tolerance = 1e-12)
  expect_equal(unname(thr$col), c(-0.687131286795469, 0.668209299725723), tolerance = 1e-12)
})

test_that("table-input latent fits attach optional inference payloads", {
  tab_2x2 <- as.table(matrix(c(61661, 1610, 85, 20), 2, 2))
  tab_3x3 <- as.table(matrix(c(13, 69, 41, 6, 113, 132, 0, 22, 104), 3, 3))

  tc <- tetrachoric(tab_2x2, ci = TRUE, p_value = TRUE)
  pc <- polychoric(tab_3x3, correct = 0, ci = TRUE, p_value = TRUE)

  expect_true(is.list(attr(tc, "ci", exact = TRUE)))
  expect_true(is.list(attr(tc, "inference", exact = TRUE)))
  expect_true(is.list(attr(pc, "ci", exact = TRUE)))
  expect_true(is.list(attr(pc, "inference", exact = TRUE)))
  expect_equal(as.numeric(tc), as.numeric(tetrachoric(tab_2x2, correct = 0.5)), tolerance = 1e-12)
  expect_equal(as.numeric(pc), as.numeric(polychoric(tab_3x3, correct = 0)), tolerance = 1e-12)
  expect_latent_wald_payloads(
    tc,
    ci_method = "wald_information_tetrachoric",
    inference_method = "wald_z_tetrachoric"
  )
  expect_latent_wald_payloads(
    pc,
    ci_method = "wald_information_polychoric",
    inference_method = "wald_z_polychoric"
  )
})

test_that("sparse zero-cell latent fits expose diagnostics and thresholds", {
  sparse_2x2 <- as.table(matrix(c(44268, 193, 14, 0), 2, 2))

  tc0 <- tetrachoric(sparse_2x2, correct = 0)
  pc0 <- polychoric(sparse_2x2, correct = 0)
  tc5 <- tetrachoric(sparse_2x2, correct = 0.5)

  expect_true(is.list(attr(tc0, "diagnostics")))
  expect_true(is.list(attr(pc0, "diagnostics")))
  expect_true(is.list(attr(tc0, "thresholds")))
  expect_true(is.list(attr(pc0, "thresholds")))

  expect_true(isTRUE(attr(tc0, "diagnostics")$boundary))
  expect_true(isTRUE(attr(pc0, "diagnostics")$boundary))
  expect_true(isTRUE(attr(tc5, "diagnostics")$corrected))
  expect_true(attr(tc0, "diagnostics")$zero_cells > 0L)
  expect_true(attr(pc0, "diagnostics")$zero_cells > 0L)
  expect_equal(attr(tc0, "diagnostics")$n_complete, sum(sparse_2x2))
  expect_equal(attr(pc0, "diagnostics")$n_complete, sum(sparse_2x2))

  expand_table_pairs <- function(tab) {
    tab <- as.matrix(tab)
    x <- integer(sum(tab))
    y <- integer(sum(tab))
    pos <- 1L
    for (i in seq_len(nrow(tab))) {
      for (j in seq_len(ncol(tab))) {
        nij <- tab[i, j]
        if (nij > 0L) {
          idx <- pos:(pos + nij - 1L)
          x[idx] <- i
          y[idx] <- j
          pos <- pos + nij
        }
      }
    }
    data.frame(
      a = ordered(x),
      b = ordered(y)
    )
  }
  sparse_df <- expand_table_pairs(sparse_2x2)
  pc_mat <- polychoric(sparse_df, na_method = "error")
  pc_mat0 <- polychoric(sparse_df, correct = 0, na_method = "error")
  diag_pc <- attr(pc_mat, "diagnostics")
  diag_pc0 <- attr(pc_mat0, "diagnostics")
  expect_true(is.list(diag_pc))
  expect_true(is.matrix(diag_pc$boundary))
  expect_true(is.matrix(diag_pc$zero_cells))
  expect_true(isTRUE(diag_pc$corrected[1, 2]))
  expect_true(isTRUE(diag_pc0$boundary[1, 2]))
  expect_true(is.list(attr(pc_mat, "thresholds")))
  expect_true(is.numeric(attr(pc_mat, "correct")))
})

test_that("matrix-style correlation summaries use the standard compact format", {
  set.seed(123)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  objs <- list(
    pearson_corr(X),
    dcor(X),
    skipped_corr(X)
  )

  for (obj in objs) {
    sm <- summary(obj)
    expect_s3_class(sm, "summary.matrixCorr")
    expect_s3_class(sm, "summary.corr_matrix")
    expect_identical(sm$n_rows, nrow(obj))
    expect_identical(sm$n_cols, ncol(obj))
    expect_true(isTRUE(sm$symmetric))

    txt <- capture.output(matrixCorr:::print.summary.matrixCorr(sm))
    expect_true(any(grepl("^Correlation summary$", txt)))
    expect_true(any(grepl("pairs", txt, fixed = TRUE)))
    expect_true(any(grepl("estimate", txt, fixed = TRUE)))
    expect_false(any(grepl("Strongest pairs by \\|estimate\\|", txt)))
  }
})

test_that("partial correlation summary follows the same matrix-style contract", {
  set.seed(456)
  X <- matrix(rnorm(160), nrow = 40, ncol = 4)
  colnames(X) <- paste0("P", seq_len(ncol(X)))

  pc <- pcorr(X, method = "ridge", lambda = 1e-2)
  sm <- summary(pc)

  expect_s3_class(sm, "summary.corr_result")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "summary.corr_matrix")
  expect_s3_class(sm, "summary.partial_corr_matrix")
  expect_identical(sm$class, "partial_corr_matrix")
  expect_identical(sm$method, "partial_correlation_ridge")
  expect_identical(sm$n_rows, 4L)
  expect_identical(sm$n_cols, 4L)
  expect_equal(attr(pc, "lambda", exact = TRUE), 1e-2)

  txt <- capture.output(print(sm))
  expect_true(any(grepl("^Partial correlation summary$", txt)))
  expect_true(any(grepl("dimensions", txt, fixed = TRUE)))
})

test_that("latent summaries retain the latent header", {
  set.seed(789)
  X <- data.frame(
    x1 = rnorm(50),
    x2 = rnorm(50)
  )
  Y <- data.frame(
    y1 = ordered(sample(1:3, 50, TRUE)),
    y2 = ordered(sample(1:4, 50, TRUE))
  )

  ps <- polyserial(X, Y)
  sm <- summary(ps)

  expect_s3_class(sm, "summary.latent_corr")
  expect_s3_class(sm, "summary.matrixCorr")
  expect_s3_class(sm, "summary.corr_matrix")
  txt <- capture.output(matrixCorr:::print.summary.latent_corr(sm))
  expect_true(any(grepl("^Latent correlation summary$", txt)))
})

#' Deprecated Compatibility Wrappers
#'
#' Temporary wrappers for functions renamed in `matrixCorr` 1.0.0. These
#' wrappers preserve the pre-1.0 entry points while warning that they will be
#' removed in 2.0.0.
#'
#' @param group1,group2 Numeric vectors of equal length.
#' @param two Positive scalar; the multiple of the standard deviation used to
#'   define the limits of agreement.
#' @param mode Integer; 1 uses `group1 - group2`, 2 uses `group2 - group1`.
#' @param conf_level Confidence level.
#' @param verbose Logical; print brief progress or diagnostic output.
#' @param data A `data.frame`, matrix, or repeated-measures dataset accepted by
#'   the corresponding replacement function.
#' @param response Numeric response vector or column name, depending on the
#'   target method.
#' @param subject Subject identifier or subject column name.
#' @param method Method label or method column name.
#' @param time Replicate/time index or time column name.
#' @param include_slope Logical; whether to estimate proportional bias.
#' @param use_ar1 Logical; whether to use AR(1) within-subject correlation.
#' @param ar1_rho AR(1) parameter.
#' @param max_iter,tol EM control parameters.
#' @param c_const Positive numeric Tukey biweight tuning constant.
#' @param max_p_outliers Numeric in `(0, 1]`; optional cap on the maximum
#'   proportion of outliers on each side.
#' @param pearson_fallback Character fallback policy used by `bicor()`.
#' @param na_method Missing-data policy used by `bicor()`.
#' @param mad_consistent Logical; if `TRUE`, uses the consistency-corrected MAD.
#' @param w Optional vector of case weights.
#' @param sparse_threshold Optional threshold controlling sparse output.
#' @param output Output representation for the computed estimates.
#' @param threshold Non-negative absolute-value filter for non-matrix outputs.
#' @param diag Logical; whether to include diagonal entries in non-matrix outputs.
#' @param n_threads Integer number of OpenMP threads.
#' @param rind Character; column identifying subjects, forwarded as `subject`
#'   to `ccc_rm_reml()`.
#' @param interaction Logical; forwarded to `ccc_rm_reml()`.
#' @param Dmat Optional distance matrix forwarded to `ccc_rm_reml()`.
#' @param Dmat_type Character selector controlling how `Dmat` is constructed.
#' @param Dmat_weights Optional weights used when `Dmat_type` requires them.
#' @param Dmat_rescale Logical; whether to rescale `Dmat`.
#' @param ci_mode Character selector for the confidence-interval scale used by
#'   `ccc_rm_reml()`.
#' @param digits Display precision forwarded to `ccc_rm_reml()`.
#' @param use_message Logical; whether the deprecated wrapper emits a lifecycle
#'   message.
#' @param ar Character selector for the within-subject residual correlation
#'   model.
#' @param ar_rho Numeric AR(1) parameter.
#' @param slope Character selector for the proportional-bias slope structure.
#' @param slope_var Optional covariance matrix for custom slopes.
#' @param slope_Z Optional design matrix for custom slopes.
#' @param drop_zero_cols Logical; whether zero-variance design columns are
#'   dropped.
#' @param vc_select Character selector controlling variance-component
#'   selection.
#' @param vc_alpha Significance level used in variance-component selection.
#' @param vc_test_order Character vector controlling the variance-component
#'   test order.
#' @param include_subj_method Optional logical override for the
#'   subject-by-method component.
#' @param include_subj_time Optional logical override for the subject-by-time
#'   component.
#' @param sb_zero_tol Numerical tolerance used when stabilising the scale-bias
#'   term.
#' @param delta Numeric power exponent for U-statistics distances.
#' @param lambda Numeric regularisation strength used by `pcorr()`.
#' @param return_cov_precision Logical; if `TRUE`, also return covariance and
#'   precision matrices.
#' @param ci Logical; if `TRUE`, request confidence intervals when supported by
#'   the replacement function.
#' @param na_method Missing-data policy forwarded to the replacement function
#'   when supported.
#' @param ... Additional arguments forwarded to the replacement function when
#'   supported.
#'
#' @details
#' Renamed functions:
#' \itemize{
#'   \item `bland_altman()` -> `ba()`
#'   \item `bland_altman_repeated()` -> `ba_rm()`
#'   \item `biweight_mid_corr()` -> `bicor()`
#'   \item `distance_corr()` -> `dcor()`
#'   \item `partial_correlation()` -> `pcorr()`
#'   \item `ccc_lmm_reml()` -> `ccc_rm_reml()`
#'   \item `ccc_pairwise_u_stat()` -> `ccc_rm_ustat()`
#' }
#'
#' The deprecated wrappers will be removed in `matrixCorr` 2.0.0.
#'
#' @name deprecated-matrixCorr
NULL

.mc_deprecate <- function(old,
                          new,
                          details = NULL,
                          remove_in = "2.0.0") {
  msg <- paste0(
    "`", old, "()` is deprecated in matrixCorr 1.0.0; use `", new,
    "()` instead. It will be removed in ", remove_in, ".",
    if (!is.null(details)) paste0(" ", details) else ""
  )

  .Deprecated(new = new, package = "matrixCorr", msg = msg)
  invisible(NULL)
}

#' @rdname deprecated-matrixCorr
#' @export
bland_altman <- function(group1,
                         group2,
                         two = 1.96,
                         mode = 1L,
                         conf_level = 0.95,
                         n_threads = getOption("matrixCorr.threads", 1L),
                         verbose = FALSE) {
  .mc_deprecate(
    old = "bland_altman",
    new = "ba",
    details = "Argument `two` was renamed to `loa_multiplier`."
  )

  ba(
    group1 = group1,
    group2 = group2,
    loa_multiplier = two,
    mode = mode,
    conf_level = conf_level,
    n_threads = n_threads,
    verbose = verbose
  )
}

#' @rdname deprecated-matrixCorr
#' @export
bland_altman_repeated <- function(data = NULL, response, subject, method, time,
                                  two = 1.96, conf_level = 0.95,
                                  n_threads = getOption("matrixCorr.threads", 1L),
                                  include_slope = FALSE,
                                  use_ar1 = FALSE, ar1_rho = NA_real_,
                                  max_iter = 200L, tol = 1e-6,
                                  verbose = FALSE) {
  .mc_deprecate(
    old = "bland_altman_repeated",
    new = "ba_rm",
    details = "Argument `two` was renamed to `loa_multiplier`."
  )

  ba_rm(
    data = data,
    response = response,
    subject = subject,
    method = method,
    time = time,
    loa_multiplier = two,
    conf_level = conf_level,
    n_threads = n_threads,
    include_slope = include_slope,
    use_ar1 = use_ar1,
    ar1_rho = ar1_rho,
    max_iter = max_iter,
    tol = tol,
    verbose = verbose
  )
}

#' @rdname deprecated-matrixCorr
#' @export
biweight_mid_corr <- function(
    data,
    c_const = 9,
    max_p_outliers = 1,
    pearson_fallback = c("hybrid", "none", "all"),
    na_method = c("error", "pairwise", "complete"),
    mad_consistent = FALSE,
    w = NULL,
    sparse_threshold = NULL,
    n_threads = getOption("matrixCorr.threads", 1L),
    output = c("matrix", "sparse", "edge_list"),
    threshold = 0,
    diag = TRUE
) {
  .mc_deprecate(old = "biweight_mid_corr", new = "bicor")

  bicor(
    data = data,
    c_const = c_const,
    max_p_outliers = max_p_outliers,
    pearson_fallback = pearson_fallback,
    na_method = na_method,
    mad_consistent = mad_consistent,
    w = w,
    sparse_threshold = sparse_threshold,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag
  )
}

#' @rdname deprecated-matrixCorr
#' @export
distance_corr <- function(data, na_method = c("error", "pairwise", "complete"), ...) {
  .mc_deprecate(old = "distance_corr", new = "dcor")
  if (missing(na_method)) {
    return(dcor(data = data, ...))
  }
  dcor(data = data, na_method = na_method, ...)
}

#' @rdname deprecated-matrixCorr
#' @export
partial_correlation <- function(data,
                                method = c("oas", "ridge", "sample"),
                                lambda = 1e-3,
                                return_cov_precision = FALSE,
                                ci = FALSE,
                                conf_level = 0.95,
                                output = c("matrix", "sparse", "edge_list"),
                                threshold = 0,
                                diag = TRUE) {
  .mc_deprecate(
    old = "partial_correlation",
    new = "pcorr",
    details = "This wrapper preserves the pre-1.0 default `method = \"oas\"`."
  )

  pcorr(
    data = data,
    method = match.arg(method),
    lambda = lambda,
    return_cov_precision = return_cov_precision,
    ci = ci,
    conf_level = conf_level,
    output = output,
    threshold = threshold,
    diag = diag
  )
}

#' @rdname deprecated-matrixCorr
#' @export
ccc_lmm_reml <- function(data, response, rind,
                         method = NULL, time = NULL, interaction = FALSE,
                         max_iter = 100, tol = 1e-6,
                         Dmat = NULL,
                         Dmat_type = c("time-avg", "typical-visit", "weighted-avg", "weighted-sq"),
                         Dmat_weights = NULL,
                         Dmat_rescale = TRUE,
                         ci = FALSE, conf_level = 0.95,
                         ci_mode = c("auto", "raw", "logit"),
                         verbose = FALSE, digits = 4, use_message = TRUE,
                         ar = c("none", "ar1"),
                         ar_rho = NA_real_,
                         slope = c("none", "subject", "method", "custom"),
                         slope_var = NULL,
                         slope_Z = NULL,
                         drop_zero_cols = TRUE,
                         vc_select = c("auto", "none"),
                         vc_alpha = 0.05,
                         vc_test_order = c("subj_time", "subj_method"),
                         include_subj_method = NULL,
                         include_subj_time = NULL,
                         sb_zero_tol = 1e-10) {
  .mc_deprecate(old = "ccc_lmm_reml", new = "ccc_rm_reml")

  ccc_rm_reml(
    data = data,
    response = response,
    subject = rind,
    method = method,
    time = time,
    interaction = interaction,
    max_iter = max_iter,
    tol = tol,
    Dmat = Dmat,
    Dmat_type = Dmat_type,
    Dmat_weights = Dmat_weights,
    Dmat_rescale = Dmat_rescale,
    ci = ci,
    conf_level = conf_level,
    ci_mode = ci_mode,
    verbose = verbose,
    digits = digits,
    use_message = use_message,
    ar = ar,
    ar_rho = ar_rho,
    slope = slope,
    slope_var = slope_var,
    slope_Z = slope_Z,
    drop_zero_cols = drop_zero_cols,
    vc_select = vc_select,
    vc_alpha = vc_alpha,
    vc_test_order = vc_test_order,
    include_subj_method = include_subj_method,
    include_subj_time = include_subj_time,
    sb_zero_tol = sb_zero_tol
  )
}

#' @rdname deprecated-matrixCorr
#' @export
ccc_pairwise_u_stat <- function(data,
                                response,
                                method,
                                subject,
                                time = NULL,
                                Dmat = NULL,
                                delta = 1,
                                ci = FALSE,
                                conf_level = 0.95,
                                n_threads = getOption("matrixCorr.threads", 1L),
                                verbose = FALSE) {
  .mc_deprecate(old = "ccc_pairwise_u_stat", new = "ccc_rm_ustat")

  ccc_rm_ustat(
    data = data,
    response = response,
    method = method,
    subject = subject,
    time = time,
    Dmat = Dmat,
    delta = delta,
    ci = ci,
    conf_level = conf_level,
    n_threads = n_threads,
    verbose = verbose
  )
}

test_that("bland_altman wrapper warns and preserves `two` mapping", {
  set.seed(1)
  x <- rnorm(40)
  y <- x + rnorm(40, sd = 0.5)

  expect_warning(
    bland_altman(x, y, two = 2),
    "removed in 2.0.0"
  )

  old_fit <- suppressWarnings(bland_altman(x, y, two = 2))
  new_fit <- ba(x, y, loa_multiplier = 2)

  expect_equal(old_fit$mean.diffs, new_fit$mean.diffs, tolerance = 1e-12)
  expect_equal(old_fit$lower.limit, new_fit$lower.limit, tolerance = 1e-12)
  expect_equal(old_fit$upper.limit, new_fit$upper.limit, tolerance = 1e-12)
  expect_equal(old_fit$loa_multiplier, new_fit$loa_multiplier)
})

test_that("repeated-measures wrappers warn and preserve outputs", {
  set.seed(2)
  n_subject <- 8L
  n_time <- 3L
  subject <- rep(seq_len(n_subject), each = n_time)
  time <- rep(seq_len(n_time), times = n_subject)
  truth <- rnorm(n_subject, 10, 1)[subject]
  mA <- truth + rnorm(length(truth), sd = 0.3)
  mB <- truth + 0.2 + rnorm(length(truth), sd = 0.4)

  dat <- rbind(
    data.frame(y = mA, subject = subject, method = "A", time = time),
    data.frame(y = mB, subject = subject, method = "B", time = time)
  )

  expect_warning(
    bland_altman_repeated(
      data = dat,
      response = "y",
      subject = "subject",
      method = "method",
      time = "time"
    ),
    "removed in 2.0.0"
  )

  old_ba <- suppressWarnings(
    bland_altman_repeated(
      data = dat,
      response = "y",
      subject = "subject",
      method = "method",
      time = "time",
      two = 2
    )
  )
  new_ba <- ba_rm(
    data = dat,
    response = "y",
    subject = "subject",
    method = "method",
    time = "time",
    loa_multiplier = 2
  )

  expect_equal(old_ba$mean.diffs, new_ba$mean.diffs, tolerance = 1e-12)
  expect_equal(old_ba$lower.limit, new_ba$lower.limit, tolerance = 1e-12)
  expect_equal(old_ba$upper.limit, new_ba$upper.limit, tolerance = 1e-12)

  df <- expand.grid(subject = 1:10, time = 1:2, method = c("A", "B", "C"))
  df$y <- rnorm(nrow(df), mean = match(df$method, c("A", "B", "C")), sd = 1)

  expect_warning(
    ccc_pairwise_u_stat(
      df,
      response = "y",
      method = "method",
      subject = "subject",
      time = "time"
    ),
    "removed in 2.0.0"
  )

  old_u <- suppressWarnings(
    ccc_pairwise_u_stat(
      df,
      response = "y",
      method = "method",
      subject = "subject",
      time = "time"
    )
  )
  new_u <- ccc_rm_ustat(
    df,
    response = "y",
    subject = "subject",
    method = "method",
    time = "time"
  )

  expect_equal(unclass(old_u), unclass(new_u), tolerance = 1e-12)

  expect_warning(
    ccc_lmm_reml(
      df,
      response = "y",
      rind = "subject",
      method = "method",
      time = "time",
      ci = TRUE
    ),
    "removed in 2.0.0"
  )

  old_reml <- suppressWarnings(
    ccc_lmm_reml(
      df,
      response = "y",
      rind = "subject",
      method = "method",
      time = "time",
      ci = TRUE
    )
  )
  new_reml <- ccc_rm_reml(
    df,
    response = "y",
    subject = "subject",
    method = "method",
    time = "time",
    ci = TRUE
  )

  expect_equal(old_reml$est, new_reml$est, tolerance = 1e-12)
})

test_that("correlation wrappers warn and preserve outputs", {
  set.seed(123)
  X <- matrix(rnorm(120), ncol = 4)
  X[1, 1] <- 8

  expect_warning(biweight_mid_corr(X), "removed in 2.0.0")
  old_bicor <- suppressWarnings(biweight_mid_corr(X))
  new_bicor <- bicor(X)
  expect_equal(unclass(old_bicor), unclass(new_bicor), tolerance = 1e-12)

  expect_warning(distance_corr(X), "removed in 2.0.0")
  old_dcor <- suppressWarnings(distance_corr(X))
  new_dcor <- dcor(X)
  expect_equal(unclass(old_dcor), unclass(new_dcor), tolerance = 1e-12)
})

test_that("partial_correlation wrapper preserves the old default method", {
  set.seed(42)
  X <- matrix(rnorm(100 * 5), ncol = 5)

  expect_warning(partial_correlation(X), "pre-1.0 default `method = \"oas\"`")

  old_fit <- suppressWarnings(partial_correlation(X))
  new_fit <- pcorr(X, method = "oas")

  expect_equal(estimate(old_fit), estimate(new_fit), tolerance = 1e-12)
  expect_equal(attr(old_fit, "partial_method", exact = TRUE), "oas")
})

#' @title Pairwise Winsorized correlation
#'
#' @description
#' Computes all pairwise Winsorized correlation coefficients for the numeric
#' columns of a matrix or data frame using a high-performance 'C++' backend.
#'
#' This function Winsorizes each margin at proportion \code{tr} and then
#' computes ordinary Pearson correlation on the Winsorized values. It is a
#' simple robust alternative to Pearson correlation when the main concern is
#' unusually large or small observations in the marginal distributions.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns will be excluded.
#' @param tr Winsorization proportion in \code{[0, 0.5)}. For a sample of size
#'   \eqn{n}, let \eqn{g = \lfloor tr \cdot n \rfloor}; the \eqn{g} smallest
#'   observations are set to the \eqn{(g+1)}-st order statistic and the
#'   \eqn{g} largest observations are set to the \eqn{(n-g)}-th order
#'   statistic. Default \code{0.2}.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each estimate on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach percentile
#'   bootstrap confidence intervals for each pairwise estimate.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach the
#'   method-specific large-sample test statistic and two-sided p-value for each
#'   pairwise estimate.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default
#'   \code{0.95}.
#' @param n_boot Integer \eqn{\geq 1}. Number of bootstrap resamples used when
#'   \code{ci = TRUE}. Default \code{500}.
#' @param seed Optional positive integer used to seed the bootstrap resampling
#'   when \code{ci = TRUE}. If \code{NULL}, the current random-number stream is
#'   used.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#' @param x An object of class \code{wincor}.
#' @param digits Integer; number of digits to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits used for confidence limits in pairwise
#'   summaries.
#' @param p_digits Integer; digits used for p-values in pairwise summaries.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to the underlying print or plot helper.
#' @param title Character; plot title.
#' @param low_color,high_color,mid_color Colors used in the heatmap.
#' @param value_text_size Numeric text size for overlaid cell values.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#'
#' @return A symmetric correlation matrix with class \code{wincor} and
#'   attributes \code{method = "winsorized_correlation"}, \code{description},
#'   and \code{package = "matrixCorr"}. When \code{ci = TRUE}, the returned
#'   object also carries a \code{ci} attribute with elements \code{est},
#'   \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and \code{ci.method},
#'   plus \code{attr(x, "conf.level")}. When \code{p_value = TRUE}, it also
#'   carries an \code{inference} attribute with elements \code{estimate},
#'   \code{statistic}, \code{parameter}, \code{p_value}, \code{n_obs}, and
#'   \code{alternative}. When either inferential option is requested, the
#'   object also carries \code{diagnostics$n_complete}.
#'
#' @details
#' Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
#' observations and columns as variables. For a column
#' \eqn{x = (x_i)_{i=1}^n}, write the order statistics as
#' \eqn{x_{(1)} \le \cdots \le x_{(n)}} and let
#' \eqn{g = \lfloor tr \cdot n \rfloor}. The Winsorized values can be written as
#' \deqn{
#' x_i^{(w)} \;=\; \max\!\bigl\{x_{(g+1)},\, \min(x_i, x_{(n-g)})\bigr\}.
#' }
#' For two columns \eqn{x} and \eqn{y}, the Winsorized correlation is the
#' ordinary Pearson correlation computed from \eqn{x^{(w)}} and \eqn{y^{(w)}}:
#' \deqn{
#' r_w(x,y) \;=\;
#' \frac{\sum_{i=1}^n (x_i^{(w)}-\bar x^{(w)})(y_i^{(w)}-\bar y^{(w)})}
#'      {\sqrt{\sum_{i=1}^n (x_i^{(w)}-\bar x^{(w)})^2}\;
#'       \sqrt{\sum_{i=1}^n (y_i^{(w)}-\bar y^{(w)})^2}}.
#' }
#'
#' In matrix form, let \eqn{X^{(w)}} contain the Winsorized columns and define
#' the centred, unit-norm columns
#' \deqn{
#' z_{\cdot j} =
#' \frac{x_{\cdot j}^{(w)} - \bar x_j^{(w)} \mathbf{1}}
#'      {\sqrt{\sum_{i=1}^n (x_{ij}^{(w)}-\bar x_j^{(w)})^2}},
#' \qquad j=1,\ldots,p.
#' }
#' If \eqn{Z = [z_{\cdot 1}, \ldots, z_{\cdot p}]}, then the Winsorized
#' correlation matrix is
#' \deqn{
#' R_w \;=\; Z^\top Z.
#' }
#'
#' Winsorization acts on each margin separately, so it guards against marginal
#' outliers and heavy tails but does not target unusual points in the joint
#' cloud. This implementation Winsorizes each column in 'C++', centres and
#' normalises it, and forms the complete-data matrix from cross-products. With
#' \code{na_method = "pairwise"}, each pair is recomputed on its overlap of
#' non-missing rows. As with Pearson correlation, the complete-data path yields
#' a symmetric positive semidefinite matrix, whereas pairwise deletion can
#' break positive semidefiniteness. If the Winsorized variance of a column is
#' zero, correlations involving that column are returned as \code{NA}.
#'
#' When \code{p_value = TRUE}, inference follows the method-specific test based
#' on
#' \deqn{
#' T_{ij} = r_{w,ij}\sqrt{\frac{n_{ij} - 2}{1 - r_{w,ij}^2}},
#' }
#' evaluated against a \eqn{t}-distribution with
#' \eqn{n_{ij} - 2g_{ij} - 2} degrees of freedom, where
#' \eqn{g_{ij} = \lfloor tr \cdot n_{ij} \rfloor} and \eqn{n_{ij}} is the
#' pairwise complete-case sample size for the corresponding column pair. The
#' p-value is reported only when the pair is not identical and the resulting
#' degrees of freedom are positive. When \code{ci = TRUE}, the interval is a
#' percentile bootstrap interval based on \eqn{n_{\mathrm{boot}}} resamples
#' drawn from the pairwise complete cases. If
#' \eqn{\tilde r_{w,(1)} \le \cdots \le \tilde r_{w,(B)}} denotes the sorted
#' bootstrap sample of finite estimates with \eqn{B} retained resamples, the
#' reported limits are
#' \deqn{
#' \tilde r_{w,(\ell)} \quad \text{and} \quad \tilde r_{w,(u)},
#' }
#' where \eqn{\ell = \lfloor (\alpha/2) B + 0.5 \rfloor} and
#' \eqn{u = \lfloor (1-\alpha/2) B + 0.5 \rfloor} for
#' \eqn{\alpha = 1 - \mathrm{conf\_level}}. Resamples that yield undefined
#' estimates are discarded before the percentile limits are formed.
#'
#' \strong{Computational complexity.} In the complete-data path, Winsorizing the
#' columns requires sorting within each column, and forming the cross-product
#' matrix costs \eqn{O(n p^2)} with \eqn{O(p^2)} output storage. When
#' \code{ci = TRUE}, the bootstrap cost is incurred separately for each column
#' pair.
#'
#' @references
#' Wilcox, R. R. (1993). Some results on a Winsorized correlation coefficient.
#' British Journal of Mathematical and Statistical Psychology, 46(2), 339-349.
#' \doi{10.1111/j.2044-8317.1993.tb01020.x}
#'
#' Wilcox, R. R. (2012). Introduction to Robust Estimation and Hypothesis
#' Testing (3rd ed.). Academic Press.
#'
#' @seealso [pbcor()], [skipped_corr()], [bicor()]
#'
#' @examples
#' set.seed(11)
#' X <- matrix(rnorm(180 * 4), ncol = 4)
#' X[sample(length(X), 6)] <- X[sample(length(X), 6)] - 12
#'
#' R <- wincor(X, tr = 0.2)
#' print(R, digits = 2)
#' summary(R)
#' estimate(R)
#' tidy(R)
#' plot(R)
#'
#' ## Bootstrap confidence intervals
#' R_ci <- wincor(X, tr = 0.2, ci = TRUE, n_boot = 49, seed = 11)
#' ci(R_ci)
#' confint(R_ci)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(R)
#' }
#'
#' @author Thiago de Paula Oliveira
#' @export
wincor <- function(data,
                   na_method = c("error", "pairwise", "complete"),
                   ci = FALSE,
                   p_value = FALSE,
                   conf_level = 0.95,
                   n_threads = getOption("matrixCorr.threads", 1L),
                   tr = 0.2,
                   n_boot = 500L,
                   seed = NULL,
                   output = c("matrix", "sparse", "edge_list"),
                   threshold = 0,
                   diag = TRUE) {
  na_method <- match.arg(na_method)
  check_scalar_numeric(tr,
                       arg = "tr",
                       lower = 0,
                       upper = 0.5,
                       closed_lower = TRUE,
                       closed_upper = FALSE)
  desc <- paste0(
    "Winsorized correlation; tr = ", tr,
    "; NA mode = ", na_method, "."
  )

  .mc_inference_corr_wrapper(
    data = data,
    na_method = na_method,
    ci = ci,
    p_value = p_value,
    conf_level = conf_level,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag,
    estimator_class = "wincor",
    method = "winsorized_correlation",
    description = desc,
    kernel_matrix = function(x, n_threads) {
      wincor_matrix_cpp(x, tr = tr, n_threads = n_threads)
    },
    kernel_pairwise = function(x, n_threads) {
      wincor_matrix_pairwise_cpp(x, tr = tr, min_n = 5L, n_threads = n_threads)
    },
    kernel_threshold = function(x, threshold, diag, n_threads) {
      wincor_threshold_triplets_cpp(
        x,
        tr = tr,
        threshold = threshold,
        diag = diag,
        n_threads = n_threads
      )
    },
    payload_builder = function(x, est, ci, p_value, conf_level, n_boot, seed) {
      .mc_wincor_pairwise_payload(
        x,
        est = est,
        tr = tr,
        ci = ci,
        p_value = p_value,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = seed
      )
    },
    min_n = 5L,
    direct_method = "wincor",
    symmetric = TRUE,
    n_boot = n_boot,
    seed = seed
  )
}

.mc_winsorize_vec <- function(z, tr = 0.2) {
  n <- length(z)
  if (!n) {
    return(z)
  }
  g <- floor(tr * n)
  if (g <= 0L) {
    return(z)
  }

  zs <- sort(z)
  low <- zs[[g + 1L]]
  high <- zs[[n - g]]
  pmax(low, pmin(z, high))
}

.mc_wincor_pair_exact <- function(x, y, tr = 0.2) {
  xw <- .mc_winsorize_vec(x, tr = tr)
  yw <- .mc_winsorize_vec(y, tr = tr)

  xc <- xw - mean(xw)
  yc <- yw - mean(yw)
  den_x <- sum(xc^2)
  den_y <- sum(yc^2)
  if (!is.finite(den_x) || !is.finite(den_y) || den_x <= 0 || den_y <= 0) {
    return(NA_real_)
  }

  sum(xc * yc) / sqrt(den_x * den_y)
}

.mc_wincor_pair_inference <- function(x,
                                      y,
                                      tr = 0.2,
                                      ci = FALSE,
                                      p_value = FALSE,
                                      conf_level = 0.95,
                                      n_boot = 500L,
                                      seed = NULL) {
  est <- .mc_wincor_pair_exact(x, y, tr = tr)
  n_complete <- length(x)
  g <- floor(tr * n_complete)
  parameter <- n_complete - 2 * g - 2
  statistic <- NA_real_
  p_val <- NA_real_
  lwr <- NA_real_
  upr <- NA_real_

  if (isTRUE(p_value) && is.finite(est) && parameter > 0L && any(x != y)) {
    if (abs(est) >= 1) {
      statistic <- sign(est) * Inf
      p_val <- 0
    } else {
      statistic <- est * sqrt((n_complete - 2) / (1 - est^2))
      p_val <- 2 * stats::pt(abs(statistic), df = parameter, lower.tail = FALSE)
    }
  }

  if (isTRUE(ci) && is.finite(est) && n_complete > 1L) {
    boot_vals <- .mc_eval_with_seed(
      seed,
      replicate(
        n_boot,
        {
          idx <- sample.int(n_complete, size = n_complete, replace = TRUE)
          .mc_wincor_pair_exact(x[idx], y[idx], tr = tr)
        }
      )
    )
    ci_vals <- .mc_percentile_boot_ci(boot_vals, conf_level = conf_level)
    lwr <- ci_vals[[1L]]
    upr <- ci_vals[[2L]]
  }

  list(
    estimate = est,
    statistic = statistic,
    p_value = p_val,
    parameter = parameter,
    lwr = lwr,
    upr = upr,
    n_complete = n_complete
  )
}

.mc_wincor_pairwise_payload <- function(X,
                                        est,
                                        tr = 0.2,
                                        ci = FALSE,
                                        p_value = FALSE,
                                        conf_level = 0.95,
                                        n_boot = 500L,
                                        seed = NULL) {
  est <- as.matrix(est)
  p <- ncol(est)
  dn <- dimnames(est)
  n_complete <- .mc_corr_n_complete(X)
  dimnames(n_complete) <- dn

  ci_lwr <- ci_upr <- NULL
  if (isTRUE(ci)) {
    ci_lwr <- matrix(NA_real_, p, p, dimnames = dn)
    ci_upr <- matrix(NA_real_, p, p, dimnames = dn)
    diag(ci_lwr) <- 1
    diag(ci_upr) <- 1
  }

  stat_mat <- p_mat <- df_mat <- NULL
  if (isTRUE(p_value)) {
    stat_mat <- matrix(NA_real_, p, p, dimnames = dn)
    p_mat <- matrix(NA_real_, p, p, dimnames = dn)
    df_mat <- matrix(NA_real_, p, p, dimnames = dn)
    diag(stat_mat) <- Inf
    diag(p_mat) <- 0
  }

  pair_id <- 0L
  for (i in seq_len(p - 1L)) {
    for (j in seq.int(i + 1L, p)) {
      pair_id <- pair_id + 1L
      ok <- is.finite(X[, i]) & is.finite(X[, j])
      n_ij <- sum(ok)
      n_complete[i, j] <- n_complete[j, i] <- n_ij
      if (n_ij < 5L) {
        next
      }

      info <- .mc_wincor_pair_inference(
        X[ok, i],
        X[ok, j],
        tr = tr,
        ci = ci,
        p_value = p_value,
        conf_level = conf_level,
        n_boot = n_boot,
        seed = .mc_seed_offset(seed, pair_id - 1L)
      )

      if (isTRUE(ci)) {
        ci_lwr[i, j] <- ci_lwr[j, i] <- info$lwr
        ci_upr[i, j] <- ci_upr[j, i] <- info$upr
      }
      if (isTRUE(p_value)) {
        stat_mat[i, j] <- stat_mat[j, i] <- info$statistic
        p_mat[i, j] <- p_mat[j, i] <- info$p_value
        df_mat[i, j] <- df_mat[j, i] <- info$parameter
      }
    }
  }

  diagnostics <- list(n_complete = n_complete)
  ci_attr <- NULL
  if (isTRUE(ci)) {
    ci_attr <- list(
      est = unclass(est),
      lwr.ci = ci_lwr,
      upr.ci = ci_upr,
      conf.level = conf_level,
      ci.method = "percentile_bootstrap"
    )
  }

  inference_attr <- NULL
  if (isTRUE(p_value)) {
    inference_attr <- list(
      method = "winsorized_t_test",
      estimate = unclass(est),
      statistic = stat_mat,
      p_value = p_mat,
      parameter = df_mat,
      n_obs = n_complete,
      alternative = "two.sided"
    )
  }

  list(
    diagnostics = diagnostics,
    ci = ci_attr,
    inference = inference_attr
  )
}

.mc_wincor_pairwise_summary <- function(object,
                                        digits = 4,
                                        ci_digits = 3,
                                        p_digits = 4,
                                        show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "wincor")

  ci <- .mc_ci_attr(object)
  inf <- .mc_inference_attr(object)
  include_p <- !is.null(inf) && !is.null(inf$p_value)
  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.wincor",
    digits = digits,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci,
    ci_attr = ci,
    inference_attr = inf,
    include_p = include_p,
    extra_columns = if (isTRUE(include_p)) {
      list(
        statistic = list(matrix = inf$statistic, digits = digits),
        p_value = list(matrix = inf$p_value, digits = p_digits)
      )
    },
    extra_attrs = list(
      inference_method = if (is.null(inf)) NA_character_ else inf$method,
      n_boot = if (is.null(ci)) NA_integer_ else attr(object, "n_boot", exact = TRUE) %||% NA_integer_
    )
  )
}

#' @rdname wincor
#' @method print wincor
#' @export
print.wincor <- function(x, digits = 4, n = NULL, topn = NULL,
                         max_vars = NULL, width = NULL,
                         show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Winsorized correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname wincor
#' @method plot wincor
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.wincor <- function(x,
                        title = "Winsorized correlation heatmap",
                        low_color = "indianred1",
                        high_color = "steelblue1",
                        mid_color = "white",
                        value_text_size = 4,
                        show_value = TRUE,
                        ...) {
  .mc_plot_corr_matrix(
    x,
    class_name = "wincor",
    fill_name = "wincor",
    title = title,
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    show_value = show_value,
    ...
  )
}

#' @rdname wincor
#' @method summary wincor
#' @param object An object of class \code{wincor}.
#' @export
summary.wincor <- function(object, n = NULL, topn = NULL,
                           max_vars = NULL, width = NULL,
                           ci_digits = 3,
                           p_digits = 4,
                           show_ci = NULL, ...) {
  check_inherits(object, "wincor")
  if (is.null(.mc_ci_attr(object)) && is.null(.mc_inference_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_wincor_pairwise_summary(
    object,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci
  )
}

#' @rdname wincor
#' @method print summary.wincor
#' @param x An object of class \code{summary.wincor}.
#' @export
print.summary.wincor <- function(x, digits = NULL, n = NULL,
                                 topn = NULL, max_vars = NULL,
                                 width = NULL, show_ci = NULL, ...) {
  extra_items <- c(
    inference = attr(x, "inference_method", exact = TRUE),
    n_boot = if (isTRUE(attr(x, "has_ci", exact = TRUE))) {
      as.character(attr(x, "n_boot", exact = TRUE))
    } else {
      NA_character_
    }
  )
  .mc_print_pairwise_summary_digest(
    x,
    title = "Winsorized correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "percentile_bootstrap",
    extra_items = extra_items,
    ...
  )
  invisible(x)
}

#' @title Pairwise (or Two-Vector) Kendall's Tau Rank Correlation
#'
#' @description
#' Computes pairwise Kendall's tau correlations for numeric data using a
#' high-performance 'C++' backend. Optional confidence intervals are available
#' for matrix and data-frame input.
#'
#' @param data
#' For matrix/data frame mode, a numeric matrix or a data frame with at least
#' two numeric columns. All non-numeric columns are excluded. For two-vector
#' mode, a numeric vector \code{x}.
#' @param y Optional numeric vector \code{y} of the same length as \code{data}
#' when \code{data} is a vector. If supplied, the function computes the
#' Kendall correlation \emph{between \code{data} and \code{y}} using a
#' low-overhead scalar path and returns a single number.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each correlation on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach pairwise
#' confidence intervals for the off-diagonal Kendall correlations in
#' matrix/data-frame mode.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#' @param ci_method Confidence-interval engine used when \code{ci = TRUE}.
#' Supported Kendall methods are \code{"fieller"} (default),
#' \code{"brown_benedetti"}, and \code{"if_el"}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return
#' \itemize{
#'   \item If \code{y} is \code{NULL} and \code{data} is a matrix/data frame: a
#'   symmetric numeric matrix where entry \code{(i, j)} is the Kendall's tau
#'   correlation between the \code{i}-th and \code{j}-th numeric columns. When
#'   \code{ci = TRUE}, the object also carries a \code{ci} attribute with
#'   elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and
#'   \code{ci.method}. Pairwise complete-case sample sizes are stored in
#'   \code{attr(x, "diagnostics")$n_complete}.
#'   \item If \code{y} is provided (two-vector mode): a single numeric scalar,
#'   the Kendall's tau correlation between \code{data} and \code{y}.
#' }
#'
#' @details
#' Kendall's tau is a rank-based measure of association between two variables.
#' For a dataset with \eqn{n} observations on variables \eqn{X} and \eqn{Y},
#' let \eqn{n_0 = n(n - 1)/2} be the number of unordered pairs, \eqn{C} the
#' number of concordant pairs, and \eqn{D} the number of discordant pairs.
#' Let \eqn{T_x = \sum_g t_g (t_g - 1)/2} and \eqn{T_y = \sum_h u_h (u_h - 1)/2}
#' be the numbers of tied pairs within \eqn{X} and within \eqn{Y}, respectively,
#' where \eqn{t_g} and \eqn{u_h} are tie-group sizes in \eqn{X} and \eqn{Y}.
#'
#' The tie-robust Kendall's tau-b is:
#' \deqn{ \tau_b = \frac{C - D}{\sqrt{(n_0 - T_x)\,(n_0 - T_y)}}. }
#' When there are no ties (\eqn{T_x = T_y = 0}), this reduces to tau-a:
#' \deqn{ \tau_a = \frac{C - D}{n(n-1)/2}. }
#'
#' The function automatically handles ties. In degenerate cases where a
#' variable is constant (\eqn{n_0 = T_x} or \eqn{n_0 = T_y}), the tau-b
#' denominator is zero and the correlation is undefined (returned as \code{NA}
#' off the diagonal).
#'
#' When \code{na_method = "pairwise"}, each \eqn{(i,j)} estimate is recomputed
#' on the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}.
#' Confidence intervals use the observed pairwise-complete Kendall estimate and
#' the same pairwise complete-case overlap.
#'
#' With \code{ci_method = "fieller"}, the interval is built on the Fisher-style
#' transformed scale \eqn{z = \operatorname{atanh}(\hat\tau)} using Fieller's
#' asymptotic standard error
#' \deqn{ \operatorname{SE}(z) = \sqrt{\frac{0.437}{n - 4}}, }
#' where \eqn{n} is the pairwise complete-case sample size. The interval is then
#' mapped back with \code{tanh()} and clipped to \eqn{[-1, 1]} for numerical
#' safety. This is the default Kendall CI and is intended to be the fast,
#' production-oriented choice.
#'
#' With \code{ci_method = "brown_benedetti"}, the interval is computed on the
#' Kendall tau scale using the Brown-Benedetti large-sample variance for
#' Kendall's tau-b. This path is tie-aware, remains on the original Kendall
#' scale, and is intended as a conventional asymptotic alternative when a
#' direct tau-scale interval is preferred.
#'
#' With \code{ci_method = "if_el"}, the interval is computed in 'C++' using an
#' influence-function empirical-likelihood construction built from the
#' linearised Kendall estimating equation. The lower and upper limits are found
#' by solving the empirical-likelihood ratio equation against the
#' \eqn{\chi^2_1}-cutoff implied by \code{conf_level}. This method is slower
#' than \code{"fieller"} and is intended for specialised inference.
#'
#' \strong{Performance:}
#' \itemize{
#'   \item In the \strong{two-vector mode} (\code{y} supplied), the C++ backend uses a
#'   raw-double path with minimal overhead.
#'   \item In the \strong{matrix/data-frame mode}, the no-missing estimate-only path
#'   uses the Knight (1966) \eqn{O(n \log n)} algorithm. Pairwise-complete
#'   inference paths recompute each pair on its complete-case overlap; the
#'   \code{"brown_benedetti"} interval adds tie-aware large-sample variance
#'   calculations and \code{"if_el"} adds extra per-pair likelihood solving.
#' }
#'
#' @note Missing values are rejected when \code{na_method = "error"}. Columns
#' with fewer than two usable observations are excluded. Confidence intervals
#' are not available in the two-vector interface.
#'
#' @references
#' Kendall, M. G. (1938). A New Measure of Rank Correlation. \emph{Biometrika},
#' 30(1/2), 81-93.
#'
#' Knight, W. R. (1966). A Computer Method for Calculating Kendall's Tau with
#' Ungrouped Data. \emph{Journal of the American Statistical Association},
#' 61(314), 436-439.
#'
#' Fieller, E. C., Hartley, H. O., & Pearson, E. S. (1957). Tests for rank
#' correlation coefficients. I. \emph{Biometrika}, 44(3/4), 470-481.
#'
#' Brown, M. B., & Benedetti, J. K. (1977). Sampling behavior of tests for
#' correlation in two-way contingency tables. \emph{Journal of the American
#' Statistical Association}, 72(358), 309-315.
#'
#' Huang, Z., & Qin, G. (2023). Influence function-based confidence intervals
#' for the Kendall rank correlation coefficient. \emph{Computational
#' Statistics}, 38(2), 1041-1055.
#'
#' Croux, C., & Dehon, C. (2010). Influence functions of the Spearman and
#' Kendall correlation measures. \emph{Statistical Methods & Applications},
#' 19, 497-515.
#'
#' @examples
#' # Basic usage with a matrix
#' mat <- cbind(a = rnorm(100), b = rnorm(100), c = rnorm(100))
#' kt <- kendall_tau(mat)
#' print(kt)
#' summary(kt)
#' plot(kt)
#'
#' # Confidence intervals
#' kt_ci <- kendall_tau(mat[, 1:3], ci = TRUE)
#' print(kt_ci, show_ci = "yes")
#' summary(kt_ci)
#' estimate(kt_ci)
#' tidy(kt_ci)
#' ci(kt_ci)
#' confint(kt_ci)
#'
#' # Two-vector mode (scalar path)
#' x <- rnorm(1000); y <- 0.5 * x + rnorm(1000)
#' kendall_tau(x, y)
#'
#' # Including ties
#' tied_df <- data.frame(
#'   v1 = rep(1:5, each = 20),
#'   v2 = rep(5:1, each = 20),
#'   v3 = rnorm(100)
#' )
#' kt_tied <- kendall_tau(tied_df, ci = TRUE, ci_method = "fieller")
#' print(kt_tied, show_ci = "yes")
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(kt)
#' }
#'
#' @seealso \code{\link{print.kendall_matrix}}, \code{\link{plot.kendall_matrix}}
#' @author Thiago de Paula Oliveira
#' @export
kendall_tau <- function(data,
                        y = NULL,
                        na_method = c("error", "pairwise", "complete"),
                        ci = FALSE,
                        conf_level = 0.95,
                        ci_method = c("fieller", "if_el", "brown_benedetti"),
                        n_threads = getOption("matrixCorr.threads", 1L),
                        output = c("matrix", "sparse", "edge_list"),
                        threshold = 0,
                        diag = TRUE,
                        ...) {
  output_cfg <- .mc_prepare_corr_output(
    output = output,
    threshold = threshold,
    diag = diag
  )
  if (is.null(y) && ...length() == 0L && missing(na_method) && isFALSE(ci)) {
    input <- .mc_prepare_corr_input(
      data,
      na_cfg = list(na_method = "error", check_na = TRUE),
      min_n = 2L
    )
    return(.mc_with_omp_threads(
      n_threads,
      {
        .mc_finalize_corr_result(
          mat = kendall_matrix_cpp(input$data),
          class_name = "kendall_matrix",
          method = "kendall",
          description = "Pairwise Kendall's tau (auto tau-a/tau-b) correlation matrix",
          output_cfg = output_cfg,
          dimnames = input$dimnames,
          symmetric = TRUE
        )
      }
    ))
  }

  na_cfg <- .mc_resolve_corr_na(
    na_method = na_method,
    dots = list(...),
    na_method_missing = missing(na_method),
    allowed = c("error", "pairwise", "complete")
  )
  if (!isFALSE(ci)) {
    check_bool(ci, arg = "ci")
  } else if (!is.logical(ci) || length(ci) != 1L || is.na(ci)) {
    check_bool(ci, arg = "ci")
  }
  ci_method <- match.arg(ci_method)
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "non-matrix output modes are unavailable in two-vector mode."
      )
    }
    if (isTRUE(ci)) {
      abort_bad_arg("ci",
        message = "Confidence intervals are not available when {.arg y} is supplied."
      )
    }
    if (!is.numeric(data) || !is.numeric(y)) {
      abort_bad_arg("data",
        message = "and {.arg y} must be numeric vectors for two-vector mode."
      )
    }
    check_same_length(data, y, arg_x = "data", arg_y = "y")
    if (na_cfg$check_na && (any(!is.finite(data)) || any(!is.finite(y)))) {
      abort_bad_arg("data",
        message = "and {.arg y} must be free of NA/NaN/Inf when {.arg na_method} is {.val error}.",
        .hint   = "Set `na_method = \"pairwise\"` to use complete-case overlaps."
      )
    }
    if (identical(na_cfg$na_method, "complete")) {
      keep <- is.finite(data) & is.finite(y)
      if (sum(keep) < 2L) {
        abort_bad_arg(
          "data",
          message = "must retain at least 2 complete rows when {.arg na_method} is {.val complete}; retained {sum(keep)}."
        )
      }
      data <- data[keep]
      y <- y[keep]
    }

    tau <- kendall_tau2_cpp(as.numeric(data), as.numeric(y))
    return(as.numeric(tau))
  }

  input <- .mc_prepare_corr_input(data, na_cfg = na_cfg, min_n = 2L)
  diagnostics <- NULL
  ci_attr <- NULL

  .mc_with_omp_threads(
    n_threads,
    {
      if (!identical(na_cfg$na_method, "pairwise") && !isTRUE(ci)) {
        result <- kendall_matrix_cpp(input$data)
      } else {
        pairwise <- kendall_matrix_pairwise_cpp(
          input$data,
          return_ci = ci,
          conf_level = conf_level,
          ci_method = ci_method
        )
        result <- pairwise$est
        diagnostics <- list(
          n_complete = .mc_set_matrix_dimnames(pairwise$n_complete, input$colnames)
        )
        if (isTRUE(ci)) {
          ci_attr <- list(
            est = .mc_set_matrix_dimnames(unclass(result), input$colnames),
            lwr.ci = .mc_set_matrix_dimnames(unclass(pairwise$lwr), input$colnames),
            upr.ci = .mc_set_matrix_dimnames(unclass(pairwise$upr), input$colnames),
            conf.level = pairwise$conf_level,
            ci.method = pairwise$ci_method
          )
        }
      }

      .mc_finalize_corr_result(
        mat = result,
        class_name = "kendall_matrix",
        method = "kendall",
        description = "Pairwise Kendall's tau (auto tau-a/tau-b) correlation matrix",
        output_cfg = output_cfg,
        diagnostics = .mc_merge_diagnostics(diagnostics, input$diagnostics),
        dimnames = input$dimnames,
        symmetric = TRUE,
        extra_attrs = if (!is.null(ci_attr)) {
          list(
            ci = ci_attr,
            conf.level = conf_level,
            ci.method = ci_method
          )
        }
      )
    }
  )
}

.mc_kendall_pairwise_summary <- function(object,
                                         digits = 4,
                                         ci_digits = 3,
                                         show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "kendall_matrix")

  ci <- .mc_ci_attr(object)
  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.kendall_matrix",
    digits = digits,
    ci_digits = ci_digits,
    show_ci = show_ci,
    ci_attr = ci,
    include_p = NULL,
    extra_attrs = list(
      ci.method = if (is.null(ci)) NA_character_ else ci$ci.method
    )
  )
}

#' @rdname kendall_tau
#' @method print kendall_matrix
#' @title Print Method for \code{kendall_matrix} Objects
#'
#' @param x An object of class \code{kendall_matrix}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits for Kendall confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print}.
#'
#' @return Invisibly returns the \code{kendall_matrix} object.
#' @export
print.kendall_matrix <- function(x, digits = 4, n = NULL,
                                 topn = NULL, max_vars = NULL,
                                 width = NULL, ci_digits = 3,
                                 show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = "Kendall correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname kendall_tau
#' @method plot kendall_matrix
#' @title Plot Method for \code{kendall_matrix} Objects
#'
#' @param x An object of class \code{kendall_matrix}.
#' @param title Plot title. Default is \code{"Kendall's Tau correlation
#' heatmap"}.
#' @param low_color Color for the minimum tau value. Default is
#' \code{"indianred1"}.
#' @param high_color Color for the maximum tau value. Default is
#' \code{"steelblue1"}.
#' @param mid_color Color for zero correlation. Default is \code{"white"}.
#' @param value_text_size Font size for displaying correlation values. Default
#' is \code{4}.
#' @param ci_text_size Text size for confidence intervals in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other
#' \code{ggplot2} layers.
#'
#' @return A \code{ggplot} object representing the heatmap.
#' @import ggplot2
#' @export
plot.kendall_matrix <- function(x, title = "Kendall's Tau correlation heatmap",
                                low_color = "indianred1", high_color = "steelblue1",
                                mid_color = "white", value_text_size = 4,
                                ci_text_size = 3, show_value = TRUE, ...) {
  check_bool(show_value, arg = "show_value")
  ci <- .mc_ci_attr(x)
  if (is.null(ci) || is.null(ci$lwr.ci) || is.null(ci$upr.ci)) {
    return(.mc_plot_corr_matrix(
      x, class_name = "kendall_matrix", fill_name = "Tau",
      title = title, low_color = low_color, high_color = high_color,
      mid_color = mid_color, value_text_size = value_text_size,
      show_value = show_value, ...
    ))
  }

  est_mat <- as.matrix(x)
  df_est <- as.data.frame(as.table(est_mat))
  names(df_est) <- c("Var1", "Var2", "tau")

  df_lwr <- as.data.frame(as.table(ci$lwr.ci))
  names(df_lwr)[3] <- "lwr"
  df_upr <- as.data.frame(as.table(ci$upr.ci))
  names(df_upr)[3] <- "upr"
  df <- Reduce(
    function(a, b) merge(a, b, by = c("Var1", "Var2"), all = TRUE),
    list(df_est, df_lwr, df_upr)
  )

  diag_idx <- df$Var1 == df$Var2
  df$lwr[diag_idx] <- NA_real_
  df$upr[diag_idx] <- NA_real_
  df$ci_label <- ifelse(
    is.na(df$lwr) | is.na(df$upr),
    NA_character_,
    sprintf("[%.3f, %.3f]", df$lwr, df$upr)
  )

  lev_row <- unique(df_est$Var1)
  lev_col <- unique(df_est$Var2)
  df$Var1 <- factor(df$Var1, levels = rev(lev_row))
  df$Var2 <- factor(df$Var2, levels = lev_col)
  df$label <- sprintf("%.2f", df$tau)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = .data$tau)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = "Tau"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value)) {
    p <- p + ggplot2::geom_text(ggplot2::aes(label = label), size = value_text_size, color = "black")
  }

  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.25),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }

  p
}

#' @rdname kendall_tau
#' @method summary kendall_matrix
#' @param object An object of class \code{kendall_matrix}.
#' @param ci_digits Integer; digits for Kendall confidence limits in the
#'   pairwise summary.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
summary.kendall_matrix <- function(object,
                                   n = NULL,
                                   topn = NULL,
                                   max_vars = NULL,
                                   width = NULL,
                                   ci_digits = 3,
                                   show_ci = NULL,
                                   ...) {
  check_inherits(object, "kendall_matrix")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  if (is.null(.mc_ci_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_kendall_pairwise_summary(
    object,
    ci_digits = ci_digits,
    show_ci = show_ci
  )
}

#' @rdname kendall_tau
#' @method print summary.kendall_matrix
#' @param x An object of class \code{summary.kendall_matrix}.
#' @export
print.summary.kendall_matrix <- function(x, digits = NULL, n = NULL,
                                         topn = NULL, max_vars = NULL,
                                         width = NULL, show_ci = NULL, ...) {
  ci_method <- attr(x, "ci.method", exact = TRUE)
  .mc_print_pairwise_summary_digest(
    x,
    title = "Kendall correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = ci_method,
    ...
  )
  invisible(x)
}

#' @title Pairwise Lin's concordance correlation coefficient
#'
#' @description
#' Computes all pairwise Lin's Concordance Correlation Coefficients (CCC)
#' from the numeric columns of a matrix or data frame. CCC measures both
#' precision (Pearson correlation) and accuracy (closeness to the 45-degree line).
#' This function is backed by a high-performance 'C++' implementation.
#'
#' Lin's CCC quantifies the concordance between a new test/measurement
#' and a gold-standard for the same variable. Like a correlation, CCC
#' ranges from -1 to 1 with perfect agreement at 1, and it cannot exceed the
#' absolute value of the Pearson correlation between variables. It can be
#' legitimately computed even with small samples (e.g., 10 observations),
#' and results are often similar to intraclass correlation coefficients.
#' CCC provides a single summary of agreement, but it may not capture
#' systematic bias; a Bland-Altman plot (differences vs. means) is recommended
#' to visualize bias, proportional trends, and heteroscedasticity (see
#' \code{\link{ba}}).
#'
#' @details
#' Lin's CCC is defined as
#' \deqn{
#' \rho_c \;=\; \frac{2\,\mathrm{cov}(X, Y)}
#'                  {\sigma_X^2 + \sigma_Y^2 + (\mu_X - \mu_Y)^2},
#' }
#' where \eqn{\mu_X,\mu_Y} are the means, \eqn{\sigma_X^2,\sigma_Y^2} the
#' variances, and \eqn{\mathrm{cov}(X,Y)} the covariance. Equivalently,
#' \deqn{
#' \rho_c \;=\; r \times C_b, \qquad
#' r \;=\; \frac{\mathrm{cov}(X,Y)}{\sigma_X \sigma_Y}, \quad
#' C_b \;=\; \frac{2 \sigma_X \sigma_Y}
#'                {\sigma_X^2 + \sigma_Y^2 + (\mu_X - \mu_Y)^2}.
#' }
#' Hence \eqn{|\rho_c| \le |r| \le 1}, \eqn{\rho_c = r} iff
#' \eqn{\mu_X=\mu_Y} and \eqn{\sigma_X=\sigma_Y}, and \eqn{\rho_c=1} iff, in
#' addition, \eqn{r=1}. CCC is symmetric in \eqn{(X,Y)} and penalises both
#' location and scale differences; unlike Pearson's \eqn{r}, it is not invariant
#' to affine transformations that change means or variances.
#'
#' When \code{ci = TRUE}, large-sample confidence intervals for
#' \eqn{\rho_c} are returned for each pair. The implementation uses Lin's
#' delta-method standard error and then forms limits on a Fisher-z transformed
#' CCC scale before mapping back to \eqn{[-1, 1]}. For speed, CIs are omitted
#' when \code{ci = FALSE}.
#'
#'If either variable has zero variance, \eqn{\rho_c} is
#' undefined and \code{NA} is returned for that pair (including the diagonal).
#'
#' Missing-data handling follows the same contract as \code{\link{pearson_corr}}
#' and \code{\link{spearman_rho}}. With \code{na_method = "error"} (default),
#' missing and non-finite values are rejected. With \code{"complete"}, rows
#' incomplete in any retained numeric column are removed once before all pairwise
#' CCC estimates are computed. With \code{"pairwise"}, each method pair is
#' computed on its own complete finite overlap, and \code{n_complete}
#' diagnostics may vary by pair. Pairwise CIs require at least three complete
#' observations for that pair.
#'
#' Negative CCC estimates are fully supported. Matrix output preserves the
#' signed estimate, while thresholded \code{"sparse"} and \code{"edge_list"}
#' outputs retain entries according to \code{abs(CCC) >= threshold}.
#'
#' Probability of agreement, available through \code{\link{prob_agree}},
#' answers a
#' different question from CCC. CCC is a coefficient-based summary of
#' concordance between paired measurements. `prob_agree()` follows Stevens and
#' Anderson-Cook (2017) and estimates the probability that two estimated
#' quantities or curves differ by no more than a user-specified practical
#' tolerance.
#'
#' @param data A numeric matrix or data frame with at least two numeric columns.
#' Non-numeric columns will be ignored.
#' @param ci Logical; if TRUE, return lower and upper confidence bounds
#' @param conf_level Confidence level for CI, default = 0.95
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"complete"} removes rows incomplete in any retained numeric column
#'   before computing all CCC estimates. \code{"pairwise"} recomputes each
#'   method pair on its own complete finite overlap.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#' @param verbose Logical; if TRUE, prints how many threads are used
#'
#' @return A symmetric numeric matrix with class \code{"ccc"} and attributes:
#' \itemize{
#'   \item \code{method}: The method used ("Lin's concordance")
#'   \item \code{description}: Description string
#' }
#'  If \code{ci = FALSE}, returns matrix of class \code{"ccc"}.
#'         If \code{ci = TRUE}, returns a list with elements: \code{est},
#'         \code{lwr.ci}, \code{upr.ci}.
#'
#' @seealso \code{\link{print.ccc}}, \code{\link{plot.ccc}},
#' \code{\link{ba}}, \code{\link{prob_agree}}, and \code{\link{cia}}.
#' \code{ccc()} answers the question "How well do two paired measurements
#' agree overall, accounting for both correlation and mean/scale bias?".
#' In contrast, \code{cia()} answers "Are two or more methods
#' interchangeable at the individual level relative to within-method
#' replicate disagreement?".
#'
#' @seealso For repeated measurements look at \code{\link{ccc_rm_reml}},
#' \code{\link{ccc_rm_ustat}} or \code{\link{ba_rm}}
#'
#' @examples
#' # Example with multivariate normal data
#' Sigma <- matrix(c(1, 0.5, 0.3,
#'                   0.5, 1, 0.4,
#'                   0.3, 0.4, 1), nrow = 3)
#' mu <- c(0, 0, 0)
#' set.seed(123)
#' mat_mvn <- MASS::mvrnorm(n = 100, mu = mu, Sigma = Sigma)
#' result_mvn <- ccc(mat_mvn, ci = TRUE)
#' print(result_mvn)
#' summary(result_mvn)
#' estimate(result_mvn)
#' tidy(result_mvn)
#' ci(result_mvn)
#' confint(result_mvn)
#' plot(result_mvn)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(result_mvn)
#' }
#'
#' @importFrom stats var cov cor
#' @importFrom graphics plot
#' @importFrom ggplot2 ggplot aes geom_tile geom_text scale_fill_gradient2
#' @importFrom ggplot2 theme_minimal element_text coord_fixed labs theme
#' @author Thiago de Paula Oliveira
#' @references
#' Lin L (1989). A concordance correlation coefficient to evaluate
#' reproducibility. Biometrics 45: 255-268.
#' @references
#' Lin L (2000). A note on the concordance correlation coefficient.
#' Biometrics 56: 324-325.
#' @references
#' Bland J, Altman D (1986). Statistical methods for assessing agreement
#' between two methods of clinical measurement. The Lancet 327: 307-310.
#' @export
ccc <- function(data, ci = FALSE, conf_level = 0.95,
                na_method = c("error", "complete", "pairwise"),
                n_threads = getOption("matrixCorr.threads", 1L),
                output = c("matrix", "sparse", "edge_list"),
                threshold = 0,
                diag = TRUE,
                verbose = FALSE) {
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )
  check_bool(ci, arg = "ci")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }
  check_bool(verbose, arg = "verbose")
  na_method <- match_arg(na_method, c("error", "complete", "pairwise"), arg_name = "na_method")

  numeric_data <- validate_corr_input(data, check_na = identical(na_method, "error"))
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(
      numeric_data,
      min_n = if (isTRUE(ci)) 3L else 2L,
      arg = "data"
    )
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }
  mat <- numeric_data
  colnames_data <- colnames(numeric_data)
  dn <- .mc_square_dimnames(colnames_data)
  if (!is.null(diagnostics_extra)) {
    diagnostics_extra$n_complete_common <- diagnostics_extra$n_complete
    diagnostics_extra$n_complete <- NULL
  }
  diag_payload <- .mc_merge_diagnostics(list(
    n_complete = matrix(
      as.integer(nrow(mat)),
      nrow = ncol(mat),
      ncol = ncol(mat),
      dimnames = dn
    )
  ), diagnostics_extra)

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  inform_if_verbose(
    "Using {openmp_threads()} OpenMP thread{?s}.",
    .verbose = verbose
  )

  if (!isTRUE(ci) && .mc_supports_direct_threshold_path(
    method = "ccc",
    na_method = na_method,
    ci = FALSE,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    pairwise = FALSE,
    has_ci = FALSE
  )) {
    trip <- ccc_threshold_triplets_cpp(
      mat,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag
    )
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = "ccc",
      method = "Lin's concordance",
      description = "Pairwise Lin's concordance correlation matrix",
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = as.integer(c(ncol(mat), ncol(mat))),
      source_dimnames = dn,
      diagnostics = diag_payload,
      symmetric = TRUE
    ))
  }

  if (identical(na_method, "pairwise")) {
    ccc_lin <- .mc_ccc_pairwise(
      mat = mat,
      colnames_data = colnames_data,
      conf_level = conf_level,
      ci = ci
    )
    diag_payload <- ccc_lin$diagnostics
    if (isTRUE(ci)) {
      ccc_out <- structure(
        list(est = ccc_lin$est, lwr.ci = ccc_lin$lwr.ci, upr.ci = ccc_lin$upr.ci),
        class = c("ccc", "ccc_ci"),
        method = "Lin's concordance",
        description = "Pairwise Lin's concordance with confidence intervals",
        package = "matrixCorr",
        conf.level = conf_level,
        ci.method = "lin_delta_fisher_z",
        diagnostics = diag_payload
      )
      if (identical(output_cfg$output, "matrix")) {
        return(ccc_out)
      }
      dense_obj <- .mc_structure_corr_matrix(
        ccc_lin$est,
        class_name = "ccc",
        method = "Lin's concordance",
        description = "Pairwise Lin's concordance with confidence intervals",
        diagnostics = diag_payload,
        dimnames = dn,
        symmetric = TRUE,
        classes = c("ccc", "matrix"),
        extra_attrs = list(
          ci = list(
            est = ccc_lin$est,
            lwr.ci = ccc_lin$lwr.ci,
            upr.ci = ccc_lin$upr.ci,
            conf.level = conf_level,
            ci.method = "lin_delta_fisher_z"
          ),
          conf.level = conf_level,
          ci.method = "lin_delta_fisher_z"
        )
      )
      return(.mc_finalize_corr_output_fast(
        dense_obj,
        output = output_cfg$output,
        threshold = output_cfg$threshold,
        diag = output_cfg$diag
      ))
    }
    dense_obj <- .mc_new_corr_matrix(
      mat = ccc_lin$est,
      estimator_class = "ccc",
      method = "Lin's concordance",
      description = "Pairwise Lin's concordance correlation matrix",
      diagnostics = diag_payload,
      symmetric = TRUE
    )
    return(.mc_finalize_corr_output_fast(
      dense_obj,
      output = output_cfg$output,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag
    ))
  }

  if (ci) {
    if (nrow(mat) <= 2L) {
      abort_bad_arg("data",
        message = "must provide at least three observations per variable when {.arg ci} = TRUE.",
        .hint   = "Increase the sample size or set ci = FALSE to obtain point estimates only."
      )
    }
    ccc_lin <- ccc_with_ci_cpp(mat, conf_level)
    diag(ccc_lin$est)    <- 1
    diag(ccc_lin$lwr.ci) <- 1
    diag(ccc_lin$upr.ci) <- 1
    ccc_lin$est <- .mc_set_matrix_dimnames(ccc_lin$est, colnames_data)
    ccc_lin$lwr.ci <- .mc_set_matrix_dimnames(ccc_lin$lwr.ci, colnames_data)
    ccc_lin$upr.ci <- .mc_set_matrix_dimnames(ccc_lin$upr.ci, colnames_data)

    ccc_lin <- structure(
      ccc_lin,
      class = c("ccc", "ccc_ci"),
      method = "Lin's concordance",
      description = "Pairwise Lin's concordance with confidence intervals",
      package = "matrixCorr",
      conf.level = conf_level,
      ci.method = "lin_delta_fisher_z",
      diagnostics = diag_payload
    )
    if (!identical(output_cfg$output, "matrix")) {
      ci_attr <- list(
        est = .mc_set_matrix_dimnames(unclass(ccc_lin$est), colnames_data),
        lwr.ci = .mc_set_matrix_dimnames(unclass(ccc_lin$lwr.ci), colnames_data),
        upr.ci = .mc_set_matrix_dimnames(unclass(ccc_lin$upr.ci), colnames_data),
        conf.level = conf_level,
        ci.method = "lin_delta_fisher_z"
      )
      dense_obj <- .mc_structure_corr_matrix(
        ccc_lin$est,
        class_name = "ccc",
        method = "Lin's concordance",
        description = "Pairwise Lin's concordance with confidence intervals",
        diagnostics = diag_payload,
        dimnames = dn,
        symmetric = TRUE,
        classes = c("ccc", "matrix"),
        extra_attrs = list(ci = ci_attr, conf.level = conf_level, ci.method = "lin_delta_fisher_z")
      )
      return(.mc_finalize_corr_output_fast(
        dense_obj,
        output = output_cfg$output,
        threshold = output_cfg$threshold,
        diag = output_cfg$diag
      ))
    }
  } else {
    est <- ccc_cpp(mat)
    est <- .mc_set_matrix_dimnames(est, colnames_data)
    ccc_lin <- .mc_new_corr_matrix(
      mat = est,
      estimator_class = "ccc",
      method = "Lin's concordance",
      description = "Pairwise Lin's concordance correlation matrix",
      diagnostics = diag_payload,
      symmetric = TRUE
    )
    if (!identical(output_cfg$output, "matrix")) {
      ccc_lin <- .mc_finalize_corr_output_fast(
        ccc_lin,
        output = output_cfg$output,
        threshold = output_cfg$threshold,
        diag = output_cfg$diag
      )
    }
  }

  ccc_lin
}

.mc_ccc_pairwise <- function(mat, colnames_data = NULL, conf_level = 0.95, ci = FALSE) {
  p <- ncol(mat)
  dn <- .mc_square_dimnames(colnames_data)
  est <- matrix(NA_real_, p, p, dimnames = dn)
  lwr <- matrix(NA_real_, p, p, dimnames = dn)
  upr <- matrix(NA_real_, p, p, dimnames = dn)
  n_complete <- matrix(0L, p, p, dimnames = dn)
  diag(est) <- 1
  diag(lwr) <- 1
  diag(upr) <- 1

  for (i in seq_len(p)) {
    xi <- mat[, i]
    ok_i <- is.finite(xi)
    n_complete[i, i] <- sum(ok_i)
    if (sum(ok_i) < 2L || stats::var(xi[ok_i]) == 0) {
      est[i, i] <- NA_real_
      lwr[i, i] <- NA_real_
      upr[i, i] <- NA_real_
    }
  }

  if (p >= 2L) {
    for (i in seq_len(p - 1L)) {
      for (j in (i + 1L):p) {
        keep <- is.finite(mat[, i]) & is.finite(mat[, j])
        n_ij <- sum(keep)
        n_complete[i, j] <- n_complete[j, i] <- as.integer(n_ij)
        min_n <- if (isTRUE(ci)) 3L else 2L
        if (n_ij < min_n) {
          next
        }
        pair_mat <- mat[keep, c(i, j), drop = FALSE]
        if (stats::var(pair_mat[, 1L]) == 0 || stats::var(pair_mat[, 2L]) == 0) {
          next
        }
        if (isTRUE(ci)) {
          pair_ci <- ccc_with_ci_cpp(pair_mat, conf_level)
          est[i, j] <- est[j, i] <- pair_ci$est[1L, 2L]
          lwr[i, j] <- lwr[j, i] <- pair_ci$lwr.ci[1L, 2L]
          upr[i, j] <- upr[j, i] <- pair_ci$upr.ci[1L, 2L]
        } else {
          pair_est <- ccc_cpp(pair_mat)
          est[i, j] <- est[j, i] <- pair_est[1L, 2L]
        }
      }
    }
  }

  list(
    est = est,
    lwr.ci = lwr,
    upr.ci = upr,
    diagnostics = list(
      n_complete = n_complete,
      na_method = "pairwise"
    )
  )
}


#' @rdname ccc
#' @method print ccc
#' @param digits Integer; decimals for CCC estimates (default 4).
#' @param ci_digits Integer; decimals for CI bounds (default 4).
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Passed to \code{\link[base]{print.data.frame}}.
#' @export
print.ccc <- function(x,
                      digits = 4,
                      ci_digits = 4,
                      n = NULL,
                      topn = NULL,
                      max_vars = NULL,
                      width = NULL,
                      show_ci = NULL,
                      ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )

  # -- identify object type ----------------------------------------------------
  is_ci_obj <- inherits(x, "ccc_ci") ||
    (is.list(x) && all(c("est", "lwr.ci", "upr.ci") %in% names(x)))

  if (is_ci_obj) {
    est <- as.matrix(x$est)
    lwr <- as.matrix(x$lwr.ci)
    upr <- as.matrix(x$upr.ci)
  } else if (is.matrix(x)) {
    est <- as.matrix(x)
    lwr <- matrix(NA_real_, nrow(est), ncol(est), dimnames = dimnames(est))
    upr <- lwr
  } else {
    abort_bad_arg("x",
      message = "must be a matrix or a list with elements `est`, `lwr.ci`, and `upr.ci`."
    )
  }

  rn <- rownames(est); cn <- colnames(est)
  if (is.null(rn)) rn <- paste0("m", seq_len(nrow(est)))
  if (is.null(cn)) cn <- rn

  .mc_print_corr_matrix(
    x,
    header = "Lin's concordance correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = est,
    ...
  )
  invisible(x)
}

#' @rdname ccc
#' @method summary ccc
#' @param object A \code{"ccc"} or \code{"ccc_ci"} object to summarize.
#' @param digits Integer; decimals for CCC estimates (default 4).
#' @param ci_digits Integer; decimals for CI bounds (default 2).
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Ignored.
#' @return For \code{summary.ccc}, a data frame with columns
#'   \code{item1}, \code{item2}, \code{estimate}, and (optionally)
#'   \code{lwr}, \code{upr}, plus \code{n_complete} when available.
#' @export
summary.ccc <- function(object,
                        digits = 4,
                        ci_digits = 2,
                        n = NULL,
                        topn = NULL,
                        max_vars = NULL,
                        width = NULL,
                        show_ci = NULL,
                        ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  # detect CI container
  is_ci_obj <- inherits(object, "ccc_ci") ||
    (is.list(object) && all(c("est", "lwr.ci", "upr.ci") %in% names(object)))

  if (is_ci_obj) {
    est <- as.matrix(object$est)
    lwr <- as.matrix(object$lwr.ci)
    upr <- as.matrix(object$upr.ci)
    conf_level <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  } else if (is.matrix(object)) {
    est <- as.matrix(object)
    lwr <- matrix(NA_real_, nrow(est), ncol(est), dimnames = dimnames(est))
    upr <- lwr
    conf_level <- NA_real_
  } else {
    abort_bad_arg("object",
      message = "must be a matrix or a list with elements `est`, `lwr.ci`, and `upr.ci`."
    )
  }

  # labels (fallback if missing)
  rn <- rownames(est); cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  # decide whether to include CI columns
  has_any_ci <- any(is.finite(lwr) | is.finite(upr))
  include_ci <- identical(show_ci, "yes") && has_any_ci
  diag_attr <- attr(object, "diagnostics", exact = TRUE)

  # 1x1 case
  if (nrow(est) == 1L && ncol(est) == 1L) {
    df <- data.frame(
      method1  = rn[1],
      method2  = cn[1],
      estimate = round(est[1, 1], digits),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
      df$n_complete <- as.integer(diag_attr$n_complete[1, 1])
    }
    if (include_ci) {
      df$lwr <- if (is.na(lwr[1,1])) NA_real_ else round(lwr[1,1], ci_digits)
      df$upr <- if (is.na(upr[1,1])) NA_real_ else round(upr[1,1], ci_digits)
    }
  } else {
    # long table over i<j
    rows <- vector("list", nrow(est) * (ncol(est) - 1L) / 2L); k <- 0L
    for (i in seq_len(nrow(est) - 1L)) {
      for (j in (i + 1L):ncol(est)) {
        k <- k + 1L
        rec <- list(
          method1  = rn[i],
          method2  = cn[j],
          estimate = round(est[i, j], digits)
        )
        if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
          rec$n_complete <- as.integer(diag_attr$n_complete[i, j])
        }
        if (include_ci) {
          rec$lwr <- if (is.na(lwr[i, j])) NA_real_ else round(lwr[i, j], ci_digits)
          rec$upr <- if (is.na(upr[i, j])) NA_real_ else round(upr[i, j], ci_digits)
        }
        rows[[k]] <- rec
      }
    }
    df <- do.call(rbind.data.frame, rows)
    rownames(df) <- NULL
    # ensure proper column types
    num_cols <- c("estimate", if (include_ci) c("lwr","upr"))
    for (nm in num_cols) df[[nm]] <- as.numeric(df[[nm]])
  }

  # carry attrs for printing
  df <- .mc_finalize_summary_df(df, class_name = "summary.ccc")
  attr(df, "overview") <- .mc_summary_corr_matrix(est, topn = topn)
  attr(df, "conf.level") <- if (is.finite(conf_level)) conf_level else NA_real_
  attr(df, "has_ci")     <- isTRUE(include_ci)
  attr(df, "digits")     <- digits
  attr(df, "ci_digits")  <- ci_digits
  df
}

#' @rdname ccc
#' @method print summary.ccc
#' @param ... Passed to \code{\link[base]{print.data.frame}}.
#' @export
print.summary.ccc <- function(x, digits = NULL, n = NULL,
                              topn = NULL, max_vars = NULL,
                              width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Lin's concordance summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "lin_delta_fisher_z",
    ...
  )
  invisible(x)
}


#' @rdname ccc
#' @method plot ccc
#' @param x An object of class \code{"ccc"} (either a matrix or a list with CIs).
#' @param title Title for the plot.
#' @param low_color Color for low CCC values.
#' @param high_color Color for high CCC values.
#' @param mid_color Color for mid CCC values.
#' @param value_text_size Text size for CCC values in the heatmap.
#' @param ci_text_size Text size for confidence intervals.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param ... Passed to \code{ggplot2::theme()}.
#' @export
plot.ccc <- function(x,
                     title = "Lin's Concordance Correlation Heatmap",
                     low_color = "indianred1",
                     high_color = "steelblue1",
                     mid_color = "white",
                     value_text_size = 4,
                     ci_text_size = 3,
                     show_value = TRUE,
                     ...) {

  check_inherits(x, "ccc")
  check_bool(show_value, arg = "show_value")

  # --- Build long data with proper alignment by (Var1, Var2) ---
  est_mat <- if (is.list(x) && !is.null(x$est)) x$est else unclass(x)
  df_est  <- as.data.frame(as.table(est_mat))
  names(df_est) <- c("Var1", "Var2", "CCC")

  if (is.list(x) && !is.null(x$lwr.ci) && !is.null(x$upr.ci)) {
    df_lwr <- as.data.frame(as.table(x$lwr.ci)); names(df_lwr)[3] <- "lwr"
    df_upr <- as.data.frame(as.table(x$upr.ci)); names(df_upr)[3] <- "upr"
    df <- Reduce(function(a, b) merge(a, b, by = c("Var1","Var2"), all = TRUE),
                 list(df_est, df_lwr, df_upr))

    # Blank CI on the diagonal (show 1.00 but no CI there)
    diag_idx <- df$Var1 == df$Var2
    df$lwr[diag_idx] <- NA_real_
    df$upr[diag_idx] <- NA_real_
    df$ci_label <- ifelse(is.na(df$lwr) | is.na(df$upr),
                          NA_character_,
                          sprintf("(%.2f, %.2f)", df$lwr, df$upr))
  } else {
    df <- df_est
    df$ci_label <- NA_character_
  }

  # Reverse Y axis for heatmap look, keep X in natural order
  lev_row <- unique(df_est$Var1)
  lev_col <- unique(df_est$Var2)
  df$Var1 <- factor(df$Var1, levels = rev(lev_row))
  df$Var2 <- factor(df$Var2, levels = lev_col)

  # Main estimate label
  df$label <- sprintf("%.2f", df$CCC)

  # --- Plot ---
  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = CCC)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color, high = high_color, mid = mid_color,
      midpoint = 0, limits = c(-1, 1), name = "CCC"
    ) +
    ggplot2::coord_fixed() +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value)) {
    p <- p + ggplot2::geom_text(ggplot2::aes(label = label), size = value_text_size)
  }

  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.25),
      size = ci_text_size, color = "gray30", na.rm = TRUE
    )
  }

  p
}

#' @title Pairwise Weighted Cohen's Kappa for Ordered Ratings
#'
#' @description
#' Computes weighted Cohen's kappa for either a pair of ordinal rating vectors
#' or all pairwise combinations of ordinal columns in a matrix or data frame.
#'
#' @param data In matrix mode, a matrix or data frame whose rows are
#'   observational units and whose columns are raters or classifiers. Each
#'   column contains ordered category ratings. In two-vector mode, the first
#'   ordinal rating vector.
#' @param y Optional second ordinal rating vector. When supplied, the function
#'   returns a single weighted kappa estimate for \code{data} and \code{y}.
#' @param weights Weight specification. The default \code{"quadratic"} uses
#'   Fleiss-Cohen-style agreement weights. \code{"linear"} uses equal-spacing
#'   agreement weights. \code{"unweighted"} reduces to ordinary Cohen's kappa.
#'   Custom numeric weight matrices must be symmetric agreement weights with
#'   diagonal 1 and entries in \code{[0, 1]}.
#' @param levels Optional ordered category levels. Weighted kappa depends on
#'   category order and will not silently alphabetise arbitrary labels. If
#'   omitted, order is inferred only when all involved ratings are ordered
#'   factors with identical levels or when all involved ratings are numeric or
#'   integer and can be ordered by their sorted unique values.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing values. \code{"complete"} applies listwise
#'   deletion across retained columns before computing the matrix. In matrix
#'   mode, \code{"pairwise"} uses pair-specific complete observations.
#' @param ci Logical; if \code{TRUE}, attach large-sample delta-method
#'   confidence intervals.
#' @param p_value Logical; if \code{TRUE}, attach large-sample delta-method
#'   Wald test statistics and p-values for \eqn{H_0: \kappa_w = 0}.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#'   \code{0.95}.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   matrix C++ backend. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for matrix mode.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries after thresholding.
#'   \item \code{"edge_list"}: long-form data frame representation.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   retain entries with \code{abs(value) >= threshold}. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in sparse and
#'   edge-list outputs.
#' @param ... Reserved for future extensions. Unsupported extra arguments are
#'   rejected.
#'
#' @details
#' This implementation uses agreement/similarity weights internally:
#' \itemize{
#'   \item \code{"unweighted"}: \eqn{w_{ij} = 1(i = j)}
#'   \item \code{"linear"}: \eqn{w_{ij} = 1 - |i - j| / (K - 1)}
#'   \item \code{"quadratic"}: \eqn{w_{ij} = 1 - (i - j)^2 / (K - 1)^2}
#'   }
#' The \code{"quadratic"} scheme is the default and corresponds to
#' Fleiss-Cohen-style agreement weights. The \code{"linear"} scheme
#' corresponds to equal-spacing agreement weights.
#'
#' For matrix mode, rows are shared observational units and columns are raters
#' or classifiers. A common category map is resolved in R and the main
#' computation is performed in C++. Missing-data handling follows the usual
#' \pkg{matrixCorr} \code{na_method} conventions. Pairwise complete counts are
#' stored in \code{attr(x, "diagnostics")$n_complete}.
#'
#' \strong{Confidence intervals and standard errors.}
#' Confidence intervals and p-values use the exact large-sample multinomial
#' delta-method formula. Let
#' \eqn{A_w = \sum_{a,b} w_{ab} p_{ab}} be the observed weighted agreement,
#' \eqn{E_w = \sum_{a,b} w_{ab} q_a r_b} the expected weighted agreement,
#' and \eqn{D = 1 - E_w}. Define the weighted margin summaries
#' \deqn{ u_a = \sum_b w_{ab} r_b, \qquad v_b = \sum_a w_{ab} q_a. }
#' For each cell \eqn{(a,b)}, the backend uses
#' \deqn{ g_{ab}
#'       \;=\;
#'       \frac{D\,w_{ab} + (A_w - 1)\,(u_a + v_b)}{D^2}. }
#' The variance estimator is
#' \deqn{ \widehat{\mathrm{Var}}(\hat\kappa_w)
#'       \;=\;
#'       \frac{1}{n}\left(
#'       \sum_{a,b} p_{ab} g_{ab}^2 -
#'       \Big(\sum_{a,b} p_{ab} g_{ab}\Big)^2
#'       \right), }
#' where \eqn{n} is the number of complete paired ratings. The standard error
#' is \eqn{\widehat{\mathrm{se}}(\hat\kappa_w)=
#' \sqrt{\widehat{\mathrm{Var}}(\hat\kappa_w)}} and the CI is the Wald interval
#' \deqn{ \hat\kappa_w \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa_w), }
#' truncated to \eqn{[-1, 1]} in the returned result. Use [cohen_kappa()] for
#' unordered nominal categories where all disagreements are equally serious.
#'
#' @return
#' If \code{y} is supplied, a scalar S3 object of class
#' \code{c("weighted_kappa", "numeric")} is returned with diagnostics and
#' optional \code{ci} and \code{inference} attributes. Otherwise a symmetric
#' matrix-style result with estimator class \code{weighted_kappa}.
#'
#' @references
#' Cohen, J. (1968). Weighted kappa: Nominal scale agreement with provision for
#' scaled disagreement or partial credit. \emph{Psychological Bulletin},
#' 70(4), 213-220. \doi{10.1037/h0026256}
#'
#' @seealso
#' [cohen_kappa()] for unweighted two-rater nominal agreement;
#' [multirater_kappa()] for panel-level nominal agreement among three or more
#' raters.
#'
#' @examples
#' raters <- data.frame(
#'   r1 = ordered(c("low", "low", "mid", "high", "high"),
#'                levels = c("low", "mid", "high")),
#'   r2 = ordered(c("low", "mid", "mid", "high", "high"),
#'                levels = c("low", "mid", "high")),
#'   r3 = ordered(c("low", "low", "high", "high", "mid"),
#'                levels = c("low", "mid", "high"))
#' )
#'
#' wk <- weighted_kappa(raters)
#' print(wk)
#' summary(wk)
#' estimate(wk)
#' tidy(wk)
#' plot(wk)
#'
#' x <- raters$r1
#' y <- raters$r2
#' weighted_kappa(x, y, weights = "linear")
#'
#' @author Thiago de Paula Oliveira
#' @export
weighted_kappa <- function(data,
                           y = NULL,
                           weights = c("quadratic", "linear", "unweighted"),
                           levels = NULL,
                           na_method = c("error", "pairwise", "complete"),
                           ci = FALSE,
                           p_value = FALSE,
                           conf_level = 0.95,
                           n_threads = getOption("matrixCorr.threads", 1L),
                           output = c("matrix", "sparse", "edge_list"),
                           threshold = 0,
                           diag = TRUE,
                           ...) {
  .mc_extract_legacy_aliases(list(...), allowed = character())
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )
  na_method <- validate_na_method(
    na_method,
    arg = "na_method",
    allowed = c("error", "pairwise", "complete")
  )
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  enc <- .mc_encode_weighted_kappa_data(
    data = data,
    y = y,
    levels = levels,
    na_method = na_method
  )
  W <- .mc_resolve_weighted_kappa_weights(weights, enc$n_levels)

  if (!is.null(y)) {
    if (!identical(output_cfg$output, "matrix")) {
      abort_bad_arg(
        "output",
        message = "must be {.val matrix} when {.arg y} is supplied."
      )
    }

    x_codes <- enc$x
    y_codes <- enc$y
    if (identical(na_method, "error") && (anyNA(x_codes) || anyNA(y_codes))) {
      abort_bad_arg(
        "data",
        message = "and {.arg y} must not contain missing values when {.arg na_method} is {.val error}.",
        .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to drop incomplete pairs."
      )
    }
    if (!identical(na_method, "error")) {
      keep <- stats::complete.cases(x_codes, y_codes)
      x_codes <- x_codes[keep]
      y_codes <- y_codes[keep]
    }

    fit <- weighted_kappa_pair_cpp(
      x_codes,
      y_codes,
      W,
      return_inference = isTRUE(ci) || isTRUE(p_value),
      conf_level = conf_level
    )
    return(.mc_weighted_kappa_scalar(
      fit,
      levels = enc$levels,
      weights = W,
      ci = ci,
      p_value = p_value,
      conf_level = conf_level
    ))
  }

  X <- enc$X
  colnames_data <- enc$diagnostics$colnames_data
  dn <- .mc_square_dimnames(colnames_data)

  if (identical(na_method, "error") && anyNA(X)) {
    abort_bad_arg(
      "data",
      message = "contains missing values when {.arg na_method} is {.val error}.",
      .hint = "Use {.arg na_method} = {.val pairwise} or {.val complete} to handle incomplete units."
    )
  }

  listwise_meta <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_weighted_kappa_complete_cases(X, min_n = 2L)
    X <- cc$X
    listwise_meta <- cc$diagnostics
  }

  direct_threshold <- .mc_supports_direct_threshold_path(
    method = "weighted_kappa",
    na_method = na_method,
    ci = ci,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    symmetric = TRUE,
    exact = TRUE,
    has_inference = p_value
  )

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  if (direct_threshold) {
    trip <- weighted_kappa_threshold_triplets_cpp(
      X,
      W,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      n_threads = n_threads
    )
    n_complete_mat <- matrix(
      as.integer(nrow(X)),
      nrow = ncol(X),
      ncol = ncol(X),
      dimnames = dn
    )
    diag(n_complete_mat) <- colSums(!is.na(X))
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = "weighted_kappa",
      method = "weighted_kappa",
      description = "Pairwise weighted Cohen's kappa agreement matrix",
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = as.integer(c(ncol(X), ncol(X))),
      source_dimnames = dn,
      diagnostics = list(
        n_complete = n_complete_mat,
        levels = enc$levels,
        weights = .mc_weighted_kappa_weights_for_diagnostics(W, enc$levels),
        weight_type = attr(W, "weight_type", exact = TRUE)
      ),
      symmetric = TRUE
    ))
  }

  fit <- weighted_kappa_matrix_cpp(
    X,
    W,
    pairwise_complete = identical(na_method, "pairwise"),
    return_inference = isTRUE(ci) || isTRUE(p_value),
    conf_level = conf_level,
    n_threads = n_threads
  )
  out <- .mc_structure_weighted_kappa_matrix(
    raw = fit,
    colnames_data = colnames_data,
    weights = W,
    levels = enc$levels,
    ci = ci,
    p_value = p_value,
    conf_level = conf_level
  )
  attr(out, "diagnostics") <- .mc_merge_diagnostics(
    attr(out, "diagnostics", exact = TRUE),
    if (identical(na_method, "pairwise")) {
      list(
        na_method = "pairwise",
        common_sample = FALSE
      )
    } else {
      listwise_meta
    }
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_weighted_kappa_prepare_vector <- function(x,
                                              arg = as.character(substitute(x))) {
  if (is.list(x) && !is.data.frame(x)) {
    abort_bad_arg(arg, message = "must not be a list.")
  }
  if (inherits(x, "Date") || inherits(x, "POSIXt") || inherits(x, "difftime")) {
    abort_bad_arg(
      arg,
      message = "must not be a Date/POSIXct/POSIXlt/difftime object."
    )
  }

  if (is.factor(x)) {
    key <- as.character(x)
    key[is.na(x)] <- NA_character_
    return(list(
      key = key,
      is_ordered = is.ordered(x),
      ordered_levels = if (is.ordered(x)) levels(x) else NULL,
      numeric_values = NULL,
      numeric_infer = FALSE
    ))
  }

  if (is.numeric(x) || is.integer(x)) {
    if (any(!is.na(x) & !is.finite(x))) {
      abort_bad_arg(arg, message = "must not contain NaN or infinite values.")
    }
    key <- as.character(x)
    key[is.na(x)] <- NA_character_
    return(list(
      key = key,
      is_ordered = FALSE,
      ordered_levels = NULL,
      numeric_values = x,
      numeric_infer = TRUE
    ))
  }

  if (is.character(x) || is.logical(x)) {
    key <- as.character(x)
    key[is.na(x)] <- NA_character_
    return(list(
      key = key,
      is_ordered = FALSE,
      ordered_levels = NULL,
      numeric_values = NULL,
      numeric_infer = FALSE
    ))
  }

  abort_bad_arg(
    arg,
    message = paste(
      "must be an ordered factor, factor, character, logical, integer,",
      "or numeric vector."
    )
  )
}

.mc_weighted_kappa_normalize_levels <- function(levels) {
  if (is.null(levels)) {
    return(NULL)
  }
  if (is.list(levels) || is.matrix(levels)) {
    abort_bad_arg("levels", message = "must be an atomic vector of ordered categories.")
  }
  lev <- if (is.factor(levels)) as.character(levels) else levels
  if (!is.atomic(lev)) {
    abort_bad_arg("levels", message = "must be an atomic vector of ordered categories.")
  }
  if (!length(lev)) {
    abort_bad_arg("levels", message = "must contain at least one category.")
  }
  if (anyNA(lev)) {
    abort_bad_arg("levels", message = "must not contain missing values.")
  }
  lev_key <- as.character(lev)
  if (anyDuplicated(lev_key)) {
    abort_bad_arg("levels", message = "must not contain duplicate categories.")
  }
  lev
}

.mc_weighted_kappa_resolve_levels <- function(prepared, levels = NULL) {
  if (!is.null(levels)) {
    return(.mc_weighted_kappa_normalize_levels(levels))
  }

  if (all(vapply(prepared, `[[`, logical(1), "is_ordered"))) {
    base_levels <- prepared[[1L]]$ordered_levels
    same_levels <- vapply(
      prepared,
      function(z) identical(z$ordered_levels, base_levels),
      logical(1)
    )
    if (all(same_levels)) {
      return(base_levels)
    }
  }

  if (all(vapply(prepared, `[[`, logical(1), "numeric_infer"))) {
    vals <- unlist(lapply(prepared, function(z) z$numeric_values[!is.na(z$numeric_values)]), use.names = FALSE)
    if (!length(vals)) {
      abort_bad_arg(
        "levels",
        message = "must be supplied when no non-missing ratings are available to infer the category order."
      )
    }
    return(sort(unique(vals)))
  }

  abort_bad_arg(
    "levels",
    message = paste(
      "must be supplied when the category order cannot be inferred.",
      "Automatic order inference is limited to identical ordered-factor levels",
      "or numeric/integer ratings."
    )
  )
}

.mc_weighted_kappa_weights_for_diagnostics <- function(weights, levels) {
  out <- unclass(weights)
  dimnames(out) <- list(as.character(levels), as.character(levels))
  out
}

.mc_weighted_kappa_scalar <- function(fit,
                                      levels,
                                      weights,
                                      ci = FALSE,
                                      p_value = FALSE,
                                      conf_level = 0.95) {
  est <- as.numeric(fit$estimate)
  out <- structure(est, class = c("weighted_kappa", "numeric"))
  attr(out, "method") <- "weighted_kappa"
  attr(out, "description") <- "Pairwise weighted Cohen's kappa agreement"
  attr(out, "package") <- "matrixCorr"
  attr(out, "estimate") <- est
  attr(out, "levels") <- levels
  attr(out, "weights") <- .mc_weighted_kappa_weights_for_diagnostics(weights, levels)
  attr(out, "weight_type") <- attr(weights, "weight_type", exact = TRUE)
  attr(out, "diagnostics") <- list(
    n_complete = as.integer(fit$n_complete),
    observed_agreement = as.numeric(fit$observed_agreement),
    expected_agreement = as.numeric(fit$expected_agreement),
    levels = levels,
    weights = .mc_weighted_kappa_weights_for_diagnostics(weights, levels),
    weight_type = attr(weights, "weight_type", exact = TRUE)
  )
  if (isTRUE(ci)) {
    attr(out, "ci") <- list(
      est = est,
      lwr.ci = as.numeric(fit$lwr),
      upr.ci = as.numeric(fit$upr),
      conf.level = conf_level,
      ci.method = "delta"
    )
    attr(out, "conf.level") <- conf_level
    attr(out, "conf_level") <- conf_level
  }
  if (isTRUE(p_value)) {
    attr(out, "inference") <- list(
      method = "delta_wald_weighted_kappa",
      estimate = est,
      se = as.numeric(fit$se),
      statistic = as.numeric(fit$statistic),
      p_value = as.numeric(fit$p_value),
      n_obs = as.integer(fit$n_complete)
    )
  }
  out
}

#' @keywords internal
#' @noRd
.mc_encode_weighted_kappa_data <- function(data,
                                           y = NULL,
                                           levels = NULL,
                                           na_method) {
  validate_na_method(
    na_method,
    arg = "na_method",
    allowed = c("error", "pairwise", "complete")
  )

  if (!is.null(y)) {
    check_same_length(data, y, arg_x = "data", arg_y = "y")
    prepared <- list(
      data = .mc_weighted_kappa_prepare_vector(data, arg = "data"),
      y = .mc_weighted_kappa_prepare_vector(y, arg = "y")
    )
    resolved_levels <- .mc_weighted_kappa_resolve_levels(prepared, levels = levels)
    level_key <- as.character(resolved_levels)

    x_codes <- match(prepared$data$key, level_key)
    y_codes <- match(prepared$y$key, level_key)
    x_codes[is.na(prepared$data$key)] <- NA_integer_
    y_codes[is.na(prepared$y$key)] <- NA_integer_

    bad_x <- !is.na(prepared$data$key) & is.na(x_codes)
    bad_y <- !is.na(prepared$y$key) & is.na(y_codes)
    if (any(bad_x)) {
      abort_bad_arg("data", message = "contains values not present in {.arg levels}.")
    }
    if (any(bad_y)) {
      abort_bad_arg("y", message = "contains values not present in {.arg levels}.")
    }

    return(list(
      x = as.integer(x_codes),
      y = as.integer(y_codes),
      levels = resolved_levels,
      n_levels = length(resolved_levels),
      diagnostics = list(mode = "pair")
    ))
  }

  cols <- NULL
  colnames_data <- NULL
  if (is.data.frame(data)) {
    cols <- unclass(data)
    colnames_data <- names(data)
  } else if (is.matrix(data)) {
    cols <- lapply(seq_len(ncol(data)), function(j) data[, j])
    colnames_data <- colnames(data)
  } else {
    abort_bad_arg(
      "data",
      message = "must be a matrix or data frame in matrix mode."
    )
  }

  if (length(cols) < 2L) {
    abort_bad_arg(
      "data",
      message = "must contain at least 2 rating columns."
    )
  }

  prepared <- vector("list", length(cols))
  for (j in seq_along(cols)) {
    col_arg <- if (!is.null(colnames_data) && nzchar(colnames_data[[j]] %||% "")) {
      sprintf("data$%s", colnames_data[[j]])
    } else {
      sprintf("data[[%d]]", j)
    }
    prepared[[j]] <- .mc_weighted_kappa_prepare_vector(cols[[j]], arg = col_arg)
  }

  resolved_levels <- .mc_weighted_kappa_resolve_levels(prepared, levels = levels)
  level_key <- as.character(resolved_levels)
  X <- matrix(
    NA_integer_,
    nrow = length(prepared[[1L]]$key),
    ncol = length(prepared),
    dimnames = list(NULL, colnames_data)
  )
  for (j in seq_along(prepared)) {
    codes <- match(prepared[[j]]$key, level_key)
    codes[is.na(prepared[[j]]$key)] <- NA_integer_
    bad <- !is.na(prepared[[j]]$key) & is.na(codes)
    if (any(bad)) {
      abort_bad_arg(
        "data",
        message = "contains values not present in {.arg levels}."
      )
    }
    X[, j] <- as.integer(codes)
  }

  list(
    X = X,
    levels = resolved_levels,
    n_levels = length(resolved_levels),
    diagnostics = list(
      mode = "matrix",
      colnames_data = colnames_data
    )
  )
}

#' @keywords internal
#' @noRd
.mc_resolve_weighted_kappa_weights <- function(weights, n_levels) {
  check_scalar_numeric(
    n_levels,
    arg = "n_levels",
    lower = 1,
    closed_lower = TRUE
  )

  if (is.character(weights)) {
    if (length(weights) != 1L) {
      if (identical(weights, c("quadratic", "linear", "unweighted"))) {
        weights <- "quadratic"
      } else {
        abort_bad_arg("weights", message = "must be a single character weight scheme or a numeric matrix.")
      }
    }
    if (is.na(weights)) {
      abort_bad_arg("weights", message = "must be a single character weight scheme or a numeric matrix.")
    }
    alias_map <- c(
      quadratic = "quadratic",
      fleiss_cohen = "quadratic",
      "fleiss-cohen" = "quadratic",
      squared = "quadratic",
      linear = "linear",
      equal_spacing = "linear",
      "equal-spacing" = "linear",
      equal = "linear",
      unweighted = "unweighted"
    )
    key <- tolower(weights[[1L]])
    resolved <- unname(alias_map[key])
    if (!length(resolved) || is.na(resolved)) {
      abort_bad_arg(
        "weights",
        message = paste(
          "must be one of {.val quadratic}, {.val linear},",
          "{.val unweighted}, or a numeric square matrix."
        )
      )
    }

    K <- as.integer(n_levels)
    if (identical(resolved, "unweighted")) {
      W <- diag(1, nrow = K, ncol = K)
    } else {
      idx <- outer(seq_len(K), seq_len(K), "-")
      if (K <= 1L) {
        W <- matrix(1, nrow = K, ncol = K)
      } else if (identical(resolved, "linear")) {
        W <- 1 - abs(idx) / (K - 1)
      } else {
        W <- 1 - (idx^2) / ((K - 1)^2)
      }
    }
    storage.mode(W) <- "double"
    attr(W, "weight_type") <- resolved
    return(W)
  }

  if (!is.numeric(weights) || !is.matrix(weights)) {
    abort_bad_arg(
      "weights",
      message = "must be a single character weight scheme or a numeric square matrix."
    )
  }
  check_matrix_dims(weights, arg = "weights")
  if (nrow(weights) != ncol(weights)) {
    abort_bad_arg("weights", message = "must be a square matrix.")
  }
  if (nrow(weights) != as.integer(n_levels)) {
    abort_bad_arg(
      "weights",
      message = "must have dimension {n_levels} x {n_levels}.",
      n_levels = as.integer(n_levels)
    )
  }
  if (any(!is.finite(weights))) {
    abort_bad_arg("weights", message = "must contain only finite values.")
  }
  tol <- .Machine$double.eps^0.5
  if (any(abs(diag(weights) - 1) > tol)) {
    abort_bad_arg("weights", message = "must have diagonal entries equal to 1.")
  }
  if (any(weights < -tol | weights > 1 + tol)) {
    abort_bad_arg("weights", message = "must have all values in the interval [0, 1].")
  }
  if (!isTRUE(isSymmetric(weights, tol = tol))) {
    abort_bad_arg(
      "weights",
      message = "must be symmetric for this matrix-oriented implementation.",
      .hint = "Asymmetric custom agreement weights are not yet supported."
    )
  }

  W <- unclass(weights)
  storage.mode(W) <- "double"
  attr(W, "weight_type") <- "custom"
  W
}

#' @keywords internal
#' @noRd
.mc_weighted_kappa_complete_cases <- function(X, min_n = 2L) {
  keep <- stats::complete.cases(X)
  list(
    X = X[keep, , drop = FALSE],
    keep = keep,
    diagnostics = list(
      na_method = "complete",
      n_original = nrow(X),
      n_complete_rows = sum(keep),
      n_removed = nrow(X) - sum(keep),
      complete_rows = keep,
      common_sample = TRUE,
      min_n = as.integer(min_n)
    )
  )
}

#' @keywords internal
#' @noRd
.mc_structure_weighted_kappa_matrix <- function(raw,
                                                colnames_data,
                                                weights,
                                                levels,
                                                ci,
                                                p_value,
                                                conf_level) {
  est <- .mc_set_matrix_dimnames(raw$est, colnames_data)
  n_complete <- .mc_set_matrix_dimnames(raw$n_complete, colnames_data)
  observed_agreement <- .mc_set_matrix_dimnames(raw$observed_agreement, colnames_data)
  expected_agreement <- .mc_set_matrix_dimnames(raw$expected_agreement, colnames_data)

  ci_attr <- NULL
  inference_attr <- NULL
  if (!is.null(raw$se)) {
    se <- .mc_set_matrix_dimnames(raw$se, colnames_data)
    lwr <- .mc_set_matrix_dimnames(raw$lwr, colnames_data)
    upr <- .mc_set_matrix_dimnames(raw$upr, colnames_data)
    statistic <- .mc_set_matrix_dimnames(raw$statistic, colnames_data)
    p_mat <- .mc_set_matrix_dimnames(raw$p_value, colnames_data)
    if (isTRUE(ci)) {
      ci_attr <- list(
        est = est,
        lwr.ci = lwr,
        upr.ci = upr,
        conf.level = conf_level,
        ci.method = "delta"
      )
    }
    if (isTRUE(p_value)) {
      inference_attr <- list(
        estimate = est,
        se = se,
        statistic = statistic,
        p_value = p_mat,
        n_obs = n_complete
      )
    }
  }

  .mc_new_corr_matrix(
    mat = est,
    estimator_class = "weighted_kappa",
    method = "weighted_kappa",
    description = "Pairwise weighted Cohen's kappa agreement matrix",
    diagnostics = list(
      n_complete = n_complete,
      observed_agreement = observed_agreement,
      expected_agreement = expected_agreement,
      levels = levels,
      weights = .mc_weighted_kappa_weights_for_diagnostics(weights, levels),
      weight_type = attr(weights, "weight_type", exact = TRUE)
    ),
    ci = ci_attr,
    conf.level = if (isTRUE(ci)) conf_level else NULL,
    symmetric = TRUE,
    extra_attrs = list(inference = inference_attr)
  )
}

#' @rdname weighted_kappa
#' @method print weighted_kappa
#' @param x A matrix-style \code{weighted_kappa} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.weighted_kappa <- function(x,
                                 digits = 4,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 show_ci = NULL,
                                 ...) {
  if (!is.matrix(x)) {
    diag_attr <- attr(x, "diagnostics", exact = TRUE)
    ci_attr <- attr(x, "ci", exact = TRUE)
    inf_attr <- attr(x, "inference", exact = TRUE)
    cat("Weighted Cohen's kappa agreement\n")
    cat("  kappa              :", formatC(as.numeric(x), format = "f", digits = digits), "\n")
    cat("  weight type        :", attr(x, "weight_type", exact = TRUE) %||% NA_character_, "\n")
    cat("  complete pairs     :", .mc_count_fmt(diag_attr$n_complete %||% NA_integer_), "\n")
    cat("  observed agreement :", formatC(diag_attr$observed_agreement %||% NA_real_, format = "f", digits = digits), "\n")
    cat("  expected agreement :", formatC(diag_attr$expected_agreement %||% NA_real_, format = "f", digits = digits), "\n")
    if (is.list(ci_attr)) {
      cat(
        "  CI                 :",
        sprintf(
          "%g%% [%s, %s]",
          100 * suppressWarnings(as.numeric(ci_attr$conf.level %||% NA_real_)),
          formatC(ci_attr$lwr.ci %||% NA_real_, format = "f", digits = digits),
          formatC(ci_attr$upr.ci %||% NA_real_, format = "f", digits = digits)
        ),
        "\n"
      )
    }
    if (is.list(inf_attr)) {
      cat("  p-value            :", formatC(inf_attr$p_value %||% NA_real_, format = "f", digits = digits), "\n")
    }
    return(invisible(x))
  }
  .mc_print_corr_matrix(
    x,
    header = "Weighted Cohen's kappa agreement matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

manual_weighted_kappa_weights <- function(type, K) {
  idx <- outer(seq_len(K), seq_len(K), "-")
  switch(
    type,
    unweighted = diag(1, K),
    linear = if (K <= 1L) matrix(1, K, K) else 1 - abs(idx) / (K - 1),
    quadratic = if (K <= 1L) matrix(1, K, K) else 1 - (idx^2) / ((K - 1)^2)
  )
}

manual_weighted_kappa <- function(x, y, levels, weight_type = "quadratic") {
  keep <- stats::complete.cases(x, y)
  x <- x[keep]
  y <- y[keep]
  if (length(x) < 2L) {
    return(NA_real_)
  }

  lev_chr <- as.character(levels)
  tab <- table(
    factor(as.character(x), levels = lev_chr),
    factor(as.character(y), levels = lev_chr)
  )
  P <- unclass(tab) / sum(tab)
  W <- manual_weighted_kappa_weights(weight_type, length(levels))
  Ao <- sum(W * P)
  Ae <- sum(W * outer(rowSums(P), colSums(P)))
  if (!is.finite(1 - Ae) || (1 - Ae) <= 0) {
    return(NA_real_)
  }
  (Ao - Ae) / (1 - Ae)
}

test_that("weighted_kappa reproduces a manual two-vector example", {
  lev <- c("low", "mid", "high")
  x <- ordered(c("low", "low", "mid", "mid", "high", "high", "high"), levels = lev)
  y <- ordered(c("low", "mid", "mid", "high", "high", "mid", "high"), levels = lev)

  expected <- manual_weighted_kappa(x, y, levels = lev, weight_type = "linear")
  fit <- weighted_kappa(x, y, weights = "linear")

  expect_s3_class(fit, "weighted_kappa")
  expect_type(fit, "double")
  expect_equal(as.numeric(fit), expected, tolerance = 1e-12)
  expect_true(is.list(attr(fit, "diagnostics", exact = TRUE)))
  expect_identical(attr(fit, "weight_type", exact = TRUE), "linear")
})

test_that("weighted_kappa two-rater regression example matches known estimate and SE", {
  lev <- c("low", "mid", "high")
  x <- ordered(
    c("low", "low", "mid", "mid", "high", "high", "high", "mid", "low", "high", "mid", "low"),
    levels = lev
  )
  y <- ordered(
    c("low", "mid", "mid", "high", "high", "mid", "high", "mid", "low", "high", "low", "mid"),
    levels = lev
  )

  fit <- weighted_kappa(x, y, weights = "quadratic")
  fit_inf <- weighted_kappa(data.frame(a = x, b = y), weights = "quadratic", p_value = TRUE)
  inf <- attr(fit_inf, "inference")

  expect_equal(as.numeric(fit), 0.6666666666666667, tolerance = 1e-12)
  expect_equal(inf$se["a", "b"], 0.1385799032138497, tolerance = 1e-12)
  expect_equal(inf$statistic["a", "b"], 4.810702354423639, tolerance = 1e-12)
})

test_that("weighted_kappa scalar inference uses attributes, not list output", {
  lev <- c("low", "mid", "high")
  x <- ordered(c("low", "low", "mid", "mid", "high", "high", "high"), levels = lev)
  y <- ordered(c("low", "mid", "mid", "high", "high", "mid", "high"), levels = lev)

  fit <- weighted_kappa(x, y, weights = "linear", ci = TRUE, p_value = TRUE)

  expect_s3_class(fit, "weighted_kappa")
  expect_type(fit, "double")
  expect_false(is.list(fit))
  expect_true(is.list(attr(fit, "ci", exact = TRUE)))
  expect_true(is.list(attr(fit, "inference", exact = TRUE)))
  expect_true(all(c("n_complete", "observed_agreement", "expected_agreement") %in% names(attr(fit, "diagnostics", exact = TRUE))))
  expect_identical(attr(fit, "ci", exact = TRUE)$ci.method, "delta")
  expect_identical(attr(fit, "inference", exact = TRUE)$method, "delta_wald_weighted_kappa")
  expect_true(length(capture.output(print(fit))) > 0L)
})

test_that("weighted_kappa resolves linear and quadratic weights and aliases", {
  W_linear <- .mc_resolve_weighted_kappa_weights("linear", 4L)
  attr(W_linear, "weight_type") <- NULL
  W_quadratic <- .mc_resolve_weighted_kappa_weights("quadratic", 4L)
  attr(W_quadratic, "weight_type") <- NULL

  expect_equal(
    W_linear,
    manual_weighted_kappa_weights("linear", 4L),
    tolerance = 1e-12
  )
  expect_equal(
    W_quadratic,
    manual_weighted_kappa_weights("quadratic", 4L),
    tolerance = 1e-12
  )

  expect_equal(
    .mc_resolve_weighted_kappa_weights("equal", 4L),
    .mc_resolve_weighted_kappa_weights("linear", 4L),
    tolerance = 1e-12
  )
  expect_equal(
    .mc_resolve_weighted_kappa_weights("equal_spacing", 4L),
    .mc_resolve_weighted_kappa_weights("linear", 4L),
    tolerance = 1e-12
  )
  expect_equal(
    .mc_resolve_weighted_kappa_weights("equal-spacing", 4L),
    .mc_resolve_weighted_kappa_weights("linear", 4L),
    tolerance = 1e-12
  )

  expect_equal(
    .mc_resolve_weighted_kappa_weights("squared", 4L),
    .mc_resolve_weighted_kappa_weights("quadratic", 4L),
    tolerance = 1e-12
  )
  expect_equal(
    .mc_resolve_weighted_kappa_weights("fleiss_cohen", 4L),
    .mc_resolve_weighted_kappa_weights("quadratic", 4L),
    tolerance = 1e-12
  )
  expect_equal(
    .mc_resolve_weighted_kappa_weights("fleiss-cohen", 4L),
    .mc_resolve_weighted_kappa_weights("quadratic", 4L),
    tolerance = 1e-12
  )
})

test_that("unweighted weighted_kappa matches cohen_kappa", {
  x <- factor(c("A", "A", "B", "B", "C", "A", "B"))
  y <- factor(c("A", "B", "B", "B", "C", "A", "C"))

  expect_equal(
    as.numeric(weighted_kappa(x, y, weights = "unweighted", levels = c("A", "B", "C"))),
    as.numeric(cohen_kappa(x, y)),
    tolerance = 1e-12
  )
})

test_that("weighted_kappa matrix mode is symmetric and matches manual pairwise values", {
  lev <- c("low", "mid", "high")
  raters <- data.frame(
    r1 = ordered(c("low", "low", "mid", "mid", "high", "high", "low", "high"), levels = lev),
    r2 = ordered(c("low", "mid", "mid", "high", "high", "mid", "low", "high"), levels = lev),
    r3 = ordered(c("low", "low", "mid", "high", "high", "high", "low", "mid"), levels = lev),
    r4 = ordered(c("mid", "mid", "mid", "high", "high", "high", "low", "mid"), levels = lev)
  )

  fit <- weighted_kappa(raters)
  mat <- as.matrix(fit)
  expect_true(isSymmetric(mat))
  expect_equal(unname(diag(mat)), rep(1, ncol(raters)))

  expected <- matrix(NA_real_, ncol(raters), ncol(raters), dimnames = dimnames(mat))
  diag(expected) <- 1
  for (j in seq_len(ncol(raters) - 1L)) {
    for (k in (j + 1L):ncol(raters)) {
      expected[j, k] <- manual_weighted_kappa(raters[[j]], raters[[k]], levels = lev)
      expected[k, j] <- expected[j, k]
    }
  }

  expect_equal(as.vector(mat), as.vector(expected), tolerance = 1e-12)
})

test_that("weighted_kappa category order handling follows the documented rules", {
  lev <- c("low", "mid", "high")
  ordered_df <- data.frame(
    a = ordered(c("low", "mid", "high", "mid"), levels = lev),
    b = ordered(c("low", "mid", "mid", "high"), levels = lev)
  )
  ordered_fit <- weighted_kappa(ordered_df)
  expect_equal(attr(ordered_fit, "diagnostics")$levels, lev)

  x_num <- c(2, 1, 3, 2, 1, 3)
  y_num <- c(2, 2, 3, 1, 1, 3)
  numeric_fit <- weighted_kappa(x_num, y_num, weights = "linear")
  expect_equal(
    as.numeric(numeric_fit),
    manual_weighted_kappa(x_num, y_num, levels = sort(unique(c(x_num, y_num))), weight_type = "linear"),
    tolerance = 1e-12
  )

  bad_nominal <- data.frame(
    a = c("low", "mid", "high"),
    b = c("low", "high", "mid"),
    stringsAsFactors = FALSE
  )
  expect_error(weighted_kappa(bad_nominal), "must be supplied")
  expect_error(weighted_kappa(bad_nominal$a, bad_nominal$b), "must be supplied")

  fit_levels <- weighted_kappa(bad_nominal, levels = lev)
  diag_attr <- attr(fit_levels, "diagnostics")
  expect_equal(diag_attr$levels, lev)
  expect_equal(dim(diag_attr$weights), c(3L, 3L))

  fit_unobs <- weighted_kappa(bad_nominal, levels = c("low", "mid", "high", "very_high"))
  expect_equal(dim(attr(fit_unobs, "diagnostics")$weights), c(4L, 4L))
})

test_that("weighted_kappa missing-data modes behave as documented", {
  lev <- c("low", "mid", "high")
  raters <- data.frame(
    a = ordered(c("low", "low", "mid", NA, "high", "low"), levels = lev),
    b = ordered(c("low", "mid", "mid", "high", NA, "low"), levels = lev),
    c = ordered(c("low", "low", NA, "high", "high", "low"), levels = lev)
  )

  expect_error(
    weighted_kappa(raters, na_method = "error"),
    "contains missing values"
  )

  fit_complete <- weighted_kappa(raters, na_method = "complete")
  keep <- stats::complete.cases(raters)
  expected_complete <- matrix(sum(keep), ncol(raters), ncol(raters))
  diag(expected_complete) <- rep(sum(keep), ncol(raters))
  dimnames(expected_complete) <- dimnames(as.matrix(fit_complete))
  expect_equal(attr(fit_complete, "diagnostics")$n_complete, expected_complete)

  fit_pairwise <- weighted_kappa(raters, na_method = "pairwise")
  n_complete <- attr(fit_pairwise, "diagnostics")$n_complete
  expect_equal(n_complete["a", "b"], sum(stats::complete.cases(raters[c("a", "b")])))
  expect_equal(n_complete["a", "c"], sum(stats::complete.cases(raters[c("a", "c")])))
  expect_equal(n_complete["b", "c"], sum(stats::complete.cases(raters[c("b", "c")])))
})

test_that("weighted_kappa attaches CI and inference metadata", {
  lev <- c("low", "mid", "high")
  raters <- data.frame(
    a = ordered(c("low", "low", "mid", "mid", "high", "high", "low", "high", "mid", "high"), levels = lev),
    b = ordered(c("low", "mid", "mid", "high", "high", "mid", "low", "high", "mid", "high"), levels = lev),
    c = ordered(c("low", "low", "mid", "high", "high", "high", "low", "mid", "mid", "high"), levels = lev)
  )

  fit_ci <- weighted_kappa(raters, ci = TRUE)
  ci_attr <- attr(fit_ci, "ci")
  expect_true(is.list(ci_attr))
  expect_true(all(c("est", "lwr.ci", "upr.ci", "conf.level", "ci.method") %in% names(ci_attr)))
  expect_identical(ci_attr$ci.method, "delta")

  fit_p <- weighted_kappa(raters, p_value = TRUE)
  inf_attr <- attr(fit_p, "inference")
  expect_true(is.list(inf_attr))
  expect_true(all(c("estimate", "se", "statistic", "p_value", "n_obs") %in% names(inf_attr)))

  degenerate <- data.frame(
    a = ordered(rep("low", 6), levels = lev),
    b = ordered(rep("low", 6), levels = lev),
    c = ordered(c("low", "mid", "low", "mid", "low", "mid"), levels = lev)
  )
  fit_deg <- weighted_kappa(degenerate, ci = TRUE, p_value = TRUE)
  ci_deg <- attr(fit_deg, "ci")
  inf_deg <- attr(fit_deg, "inference")
  expect_true(is.na(ci_deg$lwr.ci["a", "b"]))
  expect_true(is.na(inf_deg$se["a", "b"]))
})

test_that("weighted_kappa handles degenerate and low-information cases", {
  lev <- c("low", "mid", "high")
  raters <- data.frame(
    a = ordered(rep("low", 6), levels = lev),
    b = ordered(rep("low", 6), levels = lev),
    c = ordered(c("low", "mid", "low", "mid", "low", "mid"), levels = lev)
  )
  fit <- weighted_kappa(raters)
  mat <- as.matrix(fit)
  expect_true(is.na(mat["a", "b"]))
  expect_equal(unname(diag(mat)), c(1, 1, 1))

  low_info <- data.frame(
    a = ordered(c("low", NA), levels = lev),
    b = ordered(c("low", "mid"), levels = lev),
    c = ordered(c("low", "low"), levels = lev)
  )
  fit_low <- weighted_kappa(low_info, na_method = "pairwise")
  mat_low <- as.matrix(fit_low)
  expect_true(is.na(mat_low["a", "b"]))
  expect_true(is.na(mat_low["a", "a"]))
  expect_equal(mat_low["b", "b"], 1)
  expect_equal(mat_low["c", "c"], 1)
})

.mc_corr_n_complete <- function(x) {
  x <- as.matrix(x)
  fin <- is.finite(x)
  n_complete <- crossprod(unclass(fin))
  storage.mode(n_complete) <- "integer"
  n_complete
}

.mc_eval_with_seed <- function(seed, expr) {
  if (is.null(seed)) {
    return(force(expr))
  }

  had_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  if (had_seed) {
    old_seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  }

  on.exit({
    if (had_seed) {
      assign(".Random.seed", old_seed, envir = .GlobalEnv)
    } else if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      rm(".Random.seed", envir = .GlobalEnv)
    }
  }, add = TRUE)

  set.seed(seed)
  force(expr)
}

.mc_seed_offset <- function(seed, offset) {
  if (is.null(seed)) {
    return(NULL)
  }

  span <- .Machine$integer.max - 1L
  out <- ((as.integer(seed) - 1L + as.integer(offset)) %% span) + 1L
  as.integer(out)
}

.mc_percentile_boot_ci <- function(values, conf_level = 0.95) {
  values <- sort(values[is.finite(values)])
  n_boot <- length(values)
  if (!n_boot) {
    return(c(NA_real_, NA_real_))
  }

  alpha <- 1 - conf_level
  ilow <- floor((alpha / 2) * n_boot + 0.5)
  ihi <- floor((1 - alpha / 2) * n_boot + 0.5)
  ilow <- min(max(ilow, 1L), n_boot)
  ihi <- min(max(ihi, 1L), n_boot)
  c(values[[ilow]], values[[ihi]])
}

#' @title Coefficient of Individual Agreement
#'
#' @description
#' `cia()` estimates the coefficient of individual agreement. CIA assesses
#' individual-level interchangeability by comparing between-method disagreement
#' with within-method replicate disagreement. Unlike CCC, CIA is not intended
#' to be driven by between-subject heterogeneity.
#'
#' The estimator requires replicated readings within method. A data set with
#' one observation per subject per method is insufficient for CIA because the
#' within-method disagreement term cannot be estimated.
#'
#' @param data A data frame in long format.
#' @param response Character scalar naming the numeric measurement column.
#' @param subject Character scalar naming the subject/unit identifier column.
#' @param method Character scalar naming the method/device/rater column.
#' @param replicate Character scalar naming the replicate identifier within each
#'   subject-method cell.
#' @param reference Optional character scalar naming the reference method. When
#'   `NULL` (default), the no-reference CIA is estimated.
#' @param scope One of `"pairwise"` or `"overall"`. `"pairwise"` returns the
#'   two-method
#'   CIA for each method pair, or the reference-scaled CIA for each
#'   method-versus-reference comparison when `reference` is supplied.
#'   `"overall"` returns one CIA coefficient across all retained methods.
#' @param estimator One of `"mom_unconstrained"` or `"vc_constrained"`.
#'   `"mom_unconstrained"` is the literal raw method-of-moments CIA ratio and
#'   may exceed 1. `"vc_constrained"` is a bounded package-level
#'   variance-component variant that constrains the estimated inter-method
#'   variance component to be non-negative before converting it back to CIA.
#' @param ci Logical; if `TRUE`, attach confidence intervals using the selected
#'   `inference` method.
#' @param conf_level Confidence level used when `ci = TRUE`. Default is `0.95`.
#' @param inference One of `"delta"`, `"bootstrap"`, or `"none"`. When
#'   confidence intervals are requested, pairwise CIA uses a delta-method normal
#'   interval by default and also supports subject-bootstrap percentile
#'   intervals. For overall CIA, delta-method normal intervals are available for
#'   the unconstrained moment estimator, and subject-bootstrap percentile
#'   intervals are also available.
#' @param B Number of subject-bootstrap resamples when `ci = TRUE` and
#'   `inference = "bootstrap"`.
#' @param seed Optional positive integer seed for reproducible bootstrap
#'   resampling.
#' @param n_threads Integer >= 1. Number of OpenMP threads passed to the C++
#'   backend.
#' @param verbose Logical; if `TRUE`, emit short progress messages.
#'
#' @details
#' Let `Y_ijk` denote replicate `k` on subject `i` measured by method `j`.
#'
#' Without a reference method, CIA is defined by
#' \deqn{
#' \mathrm{CIA}^N =
#' \frac{\sum_j E\{(Y_{ijk} - Y_{ijk'})^2\} / 2}
#'      {\sum_{j < j'} E\{(Y_{ij} - Y_{ij'})^2\} / (J - 1)}.
#' }
#'
#' With a reference method `J`, CIA is defined by
#' \deqn{
#' \mathrm{CIA}^R =
#' \frac{E\{(Y_{iJk} - Y_{iJk'})^2\}}
#'      {\sum_{j=1}^{J-1} E\{(Y_{ij} - Y_{iJ})^2\} / (J - 1)}.
#' }
#'
#' This implementation uses method-of-moments estimators built from
#' subject-level disagreement functions.
#'
#' For pairwise CIA, consider two methods `X` and `Y`. For each eligible
#' subject `i`, define
#' \deqn{
#' G_{1i} = \mathrm{mean}\{(X_{ik} - X_{ik'})^2\},
#' }
#' \deqn{
#' G_{2i} = \mathrm{mean}\{(Y_{il} - Y_{il'})^2\},
#' }
#' \deqn{
#' G_{3i} = \mathrm{mean}\{(X_{ik} - Y_{il})^2\},
#' }
#' where the first two means are over all distinct within-method replicate
#' pairs and the third mean is over all cross-method replicate combinations.
#' Let \eqn{\bar G_1}, \eqn{\bar G_2}, and \eqn{\bar G_3} denote the averages
#' of these subject-level quantities across eligible subjects. Then the
#' no-reference pairwise estimator is
#' \deqn{
#' \widehat{\mathrm{CIA}}_{XY}^{\mathrm{raw}} =
#' \frac{(\bar G_1 + \bar G_2)/2}{\bar G_3},
#' }
#' and the reference pairwise estimator is
#' \deqn{
#' \widehat{\mathrm{CIA}}_{XR}^{\mathrm{raw}} = \frac{\bar G_1}{\bar G_3},
#' }
#' where `X` is the reference method.
#'
#' For overall CIA, the current implementation follows the balanced replicated
#' formulas in the cited papers. This requires `n` common subjects, `J`
#' retained methods, and the same replicate count `K >= 2` in every retained
#' subject-method cell. If the data are not balanced in this sense,
#' `scope = "overall"` returns an informative error rather than using an
#' approximation.
#'
#' Write \eqn{\bar Y_{ij}} for the mean of the `K` replicates on subject `i`
#' and method `j`. Define the within-method mean square
#' \deqn{
#' A_{ij} = \frac{\sum_{k=1}^{K} (Y_{ijk} - \bar Y_{ij})^2}{K - 1}.
#' }
#'
#' For the no-reference overall estimator, define the subject-level within term
#' \deqn{
#' A_{i\cdot} = \frac{1}{J} \sum_{j=1}^{J} A_{ij},
#' }
#' the subject-wide mean
#' \deqn{
#' \bar Y_{i\cdot\cdot} = \frac{1}{J} \sum_{j=1}^{J} \bar Y_{ij},
#' }
#' and the subject-level denominator
#' \deqn{
#' B_{Ni} =
#' \frac{\sum_{j=1}^{J} (\bar Y_{ij} - \bar Y_{i\cdot\cdot})^2}{J - 1}
#' + \left(1 - \frac{1}{K}\right) A_{i\cdot}.
#' }
#' With \eqn{\bar A = n^{-1}\sum_i A_{i\cdot}} and
#' \eqn{\bar B = n^{-1}\sum_i B_{Ni}}, the overall no-reference estimator is
#' \deqn{
#' \widehat{\mathrm{CIA}}_{N}^{\mathrm{raw}} = \frac{\bar A}{\bar B}.
#' }
#'
#' For the reference overall estimator, let method `R` be the retained
#' reference method. Define
#' \deqn{
#' A_{iR} = A_{i,R}
#' }
#' and
#' \deqn{
#' B_{Ri} =
#' \frac{\sum_{j \ne R} (\bar Y_{ij} - \bar Y_{iR})^2}{J - 1}
#' + \left(1 - \frac{1}{K}\right)
#'   \frac{\sum_{j \ne R} A_{ij}}{J - 1}
#' + \left(1 - \frac{1}{K}\right) A_{iR}.
#' }
#' With \eqn{\bar A_R = n^{-1}\sum_i A_{iR}} and
#' \eqn{\bar B_R = n^{-1}\sum_i B_{Ri}}, the overall reference estimator is
#' \deqn{
#' \widehat{\mathrm{CIA}}_{R}^{\mathrm{raw}} = \frac{2 \bar A_R}{\bar B_R}.
#' }
#'
#' Two estimators are available. `estimator = "mom_unconstrained"` reports the
#' raw ratio estimator directly. `estimator = "vc_constrained"` applies the
#' non-negative variance-component boundary from the cited papers. In the
#' no-reference setting this uses
#' \eqn{\hat\tau^2_{\mathrm{raw}} = \bar B - \bar A}, and in the reference
#' setting it uses \eqn{\hat\tau^2_{\mathrm{raw}} = \bar B_R/2 - \bar A_R}.
#' The constrained estimator then sets
#' \eqn{\hat\tau^2 = \max(\hat\tau^2_{\mathrm{raw}}, 0)} and reports
#' \eqn{\widehat{\mathrm{CIA}} = \widehat{\sigma}_W^2 /
#' (\widehat{\sigma}_W^2 + \hat\tau^2)} on the corresponding scale. This
#' applies the boundary on the implied inter-method variance component rather
#' than clamping CIA directly.
#'
#' When confidence intervals are requested, pairwise CIA uses a large-sample
#' delta-method normal interval by default and also supports subject-bootstrap
#' percentile intervals. For overall CIA, delta-method normal intervals are
#' available for the unconstrained moment estimator, and subject-bootstrap
#' percentile intervals are also available.
#'
#' High CIA indicates stronger individual agreement. The FDA individual
#' bioequivalence boundary `IEC <= 2.4948` corresponds to `CIA >= 0.445`, and
#' `CIA >= 0.8` is sometimes used as a stronger practical rule. Such thresholds
#' are context-dependent and are not hard-coded by this function.
#'
#' Missing rows in the required columns are removed before estimation and the
#' counts are recorded in `attr(x, "diagnostics")`.
#'
#' @return
#' For `scope = "overall"`, a one-row data frame with class
#' `c("cia_overall", "cia", "data.frame")`. For `scope = "pairwise"` and
#' `ci = FALSE`, a dense matrix-style object using the package's standard
#' correlation-result infrastructure. For `scope = "pairwise"` and `ci = TRUE`,
#' a list with elements `est`, `lwr.ci`, and `upr.ci`, classed as
#' `c("cia", "cia_ci")`.
#'
#' @references
#' Barnhart HX, Kosinski AS, Haber M. (2007). Assessing individual agreement.
#' \emph{Journal of Biopharmaceutical Statistics}, 17(4), 697-719.
#' \doi{10.1080/10543400701329489}
#'
#' Barnhart HX, Haber M, Lokhnygina Y, Kosinski AS. (2007). Comparison of
#' concordance correlation coefficient and coefficient of individual agreement
#' in assessing agreement. \emph{Journal of Biopharmaceutical Statistics},
#' 17(4), 721-738.
#'
#' Pan Y, Gao J, Haber M, Barnhart HX. (2010). Estimation of coefficients of
#' individual agreement (CIA's) for quantitative and binary data using SAS and
#' R. \emph{Computer Methods and Programs in Biomedicine}.
#'
#' @seealso \code{\link{ccc}}, \code{\link{ba}}, and \code{\link{prob_agree}}.
#' \code{cia()} answers the question "Are methods interchangeable for
#' individual subjects once within-method replicate disagreement is taken into
#' account?". In contrast, \code{ccc()} answers "How well do two paired
#' measurements agree overall, combining correlation with location/scale
#' agreement?".
#'
#' @examples
#' set.seed(1)
#' subjects <- sprintf("s%02d", 1:30)
#' methods <- c("A", "B", "C")
#' replicates <- sprintf("r%02d", 1:20)
#' dat <- expand.grid(
#'   subject = subjects,
#'   method = methods,
#'   replicate = replicates,
#'   KEEP.OUT.ATTRS = FALSE,
#'   stringsAsFactors = FALSE
#' )
#' subject_effect <- stats::rnorm(length(subjects), sd = 3)
#' method_shift <- c(A = 0, B = 0.15, C = -0.10)
#' method_sd <- c(A = 0.45, B = 0.45, C = 0.30)
#' dat$value <- subject_effect[match(dat$subject, subjects)] +
#'   method_shift[dat$method] +
#'   stats::rnorm(nrow(dat), sd = method_sd[dat$method])
#'
#' fit_overall <- cia(
#'   dat,
#'   response = "value",
#'   subject = "subject",
#'   method = "method",
#'   replicate = "replicate",
#'   scope = "overall"
#' )
#' print(fit_overall)
#' summary(fit_overall)
#' estimate(fit_overall)
#' tidy(fit_overall)
#' plot(fit_overall)
#'
#' fit_pairwise <- cia(
#'   dat,
#'   response = "value",
#'   subject = "subject",
#'   method = "method",
#'   replicate = "replicate",
#'   scope = "pairwise"
#' )
#' print(fit_pairwise)
#' summary(fit_pairwise)
#' tidy(fit_pairwise)
#' plot(fit_pairwise)
#'
#' @author Thiago de Paula Oliveira
#' @export
cia <- function(data,
                response,
                subject,
                method,
                replicate,
                reference = NULL,
                scope = c("pairwise", "overall"),
                estimator = c("mom_unconstrained", "vc_constrained"),
                ci = FALSE,
                conf_level = 0.95,
                inference = c("delta", "bootstrap", "none"),
                B = 1000L,
                seed = NULL,
                n_threads = getOption("matrixCorr.threads", 1L),
                verbose = FALSE) {
  scope <- match_arg(scope, c("pairwise", "overall"), arg_name = "scope")
  estimator <- match_arg(
    estimator,
    c("mom_unconstrained", "vc_constrained"),
    arg_name = "estimator"
  )
  inference <- if (!missing(inference)) {
    match_arg(inference, c("delta", "bootstrap", "none"), arg_name = "inference")
  } else {
    NULL
  }
  check_bool(ci, arg = "ci")
  check_bool(verbose, arg = "verbose")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  B <- check_scalar_int_pos(B, arg = "B")
  if (B < 2L) {
    abort_bad_arg("B", message = "must be >= 2.")
  }
  if (!is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  }
  inference <- .mc_cia_resolve_inference(
    ci = ci,
    scope = scope,
    estimator = estimator,
    inference = inference,
    inference_missing = missing(inference)
  )

  prep <- .mc_cia_prepare_long(
    data = data,
    response = response,
    subject = subject,
    method = method,
    replicate = replicate,
    reference = reference,
    verbose = verbose
  )

  raw <- cia_moments_cpp(
    y = prep$y,
    subject = prep$subject_code,
    method = prep$method_code,
    replicate = prep$replicate_code,
    n_methods = prep$n_methods,
    reference_method = prep$reference_index,
    has_reference = prep$has_reference,
    pairwise = identical(scope, "pairwise"),
    n_threads = n_threads
  )

  diagnostics <- .mc_cia_build_diagnostics(prep, raw)

  if (identical(scope, "overall")) {
    overall <- .mc_cia_overall_moments(
      prep,
      estimator = estimator,
      n_threads = n_threads
    )
    out <- .mc_cia_build_overall_from_moments(
      prep = prep,
      diagnostics = diagnostics,
      moments = overall,
      estimator = estimator
    )
    if (isTRUE(ci) && identical(inference, "delta")) {
      if (!identical(estimator, "mom_unconstrained")) {
        abort_bad_arg(
          "inference",
          message = paste(
            "Delta-method confidence intervals are implemented for the unconstrained moment estimator only;",
            "use {.val bootstrap} for the constrained estimator."
          )
        )
      }
      ci_res <- .mc_cia_overall_delta_ci(overall, conf_level = conf_level)
      out$lwr.ci <- ci_res$lwr
      out$upr.ci <- ci_res$upr
      out$se <- ci_res$se
      if (!is.null(ci_res$diagnostic)) {
        attr(out, "ci.diagnostic") <- ci_res$diagnostic
      }
      attr(out, "ci.method") <- "delta_normal"
    } else if (isTRUE(ci) && identical(inference, "bootstrap")) {
      boot <- .mc_cia_bootstrap_overall(
        prep = prep,
        estimator = estimator,
        B = B,
        conf_level = conf_level,
        seed = seed,
        n_threads = n_threads
      )
      out$lwr.ci <- boot$lwr
      out$upr.ci <- boot$upr
      attr(out, "bootstrap") <- boot[c("n_successful", "B")]
      attr(out, "ci.method") <- "subject_bootstrap_percentile"
    }
    attr(out, "conf.level") <- if (isTRUE(ci)) conf_level else NULL
    return(out)
  }

  pair_payload <- .mc_cia_pairwise_payload(
    prep,
    diagnostics = diagnostics,
    estimator = estimator,
    conf_level = if (isTRUE(ci) && identical(inference, "delta")) conf_level else NULL,
    return_delta = isTRUE(ci) && identical(inference, "delta"),
    n_threads = n_threads
  )

  .mc_cia_assert_estimable(
    prep,
    diagnostics,
    scope = scope,
    n_threads = n_threads,
    payload = pair_payload
  )

  if (!isTRUE(ci)) {
    return(.mc_cia_build_pairwise_matrix(
      prep,
      diagnostics,
      estimator = estimator,
      n_threads = n_threads,
      payload = pair_payload
    ))
  }

  boot <- NULL
  if (identical(inference, "bootstrap")) {
    boot <- .mc_cia_bootstrap_pairwise(
      prep = prep,
      estimator = estimator,
      B = B,
      conf_level = conf_level,
      seed = seed,
      n_threads = n_threads
    )
  }
  .mc_cia_build_pairwise_ci(
    prep,
    diagnostics,
    estimator = estimator,
    inference = inference,
    boot = boot,
    conf_level = conf_level,
    n_threads = n_threads,
    payload = pair_payload
  )
}

.mc_cia_prepare_long <- function(data,
                                 response,
                                 subject,
                                 method,
                                 replicate,
                                 reference,
                                 verbose = FALSE) {
  if (!is.data.frame(data)) {
    abort_bad_arg("data", message = "must be a data frame.")
  }
  cols <- c(response = response, subject = subject, method = method, replicate = replicate)
  for (nm in names(cols)) {
    val <- cols[[nm]]
    if (!is.character(val) || length(val) != 1L || is.na(val) || !nzchar(val)) {
      abort_bad_arg(nm, message = "must be a single column name.")
    }
  }
  check_required_cols(data, unname(cols), df_arg = "data")

  dat <- data[, unname(cols), drop = FALSE]
  names(dat) <- names(cols)
  keep <- stats::complete.cases(dat)
  n_removed <- sum(!keep)
  if (n_removed > 0L) {
    inform_if_verbose(
      "Removed {n_removed} incomplete row{?s} before CIA estimation.",
      .verbose = verbose,
      n_removed = n_removed
    )
    dat <- dat[keep, , drop = FALSE]
  }
  if (!nrow(dat)) {
    abort_bad_arg("data", message = "does not contain any complete rows in the required CIA columns.")
  }

  y <- suppressWarnings(as.numeric(dat$response))
  if (any(!is.finite(y))) {
    abort_bad_arg("response", message = "must name a numeric column with finite values.")
  }

  subj_chr <- as.character(dat$subject)
  meth_chr <- as.character(dat$method)
  rep_chr <- as.character(dat$replicate)

  dup_key <- paste(subj_chr, meth_chr, rep_chr, sep = "\r")
  if (anyDuplicated(dup_key)) {
    abort_bad_arg(
      "data",
      message = "contains duplicated subject-method-replicate combinations.",
      .hint = "Each replicate identifier must be unique within each subject-method cell."
    )
  }

  method_levels <- unique(meth_chr)
  if (length(method_levels) < 2L) {
    abort_bad_arg("method", message = "must retain at least two methods after removing incomplete rows.")
  }
  subject_levels <- unique(subj_chr)
  if (length(subject_levels) < 1L) {
    abort_bad_arg("subject", message = "must retain at least one subject after removing incomplete rows.")
  }

  if (is.null(reference)) {
    reference_index <- 0L
    reference_name <- NA_character_
    has_reference <- FALSE
    coefficient <- "CIA_N"
  } else {
    if (!is.character(reference) || length(reference) != 1L || is.na(reference) || !nzchar(reference)) {
      abort_bad_arg("reference", message = "must be NULL or a single observed method name.")
    }
    reference_index <- match(reference, method_levels, nomatch = 0L)
    if (reference_index <= 0L) {
      abort_bad_arg(
        "reference",
        message = "must match one of the retained method labels.",
        .hint = "Check the spelling of the reference method after missing-row removal."
      )
    }
    reference_name <- reference
    has_reference <- TRUE
    coefficient <- "CIA_R"
  }

  subject_code <- as.integer(factor(subj_chr, levels = subject_levels))
  method_code <- as.integer(factor(meth_chr, levels = method_levels))
  replicate_levels <- unique(rep_chr)
  replicate_code <- as.integer(factor(rep_chr, levels = replicate_levels))
  subject_rows <- split(seq_len(nrow(dat)), subject_code)
  by_subject_method <- lapply(subject_rows, function(idx) split(y[idx], method_code[idx]))

  list(
    data = dat,
    y = y,
    subject_code = subject_code,
    method_code = method_code,
    replicate_code = replicate_code,
    subject_rows = subject_rows,
    by_subject_method = by_subject_method,
    method_levels = method_levels,
    subject_levels = subject_levels,
    replicate_levels = replicate_levels,
    n_methods = length(method_levels),
    n_subjects = length(subject_levels),
    has_reference = has_reference,
    reference_index = as.integer(reference_index),
    reference = reference_name,
    coefficient = coefficient,
    na_rows_removed = as.integer(n_removed),
    n_original_rows = as.integer(nrow(data)),
    n_complete_rows = as.integer(nrow(dat))
  )
}

.mc_cia_named_vector <- function(x, nm) {
  x <- as.numeric(x)
  names(x) <- nm
  x
}

.mc_cia_named_matrix <- function(x, nm) {
  x <- as.matrix(x)
  dimnames(x) <- .mc_square_dimnames(nm)
  x
}

.mc_cia_named_rect_matrix <- function(x, row_nm, col_nm) {
  x <- as.matrix(x)
  dimnames(x) <- list(row_nm, col_nm)
  x
}

.mc_cia_build_diagnostics <- function(prep, raw) {
  within_msd <- .mc_cia_named_vector(raw$within_msd, prep$method_levels)
  sigma_within <- .mc_cia_named_vector(raw$sigma_within, prep$method_levels)
  n_replicate_pairs <- setNames(as.integer(raw$n_replicate_pairs), prep$method_levels)
  between_msd <- .mc_cia_named_matrix(raw$between_msd, prep$method_levels)
  n_between_pairs <- matrix(
    as.integer(raw$n_between_pairs),
    nrow = prep$n_methods,
    ncol = prep$n_methods,
    dimnames = .mc_square_dimnames(prep$method_levels)
  )
  list(
    n_original_rows = prep$n_original_rows,
    n_complete_rows = prep$n_complete_rows,
    na_rows_removed = prep$na_rows_removed,
    n_subjects = as.integer(raw$n_subjects),
    n_obs = as.integer(raw$n_obs),
    method_levels = prep$method_levels,
    reference = prep$reference,
    coefficient = prep$coefficient,
    within_msd = within_msd,
    sigma_within = sigma_within,
    between_msd = between_msd,
    n_replicate_pairs = n_replicate_pairs,
    n_between_pairs = n_between_pairs,
    n_complete = n_between_pairs
  )
}

.mc_cia_resolve_inference <- function(ci, scope, estimator, inference, inference_missing = FALSE) {
  if (!isTRUE(ci)) {
    return("none")
  }
  if (isTRUE(inference_missing) || is.null(inference)) {
    if (identical(scope, "pairwise")) {
      return("delta")
    }
    return(if (identical(estimator, "mom_unconstrained")) "delta" else "bootstrap")
  }
  if (identical(inference, "none")) {
    abort_bad_arg("inference", message = "must not be {.val none} when {.arg ci} is TRUE.")
  }
  if (identical(scope, "overall") &&
      identical(inference, "delta") &&
      !identical(estimator, "mom_unconstrained")) {
    abort_bad_arg(
      "inference",
      message = paste(
        "Delta-method confidence intervals are implemented for the unconstrained moment estimator only;",
        "use {.val bootstrap} for the constrained estimator."
      )
    )
  }
  inference
}

.mc_cia_subject_within_msd <- function(x) {
  n <- length(x)
  if (n < 2L) {
    return(NA_real_)
  }
  d2 <- outer(x, x, FUN = "-")^2
  mean(d2[upper.tri(d2)])
}

.mc_cia_subject_between_msd <- function(x, y) {
  if (!length(x) || !length(y)) {
    return(NA_real_)
  }
  mean(outer(x, y, FUN = "-")^2)
}

.mc_cia_subject_values <- function(prep, subject_idx, method_idx) {
  vals <- prep$by_subject_method[[subject_idx]][[as.character(method_idx)]]
  if (is.null(vals)) numeric(0) else as.numeric(vals)
}

.mc_cia_overall_arrays <- function(prep, n_threads = 1L) {
  raw <- tryCatch(
    cia_overall_balanced_cpp(
      y = prep$y,
      subject = prep$subject_code,
      method = prep$method_code,
      replicate = prep$replicate_code,
      n_methods = prep$n_methods,
      reference_method = prep$reference_index,
      has_reference = prep$has_reference,
      n_threads = n_threads
    ),
    error = function(e) {
      abort_bad_arg("data", message = conditionMessage(e))
    }
  )

  Ybar <- .mc_cia_named_rect_matrix(
    raw$Ybar,
    row_nm = as.character(prep$subject_levels),
    col_nm = prep$method_levels
  )
  A <- .mc_cia_named_rect_matrix(
    raw$A,
    row_nm = as.character(prep$subject_levels),
    col_nm = prep$method_levels
  )

  list(
    Ybar = Ybar,
    A = A,
    n = as.integer(raw$n_subjects),
    J = as.integer(raw$n_methods),
    K = as.integer(raw$K),
    ref_idx = as.integer(raw$ref_idx),
    A_i = as.numeric(raw$A_i),
    B_i = as.numeric(raw$B_i),
    A_bar = as.numeric(raw$A_bar),
    B_bar = as.numeric(raw$B_bar),
    estimate_raw = as.numeric(raw$estimate_raw)
  )
}

.mc_cia_overall_payload <- function(A_bar, B_bar, reference = FALSE, estimator) {
  if (isTRUE(reference)) {
    numerator_term <- 2 * A_bar
    cia_raw <- suppressWarnings(numerator_term / B_bar)
    if (isTRUE(is.nan(cia_raw))) {
      cia_raw <- NA_real_
    }
    tau2_raw <- B_bar / 2 - A_bar
    boundary <- is.finite(tau2_raw) && tau2_raw < 0
    tau2 <- if (is.finite(tau2_raw)) max(tau2_raw, 0) else NA_real_
    cia <- if (identical(estimator, "mom_unconstrained")) {
      cia_raw
    } else {
      denom <- A_bar + tau2
      if (!is.finite(denom) || denom == 0) NA_real_ else A_bar / denom
    }
  } else {
    numerator_term <- A_bar
    cia_raw <- suppressWarnings(A_bar / B_bar)
    if (isTRUE(is.nan(cia_raw))) {
      cia_raw <- NA_real_
    }
    tau2_raw <- B_bar - A_bar
    boundary <- is.finite(tau2_raw) && tau2_raw < 0
    tau2 <- if (is.finite(tau2_raw)) max(tau2_raw, 0) else NA_real_
    cia <- if (identical(estimator, "mom_unconstrained")) {
      cia_raw
    } else {
      denom <- A_bar + tau2
      if (!is.finite(denom) || denom == 0) NA_real_ else A_bar / denom
    }
  }

  list(
    cia = as.numeric(cia),
    cia_raw = as.numeric(cia_raw),
    tau2 = as.numeric(tau2),
    tau2_raw = as.numeric(tau2_raw),
    boundary = isTRUE(boundary),
    numerator_term = as.numeric(numerator_term),
    denominator_term = as.numeric(B_bar),
    A_bar = as.numeric(A_bar)
  )
}

.mc_cia_overall_moments <- function(prep, estimator, n_threads = 1L) {
  arr <- .mc_cia_overall_arrays(prep, n_threads = n_threads)
  payload <- .mc_cia_overall_payload(
    A_bar = arr$A_bar,
    B_bar = arr$B_bar,
    reference = isTRUE(prep$has_reference),
    estimator = estimator
  )

  c(
    list(
      Ybar = arr$Ybar,
      A = arr$A,
      A_i = arr$A_i,
      B_i = arr$B_i,
      n = arr$n,
      J = arr$J,
      K = arr$K,
      ref_idx = arr$ref_idx
    ),
    payload
  )
}

.mc_cia_overall_delta_ci <- function(moments, conf_level = 0.95) {
  A_i <- as.numeric(moments$A_i)
  B_i <- as.numeric(moments$B_i)
  n <- as.integer(moments$n)
  if (n < 2L) {
    return(list(lwr = NA_real_, upr = NA_real_, se = NA_real_, var = NA_real_, diagnostic = "At least two subjects are required for a delta-method CI."))
  }

  A_bar <- mean(A_i)
  B_bar <- mean(B_i)
  if (!is.finite(B_bar) || B_bar <= 0) {
    return(list(lwr = NA_real_, upr = NA_real_, se = NA_real_, var = NA_real_, diagnostic = "The overall denominator term is non-finite or non-positive, so the delta-method CI is not available."))
  }

  var_A <- stats::var(A_i) / n
  var_B <- stats::var(B_i) / n
  cov_A_B <- stats::cov(A_i, B_i) / n
  var_ratio <-
    (1 / (B_bar^2)) * var_A +
    ((A_bar^2) / (B_bar^4)) * var_B -
    ((2 * A_bar) / (B_bar^3)) * cov_A_B
  ratio_multiplier <- if (moments$ref_idx > 0L) 2 else 1
  var_psi <- (ratio_multiplier^2) * var_ratio
  if (!is.finite(var_psi)) {
    return(list(lwr = NA_real_, upr = NA_real_, se = NA_real_, var = NA_real_, diagnostic = "The delta-method variance was non-finite."))
  }
  tol <- sqrt(.Machine$double.eps) * max(1, abs(var_psi))
  if (var_psi < 0 && abs(var_psi) <= tol) {
    var_psi <- 0
  }
  if (var_psi < 0) {
    return(list(lwr = NA_real_, upr = NA_real_, se = NA_real_, var = var_psi, diagnostic = "The delta-method variance was negative beyond floating-point tolerance."))
  }

  psi <- as.numeric(moments$cia_raw)
  se <- sqrt(var_psi)
  z <- stats::qnorm(1 - (1 - conf_level) / 2)
  list(
    lwr = psi - z * se,
    upr = psi + z * se,
    se = se,
    var = var_psi,
    diagnostic = NULL
  )
}

.mc_cia_component_payload <- function(sigma2_within, between_term, estimator) {
  cia_raw <- suppressWarnings(sigma2_within / between_term)
  if (isTRUE(is.nan(cia_raw))) {
    cia_raw <- NA_real_
  }
  tau2_raw <- between_term - sigma2_within
  boundary <- is.finite(tau2_raw) && tau2_raw < 0
  tau2 <- if (is.finite(tau2_raw)) max(tau2_raw, 0) else NA_real_
  cia <- if (identical(estimator, "mom_unconstrained")) {
    cia_raw
  } else {
    denom <- sigma2_within + tau2
    if (!is.finite(denom) || denom == 0) NA_real_ else sigma2_within / denom
  }
  list(
    cia = as.numeric(cia),
    cia_raw = as.numeric(cia_raw),
    tau2_raw = as.numeric(tau2_raw),
    tau2 = as.numeric(tau2),
    boundary = isTRUE(boundary),
    sigma2_within = as.numeric(sigma2_within),
    between_term = as.numeric(between_term)
  )
}

.mc_cia_pairwise_delta_ci <- function(A,
                                      B,
                                      N,
                                      S11,
                                      S33,
                                      S13,
                                      S22 = NA_real_,
                                      S12 = NA_real_,
                                      S23 = NA_real_,
                                      reference_pair = FALSE,
                                      conf_level = 0.95,
                                      center) {
  if (N < 2L || !is.finite(A) || !is.finite(B) || B == 0) {
    return(list(lwr = NA_real_, upr = NA_real_, se = NA_real_, var = NA_real_))
  }

  if (isTRUE(reference_pair)) {
    var_A <- S11 / N
    var_B <- S33 / N
    cov_A_B <- S13 / N
  } else {
    var_A <- (S11 + S22 + 2 * S12) / (4 * N)
    var_B <- S33 / N
    cov_A_B <- (S13 + S23) / (2 * N)
  }

  var_cia <-
    (1 / (B^2)) * var_A +
    ((A^2) / (B^4)) * var_B -
    ((2 * A) / (B^3)) * cov_A_B
  se <- sqrt(max(var_cia, 0))
  z <- stats::qnorm(1 - (1 - conf_level) / 2)
  list(
    lwr = center - z * se,
    upr = center + z * se,
    se = se,
    var = var_cia
  )
}

.mc_cia_pairwise_payload <- function(prep,
                                     diagnostics = NULL,
                                     estimator,
                                     conf_level = NULL,
                                     return_delta = FALSE,
                                     n_threads = 1L) {
  raw <- cia_pairwise_stats_cpp(
    y = prep$y,
    subject = prep$subject_code,
    method = prep$method_code,
    replicate = prep$replicate_code,
    n_methods = prep$n_methods,
    reference_method = prep$reference_index,
    has_reference = prep$has_reference,
    n_threads = n_threads
  )
  nm <- prep$method_levels
  est <- matrix(NA_real_, prep$n_methods, prep$n_methods, dimnames = .mc_square_dimnames(nm))
  cia_raw <- est
  tau2_raw <- est
  tau2 <- est
  sigma2_within <- est
  lwr <- est
  upr <- est
  n_eligible <- matrix(0L, prep$n_methods, prep$n_methods, dimnames = .mc_square_dimnames(nm))
  boundary <- matrix(FALSE, prep$n_methods, prep$n_methods, dimnames = .mc_square_dimnames(nm))

  diag(est) <- 1
  diag(cia_raw) <- 1
  diag(tau2_raw) <- 0
  diag(tau2) <- 0
  diag(sigma2_within) <- 0
  diag(lwr) <- 1
  diag(upr) <- 1
  diag(n_eligible) <- prep$n_subjects

  if (isTRUE(prep$has_reference)) {
    ref_idx <- prep$reference_index
    for (j in seq_len(prep$n_methods)) {
      if (j == ref_idx) {
        next
      }
      N <- as.integer(raw$n_eligible[j, ref_idx])
      n_eligible[j, ref_idx] <- N
      n_eligible[ref_idx, j] <- N
      if (N <= 0L) {
        next
      }
      A <- as.numeric(raw$A_bar[j, ref_idx])
      B <- as.numeric(raw$B_bar[j, ref_idx])
      payload <- .mc_cia_component_payload(
        sigma2_within = A,
        between_term = B,
        estimator = estimator
      )
      est[j, ref_idx] <- payload$cia
      est[ref_idx, j] <- payload$cia
      cia_raw[j, ref_idx] <- payload$cia_raw
      cia_raw[ref_idx, j] <- payload$cia_raw
      tau2_raw[j, ref_idx] <- payload$tau2_raw
      tau2_raw[ref_idx, j] <- payload$tau2_raw
      tau2[j, ref_idx] <- payload$tau2
      tau2[ref_idx, j] <- payload$tau2
      sigma2_within[j, ref_idx] <- payload$sigma2_within
      sigma2_within[ref_idx, j] <- payload$sigma2_within
      boundary[j, ref_idx] <- payload$boundary
      boundary[ref_idx, j] <- payload$boundary
      if (isTRUE(return_delta)) {
        ci <- .mc_cia_pairwise_delta_ci(
          A = A,
          B = B,
          N = N,
          S11 = as.numeric(raw$S11[j, ref_idx]),
          S33 = as.numeric(raw$S33[j, ref_idx]),
          S13 = as.numeric(raw$S13[j, ref_idx]),
          reference_pair = TRUE,
          conf_level = conf_level,
          center = payload$cia
        )
        lwr[j, ref_idx] <- ci$lwr
        lwr[ref_idx, j] <- ci$lwr
        upr[j, ref_idx] <- ci$upr
        upr[ref_idx, j] <- ci$upr
      }
    }
    return(list(
      est = est,
      cia_raw = cia_raw,
      tau2_raw = tau2_raw,
      tau2 = tau2,
      sigma2_within = sigma2_within,
      boundary = boundary,
      n_eligible = n_eligible,
      lwr = lwr,
      upr = upr
    ))
  }

  for (j in seq_len(prep$n_methods - 1L)) {
    for (k in (j + 1L):prep$n_methods) {
      N <- as.integer(raw$n_eligible[j, k])
      n_eligible[j, k] <- N
      n_eligible[k, j] <- N
      if (N <= 0L) {
        next
      }
      A <- as.numeric(raw$A_bar[j, k])
      B <- as.numeric(raw$B_bar[j, k])
      payload <- .mc_cia_component_payload(
        sigma2_within = A,
        between_term = B,
        estimator = estimator
      )
      est[j, k] <- payload$cia
      est[k, j] <- payload$cia
      cia_raw[j, k] <- payload$cia_raw
      cia_raw[k, j] <- payload$cia_raw
      tau2_raw[j, k] <- payload$tau2_raw
      tau2_raw[k, j] <- payload$tau2_raw
      tau2[j, k] <- payload$tau2
      tau2[k, j] <- payload$tau2
      sigma2_within[j, k] <- payload$sigma2_within
      sigma2_within[k, j] <- payload$sigma2_within
      boundary[j, k] <- payload$boundary
      boundary[k, j] <- payload$boundary
      if (isTRUE(return_delta)) {
        ci <- .mc_cia_pairwise_delta_ci(
          A = A,
          B = B,
          N = N,
          S11 = as.numeric(raw$S11[j, k]),
          S22 = as.numeric(raw$S22[j, k]),
          S33 = as.numeric(raw$S33[j, k]),
          S12 = as.numeric(raw$S12[j, k]),
          S13 = as.numeric(raw$S13[j, k]),
          S23 = as.numeric(raw$S23[j, k]),
          reference_pair = FALSE,
          conf_level = conf_level,
          center = payload$cia
        )
        lwr[j, k] <- ci$lwr
        lwr[k, j] <- ci$lwr
        upr[j, k] <- ci$upr
        upr[k, j] <- ci$upr
      }
    }
  }
  list(
    est = est,
    cia_raw = cia_raw,
    tau2_raw = tau2_raw,
    tau2 = tau2,
    sigma2_within = sigma2_within,
    boundary = boundary,
    n_eligible = n_eligible,
    lwr = lwr,
    upr = upr
  )
}

.mc_cia_assert_estimable <- function(prep,
                                     diagnostics,
                                     scope = c("overall", "pairwise"),
                                     n_threads = 1L,
                                     payload = NULL) {
  scope <- match.arg(scope)
  if (!any(diagnostics$n_replicate_pairs > 0L)) {
    abort_bad_arg(
      "data",
      message = "does not contain any subject-method cell with at least two replicated readings.",
      .hint = "CIA requires replicated readings within method."
    )
  }

  if (!identical(scope, "overall")) {
    est <- if (is.null(payload)) {
      .mc_cia_pairwise_payload(
        prep,
        diagnostics = diagnostics,
        estimator = "vc_constrained",
        n_threads = n_threads
      )$est
    } else {
      payload$est
    }
    if (!any(is.finite(est[upper.tri(est, diag = FALSE)]))) {
      abort_bad_arg(
        "data",
        message = "does not contain enough replicated and cross-method information to estimate any pairwise CIA value."
      )
    }
  }
  invisible(NULL)
}

.mc_cia_build_overall_from_moments <- function(prep, diagnostics, moments, estimator) {
  out <- data.frame(
    coefficient = prep$coefficient,
    estimator = estimator,
    cia = moments$cia,
    cia_raw = moments$cia_raw,
    tau2 = moments$tau2,
    tau2_raw = moments$tau2_raw,
    boundary = moments$boundary,
    reference = if (isTRUE(prep$has_reference)) prep$reference else NA_character_,
    n_methods = prep$n_methods,
    n_subjects = moments$n,
    n_obs = diagnostics$n_obs,
    K = moments$K,
    numerator_term = moments$numerator_term,
    denominator_term = moments$denominator_term,
    within_msd = moments$numerator_term,
    between_msd = moments$denominator_term,
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  class(out) <- c("cia_overall", "cia", "data.frame")
  attr(out, "method") <- "Coefficient of individual agreement"
  attr(out, "description") <- if (isTRUE(prep$has_reference)) {
    "Overall reference-scaled coefficient of individual agreement"
  } else {
    "Overall coefficient of individual agreement"
  }
  attr(out, "package") <- "matrixCorr"
  attr(out, "scope") <- "overall"
  attr(out, "reference") <- prep$reference
  attr(out, "coefficient") <- prep$coefficient
  attr(out, "estimator") <- estimator
  attr(out, "boundary") <- moments$boundary
  attr(out, "diagnostics") <- c(diagnostics, list(K = moments$K))
  attr(out, "overall_moments") <- moments
  out
}

.mc_cia_warn_small_pairwise_n <- function(payload) {
  off_diag <- upper.tri(payload$n_eligible)
  small <- payload$n_eligible[off_diag] > 0L & payload$n_eligible[off_diag] < 10L
  if (any(small, na.rm = TRUE)) {
    cli::cli_warn(
      "Some pairwise CIA estimates use fewer than 10 eligible subjects.",
      class = "matrixCorr_cia_small_n"
    )
  }
}

.mc_cia_pairwise_diagnostics <- function(diagnostics, payload) {
  diagnostics$n_complete <- payload$n_eligible
  diagnostics$n_eligible_subjects <- payload$n_eligible
  diagnostics
}

.mc_cia_build_pairwise_matrix <- function(prep,
                                          diagnostics,
                                          estimator,
                                          n_threads = 1L,
                                          payload = NULL) {
  if (is.null(payload)) {
    payload <- .mc_cia_pairwise_payload(
      prep,
      diagnostics = diagnostics,
      estimator = estimator,
      n_threads = n_threads
    )
  }
  .mc_cia_warn_small_pairwise_n(payload)
  pair_diag <- .mc_cia_pairwise_diagnostics(diagnostics, payload)
  out <- .mc_new_corr_matrix(
    mat = payload$est,
    estimator_class = "cia",
    method = "Coefficient of individual agreement",
    description = if (isTRUE(prep$has_reference)) {
      "Reference-scaled pairwise coefficient of individual agreement"
    } else {
      "Pairwise coefficient of individual agreement"
    },
    diagnostics = pair_diag,
    symmetric = TRUE,
    extra_attrs = list(
      scope = "pairwise",
      reference = prep$reference,
      coefficient = prep$coefficient,
      estimator = estimator,
      cia_raw = payload$cia_raw,
      tau2_raw = payload$tau2_raw,
      tau2 = payload$tau2,
      sigma2_within = payload$sigma2_within,
      boundary = payload$boundary
    )
  )
  class(out) <- unique(c("cia", class(out)))
  out
}

.mc_cia_build_pairwise_ci <- function(prep,
                                      diagnostics,
                                      estimator,
                                      inference,
                                      boot = NULL,
                                      conf_level,
                                      n_threads = 1L,
                                      payload = NULL) {
  if (is.null(payload)) {
    payload <- .mc_cia_pairwise_payload(
      prep,
      diagnostics = diagnostics,
      estimator = estimator,
      conf_level = conf_level,
      return_delta = identical(inference, "delta"),
      n_threads = n_threads
    )
  }
  .mc_cia_warn_small_pairwise_n(payload)
  out <- list(
    est = payload$est,
    lwr.ci = if (identical(inference, "delta")) payload$lwr else boot$lwr,
    upr.ci = if (identical(inference, "delta")) payload$upr else boot$upr
  )
  class(out) <- c("cia", "cia_ci")
  attr(out, "method") <- "Coefficient of individual agreement"
  attr(out, "description") <- if (isTRUE(prep$has_reference)) {
    "Reference-scaled pairwise coefficient of individual agreement with confidence intervals"
  } else {
    "Pairwise coefficient of individual agreement with confidence intervals"
  }
  attr(out, "package") <- "matrixCorr"
  attr(out, "scope") <- "pairwise"
  attr(out, "reference") <- prep$reference
  attr(out, "coefficient") <- prep$coefficient
  attr(out, "estimator") <- estimator
  attr(out, "cia_raw") <- payload$cia_raw
  attr(out, "tau2_raw") <- payload$tau2_raw
  attr(out, "tau2") <- payload$tau2
  attr(out, "sigma2_within") <- payload$sigma2_within
  attr(out, "boundary") <- payload$boundary
  attr(out, "diagnostics") <- .mc_cia_pairwise_diagnostics(diagnostics, payload)
  attr(out, "conf.level") <- conf_level
  attr(out, "ci.method") <- if (identical(inference, "delta")) {
    "delta_normal"
  } else {
    "subject_bootstrap_percentile"
  }
  if (!is.null(boot)) {
    attr(out, "bootstrap") <- boot[c("n_successful", "B")]
  }
  out
}

.mc_cia_bootstrap_vectors <- function(prep, sampled_subjects) {
  total <- sum(lengths(prep$subject_rows[sampled_subjects]))
  y <- numeric(total)
  subject <- integer(total)
  method <- integer(total)
  replicate <- integer(total)
  pos <- 1L
  for (draw in seq_along(sampled_subjects)) {
    idx <- prep$subject_rows[[sampled_subjects[[draw]]]]
    n_i <- length(idx)
    rng <- pos:(pos + n_i - 1L)
    y[rng] <- prep$y[idx]
    method[rng] <- prep$method_code[idx]
    replicate[rng] <- prep$replicate_code[idx]
    subject[rng] <- draw
    pos <- pos + n_i
  }
  list(y = y, subject = subject, method = method, replicate = replicate)
}

.mc_cia_bootstrap_prep <- function(prep, boot_vec) {
  subject_rows <- split(seq_along(boot_vec$y), boot_vec$subject)
  by_subject_method <- lapply(subject_rows, function(idx) split(boot_vec$y[idx], boot_vec$method[idx]))
  list(
    y = boot_vec$y,
    subject_code = boot_vec$subject,
    method_code = boot_vec$method,
    replicate_code = boot_vec$replicate,
    subject_rows = subject_rows,
    by_subject_method = by_subject_method,
    method_levels = prep$method_levels,
    subject_levels = seq_along(subject_rows),
    replicate_levels = prep$replicate_levels,
    n_methods = prep$n_methods,
    n_subjects = length(subject_rows),
    has_reference = prep$has_reference,
    reference_index = prep$reference_index,
    reference = prep$reference,
    coefficient = prep$coefficient,
    n_original_rows = length(boot_vec$y),
    n_complete_rows = length(boot_vec$y),
    na_rows_removed = 0L
  )
}

.mc_cia_bootstrap_overall <- function(prep, estimator, B, conf_level, seed = NULL, n_threads = 1L) {
  .mc_eval_with_seed(seed, {
    alpha <- (1 - conf_level) / 2
    est <- rep(NA_real_, B)
    for (b in seq_len(B)) {
      sampled <- sample.int(prep$n_subjects, size = prep$n_subjects, replace = TRUE)
      boot_vec <- .mc_cia_bootstrap_vectors(prep, sampled)
      boot_prep <- .mc_cia_bootstrap_prep(prep, boot_vec)
      est[[b]] <- tryCatch(
        .mc_cia_overall_moments(
          boot_prep,
          estimator = estimator,
          n_threads = n_threads
        )$cia,
        error = function(...) NA_real_
      )
    }
    keep <- is.finite(est)
    ci <- if (sum(keep) >= 2L) {
      stats::quantile(est[keep], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
    } else {
      c(NA_real_, NA_real_)
    }
    list(
      lwr = as.numeric(ci[[1L]]),
      upr = as.numeric(ci[[2L]]),
      n_successful = sum(keep),
      B = B
    )
  })
}

.mc_cia_bootstrap_pairwise <- function(prep, estimator, B, conf_level, seed = NULL, n_threads = 1L) {
  .mc_eval_with_seed(seed, {
    alpha <- (1 - conf_level) / 2
    pair_idx <- which(upper.tri(matrix(FALSE, prep$n_methods, prep$n_methods)), arr.ind = TRUE)
    pair_linear <- pair_idx[, 1L] + (pair_idx[, 2L] - 1L) * prep$n_methods
    n_pairs <- nrow(pair_idx)
    boot_est <- matrix(NA_real_, nrow = B, ncol = n_pairs)
    for (b in seq_len(B)) {
      sampled <- sample.int(prep$n_subjects, size = prep$n_subjects, replace = TRUE)
      est <- cia_pairwise_bootstrap_est_cpp(
        y = prep$y,
        subject = prep$subject_code,
        method = prep$method_code,
        replicate = prep$replicate_code,
        n_methods = prep$n_methods,
        reference_method = prep$reference_index,
        has_reference = prep$has_reference,
        sampled_subjects = sampled,
        estimator = estimator,
        n_threads = n_threads
      )
      boot_est[b, ] <- est[pair_linear]
    }

    lwr <- matrix(NA_real_, prep$n_methods, prep$n_methods, dimnames = .mc_square_dimnames(prep$method_levels))
    upr <- lwr
    for (p in seq_len(n_pairs)) {
      vals <- boot_est[, p]
      ok <- is.finite(vals)
      if (sum(ok) >= 2L) {
        qs <- stats::quantile(vals[ok], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
        i <- pair_idx[p, 1L]
        j <- pair_idx[p, 2L]
        lwr[i, j] <- as.numeric(qs[[1L]])
        lwr[j, i] <- as.numeric(qs[[1L]])
        upr[i, j] <- as.numeric(qs[[2L]])
        upr[j, i] <- as.numeric(qs[[2L]])
      }
    }
    diag(lwr) <- 1
    diag(upr) <- 1
    list(
      lwr = lwr,
      upr = upr,
      n_successful = sum(rowSums(is.finite(boot_est)) > 0L),
      B = B
    )
  })
}

.mc_cia_as_corr_result <- function(x) {
  if (inherits(x, "corr_result")) {
    return(x)
  }
  if (inherits(x, "cia_ci")) {
    ci_attr <- list(
      est = as.matrix(x$est),
      lwr.ci = as.matrix(x$lwr.ci),
      upr.ci = as.matrix(x$upr.ci),
      conf.level = attr(x, "conf.level", exact = TRUE),
      ci.method = attr(x, "ci.method", exact = TRUE) %||% "subject_bootstrap_percentile"
    )
    return(.mc_new_corr_matrix(
      mat = as.matrix(x$est),
      estimator_class = "cia",
      method = attr(x, "method", exact = TRUE) %||% "Coefficient of individual agreement",
      description = attr(x, "description", exact = TRUE) %||% "Pairwise coefficient of individual agreement with confidence intervals",
      diagnostics = attr(x, "diagnostics", exact = TRUE),
      ci = ci_attr,
      conf.level = attr(x, "conf.level", exact = TRUE),
      symmetric = TRUE,
      extra_attrs = list(
        scope = attr(x, "scope", exact = TRUE),
        reference = attr(x, "reference", exact = TRUE),
        coefficient = attr(x, "coefficient", exact = TRUE),
        estimator = attr(x, "estimator", exact = TRUE),
        cia_raw = attr(x, "cia_raw", exact = TRUE),
        tau2_raw = attr(x, "tau2_raw", exact = TRUE),
        tau2 = attr(x, "tau2", exact = TRUE),
        sigma2_within = attr(x, "sigma2_within", exact = TRUE),
        boundary = attr(x, "boundary", exact = TRUE)
      )
    ))
  }
  abort_bad_arg("x", message = "must be a CIA object.")
}

.mc_cia_has_boundary <- function(x) {
  boundary <- attr(x, "boundary", exact = TRUE)
  if (is.null(boundary) && is.data.frame(x) && "boundary" %in% names(x)) {
    boundary <- x$boundary
  }
  isTRUE(any(boundary, na.rm = TRUE))
}

.mc_cia_boundary_note <- function() {
  paste(
    "Note: the raw method-of-moments estimator implied negative inter-method variability",
    "for at least one comparison, so the reported CIA was constrained to the",
    "valid variance-component parameter space."
  )
}

.mc_cia_summary_add_pairwise_fields <- function(df, corr_obj) {
  raw_mat <- attr(corr_obj, "cia_raw", exact = TRUE)
  tau2_raw_mat <- attr(corr_obj, "tau2_raw", exact = TRUE)
  tau2_mat <- attr(corr_obj, "tau2", exact = TRUE)
  boundary_mat <- attr(corr_obj, "boundary", exact = TRUE)
  if (is.null(raw_mat) || !all(c("item1", "item2") %in% names(df))) {
    return(df)
  }
  idx <- cbind(
    match(df$item1, rownames(raw_mat)),
    match(df$item2, colnames(raw_mat))
  )
  df$cia_raw <- raw_mat[idx]
  df$tau2_raw <- tau2_raw_mat[idx]
  df$tau2 <- tau2_mat[idx]
  df$boundary <- as.logical(boundary_mat[idx])
  df
}

#' @rdname cia
#' @method summary cia
#' @param object A `cia` object.
#' @param digits Integer; number of decimal places for estimates.
#' @param ci_digits Integer; number of decimal places for confidence limits.
#' @param ... Unused.
#' @export
summary.cia <- function(object,
                        digits = 4,
                        ci_digits = 3,
                        ...) {
  if (inherits(object, "cia_overall") && is.data.frame(object)) {
    out <- as.data.frame(object, stringsAsFactors = FALSE, check.names = FALSE)
    out <- .mc_finalize_summary_df(out, class_name = "summary.cia")
    attr(out, "result_type") <- "overall"
    attr(out, "digits") <- digits
    attr(out, "ci_digits") <- ci_digits
    attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE) %||% NA_real_
    attr(out, "coefficient") <- attr(object, "coefficient", exact = TRUE) %||% NA_character_
    attr(out, "reference") <- attr(object, "reference", exact = TRUE) %||% NA_character_
    attr(out, "estimator") <- attr(object, "estimator", exact = TRUE) %||% NA_character_
    attr(out, "boundary") <- attr(object, "boundary", exact = TRUE) %||% FALSE
    return(out)
  }
  corr_obj <- .mc_cia_as_corr_result(object)
  out <- summary.corr_matrix(corr_obj, ...)
  if ("fisher_z" %in% names(out)) {
    out$fisher_z <- NULL
  }
  out <- .mc_cia_summary_add_pairwise_fields(out, corr_obj)
  top_tbl <- attr(out, "top_results", exact = TRUE)
  if (is.data.frame(top_tbl) && "fisher_z" %in% names(top_tbl)) {
    top_tbl$fisher_z <- NULL
  }
  if (is.data.frame(top_tbl)) {
    top_tbl <- .mc_cia_summary_add_pairwise_fields(top_tbl, corr_obj)
    attr(out, "top_results") <- top_tbl
  }
  class(out) <- unique(c("summary.cia", class(out)))
  attr(out, "estimator") <- attr(corr_obj, "estimator", exact = TRUE) %||% NA_character_
  attr(out, "boundary") <- attr(corr_obj, "boundary", exact = TRUE)
  attr(out, "summary_title") <- "Coefficient of individual agreement summary"
  out
}

#' @rdname cia
#' @method print cia
#' @param x A `cia` object.
#' @param digits Integer; number of decimal places for estimates.
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Additional arguments passed to downstream print helpers.
#' @export
print.cia <- function(x,
                      digits = 4,
                      n = NULL,
                      topn = NULL,
                      max_vars = NULL,
                      width = NULL,
                      show_ci = NULL,
                      ...) {
  if (inherits(x, "cia_overall") && is.data.frame(x)) {
    return(invisible(print.summary.cia(
      summary.cia(x),
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      ...
    )))
  }

  .mc_print_corr_matrix(
    .mc_cia_as_corr_result(x),
    header = if (isTRUE(nzchar(attr(.mc_cia_as_corr_result(x), "reference", exact = TRUE) %||% ""))) {
      "Coefficient of individual agreement matrix"
    } else {
      "Coefficient of individual agreement matrix"
    },
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  if (identical(attr(.mc_cia_as_corr_result(x), "estimator", exact = TRUE), "vc_constrained") &&
      .mc_cia_has_boundary(.mc_cia_as_corr_result(x))) {
    cat("\n", .mc_cia_boundary_note(), "\n", sep = "")
  }
  invisible(x)
}

#' @rdname cia
#' @method print summary.cia
#' @param x A `summary.cia` object.
#' @export
print.summary.cia <- function(x,
                              digits = NULL,
                              n = NULL,
                              topn = NULL,
                              max_vars = NULL,
                              width = NULL,
                              show_ci = NULL,
                              ...) {
  if (!identical(attr(x, "result_type", exact = TRUE), "overall")) {
    print.summary.corr_result(
      x,
      digits = digits,
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      ...
    )
    if (identical(attr(x, "estimator", exact = TRUE), "vc_constrained") &&
        .mc_cia_has_boundary(x)) {
      cat("\n", .mc_cia_boundary_note(), "\n", sep = "")
    }
    return(invisible(x))
  }

  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  digits <- .mc_coalesce(digits, .mc_coalesce(attr(x, "digits", exact = TRUE), 4L))
  ci_digits <- .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)

  digest <- c(
    coefficient = x$coefficient[[1L]],
    estimator = x$estimator[[1L]],
    cia = formatC(as.numeric(x$cia[[1L]]), format = "f", digits = digits),
    n_methods = .mc_count_fmt(x$n_methods[[1L]]),
    n_subjects = .mc_count_fmt(x$n_subjects[[1L]]),
    n_obs = .mc_count_fmt(x$n_obs[[1L]])
  )
  if (!is.na(x$reference[[1L]]) && nzchar(x$reference[[1L]])) {
    digest <- c(digest, reference = x$reference[[1L]])
  }
  if (isTRUE(identical(show_ci, "yes")) &&
      all(c("lwr.ci", "upr.ci") %in% names(x)) &&
      is.finite(x$lwr.ci[[1L]]) &&
      is.finite(x$upr.ci[[1L]])) {
    digest <- c(
      digest,
      ci = sprintf("%g%%", 100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))))
    )
  }
  .mc_print_named_digest(digest, header = "Coefficient of individual agreement")
  cat("\n")

  preview <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  num_cols <- intersect(
    c(
      "cia",
      "cia_raw",
      "tau2",
      "tau2_raw",
      "K",
      "numerator_term",
      "denominator_term",
      "within_msd",
      "between_msd",
      "se"
    ),
    names(preview)
  )
  for (nm in num_cols) preview[[nm]] <- round(preview[[nm]], digits)
  if ("lwr.ci" %in% names(preview)) preview$lwr.ci <- round(preview$lwr.ci, ci_digits)
  if ("upr.ci" %in% names(preview)) preview$upr.ci <- round(preview$upr.ci, ci_digits)
  has_ci_cols <- all(c("lwr.ci", "upr.ci") %in% names(preview))
  if (identical(show_ci, "no") ||
      (isTRUE(has_ci_cols) &&
       !any(is.finite(preview$lwr.ci) | is.finite(preview$upr.ci)))) {
    preview <- preview[, setdiff(names(preview), c("lwr.ci", "upr.ci")), drop = FALSE]
  }

  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  .mc_print_preview_table(
    preview,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = FALSE,
    summary_hint = FALSE
  )
  if (identical(attr(x, "estimator", exact = TRUE), "vc_constrained") &&
      .mc_cia_has_boundary(x)) {
    cat("\n", .mc_cia_boundary_note(), "\n", sep = "")
  }
  invisible(x)
}

#' @rdname cia
#' @method plot cia
#' @param x A `cia` object.
#' @param title Optional plot title.
#' @param low_color Color used for lower agreement values.
#' @param high_color Color used for higher agreement values.
#' @param mid_color Midpoint color for pairwise heatmaps.
#' @param value_text_size Text size for overlaid estimate labels.
#' @param ci_text_size Text size for CI labels.
#' @param show_value Logical; whether to overlay numeric values.
#' @export
plot.cia <- function(x,
                     title = NULL,
                     low_color = "indianred1",
                     high_color = "steelblue1",
                     mid_color = "white",
                     value_text_size = 4,
                     ci_text_size = 3,
                     show_value = TRUE,
                     ...) {
  if (inherits(x, "cia_overall") && is.data.frame(x)) {
    check_bool(show_value, arg = "show_value")
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
    }
    df <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
    est <- as.numeric(df$cia[[1L]])
    lwr <- if ("lwr.ci" %in% names(df)) as.numeric(df$lwr.ci[[1L]]) else NA_real_
    upr <- if ("upr.ci" %in% names(df)) as.numeric(df$upr.ci[[1L]]) else NA_real_
    x_min <- min(c(0, est, lwr), na.rm = TRUE)
    x_max <- max(c(1, est, upr), na.rm = TRUE)
    if (!is.finite(x_min) || !is.finite(x_max)) {
      x_min <- 0
      x_max <- max(1, est + 0.1)
    }
    if (identical(x_min, x_max)) {
      x_min <- x_min - 0.1
      x_max <- x_max + 0.1
    }
    pad <- max(0.04, 0.06 * (x_max - x_min))
    point_color <- if (is.finite(est) && est < 0.5) low_color else high_color
    x_limits <- c(x_min - pad, x_max + pad)
    band_df <- data.frame(
      xmin = c(0, 0.445, 0.8, 1),
      xmax = c(0.445, 0.8, 1, x_limits[[2L]]),
      ymin = 0.7,
      ymax = 1.3,
      fill = c("#f6d6d6", "#f4ead0", "#dbeedc", "#d9e7f7")
    )
    band_df <- band_df[band_df$xmin < x_limits[[2L]], , drop = FALSE]
    band_df$xmin <- pmax(band_df$xmin, x_limits[[1L]])
    band_df$xmax <- pmin(band_df$xmax, x_limits[[2L]])

    guide_df <- data.frame(
      x = c(0.445, 0.8),
      label = c("0.445", "0.800"),
      text = c("IEC-linked reference", "strong-agreement reference")
    )
    guide_df <- guide_df[guide_df$x >= x_limits[[1L]] & guide_df$x <= x_limits[[2L]], , drop = FALSE]

    label_df <- data.frame(
      x = c(0.2225, (0.445 + 0.8) / 2, 0.9),
      y = c(1.18, 1.18, 1.18),
      label = c("lower", "intermediate", "strong")
    )
    label_df <- label_df[label_df$x >= x_limits[[1L]] & label_df$x <= x_limits[[2L]], , drop = FALSE]

    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$cia, y = 1)) +
      ggplot2::geom_rect(
        data = band_df,
        ggplot2::aes(
          xmin = .data$xmin,
          xmax = .data$xmax,
          ymin = .data$ymin,
          ymax = .data$ymax
        ),
        inherit.aes = FALSE,
        fill = band_df$fill,
        color = NA,
        alpha = 0.85
      ) +
      ggplot2::geom_segment(
        inherit.aes = FALSE,
        data = data.frame(x = x_limits[[1L]], xend = x_limits[[2L]], y = 1, yend = 1),
        ggplot2::aes(x = .data$x, xend = .data$xend, y = .data$y, yend = .data$yend),
        linewidth = 6,
        color = "white",
        lineend = "round"
      ) +
      ggplot2::geom_point(size = 3.6, color = point_color)
    if (is.finite(lwr) && is.finite(upr)) {
      p <- p + ggplot2::geom_segment(
        ggplot2::aes(x = .data$lwr.ci, xend = .data$upr.ci, y = 1, yend = 1),
        linewidth = 1.4,
        color = point_color
      )
    }
    if (nrow(guide_df)) {
      p <- p + ggplot2::geom_vline(
        data = guide_df,
        ggplot2::aes(xintercept = .data$x),
        inherit.aes = FALSE,
        color = "gray45",
        linetype = "22",
        linewidth = 0.5
      ) +
        ggplot2::geom_text(
          data = guide_df,
          ggplot2::aes(x = .data$x, y = 0.76, label = .data$label),
          inherit.aes = FALSE,
          size = 3,
          color = "gray35",
          vjust = 1
        )
    }
    if (nrow(label_df)) {
      p <- p + ggplot2::geom_text(
        data = label_df,
        ggplot2::aes(x = .data$x, y = .data$y, label = .data$label),
        inherit.aes = FALSE,
        size = 3.2,
        color = "gray25",
        fontface = "bold"
      )
    }
    if (isTRUE(show_value)) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(label = sprintf("%.2f", .data$cia)),
        nudge_y = 0.16,
        size = value_text_size
      )
    }
    subtitle <- if (!is.na(df$reference[[1L]]) && nzchar(df$reference[[1L]])) {
      sprintf("Overall reference-scaled CIA using %s as the reference method", df$reference[[1L]])
    } else {
      "Overall no-reference CIA from within-method replicate disagreement"
    }
    p <- p +
      ggplot2::scale_x_continuous(limits = x_limits) +
      ggplot2::scale_y_continuous(NULL, breaks = NULL) +
      ggplot2::labs(
        title = title %||% "Coefficient of individual agreement",
        subtitle = subtitle,
        x = "CIA",
        y = NULL,
        caption = "Guide lines at 0.445 and 0.8 are contextual references, not fixed pass/fail thresholds."
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        panel.grid.major.y = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank(),
        panel.grid.major.x = ggplot2::element_blank(),
        axis.text.y = ggplot2::element_blank(),
        plot.caption = ggplot2::element_text(hjust = 0)
      ) +
      ggplot2::coord_cartesian(clip = "off")
    return(p)
  }

  .mc_plot_corr_result(
    .mc_cia_as_corr_result(x),
    title = title %||% "Coefficient of individual agreement heatmap",
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    ci_text_size = ci_text_size,
    show_value = show_value,
    ...
  )
}

#' @rdname cia
#' @method print cia_ci
#' @export
print.cia_ci <- function(x, ...) {
  print.cia(x, ...)
}

make_cia_perf_data <- function(n_subjects = 12L, n_methods = 3L, n_replicates = 3L) {
  subjects <- sprintf("s%02d", seq_len(n_subjects))
  methods <- LETTERS[seq_len(n_methods)]
  reps <- sprintf("r%02d", seq_len(n_replicates))
  out <- expand.grid(
    subject = subjects,
    method = methods,
    replicate = reps,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  s_idx <- match(out$subject, subjects)
  m_idx <- match(out$method, methods)
  r_idx <- match(out$replicate, reps)
  out$value <- 4 + 0.7 * s_idx + 0.2 * m_idx + 0.05 * r_idx +
    sin(s_idx + m_idx + r_idx) / 10
  out
}

cia_perf_prep <- function(dat, reference = NULL) {
  matrixCorr:::.mc_cia_prepare_long(
    data = dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = reference
  )
}

cia_perf_diagnostics <- function(prep) {
  raw <- matrixCorr:::cia_moments_cpp(
    y = prep$y,
    subject = prep$subject_code,
    method = prep$method_code,
    replicate = prep$replicate_code,
    n_methods = prep$n_methods,
    reference_method = prep$reference_index,
    has_reference = prep$has_reference,
    pairwise = TRUE,
    n_threads = 1L
  )
  matrixCorr:::.mc_cia_build_diagnostics(prep, raw)
}

test_that("pairwise payload reuse preserves matrix output", {
  dat <- make_cia_perf_data()
  prep <- cia_perf_prep(dat)
  diagnostics <- cia_perf_diagnostics(prep)
  payload <- matrixCorr:::.mc_cia_pairwise_payload(
    prep,
    diagnostics = diagnostics,
    estimator = "mom_unconstrained",
    n_threads = 1L
  )

  old_path <- matrixCorr:::.mc_cia_build_pairwise_matrix(
    prep,
    diagnostics,
    estimator = "mom_unconstrained",
    n_threads = 1L
  )
  new_path <- matrixCorr:::.mc_cia_build_pairwise_matrix(
    prep,
    diagnostics,
    estimator = "mom_unconstrained",
    n_threads = 1L,
    payload = payload
  )

  expect_identical(class(new_path), class(old_path))
  expect_identical(dimnames(new_path), dimnames(old_path))
  expect_identical(attributes(new_path), attributes(old_path))
  expect_equal(unclass(new_path), unclass(old_path), tolerance = 1e-12)
})

test_that("pairwise payload reuse preserves delta CI output with reference", {
  dat <- make_cia_perf_data()
  prep <- cia_perf_prep(dat, reference = "A")
  diagnostics <- cia_perf_diagnostics(prep)
  payload <- matrixCorr:::.mc_cia_pairwise_payload(
    prep,
    diagnostics = diagnostics,
    estimator = "vc_constrained",
    conf_level = 0.9,
    return_delta = TRUE,
    n_threads = 1L
  )

  old_path <- matrixCorr:::.mc_cia_build_pairwise_ci(
    prep,
    diagnostics,
    estimator = "vc_constrained",
    inference = "delta",
    conf_level = 0.9,
    n_threads = 1L
  )
  new_path <- matrixCorr:::.mc_cia_build_pairwise_ci(
    prep,
    diagnostics,
    estimator = "vc_constrained",
    inference = "delta",
    conf_level = 0.9,
    n_threads = 1L,
    payload = payload
  )

  expect_identical(class(new_path), class(old_path))
  expect_identical(names(new_path), names(old_path))
  expect_identical(attributes(new_path), attributes(old_path))
  expect_equal(new_path, old_path, tolerance = 1e-12)
})

test_that("pairwise bootstrap remains reproducible and reports retained resamples", {
  dat <- make_cia_perf_data()
  fit1 <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    ci = TRUE,
    inference = "bootstrap",
    B = 49L,
    seed = 42L
  )
  fit2 <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    ci = TRUE,
    inference = "bootstrap",
    B = 49L,
    seed = 42L
  )

  expect_equal(fit1, fit2, tolerance = 1e-12)
  expect_identical(attr(fit1, "bootstrap", exact = TRUE)$B, 49L)
  expect_identical(attr(fit1, "bootstrap", exact = TRUE)$n_successful, 49L)
})

test_that("reference pairwise bootstrap remains reproducible", {
  dat <- make_cia_perf_data()
  fit1 <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "pairwise",
    estimator = "vc_constrained",
    ci = TRUE,
    inference = "bootstrap",
    B = 49L,
    seed = 42L
  )
  fit2 <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "pairwise",
    estimator = "vc_constrained",
    ci = TRUE,
    inference = "bootstrap",
    B = 49L,
    seed = 42L
  )

  expect_equal(fit1, fit2, tolerance = 1e-12)
  expect_identical(attr(fit1, "bootstrap", exact = TRUE), list(n_successful = 49L, B = 49L))
})

test_that("overall CIA public paths remain stable", {
  dat <- make_cia_perf_data()
  plain <- cia(dat, "value", "subject", "method", "replicate", scope = "overall")
  ref <- cia(dat, "value", "subject", "method", "replicate", reference = "A", scope = "overall")
  delta <- cia(dat, "value", "subject", "method", "replicate", scope = "overall", ci = TRUE, inference = "delta")
  boot <- cia(dat, "value", "subject", "method", "replicate", scope = "overall", ci = TRUE,
              inference = "bootstrap", B = 49L, seed = 42L)

  expect_s3_class(plain, "cia_overall")
  expect_s3_class(ref, "cia_overall")
  expect_identical(attr(delta, "ci.method", exact = TRUE), "delta_normal")
  expect_identical(attr(boot, "bootstrap", exact = TRUE), list(n_successful = 49L, B = 49L))
  expect_named(
    plain,
    c("coefficient", "estimator", "cia", "cia_raw", "tau2", "tau2_raw",
      "boundary", "reference", "n_methods", "n_subjects", "n_obs", "K",
      "numerator_term", "denominator_term", "within_msd", "between_msd")
  )
})

test_that("CIA error and warning behaviour remains unchanged", {
  dat <- make_cia_perf_data()
  dup <- rbind(dat, dat[1L, , drop = FALSE])
  one_rep <- subset(dat, replicate == "r01")
  unbalanced <- dat[-1L, , drop = FALSE]
  small <- make_cia_perf_data(n_subjects = 8L)

  expect_error(
    cia(dup, "value", "subject", "method", "replicate"),
    "duplicated subject-method-replicate"
  )
  expect_error(
    cia(one_rep, "value", "subject", "method", "replicate"),
    "at least two replicated readings"
  )
  expect_error(
    cia(unbalanced, "value", "subject", "method", "replicate", scope = "overall"),
    "balanced-replication"
  )
  expect_error(
    cia(dat, "value", "subject", "method", "replicate", reference = "missing"),
    "reference"
  )
  expect_error(
    cia(dat, "value", "subject", "method", "replicate", ci = TRUE, inference = "none"),
    "must not be"
  )
  expect_error(
    cia(dat, "value", "subject", "method", "replicate", scope = "overall",
        estimator = "vc_constrained", ci = TRUE, inference = "delta"),
    "unconstrained moment estimator only"
  )
  expect_warning(
    cia(small, "value", "subject", "method", "replicate", scope = "pairwise"),
    "fewer than 10 eligible subjects"
  )
})

test_that("cia matches a simple two-method hand calculation", {
  dat <- data.frame(
    subject = rep(c("s1", "s2"), each = 4),
    method = rep(c("A", "A", "B", "B"), times = 2),
    replicate = rep(c("r1", "r2"), times = 4),
    value = c(0, 2, 0, 0, 10, 12, 10, 10)
  )

  overall <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "overall"
  )
  pairwise <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    estimator = "vc_constrained"
  ))

  expect_s3_class(overall, "cia_overall")
  expect_equal(overall$cia[[1]], 1)
  expect_false(any(c("lwr.ci", "upr.ci") %in% names(overall)))
  expect_s3_class(pairwise, "cia")
  expect_equal(unname(pairwise["A", "B"]), 1)
  expect_equal(unname(pairwise["B", "A"]), 1)
  expect_equal(unname(diag(pairwise)), c(1, 1))
  expect_false("fisher_z" %in% names(summary(pairwise)))
})

test_that("no-reference cia retains raw ratios and reports constrained defaults", {
  dat <- data.frame(
    subject = rep(c("s1", "s2"), each = 6),
    method = rep(c("A", "A", "B", "B", "C", "C"), times = 2),
    replicate = rep(c("r1", "r2"), times = 6),
    value = c(
      0, 2, 0, 0, 1, 1,
      10, 12, 10, 10, 11, 11
    )
  )

  overall <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "overall"
  )
  pairwise <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise"
  ))
  pairwise_constrained <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    scope = "pairwise",
    estimator = "vc_constrained"
  ))

  expect_equal(overall$cia[[1]], 1, tolerance = 1e-12)
  expect_equal(overall$cia_raw[[1]], 1, tolerance = 1e-12)
  expect_equal(unname(pairwise["A", "B"]), 1, tolerance = 1e-12)
  expect_equal(unname(pairwise["A", "C"]), 2, tolerance = 1e-12)
  expect_equal(unname(pairwise["B", "C"]), 0, tolerance = 1e-12)
  expect_equal(unname(attr(pairwise, "cia_raw")["A", "C"]), 2, tolerance = 1e-12)
  expect_equal(unname(attr(pairwise, "tau2_raw")["A", "C"]), -1, tolerance = 1e-12)
  expect_true(isTRUE(attr(pairwise, "boundary")["A", "C"]))
  expect_equal(unname(pairwise_constrained["A", "C"]), 1, tolerance = 1e-12)
})

test_that("cia supports raw and constrained reference-scaled estimation", {
  dat <- data.frame(
    subject = rep(c("s1", "s2"), each = 6),
    method = rep(c("A", "A", "B", "B", "C", "C"), times = 2),
    replicate = rep(c("r1", "r2"), times = 6),
    value = c(
      0, 2, 1, 1, 2, 2,
      10, 12, 11, 11, 14, 14
    )
  )

  overall <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "overall"
  )
  overall_raw <- cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "overall",
    estimator = "mom_unconstrained"
  )
  pairwise <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "pairwise"
  ))
  pairwise_raw <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "pairwise",
    estimator = "mom_unconstrained"
  ))

  expect_equal(overall$cia[[1]], 8 / 7, tolerance = 1e-12)
  expect_equal(overall$cia_raw[[1]], 8 / 7, tolerance = 1e-12)
  expect_equal(overall$tau2_raw[[1]], -0.25, tolerance = 1e-12)
  expect_true(isTRUE(overall$boundary[[1]]))
  expect_equal(overall$numerator_term[[1]], 4, tolerance = 1e-12)
  expect_equal(overall$denominator_term[[1]], 3.5, tolerance = 1e-12)
  expect_equal(overall_raw$cia[[1]], 8 / 7, tolerance = 1e-12)
  expect_equal(unname(pairwise["A", "B"]), 4, tolerance = 1e-12)
  expect_equal(unname(pairwise["A", "C"]), 4 / 6, tolerance = 1e-12)
  expect_equal(unname(attr(pairwise, "cia_raw")["A", "B"]), 4, tolerance = 1e-12)
  expect_true(isTRUE(attr(pairwise, "boundary")["A", "B"]))
  expect_equal(unname(pairwise_raw["A", "B"]), 4, tolerance = 1e-12)
  expect_equal(unname(pairwise["B", "C"]), NA_real_)
})

test_that("cia summaries and printing expose raw boundary diagnostics", {
  dat <- data.frame(
    subject = rep(c("s1", "s2"), each = 6),
    method = rep(c("A", "A", "B", "B", "C", "C"), times = 2),
    replicate = rep(c("r1", "r2"), times = 6),
    value = c(
      0, 2, 1, 1, 2, 2,
      10, 12, 11, 11, 14, 14
    )
  )

  fit <- suppressWarnings(cia(
    dat,
    response = "value",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    reference = "A",
    scope = "pairwise",
    estimator = "vc_constrained"
  ))
  smry <- summary(fit)

  expect_true(all(c("cia_raw", "tau2_raw", "tau2", "boundary") %in% names(smry)))
  expect_true(any(smry$boundary, na.rm = TRUE))
  expect_match(
    paste(capture.output(print(fit)), collapse = "\n"),
    "negative inter-method variability"
  )
  expect_match(
    paste(capture.output(print(smry)), collapse = "\n"),
    "negative inter-method variability"
  )
})

test_that("cia requires within-method replication", {
  dat <- data.frame(
    subject = rep(c("s1", "s2"), each = 2),
    method = rep(c("A", "B"), times = 2),
    replicate = "r1",
    value = c(1, 1, 2, 2)
  )

  expect_error(
    cia(
      dat,
      response = "value",
      subject = "subject",
      method = "method",
      replicate = "replicate",
      scope = "overall"
    ),
    "balanced-replication|replicated readings"
  )
})

#' @title Pairwise skipped correlation
#'
#' @description
#' Computes all pairwise skipped correlation coefficients for the numeric
#' columns of a matrix or data frame using a high-performance 'C++' backend.
#'
#' Skipped correlation detects bivariate outliers using a projection rule and
#' then computes Pearson or Spearman correlation on the retained observations.
#' It is designed for situations where marginally robust methods can still be
#' distorted by unusual points in the joint data cloud.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns will be excluded.
#' @param method Correlation computed after removing projected outliers. One of
#'   \code{"pearson"} (default) or \code{"spearman"}.
#' @param na_method Character scalar controlling missing-data handling.
#'   With \code{"error"}, the function requires all retained numeric columns to
#'   be free of missing or non-finite values and aborts otherwise. This is the
#'   recommended setting when you want a single common sample size across all
#'   pairs, reproducible skipped-row diagnostics on the same rows, or bootstrap
#'   inference via \code{ci = TRUE} / \code{p_value = TRUE}.
#'   \code{"pairwise"} recomputes each estimate on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param stand Logical; if \code{TRUE} (default), each variable in the pair is
#'   centred by its median and divided by a robust scale estimate before the
#'   projection outlier search. The scale estimate is the MAD when positive,
#'   with fallback to \eqn{\mathrm{IQR}/1.34898} and then the usual sample
#'   standard deviation if needed. This standardisation affects only outlier
#'   detection, not the final correlation computed on the retained
#'   observations.
#' @param outlier_rule One of \code{"idealf"} (default) or \code{"mad"}.
#'   The default uses the ideal-fourths interquartile width of projected
#'   distances; \code{"mad"} uses the median absolute deviation of projected
#'   distances.
#' @param cutoff Positive numeric constant multiplying the projected spread in
#'   the outlier rule
#'   \eqn{\mathrm{med}(d_{i\cdot}) + cutoff \times s(d_{i\cdot})}. Larger
#'   values flag fewer observations as outliers; smaller values flag more.
#'   Default \code{sqrt(qchisq(0.975, df = 2))}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param return_masks Logical; if \code{TRUE}, attach compact pairwise skipped-row
#'   indices as an attribute. Default \code{FALSE}.
#' @param ci Logical; if \code{TRUE}, attach percentile-bootstrap confidence
#'   intervals for each skipped correlation using the Wilcox (2015) B2
#'   resampling scheme. Default \code{FALSE}.
#' @param p_value Logical; if \code{TRUE}, attach bootstrap p-values for testing
#'   whether each skipped correlation is zero. Default \code{FALSE}.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default
#'   \code{0.95}.
#' @param n_boot Integer \eqn{\geq 2}. Number of bootstrap resamples used when
#'   \code{ci = TRUE} or \code{p_value = TRUE}. Default \code{2000}.
#' @param p_adjust One of \code{"none"} (default), \code{"hochberg"}, or
#'   \code{"ecp"}. Optional familywise-error procedure applied to the matrix of
#'   bootstrap p-values. \code{"hochberg"} corresponds to method H in Wilcox,
#'   Rousselet, and Pernet (2018); \code{"ecp"} corresponds to their simulated
#'   critical-p-value method ECP.
#' @param fwe_level Familywise-error level used when
#'   \code{p_adjust = "hochberg"} or \code{"ecp"}. Default \code{0.05}.
#' @param n_mc Integer \eqn{\geq 10}. Number of null Monte Carlo data sets used
#'   when \code{p_adjust = "ecp"} to estimate the critical p-value. Default
#'   \code{1000}.
#' @param seed Optional positive integer used to seed the bootstrap resampling
#'   when \code{ci = TRUE} or \code{p_value = TRUE}. If \code{NULL}, a fresh
#'   internal seed is generated.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#' @param x An object of class \code{skipped_corr}.
#' @param var1,var2 Optional column names or 1-based column indices used by
#'   [skipped_corr_masks()] to extract the skipped-row indices for one pair.
#' @param digits Integer; number of digits to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ... Additional arguments passed to the underlying print or plot helper.
#' @param title Character; plot title.
#' @param low_color,high_color,mid_color Colors used in the heatmap.
#' @param value_text_size Numeric text size for overlaid cell values.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#'
#' @return A symmetric correlation matrix with class \code{skipped_corr} and
#'   attributes \code{method = "skipped_correlation"}, \code{description}, and
#'   \code{package = "matrixCorr"}. When \code{return_masks = TRUE}, the matrix
#'   also carries a \code{skipped_masks} attribute containing compact pairwise
#'   skipped-row indices. The \code{diagnostics} attribute stores per-pair
#'   complete-case counts and skipped-row counts/proportions. When
#'   \code{ci = TRUE} or \code{p_value = TRUE}, bootstrap inference matrices are
#'   attached via attributes.
#'
#' @details
#' Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
#' observations and columns as variables. For a given pair of columns
#' \eqn{(x, y)}, write the observed bivariate points as
#' \eqn{u_i = (x_i, y_i)^\top}, \eqn{i=1,\ldots,n}. If \code{stand = TRUE},
#' each margin is first centred by its median and divided by a robust scale
#' estimate before outlier detection; otherwise the original pair is used. The
#' robust scale is the MAD when positive, with fallback to
#' \eqn{\mathrm{IQR}/1.34898} and then the usual sample standard deviation if
#' needed. Let \eqn{\tilde u_i} denote the resulting points and let \eqn{c} be
#' the componentwise median center of the detection cloud.
#'
#' For each observation \eqn{i}, define the direction vector
#' \eqn{b_i = \tilde u_i - c}. When \eqn{\|b_i\| > 0}, all observations are
#' projected onto the line through \eqn{c} in direction \eqn{b_i}. The
#' projected distances are
#' \deqn{
#' d_{ij} \;=\; \frac{|(\tilde u_j - c)^\top b_i|}{\|b_i\|},
#' \qquad j=1,\ldots,n.
#' }
#' For each direction \eqn{i}, observation \eqn{j} is flagged as an outlier if
#' 
#' \deqn{
#' d_{ij} \;>\; \mathrm{med}(d_{i\cdot}) + g\, s(d_{i\cdot}),
#' \qquad g = \code{cutoff},
#' }
#' where \eqn{s(\cdot)} is either the ideal-fourths interquartile width
#' (\code{outlier_rule = "idealf"}) or the median absolute deviation
#' (\code{outlier_rule = "mad"}). An observation is removed if it is flagged
#' for at least one projection direction. The skipped correlation is then the
#' ordinary Pearson or Spearman correlation computed from the retained
#' observations:
#' \deqn{
#' r_{\mathrm{skip}}(x,y) \;=\;
#' \mathrm{cor}\!\left(x_{\mathcal{K}}, y_{\mathcal{K}}\right),
#' }
#' where \eqn{\mathcal{K}} is the index set of observations not flagged as
#' outliers.
#'
#' Unlike marginally robust methods such as \code{pbcor()}, \code{wincor()},
#' or \code{bicor()}, skipped correlation is explicitly pairwise because
#' outlier detection depends on the joint geometry of each variable pair. As a
#' result, the reported matrix need not be positive semidefinite, even with
#' complete data.
#'
#' \strong{Computational notes.} In the complete-data path, each column pair
#' requires a full bivariate projection search, so the dominant cost is higher
#' than for marginal robust methods. The implementation evaluates pairs in
#' 'C++'; where available, pairs are processed with 'OpenMP' parallelism. With
#' \code{na_method = "pairwise"}, each pair is recomputed on its overlap of
#' non-missing rows. With \code{na_method = "complete"}, rows with any
#' non-finite value across the retained numeric columns are removed before any
#' pairwise outlier search. This gives a common row universe for all
#' skipped-correlation masks.
#'
#' \strong{Bootstrap inference.} When \code{ci = TRUE} or \code{p_value = TRUE},
#' the implementation uses the percentile-bootstrap strategy studied by Wilcox
#' (2015). Each bootstrap replicate resamples whole observation pairs with
#' replacement, reruns the skipped-correlation outlier detection on the
#' resampled data, and recomputes the skipped correlation on the retained
#' observations. This corresponds to Wilcox's B2 method and avoids the
#' statistically unsatisfactory shortcut of removing outliers only once before
#' bootstrapping. Bootstrap inference currently requires complete data
#' (\code{na_method = "error"} or \code{"complete"}). When
#' \code{p_adjust = "hochberg"}, the
#' bootstrap p-values are processed with Hochberg's step-up procedure (method H
#' in Wilcox, Rousselet, and Pernet, 2018). When \code{p_adjust = "ecp"}, the
#' package follows their ECP method and simulates \code{n_mc} null data sets
#' from a \eqn{p}-variate normal distribution with identity covariance,
#' recomputes the pairwise bootstrap p-values for each null data set, stores the
#' minimum p-value from each run, and estimates the \code{fwe_level} quantile of
#' that null distribution using the Harrell-Davis estimator. Hypotheses are then
#' rejected when their observed bootstrap p-values are less than or equal to the
#' estimated critical p-value. The calibrated H1 procedure from Wilcox,
#' Rousselet, and Pernet (2018) is not currently implemented.
#'
#' @references
#' Wilcox, R. R. (2004). Inferences based on a skipped correlation coefficient.
#' Journal of Applied Statistics, 31(2), 131-143.
#' \doi{10.1080/0266476032000148821}
#'
#' Wilcox, R. R. (2015). Inferences about the skipped correlation coefficient:
#' Dealing with heteroscedasticity and non-normality. Journal of Modern Applied
#' Statistical Methods, 14(1), 172-188.
#' \doi{10.22237/jmasm/1430453580}
#'
#' Wilcox, R. R., Rousselet, G. A., & Pernet, C. R. (2018). Improved methods
#' for making inferences about multiple skipped correlations. Journal of
#' Statistical Computation and Simulation, 88(16), 3116-3131.
#' \doi{10.1080/00949655.2018.1501051}
#'
#' @seealso [pbcor()], [wincor()], [bicor()]
#'
#' @examples
#' set.seed(12)
#' X <- matrix(rnorm(160 * 4), ncol = 4)
#' X[1, 1] <- 9
#' X[1, 2] <- -8
#'
#' R <- skipped_corr(X, method = "pearson")
#' print(R, digits = 2)
#' summary(R)
#' plot(R)
#'
#' Rm <- skipped_corr(X, method = "pearson", return_masks = TRUE)
#' skipped_corr_masks(Rm, 1, 2)
#'
#' # Example 1:
#' Xm <- as.matrix(datasets::mtcars[, c("mpg", "disp", "hp", "wt")])
#' Rm2 <- skipped_corr(Xm, method = "spearman")
#' print(Rm2, digits = 2)
#'
#' # Example 2:
#' Ri <- skipped_corr(Xm, method = "pearson", ci = TRUE, n_boot = 40, seed = 1)
#' ci(Ri)
#' confint(Ri)
#' tidy(Ri)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(R)
#' }
#'
#' @author Thiago de Paula Oliveira
#' @export
skipped_corr <- function(data,
                    method = c("pearson", "spearman"),
                    na_method = c("error", "pairwise", "complete"),
                    ci = FALSE,
                    p_value = FALSE,
                    conf_level = 0.95,
                    n_threads = getOption("matrixCorr.threads", 1L),
                    return_masks = FALSE,
                    stand = TRUE,
                    outlier_rule = c("idealf", "mad"),
                    cutoff = sqrt(stats::qchisq(0.975, df = 2)),
                    n_boot = 2000L,
                    p_adjust = c("none", "hochberg", "ecp"),
                    fwe_level = 0.05,
                    n_mc = 1000L,
                    seed = NULL,
                    output = c("matrix", "sparse", "edge_list"),
                    threshold = 0,
                    diag = TRUE) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )
  method <- match.arg(method)
  na_method <- match.arg(na_method)
  outlier_rule <- match.arg(outlier_rule)
  p_adjust <- match.arg(p_adjust)
  check_bool(stand, arg = "stand")
  check_bool(return_masks, arg = "return_masks")
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_scalar_numeric(cutoff, arg = "cutoff", lower = 0, closed_lower = FALSE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  do_inference <- isTRUE(ci) || isTRUE(p_value)
  if (isTRUE(ci)) {
    conf_level <- check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  } else {
    conf_level <- 0.95
  }
  if (isTRUE(do_inference)) {
    n_boot <- check_scalar_int_pos(n_boot, arg = "n_boot")
  } else {
    n_boot <- 1L
  }
  if (isTRUE(p_value) && identical(p_adjust, "ecp")) {
    check_prob_scalar(fwe_level, arg = "fwe_level", open_ends = TRUE)
    n_mc <- check_scalar_int_pos(n_mc, arg = "n_mc")
    if (n_mc < 10L) {
      abort_bad_arg("n_mc", message = "must be >= 10.")
    }
  } else {
    fwe_level <- 0.05
    n_mc <- 10L
  }
  if (isTRUE(do_inference) && !is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  } else {
    seed <- NULL
  }
  if (isTRUE(do_inference) && identical(na_method, "pairwise")) {
    abort_bad_arg(
      "na_method",
      message = "{.arg ci} and {.arg p_value} currently require {.code na_method = \"error\"} or {.code na_method = \"complete\"}."
    )
  }
  if (!isTRUE(p_value) && !identical(p_adjust, "none")) {
    abort_bad_arg(
      "p_adjust",
      message = "{.arg p_adjust} can only be used when {.arg p_value} = TRUE."
    )
  }

  numeric_data <- if (na_method == "error") {
    validate_corr_input(data)
  } else {
    validate_corr_input(data, check_na = FALSE)
  }
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 5L, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }
  colnames_data <- colnames(numeric_data)

  method_int <- switch(method, pearson = 0L, spearman = 1L)
  use_mad <- identical(outlier_rule, "mad")
  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  res <- skipcor_matrix_cpp(
    numeric_data,
    method_int = method_int,
    stand = stand,
    use_mad = use_mad,
    gval = cutoff,
    min_n = 5L,
    n_threads = n_threads,
    return_masks = return_masks,
    return_inference = do_inference,
    conf_level = conf_level,
    n_boot = n_boot,
    seed = if (is.null(seed)) sample.int(.Machine$integer.max, 1L) else seed,
    multiple_method_int = switch(p_adjust, none = 0L, hochberg = 1L, ecp = 2L),
    fwe_level = fwe_level,
    n_mc = n_mc
  )

  mask_payload <- NULL
  if (isTRUE(return_masks)) {
    mask_payload <- structure(
      list(
        pair_i = as.integer(res$pair_i),
        pair_j = as.integer(res$pair_j),
        skipped_rows = unname(lapply(res$skipped_rows, as.integer)),
        n_rows = nrow(numeric_data),
        n_cols = ncol(numeric_data),
        colnames = colnames_data
      ),
      class = "skipped_corr_masks"
    )
  }

  diag_payload <- list(
    n_complete = unclass(res$n_complete),
    skipped_n = unclass(res$skipped_n),
    skipped_prop = unclass(res$skipped_prop)
  )
  res_cor <- .mc_set_matrix_dimnames(res$cor, colnames_data)
  for (nm in names(diag_payload)) {
    diag_payload[[nm]] <- .mc_set_matrix_dimnames(diag_payload[[nm]], colnames_data)
  }
  diag_payload <- .mc_merge_diagnostics(diag_payload, diagnostics_extra)

  infer_payload <- NULL
  if (isTRUE(ci) || isTRUE(p_value)) {
    infer_payload <- list(
      conf_level = conf_level,
      n_boot = n_boot,
      method = switch(
        p_adjust,
        none = "bootstrap_b2",
        hochberg = "method_h",
        ecp = "ecp"
      )
    )
    if (isTRUE(ci)) {
      lwr <- unclass(res$lwr)
      upr <- unclass(res$upr)
      lwr <- .mc_set_matrix_dimnames(lwr, colnames_data)
      upr <- .mc_set_matrix_dimnames(upr, colnames_data)
      infer_payload$lwr <- lwr
      infer_payload$upr <- upr
    }
    if (isTRUE(p_value)) {
      p_mat <- unclass(res$p_value)
      p_mat <- .mc_set_matrix_dimnames(p_mat, colnames_data)
      infer_payload$p_value <- p_mat
      if (identical(p_adjust, "hochberg")) {
        padj <- unclass(res$p_value_adjusted)
        reject <- unclass(res$reject) > 0.5
        padj <- .mc_set_matrix_dimnames(padj, colnames_data)
        reject <- .mc_set_matrix_dimnames(reject, colnames_data)
        infer_payload$p_value_adjusted <- padj
        infer_payload$reject <- reject
        infer_payload$p_adjust <- "hochberg"
      } else if (identical(p_adjust, "ecp")) {
        reject <- unclass(res$reject) > 0.5
        reject <- .mc_set_matrix_dimnames(reject, colnames_data)
        infer_payload$reject <- reject
        infer_payload$critical_p_value <- unname(res$critical_p_value)
        infer_payload$p_adjust <- "ecp"
      } else {
        infer_payload$p_adjust <- "none"
      }
    }
  }

  ci_attr <- NULL
  inference_attr <- NULL
  if (!is.null(infer_payload)) {
    if (isTRUE(ci)) {
      ci_attr <- list(
        est = unclass(res_cor),
        lwr.ci = infer_payload$lwr,
        upr.ci = infer_payload$upr,
        conf.level = infer_payload$conf_level
      )
    }

    inference_attr <- list(
      method = infer_payload$method,
      n_boot = infer_payload$n_boot,
      conf.level = infer_payload$conf_level,
      fwe_level = fwe_level,
      p_adjust = infer_payload$p_adjust %||% "none"
    )
    if (isTRUE(ci)) {
      inference_attr$ci <- ci_attr
    }
    if (isTRUE(p_value)) {
      inference_attr$p_value <- infer_payload$p_value
      if (!is.null(infer_payload$p_value_adjusted)) {
        inference_attr$p_value_adjusted <- infer_payload$p_value_adjusted
      }
      if (!is.null(infer_payload$reject)) {
        inference_attr$reject <- infer_payload$reject
      }
      if (!is.null(infer_payload$critical_p_value)) {
        inference_attr$critical_p_value <- infer_payload$critical_p_value
      }
    }
    if (identical(p_adjust, "ecp")) inference_attr$n_mc <- n_mc
  }

  out <- .mc_structure_corr_matrix(
    res_cor,
    class_name = "skipped_corr",
    method = "skipped_correlation",
    description = paste0(
      "Skipped correlation; base = ", method,
      "; rule = ", outlier_rule,
      "; standardise = ", stand,
      "; NA mode = ", na_method, "."
    ),
    symmetric = TRUE,
    diagnostics = diag_payload,
    extra_attrs = c(
      if (isTRUE(return_masks)) list(skipped_masks = mask_payload),
      if (!is.null(ci_attr)) list(ci = ci_attr),
      if (!is.null(inference_attr)) list(inference = inference_attr)
    )
  )
  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_skipcor_ci_attr <- function(x) {
  ci <- attr(x, "ci", exact = TRUE)
  if (!is.null(ci)) return(ci)

  lwr <- attr(x, "lwr.ci", exact = TRUE)
  upr <- attr(x, "upr.ci", exact = TRUE)
  if (is.null(lwr) && is.null(upr)) return(NULL)

  list(
    est = unclass(as.matrix(x)),
    lwr.ci = lwr,
    upr.ci = upr,
    conf.level = attr(x, "conf.level", exact = TRUE)
  )
}

.mc_skipcor_inference_attr <- function(x) {
  inf <- attr(x, "inference", exact = TRUE)
  if (!is.null(inf)) return(inf)

  legacy <- list(
    method = attr(x, "inference_method", exact = TRUE),
    n_boot = attr(x, "n_boot", exact = TRUE),
    conf.level = attr(x, "conf.level", exact = TRUE),
    fwe_level = attr(x, "fwe_level", exact = TRUE),
    p_adjust = attr(x, "p_adjust", exact = TRUE),
    p_value = attr(x, "p_value", exact = TRUE),
    p_value_adjusted = attr(x, "p_value_adjusted", exact = TRUE),
    reject = attr(x, "reject", exact = TRUE),
    critical_p_value = attr(x, "critical_p_value", exact = TRUE)
  )
  if (all(vapply(legacy, is.null, logical(1)))) return(NULL)
  legacy$ci <- .mc_skipcor_ci_attr(x)
  legacy
}

.mc_skipcor_component_names <- function(x) {
  out <- c("estimate")
  if (!is.null(.mc_skipcor_ci_attr(x))) out <- c(out, "ci")
  inf <- .mc_skipcor_inference_attr(x)
  if (!is.null(inf)) {
    if (!is.null(inf$p_value)) out <- c(out, "p_value")
    if (!is.null(inf$p_value_adjusted)) out <- c(out, "p_value_adjusted")
    if (!is.null(inf$reject)) out <- c(out, "reject")
    if (!is.null(inf$critical_p_value)) out <- c(out, "critical_p_value")
    out <- c(out, "inference")
  }
  if (!is.null(attr(x, "diagnostics", exact = TRUE))) out <- c(out, "diagnostics")
  if (!is.null(attr(x, "skipped_masks", exact = TRUE))) out <- c(out, "skipped_masks")
  unique(out)
}

#' @export
names.skipped_corr <- function(x) {
  .mc_skipcor_component_names(x)
}

#' @export
`$.skipped_corr` <- function(x, name) {
  ci <- .mc_skipcor_ci_attr(x)
  inf <- .mc_skipcor_inference_attr(x)
  switch(name,
    estimate = unclass(as.matrix(x)),
    est = unclass(as.matrix(x)),
    cor = unclass(as.matrix(x)),
    ci = ci,
    p_value = if (is.null(inf)) NULL else inf$p_value,
    p_value_adjusted = if (is.null(inf)) NULL else inf$p_value_adjusted,
    reject = if (is.null(inf)) NULL else inf$reject,
    critical_p_value = if (is.null(inf)) NULL else inf$critical_p_value,
    inference = inf,
    diagnostics = attr(x, "diagnostics", exact = TRUE),
    skipped_masks = attr(x, "skipped_masks", exact = TRUE),
    NULL
  )
}

#' @export
`[[.skipped_corr` <- function(x, i, ...) {
  if (is.numeric(i)) {
    nms <- names(x)
    if (length(i) != 1L || is.na(i) || i < 1L || i > length(nms)) {
      abort_bad_arg("i", message = "must be a valid component index.")
    }
    i <- nms[[i]]
  }
  `$.skipped_corr`(x, i)
}

#' @rdname skipped_corr
#' @export
skipped_corr_masks <- function(x, var1 = NULL, var2 = NULL) {
  check_inherits(x, "skipped_corr")
  masks <- attr(x, "skipped_masks", exact = TRUE)
  if (is.null(masks)) return(NULL)
  if (is.null(var1) && is.null(var2)) return(masks)
  if (is.null(var1) || is.null(var2)) {
    abort_bad_arg("var1", message = "and {.arg var2} must both be supplied when extracting a specific pair mask.")
  }

  resolve_one <- function(var, arg) {
    if (is.character(var)) {
      if (length(var) != 1L || is.na(var)) {
        abort_bad_arg(arg, message = "must be a single non-missing column name or index.")
      }
      idx <- match(var, colnames(x))
      if (is.na(idx)) {
        abort_bad_arg(arg, message = "must match a column name in {.arg x}.")
      }
      idx
    } else {
      idx <- check_scalar_int_pos(var, arg = arg)
      if (idx > ncol(x)) {
        abort_bad_arg(arg, message = "must be <= ncol(x).")
      }
      idx
    }
  }

  i <- resolve_one(var1, "var1")
  j <- resolve_one(var2, "var2")
  if (i == j) return(integer())
  lo <- min(i, j)
  hi <- max(i, j)
  hit <- which(masks$pair_i == lo & masks$pair_j == hi)
  if (!length(hit)) integer() else masks$skipped_rows[[hit[[1L]]]]
}

#' @rdname skipped_corr
#' @method print skipped_corr
#' @param ci_digits Integer; digits for skipped-correlation confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param show_p One of \code{"auto"}, \code{"yes"}, \code{"no"}. For
#'   \code{print()}, \code{"auto"} keeps the compact matrix-only display;
#'   use \code{"yes"} to also print pairwise p-values.
#' @export
print.skipped_corr <- function(x,
                               digits = 4,
                               n = NULL,
                               topn = NULL,
                               max_vars = NULL,
                               width = NULL,
                               ci_digits = 4,
                               show_ci = NULL,
                               show_p = c("auto", "yes", "no"),
                               ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )
  show_p <- match.arg(show_p)
  .mc_print_corr_matrix(
    x,
    header = "Skipped correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  ci <- .mc_skipcor_ci_attr(x)
  inf <- .mc_skipcor_inference_attr(x)

  include_ci <- identical(show_ci, "yes") && !is.null(ci)
  include_p <- identical(show_p, "yes") && !is.null(inf) && !is.null(inf$p_value)
  if (include_p && !include_ci) {
    cli::cli_inform("Use {.code summary(x, show_ci = \"no\")} for bounded pairwise p-value output.")
  }
  invisible(x)
}

#' @rdname skipped_corr
#' @method plot skipped_corr
#' @param ci_text_size Text size for confidence intervals in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @export
plot.skipped_corr <- function(x,
                         title = "Skipped correlation heatmap",
                         low_color = "indianred1",
                         high_color = "steelblue1",
                         mid_color = "white",
                         value_text_size = 4,
                         ci_text_size = 3,
                         show_value = TRUE,
                         ...) {
  check_inherits(x, "skipped_corr")
  check_bool(show_value, arg = "show_value")

  est_mat <- as.matrix(x)
  df_est <- as.data.frame(as.table(est_mat))
  names(df_est) <- c("Var1", "Var2", "skipped_corr")

  ci <- .mc_skipcor_ci_attr(x)
  if (!is.null(ci) && !is.null(ci$lwr.ci) && !is.null(ci$upr.ci)) {
    df_lwr <- as.data.frame(as.table(ci$lwr.ci))
    names(df_lwr)[3] <- "lwr"
    df_upr <- as.data.frame(as.table(ci$upr.ci))
    names(df_upr)[3] <- "upr"
    df <- Reduce(
      function(a, b) merge(a, b, by = c("Var1", "Var2"), all = TRUE),
      list(df_est, df_lwr, df_upr)
    )

    diag_idx <- df$Var1 == df$Var2
    df$lwr[diag_idx] <- NA_real_
    df$upr[diag_idx] <- NA_real_
    df$ci_label <- ifelse(
      is.na(df$lwr) | is.na(df$upr),
      NA_character_,
      sprintf("(%.2f, %.2f)", df$lwr, df$upr)
    )
  } else {
    df <- df_est
    df$ci_label <- NA_character_
  }

  lev_row <- unique(df_est$Var1)
  lev_col <- unique(df_est$Var2)
  df$Var1 <- factor(df$Var1, levels = rev(lev_row))
  df$Var2 <- factor(df$Var2, levels = lev_col)
  df$label <- sprintf("%.2f", df$skipped_corr)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = skipped_corr)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = "skipped_corr"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value)) {
    p <- p + ggplot2::geom_text(ggplot2::aes(label = label), size = value_text_size, color = "black")
  }

  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.25),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }

  p
}

.mc_skipcor_pairwise_summary <- function(object,
                                         digits = 4,
                                         ci_digits = 2,
                                         p_digits = 4,
                                         show_ci = NULL,
                                         show_p = c("auto", "yes", "no")) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  show_p <- match.arg(show_p)
  check_inherits(object, "skipped_corr")

  ci <- .mc_skipcor_ci_attr(object)
  inf <- .mc_skipcor_inference_attr(object)
  diag_attr <- attr(object, "diagnostics", exact = TRUE)

  include_p <- switch(show_p, auto = !is.null(inf) && !is.null(inf$p_value), yes = TRUE, no = FALSE)

  base <- .mc_summary_corr_matrix(object)
  extra_columns <- list()
  if (isTRUE(include_p)) {
    extra_columns$p_value <- list(matrix = if (is.null(inf)) NULL else inf$p_value, digits = p_digits)
    if (!is.null(inf$p_value_adjusted)) {
      extra_columns$p_value_adjusted <- list(matrix = inf$p_value_adjusted, digits = p_digits)
    }
    if (!is.null(inf$reject)) {
      extra_columns$reject <- list(matrix = inf$reject, type = "logical")
    }
  }
  if (is.list(diag_attr)) {
    if (is.matrix(diag_attr$skipped_n)) {
      extra_columns$skipped_n <- list(matrix = diag_attr$skipped_n, type = "integer")
    }
    if (is.matrix(diag_attr$skipped_prop)) {
      extra_columns$skipped_prop <- list(matrix = diag_attr$skipped_prop, digits = p_digits)
    }
  }

  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.skipped_corr",
    digits = digits,
    ci_digits = ci_digits,
    p_digits = p_digits,
    show_ci = show_ci,
    ci_attr = ci,
    inference_attr = inf,
    diagnostics_attr = diag_attr,
    include_p = include_p,
    extra_columns = extra_columns,
    extra_attrs = list(
      inference_method = if (is.null(inf)) NA_character_ else inf$method,
      p_adjust = if (is.null(inf) || is.null(inf$p_adjust)) "none" else inf$p_adjust,
      critical_p_value = if (is.null(inf)) NA_real_ else inf$critical_p_value %||% NA_real_
    ),
    overview = base
  )
}

#' @rdname skipped_corr
#' @method summary skipped_corr
#' @param object An object of class \code{skipped_corr}.
#' @export
summary.skipped_corr <- function(object, n = NULL, topn = NULL,
                                 max_vars = NULL, width = NULL,
                                 show_ci = NULL, ...) {
  check_inherits(object, "skipped_corr")
  ci <- .mc_skipcor_ci_attr(object)
  inf <- .mc_skipcor_inference_attr(object)

  if (is.null(ci) && (is.null(inf) || is.null(inf$p_value))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }

  .mc_skipcor_pairwise_summary(
    object,
    show_ci = .mc_validate_yes_no(
      show_ci,
      arg = "show_ci",
      default = .mc_display_option("summary_show_ci", "yes")
    ),
    show_p = "auto"
  )
}

#' @rdname skipped_corr
#' @method print summary.skipped_corr
#' @param x An object of class \code{summary.skipped_corr}.
#' @export
print.summary.skipped_corr <- function(x, digits = NULL, n = NULL,
                                       topn = NULL, max_vars = NULL,
                                       width = NULL, show_ci = NULL, ...) {
  has_ci <- isTRUE(attr(x, "has_ci"))
  has_p <- isTRUE(attr(x, "has_p"))
  p_digits <- attr(x, "p_digits"); if (!is.numeric(p_digits)) p_digits <- 4
  extra_items <- c(
    if (has_p) c(inference = attr(x, "inference_method")),
    if (has_p) c(multiplicity = attr(x, "p_adjust"))
  )
  crit <- suppressWarnings(as.numeric(attr(x, "critical_p_value")))
  if (is.finite(crit)) {
    extra_items <- c(extra_items, critical_p = format(signif(crit, digits = p_digits)))
  }
  .mc_print_pairwise_summary_digest(
    x,
    title = "Skipped correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = if (has_ci) "bootstrap_b2" else NULL,
    extra_items = extra_items,
    ...
  )
  invisible(x)
}

pbcor_manual_R <- function(x, y, beta = 0.2) {
  stopifnot(length(x) == length(y), !anyNA(x), !anyNA(y))

  pb_scores <- function(z) {
    med <- stats::median(z)
    omega <- sort(abs(z - med))[floor((1 - beta) * length(z))]
    if (!is.finite(omega) || omega <= 0) return(rep(NA_real_, length(z)))
    psi <- (z - med) / omega
    i1 <- sum(psi < -1)
    i2 <- sum(psi > 1)
    s <- z
    s[psi < -1 | psi > 1] <- 0
    denom <- length(z) - i1 - i2
    if (denom <= 0) return(rep(NA_real_, length(z)))
    pbos <- (sum(s) + omega * (i2 - i1)) / denom
    a <- (z - pbos) / omega
    a[a <= -1] <- -1
    a[a >= 1] <- 1
    a
  }

  ax <- pb_scores(x)
  ay <- pb_scores(y)
  if (anyNA(ax) || anyNA(ay)) return(NA_real_)
  sum(ax * ay) / sqrt(sum(ax^2) * sum(ay^2))
}

pbcor_manual_matrix_R <- function(X, beta = 0.2) {
  X <- as.matrix(X)
  p <- ncol(X)
  out <- diag(1, nrow = p, ncol = p)
  colnames(out) <- rownames(out) <- colnames(X)
  for (j in seq_len(p - 1L)) {
    for (k in seq.int(j + 1L, p)) {
      val <- pbcor_manual_R(X[, j], X[, k], beta = beta)
      out[j, k] <- val
      out[k, j] <- val
    }
  }
  out
}

plain_matrix_R <- function(X) {
  X <- as.matrix(X)
  matrix(
    data = as.vector(X),
    nrow = nrow(X),
    ncol = ncol(X),
    dimnames = dimnames(X)
  )
}

wincor_manual_R <- function(x, y, tr = 0.2) {
  stopifnot(length(x) == length(y), !anyNA(x), !anyNA(y))

  winval <- function(z) {
    zs <- sort(z)
    g <- floor(tr * length(z))
    ibot <- g + 1L
    itop <- length(z) - g
    zbot <- zs[ibot]
    ztop <- zs[itop]
    z[z <= zbot] <- zbot
    z[z >= ztop] <- ztop
    z
  }

  stats::cor(winval(x), winval(y))
}

percentile_boot_ci_manual_R <- function(values, conf_level = 0.95) {
  values <- sort(values[is.finite(values)])
  n_boot <- length(values)
  if (!n_boot) {
    return(c(NA_real_, NA_real_))
  }

  alpha <- 1 - conf_level
  ilow <- floor((alpha / 2) * n_boot + 0.5)
  ihi <- floor((1 - alpha / 2) * n_boot + 0.5)
  ilow <- min(max(ilow, 1L), n_boot)
  ihi <- min(max(ihi, 1L), n_boot)
  c(values[[ilow]], values[[ihi]])
}

pbcor_manual_inference_R <- function(x,
                                     y,
                                     beta = 0.2,
                                     conf_level = 0.95,
                                     n_boot = 60L,
                                     seed = 1L) {
  est <- pbcor_manual_R(x, y, beta = beta)
  statistic <- if (abs(est) >= 1) sign(est) * Inf else est * sqrt((length(x) - 2) / (1 - est^2))
  p_value <- if (is.finite(statistic)) {
    2 * stats::pt(abs(statistic), df = length(x) - 2, lower.tail = FALSE)
  } else {
    0
  }

  set.seed(seed)
  boot_vals <- replicate(
    n_boot,
    {
      idx <- sample.int(length(x), size = length(x), replace = TRUE)
      pbcor_manual_R(x[idx], y[idx], beta = beta)
    }
  )
  ci <- percentile_boot_ci_manual_R(boot_vals, conf_level = conf_level)

  list(
    estimate = est,
    statistic = statistic,
    p_value = p_value,
    lwr = ci[[1L]],
    upr = ci[[2L]]
  )
}

wincor_manual_inference_R <- function(x,
                                      y,
                                      tr = 0.2,
                                      conf_level = 0.95,
                                      n_boot = 60L,
                                      seed = 1L) {
  est <- wincor_manual_R(x, y, tr = tr)
  g <- floor(tr * length(x))
  df <- length(x) - 2 * g - 2
  if (any(x != y) && df > 0) {
    statistic <- if (abs(est) >= 1) sign(est) * Inf else est * sqrt((length(x) - 2) / (1 - est^2))
    p_value <- if (is.finite(statistic)) {
      2 * stats::pt(abs(statistic), df = df, lower.tail = FALSE)
    } else {
      0
    }
  } else {
    statistic <- NA_real_
    p_value <- NA_real_
  }

  set.seed(seed)
  boot_vals <- replicate(
    n_boot,
    {
      idx <- sample.int(length(x), size = length(x), replace = TRUE)
      wincor_manual_R(x[idx], y[idx], tr = tr)
    }
  )
  ci <- percentile_boot_ci_manual_R(boot_vals, conf_level = conf_level)

  list(
    estimate = est,
    statistic = statistic,
    p_value = p_value,
    lwr = ci[[1L]],
    upr = ci[[2L]]
  )
}

idealf_iqr_R <- function(x) {
  n <- length(x)
  if (n < 2) return(0)
  j <- floor(n / 4 + 5 / 12)
  y <- sort(x)
  g <- (n / 4) - j + 5 / 12
  low <- (1 - g) * y[j] + g * y[j + 1]
  k <- n - j + 1
  up <- (1 - g) * y[k] + g * y[k - 1]
  up - low
}

skipcor_manual_R <- function(x, y,
                             method = c("pearson", "spearman"),
                             stand = TRUE,
                             outlier_rule = c("idealf", "mad"),
                             cutoff = sqrt(stats::qchisq(0.975, df = 2))) {
  method <- match.arg(method)
  outlier_rule <- match.arg(outlier_rule)
  ok <- is.finite(x) & is.finite(y)
  x <- x[ok]
  y <- y[ok]
  if (length(x) < 5) return(NA_real_)

  X <- cbind(x, y)
  if (stand) {
    for (j in 1:2) {
      med <- stats::median(X[, j])
      sc <- stats::mad(X[, j], constant = 1)
      if (!is.finite(sc) || sc <= 0) {
        qs <- stats::quantile(X[, j], c(0.25, 0.75), names = FALSE, type = 7)
        sc <- diff(qs) / 1.3489795003921634
      }
      if (!is.finite(sc) || sc <= 0) sc <- 1
      X[, j] <- (X[, j] - med) / sc
    }
  }

  center <- apply(X, 2, stats::median)
  B <- sweep(X, 2, center, "-")
  bot <- rowSums(B^2)
  outlier <- rep(FALSE, nrow(B))

  for (i in seq_len(nrow(B))) {
    if (!(bot[i] > 0)) next
    dis <- abs(drop(B %*% B[i, ])) / sqrt(bot[i])
    med <- stats::median(dis)
    spread <- if (outlier_rule == "mad") {
      stats::mad(dis, center = med, constant = 1)
    } else {
      idealf_iqr_R(dis)
    }
    if (!is.finite(spread) || spread <= 0) next
    thresh <- med + cutoff * spread
    outlier <- outlier | (dis > thresh)
  }

  if (sum(!outlier) < 5) return(NA_real_)
  if (method == "spearman") {
    stats::cor(x[!outlier], y[!outlier], method = "spearman")
  } else {
    stats::cor(x[!outlier], y[!outlier], method = "pearson")
  }
}

skipcor_manual_mask_R <- function(x, y,
                                  stand = TRUE,
                                  outlier_rule = c("idealf", "mad"),
                                  cutoff = sqrt(stats::qchisq(0.975, df = 2))) {
  outlier_rule <- match.arg(outlier_rule)
  ok <- is.finite(x) & is.finite(y)
  x_ok <- x[ok]
  y_ok <- y[ok]
  idx_ok <- which(ok)
  if (length(x_ok) < 5) return(integer())

  X <- cbind(x_ok, y_ok)
  if (stand) {
    for (j in 1:2) {
      med <- stats::median(X[, j])
      sc <- stats::mad(X[, j], constant = 1)
      if (!is.finite(sc) || sc <= 0) {
        qs <- stats::quantile(X[, j], c(0.25, 0.75), names = FALSE, type = 7)
        sc <- diff(qs) / 1.3489795003921634
      }
      if (!is.finite(sc) || sc <= 0) sc <- 1
      X[, j] <- (X[, j] - med) / sc
    }
  }

  center <- apply(X, 2, stats::median)
  B <- sweep(X, 2, center, "-")
  bot <- rowSums(B^2)
  outlier <- rep(FALSE, nrow(B))

  for (i in seq_len(nrow(B))) {
    if (!(bot[i] > 0)) next
    dis <- abs(drop(B %*% B[i, ])) / sqrt(bot[i])
    med <- stats::median(dis)
    spread <- if (outlier_rule == "mad") {
      stats::mad(dis, center = med, constant = 1)
    } else {
      idealf_iqr_R(dis)
    }
    if (!is.finite(spread) || spread <= 0) next
    thresh <- med + cutoff * spread
    outlier <- outlier | (dis > thresh)
  }

  idx_ok[outlier]
}

skipcor_manual_matrix_R <- function(X,
                                    method = c("pearson", "spearman"),
                                    stand = TRUE,
                                    outlier_rule = c("idealf", "mad"),
                                    cutoff = sqrt(stats::qchisq(0.975, df = 2))) {
  method <- match.arg(method)
  outlier_rule <- match.arg(outlier_rule)
  X <- as.matrix(X)
  p <- ncol(X)
  out <- diag(1, nrow = p, ncol = p)
  colnames(out) <- rownames(out) <- colnames(X)
  for (j in seq_len(p - 1L)) {
    for (k in seq.int(j + 1L, p)) {
      val <- skipcor_manual_R(
        X[, j],
        X[, k],
        method = method,
        stand = stand,
        outlier_rule = outlier_rule,
        cutoff = cutoff
      )
      out[j, k] <- val
      out[k, j] <- val
    }
  }
  out
}

test_that("pbcor matches manual percentage bend calculation", {
  set.seed(1001)
  x <- rnorm(60)
  y <- 0.7 * x + rnorm(60, sd = 0.4)
  x[1] <- 9
  y[1] <- -8

  R <- pbcor(cbind(x = x, y = y))
  expect_s3_class(R, "pbcor")
  expect_equal(R["x", "y"], pbcor_manual_R(x, y), tolerance = 1e-12)
})

test_that("pbcor compiled backend matches a fixed known example", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)

  R_cpp <- pbcor_matrix_cpp(X, beta = 0.2, n_threads = 1L)
  expect_equal(R_cpp[1, 2], 0.5901576900261898, tolerance = 1e-12)
  expect_equal(R_cpp[1, 2], pbcor_manual_R(x, y, beta = 0.2), tolerance = 1e-12)
})

test_that("pbcor matches the manual percentage bend matrix on wider input", {
  set.seed(1005)
  X <- matrix(rnorm(120 * 6), nrow = 120, ncol = 6)
  X[, 2] <- 0.75 * X[, 1] + rnorm(120, sd = 0.35)
  X[, 4] <- -0.60 * X[, 3] + rnorm(120, sd = 0.40)
  X[c(2, 11, 77), 5] <- X[c(2, 11, 77), 5] + c(12, -11, 9)
  X[c(4, 39), 6] <- X[c(4, 39), 6] - c(13, 10)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  R <- pbcor(X, beta = 0.2)
  expect_equal(
    unname(plain_matrix_R(R)),
    unname(plain_matrix_R(pbcor_manual_matrix_R(X, beta = 0.2))),
    tolerance = 1e-12
  )
})

test_that("pbcor honors n_threads without changing estimates", {
  set.seed(1006)
  X <- matrix(rnorm(120 * 5), nrow = 120, ncol = 5)
  X[, 2] <- 0.7 * X[, 1] + rnorm(120, sd = 0.4)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  fit1 <- pbcor(X, n_threads = 1L)
  fit2 <- pbcor(X, n_threads = 2L)

  expect_equal(unname(plain_matrix_R(fit1)), unname(plain_matrix_R(fit2)), tolerance = 1e-12)
})

test_that("pbcor compiled backend matches the manual matrix on wider input", {
  set.seed(1005)
  X <- matrix(rnorm(120 * 6), nrow = 120, ncol = 6)
  X[, 2] <- 0.75 * X[, 1] + rnorm(120, sd = 0.35)
  X[, 4] <- -0.60 * X[, 3] + rnorm(120, sd = 0.40)
  X[c(2, 11, 77), 5] <- X[c(2, 11, 77), 5] + c(12, -11, 9)
  X[c(4, 39), 6] <- X[c(4, 39), 6] - c(13, 10)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  expect_equal(
    unname(pbcor_matrix_cpp(X, beta = 0.2, n_threads = 1L)),
    unname(pbcor_manual_matrix_R(X, beta = 0.2)),
    tolerance = 1e-12
  )
})

test_that("wincor matches a fixed known example", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)

  R <- wincor(X, tr = 0.2)
  expect_equal(R[1, 2], 0.6887508025909920, tolerance = 1e-12)
  expect_equal(R[1, 2], wincor_manual_R(x, y, tr = 0.2), tolerance = 1e-12)
})

test_that("wincor matches manual winsorized Pearson calculation", {
  set.seed(1002)
  x <- rnorm(80)
  y <- 0.6 * x + rnorm(80, sd = 0.5)
  x[c(2, 5)] <- c(-10, 11)
  y[c(2, 5)] <- c(-9, 10)

  R <- wincor(cbind(x = x, y = y), tr = 0.2)
  expect_s3_class(R, "wincor")
  expect_equal(R["x", "y"], wincor_manual_R(x, y, tr = 0.2), tolerance = 1e-12)
})

test_that("pbcor and wincor pairwise NA mode use overlap only", {
  x <- c(-3, -2, -1, 0, 1, 2, 3, NA)
  y <- c(-2.8, -2, -1.2, 0.1, 1.1, 2.2, 3.1, 4)
  z <- c(1, NA, 2, 3, 4, 5, 6, 7)
  M <- cbind(x = x, y = y, z = z)

  Rp <- pbcor(M, na_method = "pairwise")
  Rw <- wincor(M, na_method = "pairwise")

  idx_xy <- is.finite(x) & is.finite(y)
  idx_xz <- is.finite(x) & is.finite(z)
  expect_equal(Rp["x", "y"], pbcor_manual_R(x[idx_xy], y[idx_xy]), tolerance = 1e-12)
  expect_equal(Rw["x", "z"], wincor_manual_R(x[idx_xz], z[idx_xz]), tolerance = 1e-12)
})

test_that("pbcor inference matches the method-specific reference path", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)
  ref <- pbcor_manual_inference_R(x, y, beta = 0.2, conf_level = 0.90, n_boot = 40L, seed = 11L)

  fit <- pbcor(
    X,
    beta = 0.2,
    ci = TRUE,
    p_value = TRUE,
    conf_level = 0.90,
    n_boot = 40L,
    seed = 11L
  )

  expect_equal(fit["x", "y"], ref$estimate, tolerance = 1e-12)
  expect_equal(attr(fit, "inference")$statistic["x", "y"], ref$statistic, tolerance = 1e-12)
  expect_equal(attr(fit, "inference")$p_value["x", "y"], ref$p_value, tolerance = 1e-12)
  expect_equal(attr(fit, "ci")$lwr.ci["x", "y"], ref$lwr, tolerance = 1e-12)
  expect_equal(attr(fit, "ci")$upr.ci["x", "y"], ref$upr, tolerance = 1e-12)
})

test_that("wincor inference matches the method-specific reference path", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)
  ref <- wincor_manual_inference_R(x, y, tr = 0.2, conf_level = 0.90, n_boot = 40L, seed = 11L)

  fit <- wincor(
    X,
    tr = 0.2,
    ci = TRUE,
    p_value = TRUE,
    conf_level = 0.90,
    n_boot = 40L,
    seed = 11L
  )

  expect_equal(fit["x", "y"], ref$estimate, tolerance = 1e-12)
  expect_equal(attr(fit, "inference")$statistic["x", "y"], ref$statistic, tolerance = 1e-12)
  expect_equal(attr(fit, "inference")$p_value["x", "y"], ref$p_value, tolerance = 1e-12)
  expect_equal(attr(fit, "ci")$lwr.ci["x", "y"], ref$lwr, tolerance = 1e-12)
  expect_equal(attr(fit, "ci")$upr.ci["x", "y"], ref$upr, tolerance = 1e-12)
})

test_that("pbcor and wincor defaults keep the pre-inference estimate path unchanged", {
  set.seed(144)
  X <- matrix(rnorm(160), nrow = 40, ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  fit_pb_old <- pbcor(X, beta = 0.2)
  fit_pb_new <- pbcor(X, beta = 0.2, ci = FALSE, p_value = FALSE)
  fit_win_old <- wincor(X, tr = 0.2)
  fit_win_new <- wincor(X, tr = 0.2, ci = FALSE, p_value = FALSE)

  expect_equal(unclass(fit_pb_old), unclass(fit_pb_new), tolerance = 0)
  expect_equal(unclass(fit_win_old), unclass(fit_win_new), tolerance = 0)
  expect_null(attr(fit_pb_new, "ci", exact = TRUE))
  expect_null(attr(fit_pb_new, "inference", exact = TRUE))
  expect_null(attr(fit_win_new, "ci", exact = TRUE))
  expect_null(attr(fit_win_new, "inference", exact = TRUE))
})

test_that("pbcor and wincor CI and p-value payloads are optional and well-formed", {
  set.seed(155)
  X <- matrix(rnorm(180), nrow = 45, ncol = 4)
  X[, 2] <- 0.7 * X[, 1] + rnorm(45, sd = 0.4)
  colnames(X) <- paste0("V", 1:4)

  fit_pb_ci <- pbcor(X, ci = TRUE, conf_level = 0.90, n_boot = 30L, seed = 2L)
  fit_pb_p <- pbcor(X, p_value = TRUE)
  fit_win_ci <- wincor(X, ci = TRUE, conf_level = 0.90, n_boot = 30L, seed = 2L)
  fit_win_p <- wincor(X, p_value = TRUE)

  expect_true(is.list(attr(fit_pb_ci, "ci", exact = TRUE)))
  expect_null(attr(fit_pb_ci, "inference", exact = TRUE))
  expect_true(all(attr(fit_pb_ci, "ci")$lwr.ci[upper.tri(fit_pb_ci)] <= attr(fit_pb_ci, "ci")$upr.ci[upper.tri(fit_pb_ci)], na.rm = TRUE))
  expect_true(all(attr(fit_pb_ci, "ci")$lwr.ci[upper.tri(fit_pb_ci)] >= -1, na.rm = TRUE))
  expect_true(all(attr(fit_pb_ci, "ci")$upr.ci[upper.tri(fit_pb_ci)] <= 1, na.rm = TRUE))
  expect_true(is.list(attr(fit_pb_p, "inference", exact = TRUE)))
  expect_null(attr(fit_pb_p, "ci", exact = TRUE))

  expect_true(is.list(attr(fit_win_ci, "ci", exact = TRUE)))
  expect_null(attr(fit_win_ci, "inference", exact = TRUE))
  expect_true(all(attr(fit_win_ci, "ci")$lwr.ci[upper.tri(fit_win_ci)] <= attr(fit_win_ci, "ci")$upr.ci[upper.tri(fit_win_ci)], na.rm = TRUE))
  expect_true(all(attr(fit_win_ci, "ci")$lwr.ci[upper.tri(fit_win_ci)] >= -1, na.rm = TRUE))
  expect_true(all(attr(fit_win_ci, "ci")$upr.ci[upper.tri(fit_win_ci)] <= 1, na.rm = TRUE))
  expect_true(is.list(attr(fit_win_p, "inference", exact = TRUE)))
  expect_null(attr(fit_win_p, "ci", exact = TRUE))
})

test_that("pbcor and wincor validate bootstrap arguments only when CI is requested", {
  X <- matrix(seq_len(30), nrow = 10, ncol = 3)
  colnames(X) <- c("A", "B", "C")

  pb_plain <- pbcor(X, ci = FALSE, p_value = FALSE, conf_level = NA_real_,
                    n_boot = 0L, seed = "bad")
  expect_s3_class(pb_plain, "pbcor")
  expect_null(attr(pb_plain, "ci", exact = TRUE))
  expect_null(attr(pb_plain, "inference", exact = TRUE))

  pb_p <- pbcor(X, p_value = TRUE, ci = FALSE, conf_level = NA_real_,
                n_boot = 0L, seed = "bad")
  expect_s3_class(pb_p, "pbcor")
  expect_true(is.list(attr(pb_p, "inference", exact = TRUE)))
  expect_null(attr(pb_p, "ci", exact = TRUE))

  win_plain <- wincor(X, ci = FALSE, p_value = FALSE, conf_level = NA_real_,
                      n_boot = 0L, seed = "bad")
  expect_s3_class(win_plain, "wincor")
  expect_null(attr(win_plain, "ci", exact = TRUE))
  expect_null(attr(win_plain, "inference", exact = TRUE))

  win_p <- wincor(X, p_value = TRUE, ci = FALSE, conf_level = NA_real_,
                  n_boot = 0L, seed = "bad")
  expect_s3_class(win_p, "wincor")
  expect_true(is.list(attr(win_p, "inference", exact = TRUE)))
  expect_null(attr(win_p, "ci", exact = TRUE))

  expect_error(
    pbcor(X, ci = TRUE, n_boot = 0L),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    pbcor(X, ci = TRUE, seed = "bad"),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    wincor(X, ci = TRUE, n_boot = 0L),
    class = "matrixCorr_arg_error"
  )
  expect_error(
    wincor(X, ci = TRUE, seed = "bad"),
    class = "matrixCorr_arg_error"
  )
})

test_that("pbcor and wincor summaries switch to pairwise inference tables when requested", {
  set.seed(166)
  X <- matrix(rnorm(150), nrow = 50, ncol = 3)
  X[, 2] <- 0.6 * X[, 1] + rnorm(50, sd = 0.5)
  colnames(X) <- c("A", "B", "C")

  pb_fit <- pbcor(X, ci = TRUE, p_value = TRUE, n_boot = 30L, seed = 5L)
  win_fit <- wincor(X, ci = TRUE, p_value = TRUE, n_boot = 30L, seed = 5L)

  pb_sm <- summary(pb_fit)
  win_sm <- summary(win_fit)

  expect_s3_class(pb_sm, "summary.pbcor")
  expect_s3_class(win_sm, "summary.wincor")
  expect_true(all(c("item1", "item2", "estimate", "lwr", "upr", "statistic", "p_value", "n_complete") %in% names(pb_sm)))
  expect_true(all(c("item1", "item2", "estimate", "lwr", "upr", "statistic", "p_value", "n_complete") %in% names(win_sm)))

  pb_txt <- capture.output(print(pb_sm))
  win_txt <- capture.output(print(win_sm))
  expect_true(any(grepl("Percentage bend correlation summary", pb_txt, fixed = TRUE)))
  expect_true(any(grepl("Winsorized correlation summary", win_txt, fixed = TRUE)))
})

test_that("skipped_corr matches fixed known examples", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)

  Rp <- skipped_corr(X, method = "pearson")
  Rs <- skipped_corr(X, method = "spearman")

  expect_equal(Rp[1, 2], 0.9978288433383867, tolerance = 1e-12)
  expect_equal(Rs[1, 2], 1.0000000000000000, tolerance = 1e-12)
  expect_equal(Rp[1, 2], skipcor_manual_R(x, y, method = "pearson"), tolerance = 1e-12)
  expect_equal(Rs[1, 2], skipcor_manual_R(x, y, method = "spearman"), tolerance = 1e-12)
})

test_that("skipped_corr matches manual skipped Pearson and Spearman calculations", {
  set.seed(1003)
  x <- rnorm(70)
  y <- 0.8 * x + rnorm(70, sd = 0.3)
  x[1] <- 7
  y[1] <- -7

  Rp <- skipped_corr(cbind(x = x, y = y), method = "pearson")
  Rs <- skipped_corr(cbind(x = x, y = y), method = "spearman")

  expect_s3_class(Rp, "skipped_corr")
  expect_equal(Rp["x", "y"], skipcor_manual_R(x, y, method = "pearson"), tolerance = 1e-12)
  expect_equal(Rs["x", "y"], skipcor_manual_R(x, y, method = "spearman"), tolerance = 1e-12)
  expect_gt(Rp["x", "y"], stats::cor(x, y))
})

test_that("skipped_corr matches the manual skipped-correlation matrix on wider input", {
  set.seed(1006)
  X <- matrix(rnorm(140 * 5), nrow = 140, ncol = 5)
  X[, 2] <- 0.65 * X[, 1] + rnorm(140, sd = 0.35)
  X[, 4] <- -0.70 * X[, 3] + rnorm(140, sd = 0.40)
  X[c(5, 17), 1] <- c(9, -8)
  X[c(5, 17), 2] <- c(-9, 8)
  X[c(33, 91), 4] <- X[c(33, 91), 4] + c(10, -11)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  R <- skipped_corr(X, method = "pearson")
  expect_equal(
    unname(plain_matrix_R(R)),
    unname(plain_matrix_R(skipcor_manual_matrix_R(X, method = "pearson"))),
    tolerance = 1e-12
  )
})

test_that("skipped_corr default output is unchanged and has no extra mask payload", {
  set.seed(1007)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  R_default <- skipped_corr(X, method = "pearson")
  R_masks <- skipped_corr(X, method = "pearson", return_masks = TRUE)

  expect_equal(unname(plain_matrix_R(R_default)), unname(plain_matrix_R(R_masks)), tolerance = 0)
  expect_null(attr(R_default, "skipped_masks", exact = TRUE))
  expect_true(inherits(attr(R_masks, "skipped_masks", exact = TRUE), "skipped_corr_masks"))
})

test_that("skipped_corr legacy example remains unchanged when inference is off", {
  set.seed(4242)
  X <- matrix(rnorm(80), ncol = 4)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- paste0("V", 1:4)

  R <- skipped_corr(X, method = "pearson")
  expected <- matrix(
    c(
      1.000000000000000,  0.442551271825158,  0.001069283910523, -0.056140658927032,
      0.442551271825158,  1.000000000000000, -0.193019542135606,  0.180008663965879,
      0.001069283910523, -0.193019542135606,  1.000000000000000,  0.072274267976520,
     -0.056140658927032,  0.180008663965879,  0.072274267976520,  1.000000000000000
    ),
    nrow = 4, byrow = TRUE,
    dimnames = list(colnames(X), colnames(X))
  )

  expect_equal(unname(plain_matrix_R(R)), unname(expected), tolerance = 1e-12)
  expect_null(attr(R, "lwr.ci", exact = TRUE))
  expect_null(attr(R, "p_value", exact = TRUE))
})

test_that("skipped_corr bootstrap inference attaches CI and p-value matrices", {
  set.seed(1101)
  X <- matrix(rnorm(60 * 3), nrow = 60, ncol = 3)
  X[, 2] <- 0.75 * X[, 1] + rnorm(60, sd = 0.35)
  X[1, 1] <- 9
  X[1, 2] <- -9
  colnames(X) <- c("A", "B", "C")

  R <- skipped_corr(
    X,
    method = "pearson",
    ci = TRUE,
    p_value = TRUE,
    n_boot = 200,
    seed = 123,
    p_adjust = "hochberg"
  )

  ci <- R$ci
  lwr <- ci$lwr.ci
  upr <- ci$upr.ci
  pval <- R$p_value
  padj <- R$p_value_adjusted
  rej <- R$reject

  expect_true(is.list(ci))
  expect_true(is.matrix(lwr))
  expect_true(is.matrix(upr))
  expect_true(is.matrix(pval))
  expect_true(is.matrix(padj))
  expect_true(is.matrix(rej))
  expect_equal(dim(lwr), dim(R))
  expect_equal(dim(upr), dim(R))
  expect_equal(dim(pval), dim(R))
  expect_equal(dim(padj), dim(R))
  expect_equal(unname(diag(lwr)), rep(1, ncol(X)), tolerance = 0)
  expect_equal(unname(diag(upr)), rep(1, ncol(X)), tolerance = 0)
  expect_equal(unname(diag(pval)), rep(0, ncol(X)), tolerance = 0)
  expect_equal(unname(diag(padj)), rep(0, ncol(X)), tolerance = 0)
  expect_true(isSymmetric(lwr))
  expect_true(isSymmetric(upr))
  expect_true(isSymmetric(pval))
  expect_true(isSymmetric(padj))
  expect_true(R["A", "B"] >= lwr["A", "B"] && R["A", "B"] <= upr["A", "B"])
  expect_identical(R$inference$method, "method_h")
  expect_identical(R$inference$p_adjust, "hochberg")

  idx <- upper.tri(pval, diag = FALSE) & is.finite(pval)
  expect_equal(
    padj[idx],
    stats::p.adjust(pval[idx], method = "hochberg"),
    tolerance = 1e-12
  )
  expect_identical(
    unname(rej[idx]),
    unname(padj[idx] <= R$inference$fwe_level)
  )
})

test_that("skipped_corr ECP inference returns a critical p-value and reject matrix", {
  set.seed(1103)
  X <- matrix(rnorm(50 * 3), nrow = 50, ncol = 3)
  X[, 2] <- 0.7 * X[, 1] + rnorm(50, sd = 0.3)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- c("A", "B", "C")

  R <- skipped_corr(
    X,
    method = "pearson",
    p_value = TRUE,
    n_boot = 80,
    p_adjust = "ecp",
    n_mc = 40,
    seed = 777
  )

  pval <- R$p_value
  reject <- R$reject
  critical <- R$critical_p_value

  expect_true(is.matrix(pval))
  expect_true(is.matrix(reject))
  expect_true(is.finite(critical))
  expect_identical(R$inference$method, "ecp")
  expect_identical(R$inference$p_adjust, "ecp")
  expect_identical(R$inference$n_mc, 40L)
  expect_equal(unname(diag(reject)), rep(FALSE, ncol(X)))
  idx <- upper.tri(pval, diag = FALSE) & is.finite(pval)
  expect_identical(unname(reject[idx]), unname(pval[idx] <= critical))
})

test_that("skipped_corr list-like accessors expose CI and inference components", {
  set.seed(1104)
  X <- matrix(rnorm(40 * 3), nrow = 40, ncol = 3)
  X[1, 1] <- 7
  X[1, 2] <- -7
  colnames(X) <- c("A", "B", "C")

  R <- skipped_corr(X, ci = TRUE, p_value = TRUE, n_boot = 60, seed = 99)

  expect_true("ci" %in% names(R))
  expect_true("p_value" %in% names(R))
  expect_true(is.list(R$ci))
  expect_true(is.matrix(R$ci$lwr.ci))
  expect_true(is.matrix(R[["p_value"]]))
  expect_true(is.list(R$inference))
})

test_that("skipped_corr plot supports CI overlays when intervals are available", {
  set.seed(1105)
  X <- matrix(rnorm(45 * 3), nrow = 45, ncol = 3)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- c("A", "B", "C")

  R <- skipped_corr(X, ci = TRUE, n_boot = 60, seed = 123)
  p <- plot(R, value_text_size = 2, ci_text_size = 2)

  expect_s3_class(p, "ggplot")
})

test_that("skipped_corr summary surfaces pairwise inference details when available", {
  set.seed(1106)
  X <- matrix(rnorm(40 * 4), nrow = 40, ncol = 4)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- paste0("V", 1:4)

  R_ci <- skipped_corr(X, ci = TRUE, n_boot = 60, seed = 123)
  sm_ci <- summary(R_ci)
  txt_ci <- capture.output(print(sm_ci, digits = 4))

  expect_s3_class(sm_ci, "summary.skipped_corr")
  expect_equal(nrow(sm_ci), choose(ncol(X), 2))
  expect_true(isTRUE(attr(sm_ci, "has_ci")))
  expect_false(isTRUE(attr(sm_ci, "has_p")))
  expect_match(paste(txt_ci, collapse = "\n"), "Skipped correlation summary")
  expect_match(paste(txt_ci, collapse = "\n"), "ci_width")

  R_p <- skipped_corr(X, ci = TRUE, p_value = TRUE, n_boot = 60, seed = 456)
  sm_p <- summary(R_p)
  txt_p <- capture.output(print(sm_p, digits = 4))

  expect_s3_class(sm_p, "summary.skipped_corr")
  expect_true(isTRUE(attr(sm_p, "has_ci")))
  expect_true(isTRUE(attr(sm_p, "has_p")))
  expect_match(paste(txt_p, collapse = "\n"), "inference")
  expect_match(paste(txt_p, collapse = "\n"), "bootstrap")
})

test_that("skipped_corr masks reconstruct the reported pairwise correlations", {
  set.seed(1008)
  X <- matrix(rnorm(90 * 3), nrow = 90, ncol = 3)
  X[, 2] <- 0.7 * X[, 1] + rnorm(90, sd = 0.25)
  X[c(5, 11), 1] <- c(9, -10)
  X[c(5, 11), 2] <- c(-9, 10)
  X[c(13, 24), 3] <- c(11, -12)
  X[7, 3] <- NA_real_
  colnames(X) <- c("A", "B", "C")

  R <- skipped_corr(X, method = "spearman", na_method = "pairwise", return_masks = TRUE)

  for (j in 1:(ncol(X) - 1L)) {
    for (k in (j + 1L):ncol(X)) {
      skipped <- skipped_corr_masks(R, j, k)
      keep <- is.finite(X[, j]) & is.finite(X[, k])
      if (length(skipped)) keep[skipped] <- FALSE
      expected <- if (sum(keep) < 5L) NA_real_ else stats::cor(X[keep, j], X[keep, k], method = "spearman")
      expect_equal(unname(R[j, k]), expected, tolerance = 1e-12)
    }
  }
})

test_that("skipped_corr mask accessor is symmetric by variable pair", {
  set.seed(1009)
  X <- matrix(rnorm(70 * 3), nrow = 70, ncol = 3)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- c("x", "y", "z")

  R <- skipped_corr(X, return_masks = TRUE)
  expect_identical(skipped_corr_masks(R, "x", "y"), skipped_corr_masks(R, "y", "x"))
})

test_that("skipped_corr two-variable masks expose expected skipped rows", {
  x <- c(1, 2, 3, 4, 5, 20, 7, 8, 9, 10)
  y <- c(1.2, 2.1, 2.8, 4.2, 5.1, -18, 6.9, 8.3, 8.8, 10.4)
  X <- cbind(x = x, y = y)

  R <- skipped_corr(X, method = "pearson", return_masks = TRUE)
  expect_identical(skipped_corr_masks(R, "x", "y"), 6L)
})

test_that("skipped_corr masks match manual skipped rows on pairwise NA input", {
  x <- c(-3, -2, -1, 0, 1, 2, 8, 3, NA)
  y <- c(-3.1, -2.2, -1.0, 0.2, 1.1, 2.0, -7.5, 2.9, 4)
  z <- c(1, 2, 3, 4, 5, 6, 7, NA, 9)
  M <- cbind(x = x, y = y, z = z)

  R <- skipped_corr(M, na_method = "pairwise", return_masks = TRUE)
  expect_identical(
    skipped_corr_masks(R, "x", "y"),
    skipcor_manual_mask_R(x, y)
  )
  expect_identical(
    skipped_corr_masks(R, "x", "z"),
    skipcor_manual_mask_R(x, z)
  )
})

test_that("new robust correlation classes support print and plot methods", {
  set.seed(1004)
  X <- matrix(rnorm(120), nrow = 30, ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  objs <- list(
    pbcor(X),
    wincor(X),
    skipped_corr(X)
  )

  for (obj in objs) {
    txt <- capture.output(print(obj, digits = 2))
    expect_true(length(txt) > 0)
    p <- plot(obj, value_text_size = 2)
    expect_s3_class(p, "ggplot")
    sm <- summary(obj)
    expect_s3_class(sm, "summary.matrixCorr")
    expect_s3_class(sm, "summary.corr_matrix")
  }
})

test_that("skipped_corr summary reports skipped counts and proportions", {
  set.seed(1102)
  X <- matrix(rnorm(50 * 4), nrow = 50, ncol = 4)
  X[1, 1] <- 8
  X[1, 2] <- -8
  colnames(X) <- paste0("V", 1:4)

  sm <- summary(skipped_corr(X))
  txt <- capture.output(print(sm, digits = 4))

  expect_match(paste(txt, collapse = "\n"), "skipped_n")
  expect_match(paste(txt, collapse = "\n"), "skipped_prop")
  expect_true(is.finite(sm$skipped_n_min))
  expect_true(is.finite(sm$skipped_n_max))
  expect_true(is.finite(sm$skipped_prop_min))
  expect_true(is.finite(sm$skipped_prop_max))
})

#' @title Intraclass Correlation for Wide Data
#'
#' @description
#' Computes intraclass correlation coefficients for the numeric columns of a
#' matrix or data frame using the classical ANOVA mean-square formulas. The
#' output can be either a pairwise matrix across columns or an overall
#' all-column coefficient table.
#'
#' @details
#' Each column is treated as a measurement channel, method, or rater and each
#' row is treated as a subject.
#'
#' The function supports two distinct analysis targets.
#'
#' - `scope = "pairwise"` answers: "how reliable is each specific column pair?"
#'   Each estimate is based on exactly two columns and the output is a symmetric
#'   matrix.
#' - `scope = "overall"` answers: "how reliable is the full set of columns when
#'   analysed jointly?" The output is the standard six-form overall ANOVA table
#'   (`ICC1`, `ICC2`, `ICC3`, `ICC1k`, `ICC2k`, `ICC3k`).
#'
#' These two scopes do not target the same quantity. The overall coefficients
#' are computed from the full multi-column ANOVA decomposition and are not
#' obtained by averaging or otherwise aggregating the pairwise matrix.
#'
#' The three main choice arguments determine the classical ICC form.
#'
#' - `model` controls the rater structure:
#'   - `"oneway"` uses the one-way random-effects formulation.
#'   - `"twoway_random"` uses the two-way random-effects formulation.
#'   - `"twoway_mixed"` uses the two-way mixed-effects formulation.
#' - `type` controls whether systematic column mean differences are penalized:
#'   - `"consistency"` targets consistency across columns.
#'   - `"agreement"` targets absolute agreement across columns.
#' - `unit` controls whether reliability refers to one measurement or to the
#'   average of multiple measurements:
#'   - `"single"` returns the single-measure coefficient.
#'   - `"average"` returns the average-measure coefficient.
#'
#' The supported mappings are:
#'
#' - `model = "oneway", type = "consistency", unit = "single"` gives `ICC1`.
#' - `model = "oneway", type = "consistency", unit = "average"` gives `ICC1k`.
#' - `model = "twoway_random", type = "agreement", unit = "single"` gives `ICC2`.
#' - `model = "twoway_random", type = "agreement", unit = "average"` gives `ICC2k`.
#' - `model = "twoway_random", type = "consistency", unit = "single"` gives `ICC3`.
#' - `model = "twoway_random", type = "consistency", unit = "average"` gives `ICC3k`.
#' - `model = "twoway_mixed", type = "agreement", unit = "single"` gives the
#'   mixed-effects absolute-agreement analogue with the same classical point
#'   formula as `ICC2`.
#' - `model = "twoway_mixed", type = "agreement", unit = "average"` gives the
#'   corresponding average-measure analogue.
#' - `model = "twoway_mixed", type = "consistency", unit = "single"` gives `ICC3`.
#' - `model = "twoway_mixed", type = "consistency", unit = "average"` gives `ICC3k`.
#'
#' The combination `model = "oneway", type = "agreement"` is not defined here
#' and returns an error.
#'
#' For `scope = "pairwise"`, the point estimates are computed in `C++` directly
#' from the two-column ANOVA mean squares for each complete pair. For
#' `unit = "average"`, the implementation uses `k = 2` because each estimate is
#' based on exactly two columns.
#'
#' For `scope = "overall"`, the point estimates are computed jointly from the
#' full wide matrix using the classical ANOVA decomposition over all columns.
#' Here the average-measure coefficients use `k = ncol(data)` after any row
#' filtering required by `na_method`.
#'
#' Missing-data handling depends on `scope`:
#'
#' - with `na_method = "error"`, missing values are rejected before estimation;
#' - with `na_method = "pairwise"` and `scope = "pairwise"`, each pair uses its
#'   own complete-case overlap;
#' - with `na_method = "pairwise"` and `scope = "overall"`, rows are restricted
#'   to complete cases across all columns because the overall ANOVA requires a
#'   common wide matrix.
#'
#' When `ci = TRUE`, confidence intervals are obtained from the classical
#' F-based ANOVA formulas corresponding to the selected coefficient. For
#' `scope = "pairwise"`, non-estimable off-diagonal pairs return `NA`. For
#' `scope = "overall"`, the coefficient table includes interval columns for all
#' six standard rows.
#'
#' @param data A numeric matrix or data frame with at least two numeric columns.
#' @param model Character scalar selecting the classical ICC model.
#'   `"oneway"` uses the one-way random-effects formulation,
#'   `"twoway_random"` uses the two-way random-effects formulation, and
#'   `"twoway_mixed"` uses the two-way mixed-effects formulation.
#' @param type Character scalar selecting the reliability target.
#'   `"consistency"` does not penalize systematic column mean differences in
#'   the same way as absolute agreement, whereas `"agreement"` targets absolute
#'   agreement and therefore incorporates those differences.
#' @param unit Character scalar selecting whether the coefficient refers to a
#'   single measurement (`"single"`) or to the mean of multiple measurements
#'   (`"average"`). For `scope = "pairwise"`, the average-measure coefficient
#'   always uses `k = 2`. For `scope = "overall"`, it uses the full number of
#'   analysed columns.
#' @param scope Character scalar selecting the analysis target.
#'   `"pairwise"` returns the symmetric matrix of two-column ICCs.
#'   `"overall"` returns the six-row overall ANOVA table for the full set of
#'   columns. Choose `"pairwise"` when the question is about specific column
#'   pairs; choose `"overall"` when the question is about the full set of
#'   columns analysed jointly.
#' @param ci Logical; if `TRUE`, return confidence intervals.
#' @param conf_level Confidence level for the interval output. Ignored when
#'   `ci = FALSE`.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values before
#'   estimation. \code{"pairwise"} uses pair-specific complete cases for
#'   \code{scope = "pairwise"} and complete rows across all analysed columns
#'   for \code{scope = "overall"}. \code{"complete"} performs listwise
#'   deletion once across all analysed columns before ICC estimation.
#' @param n_threads Integer number of OpenMP threads.
#' @param verbose Logical; if `TRUE`, report how many threads are requested.
#'
#' @return
#' For `scope = "pairwise"`, if `ci = FALSE`, a symmetric matrix of class
#' `icc`. If `ci = TRUE`, a list with elements `est`, `lwr.ci`, and `upr.ci`
#' and class `c("icc", "icc_ci")`.
#'
#' For `scope = "overall"`, a list of class `c("icc_overall", "icc")` with a
#' coefficient table, ANOVA table, and mean-square metadata. The coefficient
#' table always includes the standard six overall coefficients. Confidence
#' interval columns are attached when `ci = TRUE`.
#'
#' All outputs carry attributes describing the selected model, type, unit, and
#' method.
#'
#' @seealso [ccc()], [ccc_rm_reml()], [ba()], [ba_rm()], [rmcorr()]
#'
#' @references
#' Shrout PE, Fleiss JL (1979). Intraclass correlations: uses in assessing
#' rater reliability. Psychological Bulletin, 86(2), 420-428.
#'
#' McGraw KO, Wong SP (1996). Forming inferences about some intraclass
#' correlation coefficients. Psychological Methods, 1(1), 30-46.
#'
#' @examples
#' set.seed(123)
#' n <- 40
#' subj <- rnorm(n, sd = 1)
#' dat <- data.frame(
#'   m1 = subj + rnorm(n, sd = 0.3),
#'   m2 = 0.2 + subj + rnorm(n, sd = 0.4),
#'   m3 = -0.1 + subj + rnorm(n, sd = 0.5)
#' )
#'
#' fit_icc <- icc(dat,
#'   model = "twoway_random",
#'   type = "agreement",
#'   unit = "single",
#'   scope = "pairwise"
#' )
#' print(fit_icc)
#' summary(fit_icc)
#' estimate(fit_icc)
#' tidy(fit_icc)
#'
#' fit_icc_overall <- icc(dat, scope = "overall", ci = TRUE)
#' print(fit_icc_overall)
#' summary(fit_icc_overall)
#' confint(fit_icc_overall)
#'
#' @export
icc <- function(data,
                model = c("oneway", "twoway_random", "twoway_mixed"),
                type = c("consistency", "agreement"),
                unit = c("single", "average"),
                scope = c("pairwise", "overall"),
                na_method = c("error", "pairwise", "complete"),
                ci = FALSE,
                conf_level = 0.95,
                n_threads = getOption("matrixCorr.threads", 1L),
                verbose = FALSE,
                ...) {
  model <- match.arg(model)
  type <- match.arg(type)
  unit <- match.arg(unit)
  scope <- match.arg(scope)
  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method),
    allowed = c("error", "pairwise", "complete")
  )

  check_bool(ci, arg = "ci")
  check_bool(verbose, arg = "verbose")
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }

  if (identical(model, "oneway") && identical(type, "agreement")) {
    abort_bad_arg("type",
      message = "cannot be {.val agreement} when {.arg model} is {.val oneway}.",
      .hint = "Use type = \"consistency\" for the one-way formulation."
    )
  }

  numeric_data <- validate_corr_input(data, check_na = na_cfg$check_na)
  diagnostics_extra <- NULL
  if (identical(na_cfg$na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 2L, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }
  mat <- as.matrix(numeric_data)
  colnames_data <- colnames(numeric_data)

  inform_if_verbose(
    "Using {n_threads} OpenMP thread{?s}.",
    .verbose = verbose
  )

  form_code <- .mc_icc_form_code(model, type)
  average_unit <- identical(unit, "average")
  selected_coefficient <- .mc_icc_selected_coefficient(model, type, unit)

  if (identical(scope, "overall")) {
    if (na_cfg$check_na || identical(na_cfg$na_method, "complete")) {
      overall_data <- numeric_data
    } else {
      overall_data <- numeric_data
      keep <- stats::complete.cases(overall_data)
      overall_data <- overall_data[keep, , drop = FALSE]
      if (nrow(overall_data) < 2L) {
        abort_bad_arg("data",
          message = "must retain at least two complete rows across all columns when {.arg scope} is {.val overall}."
        )
      }
    }

    mat <- as.matrix(overall_data)
    inform_if_verbose(
      "Using native ANOVA backend.",
      .verbose = verbose
    )

    fit <- icc_overall_cpp(
      mat,
      return_ci = ci,
      conf_level = conf_level
    )

    coefficients <- fit$coefficients
    coefficients$selected <- coefficients$coefficient == selected_coefficient
    anova <- fit$anova

    out <- structure(
      list(
        coefficients = coefficients,
        anova = anova,
        mean_squares = fit$mean_squares
      ),
      class = c("icc_overall", "icc")
    )

    attr(out, "method") <- "Overall intraclass correlation table"
    attr(out, "description") <- "Overall intraclass correlation coefficients from the classical ANOVA decomposition"
    attr(out, "package") <- "matrixCorr"
    attr(out, "scope") <- scope
    attr(out, "model") <- model
    attr(out, "type") <- type
    attr(out, "unit") <- unit
    attr(out, "selected_coefficient") <- selected_coefficient
    attr(out, "selected_row") <- coefficients[coefficients$selected, , drop = FALSE]
    attr(out, "conf.level") <- conf_level
    attr(out, "ci.method") <- if (isTRUE(ci)) "anova_f" else NULL
    attr(out, "diagnostics") <- .mc_merge_diagnostics(list(
      n_complete = nrow(mat),
      n_subjects = fit$n_subjects,
      n_raters = fit$n_raters,
      dropped_rows = if (na_cfg$check_na) 0L else nrow(numeric_data) - nrow(mat)
    ), diagnostics_extra)
    return(out)
  }

  prev_threads <- get_omp_threads()
  on.exit(set_omp_threads(as.integer(prev_threads)), add = TRUE)

  fit <- icc_matrix_cpp(
    mat,
    form_code = form_code,
    average_unit = average_unit,
    pairwise_complete = identical(na_cfg$na_method, "pairwise"),
    return_ci = ci,
    conf_level = conf_level,
    n_threads = n_threads
  )

  fit$est <- `dimnames<-`(fit$est, list(colnames_data, colnames_data))
  fit$n_complete <- `dimnames<-`(fit$n_complete, list(colnames_data, colnames_data))
  if (isTRUE(ci)) {
    fit$lwr.ci <- `dimnames<-`(fit$lwr.ci, list(colnames_data, colnames_data))
    fit$upr.ci <- `dimnames<-`(fit$upr.ci, list(colnames_data, colnames_data))
  }

  method_label <- .mc_icc_method_label(model = model, type = type, unit = unit)
  description <- paste0("Pairwise intraclass correlation matrix (", method_label, ")")

  if (isTRUE(ci)) {
    out <- structure(
      list(est = fit$est, lwr.ci = fit$lwr.ci, upr.ci = fit$upr.ci),
      class = c("icc", "icc_ci")
    )
  } else {
    out <- structure(fit$est, class = c("icc", "matrix"))
  }

  attr(out, "method") <- method_label
  attr(out, "description") <- description
  attr(out, "package") <- "matrixCorr"
  attr(out, "scope") <- scope
  attr(out, "model") <- model
  attr(out, "type") <- type
  attr(out, "unit") <- unit
  attr(out, "k") <- 2L
  attr(out, "conf.level") <- conf_level
  attr(out, "ci.method") <- if (isTRUE(ci)) "anova_f" else NULL
  attr(out, "diagnostics") <- .mc_merge_diagnostics(
    list(n_complete = fit$n_complete),
    diagnostics_extra
  )
  out
}

.mc_icc_form_code <- function(model, type) {
  if (identical(model, "oneway")) {
    return(0L)
  }
  if (identical(type, "agreement")) {
    return(1L)
  }
  2L
}

.mc_icc_method_label <- function(model, type, unit) {
  base <- switch(model,
    oneway = "one-way",
    twoway_random = "two-way random",
    twoway_mixed = "two-way mixed"
  )
  paste("Intraclass correlation", sprintf("(%s, %s, %s)", base, type, unit))
}

.mc_icc_selected_coefficient <- function(model, type, unit) {
  base <- if (identical(model, "oneway")) {
    "ICC1"
  } else if (identical(type, "agreement")) {
    "ICC2"
  } else {
    "ICC3"
  }
  if (identical(unit, "average")) paste0(base, "k") else base
}

#' @rdname icc
#' @method print icc
#' @param x An intraclass-correlation object returned by `icc()`, or a summary
#'   object returned by `summary()` for that fit.
#' @param digits Integer; number of digits to print.
#' @param ci_digits Integer; number of digits for CI bounds.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to the underlying print helper.
#' @export
print.icc <- function(x,
                      digits = 4,
                      ci_digits = 4,
                      n = NULL,
                      topn = NULL,
                      max_vars = NULL,
                      width = NULL,
                      show_ci = NULL,
                      ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )

  is_ci_obj <- inherits(x, "icc_ci") ||
    (is.list(x) && all(c("est", "lwr.ci", "upr.ci") %in% names(x)))

  est <- if (is_ci_obj) as.matrix(x$est) else as.matrix(x)
  .mc_print_corr_matrix(
    x,
    header = "Intraclass correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = est,
    ...
  )
  invisible(x)
}

#' @rdname icc
#' @method summary icc
#' @param object An intraclass-correlation object returned by `icc()`.
#' @export
summary.icc <- function(object,
                        digits = 4,
                        ci_digits = 2,
                        n = NULL,
                        topn = NULL,
                        max_vars = NULL,
                        width = NULL,
                        show_ci = NULL,
                        ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  is_ci_obj <- inherits(object, "icc_ci") ||
    (is.list(object) && all(c("est", "lwr.ci", "upr.ci") %in% names(object)))

  if (is_ci_obj) {
    est <- as.matrix(object$est)
    lwr <- as.matrix(object$lwr.ci)
    upr <- as.matrix(object$upr.ci)
    conf_level <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  } else if (is.matrix(object)) {
    est <- as.matrix(object)
    lwr <- matrix(NA_real_, nrow(est), ncol(est), dimnames = dimnames(est))
    upr <- lwr
    conf_level <- suppressWarnings(as.numeric(attr(object, "conf.level")))
  } else {
    abort_bad_arg("object",
      message = "must be a matrix or a list with elements `est`, `lwr.ci`, and `upr.ci`."
    )
  }
  if (length(conf_level) != 1L || !is.finite(conf_level)) conf_level <- NA_real_

  rn <- rownames(est); cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  has_any_ci <- any(is.finite(lwr) | is.finite(upr))
  include_ci <- identical(show_ci, "yes") && has_any_ci

  rows <- vector("list", nrow(est) * (ncol(est) - 1L) / 2L)
  k <- 0L
  for (i in seq_len(nrow(est) - 1L)) {
    for (j in (i + 1L):ncol(est)) {
      k <- k + 1L
      rec <- list(
        method1 = rn[i],
        method2 = cn[j],
        estimate = round(est[i, j], digits)
      )
      if (include_ci) {
        rec$lwr <- if (is.na(lwr[i, j])) NA_real_ else round(lwr[i, j], ci_digits)
        rec$upr <- if (is.na(upr[i, j])) NA_real_ else round(upr[i, j], ci_digits)
      }
      rows[[k]] <- rec
    }
  }

  df <- do.call(rbind.data.frame, rows)
  rownames(df) <- NULL
  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) {
    vals <- integer(nrow(df))
    kk <- 0L
    for (i in seq_len(nrow(est) - 1L)) {
      for (j in (i + 1L):ncol(est)) {
        kk <- kk + 1L
        vals[kk] <- as.integer(diag_attr$n_complete[i, j])
      }
    }
    df$n_complete <- vals
  }
  df$estimate <- as.numeric(df$estimate)
  if (include_ci) {
    df$lwr <- as.numeric(df$lwr)
    df$upr <- as.numeric(df$upr)
  }

  df <- .mc_finalize_summary_df(df, class_name = "summary.icc")
  attr(df, "overview") <- .mc_summary_corr_matrix(est, topn = topn)
  attr(df, "conf.level") <- if (is.finite(conf_level)) conf_level else NA_real_
  attr(df, "has_ci") <- isTRUE(include_ci)
  attr(df, "digits") <- digits
  attr(df, "ci_digits") <- ci_digits
  attr(df, "ci_method") <- attr(object, "ci.method", exact = TRUE) %||% NA_character_
  df
}

#' @rdname icc
#' @method print summary.icc
#' @export
print.summary.icc <- function(x, digits = NULL, n = NULL,
                              topn = NULL, max_vars = NULL,
                              width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Intraclass correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = .mc_coalesce(attr(x, "ci_method"), "anova_f"),
    ...
  )
  invisible(x)
}

#' @rdname icc
#' @method print icc_overall
#' @param x An intraclass-correlation object returned by `icc()`, or a summary
#'   object returned by `summary()` for that fit.
#' @export
print.icc_overall <- function(x,
                              digits = 4,
                              ci_digits = 4,
                              n = NULL,
                              topn = NULL,
                              max_vars = NULL,
                              width = NULL,
                              show_ci = NULL,
                              ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )

  coeff <- summary(x,
    digits = digits,
    ci_digits = ci_digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )

  diag_info <- attr(x, "diagnostics", exact = TRUE) %||% list()
  digest <- c(method = attr(x, "method", exact = TRUE) %||% "Overall intraclass correlation")
  if (is.finite(diag_info$n_subjects %||% NA_real_)) {
    digest <- c(digest, subjects = .mc_count_fmt(diag_info$n_subjects))
  }
  if (is.finite(diag_info$n_raters %||% NA_real_)) {
    digest <- c(digest, raters = .mc_count_fmt(diag_info$n_raters))
  }
  if (!is.null(attr(x, "selected_coefficient", exact = TRUE))) {
    digest <- c(digest, selected = attr(x, "selected_coefficient", exact = TRUE))
  }
  .mc_print_named_digest(digest, header = "Overall intraclass correlation")

  cfg <- .mc_resolve_display_args(
    context = "print",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  cat("\nCoefficient table\n\n")
  .mc_print_preview_table(
    if (identical(show_ci, "no")) coeff[, setdiff(names(coeff), c("lwr", "upr")), drop = FALSE] else coeff,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "print",
    full_hint = TRUE,
    summary_hint = FALSE,
    ...
  )
  invisible(x)
}

#' @rdname icc
#' @method summary icc_overall
#' @param object An intraclass-correlation object returned by `icc()`.
#' @export
summary.icc_overall <- function(object,
                                digits = 4,
                                ci_digits = 2,
                                n = NULL,
                                topn = NULL,
                                max_vars = NULL,
                                width = NULL,
                                show_ci = NULL,
                                ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  coeff <- object$coefficients
  num_cols <- intersect(c("estimate", "statistic", "df1", "df2", "p_value"), names(coeff))
  for (nm in num_cols) coeff[[nm]] <- round(as.numeric(coeff[[nm]]), digits)
  if (all(c("lwr", "upr") %in% names(coeff))) {
    coeff$lwr <- round(as.numeric(coeff$lwr), ci_digits)
    coeff$upr <- round(as.numeric(coeff$upr), ci_digits)
  }
  coeff$selected <- as.logical(coeff$selected)

  anova <- object$anova
  if (!is.null(anova)) {
    anova$df <- round(as.numeric(anova$df), digits)
    anova$sum_sq <- round(as.numeric(anova$sum_sq), digits)
    anova$mean_sq <- round(as.numeric(anova$mean_sq), digits)
    anova$statistic <- round(as.numeric(anova$statistic), digits)
    anova$p_value <- round(as.numeric(anova$p_value), digits)
  }

  out <- structure(coeff, class = c("summary.icc_overall", "summary.matrixCorr", "data.frame"))
  attr(out, "anova") <- anova
  attr(out, "mean_squares") <- object$mean_squares
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE)
  attr(out, "has_ci") <- all(c("lwr", "upr") %in% names(coeff))
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "selected_coefficient") <- attr(object, "selected_coefficient", exact = TRUE)
  attr(out, "diagnostics") <- attr(object, "diagnostics", exact = TRUE)
  out
}

#' @rdname icc
#' @method print summary.icc_overall
#' @param x An intraclass-correlation object returned by `icc()`, or a summary
#'   object returned by `summary()` for that fit.
#' @export
print.summary.icc_overall <- function(x,
                                      digits = NULL,
                                      n = NULL,
                                      topn = NULL,
                                      max_vars = NULL,
                                      width = NULL,
                                      show_ci = NULL,
                                      ...) {
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  digits <- .mc_coalesce(digits, attr(x, "digits", exact = TRUE) %||% 4)
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  diag_info <- attr(x, "diagnostics", exact = TRUE) %||% list()
  ms <- attr(x, "mean_squares", exact = TRUE)

  digest <- character()
  if (!is.null(attr(x, "selected_coefficient", exact = TRUE))) {
    digest <- c(digest, selected = attr(x, "selected_coefficient", exact = TRUE))
  }
  if (is.finite(diag_info$n_subjects %||% NA_real_)) {
    digest <- c(digest, subjects = .mc_count_fmt(diag_info$n_subjects))
  }
  if (is.finite(diag_info$n_raters %||% NA_real_)) {
    digest <- c(digest, raters = .mc_count_fmt(diag_info$n_raters))
  }
  if (is.finite(diag_info$n_complete %||% NA_real_)) {
    digest <- c(digest, n_complete = .mc_count_fmt(diag_info$n_complete))
  }
  if (is.numeric(ms) && length(ms)) {
    digest <- c(
      digest,
      ms_subject = formatC(unname(ms["ms_subject"]), format = "f", digits = digits),
      ms_rater = formatC(unname(ms["ms_rater"]), format = "f", digits = digits),
      ms_error = formatC(unname(ms["ms_error"]), format = "f", digits = digits)
    )
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = "Overall intraclass correlation summary")

  cat("\nICC coefficients\n\n")
  .mc_print_preview_table(
    if (identical(show_ci, "no")) x[, setdiff(names(x), c("lwr", "upr")), drop = FALSE] else x,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = FALSE,
    summary_hint = FALSE,
    ...
  )

  anova <- attr(x, "anova", exact = TRUE)
  if (is.data.frame(anova) && nrow(anova)) {
    cat("\nANOVA decomposition\n\n")
    .mc_print_preview_table(
      anova,
      n = cfg$n,
      topn = cfg$topn,
      max_vars = cfg$max_vars,
      width = cfg$width,
      context = "summary",
      full_hint = FALSE,
      summary_hint = FALSE,
      ...
    )
  }
  invisible(x)
}

#' @title Repeated-Measures Intraclass Correlation via REML
#'
#' @description
#' Computes pairwise repeated-measures intraclass correlation coefficients from
#' long-format data using the same REML and Woodbury-identity backend used by
#' the repeated-measures agreement models.
#'
#' @details
#' The repeated-measures model is fit separately for each method pair using the
#' same REML and Woodbury-identity backend used by the repeated-measures
#' concordance estimator.
#'
#' \strong{Kernel \eqn{D_m} and the fixed-bias term \eqn{S_B}.}
#' Let \eqn{d = L^\top \hat\beta} stack the within-time, pairwise method
#' differences, grouped by time. The symmetric positive semidefinite kernel
#' \eqn{D_m \succeq 0} selects which functional of the bias profile is targeted
#' by \eqn{S_B}. Internally, the code rescales any supplied or constructed
#' \eqn{D_m} to satisfy \eqn{1^\top D_m 1 = n_t} for stability and comparability.
#'
#' \itemize{
#'   \item \code{Dmat_type = "time-avg"} targets the square of the
#'   time-averaged bias.
#'   \item \code{Dmat_type = "typical-visit"} targets the average of squared
#'   per-time biases.
#'   \item \code{Dmat_type = "weighted-avg"} targets the square of a weighted
#'   time average.
#'   \item \code{Dmat_type = "weighted-sq"} targets the weighted average of
#'   squared per-time biases.
#' }
#'
#' As in the repeated-measures concordance implementation, \eqn{S_B} is the
#' fitted fixed-effect dispersion term induced by \eqn{D_m}. It enters the
#' denominator only for `type = "agreement"`.
#'
#' \strong{Time-averaging and shrinkage factors.}
#' The fitted variance components reported in the summary are
#' \eqn{\sigma_S^2}, \eqn{\sigma_{S\times M}^2}, \eqn{\sigma_{S\times T}^2},
#' and \eqn{\sigma_E^2}, stored respectively as `sigma2_subject`,
#' `sigma2_subject_method`, `sigma2_subject_time`, and `sigma2_error`.
#'
#' The repeated-measures ICC always uses \eqn{\sigma_S^2} in the numerator only.
#' The denominator uses the same time-averaging logic already used by the
#' repeated-measures concordance backend, through two pair-specific factors
#' \eqn{\bar{\kappa}_g} and \eqn{\bar{\kappa}_e}.
#'
#' If `time` is absent, the implementation sets
#' \deqn{ \bar{\kappa}_g = 0, \qquad \bar{\kappa}_e = 1. }
#'
#' If `time` is present and the target is a single visit
#' (\code{Dmat_type \%in\% c("typical-visit", "weighted-sq")}), the implementation
#' leaves the time-varying terms unshrunk:
#' \deqn{ \bar{\kappa}_g = 1, \qquad \bar{\kappa}_e = 1. }
#'
#' If `time` is present and the target is time-averaged
#' (\code{Dmat_type \%in\% c("time-avg", "weighted-avg")}), then for each observed
#' subject-method unit with \eqn{T} distinct observed visits:
#' \itemize{
#'   \item with equal weights,
#'   \deqn{ \kappa_g = \frac{1}{T}, \qquad
#'          \kappa_e = \frac{T + 2\sum_{h=1}^{T-1}(T-h)\rho^h}{T^2}, }
#'   with \eqn{\kappa_e=\kappa_g} when residuals are iid;
#'   \item with normalized visit weights \eqn{w_1,\ldots,w_T},
#'   \deqn{ \kappa_g = \sum_{t=1}^{T} w_t^2, \qquad
#'          \kappa_e = \sum_{t=1}^{T}\sum_{s=1}^{T} w_t w_s \rho^{|t-s|}, }
#'   with \eqn{\kappa_e=\kappa_g} when residuals are iid.
#' }
#'
#' With unbalanced \eqn{T}, the implementation averages the per-unit
#' \eqn{\kappa} values across the observations contributing to the pair and then
#' clamps both \eqn{\bar{\kappa}_g} and \eqn{\bar{\kappa}_e} to
#' \eqn{[10^{-12},\,1]} for numerical stability.
#'
#' \strong{Repeated-measures ICC.}
#' Let \eqn{\sigma_{S\times M,\mathrm{eff}}^2} denote the effective
#' subject-by-method variance term, equal to \eqn{\sigma_{S\times M}^2} when
#' the subject-by-method random effect is included and \eqn{0} otherwise. Let
#' \eqn{\sigma_{S\times T,\mathrm{eff}}^2} denote the effective subject-by-time
#' variance term, equal to \eqn{\sigma_{S\times T}^2} when the subject-by-time
#' random effect is included and \eqn{0} otherwise.
#'
#' For `type = "consistency"`, the reported ICC is
#' \deqn{ \mathrm{ICC}_{\mathrm{consistency}} \;=\;
#'       \frac{\sigma_S^2}{
#'       \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
#'       \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
#'       \bar{\kappa}_e\,\sigma_E^2}. }
#'
#' For `type = "agreement"`, the denominator additionally includes \eqn{S_B}:
#' \deqn{ \mathrm{ICC}_{\mathrm{agreement}} \;=\;
#'       \frac{\sigma_S^2}{
#'       \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
#'       \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
#'       \bar{\kappa}_e\,\sigma_E^2 + S_B}. }
#'
#' This differs from repeated-measures concordance because ICC uses only
#' \eqn{\sigma_S^2} in the numerator, whereas the concordance numerator also
#' includes the time-averaged subject-time term. Extra random-effect variances
#' \eqn{\{\sigma_{Z,j}^2\}} from `slope` / `slope_Z` are estimated by the shared
#' backend but are not included in the ICC denominator.
#'
#' \strong{CIs / SEs (delta method for ICC).}
#' Let \eqn{I_A = 1} for \code{type = "agreement"} and \eqn{I_A = 0} for
#' \code{type = "consistency"}, and define
#' \deqn{ \theta \;=\; \big(\sigma_S^2,\ \sigma_{S\times M,\mathrm{eff}}^2,\
#' \sigma_{S\times T,\mathrm{eff}}^2,\ \sigma_E^2,\ S_B\big)^\top. }
#' Write \eqn{\mathrm{ICC}(\theta)=N/D} with
#' \deqn{ N = \sigma_S^2, \qquad
#'        D = \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
#'            \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
#'            \bar{\kappa}_e\,\sigma_E^2 + I_A S_B. }
#' The gradient used in the delta method is
#' \deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_S^2}
#'       \;=\; \frac{
#'       \sigma_{S\times M,\mathrm{eff}}^2 +
#'       \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
#'       \bar{\kappa}_e\,\sigma_E^2 + I_A S_B}{D^2}, }
#' \deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_{S\times M,\mathrm{eff}}^2}
#'       \;=\; -\,\frac{N}{D^2}, \qquad
#'        \frac{\partial\,\mathrm{ICC}}{\partial \sigma_{S\times T,\mathrm{eff}}^2}
#'       \;=\; -\,\frac{\bar{\kappa}_g\,N}{D^2}, }
#' \deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_E^2}
#'       \;=\; -\,\frac{\bar{\kappa}_e\,N}{D^2}, \qquad
#'        \frac{\partial\,\mathrm{ICC}}{\partial S_B}
#'       \;=\; -\,\frac{I_A\,N}{D^2}. }
#'
#' The covariance matrix \eqn{\widehat{\mathrm{Var}}(\hat\theta)} is assembled
#' from the same REML fit:
#' \itemize{
#'   \item the
#'   \eqn{(\sigma_S^2,\sigma_{S\times M,\mathrm{eff}}^2,
#'   \sigma_{S\times T,\mathrm{eff}}^2)} block comes from the empirical
#'   subject-level covariance of the per-subject REML component updates;
#'   \item \eqn{\widehat{\mathrm{Var}}(\hat\sigma_E^2)} is approximated as the
#'   variance of the weighted mean of subject-level residual quadratic forms;
#'   \item \eqn{\widehat{\mathrm{Var}}(S_B)} uses the fixed-effect quadratic-form
#'   delta method already computed in the backend.
#' }
#' Cross-covariances across these blocks are ignored as a large-sample
#' simplification, so
#' \deqn{ \widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}
#'       \;=\; \sqrt{\,\nabla \mathrm{ICC}(\hat\theta)^\top\,
#'                     \widehat{\mathrm{Var}}(\hat\theta)\,
#'                     \nabla \mathrm{ICC}(\hat\theta)\,}. }
#'
#' If `ci_mode = "raw"`, a Wald interval is formed on the ICC scale,
#' \deqn{ \widehat{\mathrm{ICC}} \;\pm\; z_{1-\alpha/2}\,
#'       \widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}, }
#' and truncated to \eqn{[0,1]}. If `ci_mode = "logit"`, the backend applies
#' the same Wald construction after the transform
#' \eqn{\phi = \mathrm{logit}(\mathrm{ICC})}, with
#' \deqn{ \widehat{\mathrm{se}}(\hat\phi)
#'       \;\approx\; \frac{\widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}}
#'       {\widehat{\mathrm{ICC}}\,(1-\widehat{\mathrm{ICC}})}, }
#' and then back-transforms
#' \deqn{ \mathrm{logit}^{-1}\!\Big(
#'       \hat\phi \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\phi)\Big). }
#' If `ci_mode = "auto"`, the backend selects between the raw-scale and
#' logit-scale interval per estimate, typically preferring the logit form near
#' the boundaries.
#'
#' \strong{Choosing \eqn{\rho} for AR(1).}
#' When \code{ar="ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
#' profiling the REML log-likelihood at \eqn{(\hat\beta,\hat G,\hat\sigma_E^2)}.
#' With very few visits per subject, \eqn{\rho} can be weakly identified; consider
#' sensitivity checks over a plausible range.
#'
#' @section Notes on stability and performance:
#' All per-subject solves are \eqn{r\times r} with
#' \eqn{r = 1 + n_m + n_t + q_Z}, so cost scales with the number of subjects and
#' the fixed-effects dimension rather than the total number of observations.
#' Solvers use symmetric positive-definite paths with a small diagonal ridge and
#' pseudo-inverse fallback, which helps for very small or unbalanced subsets and
#' near-boundary estimates. For \code{AR(1)}, observations are ordered by time
#' within subject; \code{NA} time codes break the run, and gaps between factor
#' levels are treated as regular steps.
#'
#' Heteroscedastic slopes across \eqn{Z} columns are supported. Each \eqn{Z}
#' column has its own variance component \eqn{\sigma_{Z,j}^2}, but
#' cross-covariances among \eqn{Z} columns are set to zero.
#'
#' @section Threading and BLAS guards:
#' The C++ backend uses OpenMP loops while also forcing vendor BLAS libraries to
#' run single-threaded so that overall CPU usage stays predictable. This guard
#' is applied to OpenBLAS, Apple's Accelerate, and Intel MKL when their runtime
#' controls are available. You can opt out manually by setting
#' \code{MATRIXCORR_DISABLE_BLAS_GUARD=1} in the environment before loading the
#' package.
#'
#' @param data A data frame.
#' @param response Character. Response variable name.
#' @param subject Character. Subject ID variable name.
#' @param method Character or \code{NULL}. Optional column name of method factor
#'   (added to fixed effects).
#' @param time Character or \code{NULL}. Optional column name of time factor
#'   (added to fixed effects).
#' @param type Character scalar; one of \code{c("consistency","agreement")}.
#'   For \code{"consistency"}, systematic fixed method differences are not
#'   penalized through \eqn{S_B}. For \code{"agreement"}, the fixed-effect
#'   dispersion term \eqn{S_B} is added to the denominator.
#' @param interaction Logical. Include \code{method:time} interaction?
#'   (default \code{FALSE}).
#' @param max_iter Integer. Maximum iterations for variance-component updates
#'   (default \code{100}).
#' @param tol Numeric. Convergence tolerance on parameter change
#'   (default \code{1e-6}).
#'
#' @param Dmat Optional \eqn{n_t \times n_t} numeric matrix to weight/aggregate
#'   time-specific fixed biases in the \eqn{S_B} quadratic form. If supplied, it
#'   is used (after optional mass rescaling; see \code{Dmat_rescale}) whenever at
#'   least two \emph{present} time levels exist; otherwise it is ignored. \strong{If
#'   \code{Dmat} is \code{NULL}}, a canonical kernel \eqn{D_m} is \emph{constructed}
#'   from \code{Dmat_type} and \code{Dmat_weights} (see below). \code{Dmat} should be
#'   symmetric positive semidefinite; small asymmetries are symmetrized internally.
#'
#' @param Dmat_type Character, one of \code{c("time-avg","typical-visit",
#'   "weighted-avg","weighted-sq")}. Only used when \code{Dmat = NULL}.
#'   It selects the aggregation target for time-specific fixed biases in
#'   \eqn{S_B}. Options are:
#'
#'   \itemize{
#'     \item \code{"time-avg"}: square of the time-averaged bias, \eqn{D_m=(1/n_t)\,11^\top}.
#'     \item \code{"typical-visit"}: average of squared per-time biases, \eqn{D_m=I_{n_t}}.
#'     \item \code{"weighted-avg"}: square of a weighted average, \eqn{D_m=n_t\,w\,w^\top} with \eqn{\sum w=1}.
#'     \item \code{"weighted-sq"}: weighted average of squared biases, \eqn{D_m=n_t\,\mathrm{diag}(w)} with \eqn{\sum w=1}.
#'   }
#'   Pick \code{"time-avg"} for ICC targeting the time-averaged measurement; pick
#'   \code{"typical-visit"} for ICC targeting a randomly sampled visit (typical
#'   occasion). Default \code{"time-avg"}.
#'
#' @param Dmat_weights Optional numeric weights \eqn{w} used when
#'   \code{Dmat_type \%in\% c("weighted-avg","weighted-sq")}. Must be nonnegative and
#'   finite. If \code{names(w)} are provided, they should match the \emph{full} time
#'   levels in \code{data}; they are aligned to the \emph{present} time subset per fit.
#'   If unnamed, the length must equal the number of present time levels. In all cases
#'   \eqn{w} is internally normalized to sum to 1.
#'
#' @param Dmat_rescale Logical. When \code{TRUE} (default), the supplied/built
#'   \eqn{D_m} is rescaled to satisfy the simple mass rule
#'   \eqn{1^\top D_m 1 = n_t}. This keeps the \eqn{S_B} denominator invariant and
#'   harmonizes with the time-averaging factors used for the variance terms.
#'
#' @param ci Logical. If \code{TRUE}, return a CI container for the repeated ICC.
#' @param conf_level Numeric in \eqn{(0,1)}. Confidence level when
#'   \code{ci = TRUE} (default \code{0.95}).
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads to use for
#'   computation. Defaults to \code{getOption("matrixCorr.threads", 1L)}.
#' @param ci_mode Character scalar; one of \code{c("auto","raw","logit")}.
#'   Controls how confidence intervals are computed when \code{ci = TRUE}.
#'   If \code{"raw"}, a Wald CI is formed on the ICC scale and truncated to
#'   \code{[0,1]}. If \code{"logit"}, a Wald CI is computed on the
#'   \eqn{\mathrm{logit}(\mathrm{ICC})} scale and back-transformed to the
#'   original scale. If \code{"auto"} (default), the backend chooses per estimate
#'   between the raw-scale and logit-scale interval.
#' @param verbose Logical. If \code{TRUE}, prints a structured summary of the
#'   fitted variance components and \eqn{S_B} for each fit. Default \code{FALSE}.
#' @param digits Integer \eqn{(\ge 0)}. Number of decimal places to use in the
#'   printed summary when \code{verbose = TRUE}. Default \code{4}.
#' @param use_message Logical. When \code{verbose = TRUE}, verbose summaries
#'   are emitted with \pkg{cli} messages.
#'
#' @param ar Character. Residual correlation structure: \code{"none"} (iid) or
#'   \code{"ar1"} for subject-level AR(1) correlation within contiguous time
#'   runs. Default \code{c("none")}.
#' @param ar_rho Numeric in \eqn{(-0.999,\,0.999)} or \code{NA}.
#'   If \code{ar = "ar1"} and \code{ar_rho} is finite, it is treated as fixed.
#'   If \code{ar = "ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
#'   profiling a 1-D objective (REML when available; an approximation otherwise).
#'   Default \code{NA_real_}.
#' @param slope Character. Optional extra random-effect design \eqn{Z}.
#'   With \code{"subject"} a single random slope is added (one column in \eqn{Z});
#'   with \code{"method"} one column per method level is added; with
#'   \code{"custom"} you provide \code{slope_Z} directly. Default
#'   \code{c("none","subject","method","custom")}.
#' @param slope_var For \code{slope \%in\% c("subject","method")}, a character
#'   string giving the name of a column in \code{data} used as the slope regressor
#'   (e.g., centered time). It is looked up inside \code{data}; do not
#'   pass the vector itself. NAs are treated as zeros in \eqn{Z}.
#' @param slope_Z For \code{slope = "custom"}, a numeric matrix with \eqn{n}
#'   rows (same order as \code{data}) providing the full extra random-effect
#'   design \eqn{Z}. \strong{Each column of \code{slope_Z} has its own variance
#'   component} \eqn{\sigma_{Z,j}^2}; columns are treated as \emph{uncorrelated}
#'   (diagonal block in \eqn{G}). Ignored otherwise.
#' @param drop_zero_cols Logical. When \code{slope = "method"}, drop all-zero
#'   columns of \eqn{Z} after subsetting (useful in pairwise fits). Default
#'   \code{TRUE}.
#'
#' @param vc_select Character scalar; one of \code{c("auto","none")}.
#'   Controls how the subject by method \eqn{\sigma^2_{A\times M}} ("subj_method") and
#'   subject by time \eqn{\sigma^2_{A\times T}} ("subj_time") variance components are
#'   included. If \code{"auto"} (default), the function performs boundary-aware
#'   REML likelihood-ratio tests (LRTs; null on the boundary at zero with a
#'   half-\eqn{\chi^2_1} reference) to decide whether to retain each component,
#'   in the order given by \code{vc_test_order}. If \code{"none"}, no testing
#'   is done and inclusion is taken from \code{include_subj_method}/\code{include_subj_time}
#'   (or, if \code{NULL}, from the mere presence of the corresponding factor in
#'   the design). In pairwise fits, the decision is made independently for each
#'   method pair.
#'
#' @param vc_alpha Numeric scalar in \eqn{(0,1)}; default \code{0.05}.
#'   Per-component significance level for the boundary-aware REML LRTs used when
#'   \code{vc_select = "auto"}. The tests are one-sided for variance components
#'   on the boundary and are not multiplicity-adjusted.
#'
#' @param vc_test_order Character vector (length 2) with a permutation of
#'   \code{c("subj_time","subj_method")}; default \code{c("subj_time","subj_method")}. Specifies the order
#'   in which the two variance components are tested when \code{vc_select = "auto"}.
#'   The component tested first may be dropped before testing the second. If a
#'   factor is absent in the design (e.g., no time factor so "subj_time" is undefined),
#'   the corresponding test is skipped.
#'
#' @param include_subj_method,include_subj_time Logical scalars or \code{NULL}.
#'   When \code{vc_select = "none"}, these control whether the
#'   \eqn{\sigma^2_{A\times M}} ("subj_method") and \eqn{\sigma^2_{A\times T}} ("subj_time")
#'   random effects are included (\code{TRUE}) or excluded (\code{FALSE}) in the model.
#'   If \code{NULL} (default), inclusion defaults to the presence of the
#'   corresponding factor in the data (i.e., at least two method/time levels).
#'   When \code{vc_select = "auto"}, these arguments are ignored (automatic
#'   selection is used instead).
#'
#' @param sb_zero_tol Non-negative numeric scalar; default \code{1e-10}.
#'   Numerical threshold for the fixed-effect dispersion term \eqn{S_B}.
#'   After computing \eqn{\widehat{S_B}} and its delta-method variance, if
#'   \eqn{\widehat{S_B} \le} \code{sb_zero_tol} or non-finite, the procedure
#'   treats \eqn{S_B} as fixed at zero in the delta step.
#' @param x For `print()`, an object returned by `icc_rm_reml()` or
#'   `summary.icc_rm_reml()`.
#' @param object For `summary()`, an object returned by `icc_rm_reml()`.
#' @param ci_digits Integer; number of digits for confidence interval bounds in
#'   printed method summaries.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to the underlying display helpers.
#'
#' @return
#' A repeated-measures pairwise ICC object. Without confidence intervals the
#' result is a symmetric matrix of class `c("icc_rm_reml", "icc", "matrix")`.
#' With confidence intervals it is a list with `est`, `lwr.ci`, and `upr.ci`
#' and class `c("icc_rm_reml", "icc_ci", "icc")`. Both carry the fitted
#' variance-component matrices as attributes.
#'
#' @seealso `icc()`, [ccc()], [ccc_rm_reml()], [ba()], [ba_rm()], [rmcorr()]
#'
#' @references
#' Shrout PE, Fleiss JL (1979). Intraclass correlations: uses in assessing
#' rater reliability. Psychological Bulletin, 86(2), 420-428.
#'
#' McGraw KO, Wong SP (1996). Forming inferences about some intraclass
#' correlation coefficients. Psychological Methods, 1(1), 30-46.
#'
#' @examples
#' set.seed(321)
#' n_id <- 20
#' n_time <- 3
#' id <- factor(rep(seq_len(n_id), each = 2 * n_time))
#' method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_id))
#' time <- factor(rep(seq_len(n_time), times = 2 * n_id))
#'
#' subj <- rnorm(n_id, sd = 1)[as.integer(id)]
#' subj_method <- rnorm(n_id * 2, sd = 0.25)
#' sm <- subj_method[(as.integer(id) - 1L) * 2L + as.integer(method)]
#' y <- subj + sm + 0.3 * (method == "B") + rnorm(length(id), sd = 0.35)
#'
#' dat_rm <- data.frame(y = y, id = id, method = method, time = time)
#'
#' fit_icc_rm <- icc_rm_reml(
#'   dat_rm,
#'   response = "y",
#'   subject = "id",
#'   method = "method",
#'   time = "time",
#'   type = "consistency",
#'   ci = TRUE
#' )
#' print(fit_icc_rm)
#' summary(fit_icc_rm)
#' confint(fit_icc_rm)
#' tidy(fit_icc_rm)
#'
#' @export
icc_rm_reml <- function(data, response, subject,
                        method = NULL, time = NULL,
                        type = c("consistency", "agreement"),
                        ci = FALSE, conf_level = 0.95,
                        n_threads = getOption("matrixCorr.threads", 1L),
                        ci_mode = c("auto","raw","logit"),
                        verbose = FALSE, digits = 4, use_message = TRUE,
                        interaction = FALSE,
                        max_iter = 100, tol = 1e-6,
                        Dmat = NULL,
                        Dmat_type = c("time-avg","typical-visit","weighted-avg","weighted-sq"),
                        Dmat_weights = NULL,
                        Dmat_rescale = TRUE,
                        ar = c("none", "ar1"),
                        ar_rho = NA_real_,
                        slope = c("none", "subject", "method", "custom"),
                        slope_var = NULL,
                        slope_Z = NULL,
                        drop_zero_cols = TRUE,
                        vc_select = c("auto","none"),
                        vc_alpha = 0.05,
                        vc_test_order = c("subj_time","subj_method"),
                        include_subj_method = NULL,
                        include_subj_time = NULL,
                        sb_zero_tol = 1e-10) {
  type <- match.arg(type)
  ar <- match.arg(ar)
  slope <- match.arg(slope)
  Dmat_type <- match.arg(Dmat_type)
  vc_select <- match.arg(vc_select)
  vc_test_order <- match.arg(vc_test_order, several.ok = TRUE)
  ci_mode <- match.arg(ci_mode)
  ci_mode_int <- switch(ci_mode, raw = 0L, logit = 1L, auto = 2L)

  check_bool(interaction, arg = "interaction")
  max_iter <- check_scalar_int_pos(max_iter, arg = "max_iter")
  check_scalar_nonneg(tol, arg = "tol", strict = TRUE)
  check_bool(ci, arg = "ci")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")
  check_bool(use_message, arg = "use_message")
  check_bool(Dmat_rescale, arg = "Dmat_rescale")
  check_bool(drop_zero_cols, arg = "drop_zero_cols")
  check_prob_scalar(vc_alpha, arg = "vc_alpha", open_ends = TRUE)

  if (identical(ar, "ar1")) {
    if (length(ar_rho) != 1L) {
      abort_bad_arg("ar_rho",
        message = "must be length 1 (or NA to estimate)."
      )
    }
    if (!is.na(ar_rho)) {
      check_scalar_numeric(ar_rho,
        arg = "ar_rho",
        lower = -0.999,
        upper = 0.999,
        closed_lower = FALSE,
        closed_upper = FALSE
      )
    }
  } else {
    ar_rho <- NA_real_
  }

  df <- as.data.frame(data)
  req_cols <- c(response, subject, method, time, slope_var)
  req_cols <- req_cols[!vapply(req_cols, is.null, logical(1))]
  check_required_cols(df, req_cols, df_arg = "data")

  df[[response]] <- as.numeric(df[[response]])
  df[[subject]] <- factor(df[[subject]])
  if (!is.null(method)) df[[method]] <- factor(df[[method]])
  if (!is.null(time)) df[[time]] <- factor(df[[time]])
  slope_Z <- .validate_rm_wrapper_inputs(
    df = df,
    response = response,
    subject = subject,
    method = method,
    time = time,
    slope = slope,
    slope_var = slope_var,
    slope_Z = slope_Z,
    include_subj_time = include_subj_time,
    vc_select = vc_select
  )
  all_time_lvls <- if (!is.null(time)) levels(df[[time]]) else character(0)

  terms_rhs <- "1"
  if (!is.null(method)) terms_rhs <- c(terms_rhs, method)
  if (!is.null(time)) terms_rhs <- c(terms_rhs, time)
  if (!is.null(method) && !is.null(time) && interaction) {
    terms_rhs <- c(terms_rhs, sprintf("%s:%s", method, time))
  }
  fml <- as.formula(paste("~", paste(terms_rhs, collapse = " + ")))

  extra_label <- switch(slope,
    subject = "random slope (subject)",
    method = "random slope (by method)",
    custom = "custom random effect",
    NULL
  )

  if (is.null(method) || nlevels(df[[method]]) < 2L) {
    cli::cli_abort(
      c(
        "At least two method levels are required to compute pairwise intraclass correlation.",
        "i" = "Supply {.arg method} with >= 2 levels; the overall coefficient is not computed."
      )
    )
  }

  metric_mode <- if (identical(type, "agreement")) 2L else 1L

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  out <- ccc_lmm_reml_pairwise(
    df = df,
    fml = fml,
    response = response,
    rind = subject,
    method = method,
    time = time,
    slope = slope,
    slope_var = slope_var,
    slope_Z = slope_Z,
    drop_zero_cols = drop_zero_cols,
    Dmat = Dmat,
    ar = ar,
    ar_rho = ar_rho,
    max_iter = max_iter,
    tol = tol,
    conf_level = conf_level,
    verbose = verbose,
    digits = digits,
    use_message = use_message,
    extra_label = extra_label,
    ci = ci,
    ci_mode_int = ci_mode_int,
    all_time_lvls = all_time_lvls,
    Dmat_type = Dmat_type,
    Dmat_weights = Dmat_weights,
    Dmat_rescale = Dmat_rescale,
    vc_select = vc_select,
    vc_alpha = vc_alpha,
    vc_test_order = vc_test_order,
    include_subj_method = include_subj_method,
    include_subj_time = include_subj_time,
    sb_zero_tol = sb_zero_tol,
    metric_mode = metric_mode,
    estimate_key = "metric",
    se_key = "se_metric",
    out_class = "icc_rm_reml",
    out_method = paste("Repeated-measures intraclass correlation", sprintf("(%s)", type)),
    out_description = "Pairwise repeated-measures intraclass correlation from random-effects REML",
    summary_title = "Repeated-measures intraclass correlation matrix",
    vc_engine_name = "icc_rm_reml",
    vc_se_label = "SE(ICC)"
  )
  attr(out, "type") <- type
  attr(out, "model") <- "reml_variance_components"
  out
}

#' @rdname icc_rm_reml
#' @method print icc_rm_reml
#' @param x For `print()`, an object returned by `icc_rm_reml()` or
#'   `summary.icc_rm_reml()`.
#' @param ci_digits Integer; number of digits for confidence interval bounds in
#'   printed method summaries.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to the underlying display helpers.
#' @export
print.icc_rm_reml <- function(x,
                              digits = 4,
                              ci_digits = 4,
                              n = NULL,
                              topn = NULL,
                              max_vars = NULL,
                              width = NULL,
                              show_ci = NULL,
                              ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )
  est <- if (is.list(x) && !is.null(x$est)) as.matrix(x$est) else as.matrix(x)
  .mc_print_corr_matrix(
    x,
    header = "Repeated-measures intraclass correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = est,
    ...
  )
  invisible(x)
}

#' @rdname icc_rm_reml
#' @method summary icc_rm_reml
#' @param object For `summary()`, an object returned by `icc_rm_reml()`.
#' @param ci_digits Integer; number of digits for confidence interval bounds in
#'   printed method summaries.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to the underlying display helpers.
#' @export
summary.icc_rm_reml <- function(object,
                                digits = 4,
                                ci_digits = 2,
                                n = NULL,
                                topn = NULL,
                                max_vars = NULL,
                                width = NULL,
                                show_ci = NULL,
                                ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  base_summary <- summary.icc(
    object,
    digits = digits,
    ci_digits = ci_digits,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )

  est_mat <- if (is.list(object) && !is.null(object$est)) as.matrix(object$est) else as.matrix(object)
  n_pairs <- nrow(base_summary)

  extract_pairs_num <- function(val) {
    out <- numeric(n_pairs)
    k <- 0L
    for (i in seq_len(nrow(est_mat) - 1L)) {
      for (j in (i + 1L):ncol(est_mat)) {
        k <- k + 1L
        out[k] <- if (is.null(val)) {
          NA_real_
        } else if (is.matrix(val)) {
          suppressWarnings(as.numeric(val[i, j]))
        } else {
          vv <- suppressWarnings(as.numeric(val))
          if (length(vv) == 1L) vv else NA_real_
        }
      }
    }
    out
  }

  extract_pairs_extra_list <- function(val) {
    if (is.matrix(val) && typeof(val) == "list") {
      out <- vector("list", n_pairs)
      k <- 0L
      for (i in seq_len(nrow(est_mat) - 1L)) {
        for (j in (i + 1L):ncol(est_mat)) {
          k <- k + 1L
          out[[k]] <- val[[i, j]]
        }
      }
      return(out)
    }
    if (is.null(val)) return(vector("list", n_pairs))
    vv <- suppressWarnings(as.numeric(val))
    replicate(n_pairs, vv, simplify = FALSE)
  }

  out <- base_summary
  out$n_subjects <- as.integer(extract_pairs_num(attr(object, "n_subjects")))
  out$n_obs <- as.integer(extract_pairs_num(attr(object, "n_obs")))
  out$sigma2_subject <- round(extract_pairs_num(attr(object, "sigma2_subject")), digits)
  out$sigma2_subject_method <- round(extract_pairs_num(attr(object, "sigma2_subject_method")), digits)
  out$sigma2_subject_time <- round(extract_pairs_num(attr(object, "sigma2_subject_time")), digits)
  out$sigma2_error <- round(extract_pairs_num(attr(object, "sigma2_error")), digits)
  out$SB <- round(extract_pairs_num(attr(object, "SB")), digits)
  out$se_icc <- round(extract_pairs_num(attr(object, "se_metric")), digits)
  out$residual_model <- rep(attr(object, "residual_model", exact = TRUE) %||% "iid", n_pairs)

  extra_list <- extract_pairs_extra_list(attr(object, "sigma2_extra"))
  max_k <- 0L
  for (v in extra_list) if (!is.null(v)) max_k <- max(max_k, length(v))
  if (max_k > 0L) {
    for (kk in seq_len(max_k)) {
      out[[paste0("sigma2_extra", kk)]] <- vapply(extra_list, function(v) {
        if (is.null(v) || length(v) < kk) return(NA_real_)
        round(as.numeric(v[[kk]]), digits)
      }, numeric(1))
    }
  }

  extract_pairs_logi <- function(val) {
    out <- logical(n_pairs)
    k <- 0L
    for (i in seq_len(nrow(est_mat) - 1L)) {
      for (j in (i + 1L):ncol(est_mat)) {
        k <- k + 1L
        out[k] <- if (is.null(val)) FALSE else isTRUE(if (is.matrix(val)) val[i, j] else val)
      }
    }
    out
  }

  if (!is.null(attr(object, "ar_rho"))) {
    out$ar1_rho <- round(extract_pairs_num(attr(object, "ar_rho")), digits)
  }
  if (!is.null(attr(object, "ar1_rho_lag1"))) {
    vals <- round(extract_pairs_num(attr(object, "ar1_rho_lag1")), digits)
    out$ar1_rho_lag1 <- vals
    out$ar1_rho_mom <- vals
  }
  if (!is.null(attr(object, "ar1_pairs"))) {
    out$ar1_pairs <- as.integer(extract_pairs_num(attr(object, "ar1_pairs")))
  }
  if (!is.null(attr(object, "ar1_pval"))) {
    out$ar1_pval <- round(extract_pairs_num(attr(object, "ar1_pval")), digits)
  }
  if (!is.null(attr(object, "use_ar1"))) {
    out$use_ar1 <- extract_pairs_logi(attr(object, "use_ar1"))
    out$ar1_recommend <- out$use_ar1
  }

  attr(out, "conf.level") <- attr(base_summary, "conf.level")
  attr(out, "has_ci") <- attr(base_summary, "has_ci")
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  .mc_finalize_summary_df(
    out,
    class_name = "summary.icc_rm_reml",
    repeated = TRUE
  )
}

#' @rdname icc_rm_reml
#' @method print summary.icc_rm_reml
#' @param x For `print()`, an object returned by `icc_rm_reml()` or
#'   `summary.icc_rm_reml()`.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to the underlying display helpers.
#' @export
print.summary.icc_rm_reml <- function(x, digits = NULL, n = NULL,
                                      topn = NULL, max_vars = NULL,
                                      width = NULL, show_ci = NULL, ...) {
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  has_ci <- isTRUE(attr(x, "has_ci")) || all(c("lwr", "upr") %in% names(x))
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  if (!is.finite(cl)) cl <- NA_real_

  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "ICC estimates",
        cols = c("item1", "item2", "estimate", "lwr", "upr", "n_subjects", "n_obs", "se_icc", "residual_model")
      ),
      list(
        title = "Variance components",
        cols = c("sigma2_subject", "sigma2_subject_method", "sigma2_subject_time",
                 "sigma2_error", "SB", grep("^sigma2_extra", names(x), value = TRUE))
      ),
      list(
        title = "AR(1) diagnostics",
        cols = c("ar1_rho", "ar1_rho_lag1", "ar1_rho_mom", "ar1_pairs",
                 "ar1_pval", "use_ar1", "ar1_recommend")
      )
    ),
    header = .mc_header_with_ci("Repeated-measures intraclass correlation (REML)", cl, if (has_ci) show_ci else "no"),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

.test_icc_reference <- function(X) {
  X <- as.matrix(X)
  n <- nrow(X)
  k <- ncol(X)
  stopifnot(k == 2L, n >= 2L)

  row_means <- rowMeans(X)
  grand_mean <- mean(X)
  col_means <- colMeans(X)

  ss_subject <- k * sum((row_means - grand_mean)^2)
  ss_rater <- n * sum((col_means - grand_mean)^2)
  ss_total <- sum((X - grand_mean)^2)
  ss_error <- ss_total - ss_subject - ss_rater
  ss_within <- ss_total - ss_subject

  ms_subject <- ss_subject / (n - 1)
  ms_rater <- ss_rater / (k - 1)
  ms_error <- ss_error / ((n - 1) * (k - 1))
  ms_within <- ss_within / (n * (k - 1))

  icc1 <- (ms_subject - ms_within) / (ms_subject + (k - 1) * ms_within)
  icc1k <- (ms_subject - ms_within) / ms_subject
  icc2 <- (ms_subject - ms_error) /
    (ms_subject + (k - 1) * ms_error + k * (ms_rater - ms_error) / n)
  icc2k <- (ms_subject - ms_error) / (ms_subject + (ms_rater - ms_error) / n)
  icc3 <- (ms_subject - ms_error) / (ms_subject + (k - 1) * ms_error)
  icc3k <- (ms_subject - ms_error) / ms_subject

  f11 <- ms_subject / ms_within
  df11n <- n - 1
  df11d <- n * (k - 1)
  f1l <- f11 / qf(0.975, df11n, df11d)
  f1u <- f11 * qf(0.975, df11d, df11n)
  l1 <- (f1l - 1) / (f1l + (k - 1))
  u1 <- (f1u - 1) / (f1u + (k - 1))

  f21 <- ms_subject / ms_error
  df21n <- n - 1
  df21d <- (n - 1) * (k - 1)
  f3l <- f21 / qf(0.975, df21n, df21d)
  f3u <- f21 * qf(0.975, df21d, df21n)
  l3_single <- (f3l - 1) / (f3l + (k - 1))
  u3_single <- (f3u - 1) / (f3u + (k - 1))

  fj <- ms_rater / ms_error
  vn <- (k - 1) * (n - 1) * ((k * icc2 * fj + n * (1 + (k - 1) * icc2) - k * icc2)^2)
  vd <- (n - 1) * k^2 * icc2^2 * fj^2 + (n * (1 + (k - 1) * icc2) - k * icc2)^2
  v <- vn / vd
  f2u <- qf(0.975, n - 1, v)
  f2l <- qf(0.975, v, n - 1)
  l2_single <- n * (ms_subject - f2u * ms_error) /
    (f2u * (k * ms_rater + (k * n - k - n) * ms_error) + n * ms_subject)
  u2_single <- n * (f2l * ms_subject - ms_error) /
    (k * ms_rater + (k * n - k - n) * ms_error + n * f2l * ms_subject)
  l2_avg <- l2_single * k / (1 + l2_single * (k - 1))
  u2_avg <- u2_single * k / (1 + u2_single * (k - 1))

  list(
    ICC1 = icc1,
    ICC1k = icc1k,
    ICC2 = icc2,
    ICC2k = icc2k,
    ICC3 = icc3,
    ICC3k = icc3k,
    ICC1_lwr = l1,
    ICC1_upr = u1,
    ICC2_lwr = l2_single,
    ICC2_upr = u2_single,
    ICC2k_lwr = l2_avg,
    ICC2k_upr = u2_avg,
    ICC3_lwr = l3_single,
    ICC3_upr = u3_single
  )
}

.test_icc_overall_reference <- function(X, conf_level = 0.95) {
  X <- as.matrix(X)
  n <- nrow(X)
  k <- ncol(X)
  stopifnot(n >= 2L, k >= 2L)

  row_means <- rowMeans(X)
  grand_mean <- mean(X)
  col_means <- colMeans(X)

  ss_subject <- k * sum((row_means - grand_mean)^2)
  ss_rater <- n * sum((col_means - grand_mean)^2)
  ss_total <- sum((X - grand_mean)^2)
  ss_error <- ss_total - ss_subject - ss_rater
  ss_within <- ss_total - ss_subject

  ms_subject <- ss_subject / (n - 1)
  ms_rater <- ss_rater / (k - 1)
  ms_error <- ss_error / ((n - 1) * (k - 1))
  ms_within <- ss_within / (n * (k - 1))

  icc1 <- (ms_subject - ms_within) / (ms_subject + (k - 1) * ms_within)
  icc2 <- (ms_subject - ms_error) /
    (ms_subject + (k - 1) * ms_error + k * (ms_rater - ms_error) / n)
  icc3 <- (ms_subject - ms_error) / (ms_subject + (k - 1) * ms_error)
  icc1k <- (ms_subject - ms_within) / ms_subject
  icc2k <- (ms_subject - ms_error) / (ms_subject + (ms_rater - ms_error) / n)
  icc3k <- (ms_subject - ms_error) / ms_subject

  f11 <- ms_subject / ms_within
  df11n <- n - 1
  df11d <- n * (k - 1)
  p11 <- pf(f11, df11n, df11d, lower.tail = FALSE)

  f21 <- ms_subject / ms_error
  df21n <- n - 1
  df21d <- (n - 1) * (k - 1)
  p21 <- pf(f21, df21n, df21d, lower.tail = FALSE)

  alpha <- 1 - conf_level
  f1l <- f11 / qf(1 - alpha / 2, df11n, df11d)
  f1u <- f11 * qf(1 - alpha / 2, df11d, df11n)
  l1 <- (f1l - 1) / (f1l + (k - 1))
  u1 <- (f1u - 1) / (f1u + (k - 1))

  f3l <- f21 / qf(1 - alpha / 2, df21n, df21d)
  f3u <- f21 * qf(1 - alpha / 2, df21d, df21n)
  l3 <- (f3l - 1) / (f3l + (k - 1))
  u3 <- (f3u - 1) / (f3u + (k - 1))

  fj <- ms_rater / ms_error
  vn <- (k - 1) * (n - 1) * ((k * icc2 * fj + n * (1 + (k - 1) * icc2) - k * icc2)^2)
  vd <- (n - 1) * k^2 * icc2^2 * fj^2 + (n * (1 + (k - 1) * icc2) - k * icc2)^2
  v <- vn / vd
  f2u <- qf(1 - alpha / 2, n - 1, v)
  f2l <- qf(1 - alpha / 2, v, n - 1)
  l2 <- n * (ms_subject - f2u * ms_error) /
    (f2u * (k * ms_rater + (k * n - k - n) * ms_error) + n * ms_subject)
  u2 <- n * (f2l * ms_subject - ms_error) /
    (k * ms_rater + (k * n - k - n) * ms_error + n * f2l * ms_subject)

  data.frame(
    coefficient = c("ICC1", "ICC2", "ICC3", "ICC1k", "ICC2k", "ICC3k"),
    estimate = c(icc1, icc2, icc3, icc1k, icc2k, icc3k),
    statistic = c(f11, f21, f21, f11, f21, f21),
    df1 = c(df11n, df21n, df21n, df11n, df21n, df21n),
    df2 = c(df11d, df21d, df21d, df11d, df21d, df21d),
    p_value = c(p11, p21, p21, p11, p21, p21),
    lwr = c(l1, l2, l3, 1 - 1 / f1l, l2 * k / (1 + l2 * (k - 1)), 1 - 1 / f3l),
    upr = c(u1, u2, u3, 1 - 1 / f1u, u2 * k / (1 + u2 * (k - 1)), 1 - 1 / f3u),
    stringsAsFactors = FALSE
  )
}

test_that("classical pairwise ICC matches the standard two-rater ANOVA formulas", {

  set.seed(101)
  n <- 80L
  subj <- rnorm(n, sd = 1.1)
  A <- subj + rnorm(n, sd = 0.3)
  B <- 0.4 + subj + rnorm(n, sd = 0.5)
  C <- -0.2 + 0.9 * subj + rnorm(n, sd = 0.4)
  X <- cbind(A = A, B = B, C = C)

  pairs <- list(
    c("A", "B"),
    c("A", "C"),
    c("B", "C")
  )

  forms <- list(
    list(model = "oneway", type = "consistency", unit = "single", ref = "ICC1"),
    list(model = "oneway", type = "consistency", unit = "average", ref = "ICC1k"),
    list(model = "twoway_random", type = "agreement", unit = "single", ref = "ICC2"),
    list(model = "twoway_random", type = "agreement", unit = "average", ref = "ICC2k"),
    list(model = "twoway_random", type = "consistency", unit = "single", ref = "ICC3"),
    list(model = "twoway_random", type = "consistency", unit = "average", ref = "ICC3k"),
    list(model = "twoway_mixed", type = "agreement", unit = "single", ref = "ICC2"),
    list(model = "twoway_mixed", type = "consistency", unit = "single", ref = "ICC3")
  )

  for (spec in forms) {
    fit <- icc(X,
      model = spec$model,
      type = spec$type,
      unit = spec$unit
    )
    for (pair in pairs) {
      ref <- .test_icc_reference(X[, pair, drop = FALSE])
      expect_equal(
        unname(fit[pair[1], pair[2]]),
        unname(ref[[spec$ref]]),
        tolerance = 1e-8
      )
    }
  }
})

test_that("classical ICC confidence intervals align with the standard reference output", {
  set.seed(202)
  n <- 60L
  subj <- rnorm(n, sd = 1)
  X <- data.frame(
    A = subj + rnorm(n, sd = 0.4),
    B = 0.3 + subj + rnorm(n, sd = 0.6)
  )

  fit <- icc(X,
    model = "twoway_random",
    type = "agreement",
    unit = "single",
    ci = TRUE
  )
  ref <- .test_icc_reference(X)

  expect_s3_class(fit, "icc_ci")
  expect_equal(fit$est["A", "B"], ref$ICC2, tolerance = 1e-8)
  expect_equal(fit$lwr.ci["A", "B"], ref$ICC2_lwr, tolerance = 1e-6)
  expect_equal(fit$upr.ci["A", "B"], ref$ICC2_upr, tolerance = 1e-6)
})

test_that("pairwise ICC stores CI payload only when requested", {
  set.seed(205)
  X <- data.frame(
    A = rnorm(30),
    B = rnorm(30),
    C = rnorm(30)
  )

  fit <- icc(X,
    model = "twoway_random",
    type = "agreement",
    unit = "single",
    ci = FALSE
  )
  fit_ci <- icc(X,
    model = "twoway_random",
    type = "agreement",
    unit = "single",
    ci = TRUE
  )

  expect_false(is.list(fit))
  expect_s3_class(fit_ci, "icc_ci")
  expect_identical(names(fit_ci), c("est", "lwr.ci", "upr.ci"))
  expect_false("n_complete" %in% names(fit_ci))
  expect_true(is.matrix(attr(fit, "diagnostics")$n_complete))
  expect_true(is.matrix(attr(fit_ci, "diagnostics")$n_complete))
})

test_that("classical ICC argument combinations and matrix behaviour are consistent", {
  set.seed(303)
  n <- 40L
  subj <- rnorm(n, sd = 1.2)
  X <- data.frame(
    A = subj + rnorm(n, sd = 0.2),
    B = 0.8 + subj + rnorm(n, sd = 0.2),
    C = rep(4, n)
  )

  expect_error(
    icc(X[, 1:2], model = "oneway", type = "agreement"),
    "cannot be.*agreement",
    ignore.case = TRUE
  )

  cons_single <- icc(X, model = "twoway_random", type = "consistency", unit = "single")
  agree_single <- icc(X[, 1:2], model = "twoway_random", type = "agreement", unit = "single")
  cons_average <- icc(X[, 1:2], model = "twoway_random", type = "consistency", unit = "average")

  expect_gt(cons_single["A", "B"], agree_single["A", "B"])
  expect_gte(cons_average["A", "B"], cons_single["A", "B"])
  expect_true(is.na(cons_single["A", "C"]))
  expect_equal(as.numeric(diag(cons_single)), c(1, 1, 1))
  expect_true(isSymmetric(unclass(cons_single)))

  fit_ci <- icc(X[, 1:2], model = "twoway_random", type = "agreement", unit = "single", ci = TRUE)
  sm <- summary(fit_ci)
  expect_s3_class(sm, "summary.icc")
  expect_true(all(c("item1", "item2", "estimate", "lwr", "upr") %in% names(sm)))

  txt_fit <- capture.output(print(fit_ci))
  txt_sm <- capture.output(print(sm))
  expect_true(any(grepl("^Intraclass correlation matrix$", txt_fit)))
  expect_true(any(grepl("^Intraclass correlation summary$", txt_sm)))
})

test_that("classical ICC respects pairwise-complete handling", {
  X <- data.frame(
    A = c(1, 2, 3, 4, 5, NA),
    B = c(1.2, 2.2, 3.1, 4.3, NA, 6.0),
    C = c(0.9, 2.0, 2.8, 4.1, 5.2, 6.1)
  )

  expect_error(icc(X, na_method = "error"), "Missing")

  fit <- icc(X, na_method = "pairwise")
  diag_n <- attr(fit, "diagnostics")$n_complete
  expect_equal(diag_n["A", "B"], 4L)
  expect_true(is.finite(fit["A", "B"]))
  expect_equal(as.numeric(diag(fit)), c(1, 1, 1))
})

test_that("overall ICC reproduces the standard all-rater ANOVA table", {
  sf <- matrix(c(
    9, 2, 5, 8,
    6, 1, 3, 2,
    8, 4, 6, 8,
    7, 1, 2, 6,
    10, 5, 6, 9,
    6, 2, 4, 7
  ), ncol = 4, byrow = TRUE)
  colnames(sf) <- paste0("J", 1:4)
  rownames(sf) <- paste0("S", 1:6)

  fit <- icc(
    sf,
    model = "twoway_random",
    type = "agreement",
    unit = "single",
    scope = "overall",
    ci = TRUE
  )
  ref <- .test_icc_overall_reference(sf)

  expect_s3_class(fit, "icc_overall")
  expect_true(all(c("coefficients", "anova", "mean_squares") %in% names(fit)))
  expect_identical(attr(fit, "selected_coefficient"), "ICC2")

  got <- fit$coefficients
  expect_equal(got$estimate, ref$estimate, tolerance = 1e-8)
  expect_equal(got$statistic, ref$statistic, tolerance = 1e-8)
  expect_equal(got$df1, ref$df1, tolerance = 1e-8)
  expect_equal(got$df2, ref$df2, tolerance = 1e-8)
  expect_equal(got$p_value, ref$p_value, tolerance = 1e-8)
  expect_equal(got$lwr, ref$lwr, tolerance = 1e-8)
  expect_equal(got$upr, ref$upr, tolerance = 1e-8)
  expect_true(got$selected[got$coefficient == "ICC2"])
})

test_that("overall ICC summary and display follow the native object layout", {
  set.seed(404)
  n <- 24
  subj <- rnorm(n, sd = 1)
  X <- data.frame(
    A = subj + rnorm(n, sd = 0.2),
    B = 0.2 + subj + rnorm(n, sd = 0.4),
    C = -0.1 + subj + rnorm(n, sd = 0.5)
  )

  fit <- icc(
    X,
    model = "twoway_mixed",
    type = "consistency",
    unit = "average",
    scope = "overall",
    ci = TRUE
  )
  sm <- summary(fit)

  expect_s3_class(sm, "summary.icc_overall")
  expect_true(all(c(
    "coefficient", "label", "estimate", "statistic",
    "df1", "df2", "p_value", "lwr", "upr", "selected"
  ) %in% names(sm)))
  expect_identical(attr(sm, "selected_coefficient"), "ICC3k")
  expect_true(is.data.frame(attr(sm, "anova")))

  txt_fit <- capture.output(print(fit))
  txt_sm <- capture.output(print(sm))
  expect_true(any(grepl("^Overall intraclass correlation$", txt_fit)))
  expect_true(any(grepl("^Overall intraclass correlation summary$", txt_sm)))
  expect_true(any(grepl("^ICC coefficients$", txt_sm)))
  expect_true(any(grepl("^ANOVA decomposition$", txt_sm)))
})

test_that("overall ICC drops incomplete rows when requested", {
  X <- data.frame(
    A = c(1, 2, 3, 4, 5, NA),
    B = c(1.2, 2.2, 3.1, 4.3, NA, 6.0),
    C = c(0.9, 2.0, 2.8, 4.1, 5.2, 6.1)
  )

  expect_error(
    icc(X, scope = "overall", na_method = "error"),
    "Missing"
  )

  fit <- icc(X, scope = "overall", na_method = "pairwise")
  expect_identical(attr(fit, "diagnostics")$n_complete, 4L)
  expect_identical(attr(fit, "diagnostics")$dropped_rows, 2L)
})

sim_icc_rm_long <- function(seed, sig_subject = 1.2, sig_method = 0.2,
                            sig_error = 0.4, bias_b = 0.0,
                            n_subj = 120L, n_time = 4L) {
  set.seed(seed)
  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  subj <- rnorm(n_subj, 0, sqrt(sig_subject))
  subj_method <- matrix(rnorm(n_subj * 2L, 0, sqrt(sig_method)), n_subj, 2L)
  y <- numeric(length(id))

  for (s in seq_len(n_subj)) {
    for (m in 1:2) {
      idx <- which(as.integer(id) == s & as.integer(method) == m)
      y[idx] <- subj[s] + subj_method[s, m] + (m == 2L) * bias_b + rnorm(n_time, 0, sqrt(sig_error))
    }
  }

  data.frame(y = y, id = id, method = method, time = time)
}

test_that("repeated-measures ICC behaves sensibly as variance components change", {
  dat_high <- sim_icc_rm_long(seed = 1, sig_subject = 1.4, sig_method = 0.1, sig_error = 0.15)
  dat_low <- sim_icc_rm_long(seed = 2, sig_subject = 0.6, sig_method = 0.3, sig_error = 1.2)

  fit_high <- icc_rm_reml(
    dat_high, "y", "id",
    method = "method", time = "time",
    ci = TRUE, ci_mode = "raw"
  )
  fit_low <- icc_rm_reml(
    dat_low, "y", "id",
    method = "method", time = "time",
    ci = TRUE, ci_mode = "raw"
  )

  expect_s3_class(fit_high, "icc_rm_reml")
  expect_gt(fit_high$est["A", "B"], 0.7)
  expect_lt(fit_low$est["A", "B"], fit_high$est["A", "B"])
  expect_true(fit_high$lwr.ci["A", "B"] <= fit_high$est["A", "B"])
  expect_true(fit_high$upr.ci["A", "B"] >= fit_high$est["A", "B"])
})

test_that("repeated-measures ICC uses the subject argument name", {
  dat <- sim_icc_rm_long(seed = 11, sig_subject = 1.0, sig_method = 0.2, sig_error = 0.3)

  fit_named <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = FALSE
  )
  fit_positional <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = "time",
    ci = FALSE
  )

  expect_equal(unname(fit_named["A", "B"]), unname(fit_positional["A", "B"]))
})

test_that("repeated-measures ICC honors n_threads without changing estimates", {
  dat <- sim_icc_rm_long(seed = 12, sig_subject = 1.0, sig_method = 0.2, sig_error = 0.3)

  fit1 <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    n_threads = 1L,
    ci = FALSE
  )
  fit2 <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    n_threads = 2L,
    ci = FALSE
  )

  expect_equal(unname(fit1["A", "B"]), unname(fit2["A", "B"]), tolerance = 1e-12)
})

test_that("repeated-measures ICC matches the fitted variance-component denominator", {
  dat <- sim_icc_rm_long(seed = 3, sig_subject = 1.0, sig_method = 0.35, sig_error = 0.4, n_time = 1L)

  fit_cons <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = NULL,
    type = "consistency",
    vc_select = "none",
    include_subj_method = TRUE,
    ci = FALSE
  )
  fit_cons_nosm <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = NULL,
    type = "consistency",
    vc_select = "none",
    include_subj_method = FALSE,
    ci = FALSE
  )
  fit_agree <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = NULL,
    type = "agreement",
    vc_select = "none",
    include_subj_method = TRUE,
    ci = FALSE
  )

  sa <- attr(fit_cons, "sigma2_subject")["A", "B"]
  sab <- attr(fit_cons, "sigma2_subject_method")["A", "B"]
  se <- attr(fit_cons, "sigma2_error")["A", "B"]
  sa_nosm <- attr(fit_cons_nosm, "sigma2_subject")["A", "B"]
  se_nosm <- attr(fit_cons_nosm, "sigma2_error")["A", "B"]
  sb <- attr(fit_agree, "SB")["A", "B"]

  expect_equal(fit_cons["A", "B"], sa / (sa + sab + se), tolerance = 1e-6)
  expect_equal(fit_cons_nosm["A", "B"], sa_nosm / (sa_nosm + se_nosm), tolerance = 1e-6)
  expect_equal(fit_agree["A", "B"], sa / (sa + sab + se + sb), tolerance = 1e-6)
  expect_gte(fit_cons_nosm["A", "B"], fit_cons["A", "B"])
  expect_gte(fit_cons["A", "B"], fit_agree["A", "B"])
})

test_that("repeated-measures ICC summaries expose the native REML payload", {
  dat <- sim_icc_rm_long(seed = 4, sig_subject = 0.9, sig_method = 0.2, sig_error = 0.3, bias_b = 0.4)

  fit <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = "time",
    type = "agreement",
    ci = TRUE,
    ar = "none"
  )
  sm <- summary(fit)

  expect_s3_class(sm, "summary.icc_rm_reml")
  expect_true(all(c(
    "item1", "item2", "estimate", "lwr", "upr", "n_subjects", "n_obs", "se_icc",
    "sigma2_subject", "sigma2_subject_method", "sigma2_subject_time",
    "sigma2_error", "SB", "residual_model"
  ) %in% names(sm)))
  expect_identical(sm$residual_model[1], "iid")

  txt_fit <- capture.output(print(fit))
  txt_sm <- capture.output(print(sm))
  expect_true(any(grepl("^Repeated-measures intraclass correlation matrix$", txt_fit)))
  expect_true(any(grepl("^ICC estimates$", txt_sm)))
  expect_true(any(grepl("^Variance components$", txt_sm)))
})

test_that("icc_rm_reml consistency covers vignette and vc-select branches", {
  set.seed(50)
  n_id <- 14L
  n_time <- 4L

  dat <- expand.grid(
    id = factor(seq_len(n_id)),
    time = factor(seq_len(n_time)),
    method = factor(c("A", "B"))
  )
  dat$time_index <- as.integer(dat$time)

  subj <- rnorm(n_id, sd = 1.0)[dat$id]
  subject_method <- rnorm(n_id * 2L, sd = 0.25)
  sm <- subject_method[(as.integer(dat$id) - 1L) * 2L + as.integer(dat$method)]
  subject_time <- rnorm(n_id * n_time, sd = 0.75)
  st <- subject_time[(as.integer(dat$id) - 1L) * n_time + as.integer(dat$time)]
  dat$y <- subj + sm + st + 0.35 * (dat$method == "B") + rnorm(nrow(dat), sd = 0.35)

  fit_cons <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time_index",
    type = "consistency",
    ci = TRUE,
    vc_select = "auto"
  )
  fit_cons_no_st <- icc_rm_reml(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time_index",
    type = "consistency",
    ci = FALSE,
    vc_select = "none",
    include_subj_time = FALSE
  )

  expect_s3_class(fit_cons, "icc_rm_reml")
  expect_true(is.finite(unname(fit_cons$est["A", "B"])))
  expect_true(is.finite(unname(fit_cons_no_st["A", "B"])))
})

test_that("icc_rm_reml rejects invalid no-time subject-time requests and handles single-level time", {
  set.seed(61)
  dat_notime <- data.frame(
    y = rnorm(16),
    id = factor(rep(seq_len(8L), each = 2L)),
    method = factor(rep(c("A", "B"), times = 8L))
  )
  expect_error(
    icc_rm_reml(
      dat_notime,
      "y",
      "id",
      method = "method",
      include_subj_time = TRUE,
      vc_select = "none"
    ),
    "cannot be TRUE when"
  )

  dat_onetime <- data.frame(
    y = rnorm(20),
    id = factor(rep(seq_len(10L), each = 2L)),
    method = factor(rep(c("A", "B"), times = 10L)),
    time = factor(rep("visit1", 20L))
  )
  fit_onetime <- icc_rm_reml(
    dat_onetime,
    "y",
    "id",
    method = "method",
    time = "time",
    type = "consistency",
    vc_select = "auto",
    ci = FALSE
  )
  expect_true(is.finite(unname(fit_onetime["A", "B"])))
})

test_that("repeated-measures ICC carries AR(1) diagnostics consistently", {
  dat <- sim_icc_rm_long(seed = 5, sig_subject = 1.1, sig_method = 0.1, sig_error = 0.35, n_time = 5L)
  fit <- icc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = "time",
    ar = "ar1",
    ar_rho = 0.4,
    ci = FALSE
  )

  expect_true(is.matrix(attr(fit, "ar_rho")))
  expect_equal(attr(fit, "ar_rho")["A", "B"], 0.4, tolerance = 1e-12)
  sm <- summary(fit)
  expect_true("ar1_rho" %in% names(sm))
})

# Internal helpers for the Poisson GLMM implementation.
.mc_ccc_glmm_ghq_rule <- function(n = 40L) {
  n <- as.integer(n)
  i <- seq_len(n - 1L)
  J <- matrix(0, n, n)
  off <- sqrt(i / 2)
  J[cbind(i, i + 1L)] <- off
  J[cbind(i + 1L, i)] <- off
  eig <- eigen(J, symmetric = TRUE)
  ord <- order(eig$values)
  nodes <- eig$values[ord]
  weights <- sqrt(pi) * eig$vectors[1L, ord]^2
  list(nodes = nodes, weights = weights)
}

.mc_ccc_glmm_replicates_per_cell <- function(subject, method, replicate) {
  if (!length(replicate)) return(1L)
  tab <- table(subject, method)
  cells <- as.numeric(tab[tab > 0])
  if (!length(cells)) return(1L)
  max(1L, as.integer(round(mean(cells))))
}

.mc_ccc_glmm_vcov_from_hessian <- function(hessian) {
  if (is.null(hessian)) return(NULL)
  hessian <- 0.5 * (hessian + t(hessian))
  if (any(!is.finite(hessian))) return(NULL)

  eig <- tryCatch(
    eigen(hessian, symmetric = TRUE, only.values = TRUE)$values,
    error = function(e) NULL
  )
  if (is.null(eig) ||
      any(!is.finite(eig)) ||
      min(eig) <= sqrt(.Machine$double.eps)) {
    return(NULL)
  }

  tryCatch(
    chol2inv(chol(hessian)),
    error = function(e) NULL
  )
}

.mc_num_grad <- function(f, x, rel_step = 1e-5) {
  g <- numeric(length(x))
  for (i in seq_along(x)) {
    h <- rel_step * max(abs(x[[i]]), 1)
    xp <- x
    xm <- x
    xp[[i]] <- xp[[i]] + h
    xm[[i]] <- xm[[i]] - h
    fp <- f(xp)
    fm <- f(xm)
    if (!is.finite(fp) || !is.finite(fm)) {
      g[[i]] <- NA_real_
    } else {
      g[[i]] <- (fp - fm) / (2 * h)
    }
  }
  g
}

.mc_delta_ci <- function(estimate,
                         f,
                         theta,
                         vcov_theta,
                         conf_level = 0.95,
                         transform = c("fisher", "wald")) {
  transform <- match.arg(transform)
  out <- list(
    estimate = estimate,
    se = NA_real_,
    lwr.ci = NA_real_,
    upr.ci = NA_real_,
    conf_level = conf_level,
    ci_method = paste0("delta_", transform)
  )

  if (is.null(vcov_theta) ||
      any(!is.finite(vcov_theta)) ||
      !is.finite(estimate)) {
    return(out)
  }

  grad <- .mc_num_grad(f, theta)
  if (any(!is.finite(grad))) return(out)

  var_est <- as.numeric(t(grad) %*% vcov_theta %*% grad)
  if (!is.finite(var_est) || var_est < 0) return(out)

  se <- sqrt(var_est)
  alpha <- 1 - conf_level
  zcrit <- stats::qnorm(1 - alpha / 2)

  if (identical(transform, "wald")) {
    lwr <- estimate - zcrit * se
    upr <- estimate + zcrit * se
  } else {
    estimate_safe <- max(min(estimate, 1 - 1e-12), -1 + 1e-12)
    z_est <- atanh(estimate_safe)
    se_z <- se / (1 - estimate_safe^2)
    lwr <- tanh(z_est - zcrit * se_z)
    upr <- tanh(z_est + zcrit * se_z)
  }

  out$se <- se
  out$lwr.ci <- lwr
  out$upr.ci <- upr
  out
}

.mc_num_grad_vec <- function(f, x, rel_step = 1e-5) {
  f0 <- f(x)
  g <- matrix(NA_real_, nrow = length(f0), ncol = length(x))
  rownames(g) <- names(f0)
  for (i in seq_along(x)) {
    h <- rel_step * max(abs(x[[i]]), 1)
    xp <- x
    xm <- x
    xp[[i]] <- xp[[i]] + h
    xm[[i]] <- xm[[i]] - h
    fp <- f(xp)
    fm <- f(xm)
    ok <- is.finite(fp) & is.finite(fm)
    g[ok, i] <- (fp[ok] - fm[ok]) / (2 * h)
  }
  g
}

.mc_delta_ci_vec <- function(estimates,
                             f,
                             theta,
                             vcov_theta,
                             conf_level = 0.95,
                             transform = c("fisher", "wald")) {
  transform <- match.arg(transform)
  out <- data.frame(
    estimate = as.numeric(estimates),
    se = NA_real_,
    lwr.ci = NA_real_,
    upr.ci = NA_real_,
    conf_level = conf_level,
    ci_method = paste0("delta_", transform),
    row.names = names(estimates)
  )

  if (is.null(vcov_theta) || any(!is.finite(vcov_theta))) return(out)

  grad <- .mc_num_grad_vec(f, theta)
  alpha <- 1 - conf_level
  zcrit <- stats::qnorm(1 - alpha / 2)

  for (i in seq_along(estimates)) {
    estimate <- estimates[[i]]
    if (!is.finite(estimate) || any(!is.finite(grad[i, ]))) next

    var_est <- as.numeric(t(grad[i, ]) %*% vcov_theta %*% grad[i, ])
    if (!is.finite(var_est) || var_est < 0) next

    se <- sqrt(var_est)
    if (identical(transform, "wald")) {
      lwr <- estimate - zcrit * se
      upr <- estimate + zcrit * se
    } else {
      estimate_safe <- max(min(estimate, 1 - 1e-12), -1 + 1e-12)
      z_est <- atanh(estimate_safe)
      se_z <- se / (1 - estimate_safe^2)
      lwr <- tanh(z_est - zcrit * se_z)
      upr <- tanh(z_est + zcrit * se_z)
    }

    out$se[[i]] <- se
    out$lwr.ci[[i]] <- lwr
    out$upr.ci[[i]] <- upr
  }

  out
}

.mc_ccc_glmm_metric_vector <- function(theta,
                                       include_subject_method,
                                       phi,
                                       m_reps) {
  clamp01 <- function(x) pmax(0, pmin(1, x))

  beta0 <- theta[[1L]]
  beta_method <- theta[[2L]]
  sigma2_subject <- exp(2 * theta[[3L]])
  sigma2_subject_method <- if (isTRUE(include_subject_method)) exp(2 * theta[[4L]]) else 0
  sigma2_method <- 0.5 * beta_method * beta_method
  sigma2_total <- sigma2_subject + sigma2_method + sigma2_subject_method
  sigma2_intra <- sigma2_subject + sigma2_subject_method

  mu <- exp(beta0 + 0.5 * beta_method + 0.5 * sigma2_total)
  mu1 <- exp(beta0 + 0.5 * sigma2_intra)
  mu2 <- exp(beta0 + beta_method + 0.5 * sigma2_intra)
  subject_cov <- mu * (exp(sigma2_subject) - 1)
  intra_cov1 <- mu1 * (exp(sigma2_intra) - 1)
  intra_cov2 <- mu2 * (exp(sigma2_intra) - 1)

  c(
    rho_ccc = clamp01(subject_cov / (mu * (exp(sigma2_total) - 1) + phi)),
    rho_ccc_inter = clamp01(subject_cov / (mu * (exp(sigma2_total) - 1) + phi / m_reps)),
    rho_ccc_intra_method1 = clamp01(intra_cov1 / (intra_cov1 + phi)),
    rho_ccc_intra_method2 = clamp01(intra_cov2 / (intra_cov2 + phi))
  )
}

.mc_ccc_glmm_delta_functions <- function(include_subject_method,
                                         phi,
                                         m_reps) {
  clamp01 <- function(x) pmax(0, pmin(1, x))

  rho_total <- function(theta) {
    beta0 <- theta[[1L]]
    beta_method <- theta[[2L]]
    sigma2_subject <- exp(2 * theta[[3L]])
    sigma2_subject_method <- if (isTRUE(include_subject_method)) exp(2 * theta[[4L]]) else 0
    sigma2_method <- 0.5 * beta_method * beta_method
    sigma2_total <- sigma2_subject + sigma2_method + sigma2_subject_method
    mu <- exp(beta0 + 0.5 * beta_method + 0.5 * sigma2_total)
    clamp01(
      mu * (exp(sigma2_subject) - 1) /
        (mu * (exp(sigma2_total) - 1) + phi)
    )
  }

  rho_inter <- function(theta) {
    beta0 <- theta[[1L]]
    beta_method <- theta[[2L]]
    sigma2_subject <- exp(2 * theta[[3L]])
    sigma2_subject_method <- if (isTRUE(include_subject_method)) exp(2 * theta[[4L]]) else 0
    sigma2_method <- 0.5 * beta_method * beta_method
    sigma2_total <- sigma2_subject + sigma2_method + sigma2_subject_method
    mu <- exp(beta0 + 0.5 * beta_method + 0.5 * sigma2_total)
    clamp01(
      mu * (exp(sigma2_subject) - 1) /
        (mu * (exp(sigma2_total) - 1) + phi / m_reps)
    )
  }

  rho_intra_method1 <- function(theta) {
    beta0 <- theta[[1L]]
    sigma2_subject <- exp(2 * theta[[3L]])
    sigma2_subject_method <- if (isTRUE(include_subject_method)) exp(2 * theta[[4L]]) else 0
    sigma2_intra <- sigma2_subject + sigma2_subject_method
    mu1 <- exp(beta0 + 0.5 * sigma2_intra)
    clamp01(
      mu1 * (exp(sigma2_intra) - 1) /
        (mu1 * (exp(sigma2_intra) - 1) + phi)
    )
  }

  rho_intra_method2 <- function(theta) {
    beta0 <- theta[[1L]]
    beta_method <- theta[[2L]]
    sigma2_subject <- exp(2 * theta[[3L]])
    sigma2_subject_method <- if (isTRUE(include_subject_method)) exp(2 * theta[[4L]]) else 0
    sigma2_intra <- sigma2_subject + sigma2_subject_method
    mu2 <- exp(beta0 + beta_method + 0.5 * sigma2_intra)
    clamp01(
      mu2 * (exp(sigma2_intra) - 1) /
        (mu2 * (exp(sigma2_intra) - 1) + phi)
    )
  }

  list(
    rho_ccc = rho_total,
    rho_ccc_inter = rho_inter,
    rho_ccc_intra_method1 = rho_intra_method1,
    rho_ccc_intra_method2 = rho_intra_method2
  )
}

.mc_ccc_glmm_fit <- function(y,
                             subject,
                             method,
                             n_subjects,
                             include_subject_method = FALSE,
                             max_iter = 1000L,
                             tol = 1e-8,
                             compute_vcov = FALSE) {
  gh <- .mc_ccc_glmm_ghq_rule(if (isTRUE(include_subject_method)) 11L else 40L)
  mean_a <- mean(y[method == 1L])
  mean_b <- mean(y[method == 2L])
  beta0_start <- log(mean_a + 0.1)
  beta_method_start <- log(mean_b + 0.1) - log(mean_a + 0.1)
  subject_means <- tapply(y, subject, mean)
  raw_subject_var <- stats::var(log(subject_means + 0.1))
  sigma_start <- sqrt(max(raw_subject_var, 0.05, na.rm = TRUE))
  sm_start <- if (isTRUE(include_subject_method)) {
    subject_method <- interaction(subject, method, drop = TRUE)
    sm_means <- tapply(y, subject_method, mean)
    sqrt(max(stats::var(log(sm_means + 0.1)) - raw_subject_var, 0.02, na.rm = TRUE))
  } else {
    NULL
  }

  starts <- if (isTRUE(include_subject_method)) {
    list(
      c(beta0_start, beta_method_start, log(sigma_start), log(sm_start)),
      c(beta0_start, beta_method_start, log(0.25), log(0.20)),
      c(beta0_start, beta_method_start, log(0.50), log(0.25)),
      c(beta0_start, beta_method_start, log(1.00), log(0.30)),
      c(beta0_start, beta_method_start, log(0.50), log(0.50)),
      c(log(mean(y) + 0.1), 0, log(0.50), log(0.25))
    )
  } else {
    list(
      c(beta0_start, beta_method_start, log(sigma_start)),
      c(beta0_start, beta_method_start, log(0.25)),
      c(beta0_start, beta_method_start, log(0.50)),
      c(beta0_start, beta_method_start, log(1.00)),
      c(beta0_start, beta_method_start, log(1.50)),
      c(beta0_start, beta_method_start, log(2.00)),
      c(log(mean(y) + 0.1), 0, log(0.50))
    )
  }

  blocks <- ccc_glmm_poisson_prepare_blocks_cpp(
    y = y,
    subject = subject,
    method_code = method,
    n_subjects = n_subjects
  )

  objective <- function(par) {
    ccc_glmm_poisson_ghq_nll_blocks_cpp(
      par = par,
      y1 = blocks$y1,
      y2 = blocks$y2,
      n1 = blocks$n1,
      n2 = blocks$n2,
      log_factorial = blocks$log_factorial,
      include_subject_method = include_subject_method,
      gh_nodes = gh$nodes,
      gh_weights = gh$weights
    )
  }

  fits <- lapply(starts, function(start) {
    tryCatch(
      stats::optim(
        par = start,
        fn = objective,
        method = "BFGS",
        control = list(maxit = max_iter, reltol = tol)
      ),
      error = function(e) NULL
    )
  })

  fits <- Filter(Negate(is.null), fits)
  fits <- Filter(function(z) is.finite(z$value), fits)
  if (!length(fits)) return(NULL)

  best <- fits[[which.min(vapply(fits, `[[`, numeric(1), "value"))]]
  sigma <- exp(best$par[[3L]])
  sigma_sm <- if (isTRUE(include_subject_method)) exp(best$par[[4L]]) else 0
  if (!identical(best$convergence, 0L) ||
      !is.finite(sigma) ||
      sigma <= 1e-6 ||
      (isTRUE(include_subject_method) && (!is.finite(sigma_sm) || sigma_sm <= 1e-6))) {
    return(NULL)
  }

  hessian <- NULL
  vcov_par <- NULL
  if (isTRUE(compute_vcov)) {
    hessian <- tryCatch(
      stats::optimHess(best$par, objective),
      error = function(e) NULL
    )
    vcov_par <- .mc_ccc_glmm_vcov_from_hessian(hessian)
  }

  list(
    par = best$par,
    value = best$value,
    converged = TRUE,
    convergence = best$convergence,
    message = best$message %||% "",
    counts = best$counts,
    gh = gh,
    include_subject_method = include_subject_method,
    hessian = hessian,
    vcov_par = vcov_par
  )
}

.mc_ccc_glmm_random_effect_hat <- function(y, subject, method, par, gh, include_subject_method) {
  beta0 <- par[[1L]]
  beta_method <- par[[2L]]
  sigma <- exp(par[[3L]])
  sigma_sm <- if (isTRUE(include_subject_method)) exp(par[[4L]]) else 0
  n_subjects <- max(subject)
  alpha_hat <- numeric(n_subjects)
  gamma1_hat <- numeric(n_subjects)
  gamma2_hat <- numeric(n_subjects)
  log_sqrt_pi <- 0.5 * log(pi)
  rows_by_subject <- split(seq_along(subject), subject)
  log_weights <- log(gh$weights)

  if (!isTRUE(include_subject_method)) {
    alpha <- sqrt(2) * sigma * gh$nodes
    log_base <- log_weights - log_sqrt_pi
  } else {
    vals <- expand.grid(
      qa = seq_along(gh$nodes),
      q1 = seq_along(gh$nodes),
      q2 = seq_along(gh$nodes)
    )
    alpha <- sqrt(2) * sigma * gh$nodes[vals$qa]
    gamma1 <- sqrt(2) * sigma_sm * gh$nodes[vals$q1]
    gamma2 <- sqrt(2) * sigma_sm * gh$nodes[vals$q2]
    log_base <- log_weights[vals$qa] + log_weights[vals$q1] +
      log_weights[vals$q2] - 3 * log_sqrt_pi
  }

  for (s in seq_len(n_subjects)) {
    idx <- rows_by_subject[[as.character(s)]]
    if (!isTRUE(include_subject_method)) {
      log_terms <- log_base
      for (q in seq_along(alpha)) {
        eta <- beta0 + ifelse(method[idx] == 2L, beta_method, 0) + alpha[[q]]
        log_terms[[q]] <- log_terms[[q]] + sum(y[idx] * eta - exp(eta) - lgamma(y[idx] + 1))
      }
      mx <- max(log_terms)
      w <- exp(log_terms - mx)
      alpha_hat[[s]] <- sum(w * alpha) / sum(w)
    } else {
      log_terms <- log_base
      for (q in seq_along(alpha)) {
        eta <- beta0 + ifelse(method[idx] == 2L, beta_method + gamma2[[q]], gamma1[[q]]) + alpha[[q]]
        log_terms[[q]] <- log_terms[[q]] + sum(y[idx] * eta - exp(eta) - lgamma(y[idx] + 1))
      }
      mx <- max(log_terms)
      w <- exp(log_terms - mx)
      denom <- sum(w)
      alpha_hat[[s]] <- sum(w * alpha) / denom
      gamma1_hat[[s]] <- sum(w * gamma1) / denom
      gamma2_hat[[s]] <- sum(w * gamma2) / denom
    }
  }
  list(alpha = alpha_hat, gamma1 = gamma1_hat, gamma2 = gamma2_hat)
}

.mc_ccc_glmm_metrics_from_fit <- function(y,
                                          subject,
                                          method,
                                          replicate,
                                          fit,
                                          overdispersion) {
  beta0 <- fit$par[[1L]]
  beta_method <- fit$par[[2L]]
  sigma2_subject <- exp(2 * fit$par[[3L]])
  sigma2_subject_method <- if (isTRUE(fit$include_subject_method)) exp(2 * fit$par[[4L]]) else 0
  sigma2_method <- 0.5 * beta_method * beta_method
  sigma2_total <- sigma2_subject + sigma2_method + sigma2_subject_method
  eta1 <- beta0
  eta2 <- beta0 + beta_method
  eta_bar <- 0.5 * (eta1 + eta2)
  mu <- exp(eta_bar + 0.5 * sigma2_total)
  m_reps <- .mc_ccc_glmm_replicates_per_cell(subject, method, replicate)

  phi <- 1
  if (identical(overdispersion, "pearson")) {
    re_hat <- .mc_ccc_glmm_random_effect_hat(
      y = y,
      subject = subject,
      method = method,
      par = fit$par,
      gh = fit$gh,
      include_subject_method = fit$include_subject_method
    )
    eta_hat <- beta0 + ifelse(method == 2L, beta_method + re_hat$gamma2[subject], re_hat$gamma1[subject]) +
      re_hat$alpha[subject]
    mu_hat <- pmax(exp(eta_hat), .Machine$double.eps)
    pearson <- sum((y - mu_hat)^2 / mu_hat)
    n_re <- max(subject) + if (isTRUE(fit$include_subject_method)) 2L * max(subject) else 0L
    df <- max(length(y) - 2L - n_re, 1L)
    phi <- max(pearson / df, .Machine$double.eps)
  }

  cov_subject <- mu^2 * (exp(sigma2_subject) - 1)
  var_marginal_mean <- mu^2 * (exp(sigma2_total) - 1)
  residual_var <- phi * mu
  rho_ccc <- cov_subject / (var_marginal_mean + residual_var)
  rho_inter <- cov_subject / (var_marginal_mean + residual_var / m_reps)

  sigma2_intra <- sigma2_subject + sigma2_subject_method
  mu1 <- exp(eta1 + 0.5 * sigma2_intra)
  mu2 <- exp(eta2 + 0.5 * sigma2_intra)
  cov_intra1 <- mu1^2 * (exp(sigma2_intra) - 1)
  cov_intra2 <- mu2^2 * (exp(sigma2_intra) - 1)
  rho_intra1 <- cov_intra1 / (cov_intra1 + phi * mu1)
  rho_intra2 <- cov_intra2 / (cov_intra2 + phi * mu2)
  precision <- mu * (exp(sigma2_subject) - 1) /
    (mu * (exp(sigma2_subject + sigma2_subject_method) - 1) + phi)
  accuracy <- if (is.finite(precision) && precision > 0) rho_ccc / precision else NA_real_

  clamp01 <- function(x) pmax(0, pmin(1, x))
  list(
    rho_ccc = clamp01(rho_ccc),
    rho_ccc_inter = clamp01(rho_inter),
    rho_ccc_intra_method1 = clamp01(rho_intra1),
    rho_ccc_intra_method2 = clamp01(rho_intra2),
    sigma2_subject = sigma2_subject,
    sigma2_method = sigma2_method,
    sigma2_subject_method = sigma2_subject_method,
    phi = phi,
    mu = mu,
    precision = clamp01(precision),
    accuracy = clamp01(accuracy),
    n_obs = length(y),
    n_subjects = max(subject),
    m_reps = m_reps,
    beta0 = beta0,
    beta_method = beta_method,
    nll = fit$value,
    logLik = -fit$value,
    fit_engine = "matrixCorr_ghq",
    approximation = if (isTRUE(fit$include_subject_method)) "tensor_ghq" else "ghq",
    convergence_code = fit$convergence,
    optimiser_message = fit$message,
    converged = TRUE,
    iterations = as.integer(fit$counts[["function"]] %||% NA_integer_),
    fit_status = "converged"
  )
}

#' Poisson GLMM concordance correlation for count agreement
#'
#' @description
#' Computes pairwise generalized concordance correlation coefficients (CCC) for
#' count outcomes measured repeatedly by two or more methods, observers, or
#' devices. The current implementation fits Poisson-log generalized linear
#' mixed models and reports total, inter-method, and intra-method agreement
#' summaries from the fitted mean and variance components.
#'
#' @param data A data frame containing the measurements.
#' @param response Character. Name of the non-negative integer-like count column.
#' @param subject Character. Name of the subject identifier column.
#' @param method Character. Name of the method/observer column.
#' @param replicate Optional character. Name of the replicate column. If
#'   \code{NULL}, one replicate per subject-method cell is assumed.
#' @param family Character. Currently only \code{"poisson"} is implemented.
#' @param link Character. Currently only \code{"log"} is implemented.
#' @param overdispersion Character. \code{"none"} fixes \eqn{\phi = 1};
#'   \code{"pearson"} estimates \eqn{\phi} from post-fit Pearson residuals.
#' @param include_subject_method Logical. If \code{TRUE}, include an additional
#'   subject-by-method random intercept variance component.
#' @param ci Logical. If \code{TRUE}, compute delta-method confidence intervals
#'   for the total, inter-method, and intra-method CCC estimates.
#' @param conf_level Confidence level for delta-method confidence intervals.
#' @param max_iter Positive integer. Maximum optimiser iterations.
#' @param tol Positive numeric convergence tolerance.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads to make
#'   available to compiled code.
#' @param verbose Logical. If \code{TRUE}, emit progress messages.
#'
#' @details
#' The fitted model for a pair of methods is
#' \deqn{
#' Y_{ijl} \mid \alpha_i, \gamma_{ij}
#'   \sim \mathrm{Poisson}(\mu_{ijl}),
#' \qquad
#' \log(\mu_{ijl}) = \eta_j + \alpha_i + \gamma_{ij},
#' }
#' where \eqn{i} indexes subjects, \eqn{j} indexes methods, and \eqn{l}
#' indexes replicate readings. The subject effect \eqn{\alpha_i} captures
#' between-subject heterogeneity. When \code{include_subject_method = TRUE},
#' \eqn{\gamma_{ij}} captures subject-specific method departures; otherwise it
#' is fixed at zero. The fixed method difference contributes to disagreement
#' through \code{sigma2_method}.
#'
#' The model is fitted by marginal maximum likelihood using Gauss-Hermite
#' quadrature. The random-intercept model uses 40 quadrature points. The
#' subject-by-method model uses tensor-product quadrature with fewer points per
#' dimension because it integrates over a three-dimensional subject block. The
#' reported CCC quantities are then computed from the fitted Poisson-log mean
#' and variance components.
#'
#' The main matrix value, \code{rho_ccc}, is the total agreement coefficient.
#' Use it as the primary overall count-agreement summary when each individual
#' reading is the unit of inference. It penalizes lack of between-subject
#' signal, systematic method bias, subject-by-method disagreement, and Poisson
#' residual variation.
#'
#' \code{rho_ccc_inter} is the inter-method agreement coefficient for averages
#' over replicated readings. It is useful when decisions are based on the mean
#' of \eqn{m} replicate readings per subject-method cell. Replication reduces
#' only the residual count variation term; it does not dilute systematic method
#' disagreement or between-subject variation.
#'
#' \code{rho_ccc_intra_method1} and \code{rho_ccc_intra_method2} are
#' method-specific repeatability coefficients. Use them to diagnose whether one
#' method is internally more repeatable than the other on the count scale. These
#' are not direct method-comparison coefficients; they describe within-method
#' reproducibility after accounting for the fitted mean and random effects.
#'
#' \code{precision} isolates the share of non-systematic variation attributable
#' to subject ranking/heterogeneity, while \code{accuracy} is the ratio
#' \code{rho_ccc / precision}. Low accuracy with reasonable precision indicates
#' that disagreement is driven mainly by method bias or extra method-specific
#' variation rather than poor subject discrimination.
#'
#' \code{overdispersion = "none"} fixes \eqn{\phi = 1}, the Poisson model.
#' \code{overdispersion = "pearson"} replaces the residual term by a Pearson
#' dispersion estimate. Use the Pearson adjustment as a sensitivity analysis
#' when counts appear more variable than the Poisson model allows; it should
#' generally reduce CCC when extra-Poisson variation is present.
#'
#' ## Confidence intervals
#' When \code{ci = TRUE}, large-sample confidence intervals are computed for
#' \code{rho_ccc}, \code{rho_ccc_inter}, \code{rho_ccc_intra_method1}, and
#' \code{rho_ccc_intra_method2}. The implementation uses a delta-method
#' standard error based on the fitted GLMM parameter vector and the inverse
#' Hessian of the marginal negative log-likelihood:
#' \deqn{
#' \mathrm{Var}\{g(\hat\theta)\}
#'   \approx
#'   \nabla g(\hat\theta)^\top
#'   \mathrm{Var}(\hat\theta)
#'   \nabla g(\hat\theta),
#' }
#' where \eqn{g(\theta)} is the relevant CCC function. Gradients are evaluated
#' numerically by central finite differences. The reported point estimates are
#' not replaced by \eqn{g(\hat\theta)}; they remain the values from the standard
#' point-estimate path.
#'
#' Intervals are formed on a Fisher-Z transformed CCC scale,
#' \eqn{z = 0.5\log\{(1+\rho)/(1-\rho)\}}, and then back-transformed to the
#' CCC scale. This is the same broad strategy used elsewhere for CCC-style
#' intervals because it is usually more stable than a raw Wald interval near
#' the boundaries. CI limits are not forcibly truncated to \eqn{[0, 1]}.
#'
#' For \code{overdispersion = "pearson"}, \eqn{\phi} is a post-fit Pearson
#' dispersion estimate rather than a likelihood parameter. Delta-method
#' confidence intervals therefore treat \eqn{\phi} as fixed and a warning is
#' issued.
#'
#' This function currently implements the Poisson-log count-data case. The
#' \code{family} and \code{link} arguments are present for API stability and
#' future extensions, but only \code{family = "poisson"} and
#' \code{link = "log"} are currently supported. The returned estimates are
#' variance-component CCCs constrained to \eqn{[0, 1]}; they are not Lin's raw
#' moment CCC and should not be expected to produce negative values.
#'
#' @return
#' A symmetric numeric matrix of class \code{c("ccc_glmm", "ccc")} containing
#' \code{rho_ccc}. Additional pairwise matrices are stored as attributes:
#' \itemize{
#'   \item \code{rho_ccc_inter}: agreement for replicated method averages.
#'   \item \code{rho_ccc_intra_method1}, \code{rho_ccc_intra_method2}:
#'     method-specific repeatability coefficients.
#'   \item \code{sigma2_subject}, \code{sigma2_method},
#'     \code{sigma2_subject_method}: fitted variance/disagreement components.
#'   \item \code{phi}, \code{mu}, \code{precision}, \code{accuracy}: fitted
#'     count-scale diagnostics used in the CCC decomposition.
#'   \item \code{beta0}, \code{beta_method}, \code{nll}, \code{logLik},
#'     \code{convergence_code}: fitting diagnostics.
#'   \item \code{n_obs}, \code{n_subjects}, \code{m_reps}: design diagnostics.
#'   \item when \code{ci = TRUE}, standard error and confidence-limit matrices
#'     for \code{rho_ccc}, \code{rho_ccc_inter},
#'     \code{rho_ccc_intra_method1}, and \code{rho_ccc_intra_method2}.
#' }
#'
#' @references
#' Carrasco JL (2010). A generalized concordance correlation coefficient based
#' on the variance components generalized linear mixed models for overdispersed
#' count data. \emph{Biometrics}.
#'
#' @examples
#' # Example 1: overall agreement for two Poisson count methods.
#' # Here the methods have similar means, so rho_ccc is mainly driven by
#' # between-subject count heterogeneity versus Poisson residual noise.
#' set.seed(1)
#' df1 <- expand.grid(
#'   subject = factor(seq_len(12)),
#'   method = factor(c("A", "B")),
#'   replicate = factor(seq_len(2))
#' )
#' subject_eff <- rnorm(12, 0, 0.5)
#' df1$eta <- 1.1 + subject_eff[as.integer(df1$subject)]
#' df1$y <- rpois(nrow(df1), exp(df1$eta))
#' \donttest{
#' fit1 <- ccc_glmm(df1, "y", "subject", "method", replicate = "replicate",
#' ci = TRUE)
#' fit1
#' summary(fit1)
#' estimate(fit1)
#' tidy(fit1)
#' ci(fit1)
#' confint(fit1)
#' }
#'
#' # Example 2: method bias lowers total agreement.
#' # This tests whether a systematic method shift is reflected in rho_ccc
#' # and the accuracy component.
#' set.seed(2)
#' df2 <- expand.grid(
#'   subject = factor(seq_len(12)),
#'   method = factor(c("A", "B")),
#'   replicate = factor(seq_len(2))
#' )
#' subject_eff <- rnorm(12, 0, 0.6)
#' df2$eta <- 1.0 + subject_eff[as.integer(df2$subject)] +
#'   ifelse(df2$method == "B", 0.6, 0)
#' df2$y <- rpois(nrow(df2), exp(df2$eta))
#' fit2 <- ccc_glmm(df2, "y", "subject", "method", replicate = "replicate")
#' summary(fit2)
#'
#' # Example 3: subject-by-method variation.
#' # This tests whether individual subjects respond differently by method.
#' # The subject-method component is useful when disagreement is not explained
#' # by a single fixed method bias.
#' set.seed(3)
#' df3 <- expand.grid(
#'   subject = factor(seq_len(10)),
#'   method = factor(c("A", "B")),
#'   replicate = factor(seq_len(2))
#' )
#' subject_eff <- rnorm(10, 0, 0.4)
#' subject_method_eff <- matrix(rnorm(20, 0, 0.25), nrow = 10)
#' method_id <- as.integer(df3$method)
#' df3$eta <- 1.1 + subject_eff[as.integer(df3$subject)] +
#'   subject_method_eff[cbind(as.integer(df3$subject), method_id)]
#' df3$y <- rpois(nrow(df3), exp(df3$eta))
#' \donttest{
#' fit3 <- ccc_glmm(
#'   df3, "y", "subject", "method",
#'   replicate = "replicate",
#'   include_subject_method = TRUE
#' )
#' attr(fit3, "sigma2_subject_method")
#' }
#'
#' # Example 4: four methods.
#' # This tests pairwise agreement across several count methods and helps
#' # identify which method pairs have the strongest total CCC.
#' set.seed(4)
#' df4 <- expand.grid(
#'   subject = factor(seq_len(10)),
#'   method = factor(c("A", "B", "C", "D")),
#'   replicate = factor(seq_len(2))
#' )
#' subject_eff <- rnorm(10, 0, 0.5)
#' method_bias <- c(A = 0, B = 0.1, C = 0.4, D = -0.2)
#' df4$eta <- 1.0 + subject_eff[as.integer(df4$subject)] +
#'   method_bias[as.character(df4$method)]
#' df4$y <- rpois(nrow(df4), exp(df4$eta))
#' fit4 <- ccc_glmm(df4, "y", "subject", "method", replicate = "replicate")
#' fit4
#' summary(fit4, n = 3)
#'
#' @export
ccc_glmm <- function(data,
                     response,
                     subject,
                     method,
                     replicate = NULL,
                     family = "poisson",
                     link = "log",
                     overdispersion = c("none", "pearson"),
                     include_subject_method = FALSE,
                     ci = FALSE,
                     conf_level = 0.95,
                     max_iter = 1000,
                     tol = 1e-8,
                     n_threads = getOption("matrixCorr.threads", 1L),
                     verbose = FALSE) {
  overdispersion <- match_arg(
    overdispersion,
    values = c("none", "pearson"),
    arg_name = "overdispersion"
  )

  if (!identical(family, "poisson")) {
    abort_bad_arg("family", message = "only {.val poisson} is implemented.")
  }
  if (!identical(link, "log")) {
    abort_bad_arg("link", message = "only {.val log} is implemented for Poisson GLMM CCC.")
  }

  check_bool(ci, arg = "ci")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
    if (identical(overdispersion, "pearson")) {
      cli::cli_warn(
        "Delta-method confidence intervals for {.arg overdispersion = \"pearson\"} treat the Pearson dispersion estimate as fixed."
      )
    }
  }
  if (!isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }
  max_iter <- check_scalar_int_pos(max_iter, arg = "max_iter")
  check_scalar_nonneg(tol, arg = "tol", strict = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")
  check_bool(include_subject_method, arg = "include_subject_method")

  df <- as.data.frame(data)
  req_cols <- c(response, subject, method, replicate)
  req_cols <- req_cols[!vapply(req_cols, is.null, logical(1))]
  check_required_cols(df, req_cols, df_arg = "data")

  complete_mask <- stats::complete.cases(df[, req_cols, drop = FALSE])
  df <- df[complete_mask, , drop = FALSE]
  if (!nrow(df)) {
    abort_bad_arg("data", message = "must contain at least one complete row.")
  }

  if (!is.numeric(df[[response]]) || any(!is.finite(df[[response]]))) {
    abort_bad_arg("response", message = "must reference a finite numeric count column.")
  }
  if (any(df[[response]] < 0)) {
    abort_bad_arg("response", message = "must contain non-negative counts.")
  }
  if (any(abs(df[[response]] - round(df[[response]])) > sqrt(.Machine$double.eps))) {
    abort_bad_arg("response", message = "must contain integer-like counts.")
  }

  df[[subject]] <- droplevels(factor(df[[subject]]))
  df[[method]] <- droplevels(factor(df[[method]]))
  if (!is.null(replicate)) {
    df[[replicate]] <- droplevels(factor(df[[replicate]]))
  }

  method_levels <- levels(df[[method]])
  L <- length(method_levels)
  if (L < 2L) {
    abort_bad_arg("method", message = "must have at least two levels.")
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  inform_if_verbose(
    "Using {get_omp_threads()} OpenMP thread{?s}.",
    .verbose = verbose
  )

  dn <- list(method_levels, method_levels)
  est_mat <- matrix(NA_real_, L, L, dimnames = dn)
  rho_inter_mat <- matrix(NA_real_, L, L, dimnames = dn)
  rho_intra_1_mat <- matrix(NA_real_, L, L, dimnames = dn)
  rho_intra_2_mat <- matrix(NA_real_, L, L, dimnames = dn)
  sigma2_subject_mat <- matrix(NA_real_, L, L, dimnames = dn)
  sigma2_method_mat <- matrix(NA_real_, L, L, dimnames = dn)
  sigma2_subject_method_mat <- matrix(NA_real_, L, L, dimnames = dn)
  phi_mat <- matrix(NA_real_, L, L, dimnames = dn)
  mu_mat <- matrix(NA_real_, L, L, dimnames = dn)
  precision_mat <- matrix(NA_real_, L, L, dimnames = dn)
  accuracy_mat <- matrix(NA_real_, L, L, dimnames = dn)
  n_obs_mat <- matrix(NA_integer_, L, L, dimnames = dn)
  n_subjects_mat <- matrix(NA_integer_, L, L, dimnames = dn)
  m_reps_mat <- matrix(NA_integer_, L, L, dimnames = dn)
  converged_mat <- matrix(NA, L, L, dimnames = dn)
  iterations_mat <- matrix(NA_integer_, L, L, dimnames = dn)
  fit_status_mat <- matrix(NA_character_, L, L, dimnames = dn)
  beta0_mat <- matrix(NA_real_, L, L, dimnames = dn)
  beta_method_mat <- matrix(NA_real_, L, L, dimnames = dn)
  nll_mat <- matrix(NA_real_, L, L, dimnames = dn)
  loglik_mat <- matrix(NA_real_, L, L, dimnames = dn)
  convergence_code_mat <- matrix(NA_integer_, L, L, dimnames = dn)
  optimiser_message_mat <- matrix(NA_character_, L, L, dimnames = dn)
  rho_ccc_se_mat <- rho_ccc_lwr_mat <- rho_ccc_upr_mat <- NULL
  rho_inter_se_mat <- rho_inter_lwr_mat <- rho_inter_upr_mat <- NULL
  rho_intra_1_se_mat <- rho_intra_1_lwr_mat <- rho_intra_1_upr_mat <- NULL
  rho_intra_2_se_mat <- rho_intra_2_lwr_mat <- rho_intra_2_upr_mat <- NULL
  if (isTRUE(ci)) {
    rho_ccc_se_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_ccc_lwr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_ccc_upr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_inter_se_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_inter_lwr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_inter_upr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_1_se_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_1_lwr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_1_upr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_2_se_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_2_lwr_mat <- matrix(NA_real_, L, L, dimnames = dn)
    rho_intra_2_upr_mat <- matrix(NA_real_, L, L, dimnames = dn)
  }

  for (i in seq_len(L - 1L)) {
    for (j in (i + 1L):L) {
      pair_levels <- method_levels[c(i, j)]
      df_sub <- df[df[[method]] %in% pair_levels, , drop = FALSE]
      df_sub[[method]] <- droplevels(factor(df_sub[[method]], levels = pair_levels))
      df_sub[[subject]] <- droplevels(factor(df_sub[[subject]]))
      if (!is.null(replicate)) {
        df_sub[[replicate]] <- droplevels(factor(df_sub[[replicate]]))
      }

      if (nlevels(df_sub[[subject]]) < 2L) {
        cli::cli_warn(
          "ccc_glmm Poisson fit skipped for pair ({.val {pair_levels[1]}}, {.val {pair_levels[2]}}): fewer than two subjects."
        )
        next
      }

      y <- as.numeric(df_sub[[response]])
      subject_int <- as.integer(df_sub[[subject]])
      method_int <- as.integer(df_sub[[method]])
      replicate_int <- if (is.null(replicate)) integer(0) else as.integer(df_sub[[replicate]])

      inform_if_verbose(
        "Fitting ccc_glmm pair {.val {pair_levels[1]}} vs {.val {pair_levels[2]}}.",
        .verbose = verbose
      )

      ans <- tryCatch(
        {
          fit <- .mc_ccc_glmm_fit(
            y = y,
            subject = subject_int,
            method = method_int,
            n_subjects = nlevels(df_sub[[subject]]),
            include_subject_method = include_subject_method,
            max_iter = max_iter,
            tol = tol,
            compute_vcov = ci
          )
          if (is.null(fit)) {
            NULL
          } else {
            metrics <- .mc_ccc_glmm_metrics_from_fit(
              y = y,
              subject = subject_int,
              method = method_int,
              replicate = replicate_int,
              fit = fit,
              overdispersion = overdispersion
            )
            list(fit = fit, metrics = metrics)
          }
        },
        error = function(e) {
          cli::cli_warn(
            "ccc_glmm Poisson fit failed for pair ({.val {pair_levels[1]}}, {.val {pair_levels[2]}}): {conditionMessage(e)}"
          )
          NULL
        }
      )

      if (!is.null(ans)) {
        fit_pair <- ans$fit
        ans <- ans$metrics
        boundary_subject <- is.finite(num_or_na(ans$sigma2_subject)) &&
          num_or_na(ans$sigma2_subject) <= 1e-7
        if (!isTRUE(ans$converged) || isTRUE(boundary_subject)) {
          reason <- if (isTRUE(boundary_subject)) "reached the subject-variance boundary" else "did not converge"
          cli::cli_warn(
            "ccc_glmm Poisson fit {reason} for pair ({.val {pair_levels[1]}}, {.val {pair_levels[2]}}); returning NA for this pair."
          )
          converged_mat[i, j] <- converged_mat[j, i] <- FALSE
          iterations_mat[i, j] <- iterations_mat[j, i] <- as.integer(ans$iterations %||% NA_integer_)
          convergence_code_mat[i, j] <- convergence_code_mat[j, i] <- as.integer(ans$convergence_code %||% NA_integer_)
          optimiser_message_mat[i, j] <- optimiser_message_mat[j, i] <- ans$optimiser_message %||% reason
          fit_status_mat[i, j] <- fit_status_mat[j, i] <- if (isTRUE(boundary_subject)) {
            "boundary_subject_variance"
          } else {
            "nonconverged"
          }
          next
        }
        est_mat[i, j] <- est_mat[j, i] <- num_or_na(ans$rho_ccc)
        rho_inter_mat[i, j] <- rho_inter_mat[j, i] <- num_or_na(ans$rho_ccc_inter)
        rho_intra_1_mat[i, j] <- num_or_na(ans$rho_ccc_intra_method1)
        rho_intra_2_mat[i, j] <- num_or_na(ans$rho_ccc_intra_method2)
        rho_intra_1_mat[j, i] <- num_or_na(ans$rho_ccc_intra_method2)
        rho_intra_2_mat[j, i] <- num_or_na(ans$rho_ccc_intra_method1)

        sigma2_subject_mat[i, j] <- sigma2_subject_mat[j, i] <- num_or_na(ans$sigma2_subject)
        sigma2_method_mat[i, j] <- sigma2_method_mat[j, i] <- num_or_na(ans$sigma2_method)
        sigma2_subject_method_mat[i, j] <- sigma2_subject_method_mat[j, i] <- num_or_na(ans$sigma2_subject_method)
        phi_mat[i, j] <- phi_mat[j, i] <- num_or_na(ans$phi)
        mu_mat[i, j] <- mu_mat[j, i] <- num_or_na(ans$mu)
        precision_mat[i, j] <- precision_mat[j, i] <- num_or_na(ans$precision)
        accuracy_mat[i, j] <- accuracy_mat[j, i] <- num_or_na(ans$accuracy)
        n_obs_mat[i, j] <- n_obs_mat[j, i] <- as.integer(ans$n_obs)
        n_subjects_mat[i, j] <- n_subjects_mat[j, i] <- as.integer(ans$n_subjects)
        m_reps_mat[i, j] <- m_reps_mat[j, i] <- as.integer(ans$m_reps)
        converged_mat[i, j] <- converged_mat[j, i] <- isTRUE(ans$converged)
        iterations_mat[i, j] <- iterations_mat[j, i] <- as.integer(ans$iterations)
        beta0_mat[i, j] <- beta0_mat[j, i] <- num_or_na(ans$beta0)
        beta_method_mat[i, j] <- num_or_na(ans$beta_method)
        beta_method_mat[j, i] <- -num_or_na(ans$beta_method)
        nll_mat[i, j] <- nll_mat[j, i] <- num_or_na(ans$nll)
        loglik_mat[i, j] <- loglik_mat[j, i] <- num_or_na(ans$logLik)
        convergence_code_mat[i, j] <- convergence_code_mat[j, i] <- as.integer(ans$convergence_code)
        optimiser_message_mat[i, j] <- optimiser_message_mat[j, i] <- ans$optimiser_message %||% ""
        fit_status_mat[i, j] <- fit_status_mat[j, i] <- ans$fit_status %||% "converged"

        if (isTRUE(ci)) {
          ci_vec <- .mc_delta_ci_vec(
            estimates = c(
              rho_ccc = ans$rho_ccc,
              rho_ccc_inter = ans$rho_ccc_inter,
              rho_ccc_intra_method1 = ans$rho_ccc_intra_method1,
              rho_ccc_intra_method2 = ans$rho_ccc_intra_method2
            ),
            f = function(theta) {
              .mc_ccc_glmm_metric_vector(
                theta,
                include_subject_method = include_subject_method,
                phi = ans$phi,
                m_reps = ans$m_reps
              )
            },
            theta = fit_pair$par,
            vcov_theta = fit_pair$vcov_par,
            conf_level = conf_level,
            transform = "fisher"
          )
          ci_total <- ci_vec["rho_ccc", , drop = FALSE]
          ci_inter <- ci_vec["rho_ccc_inter", , drop = FALSE]
          ci_intra1 <- ci_vec["rho_ccc_intra_method1", , drop = FALSE]
          ci_intra2 <- ci_vec["rho_ccc_intra_method2", , drop = FALSE]

          rho_ccc_se_mat[i, j] <- rho_ccc_se_mat[j, i] <- num_or_na(ci_total$se)
          rho_ccc_lwr_mat[i, j] <- rho_ccc_lwr_mat[j, i] <- num_or_na(ci_total$lwr.ci)
          rho_ccc_upr_mat[i, j] <- rho_ccc_upr_mat[j, i] <- num_or_na(ci_total$upr.ci)
          rho_inter_se_mat[i, j] <- rho_inter_se_mat[j, i] <- num_or_na(ci_inter$se)
          rho_inter_lwr_mat[i, j] <- rho_inter_lwr_mat[j, i] <- num_or_na(ci_inter$lwr.ci)
          rho_inter_upr_mat[i, j] <- rho_inter_upr_mat[j, i] <- num_or_na(ci_inter$upr.ci)

          rho_intra_1_se_mat[i, j] <- num_or_na(ci_intra1$se)
          rho_intra_1_lwr_mat[i, j] <- num_or_na(ci_intra1$lwr.ci)
          rho_intra_1_upr_mat[i, j] <- num_or_na(ci_intra1$upr.ci)
          rho_intra_2_se_mat[i, j] <- num_or_na(ci_intra2$se)
          rho_intra_2_lwr_mat[i, j] <- num_or_na(ci_intra2$lwr.ci)
          rho_intra_2_upr_mat[i, j] <- num_or_na(ci_intra2$upr.ci)

          rho_intra_1_se_mat[j, i] <- num_or_na(ci_intra2$se)
          rho_intra_1_lwr_mat[j, i] <- num_or_na(ci_intra2$lwr.ci)
          rho_intra_1_upr_mat[j, i] <- num_or_na(ci_intra2$upr.ci)
          rho_intra_2_se_mat[j, i] <- num_or_na(ci_intra1$se)
          rho_intra_2_lwr_mat[j, i] <- num_or_na(ci_intra1$lwr.ci)
          rho_intra_2_upr_mat[j, i] <- num_or_na(ci_intra1$upr.ci)
        }
      }
    }
  }

  diag(est_mat) <- 1
  diag(rho_inter_mat) <- 1
  diag(rho_intra_1_mat) <- 1
  diag(rho_intra_2_mat) <- 1
  diag(sigma2_subject_mat) <- 0
  diag(sigma2_method_mat) <- 0
  diag(sigma2_subject_method_mat) <- 0
  diag(precision_mat) <- 1
  diag(accuracy_mat) <- 1
  diag(phi_mat) <- if (identical(overdispersion, "none")) 1 else NA_real_

  out <- structure(
    est_mat,
    class = c("ccc_glmm", "ccc")
  )
  attr(out, "method") <- "GLMM variance-components CCC for count data"
  attr(out, "description") <- "Poisson GLMM concordance correlation for count data"
  attr(out, "package") <- "matrixCorr"
  attr(out, "family") <- family
  attr(out, "link") <- link
  attr(out, "overdispersion") <- overdispersion
  attr(out, "include_subject_method") <- include_subject_method
  attr(out, "rho_ccc") <- est_mat
  attr(out, "rho_ccc_inter") <- rho_inter_mat
  attr(out, "rho_ccc_intra_method1") <- rho_intra_1_mat
  attr(out, "rho_ccc_intra_method2") <- rho_intra_2_mat
  attr(out, "sigma2_subject") <- sigma2_subject_mat
  attr(out, "sigma2_method") <- sigma2_method_mat
  attr(out, "sigma2_subject_method") <- sigma2_subject_method_mat
  attr(out, "phi") <- phi_mat
  attr(out, "mu") <- mu_mat
  attr(out, "precision") <- precision_mat
  attr(out, "accuracy") <- accuracy_mat
  attr(out, "n_obs") <- n_obs_mat
  attr(out, "n_subjects") <- n_subjects_mat
  attr(out, "m_reps") <- m_reps_mat
  attr(out, "converged") <- converged_mat
  attr(out, "iterations") <- iterations_mat
  attr(out, "fit_status") <- fit_status_mat
  attr(out, "beta0") <- beta0_mat
  attr(out, "beta_method") <- beta_method_mat
  attr(out, "nll") <- nll_mat
  attr(out, "logLik") <- loglik_mat
  attr(out, "fit_engine") <- "matrixCorr_ghq"
  attr(out, "approximation") <- if (isTRUE(include_subject_method)) "tensor_ghq" else "ghq"
  attr(out, "convergence_code") <- convergence_code_mat
  attr(out, "optimiser_message") <- optimiser_message_mat
  if (isTRUE(ci)) {
    attr(out, "rho_ccc_se") <- rho_ccc_se_mat
    attr(out, "rho_ccc_lwr.ci") <- rho_ccc_lwr_mat
    attr(out, "rho_ccc_upr.ci") <- rho_ccc_upr_mat
    attr(out, "rho_ccc_inter_se") <- rho_inter_se_mat
    attr(out, "rho_ccc_inter_lwr.ci") <- rho_inter_lwr_mat
    attr(out, "rho_ccc_inter_upr.ci") <- rho_inter_upr_mat
    attr(out, "rho_ccc_intra_method1_se") <- rho_intra_1_se_mat
    attr(out, "rho_ccc_intra_method1_lwr.ci") <- rho_intra_1_lwr_mat
    attr(out, "rho_ccc_intra_method1_upr.ci") <- rho_intra_1_upr_mat
    attr(out, "rho_ccc_intra_method2_se") <- rho_intra_2_se_mat
    attr(out, "rho_ccc_intra_method2_lwr.ci") <- rho_intra_2_lwr_mat
    attr(out, "rho_ccc_intra_method2_upr.ci") <- rho_intra_2_upr_mat
    attr(out, "ci") <- TRUE
    attr(out, "conf_level") <- conf_level
    attr(out, "conf.level") <- conf_level
    attr(out, "ci_method") <- "delta_fisher"
  } else {
    attr(out, "ci") <- FALSE
  }
  out
}

#' @export
#' @method print ccc_glmm
print.ccc_glmm <- function(x,
                           digits = 4,
                           ci_digits = 4,
                           n = NULL,
                           topn = NULL,
                           max_vars = NULL,
                           width = NULL,
                           show_ci = NULL,
                           ...) {
  est <- if (is.list(x) && !is.null(x$est)) as.matrix(x$est) else as.matrix(x)
  .mc_print_corr_matrix(
    x,
    header = "GLMM concordance correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = est,
    ...
  )
  invisible(x)
}

#' @export
#' @method summary ccc_glmm
summary.ccc_glmm <- function(object,
                             digits = 4,
                             ci_digits = 2,
                             n = NULL,
                             topn = NULL,
                             max_vars = NULL,
                             width = NULL,
                             show_ci = NULL,
                             ...) {
  base <- summary.ccc(
    object,
    digits = digits,
    ci_digits = ci_digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  if (all(c("item1", "item2") %in% names(base))) {
    names(base)[match(c("item1", "item2"), names(base))] <- c("method1", "method2")
  }
  est <- as.matrix(object)
  extract_pair <- function(mat) {
    out <- numeric(nrow(est) * (ncol(est) - 1L) / 2L)
    k <- 0L
    for (i in seq_len(nrow(est) - 1L)) {
      for (j in (i + 1L):ncol(est)) {
        k <- k + 1L
        out[k] <- suppressWarnings(as.numeric(mat[i, j]))
      }
    }
    out
  }

  attrs <- c(
    "rho_ccc_inter",
    "rho_ccc_intra_method1",
    "rho_ccc_intra_method2",
    "rho_ccc_inter_se",
    "rho_ccc_inter_lwr.ci",
    "rho_ccc_inter_upr.ci",
    "rho_ccc_intra_method1_se",
    "rho_ccc_intra_method1_lwr.ci",
    "rho_ccc_intra_method1_upr.ci",
    "rho_ccc_intra_method2_se",
    "rho_ccc_intra_method2_lwr.ci",
    "rho_ccc_intra_method2_upr.ci",
    "sigma2_subject",
    "sigma2_method",
    "sigma2_subject_method",
    "phi",
    "mu",
    "precision",
    "accuracy",
    "beta0",
    "beta_method",
    "nll",
    "logLik",
    "convergence_code",
    "n_subjects",
    "n_obs",
    "m_reps"
  )
  for (nm in attrs) {
    val <- attr(object, nm, exact = TRUE)
    if (is.matrix(val)) {
      base[[nm]] <- extract_pair(val)
    }
  }
  if (isTRUE(attr(object, "ci", exact = TRUE))) {
    ci_map <- c(
      se = "rho_ccc_se",
      lwr.ci = "rho_ccc_lwr.ci",
      upr.ci = "rho_ccc_upr.ci"
    )
    for (nm in names(ci_map)) {
      val <- attr(object, ci_map[[nm]], exact = TRUE)
      if (is.matrix(val)) {
        base[[nm]] <- extract_pair(val)
      }
    }
  }

  num_cols <- names(base)[vapply(base, is.numeric, logical(1))]
  for (nm in num_cols) {
    if (!nm %in% c("n_subjects", "n_obs", "m_reps")) {
      base[[nm]] <- round(base[[nm]], digits)
    }
  }
  if ("n_subjects" %in% names(base)) base$n_subjects <- as.integer(base$n_subjects)
  if ("n_obs" %in% names(base)) base$n_obs <- as.integer(base$n_obs)
  if ("m_reps" %in% names(base)) base$m_reps <- as.integer(base$m_reps)

  class(base) <- c("summary.ccc_glmm", class(base))
  attr(base, "method") <- attr(object, "method", exact = TRUE)
  attr(base, "family") <- attr(object, "family", exact = TRUE)
  attr(base, "link") <- attr(object, "link", exact = TRUE)
  attr(base, "overdispersion") <- attr(object, "overdispersion", exact = TRUE)
  attr(base, "has_ci") <- isTRUE(attr(object, "ci", exact = TRUE))
  attr(base, "conf.level") <- attr(object, "conf.level", exact = TRUE)
  attr(base, "ci_method") <- attr(object, "ci_method", exact = TRUE)
  attr(base, "digits") <- digits
  attr(base, "ci_digits") <- ci_digits
  attr(base, "print_n") <- n
  attr(base, "print_topn") <- topn
  base
}

#' @export
#' @method print summary.ccc_glmm
print.summary.ccc_glmm <- function(x,
                                   digits = NULL,
                                   n = NULL,
                                   topn = NULL,
                                   max_vars = NULL,
                                   width = NULL,
                                   show_ci = NULL,
                                   style = c("auto", "compact", "pairwise"),
                                   ...) {
  style <- match.arg(style)
  digits <- .mc_coalesce(digits, attr(x, "digits", exact = TRUE))
  if (is.null(digits) || !is.finite(digits)) digits <- 4L
  n <- .mc_coalesce(n, attr(x, "print_n", exact = TRUE))
  topn <- .mc_coalesce(topn, attr(x, "print_topn", exact = TRUE))

  if (identical(style, "auto")) {
    style <- if (nrow(x) <= 1L) "pairwise" else "compact"
  }

  if (identical(style, "compact")) {
    rows <- seq_len(nrow(x))
    if (!is.null(n)) {
      n <- check_scalar_int_pos(n, arg = "n")
      rows <- rows[seq_len(min(length(rows), n))]
    }
    omitted <- nrow(x) - length(rows)

    print_section <- function(title, cols) {
      cols <- cols[cols %in% names(x)]
      if (!length(cols)) return(invisible(NULL))
      cli::cat_line("")
      cli::cat_line(title)
      cli::cat_line("")
      print.data.frame(x[rows, cols, drop = FALSE], row.names = FALSE, right = FALSE, ...)
      invisible(NULL)
    }

    cli::cat_line("GLMM concordance correlation summary")
    if (omitted > 0L) {
      cli::cat_line("Showing ", length(rows), " of ", nrow(x), " method pairs.")
    }

    print_section(
      "Concordance estimates",
      c("method1", "method2", "estimate", "se", "lwr.ci", "upr.ci", "precision", "accuracy")
    )
    print_section(
      "Inter-method agreement for replicate means",
      c(
        "method1", "method2",
        "rho_ccc_inter",
        "rho_ccc_inter_se",
        "rho_ccc_inter_lwr.ci",
        "rho_ccc_inter_upr.ci"
      )
    )
    print_section(
      "Intra-method repeatability",
      c(
        "method1", "method2",
        "rho_ccc_intra_method1",
        "rho_ccc_intra_method1_se",
        "rho_ccc_intra_method1_lwr.ci",
        "rho_ccc_intra_method1_upr.ci",
        "rho_ccc_intra_method2",
        "rho_ccc_intra_method2_se",
        "rho_ccc_intra_method2_lwr.ci",
        "rho_ccc_intra_method2_upr.ci"
      )
    )
    print_section(
      "Variance components and mean",
      c("method1", "method2", "sigma2_subject", "sigma2_method", "sigma2_subject_method", "mu", "phi")
    )
    print_section(
      "Fit diagnostics",
      c("method1", "method2", "beta0", "beta_method", "nll", "convergence_code")
    )
    print_section(
      "Design information",
      c("method1", "method2", "n_subjects", "n_obs", "m_reps")
    )
    if (omitted > 0L) {
      cli::cat_line(
        "",
        "... ", omitted, " more method pair", if (omitted == 1L) "" else "s",
        " not shown. Use a larger `n` or `as.data.frame()` to inspect all rows."
      )
    }
    return(invisible(x))
  }

  rows <- seq_len(nrow(x))
  if (!is.null(n)) {
    n <- check_scalar_int_pos(n, arg = "n")
    rows <- rows[seq_len(min(length(rows), n))]
  }

  value <- function(row, name) {
    if (!name %in% names(row)) return(NA)
    out <- row[[name]]
    if (is.numeric(out) && name != "convergence_code") {
      out <- round(out, digits)
    }
    out
  }

  make_table <- function(row, labels, names) {
    data.frame(
      quantity = labels,
      value = vapply(names, function(nm) as.character(value(row, nm)), character(1)),
      check.names = FALSE
    )
  }

  make_concordance_table <- function(row, method1, method2) {
    labels <- c(
      "Total CCC (single count)",
      "Inter-CCC (replicate mean)",
      paste0("Intra CCC (", method1, ")"),
      paste0("Intra CCC (", method2, ")"),
      "Precision",
      "Accuracy"
    )
    estimate_names <- c(
      "estimate",
      "rho_ccc_inter",
      "rho_ccc_intra_method1",
      "rho_ccc_intra_method2",
      "precision",
      "accuracy"
    )
    out <- data.frame(
      quantity = labels,
      estimate = vapply(estimate_names, function(nm) as.character(value(row, nm)), character(1)),
      check.names = FALSE
    )

    if (isTRUE(attr(x, "has_ci", exact = TRUE))) {
      se_names <- c(
        "se",
        "rho_ccc_inter_se",
        "rho_ccc_intra_method1_se",
        "rho_ccc_intra_method2_se",
        NA_character_,
        NA_character_
      )
      lwr_names <- c(
        "lwr.ci",
        "rho_ccc_inter_lwr.ci",
        "rho_ccc_intra_method1_lwr.ci",
        "rho_ccc_intra_method2_lwr.ci",
        NA_character_,
        NA_character_
      )
      upr_names <- c(
        "upr.ci",
        "rho_ccc_inter_upr.ci",
        "rho_ccc_intra_method1_upr.ci",
        "rho_ccc_intra_method2_upr.ci",
        NA_character_,
        NA_character_
      )
      pull <- function(nms) {
        vapply(nms, function(nm) {
          if (is.na(nm)) "" else as.character(value(row, nm))
        }, character(1))
      }
      out$se <- pull(se_names)
      out$lwr.ci <- pull(lwr_names)
      out$upr.ci <- pull(upr_names)
    }

    out
  }

  cli::cat_line("GLMM concordance correlation summary")
  if (length(rows) < nrow(x)) {
    cli::cat_line("Showing ", length(rows), " of ", nrow(x), " method pairs.")
  }

  for (idx in rows) {
    row <- x[idx, , drop = FALSE]
    method1 <- as.character(row$method1)
    method2 <- as.character(row$method2)

    cli::cat_line("")
    cli::cat_line("Pair: ", method1, " vs ", method2)
    cli::cat_line("")

    concordance <- make_concordance_table(row, method1, method2)
    cli::cat_line("Concordance")
    print.data.frame(concordance, row.names = FALSE, right = FALSE, ...)

    variance <- make_table(
      row,
      labels = c(
        "Subject variance",
        "Method disagreement variance",
        "Subject-method variance",
        "Marginal mean",
        "Dispersion phi"
      ),
      names = c(
        "sigma2_subject",
        "sigma2_method",
        "sigma2_subject_method",
        "mu",
        "phi"
      )
    )
    cli::cat_line("")
    cli::cat_line("Variance components and mean")
    print.data.frame(variance, row.names = FALSE, right = FALSE, ...)

    fit <- make_table(
      row,
      labels = c(
        paste0("Intercept (", method1, ")"),
        paste0("Method effect (", method2, " - ", method1, ")"),
        "Negative log-likelihood",
        "Log-likelihood",
        "Convergence code",
        "Subjects",
        "Observations",
        "Replicates per subject-method"
      ),
      names = c(
        "beta0",
        "beta_method",
        "nll",
        "logLik",
        "convergence_code",
        "n_subjects",
        "n_obs",
        "m_reps"
      )
    )
    cli::cat_line("")
    cli::cat_line("Fit and design")
    print.data.frame(fit, row.names = FALSE, right = FALSE, ...)
  }

  invisible(x)
}

make_ccc_glmm_perf_data <- function(n_subjects = 20,
                                    reps = 2,
                                    method_bias = 0.2,
                                    subject_sd = 0.7,
                                    seed = 1) {
  set.seed(seed)
  df <- expand.grid(
    subject = factor(seq_len(n_subjects)),
    method = factor(c("A", "B")),
    replicate = factor(seq_len(reps))
  )
  subj_eff <- stats::rnorm(n_subjects, 0, subject_sd)
  df$eta <- 1.5 + subj_eff[as.integer(df$subject)] +
    ifelse(df$method == "B", method_bias, 0)
  df$y <- stats::rpois(nrow(df), exp(df$eta))
  df
}

test_that("block-based Poisson GLMM likelihood matches raw likelihood", {
  df <- make_ccc_glmm_perf_data(n_subjects = 8, reps = 2, method_bias = 0.4, seed = 101)
  y <- as.numeric(df$y)
  subject <- as.integer(droplevels(factor(df$subject)))
  method <- as.integer(droplevels(factor(df$method)))
  n_subjects <- max(subject)

  blocks <- matrixCorr:::ccc_glmm_poisson_prepare_blocks_cpp(y, subject, method, n_subjects)
  gh_simple <- matrixCorr:::.mc_ccc_glmm_ghq_rule(11L)
  gh_sm <- matrixCorr:::.mc_ccc_glmm_ghq_rule(7L)

  theta_simple <- list(
    c(1.2, 0.15, log(0.35)),
    c(1.4, -0.2, log(0.65)),
    c(0.8, 0.6, log(1.1))
  )
  for (theta in theta_simple) {
    raw <- matrixCorr:::ccc_glmm_poisson_ghq_nll_cpp(
      theta, y, subject, method, n_subjects, FALSE,
      gh_simple$nodes, gh_simple$weights
    )
    prepared <- matrixCorr:::ccc_glmm_poisson_ghq_nll_blocks_cpp(
      theta,
      blocks$y1, blocks$y2, blocks$n1, blocks$n2, blocks$log_factorial,
      FALSE, gh_simple$nodes, gh_simple$weights
    )
    expect_equal(prepared, raw, tolerance = 1e-12)
  }

  theta_sm <- list(
    c(1.2, 0.15, log(0.35), log(0.25)),
    c(1.4, -0.2, log(0.65), log(0.40)),
    c(0.8, 0.6, log(1.1), log(0.55))
  )
  for (theta in theta_sm) {
    raw <- matrixCorr:::ccc_glmm_poisson_ghq_nll_cpp(
      theta, y, subject, method, n_subjects, TRUE,
      gh_sm$nodes, gh_sm$weights
    )
    prepared <- matrixCorr:::ccc_glmm_poisson_ghq_nll_blocks_cpp(
      theta,
      blocks$y1, blocks$y2, blocks$n1, blocks$n2, blocks$log_factorial,
      TRUE, gh_sm$nodes, gh_sm$weights
    )
    expect_equal(prepared, raw, tolerance = 1e-12)
  }
})

test_that("vectorized ccc_glmm delta CIs match scalar delta helper", {
  theta <- c(1.1, 0.25, log(0.55))
  vcov_theta <- matrix(
    c(0.02, 0.001, 0.0005,
      0.001, 0.03, 0.002,
      0.0005, 0.002, 0.015),
    nrow = 3L
  )
  include_subject_method <- FALSE
  phi <- 1.2
  m_reps <- 2L

  funcs <- matrixCorr:::.mc_ccc_glmm_delta_functions(include_subject_method, phi, m_reps)
  estimates <- matrixCorr:::.mc_ccc_glmm_metric_vector(theta, include_subject_method, phi, m_reps)
  vec <- matrixCorr:::.mc_delta_ci_vec(
    estimates,
    function(x) matrixCorr:::.mc_ccc_glmm_metric_vector(x, include_subject_method, phi, m_reps),
    theta,
    vcov_theta,
    conf_level = 0.9,
    transform = "fisher"
  )

  scalar <- lapply(names(estimates), function(nm) {
    matrixCorr:::.mc_delta_ci(
      estimate = estimates[[nm]],
      f = funcs[[nm]],
      theta = theta,
      vcov_theta = vcov_theta,
      conf_level = 0.9,
      transform = "fisher"
    )
  })
  names(scalar) <- names(estimates)

  for (nm in names(estimates)) {
    expect_equal(vec[nm, "se"], scalar[[nm]]$se, tolerance = 1e-12, info = nm)
    expect_equal(vec[nm, "lwr.ci"], scalar[[nm]]$lwr.ci, tolerance = 1e-12, info = nm)
    expect_equal(vec[nm, "upr.ci"], scalar[[nm]]$upr.ci, tolerance = 1e-12, info = nm)
  }
})

test_that("ccc_glmm optimized path preserves CI object structure", {
  df <- make_ccc_glmm_perf_data(n_subjects = 16, reps = 2, method_bias = 0.5, seed = 102)
  expect_warning(
    fit <- ccc_glmm(
      df,
      "y", "subject", "method",
      replicate = "replicate",
      overdispersion = "pearson",
      ci = TRUE,
      max_iter = 500
    ),
    "treat the Pearson dispersion estimate as fixed"
  )

  expect_s3_class(fit, "ccc_glmm")
  expect_identical(class(fit), c("ccc_glmm", "ccc"))
  expect_identical(attr(fit, "ci", exact = TRUE), TRUE)
  expect_identical(attr(fit, "ci_method", exact = TRUE), "delta_fisher")
  for (nm in c(
    "rho_ccc_se", "rho_ccc_lwr.ci", "rho_ccc_upr.ci",
    "rho_ccc_inter_se", "rho_ccc_intra_method1_se",
    "rho_ccc_intra_method2_se"
  )) {
    val <- attr(fit, nm, exact = TRUE)
    expect_identical(dim(val), dim(fit), info = nm)
    expect_identical(dimnames(val), dimnames(fit), info = nm)
  }
})

make_ccc_glmm_data <- function(n_subjects = 20,
                               reps = 2,
                               method_bias = 0.2,
                               subject_sd = 0.7,
                               seed = 1) {
  set.seed(seed)
  df <- expand.grid(
    subject = factor(seq_len(n_subjects)),
    method = factor(c("A", "B")),
    replicate = factor(seq_len(reps))
  )
  subj_eff <- rnorm(n_subjects, 0, subject_sd)
  df$eta <- 1.5 + subj_eff[as.integer(df$subject)] +
    ifelse(df$method == "B", method_bias, 0)
  df$y <- rpois(nrow(df), exp(df$eta))
  df
}

test_that("ccc_glmm returns matrixCorr CCC object", {
  df <- make_ccc_glmm_data()

  fit <- ccc_glmm(
    df,
    response = "y",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    overdispersion = "none"
  )

  expect_s3_class(fit, "ccc_glmm")
  expect_s3_class(fit, "ccc")
  expect_identical(class(fit), c("ccc_glmm", "ccc"))
  expect_true(is.matrix(fit))
  expect_equal(diag(fit), c(A = 1, B = 1), ignore_attr = TRUE)
})

test_that("ccc_glmm exposes symmetric bounded CCC matrices and components", {
  fit <- ccc_glmm(
    make_ccc_glmm_data(),
    response = "y",
    subject = "subject",
    method = "method",
    replicate = "replicate"
  )

  expect_true(isSymmetric(unclass(fit), check.attributes = FALSE))
  expect_equal(diag(fit), c(A = 1, B = 1), ignore_attr = TRUE)
  expect_true(is.finite(fit[1, 2]))
  expect_gte(fit[1, 2], 0)
  expect_lte(fit[1, 2], 1)

  for (nm in c("rho_ccc_inter", "rho_ccc_intra_method1", "rho_ccc_intra_method2")) {
    val <- attr(fit, nm, exact = TRUE)
    expect_true(is.matrix(val), info = nm)
    expect_true(is.finite(val[1, 2]), info = nm)
    expect_gte(val[1, 2], 0)
    expect_lte(val[1, 2], 1)
  }
})

test_that("ccc_glmm handles overdispersion phi modes", {
  df <- make_ccc_glmm_data()
  fit_none <- ccc_glmm(
    df,
    response = "y",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    overdispersion = "none"
  )
  fit_pearson <- ccc_glmm(
    df,
    response = "y",
    subject = "subject",
    method = "method",
    replicate = "replicate",
    overdispersion = "pearson"
  )

  expect_equal(attr(fit_none, "phi", exact = TRUE)[1, 2], 1)
  phi <- attr(fit_pearson, "phi", exact = TRUE)[1, 2]
  expect_true(is.finite(phi))
  expect_gt(phi, 0)
})

test_that("larger fixed method bias lowers GLMM CCC", {
  low_bias <- ccc_glmm(
    make_ccc_glmm_data(method_bias = 0, seed = 2),
    "y", "subject", "method",
    replicate = "replicate",
    include_subject_method = FALSE
  )
  high_bias <- ccc_glmm(
    make_ccc_glmm_data(method_bias = 1.5, seed = 2),
    "y", "subject", "method",
    replicate = "replicate",
    include_subject_method = FALSE
  )

  expect_lt(high_bias[1, 2], low_bias[1, 2])
})

test_that("ccc_glmm validates count data and implemented model choices", {
  df <- make_ccc_glmm_data()
  bad_non_integer <- df
  bad_non_integer$y[1] <- 1.25
  bad_negative <- df
  bad_negative$y[1] <- -1

  expect_error(
    ccc_glmm(bad_non_integer, "y", "subject", "method"),
    "integer-like"
  )
  expect_error(
    ccc_glmm(bad_negative, "y", "subject", "method"),
    "non-negative"
  )
  expect_error(
    ccc_glmm(df, "y", "subject", "method", family = "binomial"),
    "poisson"
  )
  expect_error(
    ccc_glmm(df, "y", "subject", "method", link = "identity"),
    "log"
  )
})

test_that("summary.ccc_glmm includes GLMM quantities", {
  fit <- ccc_glmm(
    make_ccc_glmm_data(),
    "y", "subject", "method",
    replicate = "replicate",
    overdispersion = "pearson"
  )
  sm <- summary(fit)

  expect_s3_class(sm, "summary.ccc_glmm")
  expect_named(
    sm,
    c(
      "method1", "method2", "estimate",
      "rho_ccc_inter",
      "rho_ccc_intra_method1",
      "rho_ccc_intra_method2",
      "sigma2_subject",
      "sigma2_method",
      "sigma2_subject_method",
      "phi",
      "mu",
      "precision",
      "accuracy",
      "beta0",
      "beta_method",
      "nll",
      "logLik",
      "convergence_code",
      "n_subjects",
      "n_obs",
      "m_reps"
    ),
    ignore.order = TRUE
  )
  expect_true(is.finite(sm$rho_ccc_inter[1]))
  expect_true(is.finite(sm$phi[1]))
})

test_that("ccc_glmm fits subject-method variance component when requested", {
  df <- make_ccc_glmm_data(n_subjects = 16, reps = 2, seed = 11)
  fit <- ccc_glmm(
    df,
    "y", "subject", "method",
    replicate = "replicate",
    include_subject_method = TRUE,
    max_iter = 500
  )

  expect_s3_class(fit, "ccc_glmm")
  expect_true(is.finite(fit[1, 2]))
  expect_true(is.finite(attr(fit, "sigma2_subject_method", exact = TRUE)[1, 2]))
  expect_gte(attr(fit, "sigma2_subject_method", exact = TRUE)[1, 2], 0)
  expect_identical(attr(fit, "include_subject_method", exact = TRUE), TRUE)
})

test_that("ccc_glmm summary includes fitted beta and variance components", {
  fit <- ccc_glmm(
    make_ccc_glmm_data(),
    "y", "subject", "method",
    replicate = "replicate"
  )
  sm <- summary(fit)

  expect_true(is.finite(sm$beta0[1]))
  expect_true(is.finite(sm$beta_method[1]))
  expect_true(is.finite(sm$sigma2_subject[1]))
  expect_true(is.finite(sm$mu[1]))
  expect_true(is.finite(sm$nll[1]))
  expect_true(is.finite(sm$logLik[1]))
})


test_that("plot.ccc works for ccc_glmm through inherited ccc class", {
  fit <- ccc_glmm(
    make_ccc_glmm_data(n_subjects = 12),
    "y", "subject", "method",
    replicate = "replicate"
  )
  p <- plot(fit)
  expect_s3_class(p, "ggplot")
})

test_that("ccc_glmm delta CIs do not alter point estimates", {
  df <- make_ccc_glmm_data(n_subjects = 18, reps = 2, seed = 21)
  fit_no_ci <- ccc_glmm(
    df,
    "y", "subject", "method",
    replicate = "replicate",
    ci = FALSE
  )
  fit_ci <- ccc_glmm(
    df,
    "y", "subject", "method",
    replicate = "replicate",
    ci = TRUE
  )

  expect_equal(
    unclass(as.matrix(fit_ci)),
    unclass(as.matrix(fit_no_ci)),
    tolerance = 0,
    ignore_attr = TRUE
  )
  point_attrs <- c(
    "rho_ccc", "rho_ccc_inter",
    "rho_ccc_intra_method1", "rho_ccc_intra_method2",
    "precision", "accuracy",
    "sigma2_subject", "sigma2_method", "sigma2_subject_method",
    "phi", "mu"
  )
  for (nm in point_attrs) {
    expect_equal(
      attr(fit_ci, nm, exact = TRUE),
      attr(fit_no_ci, nm, exact = TRUE),
      tolerance = 0,
      info = nm
    )
  }
})

test_that("ccc_glmm delta CI attributes exist and are shaped like estimates", {
  df <- make_ccc_glmm_data(n_subjects = 18, reps = 3, method_bias = 0.7, seed = 22)
  fit <- ccc_glmm(
    df,
    "y", "subject", "method",
    replicate = "replicate",
    ci = TRUE
  )

  ci_attrs <- c(
    "rho_ccc_se",
    "rho_ccc_lwr.ci",
    "rho_ccc_upr.ci",
    "rho_ccc_inter_se",
    "rho_ccc_intra_method1_se",
    "rho_ccc_intra_method2_se"
  )
  for (nm in ci_attrs) {
    val <- attr(fit, nm, exact = TRUE)
    expect_true(is.matrix(val), info = nm)
    expect_identical(dim(val), dim(fit), info = nm)
    expect_identical(dimnames(val), dimnames(fit), info = nm)
    expect_true(is.finite(val[1, 2]), info = nm)
  }
  expect_true(isTRUE(attr(fit, "ci", exact = TRUE)))
  expect_identical(attr(fit, "ci_method", exact = TRUE), "delta_fisher")
  sm <- summary(fit)
  expect_true(all(c("se", "lwr.ci", "upr.ci") %in% names(sm)))
  expect_true(is.finite(sm$se[1]))
  expect_true(is.finite(sm$lwr.ci[1]))
  expect_true(is.finite(sm$upr.ci[1]))
})

test_that("ccc_glmm ci = FALSE remains lean", {
  fit <- ccc_glmm(
    make_ccc_glmm_data(seed = 23),
    "y", "subject", "method",
    replicate = "replicate",
    ci = FALSE
  )

  expect_null(attr(fit, "rho_ccc_se", exact = TRUE))
  expect_null(attr(fit, "rho_ccc_lwr.ci", exact = TRUE))
  expect_false(isTRUE(attr(fit, "ci", exact = TRUE)))
})

test_that("ccc_glmm delta CI handles unavailable covariance safely", {
  ci <- .mc_delta_ci(
    estimate = 0.5,
    f = function(theta) theta[[1L]],
    theta = 0,
    vcov_theta = NULL
  )
  expect_true(is.na(ci$se))
  expect_true(is.na(ci$lwr.ci))
  expect_true(is.na(ci$upr.ci))
})

test_that("ccc_glmm inter and intra CIs are metric-specific", {
  df <- make_ccc_glmm_data(n_subjects = 20, reps = 4, method_bias = 1.0, seed = 24)
  fit <- ccc_glmm(
    df,
    "y", "subject", "method",
    replicate = "replicate",
    ci = TRUE
  )

  expect_false(isTRUE(all.equal(
    attr(fit, "rho_ccc_se", exact = TRUE)[1, 2],
    attr(fit, "rho_ccc_inter_se", exact = TRUE)[1, 2]
  )))
  expect_false(isTRUE(all.equal(
    attr(fit, "rho_ccc_intra_method1_se", exact = TRUE)[1, 2],
    attr(fit, "rho_ccc_intra_method2_se", exact = TRUE)[1, 2]
  )))
})

test_that("ccc_glmm Pearson overdispersion CIs warn and compute", {
  df <- make_ccc_glmm_data(n_subjects = 18, reps = 2, seed = 25)
  expect_warning(
    fit <- ccc_glmm(
      df,
      "y", "subject", "method",
      replicate = "replicate",
      overdispersion = "pearson",
      ci = TRUE
    ),
    "treat the Pearson dispersion estimate as fixed"
  )
  expect_true(is.finite(attr(fit, "rho_ccc_se", exact = TRUE)[1, 2]))
})

#' Pairwise Hilbert-Schmidt Independence Criterion
#'
#' Computes pairwise kernel dependence measures for the numeric columns of a
#' matrix or data frame using the Hilbert-Schmidt Independence Criterion
#' (HSIC). By default, `hsic()` returns a normalised kernel independence
#' correlation in `[0, 1]`, analogous to a distance-correlation matrix.
#'
#' @param data A numeric matrix or data frame with at least two numeric columns.
#'   Non-numeric columns are dropped.
#' @param kernel Kernel used to build univariate Gram matrices. Supported
#'   options are `"gaussian"` (RBF), `"linear"`, `"laplace"`, and
#'   `"polynomial"`.
#' @param bandwidth Bandwidth rule for Gaussian and Laplace kernels.
#'   `"median"` uses the upper median of finite off-diagonal pairwise
#'   distances divided by \eqn{\sqrt{2}}, with fallback `0.001` for degenerate
#'   columns. `"silverman"` uses Silverman's univariate rule, and `"scott"`
#'   uses Scott's univariate rule. Linear and polynomial kernels ignore this
#'   argument.
#' @param normalise Logical. If `TRUE`, return the normalised kernel
#'   independence correlation. If `FALSE`, return raw HSIC.
#' @param estimator HSIC estimator. `"biased"` uses the stable V-statistic
#'   \eqn{n^{-2}\mathrm{tr}(KHLH)}. `"unbiased"` uses the U-statistic estimator
#'   with zero diagonal Gram matrices and requires at least four complete
#'   observations per pair.
#' @param na_method Missing-data handling. `"error"` rejects missing, `NaN`, and
#'   infinite values; `"complete"` performs one listwise finite-row deletion;
#'   `"pairwise"` recomputes each pair on its pairwise complete rows.
#' @param p_value Logical. If `TRUE`, attach permutation p-values for the HSIC
#'   independence test.
#' @param B Positive integer. Number of permutations used when
#'   `p_value = TRUE`.
#' @param seed Optional positive integer seed for reproducible permutation
#'   inference. The seed is passed to the C++ permutation engine and does not
#'   mutate the user's global R RNG state.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   `getOption("matrixCorr.threads", 1L)`.
#' @param output Output representation: `"matrix"`, `"sparse"`, or
#'   `"edge_list"`.
#' @param threshold Non-negative absolute-value filter for non-matrix outputs.
#'   Must be `0` when `output = "matrix"`.
#' @param diag Logical; whether to include diagonal entries in sparse and
#'   edge-list outputs.
#'
#' @details
#' For paired observations \eqn{(x_i, y_i)}, HSIC measures the squared
#' Hilbert-Schmidt norm of the RKHS cross-covariance operator. With centred Gram
#' matrices \eqn{K_c = HKH} and \eqn{L_c = HLH}, the biased empirical estimator
#' is
#' \deqn{
#'   \widehat{\mathrm{HSIC}}_b =
#'   \frac{1}{n^2}\mathrm{tr}(KHLH)
#'   =
#'   \frac{1}{n^2}\sum_{ij} (K_c)_{ij}(L_c)_{ij}.
#' }
#'
#' With `normalise = TRUE`, the returned matrix contains the kernel
#' independence correlation
#' \deqn{
#'   \widehat{\mathrm{kCor}}(X,Y) =
#'   \frac{\widehat{\mathrm{HSIC}}(X,Y)}
#'        {\sqrt{\widehat{\mathrm{HSIC}}(X,X)
#'               \widehat{\mathrm{HSIC}}(Y,Y)}}.
#' }
#' The raw HSIC matrix is stored in `attr(x, "hsic_raw")`. When
#' `estimator = "unbiased"`, raw HSIC can be slightly negative in finite
#' samples; these signed raw values are retained in `hsic_raw`, while the
#' displayed normalised coefficient is clipped only for the user-facing matrix.
#'
#' Kernel formulas are:
#' Gaussian \eqn{\exp(-|x_i - x_j|^2 / (2\sigma^2))}, Laplace
#' \eqn{\exp(-|x_i - x_j| / \sigma)}, linear \eqn{x_i x_j}, and polynomial
#' \eqn{(x_i x_j + 1)^2}. For `bandwidth = "median"`, \eqn{\sigma} is the
#' upper median of \eqn{\{|x_i - x_j|: i < j\}} divided by \eqn{\sqrt{2}}; if
#' this is non-finite or non-positive, \eqn{\sigma = 0.001}.
#'
#' For `p_value = TRUE`, the test statistic is the raw HSIC estimate. The null
#' distribution is generated by permuting one variable within each pair, and the
#' p-value is \eqn{(1 + \#\{T_b \geq T_{\mathrm{obs}}\})/(B + 1)}. This is an
#' \eqn{O(B n^2)} pairwise procedure and can be expensive for large matrices or
#' large `B`.
#'
#' The polynomial kernel currently uses the fixed form
#' \eqn{(x_i x_j + 1)^2}. More polynomial controls can be added later without
#' changing the main HSIC object contract.
#'
#' @return A matrix-like correlation result. With `normalise = TRUE`, entries
#'   are normalised kernel independence correlations and the diagonal is `1`
#'   when the variable has positive kernel self-dependence. With
#'   `normalise = FALSE`, entries are raw HSIC estimates. Attributes record the
#'   kernel, bandwidth rule, estimator, raw HSIC matrix, diagnostics, and
#'   optional permutation inference.
#'
#' @references
#' Gretton, A., Bousquet, O., Smola, A., & Schoelkopf, B. (2005).
#' Measuring statistical dependence with Hilbert-Schmidt norms.
#' \emph{Algorithmic Learning Theory}.
#'
#' Gretton, A., Fukumizu, K., Teo, C. H., Song, L., Schoelkopf, B., & Smola,
#' A. J. (2007). A kernel statistical test of independence.
#' \emph{Advances in Neural Information Processing Systems}.
#'
#' Gretton, A., Bousquet, O., Smola, A., & Schoelkopf, B. (2005).
#' Kernel methods for measuring independence.
#' \emph{Journal of Machine Learning Research}, 6, 2075-2129.
#'
#' Pfister, N., Buhlmann, P., Scholkopf, B., & Peters, J. (2018).
#' Kernel-based tests for joint independence.
#' \emph{Journal of the Royal Statistical Society: Series B}, 80(1), 5-31.
#'
#' @examples
#' set.seed(1)
#' x <- rnorm(200)
#' y <- rnorm(200)
#' H0 <- hsic(cbind(x = x, y = y))
#' H0
#'
#' set.seed(2)
#' x <- rnorm(200)
#' y <- x^2 + rnorm(200, sd = 0.1)
#' H1 <- hsic(cbind(x = x, y = y))
#' H1
#'
#' set.seed(3)
#' X <- cbind(a = rnorm(80), b = rnorm(80), c = rnorm(80))
#' H_perm <- hsic(X, p_value = TRUE, B = 19, seed = 1)
#' summary(H_perm)
#' estimate(H_perm)
#' tidy(H_perm)
#'
#' @author Thiago de Paula Oliveira
#' @export
hsic <- function(data,
                 kernel = c("gaussian", "linear", "laplace", "polynomial"),
                 bandwidth = c("median", "silverman", "scott"),
                 normalise = TRUE,
                 estimator = c("biased", "unbiased"),
                 na_method = c("error", "pairwise", "complete"),
                 p_value = FALSE,
                 B = 499L,
                 seed = NULL,
                 n_threads = getOption("matrixCorr.threads", 1L),
                 output = c("matrix", "sparse", "edge_list"),
                 threshold = 0,
                 diag = TRUE) {
  if (
    is.matrix(data) &&
      is.numeric(data) &&
      (missing(kernel) || identical(kernel, "gaussian")) &&
      (missing(bandwidth) || identical(bandwidth, "median")) &&
      (missing(estimator) || identical(estimator, "biased")) &&
      (missing(na_method) || identical(na_method, "error")) &&
      !isTRUE(normalise) &&
      !isTRUE(p_value) &&
      (missing(output) || identical(output, "matrix")) &&
      isTRUE(threshold == 0) &&
      isTRUE(diag)
  ) {
    numeric_data <- validate_corr_input(data, check_na = TRUE)
    colnames_data <- colnames(numeric_data)
    if (is.null(colnames_data)) {
      colnames_data <- paste0("V", seq_len(ncol(numeric_data)))
    }
    return(hsic_raw_biased_matrix_object_cpp(
      numeric_data,
      kernel_code = 0L,
      bandwidth_code = 0L,
      kernel = "gaussian",
      bandwidth = "median",
      names = colnames_data
    ))
  }

  kernel <- match_arg(
    kernel,
    values = c("gaussian", "linear", "laplace", "polynomial"),
    arg_name = "kernel"
  )
  bandwidth <- match_arg(
    bandwidth,
    values = c("median", "silverman", "scott"),
    arg_name = "bandwidth"
  )
  estimator <- match_arg(
    estimator,
    values = c("biased", "unbiased"),
    arg_name = "estimator"
  )
  na_method <- match_arg(
    na_method,
    values = c("error", "pairwise", "complete"),
    arg_name = "na_method"
  )
  check_bool(normalise, arg = "normalise")
  check_bool(p_value, arg = "p_value")
  if (isTRUE(p_value)) {
    B <- check_scalar_int_pos(B, arg = "B")
    if (!is.null(seed)) {
      seed <- check_scalar_int_pos(seed, arg = "seed")
    }
  }
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )

  kernel_code <- match(kernel, c("gaussian", "linear", "laplace", "polynomial")) - 1L
  bandwidth_code <- match(bandwidth, c("median", "silverman", "scott")) - 1L
  estimator_code <- match(estimator, c("biased", "unbiased")) - 1L

  if (
    identical(na_method, "error") &&
      !isTRUE(normalise) &&
      identical(estimator, "biased") &&
      !isTRUE(p_value) &&
      identical(output_cfg$output, "matrix") &&
      identical(output_cfg$threshold, 0) &&
      isTRUE(output_cfg$diag) &&
      is.matrix(data) &&
      is.numeric(data) &&
      nrow(data) >= 2L &&
      ncol(data) >= 2L
  ) {
    numeric_data <- validate_corr_input(data, check_na = TRUE)
    colnames_data <- colnames(numeric_data)
    if (is.null(colnames_data)) {
      colnames_data <- paste0("V", seq_len(ncol(numeric_data)))
    }
    return(hsic_raw_biased_matrix_object_cpp(
      numeric_data,
      kernel_code = kernel_code,
      bandwidth_code = bandwidth_code,
      kernel = kernel,
      bandwidth = bandwidth,
      names = colnames_data
    ))
  }

  numeric_data <- validate_corr_input(
    data,
    check_na = identical(na_method, "error")
  )
  min_n <- if (identical(estimator, "unbiased")) 4L else 2L
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = min_n, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }

  colnames_data <- colnames(numeric_data)
  dn <- .mc_square_dimnames(colnames_data)

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  pairwise_needed <- isTRUE(p_value) || identical(na_method, "pairwise") || isTRUE(normalise)
  if (isTRUE(pairwise_needed)) {
    pairwise <- hsic_matrix_pairwise_cpp(
      numeric_data,
      kernel_code = kernel_code,
      bandwidth_code = bandwidth_code,
      normalise = isTRUE(normalise),
      estimator_code = estimator_code,
      return_inference = isTRUE(p_value),
      B = if (isTRUE(p_value)) as.integer(B) else 1L,
      seed = if (is.null(seed)) NULL else as.integer(seed)
    )
    hsic_matrix <- pairwise$est
    raw_hsic <- pairwise$raw
    diagnostics <- list(
      n_complete = .mc_set_matrix_dimnames(unclass(pairwise$n_complete), colnames_data)
    )
    inference_attr <- if (isTRUE(p_value)) {
      list(
        method = "hsic_permutation_test",
        statistic = .mc_set_matrix_dimnames(unclass(pairwise$raw), colnames_data),
        p_value = .mc_set_matrix_dimnames(unclass(pairwise$p_value), colnames_data),
        B = as.integer(B),
        alternative = "greater"
      )
    } else {
      NULL
    }
  } else {
    hsic_matrix <- hsic_matrix_cpp(
      numeric_data,
      kernel_code = kernel_code,
      bandwidth_code = bandwidth_code,
      normalise = FALSE,
      estimator_code = estimator_code
    )
    raw_hsic <- hsic_matrix
    diagnostics <- NULL
    inference_attr <- NULL
  }

  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)
  hsic_matrix <- .mc_set_matrix_dimnames(unclass(hsic_matrix), colnames_data)
  raw_hsic <- .mc_set_matrix_dimnames(unclass(raw_hsic), colnames_data)

  out <- .mc_structure_corr_matrix(
    hsic_matrix,
    class_name = "hsic",
    method = if (isTRUE(normalise)) {
      "kernel_independence_correlation"
    } else {
      "hsic"
    },
    description = if (isTRUE(normalise)) {
      "Pairwise normalised Hilbert-Schmidt independence criterion"
    } else {
      "Pairwise raw Hilbert-Schmidt independence criterion"
    },
    symmetric = TRUE,
    diagnostics = diagnostics,
    dimnames = dn,
    classes = c("hsic", "matrixCorr_dependence", "matrix"),
    extra_attrs = c(
      list(
        kernel = kernel,
        bandwidth = bandwidth,
        estimator = estimator,
        normalise = isTRUE(normalise),
        hsic_raw = raw_hsic
      ),
      if (!is.null(inference_attr)) list(inference = inference_attr) else list()
    )
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_hsic_inference_attr <- function(x) {
  attr(x, "inference", exact = TRUE)
}

.mc_hsic_pairwise_summary <- function(object,
                                      digits = 4,
                                      p_digits = 4) {
  check_inherits(object, "hsic")
  est <- as.matrix(object)
  rn <- rownames(est)
  cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est)))

  inf <- .mc_hsic_inference_attr(object)
  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  raw <- attr(object, "hsic_raw", exact = TRUE)

  n_pairs <- nrow(est) * (ncol(est) - 1L) / 2L
  var1 <- character(n_pairs)
  var2 <- character(n_pairs)
  estimate <- numeric(n_pairs)
  hsic_raw <- numeric(n_pairs)
  n_complete <- if (is.list(diag_attr) && is.matrix(diag_attr$n_complete)) integer(n_pairs) else NULL
  p_value <- if (is.list(inf)) numeric(n_pairs) else NULL

  k <- 0L
  for (i in seq_len(nrow(est) - 1L)) {
    for (j in (i + 1L):ncol(est)) {
      k <- k + 1L
      var1[k] <- rn[i]
      var2[k] <- cn[j]
      estimate[k] <- round(est[i, j], digits)
      hsic_raw[k] <- if (is.matrix(raw)) round(raw[i, j], digits) else NA_real_
      if (!is.null(n_complete)) n_complete[k] <- as.integer(diag_attr$n_complete[i, j])
      if (!is.null(p_value)) {
        p_value[k] <- if (is.matrix(inf$p_value) && is.finite(inf$p_value[i, j])) {
          round(inf$p_value[i, j], p_digits)
        } else {
          NA_real_
        }
      }
    }
  }

  df <- data.frame(
    var1 = var1,
    var2 = var2,
    estimate = as.numeric(estimate),
    hsic_raw = as.numeric(hsic_raw),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (!is.null(n_complete)) df$n_complete <- as.integer(n_complete)
  if (!is.null(p_value)) df$p_value <- as.numeric(p_value)
  rownames(df) <- NULL

  out <- .mc_finalize_summary_df(df, class_name = "summary.hsic")
  attr(out, "overview") <- .mc_summary_corr_matrix(object)
  attr(out, "has_p") <- is.list(inf) && is.matrix(inf$p_value)
  attr(out, "digits") <- digits
  attr(out, "p_digits") <- p_digits
  attr(out, "inference_method") <- if (isTRUE(attr(out, "has_p", exact = TRUE))) inf$method %||% NA_character_ else NA_character_
  out
}

#' @rdname hsic
#' @method print hsic
#' @param x An object of class `hsic`.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of `"yes"` or `"no"`; accepted for compatibility with
#'   shared matrix-printing methods.
#' @param ... Additional arguments passed to print or plot methods.
#' @export
print.hsic <- function(x, digits = 4, n = NULL, topn = NULL,
                       max_vars = NULL, width = NULL, show_ci = NULL, ...) {
  .mc_print_corr_matrix(
    x,
    header = if (isTRUE(attr(x, "normalise", exact = TRUE))) {
      "Kernel independence correlation matrix"
    } else {
      "HSIC matrix"
    },
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

#' @rdname hsic
#' @method summary hsic
#' @param object An object of class `hsic`.
#' @export
summary.hsic <- function(object, n = NULL, topn = NULL,
                         max_vars = NULL, width = NULL,
                         show_ci = NULL, ...) {
  check_inherits(object, "hsic")
  inf <- .mc_hsic_inference_attr(object)
  diagnostics <- attr(object, "diagnostics", exact = TRUE)
  if (!is.list(inf) && !(is.list(diagnostics) && is.matrix(diagnostics$n_complete))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_hsic_pairwise_summary(object)
}

#' @rdname hsic
#' @method print summary.hsic
#' @export
print.summary.hsic <- function(x, digits = NULL, n = NULL,
                               topn = NULL, max_vars = NULL,
                               width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "HSIC summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    extra_items = c(inference = attr(x, "inference_method", exact = TRUE)),
    ...
  )
  invisible(x)
}

#' @rdname hsic
#' @method plot hsic
#' @param title Optional plot title.
#' @param low_color Colour for low HSIC values.
#' @param high_color Colour for high HSIC values.
#' @param value_text_size Font size for tile labels.
#' @param show_value Logical; whether to overlay numeric values on the heatmap.
#' @export
plot.hsic <- function(x,
                      title = NULL,
                      low_color = "white",
                      high_color = "steelblue1",
                      value_text_size = 4,
                      show_value = TRUE,
                      ...) {
  check_inherits(x, "hsic")
  check_bool(show_value, arg = "show_value")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  mat <- as.matrix(x)
  df <- as.data.frame(as.table(mat))
  colnames(df) <- c("Var1", "Var2", "HSIC")
  df$Var1 <- factor(df$Var1, levels = rev(unique(df$Var1)))

  p <- ggplot2::ggplot(df, ggplot2::aes(.data$Var2, .data$Var1, fill = .data$HSIC)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient(
      low = low_color, high = high_color,
      limits = if (isTRUE(attr(x, "normalise", exact = TRUE))) c(0, 1) else NULL,
      name = if (isTRUE(attr(x, "normalise", exact = TRUE))) "kCor" else "HSIC"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(
      title = title %||% if (isTRUE(attr(x, "normalise", exact = TRUE))) {
        "Kernel independence correlation heatmap"
      } else {
        "HSIC heatmap"
      },
      x = NULL,
      y = NULL
    )

  if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", .data$HSIC)),
      size = value_text_size,
      color = "black"
    )
  }

  p
}

upper_median_bandwidth_ref <- function(x) {
  D <- abs(outer(x, x, "-"))
  d <- sort(as.numeric(D[upper.tri(D, diag = FALSE)]))

  if (!length(d)) {
    return(0.001)
  }

  sigma <- d[floor(length(d) / 2) + 1L] / sqrt(2)

  if (!is.finite(sigma) || sigma <= 0) {
    return(0.001)
  }

  sigma
}

silverman_bandwidth_ref <- function(x) {
  n <- length(x)
  s <- stats::sd(x)
  if (!is.finite(s) || s <= 1e-12) {
    return(1.0)
  }
  max(1.06 * s * n^(-1 / 5), 1e-12)
}

scott_bandwidth_ref <- function(x) {
  n <- length(x)
  s <- stats::sd(x)
  if (!is.finite(s) || s <= 1e-12) {
    return(1.0)
  }
  max(s * n^(-1 / 5), 1e-12)
}

bandwidth_ref <- function(x, bandwidth) {
  switch(
    bandwidth,
    median = upper_median_bandwidth_ref(x),
    silverman = silverman_bandwidth_ref(x),
    scott = scott_bandwidth_ref(x),
    stop("unsupported bandwidth")
  )
}

gram_ref <- function(x,
                     kernel = c("gaussian", "linear", "laplace", "polynomial"),
                     bandwidth = c("median", "silverman", "scott")) {
  kernel <- match.arg(kernel)
  bandwidth <- match.arg(bandwidth)

  if (identical(kernel, "linear")) {
    return(outer(x, x, "*"))
  }

  if (identical(kernel, "polynomial")) {
    return((outer(x, x, "*") + 1)^2)
  }

  bw <- bandwidth_ref(x, bandwidth)
  D <- abs(outer(x, x, "-"))

  if (identical(kernel, "gaussian")) {
    return(exp(-(D^2) / (2 * bw^2)))
  }

  if (identical(kernel, "laplace")) {
    return(exp(-D / bw))
  }

  stop("unsupported kernel")
}

center_ref <- function(K) {
  n <- nrow(K)
  H <- diag(n) - matrix(1 / n, n, n)
  H %*% K %*% H
}

hsic_biased_ref <- function(x, y, kernel, bandwidth) {
  K <- gram_ref(x, kernel = kernel, bandwidth = bandwidth)
  L <- gram_ref(y, kernel = kernel, bandwidth = bandwidth)
  Kc <- center_ref(K)
  Lc <- center_ref(L)
  sum(Kc * Lc) / length(x)^2
}

hsic_kcor_ref <- function(x, y, kernel, bandwidth) {
  K <- gram_ref(x, kernel = kernel, bandwidth = bandwidth)
  L <- gram_ref(y, kernel = kernel, bandwidth = bandwidth)

  Kc <- center_ref(K)
  Lc <- center_ref(L)

  xy <- sum(Kc * Lc)
  xx <- sum(Kc * Kc)
  yy <- sum(Lc * Lc)

  if (
    !is.finite(xy) || !is.finite(xx) || !is.finite(yy) ||
      xx <= 1e-12 || yy <= 1e-12
  ) {
    return(NA_real_)
  }

  out <- xy / sqrt(xx * yy)
  max(0, min(1, out))
}

hsic_unbiased_ref <- function(x, y, kernel, bandwidth) {
  n <- length(x)
  K <- gram_ref(x, kernel = kernel, bandwidth = bandwidth)
  L <- gram_ref(y, kernel = kernel, bandwidth = bandwidth)

  diag(K) <- 0
  diag(L) <- 0

  rK <- rowSums(K)
  rL <- rowSums(L)

  (
    sum(K * L) +
      sum(K) * sum(L) / ((n - 1) * (n - 2)) -
      2 * sum(rK * rL) / (n - 2)
  ) / (n * (n - 3))
}

hsic_math_scenario <- function(name, n) {
  set.seed(
    switch(
      name,
      independent_normal = 101L,
      linear_dependence = 102L,
      quadratic_dependence = 103L,
      duplicate_columns = 104L,
      duplicated_observations = 105L,
      constant_column = 106L
    ) + n
  )

  if (identical(name, "independent_normal")) {
    return(list(x = rnorm(n), y = rnorm(n)))
  }
  if (identical(name, "linear_dependence")) {
    x <- rnorm(n)
    return(list(x = x, y = 0.8 * x + rnorm(n, sd = 0.2)))
  }
  if (identical(name, "quadratic_dependence")) {
    x <- rnorm(n)
    return(list(x = x, y = x^2 + rnorm(n, sd = 0.1)))
  }
  if (identical(name, "duplicate_columns")) {
    x <- rnorm(n)
    return(list(x = x, y = x))
  }
  if (identical(name, "duplicated_observations")) {
    x <- as.numeric(sample(c(-1, 0, 1), n, replace = TRUE))
    return(list(x = x, y = x + rnorm(n, sd = 0.1)))
  }
  if (identical(name, "constant_column")) {
    return(list(x = rnorm(n), y = rep(1, n)))
  }

  stop("unknown scenario")
}

test_that("hsic matches direct biased and normalised mathematical references", {
  kernels <- c("gaussian", "linear", "laplace", "polynomial")
  bandwidths <- c("median", "silverman", "scott")
  scenarios <- c(
    "independent_normal",
    "linear_dependence",
    "quadratic_dependence",
    "duplicate_columns",
    "duplicated_observations",
    "constant_column"
  )

  for (scenario in scenarios) {
    for (n in c(5L, 100L)) {
      dat <- hsic_math_scenario(scenario, n)
      X <- cbind(x = dat$x, y = dat$y)

      for (kernel in kernels) {
        for (bandwidth in bandwidths) {
          raw <- hsic(
            X,
            kernel = kernel,
            bandwidth = bandwidth,
            estimator = "biased",
            normalise = FALSE
          )
          norm <- hsic(
            X,
            kernel = kernel,
            bandwidth = bandwidth,
            estimator = "biased",
            normalise = TRUE
          )

          expect_equal(
            as.numeric(raw[1, 2]),
            hsic_biased_ref(dat$x, dat$y, kernel, bandwidth),
            tolerance = 1e-10,
            info = paste(scenario, n, kernel, bandwidth, "raw")
          )
          expect_equal(
            as.numeric(norm[1, 2]),
            hsic_kcor_ref(dat$x, dat$y, kernel, bandwidth),
            tolerance = 1e-10,
            info = paste(scenario, n, kernel, bandwidth, "normalised")
          )
        }
      }
    }
  }
})

test_that("hsic unbiased estimator matches direct mathematical reference", {
  kernels <- c("gaussian", "linear", "laplace", "polynomial")
  bandwidths <- c("median", "silverman", "scott")
  dat <- hsic_math_scenario("quadratic_dependence", 20L)
  X <- cbind(x = dat$x, y = dat$y)

  for (kernel in kernels) {
    for (bandwidth in bandwidths) {
      raw <- hsic(
        X,
        kernel = kernel,
        bandwidth = bandwidth,
        estimator = "unbiased",
        normalise = FALSE
      )
      expect_equal(
        as.numeric(raw[1, 2]),
        hsic_unbiased_ref(dat$x, dat$y, kernel, bandwidth),
        tolerance = 1e-10,
        info = paste(kernel, bandwidth)
      )
    }
  }
})

test_that("linear and polynomial HSIC ignore bandwidth", {
  dat <- hsic_math_scenario("linear_dependence", 50L)
  X <- cbind(x = dat$x, y = dat$y)

  for (kernel in c("linear", "polynomial")) {
    vals <- vapply(
      c("median", "silverman", "scott"),
      function(bandwidth) {
        as.numeric(hsic(X, kernel = kernel, bandwidth = bandwidth, normalise = FALSE)[1, 2])
      },
      numeric(1)
    )
    expect_equal(unname(vals), rep(vals[[1]], length(vals)), tolerance = 1e-12)
  }
})

test_that("gaussian and laplace HSIC respond to bandwidth", {
  dat <- hsic_math_scenario("linear_dependence", 100L)
  X <- cbind(x = dat$x, y = dat$y)

  for (kernel in c("gaussian", "laplace")) {
    vals <- vapply(
      c("median", "silverman", "scott"),
      function(bandwidth) {
        as.numeric(hsic(X, kernel = kernel, bandwidth = bandwidth, normalise = FALSE)[1, 2])
      },
      numeric(1)
    )
    expect_true(length(unique(round(vals, 14))) > 1)
  }
})

test_that("hsic seed does not alter global R RNG state", {
  set.seed(42)
  before <- .Random.seed
  X <- cbind(x = rnorm(30), y = rnorm(30))
  set.seed(42)
  before <- .Random.seed
  invisible(hsic(X, p_value = TRUE, B = 19L, seed = 10L))
  after <- .Random.seed
  expect_identical(after, before)
})

hsic_gaussian_raw_R <- function(x, y) {
  med_bw <- function(z) {
    D <- abs(outer(z, z, "-"))
    d <- sort(as.numeric(D[upper.tri(D, diag = FALSE)]))
    d <- d[is.finite(d)]
    if (!length(d)) return(0.001)
    sigma <- d[floor(length(d) / 2) + 1L] / sqrt(2)
    if (!is.finite(sigma) || sigma <= 0) return(0.001)
    sigma
  }
  gram <- function(z) {
    bw <- med_bw(z)
    D <- outer(z, z, "-")
    exp(-(D^2) / (2 * bw^2))
  }
  n <- length(x)
  H <- diag(n) - matrix(1 / n, n, n)
  Kc <- H %*% gram(x) %*% H
  Lc <- H %*% gram(y) %*% H
  sum(Kc * Lc) / n^2
}

test_that("hsic detects independence and non-linear dependence", {
  set.seed(1)
  x0 <- rnorm(200)
  y0 <- rnorm(200)
  H0 <- hsic(cbind(x = x0, y = y0))
  expect_s3_class(H0, "hsic")
  expect_lt(H0[1, 2], 0.10)

  set.seed(2)
  x1 <- rnorm(200)
  y1 <- x1^2 + rnorm(200, sd = 0.1)
  H1 <- hsic(cbind(x = x1, y = y1))
  expect_gt(H1[1, 2], 0.20)
})

test_that("hsic output is symmetric with expected diagonal", {
  set.seed(3)
  X <- matrix(rnorm(120), ncol = 3)
  colnames(X) <- c("a", "b", "c")
  H <- hsic(X)

  expect_equal(unname(diag(H)), rep(1, ncol(X)))
  expect_equal(H, t(H), tolerance = 1e-12)
  expect_equal(dimnames(H), list(colnames(X), colnames(X)))
  expect_true(is.matrix(attr(H, "hsic_raw")))
})

test_that("biased HSIC matches a direct R implementation", {
  x <- c(-1.2, -0.4, 0.1, 0.9, 1.4)
  y <- c(0.2, 0.5, -0.1, 1.0, 1.8)
  H <- hsic(cbind(x = x, y = y), normalise = FALSE, estimator = "biased")

  expect_equal(
    as.numeric(H[1, 2]),
    hsic_gaussian_raw_R(x, y),
    tolerance = 1e-12
  )
})

test_that("unbiased HSIC retains signed raw estimates", {
  set.seed(4)
  X <- cbind(a = rnorm(30), b = rnorm(30), c = rnorm(30))
  H <- hsic(X, estimator = "unbiased")
  raw <- attr(H, "hsic_raw")

  expect_true(is.matrix(raw))
  expect_equal(dim(raw), dim(H))
  expect_true(all(is.na(H[upper.tri(H)]) | (H[upper.tri(H)] >= 0 & H[upper.tri(H)] <= 1)))
})

test_that("hsic missing-data modes follow dcor-style handling", {
  X <- cbind(
    a = c(1, 2, 3, 4, 5, 6),
    b = c(1, NA, 2, 4, 5, 6),
    c = c(6, 5, 4, 3, 2, 1)
  )

  expect_error(hsic(X, na_method = "error"), "missing|finite|NA|NaN|infinite")

  H_complete <- hsic(X, na_method = "complete")
  expect_s3_class(H_complete, "hsic")
  expect_equal(attr(H_complete, "diagnostics")$n_complete, 5L)

  H_pairwise <- hsic(X, na_method = "pairwise")
  n_complete <- attr(H_pairwise, "diagnostics")$n_complete
  expect_equal(n_complete["a", "b"], 5)
  expect_equal(n_complete["a", "c"], 6)
})

test_that("hsic permutation inference is reproducible and stored as metadata", {
  set.seed(5)
  x <- rnorm(60)
  y <- x^2 + rnorm(60, sd = 0.2)
  H1 <- hsic(cbind(x = x, y = y), p_value = TRUE, B = 19L, seed = 11L)
  H2 <- hsic(cbind(x = x, y = y), p_value = TRUE, B = 19L, seed = 11L)

  inf1 <- attr(H1, "inference")
  inf2 <- attr(H2, "inference")
  expect_equal(inf1$p_value, inf2$p_value)
  expect_equal(inf1$B, 19L)
  expect_equal(inf1$alternative, "greater")
  expect_true(is.finite(inf1$p_value[1, 2]))
})

test_that("hsic supports sparse and edge-list output conversion", {
  set.seed(6)
  X <- matrix(rnorm(80), ncol = 4)
  colnames(X) <- paste0("v", 1:4)

  sp <- hsic(X, output = "sparse", threshold = 0, diag = FALSE)
  ed <- hsic(X, output = "edge_list", threshold = 0, diag = FALSE)

  expect_s4_class(sp, "sparseMatrix")
  expect_equal(attr(sp, "corr_output_class"), "corr_sparse")
  expect_s3_class(ed, "corr_edge_list")
  expect_true(all(c("row", "col", "value") %in% names(ed)))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bicor_matrix_cpp <- function(X, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L, n_threads = 1L) {
    .Call(`_matrixCorr_bicor_matrix_cpp`, X, c_const, maxPOutliers, pearson_fallback, n_threads)
}

bicor_vec_cpp <- function(x, y, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L) {
    .Call(`_matrixCorr_bicor_vec_cpp`, x, y, c_const, maxPOutliers, pearson_fallback)
}

bicor_matrix_pairwise_cpp <- function(X, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L, min_n = 5L, n_threads = 1L) {
    .Call(`_matrixCorr_bicor_matrix_pairwise_cpp`, X, c_const, maxPOutliers, pearson_fallback, min_n, n_threads)
}

bicor_threshold_triplets_cpp <- function(X, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_bicor_threshold_triplets_cpp`, X, c_const, maxPOutliers, pearson_fallback, threshold, diag, block_size, n_threads)
}

bicor_matrix_weighted_cpp <- function(X, w, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L, n_threads = 1L) {
    .Call(`_matrixCorr_bicor_matrix_weighted_cpp`, X, w, c_const, maxPOutliers, pearson_fallback, n_threads)
}

bicor_matrix_weighted_pairwise_cpp <- function(X, w, c_const = 9.0, maxPOutliers = 1.0, pearson_fallback = 1L, min_n = 5L, n_threads = 1L) {
    .Call(`_matrixCorr_bicor_matrix_weighted_pairwise_cpp`, X, w, c_const, maxPOutliers, pearson_fallback, min_n, n_threads)
}

bland_altman_cpp <- function(group1, group2, loa_multiplier = 1.96, mode = 1L, conf_level = 0.95, n_threads = 1L, include_legacy_payload = TRUE) {
    .Call(`_matrixCorr_bland_altman_cpp`, group1, group2, loa_multiplier, mode, conf_level, n_threads, include_legacy_payload)
}

ba_openmp_threads <- function() {
    .Call(`_matrixCorr_ba_openmp_threads`)
}

ba_rm_slope_scale_cpp <- function(mean_values) {
    .Call(`_matrixCorr_ba_rm_slope_scale_cpp`, mean_values)
}

ba_rm_complete_pairs_cpp <- function(y, subject, method, time) {
    .Call(`_matrixCorr_ba_rm_complete_pairs_cpp`, y, subject, method, time)
}

bland_altman_repeated_em_ext_cpp <- function(y, subject, method, time, include_slope = FALSE, use_ar1 = FALSE, ar1_rho = NA_real_, max_iter = 200L, tol = 1e-6, conf_level = 0.95, loa_multiplier_arg = NA_real_, use_cov_su_se = TRUE, n_threads = 1L) {
    .Call(`_matrixCorr_bland_altman_repeated_em_ext_cpp`, y, subject, method, time, include_slope, use_ar1, ar1_rho, max_iter, tol, conf_level, loa_multiplier_arg, use_cov_su_se, n_threads)
}

ccc_glmm_poisson_prepare_blocks_cpp <- function(y, subject, method_code, n_subjects) {
    .Call(`_matrixCorr_ccc_glmm_poisson_prepare_blocks_cpp`, y, subject, method_code, n_subjects)
}

ccc_glmm_poisson_ghq_nll_blocks_cpp <- function(par, y1, y2, n1, n2, log_factorial, include_subject_method, gh_nodes, gh_weights) {
    .Call(`_matrixCorr_ccc_glmm_poisson_ghq_nll_blocks_cpp`, par, y1, y2, n1, n2, log_factorial, include_subject_method, gh_nodes, gh_weights)
}

ccc_glmm_poisson_ghq_nll_cpp <- function(par, y, subject, method_code, n_subjects, include_subject_method, gh_nodes, gh_weights) {
    .Call(`_matrixCorr_ccc_glmm_poisson_ghq_nll_cpp`, par, y, subject, method_code, n_subjects, include_subject_method, gh_nodes, gh_weights)
}

cccUst_rcpp <- function(y_vec, met_vec, time_vec, subj_vec, nmet0, nmet1, ntime, ns, Dmat, delta, cl) {
    .Call(`_matrixCorr_cccUst_rcpp`, y_vec, met_vec, time_vec, subj_vec, nmet0, nmet1, ntime, ns, Dmat, delta, cl)
}

set_omp_threads <- function(n) {
    invisible(.Call(`_matrixCorr_set_omp_threads`, n))
}

get_omp_threads <- function() {
    .Call(`_matrixCorr_get_omp_threads`)
}

build_L_Dm_cpp <- function(colnames_X, rmet_name, rtime_name, method_levels, time_levels, has_interaction, Dmat_global) {
    .Call(`_matrixCorr_build_L_Dm_cpp`, colnames_X, rmet_name, rtime_name, method_levels, time_levels, has_interaction, Dmat_global)
}

build_L_Dm_Z_cpp <- function(colnames_X, rmet_name, rtime_name, method_levels, time_levels, has_interaction, Dmat_global, slope_mode, slope_var, method_codes, drop_zero_cols) {
    .Call(`_matrixCorr_build_L_Dm_Z_cpp`, colnames_X, rmet_name, rtime_name, method_levels, time_levels, has_interaction, Dmat_global, slope_mode, slope_var, method_codes, drop_zero_cols)
}

ccc_vc_cpp <- function(Xr, yr, subject, method, time, nm, nt, max_iter = 200L, tol = 1e-6, conf_level = 0.95, ci_mode = 2L, Lr = NULL, auxDr = NULL, Zr = NULL, use_ar1 = FALSE, ar1_rho = 0.0, include_subj_method = TRUE, include_subj_time = TRUE, sb_zero_tol = 1e-10, eval_single_visit = FALSE, time_weights = NULL, metric_mode = 0L, ll_only = FALSE, need_loglik = TRUE) {
    .Call(`_matrixCorr_ccc_vc_cpp`, Xr, yr, subject, method, time, nm, nt, max_iter, tol, conf_level, ci_mode, Lr, auxDr, Zr, use_ar1, ar1_rho, include_subj_method, include_subj_time, sb_zero_tol, eval_single_visit, time_weights, metric_mode, ll_only, need_loglik)
}

chatterjee_xi_vec_cpp <- function(x, y, tie_method = "random", bias_correction = "none") {
    .Call(`_matrixCorr_chatterjee_xi_vec_cpp`, x, y, tie_method, bias_correction)
}

chatterjee_xi_matrix_cpp <- function(X, tie_method = "random", bias_correction = "none", n_threads = 1L) {
    .Call(`_matrixCorr_chatterjee_xi_matrix_cpp`, X, tie_method, bias_correction, n_threads)
}

chatterjee_xi_matrix_pairwise_cpp <- function(X_, return_ci = FALSE, conf_level = 0.95, ci_method = "auto", bootstrap_reps = 999L, m = NULL, large_sample_cutoff = 1000L, tie_method = "random", bias_correction = "none", n_threads = 1L) {
    .Call(`_matrixCorr_chatterjee_xi_matrix_pairwise_cpp`, X_, return_ci, conf_level, ci_method, bootstrap_reps, m, large_sample_cutoff, tie_method, bias_correction, n_threads)
}

cia_moments_cpp <- function(y, subject, method, replicate, n_methods, reference_method, has_reference, pairwise, n_threads = 1L) {
    .Call(`_matrixCorr_cia_moments_cpp`, y, subject, method, replicate, n_methods, reference_method, has_reference, pairwise, n_threads)
}

cia_pairwise_stats_cpp <- function(y, subject, method, replicate, n_methods, reference_method, has_reference, n_threads = 1L) {
    .Call(`_matrixCorr_cia_pairwise_stats_cpp`, y, subject, method, replicate, n_methods, reference_method, has_reference, n_threads)
}

cia_pairwise_bootstrap_est_cpp <- function(y, subject, method, replicate, n_methods, reference_method, has_reference, sampled_subjects, estimator, n_threads = 1L) {
    .Call(`_matrixCorr_cia_pairwise_bootstrap_est_cpp`, y, subject, method, replicate, n_methods, reference_method, has_reference, sampled_subjects, estimator, n_threads)
}

cia_overall_balanced_cpp <- function(y, subject, method, replicate, n_methods, reference_method, has_reference, n_threads = 1L) {
    .Call(`_matrixCorr_cia_overall_balanced_cpp`, y, subject, method, replicate, n_methods, reference_method, has_reference, n_threads)
}

cia_rm_anova_cpp <- function(y, subject, method, time, n_subjects, n_methods, n_times, homogeneous, constrain_vc, n_threads = 1L) {
    .Call(`_matrixCorr_cia_rm_anova_cpp`, y, subject, method, time, n_subjects, n_methods, n_times, homogeneous, constrain_vc, n_threads)
}

cia_rm_delta_cpp <- function(y, subject, method, time, n_subjects, n_methods, n_times, homogeneous, constrain_vc, conf_level, n_threads = 1L) {
    .Call(`_matrixCorr_cia_rm_delta_cpp`, y, subject, method, time, n_subjects, n_methods, n_times, homogeneous, constrain_vc, conf_level, n_threads)
}

cohen_kappa_pair_cpp <- function(x, y, n_levels, return_inference = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_cohen_kappa_pair_cpp`, x, y, n_levels, return_inference, conf_level)
}

cohen_kappa_matrix_cpp <- function(X, n_levels, pairwise_complete = FALSE, return_inference = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_cohen_kappa_matrix_cpp`, X, n_levels, pairwise_complete, return_inference, conf_level, n_threads)
}

cohen_kappa_threshold_triplets_cpp <- function(X, n_levels, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_cohen_kappa_threshold_triplets_cpp`, X, n_levels, threshold, diag, block_size, n_threads)
}

ccc_cpp <- function(X) {
    .Call(`_matrixCorr_ccc_cpp`, X)
}

ccc_with_ci_cpp <- function(X, conf_level = 0.95) {
    .Call(`_matrixCorr_ccc_with_ci_cpp`, X, conf_level)
}

openmp_threads <- function() {
    .Call(`_matrixCorr_openmp_threads`)
}

ccc_threshold_triplets_cpp <- function(X, threshold = 0.0, diag = TRUE, block_size = 256L) {
    .Call(`_matrixCorr_ccc_threshold_triplets_cpp`, X, threshold, diag, block_size)
}

ustat_dcor <- function(x, y) {
    .Call(`_matrixCorr_ustat_dcor`, x, y)
}

ustat_dcor_matrix_cpp <- function(X) {
    .Call(`_matrixCorr_ustat_dcor_matrix_cpp`, X)
}

ustat_dcor_matrix_pairwise_cpp <- function(X, return_inference = FALSE) {
    .Call(`_matrixCorr_ustat_dcor_matrix_pairwise_cpp`, X, return_inference)
}

robust_dcor_matrix_cpp <- function(X, c_const = 4.0, n_threads = 1L) {
    .Call(`_matrixCorr_robust_dcor_matrix_cpp`, X, c_const, n_threads)
}

robust_dcor_matrix_pairwise_cpp <- function(X, c_const = 4.0, return_inference = FALSE, n_perm = 999L, seed = NULL, n_threads = 1L) {
    .Call(`_matrixCorr_robust_dcor_matrix_pairwise_cpp`, X, c_const, return_inference, n_perm, seed, n_threads)
}

gwet_ac_pair_cpp <- function(x, y, weights, drop_unused_levels = FALSE, return_inference = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_gwet_ac_pair_cpp`, x, y, weights, drop_unused_levels, return_inference, conf_level)
}

gwet_ac_matrix_cpp <- function(X, weights, drop_unused_levels = FALSE, pairwise_complete = FALSE, return_inference = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_gwet_ac_matrix_cpp`, X, weights, drop_unused_levels, pairwise_complete, return_inference, conf_level, n_threads)
}

gwet_ac_threshold_triplets_cpp <- function(X, weights, drop_unused_levels = FALSE, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_gwet_ac_threshold_triplets_cpp`, X, weights, drop_unused_levels, threshold, diag, block_size, n_threads)
}

gwet_ac_counts_cpp <- function(counts, weights, by_category = FALSE, inference_code = 0L, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_gwet_ac_counts_cpp`, counts, weights, by_category, inference_code, conf_level, n_threads)
}

gwet_ac_ratings_cpp <- function(ratings, n_levels, weights, na_code = 1L, min_raters = 2L, by_category = FALSE, inference_code = 0L, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_gwet_ac_ratings_cpp`, ratings, n_levels, weights, na_code, min_raters, by_category, inference_code, conf_level, n_threads)
}

hsic_matrix_cpp <- function(X, kernel_code, bandwidth_code, normalise, estimator_code) {
    .Call(`_matrixCorr_hsic_matrix_cpp`, X, kernel_code, bandwidth_code, normalise, estimator_code)
}

hsic_raw_biased_matrix_object_cpp <- function(X, kernel_code, bandwidth_code, kernel, bandwidth, names) {
    .Call(`_matrixCorr_hsic_raw_biased_matrix_object_cpp`, X, kernel_code, bandwidth_code, kernel, bandwidth, names)
}

hsic_matrix_pairwise_cpp <- function(X, kernel_code, bandwidth_code, normalise, estimator_code, return_inference, B, seed = NULL) {
    .Call(`_matrixCorr_hsic_matrix_pairwise_cpp`, X, kernel_code, bandwidth_code, normalise, estimator_code, return_inference, B, seed)
}

icc_matrix_cpp <- function(X, form_code = 0L, average_unit = FALSE, pairwise_complete = FALSE, return_ci = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_icc_matrix_cpp`, X, form_code, average_unit, pairwise_complete, return_ci, conf_level, n_threads)
}

icc_overall_cpp <- function(X, return_ci = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_icc_overall_cpp`, X, return_ci, conf_level)
}

kendall_matrix_cpp <- function(mat) {
    .Call(`_matrixCorr_kendall_matrix_cpp`, mat)
}

kendall_tau2_cpp <- function(x, y) {
    .Call(`_matrixCorr_kendall_tau2_cpp`, x, y)
}

kendall_tau2_from_mat_cpp <- function(mat) {
    .Call(`_matrixCorr_kendall_tau2_from_mat_cpp`, mat)
}

kendall_matrix_pairwise_cpp <- function(X_, return_ci = FALSE, conf_level = 0.95, ci_method = "fieller") {
    .Call(`_matrixCorr_kendall_matrix_pairwise_cpp`, X_, return_ci, conf_level, ci_method)
}

krippendorff_alpha_core_cpp <- function(counts, delta2, method_code = 1L, return_matrices = FALSE, n_threads = 1L) {
    .Call(`_matrixCorr_krippendorff_alpha_core_cpp`, counts, delta2, method_code, return_matrices, n_threads)
}

multirater_kappa_counts_cpp <- function(counts, method_code = 1L, by_category = FALSE, inference_code = 0L, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_multirater_kappa_counts_cpp`, counts, method_code, by_category, inference_code, conf_level, n_threads)
}

multirater_kappa_ratings_cpp <- function(ratings, n_levels, method_code = 1L, na_code = 1L, min_raters = 2L, by_category = FALSE, inference_code = 0L, conf_level = 0.95, n_threads = 1L, exact = FALSE) {
    .Call(`_matrixCorr_multirater_kappa_ratings_cpp`, ratings, n_levels, method_code, na_code, min_raters, by_category, inference_code, conf_level, n_threads, exact)
}

partial_correlation_cpp <- function(X_, method = "sample", lambda = 1e-3, return_cov_precision = TRUE, return_p_value = FALSE) {
    .Call(`_matrixCorr_partial_correlation_cpp`, X_, method, lambda, return_cov_precision, return_p_value)
}

pearson_matrix_cpp <- function(X_) {
    .Call(`_matrixCorr_pearson_matrix_cpp`, X_)
}

pearson_threshold_triplets_cpp <- function(X_, threshold = 0.0, diag = TRUE, block_size = 256L) {
    .Call(`_matrixCorr_pearson_threshold_triplets_cpp`, X_, threshold, diag, block_size)
}

pearson_matrix_pairwise_cpp <- function(X_, return_ci = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_pearson_matrix_pairwise_cpp`, X_, return_ci, conf_level)
}

prob_agree_fit_cpp <- function(response, predictor, group, eval_predictor, link_code, lower, upper, ci, ci_method, conf_level, max_iter, tol) {
    .Call(`_matrixCorr_prob_agree_fit_cpp`, response, predictor, group, eval_predictor, link_code, lower, upper, ci, ci_method, conf_level, max_iter, tol)
}

rmcorr_pair_cpp <- function(x, y, subject, conf_level = 0.95) {
    .Call(`_matrixCorr_rmcorr_pair_cpp`, x, y, subject, conf_level)
}

rmcorr_pair_weighted_cpp <- function(x, y, subject, conf_level = 0.95) {
    .Call(`_matrixCorr_rmcorr_pair_weighted_cpp`, x, y, subject, conf_level)
}

rmcorr_matrix_cpp <- function(x, y, subject, symmetric = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_rmcorr_matrix_cpp`, x, y, subject, symmetric, conf_level, n_threads)
}

rmcorr_matrix_weighted_cpp <- function(x, y, subject, symmetric = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_rmcorr_matrix_weighted_cpp`, x, y, subject, symmetric, conf_level, n_threads)
}

pbcor_matrix_cpp <- function(X, beta = 0.2, n_threads = 1L) {
    .Call(`_matrixCorr_pbcor_matrix_cpp`, X, beta, n_threads)
}

pbcor_threshold_triplets_cpp <- function(X, beta = 0.2, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_pbcor_threshold_triplets_cpp`, X, beta, threshold, diag, block_size, n_threads)
}

pbcor_matrix_pairwise_cpp <- function(X, beta = 0.2, min_n = 5L, n_threads = 1L) {
    .Call(`_matrixCorr_pbcor_matrix_pairwise_cpp`, X, beta, min_n, n_threads)
}

wincor_matrix_cpp <- function(X, tr = 0.2, n_threads = 1L) {
    .Call(`_matrixCorr_wincor_matrix_cpp`, X, tr, n_threads)
}

wincor_threshold_triplets_cpp <- function(X, tr = 0.2, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_wincor_threshold_triplets_cpp`, X, tr, threshold, diag, block_size, n_threads)
}

wincor_matrix_pairwise_cpp <- function(X, tr = 0.2, min_n = 5L, n_threads = 1L) {
    .Call(`_matrixCorr_wincor_matrix_pairwise_cpp`, X, tr, min_n, n_threads)
}

skipcor_matrix_cpp <- function(X, method_int = 0L, stand = TRUE, use_mad = FALSE, gval = 2.717803, min_n = 5L, n_threads = 1L, return_masks = FALSE, return_inference = FALSE, conf_level = 0.95, n_boot = 2000L, seed = 0L, multiple_method_int = 0L, fwe_level = 0.05, n_mc = 1000L) {
    .Call(`_matrixCorr_skipcor_matrix_cpp`, X, method_int, stand, use_mad, gval, min_n, n_threads, return_masks, return_inference, conf_level, n_boot, seed, multiple_method_int, fwe_level, n_mc)
}

sss_cor_cpp <- function(X_) {
    .Call(`_matrixCorr_sss_cor_cpp`, X_)
}

spearman_matrix_cpp <- function(X_) {
    .Call(`_matrixCorr_spearman_matrix_cpp`, X_)
}

spearman_matrix_pairwise_cpp <- function(X_, return_ci = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_spearman_matrix_pairwise_cpp`, X_, return_ci, conf_level)
}

spearman_threshold_triplets_cpp <- function(X_, threshold = 0.0, diag = TRUE, block_size = 256L) {
    .Call(`_matrixCorr_spearman_threshold_triplets_cpp`, X_, threshold, diag, block_size)
}

matrixCorr_tetrachoric_mle_cpp <- function(tab, correct = 0.5) {
    .Call(`_matrixCorr_matrixCorr_tetrachoric_mle_cpp`, tab, correct)
}

matrixCorr_tetrachoric_fixed_cpp <- function(tab, rc, cc, correct = 0.5) {
    .Call(`_matrixCorr_matrixCorr_tetrachoric_fixed_cpp`, tab, rc, cc, correct)
}

matrixCorr_polychoric_mle_cpp <- function(tab, correct = 0.5) {
    .Call(`_matrixCorr_matrixCorr_polychoric_mle_cpp`, tab, correct)
}

matrixCorr_polychoric_fixed_cpp <- function(tab, rc_in, cc_in) {
    .Call(`_matrixCorr_matrixCorr_polychoric_fixed_cpp`, tab, rc_in, cc_in)
}

matrixCorr_tetrachoric_matrix_cpp <- function(x, tau, correct = 0.5, pairwise_complete = TRUE) {
    .Call(`_matrixCorr_matrixCorr_tetrachoric_matrix_cpp`, x, tau, correct, pairwise_complete)
}

matrixCorr_polychoric_matrix_cpp <- function(x, n_levels, tau_mat, global_all = FALSE, correct = 0.5, pairwise_complete = TRUE) {
    .Call(`_matrixCorr_matrixCorr_polychoric_matrix_cpp`, x, n_levels, tau_mat, global_all, correct, pairwise_complete)
}

matrixCorr_biserial_latent_cpp <- function(x, y) {
    .Call(`_matrixCorr_matrixCorr_biserial_latent_cpp`, x, y)
}

matrixCorr_polyserial_mle_cpp <- function(x, y) {
    .Call(`_matrixCorr_matrixCorr_polyserial_mle_cpp`, x, y)
}

matrixCorr_polyserial_negloglik_cpp <- function(z, y, pars, maxcor = 0.9999) {
    .Call(`_matrixCorr_matrixCorr_polyserial_negloglik_cpp`, z, y, pars, maxcor)
}

matrixCorr_tetrachoric_inference_cpp <- function(tab, correct = 0.5, conf_level = 0.95) {
    .Call(`_matrixCorr_matrixCorr_tetrachoric_inference_cpp`, tab, correct, conf_level)
}

matrixCorr_polychoric_inference_cpp <- function(tab, correct = 0.5, conf_level = 0.95) {
    .Call(`_matrixCorr_matrixCorr_polychoric_inference_cpp`, tab, correct, conf_level)
}

matrixCorr_polyserial_inference_cpp <- function(x, y, conf_level = 0.95) {
    .Call(`_matrixCorr_matrixCorr_polyserial_inference_cpp`, x, y, conf_level)
}

matrixCorr_polydi_mle_cpp <- function(tab, correct = 0.5) {
    .Call(`_matrixCorr_matrixCorr_polydi_mle_cpp`, tab, correct)
}

validate_corr_input_cpp <- function(data, check_na = TRUE) {
    .Call(`_matrixCorr_validate_corr_input_cpp`, data, check_na)
}

complete_case_matrix_cpp <- function(x) {
    .Call(`_matrixCorr_complete_case_matrix_cpp`, x)
}

weighted_kappa_pair_cpp <- function(x, y, weights, return_inference = FALSE, conf_level = 0.95) {
    .Call(`_matrixCorr_weighted_kappa_pair_cpp`, x, y, weights, return_inference, conf_level)
}

weighted_kappa_matrix_cpp <- function(X, weights, pairwise_complete = FALSE, return_inference = FALSE, conf_level = 0.95, n_threads = 1L) {
    .Call(`_matrixCorr_weighted_kappa_matrix_cpp`, X, weights, pairwise_complete, return_inference, conf_level, n_threads)
}

weighted_kappa_threshold_triplets_cpp <- function(X, weights, threshold = 0.0, diag = TRUE, block_size = 256L, n_threads = 1L) {
    .Call(`_matrixCorr_weighted_kappa_threshold_triplets_cpp`, X, weights, threshold, diag, block_size, n_threads)
}

# AR(1) simulator for within-subject residuals
ar1_sim <- function(n, rho, sd = 1) {
  z <- rnorm(n)
  e <- numeric(n)
  e[1] <- z[1] * sd
  if (n > 1) for (t in 2:n) e[t] <- rho * e[t - 1] + sqrt(1 - rho^2) * z[t] * sd
  e
}

# Two-method data with known difference model:
# d_{s,t} = (y2 - y1) = mu + b_s + e_{s,t},
# with b_s ~ N(0, sig_s^2), e ~ N(0, sig_e^2) or AR(1)
# ==> truth: bias = mu; sd_loa = sqrt(sig_s^2 + sig_e^2)
sim_two_method_known <- function(S = 60L, Tm = 25L,
                                 mu = 1.25, sig_s = 1.7, sig_e = 2.3,
                                 rho = NULL, seed = 1L) {
  set.seed(seed)
  subj <- rep(seq_len(S), each = Tm)
  time <- rep(seq_len(Tm), times = S)
  b_s  <- rnorm(S, 0, sig_s)
  b    <- b_s[subj]
  e    <- if (is.null(rho)) rnorm(S * Tm, 0, sig_e) else
    unlist(lapply(split(seq_along(time), subj), function(ix) ar1_sim(length(ix), rho, sig_e)))

  # Baseline cancels in differences; include to mimic realistic responses
  baseline <- rnorm(S * Tm, 10, 3)
  y1 <- baseline
  y2 <- baseline + mu + b + e

  data.frame(
    y = c(y1, y2),
    subject = rep(subj, 2),
    method  = factor(rep(c("M1", "M2"), each = S * Tm), levels = c("M1", "M2")),
    time    = rep(time, 2),
    check.names = FALSE
  )
}

# Multi-method data for matrix invariants
# M2 ~= M1 + small noise; M3 = -M1; M4 = unrelated
sim_multi_method <- function(S = 30L, Tm = 15L, seed = 11L) {
  set.seed(seed)
  subj <- rep(seq_len(S), each = Tm)
  time <- rep(seq_len(Tm), times = S)

  mu_s <- rnorm(S, mean = 0, sd = 8)
  true <- mu_s[subj]
  e0   <- rnorm(length(true), 0, 2)

  y1 <- true + e0
  y2 <- y1 + rnorm(length(true), 0, 0.01)
  y3 <- -y1
  y4 <- rnorm(length(true), 3, 6)

  out <- rbind(
    data.frame(y = y1, subject = subj, method = "M1", time = time),
    data.frame(y = y2, subject = subj, method = "M2", time = time),
    data.frame(y = y3, subject = subj, method = "M3", time = time),
    data.frame(y = y4, subject = subj, method = "M4", time = time),
    make.row.names = FALSE
  )
  out$method <- factor(out$method, levels = c("M1", "M2", "M3", "M4"))
  out
}

sim_two_method_from_pairs <- function(means, diffs, S, Tm) {
  stopifnot(length(means) == length(diffs), length(means) == S * Tm)
  subj <- rep(seq_len(S), each = Tm)
  time <- rep(seq_len(Tm), times = S)
  y1 <- means - diffs / 2
  y2 <- means + diffs / 2

  data.frame(
    y = c(y1, y2),
    subject = rep(subj, 2),
    method = factor(rep(c("A", "B"), each = length(y1)), levels = c("A", "B")),
    time = rep(time, 2),
    check.names = FALSE
  )
}

test_that("two-method contrast matches factor order (method2 - method1)", {
  dat <- sim_two_method_known(S = 50, Tm = 20, mu = 1.0, sig_s = 1.2, sig_e = 1.5, seed = 1)
  dat$method <- factor(dat$method, levels = c("M1","M2"))
  fit <- ba_rm(data = dat, response="y", subject="subject",
                               method="method", time="time", use_ar1=FALSE)
  expect_gt(as.numeric(fit$mean.diffs), 0)  # should be ~ +1.0
})

test_that("Two-method BA recovers bias and sd_loa under i.i.d. residuals", {
  dat <- sim_two_method_known(S = 80, Tm = 50, mu = 1.25, sig_s = 1.7,
                              sig_e = 2.3, rho = NULL, seed = 100)
  truth_sd <- sqrt(1.7^2 + 2.3^2)

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, loa_multiplier = 1.96, conf_level = 0.95
  )

  expect_s3_class(fit, "ba_repeated")

  # Bias and sd_loa close to truth
  expect_equal(as.numeric(fit$mean.diffs), 1.25, tolerance = 0.08)
  expect_equal(as.numeric(fit$critical.diff) / as.numeric(fit$loa_multiplier), truth_sd, tolerance = 0.08)

  # LoA consistent with sd and 'loa_multiplier'
  sd_hat <- as.numeric(fit$critical.diff) / as.numeric(fit$loa_multiplier)
  expect_equal(as.numeric(fit$upper.limit) - as.numeric(fit$mean.diffs), 1.96 * sd_hat, tolerance = 1e-6)
  expect_equal(as.numeric(fit$mean.diffs) - as.numeric(fit$lower.limit), 1.96 * sd_hat, tolerance = 1e-6)

  # Summary object sanity
  sm <- summary(fit)
  expect_s3_class(sm, "summary.ba_repeated")
  expect_true(all(c("bias","sd_loa","loa_low","loa_up","width","n_obs") %in% names(sm)))
  expect_false(any(c("ar1_rho", "ar1_estimated") %in% names(sm)))
})

test_that("ba_rm honors n_threads without changing estimates", {
  dat2 <- sim_two_method_known(S = 20L, Tm = 8L, mu = 0.8, sig_s = 1.1, sig_e = 1.3, seed = 77L)
  fit2_1 <- ba_rm(
    data = dat2,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, n_threads = 1L
  )
  fit2_2 <- ba_rm(
    data = dat2,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, n_threads = 2L
  )

  expect_equal(unclass(fit2_1), unclass(fit2_2), tolerance = 1e-12)

  datm <- sim_multi_method(S = 12L, Tm = 6L, seed = 78L)
  fitm_1 <- ba_rm(
    data = datm,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, n_threads = 1L
  )
  fitm_2 <- ba_rm(
    data = datm,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, n_threads = 2L
  )

  expect_equal(unclass(fitm_1), unclass(fitm_2), tolerance = 1e-12)
})

test_that("Two-method BA with AR(1) honours supplied rho and recovers truth", {
  skip_on_cran()

  mu    <- 0.8
  sig_s <- 1.2
  sig_e <- 1.5
  rho   <- 0.45
  dat <- sim_two_method_known(S = 70, Tm = 40, mu = mu, sig_s = sig_s, sig_e = sig_e, rho = rho, seed = 200)
  truth_sd <- sqrt(sig_s^2 + sig_e^2)  # definition of sd_loa in docs

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = TRUE, ar1_rho = rho, loa_multiplier = 2.0, conf_level = 0.9
  )

  expect_true(isTRUE(fit$use_ar1))
  expect_false(is.na(fit$ar1_rho))
  expect_equal(as.numeric(fit$ar1_rho), rho, tolerance = 1e-12)
  expect_false(isTRUE(fit$ar1_estimated))  # supplied, so not estimated

  expect_equal(as.numeric(fit$mean.diffs), mu,   tolerance = 0.08)
  expect_equal(as.numeric(fit$critical.diff) / 2.0, truth_sd, tolerance = 0.09)
})

test_that("Two-method BA estimates positive AR(1) dependence on short panels", {
  skip_on_cran()

  mu <- 0.35
  sig_s <- sqrt(0.28^2 + 0.38^2)
  sig_e <- sqrt(0.45^2 + 0.60^2)
  rho <- 0.60
  dat <- sim_two_method_known(
    S = 120,
    Tm = 10,
    mu = mu,
    sig_s = sig_s,
    sig_e = sig_e,
    rho = rho,
    seed = 20260401L
  )

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = TRUE, ar1_rho = NA_real_,
    loa_multiplier = 1.96, conf_level = 0.95
  )

  expect_true(isTRUE(fit$use_ar1))
  expect_true(isTRUE(fit$ar1_estimated))
  expect_gt(as.numeric(fit$ar1_rho), 0.30)
  expect_equal(as.numeric(fit$ar1_rho), rho, tolerance = 0.20)
  expect_true(abs(as.numeric(fit$mean.diffs) - mu) < 0.08)
  expect_equal(as.numeric(fit$critical.diff) / 1.96, sqrt(sig_s^2 + sig_e^2), tolerance = 0.08)
})

test_that("AR(1) requests simplify to iid with a warning when needed for some pairs", {
  set.seed(123)
  S <- 40L
  Tm <- 50L
  methods <- c("A", "B", "C")
  rho <- 0.4

  ar1_sim <- function(n, rho, sd = 1) {
    z <- rnorm(n)
    e <- numeric(n)
    e[1] <- z[1] * sd
    if (n > 1) for (t in 2:n) e[t] <- rho * e[t - 1] + sqrt(1 - rho^2) * z[t] * sd
    e
  }

  subj <- rep(seq_len(S), each = Tm)
  time <- rep(seq_len(Tm), times = S)
  mu_s <- rnorm(S, 50, 7)
  trend <- rep(seq_len(Tm) - mean(seq_len(Tm)), times = S) * 0.8
  true <- mu_s[subj] + trend
  bias <- c(A = 0, B = 1.5, C = -0.5)
  prop <- c(A = 0.00, B = 0.00, C = 0.10)

  make_method <- function(meth, sd = 3) {
    e <- unlist(lapply(split(seq_along(time), subj),
                       function(ix) ar1_sim(length(ix), rho, sd)))
    y <- true * (1 + prop[meth]) + bias[meth] + e
    data.frame(y = y, subject = subj, method = meth, time = time,
               check.names = FALSE)
  }

  dat <- do.call(rbind, lapply(methods, make_method))
  dat$method <- factor(dat$method, levels = methods)

 fit <- ba_rm(
     response = dat$y, subject = dat$subject, method = dat$method, time = dat$time,
    include_slope = FALSE, use_ar1 = TRUE, ar1_rho = rho)


  expect_s3_class(fit, "ba_repeated_matrix")
  simplified <- fit$residual_model == "iid"
  expect_true(all(is.na(fit$ar1_rho_pair[simplified])))

  sm <- summary(fit)
  expect_true(all(c("residual_model", "ar1_rho") %in% names(sm)))
})

test_that("AR(1) fallback warning text is explicit", {
  expect_warning(
    matrixCorr:::.warn_ba_rm_ar1_fallback(),
    "Requested AR\\(1\\) residual structure could not be fit for this fit; using iid residuals instead\\."
  )

  expect_warning(
    matrixCorr:::.warn_ba_rm_ar1_fallback(c("A-B", "B-C", "A-B")),
    "Requested AR\\(1\\) residual structure could not be fit for pair\\(s\\): A-B, B-C; using iid residuals instead\\."
  )
})

test_that("Edge case: constant difference gives zero sd_loa and degenerate LoA", {
  # Small jitter to avoid numerical singularities in optimisers; still near-zero SD
  set.seed(1)
  S <- 25; Tm <- 8; mu <- 5
  subj <- rep(seq_len(S), each = Tm)
  time <- rep(seq_len(Tm), times = S)
  base <- rnorm(S * Tm, 12, 2)
  y1 <- base
  y2 <- base + mu + rnorm(S * Tm, 0, 1e-6)

  dat <- data.frame(y = c(y1,y2), subject = rep(subj, 2), method = rep(c("A","B"), each = length(y1)),
                    time = rep(time, 2))
  dat$method <- factor(dat$method, levels = c("A","B"))

  fit <- ba_rm(
    data = dat, response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE
  )

  expect_equal(as.numeric(fit$mean.diffs), mu, tolerance = 1e-3)
  expect_lt(as.numeric(fit$critical.diff), 1e-2)  # ~ zero
  expect_equal(as.numeric(fit$lower.limit), as.numeric(fit$upper.limit), tolerance = 1e-2)
})

test_that("Temporary positive residual-variance initialization remains only an EM starting heuristic", {
  S <- 10L
  Tm <- 2L
  subject_means <- seq(12, 30, length.out = S)
  means <- rep(subject_means, each = Tm)
  slope_true <- 0.25
  diffs <- rep(0.8 + slope_true * (subject_means - mean(subject_means)), each = Tm)
  dat <- sim_two_method_from_pairs(means, diffs, S = S, Tm = Tm)

  fit_cpp <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat$y,
    subject = dat$subject,
    method = as.integer(dat$method == "B") + 1L,
    time = dat$time,
    include_slope = TRUE,
    use_ar1 = FALSE
  )
  expect_match(
    fit_cpp$warn,
    "Constrained boundary fit used with sigma2_subject fixed at 0.",
    perl = TRUE
  )
  expect_true(isTRUE(fit_cpp$converged))
  expect_true(is.finite(as.numeric(fit_cpp$sigma2_subject)))
  expect_true(is.finite(as.numeric(fit_cpp$sigma2_resid)))

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = TRUE, use_ar1 = FALSE
  )
  expect_true(is.finite(as.numeric(fit$beta_slope)))
  expect_equal(as.numeric(fit$beta_slope), slope_true, tolerance = 0.05)
})

test_that("Slope scale selection uses SD when paired means have ordinary spread", {
  normal_m <- seq(1, 8)
  info_normal <- matrixCorr:::ba_rm_slope_scale_cpp(normal_m)
  expect_identical(info_normal$source, "sd")
  expect_equal(info_normal$s_sd, stats::sd(normal_m), tolerance = 1e-15)
  expect_equal(info_normal$s_iqr, stats::IQR(normal_m) / 1.349, tolerance = 1e-15)
  expect_equal(
    info_normal$s_mad,
    stats::mad(normal_m, center = stats::median(normal_m), constant = 1.4826, na.rm = TRUE),
    tolerance = 1e-15
  )
  expect_equal(info_normal$s_m_star, info_normal$s_sd, tolerance = 1e-15)
})

test_that("Slope scale selection falls back to IQR when SD is near zero", {
  near_constant_m <- c(rep(0, 16), rep(5e-324, 16))
  info_near <- matrixCorr:::ba_rm_slope_scale_cpp(near_constant_m)

  expect_identical(info_near$source, "iqr")
  expect_equal(info_near$s_sd, 0, tolerance = 0)
  expect_true(is.finite(info_near$s_iqr) && info_near$s_iqr > 0)
  expect_equal(info_near$s_m_star, info_near$s_iqr, tolerance = 0)
  expect_true(info_near$s_sd <= info_near$tau * info_near$s_ref)
})

test_that("Slope fit is recovered when pair means have ordinary spread", {
  set.seed(321)
  S <- 18L
  Tm <- 4L
  n <- S * Tm
  means <- seq(10, 40, length.out = n)
  subj_eff <- rep(rnorm(S, 0, 0.15), each = Tm)
  slope_true <- 0.35
  diffs <- 0.75 + slope_true * (means - mean(means)) + subj_eff + rnorm(n, 0, 0.04)
  dat <- sim_two_method_from_pairs(means, diffs, S = S, Tm = Tm)

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = TRUE, use_ar1 = FALSE
  )

  expect_true(is.finite(as.numeric(fit$beta_slope)))
  expect_equal(as.numeric(fit$beta_slope), slope_true, tolerance = 0.05)
})

test_that("Slope scaling errors when paired means are fully degenerate", {
  expect_error(
    matrixCorr:::ba_rm_slope_scale_cpp(rep(25, 32)),
    "The proportional-bias slope is not estimable because the paired means are near-degenerate on the observed data scale.",
    fixed = TRUE
  )

  set.seed(323)
  S <- 14L
  Tm <- 4L
  n <- S * Tm
  means <- rep(25, n)
  diffs <- 1.1 + rep(rnorm(S, 0, 0.12), each = Tm) + rnorm(n, 0, 0.03)
  dat <- sim_two_method_from_pairs(means, diffs, S = S, Tm = Tm)

  expect_error(
    ba_rm(
      data = dat,
      response = "y", subject = "subject", method = "method", time = "time",
      include_slope = TRUE, use_ar1 = FALSE
    ),
    "The proportional-bias slope is not estimable because the paired means are near-degenerate on the observed data scale.",
    fixed = TRUE
  )
})

test_that("Ordinary repeated-measures cases still converge successfully", {
  dat <- sim_two_method_known(S = 40L, Tm = 12L, mu = 1.1, sig_s = 1.0, sig_e = 0.8, seed = 913)

  fit_cpp <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat$y,
    subject = dat$subject,
    method = as.integer(dat$method == "M2") + 1L,
    time = dat$time,
    include_slope = FALSE,
    use_ar1 = FALSE
  )

  expect_true(isTRUE(fit_cpp$converged))
  expect_true(is.finite(as.numeric(fit_cpp$sigma2_subject)))
  expect_true(is.finite(as.numeric(fit_cpp$sigma2_resid)))

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE
  )

  expect_s3_class(fit, "ba_repeated")
  expect_true(is.finite(as.numeric(fit$mean.diffs)))
  expect_true(is.finite(as.numeric(fit$critical.diff)))
  expect_gt(as.numeric(fit$critical.diff), 0)
  expect_true(is.finite(as.numeric(fit$sigma2_subject)))
  expect_true(is.finite(as.numeric(fit$sigma2_resid)))
})

test_that("Ordinary slope cases still converge successfully", {
  set.seed(914)
  S <- 18L
  Tm <- 4L
  n <- S * Tm
  means <- seq(10, 40, length.out = n)
  subj_eff <- rep(rnorm(S, 0, 0.15), each = Tm)
  slope_true <- 0.35
  diffs <- 0.75 + slope_true * (means - mean(means)) + subj_eff + rnorm(n, 0, 0.04)
  dat <- sim_two_method_from_pairs(means, diffs, S = S, Tm = Tm)

  fit_cpp <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat$y,
    subject = dat$subject,
    method = as.integer(dat$method == "B") + 1L,
    time = dat$time,
    include_slope = TRUE,
    use_ar1 = FALSE
  )

  expect_true(isTRUE(fit_cpp$converged))
  expect_true(is.finite(as.numeric(fit_cpp$beta_slope)))

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = TRUE, use_ar1 = FALSE
  )

  expect_true(is.finite(as.numeric(fit$beta_slope)))
  expect_equal(as.numeric(fit$beta_slope), slope_true, tolerance = 0.05)
})

test_that("Pairwise matrix algebraic invariants hold", {
  dat <- sim_multi_method(S = 24, Tm = 12, seed = 99)

  fit <- ba_rm(
    data = dat,
    response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, loa_multiplier = 1.96, conf_level = 0.95
  )

  expect_s3_class(fit, "ba_repeated_matrix")
  m <- length(fit$methods)

  # Symmetry / antisymmetry conditions
  expect_true(all(is.na(diag(fit$bias))))
  expect_true(all(abs(fit$bias + t(fit$bias)) < 1e-8, na.rm = TRUE))

  expect_true(all(abs(fit$sd_loa - t(fit$sd_loa)) < 1e-12, na.rm = TRUE))
  expect_true(all(abs(fit$width  - t(fit$width )) < 1e-12, na.rm = TRUE))

  # LoA sign conventions (row - column)
  for (i in 1:(m-1)) for (j in (i+1):m) {
    expect_equal(fit$loa_lower[j,i], -fit$loa_upper[i,j], tolerance = 1e-10)
    expect_equal(fit$loa_upper[j,i], -fit$loa_lower[i,j], tolerance = 1e-10)
  }

  # Width equals 2 * loa_multiplier * sd_loa
  expect_equal(fit$width, 2 * fit$loa_multiplier * fit$sd_loa, tolerance = 1e-10)
})

test_that("Pairwise result equals two-method fit for the same pair", {
  dat <- sim_multi_method(S = 30, Tm = 15, seed = 7)
  # Compare M1 vs M2
  dat12 <- subset(dat, method %in% c("M1","M2"))

  fit2 <- ba_rm(
    data = dat12, response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, loa_multiplier = 1.96, conf_level = 0.95
  )

  fitN <- ba_rm(
    data = dat, response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE, loa_multiplier = 1.96, conf_level = 0.95
  )

  i <- match("M1", fitN$methods); j <- match("M2", fitN$methods)
  expect_equal(as.numeric(fit2$mean.diffs), fitN$bias[i,j], tolerance = 1e-8)
  expect_equal(as.numeric(fit2$critical.diff) / as.numeric(fit2$loa_multiplier), fitN$sd_loa[i,j], tolerance = 1e-8)
  expect_equal(as.numeric(fit2$lower.limit), fitN$loa_lower[i,j], tolerance = 1e-8)
  expect_equal(as.numeric(fit2$upper.limit), fitN$loa_upper[i,j], tolerance = 1e-8)
})

test_that("n matrix equals number of subject-time complete pairs per contrast", {
  # Create imbalance: drop some method-time rows
  dat <- sim_multi_method(S = 12, Tm = 10, seed = 123)
  # Remove random 15% of M3 rows to induce missingness
  set.seed(123)
  drop_idx <- which(dat$method == "M3")
  rm <- sample(drop_idx, size = floor(0.15 * length(drop_idx)))
  dat2 <- dat[-rm, ]

  fit <- ba_rm(
    data = dat2, response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE, use_ar1 = FALSE
  )

  # Manual complete-case counts for M1 vs M3
  cc <- merge(
    subset(dat2, method == "M1")[, c("subject","time")],
    subset(dat2, method == "M3")[, c("subject","time")],
    by = c("subject","time"),
    all = FALSE
  )
  i <- match("M1", fit$methods); j <- match("M3", fit$methods)
  expect_equal(as.integer(fit$n[i,j]), nrow(cc))
})

test_that("native complete-pair counter matches the expected matched-pair total", {
  dat <- data.frame(
    y = c(1, 2, 3, 4, 5, NA, 7, 8),
    subject = c(1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L),
    method = c(1L, 2L, 1L, 1L, 1L, 2L, 2L, 1L),
    time = c(1L, 1L, 2L, 2L, 1L, 1L, 2L, 2L)
  )

  expect_equal(
    matrixCorr:::ba_rm_complete_pairs_cpp(dat$y, dat$subject, dat$method, dat$time),
    2L
  )
})

test_that("Column mapping works both with data= and with vectors", {
  dat <- sim_multi_method(S = 10, Tm = 6, seed = 42)
  # With data=
  fit1 <- ba_rm(
    data = dat, response = "y", subject = "subject", method = "method", time = "time"
  )
  expect_true(inherits(fit1, "ba_repeated_matrix"))

  # With vectors (no data=)
  fit2 <- ba_rm(
    response = dat$y, subject = dat$subject, method = dat$method, time = dat$time
  )
  expect_true(inherits(fit2, "ba_repeated_matrix"))

  expect_true(all(c("data_long", "mapping") %in% names(fit1)))
  expect_true(all(c("data_long", "mapping") %in% names(fit2)))
  expect_equal(names(fit1$data_long), c(".response", ".subject", ".method", ".time"))
  expect_equal(names(fit2$data_long), c(".response", ".subject", ".method", ".time"))
  expect_identical(fit1$mapping, list(
    response = ".response",
    subject = ".subject",
    method = ".method",
    time = ".time"
  ))
  expect_identical(fit2$mapping, list(
    response = ".response",
    subject = ".subject",
    method = ".method",
    time = ".time"
  ))
})

test_that("summary/print produce expected classes and do not error", {
  dat <- sim_multi_method(S = 10, Tm = 6, seed = 2)

  fit_mat <- ba_rm(
    data = dat, response = "y", subject = "subject", method = "method", time = "time"
  )
  out_fit_mat <- capture.output(print(fit_mat))
  expect_false(any(grepl("bias_lwr|sigma2_subject|ar1_rho|Confidence intervals|Model details",
                         out_fit_mat)))

  sm_mat <- summary(fit_mat)
  expect_s3_class(sm_mat, "summary.ba_repeated_matrix")
  expect_true(all(c("item1","item2","bias","sd_loa","loa_low","loa_up","width","n_obs") %in% names(sm_mat)))
  expect_false(any(c("ar1_rho", "ar1_estimated") %in% names(sm_mat)))
  out_mat <- capture.output(print(sm_mat))
  expect_true(any(grepl("^Agreement estimates$", out_mat)))
  expect_true(any(grepl("^Confidence intervals$", out_mat)))
  expect_true(any(grepl("^Model details$", out_mat)))
  expect_false(any(grepl("ar1_rho|ar1_estimated", out_mat)))

  # Two-method
  dat12 <- subset(dat, method %in% c("M1","M2"))
  fit2  <- ba_rm(
    data = dat12, response = "y", subject = "subject", method = droplevels(dat12$method), time = "time"
  )
  sm2 <- summary(fit2)
  expect_s3_class(sm2, "summary.ba_repeated")
  expect_true(all(c("bias","sd_loa","loa_low","loa_up","width","n_obs") %in% names(sm2)))
  expect_false(any(c("ar1_rho", "ar1_estimated") %in% names(sm2)))
  out2 <- capture.output(print(sm2))
  expect_true(any(grepl("^Agreement estimates$", out2)))
  expect_true(any(grepl("^Confidence intervals$", out2)))
  expect_true(any(grepl("^Model details$", out2)))
  expect_false(any(grepl("ar1_rho|ar1_estimated", out2)))
})

test_that("two-method summary omits AR columns when final residual model is iid", {
  fit <- structure(
    list(
      based.on = 12L,
      mean.diffs = 0.8,
      critical.diff = 1.96,
      loa_multiplier = 1.96,
      lower.limit = -1.16,
      upper.limit = 2.76,
      CI.lines = c(
        "mean.diff.ci.lower" = 0.5,
        "mean.diff.ci.upper" = 1.1,
        "lower.limit.ci.lower" = -1.6,
        "lower.limit.ci.upper" = -0.7,
        "upper.limit.ci.lower" = 2.3,
        "upper.limit.ci.upper" = 3.2
      ),
      include_slope = FALSE,
      sigma2_subject = 0.4,
      sigma2_resid = 1.1,
      use_ar1 = TRUE,
      residual_model = "iid",
      ar1_rho = NA_real_,
      ar1_estimated = FALSE
    ),
    class = "ba_repeated"
  )
  attr(fit, "conf.level") <- 0.95

  sm <- summary(fit)

  expect_false(any(c("ar1_rho", "ar1_estimated") %in% names(sm)))
})

test_that("plot methods return a ggplot object and do not error", {
  skip_if_not_installed("ggplot2")
  dat <- sim_multi_method(S = 12, Tm = 8, seed = 5)

  # Matrix plot
  fit_mat <- ba_rm(
    data = dat, response = "y", subject = "subject", method = "method", time = "time",
    include_slope = FALSE
  )
  p_mat <- plot(fit_mat, smoother = "lm", facet_scales = "free_y")
  expect_s3_class(p_mat, "ggplot")
  has_point_layer <- any(vapply(p_mat$layers, function(layer) {
    inherits(layer$geom, "GeomPoint")
  }, logical(1)))
  expect_true(has_point_layer)
  expect_identical(p_mat$labels$x, "Pair mean")

  # Two-method plot
  dat12 <- subset(dat, method %in% c("M1","M2"))
  fit2  <- ba_rm(
    data = dat12, response = "y", subject = "subject", method = droplevels(dat12$method), time = "time",
    include_slope = TRUE
  )
  p2 <- plot(fit2, smoother = "loess", symmetrize_y = TRUE)
  expect_s3_class(p2, "ggplot")
})

test_that("Input argument validation works", {
  dat <- sim_multi_method(S = 6, Tm = 4, seed = 1)

  # bad 'loa_multiplier'
  expect_error(
    ba_rm(data = dat, response = "y", subject = "subject", method = "method", time = "time",
                          loa_multiplier = -1),
    "`loa_multiplier` must be a positive scalar.", fixed = TRUE
  )

  # bad conf_level
  expect_error(
    ba_rm(data = dat, response = "y", subject = "subject", method = "method", time = "time",
                          conf_level = 1.1),
    "`conf_level` must be in (0,1).", fixed = TRUE
  )

  # AR1 rho bounds when use_ar1 = TRUE
  expect_error(
    ba_rm(data = dat, response = "y", subject = "subject", method = "method", time = "time",
                          use_ar1 = TRUE, ar1_rho = 1.2),
    "`ar1_rho` must be in (-0.999, 0.999).", fixed = TRUE
  )

  # Need at least 2 methods
  dat1 <- subset(dat, method == "M1")
  expect_error(
    ba_rm(data = dat1, response = "y", subject = "subject", method = "method", time = "time"),
    "Need at least 2 distinct methods in `method`.", fixed = TRUE
  )

  # Wrong column name
  expect_error(
    ba_rm(data = dat, response = "y", subject = "subject", method = "NOPE", time = "time"),
    "Column 'NOPE' not found in `data`.", fixed = TRUE
  )
})

test_that("two-method path requires at least two matched pairs", {
  dat <- data.frame(
    y = c(10, 11, 12),
    subject = c(1L, 1L, 1L),
    method = factor(c("A", "B", "A"), levels = c("A", "B")),
    time = c(1L, 1L, 2L),
    check.names = FALSE
  )

  expect_error(
    ba_rm(dat$y, dat$subject, dat$method, dat$time,
                          include_slope = FALSE, use_ar1 = FALSE),
    "at least two subject-time matched pairs",
    fixed = FALSE
  )
})

test_that("pairwise matrix drops contrasts with <2 matched pairs", {
  dat <- data.frame(
    y = c(1, 2, 3, 4, 5, 6, 7),
    subject = c(1L, 1L, 1L, 1L, 1L, 2L, 2L),
    method = factor(c("A", "B", "C", "A", "C", "A", "C"), levels = c("A", "B", "C")),
    time = c(1L, 1L, 1L, 2L, 2L, 1L, 1L),
    check.names = FALSE
  )

  fit <- ba_rm(dat$y, dat$subject, dat$method, dat$time,
                               include_slope = FALSE, use_ar1 = FALSE)

  expect_true(is.na(fit$bias["A", "B"]))
  expect_equal(fit$n["A", "B"], 1L)
  expect_true(is.na(fit$sd_loa["A", "B"]))
  expect_true(is.na(fit$loa_lower["A", "B"]))
  expect_true(is.na(fit$loa_upper["A", "B"]))
  expect_equal(fit$n["A", "C"], 3L)
  expect_true(is.finite(fit$bias["A", "C"]))
})

test_that("backend refactor preserves key model-based BA estimands", {
  extract_fit <- function(fit) {
    with(fit, list(
      bias_mu0 = bias_mu0,
      bias_lwr = bias_lwr,
      bias_upr = bias_upr,
      sigma2_subject = sigma2_subject,
      sigma2_resid = sigma2_resid,
      sd_loa = sd_loa,
      loa_lower = loa_lower,
      loa_upper = loa_upper,
      loa_lower_lwr = loa_lower_lwr,
      loa_lower_upr = loa_lower_upr,
      loa_upper_lwr = loa_upper_lwr,
      loa_upper_upr = loa_upper_upr,
      ar1_rho = ar1_rho,
      ar1_estimated = ar1_estimated
    ))
  }

  expect_close_to_reference <- function(observed, expected, tol = 5e-6) {
    for (nm in names(expected)) {
      if (is.logical(expected[[nm]])) {
        expect_identical(observed[[nm]], expected[[nm]], info = nm)
      } else if (is.na(expected[[nm]])) {
        expect_true(is.na(observed[[nm]]), info = nm)
      } else {
        expect_equal(observed[[nm]], expected[[nm]], tolerance = tol, info = nm)
      }
    }
  }

  expected <- list(
    iid_boundary = list(
      bias_mu0 = 1.10880351648566,
      bias_lwr = 0.881755926331974,
      bias_upr = 1.33585110663935,
      sigma2_subject = 0,
      sigma2_resid = 2.65706881134442,
      sd_loa = 1.63005178179849,
      loa_lower = -2.08609797583937,
      loa_upper = 4.30370500881069,
      loa_lower_lwr = -2.57353079579423,
      loa_lower_upr = -1.5986651558845,
      loa_upper_lwr = 3.81627218885742,
      loa_upper_upr = 4.79113782876397,
      ar1_rho = NA_real_,
      ar1_estimated = FALSE
    ),
    ar1_fixed_slope = list(
      bias_mu0 = 0.101168854430398,
      bias_lwr = -0.232691880183468,
      bias_upr = 0.435029589044263,
      sigma2_subject = 0.603133877867929,
      sigma2_resid = 0.604988223925724,
      sd_loa = 1.09914607845984,
      loa_lower = -2.09712330248929,
      loa_upper = 2.29946101135009,
      loa_lower_lwr = -2.61556143639825,
      loa_lower_upr = -1.57868516858033,
      loa_upper_lwr = 1.78104737272341,
      loa_upper_upr = 2.81787464997676,
      ar1_rho = 0.35,
      ar1_estimated = FALSE
    ),
    ar1_profiled = list(
      bias_mu0 = 0.35051291798968,
      bias_lwr = 0.024251325329714,
      bias_upr = 0.676774510649645,
      sigma2_subject = 0.247592888510583,
      sigma2_resid = 1.13252478725102,
      sd_loa = 1.17478409750967,
      loa_lower = -1.95206391312926,
      loa_upper = 2.65308974910862,
      loa_lower_lwr = -2.45695199981367,
      loa_lower_upr = -1.44717582644486,
      loa_upper_lwr = 2.14433296879168,
      loa_upper_upr = 3.16184652942557,
      ar1_rho = 0.555555555555556,
      ar1_estimated = TRUE
    ),
    exact_boundary = list(
      bias_mu0 = 0.799999999999998,
      bias_lwr = 0.799999561738728,
      bias_upr = 0.800000438261268,
      sigma2_subject = 0,
      sigma2_resid = 1e-12,
      sd_loa = 1e-06,
      loa_lower = 0.799998039999998,
      loa_upper = 0.800001959999998,
      loa_lower_lwr = 0.799997601738728,
      loa_lower_upr = 0.799998478261268,
      loa_upper_lwr = 0.800001521738728,
      loa_upper_upr = 0.800002398261268,
      ar1_rho = NA_real_,
      ar1_estimated = FALSE
    )
  )

  dat_iid <- sim_two_method_known(S = 22L, Tm = 9L, mu = 1.1, sig_s = 0.9, sig_e = 1.3, seed = 42L)
  fit_iid <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat_iid$y,
    subject = dat_iid$subject,
    method = as.integer(dat_iid$method == "M2") + 1L,
    time = dat_iid$time,
    include_slope = FALSE,
    use_ar1 = FALSE,
    conf_level = 0.95,
    loa_multiplier_arg = 1.96
  )
  expect_close_to_reference(extract_fit(fit_iid), expected$iid_boundary, tol = 5e-6)

  dat_ar1_fixed <- sim_two_method_known(S = 18L, Tm = 8L, mu = 0.6, sig_s = 1.0, sig_e = 0.8, rho = 0.35, seed = 314L)
  fit_ar1_fixed <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat_ar1_fixed$y,
    subject = dat_ar1_fixed$subject,
    method = as.integer(dat_ar1_fixed$method == "M2") + 1L,
    time = dat_ar1_fixed$time,
    include_slope = TRUE,
    use_ar1 = TRUE,
    ar1_rho = 0.35,
    conf_level = 0.90,
    loa_multiplier_arg = 2.0
  )
  expect_close_to_reference(extract_fit(fit_ar1_fixed), expected$ar1_fixed_slope, tol = 5e-6)

  dat_ar1_profiled <- sim_two_method_known(S = 24L, Tm = 7L, mu = 0.25, sig_s = 0.7, sig_e = 1.1, rho = 0.55, seed = 20260401L)
  fit_ar1_profiled <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat_ar1_profiled$y,
    subject = dat_ar1_profiled$subject,
    method = as.integer(dat_ar1_profiled$method == "M2") + 1L,
    time = dat_ar1_profiled$time,
    include_slope = FALSE,
    use_ar1 = TRUE,
    ar1_rho = NA_real_,
    conf_level = 0.95,
    loa_multiplier_arg = 1.96
  )
  expect_close_to_reference(extract_fit(fit_ar1_profiled), expected$ar1_profiled, tol = 5e-6)

  S <- 10L
  Tm <- 2L
  subject_means <- seq(12, 30, length.out = S)
  means <- rep(subject_means, each = Tm)
  diffs <- rep(0.8 + 0.25 * (subject_means - mean(subject_means)), each = Tm)
  dat_boundary <- sim_two_method_from_pairs(means, diffs, S = S, Tm = Tm)
  fit_boundary <- matrixCorr:::bland_altman_repeated_em_ext_cpp(
    y = dat_boundary$y,
    subject = dat_boundary$subject,
    method = as.integer(dat_boundary$method == "B") + 1L,
    time = dat_boundary$time,
    include_slope = TRUE,
    use_ar1 = FALSE,
    conf_level = 0.95,
    loa_multiplier_arg = 1.96
  )
  expect_close_to_reference(extract_fit(fit_boundary), expected$exact_boundary, tol = 1e-8)
  expect_equal(as.numeric(fit_boundary$sigma2_subject), 0, tolerance = 1e-12)
})

test_that("ccc_rm_ustat: basic structure, symmetry, CI container", {
  set.seed(123)
  n_subj <- 600L
  id     <- factor(rep(seq_len(n_subj), each = 2L))
  method <- factor(rep(c("A","B"), times = n_subj))

  # model: y_A = u + e ; y_B = b + u + e
  sigA <- 1.0   # subject variance
  sigE <- 0.5   # error variance
  biasB <- 0.2  # fixed method bias for B
  u <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
  e <- rnorm(n_subj * 2L, 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + e
  df <- data.frame(y, id, method)

  # Theoretical CCC for 2 methods, T = 1, no AxM/AxT:
  # N = sA; D = sA + S_B + sE
  ccc_theory <- sigA / (sigA + biasB^2 + sigE)

  # estimates only
  c1 <- ccc_rm_ustat(df, response = "y", subject = "id", method = "method")
  expect_s3_class(c1, "ccc")
  expect_true(is.matrix(c1) && all(rownames(c1) == c("A","B")))
  expect_equal(as.numeric(diag(c1)), c(1,1))
  expect_equal(c1["A","B"], c1["B","A"])
  expect_true(c1["A","B"] > 0 && c1["A","B"] < 1)

  # within Monte Carlo tolerance
  expect_lt(abs(c1["A","B"] - ccc_theory), 0.05)

  # with CI container
  c2 <- ccc_rm_ustat(df, response = "y", subject = "id", method = "method", ci = TRUE, conf_level = 0.95)
  expect_s3_class(c2, "ccc_ci")
  expect_named(c2, c("est","lwr.ci","upr.ci"))
  expect_equal(dim(c2$est), c(2,2))
  expect_true(is.na(diag(c2$lwr.ci))[1] && is.na(diag(c2$upr.ci))[1])
  est <- c2$est["A","B"]; lwr <- c2$lwr.ci["A","B"]; upr <- c2$upr.ci["A","B"]
  expect_true(lwr <= est && est <= upr)
})

test_that("ccc_rm_ustat uses the aligned repeated-measures argument order", {
  set.seed(124)
  n_subj <- 80L
  n_time <- 3L
  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  u <- rnorm(n_subj, 0, 1)[as.integer(id)]
  y <- u + 0.2 * (method == "B") + rnorm(length(id), 0, 0.4)
  df <- data.frame(y, id, method, time)

  fit_named <- ccc_rm_ustat(
    df,
    response = "y",
    subject = "id",
    method = "method",
    time = "time"
  )
  fit_positional <- ccc_rm_ustat(
    df,
    "y",
    "id",
    "method",
    "time"
  )

  expect_equal(unname(fit_named["A", "B"]), unname(fit_positional["A", "B"]))
})

test_that("ccc_rm_reml (pairwise, no time): matches simple theory and returns VCs", {
  set.seed(123)
  n_subj <- 500L
  id     <- factor(rep(seq_len(n_subj), each = 2L))
  method <- factor(rep(c("A","B"), times = n_subj))

  sigA <- 1.0
  sigE <- 0.5
  biasB <- 0.2
  u <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
  e <- rnorm(n_subj * 2L, 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + e
  df <- data.frame(y, id, method)

  cfit <- ccc_rm_reml(df, response = "y", subject = "id",
                       method = "method", ci = TRUE)
  expect_s3_class(cfit, "ccc_rm_reml")
  expect_named(cfit, c("est","lwr.ci","upr.ci"))
  expect_equal(rownames(cfit$est), c("A","B"))
  expect_equal(colnames(cfit$est), c("A","B"))
  expect_equal(as.numeric(diag(cfit$est)), c(1,1))

  # theory for T=1 (no AxM/AxT): sA / (sA + b^2 + sE)
  ccc_theory <- sigA / (sigA + biasB^2 + sigE)
  est <- cfit$est["A","B"]
  expect_lt(abs(est - ccc_theory), 0.05)

  # variance-component attributes exist and are numeric matrices
  for (nm in c("sigma2_subject","sigma2_subject_method","sigma2_subject_time",
               "sigma2_error","SB","se_ccc")) {
    v <- attr(cfit, nm)
    expect_true(is.matrix(v))
    expect_equal(dim(v), c(2,2))
  }

  out_print <- capture.output(print(cfit))
  expect_false(any(grepl("sigma2_subject|sigma2_subject_method|sigma2_subject_time|sigma2_error|SB|se_ccc|Variance components|AR\\(1\\) diagnostics",
                         out_print)))

  # summary data frame columns present
  sm <- summary(cfit, show_ci = "yes", digits = 4)
  expect_s3_class(sm, "summary.ccc_rm_reml")
  expect_true(all(c("item1","item2","estimate","lwr","upr","n_subjects","n_obs",
                    "sigma2_subject","sigma2_subject_method","sigma2_subject_time",
                    "sigma2_error","SB","se_ccc") %in% names(sm)))
  out <- capture.output(print(sm))
  expect_true(any(grepl("^Concordance estimates$", out)))
  expect_true(any(grepl("^Variance components$", out)))
})

test_that("ccc_rm_reml no-time auto path returns safely with fallback structure", {
  set.seed(123)
  n_subj <- 200L
  id <- factor(rep(seq_len(n_subj), each = 2L))
  method <- factor(rep(c("A", "B"), times = n_subj))

  sigA <- 1.0
  sigE <- 0.5
  biasB <- 0.2
  u <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
  e <- rnorm(n_subj * 2L, 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + e
  df <- data.frame(y, id, method)

  fit_auto <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    ci = TRUE,
    vc_select = "auto"
  )
  fit_none <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    ci = TRUE,
    vc_select = "none"
  )

  expect_s3_class(fit_auto, "ccc_rm_reml")
  expect_named(fit_auto, c("est", "lwr.ci", "upr.ci"))
  expect_equal(dim(fit_auto$est), c(2L, 2L))
  expect_equal(unname(diag(fit_auto$est)), c(1, 1))
  expect_match(attr(fit_auto, "method"), "no-time fallback", fixed = TRUE)
  expect_match(attr(fit_auto, "description"), "fallback via ccc_rm_ustat", fixed = TRUE)

  for (nm in c("sigma2_subject", "sigma2_subject_method", "sigma2_subject_time",
               "sigma2_error", "SB", "se_ccc", "n_obs", "n_subjects")) {
    v <- attr(fit_auto, nm)
    expect_true(is.matrix(v))
    expect_equal(dim(v), c(2L, 2L))
  }

  expect_equal(fit_auto$est, fit_none$est, tolerance = 1e-6)
})

test_that("ccc_vc_cpp handles absent optional method/time vectors safely", {
  set.seed(125)
  n_subj <- 20L
  id <- rep(seq_len(n_subj), each = 2L)
  method <- rep(c(1L, 2L), times = n_subj)
  y <- rnorm(length(id))
  X <- cbind("(Intercept)" = 1, method_b = as.numeric(method == 2L))

  expect_error(
    fit <- matrixCorr:::ccc_vc_cpp(
      Xr = X,
      yr = y,
      subject = id,
      method = method,
      time = integer(0),
      nm = 2L,
      nt = 0L,
      max_iter = 20L,
      tol = 1e-6,
      conf_level = 0.95,
      ci_mode = 2L,
      Lr = matrix(c(0, 1), nrow = 2L),
      auxDr = matrix(1, nrow = 1L, ncol = 1L),
      Zr = NULL,
      use_ar1 = FALSE,
      ar1_rho = 0,
      include_subj_method = FALSE,
      include_subj_time = FALSE,
      sb_zero_tol = 1e-10,
      eval_single_visit = FALSE,
      time_weights = NULL,
      metric_mode = 0L,
      ll_only = FALSE,
      need_loglik = TRUE
    ),
    NA
  )
  expect_type(fit, "list")
  expect_true("ccc" %in% names(fit))
  expect_true(is.finite(as.numeric(fit$ccc)))
  expect_true("sigma2_subject" %in% names(fit))
  expect_true(is.finite(as.numeric(fit$sigma2_subject)))
})

test_that("ccc_vc_cpp S_B uses undirected method-pair denominator", {
  n_subj <- 80L
  mu <- c(10, 12)
  subject <- rep(seq_len(n_subj), each = 2L)
  method <- rep(1:2, times = n_subj)
  time <- rep(1L, length(subject))
  subject_effect <- rep(seq(-1, 1, length.out = n_subj), each = 2L)
  y <- subject_effect + mu[method]
  X <- cbind(1, as.numeric(method == 2L))
  colnames(X) <- c("(Intercept)", "methodB")

  Laux <- matrixCorr:::build_L_Dm_cpp(
    colnames_X = colnames(X),
    rmet_name = "method",
    rtime_name = NULL,
    method_levels = c("A", "B"),
    time_levels = character(0),
    has_interaction = FALSE,
    Dmat_global = NULL
  )
  expect_equal(ncol(Laux$L), 1L)

  fit <- matrixCorr:::ccc_vc_cpp(
    Xr = X,
    yr = y,
    subject = as.integer(subject),
    method = as.integer(method),
    time = as.integer(time),
    nm = 2L,
    nt = 1L,
    max_iter = 200L,
    tol = 1e-6,
    conf_level = 0.95,
    ci_mode = 2L,
    Lr = Laux$L,
    auxDr = Laux$Dm,
    Zr = NULL,
    use_ar1 = FALSE,
    ar1_rho = 0,
    include_subj_method = FALSE,
    include_subj_time = FALSE,
    sb_zero_tol = 0,
    eval_single_visit = FALSE,
    time_weights = NULL,
    metric_mode = 0L,
    ll_only = FALSE,
    need_loglik = TRUE
  )

  expected_sb <- (mu[[1L]] - mu[[2L]])^2
  expect_equal(as.numeric(fit$SB), expected_sb, tolerance = 1e-6)
  expect_gt(abs(as.numeric(fit$SB) - expected_sb / 2), 1)
})

test_that("ccc_vc_cpp rejects malformed time and optional design inputs before native indexing", {
  set.seed(126)
  n_subj <- 12L
  id <- rep(seq_len(n_subj), each = 2L)
  method <- rep(c(1L, 2L), times = n_subj)
  y <- rnorm(length(id))
  X <- cbind("(Intercept)" = 1, method_b = as.numeric(method == 2L))

  expect_error(
    matrixCorr:::ccc_vc_cpp(
      Xr = X,
      yr = y,
      subject = id,
      method = method,
      time = c(rep(1L, length(id) - 1L), 3L),
      nm = 2L,
      nt = 2L,
      max_iter = 20L,
      tol = 1e-6,
      conf_level = 0.95,
      ci_mode = 2L,
      Lr = matrix(c(0, 1), nrow = 2L),
      auxDr = matrix(1, nrow = 1L, ncol = 1L),
      Zr = NULL,
      use_ar1 = FALSE,
      ar1_rho = 0,
      include_subj_method = TRUE,
      include_subj_time = FALSE,
      sb_zero_tol = 1e-10,
      eval_single_visit = FALSE,
      time_weights = NULL,
      metric_mode = 0L,
      ll_only = FALSE,
      need_loglik = TRUE
    ),
    "contiguous 1..K integer codes|maximum"
  )

  expect_error(
    matrixCorr:::ccc_vc_cpp(
      Xr = X,
      yr = y,
      subject = id,
      method = method,
      time = integer(0),
      nm = 2L,
      nt = 0L,
      max_iter = 20L,
      tol = 1e-6,
      conf_level = 0.95,
      ci_mode = 2L,
      Lr = matrix(c(0, 1), nrow = 2L),
      auxDr = matrix(1, nrow = 1L, ncol = 1L),
      Zr = matrix(numeric(0), nrow = length(id), ncol = 0L),
      use_ar1 = FALSE,
      ar1_rho = 0,
      include_subj_method = TRUE,
      include_subj_time = TRUE,
      sb_zero_tol = 1e-10,
      eval_single_visit = FALSE,
      time_weights = NULL,
      metric_mode = 0L,
      ll_only = FALSE,
      need_loglik = TRUE
    ),
    "include_subj_time=TRUE"
  )

  expect_error(
    matrixCorr:::ccc_vc_cpp(
      Xr = X,
      yr = y,
      subject = id,
      method = method,
      time = rep(1L, length(id)),
      nm = 2L,
      nt = 1L,
      max_iter = 20L,
      tol = 1e-6,
      conf_level = 0.95,
      ci_mode = 2L,
      Lr = matrix(c(0, 1), nrow = 2L),
      auxDr = matrix(1, nrow = 1L, ncol = 1L),
      Zr = NULL,
      use_ar1 = FALSE,
      ar1_rho = 0,
      include_subj_method = TRUE,
      include_subj_time = FALSE,
      sb_zero_tol = 1e-10,
      eval_single_visit = FALSE,
      time_weights = c(0.5, 0.5),
      metric_mode = 0L,
      ll_only = FALSE,
      need_loglik = TRUE
    ),
    "time_weights length"
  )
})

test_that("repeated-measures REML wrappers reject malformed optional design state cleanly", {
  bad <- data.frame(
    y = rnorm(6),
    id = factor(rep(1:3, each = 2L)),
    method = factor(rep(c("A", "B"), times = 3L)),
    time = factor(c("1", "1", "1", NA, "1", "1"))
  )
  expect_error(
    ccc_rm_reml(bad, "y", "id", method = "method", time = "time"),
    "must not contain missing values"
  )

  dat <- data.frame(
    y = rnorm(8),
    id = factor(rep(1:4, each = 2L)),
    method = factor(rep(c("A", "B"), times = 4L))
  )
  expect_error(
    ccc_rm_reml(
      dat,
      "y",
      "id",
      method = "method",
      include_subj_time = TRUE,
      vc_select = "none"
    ),
    "cannot be TRUE when"
  )

  expect_error(
    ccc_rm_reml(dat, "y", "id", method = "method", time = "time"),
    "missing required column"
  )
})

test_that("pairwise repeated-measures REML rebuilds pair-specific time/design state", {
  set.seed(130)
  ids <- factor(rep(seq_len(18L), each = 6L))
  dat <- data.frame(
    id = ids,
    method = factor(rep(c("A", "A", "B", "B", "C", "C"), times = 18L),
                    levels = c("A", "B", "C")),
    time = factor(rep(c("1", "2", "1", "3", "2", "3"), times = 18L),
                  levels = c("1", "2", "3"))
  )
  subj_eff <- rnorm(nlevels(dat$id), sd = 0.6)[dat$id]
  method_eff <- c(A = 0, B = 0.25, C = -0.15)[as.character(dat$method)]
  time_eff <- c("1" = 0.0, "2" = 0.2, "3" = -0.1)[as.character(dat$time)]
  dat$y <- subj_eff + method_eff + time_eff + rnorm(nrow(dat), sd = 0.35)

  fit <- ccc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "auto",
    ci = FALSE
  )

  expect_true(all(is.finite(fit[upper.tri(fit)])))
})

test_that("single-level time variables are handled as no-time repeated-measures fits", {
  set.seed(131)
  n_subj <- 30L
  dat <- data.frame(
    y = rnorm(n_subj * 2L),
    id = factor(rep(seq_len(n_subj), each = 2L)),
    method = factor(rep(c("A", "B"), times = n_subj)),
    time = factor(rep("visit1", n_subj * 2L))
  )

  fit <- ccc_rm_reml(
    dat,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "auto",
    ci = FALSE
  )

  expect_true(is.finite(unname(fit["A", "B"])))
  expect_equal(unname(attr(fit, "sigma2_subject_time")["A", "B"]), 0, tolerance = 1e-8)
})

test_that("native slope builders reject malformed slope state before use", {
  expect_error(
    matrixCorr:::build_L_Dm_Z_cpp(
      colnames_X = c("(Intercept)", "methodB"),
      rmet_name = "method",
      rtime_name = NULL,
      method_levels = c("A", "B"),
      time_levels = character(0),
      has_interaction = FALSE,
      Dmat_global = NULL,
      slope_mode = "method",
      slope_var = c(1, Inf),
      method_codes = c(1L, 2L),
      drop_zero_cols = TRUE
    ),
    "slope_var"
  )

  expect_error(
    matrixCorr:::build_L_Dm_Z_cpp(
      colnames_X = c("(Intercept)", "methodB"),
      rmet_name = "method",
      rtime_name = NULL,
      method_levels = c("A", "B"),
      time_levels = character(0),
      has_interaction = FALSE,
      Dmat_global = NULL,
      slope_mode = "method",
      slope_var = c(1, 2),
      method_codes = c(1L, 3L),
      drop_zero_cols = TRUE
    ),
    "method_codes"
  )
})

test_that("randomized repeated-measures REML stress cases never crash", {
  simulate_case <- function(seed) {
    set.seed(seed)
    n_subj <- sample(2:7, 1)
    n_methods <- sample(2:4, 1)
    has_time <- sample(c(TRUE, FALSE), 1)
    n_time <- if (has_time) sample(1:4, 1) else 1L
    methods <- LETTERS[seq_len(n_methods)]
    rows <- vector("list", n_subj * n_methods)
    kk <- 0L
    subj_eff <- rnorm(n_subj, sd = runif(1, 0, 0.8))
    for (s in seq_len(n_subj)) {
      for (m in seq_len(n_methods)) {
        keep_t <- if (has_time) sort(sample(seq_len(n_time), sample(seq_len(n_time), 1))) else 1L
        y <- subj_eff[s] + (m - 1) * runif(1, -0.3, 0.3) + rnorm(length(keep_t), sd = runif(1, 0, 0.6))
        if (runif(1) < 0.1) y[] <- y[1]
        kk <- kk + 1L
        rows[[kk]] <- data.frame(
          y = y,
          id = factor(s),
          method = factor(methods[m], levels = methods),
          time = factor(keep_t, levels = seq_len(n_time))
        )
      }
    }
    dat <- do.call(rbind, rows)
    if (!has_time) dat$time <- NULL
    dat
  }

  for (seed in 1:20) {
    dat <- simulate_case(900 + seed)
    fit <- tryCatch(
      ccc_rm_reml(
        dat,
        "y",
        "id",
        method = "method",
        time = if ("time" %in% names(dat)) "time" else NULL,
        ci = (seed %% 2L) == 0L,
        vc_select = if ((seed %% 3L) == 0L) "auto" else "none",
        include_subj_time = if (!("time" %in% names(dat)) && (seed %% 5L) == 0L) TRUE else NULL
      ),
      error = identity
    )
    if (inherits(fit, "error")) {
      expect_s3_class(fit, "error")
    } else {
      est <- if (is.list(fit)) fit$est else fit
      expect_true(all(is.finite(est[upper.tri(est)])))
    }
  }
})

test_that("ccc_rm_reml handles CRAN REML branch combinations without crashing", {
  set.seed(102)
  n_subj <- 60L
  n_time <- 8L

  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))

  sigA <- 0.6
  sigAM <- 0.3
  sigAT <- 0.5
  sigE <- 0.4
  biasB <- 0.2

  u_i <- rnorm(n_subj, 0, sqrt(sigA))
  u <- u_i[as.integer(id)]
  sm <- interaction(id, method, drop = TRUE)
  w_im_lv <- rnorm(nlevels(sm), 0, sqrt(sigAM))
  w_im <- w_im_lv[as.integer(sm)]
  st <- interaction(id, time, drop = TRUE)
  g_it_lv <- rnorm(nlevels(st), 0, sqrt(sigAT))
  g_it <- g_it_lv[as.integer(st)]
  e <- rnorm(length(id), 0, sqrt(sigE))
  y <- (method == "B") * biasB + u + w_im + g_it + e
  dat_both <- data.frame(y, id, method, time)

  fit_auto <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "auto",
    ci = FALSE
  )
  fit_full <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "none",
    ci = FALSE
  )
  fit_no_st <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "none",
    include_subj_time = FALSE,
    ci = TRUE
  )
  fit_no_sm <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "none",
    include_subj_method = FALSE,
    ci = FALSE
  )
  fit_none <- ccc_rm_reml(
    dat_both,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "none",
    include_subj_method = FALSE,
    include_subj_time = FALSE,
    ci = FALSE
  )

  for (fit in list(fit_auto, fit_no_st, fit_no_sm, fit_none)) {
    est <- if (is.list(fit)) fit$est["A", "B"] else fit["A", "B"]
    expect_true(is.finite(unname(est)))
  }
  expect_equal(unname(fit_auto["A", "B"]), unname(fit_full["A", "B"]), tolerance = 1e-8)

  set.seed(11)
  ids <- seq_len(24L)
  methods <- c("A", "B", "C")
  rows <- vector("list", length(ids) * length(methods))
  kk <- 0L
  subj_eff <- rnorm(length(ids), 0, 0.5)
  subj_time_eff <- matrix(rnorm(length(ids) * 5L, 0, 0.7), length(ids), 5L)
  for (s in ids) {
    for (m in seq_along(methods)) {
      keep_t <- sort(sample(seq_len(5L), size = max(2L, ceiling((1 - 0.4) * 5L))))
      eps <- as.numeric(stats::arima.sim(list(ar = 0.1), n = length(keep_t), sd = 0.8))
      y_keep <- subj_eff[s] + subj_time_eff[s, keep_t] +
        (m - 1) * 0.5 + eps
      kk <- kk + 1L
      rows[[kk]] <- data.frame(
        y = y_keep,
        id = factor(s),
        method = factor(methods[m], levels = methods),
        time = factor(keep_t, levels = seq_len(5L))
      )
    }
  }
  dat_unbalanced <- do.call(rbind, rows)
  fit_unbalanced <- ccc_rm_reml(
    dat_unbalanced,
    "y",
    "id",
    method = "method",
    time = "time",
    vc_select = "auto",
    ci = FALSE
  )
  expect_true(is.finite(unname(fit_unbalanced["A", "B"])))
})

# helper to center time within subject
center_by_id <- function(x, id) ave(x, id, FUN = function(v) v - mean(v))

sim_ccc_rm_dat <- function(seed, rho = 0, n_subj = 120L, n_time = 6L,
                           sig_subject = 1, sig_error = 0.7, bias_b = 0.2) {
  set.seed(seed)
  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))
  subj_eff <- rnorm(n_subj, 0, sig_subject)
  y <- numeric(length(id))

  for (s in seq_len(n_subj)) {
    for (m in c("A", "B")) {
      idx <- which(id == levels(id)[s] & method == m)
      eps <- if (abs(rho) < 1e-12) {
        rnorm(n_time, 0, sig_error)
      } else {
        as.numeric(stats::arima.sim(list(ar = rho), n = n_time, sd = sig_error))
      }
      y[idx] <- subj_eff[s] + (m == "B") * bias_b + eps
    }
  }

  data.frame(y = y, id = id, method = method, time = time)
}

sim_ccc_rm_irregular <- function(seed, rho = 0, n_subj = 30L, n_time = 6L, n_method = 3L,
                                 miss = 0.35, sig_subject = 0.5, sig_error = 0.8,
                                 bias_scale = 0.5, time_effect = 1.0,
                                 sig_subject_time = 0.7) {
  set.seed(seed)
  ids <- seq_len(n_subj)
  methods <- LETTERS[seq_len(n_method)]
  rows <- vector("list", n_subj * n_method)
  kk <- 0L
  subj_eff <- rnorm(n_subj, 0, sig_subject)
  subj_time_eff <- matrix(rnorm(n_subj * n_time, 0, sig_subject_time), n_subj, n_time)

  for (s in ids) {
    for (m in seq_along(methods)) {
      keep_t <- sort(sample(seq_len(n_time), size = max(2L, ceiling((1 - miss) * n_time))))
      eps <- if (abs(rho) < 1e-12) {
        rnorm(length(keep_t), 0, sig_error)
      } else {
        as.numeric(stats::arima.sim(list(ar = rho), n = length(keep_t), sd = sig_error))
      }
      y <- subj_eff[s] + subj_time_eff[s, keep_t] +
        time_effect * (keep_t - mean(seq_len(n_time))) / n_time +
        (m - 1) * bias_scale + eps
      kk <- kk + 1L
      rows[[kk]] <- data.frame(
        y = y,
        id = factor(s),
        method = factor(methods[m], levels = methods),
        time = factor(keep_t, levels = seq_len(n_time))
      )
    }
  }

  do.call(rbind, rows[seq_len(kk)])
}

ccc_rm_raw_ar1_recommend <- function(row) {
  scalar_or_na <- function(x) {
    if (is.null(x) || !length(x)) return(NA_real_)
    suppressWarnings(as.numeric(x[[1L]]))
  }

  sag_hat <- scalar_or_na(row[["sigma2_subject_time"]])
  se_hat  <- scalar_or_na(row[["sigma2_error"]])
  rho_hat <- scalar_or_na(row[["ar1_rho_lag1"]])
  pval    <- scalar_or_na(row[["ar1_pval"]])

  sag_share <- if (is.finite(sag_hat) && is.finite(se_hat)) {
    sag_hat / max(1e-12, sag_hat + se_hat)
  } else {
    0
  }
  thr   <- if (sag_share > 0.25) 0.20 else 0.10
  p_thr <- if (sag_share > 0.25) 0.01 else 0.05

  is.finite(rho_hat) && is.finite(pval) && rho_hat >= thr && pval < p_thr
}

find_ccc_rm_false_positive_candidate <- function() {
  param_grid <- list(
    list(n_subj = 30L, n_time = 6L, n_method = 3L, miss = 0.35, sig_subject = 0.5, sig_error = 0.8, bias_scale = 0.5, time_effect = 1.0, sig_subject_time = 0.7),
    list(n_subj = 24L, n_time = 6L, n_method = 3L, miss = 0.25, sig_subject = 0.4, sig_error = 0.9, bias_scale = 0.6, time_effect = 0.9, sig_subject_time = 0.8),
    list(n_subj = 36L, n_time = 5L, n_method = 3L, miss = 0.30, sig_subject = 0.6, sig_error = 0.7, bias_scale = 0.5, time_effect = 1.1, sig_subject_time = 0.6)
  )

  for (params in param_grid) {
    for (seed in 1:80) {
      dat <- do.call(sim_ccc_rm_irregular, c(list(seed = seed, rho = 0), params))
      fit <- suppressMessages(
        ccc_rm_reml(dat, "y", "id", method = "method", time = "time", ar = "none")
      )
      sm <- as.data.frame(summary(fit))
      raw_reco <- vapply(seq_len(nrow(sm)), function(i) ccc_rm_raw_ar1_recommend(sm[i, , drop = FALSE]), logical(1))
      final_reco <- sm$use_ar1 %in% TRUE
      hit <- which(raw_reco & !final_reco)
      if (length(hit)) {
        return(list(
          seed = seed,
          params = params,
          summary = sm[hit[1], , drop = FALSE]
        ))
      }
    }
  }

  NULL
}

test_that("Dmat_type affects CCC as expected when biases flip over time", {
  set.seed(42)
  n_subj <- 200L
  n_time <- 6L
  id     <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time   <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))

  sigA  <- 0.8
  sigAT <- 0.3
  sigE  <- 0.4

  # time-varying bias for B: +b, -b, +b, -b, ...
  b0 <- 0.35
  tnum <- as.integer(time)
  bias_t <- ifelse(tnum %% 2L == 1L,  b0, -b0)
  bias  <- ifelse(method == "B", bias_t, 0)

  u  <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
  gIT <- rnorm(n_subj * n_time, 0, sqrt(sigAT))
  g  <- gIT[(as.integer(id) - 1L) * n_time + as.integer(time)]
  y  <- bias + u + g + rnorm(length(id), 0, sqrt(sigE))
  df <- data.frame(y, id, method, time)

  fit_avg <- suppressMessages(
    ccc_rm_reml(df, "y", "id", method = "method", time = "time",
                Dmat_type = "time-avg")
  )
  fit_typ <- suppressMessages(
    ccc_rm_reml(df, "y", "id", method = "method", time = "time",
                Dmat_type = "typical-visit")
  )

  # With alternating biases, squared-average is ~0, average of squares > 0,
  # so CCC(time-avg) should be >= CCC(typical-visit)
  c_avg <- fit_avg["A","B"]; c_typ <- fit_typ["A","B"]
  expect_gte(c_avg, c_typ)
})

test_that("Supplying a wrong-sized Dmat errors clearly", {
  set.seed(1)
  n_subj <- 10L; n_time <- 3L
  id     <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time   <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
  y <- rnorm(length(id))
  df <- data.frame(y, id, method, time)

  badD <- diag(5L)   # wrong dimension
  expect_error(
    ccc_rm_reml(df, "y", "id", method = "method", time = "time", Dmat = badD),
    "Dmat has incompatible dimension", fixed = FALSE
  )
})

test_that("summary adds sigma2_extra* columns when slope is enabled", {
  set.seed(2)
  n_subj <- 120L; n_time <- 4L
  id     <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time   <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))

  # Build a genuine slope effect so sigma2_extra > 0
  tnum <- as.integer(time)
  t_c  <- center_by_id(tnum, id)
  slope_subj <- rnorm(n_subj, 0, 0.15)[as.integer(id)]

  y <- 0.3 * (method == "B") +
    slope_subj * t_c +
    rnorm(length(id), 0, 0.6)

  df <- data.frame(y, id, method, time, t_c = t_c)

  fit_slope <- ccc_rm_reml(df, "y", "id", method = "method", time = "time",
                            slope = "subject", slope_var = "t_c", ci = TRUE)

  # Summary should expose sigma2_extra columns
  sm <- summary(fit_slope, show_ci = "yes")
  extra_cols <- grep("^sigma2_extra", names(sm), value = TRUE)
  expect_true(length(extra_cols) >= 1)
  expect_true(any(is.finite(sm[[extra_cols[1]]])))

  # In the no-slope case, those columns should not be present
  fit_noslope <- ccc_rm_reml(df, "y", "id", method = "method", time = "time", ci = TRUE)
  sm0 <- summary(fit_noslope, show_ci = "yes")
  expect_length(grep("^sigma2_extra", names(sm0), value = TRUE), 0)
})

test_that("AR(1) path: fixed rho is carried in attributes", {
  set.seed(10)
  n_subj <- 50L; n_time <- 6L
  id     <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  time   <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))

  rho_true <- 0.6
  # simulate AR(1) per (subject,method) run
  y <- numeric(length(id))
  for (s in seq_len(n_subj)) {
    for (m in c("A","B")) {
      idx <- which(id == levels(id)[s] & method == m)
      y[idx] <- 0.2 * (m == "B") +
        as.numeric(stats::arima.sim(list(ar = rho_true), n = n_time, sd = 0.7))
    }
  }
  df <- data.frame(y, id, method, time)

  fit_ar <- ccc_rm_reml(df, "y", "id", method = "method", time = "time",
                         ar = "ar1", ar_rho = rho_true)
  rr <- attr(fit_ar, "ar_rho")
  expect_true(is.matrix(rr))
  expect_equal(rr["A","B"], rho_true, tolerance = 1e-12)

  # summary returns AR1 diagnostics columns (may have NAs but should exist)
  sm <- summary(fit_ar)
  expect_true("ar1_rho" %in% names(sm) || "ar1_rho_lag1" %in% names(sm))
  out <- capture.output(print(sm))
  expect_true(any(grepl("^AR\\(1\\) diagnostics$", out)))
})

test_that("AR(1) recommendation distinguishes IID from positive serial correlation", {
  dat_iid <- sim_ccc_rm_dat(seed = 7, rho = 0)
  fit_iid <- expect_no_message(
    ccc_rm_reml(dat_iid, "y", "id", method = "method", time = "time", ar = "none")
  )
  sm_iid <- as.data.frame(summary(fit_iid))
  expect_false(isTRUE(sm_iid$use_ar1[1]))
  expect_lt(sm_iid$ar1_rho_lag1[1], 0)

  dat_ar1 <- sim_ccc_rm_dat(seed = 1, rho = 0.6)
  fit_ar1_diag <- suppressMessages(
    ccc_rm_reml(dat_ar1, "y", "id", method = "method", time = "time", ar = "none")
  )
  sm_ar1 <- as.data.frame(summary(fit_ar1_diag))
  expect_true(isTRUE(sm_ar1$use_ar1[1]))
  expect_gt(sm_ar1$ar1_rho_lag1[1], 0.1)
})

test_that("AR(1) fallback message aligns with ba_rm wording", {
  expect_message(
    matrixCorr:::inform_ccc_rm_ar1_fallback(pair_label = "A vs B", .verbose = TRUE),
    "Requested AR\\(1\\) residual structure could not be fit for pair\\(s\\): A vs B; using iid residuals instead\\."
  )
})

test_that("AR(1) recommendation is not triggered by an IID irregular-panel false positive", {
  cand <- find_ccc_rm_false_positive_candidate()
  skip_if(is.null(cand), "No IID irregular-panel raw false-positive candidate found on this platform/configuration.")

  row <- cand$summary[1, , drop = FALSE]
  expect_true(ccc_rm_raw_ar1_recommend(row))
  expect_false(isTRUE(row$use_ar1))
})


ccc_lin <- function(x, y, na.rm = TRUE) {
  stopifnot(length(x) == length(y))
  if (na.rm) {
    keep <- is.finite(x) & is.finite(y)
    x <- x[keep]; y <- y[keep]
  }
  if (length(x) < 2L) return(NA_real_)
  mx <- mean(x); my <- mean(y)
  vx <- stats::var(x); vy <- stats::var(y)
  sxy <- stats::cov(x, y)
  den <- vx + vy + (mx - my)^2
  if (!is.finite(den) || den <= 0) return(NA_real_)
  2 * sxy / den
}

test_that("ccc_rm_ustat reduces to Lin's CCC when T = 1", {
  set.seed(123)
  n <- 1500L
  u  <- rnorm(n, 0, 1.0)
  eA <- rnorm(n, 0, 0.7)
  eB <- rnorm(n, 0, 0.7)
  bias <- 0.3
  xA <- u + eA
  xB <- bias + u + eB

  # Lin on paired data
  c_lin <- ccc_lin(xA, xB)

  df <- data.frame(
    id     = factor(rep(seq_len(n), each = 2L)),
    method = factor(rep(c("A","B"), times = n)),
    time   = factor(rep(1L, 2L * n))
  )

  # INTERLEAVE A,B per subject (A1,B1,A2,B2,...)
  df$y <- c(rbind(xA, xB))

  c_us <- ccc_rm_ustat(df, response = "y", subject = "id", method = "method")
  c_us_AB <- unname(c_us["A","B"])

  # should match Lin very closely
  expect_equal(c_us_AB, c_lin, tolerance = 5e-3)
})

test_that("ccc_rm_reml (no time) approximates Lin's CCC when T = 1", {
  set.seed(124)
  n <- 1500L
  u  <- rnorm(n, 0, 1.0)
  eA <- rnorm(n, 0, 0.7)
  eB <- rnorm(n, 0, 0.7)
  bias <- 0.25
  xA <- u + eA
  xB <- bias + u + eB
  c_lin <- ccc_lin(xA, xB)

  df <- data.frame(
    id     = factor(rep(seq_len(n), each = 2L)),
    method = factor(rep(c("A","B"), times = n)),
    y      = c(rbind(xA, xB))
  )

  fit <- ccc_rm_reml(df, response = "y", subject = "id", method = "method")
  c_reml <- unname(fit["A","B"])

  expect_equal(c_reml, c_lin, tolerance = 1e-2)
})

test_that("repeated-measures CCC uses the subject argument name", {
  set.seed(432)
  n_subj <- 40L
  n_time <- 3L
  id <- factor(rep(seq_len(n_subj), each = 2L * n_time))
  method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_subj))
  time <- factor(rep(rep(seq_len(n_time), times = 2L), times = n_subj))
  u <- rnorm(n_subj, 0, 1)[as.integer(id)]
  y <- u + 0.15 * (method == "B") + rnorm(length(id), 0, 0.3)
  df <- data.frame(y, id, method, time)

  fit_named <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = FALSE
  )
  fit_positional <- ccc_rm_reml(
    df,
    "y",
    "id",
    method = "method",
    time = "time",
    ci = FALSE
  )

  expect_equal(unname(fit_named["A", "B"]), unname(fit_positional["A", "B"]))
})

test_that("repeated-measures CCC honors n_threads without changing estimates", {
  set.seed(433)
  df <- sim_ccc_rm_dat(seed = 9, rho = 0.2, n_subj = 50L, n_time = 4L)

  fit1 <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    n_threads = 1L,
    ci = FALSE
  )
  fit2 <- ccc_rm_reml(
    df,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    n_threads = 2L,
    ci = FALSE
  )

  expect_equal(unname(fit1["A", "B"]), unname(fit2["A", "B"]), tolerance = 1e-12)
})

manual_multirater_kappa_counts <- function(counts, method = c("fleiss", "randolph")) {
  method <- match.arg(method)
  counts <- as.matrix(counts)
  n_items <- nrow(counts)
  n_categories <- ncol(counts)
  if (n_items < 2L || n_categories < 2L) {
    return(list(kappa = NA_real_, observed = NA_real_, expected = NA_real_))
  }

  n_i <- rowSums(counts)
  item_agreement <- rowSums(counts * (counts - 1)) / (n_i * (n_i - 1))
  observed <- mean(item_agreement)
  expected <- if (identical(method, "fleiss")) {
    p_j <- colSums(counts) / sum(counts)
    sum(p_j^2)
  } else {
    1 / n_categories
  }
  if (!is.finite(1 - expected) || (1 - expected) <= 0) {
    kappa <- NA_real_
  } else {
    kappa <- (observed - expected) / (1 - expected)
  }

  list(
    kappa = kappa,
    observed = observed,
    expected = expected,
    item_agreement = item_agreement,
    n_i = n_i,
    category_counts = colSums(counts),
    category_proportions = colSums(counts) / sum(counts)
  )
}

manual_multirater_kappa_exact_ratings <- function(ratings) {
  ratings <- as.matrix(ratings)
  ratings <- ratings[stats::complete.cases(ratings), , drop = FALSE]
  n_items <- nrow(ratings)
  n_raters <- ncol(ratings)
  lev <- unique(as.character(ratings))
  counts <- vapply(seq_len(n_items), function(i) {
    tab <- table(factor(ratings[i, ], levels = lev))
    as.numeric(tab)
  }, numeric(length(lev)))
  counts <- t(counts)
  observed <- mean(rowSums(counts * (counts - 1)) / (n_raters * (n_raters - 1)))
  chance_base <- sum(colSums(counts)^2) / (n_items * n_raters)^2
  rtab <- vapply(seq_len(n_raters), function(j) {
    tab <- table(factor(ratings[, j], levels = lev))
    as.numeric(tab) / n_items
  }, numeric(length(lev)))
  chance_exact <- chance_base - sum(apply(rtab, 1, stats::var)) / n_raters
  if (!is.finite(1 - chance_exact) || (1 - chance_exact) <= 0) {
    return(NA_real_)
  }
  (observed - chance_exact) / (1 - chance_exact)
}

counts_to_ratings <- function(counts, categories = colnames(counts)) {
  counts <- as.matrix(counts)
  max_raters <- max(rowSums(counts))
  out <- matrix(NA_character_, nrow(counts), max_raters)
  for (i in seq_len(nrow(counts))) {
    labs <- rep(categories, times = counts[i, ])
    out[i, seq_along(labs)] <- labs
  }
  out
}

collapse_counts_for_category_r <- function(counts, j) {
  counts <- as.matrix(counts)
  cbind(counts[, j], rowSums(counts) - counts[, j])
}

test_that("multirater_kappa reproduces the standard Fleiss count example", {
  counts <- rbind(
    c(0, 0, 0, 0, 14),
    c(0, 2, 6, 4, 2),
    c(0, 0, 3, 5, 6),
    c(0, 3, 9, 2, 0),
    c(2, 2, 8, 1, 1),
    c(7, 7, 0, 0, 0),
    c(3, 2, 6, 3, 0),
    c(2, 5, 3, 2, 2),
    c(6, 5, 2, 1, 0),
    c(0, 2, 2, 3, 7)
  )
  colnames(counts) <- paste0("c", 1:5)

  fit <- multirater_kappa(counts, input = "counts", method = "fleiss")
  expect_s3_class(fit, c("multirater_kappa", "agreement_result", "data.frame"))
  expect_equal(fit$kappa[[1L]], 0.2099307, tolerance = 1e-6)
})

test_that("multirater_kappa ratings and counts inputs agree", {
  counts <- rbind(
    c(0, 0, 0, 0, 14),
    c(0, 2, 6, 4, 2),
    c(0, 0, 3, 5, 6),
    c(0, 3, 9, 2, 0),
    c(2, 2, 8, 1, 1),
    c(7, 7, 0, 0, 0),
    c(3, 2, 6, 3, 0),
    c(2, 5, 3, 2, 2),
    c(6, 5, 2, 1, 0),
    c(0, 2, 2, 3, 7)
  )
  colnames(counts) <- paste0("c", 1:5)
  ratings <- counts_to_ratings(counts, categories = colnames(counts))

  fit_counts <- multirater_kappa(counts, input = "counts")
  fit_ratings <- multirater_kappa(ratings, input = "ratings")
  expect_equal(fit_ratings$kappa[[1L]], fit_counts$kappa[[1L]], tolerance = 1e-12)
})

test_that("multirater_kappa reproduces Randolph's free-marginal kappa", {
  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 3, 0),
    c(0, 1, 2)
  )
  manual <- manual_multirater_kappa_counts(counts, method = "randolph")
  fit <- multirater_kappa(counts, input = "counts", method = "randolph")

  expect_equal(fit$observed_agreement[[1L]], manual$observed, tolerance = 1e-12)
  expect_equal(fit$expected_agreement[[1L]], manual$expected, tolerance = 1e-12)
  expect_equal(fit$kappa[[1L]], manual$kappa, tolerance = 1e-12)
})

test_that("multirater_kappa exact Fleiss matches the irr-compatible exact formula", {
  ratings <- data.frame(
    r1 = c("A", "A", "B", "B", "C", "A", "B", "C"),
    r2 = c("A", "B", "B", "B", "C", "A", "B", "C"),
    r3 = c("A", "A", "B", "C", "C", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  fit_std <- multirater_kappa(ratings, method = "fleiss")
  fit_exact <- multirater_kappa(ratings, method = "fleiss", exact = TRUE)
  expected_exact <- manual_multirater_kappa_exact_ratings(ratings)

  expect_equal(fit_exact$kappa[[1L]], expected_exact, tolerance = 1e-12)
  expect_false(isTRUE(all.equal(fit_std$kappa[[1L]], fit_exact$kappa[[1L]], tolerance = 1e-12)))
  expect_true(isTRUE(attr(fit_exact, "exact")))
})

test_that("multirater_kappa balanced 3-rater regression examples match known estimate and SE", {
  set.seed(20260514)
  n <- 250
  latent <- rnorm(n)
  mk <- function(s) factor(
    cut(latent + rnorm(n, sd = s), breaks = c(-Inf, -0.5, 0.5, Inf), labels = c("A", "B", "C")),
    levels = c("A", "B", "C")
  )
  dat <- data.frame(r1 = mk(0.8), r2 = mk(0.9), r3 = mk(0.85))

  fit_std <- multirater_kappa(dat, method = "fleiss", p_value = TRUE)
  fit_exact <- multirater_kappa(
    dat,
    method = "fleiss",
    exact = TRUE,
    p_value = TRUE,
    se_method = "jackknife"
  )

  expect_equal(fit_std$kappa[[1L]], 0.3159716, tolerance = 1e-7)
  expect_equal(fit_std$se[[1L]], 0.02586107388050949, tolerance = 1e-12)
  expect_equal(fit_std$statistic[[1L]], 12.2180397, tolerance = 1e-7)
  expect_identical(attr(fit_std, "se.method"), "asymptotic")

  expect_equal(fit_exact$kappa[[1L]], 0.3163428, tolerance = 1e-7)
  expect_equal(fit_exact$se[[1L]], 0.03317357033930296, tolerance = 1e-12)
  expect_equal(fit_exact$statistic[[1L]], 9.5359894, tolerance = 1e-7)
  expect_identical(attr(fit_exact, "se.method"), "jackknife")
})

test_that("multirater_kappa resolves categories for character, factor, and supplied levels", {
  char_ratings <- data.frame(
    r1 = c("low", "mid", "high", "low"),
    r2 = c("low", "high", "high", "mid"),
    r3 = c("low", "mid", "high", "mid"),
    stringsAsFactors = FALSE
  )
  fit_char <- multirater_kappa(char_ratings)
  expect_equal(attr(fit_char, "categories"), c("low", "mid", "high"))

  fac_ratings <- data.frame(
    r1 = factor(c("A", "B", "A", "C"), levels = c("A", "B", "C", "D")),
    r2 = factor(c("A", "B", "C", "C"), levels = c("A", "B", "C", "D")),
    r3 = factor(c("A", "B", "A", "D"), levels = c("A", "B", "D", "C"))
  )
  fit_fac <- multirater_kappa(fac_ratings)
  expect_equal(attr(fit_fac, "categories"), c("A", "B", "C", "D"))

  fit_unobs <- multirater_kappa(char_ratings, levels = c("low", "mid", "high", "very_high"))
  expect_equal(attr(fit_unobs, "categories"), c("low", "mid", "high", "very_high"))

  expect_error(
    multirater_kappa(char_ratings, levels = c("low", "mid")),
    "must contain every non-missing observed category"
  )
})

test_that("multirater_kappa requires explicit levels for mixed factor and non-factor ratings", {
  mixed <- data.frame(
    r1 = factor(c("mid", "low", "high"), levels = c("low", "mid", "high")),
    r2 = c("mid", "low", "high"),
    r3 = c("low", "low", "high"),
    stringsAsFactors = FALSE
  )

  expect_error(
    multirater_kappa(mixed),
    "factor and non-factor"
  )

  fit <- multirater_kappa(mixed, levels = c("low", "mid", "high"))
  expect_identical(attr(fit, "categories"), c("low", "mid", "high"))
})

test_that("multirater_kappa missing-data modes follow the documented rules", {
  raters <- data.frame(
    r1 = c("A", "A", "B", NA, "C", "A"),
    r2 = c("A", "B", "B", "B", NA, "A"),
    r3 = c("A", "A", NA, "B", "C", "A"),
    r4 = c("A", NA, "B", "B", "C", "A"),
    stringsAsFactors = FALSE
  )

  expect_error(multirater_kappa(raters, na_method = "error"), "contains missing ratings")

  fit_complete <- multirater_kappa(raters, na_method = "complete")
  keep_complete <- stats::complete.cases(raters)
  expect_equal(fit_complete$n_items[[1L]], sum(keep_complete))

  fit_available <- multirater_kappa(raters, na_method = "available", min_raters = 3L)
  keep_available <- rowSums(!is.na(raters)) >= 3L
  expect_equal(fit_available$n_items[[1L]], sum(keep_available))
  expect_true(!is.null(attr(fit_available, "diagnostics")$n_raters_by_item))

  expect_error(
    multirater_kappa(raters, na_method = "pairwise"),
    "panel-level multi-rater kappa"
  )
  expect_error(
    multirater_kappa(raters, na_method = "available", exact = TRUE),
    "must be FALSE when"
  )
  fit_available_auto <- multirater_kappa(raters, na_method = "available", p_value = TRUE)
  expect_identical(attr(fit_available_auto, "se.method"), "jackknife")
  expect_true(is.finite(fit_available_auto$se[[1L]]) || is.na(fit_available_auto$se[[1L]]))
  expect_error(
    multirater_kappa(raters, na_method = "available", p_value = TRUE, se_method = "asymptotic"),
    "asymptotic"
  )
  fit_available_jk <- multirater_kappa(raters, na_method = "available", p_value = TRUE, se_method = "jackknife")
  expect_identical(attr(fit_available_jk, "se.method"), "jackknife")
})

test_that("multirater_kappa counts input validates counts and category labels explicitly", {
  counts <- matrix(
    c(
      2, 1, 0,
      0, 3, 0,
      0, 0, 1,
      1, 1, 1
    ),
    ncol = 3,
    byrow = TRUE,
    dimnames = list(NULL, c("A", "B", "C"))
  )

  fit <- multirater_kappa(counts, input = "counts", levels = c("low", "mid", "high"), min_raters = 2L)
  expect_identical(attr(fit, "categories"), c("low", "mid", "high"))
  expect_equal(fit$n_items[[1L]], 3L)
  expect_equal(attr(fit, "diagnostics")$row_sums, c(3, 3, 1, 3))

  expect_error(
    multirater_kappa(counts, input = "counts", levels = c("A", "B")),
    "length equal to ncol"
  )
  expect_error(
    multirater_kappa(replace(counts, 1, NA), input = "counts"),
    "missing counts"
  )
  expect_error(
    multirater_kappa(replace(counts, 1, -1), input = "counts"),
    "non-negative"
  )
  expect_error(
    multirater_kappa(replace(counts, 1, 1.5), input = "counts"),
    "integer-like"
  )
})

test_that("multirater_kappa validates inference-only arguments only when inference is requested", {
  raters <- data.frame(
    r1 = c("A", "A", "B", "B"),
    r2 = c("A", "B", "B", "B"),
    r3 = c("A", "A", "B", "A"),
    stringsAsFactors = FALSE
  )

  fit <- multirater_kappa(raters, exact = TRUE, se_method = "not-a-method", conf_level = NA_real_)
  expect_true(isTRUE(attr(fit, "exact")))
  expect_identical(attr(fit, "se.method"), "none")

  expect_error(
    multirater_kappa(raters, p_value = TRUE, se_method = "not-a-method"),
    class = "rlang_error"
  )
  expect_error(
    multirater_kappa(raters, p_value = TRUE, conf_level = NA_real_),
    class = "matrixCorr_arg_error"
  )
})

test_that("multirater_kappa records unbalanced available-rater data", {
  raters <- data.frame(
    r1 = c("A", "A", "B", "C", "A"),
    r2 = c("A", "B", "B", NA, "A"),
    r3 = c("A", NA, "B", NA, "C"),
    r4 = c("A", "B", NA, NA, "A"),
    stringsAsFactors = FALSE
  )

  fit <- multirater_kappa(raters, na_method = "available", min_raters = 2L)
  expect_false(fit$balanced[[1L]])
  expect_equal(fit$n_raters_min[[1L]], 3L)
  expect_equal(fit$n_raters_max[[1L]], 4L)
})

test_that("multirater_kappa handles degenerate validation and denominator cases", {
  expect_error(
    multirater_kappa(matrix(c("A", "A"), ncol = 2), input = "ratings"),
    "at least two categories"
  )

  expect_error(
    multirater_kappa(matrix(c(1, 1, 1, 1), ncol = 1), input = "counts"),
    "at least two category columns"
  )
  expect_error(
    multirater_kappa(matrix(c(1, 1, 1, 1), ncol = 2), input = "counts", exact = TRUE),
    "must be FALSE when"
  )
  expect_error(
    multirater_kappa(data.frame(r1 = c("A", "B"), r2 = c("A", "B")), method = "randolph", exact = TRUE),
    "must be FALSE unless"
  )

  raters <- data.frame(
    r1 = rep("A", 4),
    r2 = rep("A", 4),
    r3 = rep("A", 4),
    stringsAsFactors = FALSE
  )
  fit <- multirater_kappa(raters, levels = c("A", "B"))
  expect_true(is.na(fit$kappa[[1L]]))

  raters_miss <- data.frame(
    r1 = c("A", NA, "B"),
    r2 = c("A", NA, "B"),
    r3 = c(NA, NA, "B"),
    stringsAsFactors = FALSE
  )
  fit_available <- multirater_kappa(raters_miss, na_method = "available", min_raters = 2L)
  expect_equal(fit_available$n_items[[1L]], 2L)
})

test_that("multirater_kappa attaches by-category binary-collapsed results", {
  counts <- rbind(
    c(3, 0, 0),
    c(2, 1, 0),
    c(0, 3, 0),
    c(0, 1, 2)
  )
  colnames(counts) <- c("A", "B", "C")
  fit <- multirater_kappa(counts, input = "counts", by_category = TRUE)
  by_cat <- attr(fit, "by_category")

  expect_true(is.data.frame(by_cat))
  expect_equal(nrow(by_cat), ncol(counts))
  for (j in seq_len(ncol(counts))) {
    manual <- manual_multirater_kappa_counts(collapse_counts_for_category_r(counts, j), method = "fleiss")
    expect_equal(by_cat$kappa[[j]], manual$kappa, tolerance = 1e-12)
  }
})

test_that("multirater_kappa jackknife inference behaves as documented", {
  counts <- rbind(
    c(0, 0, 0, 0, 14),
    c(0, 2, 6, 4, 2),
    c(0, 0, 3, 5, 6),
    c(0, 3, 9, 2, 0),
    c(2, 2, 8, 1, 1),
    c(7, 7, 0, 0, 0),
    c(3, 2, 6, 3, 0),
    c(2, 5, 3, 2, 2),
    c(6, 5, 2, 1, 0),
    c(0, 2, 2, 3, 7)
  )
  fit <- multirater_kappa(counts, input = "counts", ci = TRUE, p_value = TRUE)
  expect_true(is.finite(fit$se[[1L]]) || is.na(fit$se[[1L]]))
  expect_true(is.finite(fit$lwr.ci[[1L]]))
  expect_true(is.finite(fit$upr.ci[[1L]]))
  expect_true(fit$lwr.ci[[1L]] >= -1 && fit$upr.ci[[1L]] <= 1)
  expect_identical(attr(fit, "se.method"), "asymptotic")

  fit_jk <- multirater_kappa(counts, input = "counts", ci = TRUE, p_value = TRUE, se_method = "jackknife")
  expect_identical(attr(fit_jk, "se.method"), "jackknife")
  expect_true(is.finite(fit_jk$se[[1L]]) || is.na(fit_jk$se[[1L]]))

  expect_error(
    multirater_kappa(counts, input = "counts", ci = TRUE, se_method = "none"),
    "must not be 'none'"
  )
  fit_exact_auto <- multirater_kappa(
    data.frame(r1 = c("A", "A", "B"), r2 = c("A", "B", "B"), r3 = c("A", "A", "B")),
    exact = TRUE,
    p_value = TRUE
  )
  expect_identical(attr(fit_exact_auto, "se.method"), "jackknife")
  expect_error(
    multirater_kappa(
      data.frame(r1 = c("A", "A", "B"), r2 = c("A", "B", "B"), r3 = c("A", "A", "B")),
      exact = TRUE,
      p_value = TRUE,
      se_method = "asymptotic"
    ),
    "asymptotic"
  )
  expect_error(
    multirater_kappa(counts, input = "counts", exact = TRUE, p_value = TRUE),
    "must be FALSE when"
  )
  expect_error(
    multirater_kappa(
      data.frame(r1 = c("A", "A", "B"), r2 = c("A", "B", "B"), r3 = c("A", "A", "B")),
      method = "randolph",
      exact = TRUE,
      p_value = TRUE
    ),
    "must be FALSE unless"
  )
})

test_that("multirater_kappa defaults to jackknife when asymptotic inference is unavailable", {
  raters <- data.frame(
    r1 = c("A", "A", "B", "C", "A", "B"),
    r2 = c("A", "B", "B", "C", "A", "B"),
    r3 = c("A", "A", "B", "B", "A", "C"),
    stringsAsFactors = FALSE
  )

  fit_randolph <- multirater_kappa(raters, method = "randolph", ci = TRUE, p_value = TRUE)
  expect_identical(attr(fit_randolph, "se.method"), "jackknife")
  expect_true(is.finite(fit_randolph$se[[1L]]) || is.na(fit_randolph$se[[1L]]))

  expect_error(
    multirater_kappa(raters, method = "randolph", ci = TRUE, p_value = TRUE, se_method = "asymptotic"),
    "asymptotic"
  )
})

test_that("multirater_kappa S3 methods return the expected object types", {
  skip_if_not_installed("ggplot2")

  raters <- data.frame(
    r1 = c("A", "A", "B", "C", "A", "B"),
    r2 = c("A", "B", "B", "C", "A", "B"),
    r3 = c("A", "A", "B", "B", "A", "C"),
    stringsAsFactors = FALSE
  )
  fit <- multirater_kappa(raters, by_category = TRUE)
  expect_s3_class(fit, c("multirater_kappa", "agreement_result", "data.frame"))
  expect_true(length(capture.output(print(fit))) > 0L)

  sm <- summary(fit)
  expect_s3_class(sm, "summary.multirater_kappa")

  expect_s3_class(plot(fit, type = "item_agreement"), "ggplot")
  expect_s3_class(plot(fit, type = "category_proportion"), "ggplot")
  expect_error(plot(multirater_kappa(raters), type = "by_category"), "by_category")
  expect_s3_class(plot(fit, type = "by_category"), "ggplot")
})

test_that("multirater_kappa C++ ratings and counts backends are consistent", {
  counts <- matrix(
    c(
      2, 1, 0,
      0, 3, 0,
      1, 1, 1,
      0, 1, 2
    ),
    ncol = 3,
    byrow = TRUE
  )
  ratings <- counts_to_ratings(counts, categories = c("A", "B", "C"))
  encoded <- matrix(match(ratings, c("A", "B", "C")), nrow = nrow(ratings))

  fit_counts_1 <- multirater_kappa_counts_cpp(counts, method_code = 1L, n_threads = 1L)
  fit_counts_2 <- multirater_kappa_counts_cpp(counts, method_code = 1L, n_threads = 2L)
  fit_ratings <- multirater_kappa_ratings_cpp(encoded, n_levels = 3L, method_code = 1L, na_code = 1L, n_threads = 1L)

  expect_equal(fit_counts_1$estimate, fit_ratings$estimate, tolerance = 1e-12)
  expect_equal(fit_counts_1$estimate, fit_counts_2$estimate, tolerance = 1e-12)

  fit_ratings_exact <- multirater_kappa_ratings_cpp(
    encoded,
    n_levels = 3L,
    method_code = 1L,
    na_code = 1L,
    n_threads = 1L,
    exact = TRUE
  )
  expect_equal(
    fit_ratings_exact$estimate,
    manual_multirater_kappa_exact_ratings(ratings),
    tolerance = 1e-12
  )
})

test_that("ccc_cpp and lins_ccc_detail_cpp produce expected values", {
  # Test 1: Perfect agreement
  x1 <- c(1, 2, 3, 4, 5)
  mat1 <- cbind(x1, x1)
  ccc_mat <- ccc_cpp(mat1)
  expect_equal(ccc_mat[1, 2], 1.0, tolerance = 1e-12)

  # Test 2: Perfect negative agreement
  x2 <- c(1, 2, 3, 4, 5)
  y2 <- c(5, 4, 3, 2, 1)
  mat2 <- cbind(x2, y2)
  ccc_val2 <- ccc_cpp(mat2)[1, 2]
  expected_r2 <- -1
  mean_diff2 <- mean(x2) - mean(y2)
  var_x2 <- var(x2) * 4 / 5  # biased var
  var_y2 <- var(y2) * 4 / 5
  expected_sxy2 <- expected_r2 * sqrt(var_x2 * var_y2)
  expected_ccc2 <- 2 * expected_sxy2 / (var_x2 + var_y2 + mean_diff2^2)
  expect_equal(ccc_val2, expected_ccc2, tolerance = 1e-12)

  # Test 3: Manually calculated CCC
  x3 <- c(1, 2, 3, 4, 5)
  y3 <- c(1.1, 2.1, 2.9, 3.8, 5.2)
  mean_x3 <- mean(x3)
  mean_y3 <- mean(y3)
  var_x3 <- var(x3) * 4 / 5
  var_y3 <- var(y3) * 4 / 5
  cov_xy3 <- sum((x3 - mean_x3) * (y3 - mean_y3)) / 5
  r3 <- cov_xy3 / sqrt(var_x3 * var_y3)
  sxy3 <- r3 * sqrt(var_x3 * var_y3)
  expected_ccc3 <- 2 * sxy3 / (var_x3 + var_y3 + (mean_x3 - mean_y3)^2)

  mat3 <- cbind(x3, y3)
  fast_ccc3 <- ccc_cpp(mat3)[1, 2]
  expect_equal(fast_ccc3, expected_ccc3, tolerance = 1e-10)
})

test_that("CCC matrix is symmetric", {
  mat <- matrix(rnorm(100 * 4), ncol = 4)
  result <- ccc_cpp(mat)
  expect_equal(result, t(result), tolerance = 1e-12)
})

test_that("CCC detects lack of agreement despite perfect correlation", {
  x <- rnorm(100)
  y <- x * 2 + 5
  mat <- cbind(x, y)
  result <- ccc_cpp(mat)
  expect_true(result[1, 2] < 1)
})

test_that("CCC with CI returns correctly structured result", {
  mat <- matrix(rnorm(100 * 3), ncol = 3)
  result <- ccc_with_ci_cpp(mat)
  expect_named(result, c("est", "lwr.ci", "upr.ci"))
  expect_equal(dim(result$est), dim(result$lwr.ci))
  expect_equal(dim(result$est), dim(result$upr.ci))
})

test_that("ccc public API honors n_threads without changing estimates", {
  set.seed(44)
  X <- matrix(rnorm(160), ncol = 4)
  colnames(X) <- paste0("V", 1:4)

  fit1 <- ccc(X, n_threads = 1L)
  fit2 <- ccc(X, n_threads = 2L)
  fit1_ci <- ccc(X, ci = TRUE, n_threads = 1L)
  fit2_ci <- ccc(X, ci = TRUE, n_threads = 2L)

  expect_equal(unname(fit1), unname(fit2), tolerance = 1e-12)
  expect_equal(unname(fit1_ci$est), unname(fit2_ci$est), tolerance = 1e-12)
})

test_that("ccc keeps only canonical CI components when intervals are requested", {
  set.seed(45)
  X <- matrix(rnorm(120), ncol = 3)
  colnames(X) <- c("A", "B", "C")

  fit <- ccc(X)
  fit_ci <- ccc(X, ci = TRUE)

  expect_false(is.list(fit))
  expect_s3_class(fit, "ccc")
  expect_s3_class(fit_ci, "ccc_ci")
  expect_identical(names(fit_ci), c("est", "lwr.ci", "upr.ci"))
  expect_false(any(c("estimate", "lwr", "upr") %in% names(fit_ci)))
  expect_true(is.matrix(attr(fit, "diagnostics")$n_complete))
  expect_true(is.matrix(attr(fit_ci, "diagnostics")$n_complete))
})

test_that("ccc missing-data modes match Pearson/Spearman semantics", {
  X <- cbind(
    a = c(1, 2, 3, 4, 5, NA),
    b = c(1, 2, NA, 4, 5, 6),
    c = c(6, 5, 4, 3, 2, 1)
  )

  expect_error(ccc(X, na_method = "error"), "Missing")

  keep_all <- apply(is.finite(X), 1L, all)
  fit_complete <- ccc(X, na_method = "complete")
  ref_complete <- ccc(X[keep_all, , drop = FALSE])
  got_complete <- unclass(as.matrix(fit_complete))
  ref_complete_mat <- unclass(as.matrix(ref_complete))
  attributes(got_complete) <- attributes(got_complete)[c("dim", "dimnames")]
  attributes(ref_complete_mat) <- attributes(ref_complete_mat)[c("dim", "dimnames")]
  expect_equal(got_complete, ref_complete_mat, tolerance = 1e-12)
  expect_equal(attr(fit_complete, "diagnostics")$n_complete, matrix(sum(keep_all), 3, 3, dimnames = dimnames(fit_complete)))
  expect_identical(attr(fit_complete, "diagnostics")$na_method, "complete")

  fit_pairwise <- ccc(X, na_method = "pairwise")
  diag_n <- attr(fit_pairwise, "diagnostics")$n_complete
  expect_equal(diag_n["a", "b"], sum(is.finite(X[, "a"]) & is.finite(X[, "b"])))
  expect_equal(diag_n["a", "c"], sum(is.finite(X[, "a"]) & is.finite(X[, "c"])))
  expect_equal(diag_n["b", "c"], sum(is.finite(X[, "b"]) & is.finite(X[, "c"])))

  ref_ab <- ccc(X[is.finite(X[, "a"]) & is.finite(X[, "b"]), c("a", "b"), drop = FALSE])
  expect_equal(fit_pairwise["a", "b"], ref_ab["a", "b"], tolerance = 1e-12)
})

test_that("ccc pairwise CIs and negative estimates are supported across output modes", {
  X <- cbind(
    x = c(1, 2, 3, 4, 5, NA),
    y = c(5.1, 3.8, 3.2, 2.1, 0.9, 0),
    z = c(1, 2, 1, 2, NA, 3)
  )

  fit_ci <- ccc(X, ci = TRUE, na_method = "pairwise")
  expect_s3_class(fit_ci, "ccc_ci")
  expect_identical(attr(fit_ci, "ci.method", exact = TRUE), "lin_delta_fisher_z")
  expect_true(is.finite(fit_ci$lwr.ci["x", "y"]))
  expect_lt(fit_ci$est["x", "y"], 0)

  edge <- ccc(X, na_method = "pairwise", output = "edge_list", threshold = 0.5)
  expect_s3_class(edge, "corr_edge_list")
  expect_true(any(edge$row == "x" & edge$col == "y" & edge$value < 0))

  edge_ci <- ccc(X, ci = TRUE, na_method = "pairwise", output = "edge_list", threshold = 0.5)
  expect_s3_class(edge_ci, "corr_edge_list")
  ci_attr <- attr(edge_ci, "ci", exact = TRUE)
  expect_true(is.matrix(ci_attr$lwr.ci))
  expect_identical(ci_attr$ci.method, "lin_delta_fisher_z")
  expect_true(any(edge_ci$row == "x" & edge_ci$col == "y" & edge_ci$value < 0))
})

#' @title Robust Distance Correlation
#'
#' @description
#' Computes robust distance correlations by applying the biloop transformation
#' to each numeric variable and then computing unbiased distance correlation on
#' the transformed variables.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns are dropped. Columns must be numeric.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each association on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param p_value Logical (default \code{FALSE}). If \code{TRUE}, attach
#' permutation p-values for pairwise independence.
#' @param inference Character scalar. Use \code{"none"} (default) for estimates
#' only or \code{"permutation"} to request permutation p-values. If
#' \code{inference = "permutation"} is supplied with \code{p_value = FALSE},
#' p-values are inferred and returned.
#' @param n_perm Positive integer; number of permutations used when
#' \code{p_value = TRUE}. Default \code{999}.
#' @param seed Optional positive integer seed for permutation inference.
#' @param c_const Positive numeric tuning constant for the biloop
#' transformation. Default \code{4}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#' \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates:
#' \code{"matrix"}, \code{"sparse"}, or \code{"edge_list"}.
#' @param threshold Non-negative absolute-value filter for non-matrix outputs.
#' Must be \code{0} when \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#' \code{"sparse"} and \code{"edge_list"} outputs.
#' @param ... Deprecated compatibility aliases. Currently only
#' \code{check_na} is supported.
#' @param x,object An object of class \code{robust_dcor}.
#' @param digits Integer; number of decimal places to print.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param title Plot title.
#' @param low_color,high_color,mid_color Colors used in the heatmap.
#' @param value_text_size,ci_text_size Text sizes for plot labels.
#' @param show_value Logical; overlay numeric values if \code{TRUE}.
#'
#' @return A symmetric correlation result. Dense output inherits from
#' \code{robust_dcor}, \code{corr_matrix}, and \code{matrix}. Sparse and
#' edge-list outputs use the package-standard \code{corr_sparse} and
#' \code{corr_edge_list} representations.
#'
#' @section Relationship to [dcor()]:
#' `robust_dcor()` is not a replacement for `dcor()`. It estimates distance
#' correlation after a bounded robust transformation. Large differences between
#' `dcor()` and `robust_dcor()` can indicate that the classical dependence
#' signal is driven by tail observations or outliers.
#'
#' @details
#' Permutation inference is computed for every unique column pair, so the
#' workload grows as \eqn{p(p-1)n_\mathrm{perm}/2}. A warning is emitted for
#' large requests; reduce \code{n_perm}, reduce the number of columns, or run
#' estimates without permutation p-values when this cost is not intended.
#'
#' For each variable \eqn{x = (x_1,\ldots,x_n)}, the wrapper first robustly
#' standardises values using the median
#' \deqn{m_x = \mathrm{median}(x)}
#' and raw median absolute deviation
#' \deqn{s_x = \mathrm{median}\{|x_i - m_x|\}.}
#' If this scale is zero or non-finite, the implementation falls back to
#' \eqn{\mathrm{IQR}(x) / 1.34898} and then the ordinary sample standard
#' deviation. Columns with no positive finite fallback scale are treated as
#' degenerate.
#'
#' The standardised value \eqn{z_i = (x_i - m_x) / s_x} is mapped to two
#' bounded coordinates by the biloop transformation
#' \deqn{\theta_i = 2\pi\tanh(z_i / c),}
#' \deqn{u_i =
#' \begin{cases}
#' c\{1-\cos(\theta_i)\}, & z_i > 0,\\
#' -c\{1-\cos(\theta_i)\}, & z_i < 0,\\
#' 0, & z_i = 0,
#' \end{cases}}
#' and
#' \deqn{v_i = \sin(\theta_i).}
#' Pairwise distances are Euclidean distances in this transformed
#' two-dimensional space,
#' \deqn{D^{(j)}_{ab} =
#' \left[(u_{aj}-u_{bj})^2 + (v_{aj}-v_{bj})^2\right]^{1/2},\quad a \ne b,}
#' with zero diagonal.
#'
#' The transformed distance matrix is U-centred as
#' \deqn{A^{(j)}_{ab} =
#' D^{(j)}_{ab} - \frac{r^{(j)}_a + r^{(j)}_b}{n - 2}
#' + \frac{S^{(j)}}{(n - 1)(n - 2)},\quad a \ne b,}
#' where \eqn{r^{(j)}_a = \sum_{b \ne a}D^{(j)}_{ab}} and
#' \eqn{S^{(j)} = \sum_{a \ne b}D^{(j)}_{ab}}. The diagonal of
#' \eqn{A^{(j)}} is zero.
#'
#' For variables \eqn{j} and \eqn{k}, the unbiased robust distance covariance
#' is
#' \deqn{\widehat{\mathrm{dCov}}^2_u(j,k) =
#' \frac{2}{n(n-3)}\sum_{a < b} A^{(j)}_{ab} A^{(k)}_{ab}.}
#' The corresponding robust distance correlation is the usual non-negative
#' distance-correlation ratio based on this covariance and the two transformed
#' distance variances. Small negative numerical artifacts are clipped to zero.
#'
#' @note
#' `robust_dcor()` is more robust to extreme observations than classical
#' `dcor()`, but it may downweight genuine tail dependence. Classical
#' `dcor()` may be preferable when tail dependence is the scientific target.
#' Comparing both methods is recommended.
#'
#' @references
#' Leyder, J., Raymaekers, J., & Rousseeuw, P. J. (2025). Robust distance
#' correlation through bounded transformations.
#'
#' @seealso [dcor()], [wincor()], [pbcor()], [skipped_corr()]
#'
#' @examples
#' ## Non-linear dependence: both estimators detect association.
#' set.seed(1)
#' n <- 200
#' x <- rnorm(n)
#' y <- x^2 + rnorm(n, sd = 0.2)
#'
#' X <- cbind(x = x, y = y)
#'
#' classical <- dcor(X)
#' robust <- robust_dcor(X)
#' round(c(
#'   dcor = classical["x", "y"],
#'   robust_dcor = robust["x", "y"]
#' ), 3)
#'
#' ## One diagonal outlier can inflate classical dCor more than robust dCor.
#' set.seed(45)
#' x <- rnorm(20)
#' y <- rnorm(20)
#' x[1] <- 10
#' y[1] <- 10
#' X_out <- cbind(x = x, y = y)
#'
#' classical <- dcor(X_out)
#' robust <- robust_dcor(X_out)
#' round(c(
#'   dcor = classical["x", "y"],
#'   robust_dcor = robust["x", "y"]
#' ), 3)
#'
#' print(classical)
#' print(robust)
#' summary(robust)
#' estimate(robust)
#' tidy(robust)
#' plot(robust)
#'
#' ## Several variables.
#' set.seed(7)
#' z <- rnorm(120)
#' X_multi <- cbind(
#'   linear = z + rnorm(120, sd = 0.3),
#'   nonlinear = z^2 + rnorm(120, sd = 0.3),
#'   noise = rnorm(120),
#'   outlier = rnorm(120)
#' )
#' X_multi[1, "outlier"] <- 12
#' X_multi[1, "noise"] <- 12
#'
#' robust_multi <- robust_dcor(X_multi)
#'
#' print(robust_multi)
#' summary(robust_multi)
#' plot(robust_multi)
#'
#' @author Thiago de Paula Oliveira
#' @export
robust_dcor <- function(data,
                        na_method = c("error", "pairwise", "complete"),
                        p_value = FALSE,
                        inference = c("none", "permutation"),
                        n_perm = 999L,
                        seed = NULL,
                        c_const = 4,
                        n_threads = getOption("matrixCorr.threads", 1L),
                        output = c("matrix", "sparse", "edge_list"),
                        threshold = 0,
                        diag = TRUE,
                        ...) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )

  legacy_args <- .mc_extract_legacy_aliases(list(...), allowed = "check_na")
  na_cfg <- resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = missing(na_method),
    allowed = c("error", "pairwise", "complete")
  )

  check_bool(p_value, arg = "p_value")
  inference <- match.arg(inference)
  if (identical(inference, "permutation")) {
    p_value <- TRUE
  }
  check_scalar_numeric(c_const, arg = "c_const", lower = 0, closed_lower = FALSE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  if (isTRUE(p_value) && !identical(inference, "permutation")) {
    abort_bad_arg(
      "inference",
      message = "must be \"permutation\" when p_value = TRUE for robust_dcor().",
      .hint = "The classical distance-correlation t-test is not used for biloop-transformed robust dCor."
    )
  }

  if (isTRUE(p_value)) {
    n_perm <- check_scalar_int_pos(n_perm, arg = "n_perm")
    if (!is.null(seed)) seed <- check_scalar_int_pos(seed, arg = "seed")
  }

  numeric_data <- validate_corr_input(data, check_na = na_cfg$check_na)
  diagnostics_extra <- NULL
  if (identical(na_cfg$na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 4L, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }
  colnames_data <- colnames(numeric_data)
  dn <- .mc_square_dimnames(colnames_data)

  if (isTRUE(p_value)) {
    .mc_warn_large_robust_dcor_permutation_request(
      n_cols = ncol(numeric_data),
      n_perm = n_perm
    )
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  diagnostics <- NULL
  inference_attr <- NULL
  if (!identical(na_cfg$na_method, "pairwise") && !isTRUE(p_value)) {
    est <- robust_dcor_matrix_cpp(
      numeric_data,
      c_const = c_const,
      n_threads = n_threads
    )
  } else {
    res <- robust_dcor_matrix_pairwise_cpp(
      numeric_data,
      c_const = c_const,
      return_inference = isTRUE(p_value),
      n_perm = n_perm,
      seed = if (is.null(seed)) NULL else as.integer(seed),
      n_threads = n_threads
    )
    est <- res$est
    diagnostics <- list(
      n_complete = .mc_set_matrix_dimnames(unclass(res$n_complete), colnames_data),
      scale_method = "median_raw_mad_with_fallback"
    )

    if (isTRUE(p_value)) {
      inference_attr <- list(
        method = "permutation",
        estimate = .mc_set_matrix_dimnames(unclass(res$estimate), colnames_data),
        statistic = NULL,
        parameter = matrix(
          n_perm,
          nrow = ncol(numeric_data),
          ncol = ncol(numeric_data),
          dimnames = dn
        ),
        p_value = .mc_set_matrix_dimnames(unclass(res$p_value), colnames_data),
        alternative = "greater",
        n_perm = n_perm
      )
    }
  }
  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)

  est <- .mc_set_matrix_dimnames(est, colnames_data)

  out <- .mc_structure_corr_matrix(
    est,
    class_name = "robust_dcor",
    method = "robust_distance_correlation",
    description = paste0(
      "Biloop-transformed robust distance correlation; c_const = ",
      c_const,
      "; NA mode = ",
      na_cfg$na_method,
      "."
    ),
    symmetric = TRUE,
    diagnostics = diagnostics,
    dimnames = dn,
    extra_attrs = c(
      list(
        transform = "biloop",
        c_const = c_const,
        standardise = "median_raw_mad",
        classical_ref = "dcor"
      ),
      if (!is.null(inference_attr)) list(inference = inference_attr)
    )
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_warn_large_robust_dcor_permutation_request <- function(n_cols,
                                                           n_perm,
                                                           warn_at = 100000L) {
  n_pairs <- choose(as.numeric(n_cols), 2)
  n_tests <- n_pairs * as.numeric(n_perm)
  if (!is.finite(n_tests) || n_tests <= warn_at) {
    return(invisible(FALSE))
  }

  cli::cli_warn(
    c(
      "Permutation inference for {.fn robust_dcor} can be expensive.",
      "i" = "This request runs {format(n_pairs, big.mark = ',')} pairwise permutation test{?s} x {format(n_perm, big.mark = ',')} permutation{?s} = {format(n_tests, big.mark = ',')} resampled pair evaluations.",
      "i" = "Reduce {.arg n_perm}, reduce the number of columns, or set {.arg inference} = {.val none} and {.arg p_value} = {.val FALSE} if p-values are not needed."
    ),
    class = c("matrixCorr_warning", "matrixCorr_permutation_warning")
  )
  invisible(TRUE)
}

#' @rdname robust_dcor
#' @method print robust_dcor
#' @export
print.robust_dcor <- function(x,
                              digits = 4,
                              n = NULL,
                              topn = NULL,
                              max_vars = NULL,
                              width = NULL,
                              show_ci = NULL,
                              ...) {
  .mc_print_corr_matrix(
    x,
    header = "Robust distance correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname robust_dcor
#' @method summary robust_dcor
#' @export
summary.robust_dcor <- function(object, topn = NULL, show_ci = NULL, ...) {
  NextMethod("summary")
}

#' @rdname robust_dcor
#' @method plot robust_dcor
#' @export
plot.robust_dcor <- function(x,
                             title = NULL,
                             low_color = "indianred1",
                             high_color = "steelblue1",
                             mid_color = "white",
                             value_text_size = 4,
                             ci_text_size = 3,
                             show_value = TRUE,
                             ...) {
  .mc_plot_corr_result(
    x,
    title = title %||% "Robust distance correlation heatmap",
    low_color = low_color,
    high_color = high_color,
    mid_color = mid_color,
    value_text_size = value_text_size,
    ci_text_size = ci_text_size,
    show_value = show_value,
    ...
  )
}

test_that("robust_dcor returns basic shape and class", {
  set.seed(1)
  X <- matrix(rnorm(100 * 5), ncol = 5)
  colnames(X) <- paste0("V", 1:5)

  R <- robust_dcor(X)

  expect_s3_class(R, "robust_dcor")
  expect_s3_class(R, "corr_matrix")
  expect_equal(dim(R), c(5L, 5L))
  expect_true(isSymmetric(unclass(R)))
  expect_equal(unname(diag(R)), rep(1, 5), tolerance = 1e-12)
})

test_that("robust_dcor routes output modes", {
  set.seed(1)
  X <- matrix(rnorm(100 * 5), ncol = 5)
  colnames(X) <- paste0("V", 1:5)

  dense <- robust_dcor(X, output = "matrix")
  sparse <- robust_dcor(X, output = "sparse", threshold = 0.2)
  edges <- robust_dcor(X, output = "edge_list", threshold = 0.2)

  expect_s3_class(dense, "corr_matrix")
  expect_s4_class(sparse, "sparseMatrix")
  expect_identical(attr(sparse, "corr_output_class", exact = TRUE), "corr_sparse")
  expect_s3_class(edges, "corr_edge_list")
})

test_that("robust_dcor rejects missing values with na_method error", {
  set.seed(1)
  X <- matrix(rnorm(100 * 5), ncol = 5)
  colnames(X) <- paste0("V", 1:5)
  X[1, 1] <- NA_real_

  expect_error(robust_dcor(X, na_method = "error"))
})

test_that("robust_dcor supports pairwise missing data", {
  set.seed(1)
  X <- matrix(rnorm(100 * 5), ncol = 5)
  colnames(X) <- paste0("V", 1:5)
  X[1, 1] <- NA_real_

  Rna <- robust_dcor(X, na_method = "pairwise")

  expect_s3_class(Rna, "robust_dcor")
  expect_true(!is.null(attr(Rna, "diagnostics")$n_complete))
})

test_that("robust_dcor detects nonlinear dependence", {
  set.seed(2)
  x <- rnorm(200)
  y <- x^2 + rnorm(200, sd = 0.2)

  R <- robust_dcor(cbind(x = x, y = y))

  expect_true(R["x", "y"] > 0.1)
})

test_that("robust_dcor is less outlier-sensitive than dcor", {
  set.seed(11)
  x <- rnorm(100)
  y <- rnorm(100)
  x[1] <- 50
  y[1] <- 50

  Dc <- dcor(cbind(x = x, y = y))
  Dr <- robust_dcor(cbind(x = x, y = y))

  expect_true(Dr["x", "y"] < Dc["x", "y"])
})

test_that("robust_dcor uses shared S3 methods", {
  skip_if_not_installed("ggplot2")

  set.seed(2)
  x <- rnorm(200)
  y <- x^2 + rnorm(200, sd = 0.2)
  R <- robust_dcor(cbind(x = x, y = y))

  expect_s3_class(summary(R), "summary.corr_result")
  expect_s3_class(plot(R), "ggplot")
  expect_true(length(capture.output(print(R))) > 0L)
})

test_that("robust_dcor supports permutation p-values", {
  set.seed(4)
  x <- rnorm(30)
  y <- x^2 + rnorm(30, sd = 0.2)

  R <- robust_dcor(cbind(x = x, y = y), p_value = TRUE, inference = "permutation", n_perm = 19L, seed = 1L)
  inf <- attr(R, "inference", exact = TRUE)

  expect_identical(inf$method, "permutation")
  expect_true(is.matrix(inf$p_value))
  expect_true(is.finite(inf$p_value["x", "y"]))
})

test_that("robust_dcor infers p-values from permutation inference", {
  set.seed(5)
  x <- rnorm(24)
  y <- x^2 + rnorm(24, sd = 0.3)

  R <- robust_dcor(cbind(x = x, y = y), inference = "permutation", n_perm = 9L, seed = 2L)
  inf <- attr(R, "inference", exact = TRUE)

  expect_identical(inf$method, "permutation")
  expect_true(is.matrix(inf$p_value))
  expect_true(is.finite(inf$p_value["x", "y"]))
})

test_that("robust_dcor warns for large permutation workloads", {
  expect_warning(
    .mc_warn_large_robust_dcor_permutation_request(n_cols = 6L, n_perm = 10L, warn_at = 100L),
    class = "matrixCorr_permutation_warning"
  )
})

#' @title Multi-Rater Kappa for Nominal Ratings
#'
#' @description
#' Estimates panel-level chance-corrected agreement among multiple raters
#' assigning items to nominal categories.
#'
#' @param data Input ratings or counts data.
#'   For \code{input = "ratings"}, rows are items/subjects and columns are
#'   raters/classifiers. For \code{input = "counts"}, rows are items/subjects,
#'   columns are categories, and values are counts of raters assigning that
#'   category to that item.
#' @param levels Optional explicit category labels. For nominal multi-rater
#'   kappa, category order is not used in the estimator itself, but explicit
#'   levels are useful when unobserved categories should be retained in the
#'   output. If factor columns are mixed with non-factor columns, \code{levels}
#'   must be supplied to avoid ambiguous implicit ordering.
#' @param input One of \code{"ratings"} or \code{"counts"}.
#' @param method One of \code{"fleiss"} (fixed-marginal multi-rater kappa) or
#'   \code{"randolph"} (free-marginal multi-rater kappa).
#' @param exact Logical; if \code{TRUE}, use the exact Fleiss fixed-marginal
#'   variant. This is available only for \code{input = "ratings"} with
#'   complete item-by-rater data and \code{method = "fleiss"}.
#' @param na_method Missing-data rule for \code{input = "ratings"}.
#'   \code{"error"} rejects missing ratings. \code{"complete"} removes any item
#'   with one or more missing ratings. \code{"available"} retains items with at
#'   least \code{min_raters} observed ratings and allows the number of raters to
#'   vary by item. For \code{input = "counts"}, missing counts are not allowed.
#' @param min_raters Minimum number of observed ratings required for an item to
#'   be retained when \code{na_method = "available"} or when
#'   \code{input = "counts"} rows are filtered by total raters. Must be at
#'   least \code{2}.
#' @param by_category Logical; if \code{TRUE}, attach category-wise kappas based
#'   on target-vs-other binary collapses of the category count matrix.
#' @param ci Logical; if \code{TRUE}, attach confidence intervals using the
#'   selected \code{se_method}.
#' @param p_value Logical; if \code{TRUE}, attach z statistics and two-sided
#'   p-values using the selected \code{se_method}.
#' @param conf_level Confidence level for intervals. Default is \code{0.95}.
#' @param se_method Standard-error method. \code{"asymptotic"} uses the
#'   closed-form large-sample variance for the standard non-exact Fleiss kappa
#'   when a common number of raters is available. \code{"jackknife"} uses
#'   leave-one-item-out jackknife inference. \code{"none"} disables
#'   inferential quantities and may not be combined with \code{ci = TRUE} or
#'   \code{p_value = TRUE}. When inferential quantities are requested and
#'   \code{se_method} is left at its default, \code{multirater_kappa()}
#'   automatically falls back to \code{"jackknife"} if asymptotic inference is
#'   unavailable for the chosen estimator/data combination. \code{se_method}
#'   and \code{conf_level} are validated only when \code{ci} or \code{p_value}
#'   requests inferential output.
#' @param n_threads Integer \eqn{\ge 1}. Number of OpenMP threads used by the
#'   C++ backend.
#' @param verbose Logical; if \code{TRUE}, emit short progress messages.
#' @param ... Reserved for future extensions. Unsupported extra arguments are
#'   rejected.
#'
#' @details
#' `multirater_kappa()` returns one panel-level agreement coefficient, not a
#' pairwise matrix. Fleiss' kappa is the default fixed-marginal estimator:
#' \deqn{\kappa = \frac{\bar P - P_e}{1 - P_e},}
#' where \eqn{\bar P} is the mean item-level agreement and
#' \eqn{P_e = \sum_j p_j^2} uses the pooled marginal category proportions.
#'
#' Randolph's free-marginal alternative replaces the expected agreement with
#' \eqn{P_e = 1 / K}. This function is for nominal categories and does not use
#' category ordering. Use [weighted_kappa()] for two-rater ordered-category
#' agreement and [cohen_kappa()] for two-rater nominal agreement.
#'
#' When \code{exact = TRUE}, the exact Fleiss fixed-marginal estimate requires
#' the original item-by-rater rating matrix. In \pkg{matrixCorr}, closed-form
#' asymptotic inference is only available for the standard non-exact Fleiss
#' estimator with a common number of raters per item. In other settings, use
#' \code{se_method = "jackknife"} when inferential quantities are needed.
#'
#' With \code{input = "ratings"} and \code{na_method = "available"}, the
#' implementation supports unbalanced item-specific numbers of raters as a
#' generalisation beyond the strict equal-rater Fleiss setting. The returned
#' diagnostics record the observed per-item rater counts.
#'
#' \strong{Confidence intervals and standard errors.}
#' Two inference paths are implemented.
#'
#' If \code{se_method = "jackknife"}, the method computes leave-one-item-out
#' estimates \eqn{\hat\kappa_{(-1)},\ldots,\hat\kappa_{(-m)}} over the
#' \eqn{m} retained items and forms
#' \deqn{ \bar\kappa_{(.)} = \frac{1}{m}\sum_{i=1}^m \hat\kappa_{(-i)}, }
#' \deqn{ \widehat{\mathrm{Var}}_{\mathrm{JK}}(\hat\kappa)
#'       \;=\;
#'       \frac{m-1}{m}\sum_{i=1}^m
#'       \left(\hat\kappa_{(-i)} - \bar\kappa_{(.)}\right)^2. }
#' The standard error is
#' \eqn{\widehat{\mathrm{se}}(\hat\kappa)=
#' \sqrt{\widehat{\mathrm{Var}}_{\mathrm{JK}}(\hat\kappa)}} and the CI is
#' \deqn{ \hat\kappa \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa), }
#' truncated to \eqn{[-1, 1]}.
#'
#' If \code{se_method = "asymptotic"}, the closed-form variance is available
#' only for the standard non-exact Fleiss estimator
#' with a common number of raters per item. Let \eqn{m} be the number of items,
#' \eqn{n} the common number of raters, \eqn{p_j} the pooled category
#' proportions, and define
#' \deqn{ S = \sum_j p_j(1-p_j), \qquad
#'        C = \sum_j p_j(1-p_j)(1-2p_j). }
#' Then the variance estimator is
#' \deqn{ \widehat{\mathrm{Var}}(\hat\kappa)
#'       \;=\;
#'       \frac{2\,(S^2 - C)}{S^2\,m\,n\,(n-1)}. }
#' The standard error and CI are then obtained from the same Wald form
#' above and truncated to \eqn{[-1, 1]}. When asymptotic inference is
#' unavailable and inferential output is requested with the default setting,
#' \code{multirater_kappa()} automatically falls back to the jackknife path.
#'
#' @return
#' A one-row data frame with class
#' \code{c("multirater_kappa", "agreement_result", "data.frame")}. The object
#' carries estimator metadata and diagnostics in attributes, and may also
#' attach category-wise binary-collapsed results in
#' \code{attr(x, "by_category")}.
#'
#' @references
#' Fleiss, J. L. (1971). Measuring nominal scale agreement among many raters.
#' \emph{Psychological Bulletin}, 76, 378-382. \doi{10.1037/h0031619}
#'
#' Randolph, J. J. (2005). Free-Marginal Multirater Kappa: An Alternative to
#' Fleiss' Fixed-Marginal Multirater Kappa.
#'
#' @seealso
#' [cohen_kappa()] for unweighted two-rater nominal agreement;
#' [weighted_kappa()] for two-rater ordered-category agreement.
#'
#' @examples
#' raters <- data.frame(
#'   r1 = c("A", "A", "B", "C", "A", "B"),
#'   r2 = c("A", "B", "B", "C", "A", "B"),
#'   r3 = c("A", "A", "B", "B", "A", "C"),
#'   stringsAsFactors = FALSE
#' )
#'
#' fit <- multirater_kappa(raters)
#' print(fit)
#' summary(fit)
#' estimate(fit)
#' tidy(fit)
#' # The default plot is an item-by-category agreement map.
#' # Rows are items, ordered from stronger to weaker item-level agreement.
#' # Columns are categories. Each tile shows how many raters assigned that
#' # category to that item, and darker fill means a larger share of raters
#' # chose that category for that item.
#' plot(fit)
#'
#' @author Thiago de Paula Oliveira
#' @export
multirater_kappa <- function(data,
                             levels = NULL,
                             input = c("ratings", "counts"),
                             method = c("fleiss", "randolph"),
                             exact = FALSE,
                             na_method = c("error", "complete", "available"),
                             min_raters = 2L,
                             by_category = FALSE,
                             ci = FALSE,
                             p_value = FALSE,
                             conf_level = 0.95,
                             se_method = c("asymptotic", "jackknife", "none"),
                             n_threads = getOption("matrixCorr.threads", 1L),
                             verbose = FALSE,
                             ...) {
  .mc_extract_legacy_aliases(list(...), allowed = character())

  input <- match_arg(input, c("ratings", "counts"), arg_name = "input")
  method <- match_arg(method, c("fleiss", "randolph"), arg_name = "method")
  check_bool(exact, arg = "exact")
  na_method <- if (missing(na_method)) {
    "error"
  } else {
    validate_na_method(
      na_method,
      arg = "na_method",
      allowed = c("error", "complete", "available", "pairwise")
    )
  }
  if (identical(na_method, "pairwise")) {
    abort_bad_arg(
      "na_method",
      message = "na_method = 'pairwise' is not defined for panel-level multi-rater kappa; use 'available' to allow item-specific missing ratings."
    )
  }
  if (isTRUE(exact) && !identical(method, "fleiss")) {
    abort_bad_arg(
      "exact",
      message = "must be FALSE unless {.arg method} is {.val fleiss}."
    )
  }
  if (isTRUE(exact) && !identical(input, "ratings")) {
    abort_bad_arg(
      "exact",
      message = "must be FALSE when {.arg input} is {.val counts}.",
      .hint = "The exact Fleiss variant requires the original item-by-rater ratings because aggregated counts do not identify rater-specific marginals."
    )
  }
  if (isTRUE(exact) && identical(na_method, "available")) {
    abort_bad_arg(
      "exact",
      message = "must be FALSE when {.arg na_method} is {.val available}.",
      .hint = "The exact Fleiss variant is defined here only for complete item-by-rater panels."
    )
  }

  min_raters <- check_scalar_int_pos(min_raters, arg = "min_raters")
  if (min_raters < 2L) {
    abort_bad_arg("min_raters", message = "must be >= 2.")
  }
  check_bool(by_category, arg = "by_category")
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  check_bool(verbose, arg = "verbose")
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  inference_requested <- isTRUE(ci) || isTRUE(p_value)
  se_method_missing <- missing(se_method)
  if (isTRUE(inference_requested)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
    se_method <- match_arg(se_method, c("asymptotic", "jackknife", "none"), arg_name = "se_method")
    if (identical(se_method, "none")) {
      abort_bad_arg(
        "se_method",
        message = "must not be 'none' when ci or p_value is TRUE."
      )
    }
  } else {
    se_method <- "none"
    conf_level <- 0.95
  }

  prep <- if (identical(input, "ratings")) {
    .mc_prepare_multirater_kappa_ratings(
      data = data,
      levels = levels,
      na_method = na_method,
      min_raters = min_raters
    )
  } else {
    .mc_prepare_multirater_kappa_counts(
      data = data,
      levels = levels,
      min_raters = min_raters
    )
  }

  method_code <- switch(method, fleiss = 1L, randolph = 2L)
  inference_code <- if (isTRUE(inference_requested)) {
    switch(se_method, none = 0L, asymptotic = 1L, jackknife = 2L)
  } else {
    0L
  }

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  run_backend <- function(inference_code) {
    if (identical(input, "ratings")) {
      multirater_kappa_ratings_cpp(
        ratings = prep$X,
        n_levels = prep$n_levels,
        method_code = method_code,
        na_code = switch(prep$na_method, error = 1L, complete = 2L, available = 3L),
        min_raters = min_raters,
        by_category = by_category,
        inference_code = inference_code,
        conf_level = conf_level,
        n_threads = n_threads,
        exact = exact
      )
    } else {
      multirater_kappa_counts_cpp(
        counts = prep$counts,
        method_code = method_code,
        by_category = by_category,
        inference_code = inference_code,
        conf_level = conf_level,
        n_threads = n_threads
      )
    }
  }

  raw <- run_backend(inference_code)
  effective_se_method <- if (isTRUE(inference_requested)) se_method else "none"

  if (isTRUE(inference_requested) &&
      identical(se_method, "asymptotic") &&
      !isTRUE(raw$asymptotic_available)) {
    if (isTRUE(se_method_missing)) {
      raw <- run_backend(2L)
      effective_se_method <- "jackknife"
      inform_if_verbose(
        "Asymptotic inference is unavailable for this estimator/data combination; using {.val jackknife} instead.",
        .verbose = verbose
      )
    } else {
      abort_bad_arg(
        "se_method",
        message = "{.val asymptotic} is not available for this estimator/data combination.",
        .hint = "Use {.arg se_method} = {.val jackknife} for exact Fleiss, Randolph, or unbalanced multi-rater panels."
      )
    }
  }

  out <- .mc_build_multirater_kappa_output(
    raw = raw,
    method = method,
    input = input,
    categories = prep$categories,
    ci = ci,
    p_value = p_value,
    conf_level = conf_level,
    na_method = if (identical(input, "ratings")) prep$na_method else na_method,
    min_raters = min_raters,
    se_method = effective_se_method,
    by_category = by_category,
    prep_diagnostics = prep$diagnostics
  )

  inform_if_verbose(
    "Computed {.val {method}}{if (isTRUE(exact)) ' exact' else ''} multi-rater kappa for {attr(out, 'n_items', exact = TRUE)} item{?s} across {length(prep$categories)} categor{?y/ies}.",
    .verbose = verbose
  )
  out
}

.mc_prepare_multirater_kappa_ratings <- function(data,
                                                 levels,
                                                 na_method,
                                                 min_raters) {
  if (!is.data.frame(data) && !is.matrix(data)) {
    abort_bad_arg("data", message = "must be a matrix or data frame when {.arg input} is {.val ratings}.")
  }

  df <- if (is.data.frame(data)) data else as.data.frame(data, stringsAsFactors = FALSE)
  if (ncol(df) < 2L) {
    abort_bad_arg("data", message = "must contain at least two rater columns.")
  }

  cols <- lapply(seq_along(df), function(j) .mc_multirater_rating_column(df[[j]], arg = names(df)[[j]] %||% "data"))
  names(cols) <- names(df)
  categories <- .mc_resolve_multirater_levels(cols, levels = levels)
  if (length(categories) < 2L) {
    abort_bad_arg("levels", message = "must define at least two categories for multi-rater kappa.")
  }

  level_keys <- as.character(categories)
  X <- matrix(
    NA_integer_,
    nrow = nrow(df),
    ncol = ncol(df),
    dimnames = list(NULL, names(df))
  )

  for (j in seq_along(cols)) {
    vals <- cols[[j]]
    keys <- as.character(vals)
    codes <- match(keys, level_keys)
    codes[is.na(vals)] <- NA_integer_
    if (any(!is.na(vals) & is.na(codes))) {
      abort_bad_arg(
        "data",
        message = "contains values not present in {.arg levels}."
      )
    }
    X[, j] <- as.integer(codes)
  }

  keep <- rep(TRUE, nrow(X))
  if (identical(na_method, "error") && anyNA(X)) {
    abort_bad_arg(
      "data",
      message = "contains missing ratings when {.arg na_method} is {.val error}."
    )
  }
  if (identical(na_method, "complete")) {
    keep <- stats::complete.cases(X)
    X <- X[keep, , drop = FALSE]
  } else if (identical(na_method, "available")) {
    keep <- rowSums(!is.na(X)) >= min_raters
    X <- X[keep, , drop = FALSE]
  }

  if (nrow(X) < 2L) {
    abort_bad_arg(
      "data",
      message = "must retain at least two items after applying missing-data handling."
    )
  }

  list(
    X = X,
    categories = categories,
    n_levels = length(categories),
    retained_rows = keep,
    dropped_rows = !keep,
    na_method = na_method,
    diagnostics = list(
      n_original = nrow(df),
      n_retained = nrow(X),
      retained_rows = keep,
      dropped_rows = !keep,
      n_available_by_item = rowSums(!is.na(if (is.data.frame(data)) as.matrix(data) else data))
    )
  )
}

.mc_prepare_multirater_kappa_counts <- function(data,
                                                levels,
                                                min_raters) {
  if (!is.data.frame(data) && !is.matrix(data)) {
    abort_bad_arg("data", message = "must be a matrix or data frame when {.arg input} is {.val counts}.")
  }

  X <- if (is.matrix(data)) data else as.matrix(data)
  if (!is.numeric(X)) {
    abort_bad_arg("data", message = "must contain numeric count columns when {.arg input} is {.val counts}.")
  }
  if (ncol(X) < 2L) {
    abort_bad_arg("data", message = "must contain at least two category columns.")
  }
  if (anyNA(X)) {
    abort_bad_arg("data", message = "must not contain missing counts.")
  }
  if (any(!is.finite(X))) {
    abort_bad_arg("data", message = "must contain only finite counts.")
  }
  if (any(X < 0)) {
    abort_bad_arg("data", message = "must contain non-negative counts.")
  }
  if (any(abs(X - round(X)) > sqrt(.Machine$double.eps))) {
    abort_bad_arg("data", message = "must contain integer-like count values.")
  }

  categories <- if (!is.null(levels)) {
    if (length(levels) != ncol(X)) {
      abort_bad_arg("levels", message = "must have length equal to ncol(data) for {.arg input} = {.val counts}.")
    }
    levels
  } else if (!is.null(colnames(X))) {
    colnames(X)
  } else {
    paste0("category_", seq_len(ncol(X)))
  }

  if (length(categories) < 2L) {
    abort_bad_arg("levels", message = "must define at least two categories for multi-rater kappa.")
  }

  row_sums <- rowSums(X)
  keep <- row_sums >= min_raters
  counts <- matrix(
    as.integer(round(X[keep, , drop = FALSE])),
    ncol = ncol(X),
    dimnames = list(NULL, as.character(categories))
  )

  if (nrow(counts) < 2L) {
    abort_bad_arg(
      "data",
      message = "must retain at least two items with at least {.val {min_raters}} raters."
    )
  }

  list(
    counts = counts,
    categories = categories,
    n_levels = length(categories),
    retained_rows = keep,
    dropped_rows = !keep,
    na_method = "available",
    diagnostics = list(
      n_original = nrow(X),
      n_retained = nrow(counts),
      retained_rows = keep,
      dropped_rows = !keep,
      row_sums = row_sums
    )
  )
}

.mc_resolve_multirater_levels <- function(data, levels) {
  cols <- if (is.list(data) && !is.data.frame(data)) data else as.list(data)

  if (!is.null(levels)) {
    if (!is.atomic(levels) || length(levels) < 1L) {
      abort_bad_arg("levels", message = "must be an atomic vector of category labels.")
    }
    if (anyNA(levels)) {
      abort_bad_arg("levels", message = "must not contain missing values.")
    }
    keys <- as.character(levels)
    if (anyDuplicated(keys)) {
      abort_bad_arg("levels", message = "must not contain duplicate categories.")
    }
    observed <- unlist(lapply(cols, function(z) as.character(z[!is.na(z)])), use.names = FALSE)
    if (length(observed) && any(!observed %in% keys)) {
      abort_bad_arg("levels", message = "must contain every non-missing observed category.")
    }
    return(levels)
  }

  all_factor <- all(vapply(cols, is.factor, logical(1)))
  any_factor <- any(vapply(cols, is.factor, logical(1)))
  all_numericish <- all(vapply(cols, function(z) is.numeric(z) || is.integer(z) || is.logical(z), logical(1)))

  if (all_factor) {
    return(unique(unlist(lapply(cols, levels), use.names = FALSE)))
  }
  if (any_factor) {
    abort_bad_arg(
      "levels",
      message = "must be supplied when factor and non-factor rating columns are mixed."
    )
  }
  if (all_numericish) {
    observed <- unlist(lapply(cols, function(z) z[!is.na(z)]), use.names = FALSE)
    return(sort(unique(observed)))
  }
  unique(unlist(lapply(cols, function(z) as.character(z[!is.na(z)])), use.names = FALSE))
}

.mc_multirater_rating_column <- function(x, arg) {
  if (is.list(x) && !is.atomic(x)) {
    abort_bad_arg(arg, message = "must not be a list column.")
  }
  if (is.matrix(x) || is.data.frame(x)) {
    abort_bad_arg(arg, message = "must be a vector-like rating column.")
  }
  if (inherits(x, "Date") || inherits(x, "POSIXct") || inherits(x, "POSIXt") || inherits(x, "difftime")) {
    abort_bad_arg(arg, message = "must not be a date-time or time-difference column.")
  }
  if (is.factor(x)) {
    return(x)
  }
  if (is.character(x) || is.logical(x) || is.numeric(x) || is.integer(x)) {
    return(x)
  }
  abort_bad_arg(
    arg,
    message = "must be factor, ordered factor, character, logical, integer, or numeric."
  )
}

.mc_build_multirater_kappa_output <- function(raw,
                                              method,
                                              input,
                                              categories,
                                              ci,
                                              p_value,
                                              conf_level,
                                              na_method,
                                              min_raters,
                                              se_method,
                                              by_category,
                                              prep_diagnostics) {
  inference_requested <- isTRUE(ci) || isTRUE(p_value)
  se <- if (!is.null(raw$se)) as.numeric(raw$se) else NA_real_
  statistic <- if (isTRUE(p_value) && !is.null(raw$statistic)) as.numeric(raw$statistic) else NA_real_
  p_val <- if (isTRUE(p_value) && !is.null(raw$p_value)) as.numeric(raw$p_value) else NA_real_
  lwr <- if (isTRUE(ci) && !is.null(raw$lwr)) as.numeric(raw$lwr) else NA_real_
  upr <- if (isTRUE(ci) && !is.null(raw$upr)) as.numeric(raw$upr) else NA_real_

  out <- data.frame(
    method = method,
    kappa = as.numeric(raw$estimate),
    observed_agreement = as.numeric(raw$observed_agreement),
    expected_agreement = as.numeric(raw$expected_agreement),
    n_items = as.integer(raw$n_items),
    n_categories = as.integer(raw$n_categories),
    n_raters_min = as.integer(raw$n_raters_min),
    n_raters_max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced),
    n_ratings_total = as.integer(raw$n_ratings_total),
    se = se,
    statistic = statistic,
    p_value = p_val,
    lwr.ci = lwr,
    upr.ci = upr,
    conf.level = if (isTRUE(ci)) conf_level else NA_real_,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  by_category_df <- NULL
  if (isTRUE(by_category)) {
    by_category_df <- data.frame(
      category = as.character(categories),
      kappa = as.numeric(raw$by_category_estimate),
      observed_agreement = as.numeric(raw$by_category_observed_agreement),
      expected_agreement = as.numeric(raw$by_category_expected_agreement),
      se = if (!is.null(raw$by_category_se)) as.numeric(raw$by_category_se) else NA_real_,
      statistic = if (isTRUE(p_value) && !is.null(raw$by_category_statistic)) as.numeric(raw$by_category_statistic) else NA_real_,
      p_value = if (isTRUE(p_value) && !is.null(raw$by_category_p_value)) as.numeric(raw$by_category_p_value) else NA_real_,
      lwr.ci = if (isTRUE(ci) && !is.null(raw$by_category_lwr)) as.numeric(raw$by_category_lwr) else NA_real_,
      upr.ci = if (isTRUE(ci) && !is.null(raw$by_category_upr)) as.numeric(raw$by_category_upr) else NA_real_,
      n_items = as.integer(raw$n_items),
      n_ratings_total = as.integer(raw$n_ratings_total),
      category_count = as.integer(raw$category_counts),
      category_proportion = as.numeric(raw$category_proportions),
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  }

  class(out) <- c("multirater_kappa", "agreement_result", "data.frame")
  attr(out, "method") <- "multirater_kappa"
  attr(out, "description") <- "Multi-rater chance-corrected nominal agreement"
  attr(out, "package") <- "matrixCorr"
  attr(out, "input") <- input
  attr(out, "kappa_method") <- method
  attr(out, "exact") <- isTRUE(raw$exact)
  attr(out, "categories") <- categories
  attr(out, "n_categories") <- length(categories)
  attr(out, "n_items") <- as.integer(raw$n_items)
  attr(out, "n_raters") <- list(
    min = as.integer(raw$n_raters_min),
    max = as.integer(raw$n_raters_max),
    balanced = as.logical(raw$balanced)
  )
  attr(out, "na_method") <- na_method
  attr(out, "min_raters") <- min_raters
  attr(out, "diagnostics") <- c(
    list(
      category_counts = as.integer(raw$category_counts),
      category_proportions = as.numeric(raw$category_proportions),
      item_category_counts = {
        mat <- raw$item_category_counts
        if (!is.null(mat)) {
          mat <- as.matrix(mat)
          colnames(mat) <- as.character(categories)
          rownames(mat) <- paste0("item_", seq_len(nrow(mat)))
        }
        mat
      },
      item_agreement = as.numeric(raw$item_agreement),
      n_raters_by_item = as.integer(raw$n_raters_by_item)
    ),
    prep_diagnostics
  )
  attr(out, "by_category") <- by_category_df
  attr(out, "se.method") <- if (isTRUE(inference_requested)) se_method else "none"
  attr(out, "conf.level") <- if (isTRUE(ci)) conf_level else NULL
  attr(out, "reference") <- switch(
    method,
    fleiss = "Fleiss (1971), Psychological Bulletin, 76, 378-382, doi:10.1037/h0031619",
    randolph = "Randolph (2005), Free-Marginal Multirater Kappa"
  )
  out
}

#' @method print multirater_kappa
#' @rdname multirater_kappa
#' @param x A \code{multirater_kappa} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param show_by_category Logical; whether to print the attached category-wise
#'   kappa table when available.
#' @param ... Additional arguments passed to [print.data.frame()].
#' @export
print.multirater_kappa <- function(x,
                                   digits = 4,
                                   show_by_category = FALSE,
                                   ...) {
  check_inherits(x, "multirater_kappa")
  check_bool(show_by_category, arg = "show_by_category")

  nr <- attr(x, "n_raters", exact = TRUE)
  method_label <- attr(x, "kappa_method", exact = TRUE)
  if (isTRUE(attr(x, "exact", exact = TRUE))) {
    method_label <- paste0(method_label, " (exact)")
  }
  cat("Multi-rater kappa agreement\n")
  cat("  method             :", method_label, "\n")
  cat("  kappa              :", formatC(x$kappa[[1L]], format = "f", digits = digits), "\n")
  cat("  observed agreement :", formatC(x$observed_agreement[[1L]], format = "f", digits = digits), "\n")
  cat("  expected agreement :", formatC(x$expected_agreement[[1L]], format = "f", digits = digits), "\n")
  cat("  items              :", .mc_count_fmt(x$n_items[[1L]]), "\n")
  cat("  categories         :", .mc_count_fmt(x$n_categories[[1L]]), "\n")
  cat(
    "  raters per item    :",
    paste0(.mc_count_fmt(nr$min), " to ", .mc_count_fmt(nr$max)),
    if (isTRUE(nr$balanced)) "(balanced)" else "(unbalanced)",
    "\n"
  )
  if (is.finite(x$lwr.ci[[1L]]) && is.finite(x$upr.ci[[1L]])) {
    cat(
      "  CI                 :",
      sprintf(
        "%g%% [%s, %s]",
        100 * suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE))),
        formatC(x$lwr.ci[[1L]], format = "f", digits = digits),
        formatC(x$upr.ci[[1L]], format = "f", digits = digits)
      ),
      "\n"
    )
  }
  if (is.finite(x$p_value[[1L]])) {
    cat("  p-value            :", formatC(x$p_value[[1L]], format = "f", digits = digits), "\n")
  }

  if (isTRUE(show_by_category)) {
    by_cat <- attr(x, "by_category", exact = TRUE)
    if (is.data.frame(by_cat) && nrow(by_cat)) {
      cat("\nBy-category kappas\n\n")
      print.data.frame(by_cat, row.names = FALSE, ...)
    }
  }
  invisible(x)
}

#' @method summary multirater_kappa
#' @rdname multirater_kappa
#' @param object A \code{multirater_kappa} object.
#' @param digits Integer; number of decimal places for displayed values.
#' @param ... Unused.
#' @export
summary.multirater_kappa <- function(object,
                                     digits = 4,
                                     ...) {
  check_inherits(object, "multirater_kappa")

  out <- as.data.frame(object, stringsAsFactors = FALSE, check.names = FALSE)
  preferred_order <- c(
    "method",
    "kappa",
    "observed_agreement",
    "expected_agreement",
    "n_items",
    "n_categories",
    "n_raters_min",
    "n_raters_max",
    "balanced",
    "n_ratings_total",
    "se",
    "statistic",
    "p_value",
    "lwr.ci",
    "upr.ci",
    "conf.level"
  )
  out <- out[, c(intersect(preferred_order, names(out)), setdiff(names(out), preferred_order)), drop = FALSE]
  drop_if_all_na <- c("se", "statistic", "p_value", "lwr.ci", "upr.ci", "conf.level")
  keep_cols <- vapply(names(out), function(nm) {
    if (!nm %in% drop_if_all_na) {
      return(TRUE)
    }
    !all(is.na(out[[nm]]))
  }, logical(1))
  out <- out[, keep_cols, drop = FALSE]
  num_cols <- vapply(out, is.numeric, logical(1))
  out[num_cols] <- lapply(out[num_cols], round, digits = digits)
  out <- .mc_finalize_summary_df(out, class_name = "summary.multirater_kappa")
  out <- out[, c(intersect(preferred_order, names(out)), setdiff(names(out), preferred_order)), drop = FALSE]
  class(out) <- unique(c("summary.multirater_kappa", "summary.agreement_result", class(out)))

  diag_attr <- attr(object, "diagnostics", exact = TRUE)
  item_agreement <- as.numeric(diag_attr$item_agreement %||% numeric())
  n_raters_by_item <- as.integer(diag_attr$n_raters_by_item %||% integer())
  attr(out, "diagnostics_summary") <- list(
    category_counts = as.integer(diag_attr$category_counts %||% integer()),
    category_proportions = as.numeric(diag_attr$category_proportions %||% numeric()),
    item_agreement_summary = if (length(item_agreement)) {
      c(
        min = min(item_agreement, na.rm = TRUE),
        mean = mean(item_agreement, na.rm = TRUE),
        median = stats::median(item_agreement, na.rm = TRUE),
        max = max(item_agreement, na.rm = TRUE)
      )
    } else {
      c(min = NA_real_, mean = NA_real_, median = NA_real_, max = NA_real_)
    },
    n_raters_summary = if (length(n_raters_by_item)) {
      c(
        min = min(n_raters_by_item, na.rm = TRUE),
        mean = mean(n_raters_by_item, na.rm = TRUE),
        median = stats::median(n_raters_by_item, na.rm = TRUE),
        max = max(n_raters_by_item, na.rm = TRUE)
      )
    } else {
      c(min = NA_real_, mean = NA_real_, median = NA_real_, max = NA_real_)
    }
  )
  attr(out, "by_category") <- attr(object, "by_category", exact = TRUE)
  attr(out, "kappa_method") <- attr(object, "kappa_method", exact = TRUE)
  attr(out, "exact") <- attr(object, "exact", exact = TRUE)
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE)
  attr(out, "se.method") <- attr(object, "se.method", exact = TRUE)
  attr(out, "digits") <- digits
  out
}

#' @method print summary.multirater_kappa
#' @rdname multirater_kappa
#' @param n Optional preview row threshold.
#' @param topn Optional number of leading/trailing rows when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
print.summary.multirater_kappa <- function(x,
                                           digits = NULL,
                                           n = NULL,
                                           topn = NULL,
                                           max_vars = NULL,
                                           width = NULL,
                                           show_ci = NULL,
                                           ...) {
  digits <- .mc_coalesce(digits, attr(x, "digits", exact = TRUE) %||% 4L)
  digest <- c(
    method = if (isTRUE(attr(x, "exact", exact = TRUE))) {
      paste0(attr(x, "kappa_method", exact = TRUE) %||% NA_character_, " (exact)")
    } else {
      attr(x, "kappa_method", exact = TRUE) %||% NA_character_
    },
    kappa = formatC(x$kappa[[1L]], format = "f", digits = digits),
    items = .mc_count_fmt(x$n_items[[1L]]),
    categories = .mc_count_fmt(x$n_categories[[1L]])
  )
  se_method <- attr(x, "se.method", exact = TRUE) %||% "none"
  if (!identical(se_method, "none")) {
    digest <- c(digest, se_method = se_method)
  }
  digest <- digest[!is.na(digest) & nzchar(digest)]
  .mc_print_named_digest(digest, header = "Multi-rater kappa agreement summary")
  cat("\n")

  .mc_print_summary_table(
    x,
    header = "Estimate table",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    print_overview = FALSE,
    ...
  )
  invisible(x)
}

#' @method plot multirater_kappa
#' @rdname multirater_kappa
#' @param type Plot type: \code{"agreement_map"}, \code{"estimate"}, \code{"item_agreement"},
#'   \code{"category_proportion"}, or \code{"by_category"}.
#' @param bins Integer number of bins retained for compatibility; currently
#'   unused by the default item-agreement profile plot.
#' @export
plot.multirater_kappa <- function(x,
                                  type = c("agreement_map", "estimate", "item_agreement", "category_proportion", "by_category"),
                                  bins = 30L,
                                  ...) {
  check_inherits(x, "multirater_kappa")
  type <- match.arg(type)
  bins <- check_scalar_int_pos(bins, arg = "bins")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  diag_attr <- attr(x, "diagnostics", exact = TRUE)
  if (identical(type, "agreement_map")) {
    counts <- diag_attr$item_category_counts
    if (is.null(counts) || !is.matrix(counts) || !nrow(counts) || !ncol(counts)) {
      abort_bad_arg("type", message = "cannot be {.val agreement_map} because retained item-category counts are unavailable.")
    }
    item_agreement <- as.numeric(diag_attr$item_agreement %||% rep(NA_real_, nrow(counts)))
    n_raters_by_item <- as.numeric(diag_attr$n_raters_by_item %||% rowSums(counts))
    row_id <- seq_len(nrow(counts))
    dominant <- apply(counts, 1L, function(z) {
      idx <- which.max(z)
      if (!length(idx) || all(z == 0)) "" else colnames(counts)[[idx[[1L]]]]
    })
    ord <- order(item_agreement, decreasing = TRUE, na.last = TRUE)
    counts <- counts[ord, , drop = FALSE]
    item_agreement <- item_agreement[ord]
    n_raters_by_item <- n_raters_by_item[ord]
    dominant <- dominant[ord]
    row_id <- row_id[ord]

    df <- as.data.frame(as.table(counts), stringsAsFactors = FALSE)
    names(df) <- c("item", "category", "count")
    df$item_index <- match(df$item, rownames(counts))
    df$n_raters <- n_raters_by_item[df$item_index]
    df$share <- ifelse(df$n_raters > 0, df$count / df$n_raters, NA_real_)
    df$item_agreement <- item_agreement[df$item_index]
    df$dominant <- dominant[df$item_index]
    df$item_label <- sprintf(
      "item %s | a=%s | top=%s",
      row_id[df$item_index],
      formatC(df$item_agreement, format = "f", digits = 2),
      df$dominant
    )
    item_levels <- unique(df$item_label)
    df$item_label <- factor(df$item_label, levels = rev(item_levels))

    return(
      ggplot2::ggplot(df, ggplot2::aes(x = .data$category, y = .data$item_label, fill = .data$share)) +
        ggplot2::geom_tile(color = "white", linewidth = 0.5) +
        ggplot2::geom_text(
          ggplot2::aes(label = ifelse(.data$count > 0, .data$count, "")),
          size = 3,
          color = "black"
        ) +
        ggplot2::scale_fill_gradient(
          low = "#F3E9DC",
          high = "#274C77",
          limits = c(0, 1),
          name = "Share of raters"
        ) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(
          panel.grid = ggplot2::element_blank(),
          axis.title.y = ggplot2::element_blank(),
          ...
        ) +
        ggplot2::labs(
          title = "Item-by-category agreement map",
          subtitle = "Items ordered by item-level agreement; tile labels are rater counts",
          x = "Category"
        )
    )
  }

  if (identical(type, "estimate")) {
    df <- data.frame(
      label = "Panel kappa",
      x = x$kappa[[1L]],
      lwr = x$lwr.ci[[1L]],
      upr = x$upr.ci[[1L]],
      stringsAsFactors = FALSE
    )
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$x, y = .data$label)) +
      ggplot2::geom_vline(xintercept = 0, linewidth = 0.5, color = "grey80") +
      ggplot2::geom_vline(xintercept = 1, linewidth = 0.4, color = "grey90", linetype = "dashed") +
      ggplot2::geom_vline(xintercept = -1, linewidth = 0.4, color = "grey90", linetype = "dashed")
    if (is.finite(df$lwr[[1L]]) && is.finite(df$upr[[1L]])) {
      p <- p + ggplot2::geom_segment(
        ggplot2::aes(x = .data$lwr, xend = .data$upr, y = .data$label, yend = .data$label),
        linewidth = 1.2,
        color = "#5B7C99"
      )
    }
    return(
      p +
        ggplot2::geom_point(size = 3.2, color = "#1F4E79") +
        ggplot2::coord_cartesian(xlim = c(-1, 1)) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(
          panel.grid.major.y = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          axis.title.y = ggplot2::element_blank(),
          ...
        ) +
        ggplot2::labs(
          title = "Multi-rater kappa estimate",
          subtitle = sprintf(
            "%s method; %s items across %s categories",
            if (isTRUE(attr(x, "exact", exact = TRUE))) {
              paste0(attr(x, "kappa_method", exact = TRUE), " (exact)")
            } else {
              attr(x, "kappa_method", exact = TRUE)
            },
            .mc_count_fmt(x$n_items[[1L]]),
            .mc_count_fmt(x$n_categories[[1L]])
          ),
          x = "Kappa"
        )
    )
  }

  if (identical(type, "item_agreement")) {
    df <- data.frame(
      item = seq_along(diag_attr$item_agreement %||% numeric()),
      item_agreement = sort(as.numeric(diag_attr$item_agreement %||% numeric())),
      stringsAsFactors = FALSE
    )
    return(
      ggplot2::ggplot(df, ggplot2::aes(x = .data$item, y = .data$item_agreement)) +
        ggplot2::geom_hline(
          yintercept = x$observed_agreement[[1L]],
          linewidth = 0.6,
          linetype = "dashed",
          color = "#B04A5A"
        ) +
        ggplot2::geom_linerange(
          ggplot2::aes(ymin = 0, ymax = .data$item_agreement),
          linewidth = 0.8,
          color = "#7AA6C2"
        ) +
        ggplot2::geom_point(size = 2.3, color = "#1F4E79") +
        ggplot2::coord_cartesian(ylim = c(0, 1)) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
        ggplot2::labs(
          title = "Item-level agreement profile",
          subtitle = sprintf(
            "Dashed line shows mean observed agreement = %s",
            formatC(x$observed_agreement[[1L]], format = "f", digits = 3)
          ),
          x = "Items ordered by agreement",
          y = "Agreement"
        )
    )
  }

  if (identical(type, "category_proportion")) {
    df <- data.frame(
      category = factor(
        as.character(attr(x, "categories", exact = TRUE)),
        levels = rev(as.character(attr(x, "categories", exact = TRUE)))
      ),
      proportion = as.numeric(diag_attr$category_proportions %||% numeric()),
      stringsAsFactors = FALSE
    )
    return(
      ggplot2::ggplot(df, ggplot2::aes(y = .data$category, x = .data$proportion)) +
        ggplot2::geom_segment(
          ggplot2::aes(x = 0, xend = .data$proportion, yend = .data$category),
          linewidth = 1,
          color = "#C5A46D"
        ) +
        ggplot2::geom_point(size = 3, color = "#8C5A2B") +
        ggplot2::coord_cartesian(xlim = c(0, 1)) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
        ggplot2::labs(
          title = "Marginal category proportions",
          x = "Proportion",
          y = "Category"
        )
    )
  }

  by_cat <- attr(x, "by_category", exact = TRUE)
  if (!is.data.frame(by_cat) || !nrow(by_cat)) {
    abort_bad_arg(
      "type",
      message = "cannot be {.val by_category} unless {.arg by_category} was requested in the original fit."
    )
  }
  by_cat$category <- factor(by_cat$category, levels = by_cat$category[order(by_cat$kappa, decreasing = TRUE)])
  p <- ggplot2::ggplot(by_cat, ggplot2::aes(y = .data$category, x = .data$kappa)) +
    ggplot2::geom_vline(xintercept = 0, linewidth = 0.5, color = "grey80") +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(panel.grid.minor = ggplot2::element_blank(), ...) +
    ggplot2::labs(
      title = "Category-wise binary-collapsed kappas",
      x = "Kappa",
      y = "Category"
    )
  if (all(c("lwr.ci", "upr.ci") %in% names(by_cat)) &&
      any(is.finite(by_cat$lwr.ci) & is.finite(by_cat$upr.ci))) {
    p <- p +
      ggplot2::geom_segment(
        data = by_cat[is.finite(by_cat$lwr.ci) & is.finite(by_cat$upr.ci), , drop = FALSE],
        ggplot2::aes(x = .data$lwr.ci, xend = .data$upr.ci, yend = .data$category),
        linewidth = 1,
        color = "#7E9F45"
      )
  }
  p +
    ggplot2::geom_point(size = 3, color = "#4C6A2F") +
    ggplot2::coord_cartesian(xlim = c(-1, 1))
}

#' @title Partial correlation matrix (sample / ridge / OAS / graphical lasso)
#'
#' @description
#' Computes Gaussian partial correlations for the numeric columns of a matrix
#' or data frame using a high-performance 'C++' backend. Covariance estimation
#' is available via the classical sample estimator, ridge regularisation, OAS
#' shrinkage, or graphical lasso. Optional p-values and Fisher-z confidence
#' intervals are available for the classical sample estimator in the ordinary
#' low-dimensional setting.
#'
#' @param data A numeric matrix or data frame with at least two numeric columns.
#'   Non-numeric columns are ignored.
#' @param method Character; one of \code{"sample"}, \code{"oas"},
#'   \code{"ridge"}, or \code{"glasso"}. Default \code{"sample"}.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"complete"} performs listwise deletion once across the retained
#'   numeric columns and then computes the estimator on the common complete
#'   sample. Pairwise deletion is not supported for partial correlations.
#' @param lambda Numeric \eqn{\ge 0}; regularisation strength. For
#'   \code{method = "ridge"}, this is the penalty added to the covariance
#'   diagonal. For \code{method = "glasso"}, this is the off-diagonal
#'   precision-matrix \eqn{\ell_1} penalty. Ignored otherwise. Default
#'   \code{1e-3}.
#' @param return_cov_precision Logical; if \code{TRUE}, also return the
#'   covariance (\code{cov}) and precision (\code{precision}) matrices used to
#'   form the partial correlations. Requires \code{return_details = TRUE}.
#'   Default to \code{FALSE}.
#' @param return_details Logical; if \code{TRUE}, return the rich
#'   \code{"partial_corr"} list containing \code{pcor}, diagnostics, method
#'   metadata, and any requested covariance, precision, p-value, or CI
#'   components. If \code{FALSE} (default), \code{output = "matrix"} returns the
#'   standard matrixCorr estimate matrix, consistent with other correlation
#'   methods.
#' @param return_p_value Logical; if \code{TRUE}, compute two-sided p-values
#'   for testing whether each sample partial correlation is zero. With the
#'   default matrix result, p-values are stored in the inference metadata and
#'   exposed by \code{tidy()}. With \code{return_details = TRUE}, they are also
#'   returned as the \code{p_value} matrix. This option is available only for
#'   \code{method = "sample"} and requires \eqn{n > p}. Default to
#'   \code{FALSE}.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach Fisher-z
#'   confidence intervals for the off-diagonal partial correlations. This
#'   option is available only for the classical \code{method = "sample"}
#'   estimator in the ordinary low-dimensional setting.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#'   \code{0.95}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return With the default \code{return_details = FALSE}, a standard
#'   matrixCorr correlation result: a dense \code{corr_matrix}, sparse matrix,
#'   or \code{corr_edge_list}, depending on \code{output}. With
#'   \code{return_details = TRUE}, an object of class \code{"partial_corr"} (a
#'   list) with elements:
#'   \itemize{
#'     \item \code{pcor}: \eqn{p \times p} partial correlation matrix.
#'     \item \code{cov} (if requested): covariance matrix used.
#'     \item \code{precision} (if requested): precision matrix \eqn{\Theta}.
#'     \item \code{p_value} (if requested): matrix of two-sided p-values for
#'     the sample partial correlations.
#'     \item \code{ci} (if requested): a list with elements \code{est},
#'     \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and \code{ci.method}.
#'     \item \code{diagnostics}: metadata used for inference, including the
#'     effective complete-case sample size and number of conditioned variables.
#'     \item \code{method}: the estimator used (\code{"oas"}, \code{"ridge"},
#'     \code{"sample"}, or \code{"glasso"}).
#'     \item \code{lambda}: ridge or graphical-lasso penalty
#'     (or \code{NA_real_}).
#'     \item \code{rho}: OAS shrinkage weight in \eqn{[0,1]} (or \code{NA_real_}).
#'     \item \code{jitter}: diagonal jitter added (if any) to ensure positive
#'     definiteness.
#'   }
#'
#' @details
#' \strong{Statistical overview.} Given an \eqn{n \times p} data matrix \eqn{X}
#' (rows are observations, columns are variables), the routine estimates a
#' \emph{partial correlation} matrix via the precision (inverse covariance)
#' matrix. Let \eqn{\mu} be the vector of column means and
#' \deqn{S = (X - \mathbf{1}\mu)^\top (X - \mathbf{1}\mu)}
#' be the centred cross-product matrix computed without forming a centred copy
#' of \eqn{X}. Two conventional covariance scalings are formed:
#' \deqn{\hat\Sigma_{\mathrm{MLE}} = S/n, \qquad
#'       \hat\Sigma_{\mathrm{unb}} = S/(n-1).}
#'
#' \itemize{
#'   \item \emph{Sample:} \eqn{\Sigma = \hat\Sigma_{\mathrm{unb}}}.
#'   \item \emph{Ridge:} \eqn{\Sigma = \hat\Sigma_{\mathrm{unb}} + \lambda I_p}
#'         with user-supplied \eqn{\lambda \ge 0} (diagonal inflation).
#'   \item \emph{OAS (Oracle Approximating Shrinkage):}
#'         shrink \eqn{\hat\Sigma_{\mathrm{MLE}}} towards a scaled identity
#'         target \eqn{\mu_I I_p}, where \eqn{\mu_I = \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})/p}.
#'         The data-driven weight \eqn{\rho \in [0,1]} is
#'         \deqn{\rho = \min\!\left\{1,\max\!\left(0,\;
#'         \frac{(1-\tfrac{2}{p})\,\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}^2)
#'               + \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})^2}
#'              {(n + 1 - \tfrac{2}{p})
#'               \left[\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}^2)
#'               - \tfrac{\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})^2}{p}\right]}
#'         \right)\right\},}
#'         and
#'         \deqn{\Sigma = (1-\rho)\,\hat\Sigma_{\mathrm{MLE}} + \rho\,\mu_I I_p.}
#'   \item \emph{Graphical lasso:} estimate a sparse precision matrix
#'         \eqn{\Theta} by maximising
#'         \deqn{\log\det(\Theta) - \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}\Theta)
#'         - \lambda\sum_{i \ne j} |\theta_{ij}|,}
#'         with \eqn{\lambda \ge 0}. The returned covariance matrix is
#'         \eqn{\Sigma = \Theta^{-1}}.
#' }
#'
#' The method then ensures positive definiteness of \eqn{\Sigma} (adding a very
#' small diagonal \emph{jitter} only if necessary) and computes the precision
#' matrix \eqn{\Theta = \Sigma^{-1}}. Partial correlations are obtained by
#' standardising the off-diagonals of \eqn{\Theta}:
#' \deqn{\mathrm{pcor}_{ij} \;=\;
#'       -\,\frac{\theta_{ij}}{\sqrt{\theta_{ii}\,\theta_{jj}}}, \qquad
#'       \mathrm{pcor}_{ii}=1.}
#'
#' If \code{return_p_value = TRUE}, the function also reports the classical
#' two-sided test p-values for the sample partial correlations, using
#' \deqn{t_{ij} = \mathrm{pcor}_{ij}
#'   \sqrt{\frac{n - p}{1 - \mathrm{pcor}_{ij}^2}}}
#' with \eqn{n - p} degrees of freedom. These p-values are returned only for
#' \code{method = "sample"}, where they match the standard full-model partial
#' correlation test.
#'
#' When \code{ci = TRUE}, the function reports Fisher-\eqn{z} confidence
#' intervals for the sample partial correlations. For a partial correlation
#' \eqn{r_{xy \cdot Z}} conditioning on \eqn{c} variables, the transformed
#' statistic is \eqn{z = \operatorname{atanh}(r_{xy \cdot Z})} with standard
#' error
#' \deqn{\operatorname{SE}(z) = \frac{1}{\sqrt{n - 3 - c}},}
#' where \eqn{n} is the effective complete-case sample size used for the
#' estimate. The two-sided normal-theory interval is formed on the transformed
#' scale using \code{conf_level} and then mapped back with \code{tanh()}. In
#' the full matrix path implemented here, each off-diagonal entry conditions on
#' all remaining variables, so \eqn{c = p - 2} and the classical CI requires
#' \eqn{n > p + 1}. This inference is only supported for
#' \code{method = "sample"} without positive-definiteness repair; in
#' unsupported or numerically singular settings, CI bounds are returned as
#' \code{NA} with an informative \pkg{cli} warning or the request is rejected.
#'
#' \strong{Interpretation.} For Gaussian data, \eqn{\mathrm{pcor}_{ij}} equals
#' the correlation between residuals from regressing variable \eqn{i} and
#' variable \eqn{j} on all the remaining variables; equivalently, it encodes
#' conditional dependence in a Gaussian graphical model, where
#' \eqn{\mathrm{pcor}_{ij}=0} if variables \eqn{i} and \eqn{j} are
#' conditionally independent given the others. Partial correlations are
#' invariant to separate rescalings of each
#' variable; in particular, multiplying \eqn{\Sigma} by any positive scalar
#' leaves the partial correlations unchanged.
#'
#' \strong{Why shrinkage/regularisation?} When \eqn{p \ge n}, the sample
#' covariance is singular and inversion is ill-posed. Ridge and OAS both yield
#' well-conditioned \eqn{\Sigma}. Ridge adds a fixed \eqn{\lambda} on the
#' diagonal, whereas OAS shrinks adaptively towards \eqn{\mu_I I_p} with a
#' weight chosen to minimise (approximately) the Frobenius risk under a
#' Gaussian model, often improving mean-square accuracy in high dimension.
#'
#' \strong{Why glasso?} Glasso is useful when the goal is not just to
#' stabilise a covariance estimate, but to recover a manageable network of
#' direct relationships rather than a dense matrix of overall associations. In
#' Gaussian models, zeros in the precision matrix correspond to conditional
#' independences, so glasso can suppress indirect associations that are
#' explained by the other variables and return a smaller, more interpretable
#' conditional-dependence graph. This is especially practical in
#' high-dimensional settings, where the sample covariance may be unstable or
#' singular. Glasso yields a positive-definite precision estimate and supports
#' edge selection, graph recovery, and downstream network analysis.
#'
#' \strong{Computational notes.} The implementation forms \eqn{S} using 'BLAS'
#' \code{syrk} when available and constructs partial correlations by traversing
#' only the upper triangle with 'OpenMP' parallelism. Positive definiteness is
#' verified via a Cholesky factorisation; if it fails, a tiny diagonal jitter is
#' increased geometrically up to a small cap, at which point the routine
#' signals an error.
#'
#' @examples
#' ## Structured MVN with known partial correlations
#' set.seed(42)
#' p <- 12; n <- 1000
#'
#' ## Build a tri-diagonal precision (Omega) so the true partial correlations
#' ## are sparse
#' phi <- 0.35
#' Omega <- diag(p)
#' for (j in 1:(p - 1)) {
#'   Omega[j, j + 1] <- Omega[j + 1, j] <- -phi
#' }
#' ## Strict diagonal dominance
#' diag(Omega) <- 1 + 2 * abs(phi) + 0.05
#' Sigma <- solve(Omega)
#'
#' ## Upper Cholesky
#' L <- chol(Sigma)
#' Z <- matrix(rnorm(n * p), n, p)
#' X <- Z %*% L
#' colnames(X) <- sprintf("V%02d", seq_len(p))
#'
#' pc <- pcorr(X)
#' summary(pc)
#' estimate(pc)
#' tidy(pc)
#'
#' ## Fisher-z confidence intervals for sample partial correlations
#' pc_ci <- pcorr(X[, 1:5], ci = TRUE)
#' ci(pc_ci)
#' confint(pc_ci)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(pc)
#' }
#'
#' ## True partial correlation from Omega
#' pcor_true <- -Omega / sqrt(diag(Omega) %o% diag(Omega))
#' diag(pcor_true) <- 1
#'
#' ## Quick visual check (first 5x5 block)
#' round(estimate(pc)[1:5, 1:5], 2)
#' round(pcor_true[1:5, 1:5], 2)
#'
#' ## Plot method
#' plot(pc)
#'
#' ## Graphical-lasso example
#' set.seed(100)
#' p <- 20; n <- 250
#' Theta_g <- diag(p)
#' Theta_g[cbind(1:5, 2:6)] <- -0.25
#' Theta_g[cbind(2:6, 1:5)] <- -0.25
#' Theta_g[cbind(8:11, 9:12)] <- -0.20
#' Theta_g[cbind(9:12, 8:11)] <- -0.20
#' diag(Theta_g) <- rowSums(abs(Theta_g)) + 0.2
#'
#' Sigma_g <- solve(Theta_g)
#' X_g <- matrix(rnorm(n * p), n, p) %*% chol(Sigma_g)
#' colnames(X_g) <- paste0("Node", seq_len(p))
#'
#' gfit_1 <- pcorr(X_g, method = "glasso", lambda = 0.02,
#'                 return_cov_precision = TRUE, return_details = TRUE)
#' gfit_2 <- pcorr(X_g, method = "glasso", lambda = 0.08,
#'                 return_cov_precision = TRUE, return_details = TRUE)
#'
#' ## Larger lambda gives a sparser conditional-dependence graph
#' edge_count <- function(M, tol = 1e-8) {
#'   sum(abs(M[upper.tri(M, diag = FALSE)]) > tol)
#' }
#'
#' c(edges_lambda_002 = edge_count(gfit_1$precision),
#'   edges_lambda_008 = edge_count(gfit_2$precision))
#'
#' ## Inspect strongest estimated conditional associations
#' pcor_g <- gfit_1$pcor
#' idx <- which(upper.tri(pcor_g), arr.ind = TRUE)
#' ord <- order(abs(pcor_g[idx]), decreasing = TRUE)
#' head(data.frame(
#'   i = rownames(pcor_g)[idx[ord, 1]],
#'   j = colnames(pcor_g)[idx[ord, 2]],
#'   pcor = round(pcor_g[idx][ord], 2)
#' ))
#'
#' ## High-dimensional case p >> n
#' set.seed(7)
#' n <- 60; p <- 120
#'
#' ar_block <- function(m, rho = 0.6) rho^abs(outer(seq_len(m), seq_len(m), "-"))
#'
#' ## Two AR(1) blocks on the diagonal
#' if (requireNamespace("Matrix", quietly = TRUE)) {
#'   Sigma_hd <- as.matrix(Matrix::bdiag(ar_block(60, 0.6), ar_block(60, 0.6)))
#' } else {
#'   Sigma_hd <- rbind(
#'     cbind(ar_block(60, 0.6), matrix(0, 60, 60)),
#'     cbind(matrix(0, 60, 60), ar_block(60, 0.6))
#'   )
#' }
#'
#' L <- chol(Sigma_hd)
#' X_hd <- matrix(rnorm(n * p), n, p) %*% L
#' colnames(X_hd) <- paste0("G", seq_len(p))
#'
#' pc_oas   <-
#'  pcorr(X_hd, method = "oas",   return_cov_precision = TRUE,
#'        return_details = TRUE)
#' pc_ridge <-
#'  pcorr(X_hd, method = "ridge", lambda = 1e-2,
#'                      return_cov_precision = TRUE, return_details = TRUE)
#' pc_samp  <-
#'  pcorr(X_hd, method = "sample", return_cov_precision = TRUE,
#'        return_details = TRUE)
#' pc_glasso <-
#'  pcorr(X_hd, method = "glasso", lambda = 5e-3,
#'                      return_cov_precision = TRUE, return_details = TRUE)
#'
#' ## Show how much diagonal regularisation was used
#' c(oas_jitter = pc_oas$jitter,
#'   ridge_lambda = pc_ridge$lambda,
#'   sample_jitter = pc_samp$jitter,
#'   glasso_lambda = pc_glasso$lambda)
#'
#' ## Compare conditioning of the estimated covariance matrices
#' c(kappa_oas = kappa(pc_oas$cov),
#'   kappa_ridge = kappa(pc_ridge$cov),
#'   kappa_sample = kappa(pc_samp$cov))
#'
#' ## Simple conditional-dependence graph from partial correlations
#' pcor <- pc_oas$pcor
#' vals <- abs(pcor[upper.tri(pcor, diag = FALSE)])
#' thresh <- quantile(vals, 0.98)  # top 2%
#' edges  <- which(abs(pcor) > thresh & upper.tri(pcor), arr.ind = TRUE)
#' head(data.frame(i = colnames(pcor)[edges[,1]],
#'                 j = colnames(pcor)[edges[,2]],
#'                 pcor = round(pcor[edges], 2)))
#'
#' @references
#' Chen, Y., Wiesel, A., & Hero, A. O. III (2011).
#' Robust Shrinkage Estimation of High-dimensional Covariance Matrices.
#' IEEE Transactions on Signal Processing.
#'
#' @references
#' Friedman, J., Hastie, T., & Tibshirani, R. (2007).
#' Sparse inverse covariance estimation with the graphical lasso.
#' Biostatistics.
#'
#' @references
#' Ledoit, O., & Wolf, M. (2004).
#' A well-conditioned estimator for large-dimensional covariance matrices.
#' Journal of Multivariate Analysis, 88(2), 365-411.
#'
#' @references
#' Schafer, J., & Strimmer, K. (2005).
#' A shrinkage approach to large-scale covariance matrix estimation and
#' implications for functional genomics.
#' Statistical Applications in Genetics and Molecular Biology, 4(1), Article 32.
#'
#' @export
pcorr <- function(data, method = c("sample","oas","ridge","glasso"),
                                na_method = c("error", "complete"),
                                ci = FALSE, conf_level = 0.95,
                                return_cov_precision = FALSE,
                                return_details = FALSE,
                                return_p_value = FALSE, lambda = 1e-3,
                                output = c("matrix", "sparse", "edge_list"),
                                threshold = 0,
                                diag = TRUE) {
  output_cfg <- .mc_validate_output_args(
    output = output,
    threshold = threshold,
    diag = diag
  )
  method <- match.arg(method)
  na_method <- match.arg(na_method)
  lambda <- check_scalar_numeric(lambda, arg = "lambda", lower = 0, closed_lower = TRUE)
  lambda <- as.numeric(lambda)
  check_bool(return_cov_precision, arg = "return_cov_precision")
  check_bool(return_details, arg = "return_details")
  check_bool(return_p_value, arg = "return_p_value")
  check_bool(ci, arg = "ci")
  if (isTRUE(ci)) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  }
  if (isTRUE(return_cov_precision) && !isTRUE(return_details)) {
    abort_bad_arg(
      "return_details",
      message = "must be TRUE when `return_cov_precision = TRUE`."
    )
  }
  if (!identical(output_cfg$output, "matrix") &&
      (isTRUE(return_cov_precision) || isTRUE(return_details) ||
       isTRUE(return_p_value) || isTRUE(ci))) {
    abort_bad_arg(
      "output",
      message = "non-matrix outputs currently support point estimates only.",
      .hint = "Set `.arg return_cov_precision = FALSE`, `.arg return_details = FALSE`, `.arg return_p_value = FALSE`, and `.arg ci = FALSE`."
    )
  }

  numeric_data <- validate_corr_input(
    data,
    check_na = identical(na_method, "error")
  )
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = 2L, arg = "data")
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }

  if (isTRUE(return_p_value) && !identical(method, "sample")) {
    cli::cli_abort(
      "{.arg return_p_value} is available only for {.code method = \"sample\"}."
    )
  }
  if (isTRUE(return_p_value) && nrow(numeric_data) <= ncol(numeric_data)) {
    cli::cli_abort(
      "{.arg return_p_value} requires {.code n > p} so that the sample partial-correlation test has positive degrees of freedom."
    )
  }
  if (isTRUE(ci) && !identical(method, "sample")) {
    cli::cli_abort(
      "{.arg ci} is available only for the classical {.code method = \"sample\"} partial correlation."
    )
  }
  if (isTRUE(ci) && nrow(numeric_data) <= (ncol(numeric_data) + 1L)) {
    cli::cli_abort(
      "{.arg ci} requires {.code n > p + 1} so that the Fisher-z partial-correlation interval has positive degrees of freedom."
    )
  }

  res <- partial_correlation_cpp(
    numeric_data,
    method,
    lambda,
    return_cov_precision,
    return_p_value
  )

  # set dimnames (cheap; attributes only)
  dn <- list(colnames(numeric_data), colnames(numeric_data))
  if (!is.null(res$pcor)) {
    res$pcor <- structure(res$pcor, dimnames = dn)
    if (!is.null(res$p_value))   res$p_value <- structure(res$p_value, dimnames = dn)
    if (!is.null(res$cov))       res$cov <- structure(res$cov, dimnames = dn)
    if (!is.null(res$precision)) res$precision <- structure(res$precision, dimnames = dn)
  } else {
    res <- list(pcor = structure(res[[1L]], dimnames = dn))
  }

  diagnostics <- list(
    n_complete = matrix(
      as.integer(nrow(numeric_data)),
      nrow = ncol(numeric_data),
      ncol = ncol(numeric_data),
      dimnames = dn
    ),
    n_conditioning = matrix(
      as.integer(ncol(numeric_data) - 2L),
      nrow = ncol(numeric_data),
      ncol = ncol(numeric_data),
      dimnames = dn
    )
  )
  diagnostics <- .mc_merge_diagnostics(diagnostics, diagnostics_extra)
  diag(diagnostics$n_conditioning) <- 0L
  ci_attr <- NULL
  if (isTRUE(ci)) {
    ci_needs_repair <- .mc_sample_covariance_needs_ci_repair(numeric_data)
    ci_attr <- .mc_partial_corr_fisher_ci(
      pcor = res$pcor,
      n_complete = nrow(numeric_data),
      n_conditioning = ncol(numeric_data) - 2L,
      conf_level = conf_level,
      jitter = res$jitter %||% NA_real_,
      needs_repair = ci_needs_repair
    )
  }

  res$method <- method
  res$lambda <- if (method %in% c("ridge", "glasso")) lambda else NA_real_
  res$rho    <- if (identical(method, "oas"))   res$rho %||% NA_real_ else NA_real_
  res$jitter <- res$jitter %||% NA_real_
  res$diagnostics <- diagnostics
  if (!is.null(ci_attr)) {
    res$ci <- ci_attr
  }
  out <- structure(
    res,
    class = c("partial_corr", "list"),
    method = method,
    ci = ci_attr,
    conf.level = if (!is.null(ci_attr)) conf_level else NULL,
    ci.method = if (!is.null(ci_attr)) ci_attr$ci.method else NULL
  )
  if (isTRUE(return_details)) {
    return(out)
  }

  inference <- NULL
  if (isTRUE(return_p_value)) {
    inference <- list(
      method = "partial_correlation_t_test",
      p.value = res$p_value,
      p_value = res$p_value,
      statistic = NULL,
      parameter = matrix(
        as.integer(nrow(numeric_data) - ncol(numeric_data)),
        nrow = ncol(numeric_data),
        ncol = ncol(numeric_data),
        dimnames = dn
      )
    )
  }
  pcor_obj <- .mc_structure_corr_matrix(
    out$pcor,
    class_name = "partial_corr_matrix",
    method = paste0("partial_correlation_", method),
    description = "Partial correlation matrix",
    symmetric = TRUE,
    diagnostics = diagnostics,
    dimnames = dn,
    classes = c("partial_corr_matrix", "matrix"),
    extra_attrs = c(
      list(
        partial_method = method,
        lambda = out$lambda,
        rho = out$rho,
        jitter = out$jitter
      ),
      if (!is.null(ci_attr)) {
        list(
          ci = ci_attr,
          conf.level = conf_level,
          ci.method = ci_attr$ci.method
        )
      } else {
        list()
      },
      if (!is.null(inference)) list(inference = inference) else list()
    )
  )
  .mc_finalize_corr_output_fast(
    pcor_obj,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_sample_covariance_needs_ci_repair <- function(x) {
  centered <- scale(as.matrix(x), center = TRUE, scale = FALSE)
  qr(centered)$rank < ncol(centered)
}

.mc_partial_corr_ci_attr <- function(x) {
  if (inherits(x, "partial_corr") && !is.null(x$ci)) {
    return(x$ci)
  }
  attr(x, "ci", exact = TRUE)
}

.mc_partial_corr_pairwise_summary <- function(object,
                                              digits = 4,
                                              ci_digits = 3,
                                              show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "partial_corr")

  ci <- object$ci %||% attr(object, "ci", exact = TRUE)
  diag_attr <- object$diagnostics %||% attr(object, "diagnostics", exact = TRUE)
  p_value <- object$p_value %||% NULL
  .mc_pairwise_matrix_summary(
    object$pcor,
    class_name = "summary.partial_corr",
    digits = digits,
    ci_digits = ci_digits,
    show_ci = show_ci,
    ci_attr = ci,
    diagnostics_attr = diag_attr,
    include_p = NULL,
    extra_columns = if (!is.null(p_value) && is.matrix(p_value) &&
      identical(dim(p_value), dim(as.matrix(object$pcor)))) {
      list(p_value = list(matrix = p_value, digits = NULL))
    },
    overview = .mc_summary_corr_matrix(object$pcor)
  )
}

.mc_partial_corr_fisher_ci <- function(pcor,
                                       n_complete,
                                       n_conditioning,
                                       conf_level = 0.95,
                                       jitter = NA_real_,
                                       needs_repair = FALSE) {
  pcor <- as.matrix(pcor)
  p <- ncol(pcor)
  dn <- dimnames(pcor)
  lwr <- matrix(NA_real_, p, p, dimnames = dn)
  upr <- matrix(NA_real_, p, p, dimnames = dn)
  diag(lwr) <- 1
  diag(upr) <- 1

  out <- list(
    est = unclass(pcor),
    lwr.ci = lwr,
    upr.ci = upr,
    conf.level = conf_level,
    ci.method = "fisher_z_partial"
  )

  if (isTRUE(needs_repair) || (!is.na(jitter) && is.finite(jitter) && jitter > 0)) {
    cli::cli_warn(
      c(
        "Partial-correlation confidence intervals are unavailable for this fit.",
        "i" = "The sample covariance was rank-deficient or required positive-definiteness repair, so the classical Fisher-z partial-correlation interval is not identified.",
        "i" = "Returning {.code NA} confidence bounds."
      ),
      class = c("matrixCorr_warning", "matrixCorr_ci_warning")
    )
    return(out)
  }

  se_denom <- as.numeric(n_complete - 3L - n_conditioning)
  if (!is.finite(se_denom) || se_denom <= 0) {
    cli::cli_abort(
      "{.arg ci} requires positive Fisher-z residual degrees of freedom; got {.code n_complete - 3 - c = {se_denom}}."
    )
  }

  crit <- stats::qnorm(0.5 * (1 + conf_level))
  se <- 1 / sqrt(se_denom)
  eps <- sqrt(.Machine$double.eps)
  boundary_pairs <- 0L

  for (j in seq_len(p - 1L)) {
    for (i in (j + 1L):p) {
      r <- pcor[j, i]
      if (!is.finite(r)) next
      if (abs(r) >= 1) {
        boundary_pairs <- boundary_pairs + 1L
        next
      }
      r_safe <- max(min(r, 1 - eps), -1 + eps)
      z <- atanh(r_safe)
      lo <- tanh(z - crit * se)
      hi <- tanh(z + crit * se)
      lwr[j, i] <- lwr[i, j] <- max(-1, lo)
      upr[j, i] <- upr[i, j] <- min(1, hi)
    }
  }

  out$lwr.ci <- lwr
  out$upr.ci <- upr
  if (boundary_pairs > 0L) {
    cli::cli_warn(
      "{boundary_pairs} partial-correlation pair{?s} were at the boundary +/-1; returning {.code NA} confidence bounds for those pair{?s}.",
      boundary_pairs = boundary_pairs,
      class = c("matrixCorr_warning", "matrixCorr_ci_warning")
    )
  }
  out
}

#' @rdname pcorr
#' @title Print method for \code{partial_corr}
#'
#' @param x An object of class \code{partial_corr}.
#' @param digits Integer; number of decimal places for display (default 3).
#' @param show_method Logical; print a one-line header with \code{method}
#'   (and \code{lambda}/\code{rho} if available). Default \code{TRUE}.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Further arguments passed to \code{print.matrix()}.
#' @return Invisibly returns \code{x}.
#' @method print partial_corr
#' @importFrom utils capture.output
#' @export
print.partial_corr <- function(
    x,
    digits = 3,
    show_method = TRUE,
    n = NULL,
    topn = NULL,
    max_vars = NULL,
    width = NULL,
    show_ci = NULL,
    ...
) {
  check_inherits(x, "partial_corr")
  M <- x$pcor
  check_matrix_dims(M, arg = "x$pcor")
  M <- as.matrix(M)

  lines <- character()
  if (isTRUE(show_method)) {
    meth <- if (!is.null(x$method)) as.character(x$method) else NA_character_
    hdr <- switch(
      tolower(meth),
      "oas"   = {
        rho <- if (!is.null(x$rho) && is.finite(x$rho)) sprintf(", OAS rho=%.3f", x$rho) else ""
        paste0("Partial correlation (OAS", rho, ")")
      },
      "ridge" = {
        lam <- if (!is.null(x$lambda) && is.finite(x$lambda)) sprintf(", lambda=%.3g", x$lambda) else ""
        paste0("Partial correlation (ridge", lam, ")")
      },
      "glasso" = {
        lam <- if (!is.null(x$lambda) && is.finite(x$lambda)) sprintf(", lambda=%.3g", x$lambda) else ""
        paste0("Partial correlation (glasso", lam, ")")
      },
      "sample" = "Partial correlation (sample covariance)",
      "Partial correlation"
    )
    lines <- c(lines, hdr)
  } else {
    lines <- c(lines, "Partial correlation matrix:")
  }

  if (length(lines)) writeLines(lines)
  cat("\n")
  .mc_print_corr_matrix(
    x,
    header = "Partial correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = M,
    ...
  )

  invisible(x)
}

#' @rdname pcorr
#' @method summary partial_corr
#' @title Summary Method for \code{partial_corr} Objects
#'
#' @param object An object of class \code{partial_corr}.
#' @param ... Unused.
#'
#' @return A compact summary object of class \code{summary.partial_corr}.
#' @export
summary.partial_corr <- function(object, n = NULL, topn = NULL,
                                 max_vars = NULL, width = NULL,
                                 show_ci = NULL, ...) {
  check_inherits(object, "partial_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  if (!is.null(.mc_partial_corr_ci_attr(object))) {
    return(.mc_partial_corr_pairwise_summary(
      object,
      show_ci = show_ci
    ))
  }
  out <- .mc_summary_corr_matrix(object$pcor, topn = topn)
  out$class <- "partial_corr"
  out$method <- object$method %||% attr(object, "method")
  out$lambda <- object$lambda %||% NA_real_
  out$rho <- object$rho %||% NA_real_
  out$jitter <- object$jitter %||% NA_real_
  out$header <- "Correlation summary"
  class(out) <- c("summary.partial_corr", "summary.matrixCorr", "summary.corr_matrix")
  out
}

#' @rdname pcorr
#' @method print summary.partial_corr
#' @export
print.summary.partial_corr <- function(x, digits = 4, n = NULL, topn = NULL,
                                       max_vars = NULL, width = NULL,
                                       show_ci = NULL, ...) {
  if (inherits(x, "data.frame")) {
    .mc_print_pairwise_summary_digest(
      x,
      title = "Partial correlation summary",
      digits = .mc_coalesce(digits, 4),
      n = n,
      topn = topn,
      max_vars = max_vars,
      width = width,
      show_ci = show_ci,
      ci_method = "fisher_z_partial",
      ...
    )
    return(invisible(x))
  }
  print.summary.matrixCorr(
    x,
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
}

print.summary_partial_corr <- print.summary.partial_corr


#' @rdname pcorr
#' @method plot partial_corr
#' @title Plot Method for \code{partial_corr} Objects
#'
#' @param x An object of class \code{partial_corr}.
#' @param title Plot title. By default, constructed from the estimator in
#' \code{x$method}.
#' @param low_color Colour for low (negative) values. Default
#' \code{"indianred1"}.
#' @param high_color Colour for high (positive) values. Default
#' \code{"steelblue1"}.
#' @param mid_color Colour for zero. Default \code{"white"}.
#' @param value_text_size Font size for cell labels. Default \code{4}.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param mask_diag Logical; if \code{TRUE}, the diagonal is masked
#' (set to \code{NA}) and not labelled. Default \code{TRUE}.
#' @param reorder Logical; if \code{TRUE}, variables are reordered by
#' hierarchical clustering of \eqn{1 - |pcor|}. Default \code{FALSE}.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other
#'   \pkg{ggplot2} layers.
#'
#' @return A \code{ggplot} object.
#' @import ggplot2
#' @importFrom stats as.dist hclust
#' @export
plot.partial_corr <- function(
    x,
    title = NULL,
    low_color  = "indianred1",
    high_color = "steelblue1",
    mid_color  = "white",
    value_text_size = 4,
    show_value = TRUE,
    mask_diag = TRUE,
    reorder   = FALSE,
    ...
) {
  check_inherits(x, "partial_corr")
  check_bool(show_value, arg = "show_value")
  check_bool(mask_diag, arg = "mask_diag")
  check_bool(reorder,   arg = "reorder")
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }

  M <- x$pcor
  check_matrix_dims(M, arg = "x$pcor")

  # Ensure dimnames for labelling
  if (is.null(colnames(M))) colnames(M) <- paste0("V", seq_len(ncol(M)))
  if (is.null(rownames(M))) rownames(M) <- colnames(M)

  # Optional reordering by hierarchical clustering of 1 - |pcor|
  if (isTRUE(reorder) && nrow(M) >= 2L) {
    if (nrow(M) == ncol(M)) {
      dist_mat <- 1 - pmin(1, abs(M))
      dist_mat <- (dist_mat + t(dist_mat)) / 2
      diag(dist_mat) <- 0
      dist_mat[!is.finite(dist_mat)] <- 0
      dist_mat <- as.matrix(dist_mat)
      if (nrow(dist_mat) == ncol(dist_mat)) {
        D <- tryCatch(
          stats::as.dist(dist_mat),
          warning = function(w) {
            if (grepl("non-square matrix", conditionMessage(w), fixed = TRUE)) {
              return(NULL)
            }
            msg <- conditionMessage(w)
            cli::cli_warn("{msg}", msg = msg)
            NULL
          }
        )
        if (!is.null(D) && isTRUE(attr(D, "Size") >= 2)) {
          hc <- stats::hclust(D, method = "average")
          ord <- hc$order
          M <- M[ord, ord, drop = FALSE]
        }
      }
    }
  }

  # Default title constructed from x$method (+ tuning, if present)
  if (is.null(title)) {
    method <- tolower(as.character(x$method %||% ""))
    extra <- switch(
      method,
      "oas"   = if (is.finite(x$rho %||% NA_real_)) sprintf(" (OAS, rho=%.3f)", x$rho) else " (OAS)",
      "ridge" = if (is.finite(x$lambda %||% NA_real_)) sprintf(" (ridge, lambda=%.3g)", x$lambda) else " (ridge)",
      "glasso" = if (is.finite(x$lambda %||% NA_real_)) sprintf(" (glasso, lambda=%.3g)", x$lambda) else " (glasso)",
      "sample" = " (sample)",
      ""
    )
    title <- paste0("Partial correlation heatmap", extra)
  }

  # Build plotting frame
  df <- as.data.frame(as.table(M))
  names(df) <- c("Var1", "Var2", "PCor")

  # Mask diagonal if requested
  if (isTRUE(mask_diag)) {
    df$PCor[df$Var1 == df$Var2] <- NA_real_
  }

  # Reverse y-axis order for a tidy heatmap
  df$Var1 <- factor(df$Var1, levels = rev(unique(df$Var1)))

  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = PCor)) +
    ggplot2::geom_tile(colour = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color, high = high_color, mid = mid_color,
      midpoint = 0, limits = c(-1, 1), name = "Partial r",
      na.value = "grey95"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid  = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value) && !is.null(value_text_size) && is.finite(value_text_size)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ifelse(is.na(PCor), "", sprintf("%.2f", PCor))),
      size = value_text_size, colour = "black"
    )
  }

  p
}

case_output_args <- function(output, threshold_non_matrix = 0.2) {
  if (identical(output, "matrix")) {
    list(output = "matrix", threshold = 0, diag = TRUE)
  } else {
    list(output = output, threshold = threshold_non_matrix, diag = FALSE)
  }
}

assert_s3_dispatch <- function(obj,
                               case_label,
                               expected_summary_class,
                               expected_output = NULL,
                               expected_has_ci = NULL,
                               expected_has_p = NULL) {
  sm <- summary(obj)
  expect_s3_class(sm, expected_summary_class)

  if (inherits(sm, "summary.corr_result")) {
    expect_false(inherits(sm, "summaryDefault"), info = case_label)
    if (!is.null(expected_output)) {
      expect_identical(attr(sm, "output", exact = TRUE), expected_output, info = case_label)
    }
    if (!is.null(expected_has_ci)) {
      expect_identical(isTRUE(attr(sm, "has_ci", exact = TRUE)), expected_has_ci, info = case_label)
    }
    if (!is.null(expected_has_p)) {
      expect_identical(isTRUE(attr(sm, "has_p", exact = TRUE)), expected_has_p, info = case_label)
    }
  }

  gp <- plot(obj, show_value = FALSE)
  expect_s3_class(gp, "ggplot")
}

test_that("S3 dispatch works across all core correlation estimators and output combinations", {
  skip_if_not_installed("ggplot2")

  set.seed(20260417)
  X <- matrix(rnorm(180 * 5), nrow = 180, ncol = 5)
  colnames(X) <- paste0("V", seq_len(ncol(X)))

  Z <- matrix(rnorm(320 * 4), nrow = 320, ncol = 4)
  X_bin <- data.frame(
    b1 = Z[, 1] > -0.2,
    b2 = Z[, 2] > 0.1,
    b3 = Z[, 3] > 0.4,
    b4 = Z[, 4] > -0.5
  )
  X_ord <- data.frame(
    o1 = ordered(cut(Z[, 1], breaks = c(-Inf, -0.4, 0.3, Inf), labels = c("L", "M", "H"))),
    o2 = ordered(cut(Z[, 2], breaks = c(-Inf, -0.8, -0.1, 0.6, Inf), labels = c("1", "2", "3", "4"))),
    o3 = ordered(cut(Z[, 3], breaks = c(-Inf, -0.2, 0.5, Inf), labels = c("A", "B", "C"))),
    o4 = ordered(cut(Z[, 4], breaks = c(-Inf, -0.3, 0.2, Inf), labels = c("Q", "R", "S")))
  )
  X_wk <- data.frame(
    w1 = ordered(cut(Z[, 1], breaks = c(-Inf, -0.4, 0.2, Inf), labels = c("low", "mid", "high")),
                 levels = c("low", "mid", "high")),
    w2 = ordered(cut(Z[, 2], breaks = c(-Inf, -0.3, 0.5, Inf), labels = c("low", "mid", "high")),
                 levels = c("low", "mid", "high")),
    w3 = ordered(cut(Z[, 3], breaks = c(-Inf, -0.1, 0.7, Inf), labels = c("low", "mid", "high")),
                 levels = c("low", "mid", "high")),
    w4 = ordered(cut(Z[, 4], breaks = c(-Inf, -0.2, 0.4, Inf), labels = c("low", "mid", "high")),
                 levels = c("low", "mid", "high"))
  )
  X_nom <- data.frame(
    k1 = c("A", "A", "B", "B", "C", "A", "B", "C"),
    k2 = c("A", "B", "B", "B", "C", "A", "B", "C"),
    k3 = c("A", "A", "B", "C", "C", "A", "B", "B"),
    k4 = c("A", "B", "A", "B", "C", "C", "B", "C"),
    stringsAsFactors = FALSE
  )

  outputs <- c("matrix", "sparse", "edge_list")

  ci_methods <- list(
    pearson_corr = function(output, ci) {
      do.call(
        pearson_corr,
        c(list(data = X, ci = ci), case_output_args(output))
      )
    },
    spearman_rho = function(output, ci) {
      do.call(
        spearman_rho,
        c(list(data = X, ci = ci), case_output_args(output))
      )
    },
    kendall_tau = function(output, ci) {
      do.call(
        kendall_tau,
        c(list(data = X, ci = ci), case_output_args(output))
      )
    },
    bicor = function(output, ci) {
      do.call(
        bicor,
        c(list(data = X, ci = ci), case_output_args(output))
      )
    },
    ccc = function(output, ci) {
      do.call(
        ccc,
        c(list(data = X, ci = ci), case_output_args(output))
      )
    },
    tetrachoric = function(output, ci) {
      do.call(
        tetrachoric,
        c(list(data = X_bin, ci = ci, p_value = ci), case_output_args(output, threshold_non_matrix = 0.1))
      )
    },
    polychoric = function(output, ci) {
      do.call(
        polychoric,
        c(list(data = X_ord, ci = ci, p_value = ci), case_output_args(output, threshold_non_matrix = 0.1))
      )
    },
    cohen_kappa = function(output, ci) {
      do.call(
        cohen_kappa,
        c(list(data = X_nom, na_method = "error", ci = ci, p_value = ci), case_output_args(output, threshold_non_matrix = 0.1))
      )
    },
    weighted_kappa = function(output, ci) {
      do.call(
        weighted_kappa,
        c(list(data = X_wk, na_method = "error", ci = ci, p_value = ci), case_output_args(output, threshold_non_matrix = 0.1))
      )
    }
  )

  for (nm in names(ci_methods)) {
    fn <- ci_methods[[nm]]
    for (output in outputs) {
      for (ci in c(FALSE, TRUE)) {
        case_label <- sprintf("%s output=%s ci=%s", nm, output, ci)
        obj <- fn(output, ci)
        expected_class <- if (identical(nm, "ccc") && identical(output, "matrix")) "summary.ccc" else "summary.corr_result"
        assert_s3_dispatch(
          obj = obj,
          case_label = case_label,
          expected_summary_class = expected_class,
          expected_output = if (identical(expected_class, "summary.corr_result")) output else NULL,
          expected_has_ci = if (identical(expected_class, "summary.corr_result")) ci else NULL
        )
      }
    }
  }

  for (output in outputs) {
    out_args <- case_output_args(output)
    for (p_value in c(FALSE, TRUE)) {
      case_label <- sprintf("dcor output=%s p_value=%s", output, p_value)
      obj <- do.call(
        dcor,
        c(list(data = X, p_value = p_value), out_args)
      )
      assert_s3_dispatch(
        obj = obj,
        case_label = case_label,
        expected_summary_class = "summary.corr_result",
        expected_output = output,
        expected_has_ci = FALSE,
        expected_has_p = p_value
      )
    }
  }

  robust_methods <- list(
    pbcor = function(output, infer) {
      do.call(
        pbcor,
        c(
          list(data = X, ci = infer, p_value = infer, n_boot = 20, seed = 42),
          case_output_args(output)
        )
      )
    },
    wincor = function(output, infer) {
      do.call(
        wincor,
        c(
          list(data = X, ci = infer, p_value = infer, n_boot = 20, seed = 42),
          case_output_args(output)
        )
      )
    },
    skipped_corr = function(output, infer) {
      do.call(
        skipped_corr,
        c(
          list(
            data = X,
            method = "pearson",
            ci = infer,
            p_value = infer,
            n_boot = 30,
            seed = 42,
            return_masks = FALSE
          ),
          case_output_args(output)
        )
      )
    }
  )

  for (nm in names(robust_methods)) {
    fn <- robust_methods[[nm]]
    for (output in outputs) {
      for (infer in c(FALSE, TRUE)) {
        case_label <- sprintf("%s output=%s infer=%s", nm, output, infer)
        obj <- fn(output, infer)
        assert_s3_dispatch(
          obj = obj,
          case_label = case_label,
          expected_summary_class = "summary.corr_result",
          expected_output = output,
          expected_has_ci = infer,
          expected_has_p = if (identical(nm, "skipped_corr")) NULL else infer
        )
      }
    }
  }

  aliases <- list(
    shrinkage_corr = function(output) {
      do.call(shrinkage_corr, c(list(data = X), case_output_args(output)))
    },
    schafer_corr = function(output) {
      do.call(schafer_corr, c(list(data = X), case_output_args(output)))
    }
  )

  for (nm in names(aliases)) {
    fn <- aliases[[nm]]
    for (output in outputs) {
      case_label <- sprintf("%s output=%s", nm, output)
      obj <- fn(output)
      assert_s3_dispatch(
        obj = obj,
        case_label = case_label,
        expected_summary_class = "summary.corr_result",
        expected_output = output,
        expected_has_ci = FALSE
      )
    }
  }
})

test_that("S3 dispatch works for partial correlation matrix and non-matrix outputs", {
  skip_if_not_installed("ggplot2")

  set.seed(20260418)
  X <- matrix(rnorm(220 * 5), nrow = 220, ncol = 5)
  colnames(X) <- paste0("P", seq_len(ncol(X)))

  matrix_fit <- pcorr(
    X,
    method = "sample",
    ci = TRUE,
    return_p_value = TRUE,
    output = "matrix"
  )
  sm_matrix <- summary(matrix_fit)
  expect_s3_class(sm_matrix, "summary.corr_result")
  expect_s3_class(sm_matrix, "summary.partial_corr_matrix")
  expect_s3_class(plot(matrix_fit, show_value = FALSE), "ggplot")

  sparse_fit <- pcorr(
    X,
    method = "sample",
    ci = FALSE,
    return_p_value = FALSE,
    output = "sparse",
    threshold = 0.2,
    diag = FALSE
  )
  assert_s3_dispatch(
    obj = sparse_fit,
    case_label = "pcorr sample sparse",
    expected_summary_class = "summary.corr_result",
    expected_output = "sparse",
    expected_has_ci = FALSE
  )

  edge_fit <- pcorr(
    X,
    method = "ridge",
    ci = FALSE,
    return_p_value = FALSE,
    output = "edge_list",
    threshold = 0.2,
    diag = FALSE
  )
  assert_s3_dispatch(
    obj = edge_fit,
    case_label = "pcorr ridge edge_list",
    expected_summary_class = "summary.corr_result",
    expected_output = "edge_list",
    expected_has_ci = FALSE
  )
})

#' @title Repeated-Measures Coefficient of Individual Agreement
#'
#' @description
#' Computes pairwise repeated-measures coefficients of individual agreement
#' (CIA) from long-format matched repeated-measures data using the categorical
#' condition ANOVA formulation of Haber, Gao, and Barnhart (2010).
#'
#' @details
#' `cia_rm()` is for matched repeated measurements under conditions such as
#' raters, visits, laboratories, treatments, or time points. It is not a
#' technical-replicate estimator. If the same subject has true technical
#' replicates within each subject-method cell, use [cia()] instead.
#'
#' Let `Y_ijk` denote the measurement on subject `i`, with method `j`, under
#' condition `k`, and consider the model
#' \deqn{
#' Y_{ijk} = \mu + \alpha_i + \beta_j + \gamma_k +
#' (\alpha\beta)_{ij} + (\alpha\gamma)_{ik} + (\beta\gamma)_{jk} + e_{ijk}.
#' }
#' Here subject is random, method and time/condition are fixed,
#' subject-method and subject-time are random interactions, method-time is a
#' fixed interaction, and `e_ijk` is the residual.
#'
#' For each method pair and condition `k`, the repeated-measures CIA is
#' \deqn{
#' \psi_k = \frac{2\sigma_e^2}{d_k^2 + 2\sigma_{\alpha\beta}^2 + 2\sigma_e^2},
#' }
#' where `d_k` is the condition-specific mean difference between the two
#' methods, \eqn{\sigma_{\alpha\beta}^2} is the subject-method variance
#' component, and \eqn{\sigma_e^2} is the residual variance component.
#'
#' The function fits this estimator separately to every method pair. The
#' homogeneity of agreement across conditions is tested by the method-time
#' interaction:
#' \deqn{
#' F = \frac{MS_{\mathrm{method}\times\mathrm{time}}}{MS_{\mathrm{error}}}.
#' }
#' The returned `homogeneity_F` attribute stores this test statistic for each
#' method pair, and `homogeneity_p` stores the corresponding upper-tail
#' `F`-test p-value using degrees of freedom `df_method_time` and `df_error`.
#' Larger `homogeneity_F` values and smaller `homogeneity_p` values indicate
#' stronger evidence that agreement changes across conditions, meaning that the
#' condition-specific CIA curves should be interpreted directly rather than
#' relying on a single pooled/common coefficient. Conversely, a small
#' `homogeneity_F` and a non-small `homogeneity_p` indicate that the data do
#' not show strong evidence of method-by-condition heterogeneity, so a common
#' estimate may be a reasonable summary if it is scientifically meaningful.
#'
#' When `homogeneous = TRUE`, the function also reports a common or pooled CIA
#' for each method pair from the reduced model that pools the method-time sum of
#' squares into the residual term. That common estimate is meaningful only when
#' agreement is reasonably homogeneous across conditions.
#'
#' Confidence intervals can be computed by a delta-method normal approximation
#' or by a subject-level percentile bootstrap. The delta-method interval is the
#' default. For each method pair and condition `k`, let
#' \eqn{\hat\psi_k} be the ANOVA estimator, let \eqn{g_k} be its numerical
#' gradient with respect to the subject-level moment vector, and let
#' \eqn{\Sigma} be the empirical covariance matrix of that moment vector divided
#' by the number of subjects. The delta-method standard error is
#' \deqn{
#' \widehat{\mathrm{se}}(\hat\psi_k) =
#' \sqrt{\max\left(g_k^\top \Sigma g_k, 0\right)}.
#' }
#' The raw normal interval is
#' \deqn{
#' \hat\psi_k \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\psi_k).
#' }
#' Under the bootstrap option, the interval is given by the empirical
#' percentile limits from subject-resampled estimates. The argument
#' `estimator` controls whether the reported result is the literal
#' method-of-moments ratio or a bounded variance-component variant. Under
#' `estimator = "vc_constrained"`, the estimated subject-method variance
#' component is constrained to be non-negative before converting it back to CIA,
#' and the reported interval limits are also truncated to the CIA parameter
#' space,
#' \deqn{
#' \mathrm{lwr} = \max(0, \mathrm{lwr}_{\mathrm{raw}}), \qquad
#' \mathrm{upr} = \min(1, \mathrm{upr}_{\mathrm{raw}}).
#' }
#' This keeps the reported interval inside the CIA parameter space `[0, 1]`.
#' Use `estimator = "mom_unconstrained"` to inspect the literal raw
#' method-of-moments estimator and the corresponding unbounded interval on the
#' estimator scale.
#'
#' The current implementation requires exactly one observation in every
#' subject-method-time cell. It therefore targets the balanced repeated-measures
#' ANOVA setting from the cited paper and returns an error otherwise.
#'
#' @param data A data frame in long format.
#' @param response Character scalar naming the numeric response column.
#' @param subject Character scalar naming the subject identifier column.
#' @param method Character scalar naming the method column. Required.
#' @param time Character scalar naming the repeated condition column. Required.
#' @param ci Logical; if `TRUE`, attach confidence intervals.
#' @param conf_level Confidence level for intervals when `ci = TRUE`.
#' @param n_threads Positive integer thread hint passed to the C++ backend.
#' @param estimator One of `"vc_constrained"` or `"mom_unconstrained"`.
#'   `"vc_constrained"` is the default bounded variance-component estimator. It
#'   constrains the estimated subject-method variance component to be
#'   non-negative before converting it back to CIA. `"mom_unconstrained"` is
#'   the raw method-of-moments CIA ratio and may exceed 1.
#' @param inference One of `"delta"`, `"bootstrap"`, or `"none"`. The delta
#'   method is the default when `ci = TRUE`.
#' @param verbose Logical; if `TRUE`, emit progress messages for CI work.
#' @param digits Integer print precision carried on the returned object.
#' @param use_message Logical; if `TRUE`, use package messaging helpers when
#'   `verbose = TRUE`.
#' @param homogeneous Logical; if `TRUE`, also compute the pooled/common CIA
#'   from the reduced model without method-time interaction.
#' @param B Number of subject-bootstrap resamples when `ci = TRUE`.
#' @param seed Optional positive integer seed for reproducible bootstrap
#'   resampling.
#' @param ... Unused.
#'
#' @return
#' If `ci = FALSE`, the result is a list of class `c("cia_rm", "cia")`.
#' If `ci = TRUE`, the result is a list of class
#' `c("cia_rm", "cia_ci", "cia")`.
#'
#' Main components:
#' \itemize{
#'   \item `est`: condition-specific pairwise CIA estimates.
#'   \item `common`: pairwise overall summary CIA estimates from the reduced
#'   homogeneous model.
#'   \item `condition`: the ordered condition/time labels used in the output.
#'   \item `se`: standard errors for `est` when `ci = TRUE`.
#'   \item `lwr.ci` and `upr.ci`: lower and upper confidence limits for `est`
#'   when `ci = TRUE`.
#'   \item `common.se`: standard errors for `common` when `ci = TRUE`.
#'   \item `common.lwr.ci` and `common.upr.ci`: lower and upper confidence
#'   limits for `common` when `ci = TRUE`.
#' }
#'
#' When there are three or more methods, additional overall multi-method
#' components are returned:
#' \itemize{
#'   \item `overall`: condition-specific overall CIA across all methods.
#'   \item `overall.common`: the overall summary CIA across all methods when
#'   `homogeneous = TRUE`.
#'   \item `overall.se`: standard errors for `overall` when `ci = TRUE`.
#'   \item `overall.lwr.ci` and `overall.upr.ci`: lower and upper confidence
#'   limits for `overall` when `ci = TRUE`.
#'   \item `overall.common.se`: standard errors for `overall.common` when
#'   `ci = TRUE`.
#'   \item `overall.common.lwr.ci` and `overall.common.upr.ci`: lower and upper
#'   confidence limits for `overall.common` when `ci = TRUE`.
#' }
#'
#' The object carries pairwise ANOVA diagnostics as attributes, including
#' `sigma2_error`, `sigma2_subject_method`, `repeatability`,
#' `homogeneity_F`, `homogeneity_p`, `df_method_time`, `df_error`,
#' `n_obs`, `n_subjects`, `n_methods`, `n_times`, `time_levels`, and
#' `method_levels`. Here `homogeneity_F` is the method-time interaction test
#' statistic for each method pair, and `homogeneity_p` is the corresponding
#' p-value for the null hypothesis of homogeneous agreement across conditions.
#'
#' @seealso [ccc_rm_reml()], [icc_rm_reml()], and [cia()].
#'
#' @references
#' Haber M, Gao J, Barnhart HX. (2010). Evaluation of agreement between
#' measurement methods from data with matched repeated measurements via the
#' coefficient of individual agreement. \emph{Journal of Data Science}, 8,
#' 457-469.
#'
#' Barnhart HX, Kosinski AS, Haber M. (2007). Assessing individual agreement.
#' \emph{Journal of Biopharmaceutical Statistics}, 17(4), 697-719.
#'
#' @examples
#' # Example 1
#' set.seed(1)
#' dat_rater <- expand.grid(
#'   id = factor(sprintf("s%02d", 1:8)),
#'   method = factor(c("Device_A", "Device_B")),
#'   time = factor(c("Rater_1", "Rater_2", "Rater_3")),
#'   KEEP.OUT.ATTRS = FALSE
#' )
#' subj_eff <- rnorm(nlevels(dat_rater$id), sd = 1)[dat_rater$id]
#' method_shift <- c(Device_A = 0, Device_B = 0.25)[dat_rater$method]
#' rater_shift <- c(Rater_1 = 0, Rater_2 = 0.15, Rater_3 = -0.10)[dat_rater$time]
#' interaction_shift <- ifelse(
#'   dat_rater$method == "Device_B" & dat_rater$time == "Rater_3",
#'   0.12, 0
#' )
#' dat_rater$y <- subj_eff + method_shift + rater_shift +
#'   interaction_shift + rnorm(nrow(dat_rater), sd = 0.25)
#'
#' fit_rater <- cia_rm(
#'   dat_rater,
#'   response = "y",
#'   subject = "id",
#'   method = "method",
#'   time = "time",
#'   homogeneous = TRUE
#' )
#' print(fit_rater)
#' summary(fit_rater)
#' estimate(fit_rater)
#' tidy(fit_rater)
#' plot(fit_rater)
#'
#' # Example 2
#' set.seed(2)
#' dat_time <- expand.grid(
#'   id = factor(sprintf("s%02d", 1:10)),
#'   method = factor(c("Assay_A", "Assay_B", "Assay_C", "Assay_D")),
#'   time = factor(
#'     c("baseline", "week2", "month1", "month2", "month3"),
#'     levels = c("baseline", "week2", "month1", "month2", "month3")
#'   ),
#'   KEEP.OUT.ATTRS = FALSE
#' )
#' subj_eff <- rnorm(nlevels(dat_time$id), sd = 0.9)[dat_time$id]
#' method_shift <- c(Assay_A = 0, Assay_B = 0.20, Assay_C = -0.10, Assay_D = 0.35)[dat_time$method]
#' time_shift <- c(
#'   baseline = 0, week2 = 0.10, month1 = 0.22, month2 = 0.32, month3 = 0.42
#' )[dat_time$time]
#' interaction_shift <- ifelse(dat_time$method == "Assay_B" & dat_time$time == "month3", 0.10, 0) +
#'   ifelse(dat_time$method == "Assay_D" & dat_time$time == "month2", -0.08, 0)
#' dat_time$y <- subj_eff + method_shift + time_shift +
#'   interaction_shift + rnorm(nrow(dat_time), sd = 0.30)
#'
#' fit_time <- cia_rm(
#'   dat_time,
#'   response = "y",
#'   subject = "id",
#'   method = "method",
#'   time = "time",
#'   ci = TRUE
#' )
#' print(fit_time)
#' plot(fit_time)
#'
#' # Example 3
#' set.seed(3)
#' dat_days <- expand.grid(
#'   id = factor(sprintf("s%02d", 1:12)),
#'   method = factor(c("Sensor_A", "Sensor_B", "Sensor_C")),
#'   time = 1:15,
#'   KEEP.OUT.ATTRS = FALSE
#' )
#' subj_eff <- rnorm(nlevels(dat_days$id), sd = 0.8)[dat_days$id]
#' method_shift <- c(Sensor_A = 0, Sensor_B = 0.15, Sensor_C = -0.08)[dat_days$method]
#' day_trend <- 0.05 * dat_days$time
#' interaction_shift <- ifelse(dat_days$method == "Sensor_B", 0.01 * dat_days$time, 0) +
#'   ifelse(dat_days$method == "Sensor_C", -0.005 * dat_days$time, 0)
#' dat_days$y <- subj_eff + method_shift + day_trend +
#'   interaction_shift + rnorm(nrow(dat_days), sd = 0.22)
#'
#' fit_days <- cia_rm(
#'   dat_days,
#'   response = "y",
#'   subject = "id",
#'   method = "method",
#'   time = "time",
#'   ci = TRUE
#' )
#' plot(fit_days, facet_by_pair = TRUE)
#'
#' @author Thiago de Paula Oliveira
#' @export
cia_rm <- function(data,
                   response,
                   subject,
                   method = NULL,
                   time = NULL,
                   ci = FALSE,
                   conf_level = 0.95,
                   n_threads = getOption("matrixCorr.threads", 1L),
                   estimator = c("vc_constrained", "mom_unconstrained"),
                   inference = c("delta", "bootstrap", "none"),
                   verbose = FALSE,
                   digits = 4,
                   use_message = TRUE,
                   homogeneous = FALSE,
                   B = 1000L,
                   seed = NULL,
                   ...) {
  estimator <- match.arg(estimator)
  inference <- match.arg(inference)
  check_bool(ci, arg = "ci")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")
  digits <- check_scalar_int_pos(digits, arg = "digits")
  check_bool(use_message, arg = "use_message")
  check_bool(homogeneous, arg = "homogeneous")
  B <- check_scalar_int_pos(B, arg = "B")
  if (B < 2L) {
    abort_bad_arg("B", message = "must be >= 2.")
  }
  if (!is.null(seed)) {
    seed <- check_scalar_int_pos(seed, arg = "seed")
  }
  if (is.null(method)) {
    abort_bad_arg("method", message = "must be supplied.")
  }
  if (is.null(time)) {
    abort_bad_arg("time", message = "must be supplied.")
  }
  if (isTRUE(ci) && identical(inference, "none")) {
    abort_bad_arg("inference", message = "must not be {.val none} when {.arg ci} is TRUE.")
  }
  if (!isTRUE(ci)) {
    inference <- "none"
  }

  prep <- .mc_cia_rm_prepare(
    data = data,
    response = response,
    subject = subject,
    method = method,
    time = time
  )

  raw <- cia_rm_anova_cpp(
    y = prep$y,
    subject = prep$subject_code,
    method = prep$method_code,
    time = prep$time_code,
    n_subjects = prep$n_subjects,
    n_methods = prep$n_methods,
    n_times = prep$n_times,
    homogeneous = homogeneous,
    constrain_vc = identical(estimator, "vc_constrained"),
    n_threads = n_threads
  )

  out <- .mc_cia_rm_build_object(
    prep = prep,
    raw = raw,
    ci = ci,
    conf_level = conf_level,
    homogeneous = homogeneous,
    digits = digits
  )
  attr(out, "estimator") <- estimator

  if (!isTRUE(ci)) {
    return(out)
  }

  if (isTRUE(verbose)) {
    .mc_cia_rm_note(
      sprintf(
        "Computing %s confidence intervals for repeated-measures CIA.",
        if (identical(inference, "delta")) "delta-method" else "bootstrap"
      ),
      use_message = use_message
    )
  }

  if (identical(inference, "delta")) {
    delta <- .mc_cia_rm_delta(
      prep = prep,
      raw = raw,
      conf_level = conf_level,
      homogeneous = homogeneous,
      estimator = estimator,
      n_threads = n_threads
    )
    out$se <- delta$se
    out$lwr.ci <- delta$lwr
    out$upr.ci <- delta$upr
    out$common.se <- delta$common_se
    out$common.lwr.ci <- delta$common_lwr
    out$common.upr.ci <- delta$common_upr
    if (prep$n_methods >= 3L) {
      out$overall.se <- delta$overall_se
      out$overall.lwr.ci <- delta$overall_lwr
      out$overall.upr.ci <- delta$overall_upr
      out$overall.common.se <- delta$overall_common_se
      out$overall.common.lwr.ci <- delta$overall_common_lwr
      out$overall.common.upr.ci <- delta$overall_common_upr
    }
    attr(out, "ci.method") <- "delta_normal"
    attr(out, "estimator") <- estimator
  } else if (identical(inference, "bootstrap")) {
    boot <- .mc_cia_rm_bootstrap(
      prep = prep,
      homogeneous = homogeneous,
      B = B,
      conf_level = conf_level,
      seed = seed,
      n_threads = n_threads,
      estimator = estimator
    )
    out$lwr.ci <- boot$lwr
    out$upr.ci <- boot$upr
    out$common.lwr.ci <- boot$common_lwr
    out$common.upr.ci <- boot$common_upr
    if (prep$n_methods >= 3L) {
      out$overall.lwr.ci <- boot$overall_lwr
      out$overall.upr.ci <- boot$overall_upr
      out$overall.common.lwr.ci <- boot$overall_common_lwr
      out$overall.common.upr.ci <- boot$overall_common_upr
    }
    attr(out, "ci.method") <- "subject_bootstrap_percentile"
    attr(out, "estimator") <- estimator
    attr(out, "bootstrap") <- boot[c("n_successful", "B")]
  }

  out
}

.mc_cia_rm_prepare <- function(data, response, subject, method, time) {
  df <- as.data.frame(data)
  cols <- c(response, subject, method, time)
  names(cols) <- c("response", "subject", "method", "time")
  for (nm in names(cols)) {
    check_scalar_character(cols[[nm]], arg = nm)
  }
  check_required_cols(df, unname(cols), df_arg = "data")

  df <- df[, unname(cols), drop = FALSE]
  names(df) <- names(cols)
  time_raw <- df$time

  df$response <- suppressWarnings(as.numeric(df$response))
  if (any(!is.finite(df$response))) {
    abort_bad_arg("response", message = "must contain only finite numeric values.")
  }
  if (anyNA(df$subject) || anyNA(df$method) || anyNA(df$time)) {
    abort_bad_arg("data", message = "must not contain missing values in subject, method, or time.")
  }

  df$subject <- droplevels(factor(df$subject))
  df$method <- droplevels(factor(df$method))
  df$time <- droplevels(factor(df$time))
  time_meta <- .mc_cia_rm_time_meta(time_raw = time_raw, time_factor = df$time)

  if (nlevels(df$method) < 2L) {
    abort_bad_arg("method", message = "must retain at least two method levels.")
  }
  if (nlevels(df$time) < 2L) {
    abort_bad_arg("time", message = "must retain at least two time/condition levels.")
  }
  if (nlevels(df$subject) < 2L) {
    abort_bad_arg("subject", message = "must retain at least two subjects.")
  }

  counts <- stats::xtabs(~ subject + method + time, data = df)
  if (any(counts != 1L)) {
    abort_bad_arg(
      "data",
      message = paste(
        "cia_rm() currently implements the balanced categorical repeated-measures ANOVA estimator;",
        "each subject-method-time cell must contain exactly one observation."
      )
    )
  }

  ord <- order(df$subject, df$method, df$time)
  df <- df[ord, , drop = FALSE]
  rownames(df) <- NULL

  subject_levels <- levels(df$subject)
  method_levels <- levels(df$method)
  time_levels <- levels(df$time)
  subject_code <- as.integer(df$subject)
  method_code <- as.integer(df$method)
  time_code <- as.integer(df$time)
  subject_rows <- split(seq_len(nrow(df)), subject_code)

  list(
    data = df,
    y = df$response,
    subject_code = subject_code,
    method_code = method_code,
    time_code = time_code,
    subject_levels = subject_levels,
    method_levels = method_levels,
    time_levels = time_levels,
    n_subjects = length(subject_levels),
    n_methods = length(method_levels),
    n_times = length(time_levels),
    n_obs = nrow(df),
    subject_rows = subject_rows,
    rows_per_subject = length(method_levels) * length(time_levels),
    time_var = cols[["time"]],
    time_scale = time_meta$scale,
    time_plot_values = time_meta$values
  )
}

.mc_cia_rm_time_meta <- function(time_raw, time_factor) {
  time_levels <- levels(time_factor)
  level_index <- match(time_levels, as.character(time_factor))
  raw_levels <- time_raw[level_index]
  scale <- "categorical"
  values <- as.character(time_levels)

  if (inherits(raw_levels, "Date")) {
    scale <- "continuous"
    values <- as.Date(raw_levels)
  } else if (inherits(raw_levels, c("POSIXct", "POSIXlt"))) {
    scale <- "continuous"
    values <- as.POSIXct(raw_levels)
  } else if (is.numeric(time_raw) || is.integer(time_raw)) {
    scale <- "continuous"
    values <- as.numeric(raw_levels)
  } else {
    num_vals <- suppressWarnings(as.numeric(as.character(time_levels)))
    if (all(is.finite(num_vals))) {
      scale <- "continuous"
      values <- num_vals
    } else {
      date_vals <- tryCatch(
        suppressWarnings(as.Date(as.character(time_levels))),
        error = function(...) rep(as.Date(NA), length(time_levels))
      )
      if (all(!is.na(date_vals))) {
        scale <- "continuous"
        values <- date_vals
      }
    }
  }

  list(scale = scale, values = values)
}

.mc_cia_rm_array <- function(x, method_levels, time_levels) {
  arr <- array(
    as.numeric(x),
    dim = c(length(method_levels), length(method_levels), length(time_levels)),
    dimnames = list(method_levels, method_levels, time_levels)
  )
  arr
}

.mc_cia_rm_matrix <- function(x, method_levels) {
  mat <- as.matrix(x)
  dimnames(mat) <- .mc_square_dimnames(method_levels)
  mat
}

.mc_cia_rm_build_object <- function(prep, raw, ci, conf_level, homogeneous, digits) {
  est <- .mc_cia_rm_array(raw$est, prep$method_levels, prep$time_levels)
  common <- .mc_cia_rm_matrix(raw$common, prep$method_levels)
  common_sigma2_error <- .mc_cia_rm_matrix(raw$common_sigma2_error, prep$method_levels)
  common_sigma2_subject_method <- .mc_cia_rm_matrix(raw$common_sigma2_subject_method, prep$method_levels)
  common_repeatability <- .mc_cia_rm_matrix(raw$common_repeatability, prep$method_levels)
  has_overall <- prep$n_methods >= 3L
  overall <- if (has_overall) stats::setNames(as.numeric(raw$overall), prep$time_levels) else NULL
  overall_common <- if (has_overall) {
    if (isTRUE(homogeneous)) as.numeric(raw$overall_common) else NA_real_
  } else {
    NULL
  }

  if (!isTRUE(homogeneous)) {
    common[row(common) != col(common)] <- NA_real_
    common_sigma2_error[row(common_sigma2_error) != col(common_sigma2_error)] <- NA_real_
    common_sigma2_subject_method[row(common_sigma2_subject_method) != col(common_sigma2_subject_method)] <- NA_real_
    common_repeatability[row(common_repeatability) != col(common_repeatability)] <- NA_real_
  }

  out <- if (isTRUE(ci)) {
    structure(
      list(
        est = est,
        se = array(NA_real_, dim = dim(est), dimnames = dimnames(est)),
        lwr.ci = array(NA_real_, dim = dim(est), dimnames = dimnames(est)),
        upr.ci = array(NA_real_, dim = dim(est), dimnames = dimnames(est)),
        common = common,
        common.se = matrix(NA_real_, nrow = prep$n_methods, ncol = prep$n_methods, dimnames = .mc_square_dimnames(prep$method_levels)),
        common.lwr.ci = matrix(NA_real_, nrow = prep$n_methods, ncol = prep$n_methods, dimnames = .mc_square_dimnames(prep$method_levels)),
        common.upr.ci = matrix(NA_real_, nrow = prep$n_methods, ncol = prep$n_methods, dimnames = .mc_square_dimnames(prep$method_levels)),
        condition = prep$time_levels
      ),
      class = c("cia_rm", "cia_ci", "cia")
    )
  } else {
    structure(
      list(
        est = est,
        common = common,
        condition = prep$time_levels
      ),
      class = c("cia_rm", "cia")
    )
  }

  if (has_overall) {
    out$overall <- overall
    out$overall.common <- overall_common
    if (isTRUE(ci)) {
      out$overall.se <- stats::setNames(rep(NA_real_, prep$n_times), prep$time_levels)
      out$overall.lwr.ci <- stats::setNames(rep(NA_real_, prep$n_times), prep$time_levels)
      out$overall.upr.ci <- stats::setNames(rep(NA_real_, prep$n_times), prep$time_levels)
      out$overall.common.se <- NA_real_
      out$overall.common.lwr.ci <- NA_real_
      out$overall.common.upr.ci <- NA_real_
    }
  }

  attr(out, "method") <- "Repeated-measures coefficient of individual agreement"
  attr(out, "description") <- "Pairwise repeated-measures CIA from categorical-condition ANOVA"
  attr(out, "package") <- "matrixCorr"
  if (isTRUE(ci)) attr(out, "conf.level") <- conf_level
  attr(out, "sigma2_error") <- .mc_cia_rm_matrix(raw$sigma2_error, prep$method_levels)
  attr(out, "sigma2_subject_method") <- .mc_cia_rm_matrix(raw$sigma2_subject_method, prep$method_levels)
  attr(out, "repeatability") <- .mc_cia_rm_matrix(raw$repeatability, prep$method_levels)
  attr(out, "method_time_diff") <- .mc_cia_rm_array(raw$method_time_diff, prep$method_levels, prep$time_levels)
  attr(out, "ms_subject_method") <- .mc_cia_rm_matrix(raw$ms_subject_method, prep$method_levels)
  attr(out, "ms_method_time") <- .mc_cia_rm_matrix(raw$ms_method_time, prep$method_levels)
  attr(out, "ms_error") <- .mc_cia_rm_matrix(raw$ms_error, prep$method_levels)
  attr(out, "ss_subject_method") <- .mc_cia_rm_matrix(raw$ss_subject_method, prep$method_levels)
  attr(out, "ss_method_time") <- .mc_cia_rm_matrix(raw$ss_method_time, prep$method_levels)
  attr(out, "ss_error") <- .mc_cia_rm_matrix(raw$ss_error, prep$method_levels)
  attr(out, "df_subject_method") <- .mc_cia_rm_matrix(raw$df_subject_method, prep$method_levels)
  attr(out, "df_method_time") <- .mc_cia_rm_matrix(raw$df_method_time, prep$method_levels)
  attr(out, "df_error") <- .mc_cia_rm_matrix(raw$df_error, prep$method_levels)
  attr(out, "homogeneity_F") <- .mc_cia_rm_matrix(raw$homogeneity_F, prep$method_levels)
  attr(out, "homogeneity_p") <- .mc_cia_rm_matrix(raw$homogeneity_p, prep$method_levels)
  attr(out, "common_sigma2_error") <- common_sigma2_error
  attr(out, "common_sigma2_subject_method") <- common_sigma2_subject_method
  attr(out, "common_repeatability") <- common_repeatability
  attr(out, "overall_sigma2_error") <- if (has_overall) as.numeric(raw$overall_sigma2_error) else NULL
  attr(out, "overall_sigma2_subject_method") <- if (has_overall) as.numeric(raw$overall_sigma2_subject_method) else NULL
  attr(out, "overall_repeatability") <- if (has_overall) as.numeric(raw$overall_repeatability) else NULL
  attr(out, "overall_homogeneity_F") <- if (has_overall) as.numeric(raw$overall_homogeneity_F) else NULL
  attr(out, "overall_homogeneity_p") <- if (has_overall) as.numeric(raw$overall_homogeneity_p) else NULL
  attr(out, "overall_df_subject_method") <- if (has_overall) as.numeric(raw$overall_df_subject_method) else NULL
  attr(out, "overall_df_method_time") <- if (has_overall) as.numeric(raw$overall_df_method_time) else NULL
  attr(out, "overall_df_error") <- if (has_overall) as.numeric(raw$overall_df_error) else NULL
  attr(out, "overall_common_sigma2_error") <- if (has_overall) {
    if (isTRUE(homogeneous)) as.numeric(raw$overall_common_sigma2_error) else NA_real_
  } else {
    NULL
  }
  attr(out, "overall_common_sigma2_subject_method") <- if (has_overall) {
    if (isTRUE(homogeneous)) as.numeric(raw$overall_common_sigma2_subject_method) else NA_real_
  } else {
    NULL
  }
  attr(out, "overall_common_repeatability") <- if (has_overall) {
    if (isTRUE(homogeneous)) as.numeric(raw$overall_common_repeatability) else NA_real_
  } else {
    NULL
  }
  attr(out, "n_obs") <- prep$n_obs
  attr(out, "n_subjects") <- prep$n_subjects
  attr(out, "n_methods") <- prep$n_methods
  attr(out, "n_times") <- prep$n_times
  attr(out, "time_levels") <- prep$time_levels
  attr(out, "time_var") <- prep$time_var
  attr(out, "time_scale") <- prep$time_scale
  attr(out, "time_plot_values") <- prep$time_plot_values
  attr(out, "method_levels") <- prep$method_levels
  attr(out, "homogeneous") <- homogeneous
  attr(out, "digits") <- digits
  out
}

.mc_cia_rm_note <- function(text, use_message = TRUE) {
  if (isTRUE(use_message)) {
    cli::cli_inform(text)
  } else {
    base::message(text)
  }
}

.mc_cia_rm_cube <- function(prep) {
  aperm(
    array(prep$y, dim = c(prep$n_times, prep$n_methods, prep$n_subjects)),
    c(3L, 2L, 1L)
  )
}

.mc_cia_rm_pair_diff_matrix <- function(cube, method_a, method_b) {
  cube[, method_a, , drop = FALSE][, 1L, ] - cube[, method_b, , drop = FALSE][, 1L, ]
}

.mc_cia_rm_subject_cell_matrix <- function(cube) {
  n <- dim(cube)[1L]
  p <- dim(cube)[2L] * dim(cube)[3L]
  out <- matrix(NA_real_, nrow = n, ncol = p)
  for (i in seq_len(n)) {
    out[i, ] <- as.vector(cube[i, , ])
  }
  out
}

.mc_cia_rm_mean_pair_sqdiff <- function(x) {
  x <- as.numeric(x)
  J <- length(x)
  if (J < 2L) {
    return(NA_real_)
  }
  acc <- 0
  count <- 0L
  for (i in seq_len(J - 1L)) {
    for (j in (i + 1L):J) {
      acc <- acc + (x[i] - x[j])^2
      count <- count + 1L
    }
  }
  acc / count
}

.mc_cia_rm_overall_payload_from_moments <- function(m,
                                                    J,
                                                    K,
                                                    n,
                                                    homogeneous = TRUE,
                                                    constrain_vc = FALSE) {
  P <- J * K
  mu <- m[seq_len(P)]
  M <- matrix(m[P + seq_len(P * P)], nrow = P, ncol = P)
  mu_mat <- matrix(mu, nrow = J, ncol = K)
  Sx <- (n / (n - 1)) * (M - tcrossprod(mu))

  Pj <- diag(J) - matrix(1 / J, nrow = J, ncol = J)
  Pk <- diag(K) - matrix(1 / K, nrow = K, ncol = K)
  T_method <- matrix(0, nrow = J, ncol = P)
  for (k in seq_len(K)) {
    idx <- ((k - 1L) * J + 1L):(k * J)
    T_method[, idx] <- diag(J) / K
  }
  S_method <- T_method %*% Sx %*% t(T_method)
  H <- kronecker(Pk, Pj)

  ss_subject_method <- K * (n - 1) * sum(diag(Pj %*% S_method %*% Pj))
  ss_error <- (n - 1) * sum(diag(H %*% Sx %*% H))

  method_mean <- rowMeans(mu_mat)
  time_mean <- colMeans(mu_mat)
  grand_mean <- mean(mu_mat)
  interaction <- mu_mat - method_mean - rep(time_mean, each = J) + grand_mean
  ss_method_time <- n * sum(interaction^2)

  df_subject_method <- (n - 1) * (J - 1)
  df_method_time <- (J - 1) * (K - 1)
  df_error <- (n - 1) * (J - 1) * (K - 1)

  ms_subject_method <- ss_subject_method / df_subject_method
  ms_method_time <- ss_method_time / df_method_time
  ms_error <- ss_error / df_error

  sigma2_error <- ms_error
  sigma2_subject_method <- (ms_subject_method - ms_error) / K
  if (isTRUE(constrain_vc)) {
    sigma2_error <- max(sigma2_error, 0)
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }

  overall <- vapply(
    seq_len(K),
    function(k) {
      mean_sq_diff <- .mc_cia_rm_mean_pair_sqdiff(mu_mat[, k])
      denom <- mean_sq_diff + 2 * sigma2_subject_method + 2 * sigma2_error
      if (is.finite(denom) && denom > 0) {
        2 * sigma2_error / denom
      } else {
        NA_real_
      }
    },
    numeric(1)
  )

  homogeneity_F <- NA_real_
  homogeneity_p <- NA_real_
  if (is.finite(ms_method_time) && is.finite(ms_error)) {
    if (ms_error > 0) {
      homogeneity_F <- ms_method_time / ms_error
      homogeneity_p <- stats::pf(homogeneity_F, df1 = df_method_time, df2 = df_error, lower.tail = FALSE)
    } else if (ms_method_time > 0) {
      homogeneity_F <- Inf
      homogeneity_p <- 0
    }
  }

  overall_common <- NA_real_
  common_sigma2_error <- NA_real_
  common_sigma2_subject_method <- NA_real_
  common_repeatability <- NA_real_
  if (isTRUE(homogeneous)) {
    ss_error_reduced <- ss_method_time + ss_error
    df_error_reduced <- df_method_time + df_error
    ms_error_reduced <- ss_error_reduced / df_error_reduced
    common_sigma2_error <- ms_error_reduced
    common_sigma2_subject_method <- (ms_subject_method - ms_error_reduced) / K
    if (isTRUE(constrain_vc)) {
      common_sigma2_error <- max(common_sigma2_error, 0)
      common_sigma2_subject_method <- max(common_sigma2_subject_method, 0)
    }
    mean_sq_diff_common <- .mc_cia_rm_mean_pair_sqdiff(method_mean)
    denom_common <- mean_sq_diff_common + 2 * common_sigma2_subject_method + 2 * common_sigma2_error
    if (is.finite(denom_common) && denom_common > 0) {
      overall_common <- 2 * common_sigma2_error / denom_common
    }
    if (is.finite(common_sigma2_error) && common_sigma2_error >= 0) {
      common_repeatability <- 1.96 * sqrt(2 * common_sigma2_error)
    }
  }

  repeatability <- if (is.finite(sigma2_error) && sigma2_error >= 0) {
    1.96 * sqrt(2 * sigma2_error)
  } else {
    NA_real_
  }

  list(
    overall = overall,
    overall_common = overall_common,
    sigma2_error = sigma2_error,
    sigma2_subject_method = sigma2_subject_method,
    repeatability = repeatability,
    ms_subject_method = ms_subject_method,
    ms_method_time = ms_method_time,
    ms_error = ms_error,
    ss_subject_method = ss_subject_method,
    ss_method_time = ss_method_time,
    ss_error = ss_error,
    df_subject_method = df_subject_method,
    df_method_time = df_method_time,
    df_error = df_error,
    homogeneity_F = homogeneity_F,
    homogeneity_p = homogeneity_p,
    common_sigma2_error = common_sigma2_error,
    common_sigma2_subject_method = common_sigma2_subject_method,
    common_repeatability = common_repeatability
  )
}

.mc_cia_rm_overall_moment_stats <- function(X, J, K, constrain_vc = FALSE, homogeneous = TRUE) {
  n <- nrow(X)
  P <- ncol(X)
  mu <- colMeans(X)
  M <- crossprod(X) / n
  g <- cbind(
    X,
    t(vapply(seq_len(n), function(i) as.vector(tcrossprod(X[i, ])), numeric(P * P)))
  )
  Sigma_mean <- if (n >= 2L) stats::cov(g) / n else matrix(NA_real_, ncol(g), ncol(g))
  m <- c(mu, as.vector(M))
  payload <- .mc_cia_rm_overall_payload_from_moments(
    m = m,
    J = J,
    K = K,
    n = n,
    homogeneous = homogeneous,
    constrain_vc = constrain_vc
  )
  c(
    list(
      n = n,
      J = J,
      K = K,
      mu = mu,
      M = M,
      Sigma_mean = Sigma_mean
    ),
    payload
  )
}

.mc_cia_rm_pair_moment_stats <- function(D, constrain_vc = FALSE) {
  n <- nrow(D)
  K <- ncol(D)
  d <- colMeans(D)
  M <- Reduce(`+`, lapply(seq_len(n), function(i) tcrossprod(D[i, ]))) / n
  C <- (n / (n - 1)) * (M - tcrossprod(d))
  off_mean <- mean(C[row(C) != col(C)])
  diag_mean <- mean(diag(C))
  sigma2_subject_method <- off_mean / 2
  sigma2_error <- (diag_mean - off_mean) / 2
  if (isTRUE(constrain_vc)) {
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }
  denom <- d^2 + 2 * sigma2_subject_method + 2 * sigma2_error
  est <- ifelse(is.finite(denom) & denom > 0, 2 * sigma2_error / denom, NA_real_)
  d_bar <- mean(d)
  P <- diag(K) - matrix(1 / K, nrow = K, ncol = K)
  Q <- M - tcrossprod(d)
  ss_time <- n * as.numeric(crossprod(d - d_bar))
  ss_error_reduced <- n * sum(P * Q)
  ms_error_reduced <- (ss_time + ss_error_reduced) / (n * (K - 1))
  mean_a2 <- mean(M)
  ms_subject_method <- K * (n / (n - 1)) * (mean_a2 - d_bar^2)
  common_sigma2_error <- ms_error_reduced / 2
  common_sigma2_subject_method <- (ms_subject_method - ms_error_reduced) / (2 * K)
  if (isTRUE(constrain_vc)) {
    common_sigma2_subject_method <- max(common_sigma2_subject_method, 0)
  }
  common_denom <- d_bar^2 + 2 * common_sigma2_subject_method + 2 * common_sigma2_error
  common_est <- if (is.finite(common_denom) && common_denom > 0) {
    2 * common_sigma2_error / common_denom
  } else {
    NA_real_
  }

  g <- cbind(
    D,
    t(vapply(seq_len(n), function(i) as.vector(tcrossprod(D[i, ])), numeric(K * K)))
  )
  Sigma_mean <- if (n >= 2L) stats::cov(g) / n else matrix(NA_real_, ncol(g), ncol(g))

  list(
    n = n,
    K = K,
    d = d,
    M = M,
    Sigma_mean = Sigma_mean,
    sigma2_error = sigma2_error,
    sigma2_subject_method = sigma2_subject_method,
    est = est,
    common_sigma2_error = common_sigma2_error,
    common_sigma2_subject_method = common_sigma2_subject_method,
    common = common_est
  )
}

.mc_cia_rm_pair_est_from_moments <- function(m, K, n, constrain_vc = FALSE) {
  d <- m[seq_len(K)]
  M <- matrix(m[K + seq_len(K * K)], nrow = K, ncol = K)
  C <- (n / (n - 1)) * (M - tcrossprod(d))
  off_mean <- mean(C[row(C) != col(C)])
  diag_mean <- mean(diag(C))
  sigma2_subject_method <- off_mean / 2
  sigma2_error <- (diag_mean - off_mean) / 2
  if (isTRUE(constrain_vc)) {
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }
  denom <- d^2 + 2 * sigma2_subject_method + 2 * sigma2_error
  ifelse(is.finite(denom) & denom > 0, 2 * sigma2_error / denom, NA_real_)
}

.mc_cia_rm_common_est_from_moments <- function(m, K, n, constrain_vc = FALSE) {
  d <- m[seq_len(K)]
  M <- matrix(m[K + seq_len(K * K)], nrow = K, ncol = K)
  d_bar <- mean(d)
  P <- diag(K) - matrix(1 / K, nrow = K, ncol = K)
  Q <- M - tcrossprod(d)
  ss_time <- n * as.numeric(crossprod(d - d_bar))
  ss_error_reduced <- n * sum(P * Q)
  ms_error_reduced <- (ss_time + ss_error_reduced) / (n * (K - 1))
  mean_a2 <- mean(M)
  ms_subject_method <- K * (n / (n - 1)) * (mean_a2 - d_bar^2)
  sigma2_error <- ms_error_reduced / 2
  sigma2_subject_method <- (ms_subject_method - ms_error_reduced) / (2 * K)
  if (isTRUE(constrain_vc)) {
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }
  denom <- d_bar^2 + 2 * sigma2_subject_method + 2 * sigma2_error
  if (is.finite(denom) && denom > 0) {
    2 * sigma2_error / denom
  } else {
    NA_real_
  }
}

.mc_cia_rm_num_grad <- function(fun, m) {
  p <- length(m)
  g <- numeric(p)
  for (idx in seq_len(p)) {
    step <- 1e-6 * max(1, abs(m[[idx]]))
    plus <- m
    minus <- m
    plus[[idx]] <- plus[[idx]] + step
    minus[[idx]] <- minus[[idx]] - step
    g[[idx]] <- (fun(plus) - fun(minus)) / (2 * step)
  }
  g
}

.mc_cia_rm_apply_bounds <- function(lwr, upr, estimator) {
  if (identical(estimator, "vc_constrained")) {
    lwr <- pmax(0, lwr)
    upr <- pmin(1, upr)
  }
  list(lwr = lwr, upr = upr)
}

.mc_cia_rm_delta <- function(prep, raw, conf_level, homogeneous, estimator, n_threads = 1L) {
  delta_raw <- cia_rm_delta_cpp(
    y = prep$y,
    subject = prep$subject_code,
    method = prep$method_code,
    time = prep$time_code,
    n_subjects = prep$n_subjects,
    n_methods = prep$n_methods,
    n_times = prep$n_times,
    homogeneous = homogeneous,
    constrain_vc = identical(estimator, "vc_constrained"),
    conf_level = conf_level,
    n_threads = n_threads
  )

  out <- list(
    se = .mc_cia_rm_array(delta_raw$se, prep$method_levels, prep$time_levels),
    lwr = .mc_cia_rm_array(delta_raw$lwr, prep$method_levels, prep$time_levels),
    upr = .mc_cia_rm_array(delta_raw$upr, prep$method_levels, prep$time_levels),
    common_se = .mc_cia_rm_matrix(delta_raw$common_se, prep$method_levels),
    common_lwr = .mc_cia_rm_matrix(delta_raw$common_lwr, prep$method_levels),
    common_upr = .mc_cia_rm_matrix(delta_raw$common_upr, prep$method_levels),
    overall_se = NULL,
    overall_lwr = NULL,
    overall_upr = NULL,
    overall_common_se = NULL,
    overall_common_lwr = NULL,
    overall_common_upr = NULL
  )

  if (prep$n_methods >= 3L) {
    out$overall_se <- stats::setNames(as.numeric(delta_raw$overall_se), prep$time_levels)
    out$overall_lwr <- stats::setNames(as.numeric(delta_raw$overall_lwr), prep$time_levels)
    out$overall_upr <- stats::setNames(as.numeric(delta_raw$overall_upr), prep$time_levels)
    out$overall_common_se <- as.numeric(delta_raw$overall_common_se)
    out$overall_common_lwr <- as.numeric(delta_raw$overall_common_lwr)
    out$overall_common_upr <- as.numeric(delta_raw$overall_common_upr)
  }

  out
}

.mc_cia_rm_bootstrap <- function(prep, homogeneous, B, conf_level, seed = NULL, n_threads = 1L, estimator = "vc_constrained") {
  .mc_eval_with_seed(seed, {
    alpha <- (1 - conf_level) / 2
    pair_idx <- which(upper.tri(matrix(FALSE, prep$n_methods, prep$n_methods)), arr.ind = TRUE)
    n_pairs <- nrow(pair_idx)
    pair_time_linear <- as.vector(vapply(seq_len(prep$n_times), function(k) {
      pair_idx[, 1L] + (pair_idx[, 2L] - 1L) * prep$n_methods +
        (k - 1L) * prep$n_methods * prep$n_methods
    }, numeric(n_pairs)))
    common_linear <- pair_idx[, 1L] + (pair_idx[, 2L] - 1L) * prep$n_methods
    pair_est_boot <- matrix(NA_real_, nrow = B, ncol = n_pairs * prep$n_times)
    pair_common_boot <- matrix(NA_real_, nrow = B, ncol = n_pairs)
    overall_arr <- if (prep$n_methods >= 3L) {
      matrix(NA_real_, nrow = prep$n_times, ncol = B)
    } else {
      NULL
    }
    overall_common_arr <- if (prep$n_methods >= 3L) rep(NA_real_, B) else NULL
    subject_code <- rep(seq_len(prep$n_subjects), each = prep$rows_per_subject)

    for (b in seq_len(B)) {
      sampled <- sample.int(prep$n_subjects, size = prep$n_subjects, replace = TRUE)
      boot_rows <- integer(prep$n_obs)
      pos <- 1L
      for (i in seq_along(sampled)) {
        idx <- prep$subject_rows[[sampled[[i]]]]
        rng <- pos:(pos + length(idx) - 1L)
        boot_rows[rng] <- idx
        pos <- pos + length(idx)
      }

      y <- prep$y[boot_rows]
      method_code <- prep$method_code[boot_rows]
      time_code <- prep$time_code[boot_rows]

      boot_raw <- tryCatch(
        cia_rm_anova_cpp(
          y = y,
          subject = subject_code,
          method = method_code,
          time = time_code,
          n_subjects = prep$n_subjects,
          n_methods = prep$n_methods,
          n_times = prep$n_times,
          homogeneous = homogeneous,
          constrain_vc = identical(estimator, "vc_constrained"),
          n_threads = n_threads
        ),
        error = function(...) NULL
      )
      if (is.null(boot_raw)) {
        next
      }
      pair_est_boot[b, ] <- as.numeric(boot_raw$est)[pair_time_linear]
      pair_common_boot[b, ] <- as.numeric(boot_raw$common)[common_linear]
      if (prep$n_methods >= 3L) {
        overall_arr[, b] <- as.numeric(boot_raw$overall)
        overall_common_arr[[b]] <- as.numeric(boot_raw$overall_common)
      }
    }

    lwr <- array(NA_real_, dim = c(prep$n_methods, prep$n_methods, prep$n_times), dimnames = list(prep$method_levels, prep$method_levels, prep$time_levels))
    upr <- lwr
    common_lwr <- matrix(NA_real_, prep$n_methods, prep$n_methods, dimnames = .mc_square_dimnames(prep$method_levels))
    common_upr <- common_lwr
    overall_lwr <- if (prep$n_methods >= 3L) stats::setNames(rep(NA_real_, prep$n_times), prep$time_levels) else NULL
    overall_upr <- if (prep$n_methods >= 3L) stats::setNames(rep(NA_real_, prep$n_times), prep$time_levels) else NULL
    overall_common_lwr <- if (prep$n_methods >= 3L) NA_real_ else NULL
    overall_common_upr <- if (prep$n_methods >= 3L) NA_real_ else NULL
    for (k in seq_len(prep$n_times)) {
      diag(lwr[, , k]) <- 1
      diag(upr[, , k]) <- 1
    }
    diag(common_lwr) <- 1
    diag(common_upr) <- 1

    for (p in seq_len(n_pairs)) {
      i <- pair_idx[p, 1L]
      j <- pair_idx[p, 2L]
      for (k in seq_len(prep$n_times)) {
        col <- p + (k - 1L) * n_pairs
        vals <- pair_est_boot[, col]
        ok <- is.finite(vals)
        if (sum(ok) >= 2L) {
          qs <- stats::quantile(vals[ok], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
          bounded <- .mc_cia_rm_apply_bounds(as.numeric(qs[[1L]]), as.numeric(qs[[2L]]), estimator)
          lwr[i, j, k] <- bounded$lwr
          lwr[j, i, k] <- bounded$lwr
          upr[i, j, k] <- bounded$upr
          upr[j, i, k] <- bounded$upr
        }
      }
      vals_common <- pair_common_boot[, p]
      ok_common <- is.finite(vals_common)
      if (sum(ok_common) >= 2L) {
        qs <- stats::quantile(vals_common[ok_common], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
        bounded_common <- .mc_cia_rm_apply_bounds(as.numeric(qs[[1L]]), as.numeric(qs[[2L]]), estimator)
        common_lwr[i, j] <- bounded_common$lwr
        common_lwr[j, i] <- bounded_common$lwr
        common_upr[i, j] <- bounded_common$upr
        common_upr[j, i] <- bounded_common$upr
      }
    }

    if (prep$n_methods >= 3L) {
      for (k in seq_len(prep$n_times)) {
        vals <- overall_arr[k, ]
        ok <- is.finite(vals)
        if (sum(ok) >= 2L) {
          qs <- stats::quantile(vals[ok], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
          bounded <- .mc_cia_rm_apply_bounds(as.numeric(qs[[1L]]), as.numeric(qs[[2L]]), estimator)
          overall_lwr[[k]] <- bounded$lwr
          overall_upr[[k]] <- bounded$upr
        }
      }
      ok_common <- is.finite(overall_common_arr)
      if (sum(ok_common) >= 2L) {
        qs <- stats::quantile(overall_common_arr[ok_common], probs = c(alpha, 1 - alpha), names = FALSE, type = 6, na.rm = TRUE)
        bounded_common <- .mc_cia_rm_apply_bounds(as.numeric(qs[[1L]]), as.numeric(qs[[2L]]), estimator)
        overall_common_lwr <- bounded_common$lwr
        overall_common_upr <- bounded_common$upr
      }
    }

    list(
      lwr = lwr,
      upr = upr,
      common_lwr = common_lwr,
      common_upr = common_upr,
      overall_lwr = overall_lwr,
      overall_upr = overall_upr,
      overall_common_lwr = overall_common_lwr,
      overall_common_upr = overall_common_upr,
      n_successful = sum(rowSums(is.finite(pair_est_boot)) > 0L),
      B = B
    )
  })
}

.mc_cia_rm_flatten <- function(object) {
  est <- object$est
  se <- object$se
  method_levels <- attr(object, "method_levels", exact = TRUE)
  time_levels <- attr(object, "time_levels", exact = TRUE)
  sigma2_error <- attr(object, "sigma2_error", exact = TRUE)
  sigma2_subject_method <- attr(object, "sigma2_subject_method", exact = TRUE)
  repeatability <- attr(object, "repeatability", exact = TRUE)
  homogeneity_F <- attr(object, "homogeneity_F", exact = TRUE)
  homogeneity_p <- attr(object, "homogeneity_p", exact = TRUE)
  common <- object$common
  common_se <- object$common.se
  common_sigma2_error <- attr(object, "common_sigma2_error", exact = TRUE)
  common_sigma2_subject_method <- attr(object, "common_sigma2_subject_method", exact = TRUE)
  common_repeatability <- attr(object, "common_repeatability", exact = TRUE)
  has_ci <- inherits(object, "cia_ci")

  rows <- vector("list", 0L)
  for (i in seq_len(length(method_levels) - 1L)) {
    for (j in (i + 1L):length(method_levels)) {
      for (k in seq_along(time_levels)) {
        row <- data.frame(
          method_1 = method_levels[[i]],
          method_2 = method_levels[[j]],
          time = time_levels[[k]],
          cia = as.numeric(est[i, j, k]),
          sigma2_error = as.numeric(sigma2_error[i, j]),
          sigma2_subject_method = as.numeric(sigma2_subject_method[i, j]),
          repeatability = as.numeric(repeatability[i, j]),
          homogeneity_F = as.numeric(homogeneity_F[i, j]),
          homogeneity_p = as.numeric(homogeneity_p[i, j]),
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        if (!is.null(se)) {
          row$se <- as.numeric(se[i, j, k])
        }
        if (has_ci) {
          row$lwr.ci <- as.numeric(object$lwr.ci[i, j, k])
          row$upr.ci <- as.numeric(object$upr.ci[i, j, k])
        }
        rows[[length(rows) + 1L]] <- row
      }

      if (is.finite(common[i, j])) {
        row_common <- data.frame(
          method_1 = method_levels[[i]],
          method_2 = method_levels[[j]],
          time = "common",
          cia = as.numeric(common[i, j]),
          sigma2_error = as.numeric(common_sigma2_error[i, j]),
          sigma2_subject_method = as.numeric(common_sigma2_subject_method[i, j]),
          repeatability = as.numeric(common_repeatability[i, j]),
          homogeneity_F = as.numeric(homogeneity_F[i, j]),
          homogeneity_p = as.numeric(homogeneity_p[i, j]),
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        if (!is.null(common_se)) {
          row_common$se <- as.numeric(common_se[i, j])
        }
        if (has_ci) {
          row_common$lwr.ci <- as.numeric(object$common.lwr.ci[i, j])
          row_common$upr.ci <- as.numeric(object$common.upr.ci[i, j])
        }
        rows[[length(rows) + 1L]] <- row_common
      }
    }
  }

  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  out
}

.mc_cia_rm_overall_df <- function(object) {
  overall <- object$overall
  if (is.null(overall)) {
    return(NULL)
  }
  has_ci <- inherits(object, "cia_ci")
  out <- data.frame(
    time = names(overall) %||% attr(object, "time_levels", exact = TRUE),
    cia = as.numeric(overall),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (!is.null(object$overall.se)) {
    out$se <- as.numeric(object$overall.se)
  }
  if (has_ci && !is.null(object$overall.lwr.ci) && !is.null(object$overall.upr.ci)) {
    out$lwr.ci <- as.numeric(object$overall.lwr.ci)
    out$upr.ci <- as.numeric(object$overall.upr.ci)
  }
  out
}

.mc_cia_rm_overall_common_df <- function(object) {
  overall_common <- object$overall.common
  if (is.null(overall_common) || !is.finite(overall_common)) {
    return(NULL)
  }
  has_ci <- inherits(object, "cia_ci")
  out <- data.frame(
    time = "common",
    cia = as.numeric(overall_common),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (!is.null(object$overall.common.se)) {
    out$se <- as.numeric(object$overall.common.se)
  }
  if (has_ci && !is.null(object$overall.common.lwr.ci) && !is.null(object$overall.common.upr.ci)) {
    out$lwr.ci <- as.numeric(object$overall.common.lwr.ci)
    out$upr.ci <- as.numeric(object$overall.common.upr.ci)
  }
  out
}

#' @rdname cia_rm
#' @method summary cia_rm
#' @param object A `cia_rm` object.
#' @param ci_digits Integer; number of digits for confidence interval bounds.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns.
#' @param width Optional display width.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Unused.
#' @export
summary.cia_rm <- function(object,
                           digits = 4,
                           ci_digits = 3,
                           n = NULL,
                           topn = NULL,
                           max_vars = NULL,
                           width = NULL,
                           show_ci = NULL,
                           ...) {
  out <- .mc_cia_rm_flatten(object)
  num_cols <- c("cia", "sigma2_error", "sigma2_subject_method", "repeatability", "homogeneity_F", "homogeneity_p")
  if ("se" %in% names(out)) {
    num_cols <- c("se", num_cols)
  }
  for (nm in intersect(num_cols, names(out))) {
    out[[nm]] <- round(out[[nm]], digits)
  }
  for (nm in intersect(c("lwr.ci", "upr.ci"), names(out))) {
    out[[nm]] <- round(out[[nm]], ci_digits)
  }
  out <- .mc_finalize_summary_df(out, class_name = "summary.cia_rm", repeated = TRUE)
  attr(out, "conf.level") <- attr(object, "conf.level", exact = TRUE)
  attr(out, "has_ci") <- inherits(object, "cia_ci")
  attr(out, "digits") <- digits
  attr(out, "ci_digits") <- ci_digits
  attr(out, "method") <- attr(object, "method", exact = TRUE)
  attr(out, "description") <- attr(object, "description", exact = TRUE)
  attr(out, "n_subjects_total") <- attr(object, "n_subjects", exact = TRUE)
  attr(out, "n_methods_total") <- attr(object, "n_methods", exact = TRUE)
  attr(out, "n_times_total") <- attr(object, "n_times", exact = TRUE)
  attr(out, "homogeneous") <- attr(object, "homogeneous", exact = TRUE)
  overall_out <- .mc_cia_rm_overall_df(object)
  if (!is.null(overall_out)) {
    for (nm in intersect(c("cia", "se"), names(overall_out))) {
      overall_out[[nm]] <- round(overall_out[[nm]], digits)
    }
    for (nm in intersect(c("lwr.ci", "upr.ci"), names(overall_out))) {
      overall_out[[nm]] <- round(overall_out[[nm]], ci_digits)
    }
  }
  overall_common_out <- .mc_cia_rm_overall_common_df(object)
  if (!is.null(overall_common_out)) {
    for (nm in intersect(c("cia", "se"), names(overall_common_out))) {
      overall_common_out[[nm]] <- round(overall_common_out[[nm]], digits)
    }
    for (nm in intersect(c("lwr.ci", "upr.ci"), names(overall_common_out))) {
      overall_common_out[[nm]] <- round(overall_common_out[[nm]], ci_digits)
    }
  }
  attr(out, "overall_section") <- overall_out
  attr(out, "overall_common_section") <- overall_common_out
  out
}

#' @rdname cia_rm
#' @method print cia_rm
#' @param x A `cia_rm` object.
#' @export
print.cia_rm <- function(x,
                         digits = 4,
                         n = NULL,
                         topn = NULL,
                         max_vars = NULL,
                         width = NULL,
                         show_ci = NULL,
                         ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )

  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "print",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  has_ci <- inherits(x, "cia_ci")
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE)))
  if (length(cl) != 1L || !is.finite(cl)) cl <- NA_real_

  digest <- c(
    n_subjects = .mc_count_fmt(attr(x, "n_subjects", exact = TRUE)),
    n_methods = .mc_count_fmt(attr(x, "n_methods", exact = TRUE)),
    n_times = .mc_count_fmt(attr(x, "n_times", exact = TRUE))
  )
  if (isTRUE(attr(x, "homogeneous", exact = TRUE))) {
    digest <- c(digest, common = "yes")
  }
  if (has_ci && identical(cfg$show_ci, "yes")) {
    digest <- c(digest, ci = sprintf("%g%%", 100 * cl))
  }
  .mc_print_named_digest(
    digest,
    header = "Repeated-measures coefficient of individual agreement"
  )
  cat("\n")

  preview <- .mc_cia_rm_flatten(x)
  keep <- c("method_1", "method_2", "time", "cia", "lwr.ci", "upr.ci")
  preview <- preview[, intersect(keep, names(preview)), drop = FALSE]
  if ("cia" %in% names(preview)) {
    preview$cia <- round(preview$cia, digits)
  }
  for (nm in intersect(c("lwr.ci", "upr.ci"), names(preview))) {
    preview[[nm]] <- round(preview[[nm]], digits)
  }
  if (!has_ci || identical(cfg$show_ci, "no")) {
    preview <- preview[, setdiff(names(preview), c("lwr.ci", "upr.ci")), drop = FALSE]
  }

  condition_rows <- preview$time != "common"
  common_rows <- preview$time == "common"
  overall_preview <- .mc_cia_rm_overall_df(x)
  overall_common_preview <- .mc_cia_rm_overall_common_df(x)
  if (!is.null(overall_preview)) {
    overall_preview$cia <- round(overall_preview$cia, digits)
    for (nm in intersect(c("lwr.ci", "upr.ci"), names(overall_preview))) {
      overall_preview[[nm]] <- round(overall_preview[[nm]], digits)
    }
    if (!has_ci || identical(cfg$show_ci, "no")) {
      overall_preview <- overall_preview[, setdiff(names(overall_preview), c("lwr.ci", "upr.ci")), drop = FALSE]
    }
  }
  if (!is.null(overall_common_preview)) {
    overall_common_preview$cia <- round(overall_common_preview$cia, digits)
    for (nm in intersect(c("lwr.ci", "upr.ci"), names(overall_common_preview))) {
      overall_common_preview[[nm]] <- round(overall_common_preview[[nm]], digits)
    }
    if (!has_ci || identical(cfg$show_ci, "no")) {
      overall_common_preview <- overall_common_preview[, setdiff(names(overall_common_preview), c("lwr.ci", "upr.ci")), drop = FALSE]
    }
  }

  if (any(condition_rows)) {
    cat("Condition-specific CIA estimates\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = preview[condition_rows, , drop = FALSE],
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "print",
          full_hint = TRUE,
          summary_hint = TRUE
        ),
        legacy$dots
      )
    )
  }
  if (any(common_rows)) {
    cat("\nCommon CIA estimates\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = preview[common_rows, , drop = FALSE],
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "print",
          full_hint = TRUE,
          summary_hint = TRUE
        ),
        legacy$dots
      )
    )
  }
  if (!is.null(overall_preview) && nrow(overall_preview)) {
    cat("\nOverall multi-method CIA\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = overall_preview,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "print",
          full_hint = TRUE,
          summary_hint = TRUE
        ),
        legacy$dots
      )
    )
  }
  if (!is.null(overall_common_preview) && nrow(overall_common_preview)) {
    cat("\nCommon overall CIA\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = overall_common_preview,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "print",
          full_hint = TRUE,
          summary_hint = TRUE
        ),
        legacy$dots
      )
    )
  }
  invisible(x)
}

#' @rdname cia_rm
#' @method print summary.cia_rm
#' @param x A `summary.cia_rm` object.
#' @export
print.summary.cia_rm <- function(x,
                                 digits = NULL,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 show_ci = NULL,
                                 ...) {
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  has_ci <- isTRUE(attr(x, "has_ci"))
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level", exact = TRUE)))
  if (length(cl) != 1L || !is.finite(cl)) cl <- NA_real_

  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  cat(
    "\n",
    .mc_header_with_ci(
      "Repeated-measures coefficient of individual agreement",
      cl,
      if (has_ci) cfg$show_ci else "no"
    ),
    "\n\n",
    sep = ""
  )

  estimates <- as.data.frame(x, stringsAsFactors = FALSE, check.names = FALSE)
  keep_est <- c("method_1", "method_2", "time", "cia", "se", "lwr.ci", "upr.ci")
  estimates <- estimates[, intersect(keep_est, names(estimates)), drop = FALSE]
  if (!has_ci || identical(cfg$show_ci, "no")) {
    estimates <- estimates[, setdiff(names(estimates), c("lwr.ci", "upr.ci")), drop = FALSE]
  }

  diagnostics <- unique(
    as.data.frame(
      x[, c(
        "method_1", "method_2",
        "sigma2_error", "sigma2_subject_method",
        "repeatability", "homogeneity_F", "homogeneity_p"
      ), drop = FALSE],
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
  )
  overall_estimates <- attr(x, "overall_section", exact = TRUE)
  overall_common_estimates <- attr(x, "overall_common_section", exact = TRUE)

  cat("CIA estimates\n\n")
  do.call(
    .mc_print_preview_table,
    c(
      list(
        df = estimates,
        n = cfg$n,
        topn = cfg$topn,
        max_vars = cfg$max_vars,
        width = cfg$width,
        context = "summary",
        full_hint = FALSE,
        summary_hint = FALSE
      ),
      legacy$dots
    )
  )

  if (!is.null(overall_estimates) && nrow(overall_estimates)) {
    cat("\nOverall multi-method CIA\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = overall_estimates,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "summary",
          full_hint = FALSE,
          summary_hint = FALSE
        ),
        legacy$dots
      )
    )
  }

  if (!is.null(overall_common_estimates) && nrow(overall_common_estimates)) {
    cat("\nCommon overall CIA\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = overall_common_estimates,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "summary",
          full_hint = FALSE,
          summary_hint = FALSE
        ),
        legacy$dots
      )
    )
  }

  if (nrow(diagnostics)) {
    cat("\nPair diagnostics\n\n")
    do.call(
      .mc_print_preview_table,
      c(
        list(
          df = diagnostics,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "summary",
          full_hint = FALSE,
          summary_hint = FALSE
        ),
        legacy$dots
      )
    )
  }
  invisible(x)
}

#' @rdname cia_rm
#' @method plot cia_rm
#' @param title Optional plot title.
#' @param facet_by_pair Logical; if `TRUE`, draw one panel per method pair.
#' @param facet_scales Passed to `ggplot2::facet_wrap(scales = ...)` when
#'   `facet_by_pair = TRUE`. One of `"fixed"` or `"free_y"`.
#' @param show_ci Logical; if `TRUE` and confidence intervals are present, show
#'   them on the plot. By default this is inferred from the object class.
#' @param show_common Logical; if `TRUE` and common estimates are available,
#'   draw dashed horizontal reference lines for the pooled/common CIA.
#' @param point_size Numeric point size passed to `ggplot2::geom_point()`.
#' @param line_size Numeric line width passed to `ggplot2::geom_line()`.
#' @param ci_alpha Numeric alpha used for confidence ribbons in continuous
#'   plots.
#' @param ci_linewidth Numeric line width used for confidence-interval
#'   error bars in categorical plots.
#' @param ... Additional arguments passed to `ggplot2::theme()`.
#' @export
plot.cia_rm <- function(x,
                        title = NULL,
                        facet_by_pair = FALSE,
                        facet_scales = c("fixed", "free_y"),
                        show_ci = NULL,
                        show_common = TRUE,
                        point_size = 2.2,
                        line_size = 0.7,
                        ci_alpha = 0.16,
                        ci_linewidth = 0.45,
                        ...) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg ggplot2} is required for plotting.")
  }
  if (!inherits(x, "cia_rm")) {
    cli::cli_abort("{.arg x} must inherit from {.cls cia_rm}.")
  }
  check_bool(facet_by_pair, arg = "facet_by_pair")
  facet_scales <- match.arg(facet_scales)
  check_bool(show_common, arg = "show_common")
  if (!is.null(show_ci)) {
    check_bool(show_ci, arg = "show_ci")
  }

  if (is.null(show_ci)) {
    show_ci <- inherits(x, "cia_ci")
  }

  df <- .mc_cia_rm_flatten(x)
  df <- df[df$time != "common", , drop = FALSE]
  if (!nrow(df)) {
    cli::cli_abort("No condition-specific CIA rows are available for plotting.")
  }

  df$pair <- paste(df$method_1, df$method_2, sep = " vs ")
  time_levels <- attr(x, "time_levels", exact = TRUE)
  time_scale <- attr(x, "time_scale", exact = TRUE)
  time_values <- attr(x, "time_plot_values", exact = TRUE)
  time_var <- attr(x, "time_var", exact = TRUE)
  if (!length(time_levels) || !length(time_values) || length(time_levels) != length(time_values)) {
    time_levels <- unique(df$time)
    time_values <- time_levels
    time_scale <- "categorical"
  }

  if (identical(time_scale, "continuous")) {
    df$time_value <- time_values[match(df$time, time_levels)]
    ord <- order(df$pair, df$time_value)
    df <- df[ord, , drop = FALSE]
  } else {
    df$time_value <- factor(df$time, levels = time_levels)
  }

  common_df <- df[FALSE, c("pair", "cia"), drop = FALSE]
  if (isTRUE(show_common) && is.matrix(x$common)) {
    common_rows <- .mc_cia_rm_flatten(x)
    common_rows <- common_rows[common_rows$time == "common", , drop = FALSE]
    if (nrow(common_rows)) {
      common_rows$pair <- paste(common_rows$method_1, common_rows$method_2, sep = " vs ")
      common_df <- common_rows[, c("pair", "cia"), drop = FALSE]
    }
  }

  has_ci_cols <- all(c("lwr.ci", "upr.ci") %in% names(df))
  show_ci <- isTRUE(show_ci) && has_ci_cols
  if (is.null(title)) {
    title <- if (identical(time_scale, "continuous")) {
      "Repeated-measures CIA across condition levels"
    } else {
      "Repeated-measures CIA by condition"
    }
  }

  subtitle <- if (identical(time_scale, "continuous")) {
    "Condition scale detected as continuous/discrete"
  } else {
    "Condition scale detected as categorical"
  }

  if (identical(time_scale, "continuous")) {
    p <- ggplot2::ggplot(
      df,
      ggplot2::aes(
        x = .data$time_value,
        y = .data$cia,
        colour = .data$pair,
        group = .data$pair
      )
    )
    if (show_ci) {
      p <- p + ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data$lwr.ci, ymax = .data$upr.ci, fill = .data$pair),
        alpha = ci_alpha,
        colour = NA,
        inherit.aes = TRUE
      )
    }
    if (nrow(common_df)) {
      p <- p + ggplot2::geom_hline(
        data = common_df,
        ggplot2::aes(yintercept = .data$cia, colour = .data$pair),
        linetype = "dashed",
        linewidth = 0.45,
        inherit.aes = FALSE
      )
    }
    p <- p +
      ggplot2::geom_line(linewidth = line_size) +
      ggplot2::geom_point(size = point_size) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        panel.grid.minor = ggplot2::element_blank(),
        ...
      ) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle,
        x = time_var,
        y = "CIA",
        colour = "Method pair",
        fill = "Method pair"
      )

    if (isTRUE(facet_by_pair)) {
      p <- p +
        ggplot2::facet_wrap(ggplot2::vars(.data$pair), scales = facet_scales) +
        ggplot2::guides(colour = "none", fill = "none")
    }
    return(p)
  }

  dodge <- if (isTRUE(facet_by_pair)) {
    ggplot2::position_identity()
  } else {
    ggplot2::position_dodge(width = 0.45)
  }

  p <- ggplot2::ggplot(
    df,
    ggplot2::aes(
      x = .data$time_value,
      y = .data$cia,
      colour = .data$pair
    )
  )
  if (show_ci) {
    p <- p + ggplot2::geom_errorbar(
      ggplot2::aes(ymin = .data$lwr.ci, ymax = .data$upr.ci),
      width = 0.15,
      linewidth = ci_linewidth,
      position = dodge
    )
  }
  if (nrow(common_df)) {
    p <- p + ggplot2::geom_hline(
      data = common_df,
      ggplot2::aes(yintercept = .data$cia, colour = .data$pair),
      linetype = "dashed",
      linewidth = 0.45,
      inherit.aes = FALSE
    )
  }
  p <- p +
    ggplot2::geom_point(size = point_size, position = dodge) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid.minor = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(angle = 25, hjust = 1),
      ...
    ) +
    ggplot2::labs(
      title = title,
      subtitle = subtitle,
      x = time_var,
      y = "CIA",
      colour = "Method pair"
    )

  if (isTRUE(facet_by_pair)) {
    p <- p +
      ggplot2::facet_wrap(ggplot2::vars(.data$pair), scales = facet_scales) +
      ggplot2::guides(colour = "none")
  }
  p
}

make_cia_rm_perf_data <- function(n_subjects = 10L, n_methods = 3L, n_times = 4L) {
  subjects <- sprintf("s%02d", seq_len(n_subjects))
  methods <- LETTERS[seq_len(n_methods)]
  times <- sprintf("t%02d", seq_len(n_times))
  dat <- expand.grid(
    subject = subjects,
    method = methods,
    time = times,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  s <- match(dat$subject, subjects)
  m <- match(dat$method, methods)
  tt <- match(dat$time, times)
  dat$value <- 5 + 0.3 * s + 0.2 * m + 0.15 * tt + sin(s + m * tt) / 20
  dat
}

test_that("cia_rm no-CI output remains structurally stable", {
  dat2 <- make_cia_rm_perf_data(n_methods = 2L)
  dat3 <- make_cia_rm_perf_data(n_methods = 3L)

  for (estimator in c("vc_constrained", "mom_unconstrained")) {
    for (homogeneous in c(FALSE, TRUE)) {
      fit2 <- cia_rm(dat2, "value", "subject", "method", "time",
                     estimator = estimator, homogeneous = homogeneous,
                     use_message = FALSE)
      fit3 <- cia_rm(dat3, "value", "subject", "method", "time",
                     estimator = estimator, homogeneous = homogeneous,
                     use_message = FALSE)

      expect_s3_class(fit2, "cia_rm")
      expect_identical(class(fit2), c("cia_rm", "cia"))
      expect_named(fit2, c("est", "common", "condition"))
      expect_identical(attr(fit2, "estimator", exact = TRUE), estimator)
      expect_identical(attr(fit2, "homogeneous", exact = TRUE), homogeneous)

      expect_s3_class(fit3, "cia_rm")
      expect_true(all(c("overall", "overall.common") %in% names(fit3)))
      expect_identical(dimnames(fit3$est)[[1L]], attr(fit3, "method_levels", exact = TRUE))
      expect_identical(names(fit3$overall), attr(fit3, "time_levels", exact = TRUE))
    }
  }
})

test_that("cia_rm delta CI remains structurally stable", {
  dat2 <- make_cia_rm_perf_data(n_methods = 2L)
  dat3 <- make_cia_rm_perf_data(n_methods = 3L)

  for (estimator in c("vc_constrained", "mom_unconstrained")) {
    for (homogeneous in c(FALSE, TRUE)) {
      fit2 <- cia_rm(dat2, "value", "subject", "method", "time",
                     ci = TRUE, inference = "delta", estimator = estimator,
                     homogeneous = homogeneous, use_message = FALSE)
      fit3 <- cia_rm(dat3, "value", "subject", "method", "time",
                     ci = TRUE, inference = "delta", estimator = estimator,
                     homogeneous = homogeneous, use_message = FALSE)

      expect_identical(class(fit2), c("cia_rm", "cia_ci", "cia"))
      expect_identical(attr(fit2, "ci.method", exact = TRUE), "delta_normal")
      expect_true(all(c("se", "lwr.ci", "upr.ci", "common.se") %in% names(fit2)))
      expect_true(all(c("overall.se", "overall.lwr.ci", "overall.upr.ci") %in% names(fit3)))
      expect_identical(dim(fit2$se), dim(fit2$est))
      expect_identical(dimnames(fit2$lwr.ci), dimnames(fit2$est))
    }
  }
})

test_that("cia_rm bootstrap CI is reproducible with fixed seed", {
  dat <- make_cia_rm_perf_data(n_methods = 3L)

  for (estimator in c("vc_constrained", "mom_unconstrained")) {
    for (homogeneous in c(FALSE, TRUE)) {
      fit1 <- cia_rm(dat, "value", "subject", "method", "time",
                     ci = TRUE, inference = "bootstrap", estimator = estimator,
                     homogeneous = homogeneous, B = 49L, seed = 77L,
                     use_message = FALSE)
      fit2 <- cia_rm(dat, "value", "subject", "method", "time",
                     ci = TRUE, inference = "bootstrap", estimator = estimator,
                     homogeneous = homogeneous, B = 49L, seed = 77L,
                     use_message = FALSE)

      expect_equal(fit1, fit2, tolerance = 1e-12)
      expect_identical(attr(fit1, "ci.method", exact = TRUE), "subject_bootstrap_percentile")
      expect_identical(attr(fit1, "bootstrap", exact = TRUE), list(n_successful = 49L, B = 49L))
      expect_true(all(c("overall.lwr.ci", "overall.upr.ci", "overall.common.lwr.ci") %in% names(fit1)))
    }
  }
})

test_that("cia_rm time metadata remains stable for categorical numeric and Date time", {
  categorical <- make_cia_rm_perf_data()
  numeric <- categorical
  numeric$time <- match(numeric$time, sort(unique(numeric$time))) * 5
  date <- categorical
  date$time <- as.Date("2024-01-01") + match(date$time, sort(unique(date$time))) - 1L

  fit_cat <- cia_rm(categorical, "value", "subject", "method", "time", use_message = FALSE)
  fit_num <- cia_rm(numeric, "value", "subject", "method", "time", use_message = FALSE)
  fit_date <- cia_rm(date, "value", "subject", "method", "time", use_message = FALSE)

  expect_identical(attr(fit_cat, "time_scale", exact = TRUE), "categorical")
  expect_identical(attr(fit_num, "time_scale", exact = TRUE), "continuous")
  expect_type(attr(fit_num, "time_plot_values", exact = TRUE), "double")
  expect_identical(attr(fit_date, "time_scale", exact = TRUE), "continuous")
  expect_s3_class(attr(fit_date, "time_plot_values", exact = TRUE), "Date")
})

test_that("cia_rm error behaviour remains stable", {
  dat <- make_cia_rm_perf_data()
  missing_response <- dat
  missing_response$value[[1L]] <- NA_real_
  missing_key <- dat
  missing_key$subject[[1L]] <- NA_character_
  one_method <- subset(dat, method == "A")
  one_time <- subset(dat, time == "t01")
  one_subject <- subset(dat, subject == "s01")
  unbalanced <- dat[-1L, , drop = FALSE]
  duplicated <- rbind(dat, dat[1L, , drop = FALSE])

  expect_error(cia_rm(missing_response, "value", "subject", "method", "time", use_message = FALSE),
               "finite numeric")
  expect_error(cia_rm(missing_key, "value", "subject", "method", "time", use_message = FALSE),
               "must not contain missing")
  expect_error(cia_rm(one_method, "value", "subject", "method", "time", use_message = FALSE),
               "at least two method")
  expect_error(cia_rm(one_time, "value", "subject", "method", "time", use_message = FALSE),
               "at least two time")
  expect_error(cia_rm(one_subject, "value", "subject", "method", "time", use_message = FALSE),
               "at least two subjects")
  expect_error(cia_rm(unbalanced, "value", "subject", "method", "time", use_message = FALSE),
               "each subject-method-time cell must contain exactly one observation")
  expect_error(cia_rm(duplicated, "value", "subject", "method", "time", use_message = FALSE),
               "each subject-method-time cell must contain exactly one observation")
  expect_error(cia_rm(dat, "value", "subject", "method", "time", ci = TRUE,
                      inference = "none", use_message = FALSE),
               "must not be")
  expect_error(cia_rm(dat, "value", "subject", "method", "time", ci = TRUE,
                      inference = "bootstrap", B = 1L, use_message = FALSE),
               "must be >= 2")
})

test_that("cia_rm print summary and plot compatibility is retained", {
  dat <- make_cia_rm_perf_data(n_methods = 3L)
  fit <- cia_rm(dat, "value", "subject", "method", "time",
                ci = TRUE, inference = "delta", homogeneous = TRUE,
                use_message = FALSE)

  expect_output(print(fit), "Repeated-measures")
  sm <- summary(fit)
  expect_s3_class(sm, "summary.cia_rm")
  expect_output(print(sm), "Repeated-measures")

  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(tmp)
  on.exit({
    grDevices::dev.off()
    unlink(tmp)
  }, add = TRUE)
  expect_silent(plot(fit))
})

make_cia_rm_dat2 <- function() {
  dat <- expand.grid(
    id = factor(c("s1", "s2", "s3")),
    method = factor(c("A", "B")),
    time = factor(c("t1", "t2", "t3")),
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
    )
  dat$y <- c(
    1.0, 2.0, 1.5,
    2.5, 2.8, 3.2,
    3.0, 4.5, 4.2,
    1.7, 2.8, 2.0,
    2.9, 3.4, 4.1,
    4.0, 5.8, 4.9
  )
  dat
}

make_cia_rm_dat3 <- function() {
  dat <- expand.grid(
    id = factor(c("s1", "s2", "s3")),
    method = factor(c("A", "B", "C")),
    time = factor(c("t1", "t2", "t3")),
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
    )
  dat$y <- c(
    1.0, 2.0, 1.5,
    2.5, 2.8, 3.2,
    3.0, 4.5, 4.2,
    1.7, 2.8, 2.0,
    2.9, 3.4, 4.1,
    4.0, 5.8, 4.9,
    0.9, 1.6, 1.2,
    2.2, 2.7, 3.0,
    2.6, 4.0, 4.1
  )
  dat
}

make_cia_rm_dat_near1 <- function() {
  set.seed(42)
  dat <- expand.grid(
    id = factor(sprintf("s%02d", 1:8)),
    method = factor(c("Sensor_A", "Sensor_B")),
    time = 1:5,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  subj_eff <- rnorm(8, sd = 0.8)[dat$id]
  time_eff <- c(0, 0.1, 0.2, 0.3, 0.4)[dat$time]
  method_eff <- c(Sensor_A = 0, Sensor_B = 0.03)[dat$method]
  interaction <- ifelse(dat$method == "Sensor_B" & dat$time >= 4, 0.02, 0)
  dat$y <- subj_eff + time_eff + method_eff + interaction + rnorm(nrow(dat), sd = 0.18)
  dat
}

make_cia_rm_dat3_near1 <- function() {
  set.seed(122)
  dat <- expand.grid(
    id = factor(sprintf("s%02d", 1:8)),
    method = factor(c("Sensor_A", "Sensor_B", "Sensor_C")),
    time = 1:5,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  subj_eff <- rnorm(8, sd = 0.75)[dat$id]
  time_eff <- c(0, 0.08, 0.17, 0.26, 0.35)[dat$time]
  method_eff <- c(Sensor_A = 0, Sensor_B = 0.03, Sensor_C = -0.02)[dat$method]
  interaction <- ifelse(dat$method == "Sensor_B" & dat$time >= 4, 0.02, 0) +
    ifelse(dat$method == "Sensor_C" & dat$time <= 2, -0.01, 0)
  dat$y <- subj_eff + time_eff + method_eff + interaction + rnorm(nrow(dat), sd = 0.25)
  dat
}

manual_cia_rm_pair <- function(dat, method_a, method_b, constrain_vc = FALSE, homogeneous = FALSE) {
  dat <- dat[dat$method %in% c(method_a, method_b), , drop = FALSE]
  dat$method <- factor(dat$method, levels = c(method_a, method_b))
  dat$time <- factor(dat$time)
  dat$id <- factor(dat$id)

  ids <- levels(dat$id)
  times <- levels(dat$time)
  n <- length(ids)
  K <- length(times)

  Z <- array(NA_real_, dim = c(n, 2L, K), dimnames = list(ids, c(method_a, method_b), times))
  for (i in seq_along(ids)) {
    for (r in 1:2) {
      for (k in seq_along(times)) {
        Z[i, r, k] <- dat$y[dat$id == ids[[i]] & dat$method == levels(dat$method)[[r]] & dat$time == times[[k]]]
      }
    }
  }

  grand <- mean(Z)
  Zi <- apply(Z, 1, mean)
  Zr <- apply(Z, 2, mean)
  Zk <- apply(Z, 3, mean)
  Zir <- apply(Z, c(1, 2), mean)
  Zik <- apply(Z, c(1, 3), mean)
  Zrk <- apply(Z, c(2, 3), mean)

  ss_subject_method <- K * sum((Zir - Zi - rep(Zr, each = n) + grand)^2)
  ss_method_time <- n * sum((Zrk - matrix(Zr, nrow = 2L, ncol = K) - matrix(Zk, nrow = 2L, ncol = K, byrow = TRUE) + grand)^2)

  ss_error <- 0
  for (i in seq_len(n)) {
    for (r in 1:2) {
      for (k in seq_len(K)) {
        resid <- Z[i, r, k] - Zir[i, r] - Zik[i, k] - Zrk[r, k] + Zi[i] + Zr[r] + Zk[k] - grand
        ss_error <- ss_error + resid^2
      }
    }
  }

  df_subject_method <- n - 1
  df_method_time <- K - 1
  df_error <- (n - 1) * (K - 1)
  ms_subject_method <- ss_subject_method / df_subject_method
  ms_method_time <- ss_method_time / df_method_time
  ms_error <- ss_error / df_error

  sigma2_error <- ms_error
  sigma2_subject_method <- (ms_subject_method - ms_error) / K
  if (isTRUE(constrain_vc)) {
    sigma2_error <- max(sigma2_error, 0)
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }

  d_k <- vapply(seq_len(K), function(k) mean(Z[, 1, k] - Z[, 2, k]), numeric(1))
  denom <- d_k^2 + 2 * sigma2_subject_method + 2 * sigma2_error
  psi <- ifelse(is.finite(denom) & denom > 0, 2 * sigma2_error / denom, NA_real_)

  F_stat <- ms_method_time / ms_error
  p_val <- stats::pf(F_stat, df1 = df_method_time, df2 = df_error, lower.tail = FALSE)

  out <- list(
    ss_subject_method = unname(ss_subject_method),
    ss_method_time = unname(ss_method_time),
    ss_error = unname(ss_error),
    ms_subject_method = unname(ms_subject_method),
    ms_method_time = unname(ms_method_time),
    ms_error = unname(ms_error),
    sigma2_error = unname(sigma2_error),
    sigma2_subject_method = unname(sigma2_subject_method),
    repeatability = unname(1.96 * sqrt(2 * sigma2_error)),
    d_k = unname(d_k),
    psi = unname(psi),
    F = unname(F_stat),
    p = unname(p_val),
    df_method_time = unname(df_method_time),
    df_error = unname(df_error)
  )

  if (isTRUE(homogeneous)) {
    ms_error_reduced <- (ss_method_time + ss_error) / (df_method_time + df_error)
    sigma2_error_common <- ms_error_reduced
    sigma2_subject_method_common <- (ms_subject_method - ms_error_reduced) / K
    if (isTRUE(constrain_vc)) {
      sigma2_error_common <- max(sigma2_error_common, 0)
      sigma2_subject_method_common <- max(sigma2_subject_method_common, 0)
    }
    d_common <- mean(vapply(seq_len(K), function(k) Z[, 1, k] - Z[, 2, k], numeric(n)))
    denom_common <- d_common^2 + 2 * sigma2_subject_method_common + 2 * sigma2_error_common
    out$common <- unname(if (is.finite(denom_common) && denom_common > 0) {
      2 * sigma2_error_common / denom_common
    } else {
      NA_real_
    })
    out$common_sigma2_error <- unname(sigma2_error_common)
    out$common_sigma2_subject_method <- unname(sigma2_subject_method_common)
  }

  out
}

manual_cia_rm_overall <- function(dat, constrain_vc = FALSE, homogeneous = FALSE) {
  dat$method <- factor(dat$method)
  dat$time <- factor(dat$time)
  dat$id <- factor(dat$id)

  ids <- levels(dat$id)
  methods <- levels(dat$method)
  times <- levels(dat$time)
  n <- length(ids)
  J <- length(methods)
  K <- length(times)

  Z <- array(NA_real_, dim = c(n, J, K), dimnames = list(ids, methods, times))
  for (i in seq_along(ids)) {
    for (j in seq_along(methods)) {
      for (k in seq_along(times)) {
        Z[i, j, k] <- dat$y[
          dat$id == ids[[i]] &
            dat$method == methods[[j]] &
            dat$time == times[[k]]
        ]
      }
    }
  }

  grand <- mean(Z)
  Zi <- apply(Z, 1, mean)
  Zj <- apply(Z, 2, mean)
  Zk <- apply(Z, 3, mean)
  Zij <- apply(Z, c(1, 2), mean)
  Zik <- apply(Z, c(1, 3), mean)
  Zjk <- apply(Z, c(2, 3), mean)

  ss_subject_method <- K * sum(vapply(
    seq_len(n),
    function(i) {
      sum((Zij[i, ] - Zi[i] - Zj + grand)^2)
    },
    numeric(1)
  ))
  ss_method_time <- n * sum(vapply(
    seq_len(J),
    function(j) {
      sum((Zjk[j, ] - Zj[j] - Zk + grand)^2)
    },
    numeric(1)
  ))

  ss_error <- 0
  for (i in seq_len(n)) {
    for (j in seq_len(J)) {
      for (k in seq_len(K)) {
        resid <- Z[i, j, k] - Zij[i, j] - Zik[i, k] - Zjk[j, k] + Zi[i] + Zj[j] + Zk[k] - grand
        ss_error <- ss_error + resid^2
      }
    }
  }

  df_subject_method <- (n - 1) * (J - 1)
  df_method_time <- (J - 1) * (K - 1)
  df_error <- (n - 1) * (J - 1) * (K - 1)

  ms_subject_method <- ss_subject_method / df_subject_method
  ms_method_time <- ss_method_time / df_method_time
  ms_error <- ss_error / df_error

  sigma2_error <- ms_error
  sigma2_subject_method <- (ms_subject_method - ms_error) / K
  if (isTRUE(constrain_vc)) {
    sigma2_error <- max(sigma2_error, 0)
    sigma2_subject_method <- max(sigma2_subject_method, 0)
  }

  pair_idx <- utils::combn(seq_len(J), 2)
  mean_pair_d2 <- vapply(
    seq_len(K),
    function(k) {
      mean(vapply(
        seq_len(ncol(pair_idx)),
        function(p) {
          idx <- pair_idx[, p]
          (Zjk[idx[1], k] - Zjk[idx[2], k])^2
        },
        numeric(1)
      ))
    },
    numeric(1)
  )
  denom <- mean_pair_d2 + 2 * sigma2_subject_method + 2 * sigma2_error
  overall <- ifelse(is.finite(denom) & denom > 0, 2 * sigma2_error / denom, NA_real_)

  out <- list(
    overall = unname(overall),
    sigma2_error = unname(sigma2_error),
    sigma2_subject_method = unname(sigma2_subject_method),
    repeatability = unname(1.96 * sqrt(2 * sigma2_error)),
    ms_subject_method = unname(ms_subject_method),
    ms_method_time = unname(ms_method_time),
    ms_error = unname(ms_error),
    ss_subject_method = unname(ss_subject_method),
    ss_method_time = unname(ss_method_time),
    ss_error = unname(ss_error),
    df_subject_method = unname(df_subject_method),
    df_method_time = unname(df_method_time),
    df_error = unname(df_error),
    F = unname(ms_method_time / ms_error),
    p = unname(stats::pf(ms_method_time / ms_error, df1 = df_method_time, df2 = df_error, lower.tail = FALSE))
  )

  if (isTRUE(homogeneous)) {
    ms_error_reduced <- (ss_method_time + ss_error) / (df_method_time + df_error)
    common_sigma2_error <- ms_error_reduced
    common_sigma2_subject_method <- (ms_subject_method - ms_error_reduced) / K
    if (isTRUE(constrain_vc)) {
      common_sigma2_error <- max(common_sigma2_error, 0)
      common_sigma2_subject_method <- max(common_sigma2_subject_method, 0)
    }
    mean_pair_d2_common <- mean(vapply(
      seq_len(ncol(pair_idx)),
      function(p) {
        idx <- pair_idx[, p]
        (Zj[idx[1]] - Zj[idx[2]])^2
      },
      numeric(1)
    ))
    denom_common <- mean_pair_d2_common + 2 * common_sigma2_subject_method + 2 * common_sigma2_error
    out$overall_common <- unname(if (is.finite(denom_common) && denom_common > 0) {
      2 * common_sigma2_error / denom_common
    } else {
      NA_real_
    })
    out$common_sigma2_error <- unname(common_sigma2_error)
    out$common_sigma2_subject_method <- unname(common_sigma2_subject_method)
    out$common_repeatability <- unname(1.96 * sqrt(2 * common_sigma2_error))
  }

  out
}

test_that("cia_rm wrapper validates required repeated-measures structure", {
  dat <- make_cia_rm_dat2()

  expect_error(cia_rm(dat, "y", "id", method = NULL, time = "time"), "method")
  expect_error(cia_rm(dat, "y", "id", method = "method", time = NULL), "time")

  dat_one_method <- subset(dat, method == "A")
  expect_error(cia_rm(dat_one_method, "y", "id", method = "method", time = "time"), "two method levels")

  dat_one_time <- subset(dat, time == "t1")
  expect_error(cia_rm(dat_one_time, "y", "id", method = "method", time = "time"), "two time/condition levels")

  dat_one_subject <- subset(dat, id == "s1")
  expect_error(cia_rm(dat_one_subject, "y", "id", method = "method", time = "time"), "two subjects")

  dat_dup <- rbind(dat, dat[1, , drop = FALSE])
  expect_error(
    cia_rm(dat_dup, "y", "id", method = "method", time = "time"),
    "each subject-method-time cell must contain exactly one observation"
  )

  dat_miss <- dat[-1, , drop = FALSE]
  expect_error(
    cia_rm(dat_miss, "y", "id", method = "method", time = "time"),
    "each subject-method-time cell must contain exactly one observation"
  )

  dat_bad_y <- dat
  dat_bad_y$y[1] <- Inf
  expect_error(cia_rm(dat_bad_y, "y", "id", method = "method", time = "time"), "finite numeric values")
})

test_that("cia_rm_anova_cpp returns the hand-computed ANOVA quantities", {
  dat <- make_cia_rm_dat2()
  manual <- manual_cia_rm_pair(dat, "A", "B")
  cpp_fun <- get("cia_rm_anova_cpp", envir = asNamespace("matrixCorr"))

  raw <- cpp_fun(
    y = dat$y,
    subject = as.integer(factor(dat$id)),
    method = as.integer(factor(dat$method, levels = c("A", "B"))),
    time = as.integer(factor(dat$time, levels = c("t1", "t2", "t3"))),
    n_subjects = 3L,
    n_methods = 2L,
    n_times = 3L,
    homogeneous = TRUE,
    constrain_vc = FALSE,
    n_threads = 1L
  )

  expect_equal(raw$ss_subject_method[1, 2], manual$ss_subject_method, tolerance = 1e-12)
  expect_equal(raw$ss_method_time[1, 2], manual$ss_method_time, tolerance = 1e-12)
  expect_equal(raw$ss_error[1, 2], manual$ss_error, tolerance = 1e-12)
  expect_equal(raw$ms_subject_method[1, 2], manual$ms_subject_method, tolerance = 1e-12)
  expect_equal(raw$ms_method_time[1, 2], manual$ms_method_time, tolerance = 1e-12)
  expect_equal(raw$ms_error[1, 2], manual$ms_error, tolerance = 1e-12)
  expect_equal(raw$sigma2_error[1, 2], manual$sigma2_error, tolerance = 1e-12)
  expect_equal(raw$sigma2_subject_method[1, 2], manual$sigma2_subject_method, tolerance = 1e-12)
})

test_that("cia_rm condition-specific CIA matches the repeated-measures formula", {
  dat <- make_cia_rm_dat2()
  manual <- manual_cia_rm_pair(dat, "A", "B", constrain_vc = TRUE)

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time"
  )

  expect_s3_class(fit, "cia_rm")
  expect_equal(unname(fit$est["A", "B", ]), manual$psi, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "method_time_diff")["A", "B", ]), manual$d_k, tolerance = 1e-12)
})

test_that("cia_rm homogeneity test matches the method-time interaction F test", {
  dat <- make_cia_rm_dat2()
  manual <- manual_cia_rm_pair(dat, "A", "B")

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time"
  )

  expect_equal(unname(attr(fit, "homogeneity_F")["A", "B"]), manual$F, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "homogeneity_p")["A", "B"]), manual$p, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "df_method_time")["A", "B"]), manual$df_method_time, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "df_error")["A", "B"]), manual$df_error, tolerance = 1e-12)
})

test_that("cia_rm common estimate matches the reduced homogeneous model", {
  dat <- make_cia_rm_dat2()
  manual <- manual_cia_rm_pair(dat, "A", "B", constrain_vc = TRUE, homogeneous = TRUE)

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  expect_equal(unname(fit$common["A", "B"]), manual$common, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "common_sigma2_error")["A", "B"]), manual$common_sigma2_error, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "common_sigma2_subject_method")["A", "B"]), manual$common_sigma2_subject_method, tolerance = 1e-12)
})

test_that("cia_rm computes each method pair independently when there are three methods", {
  dat <- make_cia_rm_dat3()
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  for (pair in list(c("A", "B"), c("A", "C"), c("B", "C"))) {
    manual <- manual_cia_rm_pair(dat, pair[[1]], pair[[2]], constrain_vc = TRUE, homogeneous = TRUE)
    expect_equal(unname(fit$est[pair[[1]], pair[[2]], ]), manual$psi, tolerance = 1e-12)
    expect_equal(unname(fit$common[pair[[1]], pair[[2]]]), manual$common, tolerance = 1e-12)
  }
})

test_that("global overall J=2 estimate matches the existing pairwise estimate", {
  dat <- make_cia_rm_dat2()
  cpp_fun <- get("cia_rm_anova_cpp", envir = asNamespace("matrixCorr"))

  raw <- cpp_fun(
    y = dat$y,
    subject = as.integer(factor(dat$id)),
    method = as.integer(factor(dat$method, levels = c("A", "B"))),
    time = as.integer(factor(dat$time, levels = c("t1", "t2", "t3"))),
    n_subjects = 3L,
    n_methods = 2L,
    n_times = 3L,
    homogeneous = TRUE,
    constrain_vc = FALSE,
    n_threads = 1L
  )

  expect_equal(as.numeric(raw$overall), c(raw$est[1, 2, ]), tolerance = 1e-12)

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )
  expect_null(fit$overall)
  expect_null(fit$overall.common)
})

test_that("cia_rm overall multi-method estimate is computed from the global ANOVA model", {
  dat <- make_cia_rm_dat3()
  manual <- manual_cia_rm_overall(dat, constrain_vc = TRUE, homogeneous = TRUE)
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  pair_mean <- vapply(
    seq_len(length(levels(dat$time))),
    function(k) mean(c(
      fit$est["A", "B", k],
      fit$est["A", "C", k],
      fit$est["B", "C", k]
    )),
    numeric(1)
  )

  expect_equal(unname(fit$overall), manual$overall, tolerance = 1e-12)
  expect_gt(max(abs(unname(fit$overall) - pair_mean)), 1e-6)
  expect_equal(unname(attr(fit, "overall_sigma2_error")), manual$sigma2_error, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_sigma2_subject_method")), manual$sigma2_subject_method, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_repeatability")), manual$repeatability, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_homogeneity_F")), manual$F, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_homogeneity_p")), manual$p, tolerance = 1e-12)
})

test_that("cia_rm overall common estimate uses the reduced homogeneous model", {
  dat <- make_cia_rm_dat3()
  manual <- manual_cia_rm_overall(dat, constrain_vc = TRUE, homogeneous = TRUE)
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  expect_equal(unname(fit$overall.common), manual$overall_common, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_common_sigma2_error")), manual$common_sigma2_error, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_common_sigma2_subject_method")), manual$common_sigma2_subject_method, tolerance = 1e-12)
  expect_equal(unname(attr(fit, "overall_common_repeatability")), manual$common_repeatability, tolerance = 1e-12)
})

test_that("cia_rm bootstrap intervals resample subjects and preserve array shapes", {
  dat <- make_cia_rm_dat3()

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "bootstrap",
    homogeneous = TRUE,
    B = 50L,
    seed = 1L
  )

  expect_s3_class(fit, "cia_rm")
  expect_identical(attr(fit, "ci.method", exact = TRUE), "subject_bootstrap_percentile")
  expect_identical(dim(fit$est), dim(fit$lwr.ci))
  expect_identical(dim(fit$est), dim(fit$upr.ci))
  expect_identical(dim(fit$common), dim(fit$common.lwr.ci))
  expect_identical(dim(fit$common), dim(fit$common.upr.ci))
})

test_that("cia_rm uses delta-method confidence intervals by default", {
  dat <- make_cia_rm_dat3()

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    homogeneous = TRUE
  )

  expect_identical(attr(fit, "ci.method", exact = TRUE), "delta_normal")
  expect_identical(attr(fit, "estimator", exact = TRUE), "vc_constrained")
  expect_true(!is.null(fit$se))
  expect_true(!is.null(fit$lwr.ci))
  expect_true(!is.null(fit$upr.ci))
  expect_identical(dim(fit$se), dim(fit$est))
  expect_identical(dim(fit$lwr.ci), dim(fit$est))
  expect_identical(dim(fit$upr.ci), dim(fit$est))
  expect_identical(dim(fit$common.se), dim(fit$common))
  expect_identical(dim(fit$common.lwr.ci), dim(fit$common))
  expect_identical(dim(fit$common.upr.ci), dim(fit$common))
})

test_that("cia_rm unconstrained and constrained estimators differ only by the boundary rule", {
  dat <- make_cia_rm_dat_near1()

  fit_raw <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "delta",
    estimator = "mom_unconstrained"
  )
  fit_bnd <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "delta",
    estimator = "vc_constrained"
  )

  expect_true(any(fit_raw$est > 1, na.rm = TRUE) || any(fit_raw$est < 0, na.rm = TRUE) ||
                any(fit_raw$upr.ci > 1, na.rm = TRUE) || any(fit_raw$lwr.ci < 0, na.rm = TRUE))
  expect_true(all(fit_bnd$est[is.finite(fit_bnd$est)] <= 1))
  expect_true(all(fit_bnd$est[is.finite(fit_bnd$est)] >= 0))
  expect_true(any(fit_raw$upr.ci > 1, na.rm = TRUE) || any(fit_raw$lwr.ci < 0, na.rm = TRUE))
  expect_true(all(fit_bnd$upr.ci[is.finite(fit_bnd$upr.ci)] <= 1))
  expect_true(all(fit_bnd$lwr.ci[is.finite(fit_bnd$lwr.ci)] >= 0))
})

test_that("cia_rm overall constrained estimator truncates negative subject-method variance and bounds CIs", {
  dat <- make_cia_rm_dat3_near1()
  manual_raw <- manual_cia_rm_overall(dat, constrain_vc = FALSE, homogeneous = TRUE)
  manual_bnd <- manual_cia_rm_overall(dat, constrain_vc = TRUE, homogeneous = TRUE)

  fit_raw <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "delta",
    estimator = "mom_unconstrained",
    homogeneous = TRUE
  )
  fit_bnd <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "delta",
    estimator = "vc_constrained",
    homogeneous = TRUE
  )

  expect_lt(manual_raw$sigma2_subject_method, 0)
  expect_equal(unname(attr(fit_bnd, "overall_sigma2_subject_method")), 0, tolerance = 1e-12)
  expect_equal(unname(attr(fit_bnd, "overall_sigma2_subject_method")), manual_bnd$sigma2_subject_method, tolerance = 1e-12)
  expect_equal(unname(fit_bnd$overall), manual_bnd$overall, tolerance = 1e-10)
  expect_true(any(fit_raw$overall.upr.ci > 1, na.rm = TRUE) || any(fit_raw$overall.lwr.ci < 0, na.rm = TRUE))
  expect_true(all(fit_bnd$overall.upr.ci[is.finite(fit_bnd$overall.upr.ci)] <= 1))
  expect_true(all(fit_bnd$overall.lwr.ci[is.finite(fit_bnd$overall.lwr.ci)] >= 0))
  expect_true(is.finite(fit_bnd$overall.common))
  expect_true(is.finite(fit_bnd$overall.common.lwr.ci))
  expect_true(is.finite(fit_bnd$overall.common.upr.ci))
  expect_true(fit_bnd$overall.common.lwr.ci >= 0)
  expect_true(fit_bnd$overall.common.upr.ci <= 1)
})

test_that("cia_rm delta gradient matches an independent finite-difference check", {
  dat <- make_cia_rm_dat2()
  prep <- matrixCorr:::.mc_cia_rm_prepare(dat, "y", "id", "method", "time")
  cube <- matrixCorr:::.mc_cia_rm_cube(prep)
  D <- matrixCorr:::.mc_cia_rm_pair_diff_matrix(cube, 1L, 2L)
  stats <- matrixCorr:::.mc_cia_rm_pair_moment_stats(D)
  m <- c(stats$d, as.vector(stats$M))

  grad_pkg <- matrixCorr:::.mc_cia_rm_num_grad(
    function(mm) matrixCorr:::.mc_cia_rm_pair_est_from_moments(mm, K = stats$K, n = stats$n)[[1L]],
    m
  )

  grad_ref <- numeric(length(m))
  for (idx in seq_along(m)) {
    h <- 1e-7 * max(1, abs(m[[idx]]))
    plus <- m
    minus <- m
    plus[[idx]] <- plus[[idx]] + h
    minus[[idx]] <- minus[[idx]] - h
    grad_ref[[idx]] <-
      (matrixCorr:::.mc_cia_rm_pair_est_from_moments(plus, K = stats$K, n = stats$n)[[1L]] -
         matrixCorr:::.mc_cia_rm_pair_est_from_moments(minus, K = stats$K, n = stats$n)[[1L]]) / (2 * h)
  }

  expect_equal(grad_pkg, grad_ref, tolerance = 1e-6)
})

test_that("cia_rm returns common delta intervals when homogeneous is requested", {
  dat <- make_cia_rm_dat2()

  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    homogeneous = TRUE
  )

  expect_true(is.matrix(fit$common.se))
  expect_true(is.matrix(fit$common.lwr.ci))
  expect_true(is.matrix(fit$common.upr.ci))
  expect_true(is.finite(fit$common.se["A", "B"]))
})

test_that("cia_rm returns overall multi-method confidence intervals when there are at least three methods", {
  dat <- make_cia_rm_dat3()

  fit_delta <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    homogeneous = TRUE
  )
  fit_boot <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "bootstrap",
    homogeneous = TRUE,
    B = 40L,
    seed = 1L
  )

  expect_length(fit_delta$overall, 3L)
  expect_length(fit_delta$overall.se, 3L)
  expect_length(fit_delta$overall.lwr.ci, 3L)
  expect_length(fit_delta$overall.upr.ci, 3L)
  expect_true(all(names(fit_delta$overall) == levels(factor(dat$time))))
  expect_true(is.finite(fit_delta$overall.common.se))
  expect_length(fit_boot$overall.lwr.ci, 3L)
  expect_length(fit_boot$overall.upr.ci, 3L)
  expect_true(is.finite(fit_boot$overall.common.lwr.ci))
  expect_true(is.finite(fit_boot$overall.common.upr.ci))
})

test_that("cia_rm objects carry repeated-measures diagnostics and summary structure", {
  dat <- make_cia_rm_dat3()
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  expect_identical(attr(fit, "method"), "Repeated-measures coefficient of individual agreement")
  expect_identical(attr(fit, "description"), "Pairwise repeated-measures CIA from categorical-condition ANOVA")
  expect_identical(attr(fit, "package"), "matrixCorr")
  expect_true(is.matrix(attr(fit, "sigma2_error")))
  expect_true(is.matrix(attr(fit, "sigma2_subject_method")))
  expect_true(is.matrix(attr(fit, "repeatability")))
  expect_true(is.matrix(attr(fit, "homogeneity_F")))
  expect_true(is.matrix(attr(fit, "homogeneity_p")))
  expect_length(fit$overall, 3L)
  expect_true(is.numeric(attr(fit, "overall_sigma2_error")))
  expect_true(is.numeric(attr(fit, "overall_sigma2_subject_method")))
  expect_true(is.numeric(attr(fit, "overall_repeatability")))
  expect_true(is.numeric(attr(fit, "overall_homogeneity_F")))
  expect_true(is.numeric(attr(fit, "overall_homogeneity_p")))

  sm <- summary(fit)
  expect_s3_class(sm, "summary.cia_rm")
  expect_true(all(c(
    "method_1", "method_2", "time", "cia",
    "sigma2_error", "sigma2_subject_method",
    "repeatability", "homogeneity_F", "homogeneity_p"
  ) %in% names(sm)))
  expect_true(any(sm$time == "common"))
  expect_true(is.data.frame(attr(sm, "overall_section", exact = TRUE)))
  expect_true(is.data.frame(attr(sm, "overall_common_section", exact = TRUE)))
})

test_that("cia_rm print is compact while summary print includes diagnostics", {
  dat <- make_cia_rm_dat3()
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  out_print <- paste(capture.output(print(fit)), collapse = "\n")
  out_summary <- paste(capture.output(print(summary(fit))), collapse = "\n")

  expect_match(out_print, "Condition-specific CIA estimates")
  expect_match(out_print, "Overall multi-method CIA")
  expect_match(out_print, "Common overall CIA")
  expect_no_match(out_print, "Pair diagnostics")
  expect_match(out_summary, "CIA estimates")
  expect_match(out_summary, "Overall multi-method CIA")
  expect_match(out_summary, "Common overall CIA")
  expect_match(out_summary, "Pair diagnostics")
})

test_that("cia_rm print preview respects digits", {
  dat <- make_cia_rm_dat2()
  fit <- cia_rm(
    dat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "bootstrap",
    B = 20L,
    seed = 1L
  )

  out_print <- paste(capture.output(print(fit, digits = 3)), collapse = "\n")
  expect_no_match(out_print, "0\\.[0-9]{10,}")
})

test_that("cia_rm plot detects categorical versus continuous condition scales", {
  dat_cat <- make_cia_rm_dat2()
  fit_cat <- cia_rm(
    dat_cat,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  expect_identical(attr(fit_cat, "time_scale", exact = TRUE), "categorical")
  p_cat <- plot(fit_cat)
  expect_s3_class(p_cat, "ggplot")

  dat_num <- make_cia_rm_dat2()
  dat_num$time <- c(1, 2, 3)[match(dat_num$time, levels(dat_num$time))]
  fit_num <- cia_rm(
    dat_num,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    homogeneous = TRUE
  )

  expect_identical(attr(fit_num, "time_scale", exact = TRUE), "continuous")
  p_num <- plot(fit_num, facet_by_pair = TRUE)
  expect_s3_class(p_num, "ggplot")
  expect_true(inherits(p_num$facet, "FacetWrap"))
})

test_that("cia_rm plot handles confidence intervals for continuous condition scales", {
  dat_num <- make_cia_rm_dat3()
  dat_num$time <- c(1, 2, 3)[match(dat_num$time, levels(dat_num$time))]

  fit_num <- cia_rm(
    dat_num,
    response = "y",
    subject = "id",
    method = "method",
    time = "time",
    ci = TRUE,
    inference = "bootstrap",
    homogeneous = TRUE,
    B = 20L,
    seed = 1L
  )

  p_num <- plot(fit_num, facet_by_pair = TRUE)
  expect_s3_class(p_num, "ggplot")
})

# Package-level documentation and namespace setup

#' matrixCorr
#'
#' Correlation, agreement, and intraclass correlation estimators with a
#' consistent S3 interface.
#'
#' `print()` methods provide compact previews and `summary()` methods provide
#' richer but still bounded digests. Truncation affects console display only;
#' full results remain available through direct extraction and coercion helpers
#' such as `as.matrix()`, `as.data.frame()`, and `tidy()` where implemented.
#'
#' Package-wide display defaults can be controlled with options:
#' \itemize{
#'   \item `matrixCorr.threads` (default `1L`; used as the default for
#'   `n_threads` in thread-aware estimators)
#'   \item `matrixCorr.print_max_rows` (default `20L`)
#'   \item `matrixCorr.print_topn` (default `5L`)
#'   \item `matrixCorr.print_max_vars` (default `NULL`, derived from console width)
#'   \item `matrixCorr.print_show_ci` (default `"yes"`)
#'   \item `matrixCorr.summary_max_rows` (default `12L`)
#'   \item `matrixCorr.summary_topn` (default `5L`)
#'   \item `matrixCorr.summary_max_vars` (default `10L`)
#'   \item `matrixCorr.summary_show_ci` (default `"yes"`)
#' }
#'
#' Display methods validate user-facing arguments with `cli` conditions.
#' Confidence-interval visibility is explicit: `show_ci` accepts only `"yes"`
#' or `"no"`.
#'
#' A common setup is:
#' \preformatted{
#' options(matrixCorr.threads = parallel::detectCores(logical = FALSE))
#' }
#'
#' @name matrixCorr-package
#' @aliases matrixCorr RcppExports
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @useDynLib matrixCorr, .registration = TRUE
## usethis namespace: end

# Silence NOTES about NSE vars in ggplot2 etc.
utils::globalVariables(c(
  "Var1", "Var2", "Tau", "Rho", "Pearson", "CCC",
  "label", "PCor", "r", "dCor", "bicor", "ci_label",
  "Tetrachoric", "Polychoric", "Polyserial", "Biserial",
  "diffs", "hi_l", "hi_u", "lab", "lo_l", "lo_u", "loaL",
  "loaU", "md", "md_l", "md_u", "means", "j", "k"
))

#' Internal display helpers
#'
#' Shared console preview and summary formatting for matrixCorr S3 methods.
#'
#' @keywords internal
#' @noRd
NULL

.mc_coalesce <- function(x, y) if (is.null(x)) y else x

.mc_display_option <- function(name, default) {
  getOption(paste0("matrixCorr.", name), default = default)
}

.mc_validate_yes_no <- function(x,
                                arg = as.character(substitute(x)),
                                default = NULL) {
  if (is.null(x)) {
    x <- default
  }
  if (!is.character(x) || length(x) != 1L || is.na(x) || !x %in% c("yes", "no")) {
    abort_bad_arg(arg, message = "must be one of \"yes\" or \"no\".")
  }
  x
}

.mc_resolve_show_ci <- function(show_ci,
                                context = c("print", "summary")) {
  context <- match.arg(context)
  .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option(paste0(context, "_show_ci"), "yes")
  )
}

.mc_extract_legacy_display_args <- function(dots,
                                            n = NULL,
                                            topn = NULL,
                                            max_vars = NULL) {
  dots <- if (is.null(dots)) list() else dots
  if (is.null(n) && !is.null(dots$max_rows)) {
    n <- dots$max_rows
    if (is.null(topn) && is.numeric(n) && is.finite(n)) {
      topn <- max(1L, floor(as.integer(n) / 2L))
    }
  }
  if (is.null(max_vars) && !is.null(dots$max_cols)) {
    max_vars <- dots$max_cols
  }
  dots$max_rows <- NULL
  dots$max_cols <- NULL
  list(dots = dots, n = n, topn = topn, max_vars = max_vars)
}

.mc_validate_optional_count <- function(x,
                                        arg = as.character(substitute(x)),
                                        allow_null = TRUE) {
  if (is.null(x)) {
    if (allow_null) return(NULL)
    abort_bad_arg(arg, message = "must not be NULL.")
  }
  if (!rlang::is_scalar_integerish(x) || is.na(x) || x <= 0) {
    abort_bad_arg(arg, message = "must be a positive integer.")
  }
  as.integer(x)
}

.mc_resolve_display_args <- function(context = c("print", "summary"),
                                     n = NULL,
                                     topn = NULL,
                                     max_vars = NULL,
                                     width = NULL,
                                     show_ci = NULL) {
  context <- match.arg(context)
  topn_supplied <- !is.null(topn)
  defaults <- if (identical(context, "print")) {
    list(
      n = .mc_display_option("print_max_rows", 20L),
      topn = .mc_display_option("print_topn", 5L),
      max_vars = .mc_display_option("print_max_vars", NULL),
      show_ci = .mc_display_option("print_show_ci", "yes")
    )
  } else {
    list(
      n = .mc_display_option("summary_max_rows", 12L),
      topn = .mc_display_option("summary_topn", 5L),
      max_vars = .mc_display_option("summary_max_vars", 10L),
      show_ci = .mc_display_option("summary_show_ci", "yes")
    )
  }

  n <- .mc_validate_optional_count(.mc_coalesce(n, defaults$n), arg = "n", allow_null = FALSE)
  topn <- .mc_validate_optional_count(.mc_coalesce(topn, defaults$topn), arg = "topn", allow_null = FALSE)
  max_vars <- .mc_validate_optional_count(.mc_coalesce(max_vars, defaults$max_vars), arg = "max_vars")
  width <- .mc_validate_optional_count(.mc_coalesce(width, getOption("width", 80L)), arg = "width", allow_null = FALSE)
  show_ci <- .mc_validate_yes_no(show_ci, arg = "show_ci", default = defaults$show_ci)

  if (n <= 1L) {
    abort_bad_arg("n", message = "must be greater than 1.")
  }

  if (topn * 2L > n) {
    if (isTRUE(topn_supplied)) {
      cli::cli_warn(
        c(
          "Display truncation parameters are inconsistent.",
          "i" = "{.arg topn} is larger than half of {.arg n}; reducing the head/tail preview size."
        ),
        class = c("matrixCorr_warning", "matrixCorr_display_warning")
      )
    }
    topn <- max(1L, floor(n / 2L))
  }

  list(
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
}

.mc_count_fmt <- function(x) {
  format(as.integer(x), big.mark = ",", scientific = FALSE, trim = TRUE)
}

.mc_pick_head_tail <- function(total, threshold, topn) {
  if (total <= threshold) {
    return(list(index = seq_len(total), omitted = 0L, truncated = FALSE))
  }
  keep <- unique(c(seq_len(topn), seq.int(total - topn + 1L, total)))
  list(index = keep, omitted = total - length(keep), truncated = TRUE)
}

.mc_width_to_max_vars <- function(labels,
                                  width,
                                  min_visible = 2L) {
  labels <- as.character(labels)
  if (!length(labels)) return(0L)
  name_width <- max(nchar(labels, type = "width"), na.rm = TRUE)
  approx_cell <- max(8L, name_width + 1L)
  available <- max(min_visible, floor((width - 12L) / approx_cell))
  min(length(labels), available)
}

.mc_pick_leading_trailing <- function(total, max_visible) {
  if (total <= max_visible) {
    return(list(index = seq_len(total), omitted = 0L, truncated = FALSE))
  }
  left <- ceiling(max_visible / 2)
  right <- floor(max_visible / 2)
  keep <- unique(c(seq_len(left), seq.int(total - right + 1L, total)))
  list(index = keep, omitted = total - length(keep), truncated = TRUE)
}

.mc_preview_columns <- function(labels, width, max_vars = NULL) {
  if (!length(labels)) return(list(index = integer(), omitted = 0L, truncated = FALSE))
  visible <- if (is.null(max_vars)) {
    .mc_width_to_max_vars(labels = labels, width = width)
  } else {
    min(length(labels), max_vars)
  }
  visible <- max(2L, visible)
  .mc_pick_leading_trailing(length(labels), visible)
}

.mc_format_matrix_values <- function(m, digits = 4) {
  if (is.numeric(m)) {
    return(matrix(
      ifelse(is.na(m), NA_character_, formatC(m, format = "f", digits = digits)),
      nrow = nrow(m),
      ncol = ncol(m),
      dimnames = dimnames(m)
    ))
  }
  matrix(as.character(m), nrow = nrow(m), ncol = ncol(m), dimnames = dimnames(m))
}

.mc_build_matrix_preview <- function(m,
                                     digits = 4,
                                     n = 20L,
                                     topn = 5L,
                                     max_vars = NULL,
                                     width = getOption("width", 80L)) {
  row_info <- .mc_pick_head_tail(nrow(m), threshold = n, topn = topn)
  col_labels <- .mc_coalesce(colnames(m), as.character(seq_len(ncol(m))))
  col_info <- .mc_preview_columns(col_labels, width = width, max_vars = max_vars)

  fmt <- .mc_format_matrix_values(m, digits = digits)
  preview <- fmt[row_info$index, col_info$index, drop = FALSE]

  if (col_info$truncated) {
    left <- ceiling(length(col_info$index) / 2)
    preview <- cbind(
      preview[, seq_len(left), drop = FALSE],
      "..." = rep("...", nrow(preview)),
      preview[, seq.int(left + 1L, ncol(preview)), drop = FALSE]
    )
  }

  if (row_info$truncated) {
    top_keep <- topn
    preview <- rbind(
      preview[seq_len(top_keep), , drop = FALSE],
      matrix("...", nrow = 1L, ncol = ncol(preview),
             dimnames = list("...", colnames(preview))),
      preview[seq.int(nrow(preview) - topn + 1L, nrow(preview)), , drop = FALSE]
    )
  }

  list(
    data = as.data.frame(preview, stringsAsFactors = FALSE, check.names = FALSE),
    row_info = row_info,
    col_info = col_info
  )
}

.mc_build_table_preview <- function(df,
                                    n = 20L,
                                    topn = 5L,
                                    max_vars = NULL,
                                    width = getOption("width", 80L)) {
  row_info <- .mc_pick_head_tail(nrow(df), threshold = n, topn = topn)
  col_info <- .mc_preview_columns(names(df), width = width, max_vars = max_vars)
  preview <- df[row_info$index, col_info$index, drop = FALSE]

  if (col_info$truncated) {
    left <- ceiling(length(col_info$index) / 2)
    preview <- data.frame(
      preview[, seq_len(left), drop = FALSE],
      "..." = rep("...", nrow(preview)),
      preview[, seq.int(left + 1L, ncol(preview)), drop = FALSE],
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  }

  if (row_info$truncated) {
    ellipsis_row <- as.list(rep("...", ncol(preview)))
    names(ellipsis_row) <- names(preview)
    preview <- rbind(
      preview[seq_len(topn), , drop = FALSE],
      as.data.frame(ellipsis_row, stringsAsFactors = FALSE, check.names = FALSE),
      preview[seq.int(nrow(preview) - topn + 1L, nrow(preview)), , drop = FALSE]
    )
    rownames(preview) <- NULL
  }

  list(
    data = preview,
    row_info = row_info,
    col_info = col_info
  )
}

.mc_display_footer <- function(row_omitted = 0L,
                               col_omitted = 0L,
                               context = c("print", "summary"),
                               full_hint = TRUE,
                               summary_hint = FALSE) {
  context <- match.arg(context)
  lines <- character()
  truncated <- (row_omitted > 0L) || (col_omitted > 0L)
  if (row_omitted > 0L) {
    lines <- c(lines, sprintf("... %s more rows not shown (omitted)", .mc_count_fmt(row_omitted)))
  }
  if (col_omitted > 0L) {
    lines <- c(lines, sprintf("... %s more variables not shown (omitted)", .mc_count_fmt(col_omitted)))
  }
  if (summary_hint && identical(context, "print") && truncated) {
    lines <- c(lines, "Use summary() for a richer digest.")
  }
  if (full_hint && truncated) {
    lines <- c(lines, "Use as.data.frame()/tidy()/as.matrix() to inspect the full result.")
  }
  lines
}

.mc_ci_column_index <- function(df) {
  if (!ncol(df)) return(integer())
  nm <- names(df)
  which(grepl("(^lwr$|^upr$|(^|[._])lwr$|(^|[._])upr$|^ci$|^ci_method$|^conf[._]level$)", nm))
}

.mc_header_with_ci <- function(base, conf_level = NA_real_, show_ci = "yes") {
  if (identical(show_ci, "yes") && is.finite(conf_level)) {
    sprintf("%s (%g%% CI)", base, 100 * conf_level)
  } else {
    base
  }
}

.mc_print_preview_table <- function(df,
                                    n = 20L,
                                    topn = 5L,
                                    max_vars = NULL,
                                    width = getOption("width", 80L),
                                    context = c("print", "summary"),
                                    full_hint = TRUE,
                                    summary_hint = FALSE,
                                    ...) {
  context <- match.arg(context)
  preview <- .mc_build_table_preview(df, n = n, topn = topn, max_vars = max_vars, width = width)
  print.data.frame(preview$data, row.names = FALSE, right = FALSE, ...)
  foot <- .mc_display_footer(
    row_omitted = preview$row_info$omitted,
    col_omitted = preview$col_info$omitted,
    context = context,
    full_hint = full_hint,
    summary_hint = summary_hint
  )
  if (length(foot)) cat(paste0(foot, collapse = "\n"), "\n", sep = "")
  invisible(preview)
}

.mc_print_preview_matrix <- function(m,
                                     digits = 4,
                                     n = 20L,
                                     topn = 5L,
                                     max_vars = NULL,
                                     width = getOption("width", 80L),
                                     context = c("print", "summary"),
                                     full_hint = TRUE,
                                     summary_hint = FALSE,
                                     ...) {
  context <- match.arg(context)
  preview <- .mc_build_matrix_preview(
    m,
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width
  )
  print.data.frame(preview$data, right = TRUE, ...)
  foot <- .mc_display_footer(
    row_omitted = preview$row_info$omitted,
    col_omitted = preview$col_info$omitted,
    context = context,
    full_hint = full_hint,
    summary_hint = summary_hint
  )
  if (length(foot)) cat(paste0(foot, collapse = "\n"), "\n", sep = "")
  invisible(preview)
}

.mc_line_item <- function(label, value) {
  cat(sprintf("  %-12s: %s\n", label, value))
}

.mc_print_named_digest <- function(items, header = NULL) {
  if (!is.null(header)) cat(header, "\n", sep = "")
  for (nm in names(items)) {
    .mc_line_item(nm, items[[nm]])
  }
  invisible(items)
}

.mc_format_scalar_or_range <- function(x_min,
                                       x_max = x_min,
                                       digits = 4,
                                       integer = FALSE) {
  if (!is.finite(x_min) || !is.finite(x_max)) {
    return(NULL)
  }
  fmt_one <- function(x) {
    if (isTRUE(integer)) {
      .mc_count_fmt(x)
    } else {
      formatC(x, format = "f", digits = digits)
    }
  }
  if (isTRUE(all.equal(x_min, x_max))) {
    return(fmt_one(x_min))
  }
  sprintf("%s to %s", fmt_one(x_min), fmt_one(x_max))
}

.mc_pair_label <- function(var1, var2) {
  sprintf("%s-%s", var1, var2)
}

.mc_top_pair_by <- function(df, which_idx, digits = 4) {
  if (!nrow(df) || is.na(which_idx) || which_idx < 1L || which_idx > nrow(df)) {
    return(NULL)
  }
  row <- df[which_idx, , drop = FALSE]
  if (!all(c("item1", "item2", "estimate") %in% names(row))) {
    return(NULL)
  }
  sprintf(
    "%s (%s)",
    .mc_pair_label(row$item1[[1L]], row$item2[[1L]]),
    formatC(row$estimate[[1L]], format = "f", digits = digits)
  )
}

.mc_pair_extremes <- function(object, digits = 4) {
  tbl <- .mc_format_pair_table(as.matrix(object), digits = digits, sort_abs = FALSE)
  if (!nrow(tbl)) {
    return(list(
      most_negative = NULL,
      most_positive = NULL
    ))
  }
  est <- tbl$estimate
  list(
    most_negative = .mc_top_pair_by(tbl, which.min(est), digits = digits),
    most_positive = .mc_top_pair_by(tbl, which.max(est), digits = digits)
  )
}

.mc_ci_digest <- function(df,
                          conf_level = NA_real_,
                          ci_method = NULL,
                          digits = 3) {
  if (!all(c("lwr", "upr") %in% names(df))) {
    return(setNames(character(), character()))
  }
  keep <- is.finite(df$lwr) & is.finite(df$upr)
  if (!any(keep)) {
    return(setNames(character(), character()))
  }
  widths <- df$upr[keep] - df$lwr[keep]
  out <- c(ci = if (is.finite(conf_level)) sprintf("%g%%", 100 * conf_level) else "available")
  if (is.character(ci_method) && length(ci_method) == 1L && !is.na(ci_method) && nzchar(ci_method)) {
    out <- c(out, ci_method = ci_method)
  }
  width_txt <- .mc_format_scalar_or_range(min(widths), max(widths), digits = digits, integer = FALSE)
  if (!is.null(width_txt)) {
    out <- c(out, ci_width = width_txt)
  }
  cross_zero <- sum(df$lwr[keep] <= 0 & df$upr[keep] >= 0, na.rm = TRUE)
  if (cross_zero > 0L) {
    out <- c(out, cross_zero = sprintf("%s pair(s)", .mc_count_fmt(cross_zero)))
  }
  out
}

.mc_pairwise_top_results <- function(df,
                                     topn = 5L,
                                     show_ci = "yes") {
  if (!nrow(df) || !"estimate" %in% names(df)) {
    return(df[0, , drop = FALSE])
  }
  topn <- min(.mc_validate_optional_count(topn, arg = "topn", allow_null = FALSE), nrow(df))
  ord <- order(abs(df$estimate), decreasing = TRUE, na.last = NA)
  out <- df[utils::head(ord, topn), , drop = FALSE]
  keep <- intersect(
    c(
      "item1", "item2", "estimate", "lwr", "upr", "statistic", "df",
      "p_value", "n_complete", "n_subjects", "n_obs",
      "p_value_adjusted", "reject", "skipped_n", "skipped_prop",
      "lwr", "upr", "p_value", "p_value_adjusted", "reject"
    ),
    names(out)
  )
  if (identical(show_ci, "no")) {
    keep <- setdiff(keep, c("lwr", "upr"))
  }
  out <- out[, keep, drop = FALSE]
  rownames(out) <- NULL
  out
}

.mc_standardize_summary_pairs <- function(df) {
  if (!is.data.frame(df)) return(df)
  nms <- names(df)
  if ("var1" %in% nms) nms[nms == "var1"] <- "item1"
  if ("var2" %in% nms) nms[nms == "var2"] <- "item2"
  if ("method1" %in% nms) nms[nms == "method1"] <- "item1"
  if ("method2" %in% nms) nms[nms == "method2"] <- "item2"
  names(df) <- nms
  df
}

.mc_standardize_summary_counts <- function(df,
                                           repeated = FALSE,
                                           rename_n = FALSE) {
  if (!is.data.frame(df)) return(df)
  if ("based.on" %in% names(df) && !"n_obs" %in% names(df)) {
    names(df)[names(df) == "based.on"] <- "n_obs"
  }
  if (isTRUE(rename_n) && "n" %in% names(df) && !"n_obs" %in% names(df)) {
    names(df)[names(df) == "n"] <- "n_obs"
  }
  if (isTRUE(repeated) && "n_complete" %in% names(df) && !"n_obs" %in% names(df)) {
    names(df)[names(df) == "n_complete"] <- "n_obs"
  }
  df
}

.mc_reorder_summary_columns <- function(df,
                                        repeated = FALSE) {
  if (!is.data.frame(df)) return(df)
  core <- c("item1", "item2", "estimate", "lwr", "upr", "statistic", "df", "p_value")
  counts <- if (isTRUE(repeated)) c("n_subjects", "n_obs") else "n_complete"
  keep <- c(core, counts)
  ordered <- c(intersect(keep, names(df)), setdiff(names(df), keep))
  df[, ordered, drop = FALSE]
}

.mc_finalize_summary_df <- function(df,
                                    class_name,
                                    repeated = FALSE,
                                    rename_n = FALSE) {
  extra_attrs <- attributes(df)
  extra_attrs[c("names", "row.names", "class")] <- NULL
  df <- .mc_standardize_summary_pairs(df)
  df <- .mc_standardize_summary_counts(df, repeated = repeated, rename_n = rename_n)
  df <- .mc_reorder_summary_columns(df, repeated = repeated)
  attributes(df) <- c(
    attributes(df),
    extra_attrs,
    list(class = c(class_name, "summary.matrixCorr", "data.frame"))
  )
  df
}

.mc_finalize_summary_list <- function(x,
                                      class_name) {
  class(x) <- c(class_name, "summary.matrixCorr")
  x
}

.mc_print_ranked_pairs_preview <- function(df,
                                           header = "Strongest pairs by |estimate|",
                                           topn = 5L,
                                           max_vars = NULL,
                                           width = getOption("width", 80L),
                                           digits = 4,
                                           ci_digits = digits,
                                           show_ci = "yes",
                                           ...) {
  top <- .mc_pairwise_top_results(df, topn = topn, show_ci = show_ci)
  if (!nrow(top)) {
    return(invisible(top))
  }
  top <- .mc_format_summary_numeric_table(top, digits = digits, ci_digits = ci_digits)
  cat("\n", header, "\n\n", sep = "")
  .mc_print_preview_table(
    top,
    n = max(2L, nrow(top) + 1L),
    topn = max(1L, min(topn, nrow(top))),
    max_vars = max_vars,
    width = width,
    context = "summary",
    full_hint = FALSE,
    summary_hint = FALSE,
    ...
  )
  foot <- .mc_display_footer(
    row_omitted = max(0L, nrow(df) - nrow(top)),
    col_omitted = 0L,
    context = "summary",
    full_hint = nrow(df) > nrow(top),
    summary_hint = FALSE
  )
  if (length(foot)) cat(paste0(foot, collapse = "\n"), "\n", sep = "")
  invisible(top)
}

.mc_format_summary_numeric_table <- function(df,
                                             digits = 4,
                                             ci_digits = digits) {
  if (!is.data.frame(df) || !nrow(df)) {
    return(df)
  }
  ci_digits <- .mc_coalesce(ci_digits, digits)

  format_cols <- function(x, cols, ndigits) {
    for (nm in intersect(cols, names(x))) {
      vals <- suppressWarnings(as.numeric(x[[nm]]))
      has_negative <- any(is.finite(vals) & vals < 0)
      fmt <- if (isTRUE(has_negative)) {
        sprintf(paste0("% .", ndigits, "f"), vals)
      } else {
        formatC(vals, format = "f", digits = ndigits)
      }
      fmt[is.na(vals)] <- NA_character_
      x[[nm]] <- fmt
    }
    x
  }

  df <- format_cols(df, c("estimate"), digits)
  df <- format_cols(df, c("lwr", "upr"), ci_digits)
  df <- format_cols(
    df,
    c("statistic", "df", "p_value", "fisher_z", "se", "p_value_adjusted", "skipped_prop"),
    digits
  )
  df
}

.mc_corr_summary_digest_items <- function(x, digits = 4, show_ci = "yes") {
  items <- c(
    method = .mc_coalesce(x$method, NA_character_),
    dimensions = sprintf("%d x %d", x$n_rows, x$n_cols),
    pairs = .mc_count_fmt(x$n_pairs)
  )
  items <- items[!is.na(items) & nzchar(items)]

  if (isTRUE(x$has_ci) && identical(show_ci, "yes")) {
    items <- c(items, ci = "yes")
  }
  if (isTRUE(x$has_p)) {
    items <- c(items, p_values = "yes")
  }
  if (isTRUE(x$n_missing > 0L)) {
    items <- c(items, missing = .mc_count_fmt(x$n_missing))
  }
  n_complete_txt <- .mc_format_scalar_or_range(x$n_complete_min, x$n_complete_max, digits = digits, integer = TRUE)
  if (!is.null(n_complete_txt)) {
    items <- c(items, n_complete = n_complete_txt)
  }
  skipped_n_txt <- .mc_format_scalar_or_range(x$skipped_n_min, x$skipped_n_max, digits = digits, integer = TRUE)
  if (!is.null(skipped_n_txt) && !(identical(skipped_n_txt, "0"))) {
    items <- c(items, skipped_n = skipped_n_txt)
  }
  skipped_prop_txt <- .mc_format_scalar_or_range(x$skipped_prop_min, x$skipped_prop_max, digits = digits, integer = FALSE)
  if (!is.null(skipped_prop_txt) && !(identical(skipped_prop_txt, formatC(0, format = "f", digits = digits)))) {
    items <- c(items, skipped_prop = skipped_prop_txt)
  }
  if (!is.null(x$correct) &&
      length(x$correct) == 1L &&
      !is.na(x$correct) &&
      is.finite(x$correct) &&
      !isTRUE(all.equal(x$correct, 0))) {
    items <- c(items, correct = formatC(x$correct, format = "f", digits = digits))
  }
  if (is.finite(x$threshold_sets) && x$threshold_sets > 0L) {
    items <- c(items, thresholds = sprintf("%s set(s)", .mc_count_fmt(x$threshold_sets)))
  }
  if (is.finite(x$zero_cell_pairs) && x$zero_cell_pairs > 0L) {
    items <- c(items, zero_cells = sprintf("%s pair(s)", .mc_count_fmt(x$zero_cell_pairs)))
  }
  if (is.finite(x$corrected_pairs) && x$corrected_pairs > 0L) {
    items <- c(items, corrected = sprintf("%s pair(s)", .mc_count_fmt(x$corrected_pairs)))
  }
  if (is.finite(x$boundary_pairs) && x$boundary_pairs > 0L) {
    items <- c(items, boundary = sprintf("%s pair(s)", .mc_count_fmt(x$boundary_pairs)))
  }
  if (is.finite(x$near_boundary_pairs) && x$near_boundary_pairs > 0L) {
    items <- c(items, near_bound = sprintf("%s pair(s)", .mc_count_fmt(x$near_boundary_pairs)))
  }
  if (is.finite(x$converged_pairs) && x$converged_pairs > 0L) {
    items <- c(items, converged = sprintf("%s pair(s)", .mc_count_fmt(x$converged_pairs)))
  }
  if (is.finite(x$optimizer_tol)) {
    items <- c(items, opt_tol = format(signif(x$optimizer_tol, digits = digits)))
  }
  est_range <- .mc_format_scalar_or_range(x$estimate_min, x$estimate_max, digits = digits, integer = FALSE)
  if (!is.null(est_range)) {
    items <- c(items, estimate = est_range)
  }
  if (!is.null(x$most_negative) && nzchar(x$most_negative)) {
    items <- c(items, most_negative = x$most_negative)
  }
  if (!is.null(x$most_positive) && nzchar(x$most_positive)) {
    items <- c(items, most_positive = x$most_positive)
  }
  items
}

.mc_print_pairwise_summary_digest <- function(x,
                                              title,
                                              digits = 4,
                                              n = NULL,
                                              topn = NULL,
                                              max_vars = NULL,
                                              width = NULL,
                                              show_ci = NULL,
                                              ci_method = NULL,
                                              extra_items = NULL,
                                              ...) {
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci
  )
  digits <- .mc_coalesce(attr(x, "digits"), digits)
  overview <- attr(x, "overview", exact = TRUE)

  digest <- if (inherits(overview, "summary.matrixCorr")) {
    .mc_corr_summary_digest_items(overview, digits = digits, show_ci = cfg$show_ci)
  } else {
    setNames(character(), character())
  }
  if (identical(cfg$show_ci, "yes") && isTRUE(attr(x, "has_ci"))) {
    digest <- c(
      digest,
      .mc_ci_digest(
        as.data.frame(x, stringsAsFactors = FALSE),
        conf_level = suppressWarnings(as.numeric(attr(x, "conf.level"))),
        ci_method = ci_method,
        digits = .mc_coalesce(attr(x, "ci_digits"), 3)
      )
    )
  }
  if (length(extra_items)) {
    digest <- c(digest, extra_items)
  }
  if (length(digest)) {
    digest <- digest[!duplicated(names(digest), fromLast = TRUE)]
  }
  digest <- digest[!is.na(digest) & nzchar(digest) & !(names(digest) == "multiplicity" & digest == "none")]

  .mc_print_named_digest(digest, header = title)
  cat("\n")
  table_x <- .mc_reorder_summary_columns(
    .mc_standardize_summary_counts(
      .mc_standardize_summary_pairs(as.data.frame(x, stringsAsFactors = FALSE)),
      repeated = any(c("n_subjects", "n_obs") %in% names(x))
    ),
    repeated = any(c("n_subjects", "n_obs") %in% names(x))
  )
  if (nrow(table_x) && "estimate" %in% names(table_x)) {
    table_x <- table_x[order(abs(table_x$estimate), decreasing = TRUE, na.last = TRUE), , drop = FALSE]
  }
  table_x <- .mc_compact_corr_summary_table(
    table_x,
    show_ci = cfg$show_ci,
    conf_level = suppressWarnings(as.numeric(attr(x, "conf.level"))),
    digits = digits,
    ci_digits = .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)
  )
  table_x <- .mc_format_summary_numeric_table(
    table_x,
    digits = digits,
    ci_digits = .mc_coalesce(attr(x, "ci_digits", exact = TRUE), digits)
  )
  .mc_print_preview_table(
    table_x,
    n = cfg$n,
    topn = cfg$topn,
    max_vars = cfg$max_vars,
    width = cfg$width,
    context = "summary",
    full_hint = TRUE,
    summary_hint = FALSE,
    ...
  )
  invisible(x)
}

.mc_print_overview_if_present <- function(x, ...) {
  overview <- attr(x, "overview", exact = TRUE)
  show_overview <- attr(x, "show_overview", exact = TRUE)
  if (isTRUE(show_overview) || is.null(show_overview)) {
    if (!is.null(overview)) {
      class(overview) <- unique(c(class(overview), "summary.matrixCorr"))
      print.summary.matrixCorr(overview, ...)
    }
  }
  invisible(NULL)
}

.mc_print_summary_table <- function(x,
                                    header,
                                    digits = NULL,
                                    n = NULL,
                                    topn = NULL,
                                    max_vars = NULL,
                                    width = NULL,
                                    show_ci = NULL,
                                    print_overview = TRUE,
                                    ...) {
  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )

  if (isTRUE(print_overview)) {
    do.call(
      .mc_print_overview_if_present,
      c(
        list(
          x = x,
          digits = if (is.null(digits)) 4 else digits,
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          show_ci = cfg$show_ci
        ),
        legacy$dots
      )
    )
  }

  cat("\n", header, "\n\n", sep = "")
  table_x <- as.data.frame(x, stringsAsFactors = FALSE)
  if (identical(cfg$show_ci, "no")) {
    ci_cols <- .mc_ci_column_index(table_x)
    if (length(ci_cols)) {
      table_x <- table_x[, -ci_cols, drop = FALSE]
    }
  }
  do.call(
    .mc_print_preview_table,
    c(
      list(
        df = table_x,
        n = cfg$n,
        topn = cfg$topn,
        max_vars = cfg$max_vars,
        width = cfg$width,
        context = "summary",
        full_hint = TRUE,
        summary_hint = FALSE
      ),
      legacy$dots
    )
  )
  invisible(x)
}

.mc_print_sectioned_table <- function(x,
                                      sections,
                                      header,
                                      n = NULL,
                                      topn = NULL,
                                      max_vars = NULL,
                                      width = NULL,
                                      show_ci = NULL,
                                      ...) {
  legacy <- .mc_extract_legacy_display_args(
    list(...),
    n = n,
    topn = topn,
    max_vars = max_vars
  )
  cfg <- .mc_resolve_display_args(
    context = "summary",
    n = legacy$n,
    topn = legacy$topn,
    max_vars = legacy$max_vars,
    width = width,
    show_ci = show_ci
  )
  table_x <- as.data.frame(x, stringsAsFactors = FALSE)
  ci_cols <- .mc_ci_column_index(table_x)

  if (!is.null(header)) {
    cat("\n", header, "\n\n", sep = "")
  }

  printed <- 0L
  truncated_any <- FALSE
  for (section in sections) {
    cols <- intersect(section$cols, names(table_x))
    if (identical(cfg$show_ci, "no") && length(ci_cols)) {
      cols <- setdiff(cols, names(table_x)[ci_cols])
    }
    if (!length(cols)) next
    if (printed > 0L) cat("\n")
    cat(section$title, "\n\n", sep = "")
    preview <- do.call(
      .mc_print_preview_table,
      c(
        list(
          df = table_x[, cols, drop = FALSE],
          n = cfg$n,
          topn = cfg$topn,
          max_vars = cfg$max_vars,
          width = cfg$width,
          context = "summary",
          full_hint = FALSE,
          summary_hint = FALSE
        ),
        legacy$dots
      )
    )
    truncated_any <- truncated_any ||
      isTRUE(preview$row_info$omitted > 0L) ||
      isTRUE(preview$col_info$omitted > 0L)
    printed <- printed + 1L
  }
  if (printed > 0L && truncated_any) {
    cat("Use as.data.frame()/tidy()/as.matrix() to inspect the full result.\n")
  }
  invisible(x)
}

.mc_format_pair_table <- function(m,
                                  digits = 4,
                                  sort_abs = TRUE) {
  m <- as.matrix(m)
  rn <- .mc_coalesce(rownames(m), as.character(seq_len(nrow(m))))
  cn <- .mc_coalesce(colnames(m), as.character(seq_len(ncol(m))))
  symmetric <- isTRUE(nrow(m) == ncol(m)) && isTRUE(isSymmetric(unclass(m)))
  selector <- if (symmetric && nrow(m) > 1L) {
    upper.tri(m, diag = FALSE)
  } else {
    matrix(TRUE, nrow(m), ncol(m))
  }
  idx <- which(selector, arr.ind = TRUE)
  out <- data.frame(
    item1 = rn[idx[, 1L]],
    item2 = cn[idx[, 2L]],
    estimate = as.numeric(m[idx]),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (sort_abs && nrow(out)) {
    out <- out[order(abs(out$estimate), decreasing = TRUE), , drop = FALSE]
    rownames(out) <- NULL
  }
  out$estimate <- round(out$estimate, digits)
  out
}

.mc_summary_top_pairs <- function(object,
                                  digits = 4,
                                  topn = 5L) {
  topn <- .mc_validate_optional_count(topn, arg = "topn", allow_null = FALSE)
  tbl <- .mc_format_pair_table(as.matrix(object), digits = digits, sort_abs = TRUE)
  utils::head(tbl, topn)
}

.mc_has_ci_payload <- function(x) {
  ci <- attr(x, "ci", exact = TRUE)
  if (is.list(ci)) return(TRUE)
  is.list(x) && all(c("est", "lwr.ci", "upr.ci") %in% names(x))
}

.mc_has_p_payload <- function(x) {
  inf <- attr(x, "inference", exact = TRUE)
  if (is.list(inf) && !is.null(inf$p_value)) return(TRUE)
  diag_attr <- attr(x, "diagnostics", exact = TRUE)
  is.list(diag_attr) && any(c("p_value", "p_value_adjusted") %in% names(diag_attr))
}

#' @title Repeated-Measures Lin's Concordance Correlation Coefficient (CCC)
#'
#' @description
#' Computes all pairwise Lin's Concordance Correlation Coefficients (CCC)
#' across multiple methods (L \eqn{\geq} 2) for repeated-measures data.
#' Each subject must be measured by all methods across the same set of time
#' points or replicates.
#'
#' CCC measures both accuracy (how close measurements are to the line of
#' equality) and precision (Pearson correlation). Confidence intervals are
#' optionally computed using a U-statistics-based estimator with Fisher's Z
#' transformation
#'
#' @param data A data frame containing the repeated-measures dataset.
#' @param response Character. Name of the numeric outcome column.
#' @param method Character. Name of the method column (factor with L
#' \eqn{\geq} 2 levels).
#' @param subject Character. Column identifying subjects. Every subject must
#'   have measurements from all methods (and times, when supplied); rows with
#'   incomplete \{subject, time, method\} coverage are dropped per pair.
#' @param time Character or NULL. Name of the time/repetition column. If NULL,
#' one time point is assumed.
#' @param Dmat Optional numeric weight matrix (T \eqn{\times} T) for
#' timepoints. Defaults to identity.
#' @param delta Numeric. Exponent applied to the absolute pointwise differences
#'   between two method trajectories before the time-weighted quadratic form is
#'   evaluated. Internally, the function forms
#'   \eqn{v_t = |X_t - Y_t|^\delta} and then computes
#'   \eqn{v^\top D v}, where \eqn{D} is \code{Dmat}.
#'   Therefore, when \code{Dmat} is diagonal:
#'   \itemize{
#'     \item \code{delta = 1} (default) gives the standard quadratic
#'       repeated-measures CCC distance,
#'       \eqn{(X - Y)^\top D (X - Y)}.
#'     \item \code{delta = 2} gives a fourth-power loss,
#'       which puts stronger emphasis on large disagreements.
#'     \item \code{delta = 0} gives a binary disagreement indicator before
#'       aggregation, analogous to a repeated-measures kappa-type distance.
#'   }
#'   In most applications, \code{delta = 1} should be used because it matches
#'   the usual quadratic distance used in repeated-measures CCC.
#
#' @param ci Logical. If TRUE, returns confidence intervals (default FALSE).
#' @param conf_level Confidence level for CI (default 0.95).
#' @param n_threads Integer (\eqn{\geq} 1). Number of OpenMP threads to use for computation.
#'   Defaults to \code{getOption("matrixCorr.threads", 1L)}.
#' @param verbose Logical. If TRUE, prints diagnostic output (default FALSE).
#'
#' @return
#' If \code{ci = FALSE}, a symmetric matrix of class \code{"ccc"} (estimates only).
#' If \code{ci = TRUE}, a list of class \code{"ccc"}, \code{"ccc_ci"} with elements:
#' \itemize{
#'   \item \code{est}: CCC estimate matrix
#'   \item \code{lwr.ci}: Lower bound matrix
#'   \item \code{upr.ci}: Upper bound matrix
#' }
#'
#' @details
#' This function computes pairwise Lin's Concordance Correlation Coefficient
#' (CCC) between methods in a repeated-measures design using a
#' U-statistics-based nonparametric estimator proposed by
#' Carrasco et al. (2013). It is computationally efficient and robust,
#' particularly for large-scale or balanced longitudinal designs.
#'
#' Lin's CCC is defined as
#' \deqn{
#' \rho_c = \frac{2 \cdot \mathrm{cov}(X, Y)}{\sigma_X^2 + \sigma_Y^2 +
#' (\mu_X - \mu_Y)^2}
#' }{
#' CCC = 2 * cov(X, Y) / [var(X) + var(Y) + (mean(X) - mean(Y))^2]
#' }
#' where:
#' \itemize{
#'   \item \eqn{X} and \eqn{Y} are paired measurements from two methods.
#'   \item \eqn{\mu_X}, \eqn{\mu_Y} are means, and \eqn{\sigma_X^2},
#'   \eqn{\sigma_Y^2} are variances.
#' }
#'
#' ## U-statistics Estimation
#' For repeated measures across \eqn{T} time points and \eqn{n} subjects we
#' assume
#' \itemize{
#'   \item all \eqn{n(n-1)} pairs of subjects are considered to compute a
#'   U-statistic estimator for within-method and cross-method distances.
#'   \item if `delta > 0`, pairwise distances are raised to a power before
#'   applying a time-weighted kernel matrix \eqn{D}.
#'   \item if `delta = 0`, the method reduces to a version similar to a
#'   repeated-measures kappa.
#' }
#'
#' ## Confidence Intervals
#' Confidence intervals are constructed using a **Fisher Z-transformation**
#' of the CCC. Specifically,
#' \itemize{
#'   \item The CCC is transformed using
#'   \eqn{Z = 0.5 \log((1 + \rho_c) / (1 - \rho_c))}.
#'   \item Standard errors are computed from the asymptotic variance of the
#'   U-statistic.
#'   \item Normal-based intervals are computed on the Z-scale and then
#'   back-transformed to the CCC scale.
#' }
#'
#' ## Assumptions
#' \itemize{
#'   \item The design must be **balanced**, where all subjects must have
#'   complete observations for all methods and time points.
#'   \item The method is **nonparametric** and does not require assumptions
#'   of normality or linear mixed effects.
#'   \item Weights (`Dmat`) allow differential importance of time points.
#' }
#'
#' For **unbalanced** or **complex hierarchical** data (e.g.,
#' missing timepoints, covariate adjustments), consider using
#' \code{\link{ccc_rm_reml}}, which uses a variance components approach
#' via linear mixed models.
#'
#' @references
#' Lin L (1989). A concordance correlation coefficient to evaluate reproducibility.
#' \emph{Biometrics}, 45: 255-268.
#'
#' Lin L (2000). A note on the concordance correlation coefficient.
#' \emph{Biometrics}, 56: 324-325.
#'
#' Carrasco JL, Jover L (2003). Estimating the concordance correlation coefficient:
#' a new approach. \emph{Computational Statistics & Data Analysis}, 47(4): 519-539.
#'
#' @seealso \code{\link{ccc}}, \code{\link{ccc_rm_reml}},
#' \code{\link{plot.ccc}}, \code{\link{print.ccc}}
#'
#' @examples
#' set.seed(123)
#' df <- expand.grid(subject = 1:10,
#'                   time = 1:2,
#'                   method = c("A", "B", "C"))
#' df$y <- rnorm(nrow(df), mean = match(df$method, c("A", "B", "C")), sd = 1)
#'
#' # CCC matrix (no CIs)
#' ccc1 <- ccc_rm_ustat(df, response = "y", subject = "subject",
#'                             method = "method", time = "time")
#' print(ccc1)
#' summary(ccc1)
#' plot(ccc1)
#'
#' # With confidence intervals
#' ccc2 <- ccc_rm_ustat(df, response = "y", subject = "subject",
#'                             method = "method", time = "time", ci = TRUE)
#' print(ccc2)
#' summary(ccc2)
#' estimate(ccc2)
#' tidy(ccc2)
#' ci(ccc2)
#' confint(ccc2)
#' plot(ccc2)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(ccc2)
#' }
#'
#' #------------------------------------------------------------------------
#' # Choosing delta based on distance sensitivity
#' #------------------------------------------------------------------------
#' # Standard quadratic RM-CCC distance: (X - Y)' D (X - Y)
#' ccc_rm_ustat(df, response = "y", subject = "subject",
#'              method = "method", time = "time", delta = 1)
#'
#' # Fourth-power loss when D is diagonal: emphasises large disagreements
#' ccc_rm_ustat(df, response = "y", subject = "subject",
#'              method = "method", time = "time", delta = 2)
#' # Binary disagreement indicator before aggregation
#' ccc_rm_ustat(df, response = "y", subject = "subject",
#'              method = "method", time = "time", delta = 0)
#' @author Thiago de Paula Oliveira
#' @export
ccc_rm_ustat <- function(data,
                        response,
                        subject,
                        method,
                        time = NULL,
                        ci = FALSE,
                        conf_level = 0.95,
                        n_threads = getOption("matrixCorr.threads", 1L),
                        verbose = FALSE,
                        Dmat = NULL,
                        delta = 1) {
  df <- as.data.frame(data)

  req_cols <- c(response, subject, method)
  if (!is.null(time)) req_cols <- c(req_cols, time)
  check_required_cols(df, req_cols, df_arg = "data")

  if (!is.numeric(df[[response]])) {
    abort_bad_arg("response",
      message = "must reference a numeric column in {.arg data}."
    )
  }
  df[[subject]] <- droplevels(factor(df[[subject]]))
  df[[method]]  <- droplevels(factor(df[[method]]))
  method_levels <- levels(df[[method]])
  L <- length(method_levels)

  if (L < 2) {
    abort_bad_arg("method",
      message = "must have at least two levels."
    )
  }

  if (!is.null(time)) {
    df[[time]] <- droplevels(factor(df[[time]]))
    df$time_i <- as.integer(df[[time]]) - 1L
    ntime <- length(levels(df[[time]]))
  } else {
    df$time_i <- 0L
    ntime <- 1L
  }
  df$subject_i <- as.integer(df[[subject]]) - 1L

  check_scalar_nonneg(delta, arg = "delta", strict = FALSE)
  check_bool(ci, arg = "ci")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  check_bool(verbose, arg = "verbose")
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  if (is.null(Dmat)) {
    Dmat <- diag(ntime)
  } else {
    Dmat <- as.matrix(Dmat)
    if (!is.numeric(Dmat) || any(!is.finite(Dmat))) {
      abort_bad_arg("Dmat",
        message = "must be a finite numeric matrix."
      )
    }
    check_matrix_dims(Dmat, nrow = ntime, ncol = ntime, arg = "Dmat")
    check_symmetric_matrix(Dmat, arg = "Dmat")
  }

  cccr_est <- matrix(1, L, L, dimnames = list(method_levels, method_levels))
  cccr_lwr <- matrix(NA_real_, L, L, dimnames = list(method_levels, method_levels))
  cccr_upr <- matrix(NA_real_, L, L, dimnames = list(method_levels, method_levels))

  needed <- c(response, method, subject)
  if (!is.null(time)) needed <- c(needed, time)
  complete_mask <- stats::complete.cases(df[, needed, drop = FALSE])
  response_vec <- df[[response]]
  subject_vec <- df$subject_i
  time_vec <- df$time_i
  method_code <- as.integer(df[[method]])
  rows_by_method <- split(seq_len(nrow(df)), method_code)

  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  inform_if_verbose(
    "Using {get_omp_threads()} OpenMP thread{?s}.",
    .verbose = verbose
  )

  # Loop over all method pairs
  for (i in 1:(L - 1)) {
    for (j in (i + 1):L) {
      m1 <- method_levels[i]
      m2 <- method_levels[j]

      pair_rows <- c(rows_by_method[[i]], rows_by_method[[j]])
      pair_rows <- pair_rows[complete_mask[pair_rows]]
      if (!length(pair_rows)) {
        cli::cli_abort(
          c(
            "No complete observations for method pair {.val {m1}} vs {.val {m2}}.",
            "i" = "Ensure each subject has both methods observed at the same time points."
          )
        )
      }

      sid <- subject_vec[pair_rows]
      tim <- time_vec[pair_rows]
      met_i <- as.integer(method_code[pair_rows] == j)
      uid <- sort(unique(sid))
      if (!length(uid)) {
        cli::cli_abort(
          "No subjects available for method pair {.val {m1}} vs {.val {m2}}."
        )
      }

      sid_idx <- match(sid, uid)
      rows_by_subj <- tabulate(sid_idx, nbins = length(uid))
      combo_code <- sid_idx + length(uid) * (tim + ntime * met_i)
      uniq_combo <- !duplicated(combo_code)
      uniq_by_subj <- tabulate(sid_idx[uniq_combo], nbins = length(uid))

      complete_subj <- (rows_by_subj == (ntime * 2L)) & (uniq_by_subj == (ntime * 2L))
      ns_complete <- sum(complete_subj)
      if (!ns_complete) {
        cli::cli_abort(
          "All subjects have incomplete method/time coverage for {.val {m1}} vs {.val {m2}}."
        )
      }

      if (ns_complete < 2L) {
        cli::cli_abort(
          c(
            "At least two subjects with complete observations are required for {.val {m1}} vs {.val {m2}}.",
            "i" = "Provide additional subjects or relax filtering so each retained subject has both methods at all time points."
          )
        )
      }

      keep_mask <- complete_subj[sid_idx]
      pair_rows <- pair_rows[keep_mask]
      tim <- tim[keep_mask]
      met_i <- met_i[keep_mask]
      complete_map <- cumsum(complete_subj) - 1L
      subject_pair <- complete_map[sid_idx[keep_mask]]
      ns <- ns_complete

      res <- cccUst_rcpp(response_vec[pair_rows],
                         met_i,
                         tim,
                         subject_pair,
                         0, 1,
                         ntime, ns,
                         Dmat, delta,
                         conf_level)

      cccr_est[i, j] <- cccr_est[j, i] <- res$CCC
      cccr_lwr[i, j] <- cccr_lwr[j, i] <- res$LL_CI
      cccr_upr[i, j] <- cccr_upr[j, i] <- res$UL_CI
    }
  }

  if (ci) {
    result <- structure(list(est = cccr_est, lwr.ci = cccr_lwr, upr.ci = cccr_upr),
                        class = c("ccc", "ccc_ci"))
  } else {
    result <- structure(cccr_est, class = c("ccc", "matrix"))
  }
  attr(result, "method") <- "Repeated-measures Lin's concordance"
  attr(result, "description") <- if (ci) {
    "Repeated-measures CCC (pairwise) with confidence intervals"
  } else {
    "Repeated-measures CCC (pairwise matrix)"
  }
  attr(result, "package") <- "matrixCorr"
  if (ci) attr(result, "conf.level") <- conf_level

  return(result)
}

#' @title Concordance Correlation via REML (Linear Mixed-Effects Model)
#'
#' @description
#' Compute Lin's Concordance Correlation Coefficient (CCC) from a linear
#' mixed-effects model fitted by REML. The fixed-effects part can include
#' \code{method} and/or \code{time} (optionally their interaction), with a
#' subject-specific random intercept to capture between-subject variation.
#' Large \eqn{n \times n} inversions are avoided by solving small per-subject
#' systems.
#'
#' \strong{Assumption:} time levels are treated as \emph{regular, equally spaced}
#' visits indexed by their order within subject. The AR(1) residual model is
#' in discrete time on the visit index (not calendar time). \code{NA} time codes
#' break the serial run. Gaps in the factor levels are \emph{ignored} (adjacent
#' observed visits are treated as lag-1).
#'
#' @param data A data frame.
#' @param response Character. Response variable name.
#' @param subject Character. Subject ID variable name.
#' @param method Character or \code{NULL}. Optional column name of method factor
#'   (added to fixed effects).
#' @param time Character or \code{NULL}. Optional column name of time factor
#'   (added to fixed effects).
#' @param interaction Logical. Include \code{method:time} interaction?
#'   (default \code{FALSE}).
#' @param max_iter Integer. Maximum iterations for variance-component updates
#'   (default \code{100}).
#' @param tol Numeric. Convergence tolerance on parameter change
#'   (default \code{1e-6}).
#'
#' @param Dmat Optional \eqn{n_t \times n_t} numeric matrix to weight/aggregate
#'   time-specific fixed biases in the \eqn{S_B} quadratic form. If supplied, it
#'   is used (after optional mass rescaling; see \code{Dmat_rescale}) whenever at
#'   least two \emph{present} time levels exist; otherwise it is ignored. \strong{If
#'   \code{Dmat} is \code{NULL}}, a canonical kernel \eqn{D_m} is \emph{constructed}
#'   from \code{Dmat_type} and \code{Dmat_weights} (see below). \code{Dmat} should be
#'   symmetric positive semidefinite; small asymmetries are symmetrized internally.
#'
#' @param Dmat_type Character, one of \code{c("time-avg","typical-visit",
#'   "weighted-avg","weighted-sq")}. Only used when \code{Dmat = NULL}.
#'   It selects the aggregation target for time-specific fixed biases in
#'   \eqn{S_B}. Options are:
#'
#'   \itemize{
#'     \item \code{"time-avg"}: square of the time-averaged bias, \eqn{D_m=(1/n_t)\,11^\top}.
#'     \item \code{"typical-visit"}: average of squared per-time biases, \eqn{D_m=I_{n_t}}.
#'     \item \code{"weighted-avg"}: square of a weighted average, \eqn{D_m=n_t\,w\,w^\top} with \eqn{\sum w=1}.
#'     \item \code{"weighted-sq"}: weighted average of squared biases, \eqn{D_m=n_t\,\mathrm{diag}(w)} with \eqn{\sum w=1}.
#'   }
#'   Pick \code{"time-avg"} for CCC targeting the time-averaged measurement; pick
#'   \code{"typical-visit"} for CCC targeting a randomly sampled visit (typical occasion).
#'   Default \code{"time-avg"}.
#'
#' @param Dmat_weights Optional numeric weights \eqn{w} used when
#'   \code{Dmat_type \%in\% c("weighted-avg","weighted-sq")}. Must be nonnegative and
#'   finite. If \code{names(w)} are provided, they should match the \emph{full} time
#'   levels in \code{data}; they are aligned to the \emph{present} time subset per fit.
#'   If unnamed, the length must equal the number of present time levels. In all cases
#'   \eqn{w} is internally normalized to sum to 1.
#'
#' @param Dmat_rescale Logical. When \code{TRUE} (default), the supplied/built
#'   \eqn{D_m} is rescaled to satisfy the simple mass rule
#'   \eqn{1^\top D_m 1 = n_t}. This keeps the \eqn{S_B} denominator invariant and
#'   harmonizes with the \eqn{\kappa}-shrinkage used for variance terms.
#'
#' @param ci Logical. If \code{TRUE}, return a CI container; limits are computed
#'   by a large-sample delta method for CCC (see \strong{CIs} note below).
#' @param conf_level Numeric in \eqn{(0,1)}. Confidence level when
#'   \code{ci = TRUE} (default \code{0.95}).
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads to use for
#'   computation. Defaults to \code{getOption("matrixCorr.threads", 1L)}.
#' @param ci_mode Character scalar; one of \code{c("auto","raw","logit")}.
#'   Controls how confidence intervals are computed when \code{ci = TRUE}.
#'   If \code{"raw"}, a Wald CI is formed on the CCC scale and truncated to
#'   \code{[0,1]}. If \code{"logit"}, a Wald CI is computed on the
#'   \eqn{\mathrm{logit}(\mathrm{CCC})} scale and back-transformed to the
#'   original scale (often more stable near 0 or 1). If \code{"auto"}
#'   (default), the method is chosen per estimate based on simple diagnostics
#'   (e.g., proximity to the \code{[0,1]} boundary / numerical stability),
#'   typically preferring \code{"logit"} near the boundaries and \code{"raw"}
#'   otherwise.
#' @param verbose Logical. If \code{TRUE}, prints a structured summary of the
#'   fitted variance components and \eqn{S_B} for each fit. Default \code{FALSE}.
#' @param digits Integer \eqn{(\ge 0)}. Number of decimal places to use in the
#'   printed summary when \code{verbose = TRUE}. Default \code{4}.
#' @param use_message Logical. When \code{verbose = TRUE}, verbose summaries
#'   are emitted with \pkg{cli} messages.
#'
#' @param ar Character. Residual correlation structure: \code{"none"} (iid) or
#'   \code{"ar1"} for subject-level AR(1) correlation within contiguous time
#'   runs. Default \code{c("none")}.
#' @param ar_rho Numeric in \eqn{(-0.999,\,0.999)} or \code{NA}.
#'   If \code{ar = "ar1"} and \code{ar_rho} is finite, it is treated as fixed.
#'   If \code{ar = "ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
#'   profiling a 1-D objective (REML when available; an approximation otherwise).
#'   Default \code{NA_real_}.
#' @param slope Character. Optional extra random-effect design \eqn{Z}.
#'   With \code{"subject"} a single random slope is added (one column in \eqn{Z});
#'   with \code{"method"} one column per method level is added; with
#'   \code{"custom"} you provide \code{slope_Z} directly. Default
#'   \code{c("none","subject","method","custom")}.
#' @param slope_var For \code{slope \%in\% c("subject","method")}, a character
#'   string giving the name of a column in \code{data} used as the slope regressor
#'   (e.g., centered time). It is looked up inside \code{data}; do not
#'   pass the vector itself. NAs are treated as zeros in \eqn{Z}.
#' @param slope_Z For \code{slope = "custom"}, a numeric matrix with \eqn{n}
#'   rows (same order as \code{data}) providing the full extra random-effect
#'   design \eqn{Z}. \strong{Each column of \code{slope_Z} has its own variance
#'   component} \eqn{\sigma_{Z,j}^2}; columns are treated as \emph{uncorrelated}
#'   (diagonal block in \eqn{G}). Ignored otherwise.
#' @param drop_zero_cols Logical. When \code{slope = "method"}, drop all-zero
#'   columns of \eqn{Z} after subsetting (useful in pairwise fits). Default
#'   \code{TRUE}.
#'
#' @param vc_select Character scalar; one of \code{c("auto","none")}.
#'   Controls how the subject by method \eqn{\sigma^2_{A\times M}} ("subj_method") and
#'   subject by time \eqn{\sigma^2_{A\times T}} ("subj_time") variance components are
#'   included. If \code{"auto"} (default), the function performs boundary-aware
#'   REML likelihood-ratio tests (LRTs; null on the boundary at zero with a
#'   half-\eqn{\chi^2_1} reference) to decide whether to retain each component,
#'   in the order given by \code{vc_test_order}. If \code{"none"}, no testing
#'   is done and inclusion is taken from \code{include_subj_method}/\code{include_subj_time}
#'   (or, if \code{NULL}, from the mere presence of the corresponding factor in
#'   the design).  In pairwise fits, the decision is made independently for each
#'   method pair.
#'
#' @param vc_alpha Numeric scalar in \eqn{(0,1)}; default \code{0.05}.
#'   Per-component significance level for the boundary-aware REML LRTs used when
#'   \code{vc_select = "auto"}. The tests are one-sided for variance components
#'   on the boundary and are *not* multiplicity-adjusted.
#'
#' @param vc_test_order Character vector (length 2) with a permutation of
#'   \code{c("subj_time","subj_method")}; default \code{c("subj_time","subj_method")}. Specifies the order
#'   in which the two variance components are tested when \code{vc_select = "auto"}.
#'   The component tested first may be dropped before testing the second. If a
#'   factor is absent in the design (e.g., no time factor so "subj_time" is undefined),
#'   the corresponding test is skipped.
#'
#' @param include_subj_method,include_subj_time Logical scalars or \code{NULL}.
#'   When \code{vc_select = "none"}, these control whether the
#'   \eqn{\sigma^2_{A\times M}} ("subj_method") and \eqn{\sigma^2_{A\times T}} ("subj_time")
#'   random effects are included (\code{TRUE}) or excluded (\code{FALSE}) in the model.
#'   If \code{NULL} (default), inclusion defaults to the presence of the
#'   corresponding factor in the data (i.e., at least two method/time levels).
#'   When \code{vc_select = "auto"}, these arguments are ignored (automatic
#'   selection is used instead).
#'
#' @param sb_zero_tol Non-negative numeric scalar; default \code{1e-10}.
#'   Numerical threshold for the fixed-effect dispersion term \eqn{S_B}.
#'   After computing \eqn{\widehat{S_B}} and its delta-method variance, if
#'   \eqn{\widehat{S_B} \le} \code{sb_zero_tol} or non-finite, the procedure
#'   treats \eqn{S_B} as fixed at zero in the delta step. It sets
#'   \eqn{d_{S_B}=0} and \eqn{\mathrm{Var}(\widehat{S_B})=0}, preventing
#'   numerical blow-ups of \code{SE(CCC)} when \eqn{\widehat{S_B}\to 0} and the
#'   fixed-effects variance is ill-conditioned for the contrast. This stabilises
#'   inference in rare boundary cases; it has no effect when
#'   \eqn{\widehat{S_B}} is comfortably above the threshold.

#'
#' @details
#' For measurement \eqn{y_{ij}} on subject \eqn{i} under fixed
#' levels (method, time), we fit
#' \deqn{ y = X\beta + Zu + \varepsilon,\qquad
#'        u \sim N(0,\,G),\ \varepsilon \sim N(0,\,R). }
#' Notation: \eqn{m} subjects, \eqn{n=\sum_i n_i} total rows;
#' \eqn{nm} method levels; \eqn{nt} time levels; \eqn{q_Z} extra
#' random-slope columns (if any); \eqn{r=1+nm+nt} (or \eqn{1+nm+nt+q_Z} with slopes).
#' Here \eqn{Z} is the subject-structured random-effects design and \eqn{G} is
#' block-diagonal at the subject level with the following \emph{per-subject}
#' parameterisation. Specifically,
#' \itemize{
#'   \item one random intercept with variance \eqn{\sigma_A^2};
#'   \item optionally, \emph{method} deviations (one column per method level)
#'         with a common variance \eqn{\sigma_{A\times M}^2} and zero
#'         covariances across levels (i.e., multiple of an identity);
#'   \item optionally, \emph{time} deviations (one column per time level)
#'         with a common variance \eqn{\sigma_{A\times T}^2} and zero
#'         covariances across levels;
#'   \item optionally, an \emph{extra} random effect aligned with \eqn{Z}
#'         (random slope), where each \emph{column} has its own variance
#'         \eqn{\sigma_{Z,j}^2} and columns are uncorrelated.
#' }
#' The fixed-effects design is \code{~ 1 + method + time} and, if
#' \code{interaction=TRUE}, \code{+ method:time}.
#'
#' \strong{Residual correlation \eqn{R} (regular, equally spaced time).}
#' Write \eqn{R_i=\sigma_E^2\,C_i(\rho)}. With \code{ar="none"}, \eqn{C_i=I}.
#' With \code{ar="ar1"}, within-subject residuals follow a \emph{discrete} AR(1)
#' process along the visit index after sorting by increasing time level. Ties
#' retain input order, and any \code{NA} time code breaks the series so each
#' contiguous block of non-\code{NA} times forms a run. The correlation
#' between \emph{adjacent observed visits} in a run is \eqn{\rho}; we do not use
#' calendar-time gaps. Internally we work with the \emph{precision} of the AR(1)
#' correlation: for a run of length \eqn{L\ge 2}, the tridiagonal inverse has
#' \deqn{ (C^{-1})_{11}=(C^{-1})_{LL}=\frac{1}{1-\rho^2},\quad
#'        (C^{-1})_{tt}=\frac{1+\rho^2}{1-\rho^2}\ (2\le t\le L-1),\quad
#'        (C^{-1})_{t,t+1}=(C^{-1})_{t+1,t}=\frac{-\rho}{1-\rho^2}. }
#' The working inverse is \eqn{\,R_i^{-1}=\sigma_E^{-2}\,C_i(\rho)^{-1}}.
#'
#' \strong{Per-subject Woodbury system.} For subject \eqn{i} with \eqn{n_i}
#' rows, define the per-subject random-effects design \eqn{U_i} (columns:
#' intercept, method indicators, time indicators; dimension
#' \eqn{\,r=1+nm+nt\,}). The core never forms
#' \eqn{V_i = R_i + U_i G U_i^\top} explicitly. Instead,
#' \deqn{ M_i \;=\; G^{-1} \;+\; U_i^\top R_i^{-1} U_i, }
#' and accumulates GLS blocks via rank-\eqn{r} corrections using
#' \eqn{\,V_i^{-1} = R_i^{-1}-R_i^{-1}U_i M_i^{-1}U_i^\top R_i^{-1}\,}:
#' \deqn{ X^\top V^{-1} X \;=\; \sum_i \Big[
#'        X_i^\top R_i^{-1}X_i \;-\; (X_i^\top R_i^{-1}U_i)\,
#'        M_i^{-1}\,(U_i^\top R_i^{-1}X_i) \Big], }
#' \deqn{ X^\top V^{-1} y \;=\; \sum_i \Big[
#'        X_i^\top R_i^{-1}y_i \;-\; (X_i^\top R_i^{-1}U_i)\,M_i^{-1}\,
#'        (U_i^\top R_i^{-1}y_i) \Big]. }
#' Because \eqn{G^{-1}} is diagonal with positive entries, each \eqn{M_i} is
#' symmetric positive definite; solves/inversions use symmetric-PD routines with
#' a small diagonal ridge and a pseudo-inverse if needed.
#'
#' \strong{Random-slope \eqn{Z}.}
#' Besides \eqn{U_i}, the function can include an extra design \eqn{Z_i}.
#' \itemize{
#'   \item \code{slope="subject"}: \eqn{Z} has one column (the regressor in
#'         \code{slope_var}); \eqn{Z_{i}} is the subject-\eqn{i} block, with its own
#'         variance \eqn{\sigma_{Z,1}^2}.
#'   \item \code{slope="method"}: \eqn{Z} has one column per method level;
#'         row \eqn{t} uses the slope regressor if its method equals level \eqn{\ell},
#'         otherwise 0; all-zero columns can be dropped via
#'         \code{drop_zero_cols=TRUE} after subsetting. Each column has its own
#'         variance \eqn{\sigma_{Z,\ell}^2}.
#'   \item \code{slope="custom"}: \eqn{Z} is provided fully via \code{slope_Z}.
#'         Each column is an independent random effect with its own variance
#'         \eqn{\sigma_{Z,j}^2}; cross-covariances among columns are set to 0.
#' }
#' Computations simply augment \eqn{\tilde U_i=[U_i\ Z_i]} and the corresponding
#' inverse-variance block. The EM updates then include, for each column \eqn{j=1,\ldots,q_Z},
#' \deqn{ \sigma_{Z,j}^{2\,(new)} \;=\; \frac{1}{m}
#'       \sum_{i=1}^m \Big( b_{i,\text{extra},j}^2 +
#'       (M_i^{-1})_{\text{extra},jj} \Big)
#'       \quad (\text{if } q_Z>0). }
#' \emph{Interpretation:} the \eqn{\sigma_{Z,j}^2} represent additional within-subject
#' variability explained by the slope regressor(s) in column \eqn{j} and are \emph{not} part of the CCC
#' denominator (agreement across methods/time).
#'
#' \strong{EM-style variance-component updates.} With current \eqn{\hat\beta},
#' form residuals \eqn{r_i = y_i - X_i\hat\beta}. The BLUPs and conditional
#' covariances are
#' \deqn{ b_i \;=\; M_i^{-1}\,(U_i^\top R_i^{-1} r_i), \qquad
#'       \mathrm{Var}(b_i\mid y) \;=\; M_i^{-1}. }
#' Let \eqn{e_i=r_i-U_i b_i}. Expected squares then yield closed-form updates:
#' \deqn{ \sigma_A^{2\,(new)} \;=\; \frac{1}{m}\sum_i \Big( b_{i,0}^2 +
#' (M_i^{-1})_{00} \Big), }
#' \deqn{ \sigma_{A\times M}^{2\,(new)} \;=\; \frac{1}{m\,nm}
#'       \sum_i \sum_{\ell=1}^{nm}\!\Big( b_{i,\ell}^2 +
#'       (M_i^{-1})_{\ell\ell} \Big)
#'       \quad (\text{if } nm>0), }
#' \deqn{ \sigma_{A\times T}^{2\,(new)} \;=\; \frac{1}{m\,nt}
#'       \sum_i \sum_{t=1}^{nt}
#'       \Big( b_{i,t}^2 + (M_i^{-1})_{tt} \Big)
#'       \quad (\text{if } nt>0), }
#' \deqn{ \sigma_E^{2\,(new)} \;=\; \frac{1}{n} \sum_i
#'       \Big( e_i^\top C_i(\rho)^{-1} e_i \;+\;
#'       \mathrm{tr}\!\big(M_i^{-1}U_i^\top C_i(\rho)^{-1} U_i\big) \Big), }
#' together with the per-column update for \eqn{\sigma_{Z,j}^2} given above.
#' Iterate until the \eqn{\ell_1} change across components is \eqn{<}\code{tol}
#' or \code{max_iter} is reached.
#'
#' \strong{Fixed-effect dispersion \eqn{S_B}: choosing the time-kernel \eqn{D_m}.}
#'
#' Let \eqn{d = L^\top \hat\beta} stack the within-time, pairwise method differences,
#' grouped by time as \eqn{d=(d_1^\top,\ldots,d_{n_t}^\top)^\top} with
#' \eqn{d_t \in \mathbb{R}^{P}} and \eqn{P = n_m(n_m-1)}. The symmetric
#' positive semidefinite kernel \eqn{D_m \succeq 0} selects which functional of the
#' bias profile \eqn{t \mapsto d_t} is targeted by \eqn{S_B}. Internally, the code
#' rescales any supplied/built \eqn{D_m} to satisfy \eqn{1^\top D_m 1 = n_t} for
#' stability and comparability.
#'
#' \itemize{
#'   \item \code{Dmat_type = "time-avg"} (square of the time-averaged bias).
#'     Let \deqn{ w \;=\; \frac{1}{n_t}\,\mathbf{1}_{n_t}, \qquad
#'                D_m \;\propto\; I_P \otimes (w\,w^\top), }
#'     so that
#'     \deqn{ d^\top D_m d \;\propto\; \sum_{p=1}^{P}
#'            \left( \frac{1}{n_t}\sum_{t=1}^{n_t} d_{t,p} \right)^{\!2}. }
#'     Methods have equal \eqn{\textit{time-averaged}}
#'     means within subject, i.e. \eqn{\sum_{t=1}^{n_t} d_{t,p}/n_t = 0} for all
#'     \eqn{p}. Appropriate when decisions depend on an average over time and
#'     opposite-signed biases are allowed to cancel.
#'
#'   \item \code{Dmat_type = "typical-visit"} (average of squared per-time biases).
#'     With equal visit probability, take
#'     \deqn{ D_m \;\propto\; I_P \otimes
#'            \mathrm{diag}\!\Big(\tfrac{1}{n_t},\ldots,\tfrac{1}{n_t}\Big), }
#'     yielding
#'     \deqn{ d^\top D_m d \;\propto\; \frac{1}{n_t}
#'            \sum_{t=1}^{n_t}\sum_{p=1}^{P} d_{t,p}^{\,2}. }
#'     Methods agree on a \eqn{\textit{typical}}
#'     occasion drawn uniformly from the visit set. Use when each visit matters
#'     on its own; alternating signs \eqn{d_{t,p}} do not cancel.
#'
#'   \item \code{Dmat_type = "weighted-avg"} (square of a weighted time average).
#'     For user weights \eqn{a=(a_1,\ldots,a_{n_t})^\top} with \eqn{a_t \ge 0}, set
#'     \deqn{ w \;=\; \frac{a}{\sum_{t=1}^{n_t} a_t}, \qquad
#'            D_m \;\propto\; I_P \otimes (w\,w^\top), }
#'     so that
#'     \deqn{ d^\top D_m d \;\propto\; \sum_{p=1}^{P}
#'            \left( \sum_{t=1}^{n_t} w_t\, d_{t,p} \right)^{\!2}. }
#'     Methods have equal \eqn{\textit{weighted
#'     time-averaged}} means, i.e. \eqn{\sum_{t=1}^{n_t} w_t\, d_{t,p} = 0} for all
#'     \eqn{p}. Use when some visits (e.g., baseline/harvest) are a priori more
#'     influential; opposite-signed biases may cancel according to \eqn{w}.
#'
#'   \item \code{Dmat_type = "weighted-sq"} (weighted average of squared per-time biases).
#'     With the same weights \eqn{w}, take
#'     \deqn{ D_m \;\propto\; I_P \otimes \mathrm{diag}(w_1,\ldots,w_{n_t}), }
#'     giving
#'     \deqn{ d^\top D_m d \;\propto\; \sum_{t=1}^{n_t} w_t
#'            \sum_{p=1}^{P} d_{t,p}^{\,2}. }
#'     Methods agree at visits sampled with
#'     probabilities \eqn{\{w_t\}}, counting each visit's discrepancy on its own.
#'     Use when per-visit agreement is required but some visits should be
#'     emphasised more than others.
#' }
#'
#' \strong{Time-averaging for CCC (regular visits).}
#' The reported CCC targets agreement of the \emph{time-averaged} measurements
#' per method within subject by default (\code{Dmat_type="time-avg"}). Averaging over \eqn{T}
#' non-\code{NA} visits shrinks time-varying components by
#' \deqn{ \kappa_T^{(g)} \;=\; 1/T, \qquad
#'       \kappa_T^{(e)} \;=\; \{T + 2\sum_{k=1}^{T-1}(T-k)\rho^k\}/T^2, }
#' with \eqn{\kappa_T^{(e)}=1/T} when residuals are i.i.d. With unbalanced \eqn{T}, the
#' implementation averages the per-(subject,method) \eqn{\kappa} values across the
#' pairs contributing to CCC and then clamps \eqn{\kappa_T^{(e)}} to
#' \eqn{[10^{-12},\,1]} for numerical stability. Choosing
#' \code{Dmat_type="typical-visit"} makes \eqn{S_B} match the interpretation of a
#' randomly sampled occasion instead.
#'
#' \strong{Concordance correlation coefficient.} The CCC used is
#' \deqn{ \mathrm{CCC} \;=\;
#'       \frac{\sigma_A^2 + \kappa_T^{(g)}\,\sigma_{A\times T}^2}
#'            {\sigma_A^2 + \sigma_{A\times M}^2 +
#'             \kappa_T^{(g)}\,\sigma_{A\times T}^2 + S_B +
#'             \kappa_T^{(e)}\,\sigma_E^2}. }
#' Special cases: with no method factor, \eqn{S_B=\sigma_{A\times M}^2=0}; with
#' a single time level, \eqn{\sigma_{A\times T}^2=0} (no \eqn{\kappa}-shrinkage).
#' When \eqn{T=1} or \eqn{\rho=0}, both \eqn{\kappa}-factors equal 1. The \emph{extra}
#' random-effect variances \eqn{\{\sigma_{Z,j}^2\}} (if used) are \emph{not} included.
#'
#' \strong{CIs / SEs (delta method for CCC).}
#' Let
#' \deqn{ \theta \;=\; \big(\sigma_A^2,\ \sigma_{A\times M}^2,\
#' \sigma_{A\times T}^2,\ \sigma_E^2,\ S_B\big)^\top, }
#' and write \eqn{\mathrm{CCC}(\theta)=N/D} with
#' \eqn{N=\sigma_A^2+\kappa_T^{(g)}\sigma_{A\times T}^2} and
#' \eqn{D=\sigma_A^2+\sigma_{A\times M}^2+\kappa_T^{(g)}\sigma_{A\times T}^2+S_B+\kappa_T^{(e)}\sigma_E^2}.
#' The gradient components are
#' \deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_A^2}
#'       \;=\; \frac{\sigma_{A\times M}^2 + S_B + \kappa_T^{(e)}\sigma_E^2}{D^2}, }
#' \deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_{A\times M}^2}
#'       \;=\; -\,\frac{N}{D^2}, \qquad
#'        \frac{\partial\,\mathrm{CCC}}{\partial \sigma_{A\times T}^2}
#'       \;=\; \frac{\kappa_T^{(g)}\big(\sigma_{A\times M}^2 + S_B +
#'                                     \kappa_T^{(e)}\sigma_E^2\big)}{D^2}, }
#' \deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_E^2}
#'       \;=\; -\,\frac{\kappa_T^{(e)}\,N}{D^2}, \qquad
#'        \frac{\partial\,\mathrm{CCC}}{\partial S_B}
#'       \;=\; -\,\frac{N}{D^2}. }
#'
#' \emph{Estimating \eqn{\mathrm{Var}(\hat\theta)}.}
#' The EM updates write each variance component as an average of per-subject
#' quantities. For subject \eqn{i},
#' \deqn{ t_{A,i} \;=\; b_{i,0}^2 + (M_i^{-1})_{00},\qquad
#'        t_{M,i} \;=\; \frac{1}{nm}\sum_{\ell=1}^{nm}
#'                        \Big(b_{i,\ell}^2 + (M_i^{-1})_{\ell\ell}\Big), }
#' \deqn{ t_{T,i} \;=\; \frac{1}{nt}\sum_{j=1}^{nt}
#'                        \Big(b_{i,j}^2 + (M_i^{-1})_{jj}\Big),\qquad
#'        s_i \;=\; \frac{e_i^\top C_i(\rho)^{-1} e_i +
#'        \mathrm{tr}\!\big(M_i^{-1}U_i^\top C_i(\rho)^{-1} U_i\big)}{n_i}, }
#' where \eqn{b_i = M_i^{-1}(U_i^\top R_i^{-1} r_i)} and
#' \eqn{M_i = G^{-1} + U_i^\top R_i^{-1} U_i}.
#' With \eqn{m} subjects, form the empirical covariance of the stacked
#' subject vectors and scale by \eqn{m} to approximate the covariance of the
#' means:
#' \deqn{ \widehat{\mathrm{Cov}}\!\left(
#'       \begin{bmatrix} t_{A,\cdot} \\ t_{M,\cdot} \\ t_{T,\cdot} \end{bmatrix}
#'       \right)
#'       \;\approx\; \frac{1}{m}\,
#'        \mathrm{Cov}_i\!\left(
#'       \begin{bmatrix} t_{A,i} \\ t_{M,i} \\ t_{T,i} \end{bmatrix}\right). }
#' (Drop rows/columns as needed when \code{nm==0} or \code{nt==0}.)
#'
#' The residual variance estimator is a weighted mean
#' \eqn{\hat\sigma_E^2=\sum_i w_i s_i} with \eqn{w_i=n_i/n}. Its variance is
#' approximated by the variance of a weighted mean of independent terms,
#' \deqn{ \widehat{\mathrm{Var}}(\hat\sigma_E^2)
#'       \;\approx\; \Big(\sum_i w_i^2\Big)\,\widehat{\mathrm{Var}}(s_i), }
#' where \eqn{\widehat{\mathrm{Var}}(s_i)} is the sample variance across
#' subjects. The method-dispersion term uses the quadratic-form delta already
#' computed for \eqn{S_B}:
#' \deqn{ \widehat{\mathrm{Var}}(S_B)
#'       \;=\; \frac{2\,\mathrm{tr}\!\big((A_{\!fix}\,\mathrm{Var}(\hat\beta))^2\big)
#'              \;+\; 4\,\hat\beta^\top A_{\!fix}\,\mathrm{Var}(\hat\beta)\,
#'              A_{\!fix}\,\hat\beta}
#'                    {\big[nm\,(nm-1)\,\max(nt,1)\big]^2}, }
#' with \eqn{A_{\!fix}=L\,D_m\,L^\top}.
#'
#' \emph{Putting it together.} Assemble
#' \eqn{\widehat{\mathrm{Var}}(\hat\theta)} by combining the
#' \eqn{(\sigma_A^2,\sigma_{A\times M}^2,\sigma_{A\times T}^2)} covariance
#' block from the subject-level empirical covariance, add the
#' \eqn{\widehat{\mathrm{Var}}(\hat\sigma_E^2)} and
#' \eqn{\widehat{\mathrm{Var}}(S_B)} terms on the diagonal,
#' and ignore cross-covariances across these blocks (a standard large-sample
#' simplification). Then
#' \deqn{ \widehat{\mathrm{se}}\{\widehat{\mathrm{CCC}}\}
#'       \;=\; \sqrt{\,\nabla \mathrm{CCC}(\hat\theta)^\top\,
#'                     \widehat{\mathrm{Var}}(\hat\theta)\,
#'                     \nabla \mathrm{CCC}(\hat\theta)\,}. }
#'
#' A two-sided \eqn{(1-\alpha)} normal CI is
#' \deqn{ \widehat{\mathrm{CCC}} \;\pm\; z_{1-\alpha/2}\,
#'       \widehat{\mathrm{se}}\{\widehat{\mathrm{CCC}}\}, }
#' truncated to \eqn{[0,1]} in the output for convenience. When \eqn{S_B} is
#' truncated at 0 or samples are very small/imbalanced, the normal CI can be
#' mildly anti-conservative near the boundary; a logit transform for CCC or a
#' subject-level (cluster) bootstrap can be used for sensitivity analysis.
#'
#' \strong{Choosing \eqn{\rho} for AR(1).}
#' When \code{ar="ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
#' profiling the REML log-likelihood at \eqn{(\hat\beta,\hat G,\hat\sigma_E^2)}.
#' With very few visits per subject, \eqn{\rho} can be weakly identified; consider
#' sensitivity checks over a plausible range.
#'
#' @section Notes on stability and performance:
#' All per-subject solves are \eqn{\,r\times r} with \eqn{r=1+nm+nt+q_Z}, so cost
#' scales with the number of subjects and the fixed-effects dimension rather
#' than the total number of observations. Solvers use symmetric-PD paths with
#' a small diagonal ridge and pseudo-inverse,
#' which helps for very small/unbalanced subsets and near-boundary estimates.
#' For \code{AR(1)}, observations are ordered by time within subject; \code{NA} time codes
#' break the run, and gaps between factor levels are treated as regular steps
#' (elapsed time is not used).
#'
#' \emph{Heteroscedastic slopes across \eqn{Z} columns are supported.}
#' Each \eqn{Z} column has its own variance component \eqn{\sigma_{Z,j}^2}, but
#' cross-covariances among \eqn{Z} columns are set to zero (diagonal block). Column
#' rescaling changes the implied prior on \eqn{b_{i,\text{extra}}} but does not
#' introduce correlations.
#'
#' @section Threading and BLAS guards:
#' The C++ backend uses OpenMP loops while also forcing vendor BLAS libraries to
#' run single-threaded so that overall CPU usage stays predictable. This guard
#' is applied to OpenBLAS, Apple's Accelerate, and Intel MKL when their runtime
#' controls are available. You can opt out manually by setting
#' \code{MATRIXCORR_DISABLE_BLAS_GUARD=1} in the environment before loading
#' \pkg{matrixCorr}.
#'
#' @seealso \code{build_L_Dm_Z_cpp}
#' for constructing \eqn{L}/\eqn{D_m}/\eqn{Z}; \code{\link{ccc_rm_ustat}}
#' for a U-statistic alternative; and \pkg{cccrm} for a reference approach via
#' \pkg{nlme}.
#'
#' @references
#' Lin L (1989). A concordance correlation coefficient to evaluate reproducibility.
#' \emph{Biometrics}, 45: 255-268.
#'
#' Lin L (2000). A note on the concordance correlation coefficient.
#' \emph{Biometrics}, 56: 324-325.
#'
#' Carrasco, J. L. et al. (2013). Estimation of the concordance
#' correlation coefficient for repeated measures using SAS and R.
#' \emph{Computer Methods and Programs in Biomedicine}, 109(3), 293-304.
#' \doi{10.1016/j.cmpb.2012.09.002}
#'
#' King et al. (2007). A Class of Repeated Measures Concordance
#' Correlation Coefficients.
#' \emph{Journal of Biopharmaceutical Statistics}, 17(4).
#' \doi{10.1080/10543400701329455}
#'
#' @examples
#' # ====================================================================
#' # 1) Subject x METHOD variance present, no time
#' #     y_{i,m} = mu + b_m + u_i + w_{i,m} + e_{i,m}
#' #     with u_i ~ N(0, s_A^2), w_{i,m} ~ N(0, s_{AxM}^2)
#' # ====================================================================
#' set.seed(102)
#' n_subj <- 60
#' n_time <- 8
#'
#' id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#' method <- factor(rep(rep(c("A","B"),    each  = n_time), times = n_subj))
#'
#' sigA  <- 0.6   # subject
#' sigAM <- 0.3   # subject x method
#' sigAT <- 0.5   # subject x time
#' sigE  <- 0.4   # residual
#' # Expected estimate S_B = 0.2^2 = 0.04
#' biasB <- 0.2   # fixed method bias
#'
#' # random effects
#' u_i <- rnorm(n_subj, 0, sqrt(sigA))
#' u   <- u_i[as.integer(id)]
#'
#' sm      <- interaction(id, method, drop = TRUE)
#' w_im_lv <- rnorm(nlevels(sm), 0, sqrt(sigAM))
#' w_im    <- w_im_lv[as.integer(sm)]
#'
#' st      <- interaction(id, time, drop = TRUE)
#' g_it_lv <- rnorm(nlevels(st), 0, sqrt(sigAT))
#' g_it    <- g_it_lv[as.integer(st)]
#'
#' # residuals & response
#' e <- rnorm(length(id), 0, sqrt(sigE))
#' y <- (method == "B") * biasB + u + w_im + g_it + e
#'
#' dat_both <- data.frame(y, id, method, time)
#'
#' # Both sigma2_subject_method and sigma2_subject_time are identifiable here
#' fit_both <- ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
#'                          vc_select = "auto", ci = TRUE, verbose = TRUE)
#' summary(fit_both)
#' estimate(fit_both)
#' tidy(fit_both)
#' ci(fit_both)
#' confint(fit_both)
#' plot(fit_both)
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(fit_both)
#' }
#'
#' # ====================================================================
#' # 2) Subject x TIME variance present (sag > 0) with two methods
#' #     y_{i,m,t} = mu + b_m + u_i + g_{i,t} + e_{i,m,t}
#' #     where g_{i,t} ~ N(0, s_{AxT}^2) shared across methods at time t
#' # ====================================================================
#' set.seed(202)
#' n_subj <- 60; n_time <- 14
#' id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
#' time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#'
#' sigA  <- 0.7
#' sigAT <- 0.5
#' sigE  <- 0.5
#' biasB <- 0.25
#'
#' u   <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
#' gIT <- rnorm(n_subj * n_time, 0, sqrt(sigAT))
#' g   <- gIT[ (as.integer(id) - 1L) * n_time + as.integer(time) ]
#' y   <- (method == "B") * biasB + u + g + rnorm(length(id), 0, sqrt(sigE))
#' dat_sag <- data.frame(y, id, method, time)
#'
#' # sigma_AT should be retained; sigma_AM may be dropped (since w_{i,m}=0)
#' fit_sag <- ccc_rm_reml(dat_sag, "y", "id", method = "method", time = "time",
#'                         vc_select = "auto", verbose = TRUE)
#' summary(fit_sag)
#' plot(fit_sag)
#'
#' # ====================================================================
#' # 3) BOTH components present: sab > 0 and sag > 0 (2 methods x T times)
#' #     y_{i,m,t} = mu + b_m + u_i + w_{i,m} + g_{i,t} + e_{i,m,t}
#' # ====================================================================
#' set.seed(303)
#' n_subj <- 60; n_time <- 4
#' id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
#' time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#'
#' sigA  <- 0.8
#' sigAM <- 0.3
#' sigAT <- 0.4
#' sigE  <- 0.5
#' biasB <- 0.2
#'
#' u   <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
#' # (subject, method) random deviations: repeat per (i,m) across its times
#' wIM <- rnorm(n_subj * 2, 0, sqrt(sigAM))
#' w   <- wIM[ (as.integer(id) - 1L) * 2 + as.integer(method) ]
#' # (subject, time) random deviations: shared across methods at time t
#' gIT <- rnorm(n_subj * n_time, 0, sqrt(sigAT))
#' g   <- gIT[ (as.integer(id) - 1L) * n_time + as.integer(time) ]
#' y   <- (method == "B") * biasB + u + w + g + rnorm(length(id), 0, sqrt(sigE))
#' dat_both <- data.frame(y, id, method, time)
#'
#' fit_both <- ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
#'                          vc_select = "auto", verbose = TRUE, ci = TRUE)
#' summary(fit_both)
#' plot(fit_both)
#'
#' # If you want to force-include both VCs (skip testing):
#' fit_both_forced <-
#'  ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
#'               vc_select = "none", include_subj_method  = TRUE,
#'               include_subj_time  = TRUE, verbose = TRUE)
#' summary(fit_both_forced)
#' plot(fit_both_forced)
#'
#' # ====================================================================
#' # 4) D_m choices: time-averaged (default) vs typical visit
#' # ====================================================================
#' # Time-average
#' ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
#'              vc_select = "none", include_subj_method  = TRUE,
#'              include_subj_time  = TRUE, Dmat_type = "time-avg")
#'
#' # Typical visit
#' ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
#'              vc_select = "none", include_subj_method  = TRUE,
#'              include_subj_time  = TRUE, Dmat_type = "typical-visit")
#'
#' # ====================================================================
#' # 5) AR(1) residual correlation with fixed rho (larger example)
#' # ====================================================================
#' \donttest{
#' set.seed(10)
#' n_subj   <- 40
#' n_time   <- 10
#' methods  <- c("A", "B", "C", "D")
#' nm       <- length(methods)
#' id     <- factor(rep(seq_len(n_subj), each = n_time * nm))
#' method <- factor(rep(rep(methods, each = n_time), times = n_subj),
#'                  levels = methods)
#' time   <- factor(rep(rep(seq_len(n_time), times = nm), times = n_subj))
#'
#' beta0    <- 0
#' beta_t   <- 0.2
#' bias_met <- c(A = 0.00, B = 0.30, C = -0.15, D = 0.05)
#' sigA     <- 1.0
#' rho_true <- 0.6
#' sigE     <- 0.7
#'
#' t_num <- as.integer(time)
#' t_c   <- t_num - mean(seq_len(n_time))
#' mu    <- beta0 + beta_t * t_c + bias_met[as.character(method)]
#'
#' u_subj <- rnorm(n_subj, 0, sqrt(sigA))
#' u      <- u_subj[as.integer(id)]
#'
#' e <- numeric(length(id))
#' for (s in seq_len(n_subj)) {
#'   for (m in methods) {
#'     idx <- which(id == levels(id)[s] & method == m)
#'     e[idx] <- stats::arima.sim(list(ar = rho_true), n = n_time, sd = sigE)
#'   }
#' }
#' y <- mu + u + e
#' dat_ar4 <- data.frame(y = y, id = id, method = method, time = time)
#'
#' fit4 <- ccc_rm_reml(dat_ar4,
#'              response = "y", subject = "id", method = "method", time = "time",
#'              ar = "ar1", ar_rho = 0.6, verbose = TRUE)
#' fit4
#' summary(fit4)
#' plot(fit4)
#' }
#'
#' # ====================================================================
#' # 6) Random slope variants (subject, method, custom Z)
#' # ====================================================================
#' \donttest{
#' ## By SUBJECT
#' set.seed(2)
#' n_subj <- 60; n_time <- 4
#' id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#' method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
#' subj <- as.integer(id)
#' slope_i <- rnorm(n_subj, 0, 0.15)
#' slope_vec <- slope_i[subj]
#' base <- rnorm(n_subj, 0, 1.0)[subj]
#' tnum <- as.integer(tim)
#' y <- base + 0.3*(method=="B") + slope_vec*(tnum - mean(seq_len(n_time))) +
#'      rnorm(length(id), 0, 0.5)
#' dat_s <- data.frame(y, id, method, time = tim)
#' dat_s$t_num <- as.integer(dat_s$time)
#' dat_s$t_c   <- ave(dat_s$t_num, dat_s$id, FUN = function(v) v - mean(v))
#' ccc_rm_reml(dat_s, "y", "id", method = "method", time = "time",
#'              slope = "subject", slope_var = "t_c", verbose = TRUE)
#'
#' ## By METHOD
#' set.seed(3)
#' n_subj <- 60; n_time <- 4
#' id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#' method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
#' slope_m <- ifelse(method=="B", 0.25, 0.00)
#' base <- rnorm(n_subj, 0, 1.0)[as.integer(id)]
#' tnum <- as.integer(tim)
#' y <- base + 0.3*(method=="B") + slope_m*(tnum - mean(seq_len(n_time))) +
#'      rnorm(length(id), 0, 0.5)
#' dat_m <- data.frame(y, id, method, time = tim)
#' dat_m$t_num <- as.integer(dat_m$time)
#' dat_m$t_c   <- ave(dat_m$t_num, dat_m$id, FUN = function(v) v - mean(v))
#' ccc_rm_reml(dat_m, "y", "id", method = "method", time = "time",
#'              slope = "method", slope_var = "t_c", verbose = TRUE)
#'
#' ## SUBJECT + METHOD random slopes (custom Z)
#' set.seed(4)
#' n_subj <- 50; n_time <- 4
#' id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
#' tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
#' method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
#' subj <- as.integer(id)
#' slope_subj <- rnorm(n_subj, 0, 0.12)[subj]
#' slope_B    <- ifelse(method=="B", 0.18, 0.00)
#' tnum <- as.integer(tim)
#' base <- rnorm(n_subj, 0, 1.0)[subj]
#' y <- base + 0.3*(method=="B") +
#'      (slope_subj + slope_B) * (tnum - mean(seq_len(n_time))) +
#'      rnorm(length(id), 0, 0.5)
#' dat_bothRS <- data.frame(y, id, method, time = tim)
#' dat_bothRS$t_num <- as.integer(dat_bothRS$time)
#' dat_bothRS$t_c   <- ave(dat_bothRS$t_num, dat_bothRS$id, FUN = function(v) v - mean(v))
#' MM <- model.matrix(~ 0 + method, data = dat_bothRS)
#' Z_custom <- cbind(
#'   subj_slope = dat_bothRS$t_c,
#'   MM * dat_bothRS$t_c
#' )
#' ccc_rm_reml(dat_bothRS, "y", "id", method = "method", time = "time",
#'              slope = "custom", slope_Z = Z_custom, verbose = TRUE)
#' }
#'
#' @author Thiago de Paula Oliveira
#' @importFrom stats as.formula model.matrix setNames qnorm optimize
#'
#' @section Model overview:
#' Internally, the call is routed to `ccc_lmm_reml_pairwise()`, which fits one
#' repeated-measures mixed model per pair of methods. Each model includes:
#' \itemize{
#'   \item subject random intercepts (always)
#'   \item optional subject-by-method (`sigma^2_{A \times M}`) and
#'         subject-by-time (`sigma^2_{A \times T}`) variance components
#'   \item optional random slopes specified via `slope`/`slope_var`/`slope_Z`
#'   \item residual structure `ar = "none"` (iid) or `ar = "ar1"`
#' }
#' D-matrix options (`Dmat_type`, `Dmat`, `Dmat_weights`) control how time
#' averaging operates when translating variance components into CCC summaries.
#'
#' @name ccc_rm_reml
NULL

#' Internal input checks for repeated-measures REML wrappers.
#' @keywords internal
#' @noRd
.validate_rm_wrapper_inputs <- function(df, response, subject, method = NULL, time = NULL,
                                        slope = c("none", "subject", "method", "custom"),
                                        slope_var = NULL, slope_Z = NULL,
                                        include_subj_time = NULL,
                                        vc_select = c("auto", "none")) {
  slope <- match.arg(slope)
  vc_select <- match.arg(vc_select)

  if (!is.numeric(df[[response]]) || any(!is.finite(df[[response]]))) {
    abort_bad_arg("response",
      message = "must reference a finite numeric column in {.arg data}."
    )
  }

  for (arg in c(subject, method, time)) {
    if (is.null(arg)) next
    if (anyNA(df[[arg]])) {
      abort_bad_arg(arg,
        message = "must not contain missing values for repeated-measures REML fits."
      )
    }
  }

  if (isTRUE(include_subj_time) && is.null(time)) {
    abort_bad_arg("include_subj_time",
      message = "cannot be TRUE when {.arg time} is NULL."
    )
  }

  if (identical(slope, "subject") || identical(slope, "method")) {
    if (is.null(slope_var)) {
      abort_bad_arg("slope_var",
        message = "must be supplied when {.arg slope} is {.val {slope}}."
      )
    }
    if (!is.numeric(df[[slope_var]]) || any(!is.finite(df[[slope_var]]))) {
      abort_bad_arg("slope_var",
        message = "must reference a finite numeric column in {.arg data}."
      )
    }
  }

  if (identical(slope, "custom")) {
    if (is.null(slope_Z)) {
      abort_bad_arg("slope_Z",
        message = "must be supplied when {.arg slope} is {.val custom}."
      )
    }
    slope_Z <- as.matrix(slope_Z)
    check_matrix_dims(slope_Z, nrow = nrow(df), arg = "slope_Z")
    if (!is.numeric(slope_Z) || ncol(slope_Z) <= 0L || any(!is.finite(slope_Z))) {
      abort_bad_arg("slope_Z",
        message = "must be a finite numeric matrix with at least one column."
      )
    }
  } else {
    slope_Z <- NULL
  }

  slope_Z
}

#' @rdname ccc_rm_reml
#' @export
ccc_rm_reml <- function(data, response, subject,
                         method = NULL, time = NULL,
                         ci = FALSE, conf_level = 0.95,
                         n_threads = getOption("matrixCorr.threads", 1L),
                         ci_mode = c("auto","raw","logit"),
                         verbose = FALSE, digits = 4, use_message = TRUE,
                         interaction = FALSE,
                         max_iter = 100, tol = 1e-6,
                         Dmat = NULL,
                         Dmat_type = c("time-avg","typical-visit","weighted-avg","weighted-sq"),
                         Dmat_weights = NULL,
                         Dmat_rescale = TRUE,
                         ar = c("none", "ar1"),
                         ar_rho = NA_real_,
                         slope = c("none", "subject", "method", "custom"),
                         slope_var = NULL,
                         slope_Z = NULL,
                         drop_zero_cols = TRUE,
                         vc_select = c("auto","none"),
                         vc_alpha = 0.05,
                         vc_test_order = c("subj_time","subj_method"),
                         include_subj_method = NULL,
                         include_subj_time = NULL,
                         sb_zero_tol = 1e-10) {

  ar         <- match.arg(ar)
  slope      <- match.arg(slope)
  Dmat_type  <- match.arg(Dmat_type)
  vc_select    <- match.arg(vc_select)
  vc_test_order<- match.arg(vc_test_order, several.ok = TRUE)
  ci_mode <- match.arg(ci_mode)
  ci_mode_int <- switch(ci_mode, raw = 0L, logit = 1L, auto = 2L)

  check_bool(interaction, arg = "interaction")
  max_iter <- check_scalar_int_pos(max_iter, arg = "max_iter")
  check_scalar_nonneg(tol, arg = "tol", strict = TRUE)
  check_bool(ci, arg = "ci")
  check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")
  check_bool(verbose, arg = "verbose")
  check_bool(use_message, arg = "use_message")
  check_bool(Dmat_rescale, arg = "Dmat_rescale")
  check_bool(drop_zero_cols, arg = "drop_zero_cols")
  check_prob_scalar(vc_alpha, arg = "vc_alpha", open_ends = TRUE)

  if (identical(ar, "ar1")) {
    if (length(ar_rho) != 1L) {
      abort_bad_arg("ar_rho",
        message = "must be length 1 (or NA to estimate)."
      )
    }
    if (!is.na(ar_rho)) {
      check_scalar_numeric(ar_rho,
                           arg = "ar_rho",
                           lower = -0.999,
                           upper = 0.999,
                           closed_lower = FALSE,
                           closed_upper = FALSE)
    }
  } else {
    ar_rho <- NA_real_
  }

  df <- as.data.frame(data)

  req_cols <- c(response, subject, method, time, slope_var)
  req_cols <- req_cols[!vapply(req_cols, is.null, logical(1))]
  check_required_cols(df, req_cols, df_arg = "data")

  df[[response]] <- as.numeric(df[[response]])
  df[[subject]] <- factor(df[[subject]])
  if (!is.null(method))  df[[method]]  <- factor(df[[method]])
  if (!is.null(time)) df[[time]] <- factor(df[[time]])
  slope_Z <- .validate_rm_wrapper_inputs(
    df = df,
    response = response,
    subject = subject,
    method = method,
    time = time,
    slope = slope,
    slope_var = slope_var,
    slope_Z = slope_Z,
    include_subj_time = include_subj_time,
    vc_select = vc_select
  )
  all_time_lvls <- if (!is.null(time)) levels(df[[time]]) else character(0)

  terms_rhs <- "1"
  if (!is.null(method)) terms_rhs <- c(terms_rhs, method)
  if (!is.null(time))   terms_rhs <- c(terms_rhs, time)
  if (!is.null(method) && !is.null(time) && interaction) {
    terms_rhs <- c(terms_rhs, sprintf("%s:%s", method, time))
  }
  fml <- as.formula(paste("~", paste(terms_rhs, collapse = " + ")))

  extra_label <- switch(slope,
                        "subject" = "random slope (subject)",
                        "method"  = "random slope (by method)",
                        "custom"  = "custom random effect",
                        NULL)

  if (is.null(method) || nlevels(df[[method]]) < 2L) {
    cli::cli_abort(
      c(
        "At least two method levels are required to compute pairwise CCC.",
        "i" = "Supply {.arg method} with >= 2 levels; the overall CCC is not computed."
      )
    )
  }

  if (is.null(time)) {
    return(.ccc_rm_reml_no_time_fallback(
      df = df,
      response = response,
      rind = subject,
      method = method,
      ci = ci,
      conf_level = conf_level,
      out_class = "ccc_rm_reml",
      out_method = "Variance Components REML - pairwise",
      out_description = "Lin's CCC per method pair from random-effects LMM"
    ))
  }

  # Only pairwise path remains
  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }

  return(
    ccc_lmm_reml_pairwise(
      df                = df,
      fml               = fml,
      response          = response,
      rind              = subject,
      method            = method,
      time              = time,
      slope             = slope,
      slope_var         = slope_var,
      slope_Z           = slope_Z,
      drop_zero_cols    = drop_zero_cols,
      Dmat              = Dmat,
      ar                = ar,
      ar_rho            = ar_rho,
      max_iter          = max_iter,
      tol               = tol,
      conf_level        = conf_level,
      verbose           = verbose,
      digits            = digits,
      use_message       = use_message,
      extra_label       = extra_label,
      ci                = ci,
      ci_mode_int       = ci_mode_int,
      all_time_lvls     = all_time_lvls,
      Dmat_type         = Dmat_type,
      Dmat_weights      = Dmat_weights,
      Dmat_rescale      = Dmat_rescale,
      vc_select         = vc_select,
      vc_alpha          = vc_alpha,
      vc_test_order     = vc_test_order,
      include_subj_method = include_subj_method,
      include_subj_time   = include_subj_time,
      sb_zero_tol       = sb_zero_tol
    )
  )
}

#' @title num_or_na
#' @description Helper to safely coerce a value to numeric or return NA if invalid.
#' @keywords internal
num_or_na <- function(x) {
  x <- suppressWarnings(as.numeric(x))
  if (length(x) != 1 || !is.finite(x)) NA_real_ else x
}

#' @title compute_ci_from_se
#' @description Compute confidence intervals from point estimate and standard error.
#' @keywords internal
compute_ci_from_se <- function(ccc, se, level) {
  if (!is.finite(ccc) || !is.finite(se)) return(c(NA_real_, NA_real_))
  z <- qnorm(1 - (1 - level)/2)
  c(max(0, min(1, ccc - z * se)), max(0, min(1, ccc + z * se)))
}

#' @keywords internal
#' @importFrom stats plogis qlogis
compute_ci_logit_from_se <- function(ccc, se, level) {
  if (!is.finite(ccc) || !is.finite(se) || ccc <= 0 || ccc >= 1) return(c(NA_real_, NA_real_))
  z <- qnorm(1 - (1 - level)/2)
  mu <- qlogis(ccc)
  se_logit <- se / (ccc * (1 - ccc))  # delta
  l <- plogis(mu - z * se_logit)
  u <- plogis(mu + z * se_logit)
  c(max(0, min(1, l)), max(0, min(1, u)))
}

#' @keywords internal
infer_ci_method <- function(ans, ci_mode_int, conf_level, tie_tol = 1e-8) {
  if (is.null(ans)) return(ans)

  # Respect explicit fields if backend already set them
  if (!is.null(ans$ci_method) || !is.null(ans$ci_mode_code)) {
    if (is.null(ans$conf_level)) ans$conf_level <- conf_level
    return(ans)
  }

  method <- ci_mode_name(ci_mode_int)   # "wald-raw" / "wald-logit" / "auto"
  code   <- ci_mode_int                 # 0 / 1 / 2

  if (ci_mode_int == 2L) {  # auto
    ccc <- num_or_na(ans$ccc)
    se  <- num_or_na(ans$se_ccc)
    lwr <- num_or_na(ans$lwr)
    upr <- num_or_na(ans$upr)

    if (is.finite(ccc) && is.finite(se)) {
      if (is.finite(lwr) && is.finite(upr)) {
        raw_ci   <- compute_ci_from_se(ccc, se, conf_level)
        logit_ci <- compute_ci_logit_from_se(ccc, se, conf_level)

        d_raw   <- sum(abs(raw_ci   - c(lwr, upr)))
        d_logit <- sum(abs(logit_ci - c(lwr, upr)))

        if (is.finite(d_raw) && is.finite(d_logit)) {
          if (abs(d_raw - d_logit) <= tie_tol) {
            # near tie: prefer logit near boundaries, else raw
            prefer_logit <- (ccc < 0.1 || ccc > 0.9)
            if (prefer_logit) { method <- "wald-logit"; code <- 1L }
            else              { method <- "wald-raw";   code <- 0L }
          } else if (d_logit < d_raw) {
            method <- "wald-logit"; code <- 1L
          } else {
            method <- "wald-raw";   code <- 0L
          }
        }
      } else {
        # R will build CI later with compute_ci_from_se -> raw
        method <- "wald-raw"; code <- 0L
      }
    }
  }

  ans$ci_method    <- method
  ans$ci_mode_code <- code
  if (is.null(ans$conf_level)) ans$conf_level <- conf_level
  ans
}

#' @title num_or_na_vec
#' @description Display variance component estimation details to the console.
#' @keywords internal
num_or_na_vec <- function(x) {
  if (is.null(x)) return(NULL)
  x <- suppressWarnings(as.numeric(x))
  if (!length(x)) return(NULL)
  x[!is.finite(x)] <- NA_real_
  x
}


#' @title .vc_message
#' @description Display variance component estimation details to the console.
#' @keywords internal
.vc_message <- function(ans, label, nm, nt, conf_level,
                        digits = 4, use_message = TRUE,
                        extra_label = NULL,
                        ar = c("none", "ar1"),
                        ar_rho = NA_real_,
                        engine_name = "ccc_rm_reml",
                        se_label = "SE(CCC)") {
  ar <- match.arg(ar)

  fmt <- function(x) {
    if (is.null(x)) return("NA")
    x <- suppressWarnings(as.numeric(x))
    if (length(x) == 0 || all(!is.finite(x))) return("NA")
    if (length(x) == 1) {
      return(formatC(x, format = "f", digits = digits))
    }
    # vector: pretty-print as a comma-separated list
    paste0("[", paste(formatC(x, format = "f", digits = digits), collapse = ", "), "]")
  }

  colw <- 38L
  v <- function(s, x) sprintf("  %-*s : %s", colw, s, fmt(x))

  out <- c(
    sprintf("---- matrixCorr::%s - variance-components (%s) ----", engine_name, label),
    sprintf("Design: methods nm = %d, times nt = %d", nm, nt)
  )
  if (identical(ar, "ar1")) {
    info <- if (is.na(ar_rho)) "AR(1) (rho estimated)" else sprintf("AR(1) with rho = %s", fmt(ar_rho))
    out <- c(out, paste("Residual correlation:", info))
  } else {
    out <- c(out, "Residual correlation: independent (iid)")
  }

  out <- c(out,
           "Estimates:",
           v("sigma_A^2 (subject)",             ans[["sigma2_subject"]]),
           v("sigma_A_M^2 (subject x method)",  ans[["sigma2_subject_method"]]),
           v("sigma_A_T^2 (subject x time)",    ans[["sigma2_subject_time"]]),
           v("sigma_E^2 (error)",               ans[["sigma2_error"]]))

  if (!is.null(ans[["sigma2_extra"]])) {
    lab <- if (is.null(extra_label)) "extra random effect" else extra_label
    out <- c(out, v(sprintf("sigma_Z^2 (%s)", lab), ans[["sigma2_extra"]]))
  }

  out <- c(out,
           v("S_B (fixed-effect dispersion)",   ans[["SB"]]),
           v(se_label,                          ans[["se_metric"]] %||% ans[["se_ccc"]]))

  has_bounds <- is.finite(num_or_na(ans$lwr)) && is.finite(num_or_na(ans$upr))
  if (!is.null(ans$ci_method) || has_bounds) {
    cm <- ans$ci_method %||% ci_mode_name(ans$ci_mode_code %||% NA_integer_)
    cl <- ans$conf_level %||% conf_level
    out <- c(out, sprintf("CI: %s (conf_level = %s)", cm, fmt(cl)))
  }
  out <- c(out,
           "--------------------------------------------------------------------------")

  msg <- paste(out, collapse = "\n")
  if (isTRUE(use_message)) {
    cli::cli_inform(msg)
  } else {
    message(msg)
  }
}

#' @title build_LDZ
#' @description Internal helper to construct L, Dm, and Z matrices for random effects.
#' @keywords internal
build_LDZ <- function(colnames_X, method_levels, time_levels, Dsub, df_sub,
                      method_name, time_name, slope, interaction, slope_var,
                      drop_zero_cols) {
  slope_mode_cpp <- switch(slope, none = "none", subject = "subject", method = "method", custom = "none")
  if (!identical(slope, "custom")) {
    build_L_Dm_Z_cpp(
      colnames_X      = colnames_X,
      rmet_name       = if (is.null(method_name)) NULL else method_name,
      rtime_name      = if (is.null(time_name)) NULL else time_name,
      method_levels   = if (is.null(method_name)) character(0) else method_levels,
      time_levels     = if (is.null(time_name)) character(0) else time_levels,
      has_interaction = interaction,
      Dmat_global     = Dsub,
      slope_mode      = slope_mode_cpp,
      slope_var       = if (!is.null(slope_var)) df_sub[[slope_var]] else NULL,
      method_codes    = if (!is.null(method_name)) as.integer(df_sub[[method_name]]) else NULL,
      drop_zero_cols  = drop_zero_cols
    )
  } else {
    build_L_Dm_cpp(
      colnames_X      = colnames_X,
      rmet_name       = if (is.null(method_name)) NULL else method_name,
      rtime_name      = if (is.null(time_name)) NULL else time_name,
      method_levels   = if (is.null(method_name)) character(0) else method_levels,
      time_levels     = if (is.null(time_name)) character(0) else time_levels,
      has_interaction = interaction,
      Dmat_global     = Dsub
    )
  }
}

#' @title run_cpp
#' @description Wrapper for calling 'C++' backend for CCC estimation.
#' @keywords internal
#' @name run_cpp
NULL

#' Internal validation for variance-component index codes.
#' @keywords internal
#' @noRd
.validate_vc_index_codes <- function(x, arg, expected_max = NULL, allow_empty = FALSE) {
  if (!is.integer(x)) {
    cli::cli_abort("{.arg {arg}} must be an integer vector before calling {.fn ccc_vc_cpp}.")
  }
  if (!length(x)) {
    if (allow_empty) return(invisible(NULL))
    cli::cli_abort("{.arg {arg}} must not be empty before calling {.fn ccc_vc_cpp}.")
  }
  if (anyNA(x)) {
    cli::cli_abort("{.arg {arg}} must not contain missing values before calling {.fn ccc_vc_cpp}.")
  }
  if (any(!is.finite(as.numeric(x))) || any(x <= 0L)) {
    cli::cli_abort("{.arg {arg}} must contain only positive finite integer codes before calling {.fn ccc_vc_cpp}.")
  }

  u <- sort.int(unique(x), method = "quick")
  if (!identical(u, seq_len(max(x)))) {
    cli::cli_abort("{.arg {arg}} must use contiguous 1..K integer codes before calling {.fn ccc_vc_cpp}.")
  }
  if (!is.null(expected_max) && max(x) != expected_max) {
    cli::cli_abort("{.arg {arg}} codes must have maximum {.val {expected_max}} before calling {.fn ccc_vc_cpp}.")
  }
}

.validate_run_cpp_inputs <- function(Xr, yr, subject, method_int, time_int, nm, nt,
                                     Lr = NULL, auxDr = NULL, Zr = NULL,
                                     include_subj_method = TRUE,
                                     include_subj_time = TRUE,
                                     time_weights = NULL) {
  if (!is.matrix(Xr) || !is.numeric(Xr)) {
    cli::cli_abort("{.arg Xr} must be a numeric matrix before calling {.fn ccc_vc_cpp}.")
  }
  if (!is.numeric(yr) || is.matrix(yr)) {
    cli::cli_abort("{.arg yr} must be a numeric vector before calling {.fn ccc_vc_cpp}.")
  }
  if (any(!is.finite(Xr))) {
    cli::cli_abort("{.arg Xr} must contain only finite numeric values before calling {.fn ccc_vc_cpp}.")
  }
  if (any(!is.finite(yr))) {
    cli::cli_abort("{.arg yr} must contain only finite numeric values before calling {.fn ccc_vc_cpp}.")
  }
  if (nrow(Xr) != length(yr)) {
    cli::cli_abort("{.arg Xr} row count must match {.arg yr} length before calling {.fn ccc_vc_cpp}.")
  }

  .validate_vc_index_codes(subject, "subject")
  .validate_vc_index_codes(method_int, "method", expected_max = nm)
  if (length(subject) != length(yr)) {
    cli::cli_abort("{.arg subject} length must match {.arg yr} before calling {.fn ccc_vc_cpp}.")
  }
  if (length(method_int) != length(yr)) {
    cli::cli_abort("{.arg method} length must match {.arg yr} before calling {.fn ccc_vc_cpp}.")
  }

  has_time <- length(time_int) > 0L
  if (has_time) {
    .validate_vc_index_codes(time_int, "time", expected_max = nt)
    if (length(time_int) != length(yr)) {
      cli::cli_abort("{.arg time} length must match {.arg yr} before calling {.fn ccc_vc_cpp}.")
    }
  } else {
    if (isTRUE(include_subj_time)) {
      cli::cli_abort("A subject-time random component requires a non-empty {.arg time} vector.")
    }
    if (!(nt %in% c(0L, 1L))) {
      cli::cli_abort("When {.arg time} is absent, {.arg nt} must be 0 or 1.")
    }
  }

  if (!is.null(Lr)) {
    if (!is.matrix(Lr) || !is.numeric(Lr)) {
      cli::cli_abort("{.arg Lr} must be a numeric matrix or {.code NULL}.")
    }
    if (any(!is.finite(Lr))) {
      cli::cli_abort("{.arg Lr} must contain only finite numeric values.")
    }
    if (nrow(Lr) != ncol(Xr)) {
      cli::cli_abort("{.arg Lr} must have {.val {ncol(Xr)}} rows.")
    }
    if (ncol(Lr) <= 0L) {
      cli::cli_abort("{.arg Lr} must have at least one column when supplied.")
    }
  } else if (nm > 0L) {
    cli::cli_abort("{.arg Lr} must be present when method contrasts are used.")
  }

  if (!is.null(auxDr)) {
    if (!is.matrix(auxDr) || !is.numeric(auxDr)) {
      cli::cli_abort("{.arg auxDr} must be a numeric matrix or {.code NULL}.")
    }
    if (any(!is.finite(auxDr))) {
      cli::cli_abort("{.arg auxDr} must contain only finite numeric values.")
    }
    if (is.null(Lr)) {
      cli::cli_abort("{.arg auxDr} cannot be supplied without {.arg Lr}.")
    }
    if (!identical(dim(auxDr), c(ncol(Lr), ncol(Lr)))) {
      cli::cli_abort("{.arg auxDr} must be a square matrix with dimension {.val {ncol(Lr)}}.")
    }
  } else if (!is.null(Lr)) {
    cli::cli_abort("{.arg auxDr} must be present when {.arg Lr} is supplied.")
  }

  if (!is.null(Zr)) {
    if (!is.matrix(Zr) || !is.numeric(Zr)) {
      cli::cli_abort("{.arg Zr} must be a numeric matrix or {.code NULL}.")
    }
    if (any(!is.finite(Zr))) {
      cli::cli_abort("{.arg Zr} must contain only finite numeric values.")
    }
    if (nrow(Zr) != nrow(Xr)) {
      cli::cli_abort("{.arg Zr} must have {.val {nrow(Xr)}} rows.")
    }
    if (ncol(Zr) == 0L) {
      Zr <- NULL
    }
  }

  if (!is.null(time_weights)) {
    if (!has_time) {
      cli::cli_abort("{.arg time_weights} requires a non-empty {.arg time} vector.")
    }
    if (!is.numeric(time_weights) || anyNA(time_weights) || any(!is.finite(time_weights)) || any(time_weights < 0)) {
      cli::cli_abort("{.arg time_weights} must contain only finite non-negative values.")
    }
    if (length(time_weights) != nt) {
      cli::cli_abort("{.arg time_weights} length must match {.arg nt}.")
    }
  }

  invisible(NULL)
}

#' @rdname run_cpp
run_cpp <- function(Xr, yr, subject, method_int, time_int, Laux, Z,
                    use_ar1, ar1_rho, max_iter, tol, conf_level, ci_mode_int,
                    include_subj_method = TRUE, include_subj_time = TRUE,
                    sb_zero_tol = 1e-10, eval_single_visit = FALSE,
                    time_weights = NULL,
                    metric_mode = 0L,
                    loglik_only = FALSE,
                    need_reml_loglik = TRUE) {

  # Guard: you cannot include a random component whose dimension is 0
  include_subj_method <- isTRUE(include_subj_method) && isTRUE(Laux$nm > 0)
  include_subj_time   <- isTRUE(include_subj_time)   && isTRUE(Laux$nt > 0)

  Lr <- if (is.null(Laux$L)) NULL else unname(as.matrix(Laux$L))
  auxDr <- if (is.null(Laux$Dm)) NULL else unname(as.matrix(Laux$Dm))
  Zr <- if (is.null(Z)) NULL else unname(as.matrix(Z))
  if (!is.null(Zr) && ncol(Zr) == 0L) {
    Zr <- NULL
  }
  time_arg <- if (length(time_int)) as.integer(time_int) else integer(0)
  time_weights_arg <- if (is.null(time_weights)) NULL else as.numeric(time_weights)

  .validate_run_cpp_inputs(
    Xr = Xr,
    yr = yr,
    subject = as.integer(subject),
    method_int = as.integer(method_int),
    time_int = time_arg,
    nm = as.integer(Laux$nm),
    nt = as.integer(Laux$nt),
    Lr = Lr,
    auxDr = auxDr,
    Zr = Zr,
    include_subj_method = include_subj_method,
    include_subj_time = include_subj_time,
    time_weights = time_weights_arg
  )

  ccc_vc_cpp(
    Xr = unname(Xr),
    yr = yr,
    subject = subject,
    method  = method_int,
    time    = time_arg,
    nm = Laux$nm, nt = Laux$nt,
    max_iter = max_iter, tol = tol,
    conf_level = conf_level,
    ci_mode = ci_mode_int,
    Lr    = Lr,
    auxDr = auxDr,
    Zr    = Zr,
    use_ar1 = use_ar1,
    ar1_rho = as.numeric(ar1_rho),
    include_subj_method = include_subj_method,
    include_subj_time   = include_subj_time,
    sb_zero_tol = as.numeric(sb_zero_tol),
    eval_single_visit = eval_single_visit,
    time_weights = time_weights_arg,
    metric_mode = as.integer(metric_mode),
    ll_only = isTRUE(loglik_only),
    need_loglik = isTRUE(need_reml_loglik)
  )
}

#' @keywords internal
ci_mode_name <- function(code) {
  switch(as.character(code),
         "0" = "wald-raw",
         "1" = "wald-logit",
         "2" = "auto",     # chosen per-estimate in C++
         "unknown")
}

#' @title estimate_rho
#' @description Estimate AR(1) correlation parameter rho by optimizing REML log-likelihood.
#' @keywords internal
estimate_rho <- function(Xr, yr, subject, method_int, time_int, Laux, Z,
                         rho_lo = -0.95, rho_hi = 0.95,
                         max_iter = 100, tol = 1e-6, conf_level = 0.95,
                         ci_mode_int,
                         include_subj_method = TRUE, include_subj_time = TRUE,
                         sb_zero_tol = 1e-10, eval_single_visit = FALSE,
                         time_weights = NULL,
                         metric_mode = 0L) {
  obj <- function(r) {
    fit <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                   use_ar1 = TRUE, ar1_rho = r,
                   max_iter = max_iter, tol = tol, conf_level = conf_level,
                   ci_mode_int = ci_mode_int,
                   include_subj_method = include_subj_method,
                   include_subj_time   = include_subj_time,
                   sb_zero_tol = sb_zero_tol,
                   eval_single_visit = eval_single_visit,
                   time_weights = time_weights,
                   metric_mode = metric_mode,
                   loglik_only = FALSE,
                   need_reml_loglik = TRUE)
    ll <- suppressWarnings(as.numeric(fit[["reml_loglik"]]))
    if (!is.finite(ll)) return(Inf)
    -ll
  }
  oo <- optimize(obj, interval = c(rho_lo, rho_hi))
  list(rho = unname(oo$minimum), used_reml = TRUE)
}

#' @keywords internal
inform_ccc_rm_ar1_fallback <- function(pair_label = NULL,
                                       .verbose = getOption("matrixCorr.verbose", TRUE)) {
  where <- if (length(pair_label)) {
    sprintf(" for pair(s): %s", pair_label)
  } else {
    " for this fit"
  }

  inform_if_verbose(
    sprintf(
      "Requested AR(1) residual structure could not be fit%s; using iid residuals instead.",
      where
    ),
    .verbose = .verbose
  )
}

#' @keywords internal
recommend_ar1_from_refit <- function(ans_iid, Xr, yr, subject, method_int, time_int, Laux, Z,
                                     max_iter = 100, tol = 1e-6, conf_level = 0.95,
                                     ci_mode_int,
                                     include_subj_method = TRUE, include_subj_time = TRUE,
                                     sb_zero_tol = 1e-10, eval_single_visit = FALSE,
                                     time_weights = NULL) {
  if (!isTRUE(ans_iid[["use_ar1"]])) return(FALSE)

  rho_fit <- tryCatch(
    estimate_rho(
      Xr, yr, subject, method_int, time_int, Laux, Z,
      max_iter = max_iter, tol = tol, conf_level = conf_level,
      ci_mode_int = ci_mode_int,
      include_subj_method = include_subj_method,
      include_subj_time   = include_subj_time,
      sb_zero_tol = sb_zero_tol,
      eval_single_visit = eval_single_visit,
      time_weights = time_weights
    ),
    error = function(e) NULL
  )
  rho_hat <- num_or_na(rho_fit$rho %||% NA_real_)
  if (!is.finite(rho_hat) || rho_hat <= 0) return(FALSE)

  sag_share <- if (isTRUE(include_subj_time)) {
    sag_hat <- num_or_na(ans_iid[["sigma2_subject_time"]])
    se_hat  <- num_or_na(ans_iid[["sigma2_error"]])
    if (is.finite(sag_hat) && is.finite(se_hat)) {
      sag_hat / max(1e-12, sag_hat + se_hat)
    } else {
      0
    }
  } else {
    0
  }
  thr <- if (sag_share > 0.25) 0.20 else 0.10
  rho_hat >= thr
}

#' @keywords internal
#' @importFrom stats pchisq
p_half_chisq1 <- function(lrt) 0.5 * pchisq(lrt, df = 1, lower.tail = FALSE)

#' @keywords internal
reml_lrt_select <- function(Xr, yr, subject, method_int, time_int, Laux, Z,
                            ar = c("none","ar1"), ar_rho = NA_real_,
                            max_iter = 100, tol = 1e-6, conf_level = 0.95,
                            ci_mode_int,
                            alpha = 0.05, test_order = c("subj_time","subj_method"),
                            sb_zero_tol = 1e-10,
                            eval_single_visit = FALSE,
                            time_weights = NULL,
                            metric_mode = 0L) {
  ar <- match.arg(ar)
  can_subj_method <- isTRUE(Laux$nm > 0)
  can_subj_time   <- isTRUE(Laux$nt > 0)

  inc_subj_method <- can_subj_method
  inc_subj_time   <- can_subj_time

  # Function to get rho for a given (inc_subj_method, inc_subj_time)
  get_rho <- function(inc_subj_method, inc_subj_time) {
    if (!identical(ar, "ar1") || !is.na(ar_rho)) return(ar_rho)
    er <- estimate_rho(Xr, yr, subject, method_int, time_int, Laux, Z,
                       max_iter = max_iter, tol = tol, conf_level = conf_level,
                       ci_mode_int = ci_mode_int,
                       include_subj_method = inc_subj_method,
                       include_subj_time   = inc_subj_time,
                       sb_zero_tol = sb_zero_tol,
                       eval_single_visit = eval_single_visit,
                       time_weights = time_weights,
                       metric_mode = metric_mode)
    er$rho
  }

  # Fit full (with rho profiled if needed)
  rho_full <- get_rho(inc_subj_method, inc_subj_time)
  fit_full <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                      use_ar1 = identical(ar, "ar1"),
                      ar1_rho = if (identical(ar, "ar1")) rho_full else 0,
                      max_iter = max_iter, tol = tol, conf_level = conf_level,
                      ci_mode_int = ci_mode_int,
                      include_subj_method = inc_subj_method,
                      include_subj_time   = inc_subj_time,
                      sb_zero_tol = sb_zero_tol,
                      eval_single_visit = eval_single_visit,
                      time_weights = time_weights,
                      metric_mode = metric_mode)

  for (what in test_order) {
    if (what == "subj_time" && inc_subj_time) {
      rho0 <- get_rho(inc_subj_method, FALSE)
      fit0_ll <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                         use_ar1 = identical(ar, "ar1"),
                         ar1_rho = if (identical(ar, "ar1")) rho0 else 0,
                         max_iter = max_iter, tol = tol, conf_level = conf_level,
                         ci_mode_int = ci_mode_int,
                         include_subj_method = inc_subj_method, include_subj_time = FALSE,
                         sb_zero_tol = sb_zero_tol,
                         eval_single_visit = eval_single_visit,
                         time_weights = time_weights,
                         metric_mode = metric_mode,
                         loglik_only = FALSE,
                         need_reml_loglik = TRUE)
      lrt <- 2 * (as.numeric(fit_full$reml_loglik) - as.numeric(fit0_ll$reml_loglik))
      p   <- p_half_chisq1(max(lrt, 0))
      if (is.finite(p) && p > alpha) {
        inc_subj_time <- FALSE
        fit_full <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                            use_ar1 = identical(ar, "ar1"),
                            ar1_rho = if (identical(ar, "ar1")) rho0 else 0,
                            max_iter = max_iter, tol = tol, conf_level = conf_level,
                            ci_mode_int = ci_mode_int,
                            include_subj_method = inc_subj_method, include_subj_time = FALSE,
                            sb_zero_tol = sb_zero_tol,
                            eval_single_visit = eval_single_visit,
                            time_weights = time_weights,
                            metric_mode = metric_mode,
                            need_reml_loglik = FALSE)
        rho_full <- rho0
      }
    }
    if (what == "subj_method" && inc_subj_method) {
      rho0 <- get_rho(FALSE, inc_subj_time)
      fit0_ll <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                         use_ar1 = identical(ar, "ar1"),
                         ar1_rho = if (identical(ar, "ar1")) rho0 else 0,
                         max_iter = max_iter, tol = tol, conf_level = conf_level,
                         ci_mode_int = ci_mode_int,
                         include_subj_method = FALSE, include_subj_time = inc_subj_time,
                         sb_zero_tol = sb_zero_tol,
                         eval_single_visit = eval_single_visit,
                         time_weights = time_weights,
                         metric_mode = metric_mode,
                         loglik_only = FALSE,
                         need_reml_loglik = TRUE)
      lrt <- 2 * (as.numeric(fit_full$reml_loglik) - as.numeric(fit0_ll$reml_loglik))
      p   <- p_half_chisq1(max(lrt, 0))
      if (is.finite(p) && p > alpha) {
        inc_subj_method <- FALSE
        fit_full <- run_cpp(Xr, yr, subject, method_int, time_int, Laux, Z,
                            use_ar1 = identical(ar, "ar1"),
                            ar1_rho = if (identical(ar, "ar1")) rho0 else 0,
                            max_iter = max_iter, tol = tol, conf_level = conf_level,
                            ci_mode_int = ci_mode_int,
                            include_subj_method = FALSE, include_subj_time = inc_subj_time,
                            sb_zero_tol = sb_zero_tol,
                            eval_single_visit = eval_single_visit,
                            time_weights = time_weights,
                            metric_mode = metric_mode,
                            need_reml_loglik = FALSE)
        rho_full <- rho0
      }
    }
  }

  list(include_subj_method = inc_subj_method,
       include_subj_time   = inc_subj_time,
       rho = rho_full,
       fit = fit_full)
}

#' @keywords internal
.ccc_rm_reml_no_time_fallback <- function(df,
                                          response,
                                          rind,
                                          method,
                                          ci,
                                          conf_level,
                                          out_class,
                                          out_method,
                                          out_description) {
  fit <- ccc_rm_ustat(
    df,
    response = response,
    subject = rind,
    method = method,
    ci = ci,
    conf_level = conf_level
  )

  method_levels <- levels(droplevels(df[[method]]))
  Lm <- length(method_levels)
  dn <- list(method_levels, method_levels)

  n_obs_mat <- matrix(NA_integer_, Lm, Lm, dimnames = dn)
  n_subjects_mat <- matrix(NA_integer_, Lm, Lm, dimnames = dn)
  method_code_all <- as.integer(df[[method]])
  idx_by_method <- lapply(seq_len(Lm), function(k) which(method_code_all == k))
  subject_all <- df[[rind]]

  for (i in seq_len(Lm - 1L)) {
    for (j in (i + 1L):Lm) {
      idx <- sort.int(c(idx_by_method[[i]], idx_by_method[[j]]), method = "quick")
      subj_pair <- droplevels(factor(subject_all[idx]))
      met_pair <- droplevels(factor(df[[method]][idx]))
      complete_mask <- stats::complete.cases(df[idx, c(response, rind, method), drop = FALSE])
      subj_pair <- subj_pair[complete_mask]
      met_pair <- met_pair[complete_mask]

      sid <- as.integer(subj_pair)
      mid <- as.integer(met_pair)
      rows_by_subj <- tabulate(sid, nbins = nlevels(subj_pair))
      combo_code <- sid + nlevels(subj_pair) * mid
      uniq_combo <- !duplicated(combo_code)
      uniq_by_subj <- tabulate(sid[uniq_combo], nbins = nlevels(subj_pair))
      complete_subj <- (rows_by_subj == 2L) & (uniq_by_subj == 2L)
      n_subjects_pair <- sum(complete_subj)
      n_obs_pair <- 2L * n_subjects_pair

      n_subjects_mat[i, j] <- n_subjects_mat[j, i] <- as.integer(n_subjects_pair)
      n_obs_mat[i, j] <- n_obs_mat[j, i] <- as.integer(n_obs_pair)
    }
  }

  diag(n_subjects_mat) <- NA_integer_
  diag(n_obs_mat) <- NA_integer_

  empty_vc <- matrix(NA_real_, Lm, Lm, dimnames = dn)
  empty_extra <- matrix(vector("list", Lm * Lm), Lm, Lm, dimnames = dn)
  ar1_na <- matrix(NA_real_, Lm, Lm, dimnames = dn)
  ar1_pairs <- matrix(NA_integer_, Lm, Lm, dimnames = dn)
  ar1_use <- matrix(FALSE, Lm, Lm, dimnames = dn)

  if (isTRUE(ci)) {
    out <- structure(
      list(
        est = as.matrix(fit$est),
        lwr.ci = as.matrix(fit$lwr.ci),
        upr.ci = as.matrix(fit$upr.ci)
      ),
      class = c("ccc_rm_reml", "matrixCorr_ccc_ci", "matrixCorr_ccc", "ccc")
    )
    attr(out, "conf.level") <- conf_level
  } else {
    out <- structure(
      as.matrix(fit),
      class = c("ccc_rm_reml", "matrixCorr_ccc", "ccc", "matrix")
    )
  }

  attr(out, "method") <- paste0(out_method, " (no-time fallback)")
  attr(out, "description") <- paste0(out_description, " [fallback via ccc_rm_ustat]")
  attr(out, "package") <- "matrixCorr"
  attr(out, "residual_model") <- "iid"
  attr(out, "sigma2_subject") <- empty_vc
  attr(out, "sigma2_subject_method") <- empty_vc
  attr(out, "sigma2_subject_time") <- empty_vc
  attr(out, "sigma2_error") <- empty_vc
  attr(out, "sigma2_extra") <- empty_extra
  attr(out, "SB") <- empty_vc
  attr(out, "se_ccc") <- empty_vc
  attr(out, "n_obs") <- n_obs_mat
  attr(out, "n_subjects") <- n_subjects_mat
  attr(out, "ar1_rho_lag1") <- ar1_na
  attr(out, "ar1_pairs") <- ar1_pairs
  attr(out, "ar1_pval") <- ar1_na
  attr(out, "use_ar1") <- ar1_use
  out
}

#' @title ccc_lmm_reml_pairwise
#' @description Internal function to handle pairwise CCC estimation for each method pair.
#' @keywords internal
ccc_lmm_reml_pairwise <- function(df, fml, response, rind, method, time,
                                  slope, slope_var, slope_Z, drop_zero_cols,
                                  Dmat, ar, ar_rho, max_iter, tol,
                                  conf_level, verbose, digits, use_message,
                                  extra_label, ci, ci_mode_int, all_time_lvls,
                                  Dmat_type = c("time-avg","typical-visit","weighted-avg","weighted-sq"),
                                  Dmat_weights = NULL,
                                  Dmat_rescale = TRUE,
                                  vc_select = c("auto","none"),
                                  vc_alpha = 0.05,
                                  vc_test_order = c("subj_time","subj_method"),
                                  include_subj_method = NULL,
                                  include_subj_time = NULL,
                                  sb_zero_tol = 1e-10,
                                  metric_mode = 0L,
                                  estimate_key = "ccc",
                                  se_key = "se_ccc",
                                  out_class = "ccc_rm_reml",
                                  out_method = "Variance Components REML - pairwise",
                                  out_description = "Lin's CCC per method pair from random-effects LMM",
                                  summary_title = "Repeated-measures concordance matrix",
                                  vc_engine_name = "ccc_rm_reml",
                                  vc_se_label = "SE(CCC)") {
  interaction_requested <- !is.null(time) &&
    grepl(":", paste(deparse(fml), collapse = ""), fixed = TRUE)

  Dmat_type <- match.arg(Dmat_type)
  eval_single_visit <- Dmat_type %in% c("typical-visit","weighted-sq")
  vc_select <- match.arg(vc_select)
  vc_test_order <- match.arg(vc_test_order, several.ok = TRUE)

  df[[method]] <- droplevels(df[[method]])
  method_levels <- levels(df[[method]])
  Lm <- length(method_levels)

  if (is.null(time) && identical(out_class, "ccc_rm_reml")) {
    return(.ccc_rm_reml_no_time_fallback(
      df = df,
      response = response,
      rind = rind,
      method = method,
      ci = ci,
      conf_level = conf_level,
      out_class = out_class,
      out_method = out_method,
      out_description = out_description
    ))
  }

  method_code_all <- as.integer(df[[method]])
  idx_by_method <- lapply(seq_len(Lm), function(k) which(method_code_all == k))
  Dfull_input <- if (!is.null(Dmat)) as.matrix(Dmat) else NULL

  est_mat <- matrix(1,  Lm, Lm, dimnames = list(method_levels, method_levels))
  if (isTRUE(ci)) {
    lwr_mat <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
    upr_mat <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  }

  rho_mat <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  n_obs_mat <- matrix(NA_integer_, Lm, Lm, dimnames = list(method_levels, method_levels))
  n_subjects_mat <- matrix(NA_integer_, Lm, Lm, dimnames = list(method_levels, method_levels))

  vc_subject        <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  vc_subject_method <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  vc_subject_time   <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  vc_error          <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  vc_extra <- matrix(vector("list", Lm * Lm), Lm, Lm,
                     dimnames = list(method_levels, method_levels))
  vc_SB             <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))
  vc_se_metric      <- matrix(NA_real_, Lm, Lm, dimnames = list(method_levels, method_levels))

  ar1_rho_lag1_mat <- matrix(NA_real_,    Lm, Lm, dimnames = list(method_levels, method_levels))
  ar1_pairs_mat    <- matrix(NA_integer_, Lm, Lm, dimnames = list(method_levels, method_levels))
  ar1_pval_mat     <- matrix(NA_real_,    Lm, Lm, dimnames = list(method_levels, method_levels))
  ar1_reco_mat     <- matrix(NA,          Lm, Lm, dimnames = list(method_levels, method_levels))
  ar_used_mat      <- matrix(FALSE,       Lm, Lm, dimnames = list(method_levels, method_levels))

  for (i in 1:(Lm - 1L)) {
    for (j in (i + 1L):Lm) {
      m1 <- method_levels[i]; m2 <- method_levels[j]

      idx       <- sort.int(c(idx_by_method[[i]], idx_by_method[[j]]), method = "quick")
      subj_fac  <- droplevels(df[[rind]][idx])
      subj_int  <- as.integer(subj_fac)
      y_sub     <- df[[response]][idx]
      met_fac   <- droplevels(df[[method]][idx])        # exactly 2 levels
      time_fac  <- if (!is.null(time)) droplevels(df[[time]][idx]) else NULL
      df_sub    <- df[idx, , drop = FALSE]
      df_sub[[rind]] <- subj_fac
      df_sub[[method]] <- met_fac
      if (!is.null(time)) df_sub[[time]] <- time_fac
      n_obs_mat[i, j] <- n_obs_mat[j, i] <- as.integer(length(y_sub))
      n_subjects_mat[i, j] <- n_subjects_mat[j, i] <- as.integer(length(unique(subj_int)))

      has_time_levels <- !is.null(time_fac) && nlevels(time_fac) >= 2L
      pair_terms_rhs <- c("1", method)
      if (has_time_levels) pair_terms_rhs <- c(pair_terms_rhs, time)
      if (has_time_levels && isTRUE(interaction_requested)) {
        pair_terms_rhs <- c(pair_terms_rhs, sprintf("%s:%s", method, time))
      }
      pair_fml <- as.formula(paste("~", paste(pair_terms_rhs, collapse = " + ")))
      Xp <- model.matrix(pair_fml, data = df_sub)

      lev_time_sub <- if (has_time_levels) levels(time_fac) else character(0)

      # -------- Build/subset Dmat for this pair (only if >= 2 time levels) --------
      if (has_time_levels) {
        if (!is.null(Dmat)) {
          Dfull <- Dfull_input
          if (!is.null(all_time_lvls) &&
              nrow(Dfull) == length(all_time_lvls) && ncol(Dfull) == length(all_time_lvls)) {
            pos  <- match(lev_time_sub, all_time_lvls)
            Dsub <- Dfull[pos, pos, drop = FALSE]
          } else if (nrow(Dfull) == length(lev_time_sub) && ncol(Dfull) == length(lev_time_sub)) {
            Dsub <- Dfull
          } else {
            cli::cli_abort(
              c(
                "Dmat has incompatible dimension for present time levels in a pairwise fit.",
                "i" = "Provide a square matrix matching either all time levels in {.arg data} or the present pairwise subset."
              )
            )
          }
          if (isTRUE(Dmat_rescale))
            Dsub <- .Dmat_normalise_mass(Dsub, length(lev_time_sub))
          Dsub <- 0.5 * (Dsub + t(Dsub))
        } else {
          w_sub <- .align_weights_to_levels(Dmat_weights, lev_time_sub, all_time_lvls)
          Dsub  <- .Dmat_build_kernel(length(lev_time_sub),
                                      type    = Dmat_type,
                                      w       = w_sub,
                                      rescale = Dmat_rescale)
        }
      } else {
        Dsub <- NULL
      }

      # --- per-pair time weights for kappa (only for weighted-avg target) ---
      time_weights_kappa <- NULL
      if (has_time_levels && identical(Dmat_type, "weighted-avg")) {
        w_sub <- .align_weights_to_levels(Dmat_weights, lev_time_sub, all_time_lvls)
        if (is.null(w_sub)) w_sub <- rep(1 / length(lev_time_sub), length(lev_time_sub))
        sw <- sum(w_sub, na.rm = TRUE)
        if (!is.finite(sw) || sw <= 0) {
          abort_bad_arg("Dmat_weights",
            message = "must sum to a positive finite number."
          )
        }
        time_weights_kappa <- as.numeric(w_sub / sw)
      }

      has_interaction <- isTRUE(interaction_requested) && has_time_levels

      Laux <- build_LDZ(
        colnames_X      = colnames(Xp),
        method_levels   = levels(met_fac),
        time_levels     = lev_time_sub,
        Dsub            = Dsub,
        df_sub          = df_sub,
        method_name     = method,
        time_name       = time,
        slope           = slope,
        interaction     = has_interaction,
        slope_var       = slope_var,
        drop_zero_cols  = drop_zero_cols
      )

      method_int <- if (nlevels(met_fac) >= 2L) as.integer(met_fac) else integer(0)

      # --------- NORMALISE TIME VECTOR (avoids 0-length in C++) ---------
      time_int <- if (has_time_levels) as.integer(time_fac) else integer(0)

      # ----------------------- LENGTH CHECKS -----------------------------
      n <- length(y_sub)
      check_same_length(subj_int,  y_sub,      arg_x = "subject", arg_y = "response")
      check_same_length(method_int, y_sub,     arg_x = "method",  arg_y = "response")
      if (length(time_int)) {
        check_same_length(time_int, y_sub, arg_x = "time", arg_y = "response")
      }

      # ------------- Decide if AR(1) is actually identifiable ------------
      has_ar1_info <- {
        if (!length(time_int)) FALSE else
          any(vapply(split(time_int, subj_int),
                     function(v) length(unique(v)) >= 2L, logical(1)))
      }
      use_ar1_eff <- identical(ar, "ar1") && isTRUE(has_ar1_info)
      ar_used_mat[i, j] <- ar_used_mat[j, i] <- isTRUE(use_ar1_eff)
      # (Optional) message once if AR1 requested but not usable:
      if (identical(ar, "ar1") && !use_ar1_eff) {
        inform_ccc_rm_ar1_fallback(
          pair_label = sprintf("%s vs %s", m1, m2),
          .verbose = verbose
        )
      }

      Zp <- if (!identical(slope, "custom")) Laux$Z else {
        if (is.null(slope_Z)) NULL else as.matrix(slope_Z)[idx, , drop = FALSE]
      }

      ## Decide inclusions for this pair
      inc_pair <- if (identical(vc_select, "none")) {
        list(
          subj_method = if (!is.null(include_subj_method)) isTRUE(include_subj_method) else Laux$nm > 0,
          subj_time   = if (!is.null(include_subj_time))   isTRUE(include_subj_time)   else Laux$nt > 0
        )
      } else if (identical(vc_select, "auto")) {
        # Use the stable full-model fit path for automatic selection. The
        # likelihood-refit branch used for variance-component LRT selection has
        # shown platform-specific native instability in CI, whereas the direct
        # pairwise REML fit remains stable.
        list(
          subj_method = Laux$nm > 0,
          subj_time   = Laux$nt > 0
        )
      } else NULL

      {
        inc_subj_method_eff <- if (is.null(inc_pair)) (Laux$nm > 0) else inc_pair$subj_method
        inc_subj_time_eff   <- if (is.null(inc_pair)) (Laux$nt > 0) else inc_pair$subj_time

        rho_used <- if (use_ar1_eff && is.na(ar_rho)) {
          er <- estimate_rho(
            Xp, y_sub, subj_int, method_int, time_int, Laux, Zp,
            max_iter = max_iter, tol = tol, conf_level = conf_level,
            ci_mode_int = ci_mode_int,
            include_subj_method = inc_subj_method_eff,
            include_subj_time   = inc_subj_time_eff,
            sb_zero_tol = sb_zero_tol,
            eval_single_visit = eval_single_visit,
            time_weights = time_weights_kappa,
            metric_mode = metric_mode
          )
          er$rho
        } else ar_rho

        ans <- tryCatch(
          run_cpp(
            Xp, y_sub, subj_int, method_int, time_int, Laux, Zp,
            use_ar1 = use_ar1_eff,
            ar1_rho = if (use_ar1_eff) rho_used else 0,
            max_iter = max_iter, tol = tol, conf_level = conf_level,
            ci_mode_int = ci_mode_int,
            include_subj_method = inc_subj_method_eff,
            include_subj_time   = inc_subj_time_eff,
            sb_zero_tol = sb_zero_tol,
            eval_single_visit = eval_single_visit,
            time_weights = time_weights_kappa,
            metric_mode = metric_mode,
            need_reml_loglik = FALSE
          ),
          error = function(e) {
            cli::cli_warn(
              "ccc_vc_cpp failed for pair ({.val {m1}}, {.val {m2}}): {conditionMessage(e)}"
            )
            NULL
          }
        )
        ans <- infer_ci_method(ans, ci_mode_int, conf_level)
      }

      # ---- record results (common to both branches) ----
      rho_mat[i, j] <- rho_mat[j, i] <- as.numeric(rho_used)

      val <- if (is.null(ans)) NA_real_ else num_or_na(ans[[estimate_key]])
      est_mat[i, j] <- est_mat[j, i] <- val

      if (!is.null(ans)) {
        vc_subject[i, j]        <- vc_subject[j, i]        <- num_or_na(ans[["sigma2_subject"]])
        vc_subject_method[i, j] <- vc_subject_method[j, i] <- num_or_na(ans[["sigma2_subject_method"]])
        vc_subject_time[i, j]   <- vc_subject_time[j, i]   <- num_or_na(ans[["sigma2_subject_time"]])
        vc_error[i, j]          <- vc_error[j, i]          <- num_or_na(ans[["sigma2_error"]])

        extra_vec <- ans[["sigma2_extra"]]
        vc_extra[i, j] <- list(extra_vec)
        vc_extra[j, i] <- list(extra_vec)

        vc_SB[i, j]             <- vc_SB[j, i]             <- num_or_na(ans[["SB"]])
        vc_se_metric[i, j]      <- vc_se_metric[j, i]      <- num_or_na(ans[[se_key]])

        ar1_rho_lag1_mat[i, j] <- ar1_rho_lag1_mat[j, i] <- num_or_na(ans[["ar1_rho_lag1"]])
        ar1_pairs_mat[i, j]    <- ar1_pairs_mat[j, i]    <- suppressWarnings(as.integer(ans[["ar1_pairs"]]))
        ar1_pval_mat[i, j]     <- ar1_pval_mat[j, i]     <- num_or_na(ans[["ar1_pval"]])
        ar1_reco <- isTRUE(ans[["use_ar1"]])
        ar1_reco_mat[i, j]     <- ar1_reco_mat[j, i]     <- ar1_reco

        if (isTRUE(verbose)) {
          .vc_message(ans, label = sprintf("Pair: %s vs %s", m1, m2),
                      nm = Laux$nm, nt = Laux$nt,
                      conf_level = conf_level, digits = digits,
                      use_message = use_message,
                      extra_label = extra_label, ar = if (use_ar1_eff) "ar1" else "none",
                      ar_rho = if (use_ar1_eff) rho_used else NA_real_,
                      engine_name = vc_engine_name,
                      se_label = vc_se_label)
        }
      }

      if (isTRUE(ci)) {
        lwr_cpp <- num_or_na(if (!is.null(ans)) ans[["lwr"]] else NA_real_)
        upr_cpp <- num_or_na(if (!is.null(ans)) ans[["upr"]] else NA_real_)
        if (is.na(lwr_cpp) || is.na(upr_cpp)) {
          se_cpp <- num_or_na(if (!is.null(ans)) ans[[se_key]] else NA_real_)
          ci2 <- compute_ci_from_se(num_or_na(val), se_cpp, conf_level)
          lwr_cpp <- ci2[1]; upr_cpp <- ci2[2]
        }
        lwr_mat[i, j] <- lwr_mat[j, i] <- lwr_cpp
        upr_mat[i, j] <- upr_mat[j, i] <- upr_cpp
      }
    }
  }

  if (!identical(ar, "ar1")) {
    if (any(ar1_reco_mat == TRUE, na.rm = TRUE)) {
      inform_if_verbose(
        "Positive lag-1 residual correlation detected in at least one pair. Consider `ar = \"ar1\"` to model within-subject serial correlation.",
        .verbose = verbose
      )
    }
  }

  residual_model_eff <- if (!identical(ar, "ar1")) {
    "iid"
  } else if (any(ar_used_mat, na.rm = TRUE)) {
    "ar1"
  } else {
    "iid"
  }

  diag(est_mat) <- 1
  if (isTRUE(ci)) {
    diag(lwr_mat) <- NA_real_
    diag(upr_mat) <- NA_real_
    ci_classes <- if (identical(out_class, "ccc_rm_reml")) {
      c("ccc_rm_reml", "matrixCorr_ccc_ci", "matrixCorr_ccc", "ccc")
    } else {
      c(out_class, "icc_ci", "icc")
    }
    out <- structure(list(est = est_mat, lwr.ci = lwr_mat, upr.ci = upr_mat),
                     class = ci_classes)
    attr(out, "method")      <- out_method
    attr(out, "description") <- out_description
    attr(out, "package")     <- "matrixCorr"
    attr(out, "conf.level")  <- conf_level
    attr(out, "residual_model") <- residual_model_eff
    if (identical(residual_model_eff, "ar1")) attr(out, "ar_rho") <- rho_mat

    attr(out, "sigma2_subject")        <- vc_subject
    attr(out, "sigma2_subject_method") <- vc_subject_method
    attr(out, "sigma2_subject_time")   <- vc_subject_time
    attr(out, "sigma2_error")          <- vc_error
    attr(out, "sigma2_extra")          <- vc_extra
    attr(out, "SB")                    <- vc_SB
    attr(out, se_key)                  <- vc_se_metric
    attr(out, "n_obs")                 <- n_obs_mat
    attr(out, "n_subjects")            <- n_subjects_mat

    if (!identical(ar, "ar1")) {
      attr(out, "ar1_rho_lag1") <- ar1_rho_lag1_mat
      attr(out, "ar1_pairs")    <- ar1_pairs_mat
      attr(out, "ar1_pval")     <- ar1_pval_mat
      attr(out, "use_ar1")      <- ar1_reco_mat
    }

    return(out)
  } else {
    est_classes <- if (identical(out_class, "ccc_rm_reml")) {
      c("ccc_rm_reml", "matrixCorr_ccc", "ccc", "matrix")
    } else {
      c(out_class, "icc", "matrix")
    }
    out <- structure(est_mat, class = est_classes)
    attr(out, "method")      <- out_method
    attr(out, "description") <- out_description
    attr(out, "package")     <- "matrixCorr"
    attr(out, "residual_model") <- residual_model_eff
    if (identical(residual_model_eff, "ar1")) attr(out, "ar_rho") <- rho_mat

    attr(out, "sigma2_subject")        <- vc_subject
    attr(out, "sigma2_subject_method") <- vc_subject_method
    attr(out, "sigma2_subject_time")   <- vc_subject_time
    attr(out, "sigma2_error")          <- vc_error
    attr(out, "sigma2_extra")          <- vc_extra
    attr(out, "SB")                    <- vc_SB
    attr(out, se_key)                  <- vc_se_metric
    attr(out, "n_obs")                 <- n_obs_mat
    attr(out, "n_subjects")            <- n_subjects_mat

    if (!identical(ar, "ar1")) {
      attr(out, "ar1_rho_lag1") <- ar1_rho_lag1_mat
      attr(out, "ar1_pairs")    <- ar1_pairs_mat
      attr(out, "ar1_pval")     <- ar1_pval_mat
      attr(out, "use_ar1")      <- ar1_reco_mat
    }

    return(out)
  }
}


#' @keywords internal
.Dmat_normalise_mass <- function(D, target_mass) {
  if (is.null(D)) return(NULL)
  one <- rep(1, nrow(D))
  mass <- as.numeric(t(one) %*% D %*% one)
  # leave as is; 'C++' will guard S_B
  if (!is.finite(mass) || mass <= 0) return(D)
  D * (target_mass / mass)
}

#' @keywords internal
.Dmat_build_kernel <- function(nt, type = c("time-avg","typical-visit","weighted-avg","weighted-sq"),
                               w = NULL, rescale = TRUE) {
  type <- match.arg(type)
  if (nt < 2) return(NULL)
  if (type %in% c("weighted-avg","weighted-sq")) {
    if (is.null(w)) w <- rep(1/nt, nt)
    w <- check_weights(w, n = nt, arg = "Dmat_weights")
    sw <- sum(w)
    if (sw <= 0) {
      abort_bad_arg("Dmat_weights",
        message = "must sum to a positive finite number."
      )
    }
    w <- w / sw
  }
  D <- switch(type,
              "time-avg"      = (1/nt) * matrix(1, nt, nt),
              "typical-visit" = diag(nt),
              "weighted-avg"  = nt * (w %o% w),
              "weighted-sq"   = nt * diag(w)
  )
  if (rescale) D <- .Dmat_normalise_mass(D, nt)
  # symmetrise softly for safety
  0.5 * (D + t(D))
}

#' Align (optional named) weights to a subset of time levels
#' @keywords internal
.align_weights_to_levels <- function(w, present_lvls, all_lvls) {
  if (is.null(w)) return(NULL)
  if (!is.null(names(w))) {
    idx <- match(present_lvls, all_lvls)
    if (anyNA(idx)) {
      abort_bad_arg("Dmat_weights",
        message = "Present time levels not found in {.arg all_time_lvls}."
      )
    }
    w_all <- rep(NA_real_, length(all_lvls))
    w_all[seq_along(all_lvls)] <- w[all_lvls]
    w_sub <- w_all[idx]
    if (anyNA(w_sub)) {
      abort_bad_arg("Dmat_weights",
        message = "Missing weights for some present time levels."
      )
    }
    as.numeric(w_sub)
  } else {
    if (length(w) != length(present_lvls)) {
      abort_bad_arg("Dmat_weights",
        message = "Unnamed weights must have length equal to the number of present time levels."
      )
    }
    as.numeric(w)
  }
}

#' Print method for matrixCorr CCC objects
#'
#' @param x A `matrixCorr_ccc` or `matrixCorr_ccc_ci` object.
#' @param digits Number of digits for CCC estimates.
#' @param ci_digits Number of digits for CI bounds.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci One of `"yes"` or `"no"`.
#' @param ... Passed to underlying printers.
#' @export
#' @method print matrixCorr_ccc
print.matrixCorr_ccc <- function(x,
                                 digits = 4,
                                 ci_digits = 4,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 show_ci = NULL,
                                 ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("print_show_ci", "yes")
  )
  is_ci_obj <- inherits(x, "matrixCorr_ccc_ci") ||
    (is.list(x) && all(c("est", "lwr.ci", "upr.ci") %in% names(x)))

  if (is_ci_obj) {
    est <- as.matrix(x$est)
    lwr <- as.matrix(x$lwr.ci)
    upr <- as.matrix(x$upr.ci)
  } else if (is.matrix(x)) {
    est <- as.matrix(x)
    lwr <- matrix(NA_real_, nrow(est), ncol(est), dimnames = dimnames(est))
    upr <- lwr
  } else {
    abort_bad_arg("x",
      message = "must be a matrix or a list with elements `est`, `lwr.ci`, and `upr.ci`."
    )
  }

  rn <- rownames(est); cn <- colnames(est)
  if (is.null(rn)) rn <- paste0("m", seq_len(nrow(est)))
  if (is.null(cn)) cn <- rn

  .mc_print_corr_matrix(
    x,
    header = "Repeated-measures concordance matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    mat = est,
    ...
  )
  invisible(x)
}

#' Print method for matrixCorr CCC objects with CIs
#'
#' @inheritParams print.matrixCorr_ccc
#' @export
#' @method print matrixCorr_ccc_ci
print.matrixCorr_ccc_ci <- function(x, ...) {
  print.matrixCorr_ccc(x, ...)
}

#' S3 print for legacy class `ccc_ci`
#'
#' For compatibility with objects that still carry class `"ccc_ci"`.
#' @inheritParams print.matrixCorr_ccc
#' @export
#' @method print ccc_ci
print.ccc_ci <- function(x, ...) {
  print.matrixCorr_ccc(x, ...)
}

#' @title Summary Method for `ccc_rm_reml` Objects
#'
#' @description Produces a detailed summary of a `"ccc_rm_reml"` object, including
#' Lin's CCC estimates and associated variance component estimates per method pair.
#'
#' @param object An object of class `"ccc_rm_reml"`, as returned by [ccc_rm_reml()].
#' @param digits Integer; number of decimal places to round CCC estimates and components.
#' @param ci_digits Integer; decimal places for confidence interval bounds.
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives
#'   this from console width.
#' @param width Optional display width; defaults to `getOption("width")`.
#' @param show_ci Character string indicating whether to show confidence
#'   intervals: `"yes"` shows CI information when available and `"no"`
#'   suppresses it.
#' @param ... Additional arguments (ignored).
#'
#' @return A data frame of class `"summary.ccc_rm_reml"` with columns:
#'   \code{item1}, \code{item2}, \code{estimate}, and optionally \code{lwr},
#'   \code{upr}, plus canonical repeated-measures counts \code{n_subjects} and
#'   \code{n_obs}. Method-specific columns retain the scientific variance
#'   component outputs: \code{sigma2_subject}, \code{sigma2_subject_method},
#'   \code{sigma2_subject_time}, \code{sigma2_error}, \code{sigma2_extra},
#'   \code{SB}, and \code{se_ccc}.
#'
#' @export
#' @method summary ccc_rm_reml
summary.ccc_rm_reml <- function(object,
                                 digits = 4,
                                 ci_digits = 2,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 show_ci = NULL,
                                 ...) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )

  # Base CCC summary (handles CI and formatting choices)
  base_summary <- summary.ccc(object, digits = digits,
                              ci_digits = ci_digits, topn = topn,
                              max_vars = max_vars, width = width,
                              show_ci = show_ci)

  # Pull the estimate matrix to know the size/order of pairs
  est_mat <- if (is.list(object) && !is.null(object$est)) {
    as.matrix(object$est)
  } else {
    as.matrix(object)
  }

  rn <- rownames(est_mat); cn <- colnames(est_mat)
  if (is.null(rn)) rn <- as.character(seq_len(nrow(est_mat)))
  if (is.null(cn)) cn <- as.character(seq_len(ncol(est_mat)))

  n_pairs <- if (nrow(est_mat) == 1L && ncol(est_mat) == 1L) 1L else {
    nrow(est_mat) * (ncol(est_mat) - 1L) / 2L
  }

  # Helper: extract VC values in the same order as summary.ccc builds rows (numeric-only)
  extract_pairs_num <- function(val) {
    # Overall 1x1
    if (nrow(est_mat) == 1L && ncol(est_mat) == 1L) {
      if (is.null(val)) return(NA_real_)
      if (is.matrix(val)) return(suppressWarnings(as.numeric(val[1, 1])))
      return(suppressWarnings(as.numeric(val[1])))
    }
    # Pairwise
    out <- numeric(n_pairs)
    k <- 0L
    for (i in seq_len(nrow(est_mat) - 1L)) {
      for (j in (i + 1L):ncol(est_mat)) {
        k <- k + 1L
        if (is.null(val)) {
          out[k] <- NA_real_
        } else if (is.matrix(val)) {
          out[k] <- suppressWarnings(as.numeric(val[i, j]))
        } else {
          vv <- suppressWarnings(as.numeric(val))
          out[k] <- if (length(vv) == 1L) vv else NA_real_
        }
      }
    }
    out
  }

  # UPDATED: extract sigma2_extra as list of numeric vectors (one per pair / overall)
  extract_pairs_extra_list <- function(val) {
    # Overall 1x1
    if (nrow(est_mat) == 1L && ncol(est_mat) == 1L) {
      if (is.null(val)) return(list(NULL))
      if (is.matrix(val) && typeof(val) == "list") {
        return(list(val[[1, 1]]))
      }
      # could be a plain numeric vector
      return(list(suppressWarnings(as.numeric(if (is.matrix(val)) val[1, 1] else val))))
    }
    # Pairwise
    if (is.matrix(val) && typeof(val) == "list") {
      out <- vector("list", n_pairs)
      k <- 0L
      for (i in seq_len(nrow(est_mat) - 1L)) {
        for (j in (i + 1L):ncol(est_mat)) {
          k <- k + 1L
          out[[k]] <- val[[i, j]]
        }
      }
      return(out)
    } else if (is.null(val)) {
      return(vector("list", n_pairs))
    } else {
      # fallback: recycle a single numeric(vector) across pairs
      vv <- suppressWarnings(as.numeric(val))
      return(replicate(n_pairs, vv, simplify = FALSE))
    }
  }

  vc_names_num <- c("sigma2_subject",
                    "sigma2_subject_method",
                    "sigma2_subject_time",
                    "sigma2_error",
                    "SB",
                    "se_ccc")

  # Numeric VCs (same as before)
  vc_cols_num <- lapply(vc_names_num, function(nm) extract_pairs_num(attr(object, nm)))
  names(vc_cols_num) <- vc_names_num

  # sigma2_extra as list of vectors (may be NULL / varying lengths)
  extra_list <- extract_pairs_extra_list(attr(object, "sigma2_extra"))
  # Round each vector
  extra_list <- lapply(extra_list, function(v) if (is.null(v)) NULL else round(as.numeric(v), digits))

  # Determine max number of extra components across pairs/overall
  max_k <- 0L
  for (v in extra_list) if (!is.null(v)) max_k <- max(max_k, length(v))
  # Build extra columns sigma2_extra1..K (NA where not available)
  extra_cols <- list()
  if (max_k > 0L) {
    for (k in seq_len(max_k)) {
      colk <- rep(NA_real_, n_pairs)
      for (r in seq_len(n_pairs)) {
        v <- extra_list[[r]]
        if (!is.null(v) && length(v) >= k) colk[r] <- v[k]
      }
      extra_cols[[paste0("sigma2_extra", k)]] <- colk
    }
  }

  # Assemble output
  out <- base_summary
  out$n_subjects <- as.integer(extract_pairs_num(attr(object, "n_subjects")))
  out$n_obs <- as.integer(extract_pairs_num(attr(object, "n_obs")))

  # Attach numeric VC columns (rounded)
  for (nm in vc_names_num) {
    out[[nm]] <- as.numeric(round(vc_cols_num[[nm]], digits))
  }

  # Attach expanded extra columns
  if (length(extra_cols)) {
    for (nm in names(extra_cols)) {
      out[[nm]] <- as.numeric(extra_cols[[nm]])
    }
  } else {
    # keep a single column to signal absence, if you prefer:
    # out[["sigma2_extra1"]] <- NA_real_
  }

  extract_pairs_num_mat <- function(val) {
    # Overall 1x1
    if (nrow(est_mat) == 1L && ncol(est_mat) == 1L) {
      return(suppressWarnings(as.numeric(if (is.matrix(val)) val[1,1] else val)))
    }
    outv <- numeric(n_pairs); k <- 0L
    for (i in seq_len(nrow(est_mat) - 1L)) {
      for (j in (i + 1L):ncol(est_mat)) {
        k <- k + 1L
        outv[k] <- suppressWarnings(as.numeric(if (is.matrix(val)) val[i, j] else val))
      }
    }
    outv
  }
  # helper to pull logical per-pair
  extract_pairs_logi_mat <- function(val) {
    as_logi <- function(x) isTRUE(x)
    # Overall 1x1
    if (nrow(est_mat) == 1L && ncol(est_mat) == 1L) {
      v <- if (is.matrix(val)) val[1,1] else val
      return(as_logi(v))
    }
    outv <- logical(n_pairs); k <- 0L
    if (is.matrix(val)) {
      for (i in seq_len(nrow(est_mat) - 1L)) {
        for (j in (i + 1L):ncol(est_mat)) {
          k <- k + 1L
          outv[k] <- as_logi(val[i, j])
        }
      }
    } else {
      outv[] <- as_logi(val)
    }
    outv
  }

  has_attr <- function(obj, nm) !is.null(attr(obj, nm))

  ar1_cols <- list()

  if (has_attr(object, "ar_rho")) {
    ar1_rho_attr <- attr(object, "ar_rho")
    ar1_cols$ar1_rho <- extract_pairs_num_mat(ar1_rho_attr)
  }

  if (has_attr(object, "ar1_rho_lag1")) {
    ar1_rho_lag1_attr <- attr(object, "ar1_rho_lag1")
    ar1_cols$ar1_rho_lag1 <- extract_pairs_num_mat(ar1_rho_lag1_attr)
  }
  if (has_attr(object, "ar1_pairs")) {
    ar1_pairs_attr <- attr(object, "ar1_pairs")
    ar1_cols$ar1_pairs <- extract_pairs_num_mat(ar1_pairs_attr)
  }
  if (has_attr(object, "ar1_pval")) {
    ar1_pval_attr <- attr(object, "ar1_pval")
    ar1_cols$ar1_pval <- extract_pairs_num_mat(ar1_pval_attr)
  }
  if (has_attr(object, "use_ar1")) {
    ar1_reco_attr <- attr(object, "use_ar1")
    ar1_cols$use_ar1 <- extract_pairs_logi_mat(ar1_reco_attr)
  }

  # then, after you build `out`, add only what you have:
  if (!is.null(ar1_cols$ar1_rho)) {
    out[["ar1_rho"]] <- ifelse(is.finite(ar1_cols$ar1_rho),
                               round(ar1_cols$ar1_rho, digits), NA_real_)
  }
  if (!is.null(ar1_cols$ar1_rho_lag1)) {
    vals <- ifelse(is.finite(ar1_cols$ar1_rho_lag1),
                   round(ar1_cols$ar1_rho_lag1, digits), NA_real_)
    out[["ar1_rho_lag1"]] <- vals
    out[["ar1_rho_mom"]]  <- vals  # synonym kept for compatibility/tests
  }
  if (!is.null(ar1_cols$ar1_pairs)) {
    out[["ar1_pairs"]] <- ifelse(is.finite(ar1_cols$ar1_pairs),
                                 as.integer(ar1_cols$ar1_pairs), NA_integer_)
  }
  if (!is.null(ar1_cols$ar1_pval)) {
    out[["ar1_pval"]] <- ifelse(is.finite(ar1_cols$ar1_pval),
                                round(ar1_cols$ar1_pval, digits), NA_real_)
  }
  if (!is.null(ar1_cols$use_ar1)) {
    out[["use_ar1"]]        <- ar1_cols$use_ar1
    out[["ar1_recommend"]]  <- ar1_cols$use_ar1
  }

  attr(out, "conf.level") <- attr(base_summary, "conf.level")
  attr(out, "has_ci")     <- attr(base_summary, "has_ci")
  attr(out, "digits")     <- digits
  attr(out, "ci_digits")  <- ci_digits
  .mc_finalize_summary_df(
    out,
    class_name = "summary.ccc_rm_reml",
    repeated = TRUE
  )
}

.print_ccc_rm_reml_summary_blocks <- function(x, ...) {
  extra_cols <- grep("^sigma2_extra", names(x), value = TRUE)

  sections <- list(
    list(
      title = "Concordance estimates",
      cols = c("item1", "item2", "estimate", "lwr", "upr", "n_subjects", "n_obs", "SB", "se_ccc")
    ),
    list(
      title = "Variance components",
      cols = c("sigma2_subject", "sigma2_subject_method", "sigma2_subject_time",
               "sigma2_error", extra_cols)
    ),
    list(
      title = "AR(1) diagnostics",
      cols = c("ar1_rho", "ar1_rho_lag1", "ar1_rho_mom", "ar1_pairs",
               "ar1_pval", "use_ar1", "ar1_recommend")
    )
  )

  printed <- 0L
  for (section in sections) {
    cols <- unique(section$cols[section$cols %in% names(x)])
    if (!length(cols)) next

    if (printed > 0L) cat("\n")
    cat(section$title, "\n\n", sep = "")
    print.data.frame(x[cols], row.names = FALSE, right = FALSE, ...)
    printed <- printed + 1L
  }

  invisible(NULL)
}

#' @rdname summary.ccc_rm_reml
#' @method print summary.ccc_rm_reml
#' @param x An object of class \code{"summary.ccc_rm_reml"}.
#' @param ... Passed to \code{\link[base]{print.data.frame}}.
#' @export
print.summary.ccc_rm_reml <- function(x, digits = NULL, n = NULL,
                                      topn = NULL, max_vars = NULL,
                                      width = NULL, show_ci = NULL, ...) {
  show_ci <- .mc_resolve_show_ci(show_ci, context = "summary")
  has_ci <- isTRUE(attr(x, "has_ci")) ||
    all(c("lwr", "upr") %in% names(x))
  cl <- suppressWarnings(as.numeric(attr(x, "conf.level")))
  if (!is.finite(cl)) cl <- NA_real_

  header <- .mc_header_with_ci("Repeated-measures concordance (REML)", cl, if (has_ci) show_ci else "no")
  .mc_print_sectioned_table(
    x,
    sections = list(
      list(
        title = "Concordance estimates",
        cols = c("item1", "item2", "estimate", "lwr", "upr", "n_subjects", "n_obs", "SB", "se_ccc")
      ),
      list(
        title = "Variance components",
        cols = c("sigma2_subject", "sigma2_subject_method", "sigma2_subject_time",
                 "sigma2_error", grep("^sigma2_extra", names(x), value = TRUE))
      ),
      list(
        title = "AR(1) diagnostics",
        cols = c("ar1_rho", "ar1_rho_lag1", "ar1_rho_mom",
                 "ar1_pairs", "ar1_pval", "use_ar1", "ar1_recommend")
      )
    ),
    header = header,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

# Shared correlation-result helpers.
#
# These helpers are used by several correlation and agreement estimators. Keep
# them outside estimator-specific files so result construction does not depend
# on latent-correlation implementation details.

.mc_square_dimnames <- function(names) {
  list(names, names)
}

.mc_set_matrix_dimnames <- function(x, row_names = NULL, col_names = row_names) {
  if (is.null(row_names) && is.null(col_names)) {
    return(x)
  }
  dimnames(x) <- list(row_names, col_names)
  x
}

.mc_resolve_corr_na <- function(na_method,
                                dots = list(),
                                na_method_missing = FALSE,
                                allowed = c("error", "pairwise", "complete"),
                                warn = TRUE) {
  if (!length(dots) && isTRUE(na_method_missing)) {
    return(list(na_method = "error", check_na = TRUE))
  }

  legacy_args <- .mc_extract_legacy_aliases(dots, allowed = "check_na")
  resolve_na_args(
    na_method = na_method,
    check_na = legacy_args$check_na %||% NULL,
    na_method_missing = na_method_missing,
    allowed = allowed,
    warn = warn
  )
}

.mc_prepare_corr_input <- function(data,
                                   na_cfg,
                                   min_n = 2L,
                                   arg = "data") {
  numeric_data <- validate_corr_input(data, check_na = na_cfg$check_na)
  diagnostics <- NULL

  if (identical(na_cfg$na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = min_n, arg = arg)
    numeric_data <- cc$data
    diagnostics <- cc$diagnostics
  }

  col_names <- colnames(numeric_data)
  list(
    data = numeric_data,
    colnames = col_names,
    dimnames = if (is.null(col_names)) NULL else .mc_square_dimnames(col_names),
    diagnostics = diagnostics
  )
}

.mc_with_omp_threads <- function(n_threads,
                                 code) {
  prev_threads <- .mc_prepare_omp_threads(
    n_threads
  )
  if (!is.null(prev_threads)) {
    on.exit(.mc_exit_omp_threads(prev_threads), add = TRUE)
  }
  force(code)
}

.mc_prepare_corr_output <- function(output = c("matrix", "sparse", "edge_list"),
                                    threshold = 0,
                                    diag = TRUE,
                                    thresholded = FALSE) {
  validate <- if (isTRUE(thresholded)) {
    .mc_validate_thresholded_output_request
  } else {
    .mc_validate_output_args
  }
  validate(output = output, threshold = threshold, diag = diag)
}

.mc_structure_corr_matrix <- function(mat, class_name, method, description,
                                      diagnostics = NULL, thresholds = NULL,
                                      correct = NULL, dimnames = NULL,
                                      symmetric = NULL,
                                      extra_attrs = NULL,
                                      classes = c(class_name, "matrix")) {
  if (!is.null(dimnames)) {
    dimnames(mat) <- dimnames
  }
  symmetric_flag <- symmetric
  if (is.null(symmetric_flag)) {
    symmetric_flag <- isTRUE(nrow(mat) == ncol(mat)) &&
      isTRUE(isSymmetric(mat, check.attributes = FALSE))
  }
  keep_classes <- setdiff(
    classes,
    c(class_name, "matrix", "corr_matrix", "corr_result")
  )
  .mc_new_corr_matrix(
    mat = mat,
    estimator_class = class_name,
    method = method,
    description = description,
    output = "matrix",
    threshold = 0,
    diag = TRUE,
    diagnostics = diagnostics,
    ci = attr(mat, "ci", exact = TRUE),
    conf.level = attr(mat, "conf.level", exact = TRUE),
    symmetric = symmetric_flag,
    extra_attrs = c(
      list(
        thresholds = thresholds,
        correct = correct
      ),
      extra_attrs %||% list()
    ),
    extra_classes = keep_classes
  )
}

.mc_finalize_corr_result <- function(mat,
                                     class_name,
                                     method,
                                     description,
                                     output_cfg,
                                     diagnostics = NULL,
                                     dimnames = NULL,
                                     symmetric = NULL,
                                     extra_attrs = NULL,
                                     classes = c(class_name, "matrix")) {
  out <- .mc_structure_corr_matrix(
    mat = mat,
    class_name = class_name,
    method = method,
    description = description,
    diagnostics = diagnostics,
    dimnames = dimnames,
    symmetric = symmetric,
    extra_attrs = extra_attrs,
    classes = classes
  )

  .mc_finalize_corr_output_fast(
    out,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    diag = output_cfg$diag
  )
}

.mc_finalize_edge_list_zero_threshold <- function(triplets,
                                                  estimator_class,
                                                  method,
                                                  description,
                                                  source_dim,
                                                  source_dimnames = NULL,
                                                  diag = TRUE,
                                                  diagnostics = NULL,
                                                  ci = NULL,
                                                  conf.level = NULL,
                                                  symmetric = TRUE,
                                                  package_name = "matrixCorr",
                                                  extra_attrs = list()) {
  ii <- as.integer(triplets$i)
  jj <- as.integer(triplets$j)
  vv <- as.numeric(triplets$x)

  rn <- NULL
  cn <- NULL
  if (is.list(source_dimnames) && length(source_dimnames) == 2L) {
    rn <- source_dimnames[[1L]]
    cn <- source_dimnames[[2L]]
  }

  row_out <- if (is.null(rn)) as.character(ii) else rn[ii]
  col_out <- if (is.null(cn)) as.character(jj) else cn[jj]

  out <- .mc_new_corr_edge_list(
    df = data.frame(
      row = row_out,
      col = col_out,
      value = vv,
      stringsAsFactors = FALSE,
      check.names = FALSE
    ),
    estimator_class = estimator_class,
    method = method,
    description = description,
    threshold = 0,
    diag = diag,
    diagnostics = diagnostics,
    ci = ci,
    conf.level = conf.level,
    source_dim = source_dim,
    source_dimnames = source_dimnames,
    symmetric = symmetric,
    package_name = package_name,
    extra_attrs = extra_attrs
  )

  attr(out, "matrixCorr_meta") <- list(
    source_class = estimator_class,
    method = method,
    description = description,
    package = package_name,
    diagnostics = diagnostics
  )
  out
}

.mc_try_direct_triplet_output <- function(input,
                                          output_cfg,
                                          kernel_threshold,
                                          estimator_class,
                                          method,
                                          description,
                                          na_method = "error",
                                          ci = FALSE,
                                          has_inference = FALSE,
                                          pairwise = FALSE,
                                          symmetric = TRUE,
                                          diagnostics = NULL,
                                          conf.level = NULL,
                                          extra_attrs = list()) {
  if (is.null(kernel_threshold)) {
    return(NULL)
  }

  source_dim <- as.integer(c(ncol(input$data), ncol(input$data)))
  can_use_error_no_ci <- identical(na_method, "error") &&
    isFALSE(ci) &&
    isFALSE(has_inference) &&
    !isTRUE(pairwise)

  if (identical(output_cfg$output, "edge_list") &&
      isTRUE(output_cfg$threshold == 0) &&
      isTRUE(can_use_error_no_ci)) {
    trip <- kernel_threshold(
      input$data,
      threshold = 0,
      diag = output_cfg$diag
    )
    return(.mc_finalize_edge_list_zero_threshold(
      triplets = trip,
      estimator_class = estimator_class,
      method = method,
      description = description,
      source_dim = source_dim,
      source_dimnames = input$dimnames,
      diag = output_cfg$diag,
      diagnostics = diagnostics,
      conf.level = conf.level,
      symmetric = symmetric,
      extra_attrs = extra_attrs
    ))
  }

  if (.mc_supports_direct_threshold_path(
    method = method,
    na_method = na_method,
    ci = ci,
    output = output_cfg$output,
    threshold = output_cfg$threshold,
    pairwise = isTRUE(pairwise),
    has_ci = ci,
    has_inference = has_inference,
    symmetric = symmetric
  )) {
    trip <- kernel_threshold(
      input$data,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag
    )
    return(.mc_finalize_triplets_output(
      triplets = trip,
      output = output_cfg$output,
      estimator_class = estimator_class,
      method = method,
      description = description,
      threshold = output_cfg$threshold,
      diag = output_cfg$diag,
      source_dim = source_dim,
      source_dimnames = input$dimnames,
      diagnostics = diagnostics,
      conf.level = conf.level,
      symmetric = symmetric,
      extra_attrs = extra_attrs
    ))
  }

  NULL
}

.mc_pairwise_ci_attr <- function(result,
                                 pairwise,
                                 colnames,
                                 conf_level_name = "conf_level") {
  list(
    est = .mc_set_matrix_dimnames(unclass(result), colnames),
    lwr.ci = .mc_set_matrix_dimnames(unclass(pairwise$lwr), colnames),
    upr.ci = .mc_set_matrix_dimnames(unclass(pairwise$upr), colnames),
    conf.level = pairwise[[conf_level_name]]
  )
}

.mc_ci_attr <- function(x) {
  attr(x, "ci", exact = TRUE)
}

.mc_inference_attr <- function(x) {
  attr(x, "inference", exact = TRUE)
}

.mc_round_finite <- function(x, digits) {
  x <- as.numeric(x)
  if (is.null(digits)) {
    return(x)
  }
  out <- rep(NA_real_, length(x))
  keep <- is.finite(x)
  out[keep] <- round(x[keep], digits)
  out
}

.mc_pairwise_matrix_summary <- function(object,
                                        class_name,
                                        digits,
                                        ci_digits = NULL,
                                        p_digits = NULL,
                                        show_ci = NULL,
                                        ci_attr = .mc_ci_attr(object),
                                        inference_attr = .mc_inference_attr(object),
                                        diagnostics_attr = attr(object, "diagnostics", exact = TRUE),
                                        include_n_complete = TRUE,
                                        include_ci = TRUE,
                                        include_p = TRUE,
                                        extra_columns = NULL,
                                        extra_attrs = list(),
                                        overview = NULL) {
  est <- as.matrix(object)
  nr <- nrow(est)
  nc <- ncol(est)
  rn <- rownames(est)
  cn <- colnames(est)
  if (is.null(rn)) rn <- as.character(seq_len(nr))
  if (is.null(cn)) cn <- as.character(seq_len(nc))

  n_pairs <- nr * (nc - 1L) / 2L
  var1 <- character(n_pairs)
  var2 <- character(n_pairs)
  estimate <- numeric(n_pairs)

  has_n_complete <- isTRUE(include_n_complete) &&
    is.list(diagnostics_attr) &&
    is.matrix(diagnostics_attr$n_complete)
  n_complete <- if (has_n_complete) integer(n_pairs) else NULL

  show_ci <- if (is.null(show_ci)) "yes" else show_ci
  has_ci <- isTRUE(include_ci) &&
    identical(show_ci, "yes") &&
    is.list(ci_attr)
  lwr <- if (has_ci) numeric(n_pairs) else NULL
  upr <- if (has_ci) numeric(n_pairs) else NULL

  extra_columns <- extra_columns %||% list()
  extra_values <- vector("list", length(extra_columns))
  names(extra_values) <- names(extra_columns)
  if (length(extra_columns)) {
    for (nm in names(extra_columns)) {
      spec <- extra_columns[[nm]]
      type <- spec$type %||% "numeric"
      extra_values[[nm]] <- switch(
        type,
        integer = integer(n_pairs),
        logical = logical(n_pairs),
        character = character(n_pairs),
        numeric(n_pairs)
      )
    }
  }

  k <- 0L
  for (i in seq_len(nr - 1L)) {
    for (j in (i + 1L):nc) {
      k <- k + 1L
      var1[[k]] <- rn[[i]]
      var2[[k]] <- cn[[j]]
      estimate[[k]] <- round(est[i, j], digits)
      if (!is.null(n_complete)) {
        n_complete[[k]] <- as.integer(diagnostics_attr$n_complete[i, j])
      }
      if (has_ci) {
        lwr[[k]] <- if (!is.null(ci_attr$lwr.ci) && is.finite(ci_attr$lwr.ci[i, j])) {
          round(ci_attr$lwr.ci[i, j], ci_digits)
        } else {
          NA_real_
        }
        upr[[k]] <- if (!is.null(ci_attr$upr.ci) && is.finite(ci_attr$upr.ci[i, j])) {
          round(ci_attr$upr.ci[i, j], ci_digits)
        } else {
          NA_real_
        }
      }
      if (length(extra_columns)) {
        for (nm in names(extra_columns)) {
          spec <- extra_columns[[nm]]
          mat <- spec$matrix
          value <- if (is.matrix(mat)) {
            mat[i, j]
          } else if (is.function(spec$value)) {
            spec$value(i, j)
          } else {
            NA
          }
          type <- spec$type %||% "numeric"
          extra_values[[nm]][[k]] <- switch(
            type,
            integer = as.integer(value),
            logical = isTRUE(value),
            character = as.character(value),
            .mc_round_finite(value, if ("digits" %in% names(spec)) spec$digits else digits)
          )
        }
      }
    }
  }

  df <- data.frame(
    var1 = var1,
    var2 = var2,
    estimate = as.numeric(estimate),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  if (!is.null(n_complete)) df$n_complete <- as.integer(n_complete)
  if (!is.null(lwr)) df$lwr <- as.numeric(lwr)
  if (!is.null(upr)) df$upr <- as.numeric(upr)
  if (length(extra_values)) {
    for (nm in names(extra_values)) {
      df[[nm]] <- extra_values[[nm]]
    }
  }
  rownames(df) <- NULL

  out <- .mc_finalize_summary_df(df, class_name = class_name)
  attr(out, "overview") <- overview %||% .mc_summary_corr_matrix(object)
  attr(out, "has_ci") <- has_ci
  if (!is.null(include_p)) {
    attr(out, "has_p") <- isTRUE(include_p)
  }
  attr(out, "conf.level") <- if (is.null(ci_attr)) NA_real_ else ci_attr$conf.level
  attr(out, "digits") <- digits
  if (!is.null(ci_digits)) attr(out, "ci_digits") <- ci_digits
  if (!is.null(p_digits)) attr(out, "p_digits") <- p_digits
  if (length(extra_attrs)) {
    for (nm in names(extra_attrs)) {
      attr(out, nm) <- extra_attrs[[nm]]
    }
  }
  out
}

.mc_corr_wrapper <- function(data,
                             dots,
                             na_method,
                             na_method_missing,
                             ci,
                             conf_level,
                             n_threads,
                             output,
                             threshold,
                             diag,
                             estimator_class,
                             method,
                             description,
                             kernel_matrix,
                             kernel_pairwise = NULL,
                             kernel_threshold = NULL,
                             min_n = 2L,
                             symmetric = TRUE,
                             thresholded_output = TRUE,
                             structure_matrix = NULL,
                             structure_matrix_fast = NULL) {
  output_cfg <- .mc_prepare_corr_output(
    output = output,
    threshold = threshold,
    diag = diag,
    thresholded = thresholded_output
  )

  na_cfg <- .mc_resolve_corr_na(
    na_method = na_method,
    dots = dots,
    na_method_missing = na_method_missing,
    allowed = c("error", "pairwise", "complete")
  )

  if (!isFALSE(ci)) {
    check_bool(ci, arg = "ci")
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
  } else if (!is.logical(ci) || length(ci) != 1L || is.na(ci)) {
    check_bool(ci, arg = "ci")
  }

  input <- .mc_prepare_corr_input(data, na_cfg = na_cfg, min_n = min_n)
  diagnostics <- NULL
  ci_attr <- NULL

  .mc_with_omp_threads(
    n_threads,
    {
      direct <- .mc_try_direct_triplet_output(
        input = input,
        output_cfg = output_cfg,
        kernel_threshold = kernel_threshold,
        estimator_class = estimator_class,
        method = method,
        description = description,
        na_method = na_cfg$na_method,
        ci = ci,
        has_inference = FALSE,
        pairwise = identical(na_cfg$na_method, "pairwise"),
        symmetric = symmetric
      )
      if (!is.null(direct)) {
        direct
      } else {
        if (!identical(na_cfg$na_method, "pairwise") && !isTRUE(ci)) {
          result <- kernel_matrix(input$data)
        } else {
          if (is.null(kernel_pairwise)) {
            abort_internal("Pairwise kernel is required for pairwise missingness or confidence intervals.")
          }
          pairwise <- kernel_pairwise(
            input$data,
            return_ci = ci,
            conf_level = conf_level
          )
          result <- pairwise$est
          diagnostics <- list(
            n_complete = .mc_set_matrix_dimnames(pairwise$n_complete, input$colnames)
          )
          if (isTRUE(ci)) {
            ci_attr <- .mc_pairwise_ci_attr(result, pairwise, input$colnames)
          }
        }

        diagnostics <- .mc_merge_diagnostics(diagnostics, input$diagnostics)

        if (identical(output_cfg$output, "matrix") &&
            is.null(diagnostics) &&
            is.null(ci_attr) &&
            !is.null(structure_matrix_fast)) {
          structure_matrix_fast(
            mat = result,
            dimnames = input$dimnames
          )
        } else {
          if (!is.null(structure_matrix)) {
            out <- structure_matrix(
              mat = result,
              dimnames = input$dimnames,
              diagnostics = diagnostics,
              ci_attr = ci_attr,
              conf_level = if (!is.null(ci_attr)) conf_level else NULL
            )
          } else {
            out <- .mc_structure_corr_matrix(
              result,
              class_name = estimator_class,
              method = method,
              description = description,
              symmetric = symmetric,
              diagnostics = diagnostics,
              dimnames = input$dimnames,
              extra_attrs = if (!is.null(ci_attr)) {
                list(
                  ci = ci_attr,
                  conf.level = conf_level
                )
              }
            )
          }

          .mc_finalize_corr_output_fast(
            out,
            output = output_cfg$output,
            threshold = output_cfg$threshold,
            diag = output_cfg$diag
          )
        }
      }
    }
  )
}

.mc_inference_corr_wrapper <- function(data,
                                       na_method,
                                       ci = FALSE,
                                       p_value = FALSE,
                                       conf_level = 0.95,
                                       n_threads = getOption("matrixCorr.threads", 1L),
                                       output = c("matrix", "sparse", "edge_list"),
                                       threshold = 0,
                                       diag = TRUE,
                                       estimator_class,
                                       method,
                                       description,
                                       kernel_matrix,
                                       kernel_pairwise = NULL,
                                       kernel_threshold = NULL,
                                       payload_builder = NULL,
                                       min_n = 2L,
                                       direct_method = method,
                                       symmetric = TRUE,
                                       extra_attrs = list(),
                                       n_boot = NULL,
                                       seed = NULL,
                                       arg_data = "data") {
  output_cfg <- .mc_validate_thresholded_output_request(
    output = output,
    threshold = threshold,
    diag = diag
  )
  na_method <- match.arg(na_method, c("error", "pairwise", "complete"))
  check_bool(ci, arg = "ci")
  check_bool(p_value, arg = "p_value")
  needs_ci <- isTRUE(ci)
  needs_p_value <- isTRUE(p_value)
  needs_bootstrap <- needs_ci
  if (needs_bootstrap) {
    check_prob_scalar(conf_level, arg = "conf_level", open_ends = TRUE)
    n_boot <- check_scalar_int_pos(n_boot %||% 500L, arg = "n_boot")
    if (!is.null(seed)) {
      seed <- check_scalar_int_pos(seed, arg = "seed")
    }
  } else {
    conf_level <- 0.95
    n_boot <- n_boot %||% 500L
    seed <- NULL
  }
  n_threads <- check_scalar_int_pos(n_threads, arg = "n_threads")

  numeric_data <- if (identical(na_method, "error")) {
    validate_corr_input(data)
  } else {
    validate_corr_input(data, check_na = FALSE)
  }
  diagnostics_extra <- NULL
  if (identical(na_method, "complete")) {
    cc <- .mc_complete_case_matrix(numeric_data, min_n = min_n, arg = arg_data)
    numeric_data <- cc$data
    diagnostics_extra <- cc$diagnostics
  }
  colnames_data <- colnames(numeric_data)
  dn <- if (is.null(colnames_data)) NULL else .mc_square_dimnames(colnames_data)

  .mc_with_omp_threads(
    n_threads,
    {
      direct <- .mc_try_direct_triplet_output(
        input = list(data = numeric_data, dimnames = dn),
        output_cfg = output_cfg,
        kernel_threshold = if (is.null(kernel_threshold)) {
          NULL
        } else {
          function(x, threshold, diag) {
            kernel_threshold(
              x,
              threshold = threshold,
              diag = diag,
              n_threads = n_threads
            )
          }
        },
        estimator_class = estimator_class,
        method = direct_method,
        description = description,
        na_method = na_method,
        ci = ci,
        has_inference = needs_p_value,
        pairwise = identical(na_method, "pairwise"),
        symmetric = symmetric
      )
      if (!is.null(direct)) {
        direct
      } else {
        use_pairwise_kernel <- identical(na_method, "pairwise")
        if (!isTRUE(use_pairwise_kernel)) {
          est <- kernel_matrix(numeric_data, n_threads = n_threads)
        } else {
          if (is.null(kernel_pairwise)) {
            abort_internal("Pairwise kernel is required when {.arg na_method} is {.val pairwise}.")
          }
          est <- kernel_pairwise(numeric_data, n_threads = n_threads)
        }

        est <- .mc_set_matrix_dimnames(est, colnames_data)
        payload <- NULL
        if ((isTRUE(ci) || isTRUE(p_value)) && !is.null(payload_builder)) {
          payload <- payload_builder(
            numeric_data,
            est = est,
            ci = ci,
            p_value = p_value,
            conf_level = conf_level,
            n_boot = n_boot,
            seed = seed
          )
        }

        diagnostics <- .mc_merge_diagnostics(
          if (is.null(payload)) NULL else payload$diagnostics,
          diagnostics_extra
        )

        out <- .mc_structure_corr_matrix(
          est,
          class_name = estimator_class,
          method = method,
          description = description,
          symmetric = symmetric,
          diagnostics = diagnostics,
          dimnames = dn,
          extra_attrs = c(
            extra_attrs,
            if (!is.null(payload) && !is.null(payload$ci)) {
              list(
                ci = payload$ci,
                conf.level = conf_level,
                n_boot = n_boot
              )
            },
            if (!is.null(payload) && !is.null(payload$inference)) {
              list(inference = payload$inference)
            }
          )
        )

        .mc_finalize_corr_output_fast(
          out,
          output = output_cfg$output,
          threshold = output_cfg$threshold,
          diag = output_cfg$diag
        )
      }
    }
  )
}

#' @title Pairwise Pearson correlation
#'
#' @description
#' Computes pairwise Pearson correlations for the numeric columns of a matrix
#' or data frame using a high-performance 'C++' backend. Optional Fisher-z
#' confidence intervals are available.
#'
#' @param data A numeric matrix or a data frame with at least two numeric
#' columns. All non-numeric columns will be excluded. Each column must have
#' at least two non-missing values.
#' @param na_method Character scalar controlling missing-data handling.
#'   \code{"error"} rejects missing, \code{NaN}, and infinite values.
#'   \code{"pairwise"} recomputes each correlation on its own pairwise
#'   complete-case overlap. This is permissive, but different matrix entries
#'   may be based on different rows. \code{"complete"} performs listwise
#'   deletion once across the retained numeric columns and then computes the
#'   estimator on the common complete sample.
#' @param ci Logical (default \code{FALSE}). If \code{TRUE}, attach pairwise
#' Fisher-\eqn{z} confidence intervals for the off-diagonal Pearson
#' correlations.
#' @param conf_level Confidence level used when \code{ci = TRUE}. Default is
#' \code{0.95}.
#' @param n_threads Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
#'   \code{getOption("matrixCorr.threads", 1L)}.
#' @param output Output representation for the computed estimates.
#'   \itemize{
#'   \item \code{"matrix"} (default): full dense matrix; best when you need
#'   matrix algebra, dense heatmaps, or full compatibility with existing code.
#'   \item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
#'   retained entries; best when many values are dropped by thresholding.
#'   \item \code{"edge_list"}: long-form data frame with columns
#'   \code{row}, \code{col}, \code{value}; convenient for filtering, joins,
#'   and network-style workflows.
#'   }
#' @param threshold Non-negative absolute-value filter for non-matrix outputs:
#'   keep entries with \code{abs(value) >= threshold}. Use
#'   \code{threshold > 0} when you want only stronger associations (typically
#'   with \code{output = "sparse"} or \code{"edge_list"}). Keep
#'   \code{threshold = 0} to retain all values. Must be \code{0} when
#'   \code{output = "matrix"}.
#' @param diag Logical; whether to include diagonal entries in
#'   \code{"sparse"} and \code{"edge_list"} outputs.
#'
#' @return A symmetric numeric matrix where the \code{(i, j)}-th element is
#' the Pearson correlation between the \code{i}-th and \code{j}-th
#' numeric columns of the input. When \code{ci = TRUE}, the object also
#' carries a \code{ci} attribute with elements \code{est}, \code{lwr.ci},
#' \code{upr.ci}, and \code{conf.level}. When pairwise-complete evaluation is
#' used, pairwise sample sizes are stored in \code{attr(x, "diagnostics")$n_complete}.
#'
#' @details
#' Let \eqn{X \in \mathbb{R}^{n \times p}} be a
#' numeric matrix with rows as observations and columns as variables, and let
#' \eqn{\mathbf{1} \in \mathbb{R}^n} denote the all-ones vector. Define the column
#' means \eqn{\mu = (1/n)\,\mathbf{1}^\top X} and the centred cross-product
#' matrix
#' \deqn{ S \;=\; (X - \mathbf{1}\mu)^\top (X - \mathbf{1}\mu)
#'       \;=\; X^\top \!\Big(I_n - \tfrac{1}{n}\mathbf{1}\mathbf{1}^\top\Big) X
#'       \;=\; X^\top X \;-\; n\,\mu\,\mu^\top. }
#' The (unbiased) sample covariance is
#' \deqn{ \widehat{\Sigma} \;=\; \tfrac{1}{n-1}\,S, }
#' and the sample standard deviations are \eqn{s_i = \sqrt{\widehat{\Sigma}_{ii}}}.
#' The Pearson correlation matrix is obtained by standardising \eqn{\widehat{\Sigma}}, and it is given by
#' \deqn{ R \;=\; D^{-1/2}\,\widehat{\Sigma}\,D^{-1/2}, \qquad
#'       D \;=\; \mathrm{diag}(\widehat{\Sigma}_{11},\ldots,\widehat{\Sigma}_{pp}), }
#' equivalently, entrywise \eqn{R_{ij} = \widehat{\Sigma}_{ij}/(s_i s_j)} for
#' \eqn{i \neq j} and \eqn{R_{ii} = 1}. With \eqn{1/(n-1)} scaling,
#' \eqn{\widehat{\Sigma}} is unbiased for the covariance; the induced
#' correlations are biased in finite samples.
#'
#' The implementation forms \eqn{X^\top X} via a BLAS
#' symmetric rank-\eqn{k} update (SYRK) on the upper triangle, then applies the
#' rank-1 correction \eqn{-\,n\,\mu\,\mu^\top} to obtain \eqn{S} without
#' explicitly materialising \eqn{X - \mathbf{1}\mu}. After scaling by
#' \eqn{1/(n-1)}, triangular normalisation by \eqn{D^{-1/2}} yields \eqn{R},
#' which is then symmetrised to remove round-off asymmetry. Tiny negative values
#' on the covariance diagonal due to floating-point rounding are truncated to
#' zero before taking square roots.
#'
#' If a variable has zero variance (\eqn{s_i = 0}), the corresponding row and
#' column of \eqn{R} are set to \code{NA}. When
#' \code{na_method = "pairwise"}, each \eqn{(i,j)} correlation is recomputed on
#' the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}.
#'
#' When \code{ci = TRUE}, Fisher-\eqn{z} confidence intervals are computed from
#' the observed pairwise Pearson correlation \eqn{r_{ij}} and the pairwise
#' complete-case sample size \eqn{n_{ij}}:
#' \deqn{
#' z_{ij} = \operatorname{atanh}(r_{ij}), \qquad
#' \operatorname{SE}(z_{ij}) = \frac{1}{\sqrt{n_{ij} - 3}}.
#' }
#' With \eqn{z_{1-\alpha/2} = \Phi^{-1}(1 - \alpha/2)}, the confidence limits are
#' \deqn{
#' \tanh\!\bigl(z_{ij} - z_{1-\alpha/2}\operatorname{SE}(z_{ij})\bigr)
#' \;\;\text{and}\;\;
#' \tanh\!\bigl(z_{ij} + z_{1-\alpha/2}\operatorname{SE}(z_{ij})\bigr).
#' }
#' Confidence intervals are reported only when \eqn{n_{ij} > 3}.
#'
#' \strong{Computational complexity.} The dominant cost is \eqn{O(n p^2)} flops
#' with \eqn{O(p^2)} memory.
#'
#' @note `na_method = "complete"` is useful when a common analysis sample is
#' required across all matrix entries. For covariance- or cross-product-based
#' correlations, it also avoids the non-positive-semidefinite matrices that can
#' arise from pairwise deletion.
#'
#' @references
#' Pearson, K. (1895). "Notes on regression and inheritance in the case of
#' two parents". Proceedings of the Royal Society of London, 58, 240-242.
#'
#' @examples
#' ## MVN with AR(1) correlation
#' set.seed(123)
#' p <- 6; n <- 300; rho <- 0.5
#' # true correlation
#' Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
#' L <- chol(Sigma)
#' # MVN(n, 0, Sigma)
#' X <- matrix(rnorm(n * p), n, p) %*% L
#' colnames(X) <- paste0("V", seq_len(p))
#'
#' pr <- pearson_corr(X)
#' print(pr, digits = 2)
#' summary(pr)
#' estimate(pr)
#' tidy(pr)
#' plot(pr)
#'
#' ## Confidence intervals
#' pr_ci <- pearson_corr(X[, 1:3], ci = TRUE)
#' ci(pr_ci)
#' confint(pr_ci)
#'
#' ## Compare the sample estimate to the truth
#' Rhat <- cor(X)
#' # estimated
#' round(Rhat[1:4, 1:4], 2)
#' # true
#' round(Sigma[1:4, 1:4], 2)
#' off <- upper.tri(Sigma, diag = FALSE)
#' # MAE on off-diagonals
#' mean(abs(Rhat[off] - Sigma[off]))
#'
#' ## Larger n reduces sampling error
#' n2 <- 2000
#' X2 <- matrix(rnorm(n2 * p), n2, p) %*% L
#' Rhat2 <- cor(X2)
#' off <- upper.tri(Sigma, diag = FALSE)
#' ## mean absolute error (MAE) of the off-diagonal correlations
#' mean(abs(Rhat2[off] - Sigma[off]))
#'
#' # Interactive viewing (requires shiny)
#' if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
#'   view_corr_shiny(pr)
#' }
#'
#' @useDynLib matrixCorr, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @seealso \code{\link{print.pearson_corr}}, \code{\link{plot.pearson_corr}}
#' @author Thiago de Paula Oliveira
#' @export
pearson_corr <- function(data,
                         na_method = c("error", "pairwise", "complete"),
                         ci = FALSE,
                         conf_level = 0.95,
                         n_threads = getOption("matrixCorr.threads", 1L),
                         output = c("matrix", "sparse", "edge_list"),
                         threshold = 0,
                         diag = TRUE,
                         ...) {
  .mc_corr_wrapper(
    data = data,
    dots = list(...),
    na_method = na_method,
    na_method_missing = missing(na_method),
    ci = ci,
    conf_level = conf_level,
    n_threads = n_threads,
    output = output,
    threshold = threshold,
    diag = diag,
    estimator_class = "pearson_corr",
    method = "pearson",
    description = "Pairwise Pearson correlation matrix",
    kernel_matrix = pearson_matrix_cpp,
    kernel_pairwise = pearson_matrix_pairwise_cpp,
    kernel_threshold = pearson_threshold_triplets_cpp,
    min_n = 2L,
    symmetric = TRUE,
    thresholded_output = TRUE,
    structure_matrix = .mc_structure_pearson_matrix,
    structure_matrix_fast = .mc_structure_pearson_matrix_fast
  )
}

.mc_structure_pearson_matrix <- function(mat,
                                         dimnames = NULL,
                                         diagnostics = NULL,
                                         ci_attr = NULL,
                                         conf_level = NULL) {
  if (!is.null(dimnames)) {
    dimnames(mat) <- dimnames
  }
  .mc_new_corr_matrix(
    mat = mat,
    estimator_class = "pearson_corr",
    method = "pearson",
    description = "Pairwise Pearson correlation matrix",
    output = "matrix",
    threshold = 0,
    diag = TRUE,
    diagnostics = diagnostics,
    ci = ci_attr,
    conf.level = conf_level,
    symmetric = TRUE
  )
}

.mc_structure_pearson_matrix_fast <- function(mat,
                                              dimnames = NULL,
                                              diagnostics = NULL,
                                              ci_attr = NULL,
                                              conf_level = NULL) {
  if (!is.null(dimnames)) {
    dimnames(mat) <- dimnames
  }
  class(mat) <- c("corr_matrix", "pearson_corr", "corr_result", "matrix")
  attr(mat, "method") <- "pearson"
  attr(mat, "description") <- "Pairwise Pearson correlation matrix"
  attr(mat, "package") <- "matrixCorr"
  attr(mat, "output") <- "matrix"
  attr(mat, "threshold") <- 0
  attr(mat, "diag") <- TRUE
  attr(mat, "corr_result") <- TRUE
  attr(mat, "corr_output_class") <- "corr_matrix"
  attr(mat, "corr_estimator_class") <- "pearson_corr"
  attr(mat, "corr_dim") <- as.integer(dim(mat))
  attr(mat, "corr_dimnames") <- dimnames(mat)
  attr(mat, "corr_symmetric") <- TRUE
  if (!is.null(diagnostics)) {
    attr(mat, "diagnostics") <- diagnostics
  }
  if (!is.null(ci_attr)) {
    attr(mat, "ci") <- ci_attr
    attr(mat, "conf.level") <- conf_level
  }
  mat
}

.mc_pearson_pairwise_summary <- function(object,
                                         digits = 4,
                                         ci_digits = 3,
                                         show_ci = NULL) {
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  check_inherits(object, "pearson_corr")

  .mc_pairwise_matrix_summary(
    object,
    class_name = "summary.pearson_corr",
    digits = digits,
    ci_digits = ci_digits,
    show_ci = show_ci,
    include_p = NULL
  )
}


#' @rdname pearson_corr
#' @method print pearson_corr
#' @title Print Method for \code{pearson_corr} Objects
#'
#' @param x An object of class \code{pearson_corr}.
#' @param digits Integer; number of decimal places to print in the concordance
#' @param n Optional row threshold for compact preview output.
#' @param topn Optional number of leading/trailing rows to show when truncated.
#' @param max_vars Optional maximum number of visible columns; `NULL` derives this
#'   from console width.
#' @param width Optional display width; defaults to \code{getOption("width")}.
#' @param ci_digits Integer; digits for Pearson confidence limits.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @param ... Additional arguments passed to \code{print}.
#'
#' @return Invisibly returns the \code{pearson_corr} object.
#' @author Thiago de Paula Oliveira
#' @export
print.pearson_corr <- function(x, digits = 4, n = NULL,
                               topn = NULL,
                               max_vars = NULL,
                               width = NULL,
                               ci_digits = 3,
                               show_ci = NULL,
                               ...) {
  .mc_print_corr_matrix(
    x,
    header = "Pearson correlation matrix",
    digits = digits,
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ...
  )
  invisible(x)
}

#' @rdname pearson_corr
#' @method plot pearson_corr
#' @title Plot Method for \code{pearson_corr} Objects
#'
#' @param x An object of class \code{pearson_corr}.
#' @param title Plot title. Default is \code{"Pearson correlation heatmap"}.
#' @param low_color Color for the minimum correlation. Default is
#' \code{"indianred1"}.
#' @param high_color Color for the maximum correlation. Default is
#' \code{"steelblue1"}.
#' @param mid_color Color for zero correlation. Default is \code{"white"}.
#' @param value_text_size Font size for displaying correlation values. Default
#' is \code{4}.
#' @param ci_text_size Text size for confidence intervals in the heatmap.
#' @param show_value Logical; if \code{TRUE} (default), overlay numeric values
#'   on the heatmap tiles.
#' @param ... Additional arguments passed to \code{ggplot2::theme()} or other
#' \code{ggplot2} layers.
#'
#' @return A \code{ggplot} object representing the heatmap.
#' @import ggplot2
#' @author Thiago de Paula Oliveira
#' @export
plot.pearson_corr <-
  function(x, title = "Pearson correlation heatmap",
           low_color = "indianred1", high_color = "steelblue1",
           mid_color = "white", value_text_size = 4,
           ci_text_size = 3, show_value = TRUE, ...) {
  check_bool(show_value, arg = "show_value")
  ci <- .mc_ci_attr(x)
  if (is.null(ci) || is.null(ci$lwr.ci) || is.null(ci$upr.ci)) {
    return(.mc_plot_corr_matrix(
      x, class_name = "pearson_corr", fill_name = "Pearson",
      title = title, low_color = low_color, high_color = high_color,
      mid_color = mid_color, value_text_size = value_text_size,
      show_value = show_value, ...
    ))
  }

  est_mat <- as.matrix(x)
  df_est <- as.data.frame(as.table(est_mat))
  names(df_est) <- c("Var1", "Var2", "pearson")

  df_lwr <- as.data.frame(as.table(ci$lwr.ci))
  names(df_lwr)[3] <- "lwr"
  df_upr <- as.data.frame(as.table(ci$upr.ci))
  names(df_upr)[3] <- "upr"
  df <- Reduce(
    function(a, b) merge(a, b, by = c("Var1", "Var2"), all = TRUE),
    list(df_est, df_lwr, df_upr)
  )

  diag_idx <- df$Var1 == df$Var2
  df$lwr[diag_idx] <- NA_real_
  df$upr[diag_idx] <- NA_real_
  df$ci_label <- ifelse(
    is.na(df$lwr) | is.na(df$upr),
    NA_character_,
    sprintf("[%.3f, %.3f]", df$lwr, df$upr)
  )

  lev_row <- unique(df_est$Var1)
  lev_col <- unique(df_est$Var2)
  df$Var1 <- factor(df$Var1, levels = rev(lev_row))
  df$Var2 <- factor(df$Var2, levels = lev_col)
  df$label <- sprintf("%.2f", df$pearson)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Var1, fill = .data$pearson)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      high = high_color,
      mid = mid_color,
      midpoint = 0,
      limits = c(-1, 1),
      name = "Pearson"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank(),
      ...
    ) +
    ggplot2::coord_fixed() +
    ggplot2::labs(title = title, x = NULL, y = NULL)

  if (isTRUE(show_value)) {
    p <- p + ggplot2::geom_text(ggplot2::aes(label = label), size = value_text_size, color = "black")
  }

  if (isTRUE(show_value) && any(!is.na(df$ci_label))) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = ci_label, y = as.numeric(Var1) - 0.25),
      size = ci_text_size,
      color = "gray30",
      na.rm = TRUE
    )
  }

  p
}

#' @rdname pearson_corr
#' @method summary pearson_corr
#' @param object An object of class \code{pearson_corr}.
#' @param ci_digits Integer; digits for Pearson confidence limits in the
#'   pairwise summary.
#' @param show_ci One of \code{"yes"} or \code{"no"}.
#' @export
summary.pearson_corr <- function(object,
                                 n = NULL,
                                 topn = NULL,
                                 max_vars = NULL,
                                 width = NULL,
                                 ci_digits = 3,
                                 show_ci = NULL,
                                 ...) {
  check_inherits(object, "pearson_corr")
  show_ci <- .mc_validate_yes_no(
    show_ci,
    arg = "show_ci",
    default = .mc_display_option("summary_show_ci", "yes")
  )
  if (is.null(.mc_ci_attr(object))) {
    return(.mc_summary_corr_matrix(object, topn = topn))
  }
  .mc_pearson_pairwise_summary(
    object,
    ci_digits = ci_digits,
    show_ci = show_ci
  )
}

#' @rdname pearson_corr
#' @method print summary.pearson_corr
#' @param x An object of class \code{summary.pearson_corr}.
#' @export
print.summary.pearson_corr <- function(x, digits = NULL, n = NULL,
                                       topn = NULL, max_vars = NULL,
                                       width = NULL, show_ci = NULL, ...) {
  .mc_print_pairwise_summary_digest(
    x,
    title = "Pearson correlation summary",
    digits = .mc_coalesce(digits, 4),
    n = n,
    topn = topn,
    max_vars = max_vars,
    width = width,
    show_ci = show_ci,
    ci_method = "fisher_z",
    ...
  )
  invisible(x)
}

test_that("pearson_corr matches base::cor(..., method = 'pearson')", {
  for (i in 1:10) {
    n <- sample(c(10, 50, 100, 500), 1)
    p <- sample(2:6, 1)
    mat <- replicate(p, rnorm(n))
    colnames(mat) <- paste0("P", seq_len(p))

    base_cor <- cor(mat, method = "pearson")
    fast_cor <- suppressWarnings(pearson_corr(mat))

    attributes(fast_cor) <- NULL
    attributes(base_cor) <- NULL

    expect_equal(
      fast_cor,
      base_cor,
      tolerance = 1e-10,
      info = paste("Mismatch on Pearson test dataset", i, "n =", n, "p =", p)
    )
  }
})

test_that("pearson_corr matches stats::cor on transformed data", {
  set.seed(123)
  X <- matrix(rnorm(200), ncol = 2)

  X_affine <- X
  X_affine[,1] <-  3 * X_affine[,1] + 5
  X_affine[,2] <- -2 * X_affine[,2] + 7

  base <- unname(cor(X_affine, method = "pearson"))
  fast <- unname(as.matrix(unclass(pearson_corr(X_affine))))

  expect_equal(as.numeric(fast),
               as.numeric(unname(base)),
               tolerance = 1e-12)
})

test_that("pearson_corr returns NA when a column is constant", {
  X <- cbind(a = rnorm(20), b = rep(1, 20))
  pc <- pearson_corr(X)

  expect_true(all(is.na(pc["b", ])))
  expect_true(all(is.na(pc[, "b"])))
})

test_that("pearson_corr matches base::cor on a known toy dataset", {
  X <- matrix(c(1, 2, 3, 4,
                2, 4, 6, 8), ncol = 2)
  colnames(X) <- c("x", "y")

  base_cor <- cor(X, method = "pearson")
  fast_cor <- pearson_corr(X)
  expect_equal(as.numeric(fast_cor),
               as.numeric(unname(base_cor)),
               tolerance = 1e-12)
})

test_that("pearson_corr recovers true correlation in large MVN sample", {
  set.seed(42)
  p <- 2; n <- 5000; rho <- 0.7
  Sigma <- matrix(c(1, rho, rho, 1), 2, 2)
  Z <- MASS::mvrnorm(n, mu = c(0,0), Sigma = Sigma)

  est <- pearson_corr(Z)[1,2]

  expect_equal(est, rho, tolerance = 0.02,
               info = paste("Estimated =", round(est,3),
                            "Expected =", rho))
})

test_that("pearson_corr print/plot methods cover optional arguments", {
  skip_if_not_installed("ggplot2")

  set.seed(202)
  X <- matrix(rnorm(60), nrow = 15, ncol = 4)
  colnames(X) <- paste0("P", seq_len(4))
  pc <- pearson_corr(X)

  out <- capture.output(print(pc, digits = 3, max_rows = 2, max_cols = 3))
  expect_true(any(grepl("omitted", out)))

  p <- plot(pc, title = "Pearson plot", low_color = "blue", high_color = "red", mid_color = "white", value_text_size = 3)
  expect_s3_class(p, "ggplot")
  scale <- p$scales$get_scales("fill")
  expect_equal(scale$limits, c(-1, 1))
})

test_that("pearson_corr rejects missing values by default", {
  X <- cbind(a = c(1, 2, NA, 4), b = c(1, 2, 3, 4))
  expect_error(
    pearson_corr(X),
    "Missing values are not allowed.*na_method = \"pairwise\".*na_method = \"complete\""
  )
})

test_that("pearson_corr pairwise mode uses complete-case overlap per pair", {
  X <- cbind(
    x = c(1, 2, 3, 4, NA, 6),
    y = c(2, 4, 6, 8, 10, NA),
    z = c(1, NA, 2, 3, 4, 5)
  )

  R <- pearson_corr(X, na_method = "pairwise")
  diag_attr <- attr(R, "diagnostics", exact = TRUE)

  keep_xy <- stats::complete.cases(X[, "x"], X[, "y"])
  keep_xz <- stats::complete.cases(X[, "x"], X[, "z"])

  expect_equal(unname(R["x", "y"]), stats::cor(X[keep_xy, "x"], X[keep_xy, "y"]))
  expect_equal(unname(R["x", "z"]), stats::cor(X[keep_xz, "x"], X[keep_xz, "z"]))
  expect_identical(diag_attr$n_complete["x", "y"], as.integer(sum(keep_xy)))
  expect_identical(diag_attr$n_complete["x", "z"], as.integer(sum(keep_xz)))
})

test_that("pearson_corr Fisher-z confidence intervals match manual calculation", {
  X <- cbind(
    x = c(1, 2, 3, 4, 5, 6, NA, 8),
    y = c(1.4, 2.1, 3.2, 4.0, 5.4, 5.9, 7.1, NA),
    z = c(8, 7, 6, 5, 4, 3, 2, 1)
  )

  R <- pearson_corr(X, na_method = "pairwise", ci = TRUE)
  ci <- attr(R, "ci", exact = TRUE)
  diag_attr <- attr(R, "diagnostics", exact = TRUE)

  keep_xy <- stats::complete.cases(X[, "x"], X[, "y"])
  x_xy <- X[keep_xy, "x"]
  y_xy <- X[keep_xy, "y"]
  r_xy <- stats::cor(x_xy, y_xy)
  n_xy <- sum(keep_xy)
  z_xy <- atanh(r_xy)
  crit <- stats::qnorm(0.975)
  se_xy <- 1 / sqrt(n_xy - 3)
  lwr_xy <- tanh(z_xy - crit * se_xy)
  upr_xy <- tanh(z_xy + crit * se_xy)

  expect_equal(unname(R["x", "y"]), r_xy, tolerance = 1e-12)
  expect_identical(diag_attr$n_complete["x", "y"], as.integer(n_xy))
  expect_equal(unname(ci$lwr.ci["x", "y"]), lwr_xy, tolerance = 1e-12)
  expect_equal(unname(ci$upr.ci["x", "y"]), upr_xy, tolerance = 1e-12)
  expect_equal(ci$conf.level, 0.95)
})

test_that("pearson_corr stores CI payload only when requested", {
  X <- cbind(
    A = c(1, 2, 3, 4, 5, 6),
    B = c(1.1, 2.1, 3.0, 4.2, 4.9, 6.2),
    C = c(6, 5, 4, 3, 2, 1)
  )

  fit <- pearson_corr(X)
  fit_ci <- pearson_corr(X, ci = TRUE)

  expect_null(attr(fit, "ci", exact = TRUE))
  expect_null(attr(fit, "diagnostics", exact = TRUE))

  ci_attr <- attr(fit_ci, "ci", exact = TRUE)
  expect_type(ci_attr, "list")
  expect_identical(names(ci_attr), c("est", "lwr.ci", "upr.ci", "conf.level"))
  expect_true(is.matrix(ci_attr$est))
  expect_true(is.matrix(ci_attr$lwr.ci))
  expect_true(is.matrix(ci_attr$upr.ci))
  expect_false("ci" %in% names(attributes(ci_attr$est)))
})

test_that("pearson_corr honors n_threads without changing estimates", {
  set.seed(321)
  X <- matrix(rnorm(240), nrow = 40, ncol = 6)
  colnames(X) <- paste0("P", seq_len(ncol(X)))

  fit1 <- pearson_corr(X, n_threads = 1L)
  fit2 <- pearson_corr(X, n_threads = 2L)
  fit1_ci <- pearson_corr(X, ci = TRUE, n_threads = 1L)
  fit2_ci <- pearson_corr(X, ci = TRUE, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(attr(fit1_ci, "ci", exact = TRUE), attr(fit2_ci, "ci", exact = TRUE), tolerance = 1e-12)
})

test_that("pearson_corr CI-aware print, summary, and plot follow the CI style", {
  skip_if_not_installed("ggplot2")

  X <- cbind(
    A = c(1, 2, 3, 4, 5, 6),
    B = c(1.2, 2.1, 3.2, 3.9, 5.1, 5.8),
    C = c(6, 5, 4, 3, 2, 1)
  )

  R <- pearson_corr(X, ci = TRUE)
  sm <- summary(R)
  txt_print <- capture.output(print(R, show_ci = "yes"))
  txt_summary <- capture.output(print(sm))
  p <- plot(R, ci_text_size = 2)

  expect_s3_class(sm, "summary.pearson_corr")
  expect_equal(nrow(sm), choose(ncol(X), 2))
  expect_true(all(c("item1", "item2", "estimate", "n_complete", "lwr", "upr") %in% names(sm)))
  expect_match(paste(txt_print, collapse = "\n"), "Pearson correlation matrix")
  expect_false(any(grepl("Pearson correlation summary", txt_print, fixed = TRUE)))
  expect_match(paste(txt_summary, collapse = "\n"), "Pearson correlation summary")
  expect_match(paste(txt_summary, collapse = "\n"), "ci_width")
  expect_match(paste(txt_summary, collapse = "\n"), "\\bn\\b")
  expect_s3_class(p, "ggplot")
})

test_that("structure, classes and lengths are correct", {
  set.seed(1)
  x <- rnorm(20, 100, 10)
  y <- x + rnorm(20, 0, 8)

  ba <- ba(x, y)

  expect_s3_class(ba, "ba")
  expect_true(all(c("means","diffs","groups","based.on",
                    "lower.limit","mean.diffs","upper.limit",
                    "lines","CI.lines","loa_multiplier","critical.diff") %in% names(ba)))
  expect_false(any(c("groups", "based.on", "lines", "CI.lines") %in% names(unclass(ba))))

  # basic size checks
  expect_equal(length(ba$means), ba$based.on)
  expect_equal(length(ba$diffs), ba$based.on)
  expect_equal(nrow(ba$groups),  ba$based.on)

  # named vectors present
  expect_true(all(c("lower.limit","mean.diffs","upper.limit") %in% names(ba$lines)))
  expect_true(all(c("lower.limit.ci.lower","lower.limit.ci.upper",
                    "mean.diff.ci.lower","mean.diff.ci.upper",
                    "upper.limit.ci.lower","upper.limit.ci.upper")
                  %in% names(ba$CI.lines)))
})

test_that("ba stores a smaller canonical object than the legacy expanded layout", {
  set.seed(31)
  fit <- ba(rnorm(200), rnorm(200))

  legacy_like <- c(
    unclass(fit),
    list(
      groups = fit$groups,
      based.on = fit$based.on,
      lines = fit$lines,
      CI.lines = fit$CI.lines
    )
  )

  expect_lt(as.numeric(object.size(fit)), as.numeric(object.size(legacy_like)))
})

test_that("matches known output on the 12-row example (exact numbers)", {
  example.data <- data.frame(
    sex = gl(2, 6, labels = c("f", "m")),
    m1  = c(16,10,14,18,16,15,18,19,14,11,11,17),
    m2  = c(18, 9,15,19,19,13,19,20,14,11,13,17)
  )
  ba <- ba(example.data$m1, example.data$m2)

  # reference values supplied in prompt (tolerance 1e-6)
  expect_equal(as.numeric(ba$based.on), 12L)
  expect_equal(as.numeric(ba$mean.diffs),  -2/3,      tolerance = 1e-6)
  expect_equal(as.numeric(ba$lower.limit), -3.353217, tolerance = 1e-6)
  expect_equal(as.numeric(ba$upper.limit),  2.019883, tolerance = 1e-6)
  expect_equal(as.numeric(ba$critical.diff), 2.68655, tolerance = 1e-5)

  # CI lines
  exp_CI <- c(
    "lower.limit.ci.lower" = -4.8616497,
    "lower.limit.ci.upper" = -1.8447839,
    "mean.diff.ci.lower"   = -1.5375608,
    "mean.diff.ci.upper"   =  0.2042275,
    "upper.limit.ci.lower" =  0.5114506,
    "upper.limit.ci.upper" =  3.5283163
  )
  # compare by name, same tolerance
  for (nm in names(exp_CI)) {
    expect_equal(as.numeric(ba$CI.lines[[nm]]), exp_CI[[nm]], tolerance = 1e-6)
  }
})

test_that("summary.ba returns a compact summary object and prints grouped sections", {
  set.seed(11)
  x <- rnorm(40, 100, 10)
  y <- x + rnorm(40, 0, 8)

  fit <- ba(x, y)
  sm <- summary(fit)

  expect_s3_class(sm, "summary.ba")
  expect_true(all(c("n_obs", "bias", "sd_loa", "loa_low", "loa_up", "width",
                    "loa_multiplier", "bias_lwr", "bias_upr",
                    "lo_lwr", "lo_upr", "up_lwr", "up_upr") %in% names(sm)))

  out <- capture.output(print(sm))
  expect_true(any(grepl("^Agreement estimates$", out)))
  expect_true(any(grepl("^Confidence intervals$", out)))
})

test_that("ba accepts wide input and returns pairwise matrices", {
  set.seed(12)
  dat <- data.frame(
    A = rnorm(40, 100, 8),
    B = rnorm(40, 101, 9),
    C = rnorm(40,  99, 7)
  )

  fit <- ba(dat)
  sm <- summary(fit)

  expect_s3_class(fit, "ba_matrix")
  expect_s3_class(sm, "summary.ba_matrix")
  expect_equal(dim(fit$bias), c(3L, 3L))
  expect_equal(fit$methods, c("A", "B", "C"))
  expect_equal(unname(diag(fit$bias)), rep(NA_real_, 3L))
  expect_equal(fit$bias[lower.tri(fit$bias)], -t(fit$bias)[lower.tri(fit$bias)], tolerance = 1e-12)
  expect_equal(fit$sd_loa[lower.tri(fit$sd_loa)], t(fit$sd_loa)[lower.tri(fit$sd_loa)], tolerance = 1e-12)
  expect_equal(nrow(sm), 3L)
  expect_true(all(c("item1", "item2", "bias", "sd_loa", "loa_low", "loa_up",
                    "width", "n_obs", "bias_lwr", "bias_upr",
                    "lo_lwr", "lo_upr", "up_lwr", "up_upr") %in% names(sm)))
})

test_that("wide ba honors mode orientation", {
  set.seed(13)
  dat <- data.frame(
    A = rnorm(30),
    B = rnorm(30),
    C = rnorm(30)
  )

  fit1 <- ba(dat, mode = 1L)
  fit2 <- ba(dat, mode = 2L)

  expect_equal(fit1$bias, -fit2$bias, tolerance = 1e-12)
  expect_equal(fit1$loa_lower, -fit2$loa_upper, tolerance = 1e-12)
  expect_equal(fit1$loa_upper, -fit2$loa_lower, tolerance = 1e-12)
})

test_that("wide ba skips contrasts with fewer than two complete pairs", {
  dat <- data.frame(
    A = c(1, 2, NA, NA),
    B = c(1, 2, 3, NA),
    C = c(NA, NA, 3, 4)
  )

  fit <- ba(dat)

  expect_equal(fit$n["A", "B"], 2L)
  expect_equal(fit$n["A", "C"], 0L)
  expect_true(is.na(fit$bias["A", "C"]))
  expect_true(is.na(fit$sd_loa["A", "C"]))
})

test_that("'loa_multiplier' scales the LoA correctly", {
  set.seed(2)
  x <- rnorm(50, 100, 10)
  y <- x + rnorm(50, 0, 8)

  a <- ba(x, y, loa_multiplier = 1.96)
  b <- ba(x, y, loa_multiplier = 2.00)

  # the LoA width is 2 * loa_multiplier * sd_diffs -> so proportional to 'loa_multiplier'
  width_a <- as.numeric(a$upper.limit - a$lower.limit)
  width_b <- as.numeric(b$upper.limit - b$lower.limit)
  expect_equal(width_b / width_a, 2.00 / 1.96, tolerance = 1e-12)

  # 'critical.diff' equals loa_multiplier * sd_diffs
  sd_a <- as.numeric(a$critical.diff) / 1.96
  sd_b <- as.numeric(b$critical.diff) / 2.00
  expect_equal(sd_a, sd_b, tolerance = 1e-12)
})

test_that("mode=2 flips the sign as expected", {
  set.seed(3)
  x <- rnorm(40, 100, 10)
  y <- x + rnorm(40, 0, 8)

  m1 <- ba(x, y, mode = 1)  # diffs = x - y
  m2 <- ba(x, y, mode = 2)  # diffs = y - x

  # means are the same, diffs flip sign
  expect_equal(as.numeric(m1$means), as.numeric(m2$means))
  expect_equal(as.numeric(m1$diffs), -as.numeric(m2$diffs))

  # scalar summaries flip sign accordingly
  expect_equal(as.numeric(m1$mean.diffs), -as.numeric(m2$mean.diffs))
  expect_equal(as.numeric(m1$lower.limit), -as.numeric(m2$upper.limit), tolerance = 1e-12)
  expect_equal(as.numeric(m1$upper.limit), -as.numeric(m2$lower.limit), tolerance = 1e-12)

  # CI lines for mean difference flip sign; LoA CIs swap and flip
  expect_equal(as.numeric(m1$CI.lines["mean.diff.ci.lower"]),
               -as.numeric(m2$CI.lines["mean.diff.ci.upper"]), tolerance = 1e-12)
  expect_equal(as.numeric(m1$CI.lines["mean.diff.ci.upper"]),
               -as.numeric(m2$CI.lines["mean.diff.ci.lower"]), tolerance = 1e-12)
})

test_that("pairwise NA removal works and 'based.on' reflects that", {
  x <- c(1, 2, NA, 4, 5, 6)
  y <- c(1, NA, 3, 4, NA, 7)
  # valid pairs: (1,1), (4,4), (6,7) => n = 3
  ba <- ba(x, y)
  expect_equal(as.integer(ba$based.on), 3L)
  expect_equal(length(ba$means), 3L)
  expect_equal(nrow(ba$groups), 3L)
})

test_that("constant difference yields zero SD and zero-width LoA/CI", {
  set.seed(4)
  g1 <- rnorm(30, 50, 5)
  g2 <- g1 + 2                       # constant +2
  ba <- ba(g1, g2)         # diffs = g1 - g2 = -2

  expect_equal(as.numeric(ba$mean.diffs), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$critical.diff), 0, tolerance = 1e-12)
  expect_equal(as.numeric(ba$lower.limit), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$upper.limit), -2, tolerance = 1e-12)

  # all CI lines collapse to the corresponding estimates
  expect_equal(as.numeric(ba$CI.lines["mean.diff.ci.lower"]), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$CI.lines["mean.diff.ci.upper"]), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$CI.lines["lower.limit.ci.lower"]), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$CI.lines["lower.limit.ci.upper"]), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$CI.lines["upper.limit.ci.lower"]), -2, tolerance = 1e-12)
  expect_equal(as.numeric(ba$CI.lines["upper.limit.ci.upper"]), -2, tolerance = 1e-12)
})

test_that("ba honors n_threads without changing estimates", {
  set.seed(44)
  x <- rnorm(80, 100, 10)
  y <- x + rnorm(80, 0, 8)

  fit1 <- ba(x, y, n_threads = 1L)
  fit2 <- ba(x, y, n_threads = 2L)

  expect_equal(unclass(fit1), unclass(fit2), tolerance = 1e-12)
  expect_equal(fit1$groups, fit2$groups, tolerance = 1e-12)
  expect_equal(fit1$lines, fit2$lines, tolerance = 1e-12)
  expect_equal(fit1$CI.lines, fit2$CI.lines, tolerance = 1e-12)
})

test_that("errors on invalid inputs", {
  expect_error(ba(1:3, 1:2), "same length")
  expect_error(ba(letters[1:3], 1:3), "numeric", ignore.case = TRUE)
  expect_error(ba(1:3, 1:3, loa_multiplier = 0), "positive")
  expect_error(ba(1:3, 1:3, mode = 0), "1 or 2")
  expect_error(ba(data.frame(a = letters[1:3], b = letters[4:6])), "numeric columns")
  expect_error(ba(1:3, 1:3, conf_level = 1.5), "in \\(0, 1\\)")
})

test_that("requires at least two complete pairs", {
  x <- c(1, NA)
  y <- c(1, 2)
  expect_error(
    ba(x, y),
    "at least two complete pairs",
    fixed = FALSE
  )
})

test_that("plot.ba returns a ggplot without line-size deprecation warnings", {
  skip_if_not_installed("ggplot2")

  set.seed(5)
  x <- rnorm(40, 100, 10)
  y <- x + rnorm(40, 0, 8)
  fit <- ba(x, y)

  expect_no_warning(
    expect_s3_class(plot(fit), "ggplot")
  )
})

test_that("plot.ba_matrix returns a ggplot without line-size deprecation warnings", {
  skip_if_not_installed("ggplot2")

  set.seed(14)
  dat <- data.frame(
    A = rnorm(40, 100, 8),
    B = rnorm(40, 101, 9),
    C = rnorm(40,  99, 7)
  )
  fit <- ba(dat)

  expect_no_warning(
    expect_s3_class(plot(fit), "ggplot")
  )
})

---
title: "4. Latent and Mixed-Scale Correlation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{4. Latent and Mixed-Scale Correlation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Scope

This vignette covers the latent-correlation estimators used when the observed
data are binary, ordinal, or mixed. These methods do not target the same
quantity as an ordinary Pearson correlation on coded categories. They are
designed for settings where the observed variables are treated as thresholded
versions of latent continuous variables.

The relevant functions are:

- `tetrachoric()`
- `polychoric()`
- `polyserial()`
- `biserial()`

## A small latent-data example

```{r}
library(matrixCorr)

set.seed(30)
n <- 500
Sigma <- matrix(c(
  1.00, 0.55, 0.35, 0.20,
  0.55, 1.00, 0.40, 0.30,
  0.35, 0.40, 1.00, 0.45,
  0.20, 0.30, 0.45, 1.00
), 4, 4, byrow = TRUE)

Z <- matrix(rnorm(n * 4), n, 4) %*% chol(Sigma)

X_bin <- data.frame(
  b1 = Z[, 1] > qnorm(0.70),
  b2 = Z[, 2] > qnorm(0.55),
  b3 = Z[, 3] > qnorm(0.50)
)

X_ord <- data.frame(
  o1 = ordered(cut(Z[, 2], breaks = c(-Inf, -0.5, 0.4, Inf),
    labels = c("low", "mid", "high")
  )),
  o2 = ordered(cut(Z[, 3], breaks = c(-Inf, -1, 0, 1, Inf),
    labels = c("1", "2", "3", "4")
  ))
)

X_cont <- data.frame(x1 = Z[, 1], x2 = Z[, 4])
```

## Binary-binary and ordinal-ordinal settings

`tetrachoric()` is used for binary variables. `polychoric()` is used for
ordered categorical variables.

```{r}
fit_tet <- tetrachoric(X_bin, ci = TRUE, p_value = TRUE)
fit_pol <- polychoric(X_ord, ci = TRUE, p_value = TRUE)

print(fit_tet, digits = 2)
summary(fit_pol)
estimate(fit_tet)
confint(fit_tet)
ci(fit_tet)
tidy(fit_tet)
```

These estimators assume a latent-normal threshold model. That assumption should
be stated whenever the results are reported, because the interpretation is not
simply "correlation between coded categories."

It is often useful to compare that latent estimate with a naive Pearson
correlation computed on coded categories.

```{r}
fit_bin_naive <- pearson_corr(data.frame(lapply(X_bin[, 1:2], as.numeric)))
fit_ord_naive <- pearson_corr(data.frame(lapply(X_ord, as.numeric)))

round(c(
  b1_b2_pearson = fit_bin_naive[1, 2],
  b1_b2_tetrachoric = fit_tet[1, 2],
  o1_o2_pearson = fit_ord_naive[1, 2],
  o1_o2_polychoric = fit_pol[1, 2]
), 2)
```

Those numbers need not agree. The latent estimators target the association
between the underlying continuous variables, not the correlation between
arbitrarily coded categories.

## Mixed continuous-discrete settings

`polyserial()` is used when one variable is continuous and the other is
ordinal. `biserial()` is used when one variable is continuous and the other is
binary.

```{r}
fit_ps <- polyserial(X_cont, X_ord, ci = TRUE, p_value = TRUE)
fit_bis <- biserial(X_cont, X_bin[, 1:2], ci = TRUE, p_value = TRUE)

summary(fit_ps)
summary(fit_bis)
```

## Confidence intervals and p-values

These functions now follow the same user-facing pattern as the rest of the
package:

- estimates are returned by default;
- confidence intervals are added only when `ci = TRUE`;
- p-values are added only when `p_value = TRUE` where supported.

The important point is that inference is tied to the fitted latent model rather
than to an ordinary Pearson-correlation formula applied to coded categories.

## Practical guidance

These estimators are appropriate when the scientific question is explicitly
about latent association under a threshold model.

- Use `tetrachoric()` for binary-binary pairs.
- Use `polychoric()` for ordinal-ordinal pairs.
- Use `polyserial()` for continuous-ordinal pairs.
- Use `biserial()` for continuous-binary pairs.

If the variables are nominal rather than ordered, these latent-correlation
functions are not the right tools.

---
title: "3. Robust and High-Dimensional Correlation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{3. Robust and High-Dimensional Correlation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Scope

This vignette covers the estimators used when ordinary wide-data correlation is
not enough. The package provides two broad extensions:

- robust estimators for outlier-prone data;
- regularised estimators for partial correlation or `p >= n` settings.

The relevant functions are:

- `bicor()`
- `pbcor()`
- `wincor()`
- `skipped_corr()`
- `shrinkage_corr()`
- `pcorr()`

## Robust correlation

Outliers can distort ordinary correlation severely, especially in moderate
sample sizes. The robust estimators in `matrixCorr` are designed to limit that
effect, but they do so in different ways.

```{r}
library(matrixCorr)

set.seed(20)
Y <- data.frame(
  x1 = rnorm(60),
  x2 = rnorm(60),
  x3 = rnorm(60),
  x4 = rnorm(60)
)

idx <- sample.int(nrow(Y), 5)
Y$x1[idx] <- Y$x1[idx] + 8
Y$x2[idx] <- Y$x2[idx] - 6

R_pear <- pearson_corr(Y)
R_bicor <- bicor(Y)
R_pb <- pbcor(Y)
R_win <- wincor(Y)
R_skip <- skipped_corr(Y)

summary(R_pear)
summary(R_bicor)
```

In practical terms:

- `bicor()` is often a good first robust alternative for wide data.
- `pbcor()` and `wincor()` follow classical robustification routes and are easy
  to compare against ordinary correlation.
- `skipped_corr()` is more aggressive because it first removes bivariate
  outliers pair by pair.

That last property is useful, but it also makes `skipped_corr()` the most
computationally expensive estimator in this group.

```{r}
summary(R_skip)
```

## Inference for robust estimators

Inference is not uniform across the robust estimators, so it is worth stating
the current scope explicitly.

- `bicor()` supports large-sample confidence intervals.
- `pbcor()` supports percentile-bootstrap confidence intervals and
  method-specific p-values.
- `wincor()` supports percentile-bootstrap confidence intervals and
  method-specific p-values.
- `skipped_corr()` supports bootstrap confidence intervals and bootstrap
  p-values.

```{r}
fit_bicor_ci <- bicor(Y, ci = TRUE)
summary(fit_bicor_ci)
estimate(fit_bicor_ci)
confint(fit_bicor_ci)
ci(fit_bicor_ci)
tidy(fit_bicor_ci)

fit_pb_inf <- pbcor(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_pb_inf)

fit_win_inf <- wincor(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_win_inf)

fit_skip_inf <- skipped_corr(Y, ci = TRUE, p_value = TRUE, n_boot = 200, seed = 1)
summary(fit_skip_inf)
```

For `pbcor()`, `wincor()`, and `skipped_corr()`, the inferential layer is more
computationally demanding than the estimate-only path because it is built from
bootstrap resampling. The skipped-correlation route remains the most expensive
of the three because each bootstrap sample also repeats the pairwise outlier
screening step. The vignette uses `n_boot = 200` only to keep runtime modest;
substantive analyses usually need more resamples.

## Partial correlation

Partial correlation addresses a different target. Instead of summarising raw
association, it estimates conditional association after accounting for the
remaining variables.

```{r}
set.seed(21)
n <- 300
x2 <- rnorm(n)
x1 <- 0.8 * x2 + rnorm(n, sd = 0.4)
x3 <- 0.8 * x2 + rnorm(n, sd = 0.4)
x4 <- 0.7 * x3 + rnorm(n, sd = 0.5)
x5 <- rnorm(n)
x6 <- rnorm(n)
X <- data.frame(x1, x2, x3, x4, x5, x6)

fit_pcor_sample <- pcorr(X, method = "sample")
fit_pcor_oas <- pcorr(X, method = "oas")
R_raw <- pearson_corr(X)

round(c(
  raw_x1_x3 = R_raw["x1", "x3"],
  partial_x1_x3 = estimate(fit_pcor_sample)["x1", "x3"]
), 2)

print(fit_pcor_sample, digits = 2)
summary(fit_pcor_oas)
```

Here `x1` and `x3` are strongly associated in the raw correlation matrix
because both depend on `x2`. Partial correlation is useful because it can
separate that indirect association from the remaining direct structure.

The choice of `method` is important:

- `"sample"` is the ordinary full-rank route.
- `"oas"` and `"ridge"` stabilise estimation in higher-dimensional settings.
- `"glasso"` is appropriate when a sparse precision structure is the target.

When the classical sample estimator is appropriate, partial correlation also
supports p-values and Fisher-z confidence intervals.

Like the other correlation wrappers, `pcorr()` returns an estimate matrix by
default. Use `estimate()`, `tidy()`, `ci()`, and `confint()` for public access.
If you need the covariance or precision matrix, set `return_details = TRUE`.

```{r}
fit_pcor_inf <- pcorr(
  X,
  method = "sample",
  return_p_value = TRUE,
  ci = TRUE
)

summary(fit_pcor_inf)
tidy(fit_pcor_inf)
ci(fit_pcor_inf)
confint(fit_pcor_inf)
```

That inference layer is available only for the ordinary sample estimator in the
low-dimensional setting. The regularised estimators are primarily estimation
tools rather than direct inferential procedures in the current interface.

## High-dimensional shrinkage correlation

When the number of variables is large relative to the sample size, direct
sample correlation can become unstable. `shrinkage_corr()` provides a shrinkage
correlation route designed for exactly this setting.

```{r}
set.seed(22)
n <- 40
p_block <- 30

make_block <- function(f) {
  sapply(seq_len(p_block), function(j) 0.8 * f + rnorm(n, sd = 0.8))
}

f1 <- rnorm(n)
f2 <- rnorm(n)
f3 <- rnorm(n)

Xd <- cbind(make_block(f1), make_block(f2), make_block(f3))
p <- ncol(Xd)
colnames(Xd) <- paste0("G", seq_len(p))

R_raw <- stats::cor(Xd)
fit_shr <- shrinkage_corr(Xd)
block <- rep(1:3, each = p_block)
within <- outer(block, block, "==") & upper.tri(R_raw)
between <- outer(block, block, "!=") & upper.tri(R_raw)

round(c(
  raw_within = mean(abs(R_raw[within])),
  raw_between = mean(abs(R_raw[between])),
  shrink_within = mean(abs(fit_shr[within])),
  shrink_between = mean(abs(fit_shr[between]))
), 3)

print(fit_shr, digits = 2, max_rows = 6, max_vars = 6)
summary(fit_shr)
```

The shrinkage estimator is not intended to reproduce the raw sample matrix. Its
purpose is to provide a better-conditioned and more stable estimate in settings
where the sample matrix is noisy or singular. In this block-factor simulation,
the raw sample matrix retains the main within-block pattern but also carries
substantial between-block noise; shrinkage pulls that background noise down
without erasing the dominant structure.

## Choosing among these methods

The choice is usually governed by the main source of difficulty in the data.

- If the concern is outliers, start with `bicor()` and compare with
  `pbcor()`, `wincor()`, or `skipped_corr()` as needed.
- If the concern is conditional dependence, use `pcorr()`.
- If the concern is `p >= n` or unstable covariance estimation, use
  `shrinkage_corr()` or a regularised `pcorr()` method.

These are not interchangeable estimators. They solve different problems, even
though they share the same matrix-oriented calling style.

---
title: "2. Wide Correlation Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{2. Wide Correlation Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Scope

This vignette covers the basic wide-data correlation estimators. These methods
start from one numeric matrix or data frame, treat columns as variables, and
return a square matrix indexed by those columns.

The main functions in this group are:

- `pearson_corr()`
- `spearman_rho()`
- `kendall_tau()`
- `dcor()`

They answer related but not identical questions, so method choice should be
driven by the structure of the data rather than by habit alone.

## A common input pattern

```{r}
library(matrixCorr)

set.seed(10)
z <- rnorm(80)
u <- rnorm(80)
X <- data.frame(
  x1 = z + rnorm(80, sd = 0.35),
  x2 = 0.85 * z + rnorm(80, sd = 0.45),
  x3 = 0.25 * z + 0.70 * u + rnorm(80, sd = 0.45),
  x4 = rnorm(80)
)
```

All four estimators accept this same wide input format.

```{r}
R_pear <- pearson_corr(X)
R_spr  <- spearman_rho(X)
R_ken  <- kendall_tau(X)
R_dcor <- dcor(X)

print(R_pear, digits = 2)
summary(R_spr)
```

This toy dataset is intentionally structured so that `x1` and `x2` form a
clear linear pair, `x3` is only moderately related to that first block, and
`x4` is close to null. That makes the shared output structure easier to
interpret than a pure-noise example.

## Pearson correlation

Pearson correlation targets linear association on the original measurement
scale. It is the natural first choice when variables are continuous, the
relationship is approximately linear, and there is no strong concern about
outlier sensitivity.

```{r}
plot(R_pear)
```

If confidence intervals are required, they can be requested directly.

```{r}
R_pear_ci <- pearson_corr(X, ci = TRUE)
summary(R_pear_ci)
estimate(R_pear_ci)
confint(R_pear_ci)
ci(R_pear_ci)
tidy(R_pear_ci)
```

## Spearman and Kendall

Spearman's rho and Kendall's tau are rank-based estimators. They are useful
when monotone association is of interest and the analysis should be less
sensitive to departures from strict linearity. Both functions also support
optional large-sample confidence intervals through `ci = TRUE`.

```{r}
set.seed(11)
x <- sort(rnorm(60))
y <- x^3 + rnorm(60, sd = 0.5)
dat_mon <- data.frame(x = x, y = y)

pearson_corr(dat_mon)
spearman_rho(dat_mon)
kendall_tau(dat_mon)
```

In this setting the relationship is monotone but not linear, so a rank-based
summary is often the clearer first description.

When interval estimation is required, the same matrix-style interface is kept.

```{r}
fit_spr_ci <- spearman_rho(X, ci = TRUE)
fit_ken_ci <- kendall_tau(X, ci = TRUE)

summary(fit_spr_ci)
summary(fit_ken_ci)
```

## Distance correlation

Distance correlation addresses a broader target. It is designed to detect
general dependence rather than only linear or monotone structure. The function
also supports optional hypothesis testing through `p_value = TRUE`.

```{r}
set.seed(12)
x <- runif(100, -2, 2)
y <- x^2 + rnorm(100, sd = 0.2)
dat_nonlin <- data.frame(x = x, y = y)

pearson_corr(dat_nonlin)
dcor(dat_nonlin)
```

This is a typical situation where Pearson correlation can be close to zero even
though the variables are clearly dependent.

If a formal inferential summary is needed, p-values can be requested directly.

```{r}
fit_dcor_p <- dcor(dat_nonlin, p_value = TRUE)
summary(fit_dcor_p)
```

## Missing values

The default wide-data behaviour is strict validation. Missing values are
rejected unless the function explicitly supports a relaxed mode through
`na_method = "pairwise"`.

```{r}
X_miss <- X
X_miss$x2[c(3, 7)] <- NA

try(pearson_corr(X_miss))
pearson_corr(X_miss, na_method = "pairwise")
```

When `na_method = "pairwise"`, the package uses pairwise complete observations
for the affected estimator. That is convenient, but it also means different
pairs may be based on different effective sample sizes.

## Practical guidance

In ordinary wide-data work, the following sequence is usually defensible.

- Start with `pearson_corr()` when the variables are continuous and linear
  association is the scientific target.
- Use `spearman_rho()` or `kendall_tau()` when a monotone summary is preferred.
- Use `dcor()` when non-linear dependence is plausible and a zero Pearson
  correlation would be misleading.

The next vignette addresses settings where this basic family is still not
sufficient because the data are contaminated by outliers or the number of
variables is large relative to sample size.

---
title: "6. Repeated-Measures Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{6. Repeated-Measures Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Scope

Repeated-measures analysis requires a different data layout and, usually, a
different estimand. The wide-data matrix workflow is not enough because the
analysis must account for the repeated structure within subject.

This vignette covers:

- `rmcorr()`
- `ba_rm()`
- `ccc_rm_ustat()`
- `ccc_rm_reml()`
- `icc_rm_reml()`

Repeated-measures functions use a `subject` argument for the subject identifier
role, including `rmcorr()`, `ba_rm()`, `ccc_rm_ustat()`, `ccc_rm_reml()`, and
`icc_rm_reml()`.

## A common long-format dataset

```{r}
library(matrixCorr)

set.seed(50)
n_id <- 14
n_time <- 4

dat <- expand.grid(
  id = factor(seq_len(n_id)),
  time = factor(seq_len(n_time)),
  method = factor(c("A", "B"))
)

dat$time_index <- as.integer(dat$time)

subj <- rnorm(n_id, sd = 1.0)[dat$id]
subject_method <- rnorm(n_id * 2, sd = 0.25)
sm <- subject_method[(as.integer(dat$id) - 1L) * 2L + as.integer(dat$method)]
subject_time <- rnorm(n_id * n_time, sd = 0.75)
st <- subject_time[(as.integer(dat$id) - 1L) * n_time + as.integer(dat$time)]

dat$y <- subj + sm + st + 0.35 * (dat$method == "B") +
  rnorm(nrow(dat), sd = 0.35)
```

## Repeated-measures correlation

`rmcorr()` targets within-subject association, not agreement. It is the right
function when the question is whether two responses vary together within
subjects after removing subject-level offsets.

```{r}
set.seed(51)
dat_rmcorr <- data.frame(
  id = rep(seq_len(n_id), each = n_time),
  x = rnorm(n_id * n_time),
  y = rnorm(n_id * n_time),
  z = rnorm(n_id * n_time)
)

dat_rmcorr$y <- 0.7 * dat_rmcorr$x +
  rnorm(n_id, sd = 1)[dat_rmcorr$id] +
  rnorm(nrow(dat_rmcorr), sd = 0.3)

fit_rmcorr <- rmcorr(dat_rmcorr, response = c("x", "y", "z"), subject = "id")
summary(fit_rmcorr)
```

## Repeated-measures agreement

Agreement methods require method labels and, for paired repeated analysis, a
time or replicate key.

`ba_rm()` is the repeated-measures Bland-Altman route. It models subject-time
matched paired differences and returns bias and limits of agreement.

```{r}
fit_ba_rm <- ba_rm(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index"
)

summary(fit_ba_rm)
```

## Repeated CCC

The package provides two repeated-measures CCC routes.

- `ccc_rm_ustat()` is a nonparametric U-statistic approach.
- `ccc_rm_reml()` uses the REML mixed-model backend.

```{r}
fit_ccc_ustat <- ccc_rm_ustat(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index"
)

fit_ccc_reml <- ccc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  ci = FALSE
)

summary(fit_ccc_ustat)
summary(fit_ccc_reml)
```

The two functions are related, but they are not interchangeable. The U-statistic
route is useful when its design assumptions are appropriate. The REML route is
the more flexible model-based path when variance components and residual
structure need to be handled explicitly.

## Repeated ICC

`icc_rm_reml()` uses the same REML and Woodbury backend as repeated CCC, but it
targets a different reliability quantity.

```{r}
fit_icc_cons <- icc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  type = "consistency",
  ci = TRUE
)

fit_icc_agr <- icc_rm_reml(
  dat,
  response = "y",
  subject = "id",
  method = "method",
  time = "time_index",
  type = "agreement",
  ci = FALSE
)

summary(fit_icc_cons)
summary(fit_icc_agr)
estimate(fit_icc_cons)
confint(fit_icc_cons)
ci(fit_icc_cons)
tidy(fit_icc_cons)
```

The simulation above gives the subject-time component a visibly non-trivial
variance contribution. That makes the CCC-versus-ICC distinction easier to see:

```{r}
data.frame(
  method = c("Repeated CCC (REML)", "Repeated ICC (agreement, REML)"),
  estimate = c(
    fit_ccc_reml[1, 2],
    fit_icc_agr[1, 2]
  )
)
```

The most important distinction between repeated CCC and repeated ICC is the
numerator. Repeated ICC uses only the stable between-subject variance in the
numerator. Repeated CCC also credits the time-averaged subject-time component.
As a result, the two summaries can differ materially even when they are fitted
through the same backend.

## Choosing among repeated-measures methods

The method choice should follow the scientific question.

- Use `rmcorr()` for within-subject association.
- Use `ba_rm()` for repeated-measures bias and limits of agreement.
- Use `ccc_rm_ustat()` or `ccc_rm_reml()` for repeated concordance.
- Use `icc_rm_reml()` for repeated reliability under a variance-components
  interpretation.

The shared long-format interface is intentional. The statistical targets are
different, but the package keeps the surrounding workflow stable.

---
title: "1. Introduction to matrixCorr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{1. Introduction to matrixCorr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Introduction

`matrixCorr` is organised around a simple idea: correlation, agreement, and
reliability workflows are often used together, so they should not require
completely different interfaces, object classes, and inspection patterns.

The package therefore combines several estimator families behind a common
matrix-oriented interface for wide data and a shared long-format workflow for
repeated-measures designs. The purpose of this vignette is not to describe
every supported method in detail, but to show how the package is structured,
what kinds of inputs it expects, and how the returned objects can be inspected.

## Main workflow families

At a practical level, the package is centred on five workflow families.

- Wide-data correlation matrices for numeric inputs.
- Robust and high-dimensional association estimation.
- Latent and mixed-scale correlation for binary, ordinal, and mixed inputs.
- Agreement and reliability analysis for wide data.
- Repeated-measures correlation and repeated-measures agreement.

The same broad inspection pattern is used throughout to fit an estimator, print
it for a compact view, call `summary()` for a longer digest, and use `plot()`
when a graphical summary is available.

## Wide matrix workflow

The matrix-style functions accept a numeric matrix or data frame and return a
square result indexed by the original column names.

```{r}
library(matrixCorr)

set.seed(1)
X <- as.data.frame(matrix(rnorm(120), ncol = 4))
names(X) <- paste0("V", 1:4)

fit_pearson <- pearson_corr(X, ci = TRUE)
fit_spearman <- spearman_rho(X, ci = TRUE)

print(fit_pearson, digits = 2)
summary(fit_spearman)
```

For fitted objects with confidence intervals, the public accessors expose the
stored pieces without asking users to read attributes directly.

```{r}
estimate(fit_pearson)
confint(fit_pearson)
ci(fit_pearson)
tidy(fit_pearson)
```

This pattern extends to other wide-data estimators such as `kendall_tau()`,
`dcor()`, `bicor()`, `pbcor()`, `wincor()`, `skipped_corr()`,
`shrinkage_corr()`, `pcorr()`, `ccc()`, and `icc()`.

## Agreement and reliability workflow

Agreement methods answer a different question from ordinary correlation.
Correlation asks whether two variables move together. Agreement asks whether
two methods give sufficiently similar values on the measurement scale itself.

```{r}
set.seed(2)
ref <- rnorm(40, mean = 10, sd = 2)
alt <- ref + 0.3 + rnorm(40, sd = 0.8)

fit_ba <- ba(ref, alt)
print(fit_ba)

wide_methods <- data.frame(
  m1 = ref + rnorm(40, sd = 0.2),
  m2 = ref + 0.2 + rnorm(40, sd = 0.3),
  m3 = ref - 0.1 + rnorm(40, sd = 0.4)
)

fit_ccc <- ccc(wide_methods)
fit_icc <- icc(wide_methods, scope = "pairwise")

summary(fit_ccc)
summary(fit_icc)
```

For ICC, `scope = "pairwise"` and `scope = "overall"` answer different
questions as well. Pairwise ICC asks how reliable each specific method pair is.
Overall ICC asks how reliable the full set of columns is when analysed jointly.

```{r}
fit_icc_overall <- icc(wide_methods, scope = "overall", ci = TRUE)
print(fit_icc_overall)
summary(fit_icc_overall)
estimate(fit_icc_overall)
confint(fit_icc_overall)
ci(fit_icc_overall)
tidy(fit_icc_overall)
```

## Repeated-measures workflow

Repeated-measures functions require long-format data and explicit identifiers
for subjects and, when relevant, methods and time.

```{r}
set.seed(3)
n_subject <- 12
n_rep <- 3

subject <- rep(seq_len(n_subject), each = n_rep)
signal <- rnorm(n_subject * n_rep)
subject_x <- rnorm(n_subject, sd = 1.2)[subject]
subject_y <- rnorm(n_subject, sd = 1.0)[subject]

dat_rm <- data.frame(
  id = subject,
  x = subject_x + signal + rnorm(n_subject * n_rep, sd = 0.2),
  y = subject_y + 0.7 * signal + rnorm(n_subject * n_rep, sd = 0.3),
  z = subject_y - 0.4 * signal + rnorm(n_subject * n_rep, sd = 0.4)
)

fit_rmcorr <- rmcorr(dat_rm, response = c("x", "y", "z"), subject = "id")
print(fit_rmcorr, digits = 2)
summary(fit_rmcorr)
```

Agreement and reliability in repeated designs use a different long-format
interface because they need method and often time identifiers.

```{r}
set.seed(4)
n_id <- 10
n_time <- 3

dat_agree <- expand.grid(
  id = factor(seq_len(n_id)),
  time = factor(seq_len(n_time)),
  method = factor(c("A", "B"))
)

subj <- rnorm(n_id, sd = 1.0)[dat_agree$id]
subj_method <- rnorm(n_id * 2, sd = 0.2)
sm <- subj_method[(as.integer(dat_agree$id) - 1L) * 2L + as.integer(dat_agree$method)]

dat_agree$y <- subj + sm + 0.25 * (dat_agree$method == "B") +
  rnorm(nrow(dat_agree), sd = 0.35)

fit_icc_rm <- icc_rm_reml(
  dat_agree,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  type = "consistency"
)

summary(fit_icc_rm)
```

## Shared inspection methods

Most returned objects support at least `print()` and `summary()`. Many also
support `plot()`. Matrix-style objects are intentionally compact when printed,
while `summary()` returns a longer digest of the strongest or most relevant
pairs.

Display defaults can be controlled through ordinary R options. For example:

```{r, eval = FALSE}
options(
  matrixCorr.threads = parallel::detectCores(logical = FALSE),
  matrixCorr.print_max_rows = 20L,
  matrixCorr.print_topn = 5L,
  matrixCorr.print_max_vars = 10L,
  matrixCorr.print_show_ci = "yes",
  matrixCorr.summary_max_rows = 12L,
  matrixCorr.summary_topn = 5L,
  matrixCorr.summary_max_vars = 10L,
  matrixCorr.summary_show_ci = "yes"
)
```

`matrixCorr.threads` defines the default for `n_threads` across thread-aware
estimators. The print options control compact console previews returned by
`print()`. The summary options control the longer digest returned by
`summary()`. Current values can be inspected with
`getOption("matrixCorr.print_max_rows")` and the same pattern for the remaining
options.

This shared display layer is part of the package design. The goal is that
users can move across workflow families without relearning how objects are
inspected, while still being able to tune how much output is shown by default.

## Where to go next

The remaining vignettes focus on specific workflow families. Specifically

- `vignette("v02-wide-correlation-workflows", package = "matrixCorr")`
- `vignette("v03-robust-and-highdim-correlation", package = "matrixCorr")`
- `vignette("v04-latent-and-mixed-scale-correlation", package = "matrixCorr")`
- `vignette("v05-agreement-and-icc-wide", package = "matrixCorr")`
- `vignette("v06-repeated-measures-workflows", package = "matrixCorr")`

---
title: "5. Agreement and ICC for Wide Data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{5. Agreement and ICC for Wide Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

## Scope

Agreement and reliability are related to correlation, but they are not the
same problem. Correlation describes co-movement. Agreement describes similarity
on the measurement scale itself. Reliability describes the proportion of
variation attributable to stable differences among subjects rather than to
measurement error or method disagreement.

This vignette focuses on `ccc()` for wide data and uses related agreement and
reliability functions as context:

- `ccc()`
- `ba()`
- `icc()`
- `ccc_rm_reml()` and `ccc_rm_ustat()`
- `icc_rm_reml()`
- `cia()` and `cia_rm()`

## Pairwise concordance and Bland-Altman analysis

Lin's concordance correlation coefficient combines precision and accuracy in a
single number. In `matrixCorr`, `ccc()` computes Lin's pairwise CCC for numeric
wide data and optionally returns large-sample confidence intervals. No formal
hypothesis test is implemented; inference is based on the estimate and its
confidence interval.

Bland-Altman analysis separates the agreement question into estimated bias and
limits of agreement.

```{r}
library(matrixCorr)

set.seed(40)
ref <- rnorm(50, mean = 100, sd = 10)
m1 <- ref + rnorm(50, sd = 2)
m2 <- ref + 1.2 + rnorm(50, sd = 3)

fit_ba <- ba(m1, m2)
fit_ccc <- ccc(data.frame(m1 = m1, m2 = m2), ci = TRUE)

print(fit_ba)
summary(fit_ccc)
estimate(fit_ccc)
confint(fit_ccc)
ci(fit_ccc)
tidy(fit_ccc)
```

The two summaries are complementary rather than redundant. `ccc()` gives a
single concordance coefficient, while `ba()` makes the scale of disagreement
explicit.

If you have at least 3 methods in wide form, `ba()` can now compute every
unordered Bland-Altman contrast directly:

```{r}
fit_ba_pairwise <- ba(data.frame(m1 = m1, m2 = m2, m3 = ref - 0.8 + rnorm(50, sd = 2.5)))
print(fit_ba_pairwise)
summary(fit_ba_pairwise)
```

## Pairwise ICC

`icc()` extends the wide-data reliability workflow in two directions. It can
return a pairwise matrix across method pairs, or it can return the overall
classical ICC table for the full set of methods.

```{r}
wide_methods <- data.frame(
  J1 = ref + rnorm(50, sd = 1.5),
  J2 = ref + 4.0 + rnorm(50, sd = 1.8),
  J3 = ref - 3.0 + rnorm(50, sd = 2.0),
  J4 = ref + rnorm(50, sd = 1.6)
)

fit_icc_pair <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "pairwise"
)

fit_icc_overall <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "overall",
  ci = TRUE
)

print(fit_icc_pair, digits = 2)
summary(fit_icc_pair)
print(fit_icc_overall)
```

## Pairwise versus overall ICC

This is the most important distinction in the ICC interface.

`scope = "pairwise"` answers:
"How reliable is each specific pair of methods?"

`scope = "overall"` answers:
"How reliable is the full set of methods when analysed jointly?"

Those are different quantities. The overall ICC cannot, in general, be
recovered by averaging the pairwise matrix.

## Consistency versus agreement

This simulation also includes systematic method bias, so it is a natural place
to contrast `type = "consistency"` with `type = "agreement"`.

```{r}
fit_icc_cons <- icc(
  wide_methods,
  model = "twoway_random",
  type = "consistency",
  unit = "single",
  scope = "overall",
  ci = FALSE
)

fit_icc_agr <- icc(
  wide_methods,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "overall",
  ci = FALSE
)

data.frame(
  type = c("consistency", "agreement"),
  selected_coefficient = c(
    attr(fit_icc_cons, "selected_coefficient"),
    attr(fit_icc_agr, "selected_coefficient")
  ),
  estimate = c(
    attr(fit_icc_cons, "selected_row")$estimate,
    attr(fit_icc_agr, "selected_row")$estimate
  )
)
```

Consistency discounts additive method shifts, whereas agreement penalises them.
When methods differ mainly by a systematic offset, consistency can therefore
look substantially better than agreement.

## Model, type, and unit

The classical ICC family is controlled by three arguments.

- `model` selects the one-way, two-way random, or two-way mixed formulation.
- `type` selects consistency or agreement.
- `unit` selects single-measure or average-measure reliability.

For pairwise ICC, average-measure output uses `k = 2` because each estimate is
based on exactly two methods. For overall ICC, average-measure output uses the
full number of analysed columns.

## Agreement indices implemented in matrixCorr

CCC, ICC and CIA address related but distinct agreement questions. CCC
quantifies agreement between measurements by combining precision and accuracy
(Lin, 1989). ICC expresses reliability or agreement through variance
components; its interpretation depends on the study design, model form, and
whether agreement or consistency is targeted (Shrout and Fleiss, 1979; McGraw
and Wong, 1996). CIA targets individual agreement or interchangeability by
comparing disagreement between methods with disagreement within methods
(Barnhart, Kosinski, and Haber, 2007; Barnhart et al., 2007).

These indices should not be treated as interchangeable without considering the
scientific question, data structure, and implemented estimator.

### CCC functions

`ccc()` is the simple wide-data CCC implementation. It estimates pairwise Lin
CCC values and, when `ci = TRUE`, returns Lin delta-method/Fisher-z confidence
intervals. It does not report a p-value or test decision.

For repeated-measures data, the package provides two CCC routes. `ccc_rm_reml()`
fits pairwise mixed models by REML and converts the fitted variance components
and fixed-effect bias term into repeated-measures CCC estimates. `ccc_rm_ustat()`
computes a nonparametric U-statistic repeated-measures CCC with optional
Fisher-z confidence intervals. These repeated-measures functions estimate
agreement parameters and optional confidence intervals; they do not implement a
formal hypothesis test of the CCC parameter. `ccc_rm_reml()` can use
boundary-aware likelihood-ratio tests for variance-component selection when
`vc_select = "auto"`, but those tests are model-selection diagnostics rather
than tests of CCC agreement.

### ICC functions

`icc()` computes classical ANOVA ICC forms for wide data. With
`scope = "pairwise"` it returns a pairwise matrix; with `scope = "overall"` it
returns the standard six-form overall coefficient table. The selected ICC form
is controlled by `model`, `type`, and `unit`, so the same numeric value should
not be interpreted without those design choices.

Pairwise `icc()` and repeated-measures `icc_rm_reml()` provide estimates and
optional confidence intervals without a formal test of a target ICC parameter.
For `icc(scope = "overall")`, the package reports ANOVA F statistics, degrees
of freedom, and p-values in the overall coefficient table. The implementation
and tests verify those reported p-values, but the package documentation does
not define a generic ICC agreement hypothesis for users; this vignette therefore
does not describe the overall ICC p-values as a generic test of agreement.

### CIA functions

`cia()` and `cia_rm()` are included here only as conceptual comparisons. CIA
targets individual agreement or interchangeability rather than the same
population concordance question targeted by CCC. In `cia()`, replicated readings
within each subject-method cell are required because the estimator compares
between-method disagreement with within-method replicate disagreement. The
function supports pairwise and overall CIA, optional reference-method scaling,
and optional confidence intervals by the selected inference method. No formal
hypothesis test is implemented; inference is based on the estimate and its
confidence interval.

`cia_rm()` is different from `cia()` because it targets matched repeated measurements
under conditions such as visits, raters, laboratories, treatments, or time
points, and it is not a technical-replicate estimator. It reports
condition-specific CIA estimates, optional confidence intervals, and a
homogeneity test for agreement across conditions. As implemented, the reported
test statistic is
`MS_method_time / MS_error`, with an upper-tail F-test p-value using
`df_method_time` and `df_error`. The null hypothesis documented by the function
is homogeneous agreement across conditions; the alternative is that agreement
changes across conditions.

### Function summary

In this package, the functions differ not only in the target index but also in
the supported data structure and inferential procedure. Some functions provide
estimates and confidence intervals only. A formal hypothesis test should only
be interpreted when the function explicitly implements and reports one.

```{r, echo = FALSE}
agreement_summary <- data.frame(
  Function = c(
    "`ccc()`",
    "`ccc_rm_reml()`",
    "`ccc_rm_ustat()`",
    "`icc()`",
    "`icc_rm_reml()`",
    "`cia()`",
    "`cia_rm()`"
  ),
  `Index family` = c(
    "Lin CCC",
    "Repeated-measures CCC",
    "Repeated-measures CCC",
    "Classical ICC",
    "Repeated-measures ICC",
    "CIA",
    "Repeated-measures CIA"
  ),
  `Target question` = c(
    "Pairwise concordance combining precision and accuracy",
    "Pairwise repeated-measures concordance from fitted variance components",
    "Pairwise repeated-measures concordance from U-statistic distances",
    "Reliability or agreement under the selected ICC model, type, and unit",
    "Pairwise repeated-measures reliability/agreement from fitted variance components",
    "Individual agreement/interchangeability relative to within-method replicate disagreement",
    "Individual agreement/interchangeability across matched repeated conditions"
  ),
  `Data/design supported` = c(
    "Numeric wide data; rows are paired observational units",
    "Long repeated-measures data; subject plus optional method/time structure",
    "Long repeated-measures data; balanced method/time coverage per pair",
    "Numeric wide data; pairwise or overall all-column scope",
    "Long repeated-measures data; subject plus optional method/time structure",
    "Long replicated method-comparison data with replicate identifiers",
    "Long matched repeated-measures data with one observation per subject-method-condition cell"
  ),
  `Estimation approach` = c(
    "Moment CCC with Lin delta-method/Fisher-z CI when requested",
    "REML mixed-model variance components and fixed-effect bias term",
    "Nonparametric U-statistic estimator with Fisher-z CI when requested",
    "Classical ANOVA mean-square formulas",
    "REML mixed-model variance components and fixed-effect bias term",
    "Method-of-moments disagreement ratios; optional bounded variance-component variant",
    "Categorical-condition ANOVA estimator"
  ),
  `Inference reported` = c(
    "Estimate; optional confidence interval",
    "Estimate; optional confidence interval; variance-component diagnostics",
    "Estimate; optional confidence interval",
    "Pairwise: estimate and optional CI. Overall: coefficient table with F statistic, df, p-value, and optional CI",
    "Estimate; optional confidence interval; variance-component diagnostics",
    "Estimate; optional confidence interval",
    "Estimate; optional confidence interval; homogeneity F statistic and p-value"
  ),
  `Formal hypothesis test implemented? Yes/No/Requires verification` = c(
    "No",
    "No for the CCC parameter; variance-component selection tests are not CCC tests",
    "No",
    "Requires verification for overall p-values; no pairwise ICC test is documented",
    "No for the ICC parameter; variance-component selection tests are not ICC tests",
    "No",
    "Yes: homogeneity of agreement across conditions"
  ),
  check.names = FALSE
)

knitr::kable(agreement_summary, escape = FALSE)
```


## Choosing among CCC, BA, ICC, and CIA

In practice these methods answer different questions.

- Use `ccc()` when one concordance coefficient per pair is the main target.
- Use `ccc_rm_ustat()` or `ccc_rm_reml()` when the concordance target is
  repeated-measures CCC rather than simple paired wide-data CCC.
- Use `ba()` when the size and direction of disagreement should be visible on
  the original measurement scale.
  Use `ba_rm()` for the repeated-measures Bland-Altman workflow.
- Use `icc()` when the target is reliability under a classical variance
  components interpretation.
- Use `icc_rm_reml()` when the target is repeated-measures ICC from the
  package's REML variance-component backend.
- Use `cia()` when replicated readings within subject-method cells are
  available and the target is individual agreement or interchangeability
  relative to within-method replicate disagreement.
- Use `cia_rm()` when the target is individual agreement across matched
  repeated conditions rather than technical replicates.

There is overlap in interpretation, but these are not interchangeable
estimators.

## References

Barnhart HX, Haber M, Lokhnygina Y, Kosinski AS. (2007). Comparison of
concordance correlation coefficient and coefficient of individual agreement in
assessing agreement. *Journal of Biopharmaceutical Statistics*, 17(4), 721-738.

Barnhart HX, Kosinski AS, Haber M. (2007). Assessing individual agreement.
*Journal of Biopharmaceutical Statistics*, 17(4), 697-719.

Carrasco JL, Jover L. (2003). Estimating the concordance correlation
coefficient: a new approach. *Computational Statistics & Data Analysis*, 47(4),
519-539.

Carrasco JL, Phillips BR, Puig-Martinez J, King TS, Chinchilli VM. (2013).
Estimation of the concordance correlation coefficient for repeated measures
using SAS and R. *Computer Methods and Programs in Biomedicine*, 109(3),
293-304.

Haber M, Gao J, Barnhart HX. (2010). Evaluation of agreement between
measurement methods from data with matched repeated measurements via the
coefficient of individual agreement. *Journal of Data Science*, 8, 457-469.

Lin L. (1989). A concordance correlation coefficient to evaluate
reproducibility. *Biometrics*, 45, 255-268.

McGraw KO, Wong SP. (1996). Forming inferences about some intraclass
correlation coefficients. *Psychological Methods*, 1(1), 30-46.

Pan Y, Gao J, Haber M, Barnhart HX. (2010). Estimation of coefficients of
individual agreement (CIA's) for quantitative and binary data using SAS and R.
*Computer Methods and Programs in Biomedicine*.

Shrout PE, Fleiss JL. (1979). Intraclass correlations: uses in assessing rater
reliability. *Psychological Bulletin*, 86(2), 420-428.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{polyserial}
\alias{polyserial}
\alias{print.polyserial_corr}
\alias{plot.polyserial_corr}
\alias{summary.polyserial_corr}
\alias{print.summary.polyserial_corr}
\title{Polyserial Correlation Between Continuous and Ordinal Variables}
\usage{
polyserial(data, y, na_method = c("error", "pairwise"), ci = FALSE, p_value = FALSE,
  conf_level = 0.95, ...)

\method{print}{polyserial_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{polyserial_corr}(
  x,
  title = "Polyserial correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{polyserial_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.polyserial_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric vector, matrix, or data frame containing continuous
variables.}

\item{y}{An ordinal vector, matrix, or data frame containing ordinal
variables. Supported columns are factors, ordered factors, logical values,
or integer-like numerics.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
complete cases.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald confidence intervals derived from the observed
information matrix of the latent-variable likelihood.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald p-values and test statistics for each
estimated latent correlation.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{...}{Additional arguments passed to \code{print()}.}

\item{x}{An object of class \code{summary.polyserial_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Polyserial correlation heatmap"}.}

\item{low_color}{Color for the minimum correlation.}

\item{high_color}{Color for the maximum correlation.}

\item{mid_color}{Color for zero correlation.}

\item{value_text_size}{Font size used in tile labels.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{polyserial_corr}.}

\item{ci_digits}{Integer; digits for confidence limits in the pairwise
summary.}

\item{p_digits}{Integer; digits for p-values in the pairwise summary.}
}
\value{
If both \code{data} and \code{y} are vectors, a numeric scalar. Otherwise a
numeric matrix of class \code{polyserial_corr} with rows corresponding to
the continuous variables in \code{data} and columns to the ordinal variables
in \code{y}. Matrix outputs carry attributes \code{method},
\code{description}, and \code{package = "matrixCorr"}. When
\code{p_value = TRUE}, the returned object also carries an \code{inference}
attribute with elements \code{estimate}, \code{statistic}, \code{parameter},
\code{p_value}, and \code{n_obs}. When \code{ci = TRUE}, it also carries a
\code{ci} attribute with elements \code{est}, \code{lwr.ci},
\code{upr.ci}, \code{conf.level}, and \code{ci.method}, plus
\code{attr(x, "conf.level")}. Scalar outputs keep the same point estimate
and gain the same metadata only when inference is requested.
}
\description{
Computes polyserial correlations between continuous variables in \code{data}
and ordinal variables in \code{y}. Both pairwise vector mode and rectangular
matrix/data-frame mode are supported.
}
\details{
The polyserial correlation assumes a latent bivariate normal model between a
continuous variable and an unobserved continuous propensity underlying an
ordinal variable. Let
\eqn{(X, Z)^\top \sim N_2(0, \Sigma)} with
\eqn{\mathrm{corr}(X,Z)=\rho}, and suppose the observed ordinal response
\eqn{Y} is formed by cut-points
\eqn{-\infty = \beta_0 < \beta_1 < \cdots < \beta_K = \infty}:
\deqn{
Y = k \iff \beta_{k-1} < Z \le \beta_k.
}
After standardising the observed continuous variable \eqn{X}, the thresholds
are estimated from the marginal proportions of \eqn{Y}. Conditional on an
observed \eqn{x_i}, the category probability is
\deqn{
\Pr(Y_i = k \mid X_i = x_i, \rho)
=
\Phi\!\left(\frac{\beta_k - \rho x_i}{\sqrt{1-\rho^2}}\right)
-
\Phi\!\left(\frac{\beta_{k-1} - \rho x_i}{\sqrt{1-\rho^2}}\right).
}
The returned estimate maximises the log-likelihood
\deqn{
\ell(\rho) = \sum_{i=1}^{n}\log \Pr(Y_i = y_i \mid X_i = x_i, \rho)
}
over \eqn{\rho \in (-1,1)} via a one-dimensional Brent search in C++.

\strong{Assumptions.} The coefficient is appropriate when the ordinal
variable is viewed as the discretised version of a latent normal variable
that is jointly normal with the observed continuous variable. The optional
p-values and confidence intervals adopt this latent-normal interpretation
and use the same likelihood that defines the polyserial estimate. These
inferential quantities are therefore model-based and should not be
interpreted as distribution-free summaries.

\strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
function refits the pairwise polyserial model by maximum likelihood and
obtains the observed information matrix numerically in C++. The reported
confidence interval is a Wald interval
\eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
reported p-value is from the large-sample Wald \eqn{z}-test for
\eqn{H_0:\rho = 0}. These inferential quantities are only computed when
explicitly requested.

In vector mode a single estimate is returned. In matrix/data-frame mode,
every numeric column of \code{data} is paired with every ordinal column of
\code{y}, producing a rectangular matrix of continuous-by-ordinal
polyserial correlations.

\strong{Computational complexity.} If \code{data} has \eqn{p_x} continuous
columns and \code{y} has \eqn{p_y} ordinal columns, the matrix path computes
\eqn{p_x p_y} separate one-parameter likelihood optimisations.
}
\examples{
\dontshow{if (requireNamespace("mnormt", quietly = TRUE)) withAutoprint(\{ # examplesIf}
set.seed(125)
n <- 1000
Sigma <- matrix(c(
  1.00, 0.30, 0.55, 0.20,
  0.30, 1.00, 0.25, 0.50,
  0.55, 0.25, 1.00, 0.40,
  0.20, 0.50, 0.40, 1.00
), 4, 4, byrow = TRUE)

Z <- mnormt::rmnorm(n = n, mean = rep(0, 4), varcov = Sigma)
X <- data.frame(x1 = Z[, 1], x2 = Z[, 2])
Y <- data.frame(
  y1 = ordered(cut(
    Z[, 3],
    breaks = c(-Inf, -0.5, 0.7, Inf),
    labels = c("low", "mid", "high")
  )),
  y2 = ordered(cut(
    Z[, 4],
    breaks = c(-Inf, -1.0, 0.0, 1.0, Inf),
    labels = c("1", "2", "3", "4")
  ))
)

ps <- polyserial(X, Y)
print(ps, digits = 3)
summary(ps)
estimate(ps)
tidy(ps)
ps_ci <- polyserial(X, Y, ci = TRUE)
ci(ps_ci)
confint(ps_ci)
plot(ps)
\dontshow{\}) # examplesIf}
}
\references{
Olsson, U., Drasgow, F., & Dorans, N. J. (1982). The polyserial
correlation coefficient. \emph{Psychometrika}, 47(3), 337-347.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chatterjee_xi.R
\name{xi_corr}
\alias{xi_corr}
\alias{print.chatterjee_xi}
\alias{print.chatterjee_xi_scalar}
\alias{summary.chatterjee_xi_scalar}
\alias{print.summary.chatterjee_xi_scalar}
\title{Pairwise Chatterjee Rank Correlation}
\usage{
xi_corr(
  data,
  y = NULL,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  ci_method = c("auto", "dette_kroll", "n_choose_m"),
  bootstrap_reps = 999L,
  m = NULL,
  large_sample_cutoff = 1000L,
  bias_correction = c("none", "upper_bound"),
  tie_method = c("random", "first"),
  seed = NULL,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{chatterjee_xi}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{print}{chatterjee_xi_scalar}(x, digits = 4, ci_digits = 4, ...)

\method{summary}{chatterjee_xi_scalar}(object, digits = 4, ci_digits = 4, ...)

\method{print}{summary.chatterjee_xi_scalar}(x, digits = NULL, ci_digits = NULL, ...)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns, or a numeric predictor vector when \code{y} is supplied. All
non-numeric columns will be excluded in matrix/data-frame mode. Each
retained column must have at least two non-missing values.}

\item{y}{Optional numeric response vector for two-vector mode. When supplied,
\code{xi_corr(data, y)} estimates \eqn{\xi(data, y)}, where \code{data}
is the sorting/predictor variable and \code{y} is the response/ranked
variable.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each directed pair on its own pairwise
complete-case overlap. This is permissive, but different directed entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes all
directed entries on the same complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
m-out-of-n bootstrap confidence intervals for the directed Chatterjee
estimates. In two-vector mode the scalar estimate carries a \code{ci}
attribute.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{ci_method}{Confidence interval method:
\itemize{
\item \code{"auto"} (default): use \code{"dette_kroll"} when the
pair-specific complete-case sample size is less than or equal to
\code{large_sample_cutoff}, and \code{"n_choose_m"} for larger samples.
\item \code{"dette_kroll"}: m-out-of-n variance estimation followed by a
normal interval, following Dette and Kroll.
\item \code{"n_choose_m"}: non-parametric m-out-of-n basic interval based
on the centred and scaled bootstrap statistic, following the large-sample
n-choose-m approach used by Dalitz, Arning and Goebbels.
}}

\item{bootstrap_reps}{Number of m-out-of-n bootstrap replicates used when
\code{ci = TRUE}. Larger values reduce Monte Carlo noise but increase
runtime.}

\item{m}{Optional subsample size for the m-out-of-n bootstrap. If
\code{NULL}, method-specific defaults are used: \code{floor(sqrt(n))} for
\code{"dette_kroll"} and \code{round(2 * sqrt(n))} for
\code{"n_choose_m"}, each bounded to \code{[2, n - 1]}.}

\item{large_sample_cutoff}{For \code{ci_method = "auto"}, use
\code{"n_choose_m"} when pair-specific \code{n_complete} is greater than
this cutoff; otherwise use \code{"dette_kroll"}.}

\item{bias_correction}{Finite-sample normalisation:
\itemize{
\item \code{"none"} (default): return Chatterjee's original finite-sample
statistic.
\item \code{"upper_bound"}: divide by the raw finite-sample upper-bound
estimate \eqn{\xi_n(Y,Y)} and return
\eqn{\max(-1, \xi_n(X,Y) / \xi_n(Y,Y))}. This can reduce the finite-sample
downward bias of the raw statistic, but it is not the default.
}}

\item{tie_method}{How ties in the sorting variable \eqn{X} are broken:
\itemize{
\item \code{"random"} (default): randomly permute observations within tied
\eqn{X} groups, matching Chatterjee's random tie-breaking definition.
\item \code{"first"}: preserve stable input order within tied \eqn{X}
groups, useful for deterministic diagnostics and reproducible comparisons.
}}

\item{seed}{Optional positive integer seed for reproducible tie breaking and
bootstrap resampling.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}. Random tie breaking and
confidence-interval paths are evaluated serially to preserve R RNG
semantics.}

\item{output}{Output representation for the computed directed estimates.
\itemize{
\item \code{"matrix"} (default): full dense directed matrix; best when you
need matrix algebra, heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing retained
directed entries; useful when thresholding leaves relatively few values.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, and \code{value}; convenient for filtering,
joins, and graph/network workflows. Because \eqn{\xi(X,Y)} is directed,
both directions are represented when retained.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger directed associations
with \code{output = "sparse"} or \code{"edge_list"}. Must be \code{0}
when \code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Compatibility arguments. The deprecated \code{check_na} alias is
accepted.}

\item{x}{An object of class \code{chatterjee_xi}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL}
derives this from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for confidence limits.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{object}{An object of class \code{chatterjee_xi_scalar}.}
}
\value{
A directed numeric matrix where the \code{(i, j)}-th element is
Chatterjee's \eqn{\xi} from the \code{i}-th numeric column to the
\code{j}-th numeric column. The dense matrix inherits from
\code{c("corr_matrix", "chatterjee_xi", "corr_result", "matrix")}. When
\code{ci = TRUE}, the object also carries a \code{ci} attribute with
elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
\code{ci.method}, \code{se}, \code{m}, and \code{bootstrap_reps}. When
pairwise-complete evaluation is used, pairwise sample sizes are stored in
\code{attr(x, "diagnostics")$n_complete}. In two-vector mode, a numeric
scalar is returned; when \code{ci = TRUE}, it carries confidence-interval
attributes.
}
\description{
Computes the directed Chatterjee rank correlation coefficient for numeric
vectors or for all directed column pairs of a numeric matrix/data frame.
The matrix orientation is \code{result[i, j] = xi(V_i, V_j)}, where
\code{V_i} is the predictor/sorting variable and \code{V_j} is the
response/ranked variable.
}
\details{
Let \eqn{(X_k,Y_k), k=1,\ldots,n}, be complete paired observations.
Chatterjee's rank correlation is directed: \eqn{\xi(X,Y)} measures how well
\eqn{Y} is functionally determined by \eqn{X}. It is not a symmetric
association measure, so in matrix mode
\deqn{\mathrm{result}_{ij} = \xi(X_{\cdot i}, X_{\cdot j})}
need not equal \eqn{\mathrm{result}_{ji}}. The row variable is always the
sorting/predictor variable and the column variable is the response/ranked
variable.

For a given directed pair, observations are sorted by \eqn{X}. Let
\eqn{r_i} be the number of response values \eqn{Y_j \le Y_i}, and let
\eqn{l_i} be the number of response values \eqn{Y_j \ge Y_i}, evaluated for
the \eqn{i}-th observation in sorted-by-\eqn{X} order. The tied-response
finite-sample estimator is
\deqn{
\xi_n = 1 -
\frac{n \sum_{i=1}^{n-1} |r_{i+1} - r_i|}
     {2 \sum_{i=1}^n l_i (n - l_i)}.
}
If all response values are equal, the denominator is zero and the estimate
is \code{NA}. When there are no ties in \eqn{Y}, this reduces to the familiar
rank-difference expression
\deqn{
\xi_n = 1 -
\frac{3 \sum_{i=1}^{n-1}|r_{i+1}-r_i|}{n^2 - 1}.
}
The raw finite-sample statistic is not forced to one on the diagonal; with
no ties and \code{X = Y}, it equals \eqn{(n - 2)/(n + 1)}. Use
\code{bias_correction = "upper_bound"} only when this finite-sample
upper-bound normalisation is desired.

Ties in the sorting variable \eqn{X} are handled before computing adjacent
rank differences. Chatterjee's definition uses random tie breaking, provided
here by \code{tie_method = "random"}. For reproducible diagnostics or when
the input order should define tied-\eqn{X} ordering, use
\code{tie_method = "first"}. Ties in the response variable \eqn{Y} are
handled by the general \eqn{r_i,l_i} formula above and do not require random
tie breaking.

\strong{Confidence intervals.} The ordinary n-out-of-n bootstrap is not
used for Chatterjee's coefficient. Both available intervals use
m-out-of-n subsampling without replacement, consistent with the bootstrap
family considered for Chatterjee's rank correlation.
With \code{ci_method = "dette_kroll"}, subsamples of size \eqn{m} are drawn
without replacement and the limiting standard deviation is estimated as
\deqn{\widehat{\sigma} = \sqrt{m}\,\mathrm{sd}(\xi_m^*).}
The reported interval is the normal interval
\deqn{
\xi_n \pm z_{1-\alpha/2}\widehat{\sigma}/\sqrt{n},
\quad \alpha = 1-\texttt{conf\_level}.
}
If \code{m = NULL}, this method uses \eqn{m=\lfloor\sqrt{n}\rfloor}, bounded
to \eqn{[2,n-1]}.

With \code{ci_method = "n_choose_m"}, subsamples are also drawn without
replacement, but the interval is obtained by inverting the empirical
quantiles of the centred and scaled statistic
\deqn{T^* = \sqrt{m}(\xi_m^* - \xi_n).}
If \eqn{q_{\alpha/2}} and \eqn{q_{1-\alpha/2}} are the bootstrap quantiles,
the basic m-out-of-n limits are
\deqn{
\xi_n - q_{1-\alpha/2}/\sqrt{n}
\quad\text{and}\quad
\xi_n - q_{\alpha/2}/\sqrt{n}.
}
If finite Monte Carlo quantiles fall entirely on one side of zero, the
inversion is anchored at zero so that the reported interval contains the
observed estimate; the bounds are not clipped to the population parameter
range. If \code{m = NULL}, this method uses
\eqn{m=\mathrm{round}(2\sqrt{n})}, bounded to \eqn{[2,n-1]}, following the
implementation rule used by Dalitz, Arning and Goebbels.

\strong{Which CI method should be used?} The Dette-Kroll interval is the
more conservative default for small to moderate complete-case sample sizes
in this implementation, because it uses the m-out-of-n bootstrap only to
estimate a normal-approximation standard error. The non-parametric
n-choose-m interval is useful for larger samples when a direct
m-out-of-n bootstrap interval is preferred, but Dalitz, Arning and Goebbels
report that this interval may need fairly large \eqn{n} to approach nominal
coverage in some settings. Therefore \code{ci_method = "auto"} uses
\code{"dette_kroll"} when pair-specific \code{n_complete <=}
\code{large_sample_cutoff} and \code{"n_choose_m"} when
\code{n_complete > large_sample_cutoff}. Increase
\code{bootstrap_reps} for final analyses to reduce Monte Carlo error, and
consider setting \code{m} explicitly when a study protocol requires a
fixed subsample size.

\strong{Computation.} For complete finite matrices without confidence
intervals, the C++ backend precomputes each column's sorting order and
response ranks and reuses them across directed pairs. This costs
\eqn{O(p\,n\log n + p^2 n)} time and \eqn{O(pn + p^2)} memory. Pairwise
missing-data evaluation and bootstrap confidence intervals recompute on the
relevant complete-case samples and are correspondingly more expensive.
}
\examples{
## Example 1: independence versus functional dependence
## Chatterjee's xi targets whether Y is determined by X.
set.seed(1)
n <- 300
x <- runif(n, -1, 1)
y_independent <- rnorm(n)
y_function <- sin(2 * pi * x)
c(
  independent = xi_corr(x, y_independent, tie_method = "first"),
  functional = xi_corr(x, y_function, tie_method = "first")
)

## Example 2: non-monotone functional dependence is directed
## x determines x^2, but x^2 does not determine the sign of x.
## The reverse raw finite-sample estimate can therefore be much smaller,
## and may be negative when sorted response ranks oscillate strongly.
x <- seq(-1, 1, length.out = 300)
y <- x^2
c(
  xi_x_to_y = xi_corr(x, y, tie_method = "first"),
  xi_y_to_x = xi_corr(y, x, tie_method = "first")
)

## Example 3: the raw finite-sample diagonal is not forced to one
## The optional upper-bound normalisation rescales this finite-sample ceiling.
z <- 1:20
c(
  raw = xi_corr(z, z, tie_method = "first"),
  upper_bound = xi_corr(
    z, z,
    tie_method = "first",
    bias_correction = "upper_bound"
  )
)

## Example 4: directed matrix workflow
X <- cbind(
  x = x,
  square = x^2,
  sine = sin(2 * pi * x),
  noise = rnorm(length(x))
)
xi <- xi_corr(X, tie_method = "first")
print(xi, digits = 3)
summary(xi)
estimate(xi)
tidy(xi)

## Example 5: Dette-Kroll bootstrap interval for a moderate sample
\donttest{
xi_ci <- xi_corr(
  X[, c("x", "sine")],
  ci = TRUE,
  ci_method = "dette_kroll",
  bootstrap_reps = 49,
  seed = 1,
  tie_method = "first"
)
summary(xi_ci)
ci(xi_ci)
confint(xi_ci)
plot(xi_ci)
}

## Example 6: n-choose-m interval for larger samples
\donttest{
set.seed(2)
x_large <- runif(1200, -1, 1)
y_large <- sin(2 * pi * x_large) + rnorm(1200, sd = 0.2)
xi_corr(
  x_large,
  y_large,
  ci = TRUE,
  ci_method = "n_choose_m",
  bootstrap_reps = 99,
  seed = 2,
  tie_method = "first"
)
}

}
\references{
Chatterjee, S. (2021). A New Coefficient of Correlation. \emph{Journal of
the American Statistical Association}, 116, 2009-2022.

Dette, H. and Kroll, M. (2025). A simple bootstrap for Chatterjee's rank
correlation. \emph{Biometrika}, 112(1), asae045.
\doi{10.1093/biomet/asae045}.

Lin, Z. and Han, F. (2025). Limit theorems of Chatterjee's rank correlation.
arXiv:2204.08031v4. \doi{10.48550/arXiv.2204.08031}.

Dalitz, C., Arning, J. and Goebbels, S. (2024). A Simple Bias Reduction for
Chatterjee's Correlation.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{compute_ci_from_se}
\alias{compute_ci_from_se}
\title{compute_ci_from_se}
\usage{
compute_ci_from_se(ccc, se, level)
}
\description{
Compute confidence intervals from point estimate and standard error.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gwet_ac.R
\name{gwet_ac}
\alias{gwet_ac}
\alias{summary.gwet_ac}
\alias{print.summary.gwet_ac}
\alias{print.gwet_ac}
\alias{plot.gwet_ac}
\title{Gwet's AC1 and AC2 Agreement Coefficients}
\usage{
gwet_ac(
  data,
  y = NULL,
  weights = c("unweighted", "linear", "quadratic", "ordinal", "radical", "ratio",
    "circular", "bipolar"),
  levels = NULL,
  input = c("pairwise", "ratings", "counts"),
  na_method = c("error", "pairwise", "complete", "available"),
  min_raters = 2L,
  by_category = FALSE,
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  se_method = c("asymptotic", "jackknife", "none"),
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  verbose = FALSE,
  ...
)

\method{summary}{gwet_ac}(object, digits = 4, ci_digits = 3, p_digits = 4, ...)

\method{print}{summary.gwet_ac}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{gwet_ac}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  show_by_category = FALSE,
  ...
)

\method{plot}{gwet_ac}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  type = c("agreement_map", "estimate", "item_agreement", "category_proportion",
    "by_category"),
  bins = 30L,
  ...
)
}
\arguments{
\item{data}{Input data. If \code{y} is supplied, \code{data} is the first
rater vector. Otherwise:
for \code{input = "pairwise"}, rows are units and columns are raters;
for \code{input = "ratings"}, rows are items and columns are raters;
for \code{input = "counts"}, rows are items and columns are categories
containing rater counts.}

\item{y}{Optional second rater vector for scalar two-rater agreement.}

\item{weights}{Weight specification. \code{"unweighted"} yields AC1.
Weighted schemes yield AC2. Supported built-in schemes are
\code{"linear"}, \code{"quadratic"}, \code{"ordinal"},
\code{"radical"}, \code{"ratio"}, \code{"circular"}, and
\code{"bipolar"}. A custom numeric agreement-weight matrix may also be
supplied. Character schemes must be supplied as a single value.}

\item{levels}{Optional explicit category labels. This is mainly useful when
unused categories should still be retained in the calculation, and is the
recommended way to define ordering for ordinal AC2 weights.}

\item{input}{One of \code{"pairwise"}, \code{"ratings"}, or \code{"counts"}.}

\item{na_method}{Missing-data rule. For scalar two-rater and pairwise matrix
modes, \code{"error"} rejects missing values, \code{"complete"} applies
listwise deletion in matrix mode, and \code{"pairwise"} uses complete
pairs within each pair of raters. For \code{input = "ratings"},
\code{"available"} retains items with at least \code{min_raters}
observed ratings. For \code{input = "counts"}, missing values are not
allowed.}

\item{min_raters}{Minimum number of observed ratings required for an item to
be retained in panel mode. Must be at least \code{2}.}

\item{by_category}{Logical; if \code{TRUE}, attach one-vs-all category-wise
AC1 estimates for panel-level fits. This is available only for the
unweighted coefficient.}

\item{ci}{Logical; if \code{TRUE}, attach confidence intervals.}

\item{p_value}{Logical; if \code{TRUE}, attach test statistics and p-values.}

\item{conf_level}{Confidence level for intervals. Default is \code{0.95}.}

\item{se_method}{Standard-error method. \code{"asymptotic"} uses the
analytic large-sample formula. \code{"jackknife"} is retained as an
alternative for panel-level fits. \code{"none"} disables inferential
output.}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
C++ backend.}

\item{output}{One of \code{"matrix"}, \code{"sparse"}, or
\code{"edge_list"} for pairwise matrix mode.}

\item{threshold}{Numeric threshold used only for thresholded pairwise matrix
output.}

\item{diag}{Logical; include diagonal entries in thresholded pairwise matrix
output.}

\item{verbose}{Logical; if \code{TRUE}, emit short progress messages.}

\item{...}{Unused.}

\item{object}{A \code{gwet_ac} object.}

\item{digits}{Integer; number of decimal places for displayed values.}

\item{ci_digits}{Integer; number of decimal places for confidence limits.}

\item{p_digits}{Integer; number of decimal places for p-values.}

\item{x}{A \code{gwet_ac} object.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{show_by_category}{Logical; whether to print attached category-wise
panel results when available.}

\item{title}{Optional plot title.}

\item{low_color}{Fill/color used for negative agreement.}

\item{high_color}{Fill/color used for positive agreement.}

\item{mid_color}{Unused placeholder for API consistency with matrix heatmaps.}

\item{value_text_size}{Text size for estimate labels in matrix plots.}

\item{ci_text_size}{Text size for CI labels in matrix plots.}

\item{show_value}{Logical; whether to print estimate and CI labels.}

\item{type}{Plot type for panel-level fits: \code{"agreement_map"},
\code{"estimate"}, \code{"item_agreement"}, \code{"category_proportion"},
or \code{"by_category"}.}

\item{bins}{Integer number of bins retained for compatibility with the
panel-level item-agreement plot.}
}
\value{
If \code{y} is supplied, a scalar numeric object of class
\code{c("gwet_ac", "numeric")}. If \code{input = "pairwise"}, a
\code{corr_result}. If \code{input} is \code{"ratings"} or \code{"counts"},
a one-row data frame with class
\code{c("gwet_ac", "agreement_result", "data.frame")}.
}
\description{
Estimates Gwet's chance-corrected agreement coefficient for either
unweighted nominal agreement (AC1) or weighted agreement (AC2) in two-rater
pairwise form or panel-level multi-rater form.
}
\details{
For two raters, let \eqn{w_{ab}} denote the agreement weight for cell
\eqn{(a,b)}, \eqn{p_{ab}} the cell proportion, and
\eqn{\pi_k = (p_{k+} + p_{+k})/2}. Then
\deqn{ P_o = \sum_{a,b} w_{ab} p_{ab}, }
\deqn{ P_e =
       \frac{\sum_{a,b} w_{ab}}{q(q-1)}
       \sum_{k=1}^q \pi_k(1-\pi_k), }
and
\deqn{ AC = \frac{P_o - P_e}{1 - P_e}. }
With identity weights \eqn{w_{ab} = 1(a=b)} this is Gwet's AC1, the
nominal-agreement coefficient. With non-identity agreement weights this is
Gwet's AC2, which extends the same chance-correction idea to ordered or
partially creditable disagreement by letting near disagreements receive
larger agreement weights than distant disagreements.

\strong{Category ordering for weighted AC2.} Built-in weighted schemes use
the category order to define near and distant disagreements. If
\code{levels} is supplied, that order is used. Otherwise, factor inputs use
their factor levels, numeric/integer/logical inputs use sorted observed
values, character inputs use first-observed order, and counts inputs use
column order (or \code{levels}, when supplied). For ordinal analyses,
supplying \code{levels} is recommended whenever first-observed character
order is not the intended scale order.

For panel-level counts, if \eqn{r_{ik}} is the number of raters assigning
item \eqn{i} to category \eqn{k}, \eqn{r_i = \sum_k r_{ik}}, and
\eqn{w_{kl}} is the agreement-weight matrix, then the item-level agreement is
\deqn{ A_i =
       \frac{1}{r_i(r_i-1)}
       \sum_k r_{ik}\left(\sum_l w_{kl}r_{il}-1\right). }
The observed agreement is the mean of \eqn{A_i} over retained items. The
chance term uses the average item-wise category proportions
\eqn{\pi_k = m^{-1}\sum_i r_{ik}/r_i}, where \eqn{m} is the number of
retained items.

\strong{Confidence intervals and inference.}

The primary inferential path is the analytic large-sample method. For two
raters, define
\deqn{ s_{ab}
       =
       w_{ab}
       -
       \frac{2(1-AC)\sum_{u,v} w_{uv}}{q(q-1)}
       \left(1 - \frac{\pi_a + \pi_b}{2}\right). }
The backend variance estimator is
\deqn{ \widehat{\mathrm{Var}}(\widehat{AC})
       =
       \frac{1}{n(1-P_e)^2}
       \left[
       \sum_{a,b} p_{ab} s_{ab}^2
       -
       \left\{P_o - 2(1-AC)P_e\right\}^2
       \right], }
where \eqn{n} is the number of complete paired ratings.

For panel-level counts, write
\deqn{ AC_i = \frac{A_i - P_e}{1 - P_e}, }
and define the item-specific chance term
\deqn{ P_{e,i}
       =
       \frac{\sum_{k,l} w_{kl}}{q(q-1)}
       \sum_{k=1}^q \frac{r_{ik}}{r_i}(1-\pi_k). }
The asymptotic linearised contribution used by the backend is
\deqn{ \xi_i
       =
       AC_i
       -
       \frac{2(1-AC)}{1-P_e}(P_{e,i} - P_e), }
giving
\deqn{ \widehat{\mathrm{Var}}(\widehat{AC})
       =
       \frac{1}{m(m-1)}
       \sum_{i=1}^m (\xi_i - AC)^2, }
where \eqn{m} is the number of retained items.

The reported standard error is
\eqn{\widehat{\mathrm{se}}(\widehat{AC}) =
\sqrt{\widehat{\mathrm{Var}}(\widehat{AC})}}, and the confidence interval is
the t interval
\deqn{ \widehat{AC} \pm t_{1-\alpha/2,\nu}\,
       \widehat{\mathrm{se}}(\widehat{AC}), }
truncated to \eqn{[-1,1]}. Here \eqn{\nu = n-1} for the two-rater path and
\eqn{\nu = m-1} for panel-level asymptotic inference. The reported test
statistic is the corresponding t ratio for \eqn{H_0: AC = 0}. For
panel-level fits, \code{se_method = "jackknife"} remains available as a
second option, using the leave-one-item-out variance
\deqn{ \widehat{\mathrm{Var}}_{\mathrm{JK}}(\widehat{AC})
       =
       \frac{m-1}{m}
       \sum_{i=1}^m
       \left(\widehat{AC}_{(-i)} - \overline{AC}_{(-\cdot)}\right)^2. }
}
\examples{
x <- c("A", "A", "B", "B", "A", "C")
y <- c("A", "B", "B", "B", "A", "C")
gwet_ac(x, y)
gwet_ac(x, y, weights = "quadratic", levels = c("A", "B", "C"))

raters <- data.frame(
  r1 = c("A", "A", "B", "C", "A", "B"),
  r2 = c("A", "B", "B", "C", "A", "B"),
  r3 = c("A", "A", "B", "B", "A", "C"),
  stringsAsFactors = FALSE
)

fit_pw <- gwet_ac(raters)
fit_panel <- gwet_ac(raters, input = "ratings")
print(fit_pw)
print(fit_panel)
estimate(fit_pw)
tidy(fit_pw)
tidy(fit_panel)

}
\references{
Gwet, K. L. (2008). Computing inter-rater reliability and its variance in
the presence of high agreement. \emph{British Journal of Mathematical and
Statistical Psychology}, 61, 29-48. \doi{10.1348/000711006X126600}
}
\seealso{
\code{\link[=cohen_kappa]{cohen_kappa()}} for two-rater nominal kappa;
\code{\link[=multirater_kappa]{multirater_kappa()}} for panel-level nominal kappa;
\code{\link[=weighted_kappa]{weighted_kappa()}} for weighted Cohen-type agreement.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_scalar_character}
\alias{check_scalar_character}
\title{Check scalar character (non-empty)}
\usage{
check_scalar_character(
  x,
  arg = as.character(substitute(x)),
  allow_empty = FALSE
)
}
\description{
Check scalar character (non-empty)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.mc_abort_condition}
\alias{.mc_abort_condition}
\title{Rethrow a caught condition as a cli error}
\usage{
.mc_abort_condition(cnd, .class = character())
}
\description{
Rethrow a caught condition as a cli error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_result_s3.R
\name{plot.corr_edge_list}
\alias{plot.corr_edge_list}
\title{S3 Plot for Edge-List Correlation Results}
\usage{
\method{plot}{corr_edge_list}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{x}{An edge-list correlation result.}

\item{title}{Optional plot title.}

\item{low_color}{Fill color for -1.}

\item{high_color}{Fill color for +1.}

\item{mid_color}{Fill color for 0.}

\item{value_text_size}{Text size for optional overlaid values.}

\item{ci_text_size}{Text size for optional confidence-interval labels.}

\item{show_value}{Logical; overlay values if \code{TRUE}.}

\item{...}{Additional theme arguments.}
}
\description{
S3 Plot for Edge-List Correlation Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{build_LDZ}
\alias{build_LDZ}
\title{build_LDZ}
\usage{
build_LDZ(
  colnames_X,
  method_levels,
  time_levels,
  Dsub,
  df_sub,
  method_name,
  time_name,
  slope,
  interaction,
  slope_var,
  drop_zero_cols
)
}
\description{
Internal helper to construct L, Dm, and Z matrices for random effects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_weights}
\alias{check_weights}
\title{Check weights vector (non-negative, finite, correct length)}
\usage{
check_weights(w, n, arg = as.character(substitute(w)))
}
\description{
Check weights vector (non-negative, finite, correct length)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cia.R
\name{cia}
\alias{cia}
\alias{summary.cia}
\alias{print.cia}
\alias{print.summary.cia}
\alias{plot.cia}
\alias{print.cia_ci}
\title{Coefficient of Individual Agreement}
\usage{
cia(
  data,
  response,
  subject,
  method,
  replicate,
  reference = NULL,
  scope = c("pairwise", "overall"),
  estimator = c("mom_unconstrained", "vc_constrained"),
  ci = FALSE,
  conf_level = 0.95,
  inference = c("delta", "bootstrap", "none"),
  B = 1000L,
  seed = NULL,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE
)

\method{summary}{cia}(object, digits = 4, ci_digits = 3, ...)

\method{print}{cia}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.cia}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{cia}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{print}{cia_ci}(x, ...)
}
\arguments{
\item{data}{A data frame in long format.}

\item{response}{Character scalar naming the numeric measurement column.}

\item{subject}{Character scalar naming the subject/unit identifier column.}

\item{method}{Character scalar naming the method/device/rater column.}

\item{replicate}{Character scalar naming the replicate identifier within each
subject-method cell.}

\item{reference}{Optional character scalar naming the reference method. When
\code{NULL} (default), the no-reference CIA is estimated.}

\item{scope}{One of \code{"pairwise"} or \code{"overall"}. \code{"pairwise"} returns the
two-method
CIA for each method pair, or the reference-scaled CIA for each
method-versus-reference comparison when \code{reference} is supplied.
\code{"overall"} returns one CIA coefficient across all retained methods.}

\item{estimator}{One of \code{"mom_unconstrained"} or \code{"vc_constrained"}.
\code{"mom_unconstrained"} is the literal raw method-of-moments CIA ratio and
may exceed 1. \code{"vc_constrained"} is a bounded package-level
variance-component variant that constrains the estimated inter-method
variance component to be non-negative before converting it back to CIA.}

\item{ci}{Logical; if \code{TRUE}, attach confidence intervals using the selected
\code{inference} method.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is \code{0.95}.}

\item{inference}{One of \code{"delta"}, \code{"bootstrap"}, or \code{"none"}. When
confidence intervals are requested, pairwise CIA uses a delta-method normal
interval by default and also supports subject-bootstrap percentile
intervals. For overall CIA, delta-method normal intervals are available for
the unconstrained moment estimator, and subject-bootstrap percentile
intervals are also available.}

\item{B}{Number of subject-bootstrap resamples when \code{ci = TRUE} and
\code{inference = "bootstrap"}.}

\item{seed}{Optional positive integer seed for reproducible bootstrap
resampling.}

\item{n_threads}{Integer >= 1. Number of OpenMP threads passed to the C++
backend.}

\item{verbose}{Logical; if \code{TRUE}, emit short progress messages.}

\item{object}{A \code{cia} object.}

\item{digits}{Integer; number of decimal places for estimates.}

\item{ci_digits}{Integer; number of decimal places for confidence limits.}

\item{...}{Additional arguments passed to downstream print helpers.}

\item{x}{A \code{cia} object.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Optional plot title.}

\item{low_color}{Color used for lower agreement values.}

\item{high_color}{Color used for higher agreement values.}

\item{mid_color}{Midpoint color for pairwise heatmaps.}

\item{value_text_size}{Text size for overlaid estimate labels.}

\item{ci_text_size}{Text size for CI labels.}

\item{show_value}{Logical; whether to overlay numeric values.}
}
\value{
For \code{scope = "overall"}, a one-row data frame with class
\code{c("cia_overall", "cia", "data.frame")}. For \code{scope = "pairwise"} and
\code{ci = FALSE}, a dense matrix-style object using the package's standard
correlation-result infrastructure. For \code{scope = "pairwise"} and \code{ci = TRUE},
a list with elements \code{est}, \code{lwr.ci}, and \code{upr.ci}, classed as
\code{c("cia", "cia_ci")}.
}
\description{
\code{cia()} estimates the coefficient of individual agreement. CIA assesses
individual-level interchangeability by comparing between-method disagreement
with within-method replicate disagreement. Unlike CCC, CIA is not intended
to be driven by between-subject heterogeneity.

The estimator requires replicated readings within method. A data set with
one observation per subject per method is insufficient for CIA because the
within-method disagreement term cannot be estimated.
}
\details{
Let \code{Y_ijk} denote replicate \code{k} on subject \code{i} measured by method \code{j}.

Without a reference method, CIA is defined by
\deqn{
\mathrm{CIA}^N =
\frac{\sum_j E\{(Y_{ijk} - Y_{ijk'})^2\} / 2}
     {\sum_{j < j'} E\{(Y_{ij} - Y_{ij'})^2\} / (J - 1)}.
}

With a reference method \code{J}, CIA is defined by
\deqn{
\mathrm{CIA}^R =
\frac{E\{(Y_{iJk} - Y_{iJk'})^2\}}
     {\sum_{j=1}^{J-1} E\{(Y_{ij} - Y_{iJ})^2\} / (J - 1)}.
}

This implementation uses method-of-moments estimators built from
subject-level disagreement functions.

For pairwise CIA, consider two methods \code{X} and \code{Y}. For each eligible
subject \code{i}, define
\deqn{
G_{1i} = \mathrm{mean}\{(X_{ik} - X_{ik'})^2\},
}
\deqn{
G_{2i} = \mathrm{mean}\{(Y_{il} - Y_{il'})^2\},
}
\deqn{
G_{3i} = \mathrm{mean}\{(X_{ik} - Y_{il})^2\},
}
where the first two means are over all distinct within-method replicate
pairs and the third mean is over all cross-method replicate combinations.
Let \eqn{\bar G_1}, \eqn{\bar G_2}, and \eqn{\bar G_3} denote the averages
of these subject-level quantities across eligible subjects. Then the
no-reference pairwise estimator is
\deqn{
\widehat{\mathrm{CIA}}_{XY}^{\mathrm{raw}} =
\frac{(\bar G_1 + \bar G_2)/2}{\bar G_3},
}
and the reference pairwise estimator is
\deqn{
\widehat{\mathrm{CIA}}_{XR}^{\mathrm{raw}} = \frac{\bar G_1}{\bar G_3},
}
where \code{X} is the reference method.

For overall CIA, the current implementation follows the balanced replicated
formulas in the cited papers. This requires \code{n} common subjects, \code{J}
retained methods, and the same replicate count \code{K >= 2} in every retained
subject-method cell. If the data are not balanced in this sense,
\code{scope = "overall"} returns an informative error rather than using an
approximation.

Write \eqn{\bar Y_{ij}} for the mean of the \code{K} replicates on subject \code{i}
and method \code{j}. Define the within-method mean square
\deqn{
A_{ij} = \frac{\sum_{k=1}^{K} (Y_{ijk} - \bar Y_{ij})^2}{K - 1}.
}

For the no-reference overall estimator, define the subject-level within term
\deqn{
A_{i\cdot} = \frac{1}{J} \sum_{j=1}^{J} A_{ij},
}
the subject-wide mean
\deqn{
\bar Y_{i\cdot\cdot} = \frac{1}{J} \sum_{j=1}^{J} \bar Y_{ij},
}
and the subject-level denominator
\deqn{
B_{Ni} =
\frac{\sum_{j=1}^{J} (\bar Y_{ij} - \bar Y_{i\cdot\cdot})^2}{J - 1}
+ \left(1 - \frac{1}{K}\right) A_{i\cdot}.
}
With \eqn{\bar A = n^{-1}\sum_i A_{i\cdot}} and
\eqn{\bar B = n^{-1}\sum_i B_{Ni}}, the overall no-reference estimator is
\deqn{
\widehat{\mathrm{CIA}}_{N}^{\mathrm{raw}} = \frac{\bar A}{\bar B}.
}

For the reference overall estimator, let method \code{R} be the retained
reference method. Define
\deqn{
A_{iR} = A_{i,R}
}
and
\deqn{
B_{Ri} =
\frac{\sum_{j \ne R} (\bar Y_{ij} - \bar Y_{iR})^2}{J - 1}
+ \left(1 - \frac{1}{K}\right)
  \frac{\sum_{j \ne R} A_{ij}}{J - 1}
+ \left(1 - \frac{1}{K}\right) A_{iR}.
}
With \eqn{\bar A_R = n^{-1}\sum_i A_{iR}} and
\eqn{\bar B_R = n^{-1}\sum_i B_{Ri}}, the overall reference estimator is
\deqn{
\widehat{\mathrm{CIA}}_{R}^{\mathrm{raw}} = \frac{2 \bar A_R}{\bar B_R}.
}

Two estimators are available. \code{estimator = "mom_unconstrained"} reports the
raw ratio estimator directly. \code{estimator = "vc_constrained"} applies the
non-negative variance-component boundary from the cited papers. In the
no-reference setting this uses
\eqn{\hat\tau^2_{\mathrm{raw}} = \bar B - \bar A}, and in the reference
setting it uses \eqn{\hat\tau^2_{\mathrm{raw}} = \bar B_R/2 - \bar A_R}.
The constrained estimator then sets
\eqn{\hat\tau^2 = \max(\hat\tau^2_{\mathrm{raw}}, 0)} and reports
\eqn{\widehat{\mathrm{CIA}} = \widehat{\sigma}_W^2 /
(\widehat{\sigma}_W^2 + \hat\tau^2)} on the corresponding scale. This
applies the boundary on the implied inter-method variance component rather
than clamping CIA directly.

When confidence intervals are requested, pairwise CIA uses a large-sample
delta-method normal interval by default and also supports subject-bootstrap
percentile intervals. For overall CIA, delta-method normal intervals are
available for the unconstrained moment estimator, and subject-bootstrap
percentile intervals are also available.

High CIA indicates stronger individual agreement. The FDA individual
bioequivalence boundary \code{IEC <= 2.4948} corresponds to \code{CIA >= 0.445}, and
\code{CIA >= 0.8} is sometimes used as a stronger practical rule. Such thresholds
are context-dependent and are not hard-coded by this function.

Missing rows in the required columns are removed before estimation and the
counts are recorded in \code{attr(x, "diagnostics")}.
}
\examples{
set.seed(1)
subjects <- sprintf("s\%02d", 1:30)
methods <- c("A", "B", "C")
replicates <- sprintf("r\%02d", 1:20)
dat <- expand.grid(
  subject = subjects,
  method = methods,
  replicate = replicates,
  KEEP.OUT.ATTRS = FALSE,
  stringsAsFactors = FALSE
)
subject_effect <- stats::rnorm(length(subjects), sd = 3)
method_shift <- c(A = 0, B = 0.15, C = -0.10)
method_sd <- c(A = 0.45, B = 0.45, C = 0.30)
dat$value <- subject_effect[match(dat$subject, subjects)] +
  method_shift[dat$method] +
  stats::rnorm(nrow(dat), sd = method_sd[dat$method])

fit_overall <- cia(
  dat,
  response = "value",
  subject = "subject",
  method = "method",
  replicate = "replicate",
  scope = "overall"
)
print(fit_overall)
summary(fit_overall)
estimate(fit_overall)
tidy(fit_overall)
plot(fit_overall)

fit_pairwise <- cia(
  dat,
  response = "value",
  subject = "subject",
  method = "method",
  replicate = "replicate",
  scope = "pairwise"
)
print(fit_pairwise)
summary(fit_pairwise)
tidy(fit_pairwise)
plot(fit_pairwise)

}
\references{
Barnhart HX, Kosinski AS, Haber M. (2007). Assessing individual agreement.
\emph{Journal of Biopharmaceutical Statistics}, 17(4), 697-719.
\doi{10.1080/10543400701329489}

Barnhart HX, Haber M, Lokhnygina Y, Kosinski AS. (2007). Comparison of
concordance correlation coefficient and coefficient of individual agreement
in assessing agreement. \emph{Journal of Biopharmaceutical Statistics},
17(4), 721-738.

Pan Y, Gao J, Haber M, Barnhart HX. (2010). Estimation of coefficients of
individual agreement (CIA's) for quantitative and binary data using SAS and
R. \emph{Computer Methods and Programs in Biomedicine}.
}
\seealso{
\code{\link{ccc}}, \code{\link{ba}}, and \code{\link{prob_agree}}.
\code{cia()} answers the question "Are methods interchangeable for
individual subjects once within-method replicate disagreement is taken into
account?". In contrast, \code{ccc()} answers "How well do two paired
measurements agree overall, combining correlation with location/scale
agreement?".
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_scalar_nonneg}
\alias{check_scalar_nonneg}
\title{Check a non-negative scalar (>= 0)}
\usage{
check_scalar_nonneg(x, arg = as.character(substitute(x)), strict = FALSE)
}
\description{
Check a non-negative scalar (>= 0)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spearman_rho.R
\name{spearman_rho}
\alias{spearman_rho}
\alias{print.spearman_rho}
\alias{plot.spearman_rho}
\alias{summary.spearman_rho}
\alias{print.summary.spearman_rho}
\title{Pairwise Spearman's rank correlation}
\usage{
spearman_rho(
  data,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{spearman_rho}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{plot}{spearman_rho}(
  x,
  title = "Spearman's rank correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{summary}{spearman_rho}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{print}{summary.spearman_rho}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns will be excluded. Each column must have
at least two non-missing values.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each correlation on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
jackknife Euclidean-likelihood confidence intervals for the off-diagonal
Spearman correlations.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other
\code{ggplot2} layers.}

\item{x}{An object of class \code{summary.spearman_rho}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for Spearman confidence limits in the
pairwise summary.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Spearman's rank correlation
heatmap"}.}

\item{low_color}{Color for the minimum rho value. Default is
\code{"indianred1"}.}

\item{high_color}{Color for the maximum rho value. Default is
\code{"steelblue1"}.}

\item{mid_color}{Color for zero correlation. Default is \code{"white"}.}

\item{value_text_size}{Font size for displaying correlation values. Default
is \code{4}.}

\item{ci_text_size}{Text size for confidence intervals in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{spearman_rho}.}
}
\value{
A symmetric numeric matrix where the \code{(i, j)}-th element is
the Spearman correlation between the \code{i}-th and \code{j}-th
numeric columns of the input. When \code{ci = TRUE}, the object also
carries a \code{ci} attribute with elements \code{est}, \code{lwr.ci},
\code{upr.ci}, and \code{conf.level}. When pairwise-complete evaluation is
used, pairwise sample sizes are stored in \code{attr(x, "diagnostics")$n_complete}.

Invisibly returns the \code{spearman_rho} object.

A \code{ggplot} object representing the heatmap.
}
\description{
Computes pairwise Spearman's rank correlations for the numeric columns of a
matrix or data frame using a high-performance 'C++' backend. Optional
confidence intervals are available via a jackknife Euclidean-likelihood
method.
}
\details{
For each column \eqn{j=1,\ldots,p}, let
\eqn{R_{\cdot j} \in \{1,\ldots,n\}^n} denote the (mid-)ranks of
\eqn{X_{\cdot j}}, assigning average ranks to ties. The mean rank is
\eqn{\bar R_j = (n+1)/2} regardless of ties. Define the centred rank
vectors \eqn{\tilde R_{\cdot j} = R_{\cdot j} - \bar R_j \mathbf{1}},
where \eqn{\mathbf{1}\in\mathbb{R}^n} is the all-ones vector. The
Spearman correlation between columns \eqn{i} and \eqn{j} is the Pearson
correlation of their rank vectors:
\deqn{
\rho_S(i,j) \;=\;
\frac{\sum_{k=1}^n (R_{ki}-\bar R_i)(R_{kj}-\bar R_j)}
     {\sqrt{\sum_{k=1}^n (R_{ki}-\bar R_i)^2}\;
      \sqrt{\sum_{k=1}^n (R_{kj}-\bar R_j)^2}}.
}
In matrix form, with \eqn{R=[R_{\cdot 1},\ldots,R_{\cdot p}]},
\eqn{\mu=(n+1)\mathbf{1}_p/2} for \eqn{\mathbf{1}_p\in\mathbb{R}^p}, and
\eqn{S_R=\bigl(R-\mathbf{1}\mu^\top\bigr)^\top
           \bigl(R-\mathbf{1}\mu^\top\bigr)/(n-1)},
the Spearman correlation matrix is
\deqn{
\widehat{\rho}_S \;=\; D^{-1/2} S_R D^{-1/2}, \qquad
D \;=\; \mathrm{diag}(\mathrm{diag}(S_R)).
}
When there are no ties, the familiar rank-difference formula obtains
\deqn{
\rho_S(i,j) \;=\; 1 - \frac{6}{n(n^2-1)} \sum_{k=1}^n d_k^2,
\quad d_k \;=\; R_{ki}-R_{kj},
}
but this expression does \emph{not} hold under ties; computing Pearson on
mid-ranks (as above) is the standard tie-robust approach. Without ties,
\eqn{\mathrm{Var}(R_{\cdot j})=(n^2-1)/12}; with ties, the variance is
smaller.

\eqn{\rho_S(i,j) \in [-1,1]} and \eqn{\widehat{\rho}_S} is symmetric
positive semi-definite by construction (up to floating-point error). The
implementation symmetrises the result to remove round-off asymmetry.
Spearman's correlation is invariant to strictly monotone transformations
applied separately to each variable.

\strong{Computation.} Each column is ranked (mid-ranks) to form \eqn{R}.
The product \eqn{R^\top R} is computed via a 'BLAS' symmetric rank update
('SYRK'), and centred using
\deqn{
(R-\mathbf{1}\mu^\top)^\top (R-\mathbf{1}\mu^\top)
\;=\; R^\top R \;-\; n\,\mu\mu^\top,
}
avoiding an explicit centred copy. Division by \eqn{n-1} yields the sample
covariance of ranks; standardising by \eqn{D^{-1/2}} gives \eqn{\widehat{\rho}_S}.
Columns with zero rank variance (all values equal) are returned as \code{NA}
along their row/column; the corresponding diagonal entry is also \code{NA}.

When \code{na_method = "pairwise"}, each \eqn{(i,j)} estimate is recomputed
on the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}. When
\code{ci = TRUE}, confidence intervals are computed in 'C++' using the
jackknife Euclidean-likelihood method of de Carvalho and Marques (2012).
For a pairwise estimate \eqn{U = \hat\rho_S}, delete-one jackknife
pseudo-values are formed as
\deqn{
Z_i = nU - (n-1)U_{(-i)}, \qquad i = 1,\ldots,n,
}
where \eqn{U_{(-i)}} is the Spearman correlation after removing observation
\eqn{i}. The confidence limits solve
\deqn{
\frac{n(U-\theta)^2}{n^{-1}\sum_{i=1}^n (Z_i - \theta)^2}
= \chi^2_{1,\;\texttt{conf\_level}}.
}

Ranking costs
\eqn{O\!\bigl(p\,n\log n\bigr)}; forming and normalising
\eqn{R^\top R} costs \eqn{O\!\bigl(n p^2\bigr)} with \eqn{O(p^2)} additional
memory. The optional jackknife Euclidean-likelihood confidence intervals add
per-pair delete-one recomputation work and are intended for inference rather
than raw-matrix throughput.
}
\note{
Missing values are rejected when \code{na_method = "error"}. Columns
with fewer than two usable observations are excluded.
}
\examples{
## Monotone transformation invariance (Spearman is rank-based)
set.seed(123)
n <- 400; p <- 6; rho <- 0.6
Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
L <- chol(Sigma)
X <- matrix(rnorm(n * p), n, p) \%*\% L
colnames(X) <- paste0("V", seq_len(p))

X_mono <- X
X_mono[, 1] <- exp(X_mono[, 1])
X_mono[, 2] <- log1p(exp(X_mono[, 2]))
X_mono[, 3] <- X_mono[, 3]^3

sp_X <- spearman_rho(X)
sp_m <- spearman_rho(X_mono)
summary(sp_X)
round(max(abs(sp_X - sp_m)), 3)
plot(sp_X)

## Confidence intervals
sp_ci <- spearman_rho(X[, 1:3], ci = TRUE)
print(sp_ci, show_ci = "yes")
summary(sp_ci)
estimate(sp_ci)
tidy(sp_ci)
ci(sp_ci)
confint(sp_ci)

## Ties handled via mid-ranks
tied <- cbind(
  a = rep(1:5, each = 20),
  b = rep(5:1, each = 20) + rnorm(100, sd = 0.1),
  c = as.numeric(gl(10, 10))
)
sp_tied <- spearman_rho(tied, ci = TRUE)
print(sp_tied, digits = 2, show_ci = "yes")

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(sp_X)
}
}
\references{
Spearman, C. (1904). The proof and measurement of association between
two things. International Journal of Epidemiology, 39(5), 1137-1150.

de Carvalho, M., & Marques, F. (2012). Jackknife Euclidean
likelihood-based inference for Spearman's rho. North American Actuarial
Journal, 16(4), 487-492.
}
\seealso{
\code{\link{print.spearman_rho}}, \code{\link{plot.spearman_rho}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_scalar_int_pos}
\alias{check_scalar_int_pos}
\title{Check strictly positive scalar integer}
\usage{
check_scalar_int_pos(x, arg = as.character(substitute(x)))
}
\description{
Check strictly positive scalar integer
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmcorr.R
\name{print.rmcorr_matrix}
\alias{print.rmcorr_matrix}
\alias{plot.rmcorr_matrix}
\alias{summary.rmcorr_matrix}
\alias{print.summary.rmcorr_matrix}
\title{Methods for rmcorr Matrix Objects}
\usage{
\method{print}{rmcorr_matrix}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{rmcorr_matrix}(
  x,
  title = "Repeated-measures correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{rmcorr_matrix}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.rmcorr_matrix}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"rmcorr_matrix"} or
\code{"summary.rmcorr_matrix"}.}

\item{digits}{Number of significant digits to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to downstream methods.}

\item{title}{Plot title for \code{plot.rmcorr_matrix()}.}

\item{low_color, high_color, mid_color}{Colours used for negative, positive,
and midpoint values in the heatmap.}

\item{value_text_size}{Size of the overlaid numeric value labels in the
heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on heatmap tiles when the plot type supports them.}

\item{object}{An object of class \code{"rmcorr_matrix"}.}
}
\description{
Print, summarize, and plot methods for repeated-measures
correlation matrix objects of class \code{"rmcorr_matrix"} and
\code{"summary.rmcorr_matrix"}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_required_cols}
\alias{check_required_cols}
\title{Check that a data frame contains required columns}
\usage{
check_required_cols(df, cols, df_arg = "data")
}
\description{
Check that a data frame contains required columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{print.summary.latent_corr}
\alias{print.summary.latent_corr}
\title{Summary Method for Latent Correlation Matrices}
\usage{
\method{print}{summary.latent_corr}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{summary.latent_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{...}{Unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints compact summary statistics returned by
\code{summary.tetrachoric_corr()}, \code{summary.polychoric_corr()},
\code{summary.polyserial_corr()}, and \code{summary.biserial_corr()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/krippendorff_alpha.R
\name{krippendorff_alpha}
\alias{krippendorff_alpha}
\alias{print.krippendorff_alpha}
\alias{summary.krippendorff_alpha}
\alias{print.summary.krippendorff_alpha}
\alias{plot.krippendorff_alpha}
\title{Krippendorff's Alpha Reliability Coefficient}
\usage{
krippendorff_alpha(
  data,
  levels = NULL,
  input = c("ratings", "counts"),
  level = c("nominal", "ordinal", "interval", "ratio"),
  method = c("customary", "analytical"),
  na_method = c("error", "complete", "available"),
  min_raters = 2L,
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  se_method = c("auto", "bootstrap", "jackknife", "none"),
  n_boot = 1000L,
  seed = NULL,
  n_threads = getOption("matrixCorr.threads", 1L),
  return_matrices = FALSE,
  verbose = FALSE,
  ...
)

\method{print}{krippendorff_alpha}(x, digits = 4, ...)

\method{summary}{krippendorff_alpha}(object, digits = 4, ...)

\method{print}{summary.krippendorff_alpha}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{krippendorff_alpha}(
  x,
  type = c("estimate", "item_disagreement", "category_proportion", "coincidence"),
  ...
)
}
\arguments{
\item{data}{Input data.
For \code{input = "ratings"}, rows are units/items and columns are
coders/raters. For \code{input = "counts"}, rows are units/items and
columns are categories containing per-item category counts.}

\item{levels}{Optional category labels in analysis order. For ordinal
character data, explicit \code{levels} are required unless the data are
ordered factors. For interval and ratio alpha, \code{levels} must be
coercible to finite numeric values.}

\item{input}{One of \code{"ratings"} or \code{"counts"}.}

\item{level}{Measurement level: \code{"nominal"}, \code{"ordinal"},
\code{"interval"}, or \code{"ratio"}.}

\item{method}{Estimator. \code{"customary"} uses the coincidence-matrix
estimator of Krippendorff/Hayes. \code{"analytical"} uses the improved
estimator and jackknife inference direction described by Hughes.}

\item{na_method}{Missing-data rule. \code{"error"} rejects missing ratings,
\code{"complete"} removes any item with one or more missing ratings, and
\code{"available"} retains items with at least \code{min_raters} observed
ratings. For \code{input = "counts"}, missing counts are not allowed and
rows with total counts below \code{min_raters} are dropped.}

\item{min_raters}{Minimum number of observed ratings required for a retained
item. Must be at least \code{2}.}

\item{ci}{Logical; if \code{TRUE}, attach confidence intervals.}

\item{p_value}{Logical; if \code{TRUE}, attach test statistics and p-values.
This is available only for \code{method = "analytical"} with jackknife
inference.}

\item{conf_level}{Confidence level for intervals. Default is \code{0.95}.}

\item{se_method}{Standard-error method. \code{"auto"} resolves to
\code{"bootstrap"} for customary alpha when \code{ci = TRUE}, and to
\code{"jackknife"} for analytical alpha when \code{ci = TRUE} or
\code{p_value = TRUE}. \code{"bootstrap"} is allowed only for customary
alpha. \code{"jackknife"} is the primary inference path for analytical
alpha. \code{"none"} disables uncertainty output.}

\item{n_boot}{Number of bootstrap resamples used for customary alpha when
\code{ci = TRUE}.}

\item{seed}{Optional positive integer seed used for reproducible bootstrap
resampling.}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
C++ backend.}

\item{return_matrices}{Logical; if \code{TRUE}, attach the retained count
matrix, observed coincidence matrix, expected coincidence matrix, and
disagreement matrix in \code{attr(x, "matrices")}.}

\item{verbose}{Logical; if \code{TRUE}, emit short progress messages.}

\item{...}{Unused.}

\item{x}{A \code{krippendorff_alpha} object.}

\item{digits}{Integer; number of decimal places for rounded numeric columns.}

\item{object}{A \code{krippendorff_alpha} object.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{type}{Plot type: \code{"estimate"}, \code{"item_disagreement"},
\code{"category_proportion"}, or \code{"coincidence"}.}
}
\value{
A one-row data frame with class
\code{c("krippendorff_alpha", "agreement_result", "data.frame")}. The
object stores method metadata, diagnostics, and optional matrices in
attributes.
}
\description{
Estimates Krippendorff's alpha as a panel-level reliability/agreement
coefficient among two or more coders, raters, observers, judges, or
instruments. Missing ratings are supported through \code{na_method}, and
nominal, ordinal, interval, and ratio disagreement functions are available.
}
\details{
Krippendorff's alpha is a panel-level reliability coefficient, not a
pairwise correlation matrix. The customary coincidence-matrix estimator is
\deqn{ \alpha = 1 - \frac{D_o}{D_e}, }
where \eqn{D_o} is the observed disagreement and \eqn{D_e} is the expected
disagreement under chance assignment.

For retained item \eqn{u} with \eqn{m_u} observed ratings and category
counts \eqn{n_{uc}}, the observed coincidence matrix updates by
\deqn{ O_{ck} \leftarrow O_{ck} +
       \frac{n_{uc}(n_{uk} - 1(c=k))}{m_u - 1}. }
Let \eqn{n_c = \sum_k O_{ck}} and \eqn{n = \sum_c n_c}. Then
\deqn{ D_o = \frac{1}{n}\sum_{c,k} O_{ck}\,\delta^2_{ck}, }
\deqn{ D_e = \frac{1}{n(n-1)}\sum_{c,k} n_c n_k\,\delta^2_{ck}, }
where \eqn{\delta^2_{ck}} is the level-specific disagreement matrix.

The disagreement functions implemented here are:
\itemize{
\item nominal: \eqn{\delta^2_{ck} = 1(c \ne k)}
\item ordinal: Krippendorff's cumulative-margin disagreement based on the
pooled category margins
\item interval: \eqn{\delta^2_{ck} = (v_c - v_k)^2}
\item ratio:
\eqn{\delta^2_{ck} = (v_c - v_k)^2 / (v_c + v_k)^2}, with
\eqn{\delta^2_{ck}=0} when both values are zero
}

\strong{Analytical estimator and inference.}
Hughes (2024) recommends a different point estimator based on within-item
and pooled pairwise disagreement. Let \eqn{a} be the number of retained
items, \eqn{m_i} the number of ratings in item \eqn{i}, and
\eqn{N = \sum_i m_i}. Define
\deqn{ \bar n = \frac{N - \sum_i m_i^2 / N}{a - 1}. }
The within-item mean square is
\deqn{ MSE =
       \frac{1}{N}\sum_{i=1}^a
       \frac{\sum_{j<k}\delta^2(x_{ij}, x_{ik})}{m_i - 1}, }
the pooled total disagreement is
\deqn{ SST = \frac{1}{N}\sum_{r<s}\delta^2(z_r, z_s), }
where \eqn{z_1,\ldots,z_N} are the pooled observed ratings, and
\deqn{ SSA = SST - (N-a)\,MSE, \qquad
       MSA = \frac{SSA}{a-1}. }
With \eqn{\theta = MSA/MSE} and \eqn{\eta = \log(\theta)}, the analytical
alpha estimate is
\deqn{ \alpha =
       \frac{\theta - 1}{\theta + \bar n - 1}. }

When analytical inference is requested, the implementation uses the
delete-one-item jackknife on \eqn{\eta}. If \eqn{\eta_{(-i)}} is the
leave-one-item-out transform and \eqn{a} is the number of retained items,
the jackknife pseudo-values are
\deqn{ p_i = a\,\hat\eta - (a-1)\,\hat\eta_{(-i)}, }
with standard error
\deqn{ \widehat{\mathrm{se}}(\hat\eta)
       = \sqrt{\mathrm{Var}(p_1,\ldots,p_a) / a}. }
The confidence interval is formed on the \eqn{\eta}-scale with a
\eqn{t_{a-1}} critical value and back-transformed to alpha using the full
data \eqn{\bar n}. The analytical p-value tests \eqn{H_0: \alpha = 0},
equivalently \eqn{H_0: \eta = 0}, via the t statistic
\deqn{ t = \hat\eta / \widehat{\mathrm{se}}(\hat\eta). }

For customary alpha, percentile bootstrap intervals are available by
resampling retained items with replacement and recomputing the customary
estimate.
}
\examples{
raters <- data.frame(
  r1 = c("A", "A", "B", "B", "C", "A"),
  r2 = c("A", "B", "B", "B", "C", "A"),
  r3 = c("A", "A", "B", "C", "C", "A"),
  stringsAsFactors = FALSE
)

fit <- krippendorff_alpha(raters, level = "nominal", na_method = "available")
print(fit)
summary(fit)
estimate(fit)
tidy(fit)
plot(fit)

}
\references{
Krippendorff, K. (2011/2013). Computing Krippendorff's alpha-reliability.

Hayes, A. F. and Krippendorff, K. (2007). Answering the call for a standard
reliability measure for coding data. \emph{Communication Methods and
Measures}, 1(1), 77-89.

Hughes, J. (2024). Toward improved inference for Krippendorff's Alpha
agreement coefficient. \emph{Journal of Statistical Planning and
Inference}, 233, 106170.
}
\seealso{
\code{\link[=multirater_kappa]{multirater_kappa()}} for nominal multi-rater kappa;
\code{\link[=gwet_ac]{gwet_ac()}} for AC1/AC2 agreement coefficients.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{run_cpp}
\alias{run_cpp}
\title{run_cpp}
\usage{
run_cpp(
  Xr,
  yr,
  subject,
  method_int,
  time_int,
  Laux,
  Z,
  use_ar1,
  ar1_rho,
  max_iter,
  tol,
  conf_level,
  ci_mode_int,
  include_subj_method = TRUE,
  include_subj_time = TRUE,
  sb_zero_tol = 1e-10,
  eval_single_visit = FALSE,
  time_weights = NULL,
  metric_mode = 0L,
  loglik_only = FALSE,
  need_reml_loglik = TRUE
)
}
\description{
Wrapper for calling 'C++' backend for CCC estimation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{match_arg}
\alias{match_arg}
\title{Match argument to allowed values}
\usage{
match_arg(arg, values, arg_name = as.character(substitute(arg)))
}
\description{
Match argument to allowed values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kendall_corr.R
\name{kendall_tau}
\alias{kendall_tau}
\alias{print.kendall_matrix}
\alias{plot.kendall_matrix}
\alias{summary.kendall_matrix}
\alias{print.summary.kendall_matrix}
\title{Pairwise (or Two-Vector) Kendall's Tau Rank Correlation}
\usage{
kendall_tau(
  data,
  y = NULL,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  ci_method = c("fieller", "if_el", "brown_benedetti"),
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{kendall_matrix}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{plot}{kendall_matrix}(
  x,
  title = "Kendall's Tau correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{summary}{kendall_matrix}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{print}{summary.kendall_matrix}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{For matrix/data frame mode, a numeric matrix or a data frame with at least
two numeric columns. All non-numeric columns are excluded. For two-vector
mode, a numeric vector \code{x}.}

\item{y}{Optional numeric vector \code{y} of the same length as \code{data}
when \code{data} is a vector. If supplied, the function computes the
Kendall correlation \emph{between \code{data} and \code{y}} using a
low-overhead scalar path and returns a single number.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each correlation on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach pairwise
confidence intervals for the off-diagonal Kendall correlations in
matrix/data-frame mode.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{ci_method}{Confidence-interval engine used when \code{ci = TRUE}.
Supported Kendall methods are \code{"fieller"} (default),
\code{"brown_benedetti"}, and \code{"if_el"}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other
\code{ggplot2} layers.}

\item{x}{An object of class \code{summary.kendall_matrix}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for Kendall confidence limits in the
pairwise summary.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Kendall's Tau correlation
heatmap"}.}

\item{low_color}{Color for the minimum tau value. Default is
\code{"indianred1"}.}

\item{high_color}{Color for the maximum tau value. Default is
\code{"steelblue1"}.}

\item{mid_color}{Color for zero correlation. Default is \code{"white"}.}

\item{value_text_size}{Font size for displaying correlation values. Default
is \code{4}.}

\item{ci_text_size}{Text size for confidence intervals in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{kendall_matrix}.}
}
\value{
\itemize{
\item If \code{y} is \code{NULL} and \code{data} is a matrix/data frame: a
symmetric numeric matrix where entry \code{(i, j)} is the Kendall's tau
correlation between the \code{i}-th and \code{j}-th numeric columns. When
\code{ci = TRUE}, the object also carries a \code{ci} attribute with
elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and
\code{ci.method}. Pairwise complete-case sample sizes are stored in
\code{attr(x, "diagnostics")$n_complete}.
\item If \code{y} is provided (two-vector mode): a single numeric scalar,
the Kendall's tau correlation between \code{data} and \code{y}.
}

Invisibly returns the \code{kendall_matrix} object.

A \code{ggplot} object representing the heatmap.
}
\description{
Computes pairwise Kendall's tau correlations for numeric data using a
high-performance 'C++' backend. Optional confidence intervals are available
for matrix and data-frame input.
}
\details{
Kendall's tau is a rank-based measure of association between two variables.
For a dataset with \eqn{n} observations on variables \eqn{X} and \eqn{Y},
let \eqn{n_0 = n(n - 1)/2} be the number of unordered pairs, \eqn{C} the
number of concordant pairs, and \eqn{D} the number of discordant pairs.
Let \eqn{T_x = \sum_g t_g (t_g - 1)/2} and \eqn{T_y = \sum_h u_h (u_h - 1)/2}
be the numbers of tied pairs within \eqn{X} and within \eqn{Y}, respectively,
where \eqn{t_g} and \eqn{u_h} are tie-group sizes in \eqn{X} and \eqn{Y}.

The tie-robust Kendall's tau-b is:
\deqn{ \tau_b = \frac{C - D}{\sqrt{(n_0 - T_x)\,(n_0 - T_y)}}. }
When there are no ties (\eqn{T_x = T_y = 0}), this reduces to tau-a:
\deqn{ \tau_a = \frac{C - D}{n(n-1)/2}. }

The function automatically handles ties. In degenerate cases where a
variable is constant (\eqn{n_0 = T_x} or \eqn{n_0 = T_y}), the tau-b
denominator is zero and the correlation is undefined (returned as \code{NA}
off the diagonal).

When \code{na_method = "pairwise"}, each \eqn{(i,j)} estimate is recomputed
on the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}.
Confidence intervals use the observed pairwise-complete Kendall estimate and
the same pairwise complete-case overlap.

With \code{ci_method = "fieller"}, the interval is built on the Fisher-style
transformed scale \eqn{z = \operatorname{atanh}(\hat\tau)} using Fieller's
asymptotic standard error
\deqn{ \operatorname{SE}(z) = \sqrt{\frac{0.437}{n - 4}}, }
where \eqn{n} is the pairwise complete-case sample size. The interval is then
mapped back with \code{tanh()} and clipped to \eqn{[-1, 1]} for numerical
safety. This is the default Kendall CI and is intended to be the fast,
production-oriented choice.

With \code{ci_method = "brown_benedetti"}, the interval is computed on the
Kendall tau scale using the Brown-Benedetti large-sample variance for
Kendall's tau-b. This path is tie-aware, remains on the original Kendall
scale, and is intended as a conventional asymptotic alternative when a
direct tau-scale interval is preferred.

With \code{ci_method = "if_el"}, the interval is computed in 'C++' using an
influence-function empirical-likelihood construction built from the
linearised Kendall estimating equation. The lower and upper limits are found
by solving the empirical-likelihood ratio equation against the
\eqn{\chi^2_1}-cutoff implied by \code{conf_level}. This method is slower
than \code{"fieller"} and is intended for specialised inference.

\strong{Performance:}
\itemize{
\item In the \strong{two-vector mode} (\code{y} supplied), the C++ backend uses a
raw-double path with minimal overhead.
\item In the \strong{matrix/data-frame mode}, the no-missing estimate-only path
uses the Knight (1966) \eqn{O(n \log n)} algorithm. Pairwise-complete
inference paths recompute each pair on its complete-case overlap; the
\code{"brown_benedetti"} interval adds tie-aware large-sample variance
calculations and \code{"if_el"} adds extra per-pair likelihood solving.
}
}
\note{
Missing values are rejected when \code{na_method = "error"}. Columns
with fewer than two usable observations are excluded. Confidence intervals
are not available in the two-vector interface.
}
\examples{
# Basic usage with a matrix
mat <- cbind(a = rnorm(100), b = rnorm(100), c = rnorm(100))
kt <- kendall_tau(mat)
print(kt)
summary(kt)
plot(kt)

# Confidence intervals
kt_ci <- kendall_tau(mat[, 1:3], ci = TRUE)
print(kt_ci, show_ci = "yes")
summary(kt_ci)
estimate(kt_ci)
tidy(kt_ci)
ci(kt_ci)
confint(kt_ci)

# Two-vector mode (scalar path)
x <- rnorm(1000); y <- 0.5 * x + rnorm(1000)
kendall_tau(x, y)

# Including ties
tied_df <- data.frame(
  v1 = rep(1:5, each = 20),
  v2 = rep(5:1, each = 20),
  v3 = rnorm(100)
)
kt_tied <- kendall_tau(tied_df, ci = TRUE, ci_method = "fieller")
print(kt_tied, show_ci = "yes")

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(kt)
}

}
\references{
Kendall, M. G. (1938). A New Measure of Rank Correlation. \emph{Biometrika},
30(1/2), 81-93.

Knight, W. R. (1966). A Computer Method for Calculating Kendall's Tau with
Ungrouped Data. \emph{Journal of the American Statistical Association},
61(314), 436-439.

Fieller, E. C., Hartley, H. O., & Pearson, E. S. (1957). Tests for rank
correlation coefficients. I. \emph{Biometrika}, 44(3/4), 470-481.

Brown, M. B., & Benedetti, J. K. (1977). Sampling behavior of tests for
correlation in two-way contingency tables. \emph{Journal of the American
Statistical Association}, 72(358), 309-315.

Huang, Z., & Qin, G. (2023). Influence function-based confidence intervals
for the Kendall rank correlation coefficient. \emph{Computational
Statistics}, 38(2), 1041-1055.

Croux, C., & Dehon, C. (2010). Influence functions of the Spearman and
Kendall correlation measures. \emph{Statistical Methods & Applications},
19, 497-515.
}
\seealso{
\code{\link{print.kendall_matrix}}, \code{\link{plot.kendall_matrix}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{ccc_rm_reml}
\alias{ccc_rm_reml}
\title{Concordance Correlation via REML (Linear Mixed-Effects Model)}
\usage{
ccc_rm_reml(
  data,
  response,
  subject,
  method = NULL,
  time = NULL,
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  ci_mode = c("auto", "raw", "logit"),
  verbose = FALSE,
  digits = 4,
  use_message = TRUE,
  interaction = FALSE,
  max_iter = 100,
  tol = 1e-06,
  Dmat = NULL,
  Dmat_type = c("time-avg", "typical-visit", "weighted-avg", "weighted-sq"),
  Dmat_weights = NULL,
  Dmat_rescale = TRUE,
  ar = c("none", "ar1"),
  ar_rho = NA_real_,
  slope = c("none", "subject", "method", "custom"),
  slope_var = NULL,
  slope_Z = NULL,
  drop_zero_cols = TRUE,
  vc_select = c("auto", "none"),
  vc_alpha = 0.05,
  vc_test_order = c("subj_time", "subj_method"),
  include_subj_method = NULL,
  include_subj_time = NULL,
  sb_zero_tol = 1e-10
)
}
\arguments{
\item{data}{A data frame.}

\item{response}{Character. Response variable name.}

\item{subject}{Character. Subject ID variable name.}

\item{method}{Character or \code{NULL}. Optional column name of method factor
(added to fixed effects).}

\item{time}{Character or \code{NULL}. Optional column name of time factor
(added to fixed effects).}

\item{ci}{Logical. If \code{TRUE}, return a CI container; limits are computed
by a large-sample delta method for CCC (see \strong{CIs} note below).}

\item{conf_level}{Numeric in \eqn{(0,1)}. Confidence level when
\code{ci = TRUE} (default \code{0.95}).}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads to use for
computation. Defaults to \code{getOption("matrixCorr.threads", 1L)}.}

\item{ci_mode}{Character scalar; one of \code{c("auto","raw","logit")}.
Controls how confidence intervals are computed when \code{ci = TRUE}.
If \code{"raw"}, a Wald CI is formed on the CCC scale and truncated to
\code{[0,1]}. If \code{"logit"}, a Wald CI is computed on the
\eqn{\mathrm{logit}(\mathrm{CCC})} scale and back-transformed to the
original scale (often more stable near 0 or 1). If \code{"auto"}
(default), the method is chosen per estimate based on simple diagnostics
(e.g., proximity to the \code{[0,1]} boundary / numerical stability),
typically preferring \code{"logit"} near the boundaries and \code{"raw"}
otherwise.}

\item{verbose}{Logical. If \code{TRUE}, prints a structured summary of the
fitted variance components and \eqn{S_B} for each fit. Default \code{FALSE}.}

\item{digits}{Integer \eqn{(\ge 0)}. Number of decimal places to use in the
printed summary when \code{verbose = TRUE}. Default \code{4}.}

\item{use_message}{Logical. When \code{verbose = TRUE}, verbose summaries
are emitted with \pkg{cli} messages.}

\item{interaction}{Logical. Include \code{method:time} interaction?
(default \code{FALSE}).}

\item{max_iter}{Integer. Maximum iterations for variance-component updates
(default \code{100}).}

\item{tol}{Numeric. Convergence tolerance on parameter change
(default \code{1e-6}).}

\item{Dmat}{Optional \eqn{n_t \times n_t} numeric matrix to weight/aggregate
time-specific fixed biases in the \eqn{S_B} quadratic form. If supplied, it
is used (after optional mass rescaling; see \code{Dmat_rescale}) whenever at
least two \emph{present} time levels exist; otherwise it is ignored. \strong{If
\code{Dmat} is \code{NULL}}, a canonical kernel \eqn{D_m} is \emph{constructed}
from \code{Dmat_type} and \code{Dmat_weights} (see below). \code{Dmat} should be
symmetric positive semidefinite; small asymmetries are symmetrized internally.}

\item{Dmat_type}{Character, one of \code{c("time-avg","typical-visit",
  "weighted-avg","weighted-sq")}. Only used when \code{Dmat = NULL}.
It selects the aggregation target for time-specific fixed biases in
\eqn{S_B}. Options are:

\itemize{
\item \code{"time-avg"}: square of the time-averaged bias, \eqn{D_m=(1/n_t)\,11^\top}.
\item \code{"typical-visit"}: average of squared per-time biases, \eqn{D_m=I_{n_t}}.
\item \code{"weighted-avg"}: square of a weighted average, \eqn{D_m=n_t\,w\,w^\top} with \eqn{\sum w=1}.
\item \code{"weighted-sq"}: weighted average of squared biases, \eqn{D_m=n_t\,\mathrm{diag}(w)} with \eqn{\sum w=1}.
}
Pick \code{"time-avg"} for CCC targeting the time-averaged measurement; pick
\code{"typical-visit"} for CCC targeting a randomly sampled visit (typical occasion).
Default \code{"time-avg"}.}

\item{Dmat_weights}{Optional numeric weights \eqn{w} used when
\code{Dmat_type \%in\% c("weighted-avg","weighted-sq")}. Must be nonnegative and
finite. If \code{names(w)} are provided, they should match the \emph{full} time
levels in \code{data}; they are aligned to the \emph{present} time subset per fit.
If unnamed, the length must equal the number of present time levels. In all cases
\eqn{w} is internally normalized to sum to 1.}

\item{Dmat_rescale}{Logical. When \code{TRUE} (default), the supplied/built
\eqn{D_m} is rescaled to satisfy the simple mass rule
\eqn{1^\top D_m 1 = n_t}. This keeps the \eqn{S_B} denominator invariant and
harmonizes with the \eqn{\kappa}-shrinkage used for variance terms.}

\item{ar}{Character. Residual correlation structure: \code{"none"} (iid) or
\code{"ar1"} for subject-level AR(1) correlation within contiguous time
runs. Default \code{c("none")}.}

\item{ar_rho}{Numeric in \eqn{(-0.999,\,0.999)} or \code{NA}.
If \code{ar = "ar1"} and \code{ar_rho} is finite, it is treated as fixed.
If \code{ar = "ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
profiling a 1-D objective (REML when available; an approximation otherwise).
Default \code{NA_real_}.}

\item{slope}{Character. Optional extra random-effect design \eqn{Z}.
With \code{"subject"} a single random slope is added (one column in \eqn{Z});
with \code{"method"} one column per method level is added; with
\code{"custom"} you provide \code{slope_Z} directly. Default
\code{c("none","subject","method","custom")}.}

\item{slope_var}{For \code{slope \%in\% c("subject","method")}, a character
string giving the name of a column in \code{data} used as the slope regressor
(e.g., centered time). It is looked up inside \code{data}; do not
pass the vector itself. NAs are treated as zeros in \eqn{Z}.}

\item{slope_Z}{For \code{slope = "custom"}, a numeric matrix with \eqn{n}
rows (same order as \code{data}) providing the full extra random-effect
design \eqn{Z}. \strong{Each column of \code{slope_Z} has its own variance
component} \eqn{\sigma_{Z,j}^2}; columns are treated as \emph{uncorrelated}
(diagonal block in \eqn{G}). Ignored otherwise.}

\item{drop_zero_cols}{Logical. When \code{slope = "method"}, drop all-zero
columns of \eqn{Z} after subsetting (useful in pairwise fits). Default
\code{TRUE}.}

\item{vc_select}{Character scalar; one of \code{c("auto","none")}.
Controls how the subject by method \eqn{\sigma^2_{A\times M}} ("subj_method") and
subject by time \eqn{\sigma^2_{A\times T}} ("subj_time") variance components are
included. If \code{"auto"} (default), the function performs boundary-aware
REML likelihood-ratio tests (LRTs; null on the boundary at zero with a
half-\eqn{\chi^2_1} reference) to decide whether to retain each component,
in the order given by \code{vc_test_order}. If \code{"none"}, no testing
is done and inclusion is taken from \code{include_subj_method}/\code{include_subj_time}
(or, if \code{NULL}, from the mere presence of the corresponding factor in
the design).  In pairwise fits, the decision is made independently for each
method pair.}

\item{vc_alpha}{Numeric scalar in \eqn{(0,1)}; default \code{0.05}.
Per-component significance level for the boundary-aware REML LRTs used when
\code{vc_select = "auto"}. The tests are one-sided for variance components
on the boundary and are \emph{not} multiplicity-adjusted.}

\item{vc_test_order}{Character vector (length 2) with a permutation of
\code{c("subj_time","subj_method")}; default \code{c("subj_time","subj_method")}. Specifies the order
in which the two variance components are tested when \code{vc_select = "auto"}.
The component tested first may be dropped before testing the second. If a
factor is absent in the design (e.g., no time factor so "subj_time" is undefined),
the corresponding test is skipped.}

\item{include_subj_method, include_subj_time}{Logical scalars or \code{NULL}.
When \code{vc_select = "none"}, these control whether the
\eqn{\sigma^2_{A\times M}} ("subj_method") and \eqn{\sigma^2_{A\times T}} ("subj_time")
random effects are included (\code{TRUE}) or excluded (\code{FALSE}) in the model.
If \code{NULL} (default), inclusion defaults to the presence of the
corresponding factor in the data (i.e., at least two method/time levels).
When \code{vc_select = "auto"}, these arguments are ignored (automatic
selection is used instead).}

\item{sb_zero_tol}{Non-negative numeric scalar; default \code{1e-10}.
Numerical threshold for the fixed-effect dispersion term \eqn{S_B}.
After computing \eqn{\widehat{S_B}} and its delta-method variance, if
\eqn{\widehat{S_B} \le} \code{sb_zero_tol} or non-finite, the procedure
treats \eqn{S_B} as fixed at zero in the delta step. It sets
\eqn{d_{S_B}=0} and \eqn{\mathrm{Var}(\widehat{S_B})=0}, preventing
numerical blow-ups of \code{SE(CCC)} when \eqn{\widehat{S_B}\to 0} and the
fixed-effects variance is ill-conditioned for the contrast. This stabilises
inference in rare boundary cases; it has no effect when
\eqn{\widehat{S_B}} is comfortably above the threshold.}
}
\description{
Compute Lin's Concordance Correlation Coefficient (CCC) from a linear
mixed-effects model fitted by REML. The fixed-effects part can include
\code{method} and/or \code{time} (optionally their interaction), with a
subject-specific random intercept to capture between-subject variation.
Large \eqn{n \times n} inversions are avoided by solving small per-subject
systems.

\strong{Assumption:} time levels are treated as \emph{regular, equally spaced}
visits indexed by their order within subject. The AR(1) residual model is
in discrete time on the visit index (not calendar time). \code{NA} time codes
break the serial run. Gaps in the factor levels are \emph{ignored} (adjacent
observed visits are treated as lag-1).
}
\details{
For measurement \eqn{y_{ij}} on subject \eqn{i} under fixed
levels (method, time), we fit
\deqn{ y = X\beta + Zu + \varepsilon,\qquad
       u \sim N(0,\,G),\ \varepsilon \sim N(0,\,R). }
Notation: \eqn{m} subjects, \eqn{n=\sum_i n_i} total rows;
\eqn{nm} method levels; \eqn{nt} time levels; \eqn{q_Z} extra
random-slope columns (if any); \eqn{r=1+nm+nt} (or \eqn{1+nm+nt+q_Z} with slopes).
Here \eqn{Z} is the subject-structured random-effects design and \eqn{G} is
block-diagonal at the subject level with the following \emph{per-subject}
parameterisation. Specifically,
\itemize{
\item one random intercept with variance \eqn{\sigma_A^2};
\item optionally, \emph{method} deviations (one column per method level)
with a common variance \eqn{\sigma_{A\times M}^2} and zero
covariances across levels (i.e., multiple of an identity);
\item optionally, \emph{time} deviations (one column per time level)
with a common variance \eqn{\sigma_{A\times T}^2} and zero
covariances across levels;
\item optionally, an \emph{extra} random effect aligned with \eqn{Z}
(random slope), where each \emph{column} has its own variance
\eqn{\sigma_{Z,j}^2} and columns are uncorrelated.
}
The fixed-effects design is \code{~ 1 + method + time} and, if
\code{interaction=TRUE}, \code{+ method:time}.

\strong{Residual correlation \eqn{R} (regular, equally spaced time).}
Write \eqn{R_i=\sigma_E^2\,C_i(\rho)}. With \code{ar="none"}, \eqn{C_i=I}.
With \code{ar="ar1"}, within-subject residuals follow a \emph{discrete} AR(1)
process along the visit index after sorting by increasing time level. Ties
retain input order, and any \code{NA} time code breaks the series so each
contiguous block of non-\code{NA} times forms a run. The correlation
between \emph{adjacent observed visits} in a run is \eqn{\rho}; we do not use
calendar-time gaps. Internally we work with the \emph{precision} of the AR(1)
correlation: for a run of length \eqn{L\ge 2}, the tridiagonal inverse has
\deqn{ (C^{-1})_{11}=(C^{-1})_{LL}=\frac{1}{1-\rho^2},\quad
       (C^{-1})_{tt}=\frac{1+\rho^2}{1-\rho^2}\ (2\le t\le L-1),\quad
       (C^{-1})_{t,t+1}=(C^{-1})_{t+1,t}=\frac{-\rho}{1-\rho^2}. }
The working inverse is \eqn{\,R_i^{-1}=\sigma_E^{-2}\,C_i(\rho)^{-1}}.

\strong{Per-subject Woodbury system.} For subject \eqn{i} with \eqn{n_i}
rows, define the per-subject random-effects design \eqn{U_i} (columns:
intercept, method indicators, time indicators; dimension
\eqn{\,r=1+nm+nt\,}). The core never forms
\eqn{V_i = R_i + U_i G U_i^\top} explicitly. Instead,
\deqn{ M_i \;=\; G^{-1} \;+\; U_i^\top R_i^{-1} U_i, }
and accumulates GLS blocks via rank-\eqn{r} corrections using
\eqn{\,V_i^{-1} = R_i^{-1}-R_i^{-1}U_i M_i^{-1}U_i^\top R_i^{-1}\,}:
\deqn{ X^\top V^{-1} X \;=\; \sum_i \Big[
       X_i^\top R_i^{-1}X_i \;-\; (X_i^\top R_i^{-1}U_i)\,
       M_i^{-1}\,(U_i^\top R_i^{-1}X_i) \Big], }
\deqn{ X^\top V^{-1} y \;=\; \sum_i \Big[
       X_i^\top R_i^{-1}y_i \;-\; (X_i^\top R_i^{-1}U_i)\,M_i^{-1}\,
       (U_i^\top R_i^{-1}y_i) \Big]. }
Because \eqn{G^{-1}} is diagonal with positive entries, each \eqn{M_i} is
symmetric positive definite; solves/inversions use symmetric-PD routines with
a small diagonal ridge and a pseudo-inverse if needed.

\strong{Random-slope \eqn{Z}.}
Besides \eqn{U_i}, the function can include an extra design \eqn{Z_i}.
\itemize{
\item \code{slope="subject"}: \eqn{Z} has one column (the regressor in
\code{slope_var}); \eqn{Z_{i}} is the subject-\eqn{i} block, with its own
variance \eqn{\sigma_{Z,1}^2}.
\item \code{slope="method"}: \eqn{Z} has one column per method level;
row \eqn{t} uses the slope regressor if its method equals level \eqn{\ell},
otherwise 0; all-zero columns can be dropped via
\code{drop_zero_cols=TRUE} after subsetting. Each column has its own
variance \eqn{\sigma_{Z,\ell}^2}.
\item \code{slope="custom"}: \eqn{Z} is provided fully via \code{slope_Z}.
Each column is an independent random effect with its own variance
\eqn{\sigma_{Z,j}^2}; cross-covariances among columns are set to 0.
}
Computations simply augment \eqn{\tilde U_i=[U_i\ Z_i]} and the corresponding
inverse-variance block. The EM updates then include, for each column \eqn{j=1,\ldots,q_Z},
\deqn{ \sigma_{Z,j}^{2\,(new)} \;=\; \frac{1}{m}
      \sum_{i=1}^m \Big( b_{i,\text{extra},j}^2 +
      (M_i^{-1})_{\text{extra},jj} \Big)
      \quad (\text{if } q_Z>0). }
\emph{Interpretation:} the \eqn{\sigma_{Z,j}^2} represent additional within-subject
variability explained by the slope regressor(s) in column \eqn{j} and are \emph{not} part of the CCC
denominator (agreement across methods/time).

\strong{EM-style variance-component updates.} With current \eqn{\hat\beta},
form residuals \eqn{r_i = y_i - X_i\hat\beta}. The BLUPs and conditional
covariances are
\deqn{ b_i \;=\; M_i^{-1}\,(U_i^\top R_i^{-1} r_i), \qquad
      \mathrm{Var}(b_i\mid y) \;=\; M_i^{-1}. }
Let \eqn{e_i=r_i-U_i b_i}. Expected squares then yield closed-form updates:
\deqn{ \sigma_A^{2\,(new)} \;=\; \frac{1}{m}\sum_i \Big( b_{i,0}^2 +
(M_i^{-1})_{00} \Big), }
\deqn{ \sigma_{A\times M}^{2\,(new)} \;=\; \frac{1}{m\,nm}
      \sum_i \sum_{\ell=1}^{nm}\!\Big( b_{i,\ell}^2 +
      (M_i^{-1})_{\ell\ell} \Big)
      \quad (\text{if } nm>0), }
\deqn{ \sigma_{A\times T}^{2\,(new)} \;=\; \frac{1}{m\,nt}
      \sum_i \sum_{t=1}^{nt}
      \Big( b_{i,t}^2 + (M_i^{-1})_{tt} \Big)
      \quad (\text{if } nt>0), }
\deqn{ \sigma_E^{2\,(new)} \;=\; \frac{1}{n} \sum_i
      \Big( e_i^\top C_i(\rho)^{-1} e_i \;+\;
      \mathrm{tr}\!\big(M_i^{-1}U_i^\top C_i(\rho)^{-1} U_i\big) \Big), }
together with the per-column update for \eqn{\sigma_{Z,j}^2} given above.
Iterate until the \eqn{\ell_1} change across components is \eqn{<}\code{tol}
or \code{max_iter} is reached.

\strong{Fixed-effect dispersion \eqn{S_B}: choosing the time-kernel \eqn{D_m}.}

Let \eqn{d = L^\top \hat\beta} stack the within-time, pairwise method differences,
grouped by time as \eqn{d=(d_1^\top,\ldots,d_{n_t}^\top)^\top} with
\eqn{d_t \in \mathbb{R}^{P}} and \eqn{P = n_m(n_m-1)}. The symmetric
positive semidefinite kernel \eqn{D_m \succeq 0} selects which functional of the
bias profile \eqn{t \mapsto d_t} is targeted by \eqn{S_B}. Internally, the code
rescales any supplied/built \eqn{D_m} to satisfy \eqn{1^\top D_m 1 = n_t} for
stability and comparability.

\itemize{
\item \code{Dmat_type = "time-avg"} (square of the time-averaged bias).
Let \deqn{ w \;=\; \frac{1}{n_t}\,\mathbf{1}_{n_t}, \qquad
               D_m \;\propto\; I_P \otimes (w\,w^\top), }
so that
\deqn{ d^\top D_m d \;\propto\; \sum_{p=1}^{P}
           \left( \frac{1}{n_t}\sum_{t=1}^{n_t} d_{t,p} \right)^{\!2}. }
Methods have equal \eqn{\textit{time-averaged}}
means within subject, i.e. \eqn{\sum_{t=1}^{n_t} d_{t,p}/n_t = 0} for all
\eqn{p}. Appropriate when decisions depend on an average over time and
opposite-signed biases are allowed to cancel.

\item \code{Dmat_type = "typical-visit"} (average of squared per-time biases).
With equal visit probability, take
\deqn{ D_m \;\propto\; I_P \otimes
           \mathrm{diag}\!\Big(\tfrac{1}{n_t},\ldots,\tfrac{1}{n_t}\Big), }
yielding
\deqn{ d^\top D_m d \;\propto\; \frac{1}{n_t}
           \sum_{t=1}^{n_t}\sum_{p=1}^{P} d_{t,p}^{\,2}. }
Methods agree on a \eqn{\textit{typical}}
occasion drawn uniformly from the visit set. Use when each visit matters
on its own; alternating signs \eqn{d_{t,p}} do not cancel.

\item \code{Dmat_type = "weighted-avg"} (square of a weighted time average).
For user weights \eqn{a=(a_1,\ldots,a_{n_t})^\top} with \eqn{a_t \ge 0}, set
\deqn{ w \;=\; \frac{a}{\sum_{t=1}^{n_t} a_t}, \qquad
           D_m \;\propto\; I_P \otimes (w\,w^\top), }
so that
\deqn{ d^\top D_m d \;\propto\; \sum_{p=1}^{P}
           \left( \sum_{t=1}^{n_t} w_t\, d_{t,p} \right)^{\!2}. }
Methods have equal \eqn{\textit{weighted
    time-averaged}} means, i.e. \eqn{\sum_{t=1}^{n_t} w_t\, d_{t,p} = 0} for all
\eqn{p}. Use when some visits (e.g., baseline/harvest) are a priori more
influential; opposite-signed biases may cancel according to \eqn{w}.

\item \code{Dmat_type = "weighted-sq"} (weighted average of squared per-time biases).
With the same weights \eqn{w}, take
\deqn{ D_m \;\propto\; I_P \otimes \mathrm{diag}(w_1,\ldots,w_{n_t}), }
giving
\deqn{ d^\top D_m d \;\propto\; \sum_{t=1}^{n_t} w_t
           \sum_{p=1}^{P} d_{t,p}^{\,2}. }
Methods agree at visits sampled with
probabilities \eqn{\{w_t\}}, counting each visit's discrepancy on its own.
Use when per-visit agreement is required but some visits should be
emphasised more than others.
}

\strong{Time-averaging for CCC (regular visits).}
The reported CCC targets agreement of the \emph{time-averaged} measurements
per method within subject by default (\code{Dmat_type="time-avg"}). Averaging over \eqn{T}
non-\code{NA} visits shrinks time-varying components by
\deqn{ \kappa_T^{(g)} \;=\; 1/T, \qquad
      \kappa_T^{(e)} \;=\; \{T + 2\sum_{k=1}^{T-1}(T-k)\rho^k\}/T^2, }
with \eqn{\kappa_T^{(e)}=1/T} when residuals are i.i.d. With unbalanced \eqn{T}, the
implementation averages the per-(subject,method) \eqn{\kappa} values across the
pairs contributing to CCC and then clamps \eqn{\kappa_T^{(e)}} to
\eqn{[10^{-12},\,1]} for numerical stability. Choosing
\code{Dmat_type="typical-visit"} makes \eqn{S_B} match the interpretation of a
randomly sampled occasion instead.

\strong{Concordance correlation coefficient.} The CCC used is
\deqn{ \mathrm{CCC} \;=\;
      \frac{\sigma_A^2 + \kappa_T^{(g)}\,\sigma_{A\times T}^2}
           {\sigma_A^2 + \sigma_{A\times M}^2 +
            \kappa_T^{(g)}\,\sigma_{A\times T}^2 + S_B +
            \kappa_T^{(e)}\,\sigma_E^2}. }
Special cases: with no method factor, \eqn{S_B=\sigma_{A\times M}^2=0}; with
a single time level, \eqn{\sigma_{A\times T}^2=0} (no \eqn{\kappa}-shrinkage).
When \eqn{T=1} or \eqn{\rho=0}, both \eqn{\kappa}-factors equal 1. The \emph{extra}
random-effect variances \eqn{\{\sigma_{Z,j}^2\}} (if used) are \emph{not} included.

\strong{CIs / SEs (delta method for CCC).}
Let
\deqn{ \theta \;=\; \big(\sigma_A^2,\ \sigma_{A\times M}^2,\
\sigma_{A\times T}^2,\ \sigma_E^2,\ S_B\big)^\top, }
and write \eqn{\mathrm{CCC}(\theta)=N/D} with
\eqn{N=\sigma_A^2+\kappa_T^{(g)}\sigma_{A\times T}^2} and
\eqn{D=\sigma_A^2+\sigma_{A\times M}^2+\kappa_T^{(g)}\sigma_{A\times T}^2+S_B+\kappa_T^{(e)}\sigma_E^2}.
The gradient components are
\deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_A^2}
      \;=\; \frac{\sigma_{A\times M}^2 + S_B + \kappa_T^{(e)}\sigma_E^2}{D^2}, }
\deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_{A\times M}^2}
      \;=\; -\,\frac{N}{D^2}, \qquad
       \frac{\partial\,\mathrm{CCC}}{\partial \sigma_{A\times T}^2}
      \;=\; \frac{\kappa_T^{(g)}\big(\sigma_{A\times M}^2 + S_B +
                                    \kappa_T^{(e)}\sigma_E^2\big)}{D^2}, }
\deqn{ \frac{\partial\,\mathrm{CCC}}{\partial \sigma_E^2}
      \;=\; -\,\frac{\kappa_T^{(e)}\,N}{D^2}, \qquad
       \frac{\partial\,\mathrm{CCC}}{\partial S_B}
      \;=\; -\,\frac{N}{D^2}. }

\emph{Estimating \eqn{\mathrm{Var}(\hat\theta)}.}
The EM updates write each variance component as an average of per-subject
quantities. For subject \eqn{i},
\deqn{ t_{A,i} \;=\; b_{i,0}^2 + (M_i^{-1})_{00},\qquad
       t_{M,i} \;=\; \frac{1}{nm}\sum_{\ell=1}^{nm}
                       \Big(b_{i,\ell}^2 + (M_i^{-1})_{\ell\ell}\Big), }
\deqn{ t_{T,i} \;=\; \frac{1}{nt}\sum_{j=1}^{nt}
                       \Big(b_{i,j}^2 + (M_i^{-1})_{jj}\Big),\qquad
       s_i \;=\; \frac{e_i^\top C_i(\rho)^{-1} e_i +
       \mathrm{tr}\!\big(M_i^{-1}U_i^\top C_i(\rho)^{-1} U_i\big)}{n_i}, }
where \eqn{b_i = M_i^{-1}(U_i^\top R_i^{-1} r_i)} and
\eqn{M_i = G^{-1} + U_i^\top R_i^{-1} U_i}.
With \eqn{m} subjects, form the empirical covariance of the stacked
subject vectors and scale by \eqn{m} to approximate the covariance of the
means:
\deqn{ \widehat{\mathrm{Cov}}\!\left(
      \begin{bmatrix} t_{A,\cdot} \\ t_{M,\cdot} \\ t_{T,\cdot} \end{bmatrix}
      \right)
      \;\approx\; \frac{1}{m}\,
       \mathrm{Cov}_i\!\left(
      \begin{bmatrix} t_{A,i} \\ t_{M,i} \\ t_{T,i} \end{bmatrix}\right). }
(Drop rows/columns as needed when \code{nm==0} or \code{nt==0}.)

The residual variance estimator is a weighted mean
\eqn{\hat\sigma_E^2=\sum_i w_i s_i} with \eqn{w_i=n_i/n}. Its variance is
approximated by the variance of a weighted mean of independent terms,
\deqn{ \widehat{\mathrm{Var}}(\hat\sigma_E^2)
      \;\approx\; \Big(\sum_i w_i^2\Big)\,\widehat{\mathrm{Var}}(s_i), }
where \eqn{\widehat{\mathrm{Var}}(s_i)} is the sample variance across
subjects. The method-dispersion term uses the quadratic-form delta already
computed for \eqn{S_B}:
\deqn{ \widehat{\mathrm{Var}}(S_B)
      \;=\; \frac{2\,\mathrm{tr}\!\big((A_{\!fix}\,\mathrm{Var}(\hat\beta))^2\big)
             \;+\; 4\,\hat\beta^\top A_{\!fix}\,\mathrm{Var}(\hat\beta)\,
             A_{\!fix}\,\hat\beta}
                   {\big[nm\,(nm-1)\,\max(nt,1)\big]^2}, }
with \eqn{A_{\!fix}=L\,D_m\,L^\top}.

\emph{Putting it together.} Assemble
\eqn{\widehat{\mathrm{Var}}(\hat\theta)} by combining the
\eqn{(\sigma_A^2,\sigma_{A\times M}^2,\sigma_{A\times T}^2)} covariance
block from the subject-level empirical covariance, add the
\eqn{\widehat{\mathrm{Var}}(\hat\sigma_E^2)} and
\eqn{\widehat{\mathrm{Var}}(S_B)} terms on the diagonal,
and ignore cross-covariances across these blocks (a standard large-sample
simplification). Then
\deqn{ \widehat{\mathrm{se}}\{\widehat{\mathrm{CCC}}\}
      \;=\; \sqrt{\,\nabla \mathrm{CCC}(\hat\theta)^\top\,
                    \widehat{\mathrm{Var}}(\hat\theta)\,
                    \nabla \mathrm{CCC}(\hat\theta)\,}. }

A two-sided \eqn{(1-\alpha)} normal CI is
\deqn{ \widehat{\mathrm{CCC}} \;\pm\; z_{1-\alpha/2}\,
      \widehat{\mathrm{se}}\{\widehat{\mathrm{CCC}}\}, }
truncated to \eqn{[0,1]} in the output for convenience. When \eqn{S_B} is
truncated at 0 or samples are very small/imbalanced, the normal CI can be
mildly anti-conservative near the boundary; a logit transform for CCC or a
subject-level (cluster) bootstrap can be used for sensitivity analysis.

\strong{Choosing \eqn{\rho} for AR(1).}
When \code{ar="ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
profiling the REML log-likelihood at \eqn{(\hat\beta,\hat G,\hat\sigma_E^2)}.
With very few visits per subject, \eqn{\rho} can be weakly identified; consider
sensitivity checks over a plausible range.
}
\section{Notes on stability and performance}{

All per-subject solves are \eqn{\,r\times r} with \eqn{r=1+nm+nt+q_Z}, so cost
scales with the number of subjects and the fixed-effects dimension rather
than the total number of observations. Solvers use symmetric-PD paths with
a small diagonal ridge and pseudo-inverse,
which helps for very small/unbalanced subsets and near-boundary estimates.
For \code{AR(1)}, observations are ordered by time within subject; \code{NA} time codes
break the run, and gaps between factor levels are treated as regular steps
(elapsed time is not used).

\emph{Heteroscedastic slopes across \eqn{Z} columns are supported.}
Each \eqn{Z} column has its own variance component \eqn{\sigma_{Z,j}^2}, but
cross-covariances among \eqn{Z} columns are set to zero (diagonal block). Column
rescaling changes the implied prior on \eqn{b_{i,\text{extra}}} but does not
introduce correlations.
}

\section{Threading and BLAS guards}{

The C++ backend uses OpenMP loops while also forcing vendor BLAS libraries to
run single-threaded so that overall CPU usage stays predictable. This guard
is applied to OpenBLAS, Apple's Accelerate, and Intel MKL when their runtime
controls are available. You can opt out manually by setting
\code{MATRIXCORR_DISABLE_BLAS_GUARD=1} in the environment before loading
\pkg{matrixCorr}.
}

\section{Model overview}{

Internally, the call is routed to \code{ccc_lmm_reml_pairwise()}, which fits one
repeated-measures mixed model per pair of methods. Each model includes:
\itemize{
\item subject random intercepts (always)
\item optional subject-by-method (\verb{sigma^2_\{A \\times M\}}) and
subject-by-time (\verb{sigma^2_\{A \\times T\}}) variance components
\item optional random slopes specified via \code{slope}/\code{slope_var}/\code{slope_Z}
\item residual structure \code{ar = "none"} (iid) or \code{ar = "ar1"}
}
D-matrix options (\code{Dmat_type}, \code{Dmat}, \code{Dmat_weights}) control how time
averaging operates when translating variance components into CCC summaries.
}

\examples{
# ====================================================================
# 1) Subject x METHOD variance present, no time
#     y_{i,m} = mu + b_m + u_i + w_{i,m} + e_{i,m}
#     with u_i ~ N(0, s_A^2), w_{i,m} ~ N(0, s_{AxM}^2)
# ====================================================================
set.seed(102)
n_subj <- 60
n_time <- 8

id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
method <- factor(rep(rep(c("A","B"),    each  = n_time), times = n_subj))

sigA  <- 0.6   # subject
sigAM <- 0.3   # subject x method
sigAT <- 0.5   # subject x time
sigE  <- 0.4   # residual
# Expected estimate S_B = 0.2^2 = 0.04
biasB <- 0.2   # fixed method bias

# random effects
u_i <- rnorm(n_subj, 0, sqrt(sigA))
u   <- u_i[as.integer(id)]

sm      <- interaction(id, method, drop = TRUE)
w_im_lv <- rnorm(nlevels(sm), 0, sqrt(sigAM))
w_im    <- w_im_lv[as.integer(sm)]

st      <- interaction(id, time, drop = TRUE)
g_it_lv <- rnorm(nlevels(st), 0, sqrt(sigAT))
g_it    <- g_it_lv[as.integer(st)]

# residuals & response
e <- rnorm(length(id), 0, sqrt(sigE))
y <- (method == "B") * biasB + u + w_im + g_it + e

dat_both <- data.frame(y, id, method, time)

# Both sigma2_subject_method and sigma2_subject_time are identifiable here
fit_both <- ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
                         vc_select = "auto", ci = TRUE, verbose = TRUE)
summary(fit_both)
estimate(fit_both)
tidy(fit_both)
ci(fit_both)
confint(fit_both)
plot(fit_both)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(fit_both)
}

# ====================================================================
# 2) Subject x TIME variance present (sag > 0) with two methods
#     y_{i,m,t} = mu + b_m + u_i + g_{i,t} + e_{i,m,t}
#     where g_{i,t} ~ N(0, s_{AxT}^2) shared across methods at time t
# ====================================================================
set.seed(202)
n_subj <- 60; n_time <- 14
id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))

sigA  <- 0.7
sigAT <- 0.5
sigE  <- 0.5
biasB <- 0.25

u   <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
gIT <- rnorm(n_subj * n_time, 0, sqrt(sigAT))
g   <- gIT[ (as.integer(id) - 1L) * n_time + as.integer(time) ]
y   <- (method == "B") * biasB + u + g + rnorm(length(id), 0, sqrt(sigE))
dat_sag <- data.frame(y, id, method, time)

# sigma_AT should be retained; sigma_AM may be dropped (since w_{i,m}=0)
fit_sag <- ccc_rm_reml(dat_sag, "y", "id", method = "method", time = "time",
                        vc_select = "auto", verbose = TRUE)
summary(fit_sag)
plot(fit_sag)

# ====================================================================
# 3) BOTH components present: sab > 0 and sag > 0 (2 methods x T times)
#     y_{i,m,t} = mu + b_m + u_i + w_{i,m} + g_{i,t} + e_{i,m,t}
# ====================================================================
set.seed(303)
n_subj <- 60; n_time <- 4
id     <- factor(rep(seq_len(n_subj), each = 2 * n_time))
method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
time   <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))

sigA  <- 0.8
sigAM <- 0.3
sigAT <- 0.4
sigE  <- 0.5
biasB <- 0.2

u   <- rnorm(n_subj, 0, sqrt(sigA))[as.integer(id)]
# (subject, method) random deviations: repeat per (i,m) across its times
wIM <- rnorm(n_subj * 2, 0, sqrt(sigAM))
w   <- wIM[ (as.integer(id) - 1L) * 2 + as.integer(method) ]
# (subject, time) random deviations: shared across methods at time t
gIT <- rnorm(n_subj * n_time, 0, sqrt(sigAT))
g   <- gIT[ (as.integer(id) - 1L) * n_time + as.integer(time) ]
y   <- (method == "B") * biasB + u + w + g + rnorm(length(id), 0, sqrt(sigE))
dat_both <- data.frame(y, id, method, time)

fit_both <- ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
                         vc_select = "auto", verbose = TRUE, ci = TRUE)
summary(fit_both)
plot(fit_both)

# If you want to force-include both VCs (skip testing):
fit_both_forced <-
 ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
              vc_select = "none", include_subj_method  = TRUE,
              include_subj_time  = TRUE, verbose = TRUE)
summary(fit_both_forced)
plot(fit_both_forced)

# ====================================================================
# 4) D_m choices: time-averaged (default) vs typical visit
# ====================================================================
# Time-average
ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
             vc_select = "none", include_subj_method  = TRUE,
             include_subj_time  = TRUE, Dmat_type = "time-avg")

# Typical visit
ccc_rm_reml(dat_both, "y", "id", method = "method", time = "time",
             vc_select = "none", include_subj_method  = TRUE,
             include_subj_time  = TRUE, Dmat_type = "typical-visit")

# ====================================================================
# 5) AR(1) residual correlation with fixed rho (larger example)
# ====================================================================
\donttest{
set.seed(10)
n_subj   <- 40
n_time   <- 10
methods  <- c("A", "B", "C", "D")
nm       <- length(methods)
id     <- factor(rep(seq_len(n_subj), each = n_time * nm))
method <- factor(rep(rep(methods, each = n_time), times = n_subj),
                 levels = methods)
time   <- factor(rep(rep(seq_len(n_time), times = nm), times = n_subj))

beta0    <- 0
beta_t   <- 0.2
bias_met <- c(A = 0.00, B = 0.30, C = -0.15, D = 0.05)
sigA     <- 1.0
rho_true <- 0.6
sigE     <- 0.7

t_num <- as.integer(time)
t_c   <- t_num - mean(seq_len(n_time))
mu    <- beta0 + beta_t * t_c + bias_met[as.character(method)]

u_subj <- rnorm(n_subj, 0, sqrt(sigA))
u      <- u_subj[as.integer(id)]

e <- numeric(length(id))
for (s in seq_len(n_subj)) {
  for (m in methods) {
    idx <- which(id == levels(id)[s] & method == m)
    e[idx] <- stats::arima.sim(list(ar = rho_true), n = n_time, sd = sigE)
  }
}
y <- mu + u + e
dat_ar4 <- data.frame(y = y, id = id, method = method, time = time)

fit4 <- ccc_rm_reml(dat_ar4,
             response = "y", subject = "id", method = "method", time = "time",
             ar = "ar1", ar_rho = 0.6, verbose = TRUE)
fit4
summary(fit4)
plot(fit4)
}

# ====================================================================
# 6) Random slope variants (subject, method, custom Z)
# ====================================================================
\donttest{
## By SUBJECT
set.seed(2)
n_subj <- 60; n_time <- 4
id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
subj <- as.integer(id)
slope_i <- rnorm(n_subj, 0, 0.15)
slope_vec <- slope_i[subj]
base <- rnorm(n_subj, 0, 1.0)[subj]
tnum <- as.integer(tim)
y <- base + 0.3*(method=="B") + slope_vec*(tnum - mean(seq_len(n_time))) +
     rnorm(length(id), 0, 0.5)
dat_s <- data.frame(y, id, method, time = tim)
dat_s$t_num <- as.integer(dat_s$time)
dat_s$t_c   <- ave(dat_s$t_num, dat_s$id, FUN = function(v) v - mean(v))
ccc_rm_reml(dat_s, "y", "id", method = "method", time = "time",
             slope = "subject", slope_var = "t_c", verbose = TRUE)

## By METHOD
set.seed(3)
n_subj <- 60; n_time <- 4
id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
slope_m <- ifelse(method=="B", 0.25, 0.00)
base <- rnorm(n_subj, 0, 1.0)[as.integer(id)]
tnum <- as.integer(tim)
y <- base + 0.3*(method=="B") + slope_m*(tnum - mean(seq_len(n_time))) +
     rnorm(length(id), 0, 0.5)
dat_m <- data.frame(y, id, method, time = tim)
dat_m$t_num <- as.integer(dat_m$time)
dat_m$t_c   <- ave(dat_m$t_num, dat_m$id, FUN = function(v) v - mean(v))
ccc_rm_reml(dat_m, "y", "id", method = "method", time = "time",
             slope = "method", slope_var = "t_c", verbose = TRUE)

## SUBJECT + METHOD random slopes (custom Z)
set.seed(4)
n_subj <- 50; n_time <- 4
id  <- factor(rep(seq_len(n_subj), each = 2 * n_time))
tim <- factor(rep(rep(seq_len(n_time), times = 2), times = n_subj))
method <- factor(rep(rep(c("A","B"), each = n_time), times = n_subj))
subj <- as.integer(id)
slope_subj <- rnorm(n_subj, 0, 0.12)[subj]
slope_B    <- ifelse(method=="B", 0.18, 0.00)
tnum <- as.integer(tim)
base <- rnorm(n_subj, 0, 1.0)[subj]
y <- base + 0.3*(method=="B") +
     (slope_subj + slope_B) * (tnum - mean(seq_len(n_time))) +
     rnorm(length(id), 0, 0.5)
dat_bothRS <- data.frame(y, id, method, time = tim)
dat_bothRS$t_num <- as.integer(dat_bothRS$time)
dat_bothRS$t_c   <- ave(dat_bothRS$t_num, dat_bothRS$id, FUN = function(v) v - mean(v))
MM <- model.matrix(~ 0 + method, data = dat_bothRS)
Z_custom <- cbind(
  subj_slope = dat_bothRS$t_c,
  MM * dat_bothRS$t_c
)
ccc_rm_reml(dat_bothRS, "y", "id", method = "method", time = "time",
             slope = "custom", slope_Z = Z_custom, verbose = TRUE)
}

}
\references{
Lin L (1989). A concordance correlation coefficient to evaluate reproducibility.
\emph{Biometrics}, 45: 255-268.

Lin L (2000). A note on the concordance correlation coefficient.
\emph{Biometrics}, 56: 324-325.

Carrasco, J. L. et al. (2013). Estimation of the concordance
correlation coefficient for repeated measures using SAS and R.
\emph{Computer Methods and Programs in Biomedicine}, 109(3), 293-304.
\doi{10.1016/j.cmpb.2012.09.002}

King et al. (2007). A Class of Repeated Measures Concordance
Correlation Coefficients.
\emph{Journal of Biopharmaceutical Statistics}, 17(4).
\doi{10.1080/10543400701329455}
}
\seealso{
\code{build_L_Dm_Z_cpp}
for constructing \eqn{L}/\eqn{D_m}/\eqn{Z}; \code{\link{ccc_rm_ustat}}
for a U-statistic alternative; and \pkg{cccrm} for a reference approach via
\pkg{nlme}.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{polychoric}
\alias{polychoric}
\alias{print.polychoric_corr}
\alias{plot.polychoric_corr}
\alias{summary.polychoric_corr}
\alias{print.summary.polychoric_corr}
\title{Pairwise Polychoric Correlation}
\usage{
polychoric(
  data,
  y = NULL,
  na_method = c("error", "pairwise"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  correct = 0.5,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{polychoric_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{polychoric_corr}(
  x,
  title = "Polychoric correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{polychoric_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.polychoric_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{An ordinal vector, matrix, or data frame. Supported columns are
factors, ordered factors, logical values, or integer-like numerics.
In matrix/data-frame mode, only supported ordinal columns are retained.}

\item{y}{Optional second ordinal vector. When supplied, the function returns
a single polychoric correlation estimate.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
complete cases.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald confidence intervals derived from the observed
information matrix of the latent-variable likelihood.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald p-values and test statistics for each
estimated latent correlation.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{correct}{Non-negative continuity correction added to zero-count cells.
Default is \code{0.5}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{print()}.}

\item{x}{An object of class \code{summary.polychoric_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Polychoric correlation heatmap"}.}

\item{low_color}{Color for the minimum correlation.}

\item{high_color}{Color for the maximum correlation.}

\item{mid_color}{Color for zero correlation.}

\item{value_text_size}{Font size used in tile labels.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{polychoric_corr}.}

\item{ci_digits}{Integer; digits for confidence limits in the pairwise
summary.}

\item{p_digits}{Integer; digits for p-values in the pairwise summary.}
}
\value{
If \code{y} is supplied, a numeric scalar with attributes
\code{diagnostics} and \code{thresholds}. Otherwise a symmetric matrix of
class \code{polychoric_corr} with attributes \code{method},
\code{description}, \code{package = "matrixCorr"}, \code{diagnostics},
\code{thresholds}, and \code{correct}. When \code{p_value = TRUE}, the
returned object also carries an \code{inference} attribute with elements
\code{estimate}, \code{statistic}, \code{parameter}, \code{p_value}, and
\code{n_obs}. When \code{ci = TRUE}, it also carries a \code{ci} attribute
with elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
and \code{ci.method}, plus \code{attr(x, "conf.level")}. Scalar outputs keep
the same point estimate and gain the same metadata only when inference is
requested. In matrix mode, \code{output = "edge_list"} returns a data frame with columns
\code{row}, \code{col}, \code{value}; \code{output = "sparse"} returns a
symmetric sparse matrix.
}
\description{
Computes the polychoric correlation for either a pair of ordinal variables
or all pairwise combinations of ordinal columns in a matrix/data frame.
}
\details{
The polychoric correlation generalises the tetrachoric model to ordered
categorical variables with more than two levels. It assumes latent
standard-normal variables \eqn{Z_1, Z_2} with correlation \eqn{\rho}, and
cut-points
\eqn{-\infty = \alpha_0 < \alpha_1 < \cdots < \alpha_R = \infty} and
\eqn{-\infty = \beta_0 < \beta_1 < \cdots < \beta_C = \infty} such that
\deqn{
X = r \iff \alpha_{r-1} < Z_1 \le \alpha_r,
\qquad
Y = c \iff \beta_{c-1} < Z_2 \le \beta_c.
}
For an observed \eqn{R \times C} contingency table with counts \eqn{n_{rc}},
the thresholds are estimated from the marginal cumulative proportions:
\deqn{
\alpha_r = \Phi^{-1}\!\Big(\sum_{k \le r} P(X = k)\Big),
\qquad
\beta_c = \Phi^{-1}\!\Big(\sum_{k \le c} P(Y = k)\Big).
}
Holding those thresholds fixed, the log-likelihood for the latent
correlation is
\deqn{
\ell(\rho) = \sum_{r=1}^{R}\sum_{c=1}^{C}
n_{rc} \log \Pr\!\big(
\alpha_{r-1} < Z_1 \le \alpha_r,\;
\beta_{c-1} < Z_2 \le \beta_c
\mid \rho \big),
}
and the estimator returned is the maximiser over \eqn{\rho \in (-1,1)}.
The C++ implementation performs a dense one-dimensional search followed by
Brent refinement.

The argument \code{correct} adds a non-negative continuity correction to
empty cells before marginal threshold estimation and likelihood evaluation.
This avoids numerical failures for sparse tables with structurally zero cells.
When \code{correct = 0} and zero cells are present, the corresponding fit can
be boundary-driven rather than a regular interior maximum-likelihood problem.
The returned object stores sparse-fit diagnostics and the thresholds used for
estimation so those cases can be inspected explicitly.

\strong{Assumptions.} The coefficient is appropriate when both observed
ordinal variables are viewed as discretisations of jointly normal latent
variables. The optional p-values and confidence intervals adopt this
latent-normal interpretation and use the same likelihood that defines the
polychoric estimate. These inferential quantities are therefore model-based
and should not be interpreted as distribution-free summaries.

\strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
function refits the pairwise polychoric model by maximum likelihood and
obtains the observed information matrix numerically in C++. The reported
confidence interval is a Wald interval
\eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
reported p-value is from the large-sample Wald \eqn{z}-test for
\eqn{H_0:\rho = 0}. These inferential quantities are only computed when
explicitly requested.

In matrix/data-frame mode, all pairwise polychoric correlations are computed
between supported ordinal columns. Diagonal entries are \code{1} for
non-degenerate columns and \code{NA} when a column has fewer than two
observed levels.

\strong{Computational complexity.} For \eqn{p} ordinal variables, the matrix
path evaluates \eqn{p(p-1)/2} bivariate likelihoods. Each pair optimises a
single scalar parameter \eqn{\rho}, so the main cost is repeated evaluation
of bivariate normal rectangle probabilities.
}
\examples{
\dontshow{if (requireNamespace("mnormt", quietly = TRUE)) withAutoprint(\{ # examplesIf}
set.seed(124)
n <- 1200
Sigma <- matrix(c(
  1.00, 0.60, 0.40,
  0.60, 1.00, 0.50,
  0.40, 0.50, 1.00
), 3, 3, byrow = TRUE)

Z <- mnormt::rmnorm(n = n, mean = rep(0, 3), varcov = Sigma)
Y <- data.frame(
  y1 = ordered(cut(
    Z[, 1],
    breaks = c(-Inf, -0.7, 0.4, Inf),
    labels = c("low", "mid", "high")
  )),
  y2 = ordered(cut(
    Z[, 2],
    breaks = c(-Inf, -1.0, -0.1, 0.8, Inf),
    labels = c("1", "2", "3", "4")
  )),
  y3 = ordered(cut(
    Z[, 3],
    breaks = c(-Inf, -0.4, 0.2, 1.1, Inf),
    labels = c("A", "B", "C", "D")
  ))
)

pc <- polychoric(Y)
print(pc, digits = 3)
summary(pc)
estimate(pc)
tidy(pc)
pc_ci <- polychoric(Y, ci = TRUE)
ci(pc_ci)
confint(pc_ci)
plot(pc)
polychoric(Y, output = "edge_list", threshold = 0.3, diag = FALSE)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(pc)
}

# latent Pearson correlations used to generate the ordinal variables
round(stats::cor(Z), 2)
\dontshow{\}) # examplesIf}
}
\references{
Olsson, U. (1979). Maximum likelihood estimation of the polychoric
correlation coefficient. \emph{Psychometrika}, 44(4), 443-460.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_bad_arg}
\alias{abort_bad_arg}
\title{Abort with a standardised argument error}
\usage{
abort_bad_arg(arg, message, ..., .hint = NULL, .class = character())
}
\description{
Abort with a standardised argument error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/skipcor.R
\name{skipped_corr}
\alias{skipped_corr}
\alias{skipped_corr_masks}
\alias{print.skipped_corr}
\alias{plot.skipped_corr}
\alias{summary.skipped_corr}
\alias{print.summary.skipped_corr}
\title{Pairwise skipped correlation}
\usage{
skipped_corr(
  data,
  method = c("pearson", "spearman"),
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  return_masks = FALSE,
  stand = TRUE,
  outlier_rule = c("idealf", "mad"),
  cutoff = sqrt(stats::qchisq(0.975, df = 2)),
  n_boot = 2000L,
  p_adjust = c("none", "hochberg", "ecp"),
  fwe_level = 0.05,
  n_mc = 1000L,
  seed = NULL,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

skipped_corr_masks(x, var1 = NULL, var2 = NULL)

\method{print}{skipped_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 4,
  show_ci = NULL,
  show_p = c("auto", "yes", "no"),
  ...
)

\method{plot}{skipped_corr}(
  x,
  title = "Skipped correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{summary}{skipped_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.skipped_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns will be excluded.}

\item{method}{Correlation computed after removing projected outliers. One of
\code{"pearson"} (default) or \code{"spearman"}.}

\item{na_method}{Character scalar controlling missing-data handling.
With \code{"error"}, the function requires all retained numeric columns to
be free of missing or non-finite values and aborts otherwise. This is the
recommended setting when you want a single common sample size across all
pairs, reproducible skipped-row diagnostics on the same rows, or bootstrap
inference via \code{ci = TRUE} / \code{p_value = TRUE}.
\code{"pairwise"} recomputes each estimate on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical; if \code{TRUE}, attach percentile-bootstrap confidence
intervals for each skipped correlation using the Wilcox (2015) B2
resampling scheme. Default \code{FALSE}.}

\item{p_value}{Logical; if \code{TRUE}, attach bootstrap p-values for testing
whether each skipped correlation is zero. Default \code{FALSE}.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{return_masks}{Logical; if \code{TRUE}, attach compact pairwise skipped-row
indices as an attribute. Default \code{FALSE}.}

\item{stand}{Logical; if \code{TRUE} (default), each variable in the pair is
centred by its median and divided by a robust scale estimate before the
projection outlier search. The scale estimate is the MAD when positive,
with fallback to \eqn{\mathrm{IQR}/1.34898} and then the usual sample
standard deviation if needed. This standardisation affects only outlier
detection, not the final correlation computed on the retained
observations.}

\item{outlier_rule}{One of \code{"idealf"} (default) or \code{"mad"}.
The default uses the ideal-fourths interquartile width of projected
distances; \code{"mad"} uses the median absolute deviation of projected
distances.}

\item{cutoff}{Positive numeric constant multiplying the projected spread in
the outlier rule
\eqn{\mathrm{med}(d_{i\cdot}) + cutoff \times s(d_{i\cdot})}. Larger
values flag fewer observations as outliers; smaller values flag more.
Default \code{sqrt(qchisq(0.975, df = 2))}.}

\item{n_boot}{Integer \eqn{\geq 2}. Number of bootstrap resamples used when
\code{ci = TRUE} or \code{p_value = TRUE}. Default \code{2000}.}

\item{p_adjust}{One of \code{"none"} (default), \code{"hochberg"}, or
\code{"ecp"}. Optional familywise-error procedure applied to the matrix of
bootstrap p-values. \code{"hochberg"} corresponds to method H in Wilcox,
Rousselet, and Pernet (2018); \code{"ecp"} corresponds to their simulated
critical-p-value method ECP.}

\item{fwe_level}{Familywise-error level used when
\code{p_adjust = "hochberg"} or \code{"ecp"}. Default \code{0.05}.}

\item{n_mc}{Integer \eqn{\geq 10}. Number of null Monte Carlo data sets used
when \code{p_adjust = "ecp"} to estimate the critical p-value. Default
\code{1000}.}

\item{seed}{Optional positive integer used to seed the bootstrap resampling
when \code{ci = TRUE} or \code{p_value = TRUE}. If \code{NULL}, a fresh
internal seed is generated.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{x}{An object of class \code{summary.skipped_corr}.}

\item{var1, var2}{Optional column names or 1-based column indices used by
\code{\link[=skipped_corr_masks]{skipped_corr_masks()}} to extract the skipped-row indices for one pair.}

\item{digits}{Integer; number of digits to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for skipped-correlation confidence limits.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{show_p}{One of \code{"auto"}, \code{"yes"}, \code{"no"}. For
\code{print()}, \code{"auto"} keeps the compact matrix-only display;
use \code{"yes"} to also print pairwise p-values.}

\item{...}{Additional arguments passed to the underlying print or plot helper.}

\item{title}{Character; plot title.}

\item{low_color, high_color, mid_color}{Colors used in the heatmap.}

\item{value_text_size}{Numeric text size for overlaid cell values.}

\item{ci_text_size}{Text size for confidence intervals in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{skipped_corr}.}
}
\value{
A symmetric correlation matrix with class \code{skipped_corr} and
attributes \code{method = "skipped_correlation"}, \code{description}, and
\code{package = "matrixCorr"}. When \code{return_masks = TRUE}, the matrix
also carries a \code{skipped_masks} attribute containing compact pairwise
skipped-row indices. The \code{diagnostics} attribute stores per-pair
complete-case counts and skipped-row counts/proportions. When
\code{ci = TRUE} or \code{p_value = TRUE}, bootstrap inference matrices are
attached via attributes.
}
\description{
Computes all pairwise skipped correlation coefficients for the numeric
columns of a matrix or data frame using a high-performance 'C++' backend.

Skipped correlation detects bivariate outliers using a projection rule and
then computes Pearson or Spearman correlation on the retained observations.
It is designed for situations where marginally robust methods can still be
distorted by unusual points in the joint data cloud.
}
\details{
Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
observations and columns as variables. For a given pair of columns
\eqn{(x, y)}, write the observed bivariate points as
\eqn{u_i = (x_i, y_i)^\top}, \eqn{i=1,\ldots,n}. If \code{stand = TRUE},
each margin is first centred by its median and divided by a robust scale
estimate before outlier detection; otherwise the original pair is used. The
robust scale is the MAD when positive, with fallback to
\eqn{\mathrm{IQR}/1.34898} and then the usual sample standard deviation if
needed. Let \eqn{\tilde u_i} denote the resulting points and let \eqn{c} be
the componentwise median center of the detection cloud.

For each observation \eqn{i}, define the direction vector
\eqn{b_i = \tilde u_i - c}. When \eqn{\|b_i\| > 0}, all observations are
projected onto the line through \eqn{c} in direction \eqn{b_i}. The
projected distances are
\deqn{
d_{ij} \;=\; \frac{|(\tilde u_j - c)^\top b_i|}{\|b_i\|},
\qquad j=1,\ldots,n.
}
For each direction \eqn{i}, observation \eqn{j} is flagged as an outlier if

\deqn{
d_{ij} \;>\; \mathrm{med}(d_{i\cdot}) + g\, s(d_{i\cdot}),
\qquad g = \code{cutoff},
}
where \eqn{s(\cdot)} is either the ideal-fourths interquartile width
(\code{outlier_rule = "idealf"}) or the median absolute deviation
(\code{outlier_rule = "mad"}). An observation is removed if it is flagged
for at least one projection direction. The skipped correlation is then the
ordinary Pearson or Spearman correlation computed from the retained
observations:
\deqn{
r_{\mathrm{skip}}(x,y) \;=\;
\mathrm{cor}\!\left(x_{\mathcal{K}}, y_{\mathcal{K}}\right),
}
where \eqn{\mathcal{K}} is the index set of observations not flagged as
outliers.

Unlike marginally robust methods such as \code{pbcor()}, \code{wincor()},
or \code{bicor()}, skipped correlation is explicitly pairwise because
outlier detection depends on the joint geometry of each variable pair. As a
result, the reported matrix need not be positive semidefinite, even with
complete data.

\strong{Computational notes.} In the complete-data path, each column pair
requires a full bivariate projection search, so the dominant cost is higher
than for marginal robust methods. The implementation evaluates pairs in
'C++'; where available, pairs are processed with 'OpenMP' parallelism. With
\code{na_method = "pairwise"}, each pair is recomputed on its overlap of
non-missing rows. With \code{na_method = "complete"}, rows with any
non-finite value across the retained numeric columns are removed before any
pairwise outlier search. This gives a common row universe for all
skipped-correlation masks.

\strong{Bootstrap inference.} When \code{ci = TRUE} or \code{p_value = TRUE},
the implementation uses the percentile-bootstrap strategy studied by Wilcox
(2015). Each bootstrap replicate resamples whole observation pairs with
replacement, reruns the skipped-correlation outlier detection on the
resampled data, and recomputes the skipped correlation on the retained
observations. This corresponds to Wilcox's B2 method and avoids the
statistically unsatisfactory shortcut of removing outliers only once before
bootstrapping. Bootstrap inference currently requires complete data
(\code{na_method = "error"} or \code{"complete"}). When
\code{p_adjust = "hochberg"}, the
bootstrap p-values are processed with Hochberg's step-up procedure (method H
in Wilcox, Rousselet, and Pernet, 2018). When \code{p_adjust = "ecp"}, the
package follows their ECP method and simulates \code{n_mc} null data sets
from a \eqn{p}-variate normal distribution with identity covariance,
recomputes the pairwise bootstrap p-values for each null data set, stores the
minimum p-value from each run, and estimates the \code{fwe_level} quantile of
that null distribution using the Harrell-Davis estimator. Hypotheses are then
rejected when their observed bootstrap p-values are less than or equal to the
estimated critical p-value. The calibrated H1 procedure from Wilcox,
Rousselet, and Pernet (2018) is not currently implemented.
}
\examples{
set.seed(12)
X <- matrix(rnorm(160 * 4), ncol = 4)
X[1, 1] <- 9
X[1, 2] <- -8

R <- skipped_corr(X, method = "pearson")
print(R, digits = 2)
summary(R)
plot(R)

Rm <- skipped_corr(X, method = "pearson", return_masks = TRUE)
skipped_corr_masks(Rm, 1, 2)

# Example 1:
Xm <- as.matrix(datasets::mtcars[, c("mpg", "disp", "hp", "wt")])
Rm2 <- skipped_corr(Xm, method = "spearman")
print(Rm2, digits = 2)

# Example 2:
Ri <- skipped_corr(Xm, method = "pearson", ci = TRUE, n_boot = 40, seed = 1)
ci(Ri)
confint(Ri)
tidy(Ri)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(R)
}

}
\references{
Wilcox, R. R. (2004). Inferences based on a skipped correlation coefficient.
Journal of Applied Statistics, 31(2), 131-143.
\doi{10.1080/0266476032000148821}

Wilcox, R. R. (2015). Inferences about the skipped correlation coefficient:
Dealing with heteroscedasticity and non-normality. Journal of Modern Applied
Statistical Methods, 14(1), 172-188.
\doi{10.22237/jmasm/1430453580}

Wilcox, R. R., Rousselet, G. A., & Pernet, C. R. (2018). Improved methods
for making inferences about multiple skipped correlations. Journal of
Statistical Computation and Simulation, 88(16), 3116-3131.
\doi{10.1080/00949655.2018.1501051}
}
\seealso{
\code{\link[=pbcor]{pbcor()}}, \code{\link[=wincor]{wincor()}}, \code{\link[=bicor]{bicor()}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multirater_kappa.R
\name{multirater_kappa}
\alias{multirater_kappa}
\alias{print.multirater_kappa}
\alias{summary.multirater_kappa}
\alias{print.summary.multirater_kappa}
\alias{plot.multirater_kappa}
\title{Multi-Rater Kappa for Nominal Ratings}
\usage{
multirater_kappa(
  data,
  levels = NULL,
  input = c("ratings", "counts"),
  method = c("fleiss", "randolph"),
  exact = FALSE,
  na_method = c("error", "complete", "available"),
  min_raters = 2L,
  by_category = FALSE,
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  se_method = c("asymptotic", "jackknife", "none"),
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE,
  ...
)

\method{print}{multirater_kappa}(x, digits = 4, show_by_category = FALSE, ...)

\method{summary}{multirater_kappa}(object, digits = 4, ...)

\method{print}{summary.multirater_kappa}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{multirater_kappa}(
  x,
  type = c("agreement_map", "estimate", "item_agreement", "category_proportion",
    "by_category"),
  bins = 30L,
  ...
)
}
\arguments{
\item{data}{Input ratings or counts data.
For \code{input = "ratings"}, rows are items/subjects and columns are
raters/classifiers. For \code{input = "counts"}, rows are items/subjects,
columns are categories, and values are counts of raters assigning that
category to that item.}

\item{levels}{Optional explicit category labels. For nominal multi-rater
kappa, category order is not used in the estimator itself, but explicit
levels are useful when unobserved categories should be retained in the
output. If factor columns are mixed with non-factor columns, \code{levels}
must be supplied to avoid ambiguous implicit ordering.}

\item{input}{One of \code{"ratings"} or \code{"counts"}.}

\item{method}{One of \code{"fleiss"} (fixed-marginal multi-rater kappa) or
\code{"randolph"} (free-marginal multi-rater kappa).}

\item{exact}{Logical; if \code{TRUE}, use the exact Fleiss fixed-marginal
variant. This is available only for \code{input = "ratings"} with
complete item-by-rater data and \code{method = "fleiss"}.}

\item{na_method}{Missing-data rule for \code{input = "ratings"}.
\code{"error"} rejects missing ratings. \code{"complete"} removes any item
with one or more missing ratings. \code{"available"} retains items with at
least \code{min_raters} observed ratings and allows the number of raters to
vary by item. For \code{input = "counts"}, missing counts are not allowed.}

\item{min_raters}{Minimum number of observed ratings required for an item to
be retained when \code{na_method = "available"} or when
\code{input = "counts"} rows are filtered by total raters. Must be at
least \code{2}.}

\item{by_category}{Logical; if \code{TRUE}, attach category-wise kappas based
on target-vs-other binary collapses of the category count matrix.}

\item{ci}{Logical; if \code{TRUE}, attach confidence intervals using the
selected \code{se_method}.}

\item{p_value}{Logical; if \code{TRUE}, attach z statistics and two-sided
p-values using the selected \code{se_method}.}

\item{conf_level}{Confidence level for intervals. Default is \code{0.95}.}

\item{se_method}{Standard-error method. \code{"asymptotic"} uses the
closed-form large-sample variance for the standard non-exact Fleiss kappa
when a common number of raters is available. \code{"jackknife"} uses
leave-one-item-out jackknife inference. \code{"none"} disables
inferential quantities and may not be combined with \code{ci = TRUE} or
\code{p_value = TRUE}. When inferential quantities are requested and
\code{se_method} is left at its default, \code{multirater_kappa()}
automatically falls back to \code{"jackknife"} if asymptotic inference is
unavailable for the chosen estimator/data combination. \code{se_method}
and \code{conf_level} are validated only when \code{ci} or \code{p_value}
requests inferential output.}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
C++ backend.}

\item{verbose}{Logical; if \code{TRUE}, emit short progress messages.}

\item{...}{Unused.}

\item{x}{A \code{multirater_kappa} object.}

\item{digits}{Integer; number of decimal places for displayed values.}

\item{show_by_category}{Logical; whether to print the attached category-wise
kappa table when available.}

\item{object}{A \code{multirater_kappa} object.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{type}{Plot type: \code{"agreement_map"}, \code{"estimate"}, \code{"item_agreement"},
\code{"category_proportion"}, or \code{"by_category"}.}

\item{bins}{Integer number of bins retained for compatibility; currently
unused by the default item-agreement profile plot.}
}
\value{
A one-row data frame with class
\code{c("multirater_kappa", "agreement_result", "data.frame")}. The object
carries estimator metadata and diagnostics in attributes, and may also
attach category-wise binary-collapsed results in
\code{attr(x, "by_category")}.
}
\description{
Estimates panel-level chance-corrected agreement among multiple raters
assigning items to nominal categories.
}
\details{
\code{multirater_kappa()} returns one panel-level agreement coefficient, not a
pairwise matrix. Fleiss' kappa is the default fixed-marginal estimator:
\deqn{\kappa = \frac{\bar P - P_e}{1 - P_e},}
where \eqn{\bar P} is the mean item-level agreement and
\eqn{P_e = \sum_j p_j^2} uses the pooled marginal category proportions.

Randolph's free-marginal alternative replaces the expected agreement with
\eqn{P_e = 1 / K}. This function is for nominal categories and does not use
category ordering. Use \code{\link[=weighted_kappa]{weighted_kappa()}} for two-rater ordered-category
agreement and \code{\link[=cohen_kappa]{cohen_kappa()}} for two-rater nominal agreement.

When \code{exact = TRUE}, the exact Fleiss fixed-marginal estimate requires
the original item-by-rater rating matrix. In \pkg{matrixCorr}, closed-form
asymptotic inference is only available for the standard non-exact Fleiss
estimator with a common number of raters per item. In other settings, use
\code{se_method = "jackknife"} when inferential quantities are needed.

With \code{input = "ratings"} and \code{na_method = "available"}, the
implementation supports unbalanced item-specific numbers of raters as a
generalisation beyond the strict equal-rater Fleiss setting. The returned
diagnostics record the observed per-item rater counts.

\strong{Confidence intervals and standard errors.}
Two inference paths are implemented.

If \code{se_method = "jackknife"}, the method computes leave-one-item-out
estimates \eqn{\hat\kappa_{(-1)},\ldots,\hat\kappa_{(-m)}} over the
\eqn{m} retained items and forms
\deqn{ \bar\kappa_{(.)} = \frac{1}{m}\sum_{i=1}^m \hat\kappa_{(-i)}, }
\deqn{ \widehat{\mathrm{Var}}_{\mathrm{JK}}(\hat\kappa)
      \;=\;
      \frac{m-1}{m}\sum_{i=1}^m
      \left(\hat\kappa_{(-i)} - \bar\kappa_{(.)}\right)^2. }
The standard error is
\eqn{\widehat{\mathrm{se}}(\hat\kappa)=
\sqrt{\widehat{\mathrm{Var}}_{\mathrm{JK}}(\hat\kappa)}} and the CI is
\deqn{ \hat\kappa \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa), }
truncated to \eqn{[-1, 1]}.

If \code{se_method = "asymptotic"}, the closed-form variance is available
only for the standard non-exact Fleiss estimator
with a common number of raters per item. Let \eqn{m} be the number of items,
\eqn{n} the common number of raters, \eqn{p_j} the pooled category
proportions, and define
\deqn{ S = \sum_j p_j(1-p_j), \qquad
       C = \sum_j p_j(1-p_j)(1-2p_j). }
Then the variance estimator is
\deqn{ \widehat{\mathrm{Var}}(\hat\kappa)
      \;=\;
      \frac{2\,(S^2 - C)}{S^2\,m\,n\,(n-1)}. }
The standard error and CI are then obtained from the same Wald form
above and truncated to \eqn{[-1, 1]}. When asymptotic inference is
unavailable and inferential output is requested with the default setting,
\code{multirater_kappa()} automatically falls back to the jackknife path.
}
\examples{
raters <- data.frame(
  r1 = c("A", "A", "B", "C", "A", "B"),
  r2 = c("A", "B", "B", "C", "A", "B"),
  r3 = c("A", "A", "B", "B", "A", "C"),
  stringsAsFactors = FALSE
)

fit <- multirater_kappa(raters)
print(fit)
summary(fit)
estimate(fit)
tidy(fit)
# The default plot is an item-by-category agreement map.
# Rows are items, ordered from stronger to weaker item-level agreement.
# Columns are categories. Each tile shows how many raters assigned that
# category to that item, and darker fill means a larger share of raters
# chose that category for that item.
plot(fit)

}
\references{
Fleiss, J. L. (1971). Measuring nominal scale agreement among many raters.
\emph{Psychological Bulletin}, 76, 378-382. \doi{10.1037/h0031619}

Randolph, J. J. (2005). Free-Marginal Multirater Kappa: An Alternative to
Fleiss' Fixed-Marginal Multirater Kappa.
}
\seealso{
\code{\link[=cohen_kappa]{cohen_kappa()}} for unweighted two-rater nominal agreement;
\code{\link[=weighted_kappa]{weighted_kappa()}} for two-rater ordered-category agreement.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{summary.ccc_rm_reml}
\alias{summary.ccc_rm_reml}
\alias{print.summary.ccc_rm_reml}
\title{Summary Method for \code{ccc_rm_reml} Objects}
\usage{
\method{summary}{ccc_rm_reml}(
  object,
  digits = 4,
  ci_digits = 2,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.ccc_rm_reml}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{"ccc_rm_reml"}, as returned by \code{\link[=ccc_rm_reml]{ccc_rm_reml()}}.}

\item{digits}{Integer; number of decimal places to round CCC estimates and components.}

\item{ci_digits}{Integer; decimal places for confidence interval bounds.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives
this from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{Character string indicating whether to show confidence
intervals: \code{"yes"} shows CI information when available and \code{"no"}
suppresses it.}

\item{...}{Passed to \code{\link[base]{print.data.frame}}.}

\item{x}{An object of class \code{"summary.ccc_rm_reml"}.}
}
\value{
A data frame of class \code{"summary.ccc_rm_reml"} with columns:
\code{item1}, \code{item2}, \code{estimate}, and optionally \code{lwr},
\code{upr}, plus canonical repeated-measures counts \code{n_subjects} and
\code{n_obs}. Method-specific columns retain the scientific variance
component outputs: \code{sigma2_subject}, \code{sigma2_subject_method},
\code{sigma2_subject_time}, \code{sigma2_error}, \code{sigma2_extra},
\code{SB}, and \code{se_ccc}.
}
\description{
Produces a detailed summary of a \code{"ccc_rm_reml"} object, including
Lin's CCC estimates and associated variance component estimates per method pair.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_symmetric_matrix}
\alias{check_symmetric_matrix}
\title{Check matrix symmetry}
\usage{
check_symmetric_matrix(
  M,
  tol = .Machine$double.eps^0.5,
  arg = as.character(substitute(M))
)
}
\description{
Check matrix symmetry
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cohen_kappa.R
\name{cohen_kappa}
\alias{cohen_kappa}
\alias{summary.cohen_kappa}
\alias{print.summary.cohen_kappa}
\alias{print.cohen_kappa}
\alias{plot.cohen_kappa}
\title{Pairwise Cohen's Kappa for Nominal Ratings}
\usage{
cohen_kappa(
  data,
  y = NULL,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{summary}{cohen_kappa}(object, digits = 4, ci_digits = 3, p_digits = 4, ...)

\method{print}{summary.cohen_kappa}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{cohen_kappa}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{cohen_kappa}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{data}{In matrix mode, a matrix or data frame whose rows are
observational units and whose columns are raters or classifiers. Supported
column types are factor, ordered factor, character, logical, integer, and
numeric, all treated as nominal categories here. If the ratings are truly
ordinal and disagreements should be weighted by distance, use
\code{\link[=weighted_kappa]{weighted_kappa()}} instead. In two-vector mode, the first nominal rating
vector.}

\item{y}{Optional second nominal rating vector. When supplied, the function
returns a single Cohen's kappa estimate for \code{data} and \code{y}.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"complete"} applies listwise
deletion across retained columns before computing the matrix. In matrix
mode, \code{"pairwise"} uses pair-specific complete observations.}

\item{ci}{Logical; if \code{TRUE}, attach large-sample delta-method
confidence intervals.}

\item{p_value}{Logical; if \code{TRUE}, attach large-sample delta-method
Wald test statistics and p-values.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
matrix C++ backend. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for matrix mode.
\itemize{
\item \code{"matrix"} (default): full dense matrix.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries after thresholding.
\item \code{"edge_list"}: long-form data frame representation.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
retain entries with \code{abs(value) >= threshold}. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in sparse and
edge-list outputs.}

\item{...}{Additional theme arguments.}

\item{object}{A scalar or matrix-style \code{cohen_kappa} object.}

\item{digits}{Integer; number of decimal places for displayed values.}

\item{ci_digits}{Integer; number of decimal places for confidence limits.}

\item{p_digits}{Integer; number of decimal places for p-values.}

\item{x}{A scalar or matrix-style \code{cohen_kappa} object.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Optional plot title.}

\item{low_color}{Fill/color used for negative agreement.}

\item{high_color}{Fill/color used for positive agreement.}

\item{mid_color}{Unused placeholder for API consistency with matrix heatmaps.}

\item{value_text_size}{Text size for the estimate label.}

\item{ci_text_size}{Text size for the CI label.}

\item{show_value}{Logical; whether to print the estimate and CI labels.}
}
\value{
If \code{y} is supplied, a scalar S3 object of class \code{"cohen_kappa"}
backed by a numeric value, with attributes \code{diagnostics}, and
optionally \code{ci}, \code{inference}, and \code{conf.level}. Otherwise a
symmetric matrix-style result with estimator class \code{cohen_kappa}.
}
\description{
Computes unweighted Cohen's kappa for either a pair of nominal rating
vectors or all pairwise combinations of nominal columns in a matrix or data
frame.
}
\details{
Cohen's kappa is an agreement coefficient for two raters assigning the same
units to mutually exclusive nominal categories. For contingency-table cell
proportions \eqn{p_{ij}},
\deqn{
\kappa = \frac{p_o - p_e}{1 - p_e},
}
where \eqn{p_o = \sum_i p_{ii}} is the observed agreement and
\eqn{p_e = \sum_i p_{i+} p_{+i}} is the chance agreement implied by the
marginal category proportions.

This implementation is strictly the original unweighted nominal-scale Cohen's
kappa. If the categories are ordinal and near disagreements should count as
less severe than distant disagreements, use \code{\link[=weighted_kappa]{weighted_kappa()}} instead.

In matrix mode, columns are treated as raters/classifiers and rows as shared
observational units. All pairwise Cohen's kappas between columns are
computed. Category labels are encoded in R before dispatch to C++, using a
common label mapping across all columns so that identical labels in different
columns correspond to the same category code.

Missing values are encoded as \code{NA_integer_} before entering the C++
backend. With \code{na_method = "error"}, missing values are rejected before
computation. With \code{na_method = "complete"}, listwise deletion is
applied across retained columns. With \code{na_method = "pairwise"}, each
pair uses its own complete observations. Pairwise complete counts are stored
in \code{attr(x, "diagnostics")$n_complete}.

\strong{Confidence intervals and standard errors.}
The implementation uses the exact large-sample formula. Let \eqn{p_{ab}}
be the empirical cell proportions,
\eqn{q_a = \sum_b p_{ab}} the row margins, \eqn{r_b = \sum_a p_{ab}} the
column margins, and \eqn{D = 1 - p_e}. For each cell \eqn{(a,b)}, define
the influence contribution
\deqn{ g_{ab}
      \;=\;
      \frac{\mathbf{1}(a=b)\,D + (p_o - 1)\,(r_a + q_b)}{D^2}. }
The variance estimator used by the code is
\deqn{ \widehat{\mathrm{Var}}(\hat\kappa)
      \;=\;
      \frac{1}{n}\left(
      \sum_{a,b} p_{ab} g_{ab}^2 -
      \Big(\sum_{a,b} p_{ab} g_{ab}\Big)^2
      \right), }
with \eqn{n} the number of complete paired ratings and
\eqn{\widehat{\mathrm{se}}(\hat\kappa)=\sqrt{\widehat{\mathrm{Var}}(\hat\kappa)}}.
The confidence interval is the Wald interval
\deqn{ \hat\kappa \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa), }
truncated to \eqn{[-1, 1]} in the returned result.
}
\examples{
x <- factor(c("A", "A", "B", "B", "A", "B"))
y <- factor(c("A", "B", "B", "B", "A", "A"))
cohen_kappa(x, y)

raters <- data.frame(
  r1 = factor(c("low", "low", "high", "high", "mid")),
  r2 = factor(c("low", "mid", "high", "high", "mid")),
  r3 = c("low", "low", "high", "mid", "mid")
)
ck <- cohen_kappa(raters)
print(ck)
summary(ck)
estimate(ck)
tidy(ck)
plot(ck)

}
\references{
Cohen, J. (1960). A coefficient of agreement for nominal scales.
\emph{Educational and Psychological Measurement}, 20(1), 37-46.
\doi{10.1177/001316446002000104}
}
\seealso{
\code{\link[=weighted_kappa]{weighted_kappa()}} for two-rater ordered-category agreement with
distance-sensitive disagreement weights; \code{\link[=multirater_kappa]{multirater_kappa()}} for
panel-level nominal agreement among three or more raters.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ba_rm.R
\name{ba_rm}
\alias{ba_rm}
\alias{print.ba_repeated}
\alias{print.ba_repeated_matrix}
\alias{plot.ba_repeated}
\alias{plot.ba_repeated_matrix}
\title{Bland-Altman for repeated measurements}
\usage{
ba_rm(
  data = NULL,
  response,
  subject,
  method,
  time,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE,
  loa_multiplier = 1.96,
  include_slope = FALSE,
  use_ar1 = FALSE,
  ar1_rho = NA_real_,
  max_iter = 200L,
  tol = 1e-06
)

\method{print}{ba_repeated}(
  x,
  digits = 3,
  ci_digits = 3,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{ba_repeated_matrix}(
  x,
  digits = 3,
  ci_digits = 3,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  style = c("pairs", "matrices"),
  ...
)

\method{plot}{ba_repeated}(
  x,
  title = "Bland-Altman (repeated measurements)",
  subtitle = NULL,
  point_alpha = 0.7,
  point_size = 2.2,
  line_size = 0.8,
  shade_ci = TRUE,
  shade_alpha = 0.08,
  smoother = c("none", "loess", "lm"),
  symmetrize_y = TRUE,
  show_points = TRUE,
  show_value = TRUE,
  ...
)

\method{plot}{ba_repeated_matrix}(
  x,
  pairs = NULL,
  against = NULL,
  facet_scales = c("free_y", "fixed"),
  title = "Bland-Altman (repeated, pairwise)",
  point_alpha = 0.6,
  point_size = 1.8,
  line_size = 0.7,
  shade_ci = TRUE,
  shade_alpha = 0.08,
  smoother = c("none", "loess", "lm"),
  show_points = TRUE,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{data}{Optional data frame-like object. If supplied, \code{response},
\code{subject}, \code{method}, and \code{time} may be column names. Objects not already
inheriting from \code{data.frame} are first coerced with \code{as.data.frame()}.}

\item{response}{Numeric response vector, or a single character string naming
the response column in \code{data}.}

\item{subject}{Subject identifier vector (integer, numeric, or factor), or a
single character string naming the subject column in \code{data}.}

\item{method}{Method label vector (character, factor, integer, or numeric), or
a single character string naming the method column in \code{data}. At least two
distinct method levels are required.}

\item{time}{Replicate/time index vector (integer or numeric), or a single
character string naming the time column in \code{data}. Values are coerced to
integer before pairing and before AR(1) contiguity checks.}

\item{conf_level}{Confidence level for Wald confidence intervals for the
reported bias and both LoA endpoints. Must lie in \verb{(0, 1)}. Default \code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{verbose}{Logical. If \code{TRUE}, print progress for the pairwise
\eqn{\ge 3}-method path. It has no material effect in the exactly-two-method
path.}

\item{loa_multiplier}{Positive scalar giving the SD multiplier used to form
the limits of agreement. Default is \code{1.96}. In the exported \code{ba_rm()}
interface this default is fixed and does not depend on \code{conf_level}.}

\item{include_slope}{Logical. If \code{TRUE}, the model includes the paired mean as
a fixed effect and estimates a proportional-bias slope. The reported BA
centre remains the fitted mean difference at the centred reference paired
mean used internally by the backend; the returned LoA remain horizontal
bands and are not regression-adjusted curves.}

\item{use_ar1}{Logical. If \code{TRUE}, request an AR(1) residual structure within
subject over contiguous integer time blocks.}

\item{ar1_rho}{Optional AR(1) parameter. Must satisfy \code{abs(ar1_rho) < 0.999}
when supplied. If \code{NA} and \code{use_ar1 = TRUE}, the backend estimates \code{rho}
separately for each analysed pair.}

\item{max_iter}{Maximum number of EM/GLS iterations used by the backend.}

\item{tol}{Convergence tolerance for the backend EM/GLS iterations.}

\item{x}{A \code{"ba_repeated_matrix"} object.}

\item{digits}{Number of digits for estimates (default 3).}

\item{ci_digits}{Number of digits for CI bounds (default 3).}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional theme adjustments passed to \code{ggplot2::theme(...)}
(e.g., \code{plot.title.position = "plot"}, \code{axis.title.x = element_text(size=11)}).}

\item{style}{Show as pairs or matrix format?}

\item{title}{Plot title (character scalar). Defaults to
\code{"Bland-Altman (repeated measurements)"} for two methods and
\code{"Bland-Altman (repeated, pairwise)"} for the faceted matrix plot.}

\item{subtitle}{Optional subtitle (character scalar). If \code{NULL}, a compact
summary is shown using the fitted object.}

\item{point_alpha}{Numeric in \verb{[0, 1]}. Transparency for scatter points
drawn at (pair mean, pair difference) when point data are available.
Passed to \code{ggplot2::geom_point(alpha = ...)}. Default \code{0.7}.}

\item{point_size}{Positive numeric. Size of scatter points; passed to
\code{ggplot2::geom_point(size = ...)}. Default \code{2.2}.}

\item{line_size}{Positive numeric. Line width for horizontal bands
(bias and both LoA) and, when requested, the proportional-bias line.
Passed to \code{ggplot2::geom_hline(linewidth = ...)} (and \code{geom_abline}).
Default \code{0.8}.}

\item{shade_ci}{Logical. If \code{TRUE} and confidence intervals are available in
the object (\code{CI.lines} for two methods; \verb{*_ci_*} matrices for the pairwise
case), semi-transparent rectangles are drawn to indicate CI bands for the
bias and each LoA. If \code{FALSE}, dashed horizontal CI lines are drawn instead.
Has no effect if CIs are not present. Default \code{TRUE}.}

\item{shade_alpha}{Numeric in \verb{[0, 1]}. Opacity of the CI shading
rectangles when \code{shade_ci = TRUE}. Passed to \code{ggplot2::annotate("rect", alpha = ...)}.
Default \code{0.08}.}

\item{smoother}{One of \code{"none"}, \code{"loess"}, or \code{"lm"}. Adds an overlaid trend
for differences vs means when points are drawn, to visualise proportional
bias. \code{"lm"} fits a straight line with no SE ribbon; \code{"loess"} draws a
locally-smoothed curve (span 0.9) with no SE ribbon; \code{"none"} draws no
smoother. Ignored if \code{show_points = FALSE} or if no point data are available.}

\item{symmetrize_y}{Logical (two-method plot only). If \code{TRUE}, the y-axis is
centred at the estimated bias and expanded symmetrically to cover all
elements used to compute the range (bands, CIs, and points if shown).
Default \code{TRUE}.}

\item{show_points}{Logical. If \code{TRUE}, per-pair points are drawn for the
exactly-two-method path from the stored \code{means} and \code{diffs} vectors.
Pairwise matrix plots draw points reconstructed from the canonical
long-form columns stored by \code{ba_rm()}. If \code{FALSE} or if point data are
unavailable, only the bands (and optional CI indicators) are drawn.
Default \code{TRUE}.}

\item{show_value}{Logical; included for a consistent plotting interface.
Repeated-measures Bland-Altman plots do not overlay numeric cell values,
so this argument currently has no effect.}

\item{pairs}{(Faceted pairwise plot only.) Optional character vector of
labels specifying which method contrasts to display. Labels must match the
"row - column" convention used by \code{print()}/\code{summary()} (e.g., \code{"B - A"}).
Defaults to all upper-triangle pairs.}

\item{against}{(Faceted pairwise plot only.) Optional single method name.
If supplied, facets are restricted to contrasts of the chosen method
against all others. Ignored when \code{pairs} is provided.}

\item{facet_scales}{(Faceted pairwise plot only.) Either \code{"free_y"} (default)
to allow each facet its own y-axis limits, or \code{"fixed"} for a common scale
across facets. Passed to \code{ggplot2::facet_wrap(scales = ...)}.}
}
\value{
Either a \code{"ba_repeated"} object (exactly two methods) or a
\code{"ba_repeated_matrix"} object (\eqn{\ge 3} methods).

\strong{If exactly two methods are supplied}, the returned
\code{"ba_repeated"} object is a list with components:
\itemize{
\item \code{means}: numeric vector of paired means
\eqn{(y_1 + y_2)/2} used for plotting helpers.
\item \code{diffs}: numeric vector of paired differences
\eqn{y_2 - y_1} used for plotting helpers.
\item \code{n_obs}: integer number of complete subject-time pairs used.
\item \code{based.on}: compatibility alias for \code{n_obs}.
\item \code{mean.diffs}: scalar model-based BA centre. When
\code{include_slope = FALSE}, this is the fitted intercept of the paired-
difference model. When \code{include_slope = TRUE}, this is the fitted mean
difference at the centred reference paired mean used internally by the
backend.
\item \code{lower.limit}, \code{upper.limit}: scalar limits of agreement,
computed as
\eqn{\mu_0 \pm \texttt{loa\_multiplier}\sqrt{\sigma_u^2 + \sigma_e^2}}.
\item \code{lines}: named numeric vector with entries \code{lower}, \code{mean}, and
\code{upper}.
\item \code{CI.lines}: named numeric vector containing Wald confidence
interval bounds for the bias and both LoA endpoints:
\code{mean.diff.ci.lower}, \code{mean.diff.ci.upper},
\code{lower.limit.ci.lower}, \code{lower.limit.ci.upper},
\code{upper.limit.ci.lower}, \code{upper.limit.ci.upper}.
\item \code{loa_multiplier}: scalar LoA multiplier actually used.
\item \code{critical.diff}: scalar LoA half-width
\eqn{\texttt{loa\_multiplier} \times \texttt{sd\_loa}}.
\item \code{include_slope}: logical, copied from the call.
\item \code{beta_slope}: proportional-bias slope on the original paired-mean
scale when \code{include_slope = TRUE}; otherwise \code{NA}.
\item \code{sigma2_subject}: estimated variance of the subject-level random
intercept on paired differences.
\item \code{sigma2_resid}: estimated residual variance on paired differences.
\item \code{use_ar1}: logical, copied from the call.
\item \code{residual_model}: either \code{"ar1"} or \code{"iid"}, indicating the final
residual structure actually used.
\item \code{ar1_rho}: AR(1) correlation actually used in the final fit when
\code{residual_model == "ar1"}; otherwise \code{NA}.
\item \code{ar1_estimated}: logical indicating whether \code{ar1_rho} was
estimated internally (\code{TRUE}) or supplied by the user (\code{FALSE}) when the
final residual model is AR(1); otherwise \code{NA}.
}

The confidence level is stored as \code{attr(x, "conf.level")}.

\strong{If \eqn{\ge 3} methods are supplied}, the returned
\code{"ba_repeated_matrix"} object is a list with components:
\itemize{
\item \code{bias}: numeric \eqn{m \times m} matrix of model-based BA centres.
For indices \eqn{(j, k)} with \eqn{j < k}, \code{bias[j, k]} estimates
\eqn{\mathrm{method}_k - \mathrm{method}_j}. Thus the matrix orientation is
\strong{column minus row}, not row minus column. The diagonal is \code{NA}.
\item \code{sd_loa}: numeric \eqn{m \times m} matrix of LoA SDs,
\eqn{\sqrt{\sigma_u^2 + \sigma_e^2}}. This matrix is symmetric.
\item \code{loa_lower}, \code{loa_upper}: numeric \eqn{m \times m} matrices
of LoA endpoints corresponding to \code{bias}. These satisfy
\eqn{\texttt{loa\_lower}[j,k] = -\texttt{loa\_upper}[k,j]} and
\eqn{\texttt{loa\_upper}[j,k] = -\texttt{loa\_lower}[k,j]}.
\item \code{width}: numeric \eqn{m \times m} matrix of LoA widths,
\code{loa_upper - loa_lower}. This matrix is symmetric.
\item \code{n}: integer \eqn{m \times m} matrix giving the number of complete
subject-time pairs used for each analysed contrast. Pairs with fewer than
two complete matches are left as \code{NA} in the estimate matrices.
\item \code{mean_ci_low}, \code{mean_ci_high}: numeric \eqn{m \times m}
matrices of Wald confidence interval bounds for \code{bias}.
\item \code{loa_lower_ci_low}, \code{loa_lower_ci_high}: numeric
\eqn{m \times m} matrices of Wald confidence interval bounds for the lower
LoA.
\item \code{loa_upper_ci_low}, \code{loa_upper_ci_high}: numeric
\eqn{m \times m} matrices of Wald confidence interval bounds for the upper
LoA.
\item \code{slope}: optional numeric \eqn{m \times m} matrix of
proportional-bias slopes on the original paired-mean scale when
\code{include_slope = TRUE}; otherwise \code{NULL}. This matrix is antisymmetric in
sign because each fitted contrast is reversed across the transpose.
\item \code{methods}: character vector of method levels defining matrix row
and column order.
\item \code{loa_multiplier}: scalar LoA multiplier actually used.
\item \code{conf_level}: scalar confidence level used for the reported Wald
intervals.
\item \code{use_ar1}: logical, copied from the call.
\item \code{ar1_rho}: scalar equal to the user-supplied common \code{ar1_rho}
when \code{use_ar1 = TRUE} and a value was supplied; otherwise \code{NA}. This field
does \emph{not} store the per-pair estimated AR(1) parameters.
\item \code{residual_model}: character \eqn{m \times m} matrix whose entries
are \code{"ar1"}, \code{"iid"}, or \code{NA}, indicating the final residual structure used
for each pair.
\item \code{sigma2_subject}: numeric \eqn{m \times m} matrix of estimated
subject-level random-intercept variances.
\item \code{sigma2_resid}: numeric \eqn{m \times m} matrix of estimated
residual variances.
\item \code{ar1_rho_pair}: optional numeric \eqn{m \times m} matrix giving
the AR(1) correlation actually used for each pair when the final residual
model is \code{"ar1"}; otherwise \code{NA} for that entry. Present only when
\code{use_ar1 = TRUE}.
\item \code{ar1_estimated}: optional logical \eqn{m \times m} matrix
indicating whether the pair-specific \code{ar1_rho_pair} was estimated internally
(\code{TRUE}) or supplied by the user (\code{FALSE}) for entries whose final residual
model is \code{"ar1"}; otherwise \code{NA}. Present only when \code{use_ar1 = TRUE}.
\item \code{data_long}: canonical long-form data frame with columns
\code{.response}, \code{.subject}, \code{.method}, and \code{.time}
retained for pairwise plotting helpers.
\item \code{mapping}: named list mapping \code{response}, \code{subject},
\code{method}, and \code{time} to the canonical stored columns in
\code{data_long}.
}
}
\description{
Repeated-measures Bland-Altman (BA) analysis for method comparison based on a
mixed-effects model fitted to \strong{subject-time matched paired differences}.
The fitted model includes a subject-specific random intercept and, optionally,
an AR(1) residual correlation structure within subject.

The function accepts either exactly two methods or \eqn{\ge 3} methods.
With exactly two methods it returns a single fitted BA object. With
\eqn{\ge 3} methods it fits the same model to every unordered method pair and
returns pairwise matrices of results.

\strong{Required variables}
\itemize{
\item \code{response}: numeric measurements.
\item \code{subject}: subject identifier.
\item \code{method}: method label with at least two distinct levels.
\item \code{time}: replicate/time key used to form within-subject pairs.
}

For any analysed pair of methods, only records where both methods are present
for the same \code{subject} and integer-coerced \code{time} contribute to the
fit. Rows with missing values in any required field are excluded for that
analysed pair.
}
\details{
For a selected pair of methods \eqn{(a, b)}, the backend first forms complete
within-subject pairs at matched \code{subject} and integer-coerced
\code{time}. Let
\deqn{
d_{it} = y_{itb} - y_{ita},
\qquad
m_{it} = \frac{y_{ita} + y_{itb}}{2},
}
where \eqn{d_{it}} is the paired difference and \eqn{m_{it}} is the paired
mean for subject \eqn{i} at time/replicate \eqn{t}. Only complete
subject-time matches contribute to that pairwise fit.

If multiple rows are present for the same \code{subject}-\code{time}-\code{method}
combination within an analysed pair, the backend keeps the last encountered
value for that combination when forming the pair. The function therefore
implicitly assumes at most one observation per \code{subject}-\code{time}-\code{method}
cell for each analysed contrast.

The fitted model for each analysed pair is
\deqn{
d_{it} = \beta_0 + \beta_1 x_{it} + u_i + \varepsilon_{it},
}
where \eqn{x_{it} = m_{it}} if \code{include_slope = TRUE} and the term is omitted
otherwise; \eqn{u_i \sim \mathcal{N}(0, \sigma_u^2)} is a subject-specific
random intercept; and the within-subject residual vector satisfies
\eqn{\mathrm{Cov}(\varepsilon_i) = \sigma_e^2 R_i}.

When \code{use_ar1 = FALSE}, \eqn{R_i = I}. When \code{use_ar1 = TRUE}, the backend
works with the residual \emph{precision} matrix \eqn{C_i = R_i^{-1}} over
contiguous time blocks within subject and uses \eqn{\sigma_e^2 C_i^{-1}} as
the residual covariance.

\subsection{AR(1) residual structure}{
Within each subject, paired observations are ordered by integer-coerced
\code{time}. AR(1) correlation is applied only over strictly contiguous runs
satisfying \eqn{t_{k+1} = t_k + 1}. Gaps break the run. Negative times, and
any isolated positions not belonging to a contiguous run, are treated as
independent singletons.

For a contiguous run of length \eqn{L} and correlation parameter \eqn{\rho},
the block precision matrix is
\deqn{
C
=
\frac{1}{1-\rho^2}
\begin{bmatrix}
1      & -\rho &        &        &  \\
-\rho  & 1+\rho^2 & -\rho &      &  \\
        & \ddots & \ddots & \ddots & \\
        &        & -\rho & 1+\rho^2 & -\rho \\
        &        &       & -\rho & 1
\end{bmatrix},
}
with a very small ridge added to the diagonal for numerical stability.

If \code{use_ar1 = TRUE} and \code{ar1_rho} is supplied, that value is used after
validation and clipping to the admissible numerical range handled by the
backend.

If \code{use_ar1 = TRUE} and \code{ar1_rho = NA}, the backend estimates \code{rho}
separately for each analysed pair by:
\enumerate{
\item fitting the corresponding iid model;
\item computing a moments-based lag-1 estimate from detrended residuals
within contiguous blocks, used only as a seed; and
\item refining that seed by a short profile search over \code{rho} using the
profiled REML log-likelihood.
}

In the exported \code{ba_rm()} wrapper, if an AR(1) fit for a given analysed pair
fails specifically because the backend EM/GLS routine did not converge to
admissible finite variance-component estimates, the wrapper retries that pair
with iid residuals. If the iid refit succeeds, the final reported residual
model for that pair is \code{"iid"} and a warning is issued. Other AR(1) failures
are not simplified and are propagated as errors.
}

\subsection{Internal centring and scaling for the proportional-bias slope}{
When \code{include_slope = TRUE}, the paired mean regressor is centred and scaled
internally before fitting. Let \eqn{\bar m} be the mean of the observed paired
means. The backend chooses a scaling denominator from:
\itemize{
\item the sample SD;
\item the IQR-based scale \eqn{\mathrm{IQR}(m)/1.349};
\item the MAD-based scale \eqn{1.4826\,\mathrm{MAD}(m)}.
}
It uses the first of these that is not judged near-zero relative to the
largest finite positive candidate scale, under a threshold proportional to
\eqn{\sqrt{\epsilon_{\mathrm{mach}}}}. If all candidate scales are treated as
near-zero, the fit stops with an error because the proportional-bias slope is
not estimable on the observed paired-mean scale.

The returned \code{beta_slope} is back-transformed to the original paired-mean
scale. The returned BA centre is the fitted mean difference at the centred
reference paired mean \eqn{\bar m}, not the original-scale intercept
coefficient.
}

\subsection{Estimation}{
The backend uses a stabilised EM/GLS scheme.

Conditional on current variance components, the fixed effects are updated by
GLS using the marginal precision of the paired differences after integrating
out the random subject intercept. The resulting fixed-effect covariance used
in the confidence-interval calculations is the GLS covariance
\deqn{
\mathrm{Var}(\hat\beta \mid \hat\theta)
=
\left( \sum_i X_i^\top V_i^{-1} X_i \right)^{-1}.
}

Given updated fixed effects, the variance components are refreshed by EM using
the conditional moments of the subject random intercept and the residual
quadratic forms. Variance updates are ratio-damped and clipped to admissible
ranges for numerical stability.
}

\subsection{Reported BA centre and limits of agreement}{
The reported BA centre is always model-based.

When \code{include_slope = FALSE}, it is the fitted intercept of the paired-
difference mixed model.

When \code{include_slope = TRUE}, it is the fitted mean difference at the centred
reference paired mean used internally by the backend.

The reported limits of agreement are
\deqn{
\mu_0 \pm \texttt{loa\_multiplier}\sqrt{\sigma_u^2 + \sigma_e^2},
}
where \eqn{\mu_0} is the reported model-based BA centre. These LoA are for a
single new paired difference from a random subject under the fitted model.

Under the implemented parameterisation, AR(1) correlation affects the
off-diagonal within-subject covariance structure and therefore the estimation
of the model parameters and their uncertainty, but not the marginal variance
of a single paired difference. Consequently \code{rho} does not appear explicitly
in the LoA point-estimate formula.
}

\subsection{Confidence intervals}{
The backend returns Wald confidence intervals for the reported BA centre and
for both LoA endpoints.

These intervals combine:
\itemize{
\item the conditional GLS uncertainty in the fixed effects at the fitted
covariance parameters; and
\item a delta-method propagation of covariance-parameter uncertainty from
the observed information matrix of the profiled REML log-likelihood.
}

The covariance-parameter vector is profiled on transformed scales:
log-variances for \eqn{\sigma_u^2} and \eqn{\sigma_e^2}, and, when \code{rho} is
estimated internally under AR(1), a transformed correlation parameter
mapped back by \eqn{\rho = 0.95\tanh(z)}.

Numerical central finite differences are used to approximate both the
observed Hessian of the profiled REML log-likelihood and the gradients of the
reported derived quantities. The resulting variances are combined and the
final intervals are formed with the normal quantile corresponding to
\code{conf_level}.
}

\subsection{Exactly two methods versus \eqn{\ge 3} methods}{
With exactly two methods, at least two complete subject-time pairs are
required; otherwise the function errors.

With \eqn{\ge 3} methods, the function analyses every unordered pair of method
levels. For a given pair with fewer than two complete subject-time matches,
that contrast is skipped and the corresponding matrix entries remain \code{NA}.

For a fitted contrast between methods in matrix positions \eqn{(j, k)} with
\eqn{j < k}, the stored orientation is:
\deqn{
\texttt{bias}[j,k] \approx \mathrm{method}_k - \mathrm{method}_j.
}
Hence the transposed entry changes sign, while \code{sd_loa} and \code{width} are
symmetric.
}

\subsection{Identifiability and safeguards}{
Separate estimation of the residual and subject-level variance components
requires sufficient complete within-subject replication after pairing. If the
paired data are not adequate to separate these components, the fit stops with
an identifiability error.

If the model is conceptually estimable but no finite positive pooled
within-subject variance can be formed during initialisation, the backend uses
\eqn{0.5 \times v_{\mathrm{ref}}} only as a temporary positive starting value
for the EM routine and records a warning string in the backend output. The
exported wrapper does not otherwise modify the final estimates.

If the EM/GLS routine fails to reach admissible finite variance-component
estimates, the backend throws an explicit convergence error rather than
returning fallback estimates.
}
}
\examples{
# -------- Simulate repeated-measures data --------
set.seed(1)

# design (no AR)
# subjects
S   <- 30L
# replicates per subject
Tm  <- 15L
subj <- rep(seq_len(S), each = Tm)
time <- rep(seq_len(Tm), times = S)

# subject signal centered at 0 so BA "bias" won't be driven by the mean level
mu_s  <- rnorm(S, mean = 0, sd = 8)
# constant within subject across replicates
true  <- mu_s[subj]

# common noise (no AR, i.i.d.)
sd_e <- 2
e0   <- rnorm(length(true), 0, sd_e)

# --- Methods ---
# M1: signal + noise
y1 <- true + e0

# M2: same precision as M1; here identical so M3 can be
#     almost perfectly the inverse of both M1 and M2
y2 <- y1 + rnorm(length(true), 0, 0.01)

# M3: perfect inverse of M1 and M2
y3 <- -y1   # = -(true + e0)

# M4: unrelated to all others (pure noise, different scale)
y4 <- rnorm(length(true), 3, 6)

data <- rbind(
  data.frame(y = y1, subject = subj, method = "M1", time = time),
  data.frame(y = y2, subject = subj, method = "M2", time = time),
  data.frame(y = y3, subject = subj, method = "M3", time = time),
  data.frame(y = y4, subject = subj, method = "M4", time = time)
)
data$method <- factor(data$method, levels = c("M1","M2","M3","M4"))

# quick sanity checks
with(data, {
  Y <- split(y, method)
  round(cor(cbind(M1 = Y$M1, M2 = Y$M2, M3 = Y$M3, M4 = Y$M4)), 3)
})

# Run BA (no AR)
ba4 <- ba_rm(
  data = data,
  response = "y", subject = "subject", method = "method", time = "time",
  loa_multiplier = 1.96, conf_level = 0.95,
  include_slope = FALSE, use_ar1 = FALSE
)
summary(ba4)
estimate(ba4)
tidy(ba4)
confint(ba4)
plot(ba4)

# -------- Simulate repeated-measures with AR(1) data --------
set.seed(123)
S <- 40L                      # subjects
Tm <- 50L                     # replicates per subject
methods <- c("A","B","C")     # N = 3 methods
rho <- 0.4                    # AR(1) within-subject across time

ar1_sim <- function(n, rho, sd = 1) {
  z <- rnorm(n)
  e <- numeric(n)
  e[1] <- z[1] * sd
  if (n > 1) for (t in 2:n) e[t] <- rho * e[t-1] + sqrt(1 - rho^2) * z[t] * sd
  e
}

# Subject baseline + time trend (latent "true" signal)
subj <- rep(seq_len(S), each = Tm)
time <- rep(seq_len(Tm), times = S)
# subject effects
mu_s  <- rnorm(S, 50, 7)
trend <- rep(seq_len(Tm) - mean(seq_len(Tm)), times = S) * 0.8
true  <- mu_s[subj] + trend

# Method-specific biases (B has +1.5 constant; C has slight proportional bias)
bias  <- c(A = 0, B = 1.5, C = -0.5)
# proportional component on "true"
prop  <- c(A = 0.00, B = 0.00, C = 0.10)

# Build long data: for each method, add AR(1) noise within subject over time
make_method <- function(meth, sd = 3) {
  e <- unlist(lapply(split(seq_along(time), subj),
                     function(ix) ar1_sim(length(ix), rho, sd)))
  y <- true * (1 + prop[meth]) + bias[meth] + e
  data.frame(y = y, subject = subj, method = meth, time = time,
             check.names = FALSE)
}

data <- do.call(rbind, lapply(methods, make_method))
data$method <- factor(data$method, levels = methods)

# -------- Repeated BA (pairwise matrix) ---------------------
baN <- ba_rm(
  response = data$y, subject = data$subject, method = data$method, time = data$time,
  loa_multiplier = 1.96, conf_level = 0.95,
  include_slope = FALSE,         # estimate proportional bias per pair
  use_ar1 = TRUE, ar1_rho = rho
)

# Matrices (row - column orientation)
print(baN)
summary(baN)
tidy(baN)

# Faceted BA scatter by pair
plot(baN, smoother = "lm", facet_scales = "free_y")

# -------- Two-method AR(1) path (A vs B only) ------------------------------
data_AB <- subset(data, method \%in\% c("A","B"))
baAB <- ba_rm(
  response = data_AB$y, subject = data_AB$subject,
  method = droplevels(data_AB$method), time = data_AB$time,
  include_slope = FALSE, use_ar1 = TRUE, ar1_rho = 0.4
)
print(baAB)
plot(baAB)

}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_same_length}
\alias{check_same_length}
\title{Check that two vectors have the same length}
\usage{
check_same_length(
  x,
  y,
  arg_x = as.character(substitute(x)),
  arg_y = as.character(substitute(y))
)
}
\description{
Check that two vectors have the same length
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{.vc_message}
\alias{.vc_message}
\title{.vc_message}
\usage{
.vc_message(
  ans,
  label,
  nm,
  nt,
  conf_level,
  digits = 4,
  use_message = TRUE,
  extra_label = NULL,
  ar = c("none", "ar1"),
  ar_rho = NA_real_,
  engine_name = "ccc_rm_reml",
  se_label = "SE(CCC)"
)
}
\description{
Display variance component estimation details to the console.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{estimate_rho}
\alias{estimate_rho}
\title{estimate_rho}
\usage{
estimate_rho(
  Xr,
  yr,
  subject,
  method_int,
  time_int,
  Laux,
  Z,
  rho_lo = -0.95,
  rho_hi = 0.95,
  max_iter = 100,
  tol = 1e-06,
  conf_level = 0.95,
  ci_mode_int,
  include_subj_method = TRUE,
  include_subj_time = TRUE,
  sb_zero_tol = 1e-10,
  eval_single_visit = FALSE,
  time_weights = NULL,
  metric_mode = 0L
)
}
\description{
Estimate AR(1) correlation parameter rho by optimizing REML log-likelihood.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_result_s3.R
\name{$.summary.corr_result}
\alias{$.summary.corr_result}
\title{Summary Accessor for Correlation Summaries}
\usage{
\method{$}{summary.corr_result}(x, name)
}
\arguments{
\item{x}{A \code{summary.corr_result} object.}

\item{name}{A column name or summary metadata key.}
}
\value{
A summary column (if present) or summary metadata entry.
}
\description{
Summary Accessor for Correlation Summaries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmcorr.R
\name{print.rmcorr}
\alias{print.rmcorr}
\alias{summary.rmcorr}
\alias{print.summary.rmcorr}
\alias{plot.rmcorr}
\title{Methods for Pairwise rmcorr Objects}
\usage{
\method{print}{rmcorr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{rmcorr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.rmcorr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{rmcorr}(
  x,
  title = NULL,
  point_alpha = 0.8,
  line_width = 0.8,
  show_legend = FALSE,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"rmcorr"} or \code{"summary.rmcorr"}.}

\item{digits}{Number of significant digits to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to downstream methods. For
\code{plot.rmcorr()}, these are passed to \code{ggplot2::theme()}.}

\item{object}{An object of class \code{"rmcorr"}.}

\item{title}{Optional plot title for \code{plot.rmcorr()}. Defaults to a
title containing the estimated repeated-measures correlation.}

\item{point_alpha}{Alpha transparency for scatterplot points.}

\item{line_width}{Line width for subject-specific fitted lines.}

\item{show_legend}{Logical; if \code{TRUE}, shows the subject legend in the
pairwise scatterplot.}

\item{show_value}{Logical; included for a consistent plotting interface.
Pairwise repeated-measures plots do not overlay numeric cell values, so this
argument currently has no effect.}
}
\description{
Print, summarize, and plot methods for pairwise
repeated-measures correlation objects of class \code{"rmcorr"} and
\code{"summary.rmcorr"}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_result_s3.R
\name{print.summary.corr_result}
\alias{print.summary.corr_result}
\title{Print Standardized Correlation Summaries}
\usage{
\method{print}{summary.corr_result}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{summary.corr_result} object.}

\item{digits}{Number of digits for numeric values.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of head/tail rows when preview is truncated.}

\item{max_vars}{Optional maximum number of visible columns in preview.}

\item{width}{Optional output width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print Standardized Correlation Summaries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcorr.R
\name{pcorr}
\alias{pcorr}
\alias{print.partial_corr}
\alias{summary.partial_corr}
\alias{print.summary.partial_corr}
\alias{plot.partial_corr}
\title{Partial correlation matrix (sample / ridge / OAS / graphical lasso)}
\usage{
pcorr(
  data,
  method = c("sample", "oas", "ridge", "glasso"),
  na_method = c("error", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  return_cov_precision = FALSE,
  return_details = FALSE,
  return_p_value = FALSE,
  lambda = 0.001,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

\method{print}{partial_corr}(
  x,
  digits = 3,
  show_method = TRUE,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{partial_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.partial_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{partial_corr}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  mask_diag = TRUE,
  reorder = FALSE,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or data frame with at least two numeric columns.
Non-numeric columns are ignored.}

\item{method}{Character; one of \code{"sample"}, \code{"oas"},
\code{"ridge"}, or \code{"glasso"}. Default \code{"sample"}.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"complete"} performs listwise deletion once across the retained
numeric columns and then computes the estimator on the common complete
sample. Pairwise deletion is not supported for partial correlations.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach Fisher-z
confidence intervals for the off-diagonal partial correlations. This
option is available only for the classical \code{method = "sample"}
estimator in the ordinary low-dimensional setting.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{return_cov_precision}{Logical; if \code{TRUE}, also return the
covariance (\code{cov}) and precision (\code{precision}) matrices used to
form the partial correlations. Requires \code{return_details = TRUE}.
Default to \code{FALSE}.}

\item{return_details}{Logical; if \code{TRUE}, return the rich
\code{"partial_corr"} list containing \code{pcor}, diagnostics, method
metadata, and any requested covariance, precision, p-value, or CI
components. If \code{FALSE} (default), \code{output = "matrix"} returns the
standard matrixCorr estimate matrix, consistent with other correlation
methods.}

\item{return_p_value}{Logical; if \code{TRUE}, compute two-sided p-values
for testing whether each sample partial correlation is zero. With the
default matrix result, p-values are stored in the inference metadata and
exposed by \code{tidy()}. With \code{return_details = TRUE}, they are also
returned as the \code{p_value} matrix. This option is available only for
\code{method = "sample"} and requires \eqn{n > p}. Default to
\code{FALSE}.}

\item{lambda}{Numeric \eqn{\ge 0}; regularisation strength. For
\code{method = "ridge"}, this is the penalty added to the covariance
diagonal. For \code{method = "glasso"}, this is the off-diagonal
precision-matrix \eqn{\ell_1} penalty. Ignored otherwise. Default
\code{1e-3}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{x}{An object of class \code{partial_corr}.}

\item{digits}{Integer; number of decimal places for display (default 3).}

\item{show_method}{Logical; print a one-line header with \code{method}
(and \code{lambda}/\code{rho} if available). Default \code{TRUE}.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other
\pkg{ggplot2} layers.}

\item{object}{An object of class \code{partial_corr}.}

\item{title}{Plot title. By default, constructed from the estimator in
\code{x$method}.}

\item{low_color}{Colour for low (negative) values. Default
\code{"indianred1"}.}

\item{high_color}{Colour for high (positive) values. Default
\code{"steelblue1"}.}

\item{mid_color}{Colour for zero. Default \code{"white"}.}

\item{value_text_size}{Font size for cell labels. Default \code{4}.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{mask_diag}{Logical; if \code{TRUE}, the diagonal is masked
(set to \code{NA}) and not labelled. Default \code{TRUE}.}

\item{reorder}{Logical; if \code{TRUE}, variables are reordered by
hierarchical clustering of \eqn{1 - |pcor|}. Default \code{FALSE}.}
}
\value{
With the default \code{return_details = FALSE}, a standard
matrixCorr correlation result: a dense \code{corr_matrix}, sparse matrix,
or \code{corr_edge_list}, depending on \code{output}. With
\code{return_details = TRUE}, an object of class \code{"partial_corr"} (a
list) with elements:
\itemize{
\item \code{pcor}: \eqn{p \times p} partial correlation matrix.
\item \code{cov} (if requested): covariance matrix used.
\item \code{precision} (if requested): precision matrix \eqn{\Theta}.
\item \code{p_value} (if requested): matrix of two-sided p-values for
the sample partial correlations.
\item \code{ci} (if requested): a list with elements \code{est},
\code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and \code{ci.method}.
\item \code{diagnostics}: metadata used for inference, including the
effective complete-case sample size and number of conditioned variables.
\item \code{method}: the estimator used (\code{"oas"}, \code{"ridge"},
\code{"sample"}, or \code{"glasso"}).
\item \code{lambda}: ridge or graphical-lasso penalty
(or \code{NA_real_}).
\item \code{rho}: OAS shrinkage weight in \eqn{[0,1]} (or \code{NA_real_}).
\item \code{jitter}: diagonal jitter added (if any) to ensure positive
definiteness.
}

Invisibly returns \code{x}.

A compact summary object of class \code{summary.partial_corr}.

A \code{ggplot} object.
}
\description{
Computes Gaussian partial correlations for the numeric columns of a matrix
or data frame using a high-performance 'C++' backend. Covariance estimation
is available via the classical sample estimator, ridge regularisation, OAS
shrinkage, or graphical lasso. Optional p-values and Fisher-z confidence
intervals are available for the classical sample estimator in the ordinary
low-dimensional setting.
}
\details{
\strong{Statistical overview.} Given an \eqn{n \times p} data matrix \eqn{X}
(rows are observations, columns are variables), the routine estimates a
\emph{partial correlation} matrix via the precision (inverse covariance)
matrix. Let \eqn{\mu} be the vector of column means and
\deqn{S = (X - \mathbf{1}\mu)^\top (X - \mathbf{1}\mu)}
be the centred cross-product matrix computed without forming a centred copy
of \eqn{X}. Two conventional covariance scalings are formed:
\deqn{\hat\Sigma_{\mathrm{MLE}} = S/n, \qquad
      \hat\Sigma_{\mathrm{unb}} = S/(n-1).}

\itemize{
\item \emph{Sample:} \eqn{\Sigma = \hat\Sigma_{\mathrm{unb}}}.
\item \emph{Ridge:} \eqn{\Sigma = \hat\Sigma_{\mathrm{unb}} + \lambda I_p}
with user-supplied \eqn{\lambda \ge 0} (diagonal inflation).
\item \emph{OAS (Oracle Approximating Shrinkage):}
shrink \eqn{\hat\Sigma_{\mathrm{MLE}}} towards a scaled identity
target \eqn{\mu_I I_p}, where \eqn{\mu_I = \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})/p}.
The data-driven weight \eqn{\rho \in [0,1]} is
\deqn{\rho = \min\!\left\{1,\max\!\left(0,\;
        \frac{(1-\tfrac{2}{p})\,\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}^2)
              + \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})^2}
             {(n + 1 - \tfrac{2}{p})
              \left[\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}^2)
              - \tfrac{\mathrm{tr}(\hat\Sigma_{\mathrm{MLE}})^2}{p}\right]}
        \right)\right\},}
and
\deqn{\Sigma = (1-\rho)\,\hat\Sigma_{\mathrm{MLE}} + \rho\,\mu_I I_p.}
\item \emph{Graphical lasso:} estimate a sparse precision matrix
\eqn{\Theta} by maximising
\deqn{\log\det(\Theta) - \mathrm{tr}(\hat\Sigma_{\mathrm{MLE}}\Theta)
        - \lambda\sum_{i \ne j} |\theta_{ij}|,}
with \eqn{\lambda \ge 0}. The returned covariance matrix is
\eqn{\Sigma = \Theta^{-1}}.
}

The method then ensures positive definiteness of \eqn{\Sigma} (adding a very
small diagonal \emph{jitter} only if necessary) and computes the precision
matrix \eqn{\Theta = \Sigma^{-1}}. Partial correlations are obtained by
standardising the off-diagonals of \eqn{\Theta}:
\deqn{\mathrm{pcor}_{ij} \;=\;
      -\,\frac{\theta_{ij}}{\sqrt{\theta_{ii}\,\theta_{jj}}}, \qquad
      \mathrm{pcor}_{ii}=1.}

If \code{return_p_value = TRUE}, the function also reports the classical
two-sided test p-values for the sample partial correlations, using
\deqn{t_{ij} = \mathrm{pcor}_{ij}
  \sqrt{\frac{n - p}{1 - \mathrm{pcor}_{ij}^2}}}
with \eqn{n - p} degrees of freedom. These p-values are returned only for
\code{method = "sample"}, where they match the standard full-model partial
correlation test.

When \code{ci = TRUE}, the function reports Fisher-\eqn{z} confidence
intervals for the sample partial correlations. For a partial correlation
\eqn{r_{xy \cdot Z}} conditioning on \eqn{c} variables, the transformed
statistic is \eqn{z = \operatorname{atanh}(r_{xy \cdot Z})} with standard
error
\deqn{\operatorname{SE}(z) = \frac{1}{\sqrt{n - 3 - c}},}
where \eqn{n} is the effective complete-case sample size used for the
estimate. The two-sided normal-theory interval is formed on the transformed
scale using \code{conf_level} and then mapped back with \code{tanh()}. In
the full matrix path implemented here, each off-diagonal entry conditions on
all remaining variables, so \eqn{c = p - 2} and the classical CI requires
\eqn{n > p + 1}. This inference is only supported for
\code{method = "sample"} without positive-definiteness repair; in
unsupported or numerically singular settings, CI bounds are returned as
\code{NA} with an informative \pkg{cli} warning or the request is rejected.

\strong{Interpretation.} For Gaussian data, \eqn{\mathrm{pcor}_{ij}} equals
the correlation between residuals from regressing variable \eqn{i} and
variable \eqn{j} on all the remaining variables; equivalently, it encodes
conditional dependence in a Gaussian graphical model, where
\eqn{\mathrm{pcor}_{ij}=0} if variables \eqn{i} and \eqn{j} are
conditionally independent given the others. Partial correlations are
invariant to separate rescalings of each
variable; in particular, multiplying \eqn{\Sigma} by any positive scalar
leaves the partial correlations unchanged.

\strong{Why shrinkage/regularisation?} When \eqn{p \ge n}, the sample
covariance is singular and inversion is ill-posed. Ridge and OAS both yield
well-conditioned \eqn{\Sigma}. Ridge adds a fixed \eqn{\lambda} on the
diagonal, whereas OAS shrinks adaptively towards \eqn{\mu_I I_p} with a
weight chosen to minimise (approximately) the Frobenius risk under a
Gaussian model, often improving mean-square accuracy in high dimension.

\strong{Why glasso?} Glasso is useful when the goal is not just to
stabilise a covariance estimate, but to recover a manageable network of
direct relationships rather than a dense matrix of overall associations. In
Gaussian models, zeros in the precision matrix correspond to conditional
independences, so glasso can suppress indirect associations that are
explained by the other variables and return a smaller, more interpretable
conditional-dependence graph. This is especially practical in
high-dimensional settings, where the sample covariance may be unstable or
singular. Glasso yields a positive-definite precision estimate and supports
edge selection, graph recovery, and downstream network analysis.

\strong{Computational notes.} The implementation forms \eqn{S} using 'BLAS'
\code{syrk} when available and constructs partial correlations by traversing
only the upper triangle with 'OpenMP' parallelism. Positive definiteness is
verified via a Cholesky factorisation; if it fails, a tiny diagonal jitter is
increased geometrically up to a small cap, at which point the routine
signals an error.
}
\examples{
## Structured MVN with known partial correlations
set.seed(42)
p <- 12; n <- 1000

## Build a tri-diagonal precision (Omega) so the true partial correlations
## are sparse
phi <- 0.35
Omega <- diag(p)
for (j in 1:(p - 1)) {
  Omega[j, j + 1] <- Omega[j + 1, j] <- -phi
}
## Strict diagonal dominance
diag(Omega) <- 1 + 2 * abs(phi) + 0.05
Sigma <- solve(Omega)

## Upper Cholesky
L <- chol(Sigma)
Z <- matrix(rnorm(n * p), n, p)
X <- Z \%*\% L
colnames(X) <- sprintf("V\%02d", seq_len(p))

pc <- pcorr(X)
summary(pc)
estimate(pc)
tidy(pc)

## Fisher-z confidence intervals for sample partial correlations
pc_ci <- pcorr(X[, 1:5], ci = TRUE)
ci(pc_ci)
confint(pc_ci)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(pc)
}

## True partial correlation from Omega
pcor_true <- -Omega / sqrt(diag(Omega) \%o\% diag(Omega))
diag(pcor_true) <- 1

## Quick visual check (first 5x5 block)
round(estimate(pc)[1:5, 1:5], 2)
round(pcor_true[1:5, 1:5], 2)

## Plot method
plot(pc)

## Graphical-lasso example
set.seed(100)
p <- 20; n <- 250
Theta_g <- diag(p)
Theta_g[cbind(1:5, 2:6)] <- -0.25
Theta_g[cbind(2:6, 1:5)] <- -0.25
Theta_g[cbind(8:11, 9:12)] <- -0.20
Theta_g[cbind(9:12, 8:11)] <- -0.20
diag(Theta_g) <- rowSums(abs(Theta_g)) + 0.2

Sigma_g <- solve(Theta_g)
X_g <- matrix(rnorm(n * p), n, p) \%*\% chol(Sigma_g)
colnames(X_g) <- paste0("Node", seq_len(p))

gfit_1 <- pcorr(X_g, method = "glasso", lambda = 0.02,
                return_cov_precision = TRUE, return_details = TRUE)
gfit_2 <- pcorr(X_g, method = "glasso", lambda = 0.08,
                return_cov_precision = TRUE, return_details = TRUE)

## Larger lambda gives a sparser conditional-dependence graph
edge_count <- function(M, tol = 1e-8) {
  sum(abs(M[upper.tri(M, diag = FALSE)]) > tol)
}

c(edges_lambda_002 = edge_count(gfit_1$precision),
  edges_lambda_008 = edge_count(gfit_2$precision))

## Inspect strongest estimated conditional associations
pcor_g <- gfit_1$pcor
idx <- which(upper.tri(pcor_g), arr.ind = TRUE)
ord <- order(abs(pcor_g[idx]), decreasing = TRUE)
head(data.frame(
  i = rownames(pcor_g)[idx[ord, 1]],
  j = colnames(pcor_g)[idx[ord, 2]],
  pcor = round(pcor_g[idx][ord], 2)
))

## High-dimensional case p >> n
set.seed(7)
n <- 60; p <- 120

ar_block <- function(m, rho = 0.6) rho^abs(outer(seq_len(m), seq_len(m), "-"))

## Two AR(1) blocks on the diagonal
if (requireNamespace("Matrix", quietly = TRUE)) {
  Sigma_hd <- as.matrix(Matrix::bdiag(ar_block(60, 0.6), ar_block(60, 0.6)))
} else {
  Sigma_hd <- rbind(
    cbind(ar_block(60, 0.6), matrix(0, 60, 60)),
    cbind(matrix(0, 60, 60), ar_block(60, 0.6))
  )
}

L <- chol(Sigma_hd)
X_hd <- matrix(rnorm(n * p), n, p) \%*\% L
colnames(X_hd) <- paste0("G", seq_len(p))

pc_oas   <-
 pcorr(X_hd, method = "oas",   return_cov_precision = TRUE,
       return_details = TRUE)
pc_ridge <-
 pcorr(X_hd, method = "ridge", lambda = 1e-2,
                     return_cov_precision = TRUE, return_details = TRUE)
pc_samp  <-
 pcorr(X_hd, method = "sample", return_cov_precision = TRUE,
       return_details = TRUE)
pc_glasso <-
 pcorr(X_hd, method = "glasso", lambda = 5e-3,
                     return_cov_precision = TRUE, return_details = TRUE)

## Show how much diagonal regularisation was used
c(oas_jitter = pc_oas$jitter,
  ridge_lambda = pc_ridge$lambda,
  sample_jitter = pc_samp$jitter,
  glasso_lambda = pc_glasso$lambda)

## Compare conditioning of the estimated covariance matrices
c(kappa_oas = kappa(pc_oas$cov),
  kappa_ridge = kappa(pc_ridge$cov),
  kappa_sample = kappa(pc_samp$cov))

## Simple conditional-dependence graph from partial correlations
pcor <- pc_oas$pcor
vals <- abs(pcor[upper.tri(pcor, diag = FALSE)])
thresh <- quantile(vals, 0.98)  # top 2\%
edges  <- which(abs(pcor) > thresh & upper.tri(pcor), arr.ind = TRUE)
head(data.frame(i = colnames(pcor)[edges[,1]],
                j = colnames(pcor)[edges[,2]],
                pcor = round(pcor[edges], 2)))

}
\references{
Chen, Y., Wiesel, A., & Hero, A. O. III (2011).
Robust Shrinkage Estimation of High-dimensional Covariance Matrices.
IEEE Transactions on Signal Processing.

Friedman, J., Hastie, T., & Tibshirani, R. (2007).
Sparse inverse covariance estimation with the graphical lasso.
Biostatistics.

Ledoit, O., & Wolf, M. (2004).
A well-conditioned estimator for large-dimensional covariance matrices.
Journal of Multivariate Analysis, 88(2), 365-411.

Schafer, J., & Strimmer, K. (2005).
A shrinkage approach to large-scale covariance matrix estimation and
implications for functional genomics.
Statistical Applications in Genetics and Molecular Biology, 4(1), Article 32.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pearson_corr.R
\name{pearson_corr}
\alias{pearson_corr}
\alias{print.pearson_corr}
\alias{plot.pearson_corr}
\alias{summary.pearson_corr}
\alias{print.summary.pearson_corr}
\title{Pairwise Pearson correlation}
\usage{
pearson_corr(
  data,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{pearson_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{plot}{pearson_corr}(
  x,
  title = "Pearson correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{summary}{pearson_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  ...
)

\method{print}{summary.pearson_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns will be excluded. Each column must have
at least two non-missing values.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each correlation on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach pairwise
Fisher-\eqn{z} confidence intervals for the off-diagonal Pearson
correlations.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other
\code{ggplot2} layers.}

\item{x}{An object of class \code{summary.pearson_corr}.}

\item{digits}{Integer; number of decimal places to print in the concordance}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for Pearson confidence limits in the
pairwise summary.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Pearson correlation heatmap"}.}

\item{low_color}{Color for the minimum correlation. Default is
\code{"indianred1"}.}

\item{high_color}{Color for the maximum correlation. Default is
\code{"steelblue1"}.}

\item{mid_color}{Color for zero correlation. Default is \code{"white"}.}

\item{value_text_size}{Font size for displaying correlation values. Default
is \code{4}.}

\item{ci_text_size}{Text size for confidence intervals in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{pearson_corr}.}
}
\value{
A symmetric numeric matrix where the \code{(i, j)}-th element is
the Pearson correlation between the \code{i}-th and \code{j}-th
numeric columns of the input. When \code{ci = TRUE}, the object also
carries a \code{ci} attribute with elements \code{est}, \code{lwr.ci},
\code{upr.ci}, and \code{conf.level}. When pairwise-complete evaluation is
used, pairwise sample sizes are stored in \code{attr(x, "diagnostics")$n_complete}.

Invisibly returns the \code{pearson_corr} object.

A \code{ggplot} object representing the heatmap.
}
\description{
Computes pairwise Pearson correlations for the numeric columns of a matrix
or data frame using a high-performance 'C++' backend. Optional Fisher-z
confidence intervals are available.
}
\details{
Let \eqn{X \in \mathbb{R}^{n \times p}} be a
numeric matrix with rows as observations and columns as variables, and let
\eqn{\mathbf{1} \in \mathbb{R}^n} denote the all-ones vector. Define the column
means \eqn{\mu = (1/n)\,\mathbf{1}^\top X} and the centred cross-product
matrix
\deqn{ S \;=\; (X - \mathbf{1}\mu)^\top (X - \mathbf{1}\mu)
      \;=\; X^\top \!\Big(I_n - \tfrac{1}{n}\mathbf{1}\mathbf{1}^\top\Big) X
      \;=\; X^\top X \;-\; n\,\mu\,\mu^\top. }
The (unbiased) sample covariance is
\deqn{ \widehat{\Sigma} \;=\; \tfrac{1}{n-1}\,S, }
and the sample standard deviations are \eqn{s_i = \sqrt{\widehat{\Sigma}_{ii}}}.
The Pearson correlation matrix is obtained by standardising \eqn{\widehat{\Sigma}}, and it is given by
\deqn{ R \;=\; D^{-1/2}\,\widehat{\Sigma}\,D^{-1/2}, \qquad
      D \;=\; \mathrm{diag}(\widehat{\Sigma}_{11},\ldots,\widehat{\Sigma}_{pp}), }
equivalently, entrywise \eqn{R_{ij} = \widehat{\Sigma}_{ij}/(s_i s_j)} for
\eqn{i \neq j} and \eqn{R_{ii} = 1}. With \eqn{1/(n-1)} scaling,
\eqn{\widehat{\Sigma}} is unbiased for the covariance; the induced
correlations are biased in finite samples.

The implementation forms \eqn{X^\top X} via a BLAS
symmetric rank-\eqn{k} update (SYRK) on the upper triangle, then applies the
rank-1 correction \eqn{-\,n\,\mu\,\mu^\top} to obtain \eqn{S} without
explicitly materialising \eqn{X - \mathbf{1}\mu}. After scaling by
\eqn{1/(n-1)}, triangular normalisation by \eqn{D^{-1/2}} yields \eqn{R},
which is then symmetrised to remove round-off asymmetry. Tiny negative values
on the covariance diagonal due to floating-point rounding are truncated to
zero before taking square roots.

If a variable has zero variance (\eqn{s_i = 0}), the corresponding row and
column of \eqn{R} are set to \code{NA}. When
\code{na_method = "pairwise"}, each \eqn{(i,j)} correlation is recomputed on
the pairwise complete-case overlap of columns \eqn{i} and \eqn{j}.

When \code{ci = TRUE}, Fisher-\eqn{z} confidence intervals are computed from
the observed pairwise Pearson correlation \eqn{r_{ij}} and the pairwise
complete-case sample size \eqn{n_{ij}}:
\deqn{
z_{ij} = \operatorname{atanh}(r_{ij}), \qquad
\operatorname{SE}(z_{ij}) = \frac{1}{\sqrt{n_{ij} - 3}}.
}
With \eqn{z_{1-\alpha/2} = \Phi^{-1}(1 - \alpha/2)}, the confidence limits are
\deqn{
\tanh\!\bigl(z_{ij} - z_{1-\alpha/2}\operatorname{SE}(z_{ij})\bigr)
\;\;\text{and}\;\;
\tanh\!\bigl(z_{ij} + z_{1-\alpha/2}\operatorname{SE}(z_{ij})\bigr).
}
Confidence intervals are reported only when \eqn{n_{ij} > 3}.

\strong{Computational complexity.} The dominant cost is \eqn{O(n p^2)} flops
with \eqn{O(p^2)} memory.
}
\note{
\code{na_method = "complete"} is useful when a common analysis sample is
required across all matrix entries. For covariance- or cross-product-based
correlations, it also avoids the non-positive-semidefinite matrices that can
arise from pairwise deletion.
}
\examples{
## MVN with AR(1) correlation
set.seed(123)
p <- 6; n <- 300; rho <- 0.5
# true correlation
Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
L <- chol(Sigma)
# MVN(n, 0, Sigma)
X <- matrix(rnorm(n * p), n, p) \%*\% L
colnames(X) <- paste0("V", seq_len(p))

pr <- pearson_corr(X)
print(pr, digits = 2)
summary(pr)
estimate(pr)
tidy(pr)
plot(pr)

## Confidence intervals
pr_ci <- pearson_corr(X[, 1:3], ci = TRUE)
ci(pr_ci)
confint(pr_ci)

## Compare the sample estimate to the truth
Rhat <- cor(X)
# estimated
round(Rhat[1:4, 1:4], 2)
# true
round(Sigma[1:4, 1:4], 2)
off <- upper.tri(Sigma, diag = FALSE)
# MAE on off-diagonals
mean(abs(Rhat[off] - Sigma[off]))

## Larger n reduces sampling error
n2 <- 2000
X2 <- matrix(rnorm(n2 * p), n2, p) \%*\% L
Rhat2 <- cor(X2)
off <- upper.tri(Sigma, diag = FALSE)
## mean absolute error (MAE) of the off-diagonal correlations
mean(abs(Rhat2[off] - Sigma[off]))

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(pr)
}

}
\references{
Pearson, K. (1895). "Notes on regression and inheritance in the case of
two parents". Proceedings of the Royal Society of London, 58, 240-242.
}
\seealso{
\code{\link{print.pearson_corr}}, \code{\link{plot.pearson_corr}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prob_agree.R
\name{plot.prob_agree}
\alias{plot.prob_agree}
\title{Plot probability-of-agreement results}
\usage{
\method{plot}{prob_agree}(
  x,
  threshold = NULL,
  title = "Probability of agreement",
  style = c("auto", "curve", "facet", "heatmap"),
  show_ci = NULL,
  ...
)
}
\arguments{
\item{x}{An object returned by \code{prob_agree()}.}

\item{threshold}{Optional probability threshold shown in curve plots.}

\item{title}{Optional plot title.}

\item{style}{Plot style: \code{"auto"} and \code{"curve"} draw pairwise probability
curves; \code{"facet"} draws one curve panel per comparison; \code{"heatmap"} shows
probability by predictor value and comparison.}

\item{show_ci}{Logical; for curve plots, controls whether confidence ribbons
are shown. By default ribbons are shown for one comparison and suppressed
for multiple comparisons.}

\item{...}{Additional arguments passed to \code{\link[ggplot2:theme]{ggplot2::theme()}}.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot probability-of-agreement results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view_rmcorr_shiny.R
\name{view_rmcorr_shiny}
\alias{view_rmcorr_shiny}
\title{Interactive Shiny Viewer for Repeated-Measures Correlation}
\usage{
view_rmcorr_shiny(x, title = NULL, default_max_vars = 40L)
}
\arguments{
\item{x}{An object of class \code{"rmcorr_matrix"} or a named list of such
objects.}

\item{title}{Optional character title shown at the top of the gadget.}

\item{default_max_vars}{Integer; maximum number of variables pre-selected
in the heatmap view when the app opens. Defaults to 40.}
}
\value{
Invisibly returns \code{NULL}; the function is called for its side
effect of launching a Shiny gadget.
}
\description{
Launches a dedicated Shiny gadget for repeated-measures correlation matrix
objects of class \code{"rmcorr_matrix"}. The viewer combines the correlation
heatmap with a pairwise scatterplot panel that rebuilds the corresponding
two-variable \code{"rmcorr"} fit for user-selected variables.
}
\details{
This helper requires the \pkg{shiny} and \pkg{shinyWidgets}
packages at runtime and will optionally use \pkg{plotly} for the heatmap
when available. The pairwise panel reuses the package's regular
\code{plot.rmcorr()} method, so the Shiny scatterplot matches the standard
pairwise repeated-measures correlation plot. To rebuild pairwise fits from a
returned \code{"rmcorr_matrix"} object, the matrix must have been created
with \code{keep_data = TRUE}.
}
\examples{
if (interactive()) {
  set.seed(2026)
  n_subjects <- 20
  n_rep <- 4
  subject <- rep(seq_len(n_subjects), each = n_rep)
  subj_eff_x <- rnorm(n_subjects, sd = 1.5)
  subj_eff_y <- rnorm(n_subjects, sd = 2.0)
  within_signal <- rnorm(n_subjects * n_rep)

  dat <- data.frame(
    subject = subject,
    x = subj_eff_x[subject] + within_signal + rnorm(n_subjects * n_rep, sd = 0.2),
    y = subj_eff_y[subject] + 0.8 * within_signal + rnorm(n_subjects * n_rep, sd = 0.3),
    z = subj_eff_y[subject] - 0.4 * within_signal + rnorm(n_subjects * n_rep, sd = 0.4)
  )

  fit_mat <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    keep_data = TRUE
  )
  view_rmcorr_shiny(fit_mat)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_inherits}
\alias{check_inherits}
\title{Check object class}
\usage{
check_inherits(x, class, arg = as.character(substitute(x)))
}
\description{
Check object class
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{print.summary.matrixCorr}
\alias{print.summary.matrixCorr}
\title{Summary Method for Correlation Matrices}
\usage{
\method{print}{summary.matrixCorr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{summary.matrixCorr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints compact summary statistics returned by matrix-style
\code{summary()} methods in \pkg{matrixCorr}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_kappa.R
\name{weighted_kappa}
\alias{weighted_kappa}
\alias{print.weighted_kappa}
\title{Pairwise Weighted Cohen's Kappa for Ordered Ratings}
\usage{
weighted_kappa(
  data,
  y = NULL,
  weights = c("quadratic", "linear", "unweighted"),
  levels = NULL,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{weighted_kappa}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{In matrix mode, a matrix or data frame whose rows are
observational units and whose columns are raters or classifiers. Each
column contains ordered category ratings. In two-vector mode, the first
ordinal rating vector.}

\item{y}{Optional second ordinal rating vector. When supplied, the function
returns a single weighted kappa estimate for \code{data} and \code{y}.}

\item{weights}{Weight specification. The default \code{"quadratic"} uses
Fleiss-Cohen-style agreement weights. \code{"linear"} uses equal-spacing
agreement weights. \code{"unweighted"} reduces to ordinary Cohen's kappa.
Custom numeric weight matrices must be symmetric agreement weights with
diagonal 1 and entries in \code{[0, 1]}.}

\item{levels}{Optional ordered category levels. Weighted kappa depends on
category order and will not silently alphabetise arbitrary labels. If
omitted, order is inferred only when all involved ratings are ordered
factors with identical levels or when all involved ratings are numeric or
integer and can be ordered by their sorted unique values.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"complete"} applies listwise
deletion across retained columns before computing the matrix. In matrix
mode, \code{"pairwise"} uses pair-specific complete observations.}

\item{ci}{Logical; if \code{TRUE}, attach large-sample delta-method
confidence intervals.}

\item{p_value}{Logical; if \code{TRUE}, attach large-sample delta-method
Wald test statistics and p-values for \eqn{H_0: \kappa_w = 0}.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
matrix C++ backend. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for matrix mode.
\itemize{
\item \code{"matrix"} (default): full dense matrix.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries after thresholding.
\item \code{"edge_list"}: long-form data frame representation.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
retain entries with \code{abs(value) >= threshold}. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in sparse and
edge-list outputs.}

\item{...}{Reserved for future extensions. Unsupported extra arguments are
rejected.}

\item{x}{A matrix-style \code{weighted_kappa} object.}

\item{digits}{Integer; number of decimal places for displayed values.}

\item{n}{Optional preview row threshold.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}
}
\value{
If \code{y} is supplied, a scalar S3 object of class
\code{c("weighted_kappa", "numeric")} is returned with diagnostics and
optional \code{ci} and \code{inference} attributes. Otherwise a symmetric
matrix-style result with estimator class \code{weighted_kappa}.
}
\description{
Computes weighted Cohen's kappa for either a pair of ordinal rating vectors
or all pairwise combinations of ordinal columns in a matrix or data frame.
}
\details{
This implementation uses agreement/similarity weights internally:
\itemize{
\item \code{"unweighted"}: \eqn{w_{ij} = 1(i = j)}
\item \code{"linear"}: \eqn{w_{ij} = 1 - |i - j| / (K - 1)}
\item \code{"quadratic"}: \eqn{w_{ij} = 1 - (i - j)^2 / (K - 1)^2}
}
The \code{"quadratic"} scheme is the default and corresponds to
Fleiss-Cohen-style agreement weights. The \code{"linear"} scheme
corresponds to equal-spacing agreement weights.

For matrix mode, rows are shared observational units and columns are raters
or classifiers. A common category map is resolved in R and the main
computation is performed in C++. Missing-data handling follows the usual
\pkg{matrixCorr} \code{na_method} conventions. Pairwise complete counts are
stored in \code{attr(x, "diagnostics")$n_complete}.

\strong{Confidence intervals and standard errors.}
Confidence intervals and p-values use the exact large-sample multinomial
delta-method formula. Let
\eqn{A_w = \sum_{a,b} w_{ab} p_{ab}} be the observed weighted agreement,
\eqn{E_w = \sum_{a,b} w_{ab} q_a r_b} the expected weighted agreement,
and \eqn{D = 1 - E_w}. Define the weighted margin summaries
\deqn{ u_a = \sum_b w_{ab} r_b, \qquad v_b = \sum_a w_{ab} q_a. }
For each cell \eqn{(a,b)}, the backend uses
\deqn{ g_{ab}
      \;=\;
      \frac{D\,w_{ab} + (A_w - 1)\,(u_a + v_b)}{D^2}. }
The variance estimator is
\deqn{ \widehat{\mathrm{Var}}(\hat\kappa_w)
      \;=\;
      \frac{1}{n}\left(
      \sum_{a,b} p_{ab} g_{ab}^2 -
      \Big(\sum_{a,b} p_{ab} g_{ab}\Big)^2
      \right), }
where \eqn{n} is the number of complete paired ratings. The standard error
is \eqn{\widehat{\mathrm{se}}(\hat\kappa_w)=
\sqrt{\widehat{\mathrm{Var}}(\hat\kappa_w)}} and the CI is the Wald interval
\deqn{ \hat\kappa_w \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\kappa_w), }
truncated to \eqn{[-1, 1]} in the returned result. Use \code{\link[=cohen_kappa]{cohen_kappa()}} for
unordered nominal categories where all disagreements are equally serious.
}
\examples{
raters <- data.frame(
  r1 = ordered(c("low", "low", "mid", "high", "high"),
               levels = c("low", "mid", "high")),
  r2 = ordered(c("low", "mid", "mid", "high", "high"),
               levels = c("low", "mid", "high")),
  r3 = ordered(c("low", "low", "high", "high", "mid"),
               levels = c("low", "mid", "high"))
)

wk <- weighted_kappa(raters)
print(wk)
summary(wk)
estimate(wk)
tidy(wk)
plot(wk)

x <- raters$r1
y <- raters$r2
weighted_kappa(x, y, weights = "linear")

}
\references{
Cohen, J. (1968). Weighted kappa: Nominal scale agreement with provision for
scaled disagreement or partial credit. \emph{Psychological Bulletin},
70(4), 213-220. \doi{10.1037/h0026256}
}
\seealso{
\code{\link[=cohen_kappa]{cohen_kappa()}} for unweighted two-rater nominal agreement;
\code{\link[=multirater_kappa]{multirater_kappa()}} for panel-level nominal agreement among three or more
raters.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/schafer_corr.R
\name{shrinkage_corr}
\alias{shrinkage_corr}
\alias{schafer_corr}
\alias{print.shrinkage_corr}
\alias{print.schafer_corr}
\alias{plot.shrinkage_corr}
\alias{plot.schafer_corr}
\alias{summary.shrinkage_corr}
\alias{summary.schafer_corr}
\title{Shrinkage Correlation}
\usage{
shrinkage_corr(
  data,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

schafer_corr(
  data,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

\method{print}{shrinkage_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{schafer_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{shrinkage_corr}(
  x,
  title = "Schafer-Strimmer shrinkage correlation",
  cluster = TRUE,
  hclust_method = "complete",
  triangle = c("upper", "lower", "full"),
  show_value = TRUE,
  show_values = NULL,
  value_text_limit = 60,
  value_text_size = 3,
  palette = c("diverging", "viridis"),
  ...
)

\method{plot}{schafer_corr}(
  x,
  title = "Schafer-Strimmer shrinkage correlation",
  cluster = TRUE,
  hclust_method = "complete",
  triangle = c("upper", "lower", "full"),
  show_value = TRUE,
  show_values = NULL,
  value_text_limit = 60,
  value_text_size = 3,
  palette = c("diverging", "viridis"),
  ...
)

\method{summary}{shrinkage_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{schafer_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns will be excluded. Columns must be numeric
and contain no \code{NA}s.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{x}{An object of class \code{shrinkage_corr} or
\code{schafer_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()}.}

\item{title}{Plot title.}

\item{cluster}{Logical; if TRUE, reorder rows/cols by hierarchical clustering
on distance \eqn{1 - r}.}

\item{hclust_method}{Linkage method for \code{hclust}; default \code{"complete"}.}

\item{triangle}{One of \code{"full"}, \code{"upper"}, \code{"lower"}.
Default to \code{upper}.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles (subject to \code{value_text_limit}).}

\item{show_values}{Deprecated compatibility alias for \code{show_value}. If
supplied, it overrides \code{show_value}.}

\item{value_text_limit}{Integer threshold controlling when values are drawn.}

\item{value_text_size}{Font size for values if shown.}

\item{palette}{Character; \code{"diverging"} (default) or \code{"viridis"}.}

\item{object}{An object of class \code{shrinkage_corr} or
\code{schafer_corr}.}
}
\value{
A symmetric numeric matrix of class \code{shrinkage_corr} (with
compatibility class \code{schafer_corr}) where entry \code{(i, j)} is the
shrunk correlation between the \code{i}-th and \code{j}-th numeric columns.
Attributes:
\itemize{
\item \code{method} = \code{"schafer_shrinkage"}
\item \code{description} = \code{"Schafer-Strimmer shrinkage correlation
  matrix"}
\item \code{package} = \code{"matrixCorr"}
}
Columns with zero variance are set to \code{NA} across row/column (including
the diagonal), matching \code{pearson_corr()} behaviour.

Invisibly returns \code{x}.

A \code{ggplot} object.
}
\description{
Computes a shrinkage correlation matrix for numeric data using a
high-performance 'C++' backend. The current implementation uses the
Schafer-Strimmer shrinkage estimator to stabilise Pearson correlation
estimates by shrinking off-diagonal entries towards zero.
}
\details{
Let \eqn{R} be the sample Pearson correlation matrix. The Schafer-Strimmer
shrinkage estimator targets the identity in correlation space and uses
\eqn{\hat\lambda = \frac{\sum_{i<j}\widehat{\mathrm{Var}}(r_{ij})}
{\sum_{i<j} r_{ij}^2}} (clamped to \eqn{[0,1]}), where
\eqn{\widehat{\mathrm{Var}}(r_{ij}) \approx \frac{(1-r_{ij}^2)^2}{n-1}}.
The returned estimator is \eqn{R_{\mathrm{shr}} = (1-\hat\lambda)R +
\hat\lambda I}.
}
\note{
No missing values are permitted. Columns with fewer than two observations
or zero variance are flagged as \code{NA} (row/column).
}
\examples{
## Multivariate normal with AR(1) dependence (Toeplitz correlation)
set.seed(1)
n <- 80; p <- 40; rho <- 0.6
d <- abs(outer(seq_len(p), seq_len(p), "-"))
Sigma <- rho^d

X <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
colnames(X) <- paste0("V", seq_len(p))

Rshr <- shrinkage_corr(X)
print(Rshr, digits = 2, n = 6, max_vars = 6)
summary(Rshr)
estimate(Rshr)
tidy(Rshr)
plot(Rshr)

## Shrinkage typically moves the sample correlation closer to the truth
Rraw <- stats::cor(X)
off  <- upper.tri(Sigma, diag = FALSE)
mae_raw <- mean(abs(Rraw[off] - Sigma[off]))
mae_shr <- mean(abs(Rshr[off] - Sigma[off]))
print(c(MAE_raw = mae_raw, MAE_shrunk = mae_shr))
plot(Rshr, title = "Schafer-Strimmer shrinkage correlation")

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(Rshr)
}

}
\references{
Schafer, J. & Strimmer, K. (2005). A shrinkage approach to large-scale
covariance matrix estimation and implications for functional genomics.
\emph{Statistical Applications in Genetics and Molecular Biology}, 4(1).
}
\seealso{
\code{\link{print.shrinkage_corr}},
\code{\link{plot.shrinkage_corr}}, \code{\link{pearson_corr}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{ccc_rm_ustat}
\alias{ccc_rm_ustat}
\title{Repeated-Measures Lin's Concordance Correlation Coefficient (CCC)}
\usage{
ccc_rm_ustat(
  data,
  response,
  subject,
  method,
  time = NULL,
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE,
  Dmat = NULL,
  delta = 1
)
}
\arguments{
\item{data}{A data frame containing the repeated-measures dataset.}

\item{response}{Character. Name of the numeric outcome column.}

\item{subject}{Character. Column identifying subjects. Every subject must
have measurements from all methods (and times, when supplied); rows with
incomplete \{subject, time, method\} coverage are dropped per pair.}

\item{method}{Character. Name of the method column (factor with L
\eqn{\geq} 2 levels).}

\item{time}{Character or NULL. Name of the time/repetition column. If NULL,
one time point is assumed.}

\item{ci}{Logical. If TRUE, returns confidence intervals (default FALSE).}

\item{conf_level}{Confidence level for CI (default 0.95).}

\item{n_threads}{Integer (\eqn{\geq} 1). Number of OpenMP threads to use for computation.
Defaults to \code{getOption("matrixCorr.threads", 1L)}.}

\item{verbose}{Logical. If TRUE, prints diagnostic output (default FALSE).}

\item{Dmat}{Optional numeric weight matrix (T \eqn{\times} T) for
timepoints. Defaults to identity.}

\item{delta}{Numeric. Exponent applied to the absolute pointwise differences
between two method trajectories before the time-weighted quadratic form is
evaluated. Internally, the function forms
\eqn{v_t = |X_t - Y_t|^\delta} and then computes
\eqn{v^\top D v}, where \eqn{D} is \code{Dmat}.
Therefore, when \code{Dmat} is diagonal:
\itemize{
\item \code{delta = 1} (default) gives the standard quadratic
repeated-measures CCC distance,
\eqn{(X - Y)^\top D (X - Y)}.
\item \code{delta = 2} gives a fourth-power loss,
which puts stronger emphasis on large disagreements.
\item \code{delta = 0} gives a binary disagreement indicator before
aggregation, analogous to a repeated-measures kappa-type distance.
}
In most applications, \code{delta = 1} should be used because it matches
the usual quadratic distance used in repeated-measures CCC.}
}
\value{
If \code{ci = FALSE}, a symmetric matrix of class \code{"ccc"} (estimates only).
If \code{ci = TRUE}, a list of class \code{"ccc"}, \code{"ccc_ci"} with elements:
\itemize{
\item \code{est}: CCC estimate matrix
\item \code{lwr.ci}: Lower bound matrix
\item \code{upr.ci}: Upper bound matrix
}
}
\description{
Computes all pairwise Lin's Concordance Correlation Coefficients (CCC)
across multiple methods (L \eqn{\geq} 2) for repeated-measures data.
Each subject must be measured by all methods across the same set of time
points or replicates.

CCC measures both accuracy (how close measurements are to the line of
equality) and precision (Pearson correlation). Confidence intervals are
optionally computed using a U-statistics-based estimator with Fisher's Z
transformation
}
\details{
This function computes pairwise Lin's Concordance Correlation Coefficient
(CCC) between methods in a repeated-measures design using a
U-statistics-based nonparametric estimator proposed by
Carrasco et al. (2013). It is computationally efficient and robust,
particularly for large-scale or balanced longitudinal designs.

Lin's CCC is defined as
\deqn{
\rho_c = \frac{2 \cdot \mathrm{cov}(X, Y)}{\sigma_X^2 + \sigma_Y^2 +
(\mu_X - \mu_Y)^2}
}{
CCC = 2 * cov(X, Y) / [var(X) + var(Y) + (mean(X) - mean(Y))^2]
}
where:
\itemize{
\item \eqn{X} and \eqn{Y} are paired measurements from two methods.
\item \eqn{\mu_X}, \eqn{\mu_Y} are means, and \eqn{\sigma_X^2},
\eqn{\sigma_Y^2} are variances.
}
\subsection{U-statistics Estimation}{

For repeated measures across \eqn{T} time points and \eqn{n} subjects we
assume
\itemize{
\item all \eqn{n(n-1)} pairs of subjects are considered to compute a
U-statistic estimator for within-method and cross-method distances.
\item if \code{delta > 0}, pairwise distances are raised to a power before
applying a time-weighted kernel matrix \eqn{D}.
\item if \code{delta = 0}, the method reduces to a version similar to a
repeated-measures kappa.
}
}

\subsection{Confidence Intervals}{

Confidence intervals are constructed using a \strong{Fisher Z-transformation}
of the CCC. Specifically,
\itemize{
\item The CCC is transformed using
\eqn{Z = 0.5 \log((1 + \rho_c) / (1 - \rho_c))}.
\item Standard errors are computed from the asymptotic variance of the
U-statistic.
\item Normal-based intervals are computed on the Z-scale and then
back-transformed to the CCC scale.
}
}

\subsection{Assumptions}{

\itemize{
\item The design must be \strong{balanced}, where all subjects must have
complete observations for all methods and time points.
\item The method is \strong{nonparametric} and does not require assumptions
of normality or linear mixed effects.
\item Weights (\code{Dmat}) allow differential importance of time points.
}

For \strong{unbalanced} or \strong{complex hierarchical} data (e.g.,
missing timepoints, covariate adjustments), consider using
\code{\link{ccc_rm_reml}}, which uses a variance components approach
via linear mixed models.
}
}
\examples{
set.seed(123)
df <- expand.grid(subject = 1:10,
                  time = 1:2,
                  method = c("A", "B", "C"))
df$y <- rnorm(nrow(df), mean = match(df$method, c("A", "B", "C")), sd = 1)

# CCC matrix (no CIs)
ccc1 <- ccc_rm_ustat(df, response = "y", subject = "subject",
                            method = "method", time = "time")
print(ccc1)
summary(ccc1)
plot(ccc1)

# With confidence intervals
ccc2 <- ccc_rm_ustat(df, response = "y", subject = "subject",
                            method = "method", time = "time", ci = TRUE)
print(ccc2)
summary(ccc2)
estimate(ccc2)
tidy(ccc2)
ci(ccc2)
confint(ccc2)
plot(ccc2)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(ccc2)
}

#------------------------------------------------------------------------
# Choosing delta based on distance sensitivity
#------------------------------------------------------------------------
# Standard quadratic RM-CCC distance: (X - Y)' D (X - Y)
ccc_rm_ustat(df, response = "y", subject = "subject",
             method = "method", time = "time", delta = 1)

# Fourth-power loss when D is diagonal: emphasises large disagreements
ccc_rm_ustat(df, response = "y", subject = "subject",
             method = "method", time = "time", delta = 2)
# Binary disagreement indicator before aggregation
ccc_rm_ustat(df, response = "y", subject = "subject",
             method = "method", time = "time", delta = 0)
}
\references{
Lin L (1989). A concordance correlation coefficient to evaluate reproducibility.
\emph{Biometrics}, 45: 255-268.

Lin L (2000). A note on the concordance correlation coefficient.
\emph{Biometrics}, 56: 324-325.

Carrasco JL, Jover L (2003). Estimating the concordance correlation coefficient:
a new approach. \emph{Computational Statistics & Data Analysis}, 47(4): 519-539.
}
\seealso{
\code{\link{ccc}}, \code{\link{ccc_rm_reml}},
\code{\link{plot.ccc}}, \code{\link{print.ccc}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pbcor.R
\name{pbcor}
\alias{pbcor}
\alias{print.pbcor}
\alias{plot.pbcor}
\alias{summary.pbcor}
\alias{print.summary.pbcor}
\title{Percentage bend correlation}
\usage{
pbcor(
  data,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  beta = 0.2,
  n_boot = 500L,
  seed = NULL,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

\method{print}{pbcor}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{pbcor}(
  x,
  title = "Percentage bend correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{pbcor}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.pbcor}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or data frame containing numeric columns.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each estimate on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.
With \code{"pairwise"}, each correlation is computed on the overlapping
complete rows for the column pair.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach percentile
bootstrap confidence intervals for each pairwise estimate.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach the
method-specific large-sample test statistic and two-sided p-value for each
pairwise estimate.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads used for the
point-estimate matrix computation. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{beta}{Bending constant in \code{[0, 0.5)} that sets the cutoff used to
bend standardised deviations toward the interval \eqn{[-1, 1]}. Larger
values cause more observations to be bent and increase resistance to
marginal outliers. Default \code{0.2}. See Details.}

\item{n_boot}{Integer \eqn{\geq 1}. Number of bootstrap resamples used when
\code{ci = TRUE}. Default \code{500}.}

\item{seed}{Optional positive integer used to seed the bootstrap resampling
when \code{ci = TRUE}. If \code{NULL}, the current random-number stream is
used.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{x}{An object of class \code{summary.pbcor}.}

\item{digits}{Integer; number of digits to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to the underlying print or plot helper.}

\item{title}{Character; plot title.}

\item{low_color, high_color, mid_color}{Colors used in the heatmap.}

\item{value_text_size}{Numeric text size for overlaid cell values.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{pbcor}.}

\item{ci_digits}{Integer; digits used for confidence limits in pairwise
summaries.}

\item{p_digits}{Integer; digits used for p-values in pairwise summaries.}
}
\value{
A symmetric correlation matrix with class \code{pbcor} and
attributes \code{method = "percentage_bend_correlation"},
\code{description}, and \code{package = "matrixCorr"}. When
\code{ci = TRUE}, the returned object also carries a \code{ci} attribute
with elements \code{est}, \code{lwr.ci}, \code{upr.ci},
\code{conf.level}, and \code{ci.method}, plus
\code{attr(x, "conf.level")}. When \code{p_value = TRUE}, it also carries
an \code{inference} attribute with elements \code{estimate},
\code{statistic}, \code{parameter}, \code{p_value}, \code{n_obs}, and
\code{alternative}. When either inferential option is requested, the
object also carries \code{diagnostics$n_complete}.
}
\description{
Computes all pairwise percentage bend correlations for the numeric columns of
a matrix or data frame. Percentage bend correlation limits the influence of
extreme marginal observations by bending standardised deviations into the
interval \eqn{[-1, 1]}, yielding a Pearson-like measure that is robust to
outliers and heavy tails.
}
\details{
Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
observations and columns as variables. For a column
\eqn{x = (x_i)_{i=1}^n}, let \eqn{m_x = \mathrm{med}(x)} and define
\eqn{\omega_\beta(x)} as the \eqn{\lfloor (1-\beta)n \rfloor}-th order
statistic of \eqn{|x_i - m_x|}. Larger values of \code{beta} reduce
\eqn{\omega_\beta(x)}, so more observations are bent to the bounds
\eqn{-1} and \eqn{1}.

The one-step percentage-bend location is
\deqn{
\hat\theta_{pb}(x) =
\frac{\sum_{i: |\psi_i| \le 1} x_i + \omega_\beta(x)(i_2 - i_1)}
     {n - i_1 - i_2},
\qquad
\psi_i = \frac{x_i - m_x}{\omega_\beta(x)},
}
where \eqn{i_1 = \sum_{i=1}^n \mathbf{1}(\psi_i < -1)} and
\eqn{i_2 = \sum_{i=1}^n \mathbf{1}(\psi_i > 1)}. The bent scores are
\deqn{
a_i = \max\!\left\{-1,\; \min\!\left(1,\frac{x_i - \hat\theta_{pb}(x)}
{\omega_\beta(x)}\right)\right\},
}
and likewise \eqn{b_i} for a second column \eqn{y}. The percentage bend
correlation for the pair \eqn{(x,y)} is
\deqn{
r_{pb}(x,y) =
\frac{\sum_{i=1}^n a_i b_i}
     {\sqrt{\sum_{i=1}^n a_i^2}\sqrt{\sum_{i=1}^n b_i^2}}.
}

In the complete-data path, the bent score vectors are computed once per
column and collected into a matrix \eqn{A = [a_{\cdot 1}, \ldots, a_{\cdot p}]},
after which the correlation matrix is formed from their cross-products:
\deqn{
R_{pb} = D_A^{-1/2} A^\top A D_A^{-1/2},
\qquad
D_A = \mathrm{diag}(a_{\cdot 1}^\top a_{\cdot 1}, \ldots,
a_{\cdot p}^\top a_{\cdot p}).
}
If a column yields an undefined bent-score denominator, the corresponding row
and column are returned as \code{NA}. With \code{na_method = "pairwise"},
each pair is recomputed on its complete-case overlap. As with pairwise
Pearson correlation, this pairwise path can break positive semidefiniteness.

When \code{p_value = TRUE}, the method-specific test statistic for a pairwise
estimate \eqn{r_{pb}} based on \eqn{n_{ij}} complete observations is
\deqn{
T_{ij} = r_{pb,ij}\sqrt{\frac{n_{ij} - 2}{1 - r_{pb,ij}^2}},
}
and the reported p-value is the two-sided Student-\eqn{t} tail probability
with \eqn{n_{ij}-2} degrees of freedom. When \code{ci = TRUE}, the interval
is a percentile bootstrap interval based on \eqn{n_{\mathrm{boot}}}
resamples drawn from the pairwise complete cases. If
\eqn{\tilde r_{pb,(1)} \le \cdots \le \tilde r_{pb,(B)}} denotes the sorted
bootstrap sample of finite estimates with \eqn{B} retained resamples, the
reported limits are
\deqn{
\tilde r_{pb,(\ell)} \quad \text{and} \quad \tilde r_{pb,(u)},
}
where \eqn{\ell = \lfloor (\alpha/2) B + 0.5 \rfloor} and
\eqn{u = \lfloor (1-\alpha/2) B + 0.5 \rfloor} for
\eqn{\alpha = 1 - \mathrm{conf\_level}}. Resamples that yield undefined
estimates are discarded before the percentile limits are formed.

\strong{Computational complexity.} In the complete-data path, forming the
bent scores requires sorting within each column and the cross-product step
costs \eqn{O(n p^2)} with \eqn{O(p^2)} output storage. When
\code{ci = TRUE}, the bootstrap cost is incurred separately for each column
pair.
}
\examples{
set.seed(10)
X <- matrix(rnorm(150 * 4), ncol = 4)
X[sample(length(X), 8)] <- X[sample(length(X), 8)] + 10

R <- pbcor(X)
print(R, digits = 2)
summary(R)
estimate(R)
tidy(R)
plot(R)

## Bootstrap confidence intervals
R_ci <- pbcor(X, ci = TRUE, n_boot = 49, seed = 10)
ci(R_ci)
confint(R_ci)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(R)
}

}
\references{
Wilcox, R. R. (1994). The percentage bend correlation coefficient.
Psychometrika, 59(4), 601-616. \doi{10.1007/BF02294395}
}
\seealso{
\code{\link[=wincor]{wincor()}}, \code{\link[=skipped_corr]{skipped_corr()}}, \code{\link[=bicor]{bicor()}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prob_agree.R
\name{prob_agree}
\alias{prob_agree}
\title{Probability of Agreement for Reliability Curves}
\usage{
prob_agree(
  data,
  response,
  predictor,
  group,
  delta = NULL,
  limits = NULL,
  link = c("logit", "probit"),
  newdata = NULL,
  grid_size = 100L,
  ci = TRUE,
  conf_level = 0.95,
  max_iter = 50L,
  tol = 1e-08,
  verbose = FALSE
)
}
\arguments{
\item{data}{A data frame containing the response, predictor, and group
variables.}

\item{response}{Character scalar naming the binary pass/fail response column.}

\item{predictor}{Character scalar naming the numeric age, time, or operating
condition column.}

\item{group}{Character scalar naming the population/generation column. All
pairwise comparisons among the observed non-missing levels are evaluated.}

\item{delta}{Positive scalar or vector tolerance for symmetric limits
\code{-delta} and \code{delta}. A length-two vector with a negative lower bound and a
positive upper bound, for example \code{c(-0.03, 0.05)}, is treated as an
asymmetric tolerance interval. No default is provided because the tolerance
is part of the scientific estimand.}

\item{limits}{Numeric vector \code{c(lower, upper)} for asymmetric tolerance
limits. Mutually exclusive with \code{delta}.}

\item{link}{Link used for the reliability curve: \code{"logit"} or \code{"probit"}.}

\item{newdata}{Optional data frame containing predictor values where the
probability of agreement is evaluated. If \code{NULL}, a regular grid over the
observed predictor range is used.}

\item{grid_size}{Integer number of evaluation points when \code{newdata = NULL}.}

\item{ci}{Logical; if \code{TRUE}, attach pointwise confidence limits.}

\item{conf_level}{Confidence level for intervals.}

\item{max_iter}{Maximum number of IRLS iterations for each fitted curve.}

\item{tol}{Convergence tolerance for IRLS coefficient updates.}

\item{verbose}{Logical; if \code{TRUE}, emits a short \code{cli} message.}
}
\value{
A data frame with class
\code{c("prob_agree_curve", "prob_agree", "data.frame")} and columns \code{group1},
\code{group2}, the predictor, \code{prob_agree}, and optional \code{lwr.ci}, \code{upr.ci}.
}
\description{
\code{prob_agree()} implements the probability of agreement following Stevens and
Anderson-Cook (2017). It fits binomial reliability curves by population or
generation and estimates, for each pair, the probability that the fitted
reliabilities differ by no more than a user-specified practically negligible
amount.

This is not a correlation coefficient and should not be interpreted as
linear association. It is a tolerance-based agreement measure. It is
computed from the sampling distribution of the estimated difference between
two reliability curves.
}
\details{
For each pair of fitted reliability curves \eqn{\hat\pi_1(a)} and
\eqn{\hat\pi_2(a)}, Stevens and Anderson-Cook define
\deqn{\theta(a) = P\{|\tilde\pi_1(a) - \tilde\pi_2(a)| \le \delta(a) \mid a\}.}
The models are fit with a binomial GLM using either a logit or probit link,
with \eqn{g(\pi_i) = \beta_0 + \beta_1 a_i}. The C++ backend evaluates the
two-population large-sample normal approximation described in the paper and
Supplementary Material A; when more than two groups are supplied, \code{prob_agree()}
applies that calculation to all pairwise group comparisons.

Missing rows in \code{response}, \code{predictor}, or \code{group} are removed before model
fitting. The response must be binary, coded as \code{0}/\code{1}, \code{FALSE}/\code{TRUE}, or a
two-level factor.
}
\examples{
# Stevens and Anderson-Cook's probability of agreement evaluates whether
# two related populations have reliability curves that are similar enough
# to be treated as practically homogeneous. At each age, the agreement
# hypothesis is that the difference between the two fitted reliabilities
# lies inside the user-specified practical tolerance interval.
set.seed(1)
n <- 160
dat <- data.frame(
  age = c(runif(n, 0, 60), runif(n, 0, 45)),
  generation = rep(c("Gen1", "Gen2"), each = n)
)
eta <- ifelse(
  dat$generation == "Gen1",
  4.3 - 0.045 * dat$age,
  4.0 - 0.040 * dat$age
)
dat$pass <- rbinom(nrow(dat), size = 1, prob = plogis(eta))

fit_pa <- prob_agree(
  dat,
  response = "pass",
  predictor = "age",
  group = "generation",
  delta = 0.05,
  link = "logit",
  ci = TRUE
)
print(fit_pa)
summary(fit_pa)
estimate(fit_pa)
tidy(fit_pa)
confint(fit_pa)
plot(fit_pa)

# Four generations are compared as all pairwise two-generation contrasts.
set.seed(2)
n4 <- 120
dat4 <- data.frame(
  age = rep(runif(n4, 0, 55), times = 4),
  generation = rep(paste0("Gen", 1:4), each = n4)
)
shifts <- c(4.2, 4.0, 3.8, 3.6)
slopes <- c(-0.040, -0.042, -0.044, -0.046)
gen_id <- match(dat4$generation, paste0("Gen", 1:4))
eta4 <- shifts[gen_id] + slopes[gen_id] * dat4$age
dat4$pass <- rbinom(nrow(dat4), size = 1, prob = plogis(eta4))

fit4 <- prob_agree(
  dat4,
  response = "pass",
  predictor = "age",
  group = "generation",
  limits = c(-0.03, 0.05),
  link = "logit",
  ci = FALSE
)
print(fit4)
plot(fit4)

}
\references{
Stevens, N. T. and Anderson-Cook, C. M. (2017). Comparing the Reliability of
Related Populations With the Probability of Agreement. \emph{Technometrics},
59(3), 371-380. doi:10.1080/00401706.2016.1214180.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bicor.R
\name{bicor}
\alias{bicor}
\alias{diag.bicor}
\alias{print.bicor}
\alias{plot.bicor}
\alias{summary.bicor}
\alias{print.summary.bicor}
\title{Biweight mid-correlation (bicor)}
\usage{
bicor(
  data,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  c_const = 9,
  max_p_outliers = 1,
  pearson_fallback = c("hybrid", "none", "all"),
  mad_consistent = FALSE,
  w = NULL,
  sparse_threshold = NULL
)

diag.bicor(x, ...)

\method{print}{bicor}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  show_ci = NULL,
  na_print = "NA",
  ...
)

\method{plot}{bicor}(
  x,
  title = "Biweight mid-correlation heatmap",
  reorder = c("none", "hclust"),
  triangle = c("full", "lower", "upper"),
  low_color = "indianred1",
  mid_color = "white",
  high_color = "steelblue1",
  value_text_size = 3,
  ci_text_size = 2.5,
  show_value = TRUE,
  na_fill = "grey90",
  ...
)

\method{summary}{bicor}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.bicor}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame containing numeric columns.
Factors, logicals and common time classes are dropped in the data-frame
path. Missing values are not allowed unless \code{na_method = "pairwise"}.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each estimate on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach approximate
Fisher-z confidence intervals for the off-diagonal biweight
mid-correlations. This confidence interval is provided as an additional
large-sample approximation.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{c_const}{Positive numeric. Tukey biweight tuning constant applied to the
\emph{raw} MAD; default \code{9} (Langfelder & Horvath's convention).}

\item{max_p_outliers}{Numeric in \code{(0, 1]}. Optional cap on the maximum
proportion of outliers \emph{on each side}; if \code{< 1}, side-specific
rescaling maps those quantiles to \code{|u|=1}. Use \code{1} to disable.}

\item{pearson_fallback}{Character scalar indicating the fallback policy.
One of:
\itemize{
\item \code{"hybrid"} (default): if a column has MAD = 0, that column
uses Pearson standardisation, yielding a hybrid correlation.
\item \code{"none"}: return \code{NA} if a column has MAD = 0 or becomes
degenerate after weighting.
\item \code{"all"}: force ordinary Pearson for all columns.
}}

\item{mad_consistent}{Logical; if \code{TRUE}, use the normal-consistent MAD
(\code{MAD_raw * 1.4826}) in the bicor weights. Default \code{FALSE} to
match Langfelder & Horvath (2012).}

\item{w}{Optional non-negative numeric vector of length \code{nrow(data)}
giving \emph{row weights}. When supplied, weighted medians/MADs are used
and Tukey weights are multiplied by \code{w} before normalisation.}

\item{sparse_threshold}{Optional numeric \eqn{\geq 0}. If supplied, sets
entries with \code{|r| < sparse_threshold} to 0 and returns a sparse
\code{"ddiMatrix"} (requires \pkg{Matrix}).}

\item{x}{An object of class \code{summary.bicor}.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other layers.}

\item{digits}{Integer; number of decimal places used for the matrix.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{ci_digits}{Integer; digits for bicor confidence limits in the pairwise
summary.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{na_print}{Character; how to display missing values.}

\item{title}{Plot title. Default is \code{"Biweight mid-correlation heatmap"}.}

\item{reorder}{Character; one of \code{"none"} (default) or \code{"hclust"}.
If \code{"hclust"}, variables are reordered by complete-linkage clustering
on the distance \eqn{d = 1 - r}, after replacing \code{NA} by 0 for
clustering purposes only.}

\item{triangle}{One of \code{"full"} (default), \code{"lower"}, or \code{"upper"}
to display the full matrix or a single triangle.}

\item{low_color, mid_color, high_color}{Colours for the gradient in
\code{scale_fill_gradient2}. Defaults are \code{"indianred1"},
\code{"white"}, \code{"steelblue1"}.}

\item{value_text_size}{Numeric; font size for cell labels. Set to \code{NULL}
to suppress labels (recommended for large matrices).}

\item{ci_text_size}{Text size for confidence-interval labels in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{na_fill}{Fill colour for \code{NA} cells. Default \code{"grey90"}.}

\item{object}{An object of class \code{bicor}.}

\item{p_digits}{Integer; digits for bicor p-values in the pairwise summary.}
}
\value{
A symmetric correlation matrix with class \code{bicor}
(or a \code{dgCMatrix} if \code{sparse_threshold} is used), with attributes:
\code{method = "biweight_mid_correlation"}, \code{description},
and \code{package = "matrixCorr"}. Downstream code should be prepared to
handle either a dense numeric matrix or a sparse \code{dgCMatrix}. When
\code{ci = TRUE}, the object also carries a \code{ci} attribute with
elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and
\code{ci.method}, together with an \code{inference} attribute containing
the standard large-sample summary matrices \code{estimate},
\code{statistic}, \code{p_value}, \code{Z}, and \code{n_obs}. Pairwise
complete-case counts are stored in \code{attr(x, "diagnostics")$n_complete}.
Internally, all medians/MADs, Tukey weights, optional pairwise-NA handling,
and OpenMP loops are implemented in the C++ helpers
(\code{bicor_*_cpp()}), so the R wrapper mostly validates arguments and
dispatches to the appropriate backend.

Invisibly returns \code{x}.

A \code{ggplot} object.
}
\description{
Computes pairwise biweight mid-correlations for numeric data. Bicor is a
robust, Pearson-like correlation that down-weights outliers and heavy-tailed
observations. Optional large-sample confidence intervals are available as a
derived feature.
}
\details{
For a column \eqn{x = (x_a)_{a=1}^m}, let \eqn{\mathrm{med}(x)} be the median and
\eqn{\mathrm{MAD}(x) = \mathrm{med}(|x - \mathrm{med}(x)|)} the (raw) median
absolute deviation. If \code{mad_consistent = TRUE}, the consistent scale
\eqn{\mathrm{MAD}^\star(x) = 1.4826\,\mathrm{MAD}(x)} is used. With tuning constant
\eqn{c>0}, define
\deqn{u_a = \frac{x_a - \mathrm{med}(x)}{c\,\mathrm{MAD}^{(\star)}(x)}.}
The Tukey biweight gives per-observation weights
\deqn{w_a = (1 - u_a^2)^2\,\mathbf{1}\{|u_a| < 1\}.}
Robust standardisation of a column is
\deqn{\tilde x_a =
\frac{(x_a - \mathrm{med}(x))\,w_a}{
      \sqrt{\sum_{b=1}^m \big[(x_b - \mathrm{med}(x))\,w_b\big]^2}}.}
For two columns \eqn{x,y}, the biweight mid-correlation is
\deqn{\mathrm{bicor}(x,y) = \sum_{a=1}^m \tilde x_a\,\tilde y_a \in [-1,1].}

\strong{Capping the maximum proportion of outliers (\code{max_p_outliers}).}
If \code{max_p_outliers < 1}, let \eqn{q_L = Q_x(\text{max\_p\_outliers})} and
\eqn{q_U = Q_x(1-\text{max\_p\_outliers})} be the lower/upper quantiles of \eqn{x}.
If the corresponding \eqn{|u|} at either quantile exceeds 1, \eqn{u} is rescaled
\emph{separately} on the negative and positive sides so that those quantiles land at
\eqn{|u|=1}. This guarantees that all observations between the two quantiles receive
positive weight. Note the bound applies per side, so up to \eqn{2\,\text{max\_p\_outliers}}
of observations can be treated as outliers overall.

\strong{Fallback when for zero MAD / degeneracy (\code{pearson_fallback}).}
If a column has \eqn{\mathrm{MAD}=0} or the robust denominator becomes zero,
the following rules apply:
\itemize{
\item \code{"none"} when correlations involving that column are \code{NA} (diagonal
remains 1).
\item \code{"hybrid"} when only the affected column switches to Pearson standardisation
\eqn{\bar x_a = (x_a - \overline{x}) / \sqrt{\sum_b (x_b - \overline{x})^2}},
yielding the hybrid correlation
\deqn{\mathrm{bicor}_{\mathrm{hyb}}(x,y) = \sum_a \bar x_a\,\tilde y_a,}
with the other column still robust-standardised.
\item \code{"all"} when all columns use ordinary Pearson standardisation; the result
equals \code{stats::cor(..., method="pearson")} when the NA policy matches.
}

\strong{Handling missing values (\code{na_method}).}
\itemize{
\item \code{"error"} (default): inputs must be finite; this yields a symmetric,
positive semidefinite (PSD) matrix since \eqn{R = \tilde X^\top \tilde X}.
\item \code{"pairwise"}: each \eqn{R_{jk}} is computed on the intersection of
rows where both columns are finite. Pairs with fewer than 5 overlapping
rows return \code{NA} (guarding against instability). Pairwise deletion can
break PSD, as in the Pearson case.
}

\strong{Row weights (\code{w}).}
When \code{w} is supplied (non-negative, length \eqn{m}), the weighted median
\eqn{\mathrm{med}_w(x)} and weighted MAD
\eqn{\mathrm{MAD}_w(x) = \mathrm{med}_w(|x - \mathrm{med}_w(x)|)} are used to form
\eqn{u}. The Tukey weights are then multiplied by the observation weights prior
to normalisation:
\deqn{\tilde x_a =
\frac{(x_a - \mathrm{med}_w(x))\,w_a\,w^{(\mathrm{obs})}_a}{
      \sqrt{\sum_b \big[(x_b - \mathrm{med}_w(x))\,w_b\,w^{(\mathrm{obs})}_b\big]^2}},}
where \eqn{w^{(\mathrm{obs})}_a \ge 0} are the user-supplied row weights and \eqn{w_a}
are the Tukey biweights built from the weighted median/MAD. Weighted pairwise
behaves analogously on each column pair's overlap.

\strong{MAD choice (\code{mad_consistent}).}
Setting \code{mad_consistent = TRUE} multiplies the raw MAD by 1.4826 inside
\eqn{u}. Equivalently, it uses an effective tuning constant
\eqn{c^\star = c \times 1.4826}. The default \code{FALSE} reproduces the convention
in Langfelder & Horvath (2012).

\strong{Optional sparsification (\code{sparse_threshold}).}
If provided, entries with \eqn{|r| < \text{sparse\_threshold}} are set to 0 and the
result is returned as a \code{"ddiMatrix"} (diagonal is forced to 1). This is a
post-processing step that does not alter the per-pair estimates.

\strong{Computation and threads.}
Columns are robust-standardised in parallel and the matrix is formed as
\eqn{R = \tilde X^\top \tilde X}. \code{n_threads} selects the number of OpenMP
threads; by default it uses \code{getOption("matrixCorr.threads", 1L)}.

\strong{Large-sample inference.}
For a pairwise estimate \eqn{r} computed from \eqn{n_{jk}} observed rows, the
standard large-sample summaries use
\deqn{
Z_{jk} = \frac{1}{2}\log\!\left(\frac{1 + r}{1 - r}\right)\sqrt{n_{jk} - 2}
}
and
\deqn{
T_{jk} = \sqrt{n_{jk} - 2}\;\frac{|r|}{\sqrt{1-r^2}}.
}
The reported p-value is the two-sided Student-\eqn{t} tail probability with
\eqn{n_{jk}-2} degrees of freedom. When \code{ci = TRUE}, the package also
reports an approximate Fisher-z confidence interval obtained from
\deqn{
z_{jk} = \operatorname{atanh}(r), \qquad
\operatorname{SE}(z_{jk}) = \frac{1}{\sqrt{n_{jk}-3}},
}
followed by back-transformation with \code{tanh()}. Confidence intervals are currently available only for dense,
unweighted outputs.

\strong{Basic properties.}
\eqn{\mathrm{bicor}(a x + b,\; c y + d) = \mathrm{sign}(ac)\,\mathrm{bicor}(x,y)}.
With no missing data (and with per-column hybrid/robust standardisation), the
output is symmetric and PSD. As with Pearson, affine equivariance does not hold
for the associated biweight midcovariance.
}
\examples{
set.seed(1)
X <- matrix(rnorm(2000 * 40), 2000, 40)
R <- bicor(X, c_const = 9, max_p_outliers = 1,
                       pearson_fallback = "hybrid")
print(attr(R, "method"))
summary(R)
R_ci <- bicor(X[, 1:5], ci = TRUE)
summary(R_ci)
estimate(R_ci)
tidy(R_ci)
ci(R_ci)
confint(R_ci)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(R)
}

}
\references{
Langfelder, P. & Horvath, S. (2012).
Fast R Functions for Robust Correlations and Hierarchical Clustering.
Journal of Statistical Software, 46(11), 1-17. \doi{10.18637/jss.v046.i11}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view_corr_shiny.R
\name{view_corr_shiny}
\alias{view_corr_shiny}
\title{Interactive Shiny viewer for matrixCorr objects}
\usage{
view_corr_shiny(x, title = NULL, default_max_vars = 40L)
}
\arguments{
\item{x}{A correlation result, a numeric matrix, or a named list of those
objects. Each element must be square with matching row/column names.}

\item{title}{Optional character title shown at the top of the gadget.}

\item{default_max_vars}{Integer; maximum number of variables pre-selected
when the app opens. Defaults to 40 so very wide matrices start collapsed.}
}
\value{
Invisibly returns \code{NULL}; the function is called for its side
effect of launching a Shiny gadget.
}
\description{
Launches an interactive Shiny gadget that displays correlation heatmaps with
filtering, clustering, and hover inspection. The viewer accepts any
matrixCorr correlation result (for example the outputs from
\code{\link[=pearson_corr]{pearson_corr()}}, \code{\link[=spearman_rho]{spearman_rho()}}, \code{\link[=kendall_tau]{kendall_tau()}}, \code{\link[=bicor]{bicor()}},
\code{\link[=pbcor]{pbcor()}}, \code{\link[=wincor]{wincor()}}, \code{\link[=skipped_corr]{skipped_corr()}}, \code{\link[=pcorr]{pcorr()}}, \code{\link[=dcor]{dcor()}}, or
\code{\link[=shrinkage_corr]{shrinkage_corr()}}), a plain
matrix, or a named list of such objects. When a list is supplied the gadget
offers a picker to switch between results.
}
\details{
This helper lives in \code{Suggests}; it requires the \code{shiny} and
\code{shinyWidgets} packages at runtime and will optionally convert the plot to
an interactive widget when \code{plotly} is installed. Variable selection uses
a searchable picker, and clustering controls let you reorder variables via
hierarchical clustering on either absolute or signed correlations with a
choice of linkage methods.
}
\examples{
if (interactive()) {
  data <- mtcars
  results <- list(
    Pearson = pearson_corr(data),
    Spearman = spearman_rho(data),
    Kendall = kendall_tau(data)
  )
  view_corr_shiny(results)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcor.R
\name{dcor}
\alias{dcor}
\alias{print.dcor}
\alias{plot.dcor}
\alias{summary.dcor}
\alias{print.summary.dcor}
\title{Pairwise Distance Correlation (dCor)}
\usage{
dcor(
  data,
  na_method = c("error", "pairwise", "complete"),
  p_value = FALSE,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{dcor}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{dcor}(
  x,
  title = "Distance correlation heatmap",
  low_color = "white",
  high_color = "steelblue1",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{dcor}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.dcor}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns are dropped. Columns must be numeric.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each association on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
pairwise p-values, test statistics, and degrees of freedom from the
distance-correlation t-test of independence.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()} or other
\code{ggplot2} layers.}

\item{x}{An object of class \code{summary.dcor}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Distance correlation heatmap"}.}

\item{low_color}{Colour for zero correlation. Default is \code{"white"}.}

\item{high_color}{Colour for strong correlation. Default is \code{"steelblue1"}.}

\item{value_text_size}{Font size for displaying values. Default is \code{4}.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{dcor}.}
}
\value{
A symmetric numeric matrix where the \code{(i, j)} entry is the
unbiased distance correlation between the \code{i}-th and \code{j}-th
numeric columns. The object has class \code{dcor} with attributes
\code{method = "distance_correlation"}, \code{description}, and
\code{package = "matrixCorr"}. When \code{p_value = TRUE}, the object also
carries an \code{inference} attribute with matrices \code{estimate},
\code{statistic}, \code{parameter}, and \code{p_value}, plus
\code{attr(x, "diagnostics")$n_complete}. The main returned matrix remains
the usual non-negative unbiased distance-correlation estimate.

Invisibly returns \code{x}.

A \code{ggplot} object representing the heatmap.
}
\description{
Computes pairwise distance correlations for the numeric columns of a matrix
or data frame using a high-performance 'C++' backend. Distance correlation
detects general dependence, including non-linear relationships. Optional
p-values are available via the bias-corrected distance-correlation t-test.
}
\details{
Let \eqn{x \in \mathbb{R}^n} and \eqn{D^{(x)}} be the pairwise distance matrix
with zero diagonal: \eqn{D^{(x)}_{ii} = 0}, \eqn{D^{(x)}_{ij} = |x_i - x_j|} for
\eqn{i \neq j}. Define row sums \eqn{r^{(x)}_i = \sum_{k \neq i} D^{(x)}_{ik}} and
grand sum \eqn{S^{(x)} = \sum_{i \neq k} D^{(x)}_{ik}}. The U-centred matrix is
\deqn{A^{(x)}_{ij} =
      \begin{cases}
        D^{(x)}_{ij} - \dfrac{r^{(x)}_i + r^{(x)}_j}{n - 2}
        + \dfrac{S^{(x)}}{(n - 1)(n - 2)}, & i \neq j,\\[6pt]
        0, & i = j~.
      \end{cases}}
For two variables \eqn{x,y}, the unbiased distance covariance and variances are
\deqn{\widehat{\mathrm{dCov}}^2_u(x,y) = \frac{2}{n(n-3)} \sum_{i<j} A^{(x)}_{ij} A^{(y)}_{ij}
\;=\; \frac{1}{n(n-3)} \sum_{i \neq j} A^{(x)}_{ij} A^{(y)}_{ij},}
with \eqn{\widehat{\mathrm{dVar}}^2_u(x)} defined analogously from \eqn{A^{(x)}}.
The unbiased distance correlation is
\deqn{\widehat{\mathrm{dCor}}_u(x,y) =
      \frac{\widehat{\mathrm{dCov}}_u(x,y)}
           {\sqrt{\widehat{\mathrm{dVar}}_u(x)\,\widehat{\mathrm{dVar}}_u(y)}} \in [0,1].}

\strong{Computation.} All heavy lifting (distance matrices, U-centering,
and unbiased scaling) is implemented in C++ (\code{ustat_dcor_matrix_cpp}),
so the R wrapper only validates/coerces the input. OpenMP parallelises the
upper-triangular loops. The implementation includes a Huo-Szekely style
univariate \eqn{O(n \log n)} dispatch for pairwise terms. We also have an exact
unbiased \eqn{O(n^2)} fallback retained for robustness in small-sample or
non-finite-path cases; no external dependencies are used.

\strong{Inference.} When \code{p_value = TRUE}, the package computes the
bias-corrected distance-correlation t-test of independence of Szekely and
Rizzo (2013). Let \eqn{\widehat{\mathrm{dCor}}^\ast(x,y)} denote the signed
bias-corrected distance correlation used internally by the test (that is,
the same ratio before the package's usual clipping to \eqn{[0,1]}). With
\deqn{M = \frac{n(n-3)}{2},}
the test statistic is
\deqn{
T = \sqrt{M - 1}\;
\frac{\widehat{\mathrm{dCor}}^\ast(x,y)}
     {\sqrt{1 - \{\widehat{\mathrm{dCor}}^\ast(x,y)\}^2}},
}
referenced to a Student \eqn{t}-distribution with \eqn{M - 1} degrees of
freedom. The reported p-value uses the upper-tail probability
\eqn{P(t_{M-1} \ge T)}. This inference payload is attached as metadata; the
main returned matrix is unchanged unless \code{p_value} is explicitly
requested. The t reference is an asymptotic approximation. For small
complete-case sample sizes, especially when \eqn{M - 1} is small, p-values
can be unstable and should be interpreted cautiously; a permutation-based
dependence test is preferable when exact small-sample calibration matters.
}
\note{
Requires \eqn{n \ge 4}. Columns with (near) zero unbiased distance
variance yield \code{NA} in their row/column. Typical per-pair cost uses
the \eqn{O(n \log n)} fast path, with \eqn{O(n^2)} fallback when needed.
}
\examples{
\donttest{
## Independent variables -> dCor ~ 0
set.seed(1)
X <- cbind(a = rnorm(200), b = rnorm(200))
D <- dcor(X)
print(D, digits = 3)
summary(D)

## Non-linear dependence: Pearson ~ 0, but unbiased dCor > 0
set.seed(42)
n <- 200
x <- rnorm(n)
y <- x^2 + rnorm(n, sd = 0.2)
XY <- cbind(x = x, y = y)
D2 <- dcor(XY)
# Compare Pearson vs unbiased distance correlation
round(c(pearson = cor(XY)[1, 2], dcor = D2["x", "y"]), 3)
summary(D2)
plot(D2, title = "Unbiased distance correlation (non-linear example)")

## Small AR(1) multivariate normal example
set.seed(7)
p <- 5; n <- 150; rho <- 0.6
Sigma <- rho^abs(outer(seq_len(p), seq_len(p), "-"))
X3 <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
colnames(X3) <- paste0("V", seq_len(p))
D3 <- dcor(X3)
print(D3[1:3, 1:3], digits = 2)

## Optional inference
D4 <- dcor(XY, p_value = TRUE)
summary(D4)
estimate(D4)
tidy(D4)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(D)
}
}

}
\references{
Szekely, G. J., Rizzo, M. L., & Bakirov, N. K. (2007).
Measuring and testing dependence by correlation of distances.
\emph{Annals of Statistics}, 35(6), 2769-2794.

Szekely, G. J., & Rizzo, M. L. (2013).
The distance correlation t-test of independence.
\emph{Journal of Multivariate Analysis}, 117, 193-213.

Rizzo, M. L., & Szekely, G. J. (2024). \pkg{energy}: E-statistics
(energy statistics). R package version 1.7-12.
}
\author{
Thiago de paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_result_s3.R
\name{summary.corr_edge_list}
\alias{summary.corr_edge_list}
\title{S3 Summary for Edge-List Correlation Results}
\usage{
\method{summary}{corr_edge_list}(object, topn = NULL, show_ci = NULL, ...)
}
\arguments{
\item{object}{An edge-list correlation result.}

\item{topn}{Optional number of head/tail rows when preview is truncated.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Unused.}
}
\value{
A standardized summary data frame with class
\code{c("summary.corr_result", "data.frame")} (plus compatibility classes).
}
\description{
Representation-first summary for edge-list outputs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmcorr.R
\name{rmcorr}
\alias{rmcorr}
\alias{rmcorr_weighted}
\title{Repeated-Measures Correlation (rmcorr)}
\usage{
rmcorr(
  data = NULL,
  response,
  subject,
  na_method = c("error", "pairwise", "complete"),
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  keep_data = FALSE,
  verbose = FALSE,
  estimator = c("ancova", "weighted"),
  ci_method = c("auto", "fisher_z", "bootstrap", "none"),
  n_boot = 999L,
  seed = NULL,
  ...
)

rmcorr_weighted(data = NULL, response, subject, ...)
}
\arguments{
\item{data}{Optional \code{data.frame}/matrix containing the repeated-measures
dataset.}

\item{response}{Either:
\itemize{
\item a character vector of at least two column names in \code{data}, or
\item a numeric matrix/data frame with at least two columns.
}
If exactly two responses are supplied, the function returns a pairwise
repeated-measures correlation object of class \code{"rmcorr"}. If three or
more responses are supplied, a symmetric matrix of class
\code{"rmcorr_matrix"} is returned.}

\item{subject}{Subject identifier (factor/character/integer/numeric) or a
single character string naming the subject column in \code{data}.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values in the selected response columns or
\code{subject}. \code{"pairwise"} uses pairwise complete cases and drops
subjects with fewer than two complete pairs for the specific contrast.
\code{"complete"} removes rows with missing or non-finite values in
\code{subject} and all selected response variables before fitting.}

\item{conf_level}{Confidence level used for Wald confidence intervals on the
repeated-measures correlation (default \code{0.95}).}

\item{n_threads}{Integer \eqn{\ge 1}. Number of OpenMP threads used by the
C++ backend in matrix mode. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{keep_data}{Logical (default \code{FALSE}). If \code{TRUE}, returned
objects retain a compact copy of the numeric response matrix plus encoded
subject identifiers. For pairwise fits this enables \code{plot()} to
reconstruct subject-level fitted lines lazily; for matrix outputs it also
allows \code{view_rmcorr_shiny()} to rebuild pairwise scatterplots from the
returned object alone.}

\item{verbose}{Logical; if \code{TRUE}, prints a short note about the thread
count used in matrix mode.}

\item{estimator}{Character scalar selecting the repeated-measures
correlation estimator. \code{"ancova"} (default) is the classical
subject-centred repeated-measures correlation. \code{"weighted"} is the
weighted repeated-measures correlation coefficient of Kondo et al. (2025),
designed for incomplete or unbalanced repeated-measurement schedules.}

\item{ci_method}{Confidence-interval engine. \code{"auto"} (default)
resolves to \code{"fisher_z"} for \code{estimator = "ancova"} and to
subject-level \code{"bootstrap"} for \code{estimator = "weighted"} when
\code{n_boot > 0}; otherwise \code{"none"}. \code{"none"} returns
\code{NA} bounds.}

\item{n_boot}{Integer \eqn{\ge 0}. Number of subject-level bootstrap
resamples when \code{ci_method = "bootstrap"}.}

\item{seed}{Optional positive integer seed used for bootstrap
reproducibility.}

\item{...}{Deprecated compatibility aliases. The legacy \code{check_na}
argument is still accepted here temporarily.}
}
\value{
Either a \code{"rmcorr"} object (exactly two responses) or a
\code{"rmcorr_matrix"} object (pairwise results when \eqn{\ge}3 responses).

\strong{If \code{"rmcorr"} (exactly two responses)}, outputs include:
\itemize{
\item \code{estimate}; repeated-measures correlation estimate.
\item \code{p_value}; two-sided p-value for the common within-subject slope.
\item \code{lwr}, \code{upr}; confidence interval limits for
\code{estimate}.
\item \code{slope}; common within-subject slope.
\item \code{df}; residual degrees of freedom \eqn{N - S - 1}.
\item \code{n_obs}; number of complete observations retained after
dropping subjects with fewer than two repeated pairs.
\item \code{n_subjects}; number of contributing subjects.
\item \code{responses}; names of the fitted response variables.
\item compatibility aliases \code{r}, \code{conf_int}, and \code{based.on}
are reconstructed on access without duplicate storage.
\item when \code{keep_data = TRUE}, compact source data are retained so
\code{plot()} can lazily reconstruct \code{data_long},
\code{intercepts}, and fitted lines; these are otherwise not stored.
}

\strong{If \code{"rmcorr_matrix"} (\eqn{\ge}3 responses)}, outputs are:
\itemize{
\item a symmetric numeric matrix of pairwise repeated-measures correlations.
\item attributes \code{method}, \code{description}, and
\code{package = "matrixCorr"}.
\item \code{diagnostics}; a list with square matrices for \code{slope},
\code{p_value}, \code{df}, \code{n_complete}, \code{n_subjects},
\code{conf_low}, and \code{conf_high}, plus scalar
\code{conf_level}.
}
}
\description{
Computes repeated-measures correlation for two or more continuous responses
observed repeatedly within subjects. Supply a \code{data.frame} plus column
names, or pass the response matrix/data frame and subject vector directly.
The repeated observations are indexed only by \code{subject}, thus no explicit
time variable is modeled, and the method targets a common within-subject
linear association after removing subject-specific means.
}
\details{
Repeated-measures correlation estimates the common \emph{within-subject}
linear association between two variables measured repeatedly on the same
subjects. It differs from agreement methods such as Lin's CCC or
Bland-Altman analysis because those target concordance or interchangeability,
whereas repeated-measures correlation targets the strength of the
subject-centred association.

For subject \eqn{i = 1,\ldots,S} and repeated observations
\eqn{j = 1,\ldots,n_i}, let \eqn{x_{ij}} and \eqn{y_{ij}} denote the two
responses. Define subject-specific means
\deqn{\bar x_i = \frac{1}{n_i}\sum_{j=1}^{n_i} x_{ij}, \qquad
      \bar y_i = \frac{1}{n_i}\sum_{j=1}^{n_i} y_{ij}.}
The repeated-measures correlation uses within-subject centred values
\deqn{x^\ast_{ij} = x_{ij} - \bar x_i, \qquad
      y^\ast_{ij} = y_{ij} - \bar y_i}
and computes
\deqn{r_{\mathrm{rm}} =
      \frac{\sum_i \sum_j x^\ast_{ij} y^\ast_{ij}}
           {\sqrt{\left(\sum_i \sum_j {x^\ast_{ij}}^2\right)
                  \left(\sum_i \sum_j {y^\ast_{ij}}^2\right)}}.}

Equivalently, this is the correlation implied by an ANCOVA model with a
common slope and subject-specific intercepts:
\deqn{y_{ij} = \alpha_i + \beta x_{ij} + \varepsilon_{ij}.}
The returned slope is
\deqn{\hat\beta =
      \frac{\sum_i \sum_j x^\ast_{ij} y^\ast_{ij}}
           {\sum_i \sum_j {x^\ast_{ij}}^2},}
and the subject-specific fitted intercepts are
\eqn{\hat\alpha_i = \bar y_i - \hat\beta \bar x_i}. Residual degrees of
freedom are \eqn{N - S - 1}, where \eqn{N = \sum_i n_i} after filtering to
complete observations and retaining only subjects with at least two repeated
pairs.

Confidence intervals are computed with a Fisher \eqn{z}-transformation of
\eqn{r_{\mathrm{rm}}} and then back-transformed to the correlation scale. In
matrix mode, the same estimator is applied to every pair of selected
response columns.

With \code{estimator = "weighted"}, the package implements the weighted
repeated-measures estimator from Kondo et al. (2025) using complete observed
\eqn{(x,y)} pairs per contrast. This estimator does not impute missing data;
it uses per-subject complete-pair sets and weighted sums of explained and
residual quantities from the fixed-subject ANCOVA decomposition.

Bootstrap confidence intervals use non-parametric subject-level resampling
(subjects are resampled as blocks, not rows).
}
\examples{
set.seed(2026)
n_subjects <- 20
n_rep <- 4
subject <- rep(seq_len(n_subjects), each = n_rep)
subj_eff_x <- rnorm(n_subjects, sd = 1.5)
subj_eff_y <- rnorm(n_subjects, sd = 2.0)
within_signal <- rnorm(n_subjects * n_rep)

dat <- data.frame(
  subject = subject,
  x = subj_eff_x[subject] + within_signal + rnorm(n_subjects * n_rep, sd = 0.2),
  y = subj_eff_y[subject] + 0.8 * within_signal + rnorm(n_subjects * n_rep, sd = 0.3),
  z = subj_eff_y[subject] - 0.4 * within_signal + rnorm(n_subjects * n_rep, sd = 0.4)
)

fit_xy <- rmcorr(dat, response = c("x", "y"), subject = "subject", keep_data = TRUE)
print(fit_xy)
summary(fit_xy)
estimate(fit_xy)
tidy(fit_xy)
ci(fit_xy)
confint(fit_xy)
plot(fit_xy)

fit_mat <- rmcorr(dat, response = c("x", "y", "z"), subject = "subject")
print(fit_mat, digits = 3)
summary(fit_mat)
tidy(fit_mat)
plot(fit_mat)

if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  fit_mat_view <- rmcorr(
    dat,
    response = c("x", "y", "z"),
    subject = "subject",
    keep_data = TRUE
  )
  view_rmcorr_shiny(fit_mat_view)
}
}
\references{
Bakdash, J. Z., & Marusich, L. R. (2017). Repeated Measures Correlation.
\emph{Frontiers in Psychology}, 8, 456.
\doi{10.3389/fpsyg.2017.00456}

Kondo, M., Nagashima, K., Isono, S., & Sato, Y. (2025). Weighted Repeated
Measures Correlation Coefficient: A New Correlation Coefficient for Handling
Missing Data With Repeated Measures. \emph{Statistics in Medicine},
44(10-12), e70046. \doi{10.1002/sim.70046}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{ccc_lmm_reml_pairwise}
\alias{ccc_lmm_reml_pairwise}
\title{ccc_lmm_reml_pairwise}
\usage{
ccc_lmm_reml_pairwise(
  df,
  fml,
  response,
  rind,
  method,
  time,
  slope,
  slope_var,
  slope_Z,
  drop_zero_cols,
  Dmat,
  ar,
  ar_rho,
  max_iter,
  tol,
  conf_level,
  verbose,
  digits,
  use_message,
  extra_label,
  ci,
  ci_mode_int,
  all_time_lvls,
  Dmat_type = c("time-avg", "typical-visit", "weighted-avg", "weighted-sq"),
  Dmat_weights = NULL,
  Dmat_rescale = TRUE,
  vc_select = c("auto", "none"),
  vc_alpha = 0.05,
  vc_test_order = c("subj_time", "subj_method"),
  include_subj_method = NULL,
  include_subj_time = NULL,
  sb_zero_tol = 1e-10,
  metric_mode = 0L,
  estimate_key = "ccc",
  se_key = "se_ccc",
  out_class = "ccc_rm_reml",
  out_method = "Variance Components REML - pairwise",
  out_description = "Lin's CCC per method pair from random-effects LMM",
  summary_title = "Repeated-measures concordance matrix",
  vc_engine_name = "ccc_rm_reml",
  vc_se_label = "SE(CCC)"
)
}
\description{
Internal function to handle pairwise CCC estimation for each method pair.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated-matrixCorr}
\alias{deprecated-matrixCorr}
\alias{bland_altman}
\alias{bland_altman_repeated}
\alias{biweight_mid_corr}
\alias{distance_corr}
\alias{partial_correlation}
\alias{ccc_lmm_reml}
\alias{ccc_pairwise_u_stat}
\title{Deprecated Compatibility Wrappers}
\usage{
bland_altman(
  group1,
  group2,
  two = 1.96,
  mode = 1L,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE
)

bland_altman_repeated(
  data = NULL,
  response,
  subject,
  method,
  time,
  two = 1.96,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  include_slope = FALSE,
  use_ar1 = FALSE,
  ar1_rho = NA_real_,
  max_iter = 200L,
  tol = 1e-06,
  verbose = FALSE
)

biweight_mid_corr(
  data,
  c_const = 9,
  max_p_outliers = 1,
  pearson_fallback = c("hybrid", "none", "all"),
  na_method = c("error", "pairwise", "complete"),
  mad_consistent = FALSE,
  w = NULL,
  sparse_threshold = NULL,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

distance_corr(data, na_method = c("error", "pairwise", "complete"), ...)

partial_correlation(
  data,
  method = c("oas", "ridge", "sample"),
  lambda = 0.001,
  return_cov_precision = FALSE,
  ci = FALSE,
  conf_level = 0.95,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

ccc_lmm_reml(
  data,
  response,
  rind,
  method = NULL,
  time = NULL,
  interaction = FALSE,
  max_iter = 100,
  tol = 1e-06,
  Dmat = NULL,
  Dmat_type = c("time-avg", "typical-visit", "weighted-avg", "weighted-sq"),
  Dmat_weights = NULL,
  Dmat_rescale = TRUE,
  ci = FALSE,
  conf_level = 0.95,
  ci_mode = c("auto", "raw", "logit"),
  verbose = FALSE,
  digits = 4,
  use_message = TRUE,
  ar = c("none", "ar1"),
  ar_rho = NA_real_,
  slope = c("none", "subject", "method", "custom"),
  slope_var = NULL,
  slope_Z = NULL,
  drop_zero_cols = TRUE,
  vc_select = c("auto", "none"),
  vc_alpha = 0.05,
  vc_test_order = c("subj_time", "subj_method"),
  include_subj_method = NULL,
  include_subj_time = NULL,
  sb_zero_tol = 1e-10
)

ccc_pairwise_u_stat(
  data,
  response,
  method,
  subject,
  time = NULL,
  Dmat = NULL,
  delta = 1,
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE
)
}
\arguments{
\item{group1, group2}{Numeric vectors of equal length.}

\item{two}{Positive scalar; the multiple of the standard deviation used to
define the limits of agreement.}

\item{mode}{Integer; 1 uses \code{group1 - group2}, 2 uses \code{group2 - group1}.}

\item{conf_level}{Confidence level.}

\item{n_threads}{Integer number of OpenMP threads.}

\item{verbose}{Logical; print brief progress or diagnostic output.}

\item{data}{A \code{data.frame}, matrix, or repeated-measures dataset accepted by
the corresponding replacement function.}

\item{response}{Numeric response vector or column name, depending on the
target method.}

\item{subject}{Subject identifier or subject column name.}

\item{method}{Method label or method column name.}

\item{time}{Replicate/time index or time column name.}

\item{include_slope}{Logical; whether to estimate proportional bias.}

\item{use_ar1}{Logical; whether to use AR(1) within-subject correlation.}

\item{ar1_rho}{AR(1) parameter.}

\item{max_iter, tol}{EM control parameters.}

\item{c_const}{Positive numeric Tukey biweight tuning constant.}

\item{max_p_outliers}{Numeric in \verb{(0, 1]}; optional cap on the maximum
proportion of outliers on each side.}

\item{pearson_fallback}{Character fallback policy used by \code{bicor()}.}

\item{na_method}{Missing-data policy forwarded to the replacement function
when supported.}

\item{mad_consistent}{Logical; if \code{TRUE}, uses the consistency-corrected MAD.}

\item{w}{Optional vector of case weights.}

\item{sparse_threshold}{Optional threshold controlling sparse output.}

\item{output}{Output representation for the computed estimates.}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs.}

\item{diag}{Logical; whether to include diagonal entries in non-matrix outputs.}

\item{...}{Additional arguments forwarded to the replacement function when
supported.}

\item{lambda}{Numeric regularisation strength used by \code{pcorr()}.}

\item{return_cov_precision}{Logical; if \code{TRUE}, also return covariance and
precision matrices.}

\item{ci}{Logical; if \code{TRUE}, request confidence intervals when supported by
the replacement function.}

\item{rind}{Character; column identifying subjects, forwarded as \code{subject}
to \code{ccc_rm_reml()}.}

\item{interaction}{Logical; forwarded to \code{ccc_rm_reml()}.}

\item{Dmat}{Optional distance matrix forwarded to \code{ccc_rm_reml()}.}

\item{Dmat_type}{Character selector controlling how \code{Dmat} is constructed.}

\item{Dmat_weights}{Optional weights used when \code{Dmat_type} requires them.}

\item{Dmat_rescale}{Logical; whether to rescale \code{Dmat}.}

\item{ci_mode}{Character selector for the confidence-interval scale used by
\code{ccc_rm_reml()}.}

\item{digits}{Display precision forwarded to \code{ccc_rm_reml()}.}

\item{use_message}{Logical; whether the deprecated wrapper emits a lifecycle
message.}

\item{ar}{Character selector for the within-subject residual correlation
model.}

\item{ar_rho}{Numeric AR(1) parameter.}

\item{slope}{Character selector for the proportional-bias slope structure.}

\item{slope_var}{Optional covariance matrix for custom slopes.}

\item{slope_Z}{Optional design matrix for custom slopes.}

\item{drop_zero_cols}{Logical; whether zero-variance design columns are
dropped.}

\item{vc_select}{Character selector controlling variance-component
selection.}

\item{vc_alpha}{Significance level used in variance-component selection.}

\item{vc_test_order}{Character vector controlling the variance-component
test order.}

\item{include_subj_method}{Optional logical override for the
subject-by-method component.}

\item{include_subj_time}{Optional logical override for the subject-by-time
component.}

\item{sb_zero_tol}{Numerical tolerance used when stabilising the scale-bias
term.}

\item{delta}{Numeric power exponent for U-statistics distances.}
}
\description{
Temporary wrappers for functions renamed in \code{matrixCorr} 1.0.0. These
wrappers preserve the pre-1.0 entry points while warning that they will be
removed in 2.0.0.
}
\details{
Renamed functions:
\itemize{
\item \code{bland_altman()} -> \code{ba()}
\item \code{bland_altman_repeated()} -> \code{ba_rm()}
\item \code{biweight_mid_corr()} -> \code{bicor()}
\item \code{distance_corr()} -> \code{dcor()}
\item \code{partial_correlation()} -> \code{pcorr()}
\item \code{ccc_lmm_reml()} -> \code{ccc_rm_reml()}
\item \code{ccc_pairwise_u_stat()} -> \code{ccc_rm_ustat()}
}

The deprecated wrappers will be removed in \code{matrixCorr} 2.0.0.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_prob_scalar}
\alias{check_prob_scalar}
\title{Check probability in unit interval}
\usage{
check_prob_scalar(x, arg = as.character(substitute(x)), open_ends = FALSE)
}
\description{
Check probability in unit interval
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{estimate}
\alias{estimate}
\alias{tidy}
\alias{ci}
\alias{estimate.corr_result}
\alias{estimate.summary.corr_result}
\alias{estimate.summary.matrixCorr}
\alias{coef.corr_result}
\alias{ci.corr_result}
\alias{ci.default}
\alias{ci.ba}
\alias{ci.ba_matrix}
\alias{ci.ba_repeated}
\alias{ci.ba_repeated_matrix}
\alias{ci.ccc}
\alias{ci.ccc_ci}
\alias{ci.ccc_glmm}
\alias{ci.cia}
\alias{ci.cia_ci}
\alias{ci.cia_rm}
\alias{ci.cohen_kappa}
\alias{ci.gwet_ac}
\alias{ci.icc}
\alias{ci.icc_overall}
\alias{ci.icc_rm_reml}
\alias{ci.krippendorff_alpha}
\alias{ci.multirater_kappa}
\alias{ci.partial_corr}
\alias{ci.prob_agree}
\alias{ci.rmcorr}
\alias{ci.rmcorr_matrix}
\alias{ci.weighted_kappa}
\alias{estimate.partial_corr}
\alias{estimate.ba}
\alias{estimate.ba_matrix}
\alias{estimate.ba_repeated}
\alias{estimate.ba_repeated_matrix}
\alias{estimate.ccc_glmm}
\alias{coef.ccc_glmm}
\alias{estimate.default}
\alias{tidy.corr_result}
\alias{tidy.summary.corr_result}
\alias{tidy.summary.matrixCorr}
\alias{tidy.partial_corr}
\alias{tidy.ba}
\alias{tidy.ba_matrix}
\alias{tidy.ba_repeated}
\alias{tidy.ba_repeated_matrix}
\alias{tidy.default}
\alias{confint.corr_result}
\alias{confint.summary.corr_result}
\alias{confint.summary.matrixCorr}
\alias{confint.ba}
\alias{confint.ba_repeated}
\alias{confint.ba_matrix}
\alias{confint.ba_repeated_matrix}
\alias{confint.ccc}
\alias{confint.ccc_ci}
\alias{confint.ccc_glmm}
\alias{confint.chatterjee_xi_scalar}
\alias{confint.cia}
\alias{confint.cia_ci}
\alias{confint.cia_rm}
\alias{confint.cohen_kappa}
\alias{confint.gwet_ac}
\alias{confint.icc}
\alias{confint.icc_overall}
\alias{confint.icc_rm_reml}
\alias{confint.krippendorff_alpha}
\alias{confint.multirater_kappa}
\alias{confint.prob_agree}
\alias{confint.partial_corr}
\alias{confint.rmcorr}
\alias{confint.rmcorr_matrix}
\alias{confint.weighted_kappa}
\title{Access MatrixCorr Estimates, Confidence Intervals, and Tidy Results}
\usage{
estimate(x, ...)

tidy(x, ...)

ci(x, ...)

\method{estimate}{corr_result}(x, ...)

\method{estimate}{summary.corr_result}(x, ...)

\method{estimate}{summary.matrixCorr}(x, ...)

\method{coef}{corr_result}(object, ...)

\method{ci}{corr_result}(x, ...)

\method{ci}{default}(x, ...)

\method{ci}{ba}(x, ...)

\method{ci}{ba_matrix}(x, ...)

\method{ci}{ba_repeated}(x, ...)

\method{ci}{ba_repeated_matrix}(x, ...)

\method{ci}{ccc}(x, ...)

\method{ci}{ccc_ci}(x, ...)

\method{ci}{ccc_glmm}(x, ...)

\method{ci}{cia}(x, ...)

\method{ci}{cia_ci}(x, ...)

\method{ci}{cia_rm}(x, ...)

\method{ci}{cohen_kappa}(x, ...)

\method{ci}{gwet_ac}(x, ...)

\method{ci}{icc}(x, ...)

\method{ci}{icc_overall}(x, ...)

\method{ci}{icc_rm_reml}(x, ...)

\method{ci}{krippendorff_alpha}(x, ...)

\method{ci}{multirater_kappa}(x, ...)

\method{ci}{partial_corr}(x, ...)

\method{ci}{prob_agree}(x, ...)

\method{ci}{rmcorr}(x, ...)

\method{ci}{rmcorr_matrix}(x, ...)

\method{ci}{weighted_kappa}(x, ...)

\method{estimate}{partial_corr}(x, ...)

\method{estimate}{ba}(x, ...)

\method{estimate}{ba_matrix}(x, ...)

\method{estimate}{ba_repeated}(x, ...)

\method{estimate}{ba_repeated_matrix}(x, ...)

\method{estimate}{ccc_glmm}(x, ...)

\method{coef}{ccc_glmm}(object, ...)

\method{estimate}{default}(x, ...)

\method{tidy}{corr_result}(x, diag = FALSE, triangle = c("upper", "lower", "full"), ...)

\method{tidy}{summary.corr_result}(x, ...)

\method{tidy}{summary.matrixCorr}(x, ...)

\method{tidy}{partial_corr}(x, diag = FALSE, triangle = c("upper", "lower", "full"), ...)

\method{tidy}{ba}(x, ...)

\method{tidy}{ba_matrix}(x, ...)

\method{tidy}{ba_repeated}(x, ...)

\method{tidy}{ba_repeated_matrix}(x, ...)

\method{tidy}{default}(x, diag = FALSE, triangle = c("upper", "lower", "full"), ...)

\method{confint}{corr_result}(object, parm, level = NULL, ...)

\method{confint}{summary.corr_result}(object, parm, level = NULL, ...)

\method{confint}{summary.matrixCorr}(object, parm, level = NULL, ...)

\method{confint}{ba}(object, parm, level = NULL, ...)

\method{confint}{ba_repeated}(object, parm, level = NULL, ...)

\method{confint}{ba_matrix}(object, parm, level = NULL, ...)

\method{confint}{ba_repeated_matrix}(object, parm, level = NULL, ...)

\method{confint}{ccc}(object, parm, level = NULL, ...)

\method{confint}{ccc_ci}(object, parm, level = NULL, ...)

\method{confint}{ccc_glmm}(object, parm, level = NULL, ...)

\method{confint}{chatterjee_xi_scalar}(object, parm, level = NULL, ...)

\method{confint}{cia}(object, parm, level = NULL, ...)

\method{confint}{cia_ci}(object, parm, level = NULL, ...)

\method{confint}{cia_rm}(object, parm, level = NULL, ...)

\method{confint}{cohen_kappa}(object, parm, level = NULL, ...)

\method{confint}{gwet_ac}(object, parm, level = NULL, ...)

\method{confint}{icc}(object, parm, level = NULL, ...)

\method{confint}{icc_overall}(object, parm, level = NULL, ...)

\method{confint}{icc_rm_reml}(object, parm, level = NULL, ...)

\method{confint}{krippendorff_alpha}(object, parm, level = NULL, ...)

\method{confint}{multirater_kappa}(object, parm, level = NULL, ...)

\method{confint}{prob_agree}(object, parm, level = NULL, ...)

\method{confint}{partial_corr}(object, parm, level = NULL, ...)

\method{confint}{rmcorr}(object, parm, level = NULL, ...)

\method{confint}{rmcorr_matrix}(object, parm, level = NULL, ...)

\method{confint}{weighted_kappa}(object, parm, level = NULL, ...)
}
\arguments{
\item{x, object}{A matrixCorr result, summary object, or scalar result.}

\item{...}{Additional arguments passed to methods.}

\item{diag}{Logical; include diagonal entries for matrix-style results.
Default is \code{FALSE}.}

\item{triangle}{For matrix-style correlation results, which triangle to
return from \code{tidy()}: \code{"upper"} (default), \code{"lower"}, or
\code{"full"}. The default avoids duplicate pairs for symmetric matrices;
directed matrices use all off-diagonal entries unless \code{triangle} is
supplied explicitly.}

\item{parm}{Ignored; included for compatibility with \code{stats::confint()}.}

\item{level}{Confidence level requested by \code{stats::confint()}. MatrixCorr
objects store pre-computed intervals; if \code{level} differs from the
stored level, recompute the original object with the desired level.}
}
\value{
\code{estimate()} returns the primary estimate in its natural shape: a
matrix-like result returns a matrix, an edge-list result returns a data frame,
and a scalar result returns a numeric value.

\code{tidy()} returns a data frame with columns such as \code{item1},
\code{item2}, \code{estimate}, \code{lwr}, \code{upr}, diagnostics, and
inferential quantities when available.

\code{confint()} returns a data frame containing confidence limits when they
are available.

\code{ci()} returns the stored confidence-interval payload. It is mainly a
structured alternative to reading \code{attr(x, "ci")} directly.
}
\description{
Lightweight accessors for matrixCorr result objects. These functions do not
change the stored object structure; they just provide a stable way to extract the
estimate matrix, confidence intervals, or a pairwise data frame without
reading attributes directly.
}
\examples{
X <- cbind(a = 1:10, b = 1:10 + rnorm(10), c = rnorm(10))
fit <- pearson_corr(X, ci = TRUE)

estimate(fit)
tidy(fit)
ci(fit)
confint(fit)

}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{tetrachoric}
\alias{tetrachoric}
\alias{print.tetrachoric_corr}
\alias{plot.tetrachoric_corr}
\alias{summary.tetrachoric_corr}
\alias{print.summary.tetrachoric_corr}
\title{Pairwise Tetrachoric Correlation}
\usage{
tetrachoric(
  data,
  y = NULL,
  na_method = c("error", "pairwise"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  correct = 0.5,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{tetrachoric_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{tetrachoric_corr}(
  x,
  title = "Tetrachoric correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{tetrachoric_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.tetrachoric_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A binary vector, matrix, or data frame. In matrix/data-frame mode,
only binary columns are retained.}

\item{y}{Optional second binary vector. When supplied, the function returns a
single tetrachoric correlation estimate.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
complete cases.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald confidence intervals derived from the observed
information matrix of the latent-variable likelihood.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample Wald p-values and test statistics for each
estimated latent correlation.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{correct}{Non-negative continuity correction added to zero-count cells.
Default is \code{0.5}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Additional arguments passed to \code{print()}.}

\item{x}{An object of class \code{summary.tetrachoric_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Tetrachoric correlation heatmap"}.}

\item{low_color}{Color for the minimum correlation.}

\item{high_color}{Color for the maximum correlation.}

\item{mid_color}{Color for zero correlation.}

\item{value_text_size}{Font size used in tile labels.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{tetrachoric_corr}.}

\item{ci_digits}{Integer; digits for confidence limits in the pairwise
summary.}

\item{p_digits}{Integer; digits for p-values in the pairwise summary.}
}
\value{
If \code{y} is supplied, a numeric scalar with attributes
\code{diagnostics} and \code{thresholds}. Otherwise a symmetric matrix of
class \code{tetrachoric_corr} with attributes \code{method},
\code{description}, \code{package = "matrixCorr"}, \code{diagnostics},
\code{thresholds}, and \code{correct}. When \code{p_value = TRUE}, the
returned object also carries an \code{inference} attribute with elements
\code{estimate}, \code{statistic}, \code{parameter}, \code{p_value}, and
\code{n_obs}. When \code{ci = TRUE}, it also carries a \code{ci} attribute
with elements \code{est}, \code{lwr.ci}, \code{upr.ci}, \code{conf.level},
and \code{ci.method}, plus \code{attr(x, "conf.level")}. Scalar outputs keep
the same point estimate and gain the same metadata only when inference is
requested. In matrix mode, \code{output = "edge_list"} returns a data frame with columns
\code{row}, \code{col}, \code{value}; \code{output = "sparse"} returns a
symmetric sparse matrix.
}
\description{
Computes the tetrachoric correlation for either a pair of binary variables
or all pairwise combinations of binary columns in a matrix/data frame.
}
\details{
The tetrachoric correlation assumes that the observed binary variables arise
by dichotomising latent standard-normal variables. Let
\eqn{Z_1, Z_2 \sim N(0, 1)} with latent correlation \eqn{\rho}, and define
observed binary variables by thresholds \eqn{\tau_1, \tau_2}:
\deqn{
X = \mathbf{1}\{Z_1 > \tau_1\},
\qquad
Y = \mathbf{1}\{Z_2 > \tau_2\}.
}
If the observed \eqn{2 \times 2} table has counts
\eqn{n_{ij}} for \eqn{i,j \in \{0,1\}}, the marginal proportions determine
the thresholds:
\deqn{
\tau_1 = \Phi^{-1}\!\big(P(X = 0)\big),
\qquad
\tau_2 = \Phi^{-1}\!\big(P(Y = 0)\big).
}
The estimator returned here is the maximum-likelihood estimate of the latent
correlation \eqn{\rho}, obtained by maximizing the multinomial log-likelihood
built from the rectangle probabilities of the bivariate normal distribution:
\deqn{
\ell(\rho) = \sum_{i=0}^1 \sum_{j=0}^1 n_{ij}\log \pi_{ij}(\rho;\tau_1,\tau_2),
}
where \eqn{\pi_{ij}} are the four bivariate-normal cell probabilities implied
by \eqn{\rho} and the fixed thresholds. The implementation evaluates the
likelihood over \eqn{\rho \in (-1,1)} by a coarse search followed by Brent
refinement in C++.

The argument \code{correct} adds a continuity correction only to zero-count
cells before threshold estimation and likelihood evaluation. This stabilises
the estimator for sparse tables and mirrors the conventional
\code{correct = 0.5} continuity-correction behaviour used in several
latent-correlation implementations.
When \code{correct = 0} and the observed contingency table contains zero
cells, the fit is non-regular and may be boundary-driven. In those cases the
returned object stores sparse-fit diagnostics, including whether the fit was
classified as \code{boundary} or \code{near_boundary}.

\strong{Assumptions.} The coefficient is appropriate when both observed
binary variables are viewed as thresholded versions of jointly normal latent
variables. The optional p-values and confidence intervals adopt this
latent-normal interpretation and use the same likelihood that defines the
tetrachoric estimate. These inferential quantities are therefore model-based
and should not be interpreted as distribution-free summaries.

\strong{Inference.} When \code{ci = TRUE} or \code{p_value = TRUE}, the
function refits the pairwise tetrachoric model by maximum likelihood and
obtains the observed information matrix numerically in C++. The reported
confidence interval is a Wald interval
\eqn{\hat\rho \pm z_{1-\alpha/2}\operatorname{SE}(\hat\rho)}, and the
reported p-value is from the large-sample Wald \eqn{z}-test for
\eqn{H_0:\rho = 0}. These inferential quantities are only computed when
explicitly requested.

In matrix/data-frame mode, all pairwise tetrachoric correlations are computed
between binary columns. Diagonal entries are \code{1} for non-degenerate
columns and \code{NA} for columns with fewer than two observed levels.
Variable-specific latent thresholds are stored in the \code{thresholds}
attribute, and pairwise sparse-fit diagnostics are stored in
\code{diagnostics}.

\strong{Computational complexity.} For \eqn{p} binary variables, the matrix
path evaluates \eqn{p(p-1)/2} pairwise likelihoods. Each pair uses a
one-dimensional optimisation with negligible memory overhead beyond the
output matrix.
}
\examples{
\dontshow{if (requireNamespace("mnormt", quietly = TRUE)) withAutoprint(\{ # examplesIf}
set.seed(123)
n <- 1000
Sigma <- matrix(c(
  1.00, 0.55, 0.35,
  0.55, 1.00, 0.45,
  0.35, 0.45, 1.00
), 3, 3, byrow = TRUE)

Z <- mnormt::rmnorm(n = n, mean = rep(0, 3), varcov = Sigma)
X <- data.frame(
  item1 = Z[, 1] > stats::qnorm(0.70),
  item2 = Z[, 2] > stats::qnorm(0.60),
  item3 = Z[, 3] > stats::qnorm(0.50)
)

tc <- tetrachoric(X)
print(tc, digits = 3)
summary(tc)
estimate(tc)
tidy(tc)
tc_ci <- tetrachoric(X, ci = TRUE)
ci(tc_ci)
confint(tc_ci)
plot(tc)
tetrachoric(X, output = "edge_list", diag = FALSE)
tetrachoric(X, output = "sparse", threshold = 0.4, diag = FALSE)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(tc)
}

# latent Pearson correlations used to generate the binary items
round(stats::cor(Z), 2)
\dontshow{\}) # examplesIf}
}
\references{
Pearson, K. (1900). Mathematical contributions to the theory of evolution.
VII. On the correlation of characters not quantitatively measurable.
\emph{Philosophical Transactions of the Royal Society A}, 195, 1-47.

Olsson, U. (1979). Maximum likelihood estimation of the polychoric
correlation coefficient. \emph{Psychometrika}, 44(4), 443-460.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_corr_input.R
\name{matrixCorr-internal}
\alias{matrixCorr-internal}
\alias{validate_corr_input}
\title{Validation and normalisation for correlation}
\usage{
validate_corr_input(data, check_na = TRUE)
}
\arguments{
\item{data}{A matrix or data frame. Non-numeric columns are dropped
(data.frame path). For matrix input, storage mode must be integer or double.}

\item{check_na}{Logical (default \code{TRUE}). If \code{TRUE}, validate and
reject inputs containing \code{NA}/\code{NaN}/\code{Inf}. Set to
\code{FALSE} when an upstream routine (e.g., pairwise-complete kernels)
will handle missingness per pair.}
}
\value{
A numeric matrix (type \code{double}) with column names preserved.
}
\description{
Validates and normalises input for correlation computations. Accepts either a
numeric matrix or a data frame, filters numeric columns, checks dimensions and
(optionally) missing values, and returns a numeric (double) matrix with
preserved column names.
}
\details{
Rules enforced:
\itemize{
\item Input must be a matrix or data.frame.
\item Only numeric (integer or double) columns are retained (data.frame path).
\item At least two numeric columns are required.
\item All columns must have the same length and \eqn{\ge} 2 observations.
\item Missing and non-finite values are not allowed when
\code{check_na = TRUE}.
\item Returns a \code{double} matrix; integer input is converted once.
}
}
\seealso{
\code{\link[=pearson_corr]{pearson_corr()}}, \code{\link[=spearman_rho]{spearman_rho()}}, \code{\link[=kendall_tau]{kendall_tau()}}
}
\author{
Thiago de Paula Oliveira
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ba_rm.R
\name{.ba_rep_two_methods}
\alias{.ba_rep_two_methods}
\title{two-method helper}
\usage{
.ba_rep_two_methods(
  response,
  subject,
  method12,
  time,
  loa_multiplier,
  conf_level,
  n_threads,
  include_slope,
  use_ar1,
  ar1_rho,
  max_iter,
  tol
)
}
\description{
two-method helper
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latent_corr.R
\name{biserial}
\alias{biserial}
\alias{print.biserial_corr}
\alias{plot.biserial_corr}
\alias{summary.biserial_corr}
\alias{print.summary.biserial_corr}
\title{Biserial Correlation Between Continuous and Binary Variables}
\usage{
biserial(data, y, na_method = c("error", "pairwise"), ci = FALSE, p_value = FALSE,
  conf_level = 0.95, ...)

\method{print}{biserial_corr}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{biserial_corr}(
  x,
  title = "Biserial correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)

\method{summary}{biserial_corr}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.biserial_corr}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric vector, matrix, or data frame containing continuous
variables.}

\item{y}{A binary vector, matrix, or data frame. In data-frame mode, only
two-level columns are retained.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing values. \code{"pairwise"} uses pairwise
complete cases.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach
approximate large-sample confidence intervals derived from a Fisher
\eqn{z}-transformation of the biserial estimate.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
model-based large-sample p-values, test statistics, and degrees of freedom
for each biserial estimate.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default is
\code{0.95}.}

\item{...}{Additional arguments passed to \code{print()}.}

\item{x}{An object of class \code{summary.biserial_corr}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title. Default is \code{"Biserial correlation heatmap"}.}

\item{low_color}{Color for the minimum correlation.}

\item{high_color}{Color for the maximum correlation.}

\item{mid_color}{Color for zero correlation.}

\item{value_text_size}{Font size used in tile labels.}

\item{ci_text_size}{Text size for confidence intervals in the heatmap.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{biserial_corr}.}

\item{ci_digits}{Integer; digits for biserial confidence limits in the
pairwise summary.}

\item{p_digits}{Integer; digits for biserial p-values in the pairwise
summary.}
}
\value{
If both \code{data} and \code{y} are vectors, a numeric scalar. Otherwise a
numeric matrix of class \code{biserial_corr} with rows corresponding to
the continuous variables in \code{data} and columns to the binary variables
in \code{y}. Matrix outputs carry attributes \code{method},
\code{description}, and \code{package = "matrixCorr"}. When
\code{p_value = TRUE}, the object also carries an \code{inference}
attribute with matrices \code{estimate}, \code{statistic},
\code{parameter}, \code{p_value}, and \code{n_obs}. When \code{ci = TRUE},
it additionally carries a \code{ci} attribute with matrices
\code{lwr.ci} and \code{upr.ci}, plus \code{attr(x, "conf.level")}. Scalar
outputs keep the same point estimate and gain the same metadata only when
inference is requested.
}
\description{
Computes biserial correlations between continuous variables in \code{data}
and binary variables in \code{y}. Both pairwise vector mode and rectangular
matrix/data-frame mode are supported.
}
\details{
The biserial correlation is the special two-category case of the polyserial
model. It assumes that a binary variable \eqn{Y} arises by thresholding an
unobserved standard-normal variable \eqn{Z} that is jointly normal with a
continuous variable \eqn{X}. Writing \eqn{p = P(Y = 1)} and
\eqn{q = 1-p}, let \eqn{z_p = \Phi^{-1}(p)} and \eqn{\phi(z_p)} be the
standard-normal density evaluated at \eqn{z_p}. If \eqn{\bar x_1} and
\eqn{\bar x_0} denote the sample means of \eqn{X} in the two observed groups
and \eqn{s_x} is the sample standard deviation of \eqn{X}, the usual
biserial estimator is
\deqn{
r_b =
\frac{\bar x_1 - \bar x_0}{s_x}
\frac{pq}{\phi(z_p)}.
}
This is exactly the estimator implemented in the underlying C++ kernel.

\strong{Assumptions.} The biserial coefficient is appropriate when the
observed binary variable is viewed as a thresholded version of an
unobserved continuous latent variable that is jointly normal with the
observed continuous variable. The optional p-values and confidence
intervals adopt this latent-normal interpretation together with the usual
large-sample approximations used for correlation coefficients. These
inferential quantities are therefore model-based and should not be
interpreted as distribution-free summaries.

\strong{Inference.} When \code{p_value = TRUE}, the package reports the
large-sample \eqn{t}-statistic
\deqn{
t = r_b \sqrt{\frac{n - 2}{1 - r_b^2}},
}
referenced to a Student \eqn{t}-distribution with \eqn{n - 2} degrees of
freedom. When \code{ci = TRUE}, the package forms an approximate Fisher
\eqn{z}-interval by transforming \eqn{r_b} with
\eqn{z = \operatorname{atanh}(r_b)}, using standard error
\eqn{1 / \sqrt{n - 3}}, and mapping the limits back with
\eqn{\tanh(\cdot)}. The CI is therefore an internal large-sample
extension and is only computed when explicitly requested.

In vector mode a single biserial correlation is returned. In
matrix/data-frame mode, every numeric column of \code{data} is paired with every
binary column of \code{y}, producing a rectangular matrix of
continuous-by-binary biserial correlations.

Unlike the point-biserial correlation, which is just Pearson correlation on a
0/1 coding of the binary variable, the biserial coefficient explicitly
assumes an underlying latent normal threshold model and rescales the mean
difference accordingly.

\strong{Computational complexity.} If \code{data} has \eqn{p_x} continuous
columns and \code{y} has \eqn{p_y} binary columns, the matrix path computes
\eqn{p_x p_y} closed-form estimates with negligible extra memory beyond the
output matrix.
}
\examples{
\dontshow{if (requireNamespace("mnormt", quietly = TRUE)) withAutoprint(\{ # examplesIf}
set.seed(126)
n <- 1000
Sigma <- matrix(c(
  1.00, 0.35, 0.50, 0.25,
  0.35, 1.00, 0.30, 0.55,
  0.50, 0.30, 1.00, 0.40,
  0.25, 0.55, 0.40, 1.00
), 4, 4, byrow = TRUE)

Z <- mnormt::rmnorm(n = n, mean = rep(0, 4), varcov = Sigma)
X <- data.frame(x1 = Z[, 1], x2 = Z[, 2])
Y <- data.frame(
  g1 = Z[, 3] > stats::qnorm(0.65),
  g2 = Z[, 4] > stats::qnorm(0.55)
)

bs <- biserial(X, Y, ci = TRUE, p_value = TRUE)
print(bs, digits = 3)
summary(bs)
estimate(bs)
tidy(bs)
ci(bs)
confint(bs)
plot(bs)
\dontshow{\}) # examplesIf}
}
\references{
Olsson, U., Drasgow, F., & Dorans, N. J. (1982). The polyserial
correlation coefficient. \emph{Psychometrika}, 47(3), 337-347.

Fisher, R. A. (1921). On the probable error of a coefficient of
correlation deduced from a small sample. \emph{Metron}, 1, 3-32.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icc.R
\name{icc}
\alias{icc}
\alias{print.icc}
\alias{summary.icc}
\alias{print.summary.icc}
\alias{print.icc_overall}
\alias{summary.icc_overall}
\alias{print.summary.icc_overall}
\title{Intraclass Correlation for Wide Data}
\usage{
icc(
  data,
  model = c("oneway", "twoway_random", "twoway_mixed"),
  type = c("consistency", "agreement"),
  unit = c("single", "average"),
  scope = c("pairwise", "overall"),
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE,
  ...
)

\method{print}{icc}(
  x,
  digits = 4,
  ci_digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{icc}(
  object,
  digits = 4,
  ci_digits = 2,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.icc}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{icc_overall}(
  x,
  digits = 4,
  ci_digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{icc_overall}(
  object,
  digits = 4,
  ci_digits = 2,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.icc_overall}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or data frame with at least two numeric columns.}

\item{model}{Character scalar selecting the classical ICC model.
\code{"oneway"} uses the one-way random-effects formulation,
\code{"twoway_random"} uses the two-way random-effects formulation, and
\code{"twoway_mixed"} uses the two-way mixed-effects formulation.}

\item{type}{Character scalar selecting the reliability target.
\code{"consistency"} does not penalize systematic column mean differences in
the same way as absolute agreement, whereas \code{"agreement"} targets absolute
agreement and therefore incorporates those differences.}

\item{unit}{Character scalar selecting whether the coefficient refers to a
single measurement (\code{"single"}) or to the mean of multiple measurements
(\code{"average"}). For \code{scope = "pairwise"}, the average-measure coefficient
always uses \code{k = 2}. For \code{scope = "overall"}, it uses the full number of
analysed columns.}

\item{scope}{Character scalar selecting the analysis target.
\code{"pairwise"} returns the symmetric matrix of two-column ICCs.
\code{"overall"} returns the six-row overall ANOVA table for the full set of
columns. Choose \code{"pairwise"} when the question is about specific column
pairs; choose \code{"overall"} when the question is about the full set of
columns analysed jointly.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values before
estimation. \code{"pairwise"} uses pair-specific complete cases for
\code{scope = "pairwise"} and complete rows across all analysed columns
for \code{scope = "overall"}. \code{"complete"} performs listwise
deletion once across all analysed columns before ICC estimation.}

\item{ci}{Logical; if \code{TRUE}, return confidence intervals.}

\item{conf_level}{Confidence level for the interval output. Ignored when
\code{ci = FALSE}.}

\item{n_threads}{Integer number of OpenMP threads.}

\item{verbose}{Logical; if \code{TRUE}, report how many threads are requested.}

\item{...}{Passed to the underlying print helper.}

\item{x}{An intraclass-correlation object returned by \code{icc()}, or a summary
object returned by \code{summary()} for that fit.}

\item{digits}{Integer; number of digits to print.}

\item{ci_digits}{Integer; number of digits for CI bounds.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives
this from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{object}{An intraclass-correlation object returned by \code{icc()}.}
}
\value{
For \code{scope = "pairwise"}, if \code{ci = FALSE}, a symmetric matrix of class
\code{icc}. If \code{ci = TRUE}, a list with elements \code{est}, \code{lwr.ci}, and \code{upr.ci}
and class \code{c("icc", "icc_ci")}.

For \code{scope = "overall"}, a list of class \code{c("icc_overall", "icc")} with a
coefficient table, ANOVA table, and mean-square metadata. The coefficient
table always includes the standard six overall coefficients. Confidence
interval columns are attached when \code{ci = TRUE}.

All outputs carry attributes describing the selected model, type, unit, and
method.
}
\description{
Computes intraclass correlation coefficients for the numeric columns of a
matrix or data frame using the classical ANOVA mean-square formulas. The
output can be either a pairwise matrix across columns or an overall
all-column coefficient table.
}
\details{
Each column is treated as a measurement channel, method, or rater and each
row is treated as a subject.

The function supports two distinct analysis targets.
\itemize{
\item \code{scope = "pairwise"} answers: "how reliable is each specific column pair?"
Each estimate is based on exactly two columns and the output is a symmetric
matrix.
\item \code{scope = "overall"} answers: "how reliable is the full set of columns when
analysed jointly?" The output is the standard six-form overall ANOVA table
(\code{ICC1}, \code{ICC2}, \code{ICC3}, \code{ICC1k}, \code{ICC2k}, \code{ICC3k}).
}

These two scopes do not target the same quantity. The overall coefficients
are computed from the full multi-column ANOVA decomposition and are not
obtained by averaging or otherwise aggregating the pairwise matrix.

The three main choice arguments determine the classical ICC form.
\itemize{
\item \code{model} controls the rater structure:
\itemize{
\item \code{"oneway"} uses the one-way random-effects formulation.
\item \code{"twoway_random"} uses the two-way random-effects formulation.
\item \code{"twoway_mixed"} uses the two-way mixed-effects formulation.
}
\item \code{type} controls whether systematic column mean differences are penalized:
\itemize{
\item \code{"consistency"} targets consistency across columns.
\item \code{"agreement"} targets absolute agreement across columns.
}
\item \code{unit} controls whether reliability refers to one measurement or to the
average of multiple measurements:
\itemize{
\item \code{"single"} returns the single-measure coefficient.
\item \code{"average"} returns the average-measure coefficient.
}
}

The supported mappings are:
\itemize{
\item \verb{model = "oneway", type = "consistency", unit = "single"} gives \code{ICC1}.
\item \verb{model = "oneway", type = "consistency", unit = "average"} gives \code{ICC1k}.
\item \verb{model = "twoway_random", type = "agreement", unit = "single"} gives \code{ICC2}.
\item \verb{model = "twoway_random", type = "agreement", unit = "average"} gives \code{ICC2k}.
\item \verb{model = "twoway_random", type = "consistency", unit = "single"} gives \code{ICC3}.
\item \verb{model = "twoway_random", type = "consistency", unit = "average"} gives \code{ICC3k}.
\item \verb{model = "twoway_mixed", type = "agreement", unit = "single"} gives the
mixed-effects absolute-agreement analogue with the same classical point
formula as \code{ICC2}.
\item \verb{model = "twoway_mixed", type = "agreement", unit = "average"} gives the
corresponding average-measure analogue.
\item \verb{model = "twoway_mixed", type = "consistency", unit = "single"} gives \code{ICC3}.
\item \verb{model = "twoway_mixed", type = "consistency", unit = "average"} gives \code{ICC3k}.
}

The combination \verb{model = "oneway", type = "agreement"} is not defined here
and returns an error.

For \code{scope = "pairwise"}, the point estimates are computed in \verb{C++} directly
from the two-column ANOVA mean squares for each complete pair. For
\code{unit = "average"}, the implementation uses \code{k = 2} because each estimate is
based on exactly two columns.

For \code{scope = "overall"}, the point estimates are computed jointly from the
full wide matrix using the classical ANOVA decomposition over all columns.
Here the average-measure coefficients use \code{k = ncol(data)} after any row
filtering required by \code{na_method}.

Missing-data handling depends on \code{scope}:
\itemize{
\item with \code{na_method = "error"}, missing values are rejected before estimation;
\item with \code{na_method = "pairwise"} and \code{scope = "pairwise"}, each pair uses its
own complete-case overlap;
\item with \code{na_method = "pairwise"} and \code{scope = "overall"}, rows are restricted
to complete cases across all columns because the overall ANOVA requires a
common wide matrix.
}

When \code{ci = TRUE}, confidence intervals are obtained from the classical
F-based ANOVA formulas corresponding to the selected coefficient. For
\code{scope = "pairwise"}, non-estimable off-diagonal pairs return \code{NA}. For
\code{scope = "overall"}, the coefficient table includes interval columns for all
six standard rows.
}
\examples{
set.seed(123)
n <- 40
subj <- rnorm(n, sd = 1)
dat <- data.frame(
  m1 = subj + rnorm(n, sd = 0.3),
  m2 = 0.2 + subj + rnorm(n, sd = 0.4),
  m3 = -0.1 + subj + rnorm(n, sd = 0.5)
)

fit_icc <- icc(dat,
  model = "twoway_random",
  type = "agreement",
  unit = "single",
  scope = "pairwise"
)
print(fit_icc)
summary(fit_icc)
estimate(fit_icc)
tidy(fit_icc)

fit_icc_overall <- icc(dat, scope = "overall", ci = TRUE)
print(fit_icc_overall)
summary(fit_icc_overall)
confint(fit_icc_overall)

}
\references{
Shrout PE, Fleiss JL (1979). Intraclass correlations: uses in assessing
rater reliability. Psychological Bulletin, 86(2), 420-428.

McGraw KO, Wong SP (1996). Forming inferences about some intraclass
correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\seealso{
\code{\link[=ccc]{ccc()}}, \code{\link[=ccc_rm_reml]{ccc_rm_reml()}}, \code{\link[=ba]{ba()}}, \code{\link[=ba_rm]{ba_rm()}}, \code{\link[=rmcorr]{rmcorr()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ba.R
\name{ba}
\alias{ba}
\alias{print.ba}
\alias{summary.ba}
\alias{print.summary.ba}
\alias{plot.ba}
\alias{print.ba_matrix}
\alias{summary.ba_matrix}
\alias{print.summary.ba_matrix}
\alias{plot.ba_matrix}
\title{Bland-Altman statistics with confidence intervals}
\usage{
ba(
  group1,
  group2,
  loa_multiplier = 1.96,
  mode = 1L,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE
)

\method{print}{ba}(
  x,
  digits = 3,
  ci_digits = 3,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{ba}(object, digits = 3, ci_digits = 3, ...)

\method{print}{summary.ba}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{ba}(
  x,
  title = "Bland-Altman Plot",
  subtitle = NULL,
  point_alpha = 0.7,
  point_size = 2.2,
  line_size = 0.8,
  shade_ci = TRUE,
  shade_alpha = 0.08,
  smoother = c("none", "loess", "lm"),
  symmetrize_y = TRUE,
  show_value = TRUE,
  ...
)

\method{print}{ba_matrix}(
  x,
  digits = 3,
  ci_digits = 3,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  style = c("pairs", "matrices"),
  ...
)

\method{summary}{ba_matrix}(object, digits = 3, ci_digits = 3, ...)

\method{print}{summary.ba_matrix}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{ba_matrix}(
  x,
  pairs = NULL,
  against = NULL,
  facet_scales = c("free_y", "fixed"),
  title = "Bland-Altman (pairwise)",
  point_alpha = 0.6,
  point_size = 1.8,
  line_size = 0.7,
  shade_ci = TRUE,
  shade_alpha = 0.08,
  smoother = c("none", "loess", "lm"),
  show_value = TRUE,
  ...
)
}
\arguments{
\item{group1}{Numeric vector of paired measurements, or a numeric matrix/data
frame with at least two numeric columns when \code{group2} is omitted.}

\item{group2}{Optional numeric vector of paired measurements. If omitted,
\code{group1} must be a numeric matrix or data frame and all unordered numeric
column pairs are analysed.}

\item{loa_multiplier}{Positive scalar; the multiple of the standard deviation used to
define the LoA (default 1.96 for nominal 95\\% agreement). The confidence
intervals always use \eqn{t_{n-1,\,1-\alpha/2}} regardless of this choice.}

\item{mode}{Integer; 1 uses \code{group1 - group2}, 2 uses \code{group2 - group1}.}

\item{conf_level}{Confidence level for CIs (default 0.95).}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{verbose}{Logical; if TRUE, prints how many OpenMP threads are used.}

\item{x}{A \code{"ba_matrix"} object.}

\item{digits}{Number of digits for estimates (default 3).}

\item{ci_digits}{Number of digits for CI bounds (default 3).}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Passed to \code{ggplot2::theme()} (ggplot path) or \code{plot()}.}

\item{object}{A \code{"ba"} object.}

\item{title}{Plot title.}

\item{subtitle}{Optional subtitle. If NULL, shows n and LoA summary.}

\item{point_alpha}{Point transparency.}

\item{point_size}{Point size.}

\item{line_size}{Line width for mean/LoA.}

\item{shade_ci}{Logical; if TRUE, draw shaded CI bands instead of 6 dashed
lines.}

\item{shade_alpha}{Transparency of CI bands.}

\item{smoother}{One of "none", "loess", "lm" to visualize proportional bias.}

\item{symmetrize_y}{Logical; if TRUE, y-axis centered at mean difference
with symmetric limits.}

\item{show_value}{Logical; included for a consistent plotting interface.
Bland-Altman plots do not overlay numeric cell values, so this argument
currently has no effect.}

\item{style}{Show the pairwise result as \code{"pairs"} or \code{"matrices"}.}

\item{pairs}{Optional character vector of pair labels to display.}

\item{against}{Optional single method name; if supplied, only contrasts
involving that method are plotted.}

\item{facet_scales}{Either \code{"free_y"} (default) or \code{"fixed"}.}
}
\value{
If \code{group1} and \code{group2} are both supplied, an object of class
\code{"ba"} (list) with elements:
\itemize{
\item \code{means}, \code{diffs}: numeric vectors
\item \code{groups}: data.frame used after NA removal
\item \code{n_obs}: integer, number of complete pairs used.
\item \code{based.on}: compatibility alias for \code{n_obs}.
\item \code{lower.limit}, \code{mean.diffs}, \code{upper.limit}
\item \code{lines}: named numeric vector (lower, mean, upper)
\item \code{CI.lines}: named numeric vector for CIs of those lines
\item \code{loa_multiplier}, \code{critical.diff}
}

If \code{group2} is omitted and \code{group1} is a wide numeric matrix or data frame,
an object of class \code{"ba_matrix"} with pairwise components:
\itemize{
\item \code{bias}: pairwise matrix of Bland-Altman mean differences.
\item \code{sd_loa}: pairwise matrix of SDs of differences.
\item \code{loa_lower}, \code{loa_upper}: pairwise matrices of LoA
endpoints.
\item \code{width}: pairwise matrix of LoA widths.
\item \code{n}: integer pairwise matrix of complete-case counts.
\item \code{mean_ci_low}, \code{mean_ci_high}: pairwise matrices of CI
bounds for the mean difference.
\item \code{loa_lower_ci_low}, \code{loa_lower_ci_high}: pairwise matrices
of CI bounds for the lower LoA.
\item \code{loa_upper_ci_low}, \code{loa_upper_ci_high}: pairwise matrices
of CI bounds for the upper LoA.
\item \code{methods}: character vector of analysed method names.
\item \code{loa_multiplier}, \code{mode}: calculation settings reused for
every pair.
}
}
\description{
Computes Bland-Altman mean difference and limits of agreement (LoA)
between two numeric measurement vectors, including t-based confidence
intervals for the mean difference and for each LoA using 'C++' backend.
If \code{group2} is omitted and \code{group1} is a numeric matrix or data frame with
at least two numeric columns, \code{ba()} computes all pairwise contrasts across
methods and returns a pairwise Bland-Altman matrix object.

Note: Lin's concordance correlation coefficient (CCC) is a complementary,
single-number summary of agreement (precision + accuracy). It is useful for
quick screening or reporting an overall CI, but may miss systematic or
magnitude-dependent bias; consider reporting CCC alongside Bland-Altman.
}
\details{
Given paired measurements \eqn{(x_i, y_i)}, Bland-Altman analysis uses
\eqn{d_i = x_i - y_i} (or \eqn{y_i - x_i} if \code{mode = 2}) and
\eqn{m_i = (x_i + y_i)/2}. The mean difference \eqn{\bar d} estimates bias.
The limits of agreement (LoA) are \eqn{\bar d \pm z \cdot s_d}, where
\eqn{s_d} is the sample standard deviation of \eqn{d_i} and \eqn{z}
(argument \code{loa_multiplier}) is typically 1.96 for nominal 95\% LoA.

When \code{group2} is omitted and \code{group1} is a wide numeric matrix or data
frame, the same two-method calculation is applied to every unordered column
pair. The returned pairwise matrices follow the requested \code{mode}: with
\code{mode = 1}, upper-triangle entries represent row minus column; with
\code{mode = 2}, upper-triangle entries represent column minus row.

Confidence intervals use Student's \eqn{t} distribution with \eqn{n-1}
degrees of freedom, with
\itemize{
\item Mean-difference CI given by \eqn{\bar d \pm t_{n-1,\,1-\alpha/2}\,
  s_d/\sqrt{n}}; and
\item LoA CI given by \eqn{(\bar d \pm z\, s_d) \;\pm\;
  t_{n-1,\,1-\alpha/2}\, s_d\,\sqrt{3/n}}.
}

Assumptions include approximately normal differences and roughly constant
variability across the measurement range; if differences increase with
magnitude, consider a transformation before analysis. Missing values are
removed pairwise (rows with an \code{NA} in either input are dropped before
calling the C++ backend).

Probability of agreement, available through \code{\link{prob_agree}}, is a
tolerance-based companion to Bland-Altman analysis. Bland-Altman reports bias
and limits of agreement for paired differences. \code{prob_agree()} instead uses
the sampling distribution of estimated differences to quantify the
probability that two estimated quantities or curves agree within a
user-specified practical tolerance.
}
\examples{
set.seed(1)
x <- rnorm(100, 100, 10)
y <- x + rnorm(100, 0, 8)
fit_ba <- ba(x, y)
print(fit_ba)
estimate(fit_ba)
tidy(fit_ba)
confint(fit_ba)
plot(fit_ba)

# Pairwise Bland-Altman across 3 methods
set.seed(7)
wide3 <- data.frame(
  ref = rnorm(80, 100, 8),
  m2  = rnorm(80, 101, 8),
  m3  = rnorm(80,  99, 9)
)
fit_ba3 <- ba(wide3)
print(fit_ba3)
summary(fit_ba3)
tidy(fit_ba3)
plot(fit_ba3)

}
\references{
Bland JM, Altman DG (1986). Statistical methods for assessing agreement
between two methods of clinical measurement. \emph{The Lancet}, 307-310.

Bland JM, Altman DG (1999). Measuring agreement in method comparison studies.
\emph{Statistical Methods in Medical Research}, 8(2), 135-160.
}
\seealso{
\code{\link{print.ba}}, \code{\link{plot.ba}},
\code{\link{ccc}}, \code{\link{prob_agree}}, \code{\link{ccc_rm_ustat}},
\code{\link{ccc_rm_reml}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_glmm.R
\name{ccc_glmm}
\alias{ccc_glmm}
\title{Poisson GLMM concordance correlation for count agreement}
\usage{
ccc_glmm(
  data,
  response,
  subject,
  method,
  replicate = NULL,
  family = "poisson",
  link = "log",
  overdispersion = c("none", "pearson"),
  include_subject_method = FALSE,
  ci = FALSE,
  conf_level = 0.95,
  max_iter = 1000,
  tol = 1e-08,
  n_threads = getOption("matrixCorr.threads", 1L),
  verbose = FALSE
)
}
\arguments{
\item{data}{A data frame containing the measurements.}

\item{response}{Character. Name of the non-negative integer-like count column.}

\item{subject}{Character. Name of the subject identifier column.}

\item{method}{Character. Name of the method/observer column.}

\item{replicate}{Optional character. Name of the replicate column. If
\code{NULL}, one replicate per subject-method cell is assumed.}

\item{family}{Character. Currently only \code{"poisson"} is implemented.}

\item{link}{Character. Currently only \code{"log"} is implemented.}

\item{overdispersion}{Character. \code{"none"} fixes \eqn{\phi = 1};
\code{"pearson"} estimates \eqn{\phi} from post-fit Pearson residuals.}

\item{include_subject_method}{Logical. If \code{TRUE}, include an additional
subject-by-method random intercept variance component.}

\item{ci}{Logical. If \code{TRUE}, compute delta-method confidence intervals
for the total, inter-method, and intra-method CCC estimates.}

\item{conf_level}{Confidence level for delta-method confidence intervals.}

\item{max_iter}{Positive integer. Maximum optimiser iterations.}

\item{tol}{Positive numeric convergence tolerance.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads to make
available to compiled code.}

\item{verbose}{Logical. If \code{TRUE}, emit progress messages.}
}
\value{
A symmetric numeric matrix of class \code{c("ccc_glmm", "ccc")} containing
\code{rho_ccc}. Additional pairwise matrices are stored as attributes:
\itemize{
\item \code{rho_ccc_inter}: agreement for replicated method averages.
\item \code{rho_ccc_intra_method1}, \code{rho_ccc_intra_method2}:
method-specific repeatability coefficients.
\item \code{sigma2_subject}, \code{sigma2_method},
\code{sigma2_subject_method}: fitted variance/disagreement components.
\item \code{phi}, \code{mu}, \code{precision}, \code{accuracy}: fitted
count-scale diagnostics used in the CCC decomposition.
\item \code{beta0}, \code{beta_method}, \code{nll}, \code{logLik},
\code{convergence_code}: fitting diagnostics.
\item \code{n_obs}, \code{n_subjects}, \code{m_reps}: design diagnostics.
\item when \code{ci = TRUE}, standard error and confidence-limit matrices
for \code{rho_ccc}, \code{rho_ccc_inter},
\code{rho_ccc_intra_method1}, and \code{rho_ccc_intra_method2}.
}
}
\description{
Computes pairwise generalized concordance correlation coefficients (CCC) for
count outcomes measured repeatedly by two or more methods, observers, or
devices. The current implementation fits Poisson-log generalized linear
mixed models and reports total, inter-method, and intra-method agreement
summaries from the fitted mean and variance components.
}
\details{
The fitted model for a pair of methods is
\deqn{
Y_{ijl} \mid \alpha_i, \gamma_{ij}
  \sim \mathrm{Poisson}(\mu_{ijl}),
\qquad
\log(\mu_{ijl}) = \eta_j + \alpha_i + \gamma_{ij},
}
where \eqn{i} indexes subjects, \eqn{j} indexes methods, and \eqn{l}
indexes replicate readings. The subject effect \eqn{\alpha_i} captures
between-subject heterogeneity. When \code{include_subject_method = TRUE},
\eqn{\gamma_{ij}} captures subject-specific method departures; otherwise it
is fixed at zero. The fixed method difference contributes to disagreement
through \code{sigma2_method}.

The model is fitted by marginal maximum likelihood using Gauss-Hermite
quadrature. The random-intercept model uses 40 quadrature points. The
subject-by-method model uses tensor-product quadrature with fewer points per
dimension because it integrates over a three-dimensional subject block. The
reported CCC quantities are then computed from the fitted Poisson-log mean
and variance components.

The main matrix value, \code{rho_ccc}, is the total agreement coefficient.
Use it as the primary overall count-agreement summary when each individual
reading is the unit of inference. It penalizes lack of between-subject
signal, systematic method bias, subject-by-method disagreement, and Poisson
residual variation.

\code{rho_ccc_inter} is the inter-method agreement coefficient for averages
over replicated readings. It is useful when decisions are based on the mean
of \eqn{m} replicate readings per subject-method cell. Replication reduces
only the residual count variation term; it does not dilute systematic method
disagreement or between-subject variation.

\code{rho_ccc_intra_method1} and \code{rho_ccc_intra_method2} are
method-specific repeatability coefficients. Use them to diagnose whether one
method is internally more repeatable than the other on the count scale. These
are not direct method-comparison coefficients; they describe within-method
reproducibility after accounting for the fitted mean and random effects.

\code{precision} isolates the share of non-systematic variation attributable
to subject ranking/heterogeneity, while \code{accuracy} is the ratio
\code{rho_ccc / precision}. Low accuracy with reasonable precision indicates
that disagreement is driven mainly by method bias or extra method-specific
variation rather than poor subject discrimination.

\code{overdispersion = "none"} fixes \eqn{\phi = 1}, the Poisson model.
\code{overdispersion = "pearson"} replaces the residual term by a Pearson
dispersion estimate. Use the Pearson adjustment as a sensitivity analysis
when counts appear more variable than the Poisson model allows; it should
generally reduce CCC when extra-Poisson variation is present.
\subsection{Confidence intervals}{

When \code{ci = TRUE}, large-sample confidence intervals are computed for
\code{rho_ccc}, \code{rho_ccc_inter}, \code{rho_ccc_intra_method1}, and
\code{rho_ccc_intra_method2}. The implementation uses a delta-method
standard error based on the fitted GLMM parameter vector and the inverse
Hessian of the marginal negative log-likelihood:
\deqn{
\mathrm{Var}\{g(\hat\theta)\}
  \approx
  \nabla g(\hat\theta)^\top
  \mathrm{Var}(\hat\theta)
  \nabla g(\hat\theta),
}
where \eqn{g(\theta)} is the relevant CCC function. Gradients are evaluated
numerically by central finite differences. The reported point estimates are
not replaced by \eqn{g(\hat\theta)}; they remain the values from the standard
point-estimate path.

Intervals are formed on a Fisher-Z transformed CCC scale,
\eqn{z = 0.5\log\{(1+\rho)/(1-\rho)\}}, and then back-transformed to the
CCC scale. This is the same broad strategy used elsewhere for CCC-style
intervals because it is usually more stable than a raw Wald interval near
the boundaries. CI limits are not forcibly truncated to \eqn{[0, 1]}.

For \code{overdispersion = "pearson"}, \eqn{\phi} is a post-fit Pearson
dispersion estimate rather than a likelihood parameter. Delta-method
confidence intervals therefore treat \eqn{\phi} as fixed and a warning is
issued.

This function currently implements the Poisson-log count-data case. The
\code{family} and \code{link} arguments are present for API stability and
future extensions, but only \code{family = "poisson"} and
\code{link = "log"} are currently supported. The returned estimates are
variance-component CCCs constrained to \eqn{[0, 1]}; they are not Lin's raw
moment CCC and should not be expected to produce negative values.
}
}
\examples{
# Example 1: overall agreement for two Poisson count methods.
# Here the methods have similar means, so rho_ccc is mainly driven by
# between-subject count heterogeneity versus Poisson residual noise.
set.seed(1)
df1 <- expand.grid(
  subject = factor(seq_len(12)),
  method = factor(c("A", "B")),
  replicate = factor(seq_len(2))
)
subject_eff <- rnorm(12, 0, 0.5)
df1$eta <- 1.1 + subject_eff[as.integer(df1$subject)]
df1$y <- rpois(nrow(df1), exp(df1$eta))
\donttest{
fit1 <- ccc_glmm(df1, "y", "subject", "method", replicate = "replicate",
ci = TRUE)
fit1
summary(fit1)
estimate(fit1)
tidy(fit1)
ci(fit1)
confint(fit1)
}

# Example 2: method bias lowers total agreement.
# This tests whether a systematic method shift is reflected in rho_ccc
# and the accuracy component.
set.seed(2)
df2 <- expand.grid(
  subject = factor(seq_len(12)),
  method = factor(c("A", "B")),
  replicate = factor(seq_len(2))
)
subject_eff <- rnorm(12, 0, 0.6)
df2$eta <- 1.0 + subject_eff[as.integer(df2$subject)] +
  ifelse(df2$method == "B", 0.6, 0)
df2$y <- rpois(nrow(df2), exp(df2$eta))
fit2 <- ccc_glmm(df2, "y", "subject", "method", replicate = "replicate")
summary(fit2)

# Example 3: subject-by-method variation.
# This tests whether individual subjects respond differently by method.
# The subject-method component is useful when disagreement is not explained
# by a single fixed method bias.
set.seed(3)
df3 <- expand.grid(
  subject = factor(seq_len(10)),
  method = factor(c("A", "B")),
  replicate = factor(seq_len(2))
)
subject_eff <- rnorm(10, 0, 0.4)
subject_method_eff <- matrix(rnorm(20, 0, 0.25), nrow = 10)
method_id <- as.integer(df3$method)
df3$eta <- 1.1 + subject_eff[as.integer(df3$subject)] +
  subject_method_eff[cbind(as.integer(df3$subject), method_id)]
df3$y <- rpois(nrow(df3), exp(df3$eta))
\donttest{
fit3 <- ccc_glmm(
  df3, "y", "subject", "method",
  replicate = "replicate",
  include_subject_method = TRUE
)
attr(fit3, "sigma2_subject_method")
}

# Example 4: four methods.
# This tests pairwise agreement across several count methods and helps
# identify which method pairs have the strongest total CCC.
set.seed(4)
df4 <- expand.grid(
  subject = factor(seq_len(10)),
  method = factor(c("A", "B", "C", "D")),
  replicate = factor(seq_len(2))
)
subject_eff <- rnorm(10, 0, 0.5)
method_bias <- c(A = 0, B = 0.1, C = 0.4, D = -0.2)
df4$eta <- 1.0 + subject_eff[as.integer(df4$subject)] +
  method_bias[as.character(df4$method)]
df4$y <- rpois(nrow(df4), exp(df4$eta))
fit4 <- ccc_glmm(df4, "y", "subject", "method", replicate = "replicate")
fit4
summary(fit4, n = 3)

}
\references{
Carrasco JL (2010). A generalized concordance correlation coefficient based
on the variance components generalized linear mixed models for overdispersed
count data. \emph{Biometrics}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{num_or_na_vec}
\alias{num_or_na_vec}
\title{num_or_na_vec}
\usage{
num_or_na_vec(x)
}
\description{
Display variance component estimation details to the console.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{num_or_na}
\alias{num_or_na}
\title{num_or_na}
\usage{
num_or_na(x)
}
\description{
Helper to safely coerce a value to numeric or return NA if invalid.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wincor.R
\name{wincor}
\alias{wincor}
\alias{print.wincor}
\alias{plot.wincor}
\alias{summary.wincor}
\alias{print.summary.wincor}
\title{Pairwise Winsorized correlation}
\usage{
wincor(
  data,
  na_method = c("error", "pairwise", "complete"),
  ci = FALSE,
  p_value = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  tr = 0.2,
  n_boot = 500L,
  seed = NULL,
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

\method{print}{wincor}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{wincor}(
  x,
  title = "Winsorized correlation heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  show_value = TRUE,
  ...
)

\method{summary}{wincor}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  ci_digits = 3,
  p_digits = 4,
  show_ci = NULL,
  ...
)

\method{print}{summary.wincor}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns will be excluded.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each estimate on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{ci}{Logical (default \code{FALSE}). If \code{TRUE}, attach percentile
bootstrap confidence intervals for each pairwise estimate.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach the
method-specific large-sample test statistic and two-sided p-value for each
pairwise estimate.}

\item{conf_level}{Confidence level used when \code{ci = TRUE}. Default
\code{0.95}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{tr}{Winsorization proportion in \code{[0, 0.5)}. For a sample of size
\eqn{n}, let \eqn{g = \lfloor tr \cdot n \rfloor}; the \eqn{g} smallest
observations are set to the \eqn{(g+1)}-st order statistic and the
\eqn{g} largest observations are set to the \eqn{(n-g)}-th order
statistic. Default \code{0.2}.}

\item{n_boot}{Integer \eqn{\geq 1}. Number of bootstrap resamples used when
\code{ci = TRUE}. Default \code{500}.}

\item{seed}{Optional positive integer used to seed the bootstrap resampling
when \code{ci = TRUE}. If \code{NULL}, the current random-number stream is
used.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{x}{An object of class \code{summary.wincor}.}

\item{digits}{Integer; number of digits to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Additional arguments passed to the underlying print or plot helper.}

\item{title}{Character; plot title.}

\item{low_color, high_color, mid_color}{Colors used in the heatmap.}

\item{value_text_size}{Numeric text size for overlaid cell values.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}

\item{object}{An object of class \code{wincor}.}

\item{ci_digits}{Integer; digits used for confidence limits in pairwise
summaries.}

\item{p_digits}{Integer; digits used for p-values in pairwise summaries.}
}
\value{
A symmetric correlation matrix with class \code{wincor} and
attributes \code{method = "winsorized_correlation"}, \code{description},
and \code{package = "matrixCorr"}. When \code{ci = TRUE}, the returned
object also carries a \code{ci} attribute with elements \code{est},
\code{lwr.ci}, \code{upr.ci}, \code{conf.level}, and \code{ci.method},
plus \code{attr(x, "conf.level")}. When \code{p_value = TRUE}, it also
carries an \code{inference} attribute with elements \code{estimate},
\code{statistic}, \code{parameter}, \code{p_value}, \code{n_obs}, and
\code{alternative}. When either inferential option is requested, the
object also carries \code{diagnostics$n_complete}.
}
\description{
Computes all pairwise Winsorized correlation coefficients for the numeric
columns of a matrix or data frame using a high-performance 'C++' backend.

This function Winsorizes each margin at proportion \code{tr} and then
computes ordinary Pearson correlation on the Winsorized values. It is a
simple robust alternative to Pearson correlation when the main concern is
unusually large or small observations in the marginal distributions.
}
\details{
Let \eqn{X \in \mathbb{R}^{n \times p}} be a numeric matrix with rows as
observations and columns as variables. For a column
\eqn{x = (x_i)_{i=1}^n}, write the order statistics as
\eqn{x_{(1)} \le \cdots \le x_{(n)}} and let
\eqn{g = \lfloor tr \cdot n \rfloor}. The Winsorized values can be written as
\deqn{
x_i^{(w)} \;=\; \max\!\bigl\{x_{(g+1)},\, \min(x_i, x_{(n-g)})\bigr\}.
}
For two columns \eqn{x} and \eqn{y}, the Winsorized correlation is the
ordinary Pearson correlation computed from \eqn{x^{(w)}} and \eqn{y^{(w)}}:
\deqn{
r_w(x,y) \;=\;
\frac{\sum_{i=1}^n (x_i^{(w)}-\bar x^{(w)})(y_i^{(w)}-\bar y^{(w)})}
     {\sqrt{\sum_{i=1}^n (x_i^{(w)}-\bar x^{(w)})^2}\;
      \sqrt{\sum_{i=1}^n (y_i^{(w)}-\bar y^{(w)})^2}}.
}

In matrix form, let \eqn{X^{(w)}} contain the Winsorized columns and define
the centred, unit-norm columns
\deqn{
z_{\cdot j} =
\frac{x_{\cdot j}^{(w)} - \bar x_j^{(w)} \mathbf{1}}
     {\sqrt{\sum_{i=1}^n (x_{ij}^{(w)}-\bar x_j^{(w)})^2}},
\qquad j=1,\ldots,p.
}
If \eqn{Z = [z_{\cdot 1}, \ldots, z_{\cdot p}]}, then the Winsorized
correlation matrix is
\deqn{
R_w \;=\; Z^\top Z.
}

Winsorization acts on each margin separately, so it guards against marginal
outliers and heavy tails but does not target unusual points in the joint
cloud. This implementation Winsorizes each column in 'C++', centres and
normalises it, and forms the complete-data matrix from cross-products. With
\code{na_method = "pairwise"}, each pair is recomputed on its overlap of
non-missing rows. As with Pearson correlation, the complete-data path yields
a symmetric positive semidefinite matrix, whereas pairwise deletion can
break positive semidefiniteness. If the Winsorized variance of a column is
zero, correlations involving that column are returned as \code{NA}.

When \code{p_value = TRUE}, inference follows the method-specific test based
on
\deqn{
T_{ij} = r_{w,ij}\sqrt{\frac{n_{ij} - 2}{1 - r_{w,ij}^2}},
}
evaluated against a \eqn{t}-distribution with
\eqn{n_{ij} - 2g_{ij} - 2} degrees of freedom, where
\eqn{g_{ij} = \lfloor tr \cdot n_{ij} \rfloor} and \eqn{n_{ij}} is the
pairwise complete-case sample size for the corresponding column pair. The
p-value is reported only when the pair is not identical and the resulting
degrees of freedom are positive. When \code{ci = TRUE}, the interval is a
percentile bootstrap interval based on \eqn{n_{\mathrm{boot}}} resamples
drawn from the pairwise complete cases. If
\eqn{\tilde r_{w,(1)} \le \cdots \le \tilde r_{w,(B)}} denotes the sorted
bootstrap sample of finite estimates with \eqn{B} retained resamples, the
reported limits are
\deqn{
\tilde r_{w,(\ell)} \quad \text{and} \quad \tilde r_{w,(u)},
}
where \eqn{\ell = \lfloor (\alpha/2) B + 0.5 \rfloor} and
\eqn{u = \lfloor (1-\alpha/2) B + 0.5 \rfloor} for
\eqn{\alpha = 1 - \mathrm{conf\_level}}. Resamples that yield undefined
estimates are discarded before the percentile limits are formed.

\strong{Computational complexity.} In the complete-data path, Winsorizing the
columns requires sorting within each column, and forming the cross-product
matrix costs \eqn{O(n p^2)} with \eqn{O(p^2)} output storage. When
\code{ci = TRUE}, the bootstrap cost is incurred separately for each column
pair.
}
\examples{
set.seed(11)
X <- matrix(rnorm(180 * 4), ncol = 4)
X[sample(length(X), 6)] <- X[sample(length(X), 6)] - 12

R <- wincor(X, tr = 0.2)
print(R, digits = 2)
summary(R)
estimate(R)
tidy(R)
plot(R)

## Bootstrap confidence intervals
R_ci <- wincor(X, tr = 0.2, ci = TRUE, n_boot = 49, seed = 11)
ci(R_ci)
confint(R_ci)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(R)
}

}
\references{
Wilcox, R. R. (1993). Some results on a Winsorized correlation coefficient.
British Journal of Mathematical and Statistical Psychology, 46(2), 339-349.
\doi{10.1111/j.2044-8317.1993.tb01020.x}

Wilcox, R. R. (2012). Introduction to Robust Estimation and Hypothesis
Testing (3rd ed.). Academic Press.
}
\seealso{
\code{\link[=pbcor]{pbcor()}}, \code{\link[=skipped_corr]{skipped_corr()}}, \code{\link[=bicor]{bicor()}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icc.R
\name{icc_rm_reml}
\alias{icc_rm_reml}
\alias{print.icc_rm_reml}
\alias{summary.icc_rm_reml}
\alias{print.summary.icc_rm_reml}
\title{Repeated-Measures Intraclass Correlation via REML}
\usage{
icc_rm_reml(
  data,
  response,
  subject,
  method = NULL,
  time = NULL,
  type = c("consistency", "agreement"),
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  ci_mode = c("auto", "raw", "logit"),
  verbose = FALSE,
  digits = 4,
  use_message = TRUE,
  interaction = FALSE,
  max_iter = 100,
  tol = 1e-06,
  Dmat = NULL,
  Dmat_type = c("time-avg", "typical-visit", "weighted-avg", "weighted-sq"),
  Dmat_weights = NULL,
  Dmat_rescale = TRUE,
  ar = c("none", "ar1"),
  ar_rho = NA_real_,
  slope = c("none", "subject", "method", "custom"),
  slope_var = NULL,
  slope_Z = NULL,
  drop_zero_cols = TRUE,
  vc_select = c("auto", "none"),
  vc_alpha = 0.05,
  vc_test_order = c("subj_time", "subj_method"),
  include_subj_method = NULL,
  include_subj_time = NULL,
  sb_zero_tol = 1e-10
)

\method{print}{icc_rm_reml}(
  x,
  digits = 4,
  ci_digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{icc_rm_reml}(
  object,
  digits = 4,
  ci_digits = 2,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.icc_rm_reml}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{response}{Character. Response variable name.}

\item{subject}{Character. Subject ID variable name.}

\item{method}{Character or \code{NULL}. Optional column name of method factor
(added to fixed effects).}

\item{time}{Character or \code{NULL}. Optional column name of time factor
(added to fixed effects).}

\item{type}{Character scalar; one of \code{c("consistency","agreement")}.
For \code{"consistency"}, systematic fixed method differences are not
penalized through \eqn{S_B}. For \code{"agreement"}, the fixed-effect
dispersion term \eqn{S_B} is added to the denominator.}

\item{ci}{Logical. If \code{TRUE}, return a CI container for the repeated ICC.}

\item{conf_level}{Numeric in \eqn{(0,1)}. Confidence level when
\code{ci = TRUE} (default \code{0.95}).}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads to use for
computation. Defaults to \code{getOption("matrixCorr.threads", 1L)}.}

\item{ci_mode}{Character scalar; one of \code{c("auto","raw","logit")}.
Controls how confidence intervals are computed when \code{ci = TRUE}.
If \code{"raw"}, a Wald CI is formed on the ICC scale and truncated to
\code{[0,1]}. If \code{"logit"}, a Wald CI is computed on the
\eqn{\mathrm{logit}(\mathrm{ICC})} scale and back-transformed to the
original scale. If \code{"auto"} (default), the backend chooses per estimate
between the raw-scale and logit-scale interval.}

\item{verbose}{Logical. If \code{TRUE}, prints a structured summary of the
fitted variance components and \eqn{S_B} for each fit. Default \code{FALSE}.}

\item{digits}{Integer \eqn{(\ge 0)}. Number of decimal places to use in the
printed summary when \code{verbose = TRUE}. Default \code{4}.}

\item{use_message}{Logical. When \code{verbose = TRUE}, verbose summaries
are emitted with \pkg{cli} messages.}

\item{interaction}{Logical. Include \code{method:time} interaction?
(default \code{FALSE}).}

\item{max_iter}{Integer. Maximum iterations for variance-component updates
(default \code{100}).}

\item{tol}{Numeric. Convergence tolerance on parameter change
(default \code{1e-6}).}

\item{Dmat}{Optional \eqn{n_t \times n_t} numeric matrix to weight/aggregate
time-specific fixed biases in the \eqn{S_B} quadratic form. If supplied, it
is used (after optional mass rescaling; see \code{Dmat_rescale}) whenever at
least two \emph{present} time levels exist; otherwise it is ignored. \strong{If
\code{Dmat} is \code{NULL}}, a canonical kernel \eqn{D_m} is \emph{constructed}
from \code{Dmat_type} and \code{Dmat_weights} (see below). \code{Dmat} should be
symmetric positive semidefinite; small asymmetries are symmetrized internally.}

\item{Dmat_type}{Character, one of \code{c("time-avg","typical-visit",
  "weighted-avg","weighted-sq")}. Only used when \code{Dmat = NULL}.
It selects the aggregation target for time-specific fixed biases in
\eqn{S_B}. Options are:

\itemize{
\item \code{"time-avg"}: square of the time-averaged bias, \eqn{D_m=(1/n_t)\,11^\top}.
\item \code{"typical-visit"}: average of squared per-time biases, \eqn{D_m=I_{n_t}}.
\item \code{"weighted-avg"}: square of a weighted average, \eqn{D_m=n_t\,w\,w^\top} with \eqn{\sum w=1}.
\item \code{"weighted-sq"}: weighted average of squared biases, \eqn{D_m=n_t\,\mathrm{diag}(w)} with \eqn{\sum w=1}.
}
Pick \code{"time-avg"} for ICC targeting the time-averaged measurement; pick
\code{"typical-visit"} for ICC targeting a randomly sampled visit (typical
occasion). Default \code{"time-avg"}.}

\item{Dmat_weights}{Optional numeric weights \eqn{w} used when
\code{Dmat_type \%in\% c("weighted-avg","weighted-sq")}. Must be nonnegative and
finite. If \code{names(w)} are provided, they should match the \emph{full} time
levels in \code{data}; they are aligned to the \emph{present} time subset per fit.
If unnamed, the length must equal the number of present time levels. In all cases
\eqn{w} is internally normalized to sum to 1.}

\item{Dmat_rescale}{Logical. When \code{TRUE} (default), the supplied/built
\eqn{D_m} is rescaled to satisfy the simple mass rule
\eqn{1^\top D_m 1 = n_t}. This keeps the \eqn{S_B} denominator invariant and
harmonizes with the time-averaging factors used for the variance terms.}

\item{ar}{Character. Residual correlation structure: \code{"none"} (iid) or
\code{"ar1"} for subject-level AR(1) correlation within contiguous time
runs. Default \code{c("none")}.}

\item{ar_rho}{Numeric in \eqn{(-0.999,\,0.999)} or \code{NA}.
If \code{ar = "ar1"} and \code{ar_rho} is finite, it is treated as fixed.
If \code{ar = "ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
profiling a 1-D objective (REML when available; an approximation otherwise).
Default \code{NA_real_}.}

\item{slope}{Character. Optional extra random-effect design \eqn{Z}.
With \code{"subject"} a single random slope is added (one column in \eqn{Z});
with \code{"method"} one column per method level is added; with
\code{"custom"} you provide \code{slope_Z} directly. Default
\code{c("none","subject","method","custom")}.}

\item{slope_var}{For \code{slope \%in\% c("subject","method")}, a character
string giving the name of a column in \code{data} used as the slope regressor
(e.g., centered time). It is looked up inside \code{data}; do not
pass the vector itself. NAs are treated as zeros in \eqn{Z}.}

\item{slope_Z}{For \code{slope = "custom"}, a numeric matrix with \eqn{n}
rows (same order as \code{data}) providing the full extra random-effect
design \eqn{Z}. \strong{Each column of \code{slope_Z} has its own variance
component} \eqn{\sigma_{Z,j}^2}; columns are treated as \emph{uncorrelated}
(diagonal block in \eqn{G}). Ignored otherwise.}

\item{drop_zero_cols}{Logical. When \code{slope = "method"}, drop all-zero
columns of \eqn{Z} after subsetting (useful in pairwise fits). Default
\code{TRUE}.}

\item{vc_select}{Character scalar; one of \code{c("auto","none")}.
Controls how the subject by method \eqn{\sigma^2_{A\times M}} ("subj_method") and
subject by time \eqn{\sigma^2_{A\times T}} ("subj_time") variance components are
included. If \code{"auto"} (default), the function performs boundary-aware
REML likelihood-ratio tests (LRTs; null on the boundary at zero with a
half-\eqn{\chi^2_1} reference) to decide whether to retain each component,
in the order given by \code{vc_test_order}. If \code{"none"}, no testing
is done and inclusion is taken from \code{include_subj_method}/\code{include_subj_time}
(or, if \code{NULL}, from the mere presence of the corresponding factor in
the design). In pairwise fits, the decision is made independently for each
method pair.}

\item{vc_alpha}{Numeric scalar in \eqn{(0,1)}; default \code{0.05}.
Per-component significance level for the boundary-aware REML LRTs used when
\code{vc_select = "auto"}. The tests are one-sided for variance components
on the boundary and are not multiplicity-adjusted.}

\item{vc_test_order}{Character vector (length 2) with a permutation of
\code{c("subj_time","subj_method")}; default \code{c("subj_time","subj_method")}. Specifies the order
in which the two variance components are tested when \code{vc_select = "auto"}.
The component tested first may be dropped before testing the second. If a
factor is absent in the design (e.g., no time factor so "subj_time" is undefined),
the corresponding test is skipped.}

\item{include_subj_method, include_subj_time}{Logical scalars or \code{NULL}.
When \code{vc_select = "none"}, these control whether the
\eqn{\sigma^2_{A\times M}} ("subj_method") and \eqn{\sigma^2_{A\times T}} ("subj_time")
random effects are included (\code{TRUE}) or excluded (\code{FALSE}) in the model.
If \code{NULL} (default), inclusion defaults to the presence of the
corresponding factor in the data (i.e., at least two method/time levels).
When \code{vc_select = "auto"}, these arguments are ignored (automatic
selection is used instead).}

\item{sb_zero_tol}{Non-negative numeric scalar; default \code{1e-10}.
Numerical threshold for the fixed-effect dispersion term \eqn{S_B}.
After computing \eqn{\widehat{S_B}} and its delta-method variance, if
\eqn{\widehat{S_B} \le} \code{sb_zero_tol} or non-finite, the procedure
treats \eqn{S_B} as fixed at zero in the delta step.}

\item{x}{For \code{print()}, an object returned by \code{icc_rm_reml()} or
\code{summary.icc_rm_reml()}.}

\item{ci_digits}{Integer; number of digits for confidence interval bounds in
printed method summaries.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives
this from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Passed to the underlying display helpers.}

\item{object}{For \code{summary()}, an object returned by \code{icc_rm_reml()}.}
}
\value{
A repeated-measures pairwise ICC object. Without confidence intervals the
result is a symmetric matrix of class \code{c("icc_rm_reml", "icc", "matrix")}.
With confidence intervals it is a list with \code{est}, \code{lwr.ci}, and \code{upr.ci}
and class \code{c("icc_rm_reml", "icc_ci", "icc")}. Both carry the fitted
variance-component matrices as attributes.
}
\description{
Computes pairwise repeated-measures intraclass correlation coefficients from
long-format data using the same REML and Woodbury-identity backend used by
the repeated-measures agreement models.
}
\details{
The repeated-measures model is fit separately for each method pair using the
same REML and Woodbury-identity backend used by the repeated-measures
concordance estimator.

\strong{Kernel \eqn{D_m} and the fixed-bias term \eqn{S_B}.}
Let \eqn{d = L^\top \hat\beta} stack the within-time, pairwise method
differences, grouped by time. The symmetric positive semidefinite kernel
\eqn{D_m \succeq 0} selects which functional of the bias profile is targeted
by \eqn{S_B}. Internally, the code rescales any supplied or constructed
\eqn{D_m} to satisfy \eqn{1^\top D_m 1 = n_t} for stability and comparability.

\itemize{
\item \code{Dmat_type = "time-avg"} targets the square of the
time-averaged bias.
\item \code{Dmat_type = "typical-visit"} targets the average of squared
per-time biases.
\item \code{Dmat_type = "weighted-avg"} targets the square of a weighted
time average.
\item \code{Dmat_type = "weighted-sq"} targets the weighted average of
squared per-time biases.
}

As in the repeated-measures concordance implementation, \eqn{S_B} is the
fitted fixed-effect dispersion term induced by \eqn{D_m}. It enters the
denominator only for \code{type = "agreement"}.

\strong{Time-averaging and shrinkage factors.}
The fitted variance components reported in the summary are
\eqn{\sigma_S^2}, \eqn{\sigma_{S\times M}^2}, \eqn{\sigma_{S\times T}^2},
and \eqn{\sigma_E^2}, stored respectively as \code{sigma2_subject},
\code{sigma2_subject_method}, \code{sigma2_subject_time}, and \code{sigma2_error}.

The repeated-measures ICC always uses \eqn{\sigma_S^2} in the numerator only.
The denominator uses the same time-averaging logic already used by the
repeated-measures concordance backend, through two pair-specific factors
\eqn{\bar{\kappa}_g} and \eqn{\bar{\kappa}_e}.

If \code{time} is absent, the implementation sets
\deqn{ \bar{\kappa}_g = 0, \qquad \bar{\kappa}_e = 1. }

If \code{time} is present and the target is a single visit
(\code{Dmat_type \%in\% c("typical-visit", "weighted-sq")}), the implementation
leaves the time-varying terms unshrunk:
\deqn{ \bar{\kappa}_g = 1, \qquad \bar{\kappa}_e = 1. }

If \code{time} is present and the target is time-averaged
(\code{Dmat_type \%in\% c("time-avg", "weighted-avg")}), then for each observed
subject-method unit with \eqn{T} distinct observed visits:
\itemize{
\item with equal weights,
\deqn{ \kappa_g = \frac{1}{T}, \qquad
         \kappa_e = \frac{T + 2\sum_{h=1}^{T-1}(T-h)\rho^h}{T^2}, }
with \eqn{\kappa_e=\kappa_g} when residuals are iid;
\item with normalized visit weights \eqn{w_1,\ldots,w_T},
\deqn{ \kappa_g = \sum_{t=1}^{T} w_t^2, \qquad
         \kappa_e = \sum_{t=1}^{T}\sum_{s=1}^{T} w_t w_s \rho^{|t-s|}, }
with \eqn{\kappa_e=\kappa_g} when residuals are iid.
}

With unbalanced \eqn{T}, the implementation averages the per-unit
\eqn{\kappa} values across the observations contributing to the pair and then
clamps both \eqn{\bar{\kappa}_g} and \eqn{\bar{\kappa}_e} to
\eqn{[10^{-12},\,1]} for numerical stability.

\strong{Repeated-measures ICC.}
Let \eqn{\sigma_{S\times M,\mathrm{eff}}^2} denote the effective
subject-by-method variance term, equal to \eqn{\sigma_{S\times M}^2} when
the subject-by-method random effect is included and \eqn{0} otherwise. Let
\eqn{\sigma_{S\times T,\mathrm{eff}}^2} denote the effective subject-by-time
variance term, equal to \eqn{\sigma_{S\times T}^2} when the subject-by-time
random effect is included and \eqn{0} otherwise.

For \code{type = "consistency"}, the reported ICC is
\deqn{ \mathrm{ICC}_{\mathrm{consistency}} \;=\;
      \frac{\sigma_S^2}{
      \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
      \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
      \bar{\kappa}_e\,\sigma_E^2}. }

For \code{type = "agreement"}, the denominator additionally includes \eqn{S_B}:
\deqn{ \mathrm{ICC}_{\mathrm{agreement}} \;=\;
      \frac{\sigma_S^2}{
      \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
      \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
      \bar{\kappa}_e\,\sigma_E^2 + S_B}. }

This differs from repeated-measures concordance because ICC uses only
\eqn{\sigma_S^2} in the numerator, whereas the concordance numerator also
includes the time-averaged subject-time term. Extra random-effect variances
\eqn{\{\sigma_{Z,j}^2\}} from \code{slope} / \code{slope_Z} are estimated by the shared
backend but are not included in the ICC denominator.

\strong{CIs / SEs (delta method for ICC).}
Let \eqn{I_A = 1} for \code{type = "agreement"} and \eqn{I_A = 0} for
\code{type = "consistency"}, and define
\deqn{ \theta \;=\; \big(\sigma_S^2,\ \sigma_{S\times M,\mathrm{eff}}^2,\
\sigma_{S\times T,\mathrm{eff}}^2,\ \sigma_E^2,\ S_B\big)^\top. }
Write \eqn{\mathrm{ICC}(\theta)=N/D} with
\deqn{ N = \sigma_S^2, \qquad
       D = \sigma_S^2 + \sigma_{S\times M,\mathrm{eff}}^2 +
           \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
           \bar{\kappa}_e\,\sigma_E^2 + I_A S_B. }
The gradient used in the delta method is
\deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_S^2}
      \;=\; \frac{
      \sigma_{S\times M,\mathrm{eff}}^2 +
      \bar{\kappa}_g\,\sigma_{S\times T,\mathrm{eff}}^2 +
      \bar{\kappa}_e\,\sigma_E^2 + I_A S_B}{D^2}, }
\deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_{S\times M,\mathrm{eff}}^2}
      \;=\; -\,\frac{N}{D^2}, \qquad
       \frac{\partial\,\mathrm{ICC}}{\partial \sigma_{S\times T,\mathrm{eff}}^2}
      \;=\; -\,\frac{\bar{\kappa}_g\,N}{D^2}, }
\deqn{ \frac{\partial\,\mathrm{ICC}}{\partial \sigma_E^2}
      \;=\; -\,\frac{\bar{\kappa}_e\,N}{D^2}, \qquad
       \frac{\partial\,\mathrm{ICC}}{\partial S_B}
      \;=\; -\,\frac{I_A\,N}{D^2}. }

The covariance matrix \eqn{\widehat{\mathrm{Var}}(\hat\theta)} is assembled
from the same REML fit:
\itemize{
\item the
\eqn{(\sigma_S^2,\sigma_{S\times M,\mathrm{eff}}^2,
  \sigma_{S\times T,\mathrm{eff}}^2)} block comes from the empirical
subject-level covariance of the per-subject REML component updates;
\item \eqn{\widehat{\mathrm{Var}}(\hat\sigma_E^2)} is approximated as the
variance of the weighted mean of subject-level residual quadratic forms;
\item \eqn{\widehat{\mathrm{Var}}(S_B)} uses the fixed-effect quadratic-form
delta method already computed in the backend.
}
Cross-covariances across these blocks are ignored as a large-sample
simplification, so
\deqn{ \widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}
      \;=\; \sqrt{\,\nabla \mathrm{ICC}(\hat\theta)^\top\,
                    \widehat{\mathrm{Var}}(\hat\theta)\,
                    \nabla \mathrm{ICC}(\hat\theta)\,}. }

If \code{ci_mode = "raw"}, a Wald interval is formed on the ICC scale,
\deqn{ \widehat{\mathrm{ICC}} \;\pm\; z_{1-\alpha/2}\,
      \widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}, }
and truncated to \eqn{[0,1]}. If \code{ci_mode = "logit"}, the backend applies
the same Wald construction after the transform
\eqn{\phi = \mathrm{logit}(\mathrm{ICC})}, with
\deqn{ \widehat{\mathrm{se}}(\hat\phi)
      \;\approx\; \frac{\widehat{\mathrm{se}}\{\widehat{\mathrm{ICC}}\}}
      {\widehat{\mathrm{ICC}}\,(1-\widehat{\mathrm{ICC}})}, }
and then back-transforms
\deqn{ \mathrm{logit}^{-1}\!\Big(
      \hat\phi \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\phi)\Big). }
If \code{ci_mode = "auto"}, the backend selects between the raw-scale and
logit-scale interval per estimate, typically preferring the logit form near
the boundaries.

\strong{Choosing \eqn{\rho} for AR(1).}
When \code{ar="ar1"} and \code{ar_rho = NA}, \eqn{\rho} is estimated by
profiling the REML log-likelihood at \eqn{(\hat\beta,\hat G,\hat\sigma_E^2)}.
With very few visits per subject, \eqn{\rho} can be weakly identified; consider
sensitivity checks over a plausible range.
}
\section{Notes on stability and performance}{

All per-subject solves are \eqn{r\times r} with
\eqn{r = 1 + n_m + n_t + q_Z}, so cost scales with the number of subjects and
the fixed-effects dimension rather than the total number of observations.
Solvers use symmetric positive-definite paths with a small diagonal ridge and
pseudo-inverse fallback, which helps for very small or unbalanced subsets and
near-boundary estimates. For \code{AR(1)}, observations are ordered by time
within subject; \code{NA} time codes break the run, and gaps between factor
levels are treated as regular steps.

Heteroscedastic slopes across \eqn{Z} columns are supported. Each \eqn{Z}
column has its own variance component \eqn{\sigma_{Z,j}^2}, but
cross-covariances among \eqn{Z} columns are set to zero.
}

\section{Threading and BLAS guards}{

The C++ backend uses OpenMP loops while also forcing vendor BLAS libraries to
run single-threaded so that overall CPU usage stays predictable. This guard
is applied to OpenBLAS, Apple's Accelerate, and Intel MKL when their runtime
controls are available. You can opt out manually by setting
\code{MATRIXCORR_DISABLE_BLAS_GUARD=1} in the environment before loading the
package.
}

\examples{
set.seed(321)
n_id <- 20
n_time <- 3
id <- factor(rep(seq_len(n_id), each = 2 * n_time))
method <- factor(rep(rep(c("A", "B"), each = n_time), times = n_id))
time <- factor(rep(seq_len(n_time), times = 2 * n_id))

subj <- rnorm(n_id, sd = 1)[as.integer(id)]
subj_method <- rnorm(n_id * 2, sd = 0.25)
sm <- subj_method[(as.integer(id) - 1L) * 2L + as.integer(method)]
y <- subj + sm + 0.3 * (method == "B") + rnorm(length(id), sd = 0.35)

dat_rm <- data.frame(y = y, id = id, method = method, time = time)

fit_icc_rm <- icc_rm_reml(
  dat_rm,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  type = "consistency",
  ci = TRUE
)
print(fit_icc_rm)
summary(fit_icc_rm)
confint(fit_icc_rm)
tidy(fit_icc_rm)

}
\references{
Shrout PE, Fleiss JL (1979). Intraclass correlations: uses in assessing
rater reliability. Psychological Bulletin, 86(2), 420-428.

McGraw KO, Wong SP (1996). Forming inferences about some intraclass
correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\seealso{
\code{icc()}, \code{\link[=ccc]{ccc()}}, \code{\link[=ccc_rm_reml]{ccc_rm_reml()}}, \code{\link[=ba]{ba()}}, \code{\link[=ba_rm]{ba_rm()}}, \code{\link[=rmcorr]{rmcorr()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_scalar_numeric}
\alias{check_scalar_numeric}
\title{Check scalar numeric (with optional bounds)}
\usage{
check_scalar_numeric(
  x,
  arg = as.character(substitute(x)),
  finite = TRUE,
  lower = -Inf,
  upper = Inf,
  allow_na = FALSE,
  closed_lower = TRUE,
  closed_upper = TRUE
)
}
\description{
Check scalar numeric (with optional bounds)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_bool}
\alias{check_bool}
\title{Check a single logical flag}
\usage{
check_bool(x, arg = as.character(substitute(x)))
}
\description{
Check a single logical flag
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_ar1_rho}
\alias{check_ar1_rho}
\title{Check AR(1) correlation parameter}
\usage{
check_ar1_rho(x, arg = as.character(substitute(x)), bound = 0.999)
}
\description{
Check AR(1) correlation parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cia_repeated.R
\name{cia_rm}
\alias{cia_rm}
\alias{summary.cia_rm}
\alias{print.cia_rm}
\alias{print.summary.cia_rm}
\alias{plot.cia_rm}
\title{Repeated-Measures Coefficient of Individual Agreement}
\usage{
cia_rm(
  data,
  response,
  subject,
  method = NULL,
  time = NULL,
  ci = FALSE,
  conf_level = 0.95,
  n_threads = getOption("matrixCorr.threads", 1L),
  estimator = c("vc_constrained", "mom_unconstrained"),
  inference = c("delta", "bootstrap", "none"),
  verbose = FALSE,
  digits = 4,
  use_message = TRUE,
  homogeneous = FALSE,
  B = 1000L,
  seed = NULL,
  ...
)

\method{summary}{cia_rm}(
  object,
  digits = 4,
  ci_digits = 3,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{cia_rm}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.cia_rm}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{cia_rm}(
  x,
  title = NULL,
  facet_by_pair = FALSE,
  facet_scales = c("fixed", "free_y"),
  show_ci = NULL,
  show_common = TRUE,
  point_size = 2.2,
  line_size = 0.7,
  ci_alpha = 0.16,
  ci_linewidth = 0.45,
  ...
)
}
\arguments{
\item{data}{A data frame in long format.}

\item{response}{Character scalar naming the numeric response column.}

\item{subject}{Character scalar naming the subject identifier column.}

\item{method}{Character scalar naming the method column. Required.}

\item{time}{Character scalar naming the repeated condition column. Required.}

\item{ci}{Logical; if \code{TRUE}, attach confidence intervals.}

\item{conf_level}{Confidence level for intervals when \code{ci = TRUE}.}

\item{n_threads}{Positive integer thread hint passed to the C++ backend.}

\item{estimator}{One of \code{"vc_constrained"} or \code{"mom_unconstrained"}.
\code{"vc_constrained"} is the default bounded variance-component estimator. It
constrains the estimated subject-method variance component to be
non-negative before converting it back to CIA. \code{"mom_unconstrained"} is
the raw method-of-moments CIA ratio and may exceed 1.}

\item{inference}{One of \code{"delta"}, \code{"bootstrap"}, or \code{"none"}. The delta
method is the default when \code{ci = TRUE}.}

\item{verbose}{Logical; if \code{TRUE}, emit progress messages for CI work.}

\item{digits}{Integer print precision carried on the returned object.}

\item{use_message}{Logical; if \code{TRUE}, use package messaging helpers when
\code{verbose = TRUE}.}

\item{homogeneous}{Logical; if \code{TRUE}, also compute the pooled/common CIA
from the reduced model without method-time interaction.}

\item{B}{Number of subject-bootstrap resamples when \code{ci = TRUE}.}

\item{seed}{Optional positive integer seed for reproducible bootstrap
resampling.}

\item{...}{Additional arguments passed to \code{ggplot2::theme()}.}

\item{object}{A \code{cia_rm} object.}

\item{ci_digits}{Integer; number of digits for confidence interval bounds.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{Logical; if \code{TRUE} and confidence intervals are present, show
them on the plot. By default this is inferred from the object class.}

\item{x}{A \code{summary.cia_rm} object.}

\item{title}{Optional plot title.}

\item{facet_by_pair}{Logical; if \code{TRUE}, draw one panel per method pair.}

\item{facet_scales}{Passed to \code{ggplot2::facet_wrap(scales = ...)} when
\code{facet_by_pair = TRUE}. One of \code{"fixed"} or \code{"free_y"}.}

\item{show_common}{Logical; if \code{TRUE} and common estimates are available,
draw dashed horizontal reference lines for the pooled/common CIA.}

\item{point_size}{Numeric point size passed to \code{ggplot2::geom_point()}.}

\item{line_size}{Numeric line width passed to \code{ggplot2::geom_line()}.}

\item{ci_alpha}{Numeric alpha used for confidence ribbons in continuous
plots.}

\item{ci_linewidth}{Numeric line width used for confidence-interval
error bars in categorical plots.}
}
\value{
If \code{ci = FALSE}, the result is a list of class \code{c("cia_rm", "cia")}.
If \code{ci = TRUE}, the result is a list of class
\code{c("cia_rm", "cia_ci", "cia")}.

Main components:
\itemize{
\item \code{est}: condition-specific pairwise CIA estimates.
\item \code{common}: pairwise overall summary CIA estimates from the reduced
homogeneous model.
\item \code{condition}: the ordered condition/time labels used in the output.
\item \code{se}: standard errors for \code{est} when \code{ci = TRUE}.
\item \code{lwr.ci} and \code{upr.ci}: lower and upper confidence limits for \code{est}
when \code{ci = TRUE}.
\item \code{common.se}: standard errors for \code{common} when \code{ci = TRUE}.
\item \code{common.lwr.ci} and \code{common.upr.ci}: lower and upper confidence
limits for \code{common} when \code{ci = TRUE}.
}

When there are three or more methods, additional overall multi-method
components are returned:
\itemize{
\item \code{overall}: condition-specific overall CIA across all methods.
\item \code{overall.common}: the overall summary CIA across all methods when
\code{homogeneous = TRUE}.
\item \code{overall.se}: standard errors for \code{overall} when \code{ci = TRUE}.
\item \code{overall.lwr.ci} and \code{overall.upr.ci}: lower and upper confidence
limits for \code{overall} when \code{ci = TRUE}.
\item \code{overall.common.se}: standard errors for \code{overall.common} when
\code{ci = TRUE}.
\item \code{overall.common.lwr.ci} and \code{overall.common.upr.ci}: lower and upper
confidence limits for \code{overall.common} when \code{ci = TRUE}.
}

The object carries pairwise ANOVA diagnostics as attributes, including
\code{sigma2_error}, \code{sigma2_subject_method}, \code{repeatability},
\code{homogeneity_F}, \code{homogeneity_p}, \code{df_method_time}, \code{df_error},
\code{n_obs}, \code{n_subjects}, \code{n_methods}, \code{n_times}, \code{time_levels}, and
\code{method_levels}. Here \code{homogeneity_F} is the method-time interaction test
statistic for each method pair, and \code{homogeneity_p} is the corresponding
p-value for the null hypothesis of homogeneous agreement across conditions.
}
\description{
Computes pairwise repeated-measures coefficients of individual agreement
(CIA) from long-format matched repeated-measures data using the categorical
condition ANOVA formulation of Haber, Gao, and Barnhart (2010).
}
\details{
\code{cia_rm()} is for matched repeated measurements under conditions such as
raters, visits, laboratories, treatments, or time points. It is not a
technical-replicate estimator. If the same subject has true technical
replicates within each subject-method cell, use \code{\link[=cia]{cia()}} instead.

Let \code{Y_ijk} denote the measurement on subject \code{i}, with method \code{j}, under
condition \code{k}, and consider the model
\deqn{
Y_{ijk} = \mu + \alpha_i + \beta_j + \gamma_k +
(\alpha\beta)_{ij} + (\alpha\gamma)_{ik} + (\beta\gamma)_{jk} + e_{ijk}.
}
Here subject is random, method and time/condition are fixed,
subject-method and subject-time are random interactions, method-time is a
fixed interaction, and \code{e_ijk} is the residual.

For each method pair and condition \code{k}, the repeated-measures CIA is
\deqn{
\psi_k = \frac{2\sigma_e^2}{d_k^2 + 2\sigma_{\alpha\beta}^2 + 2\sigma_e^2},
}
where \code{d_k} is the condition-specific mean difference between the two
methods, \eqn{\sigma_{\alpha\beta}^2} is the subject-method variance
component, and \eqn{\sigma_e^2} is the residual variance component.

The function fits this estimator separately to every method pair. The
homogeneity of agreement across conditions is tested by the method-time
interaction:
\deqn{
F = \frac{MS_{\mathrm{method}\times\mathrm{time}}}{MS_{\mathrm{error}}}.
}
The returned \code{homogeneity_F} attribute stores this test statistic for each
method pair, and \code{homogeneity_p} stores the corresponding upper-tail
\code{F}-test p-value using degrees of freedom \code{df_method_time} and \code{df_error}.
Larger \code{homogeneity_F} values and smaller \code{homogeneity_p} values indicate
stronger evidence that agreement changes across conditions, meaning that the
condition-specific CIA curves should be interpreted directly rather than
relying on a single pooled/common coefficient. Conversely, a small
\code{homogeneity_F} and a non-small \code{homogeneity_p} indicate that the data do
not show strong evidence of method-by-condition heterogeneity, so a common
estimate may be a reasonable summary if it is scientifically meaningful.

When \code{homogeneous = TRUE}, the function also reports a common or pooled CIA
for each method pair from the reduced model that pools the method-time sum of
squares into the residual term. That common estimate is meaningful only when
agreement is reasonably homogeneous across conditions.

Confidence intervals can be computed by a delta-method normal approximation
or by a subject-level percentile bootstrap. The delta-method interval is the
default. For each method pair and condition \code{k}, let
\eqn{\hat\psi_k} be the ANOVA estimator, let \eqn{g_k} be its numerical
gradient with respect to the subject-level moment vector, and let
\eqn{\Sigma} be the empirical covariance matrix of that moment vector divided
by the number of subjects. The delta-method standard error is
\deqn{
\widehat{\mathrm{se}}(\hat\psi_k) =
\sqrt{\max\left(g_k^\top \Sigma g_k, 0\right)}.
}
The raw normal interval is
\deqn{
\hat\psi_k \pm z_{1-\alpha/2}\,\widehat{\mathrm{se}}(\hat\psi_k).
}
Under the bootstrap option, the interval is given by the empirical
percentile limits from subject-resampled estimates. The argument
\code{estimator} controls whether the reported result is the literal
method-of-moments ratio or a bounded variance-component variant. Under
\code{estimator = "vc_constrained"}, the estimated subject-method variance
component is constrained to be non-negative before converting it back to CIA,
and the reported interval limits are also truncated to the CIA parameter
space,
\deqn{
\mathrm{lwr} = \max(0, \mathrm{lwr}_{\mathrm{raw}}), \qquad
\mathrm{upr} = \min(1, \mathrm{upr}_{\mathrm{raw}}).
}
This keeps the reported interval inside the CIA parameter space \verb{[0, 1]}.
Use \code{estimator = "mom_unconstrained"} to inspect the literal raw
method-of-moments estimator and the corresponding unbounded interval on the
estimator scale.

The current implementation requires exactly one observation in every
subject-method-time cell. It therefore targets the balanced repeated-measures
ANOVA setting from the cited paper and returns an error otherwise.
}
\examples{
# Example 1
set.seed(1)
dat_rater <- expand.grid(
  id = factor(sprintf("s\%02d", 1:8)),
  method = factor(c("Device_A", "Device_B")),
  time = factor(c("Rater_1", "Rater_2", "Rater_3")),
  KEEP.OUT.ATTRS = FALSE
)
subj_eff <- rnorm(nlevels(dat_rater$id), sd = 1)[dat_rater$id]
method_shift <- c(Device_A = 0, Device_B = 0.25)[dat_rater$method]
rater_shift <- c(Rater_1 = 0, Rater_2 = 0.15, Rater_3 = -0.10)[dat_rater$time]
interaction_shift <- ifelse(
  dat_rater$method == "Device_B" & dat_rater$time == "Rater_3",
  0.12, 0
)
dat_rater$y <- subj_eff + method_shift + rater_shift +
  interaction_shift + rnorm(nrow(dat_rater), sd = 0.25)

fit_rater <- cia_rm(
  dat_rater,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  homogeneous = TRUE
)
print(fit_rater)
summary(fit_rater)
estimate(fit_rater)
tidy(fit_rater)
plot(fit_rater)

# Example 2
set.seed(2)
dat_time <- expand.grid(
  id = factor(sprintf("s\%02d", 1:10)),
  method = factor(c("Assay_A", "Assay_B", "Assay_C", "Assay_D")),
  time = factor(
    c("baseline", "week2", "month1", "month2", "month3"),
    levels = c("baseline", "week2", "month1", "month2", "month3")
  ),
  KEEP.OUT.ATTRS = FALSE
)
subj_eff <- rnorm(nlevels(dat_time$id), sd = 0.9)[dat_time$id]
method_shift <- c(Assay_A = 0, Assay_B = 0.20, Assay_C = -0.10, Assay_D = 0.35)[dat_time$method]
time_shift <- c(
  baseline = 0, week2 = 0.10, month1 = 0.22, month2 = 0.32, month3 = 0.42
)[dat_time$time]
interaction_shift <- ifelse(dat_time$method == "Assay_B" & dat_time$time == "month3", 0.10, 0) +
  ifelse(dat_time$method == "Assay_D" & dat_time$time == "month2", -0.08, 0)
dat_time$y <- subj_eff + method_shift + time_shift +
  interaction_shift + rnorm(nrow(dat_time), sd = 0.30)

fit_time <- cia_rm(
  dat_time,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  ci = TRUE
)
print(fit_time)
plot(fit_time)

# Example 3
set.seed(3)
dat_days <- expand.grid(
  id = factor(sprintf("s\%02d", 1:12)),
  method = factor(c("Sensor_A", "Sensor_B", "Sensor_C")),
  time = 1:15,
  KEEP.OUT.ATTRS = FALSE
)
subj_eff <- rnorm(nlevels(dat_days$id), sd = 0.8)[dat_days$id]
method_shift <- c(Sensor_A = 0, Sensor_B = 0.15, Sensor_C = -0.08)[dat_days$method]
day_trend <- 0.05 * dat_days$time
interaction_shift <- ifelse(dat_days$method == "Sensor_B", 0.01 * dat_days$time, 0) +
  ifelse(dat_days$method == "Sensor_C", -0.005 * dat_days$time, 0)
dat_days$y <- subj_eff + method_shift + day_trend +
  interaction_shift + rnorm(nrow(dat_days), sd = 0.22)

fit_days <- cia_rm(
  dat_days,
  response = "y",
  subject = "id",
  method = "method",
  time = "time",
  ci = TRUE
)
plot(fit_days, facet_by_pair = TRUE)

}
\references{
Haber M, Gao J, Barnhart HX. (2010). Evaluation of agreement between
measurement methods from data with matched repeated measurements via the
coefficient of individual agreement. \emph{Journal of Data Science}, 8,
457-469.

Barnhart HX, Kosinski AS, Haber M. (2007). Assessing individual agreement.
\emph{Journal of Biopharmaceutical Statistics}, 17(4), 697-719.
}
\seealso{
\code{\link[=ccc_rm_reml]{ccc_rm_reml()}}, \code{\link[=icc_rm_reml]{icc_rm_reml()}}, and \code{\link[=cia]{cia()}}.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_matrix_dims}
\alias{check_matrix_dims}
\title{Check matrix dimensions}
\usage{
check_matrix_dims(
  M,
  nrow = NULL,
  ncol = NULL,
  arg = as.character(substitute(M))
)
}
\description{
Check matrix dimensions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robust_dcor.R
\name{robust_dcor}
\alias{robust_dcor}
\alias{print.robust_dcor}
\alias{summary.robust_dcor}
\alias{plot.robust_dcor}
\title{Robust Distance Correlation}
\usage{
robust_dcor(
  data,
  na_method = c("error", "pairwise", "complete"),
  p_value = FALSE,
  inference = c("none", "permutation"),
  n_perm = 999L,
  seed = NULL,
  c_const = 4,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  ...
)

\method{print}{robust_dcor}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{robust_dcor}(object, topn = NULL, show_ci = NULL, ...)

\method{plot}{robust_dcor}(
  x,
  title = NULL,
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or a data frame with at least two numeric
columns. All non-numeric columns are dropped. Columns must be numeric.}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"pairwise"} recomputes each association on its own pairwise
complete-case overlap. This is permissive, but different matrix entries
may be based on different rows. \code{"complete"} performs listwise
deletion once across the retained numeric columns and then computes the
estimator on the common complete sample.}

\item{p_value}{Logical (default \code{FALSE}). If \code{TRUE}, attach
permutation p-values for pairwise independence.}

\item{inference}{Character scalar. Use \code{"none"} (default) for estimates
only or \code{"permutation"} to request permutation p-values. If
\code{inference = "permutation"} is supplied with \code{p_value = FALSE},
p-values are inferred and returned.}

\item{n_perm}{Positive integer; number of permutations used when
\code{p_value = TRUE}. Default \code{999}.}

\item{seed}{Optional positive integer seed for permutation inference.}

\item{c_const}{Positive numeric tuning constant for the biloop
transformation. Default \code{4}.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates:
\code{"matrix"}, \code{"sparse"}, or \code{"edge_list"}.}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs.
Must be \code{0} when \code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{...}{Deprecated compatibility aliases. Currently only
\code{check_na} is supported.}

\item{x, object}{An object of class \code{robust_dcor}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{title}{Plot title.}

\item{low_color, high_color, mid_color}{Colors used in the heatmap.}

\item{value_text_size, ci_text_size}{Text sizes for plot labels.}

\item{show_value}{Logical; overlay numeric values if \code{TRUE}.}
}
\value{
A symmetric correlation result. Dense output inherits from
\code{robust_dcor}, \code{corr_matrix}, and \code{matrix}. Sparse and
edge-list outputs use the package-standard \code{corr_sparse} and
\code{corr_edge_list} representations.
}
\description{
Computes robust distance correlations by applying the biloop transformation
to each numeric variable and then computing unbiased distance correlation on
the transformed variables.
}
\details{
Permutation inference is computed for every unique column pair, so the
workload grows as \eqn{p(p-1)n_\mathrm{perm}/2}. A warning is emitted for
large requests; reduce \code{n_perm}, reduce the number of columns, or run
estimates without permutation p-values when this cost is not intended.

For each variable \eqn{x = (x_1,\ldots,x_n)}, the wrapper first robustly
standardises values using the median
\deqn{m_x = \mathrm{median}(x)}
and raw median absolute deviation
\deqn{s_x = \mathrm{median}\{|x_i - m_x|\}.}
If this scale is zero or non-finite, the implementation falls back to
\eqn{\mathrm{IQR}(x) / 1.34898} and then the ordinary sample standard
deviation. Columns with no positive finite fallback scale are treated as
degenerate.

The standardised value \eqn{z_i = (x_i - m_x) / s_x} is mapped to two
bounded coordinates by the biloop transformation
\deqn{\theta_i = 2\pi\tanh(z_i / c),}
\deqn{u_i =
\begin{cases}
c\{1-\cos(\theta_i)\}, & z_i > 0,\\
-c\{1-\cos(\theta_i)\}, & z_i < 0,\\
0, & z_i = 0,
\end{cases}}
and
\deqn{v_i = \sin(\theta_i).}
Pairwise distances are Euclidean distances in this transformed
two-dimensional space,
\deqn{D^{(j)}_{ab} =
\left[(u_{aj}-u_{bj})^2 + (v_{aj}-v_{bj})^2\right]^{1/2},\quad a \ne b,}
with zero diagonal.

The transformed distance matrix is U-centred as
\deqn{A^{(j)}_{ab} =
D^{(j)}_{ab} - \frac{r^{(j)}_a + r^{(j)}_b}{n - 2}
+ \frac{S^{(j)}}{(n - 1)(n - 2)},\quad a \ne b,}
where \eqn{r^{(j)}_a = \sum_{b \ne a}D^{(j)}_{ab}} and
\eqn{S^{(j)} = \sum_{a \ne b}D^{(j)}_{ab}}. The diagonal of
\eqn{A^{(j)}} is zero.

For variables \eqn{j} and \eqn{k}, the unbiased robust distance covariance
is
\deqn{\widehat{\mathrm{dCov}}^2_u(j,k) =
\frac{2}{n(n-3)}\sum_{a < b} A^{(j)}_{ab} A^{(k)}_{ab}.}
The corresponding robust distance correlation is the usual non-negative
distance-correlation ratio based on this covariance and the two transformed
distance variances. Small negative numerical artifacts are clipped to zero.
}
\note{
\code{robust_dcor()} is more robust to extreme observations than classical
\code{dcor()}, but it may downweight genuine tail dependence. Classical
\code{dcor()} may be preferable when tail dependence is the scientific target.
Comparing both methods is recommended.
}
\section{Relationship to \code{\link[=dcor]{dcor()}}}{

\code{robust_dcor()} is not a replacement for \code{dcor()}. It estimates distance
correlation after a bounded robust transformation. Large differences between
\code{dcor()} and \code{robust_dcor()} can indicate that the classical dependence
signal is driven by tail observations or outliers.
}

\examples{
## Non-linear dependence: both estimators detect association.
set.seed(1)
n <- 200
x <- rnorm(n)
y <- x^2 + rnorm(n, sd = 0.2)

X <- cbind(x = x, y = y)

classical <- dcor(X)
robust <- robust_dcor(X)
round(c(
  dcor = classical["x", "y"],
  robust_dcor = robust["x", "y"]
), 3)

## One diagonal outlier can inflate classical dCor more than robust dCor.
set.seed(45)
x <- rnorm(20)
y <- rnorm(20)
x[1] <- 10
y[1] <- 10
X_out <- cbind(x = x, y = y)

classical <- dcor(X_out)
robust <- robust_dcor(X_out)
round(c(
  dcor = classical["x", "y"],
  robust_dcor = robust["x", "y"]
), 3)

print(classical)
print(robust)
summary(robust)
estimate(robust)
tidy(robust)
plot(robust)

## Several variables.
set.seed(7)
z <- rnorm(120)
X_multi <- cbind(
  linear = z + rnorm(120, sd = 0.3),
  nonlinear = z^2 + rnorm(120, sd = 0.3),
  noise = rnorm(120),
  outlier = rnorm(120)
)
X_multi[1, "outlier"] <- 12
X_multi[1, "noise"] <- 12

robust_multi <- robust_dcor(X_multi)

print(robust_multi)
summary(robust_multi)
plot(robust_multi)

}
\references{
Leyder, J., Raymaekers, J., & Rousseeuw, P. J. (2025). Robust distance
correlation through bounded transformations.
}
\seealso{
\code{\link[=dcor]{dcor()}}, \code{\link[=wincor]{wincor()}}, \code{\link[=pbcor]{pbcor()}}, \code{\link[=skipped_corr]{skipped_corr()}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_internal}
\alias{abort_internal}
\title{Abort for internal errors (should not happen)}
\usage{
abort_internal(message, ..., .class = character())
}
\description{
Abort for internal errors (should not happen)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{print.matrixCorr_ccc_ci}
\alias{print.matrixCorr_ccc_ci}
\title{Print method for matrixCorr CCC objects with CIs}
\usage{
\method{print}{matrixCorr_ccc_ci}(x, ...)
}
\arguments{
\item{x}{A \code{matrixCorr_ccc} or \code{matrixCorr_ccc_ci} object.}

\item{...}{Passed to underlying printers.}
}
\description{
Print method for matrixCorr CCC objects with CIs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{inform_if_verbose}
\alias{inform_if_verbose}
\title{Inform only when verbose}
\usage{
inform_if_verbose(
  ...,
  .verbose = getOption("matrixCorr.verbose", TRUE),
  .envir = parent.frame()
)
}
\description{
Inform only when verbose
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concordance_corr.R
\name{ccc}
\alias{ccc}
\alias{print.ccc}
\alias{summary.ccc}
\alias{print.summary.ccc}
\alias{plot.ccc}
\title{Pairwise Lin's concordance correlation coefficient}
\usage{
ccc(
  data,
  ci = FALSE,
  conf_level = 0.95,
  na_method = c("error", "complete", "pairwise"),
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE,
  verbose = FALSE
)

\method{print}{ccc}(
  x,
  digits = 4,
  ci_digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{ccc}(
  object,
  digits = 4,
  ci_digits = 2,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.ccc}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{ccc}(
  x,
  title = "Lin's Concordance Correlation Heatmap",
  low_color = "indianred1",
  high_color = "steelblue1",
  mid_color = "white",
  value_text_size = 4,
  ci_text_size = 3,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or data frame with at least two numeric columns.
Non-numeric columns will be ignored.}

\item{ci}{Logical; if TRUE, return lower and upper confidence bounds}

\item{conf_level}{Confidence level for CI, default = 0.95}

\item{na_method}{Character scalar controlling missing-data handling.
\code{"error"} rejects missing, \code{NaN}, and infinite values.
\code{"complete"} removes rows incomplete in any retained numeric column
before computing all CCC estimates. \code{"pairwise"} recomputes each
method pair on its own complete finite overlap.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation for the computed estimates.
\itemize{
\item \code{"matrix"} (default): full dense matrix; best when you need
matrix algebra, dense heatmaps, or full compatibility with existing code.
\item \code{"sparse"}: sparse matrix from \pkg{Matrix} containing only
retained entries; best when many values are dropped by thresholding.
\item \code{"edge_list"}: long-form data frame with columns
\code{row}, \code{col}, \code{value}; convenient for filtering, joins,
and network-style workflows.
}}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs:
keep entries with \code{abs(value) >= threshold}. Use
\code{threshold > 0} when you want only stronger associations (typically
with \code{output = "sparse"} or \code{"edge_list"}). Keep
\code{threshold = 0} to retain all values. Must be \code{0} when
\code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in
\code{"sparse"} and \code{"edge_list"} outputs.}

\item{verbose}{Logical; if TRUE, prints how many threads are used}

\item{x}{An object of class \code{"ccc"} (either a matrix or a list with CIs).}

\item{digits}{Integer; decimals for CCC estimates (default 4).}

\item{ci_digits}{Integer; decimals for CI bounds (default 2).}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives this
from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Passed to \code{ggplot2::theme()}.}

\item{object}{A \code{"ccc"} or \code{"ccc_ci"} object to summarize.}

\item{title}{Title for the plot.}

\item{low_color}{Color for low CCC values.}

\item{high_color}{Color for high CCC values.}

\item{mid_color}{Color for mid CCC values.}

\item{value_text_size}{Text size for CCC values in the heatmap.}

\item{ci_text_size}{Text size for confidence intervals.}

\item{show_value}{Logical; if \code{TRUE} (default), overlay numeric values
on the heatmap tiles.}
}
\value{
A symmetric numeric matrix with class \code{"ccc"} and attributes:
\itemize{
\item \code{method}: The method used ("Lin's concordance")
\item \code{description}: Description string
}
If \code{ci = FALSE}, returns matrix of class \code{"ccc"}.
If \code{ci = TRUE}, returns a list with elements: \code{est},
\code{lwr.ci}, \code{upr.ci}.

For \code{summary.ccc}, a data frame with columns
\code{item1}, \code{item2}, \code{estimate}, and (optionally)
\code{lwr}, \code{upr}, plus \code{n_complete} when available.
}
\description{
Computes all pairwise Lin's Concordance Correlation Coefficients (CCC)
from the numeric columns of a matrix or data frame. CCC measures both
precision (Pearson correlation) and accuracy (closeness to the 45-degree line).
This function is backed by a high-performance 'C++' implementation.

Lin's CCC quantifies the concordance between a new test/measurement
and a gold-standard for the same variable. Like a correlation, CCC
ranges from -1 to 1 with perfect agreement at 1, and it cannot exceed the
absolute value of the Pearson correlation between variables. It can be
legitimately computed even with small samples (e.g., 10 observations),
and results are often similar to intraclass correlation coefficients.
CCC provides a single summary of agreement, but it may not capture
systematic bias; a Bland-Altman plot (differences vs. means) is recommended
to visualize bias, proportional trends, and heteroscedasticity (see
\code{\link{ba}}).
}
\details{
Lin's CCC is defined as
\deqn{
\rho_c \;=\; \frac{2\,\mathrm{cov}(X, Y)}
                 {\sigma_X^2 + \sigma_Y^2 + (\mu_X - \mu_Y)^2},
}
where \eqn{\mu_X,\mu_Y} are the means, \eqn{\sigma_X^2,\sigma_Y^2} the
variances, and \eqn{\mathrm{cov}(X,Y)} the covariance. Equivalently,
\deqn{
\rho_c \;=\; r \times C_b, \qquad
r \;=\; \frac{\mathrm{cov}(X,Y)}{\sigma_X \sigma_Y}, \quad
C_b \;=\; \frac{2 \sigma_X \sigma_Y}
               {\sigma_X^2 + \sigma_Y^2 + (\mu_X - \mu_Y)^2}.
}
Hence \eqn{|\rho_c| \le |r| \le 1}, \eqn{\rho_c = r} iff
\eqn{\mu_X=\mu_Y} and \eqn{\sigma_X=\sigma_Y}, and \eqn{\rho_c=1} iff, in
addition, \eqn{r=1}. CCC is symmetric in \eqn{(X,Y)} and penalises both
location and scale differences; unlike Pearson's \eqn{r}, it is not invariant
to affine transformations that change means or variances.

When \code{ci = TRUE}, large-sample confidence intervals for
\eqn{\rho_c} are returned for each pair. The implementation uses Lin's
delta-method standard error and then forms limits on a Fisher-z transformed
CCC scale before mapping back to \eqn{[-1, 1]}. For speed, CIs are omitted
when \code{ci = FALSE}.

If either variable has zero variance, \eqn{\rho_c} is
undefined and \code{NA} is returned for that pair (including the diagonal).

Missing-data handling follows the same contract as \code{\link{pearson_corr}}
and \code{\link{spearman_rho}}. With \code{na_method = "error"} (default),
missing and non-finite values are rejected. With \code{"complete"}, rows
incomplete in any retained numeric column are removed once before all pairwise
CCC estimates are computed. With \code{"pairwise"}, each method pair is
computed on its own complete finite overlap, and \code{n_complete}
diagnostics may vary by pair. Pairwise CIs require at least three complete
observations for that pair.

Negative CCC estimates are fully supported. Matrix output preserves the
signed estimate, while thresholded \code{"sparse"} and \code{"edge_list"}
outputs retain entries according to \code{abs(CCC) >= threshold}.

Probability of agreement, available through \code{\link{prob_agree}},
answers a
different question from CCC. CCC is a coefficient-based summary of
concordance between paired measurements. \code{prob_agree()} follows Stevens and
Anderson-Cook (2017) and estimates the probability that two estimated
quantities or curves differ by no more than a user-specified practical
tolerance.
}
\examples{
# Example with multivariate normal data
Sigma <- matrix(c(1, 0.5, 0.3,
                  0.5, 1, 0.4,
                  0.3, 0.4, 1), nrow = 3)
mu <- c(0, 0, 0)
set.seed(123)
mat_mvn <- MASS::mvrnorm(n = 100, mu = mu, Sigma = Sigma)
result_mvn <- ccc(mat_mvn, ci = TRUE)
print(result_mvn)
summary(result_mvn)
estimate(result_mvn)
tidy(result_mvn)
ci(result_mvn)
confint(result_mvn)
plot(result_mvn)

# Interactive viewing (requires shiny)
if (interactive() && requireNamespace("shiny", quietly = TRUE)) {
  view_corr_shiny(result_mvn)
}

}
\references{
Lin L (1989). A concordance correlation coefficient to evaluate
reproducibility. Biometrics 45: 255-268.

Lin L (2000). A note on the concordance correlation coefficient.
Biometrics 56: 324-325.

Bland J, Altman D (1986). Statistical methods for assessing agreement
between two methods of clinical measurement. The Lancet 327: 307-310.
}
\seealso{
\code{\link{print.ccc}}, \code{\link{plot.ccc}},
\code{\link{ba}}, \code{\link{prob_agree}}, and \code{\link{cia}}.
\code{ccc()} answers the question "How well do two paired measurements
agree overall, accounting for both correlation and mean/scale bias?".
In contrast, \code{cia()} answers "Are two or more methods
interchangeable at the individual level relative to within-method
replicate disagreement?".

For repeated measurements look at \code{\link{ccc_rm_reml}},
\code{\link{ccc_rm_ustat}} or \code{\link{ba_rm}}
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{print.matrixCorr_ccc}
\alias{print.matrixCorr_ccc}
\title{Print method for matrixCorr CCC objects}
\usage{
\method{print}{matrixCorr_ccc}(
  x,
  digits = 4,
  ci_digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{matrixCorr_ccc} or \code{matrixCorr_ccc_ci} object.}

\item{digits}{Number of digits for CCC estimates.}

\item{ci_digits}{Number of digits for CI bounds.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns; \code{NULL} derives
this from console width.}

\item{width}{Optional display width; defaults to \code{getOption("width")}.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}.}

\item{...}{Passed to underlying printers.}
}
\description{
Print method for matrixCorr CCC objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hsic.R
\name{hsic}
\alias{hsic}
\alias{print.hsic}
\alias{summary.hsic}
\alias{print.summary.hsic}
\alias{plot.hsic}
\title{Pairwise Hilbert-Schmidt Independence Criterion}
\usage{
hsic(
  data,
  kernel = c("gaussian", "linear", "laplace", "polynomial"),
  bandwidth = c("median", "silverman", "scott"),
  normalise = TRUE,
  estimator = c("biased", "unbiased"),
  na_method = c("error", "pairwise", "complete"),
  p_value = FALSE,
  B = 499L,
  seed = NULL,
  n_threads = getOption("matrixCorr.threads", 1L),
  output = c("matrix", "sparse", "edge_list"),
  threshold = 0,
  diag = TRUE
)

\method{print}{hsic}(
  x,
  digits = 4,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{summary}{hsic}(
  object,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{print}{summary.hsic}(
  x,
  digits = NULL,
  n = NULL,
  topn = NULL,
  max_vars = NULL,
  width = NULL,
  show_ci = NULL,
  ...
)

\method{plot}{hsic}(
  x,
  title = NULL,
  low_color = "white",
  high_color = "steelblue1",
  value_text_size = 4,
  show_value = TRUE,
  ...
)
}
\arguments{
\item{data}{A numeric matrix or data frame with at least two numeric columns.
Non-numeric columns are dropped.}

\item{kernel}{Kernel used to build univariate Gram matrices. Supported
options are \code{"gaussian"} (RBF), \code{"linear"}, \code{"laplace"}, and
\code{"polynomial"}.}

\item{bandwidth}{Bandwidth rule for Gaussian and Laplace kernels.
\code{"median"} uses the upper median of finite off-diagonal pairwise
distances divided by \eqn{\sqrt{2}}, with fallback \code{0.001} for degenerate
columns. \code{"silverman"} uses Silverman's univariate rule, and \code{"scott"}
uses Scott's univariate rule. Linear and polynomial kernels ignore this
argument.}

\item{normalise}{Logical. If \code{TRUE}, return the normalised kernel
independence correlation. If \code{FALSE}, return raw HSIC.}

\item{estimator}{HSIC estimator. \code{"biased"} uses the stable V-statistic
\eqn{n^{-2}\mathrm{tr}(KHLH)}. \code{"unbiased"} uses the U-statistic estimator
with zero diagonal Gram matrices and requires at least four complete
observations per pair.}

\item{na_method}{Missing-data handling. \code{"error"} rejects missing, \code{NaN}, and
infinite values; \code{"complete"} performs one listwise finite-row deletion;
\code{"pairwise"} recomputes each pair on its pairwise complete rows.}

\item{p_value}{Logical. If \code{TRUE}, attach permutation p-values for the HSIC
independence test.}

\item{B}{Positive integer. Number of permutations used when
\code{p_value = TRUE}.}

\item{seed}{Optional positive integer seed for reproducible permutation
inference. The seed is passed to the C++ permutation engine and does not
mutate the user's global R RNG state.}

\item{n_threads}{Integer \eqn{\geq 1}. Number of OpenMP threads. Defaults to
\code{getOption("matrixCorr.threads", 1L)}.}

\item{output}{Output representation: \code{"matrix"}, \code{"sparse"}, or
\code{"edge_list"}.}

\item{threshold}{Non-negative absolute-value filter for non-matrix outputs.
Must be \code{0} when \code{output = "matrix"}.}

\item{diag}{Logical; whether to include diagonal entries in sparse and
edge-list outputs.}

\item{x}{An object of class \code{hsic}.}

\item{digits}{Integer; number of decimal places to print.}

\item{n}{Optional row threshold for compact preview output.}

\item{topn}{Optional number of leading/trailing rows to show when truncated.}

\item{max_vars}{Optional maximum number of visible columns.}

\item{width}{Optional display width.}

\item{show_ci}{One of \code{"yes"} or \code{"no"}; accepted for compatibility with
shared matrix-printing methods.}

\item{...}{Additional arguments passed to print or plot methods.}

\item{object}{An object of class \code{hsic}.}

\item{title}{Optional plot title.}

\item{low_color}{Colour for low HSIC values.}

\item{high_color}{Colour for high HSIC values.}

\item{value_text_size}{Font size for tile labels.}

\item{show_value}{Logical; whether to overlay numeric values on the heatmap.}
}
\value{
A matrix-like correlation result. With \code{normalise = TRUE}, entries
are normalised kernel independence correlations and the diagonal is \code{1}
when the variable has positive kernel self-dependence. With
\code{normalise = FALSE}, entries are raw HSIC estimates. Attributes record the
kernel, bandwidth rule, estimator, raw HSIC matrix, diagnostics, and
optional permutation inference.
}
\description{
Computes pairwise kernel dependence measures for the numeric columns of a
matrix or data frame using the Hilbert-Schmidt Independence Criterion
(HSIC). By default, \code{hsic()} returns a normalised kernel independence
correlation in \verb{[0, 1]}, analogous to a distance-correlation matrix.
}
\details{
For paired observations \eqn{(x_i, y_i)}, HSIC measures the squared
Hilbert-Schmidt norm of the RKHS cross-covariance operator. With centred Gram
matrices \eqn{K_c = HKH} and \eqn{L_c = HLH}, the biased empirical estimator
is
\deqn{
  \widehat{\mathrm{HSIC}}_b =
  \frac{1}{n^2}\mathrm{tr}(KHLH)
  =
  \frac{1}{n^2}\sum_{ij} (K_c)_{ij}(L_c)_{ij}.
}

With \code{normalise = TRUE}, the returned matrix contains the kernel
independence correlation
\deqn{
  \widehat{\mathrm{kCor}}(X,Y) =
  \frac{\widehat{\mathrm{HSIC}}(X,Y)}
       {\sqrt{\widehat{\mathrm{HSIC}}(X,X)
              \widehat{\mathrm{HSIC}}(Y,Y)}}.
}
The raw HSIC matrix is stored in \code{attr(x, "hsic_raw")}. When
\code{estimator = "unbiased"}, raw HSIC can be slightly negative in finite
samples; these signed raw values are retained in \code{hsic_raw}, while the
displayed normalised coefficient is clipped only for the user-facing matrix.

Kernel formulas are:
Gaussian \eqn{\exp(-|x_i - x_j|^2 / (2\sigma^2))}, Laplace
\eqn{\exp(-|x_i - x_j| / \sigma)}, linear \eqn{x_i x_j}, and polynomial
\eqn{(x_i x_j + 1)^2}. For \code{bandwidth = "median"}, \eqn{\sigma} is the
upper median of \eqn{\{|x_i - x_j|: i < j\}} divided by \eqn{\sqrt{2}}; if
this is non-finite or non-positive, \eqn{\sigma = 0.001}.

For \code{p_value = TRUE}, the test statistic is the raw HSIC estimate. The null
distribution is generated by permuting one variable within each pair, and the
p-value is \eqn{(1 + \#\{T_b \geq T_{\mathrm{obs}}\})/(B + 1)}. This is an
\eqn{O(B n^2)} pairwise procedure and can be expensive for large matrices or
large \code{B}.

The polynomial kernel currently uses the fixed form
\eqn{(x_i x_j + 1)^2}. More polynomial controls can be added later without
changing the main HSIC object contract.
}
\examples{
set.seed(1)
x <- rnorm(200)
y <- rnorm(200)
H0 <- hsic(cbind(x = x, y = y))
H0

set.seed(2)
x <- rnorm(200)
y <- x^2 + rnorm(200, sd = 0.1)
H1 <- hsic(cbind(x = x, y = y))
H1

set.seed(3)
X <- cbind(a = rnorm(80), b = rnorm(80), c = rnorm(80))
H_perm <- hsic(X, p_value = TRUE, B = 19, seed = 1)
summary(H_perm)
estimate(H_perm)
tidy(H_perm)

}
\references{
Gretton, A., Bousquet, O., Smola, A., & Schoelkopf, B. (2005).
Measuring statistical dependence with Hilbert-Schmidt norms.
\emph{Algorithmic Learning Theory}.

Gretton, A., Fukumizu, K., Teo, C. H., Song, L., Schoelkopf, B., & Smola,
A. J. (2007). A kernel statistical test of independence.
\emph{Advances in Neural Information Processing Systems}.

Gretton, A., Bousquet, O., Smola, A., & Schoelkopf, B. (2005).
Kernel methods for measuring independence.
\emph{Journal of Machine Learning Research}, 6, 2075-2129.

Pfister, N., Buhlmann, P., Scholkopf, B., & Peters, J. (2018).
Kernel-based tests for joint independence.
\emph{Journal of the Royal Statistical Society: Series B}, 80(1), 5-31.
}
\author{
Thiago de Paula Oliveira
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_result_s3.R
\name{as.data.frame.corr_edge_list}
\alias{as.data.frame.corr_edge_list}
\title{Edge-list data-frame view}
\usage{
\method{as.data.frame}{corr_edge_list}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{A \code{corr_edge_list} object.}

\item{row.names}{Ignored.}

\item{optional}{Ignored.}

\item{...}{Unused.}
}
\value{
A data frame with columns \code{row}, \code{col}, \code{value}.
}
\description{
Edge-list data-frame view
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrixCorr-package.R
\docType{package}
\name{matrixCorr-package}
\alias{matrixCorr-package}
\alias{matrixCorr}
\alias{RcppExports}
\title{matrixCorr}
\description{
Correlation, agreement, and intraclass correlation estimators with a
consistent S3 interface.
}
\details{
\code{print()} methods provide compact previews and \code{summary()} methods provide
richer but still bounded digests. Truncation affects console display only;
full results remain available through direct extraction and coercion helpers
such as \code{as.matrix()}, \code{as.data.frame()}, and \code{tidy()} where implemented.

Package-wide display defaults can be controlled with options:
\itemize{
\item \code{matrixCorr.threads} (default \code{1L}; used as the default for
\code{n_threads} in thread-aware estimators)
\item \code{matrixCorr.print_max_rows} (default \code{20L})
\item \code{matrixCorr.print_topn} (default \code{5L})
\item \code{matrixCorr.print_max_vars} (default \code{NULL}, derived from console width)
\item \code{matrixCorr.print_show_ci} (default \code{"yes"})
\item \code{matrixCorr.summary_max_rows} (default \code{12L})
\item \code{matrixCorr.summary_topn} (default \code{5L})
\item \code{matrixCorr.summary_max_vars} (default \code{10L})
\item \code{matrixCorr.summary_show_ci} (default \code{"yes"})
}

Display methods validate user-facing arguments with \code{cli} conditions.
Confidence-interval visibility is explicit: \code{show_ci} accepts only \code{"yes"}
or \code{"no"}.

A common setup is:
\preformatted{
options(matrixCorr.threads = parallel::detectCores(logical = FALSE))
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Prof-ThiagoOliveira/matrixCorr}
  \item Report bugs at \url{https://github.com/Prof-ThiagoOliveira/matrixCorr/issues}
}

}
\author{
\strong{Maintainer}: Thiago de Paula Oliveira \email{thiago.paula.oliveira@gmail.com} (\href{https://orcid.org/0000-0002-4555-2584}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{.align_weights_to_levels}
\alias{.align_weights_to_levels}
\title{Align (optional named) weights to a subset of time levels}
\usage{
.align_weights_to_levels(w, present_lvls, all_lvls)
}
\description{
Align (optional named) weights to a subset of time levels
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccc_repeated.R
\name{print.ccc_ci}
\alias{print.ccc_ci}
\title{S3 print for legacy class \code{ccc_ci}}
\usage{
\method{print}{ccc_ci}(x, ...)
}
\arguments{
\item{x}{A \code{matrixCorr_ccc} or \code{matrixCorr_ccc_ci} object.}

\item{...}{Passed to underlying printers.}
}
\description{
For compatibility with objects that still carry class \code{"ccc_ci"}.
}
